/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/gather-dep-set.js TAP > set with a 1`] = `
Array [
  "node_modules/a",
  "node_modules/b",
  "node_modules/i",
  "node_modules/j",
  "node_modules/k",
  "node_modules/x",
  "node_modules/y",
]
`

exports[`test/gather-dep-set.js TAP > set with a and x 1`] = `
Array [
  "node_modules/a",
  "node_modules/b",
  "node_modules/i",
  "node_modules/j",
  "node_modules/k",
  "node_modules/x",
  "node_modules/y",
]
`

exports[`test/gather-dep-set.js TAP > set with b 1`] = `
Array []
`

exports[`test/gather-dep-set.js TAP > set with c only 1`] = `
Array [
  "node_modules/c",
  "node_modules/r",
  "node_modules/s",
  "node_modules/t",
]
`
