// generated from test/fixtures/peer-dep-cycle-nested-with-sw
module.exports = t => {
  const path = t.testdir({
  "package-lock.json": JSON.stringify({
    "name": "@isaacs/peer-dep-nested",
    "version": "1.0.0",
    "lockfileVersion": 2,
    "requires": true,
    "packages": {
      "": {
        "name": "@isaacs/peer-dep-nested",
        "version": "1.0.0",
        "dependencies": {
          "@isaacs/peer-dep-cycle": "1"
        }
      },
      "node_modules/@isaacs/peer-dep-cycle": {
        "name": "@isaacs/peer-dep-cycle",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
        "integrity": "sha512-VpDmOiv9MGcPlt5F6x8mCqQvactg+BOufGTwyvQWxoqKoScZA5CimqjbWS8yDkGYXi5nKQNSgZ/zbWAo7lAxBQ==",
        "dependencies": {
          "@isaacs/peer-dep-cycle-a": "1"
        }
      },
      "node_modules/@isaacs/peer-dep-cycle-a": {
        "name": "@isaacs/peer-dep-cycle-a",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
        "integrity": "sha512-TrtKy03X2HSESjcRbWGIpCJWLZifvW6aViaOnWXjZ7xC2bhuWT/ZSpvE5weStWZRQZIfAO9dmG7EK3D9TbauUg==",
        "peerDependencies": {
          "@isaacs/peer-dep-cycle-b": "1"
        }
      },
      "node_modules/@isaacs/peer-dep-cycle-b": {
        "name": "@isaacs/peer-dep-cycle-b",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
        "integrity": "sha512-lRSvQ4sIRhXjMBcn9z8wCxNTySpCvMyDJvQfAGrE9FsAhmugJRRDYcCHTEpTMRqYJQR0Rn97Np5P7QbiBuH1aQ==",
        "peerDependencies": {
          "@isaacs/peer-dep-cycle-c": "1"
        }
      },
      "node_modules/@isaacs/peer-dep-cycle-c": {
        "name": "@isaacs/peer-dep-cycle-c",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
        "integrity": "sha512-mE1f1QdCY1+hMkdlGruWYauq8wZaUJ5uA9U8vc2LBs0pPdq9cPESLk6NY7n338WTiZkuqdh3+ooIjTzgR9IGiQ==",
        "peerDependencies": {
          "@isaacs/peer-dep-cycle-a": "1"
        }
      }
    },
    "dependencies": {
      "@isaacs/peer-dep-cycle": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
        "integrity": "sha512-VpDmOiv9MGcPlt5F6x8mCqQvactg+BOufGTwyvQWxoqKoScZA5CimqjbWS8yDkGYXi5nKQNSgZ/zbWAo7lAxBQ==",
        "requires": {
          "@isaacs/peer-dep-cycle-a": "1"
        }
      },
      "@isaacs/peer-dep-cycle-a": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
        "integrity": "sha512-TrtKy03X2HSESjcRbWGIpCJWLZifvW6aViaOnWXjZ7xC2bhuWT/ZSpvE5weStWZRQZIfAO9dmG7EK3D9TbauUg==",
        "requires": {
          "@isaacs/peer-dep-cycle-b": "1"
        }
      },
      "@isaacs/peer-dep-cycle-b": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
        "integrity": "sha512-lRSvQ4sIRhXjMBcn9z8wCxNTySpCvMyDJvQfAGrE9FsAhmugJRRDYcCHTEpTMRqYJQR0Rn97Np5P7QbiBuH1aQ==",
        "requires": {
          "@isaacs/peer-dep-cycle-c": "1"
        }
      },
      "@isaacs/peer-dep-cycle-c": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
        "integrity": "sha512-mE1f1QdCY1+hMkdlGruWYauq8wZaUJ5uA9U8vc2LBs0pPdq9cPESLk6NY7n338WTiZkuqdh3+ooIjTzgR9IGiQ==",
        "requires": {
          "@isaacs/peer-dep-cycle-a": "1"
        }
      }
    }
  }),
  "package.json": JSON.stringify({
    "name": "@isaacs/peer-dep-nested",
    "version": "1.0.0",
    "description": "dep -> a -> b -> c -> a",
    "dependencies": {
      "@isaacs/peer-dep-cycle": "1"
    }
  })
})
  return path
}
