/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/tag.js TAP generate args from options > default options 1`] = `
Array [
  "-m",
  "v1.2.3",
  "undefined1.2.3",
]
`

exports[`test/tag.js TAP generate args from options > non-default options 1`] = `
Array [
  "-sfm",
  "hello, 1.2.3, this is a message for you about 1.2.3.",
  "undefined1.2.3",
]
`
