/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/lib/commands/completion.js TAP completion --no- flags > flags 1`] = `
Array [
  Array [
    String(
      --no-version
      --no-versions
    ),
  ],
]
`

exports[`test/lib/commands/completion.js TAP completion commands with no completion > no results 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion completion cannot complete options that take a value in mid-command > does not try to complete option arguments in the middle of a command 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion completion completion > both shells 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion completion completion no known shells > no responses 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion completion completion wrong word count > no responses 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion completion of invalid command name does nothing > no results 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion double dashes escape from flag completion > full command list 1`] = `
Array [
  Array [
    String(
      access
      adduser
      audit
      bugs
      cache
      ci
      completion
      config
      dedupe
      deprecate
      diff
      dist-tag
      docs
      doctor
      edit
      exec
      explain
      explore
      find-dupes
      fund
      get
      help
      hook
      init
      install
      install-ci-test
      install-test
      link
      ll
      login
      logout
      ls
      org
      outdated
      owner
      pack
      ping
      pkg
      prefix
      profile
      prune
      publish
      query
      rebuild
      repo
      restart
      root
      run-script
      search
      set
      shrinkwrap
      star
      stars
      start
      stop
      team
      test
      token
      uninstall
      unpublish
      unstar
      update
      version
      view
      whoami
      author
      home
      issues
      info
      show
      find
      add
      unlink
      remove
      rm
      r
      un
      rb
      list
      ln
      create
      i
      it
      cit
      up
      c
      s
      se
      tst
      t
      ddp
      v
      run
      clean-install
      clean-install-test
      x
      why
      la
      verison
      ic
      innit
      in
      ins
      inst
      insta
      instal
      isnt
      isnta
      isntal
      isntall
      install-clean
      isntall-clean
      hlep
      dist-tags
      upgrade
      udpate
      rum
      sit
      urn
      ogr
      add-user
    ),
  ],
]
`

exports[`test/lib/commands/completion.js TAP completion filtered subcommands > filtered subcommands 1`] = `
Array []
`

exports[`test/lib/commands/completion.js TAP completion flags > flags 1`] = `
Array [
  Array [
    String(
      --version
      --versions
      --viewer
      --verbose
      --v
    ),
  ],
]
`

exports[`test/lib/commands/completion.js TAP completion multiple command names > multiple command names 1`] = `
Array [
  Array [
    String(
      access
      adduser
      audit
      author
      add
      add-user
    ),
  ],
]
`

exports[`test/lib/commands/completion.js TAP completion single command name > single command name 1`] = `
Array [
  Array [
    "config",
  ],
]
`

exports[`test/lib/commands/completion.js TAP completion subcommand completion > subcommands 1`] = `
Array [
  Array [
    String(
      get
      grant
      list
      revoke
      set
    ),
  ],
]
`

exports[`test/lib/commands/completion.js TAP windows without bash > no output 1`] = `
Array []
`
