module.exports = {
  methane: 'CH4',
  access: null,
  all: false,
  'allow-same-version': false,
  'always-auth': false,
  also: null,
  audit: true,
  'audit-level': null,
  'auth-type': 'legacy',

  before: null,
  'bin-links': true,
  browser: null,

  ca: null,
  cafile: null,

  cache: '~/.npm',

  'cache-lock-stale': 60000,
  'cache-lock-retries': 10,
  'cache-lock-wait': 10000,

  'cache-max': Infinity,
  'cache-min': 10,

  cert: null,

  cidr: null,

  color: true,
  call: '',
  depth: 0,
  description: true,
  dev: false,
  'dry-run': false,
  editor: 'vim',
  'engine-strict': false,
  force: false,
  'format-package-lock': true,

  fund: true,

  'fetch-retries': 2,
  'fetch-retry-factor': 10,
  'fetch-retry-mintimeout': 10000,
  'fetch-retry-maxtimeout': 60000,

  git: 'git',
  'git-tag-version': true,
  'commit-hooks': true,

  global: false,
  'global-style': false,
  heading: 'npm',
  'if-present': false,
  include: [],
  'include-staged': false,
  'ignore-prepublish': false,
  'ignore-scripts': false,
  'init-module': '~/.npm-init.js',
  'init-author-name': '',
  'init-author-email': '',
  'init-author-url': '',
  'init-version': '1.0.0',
  'init-license': 'ISC',
  json: false,
  key: null,
  'legacy-bundling': false,
  'legacy-peer-deps': false,
  link: false,
  'local-address': undefined,
  loglevel: 'notice',
  'logs-max': 10,
  long: false,
  maxsockets: 50,
  message: '%s',
  'metrics-registry': null,
  'node-options': null,
  'node-version': process.version,
  offline: false,
  omit: [],
  only: null,
  optional: true,
  otp: null,
  package: [],
  'package-lock': true,
  'package-lock-only': false,
  parseable: false,
  'prefer-offline': false,
  'prefer-online': false,
  preid: '',
  production: true,
  progress: true,
  proxy: null,
  'https-proxy': null,
  noproxy: null,
  'user-agent': 'npm/{npm-version} ' +
                'node/{node-version} ' +
                '{platform} ' +
                '{arch} ' +
                '{ci}',
  'read-only': false,
  'rebuild-bundle': true,
  registry: 'https://registry.npmjs.org/',
  rollback: true,
  save: true,
  'save-bundle': false,
  'save-dev': false,
  'save-exact': false,
  'save-optional': false,
  'save-prefix': '^',
  'save-prod': false,
  scope: '',
  'script-shell': null,
  'scripts-prepend-node-path': 'warn-only',
  searchopts: '',
  searchexclude: null,
  searchlimit: 20,
  searchstaleness: 15 * 60,
  'send-metrics': false,
  shell: '/bin/sh',
  shrinkwrap: true,
  'sign-git-commit': false,
  'sign-git-tag': false,
  'sso-poll-frequency': 500,
  'sso-type': 'oauth',
  'strict-ssl': true,
  tag: 'latest',
  'tag-version-prefix': 'v',
  timing: false,
  unicode: /UTF-?8$/i.test(
    process.env.LC_ALL || process.env.LC_CTYPE || process.env.LANG
  ),
  'update-notifier': true,
  usage: false,
  userconfig: '~/.npmrc',
  umask: 0o22,
  version: false,
  versions: false,
  viewer: 'man',
}
