/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/type-description.js TAP > must match snapshot 1`] = `
Object {
  "_exit": Array [
    "boolean value (true or false)",
  ],
  "access": Array [
    null,
    "restricted",
    "public",
  ],
  "all": Array [
    "boolean value (true or false)",
  ],
  "allow-same-version": Array [
    "boolean value (true or false)",
  ],
  "also": Array [
    null,
    "dev",
    "development",
  ],
  "always-auth": Array [
    "boolean value (true or false)",
  ],
  "audit": Array [
    "boolean value (true or false)",
  ],
  "audit-level": Array [
    "low",
    "moderate",
    "high",
    "critical",
    "none",
    null,
  ],
  "auth-type": Array [
    "legacy",
    "sso",
    "saml",
    "oauth",
  ],
  "before": Array [
    null,
    "valid Date string",
  ],
  "bin-links": Array [
    "boolean value (true or false)",
  ],
  "browser": Array [
    null,
    "boolean value (true or false)",
    Function String(),
  ],
  "ca": Array [
    null,
    Function String(),
    Function Array(),
  ],
  "cache": Array [
    "valid filesystem path",
  ],
  "cache-lock-retries": Array [
    "numeric value",
  ],
  "cache-lock-stale": Array [
    "numeric value",
  ],
  "cache-lock-wait": Array [
    "numeric value",
  ],
  "cache-max": Array [
    "numeric value",
  ],
  "cache-min": Array [
    "numeric value",
  ],
  "cafile": Array [
    "valid filesystem path",
  ],
  "call": Array [
    Function String(),
  ],
  "cert": Array [
    null,
    Function String(),
  ],
  "cidr": Array [
    null,
    Function String(),
    Function Array(),
  ],
  "color": Array [
    "always",
    "boolean value (true or false)",
  ],
  "commit-hooks": Array [
    "boolean value (true or false)",
  ],
  "depth": Array [
    "numeric value",
  ],
  "description": Array [
    "boolean value (true or false)",
  ],
  "dev": Array [
    "boolean value (true or false)",
  ],
  "dry-run": Array [
    "boolean value (true or false)",
  ],
  "editor": Array [
    Function String(),
  ],
  "engine-strict": Array [
    "boolean value (true or false)",
  ],
  "fetch-retries": Array [
    "numeric value",
  ],
  "fetch-retry-factor": Array [
    "numeric value",
  ],
  "fetch-retry-maxtimeout": Array [
    "numeric value",
  ],
  "fetch-retry-mintimeout": Array [
    "numeric value",
  ],
  "force": Array [
    "boolean value (true or false)",
  ],
  "format-package-lock": Array [
    "boolean value (true or false)",
  ],
  "fund": Array [
    "boolean value (true or false)",
  ],
  "git": Array [
    Function String(),
  ],
  "git-tag-version": Array [
    "boolean value (true or false)",
  ],
  "global": Array [
    "boolean value (true or false)",
  ],
  "global-style": Array [
    "boolean value (true or false)",
  ],
  "globalconfig": Array [
    "valid filesystem path",
  ],
  "heading": Array [
    Function String(),
  ],
  "https-proxy": Array [
    null,
    "full url with \\"http://\\"",
  ],
  "if-present": Array [
    "boolean value (true or false)",
  ],
  "ignore-prepublish": Array [
    "boolean value (true or false)",
  ],
  "ignore-scripts": Array [
    "boolean value (true or false)",
  ],
  "include": Array [
    Function Array(),
    "prod",
    "dev",
    "optional",
    "peer",
  ],
  "include-staged": Array [
    "boolean value (true or false)",
  ],
  "init-author-email": Array [
    Function String(),
  ],
  "init-author-name": Array [
    Function String(),
  ],
  "init-author-url": Array [
    "",
    "full url with \\"http://\\"",
  ],
  "init-license": Array [
    Function String(),
  ],
  "init-module": Array [
    "valid filesystem path",
  ],
  "init-version": Array [
    "full valid SemVer string",
  ],
  "json": Array [
    "boolean value (true or false)",
  ],
  "key": Array [
    null,
    Function String(),
  ],
  "legacy-bundling": Array [
    "boolean value (true or false)",
  ],
  "legacy-peer-deps": Array [
    "boolean value (true or false)",
  ],
  "link": Array [
    "boolean value (true or false)",
  ],
  "loglevel": Array [
    "silent",
    "error",
    "warn",
    "notice",
    "http",
    "timing",
    "info",
    "verbose",
    "silly",
  ],
  "logs-max": Array [
    "numeric value",
  ],
  "long": Array [
    "boolean value (true or false)",
  ],
  "maxsockets": Array [
    "numeric value",
  ],
  "message": Array [
    Function String(),
  ],
  "metrics-registry": Array [
    null,
    Function String(),
  ],
  "multiple-numbers": Array [
    Function Array(),
    "numeric value",
  ],
  "node-options": Array [
    null,
    Function String(),
  ],
  "node-version": Array [
    null,
    "full valid SemVer string",
  ],
  "noproxy": Array [
    null,
    Function String(),
    Function Array(),
  ],
  "offline": Array [
    "boolean value (true or false)",
  ],
  "omit": Array [
    Function Array(),
    "dev",
    "optional",
    "peer",
  ],
  "only": Array [
    null,
    "dev",
    "development",
    "prod",
    "production",
  ],
  "optional": Array [
    "boolean value (true or false)",
  ],
  "otp": Array [
    null,
    Function String(),
  ],
  "package": Array [
    Function String(),
    Function Array(),
  ],
  "package-lock": Array [
    "boolean value (true or false)",
  ],
  "package-lock-only": Array [
    "boolean value (true or false)",
  ],
  "parseable": Array [
    "boolean value (true or false)",
  ],
  "prefer-offline": Array [
    "boolean value (true or false)",
  ],
  "prefer-online": Array [
    "boolean value (true or false)",
  ],
  "prefix": Array [
    "valid filesystem path",
  ],
  "preid": Array [
    Function String(),
  ],
  "production": Array [
    "boolean value (true or false)",
  ],
  "progress": Array [
    "boolean value (true or false)",
  ],
  "proxy": Array [
    null,
    false,
    "full url with \\"http://\\"",
  ],
  "read-only": Array [
    "boolean value (true or false)",
  ],
  "rebuild-bundle": Array [
    "boolean value (true or false)",
  ],
  "registry": Array [
    null,
    "full url with \\"http://\\"",
  ],
  "rollback": Array [
    "boolean value (true or false)",
  ],
  "save": Array [
    "boolean value (true or false)",
  ],
  "save-bundle": Array [
    "boolean value (true or false)",
  ],
  "save-dev": Array [
    "boolean value (true or false)",
  ],
  "save-exact": Array [
    "boolean value (true or false)",
  ],
  "save-optional": Array [
    "boolean value (true or false)",
  ],
  "save-prefix": Array [
    Function String(),
  ],
  "save-prod": Array [
    "boolean value (true or false)",
  ],
  "scope": Array [
    Function String(),
  ],
  "script-shell": Array [
    null,
    Function String(),
  ],
  "scripts-prepend-node-path": Array [
    "boolean value (true or false)",
    "auto",
    "warn-only",
  ],
  "searchexclude": Array [
    null,
    Function String(),
  ],
  "searchlimit": Array [
    "numeric value",
  ],
  "searchopts": Array [
    Function String(),
  ],
  "searchstaleness": Array [
    "numeric value",
  ],
  "send-metrics": Array [
    "boolean value (true or false)",
  ],
  "shell": Array [
    Function String(),
  ],
  "shrinkwrap": Array [
    "boolean value (true or false)",
  ],
  "sign-git-commit": Array [
    "boolean value (true or false)",
  ],
  "sign-git-tag": Array [
    "boolean value (true or false)",
  ],
  "sso-poll-frequency": Array [
    "numeric value",
  ],
  "sso-type": Array [
    null,
    "oauth",
    "saml",
  ],
  "strict-ssl": Array [
    "boolean value (true or false)",
  ],
  "tag": Array [
    Function String(),
  ],
  "tag-version-prefix": Array [
    Function String(),
  ],
  "timing": Array [
    "boolean value (true or false)",
  ],
  "tmp": Array [
    "valid filesystem path",
  ],
  "umask": Array [
    "octal number in range 0o000..0o777 (0..511)",
  ],
  "unicode": Array [
    "boolean value (true or false)",
  ],
  "update-notifier": Array [
    "boolean value (true or false)",
  ],
  "usage": Array [
    "boolean value (true or false)",
  ],
  "user-agent": Array [
    Function String(),
  ],
  "userconfig": Array [
    "valid filesystem path",
  ],
  "version": Array [
    "boolean value (true or false)",
  ],
  "versions": Array [
    "boolean value (true or false)",
  ],
  "viewer": Array [
    Function String(),
  ],
}
`
