/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/run-script.js TAP colorized interactive mode msg > should print colorized output 1`] = `
[0m[0m
[0mEntering npm script environment[0m[0m at location:[0m
[0m[2m/foo/[22m[0m[1m[22m
[1mType 'exit' or ^D when finished[22m
[1m[22m
`

exports[`test/run-script.js TAP no color interactive mode msg > should print non-colorized output 1`] = `

Entering npm script environment at location:
/foo/
Type 'exit' or ^D when finished

`
