// generated from test/fixtures/testing-bundledeps-2
module.exports = t => {
  const path = t.testdir({
  "index.js": "",
  "lib": {
    "foo": ""
  },
  "package.json": JSON.stringify({
    "name": "@isaacs/testing-bundledeps",
    "version": "1.0.0",
    "dependencies": {
      "@isaacs/testing-bundledeps-a": "*",
      "@isaacs/testing-bundledeps-c": "*"
    },
    "bundleDependencies": [
      "@isaacs/testing-bundledeps-a"
    ],
    "files": [
      "lib/foo"
    ]
  })
})
  return path
}
