// generated from test/fixtures/tap-react15-collision
module.exports = t => {
  const path = t.testdir({
  "package.json": JSON.stringify({
    "name": "tap-14.7-react-15-peerdep-collision",
    "version": "1.2.3",
    "dependencies": {
      "react": "15"
    },
    "devDependencies": {
      "tap": "unbundled"
    }
  })
})
  return path
}
