/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/index.js TAP credentials management def_auth > default registry 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management def_auth > default registry after set 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management def_auth > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management def_passNoUser > default registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management def_passNoUser > other registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management def_userNoPass > default registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management def_userNoPass > other registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management def_userpass > default registry 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "email": "i@izs.me",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management def_userpass > default registry after set 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "email": "i@izs.me",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management def_userpass > other registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management nerfed_auth > default registry 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_auth > default registry after set 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_auth > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management nerfed_authToken > default registry 1`] = `
Object {
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management nerfed_authToken > default registry after set 1`] = `
Object {
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management nerfed_authToken > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management nerfed_mtls > default registry 1`] = `
Object {
  "certfile": "/path/to/cert",
  "keyfile": "/path/to/key",
}
`

exports[`test/index.js TAP credentials management nerfed_mtls > default registry after set 1`] = `
Object {
  "certfile": "/path/to/cert",
  "keyfile": "/path/to/key",
}
`

exports[`test/index.js TAP credentials management nerfed_mtls > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management nerfed_mtlsAuthToken > default registry 1`] = `
Object {
  "certfile": "/path/to/cert",
  "keyfile": "/path/to/key",
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management nerfed_mtlsAuthToken > default registry after set 1`] = `
Object {
  "certfile": "/path/to/cert",
  "keyfile": "/path/to/key",
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management nerfed_mtlsAuthToken > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management nerfed_mtlsUserPass > default registry 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "certfile": "/path/to/cert",
  "email": "i@izs.me",
  "keyfile": "/path/to/key",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_mtlsUserPass > default registry after set 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "certfile": "/path/to/cert",
  "email": "i@izs.me",
  "keyfile": "/path/to/key",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_mtlsUserPass > other registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management nerfed_userpass > default registry 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "email": "i@izs.me",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_userpass > default registry after set 1`] = `
Object {
  "auth": "aGVsbG86d29ybGQ=",
  "email": "i@izs.me",
  "password": "world",
  "username": "hello",
}
`

exports[`test/index.js TAP credentials management nerfed_userpass > other registry 1`] = `
Object {
  "email": "i@izs.me",
}
`

exports[`test/index.js TAP credentials management none_authToken > default registry 1`] = `
Object {
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management none_authToken > default registry after set 1`] = `
Object {
  "token": "0bad1de4",
}
`

exports[`test/index.js TAP credentials management none_authToken > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_emptyConfig > default registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_emptyConfig > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_lcAuthToken > default registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_lcAuthToken > other registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_noConfig > default registry 1`] = `
Object {}
`

exports[`test/index.js TAP credentials management none_noConfig > other registry 1`] = `
Object {}
`
