/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/lib/utils/reify-finish.js TAP should write if everything above passes > written config 1`] = `
hasBuiltinConfig=true
x=y

[nested]
foo=bar

`
