/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/link.js TAP > instantiate with target specified 1`] = `
Link {
  "dev": true,
  "devOptional": true,
  "dummy": false,
  "edgesIn": Set {},
  "edgesOut": Map {},
  "errors": Array [],
  "extraneous": true,
  "fsChildren": Set {},
  "hasShrinkwrap": false,
  "installLinks": false,
  "integrity": null,
  "inventory": Inventory {},
  "legacyPeerDeps": false,
  "linksIn": Set {},
  "location": "../../../../../some/other/path",
  "name": "path",
  "optional": true,
  "path": "/home/user/some/other/path",
  "peer": true,
  "queryContext": Object {},
  "realpath": "/home/user/projects/some/kind/of/path",
  "sourceReference": null,
  "tops": Set {},
}
`

exports[`test/link.js TAP > instantiate without providing target 1`] = `
&ref_1 Link {
  "dev": true,
  "devOptional": true,
  "dummy": false,
  "edgesIn": Set {},
  "edgesOut": Map {},
  "errors": Array [],
  "extraneous": true,
  "fsChildren": Set {},
  "hasShrinkwrap": false,
  "installLinks": false,
  "integrity": null,
  "inventory": Inventory {
    "../../../../../some/other/path" => <*ref_1>,
    "" => Node {
      "children": Map {},
      "dev": true,
      "devOptional": true,
      "dummy": false,
      "edgesIn": Set {},
      "edgesOut": Map {},
      "errors": Array [],
      "extraneous": true,
      "fsChildren": Set {},
      "hasShrinkwrap": false,
      "installLinks": false,
      "integrity": null,
      "inventory": Inventory {},
      "legacyPeerDeps": false,
      "linksIn": Set {
        <*ref_1>,
      },
      "location": "",
      "name": "path",
      "optional": true,
      "path": "/home/user/projects/some/kind/of/path",
      "peer": true,
      "queryContext": Object {},
      "realpath": "/home/user/projects/some/kind/of/path",
      "resolved": null,
      "sourceReference": null,
      "tops": Set {},
    },
  },
  "legacyPeerDeps": false,
  "linksIn": Set {},
  "location": "../../../../../some/other/path",
  "name": "path",
  "optional": true,
  "path": "/home/user/some/other/path",
  "peer": true,
  "queryContext": Object {},
  "realpath": "/home/user/projects/some/kind/of/path",
  "sourceReference": null,
  "tops": Set {
    Node {
      "children": Map {},
      "dev": true,
      "devOptional": true,
      "dummy": false,
      "edgesIn": Set {},
      "edgesOut": Map {},
      "errors": Array [],
      "extraneous": true,
      "fsChildren": Set {},
      "hasShrinkwrap": false,
      "installLinks": false,
      "integrity": null,
      "inventory": Inventory {},
      "legacyPeerDeps": false,
      "linksIn": Set {
        <*ref_1>,
      },
      "location": "",
      "name": "path",
      "optional": true,
      "path": "/home/user/projects/some/kind/of/path",
      "peer": true,
      "queryContext": Object {},
      "realpath": "/home/user/projects/some/kind/of/path",
      "resolved": null,
      "sourceReference": null,
      "tops": Set {},
    },
  },
}
`
