/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/read-json.js TAP do not strip or mutate anything > crlf-tabs.json 1`] = `
Object {
  "name": "curly leaflets tabula rasa",
  "version": "9",
}
`

exports[`test/read-json.js TAP do not strip or mutate anything > package.json 1`] = `
Object {
  "_someField": "someValue",
  "bin": "../../../../../../etc/passwd",
  "name": "foo",
}
`

exports[`test/read-json.js TAP do not strip or mutate anything > space-tabs.json 1`] = `
Object {
  "name": "spacetabular",
  "version": "9000.0.1",
}
`
