/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/peer-entry-sets.js TAP basic test > b 1`] = `
Map {
  Array [
    "",
    "prod",
    "b",
    "",
  ] => Array [
    Array [
      "node_modules/b",
      "1.0.0",
    ],
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > c 1`] = `
Map {
  Array [
    "",
    "prod",
    "c",
    "",
  ] => Array [
    Array [
      "node_modules/c",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/h",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > d 1`] = `
Map {
  Array [
    "",
    "prod",
    "d",
    "",
  ] => Array [
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "b",
    "",
  ] => Array [
    Array [
      "node_modules/b",
      "1.0.0",
    ],
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > e 1`] = `
Map {
  Array [
    "",
    "prod",
    "d",
    "",
  ] => Array [
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "b",
    "",
  ] => Array [
    Array [
      "node_modules/b",
      "1.0.0",
    ],
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > f 1`] = `
Map {
  Array [
    "",
    "prod",
    "c",
    "",
  ] => Array [
    Array [
      "node_modules/c",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/h",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "d",
    "",
  ] => Array [
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "b",
    "",
  ] => Array [
    Array [
      "node_modules/b",
      "1.0.0",
    ],
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > g 1`] = `
Map {
  Array [
    "",
    "prod",
    "c",
    "",
  ] => Array [
    Array [
      "node_modules/c",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/h",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "d",
    "",
  ] => Array [
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
  Array [
    "",
    "prod",
    "b",
    "",
  ] => Array [
    Array [
      "node_modules/b",
      "1.0.0",
    ],
    Array [
      "node_modules/d",
      "1.0.0",
    ],
    Array [
      "node_modules/e",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`

exports[`test/peer-entry-sets.js TAP basic test > h 1`] = `
Map {
  Array [
    "",
    "prod",
    "c",
    "",
  ] => Array [
    Array [
      "node_modules/c",
      "1.0.0",
    ],
    Array [
      "node_modules/f",
      "1.0.0",
    ],
    Array [
      "node_modules/h",
      "1.0.0",
    ],
    Array [
      "node_modules/g",
      "1.0.0",
    ],
  ],
}
`
