/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/lib/utils/update-notifier.js TAP notification situations major to current > color 1`] = `

New [31mmajor[39m version of npm available! [31m122.420.69[39m -> [32m123.420.69[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v123.420.69[39m
Run [32mnpm install -g npm@123.420.69[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations major to current > no color 1`] = `

New major version of npm available! 122.420.69 -> 123.420.69
Changelog: <https://github.com/npm/cli/releases/tag/v123.420.69>
Run \`npm install -g npm@123.420.69\` to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations minor to current > color 1`] = `

New [33mminor[39m version of npm available! [31m123.419.69[39m -> [32m123.420.69[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v123.420.69[39m
Run [32mnpm install -g npm@123.420.69[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations minor to current > no color 1`] = `

New minor version of npm available! 123.419.69 -> 123.420.69
Changelog: <https://github.com/npm/cli/releases/tag/v123.420.69>
Run \`npm install -g npm@123.420.69\` to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations minor to next version > color 1`] = `

New [33mminor[39m version of npm available! [31m123.420.70[39m -> [32m123.421.70[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v123.421.70[39m
Run [32mnpm install -g npm@123.421.70[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations minor to next version > no color 1`] = `

New minor version of npm available! 123.420.70 -> 123.421.70
Changelog: <https://github.com/npm/cli/releases/tag/v123.421.70>
Run \`npm install -g npm@123.421.70\` to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations new beta available > color 1`] = `

New [32mprerelease[39m version of npm available! [31m124.0.0-beta.0[39m -> [32m124.0.0-beta.99999[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v124.0.0-beta.99999[39m
Run [32mnpm install -g npm@124.0.0-beta.99999[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations new beta available > no color 1`] = `

New prerelease version of npm available! 124.0.0-beta.0 -> 124.0.0-beta.99999
Changelog: <https://github.com/npm/cli/releases/tag/v124.0.0-beta.99999>
Run \`npm install -g npm@124.0.0-beta.99999\` to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations patch to current > color 1`] = `

New [32mpatch[39m version of npm available! [31m123.420.68[39m -> [32m123.420.69[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v123.420.69[39m
Run [32mnpm install -g npm@123.420.69[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations patch to current > no color 1`] = `

New patch version of npm available! 123.420.68 -> 123.420.69
Changelog: <https://github.com/npm/cli/releases/tag/v123.420.69>
Run \`npm install -g npm@123.420.69\` to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations patch to next version > color 1`] = `

New [32mpatch[39m version of npm available! [31m123.421.69[39m -> [32m123.421.70[39m
Changelog: [36mhttps://github.com/npm/cli/releases/tag/v123.421.70[39m
Run [32mnpm install -g npm@123.421.70[39m to update!

`

exports[`test/lib/utils/update-notifier.js TAP notification situations patch to next version > no color 1`] = `

New patch version of npm available! 123.421.69 -> 123.421.70
Changelog: <https://github.com/npm/cli/releases/tag/v123.421.70>
Run \`npm install -g npm@123.421.70\` to update!

`
