// generated from test/fixtures/flow-outdated
module.exports = t => {
  const path = t.testdir({
  "package-lock.json": JSON.stringify({
    "name": "flow-outdated",
    "version": "1.0.0",
    "lockfileVersion": 2,
    "requires": true,
    "packages": {
      "": {
        "name": "flow-outdated",
        "version": "1.0.0",
        "license": "ISC",
        "dependencies": {
          "flow-parser": "0.x",
          "flow-remove-types": "^2.112.0"
        }
      },
      "node_modules/flow-parser": {
        "version": "0.112.0",
        "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.112.0.tgz",
        "integrity": "sha512-sxjnwhR76B/fUN6n/XerYzn8R1HvtVo3SM8Il3WiZ4nkAlb2BBzKe1TSVKGSyZgD6FW9Bsxom/57ktkqrqmXGA=="
      },
      "node_modules/flow-remove-types": {
        "version": "2.112.0",
        "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.112.0.tgz",
        "integrity": "sha512-h3bwcfh41nR9kvlhZFr5ySGmzzOyG4VUsnN4OBl9R6anbWAiX4H5lPhKTwZ7AelWF8Rtqmw/Vnq+VLEMg7PdAw==",
        "dependencies": {
          "flow-parser": "^0.112.0",
          "pirates": "^3.0.2",
          "vlq": "^0.2.1"
        }
      },
      "node_modules/node-modules-regexp": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
        "integrity": "sha1-jZ2+KJZKSsVxLpExZCEHxx6Q7EA="
      },
      "node_modules/pirates": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
        "integrity": "sha512-c5CgUJq6H2k6MJz72Ak1F5sN9n9wlSlJyEnwvpm9/y3WB4E3pHBDT2c6PEiS1vyJvq2bUxUAIu0EGf8Cx4Ic7Q==",
        "dependencies": {
          "node-modules-regexp": "^1.0.0"
        }
      },
      "node_modules/vlq": {
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.1.tgz",
        "integrity": "sha512-nHsLkUCZjkeZgTWmMA1htGsht29a8Qdf+o2LhJlMN3cIY24DNpHapLWnUTJ3rpZ3eD3240yiS6nRya/km8L7lQ=="
      }
    },
    "dependencies": {
      "flow-parser": {
        "version": "0.112.0",
        "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.112.0.tgz",
        "integrity": "sha512-sxjnwhR76B/fUN6n/XerYzn8R1HvtVo3SM8Il3WiZ4nkAlb2BBzKe1TSVKGSyZgD6FW9Bsxom/57ktkqrqmXGA=="
      },
      "flow-remove-types": {
        "version": "2.112.0",
        "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.112.0.tgz",
        "integrity": "sha512-h3bwcfh41nR9kvlhZFr5ySGmzzOyG4VUsnN4OBl9R6anbWAiX4H5lPhKTwZ7AelWF8Rtqmw/Vnq+VLEMg7PdAw==",
        "requires": {
          "flow-parser": "^0.112.0",
          "pirates": "^3.0.2",
          "vlq": "^0.2.1"
        }
      },
      "node-modules-regexp": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
        "integrity": "sha1-jZ2+KJZKSsVxLpExZCEHxx6Q7EA="
      },
      "pirates": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
        "integrity": "sha512-c5CgUJq6H2k6MJz72Ak1F5sN9n9wlSlJyEnwvpm9/y3WB4E3pHBDT2c6PEiS1vyJvq2bUxUAIu0EGf8Cx4Ic7Q==",
        "requires": {
          "node-modules-regexp": "^1.0.0"
        }
      },
      "vlq": {
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.1.tgz",
        "integrity": "sha512-nHsLkUCZjkeZgTWmMA1htGsht29a8Qdf+o2LhJlMN3cIY24DNpHapLWnUTJ3rpZ3eD3240yiS6nRya/km8L7lQ=="
      }
    }
  }),
  "package.json": JSON.stringify({
    "name": "flow-outdated",
    "version": "1.0.0",
    "description": "",
    "main": "index.js",
    "dependencies": {
      "flow-parser": "0.x",
      "flow-remove-types": "^2.112.0"
    },
    "devDependencies": {},
    "scripts": {
      "test": "echo \"Error: no test specified\" && exit 1"
    },
    "keywords": [],
    "author": "",
    "license": "ISC"
  })
})
  return path
}
