/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/arborist/build-ideal-tree.js TAP a direct link dep has a dep with optional dependencies > should not mark children of the optional dep as extraneous 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "./a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep/node_modules/a",
      "realpath": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "graceful-fs",
          "spec": "~2",
          "type": "optional",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "optional": true,
      "path": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-2.0.3.tgz",
      "version": "2.0.3",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "rimraf",
          "spec": "2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "~2",
          "to": "node_modules/graceful-fs",
          "type": "optional",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.2.4.tgz",
      "version": "2.2.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "./a",
      "to": "node_modules/a",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "2.2.4",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "link-dep-has-dep-with-optional-dep",
  "path": "{CWD}/test/fixtures/link-dep-has-dep-with-optional-dep",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP a tree with an outdated dep, missing dep, no lockfile > should not update all 1`] = `
ArboristNode {
  "children": Map {
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/outdated-no-lockfile/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/outdated-no-lockfile/node_modules/once",
      "version": "1.3.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/outdated-no-lockfile/node_modules/wrappy",
      "version": "1.0.1",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "1",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "^1.3.3",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "outdated-no-lockfile",
  "path": "{CWD}/test/fixtures/outdated-no-lockfile",
}
`

exports[`test/arborist/build-ideal-tree.js TAP a workspace with a conflicted nested duplicated dep > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "bar" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "bar",
          "spec": "file:packages/bar",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/foo",
          "name": "bar",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bar",
      "name": "bar",
      "path": "{CWD}/test/fixtures/workspace4/node_modules/bar",
      "realpath": "{CWD}/test/fixtures/workspace4/packages/bar",
      "resolved": "file:../packages/bar",
      "target": ArboristNode {
        "location": "packages/bar",
      },
      "version": "1.2.3",
    },
    "foo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "foo",
          "spec": "file:packages/foo",
          "type": "prod",
        },
      },
      "location": "node_modules/foo",
      "name": "foo",
      "path": "{CWD}/test/fixtures/workspace4/node_modules/foo",
      "realpath": "{CWD}/test/fixtures/workspace4/packages/foo",
      "resolved": "file:../packages/foo",
      "target": ArboristNode {
        "location": "packages/foo",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/workspace4/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/bar",
          "name": "mkdirp",
          "spec": "0.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/workspace4/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
  },
  "edgesOut": Map {
    "bar" => EdgeOut {
      "name": "bar",
      "spec": "file:packages/bar",
      "to": "node_modules/bar",
      "type": "prod",
    },
    "foo" => EdgeOut {
      "name": "foo",
      "spec": "file:packages/foo",
      "to": "node_modules/foo",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "0.x",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "packages/bar",
      "name": "bar",
      "path": "{CWD}/test/fixtures/workspace4/packages/bar",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/foo",
              "name": "mkdirp",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "packages/foo/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/fixtures/workspace4/packages/foo/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesOut": Map {
        "bar" => EdgeOut {
          "name": "bar",
          "spec": "^1.0.0",
          "to": "node_modules/bar",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "1",
          "to": "packages/foo/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "packages/foo",
      "name": "foo",
      "path": "{CWD}/test/fixtures/workspace4/packages/foo",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspace4",
  "path": "{CWD}/test/fixtures/workspace4",
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps and include workspace-root add mkdirp 0.5.0 to b > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "node_modules/mkdirp/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/mkdirp/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/b",
          "name": "mkdirp",
          "spec": "0.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/mkdirp/node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
      "version": "0.5.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "0.5.0",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps and include workspace-root no args > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-and-include-workspace-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps to workspaces KEEP in the root, prune out unnecessary dupe > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "mkdirp",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces-KEEP-in-the-root-prune-out-unnecessary-dupe",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps to workspaces add mkdirp 0.5.0 to b > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b/node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b",
              "name": "mkdirp",
              "spec": "0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "0.0.8",
              "to": "packages/b/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
          "version": "0.5.0",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "0.5.0",
          "to": "packages/b/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps to workspaces no args > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps to workspaces remove mkdirp from a > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "minimist",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "mkdirp",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "minimist" => EdgeOut {
      "name": "minimist",
      "spec": "1",
      "to": "node_modules/minimist",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-deps-to-workspaces",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add one workspace to another > tree with workspace a added to workspace c 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
          "type": "workspace",
        },
        EdgeIn {
          "from": "packages/c",
          "name": "a",
          "spec": "file:../a",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/b",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/c",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/c",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "1.0.0",
      "to": "node_modules/wrappy",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "a" => EdgeOut {
          "name": "a",
          "spec": "file:../a",
          "to": "node_modules/a",
          "type": "prod",
        },
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-not-root",
  "path": "{CWD}/test/fixtures/workspaces-not-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add packages to workspaces, not root > tree with abbrev removed from a and b 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/c",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/c",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "1.0.0",
      "to": "node_modules/wrappy",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-not-root",
  "path": "{CWD}/test/fixtures/workspaces-not-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add packages to workspaces, not root > tree with wrappy added to a and c 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/b",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/c",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/c",
      "realpath": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/workspaces-not-root/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "1.0.0",
      "to": "node_modules/wrappy",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "wrappy",
              "spec": "1.0.1",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/fixtures/workspaces-not-root/packages/a/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.1",
          "to": "packages/a/node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/b",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/c",
              "name": "wrappy",
              "spec": "1.0.1",
              "type": "prod",
            },
          },
          "location": "packages/c/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/fixtures/workspaces-not-root/packages/c/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.1",
          "to": "packages/c/node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-not-root/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-not-root",
  "path": "{CWD}/test/fixtures/workspaces-not-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP add symlink that points to a symlink > should follow symlinks to find final realpath destination 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:../linked-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-symlink-that-points-to-a-symlink/my-project/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-symlink-that-points-to-a-symlink/linked-pkg",
      "resolved": "file:../../linked-pkg",
      "target": ArboristNode {
        "location": "../linked-pkg",
        "name": "a",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-symlink-that-points-to-a-symlink/linked-pkg",
        "version": "1.0.0",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:../linked-pkg",
      "to": "node_modules/a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "my-project",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-add-symlink-that-points-to-a-symlink/my-project",
}
`

exports[`test/arborist/build-ideal-tree.js TAP adding existing dep with updateable version in package.json > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "lodash",
          "spec": "^3.9.1",
          "type": "dev",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-existing-dep-with-updateable-version-in-package.json/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-3.10.1.tgz",
      "version": "3.10.1",
    },
  },
  "edgesOut": Map {
    "lodash" => EdgeOut {
      "name": "lodash",
      "spec": "^3.9.1",
      "to": "node_modules/lodash",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-adding-existing-dep-with-updateable-version-in-package.json",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-existing-dep-with-updateable-version-in-package.json",
}
`

exports[`test/arborist/build-ideal-tree.js TAP adding tarball to global prefix that is a symlink at a different path depth > must match snapshot 1`] = `
ArboristLink {
  "isProjectRoot": true,
  "location": "../another-path/global-root",
  "name": "global-root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-tarball-to-global-prefix-that-is-a-symlink-at-a-different-path-depth/another-path/global-root",
  "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-tarball-to-global-prefix-that-is-a-symlink-at-a-different-path-depth/real-root",
  "resolved": "file:../real-root",
  "target": ArboristNode {
    "children": Map {
      "mkdirp" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "mkdirp",
            "spec": "file:../../../fixtures/registry-mocks/content/mkdirp/-/mkdirp-1.0.2.tgz",
            "type": "prod",
          },
        },
        "location": "node_modules/mkdirp",
        "name": "mkdirp",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-tarball-to-global-prefix-that-is-a-symlink-at-a-different-path-depth/real-root/node_modules/mkdirp",
        "resolved": "file:{CWD}/test/fixtures/registry-mocks/content/mkdirp/-/mkdirp-1.0.2.tgz",
        "version": "1.0.2",
      },
    },
    "edgesOut": Map {
      "mkdirp" => EdgeOut {
        "name": "mkdirp",
        "spec": "file:../../../fixtures/registry-mocks/content/mkdirp/-/mkdirp-1.0.2.tgz",
        "to": "node_modules/mkdirp",
        "type": "prod",
      },
    },
    "isProjectRoot": true,
    "location": "",
    "name": "real-root",
    "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-adding-tarball-to-global-prefix-that-is-a-symlink-at-a-different-path-depth/real-root",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source both direct and peer of the same type dependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "prod",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-dependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-dependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source both direct and peer of the same type devDependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "dev",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "dev",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-devDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-devDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source both direct and peer of the same type optionalDependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "optional",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "optional",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-optionalDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-optionalDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source both direct and peer of the same type peerDependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-peerDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-peerDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "peer",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "peer",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-peerDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-both-direct-and-peer-of-the-same-type-peerDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type dependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "peer",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type dependencies > use the force 2`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "prod",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "peer",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-dependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type devDependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "devOptional": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "dev",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "peer",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type devDependencies > use the force 2`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "dev",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "peer",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-devDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type optionalDependencies > use the force 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "optional",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "peer",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow ERESOLVE to be forced when not in the source peer is peer, b is some other type optionalDependencies > use the force 2`] = `
ArboristNode {
  "children": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "spec": "*",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-b/-/conflicted-peer-optional-from-dev-dep-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
          "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
          "peerConflicted": true,
          "spec": "2",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies/node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/conflicted-peer-optional-from-dev-dep-peer/-/conflicted-peer-optional-from-dev-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/conflicted-peer-optional-from-dev-dep-b" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "spec": "*",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-b",
      "type": "optional",
    },
    "@isaacs/conflicted-peer-optional-from-dev-dep-peer" => EdgeOut {
      "name": "@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/conflicted-peer-optional-from-dev-dep-peer",
      "type": "peer",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies",
  "packageName": "@isaacs/conflicted-peer-optional-from-dev-dep",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-ERESOLVE-to-be-forced-when-not-in-the-source-peer-is-peer-b-is-some-other-type-optionalDependencies",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow a link dep to satisfy a peer dep > reified link avoids conflict 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "file:../v2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/v2",
      "resolved": "file:../../../v2",
      "target": ArboristNode {
        "location": "../v2",
        "name": "v2",
        "packageName": "@isaacs/testing-peer-dep-conflict-chain-v",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/v2",
        "version": "2.0.0",
      },
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-vv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-vv/-/testing-peer-dep-conflict-chain-vv-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "file:../v2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-vv" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "main",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow a link dep to satisfy a peer dep > unmet link avoids conflict 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "file:../v2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/v2",
      "resolved": "file:../../../v2",
      "target": ArboristNode {
        "location": "../v2",
        "name": "@isaacs/testing-peer-dep-conflict-chain-v",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/v2",
        "version": "2.0.0",
      },
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-vv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main/node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-vv/-/testing-peer-dep-conflict-chain-vv-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "file:../v2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-vv" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-vv",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-vv",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "main",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-a-link-dep-to-satisfy-a-peer-dep/main",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow updating when peer outside of explicit update set conflict, but resolves appropriately with --force > succeed if force applied 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-single-a/-/testing-peer-dep-conflict-chain-single-a-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-single-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force/node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-single-b/-/testing-peer-dep-conflict-chain-single-b-2.0.1.tgz",
      "version": "2.0.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force",
  "packageName": "x",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-conflict-but-resolves-appropriately-with---force",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP allow updating when peer outside of explicit update set valid, no force required > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-single-a/-/testing-peer-dep-conflict-chain-single-a-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-single-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required/node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-single-b/-/testing-peer-dep-conflict-chain-single-b-2.0.1.tgz",
      "version": "2.0.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-single-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-single-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-single-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required",
  "packageName": "x",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-allow-updating-when-peer-outside-of-explicit-update-set-valid-no-force-required",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP always prefer deduping peer deps > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@pmmmwh/react-refresh-webpack-plugin" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@pmmmwh/react-refresh-webpack-plugin",
          "spec": "0.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/webpack" => EdgeOut {
          "name": "@types/webpack",
          "spec": "4.x",
          "to": null,
          "type": "peerOptional",
        },
        "ansi-html" => EdgeOut {
          "name": "ansi-html",
          "spec": "^0.0.7",
          "to": "node_modules/ansi-html",
          "type": "prod",
        },
        "error-stack-parser" => EdgeOut {
          "name": "error-stack-parser",
          "spec": "^2.0.6",
          "to": "node_modules/error-stack-parser",
          "type": "prod",
        },
        "html-entities" => EdgeOut {
          "name": "html-entities",
          "spec": "^1.2.1",
          "to": "node_modules/html-entities",
          "type": "prod",
        },
        "native-url" => EdgeOut {
          "name": "native-url",
          "spec": "^0.2.6",
          "to": "node_modules/native-url",
          "type": "prod",
        },
        "react-refresh" => EdgeOut {
          "name": "react-refresh",
          "spec": "^0.8.3",
          "to": "node_modules/react-refresh",
          "type": "peer",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^2.6.5",
          "to": "node_modules/schema-utils",
          "type": "prod",
        },
        "sockjs-client" => EdgeOut {
          "name": "sockjs-client",
          "spec": "^1.4.0",
          "to": null,
          "type": "peerOptional",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.7.3",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "error": "INVALID",
          "name": "type-fest",
          "spec": "^0.13.1",
          "to": "node_modules/type-fest",
          "type": "peerOptional",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": ">=4.43.0 <6.0.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
        "webpack-dev-server" => EdgeOut {
          "name": "webpack-dev-server",
          "spec": "3.x",
          "to": null,
          "type": "peerOptional",
        },
        "webpack-hot-middleware" => EdgeOut {
          "name": "webpack-hot-middleware",
          "spec": "2.x",
          "to": null,
          "type": "peerOptional",
        },
        "webpack-plugin-serve" => EdgeOut {
          "name": "webpack-plugin-serve",
          "spec": "0.x || 1.x",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "name": "@pmmmwh/react-refresh-webpack-plugin",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "resolved": "https://registry.npmjs.org/@pmmmwh/react-refresh-webpack-plugin/-/react-refresh-webpack-plugin-0.4.2.tgz",
      "version": "0.4.2",
    },
    "@types/eslint" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/eslint-scope",
          "name": "@types/eslint",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/estree" => EdgeOut {
          "name": "@types/estree",
          "spec": "*",
          "to": "node_modules/@types/estree",
          "type": "prod",
        },
        "@types/json-schema" => EdgeOut {
          "name": "@types/json-schema",
          "spec": "*",
          "to": "node_modules/@types/json-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/eslint",
      "name": "@types/eslint",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/eslint",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@types/eslint/-/eslint-7.2.4.tgz",
      "version": "7.2.4",
    },
    "@types/eslint-scope" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@types/eslint-scope",
          "spec": "^3.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/eslint" => EdgeOut {
          "name": "@types/eslint",
          "spec": "*",
          "to": "node_modules/@types/eslint",
          "type": "prod",
        },
        "@types/estree" => EdgeOut {
          "name": "@types/estree",
          "spec": "*",
          "to": "node_modules/@types/estree",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/eslint-scope",
      "name": "@types/eslint-scope",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/eslint-scope",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@types/eslint-scope/-/eslint-scope-3.7.0.tgz",
      "version": "3.7.0",
    },
    "@types/estree" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/eslint",
          "name": "@types/estree",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/eslint-scope",
          "name": "@types/estree",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@types/estree",
          "spec": "^0.0.45",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/estree",
      "name": "@types/estree",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/estree",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@types/estree/-/estree-0.0.45.tgz",
      "version": "0.0.45",
    },
    "@types/json-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/eslint",
          "name": "@types/json-schema",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/json-schema",
      "name": "@types/json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/json-schema",
      "resolved": "https://registry.npmjs.org/@types/json-schema/-/json-schema-7.0.6.tgz",
      "version": "7.0.6",
    },
    "@types/node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jest-worker",
          "name": "@types/node",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/node",
      "name": "@types/node",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/node",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@types/node/-/node-14.11.8.tgz",
      "version": "14.11.8",
    },
    "@types/yoga-layout" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yoga-layout-prebuilt",
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/yoga-layout",
      "name": "@types/yoga-layout",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@types/yoga-layout",
      "resolved": "https://registry.npmjs.org/@types/yoga-layout/-/yoga-layout-1.9.2.tgz",
      "version": "1.9.2",
    },
    "@webassemblyjs/ast" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-module-context",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/helper-module-context" => EdgeOut {
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-module-context",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/wast-parser" => EdgeOut {
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/ast",
      "name": "@webassemblyjs/ast",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/ast",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/ast/-/ast-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/floating-point-hex-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/floating-point-hex-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/floating-point-hex-parser",
      "name": "@webassemblyjs/floating-point-hex-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/floating-point-hex-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/floating-point-hex-parser/-/floating-point-hex-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-api-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-api-error",
      "name": "@webassemblyjs/helper-api-error",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-api-error",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-api-error/-/helper-api-error-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-buffer",
      "name": "@webassemblyjs/helper-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-buffer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-buffer/-/helper-buffer-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-code-frame",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/wast-printer" => EdgeOut {
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-printer",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-code-frame",
      "name": "@webassemblyjs/helper-code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-code-frame",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-code-frame/-/helper-code-frame-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-fsm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-fsm",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-fsm",
      "name": "@webassemblyjs/helper-fsm",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-fsm",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-fsm/-/helper-fsm-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-module-context" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-module-context",
      "name": "@webassemblyjs/helper-module-context",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-module-context",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-module-context/-/helper-module-context-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-wasm-bytecode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-wasm-bytecode",
      "name": "@webassemblyjs/helper-wasm-bytecode",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-wasm-bytecode",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-wasm-bytecode/-/helper-wasm-bytecode-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-wasm-section" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-wasm-section",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-wasm-section",
      "name": "@webassemblyjs/helper-wasm-section",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/helper-wasm-section",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-wasm-section/-/helper-wasm-section-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/ieee754" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@xtuc/ieee754" => EdgeOut {
          "name": "@xtuc/ieee754",
          "spec": "^1.2.0",
          "to": "node_modules/@xtuc/ieee754",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/ieee754",
      "name": "@webassemblyjs/ieee754",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/ieee754",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/ieee754/-/ieee754-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/leb128" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/leb128",
      "name": "@webassemblyjs/leb128",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/leb128",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/leb128/-/leb128-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/utf8" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/utf8",
      "name": "@webassemblyjs/utf8",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/utf8",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/utf8/-/utf8-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-edit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/wasm-edit",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-section" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-section",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-section",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
        "@webassemblyjs/wasm-opt" => EdgeOut {
          "name": "@webassemblyjs/wasm-opt",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-opt",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
        "@webassemblyjs/wast-printer" => EdgeOut {
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-printer",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-edit",
      "name": "@webassemblyjs/wasm-edit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wasm-edit",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-edit/-/wasm-edit-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-gen" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/ieee754" => EdgeOut {
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ieee754",
          "type": "prod",
        },
        "@webassemblyjs/leb128" => EdgeOut {
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/leb128",
          "type": "prod",
        },
        "@webassemblyjs/utf8" => EdgeOut {
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/utf8",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-gen",
      "name": "@webassemblyjs/wasm-gen",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wasm-gen",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-gen/-/wasm-gen-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-opt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-opt",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-opt",
      "name": "@webassemblyjs/wasm-opt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wasm-opt",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-opt/-/wasm-opt-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-api-error" => EdgeOut {
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-api-error",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/ieee754" => EdgeOut {
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ieee754",
          "type": "prod",
        },
        "@webassemblyjs/leb128" => EdgeOut {
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/leb128",
          "type": "prod",
        },
        "@webassemblyjs/utf8" => EdgeOut {
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/utf8",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-parser",
      "name": "@webassemblyjs/wasm-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wasm-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-parser/-/wasm-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wast-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/floating-point-hex-parser" => EdgeOut {
          "name": "@webassemblyjs/floating-point-hex-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/floating-point-hex-parser",
          "type": "prod",
        },
        "@webassemblyjs/helper-api-error" => EdgeOut {
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-api-error",
          "type": "prod",
        },
        "@webassemblyjs/helper-code-frame" => EdgeOut {
          "name": "@webassemblyjs/helper-code-frame",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-code-frame",
          "type": "prod",
        },
        "@webassemblyjs/helper-fsm" => EdgeOut {
          "name": "@webassemblyjs/helper-fsm",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-fsm",
          "type": "prod",
        },
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wast-parser",
      "name": "@webassemblyjs/wast-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wast-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wast-parser/-/wast-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wast-printer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-code-frame",
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/wast-parser" => EdgeOut {
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-parser",
          "type": "prod",
        },
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wast-printer",
      "name": "@webassemblyjs/wast-printer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@webassemblyjs/wast-printer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wast-printer/-/wast-printer-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@xtuc/ieee754" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ieee754",
          "name": "@xtuc/ieee754",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@xtuc/ieee754",
      "name": "@xtuc/ieee754",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@xtuc/ieee754",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@xtuc/ieee754/-/ieee754-1.2.0.tgz",
      "version": "1.2.0",
    },
    "@xtuc/long" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/leb128",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@xtuc/long",
      "name": "@xtuc/long",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/@xtuc/long",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@xtuc/long/-/long-4.2.2.tgz",
      "version": "4.2.2",
    },
    "acorn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "acorn",
          "spec": "^8.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/acorn",
      "peer": true,
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-8.0.4.tgz",
      "version": "8.0.4",
    },
    "ajv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv-keywords",
          "name": "ajv",
          "spec": "^6.9.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.12.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.12.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.12.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ajv-keywords" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.9.1",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-keywords",
      "name": "ajv-keywords",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ajv-keywords",
      "resolved": "https://registry.npmjs.org/ajv-keywords/-/ajv-keywords-3.5.2.tgz",
      "version": "3.5.2",
    },
    "ansi-escapes" => ArboristNode {
      "children": Map {
        "type-fest" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-escapes",
              "name": "type-fest",
              "spec": "^0.11.0",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-escapes/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ansi-escapes/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.11.0.tgz",
          "version": "0.11.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.11.0",
          "to": "node_modules/ansi-escapes/node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.1.tgz",
      "version": "4.3.1",
    },
    "ansi-html" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "ansi-html",
          "spec": "^0.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-html",
      "name": "ansi-html",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ansi-html",
      "resolved": "https://registry.npmjs.org/ansi-html/-/ansi-html-0.0.7.tgz",
      "version": "0.0.7",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
      "version": "4.1.0",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^2.0.1",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
      "version": "4.3.0",
    },
    "astral-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "auto-bind" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "auto-bind",
          "spec": "4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/auto-bind",
      "name": "auto-bind",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/auto-bind",
      "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-4.0.0.tgz",
      "version": "4.0.0",
    },
    "browserslist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "browserslist",
          "spec": "^4.14.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caniuse-lite" => EdgeOut {
          "name": "caniuse-lite",
          "spec": "^1.0.30001135",
          "to": "node_modules/caniuse-lite",
          "type": "prod",
        },
        "electron-to-chromium" => EdgeOut {
          "name": "electron-to-chromium",
          "spec": "^1.3.571",
          "to": "node_modules/electron-to-chromium",
          "type": "prod",
        },
        "escalade" => EdgeOut {
          "name": "escalade",
          "spec": "^3.1.0",
          "to": "node_modules/escalade",
          "type": "prod",
        },
        "node-releases" => EdgeOut {
          "name": "node-releases",
          "spec": "^1.1.61",
          "to": "node_modules/node-releases",
          "type": "prod",
        },
      },
      "location": "node_modules/browserslist",
      "name": "browserslist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/browserslist",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserslist/-/browserslist-4.14.5.tgz",
      "version": "4.14.5",
    },
    "buffer-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/buffer-from",
      "peer": true,
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caniuse-lite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "caniuse-lite",
          "spec": "^1.0.30001135",
          "type": "prod",
        },
      },
      "location": "node_modules/caniuse-lite",
      "name": "caniuse-lite",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/caniuse-lite",
      "peer": true,
      "resolved": "https://registry.npmjs.org/caniuse-lite/-/caniuse-lite-1.0.30001148.tgz",
      "version": "1.0.30001148",
    },
    "chalk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "chalk",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-4.1.0.tgz",
      "version": "4.1.0",
    },
    "chrome-trace-event" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "chrome-trace-event",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.9.0",
          "to": "node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/chrome-trace-event",
      "name": "chrome-trace-event",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/chrome-trace-event",
      "peer": true,
      "resolved": "https://registry.npmjs.org/chrome-trace-event/-/chrome-trace-event-1.0.2.tgz",
      "version": "1.0.2",
    },
    "ci-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-boxes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-boxes",
          "spec": "^2.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-boxes",
      "name": "cli-boxes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/cli-boxes",
      "resolved": "https://registry.npmjs.org/cli-boxes/-/cli-boxes-2.2.1.tgz",
      "version": "2.2.1",
    },
    "cli-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-truncate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-truncate",
      "name": "cli-truncate",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/cli-truncate",
      "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-2.1.0.tgz",
      "version": "2.1.0",
    },
    "code-excerpt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "code-excerpt",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "convert-to-spaces" => EdgeOut {
          "name": "convert-to-spaces",
          "spec": "^1.0.1",
          "to": "node_modules/convert-to-spaces",
          "type": "prod",
        },
      },
      "location": "node_modules/code-excerpt",
      "name": "code-excerpt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/code-excerpt",
      "resolved": "https://registry.npmjs.org/code-excerpt/-/code-excerpt-3.0.0.tgz",
      "version": "3.0.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "~1.1.4",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
      "version": "2.0.1",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "~1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
      "version": "1.1.4",
    },
    "commander" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser",
          "name": "commander",
          "spec": "^2.20.0",
          "type": "prod",
        },
      },
      "location": "node_modules/commander",
      "name": "commander",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/commander",
      "peer": true,
      "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.3.tgz",
      "version": "2.20.3",
    },
    "convert-to-spaces" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/code-excerpt",
          "name": "convert-to-spaces",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-to-spaces",
      "name": "convert-to-spaces",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/convert-to-spaces",
      "resolved": "https://registry.npmjs.org/convert-to-spaces/-/convert-to-spaces-1.0.2.tgz",
      "version": "1.0.2",
    },
    "electron-to-chromium" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "electron-to-chromium",
          "spec": "^1.3.571",
          "type": "prod",
        },
      },
      "location": "node_modules/electron-to-chromium",
      "name": "electron-to-chromium",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/electron-to-chromium",
      "peer": true,
      "resolved": "https://registry.npmjs.org/electron-to-chromium/-/electron-to-chromium-1.3.582.tgz",
      "version": "1.3.582",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "enhanced-resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "enhanced-resolve",
          "spec": "^5.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.2.4",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "tapable" => EdgeOut {
          "name": "tapable",
          "spec": "^2.0.0",
          "to": "node_modules/tapable",
          "type": "prod",
        },
      },
      "location": "node_modules/enhanced-resolve",
      "name": "enhanced-resolve",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/enhanced-resolve",
      "peer": true,
      "resolved": "https://registry.npmjs.org/enhanced-resolve/-/enhanced-resolve-5.3.1.tgz",
      "version": "5.3.1",
    },
    "error-stack-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "error-stack-parser",
          "spec": "^2.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "stackframe" => EdgeOut {
          "name": "stackframe",
          "spec": "^1.1.1",
          "to": "node_modules/stackframe",
          "type": "prod",
        },
      },
      "location": "node_modules/error-stack-parser",
      "name": "error-stack-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/error-stack-parser",
      "resolved": "https://registry.npmjs.org/error-stack-parser/-/error-stack-parser-2.0.6.tgz",
      "version": "2.0.6",
    },
    "escalade" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "escalade",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/escalade",
      "name": "escalade",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/escalade",
      "peer": true,
      "resolved": "https://registry.npmjs.org/escalade/-/escalade-3.1.1.tgz",
      "version": "3.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/stack-utils",
          "name": "escape-string-regexp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "eslint-scope" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.3.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-scope",
      "name": "eslint-scope",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/eslint-scope",
      "peer": true,
      "resolved": "https://registry.npmjs.org/eslint-scope/-/eslint-scope-5.1.1.tgz",
      "version": "5.1.1",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/esrecurse/node_modules/estraverse",
          "peer": true,
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "esrecurse",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.2.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/esrecurse",
      "peer": true,
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "estraverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "estraverse",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/estraverse",
      "name": "estraverse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/estraverse",
      "peer": true,
      "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "events" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "events",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/events",
      "name": "events",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/events",
      "peer": true,
      "resolved": "https://registry.npmjs.org/events/-/events-3.2.0.tgz",
      "version": "3.2.0",
    },
    "fast-deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^5.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^4.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/find-up",
      "peer": true,
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-4.1.0.tgz",
      "version": "4.1.0",
    },
    "glob-to-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/watchpack",
          "name": "glob-to-regexp",
          "spec": "^0.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "glob-to-regexp",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-to-regexp",
      "name": "glob-to-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/glob-to-regexp",
      "peer": true,
      "resolved": "https://registry.npmjs.org/glob-to-regexp/-/glob-to-regexp-0.4.1.tgz",
      "version": "0.4.1",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enhanced-resolve",
          "name": "graceful-fs",
          "spec": "^4.2.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "graceful-fs",
          "spec": "^4.2.4",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/graceful-fs",
      "peer": true,
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
      "version": "4.0.0",
    },
    "html-entities" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "html-entities",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/html-entities",
      "name": "html-entities",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/html-entities",
      "resolved": "https://registry.npmjs.org/html-entities/-/html-entities-1.3.1.tgz",
      "version": "1.3.1",
    },
    "indent-string" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "indent-string",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/indent-string",
      "name": "indent-string",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/indent-string",
      "resolved": "https://registry.npmjs.org/indent-string/-/indent-string-4.0.0.tgz",
      "version": "4.0.0",
    },
    "ink" => ArboristNode {
      "children": Map {
        "react-reconciler" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink",
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.0.0",
              "to": "node_modules/react",
              "type": "peer",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/scheduler",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/react-reconciler",
          "name": "react-reconciler",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ink/node_modules/react-reconciler",
          "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.24.0.tgz",
          "version": "0.24.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "ink",
          "spec": "3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": ">=16.8.0",
          "to": null,
          "type": "peerOptional",
        },
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "auto-bind" => EdgeOut {
          "name": "auto-bind",
          "spec": "4.0.0",
          "to": "node_modules/auto-bind",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-boxes" => EdgeOut {
          "name": "cli-boxes",
          "spec": "^2.2.0",
          "to": "node_modules/cli-boxes",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-truncate" => EdgeOut {
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "to": "node_modules/cli-truncate",
          "type": "prod",
        },
        "code-excerpt" => EdgeOut {
          "name": "code-excerpt",
          "spec": "^3.0.0",
          "to": "node_modules/code-excerpt",
          "type": "prod",
        },
        "indent-string" => EdgeOut {
          "name": "indent-string",
          "spec": "^4.0.0",
          "to": "node_modules/indent-string",
          "type": "prod",
        },
        "is-ci" => EdgeOut {
          "name": "is-ci",
          "spec": "^2.0.0",
          "to": "node_modules/is-ci",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.20",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "patch-console" => EdgeOut {
          "name": "patch-console",
          "spec": "^1.0.0",
          "to": "node_modules/patch-console",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": ">=16.8.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "react-devtools-core" => EdgeOut {
          "name": "react-devtools-core",
          "spec": "^4.6.0",
          "to": "node_modules/react-devtools-core",
          "type": "prod",
        },
        "react-reconciler" => EdgeOut {
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "to": "node_modules/ink/node_modules/react-reconciler",
          "type": "prod",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^2.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "string-length" => EdgeOut {
          "name": "string-length",
          "spec": "^3.1.0",
          "to": "node_modules/string-length",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.12.0",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
        "widest-line" => EdgeOut {
          "name": "widest-line",
          "spec": "^3.1.0",
          "to": "node_modules/widest-line",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
        "ws" => EdgeOut {
          "name": "ws",
          "spec": "^7.2.5",
          "to": "node_modules/ws",
          "type": "prod",
        },
        "yoga-layout-prebuilt" => EdgeOut {
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.6",
          "to": "node_modules/yoga-layout-prebuilt",
          "type": "prod",
        },
      },
      "location": "node_modules/ink",
      "name": "ink",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ink",
      "resolved": "https://registry.npmjs.org/ink/-/ink-3.0.8.tgz",
      "version": "3.0.8",
    },
    "is-ci" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "jest-worker" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "jest-worker",
          "spec": "^26.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/node" => EdgeOut {
          "name": "@types/node",
          "spec": "*",
          "to": "node_modules/@types/node",
          "type": "prod",
        },
        "merge-stream" => EdgeOut {
          "name": "merge-stream",
          "spec": "^2.0.0",
          "to": "node_modules/merge-stream",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.0.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/jest-worker",
      "name": "jest-worker",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/jest-worker",
      "peer": true,
      "resolved": "https://registry.npmjs.org/jest-worker/-/jest-worker-26.6.2.tgz",
      "version": "26.6.2",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "json-parse-better-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "json-parse-better-errors",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/json-parse-better-errors",
      "peer": true,
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema-traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "loader-runner" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "loader-runner",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/loader-runner",
      "name": "loader-runner",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/loader-runner",
      "peer": true,
      "resolved": "https://registry.npmjs.org/loader-runner/-/loader-runner-4.1.0.tgz",
      "version": "4.1.0",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^4.1.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/locate-path",
      "peer": true,
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-5.0.0.tgz",
      "version": "5.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "lodash",
          "spec": "^4.17.20",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink/node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler/node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "merge-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jest-worker",
          "name": "merge-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-stream",
      "name": "merge-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/merge-stream",
      "peer": true,
      "resolved": "https://registry.npmjs.org/merge-stream/-/merge-stream-2.0.0.tgz",
      "version": "2.0.0",
    },
    "mime-db" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.44.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/mime-db",
      "peer": true,
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.44.0.tgz",
      "version": "1.44.0",
    },
    "mime-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "mime-types",
          "spec": "^2.1.27",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.44.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/mime-types",
      "peer": true,
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.27.tgz",
      "version": "2.1.27",
    },
    "mimic-fn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "native-url" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "native-url",
          "spec": "^0.2.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "querystring" => EdgeOut {
          "name": "querystring",
          "spec": "^0.2.0",
          "to": "node_modules/querystring",
          "type": "prod",
        },
      },
      "location": "node_modules/native-url",
      "name": "native-url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/native-url",
      "resolved": "https://registry.npmjs.org/native-url/-/native-url-0.2.6.tgz",
      "version": "0.2.6",
    },
    "neo-async" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "neo-async",
          "spec": "^2.6.2",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/neo-async",
      "peer": true,
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.2.tgz",
      "version": "2.6.2",
    },
    "node-releases" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "node-releases",
          "spec": "^1.1.61",
          "type": "prod",
        },
      },
      "location": "node_modules/node-releases",
      "name": "node-releases",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/node-releases",
      "peer": true,
      "resolved": "https://registry.npmjs.org/node-releases/-/node-releases-1.1.63.tgz",
      "version": "1.1.63",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink/node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler/node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "onetime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.2.tgz",
      "version": "5.1.2",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/p-limit",
      "peer": true,
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
      "version": "2.3.0",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.2.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/p-locate",
      "peer": true,
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-4.1.0.tgz",
      "version": "4.1.0",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/p-try",
      "peer": true,
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "patch-console" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "patch-console",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/patch-console",
      "name": "patch-console",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/patch-console",
      "resolved": "https://registry.npmjs.org/patch-console/-/patch-console-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "path-exists",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/path-exists",
      "peer": true,
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-4.0.0.tgz",
      "version": "4.0.0",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "pkg-dir",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/pkg-dir",
      "peer": true,
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-4.2.0.tgz",
      "version": "4.2.0",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink/node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "punycode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "querystring" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/native-url",
          "name": "querystring",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/querystring",
      "name": "querystring",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/querystring",
      "resolved": "https://registry.npmjs.org/querystring/-/querystring-0.2.0.tgz",
      "version": "0.2.0",
    },
    "randombytes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/serialize-javascript",
          "name": "randombytes",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/randombytes",
      "name": "randombytes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/randombytes",
      "peer": true,
      "resolved": "https://registry.npmjs.org/randombytes/-/randombytes-2.1.0.tgz",
      "version": "2.1.0",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/ink/node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react",
      "peer": true,
      "resolved": "https://registry.npmjs.org/react/-/react-16.14.0.tgz",
      "version": "16.14.0",
    },
    "react-devtools-core" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react-devtools-core",
          "spec": "^4.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shell-quote" => EdgeOut {
          "name": "shell-quote",
          "spec": "^1.6.1",
          "to": "node_modules/shell-quote",
          "type": "prod",
        },
        "ws" => EdgeOut {
          "name": "ws",
          "spec": "^7",
          "to": "node_modules/ws",
          "type": "prod",
        },
      },
      "location": "node_modules/react-devtools-core",
      "name": "react-devtools-core",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react-devtools-core",
      "resolved": "https://registry.npmjs.org/react-devtools-core/-/react-devtools-core-4.9.0.tgz",
      "version": "4.9.0",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.13.1.tgz",
      "version": "16.13.1",
    },
    "react-reconciler" => ArboristNode {
      "children": Map {
        "scheduler" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/react-reconciler",
              "name": "scheduler",
              "spec": "^0.19.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/object-assign",
              "type": "prod",
            },
          },
          "location": "node_modules/react-reconciler/node_modules/scheduler",
          "name": "scheduler",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react-reconciler/node_modules/scheduler",
          "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.19.1.tgz",
          "version": "0.19.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "react-reconciler",
          "spec": "0.25.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.0.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.19.0",
          "to": "node_modules/react-reconciler/node_modules/scheduler",
          "type": "prod",
        },
      },
      "location": "node_modules/react-reconciler",
      "name": "react-reconciler",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react-reconciler",
      "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.25.0.tgz",
      "version": "0.25.0",
    },
    "react-refresh" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "react-refresh",
          "spec": "^0.8.3",
          "type": "peer",
        },
      },
      "location": "node_modules/react-refresh",
      "name": "react-refresh",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/react-refresh",
      "peer": true,
      "resolved": "https://registry.npmjs.org/react-refresh/-/react-refresh-0.8.3.tgz",
      "version": "0.8.3",
    },
    "restore-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/randombytes",
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/safe-buffer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.1.tgz",
      "version": "5.2.1",
    },
    "scheduler" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink/node_modules/react-reconciler",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/scheduler",
      "name": "scheduler",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/scheduler",
      "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.18.0.tgz",
      "version": "0.18.0",
    },
    "schema-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "schema-utils",
          "spec": "^2.6.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/json-schema" => EdgeOut {
          "name": "@types/json-schema",
          "spec": "^7.0.5",
          "to": "node_modules/@types/json-schema",
          "type": "prod",
        },
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.4",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
      },
      "location": "node_modules/schema-utils",
      "name": "schema-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/schema-utils",
      "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-2.7.1.tgz",
      "version": "2.7.1",
    },
    "serialize-javascript" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "serialize-javascript",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.1.0",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
      },
      "location": "node_modules/serialize-javascript",
      "name": "serialize-javascript",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/serialize-javascript",
      "peer": true,
      "resolved": "https://registry.npmjs.org/serialize-javascript/-/serialize-javascript-5.0.1.tgz",
      "version": "5.0.1",
    },
    "shell-quote" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react-devtools-core",
          "name": "shell-quote",
          "spec": "^1.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/shell-quote",
      "name": "shell-quote",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/shell-quote",
      "resolved": "https://registry.npmjs.org/shell-quote/-/shell-quote-1.7.2.tgz",
      "version": "1.7.2",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "slice-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^2.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "source-list-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-sources",
          "name": "source-list-map",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/source-list-map",
      "name": "source-list-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/source-list-map",
      "peer": true,
      "resolved": "https://registry.npmjs.org/source-list-map/-/source-list-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "source-map",
          "spec": "^0.7.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser",
          "name": "source-map",
          "spec": "~0.7.2",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.7.3.tgz",
      "version": "0.7.3",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/source-map-support/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser",
          "name": "source-map-support",
          "spec": "~0.5.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/source-map-support",
      "peer": true,
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
      "version": "0.5.19",
    },
    "stack-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "stack-utils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^2.0.0",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-2.0.2.tgz",
      "version": "2.0.2",
    },
    "stackframe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-stack-parser",
          "name": "stackframe",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/stackframe",
      "name": "stackframe",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/stackframe",
      "resolved": "https://registry.npmjs.org/stackframe/-/stackframe-1.2.0.tgz",
      "version": "1.2.0",
    },
    "string-length" => ArboristNode {
      "children": Map {
        "astral-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "astral-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/string-length/node_modules/astral-regex",
          "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/string-length/node_modules/astral-regex",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-length",
      "name": "string-length",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/string-length",
      "resolved": "https://registry.npmjs.org/string-length/-/string-length-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string-width" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/string-width/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/string-width/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/string-width/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line",
          "name": "string-width",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/string-width/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-length",
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
      "version": "5.2.0",
    },
    "supports-color" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jest-worker",
          "name": "supports-color",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^4.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
      "version": "7.2.0",
    },
    "tapable" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enhanced-resolve",
          "name": "tapable",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "tapable",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tapable",
      "name": "tapable",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/tapable",
      "peer": true,
      "resolved": "https://registry.npmjs.org/tapable/-/tapable-2.0.0.tgz",
      "version": "2.0.0",
    },
    "terser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "terser",
          "spec": "^5.3.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "^2.20.0",
          "to": "node_modules/commander",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.7.2",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "~0.5.19",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/terser",
      "name": "terser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/terser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/terser/-/terser-5.3.8.tgz",
      "version": "5.3.8",
    },
    "terser-webpack-plugin" => ArboristNode {
      "children": Map {
        "p-limit" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser-webpack-plugin",
              "name": "p-limit",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^2.0.0",
              "to": "node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/terser-webpack-plugin/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/terser-webpack-plugin/node_modules/p-limit",
          "peer": true,
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-3.0.2.tgz",
          "version": "3.0.2",
        },
        "schema-utils" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser-webpack-plugin",
              "name": "schema-utils",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/json-schema" => EdgeOut {
              "name": "@types/json-schema",
              "spec": "^7.0.6",
              "to": "node_modules/@types/json-schema",
              "type": "prod",
            },
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.12.5",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "ajv-keywords" => EdgeOut {
              "name": "ajv-keywords",
              "spec": "^3.5.2",
              "to": "node_modules/ajv-keywords",
              "type": "prod",
            },
          },
          "location": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "schema-utils",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "peer": true,
          "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-3.0.0.tgz",
          "version": "3.0.0",
        },
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser-webpack-plugin",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/terser-webpack-plugin/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/terser-webpack-plugin/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "terser-webpack-plugin",
          "spec": "^5.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jest-worker" => EdgeOut {
          "name": "jest-worker",
          "spec": "^26.6.1",
          "to": "node_modules/jest-worker",
          "type": "prod",
        },
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^3.0.2",
          "to": "node_modules/terser-webpack-plugin/node_modules/p-limit",
          "type": "prod",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^3.0.0",
          "to": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "type": "prod",
        },
        "serialize-javascript" => EdgeOut {
          "name": "serialize-javascript",
          "spec": "^5.0.1",
          "to": "node_modules/serialize-javascript",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/terser-webpack-plugin/node_modules/source-map",
          "type": "prod",
        },
        "terser" => EdgeOut {
          "name": "terser",
          "spec": "^5.3.8",
          "to": "node_modules/terser",
          "type": "prod",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": "^5.1.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
      },
      "location": "node_modules/terser-webpack-plugin",
      "name": "terser-webpack-plugin",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/terser-webpack-plugin",
      "peer": true,
      "resolved": "https://registry.npmjs.org/terser-webpack-plugin/-/terser-webpack-plugin-5.0.3.tgz",
      "version": "5.0.3",
    },
    "tslib" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chrome-trace-event",
          "name": "tslib",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tslib",
      "name": "tslib",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/tslib",
      "peer": true,
      "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
      "version": "1.14.1",
    },
    "type-fest" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "type-fest",
          "spec": "^0.13.1",
          "type": "peerOptional",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "type-fest",
          "spec": "^0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.12.0.tgz",
      "version": "0.12.0",
    },
    "uri-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "watchpack" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "watchpack",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob-to-regexp" => EdgeOut {
          "name": "glob-to-regexp",
          "spec": "^0.4.1",
          "to": "node_modules/glob-to-regexp",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
      },
      "location": "node_modules/watchpack",
      "name": "watchpack",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/watchpack",
      "peer": true,
      "resolved": "https://registry.npmjs.org/watchpack/-/watchpack-2.0.1.tgz",
      "version": "2.0.1",
    },
    "webpack" => ArboristNode {
      "children": Map {
        "schema-utils" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack",
              "name": "schema-utils",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/json-schema" => EdgeOut {
              "name": "@types/json-schema",
              "spec": "^7.0.6",
              "to": "node_modules/@types/json-schema",
              "type": "prod",
            },
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.12.5",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "ajv-keywords" => EdgeOut {
              "name": "ajv-keywords",
              "spec": "^3.5.2",
              "to": "node_modules/ajv-keywords",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack/node_modules/schema-utils",
          "name": "schema-utils",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/webpack/node_modules/schema-utils",
          "peer": true,
          "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "webpack",
          "spec": ">=4.43.0 <6.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "webpack",
          "spec": "^5.1.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@types/eslint-scope" => EdgeOut {
          "name": "@types/eslint-scope",
          "spec": "^3.7.0",
          "to": "node_modules/@types/eslint-scope",
          "type": "prod",
        },
        "@types/estree" => EdgeOut {
          "name": "@types/estree",
          "spec": "^0.0.45",
          "to": "node_modules/@types/estree",
          "type": "prod",
        },
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-module-context" => EdgeOut {
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-module-context",
          "type": "prod",
        },
        "@webassemblyjs/wasm-edit" => EdgeOut {
          "name": "@webassemblyjs/wasm-edit",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-edit",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^8.0.4",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "browserslist" => EdgeOut {
          "name": "browserslist",
          "spec": "^4.14.5",
          "to": "node_modules/browserslist",
          "type": "prod",
        },
        "chrome-trace-event" => EdgeOut {
          "name": "chrome-trace-event",
          "spec": "^1.0.2",
          "to": "node_modules/chrome-trace-event",
          "type": "prod",
        },
        "enhanced-resolve" => EdgeOut {
          "name": "enhanced-resolve",
          "spec": "^5.3.1",
          "to": "node_modules/enhanced-resolve",
          "type": "prod",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "events" => EdgeOut {
          "name": "events",
          "spec": "^3.2.0",
          "to": "node_modules/events",
          "type": "prod",
        },
        "glob-to-regexp" => EdgeOut {
          "name": "glob-to-regexp",
          "spec": "^0.4.1",
          "to": "node_modules/glob-to-regexp",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.2.4",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.2",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
        "loader-runner" => EdgeOut {
          "name": "loader-runner",
          "spec": "^4.1.0",
          "to": "node_modules/loader-runner",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.27",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.2",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^4.2.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^3.0.0",
          "to": "node_modules/webpack/node_modules/schema-utils",
          "type": "prod",
        },
        "tapable" => EdgeOut {
          "name": "tapable",
          "spec": "^2.0.0",
          "to": "node_modules/tapable",
          "type": "prod",
        },
        "terser-webpack-plugin" => EdgeOut {
          "name": "terser-webpack-plugin",
          "spec": "^5.0.3",
          "to": "node_modules/terser-webpack-plugin",
          "type": "prod",
        },
        "watchpack" => EdgeOut {
          "name": "watchpack",
          "spec": "^2.0.0",
          "to": "node_modules/watchpack",
          "type": "prod",
        },
        "webpack-sources" => EdgeOut {
          "name": "webpack-sources",
          "spec": "^2.1.1",
          "to": "node_modules/webpack-sources",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack",
      "name": "webpack",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/webpack",
      "peer": true,
      "resolved": "https://registry.npmjs.org/webpack/-/webpack-5.3.2.tgz",
      "version": "5.3.2",
    },
    "webpack-sources" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack-sources",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack-sources/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/webpack-sources/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "webpack-sources",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-list-map" => EdgeOut {
          "name": "source-list-map",
          "spec": "^2.0.1",
          "to": "node_modules/source-list-map",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/webpack-sources/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack-sources",
      "name": "webpack-sources",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/webpack-sources",
      "peer": true,
      "resolved": "https://registry.npmjs.org/webpack-sources/-/webpack-sources-2.2.0.tgz",
      "version": "2.2.0",
    },
    "widest-line" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/wrap-ansi/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/wrap-ansi/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/wrap-ansi/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
      "version": "6.2.0",
    },
    "ws" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "ws",
          "spec": "^7.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-devtools-core",
          "name": "ws",
          "spec": "^7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bufferutil" => EdgeOut {
          "name": "bufferutil",
          "spec": "^4.0.1",
          "to": null,
          "type": "peerOptional",
        },
        "utf-8-validate" => EdgeOut {
          "name": "utf-8-validate",
          "spec": "^5.0.2",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/ws",
      "name": "ws",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/ws",
      "resolved": "https://registry.npmjs.org/ws/-/ws-7.3.1.tgz",
      "version": "7.3.1",
    },
    "yoga-layout-prebuilt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/yoga-layout" => EdgeOut {
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "to": "node_modules/@types/yoga-layout",
          "type": "prod",
        },
      },
      "location": "node_modules/yoga-layout-prebuilt",
      "name": "yoga-layout-prebuilt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps/node_modules/yoga-layout-prebuilt",
      "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.10.0.tgz",
      "version": "1.10.0",
    },
  },
  "edgesOut": Map {
    "@pmmmwh/react-refresh-webpack-plugin" => EdgeOut {
      "name": "@pmmmwh/react-refresh-webpack-plugin",
      "spec": "0.4.2",
      "to": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "type": "prod",
    },
    "ink" => EdgeOut {
      "name": "ink",
      "spec": "3.0.8",
      "to": "node_modules/ink",
      "type": "prod",
    },
    "react-reconciler" => EdgeOut {
      "name": "react-reconciler",
      "spec": "0.25.0",
      "to": "node_modules/react-reconciler",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-always-prefer-deduping-peer-deps",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bad shrinkwrap file > bad shrinkwrap 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps-b" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-deps-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-deps-d" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-d",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-deps-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-peer-deps-bad-sw",
  "packageName": "@isaacs/testing-peer-deps",
  "path": "{CWD}/test/fixtures/testing-peer-deps-bad-sw",
  "version": "2.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, complete:true > no missing deps, because complete: true 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-b" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "name": "@isaacs/testing-bundledeps-b",
          "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-empty",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-empty",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, complete:true > no missing deps, because complete: true, add dep, save bundled 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps",
  ],
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundled": true,
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-empty",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-empty",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, empty > bundle deps testing 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "error": "MISSING",
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": null,
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-empty",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-empty",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, empty > should have some missing deps in the ideal tree 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps",
  ],
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundled": true,
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "error": "MISSING",
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": null,
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-empty",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-empty",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, full > add stuff, no missing deps 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps",
  ],
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundled": true,
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 1, full > no missing deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-b" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "name": "@isaacs/testing-bundledeps-b",
          "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2 > add new bundled dep c 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps-a",
    "@isaacs/testing-bundledeps-c",
  ],
  "children": Map {
    "@isaacs/testing-bundledeps-a" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-a",
      "name": "@isaacs/testing-bundledeps-a",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-a" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-a",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-a",
      "type": "prod",
    },
    "@isaacs/testing-bundledeps-c" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-c",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-c",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-2",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-2",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2 > bundle deps testing 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps-a",
  ],
  "children": Map {
    "@isaacs/testing-bundledeps-a" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-a",
      "name": "@isaacs/testing-bundledeps-a",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-a" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-a",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-a",
      "type": "prod",
    },
    "@isaacs/testing-bundledeps-c" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-c",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-c",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-2",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-2",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2 > remove bundled dependency a 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-c" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-c",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-c",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-bundledeps-2",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-2",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2, link > add new bundled dep c 1`] = `
ArboristLink {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps-a",
    "@isaacs/testing-bundledeps-c",
  ],
  "isProjectRoot": true,
  "location": "../testing-bundledeps-link",
  "name": "testing-bundledeps-link",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-link",
  "realpath": "{CWD}/test/fixtures/testing-bundledeps-2",
  "resolved": "file:testing-bundledeps-2",
  "target": ArboristNode {
    "bundleDependencies": Array [
      "@isaacs/testing-bundledeps-a",
      "@isaacs/testing-bundledeps-c",
    ],
    "children": Map {
      "@isaacs/testing-bundledeps-a" => ArboristNode {
        "bundled": true,
        "bundler": "",
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-bundledeps-a",
            "spec": "*",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-bundledeps-b" => EdgeOut {
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "to": "node_modules/@isaacs/testing-bundledeps-b",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-a",
        "name": "@isaacs/testing-bundledeps-a",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-a",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
        "version": "1.0.0",
      },
      "@isaacs/testing-bundledeps-b" => ArboristNode {
        "bundled": true,
        "bundler": "",
        "edgesIn": Set {
          EdgeIn {
            "from": "node_modules/@isaacs/testing-bundledeps-a",
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "type": "prod",
          },
          EdgeIn {
            "from": "node_modules/@isaacs/testing-bundledeps-c",
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-b",
        "name": "@isaacs/testing-bundledeps-b",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
        "version": "1.0.0",
      },
      "@isaacs/testing-bundledeps-c" => ArboristNode {
        "bundled": true,
        "bundler": "",
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-bundledeps-c",
            "spec": "*",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-bundledeps-b" => EdgeOut {
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "to": "node_modules/@isaacs/testing-bundledeps-b",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-c",
        "name": "@isaacs/testing-bundledeps-c",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
        "version": "2.0.0",
      },
    },
    "edgesOut": Map {
      "@isaacs/testing-bundledeps-a" => EdgeOut {
        "name": "@isaacs/testing-bundledeps-a",
        "spec": "*",
        "to": "node_modules/@isaacs/testing-bundledeps-a",
        "type": "prod",
      },
      "@isaacs/testing-bundledeps-c" => EdgeOut {
        "name": "@isaacs/testing-bundledeps-c",
        "spec": "*",
        "to": "node_modules/@isaacs/testing-bundledeps-c",
        "type": "prod",
      },
    },
    "isProjectRoot": true,
    "location": "",
    "name": "testing-bundledeps-2",
    "packageName": "@isaacs/testing-bundledeps",
    "path": "{CWD}/test/fixtures/testing-bundledeps-2",
    "version": "1.0.0",
  },
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2, link > bundle deps testing 1`] = `
ArboristLink {
  "bundleDependencies": Array [
    "@isaacs/testing-bundledeps-a",
  ],
  "isProjectRoot": true,
  "location": "../testing-bundledeps-link",
  "name": "testing-bundledeps-link",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-link",
  "realpath": "{CWD}/test/fixtures/testing-bundledeps-2",
  "resolved": "file:testing-bundledeps-2",
  "target": ArboristNode {
    "bundleDependencies": Array [
      "@isaacs/testing-bundledeps-a",
    ],
    "children": Map {
      "@isaacs/testing-bundledeps-a" => ArboristNode {
        "bundled": true,
        "bundler": "",
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-bundledeps-a",
            "spec": "*",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-bundledeps-b" => EdgeOut {
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "to": "node_modules/@isaacs/testing-bundledeps-b",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-a",
        "name": "@isaacs/testing-bundledeps-a",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-a",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
        "version": "1.0.0",
      },
      "@isaacs/testing-bundledeps-b" => ArboristNode {
        "bundled": true,
        "bundler": "",
        "edgesIn": Set {
          EdgeIn {
            "from": "node_modules/@isaacs/testing-bundledeps-a",
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "type": "prod",
          },
          EdgeIn {
            "from": "node_modules/@isaacs/testing-bundledeps-c",
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-b",
        "name": "@isaacs/testing-bundledeps-b",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
        "version": "1.0.0",
      },
      "@isaacs/testing-bundledeps-c" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-bundledeps-c",
            "spec": "*",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-bundledeps-b" => EdgeOut {
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "to": "node_modules/@isaacs/testing-bundledeps-b",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-c",
        "name": "@isaacs/testing-bundledeps-c",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
        "version": "2.0.0",
      },
    },
    "edgesOut": Map {
      "@isaacs/testing-bundledeps-a" => EdgeOut {
        "name": "@isaacs/testing-bundledeps-a",
        "spec": "*",
        "to": "node_modules/@isaacs/testing-bundledeps-a",
        "type": "prod",
      },
      "@isaacs/testing-bundledeps-c" => EdgeOut {
        "name": "@isaacs/testing-bundledeps-c",
        "spec": "*",
        "to": "node_modules/@isaacs/testing-bundledeps-c",
        "type": "prod",
      },
    },
    "isProjectRoot": true,
    "location": "",
    "name": "testing-bundledeps-2",
    "packageName": "@isaacs/testing-bundledeps",
    "path": "{CWD}/test/fixtures/testing-bundledeps-2",
    "version": "1.0.0",
  },
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP bundle deps example 2, link > remove bundled dependency a 1`] = `
ArboristLink {
  "isProjectRoot": true,
  "location": "../testing-bundledeps-link",
  "name": "testing-bundledeps-link",
  "packageName": "@isaacs/testing-bundledeps",
  "path": "{CWD}/test/fixtures/testing-bundledeps-link",
  "realpath": "{CWD}/test/fixtures/testing-bundledeps-2",
  "resolved": "file:testing-bundledeps-2",
  "target": ArboristNode {
    "children": Map {
      "@isaacs/testing-bundledeps-b" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "node_modules/@isaacs/testing-bundledeps-c",
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-b",
        "name": "@isaacs/testing-bundledeps-b",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-b",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
        "version": "1.0.0",
      },
      "@isaacs/testing-bundledeps-c" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-bundledeps-c",
            "spec": "*",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-bundledeps-b" => EdgeOut {
            "name": "@isaacs/testing-bundledeps-b",
            "spec": "*",
            "to": "node_modules/@isaacs/testing-bundledeps-b",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-bundledeps-c",
        "name": "@isaacs/testing-bundledeps-c",
        "path": "{CWD}/test/fixtures/testing-bundledeps-2/node_modules/@isaacs/testing-bundledeps-c",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
        "version": "2.0.0",
      },
    },
    "edgesOut": Map {
      "@isaacs/testing-bundledeps-c" => EdgeOut {
        "name": "@isaacs/testing-bundledeps-c",
        "spec": "*",
        "to": "node_modules/@isaacs/testing-bundledeps-c",
        "type": "prod",
      },
    },
    "isProjectRoot": true,
    "location": "",
    "name": "testing-bundledeps-2",
    "packageName": "@isaacs/testing-bundledeps",
    "path": "{CWD}/test/fixtures/testing-bundledeps-2",
    "version": "1.0.0",
  },
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP carbonium eslint conflicts > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/highlight" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/@babel/highlight/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight",
              "name": "chalk",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "to": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^5.3.0",
              "to": "node_modules/@babel/highlight/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
          "version": "2.4.2",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/@babel/highlight/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "supports-color",
              "spec": "^5.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/@babel/highlight/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
          "version": "5.5.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/@babel/highlight/node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@eslint/eslintrc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "@eslint/eslintrc",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.4",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^7.3.0",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^12.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^4.0.6",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "import-fresh" => EdgeOut {
          "name": "import-fresh",
          "spec": "^3.2.1",
          "to": "node_modules/import-fresh",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "^3.1.1",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
      },
      "location": "node_modules/@eslint/eslintrc",
      "name": "@eslint/eslintrc",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@eslint/eslintrc",
      "resolved": "https://registry.npmjs.org/@eslint/eslintrc/-/eslintrc-0.1.3.tgz",
      "version": "0.1.3",
    },
    "@nodelib/fs.scandir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.walk",
          "name": "@nodelib/fs.scandir",
          "spec": "2.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.stat" => EdgeOut {
          "name": "@nodelib/fs.stat",
          "spec": "2.0.3",
          "to": "node_modules/@nodelib/fs.stat",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.9",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.scandir",
      "name": "@nodelib/fs.scandir",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@nodelib/fs.scandir",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.scandir/-/fs.scandir-2.1.3.tgz",
      "version": "2.1.3",
    },
    "@nodelib/fs.stat" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.scandir",
          "name": "@nodelib/fs.stat",
          "spec": "2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "@nodelib/fs.stat",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.stat",
      "name": "@nodelib/fs.stat",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@nodelib/fs.stat",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.stat/-/fs.stat-2.0.3.tgz",
      "version": "2.0.3",
    },
    "@nodelib/fs.walk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "@nodelib/fs.walk",
          "spec": "^1.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.scandir" => EdgeOut {
          "name": "@nodelib/fs.scandir",
          "spec": "2.1.3",
          "to": "node_modules/@nodelib/fs.scandir",
          "type": "prod",
        },
        "fastq" => EdgeOut {
          "name": "fastq",
          "spec": "^1.6.0",
          "to": "node_modules/fastq",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.walk",
      "name": "@nodelib/fs.walk",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@nodelib/fs.walk",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.walk/-/fs.walk-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@types/json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/json-schema",
      "name": "@types/json-schema",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@types/json-schema",
      "resolved": "https://registry.npmjs.org/@types/json-schema/-/json-schema-7.0.6.tgz",
      "version": "7.0.6",
    },
    "@typescript-eslint/eslint-plugin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@typescript-eslint/eslint-plugin",
          "spec": "4",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/experimental-utils" => EdgeOut {
          "name": "@typescript-eslint/experimental-utils",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/experimental-utils",
          "type": "prod",
        },
        "@typescript-eslint/parser" => EdgeOut {
          "name": "@typescript-eslint/parser",
          "spec": "^4.0.0",
          "to": "node_modules/@typescript-eslint/parser",
          "type": "peer",
        },
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "functional-red-black-tree" => EdgeOut {
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "to": "node_modules/functional-red-black-tree",
          "type": "prod",
        },
        "regexpp" => EdgeOut {
          "name": "regexpp",
          "spec": "^3.0.0",
          "to": "node_modules/regexpp",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "tsutils" => EdgeOut {
          "name": "tsutils",
          "spec": "^3.17.1",
          "to": "node_modules/tsutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/eslint-plugin",
      "name": "@typescript-eslint/eslint-plugin",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/eslint-plugin",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/eslint-plugin/-/eslint-plugin-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/experimental-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/experimental-utils",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/json-schema" => EdgeOut {
          "name": "@types/json-schema",
          "spec": "^7.0.3",
          "to": "node_modules/@types/json-schema",
          "type": "prod",
        },
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/typescript-estree" => EdgeOut {
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/typescript-estree",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "*",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^5.0.0",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/experimental-utils",
      "name": "@typescript-eslint/experimental-utils",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/experimental-utils",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/experimental-utils/-/experimental-utils-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@typescript-eslint/parser",
          "spec": "4",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/parser",
          "spec": "^4.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/typescript-estree" => EdgeOut {
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/typescript-estree",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/@typescript-eslint/parser",
      "name": "@typescript-eslint/parser",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/parser",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/parser/-/parser-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/scope-manager" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/visitor-keys" => EdgeOut {
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/scope-manager",
      "name": "@typescript-eslint/scope-manager",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/scope-manager",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/scope-manager/-/scope-manager-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/scope-manager",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/visitor-keys",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/types",
      "name": "@typescript-eslint/types",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/types",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/types/-/types-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/typescript-estree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/visitor-keys" => EdgeOut {
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/visitor-keys",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^11.0.1",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.15",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "tsutils" => EdgeOut {
          "name": "tsutils",
          "spec": "^3.17.1",
          "to": "node_modules/tsutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/typescript-estree",
      "name": "@typescript-eslint/typescript-estree",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/typescript-estree",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/typescript-estree/-/typescript-estree-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/visitor-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/scope-manager",
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/visitor-keys",
      "name": "@typescript-eslint/visitor-keys",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/@typescript-eslint/visitor-keys",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/visitor-keys/-/visitor-keys-4.4.1.tgz",
      "version": "4.4.1",
    },
    "acorn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/acorn-jsx",
          "name": "acorn",
          "spec": "^6.0.0 || ^7.0.0 || ^8.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/acorn",
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-7.4.1.tgz",
      "version": "7.4.1",
    },
    "acorn-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^6.0.0 || ^7.0.0 || ^8.0.0",
          "to": "node_modules/acorn",
          "type": "peer",
        },
      },
      "location": "node_modules/acorn-jsx",
      "name": "acorn-jsx",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/acorn-jsx",
      "resolved": "https://registry.npmjs.org/acorn-jsx/-/acorn-jsx-5.3.1.tgz",
      "version": "5.3.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "ajv",
          "spec": "^6.12.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ajv",
          "spec": "^6.10.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv",
          "spec": "^6.10.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ansi-colors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enquirer",
          "name": "ansi-colors",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-colors",
      "name": "ansi-colors",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ansi-colors",
      "resolved": "https://registry.npmjs.org/ansi-colors/-/ansi-colors-4.1.1.tgz",
      "version": "4.1.1",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
      "version": "5.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^2.0.1",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
      "version": "4.3.0",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "array-union" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-2.1.0.tgz",
      "version": "2.1.0",
    },
    "astral-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "braces",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parent-module",
          "name": "callsites",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-3.1.0.tgz",
      "version": "3.1.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "chalk",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-4.1.0.tgz",
      "version": "4.1.0",
    },
    "color-convert" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "~1.1.4",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
      "version": "2.0.1",
    },
    "color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "~1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
      "version": "1.1.4",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "cross-spawn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "cross-spawn",
          "spec": "^7.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^3.1.0",
          "to": "node_modules/path-key",
          "type": "prod",
        },
        "shebang-command" => EdgeOut {
          "name": "shebang-command",
          "spec": "^2.0.0",
          "to": "node_modules/shebang-command",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.1",
          "to": "node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-7.0.3.tgz",
      "version": "7.0.3",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "debug",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "deep-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "deep-is",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-is",
      "name": "deep-is",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/deep-is",
      "resolved": "https://registry.npmjs.org/deep-is/-/deep-is-0.1.3.tgz",
      "version": "0.1.3",
    },
    "dir-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "dir-glob",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^4.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/dir-glob",
      "name": "dir-glob",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/dir-glob",
      "resolved": "https://registry.npmjs.org/dir-glob/-/dir-glob-3.0.1.tgz",
      "version": "3.0.1",
    },
    "doctrine" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "doctrine",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
      },
      "location": "node_modules/doctrine",
      "name": "doctrine",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/doctrine",
      "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-3.0.0.tgz",
      "version": "3.0.0",
    },
    "emoji-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
      "version": "7.0.3",
    },
    "enquirer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "enquirer",
          "spec": "^2.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-colors" => EdgeOut {
          "name": "ansi-colors",
          "spec": "^4.1.1",
          "to": "node_modules/ansi-colors",
          "type": "prod",
        },
      },
      "location": "node_modules/enquirer",
      "name": "enquirer",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/enquirer",
      "resolved": "https://registry.npmjs.org/enquirer/-/enquirer-2.3.6.tgz",
      "version": "2.3.6",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "eslint" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "eslint",
          "spec": "^7",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint",
          "spec": "*",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@eslint/eslintrc" => EdgeOut {
          "name": "@eslint/eslintrc",
          "spec": "^0.1.3",
          "to": "node_modules/@eslint/eslintrc",
          "type": "prod",
        },
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.10.0",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.2",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.0.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^3.0.0",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "enquirer" => EdgeOut {
          "name": "enquirer",
          "spec": "^2.3.5",
          "to": "node_modules/enquirer",
          "type": "prod",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.1.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-visitor-keys",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^7.3.0",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "esquery" => EdgeOut {
          "name": "esquery",
          "spec": "^1.2.0",
          "to": "node_modules/esquery",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "file-entry-cache" => EdgeOut {
          "name": "file-entry-cache",
          "spec": "^5.0.1",
          "to": "node_modules/file-entry-cache",
          "type": "prod",
        },
        "functional-red-black-tree" => EdgeOut {
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "to": "node_modules/functional-red-black-tree",
          "type": "prod",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "^5.0.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^12.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^4.0.6",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "import-fresh" => EdgeOut {
          "name": "import-fresh",
          "spec": "^3.0.0",
          "to": "node_modules/import-fresh",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.0",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "json-stable-stringify-without-jsonify" => EdgeOut {
          "name": "json-stable-stringify-without-jsonify",
          "spec": "^1.0.1",
          "to": "node_modules/json-stable-stringify-without-jsonify",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.4.1",
          "to": "node_modules/levn",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "natural-compare" => EdgeOut {
          "name": "natural-compare",
          "spec": "^1.4.0",
          "to": "node_modules/natural-compare",
          "type": "prod",
        },
        "optionator" => EdgeOut {
          "name": "optionator",
          "spec": "^0.9.1",
          "to": "node_modules/optionator",
          "type": "prod",
        },
        "progress" => EdgeOut {
          "name": "progress",
          "spec": "^2.0.0",
          "to": "node_modules/progress",
          "type": "prod",
        },
        "regexpp" => EdgeOut {
          "name": "regexpp",
          "spec": "^3.1.0",
          "to": "node_modules/regexpp",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.2.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "^3.1.0",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
        "table" => EdgeOut {
          "name": "table",
          "spec": "^5.2.3",
          "to": "node_modules/table",
          "type": "prod",
        },
        "text-table" => EdgeOut {
          "name": "text-table",
          "spec": "^0.2.0",
          "to": "node_modules/text-table",
          "type": "prod",
        },
        "v8-compile-cache" => EdgeOut {
          "name": "v8-compile-cache",
          "spec": "^2.0.3",
          "to": "node_modules/v8-compile-cache",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint",
      "name": "eslint",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/eslint",
      "resolved": "https://registry.npmjs.org/eslint/-/eslint-7.11.0.tgz",
      "version": "7.11.0",
    },
    "eslint-scope" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint-scope",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.3.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-scope",
      "name": "eslint-scope",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/eslint-scope",
      "resolved": "https://registry.npmjs.org/eslint-scope/-/eslint-scope-5.1.1.tgz",
      "version": "5.1.1",
    },
    "eslint-utils" => ArboristNode {
      "children": Map {
        "eslint-visitor-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-utils",
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "name": "eslint-visitor-keys",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-1.3.0.tgz",
          "version": "1.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-utils",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^1.1.0",
          "to": "node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-utils",
      "name": "eslint-utils",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/eslint-utils",
      "resolved": "https://registry.npmjs.org/eslint-utils/-/eslint-utils-2.1.0.tgz",
      "version": "2.1.0",
    },
    "eslint-visitor-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/visitor-keys",
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-visitor-keys",
      "name": "eslint-visitor-keys",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/eslint-visitor-keys",
      "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-2.0.0.tgz",
      "version": "2.0.0",
    },
    "espree" => ArboristNode {
      "children": Map {
        "eslint-visitor-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/espree",
              "name": "eslint-visitor-keys",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/espree/node_modules/eslint-visitor-keys",
          "name": "eslint-visitor-keys",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/espree/node_modules/eslint-visitor-keys",
          "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-1.3.0.tgz",
          "version": "1.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "espree",
          "spec": "^7.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "espree",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^7.4.0",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "acorn-jsx" => EdgeOut {
          "name": "acorn-jsx",
          "spec": "^5.2.0",
          "to": "node_modules/acorn-jsx",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^1.3.0",
          "to": "node_modules/espree/node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/espree",
      "name": "espree",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/espree",
      "resolved": "https://registry.npmjs.org/espree/-/espree-7.3.0.tgz",
      "version": "7.3.0",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esquery" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esquery",
              "name": "estraverse",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esquery/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/esquery/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "esquery",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.1.0",
          "to": "node_modules/esquery/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esquery",
      "name": "esquery",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/esquery",
      "resolved": "https://registry.npmjs.org/esquery/-/esquery-1.3.1.tgz",
      "version": "1.3.1",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/esrecurse/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "esrecurse",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.2.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/esrecurse",
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "estraverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "estraverse",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/estraverse",
      "name": "estraverse",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/estraverse",
      "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "esutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/doctrine",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "fast-glob",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.stat" => EdgeOut {
          "name": "@nodelib/fs.stat",
          "spec": "^2.0.2",
          "to": "node_modules/@nodelib/fs.stat",
          "type": "prod",
        },
        "@nodelib/fs.walk" => EdgeOut {
          "name": "@nodelib/fs.walk",
          "spec": "^1.2.3",
          "to": "node_modules/@nodelib/fs.walk",
          "type": "prod",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "^5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "merge2" => EdgeOut {
          "name": "merge2",
          "spec": "^1.3.0",
          "to": "node_modules/merge2",
          "type": "prod",
        },
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^4.0.2",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.1",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-glob",
      "name": "fast-glob",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fast-glob",
      "resolved": "https://registry.npmjs.org/fast-glob/-/fast-glob-3.2.4.tgz",
      "version": "3.2.4",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fast-levenshtein" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "fast-levenshtein",
          "spec": "^2.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-levenshtein",
      "name": "fast-levenshtein",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fast-levenshtein",
      "resolved": "https://registry.npmjs.org/fast-levenshtein/-/fast-levenshtein-2.0.6.tgz",
      "version": "2.0.6",
    },
    "fastq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.walk",
          "name": "fastq",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "reusify" => EdgeOut {
          "name": "reusify",
          "spec": "^1.0.4",
          "to": "node_modules/reusify",
          "type": "prod",
        },
      },
      "location": "node_modules/fastq",
      "name": "fastq",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fastq",
      "resolved": "https://registry.npmjs.org/fastq/-/fastq-1.8.0.tgz",
      "version": "1.8.0",
    },
    "file-entry-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flat-cache" => EdgeOut {
          "name": "flat-cache",
          "spec": "^2.0.1",
          "to": "node_modules/flat-cache",
          "type": "prod",
        },
      },
      "location": "node_modules/file-entry-cache",
      "name": "file-entry-cache",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/file-entry-cache",
      "resolved": "https://registry.npmjs.org/file-entry-cache/-/file-entry-cache-5.0.1.tgz",
      "version": "5.0.1",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "flat-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "flat-cache",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flatted" => EdgeOut {
          "name": "flatted",
          "spec": "^2.0.0",
          "to": "node_modules/flatted",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "write" => EdgeOut {
          "name": "write",
          "spec": "1.0.3",
          "to": "node_modules/write",
          "type": "prod",
        },
      },
      "location": "node_modules/flat-cache",
      "name": "flat-cache",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/flat-cache",
      "resolved": "https://registry.npmjs.org/flat-cache/-/flat-cache-2.0.1.tgz",
      "version": "2.0.1",
    },
    "flatted" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "flatted",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flatted",
      "name": "flatted",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/flatted",
      "resolved": "https://registry.npmjs.org/flatted/-/flatted-2.0.2.tgz",
      "version": "2.0.2",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "functional-red-black-tree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/functional-red-black-tree",
      "name": "functional-red-black-tree",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/functional-red-black-tree",
      "resolved": "https://registry.npmjs.org/functional-red-black-tree/-/functional-red-black-tree-1.0.1.tgz",
      "version": "1.0.1",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "glob-parent",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "glob-parent",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.1.tgz",
      "version": "5.1.1",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "globals",
          "spec": "^12.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "globals",
          "spec": "^12.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.1",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-12.4.0.tgz",
      "version": "12.4.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "ignore" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "ignore",
              "spec": "^5.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/ignore",
          "name": "ignore",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/globby/node_modules/ignore",
          "resolved": "https://registry.npmjs.org/ignore/-/ignore-5.1.8.tgz",
          "version": "5.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "globby",
          "spec": "^11.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^2.1.0",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "dir-glob" => EdgeOut {
          "name": "dir-glob",
          "spec": "^3.0.1",
          "to": "node_modules/dir-glob",
          "type": "prod",
        },
        "fast-glob" => EdgeOut {
          "name": "fast-glob",
          "spec": "^3.1.1",
          "to": "node_modules/fast-glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^5.1.4",
          "to": "node_modules/globby/node_modules/ignore",
          "type": "prod",
        },
        "merge2" => EdgeOut {
          "name": "merge2",
          "spec": "^1.3.0",
          "to": "node_modules/merge2",
          "type": "prod",
        },
        "slash" => EdgeOut {
          "name": "slash",
          "spec": "^3.0.0",
          "to": "node_modules/slash",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-11.0.1.tgz",
      "version": "11.0.1",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
      "version": "4.0.0",
    },
    "ignore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "ignore",
          "spec": "^4.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ignore",
          "spec": "^4.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/ignore",
      "name": "ignore",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ignore",
      "resolved": "https://registry.npmjs.org/ignore/-/ignore-4.0.6.tgz",
      "version": "4.0.6",
    },
    "import-fresh" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "import-fresh",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "import-fresh",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "parent-module" => EdgeOut {
          "name": "parent-module",
          "spec": "^1.0.0",
          "to": "node_modules/parent-module",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/import-fresh",
      "name": "import-fresh",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/import-fresh",
      "resolved": "https://registry.npmjs.org/import-fresh/-/import-fresh-3.2.1.tgz",
      "version": "3.2.1",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-glob",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.14.0.tgz",
      "version": "3.14.0",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stable-stringify-without-jsonify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "json-stable-stringify-without-jsonify",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stable-stringify-without-jsonify",
      "name": "json-stable-stringify-without-jsonify",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/json-stable-stringify-without-jsonify",
      "resolved": "https://registry.npmjs.org/json-stable-stringify-without-jsonify/-/json-stable-stringify-without-jsonify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "levn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "levn",
          "spec": "^0.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "levn",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
        "type-check" => EdgeOut {
          "name": "type-check",
          "spec": "~0.4.0",
          "to": "node_modules/type-check",
          "type": "prod",
        },
      },
      "location": "node_modules/levn",
      "name": "levn",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/levn",
      "resolved": "https://registry.npmjs.org/levn/-/levn-0.4.1.tgz",
      "version": "0.4.1",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "lodash",
          "spec": "^4.17.15",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "lodash",
          "spec": "^4.17.14",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "merge2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "merge2",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "merge2",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/merge2",
      "name": "merge2",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/merge2",
      "resolved": "https://registry.npmjs.org/merge2/-/merge2-1.4.1.tgz",
      "version": "1.4.1",
    },
    "micromatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "micromatch",
          "spec": "^4.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "^3.0.1",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.5",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/micromatch",
      "name": "micromatch",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/micromatch",
      "resolved": "https://registry.npmjs.org/micromatch/-/micromatch-4.0.2.tgz",
      "version": "4.0.2",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "natural-compare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/natural-compare",
      "name": "natural-compare",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/natural-compare",
      "resolved": "https://registry.npmjs.org/natural-compare/-/natural-compare-1.4.0.tgz",
      "version": "1.4.0",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "optionator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "optionator",
          "spec": "^0.9.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deep-is" => EdgeOut {
          "name": "deep-is",
          "spec": "^0.1.3",
          "to": "node_modules/deep-is",
          "type": "prod",
        },
        "fast-levenshtein" => EdgeOut {
          "name": "fast-levenshtein",
          "spec": "^2.0.6",
          "to": "node_modules/fast-levenshtein",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.4.1",
          "to": "node_modules/levn",
          "type": "prod",
        },
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
        "type-check" => EdgeOut {
          "name": "type-check",
          "spec": "^0.4.0",
          "to": "node_modules/type-check",
          "type": "prod",
        },
        "word-wrap" => EdgeOut {
          "name": "word-wrap",
          "spec": "^1.2.3",
          "to": "node_modules/word-wrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optionator",
      "name": "optionator",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/optionator",
      "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.9.1.tgz",
      "version": "0.9.1",
    },
    "parent-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-fresh",
          "name": "parent-module",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^3.0.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/parent-module",
      "name": "parent-module",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/parent-module",
      "resolved": "https://registry.npmjs.org/parent-module/-/parent-module-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-3.1.1.tgz",
      "version": "3.1.1",
    },
    "path-type" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dir-glob",
          "name": "path-type",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-4.0.0.tgz",
      "version": "4.0.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "picomatch",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.2.2.tgz",
      "version": "2.2.2",
    },
    "prelude-ls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/levn",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/type-check",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/prelude-ls",
      "name": "prelude-ls",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/prelude-ls",
      "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.2.1.tgz",
      "version": "1.2.1",
    },
    "progress" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "progress",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/progress",
      "name": "progress",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/progress",
      "resolved": "https://registry.npmjs.org/progress/-/progress-2.0.3.tgz",
      "version": "2.0.3",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "regexpp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "regexpp",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "regexpp",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/regexpp",
      "name": "regexpp",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/regexpp",
      "resolved": "https://registry.npmjs.org/regexpp/-/regexpp-3.1.0.tgz",
      "version": "3.1.0",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-fresh",
          "name": "resolve-from",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
      "version": "4.0.0",
    },
    "reusify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fastq",
          "name": "reusify",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/reusify",
      "name": "reusify",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/reusify",
      "resolved": "https://registry.npmjs.org/reusify/-/reusify-1.0.4.tgz",
      "version": "1.0.4",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "rimraf",
          "spec": "2.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
      "version": "2.6.3",
    },
    "run-parallel" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.scandir",
          "name": "run-parallel",
          "spec": "^1.1.9",
          "type": "prod",
        },
      },
      "location": "node_modules/run-parallel",
      "name": "run-parallel",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/run-parallel",
      "resolved": "https://registry.npmjs.org/run-parallel/-/run-parallel-1.1.9.tgz",
      "version": "1.1.9",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "semver",
          "spec": "^7.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "semver",
          "spec": "^7.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "semver",
          "spec": "^7.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
      "version": "7.3.2",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-3.0.0.tgz",
      "version": "3.0.0",
    },
    "slash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "slash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/slash",
      "name": "slash",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/slash",
      "resolved": "https://registry.npmjs.org/slash/-/slash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "slice-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/slice-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/slice-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/slice-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/slice-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/slice-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "slice-ansi",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "to": "node_modules/slice-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-2.1.0.tgz",
      "version": "2.1.0",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "string-width" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/string-width/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/string-width/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/carbonium/node_modules/string-width/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "string-width",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.1.0",
          "to": "node_modules/string-width/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
      "version": "3.1.0",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
      "version": "6.0.0",
    },
    "strip-json-comments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "strip-json-comments",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-json-comments",
      "name": "strip-json-comments",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/strip-json-comments",
      "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-3.1.1.tgz",
      "version": "3.1.1",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^4.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
      "version": "7.2.0",
    },
    "table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "table",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.10.2",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.14",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^2.1.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/table",
      "name": "table",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/table",
      "resolved": "https://registry.npmjs.org/table/-/table-5.4.6.tgz",
      "version": "5.4.6",
    },
    "text-table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "text-table",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/text-table",
      "name": "text-table",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/text-table",
      "resolved": "https://registry.npmjs.org/text-table/-/text-table-0.2.0.tgz",
      "version": "0.2.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tslib" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tsutils",
          "name": "tslib",
          "spec": "^1.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tslib",
      "name": "tslib",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/tslib",
      "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
      "version": "1.14.1",
    },
    "tsutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "tsutils",
          "spec": "^3.17.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "tsutils",
          "spec": "^3.17.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.8.1",
          "to": "node_modules/tslib",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.8.0 || >= 3.2.0-dev || >= 3.3.0-dev || >= 3.4.0-dev || >= 3.5.0-dev || >= 3.6.0-dev || >= 3.6.0-beta || >= 3.7.0-dev || >= 3.7.0-beta",
          "to": "node_modules/typescript",
          "type": "peer",
        },
      },
      "location": "node_modules/tsutils",
      "name": "tsutils",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/tsutils",
      "resolved": "https://registry.npmjs.org/tsutils/-/tsutils-3.17.1.tgz",
      "version": "3.17.1",
    },
    "type-check" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/levn",
          "name": "type-check",
          "spec": "~0.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "type-check",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
      },
      "location": "node_modules/type-check",
      "name": "type-check",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/type-check",
      "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.4.0.tgz",
      "version": "0.4.0",
    },
    "type-fest" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globals",
          "name": "type-fest",
          "spec": "^0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
      "version": "0.8.1",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tsutils",
          "name": "typescript",
          "spec": ">=2.8.0 || >= 3.2.0-dev || >= 3.3.0-dev || >= 3.4.0-dev || >= 3.5.0-dev || >= 3.6.0-dev || >= 3.6.0-beta || >= 3.7.0-dev || >= 3.7.0-beta",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/typescript",
      "peer": true,
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.0.3.tgz",
      "version": "4.0.3",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "v8-compile-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "v8-compile-cache",
          "spec": "^2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/v8-compile-cache",
      "name": "v8-compile-cache",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/v8-compile-cache",
      "resolved": "https://registry.npmjs.org/v8-compile-cache/-/v8-compile-cache-2.1.1.tgz",
      "version": "2.1.1",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "which",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "word-wrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "word-wrap",
          "spec": "^1.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/word-wrap",
      "name": "word-wrap",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/word-wrap",
      "resolved": "https://registry.npmjs.org/word-wrap/-/word-wrap-1.2.3.tgz",
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "write",
          "spec": "1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/write",
      "name": "write",
      "path": "{CWD}/test/fixtures/carbonium/node_modules/write",
      "resolved": "https://registry.npmjs.org/write/-/write-1.0.3.tgz",
      "version": "1.0.3",
    },
  },
  "edgesOut": Map {
    "@typescript-eslint/eslint-plugin" => EdgeOut {
      "name": "@typescript-eslint/eslint-plugin",
      "spec": "4",
      "to": "node_modules/@typescript-eslint/eslint-plugin",
      "type": "dev",
    },
    "@typescript-eslint/parser" => EdgeOut {
      "name": "@typescript-eslint/parser",
      "spec": "4",
      "to": "node_modules/@typescript-eslint/parser",
      "type": "dev",
    },
    "eslint" => EdgeOut {
      "name": "eslint",
      "spec": "^7",
      "to": "node_modules/eslint",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "carbonium",
  "path": "{CWD}/test/fixtures/carbonium",
  "version": "1.0.1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cases requiring peer sets to be nested multi > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-nesting-p" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-nesting-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-p",
      "name": "@isaacs/testing-peer-dep-nesting-p",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-p/-/testing-peer-dep-nesting-p-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-nesting-s" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-nesting-p" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
          "name": "@isaacs/testing-peer-dep-nesting-p",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-p/-/testing-peer-dep-nesting-p-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-nesting-q" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
              "name": "@isaacs/testing-peer-dep-nesting-q",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
          "name": "@isaacs/testing-peer-dep-nesting-q",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-q/-/testing-peer-dep-nesting-q-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-nesting-x" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s",
              "name": "@isaacs/testing-peer-dep-nesting-x",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
              "type": "peer",
            },
            "@isaacs/testing-peer-dep-nesting-q" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-q",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "name": "@isaacs/testing-peer-dep-nesting-x",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-x/-/testing-peer-dep-nesting-x-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-nesting-s",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-nesting-x" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-nesting-x",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-nesting-y" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-nesting-y",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-peer-dep-nesting-y",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-s",
      "name": "@isaacs/testing-peer-dep-nesting-s",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-s",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-s/-/testing-peer-dep-nesting-s-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-nesting-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-nesting-s",
          "name": "@isaacs/testing-peer-dep-nesting-y",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-nesting-z" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-nesting-z",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-peer-dep-nesting-z",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-y",
      "name": "@isaacs/testing-peer-dep-nesting-y",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-y/-/testing-peer-dep-nesting-y-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-nesting-z" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-nesting-y",
          "name": "@isaacs/testing-peer-dep-nesting-z",
          "spec": "*",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-z",
      "name": "@isaacs/testing-peer-dep-nesting-z",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi/node_modules/@isaacs/testing-peer-dep-nesting-z",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-z/-/testing-peer-dep-nesting-z-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-nesting-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-nesting-p",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-nesting-s" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-nesting-s",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-nesting-s",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "multi",
  "packageName": "@isaacs/testing-peer-dep-nesting-multi",
  "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/multi",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cases requiring peer sets to be nested simple > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-nesting-p" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-nesting-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-p",
      "name": "@isaacs/testing-peer-dep-nesting-p",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple/node_modules/@isaacs/testing-peer-dep-nesting-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-p/-/testing-peer-dep-nesting-p-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-nesting-s" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-nesting-p" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
          "name": "@isaacs/testing-peer-dep-nesting-p",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-p/-/testing-peer-dep-nesting-p-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-nesting-q" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
              "name": "@isaacs/testing-peer-dep-nesting-q",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
          "name": "@isaacs/testing-peer-dep-nesting-q",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-q/-/testing-peer-dep-nesting-q-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-nesting-x" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-nesting-s",
              "name": "@isaacs/testing-peer-dep-nesting-x",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-p",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-p",
              "type": "peer",
            },
            "@isaacs/testing-peer-dep-nesting-q" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-nesting-q",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-q",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "name": "@isaacs/testing-peer-dep-nesting-x",
          "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple/node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-x/-/testing-peer-dep-nesting-x-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-nesting-s",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-nesting-x" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-nesting-x",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-peer-dep-nesting-s/node_modules/@isaacs/testing-peer-dep-nesting-x",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-nesting-s",
      "name": "@isaacs/testing-peer-dep-nesting-s",
      "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple/node_modules/@isaacs/testing-peer-dep-nesting-s",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-nesting-s/-/testing-peer-dep-nesting-s-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-nesting-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-nesting-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-nesting-p",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-nesting-s" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-nesting-s",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-nesting-s",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "simple",
  "packageName": "@isaacs/testing-peer-dep-nesting-simple",
  "path": "{CWD}/test/fixtures/testing-peer-dep-nesting/simple",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace conflicting peerSets do warn > root tree 1`] = `
ArboristNode {
  "children": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-b" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "type": "prod",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-a/-/workspace-peer-dep-infinite-loop-a-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "error": "INVALID",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "peerConflicted": true,
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "error": "INVALID",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "peerConflicted": true,
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-b/-/workspace-peer-dep-infinite-loop-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "peerConflicted": true,
          "spec": "1.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-c/-/workspace-peer-dep-infinite-loop-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "peerConflicted": true,
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-d/-/workspace-peer-dep-infinite-loop-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => EdgeOut {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "spec": "2.0.1",
      "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace conflicting peerSets do warn > root warnings 1`] = `
Object {
  "code": "ERESOLVE",
  "current": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
              "spec": "2.0.1",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "version": "2.0.1",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "2.0.1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "2.0.1",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "2.0.0",
  },
  "dep": Object {
    "dependents": Array [
      Object {
        "error": "INVALID",
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "2.0.1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "2.0.1",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
              "spec": "1.0.0",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "version": "1.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "1.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "1.0.0",
    "whileInstalling": Object {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "version": "1.0.0",
    },
  },
  "edge": Object {
    "error": "INVALID",
    "from": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "2.0.1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "2.0.1",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "version": "1.0.0",
    },
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "spec": "1.0.0",
    "type": "peer",
  },
  "force": false,
  "isMine": false,
  "peerConflict": Object {
    "current": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "2.0.1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "2.0.1",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "type": "peer",
        },
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "dependents": Array [
                    Object {
                      "from": Object {
                        "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                      },
                      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                      "spec": "2.0.1",
                      "type": "prod",
                    },
                  ],
                  "isWorkspace": false,
                  "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "version": "2.0.1",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
                "spec": "2.0.0",
                "type": "peer",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
            "version": "2.0.0",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "2.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "version": "2.0.0",
    },
    "peer": Object {
      "dependents": Array [
        Object {
          "error": "INVALID",
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "dependents": Array [
                    Object {
                      "from": Object {
                        "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                      },
                      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                      "spec": "2.0.1",
                      "type": "prod",
                    },
                  ],
                  "isWorkspace": false,
                  "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "version": "2.0.1",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
                "spec": "1.0.0",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
            "version": "1.0.0",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "version": "1.0.0",
      "whileInstalling": Object {
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
        "version": "1.0.0",
      },
    },
  },
  "strictPeerDeps": false,
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace conflicting peerSets do warn > workspace tree 1`] = `
ArboristNode {
  "children": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-b" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "type": "prod",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "error": "INVALID",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "peerConflicted": true,
          "spec": "2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "error": "INVALID",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "peerConflicted": true,
          "spec": "2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-a/-/workspace-peer-dep-infinite-loop-a-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-b/-/workspace-peer-dep-infinite-loop-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "peerConflicted": true,
          "spec": "2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-c/-/workspace-peer-dep-infinite-loop-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "peerConflicted": true,
          "spec": "2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-d/-/workspace-peer-dep-infinite-loop-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-a" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "2.0.1",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "ws",
  "packageName": "workspace",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "a",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace conflicting peerSets do warn > workspace warnings 1`] = `
Object {
  "code": "ERESOLVE",
  "current": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
              "spec": "2.0.1",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "version": "2.0.1",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
    "version": "2.0.0",
  },
  "dep": Object {
    "dependents": Array [
      Object {
        "error": "INVALID",
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "2.0.1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "2.0.1",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
              "spec": "1.0.0",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "version": "1.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
        "spec": "1.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
    "version": "1.0.0",
    "whileInstalling": Object {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "version": "1.0.0",
    },
  },
  "edge": Object {
    "error": "INVALID",
    "from": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "2.0.1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "2.0.1",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "version": "1.0.0",
    },
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
    "spec": "1.0.0",
    "type": "peer",
  },
  "force": false,
  "isMine": false,
  "peerConflict": Object {
    "current": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "2.0.1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "2.0.1",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "2.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "version": "2.0.0",
    },
    "peer": Object {
      "dependents": Array [
        Object {
          "error": "INVALID",
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "dependents": Array [
                    Object {
                      "from": Object {
                        "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                      },
                      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                      "spec": "2.0.1",
                      "type": "prod",
                    },
                  ],
                  "isWorkspace": false,
                  "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                  "version": "2.0.1",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
                "spec": "1.0.0",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
            "version": "1.0.0",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "version": "1.0.0",
      "whileInstalling": Object {
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
        "version": "1.0.0",
      },
    },
  },
  "strictPeerDeps": false,
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace overlapping peerSets dont warn > root tree 1`] = `
ArboristNode {
  "children": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-b" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "type": "prod",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0 || 2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-a/-/workspace-peer-dep-infinite-loop-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-b/-/workspace-peer-dep-infinite-loop-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-c/-/workspace-peer-dep-infinite-loop-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-d/-/workspace-peer-dep-infinite-loop-d-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => EdgeOut {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "spec": "1",
      "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace overlapping peerSets dont warn > root warnings 1`] = `
Object {
  "code": "ERESOLVE",
  "current": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "2.0.0",
  },
  "dep": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "1.0.0",
    "whileInstalling": Object {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "version": "1.0.0",
    },
  },
  "edge": Object {
    "from": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "1.0.0",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "version": "2.0.0",
    },
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "spec": "2.0.0",
    "type": "peer",
  },
  "force": false,
  "isMine": false,
  "peerConflict": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/root",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "2.0.0",
  },
  "strictPeerDeps": false,
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace overlapping peerSets dont warn > workspace tree 1`] = `
ArboristNode {
  "children": Map {
    "@lukekarrys/workspace-peer-dep-infinite-loop-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-b" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "type": "prod",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0 || 2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-a/-/workspace-peer-dep-infinite-loop-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
        "@lukekarrys/workspace-peer-dep-infinite-loop-d" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-b/-/workspace-peer-dep-infinite-loop-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-c/-/workspace-peer-dep-infinite-loop-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@lukekarrys/workspace-peer-dep-infinite-loop-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-b",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-c" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "spec": "1.0.0",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@lukekarrys/workspace-peer-dep-infinite-loop-d/-/workspace-peer-dep-infinite-loop-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@lukekarrys/workspace-peer-dep-infinite-loop-a" => EdgeOut {
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "spec": "1",
          "to": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "ws",
  "packageName": "workspace",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "a",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP competing peerSets resolve in both root and workspace overlapping peerSets dont warn > workspace warnings 1`] = `
Object {
  "code": "ERESOLVE",
  "current": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "dependents": Array [],
                      "isWorkspace": true,
                      "linksIn": Array [
                        Object {
                          "dependents": Array [
                            Object {
                              "from": Object {
                                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
                              },
                              "name": "a",
                              "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
                              "type": "workspace",
                            },
                          ],
                          "isWorkspace": true,
                          "location": "node_modules/a",
                          "name": "a",
                          "version": "1.0.0",
                        },
                      ],
                      "location": "a",
                      "name": "a",
                      "version": "1.0.0",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "2.0.0",
  },
  "dep": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "dependents": Array [],
                      "isWorkspace": true,
                      "linksIn": Array [
                        Object {
                          "dependents": Array [
                            Object {
                              "from": Object {
                                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
                              },
                              "name": "a",
                              "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
                              "type": "workspace",
                            },
                          ],
                          "isWorkspace": true,
                          "location": "node_modules/a",
                          "name": "a",
                          "version": "1.0.0",
                        },
                      ],
                      "location": "a",
                      "name": "a",
                      "version": "1.0.0",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "1.0.0",
    "whileInstalling": Object {
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
      "version": "1.0.0",
    },
  },
  "edge": Object {
    "from": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "dependents": Array [],
                  "isWorkspace": true,
                  "linksIn": Array [
                    Object {
                      "dependents": Array [
                        Object {
                          "from": Object {
                            "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
                          },
                          "name": "a",
                          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
                          "type": "workspace",
                        },
                      ],
                      "isWorkspace": true,
                      "location": "node_modules/a",
                      "name": "a",
                      "version": "1.0.0",
                    },
                  ],
                  "location": "a",
                  "name": "a",
                  "version": "1.0.0",
                },
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "spec": "1",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
            "version": "1.0.0",
          },
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "spec": "1.0.0 || 2.0.0",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
      "version": "2.0.0",
    },
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "spec": "2.0.0",
    "type": "peer",
  },
  "force": false,
  "isMine": false,
  "peerConflict": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "dependents": Array [],
                      "isWorkspace": true,
                      "linksIn": Array [
                        Object {
                          "dependents": Array [
                            Object {
                              "from": Object {
                                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws",
                              },
                              "name": "a",
                              "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-competing-peerSets-resolve-in-both-root-and-workspace/ws/a",
                              "type": "workspace",
                            },
                          ],
                          "isWorkspace": true,
                          "location": "node_modules/a",
                          "name": "a",
                          "version": "1.0.0",
                        },
                      ],
                      "location": "a",
                      "name": "a",
                      "version": "1.0.0",
                    },
                    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                    "spec": "1",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "name": "@lukekarrys/workspace-peer-dep-infinite-loop-a",
                "version": "1.0.0",
              },
              "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
              "spec": "1.0.0 || 2.0.0",
              "type": "peer",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "name": "@lukekarrys/workspace-peer-dep-infinite-loop-d",
          "version": "2.0.0",
        },
        "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
        "spec": "2.0.0",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "name": "@lukekarrys/workspace-peer-dep-infinite-loop-c",
    "version": "2.0.0",
  },
  "strictPeerDeps": false,
}
`

exports[`test/arborist/build-ideal-tree.js TAP complete build for project with old lockfile > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/fixtures/link-dep-empty/node_modules/linked-dep",
      "realpath": "{CWD}/test/fixtures/link-dep-empty/target",
      "resolved": "file:../target",
      "target": ArboristNode {
        "location": "target",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "linked-dep" => EdgeOut {
      "name": "linked-dep",
      "spec": "file:target",
      "to": "node_modules/linked-dep",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "target",
      "name": "target",
      "packageName": "linked-dep",
      "path": "{CWD}/test/fixtures/link-dep-empty/target",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "link-dep-empty",
  "packageName": "@isaacs/testing-link-dep",
  "path": "{CWD}/test/fixtures/link-dep-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > add b@2.x with legacy peer deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2.x",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > can add b@2 if we remove a@1 dep 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "2.0.0",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > cyclical peer deps - reload a dependency 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > cyclical peer deps - upgrade a package 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > cyclical peer deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle > remove the dep, prune everything 1`] = `
ArboristNode {
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > add b@2.x with legacy peer deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2.x",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > can add b@2 if we remove a@1 dep 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "2.0.0",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > cyclical peer deps - reload a dependency 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > cyclical peer deps - upgrade a package 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > cyclical peer deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP cyclical peer deps peer-dep-cycle-with-sw > remove the dep, prune everything 1`] = `
ArboristNode {
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-with-sw",
  "packageName": "@isaacs/peer-dep-cycle",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP dedupe example - deduped > dedupe testing 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/dedupe-tests-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-a",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/dedupe-tests-b" => EdgeOut {
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "to": "node_modules/@isaacs/dedupe-tests-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-a",
      "name": "@isaacs/dedupe-tests-a",
      "path": "{CWD}/test/fixtures/dedupe-tests-2/node_modules/@isaacs/dedupe-tests-a",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-a/-/dedupe-tests-a-1.0.1.tgz",
      "version": "1.0.1",
    },
    "@isaacs/dedupe-tests-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/dedupe-tests-a",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-b",
      "name": "@isaacs/dedupe-tests-b",
      "path": "{CWD}/test/fixtures/dedupe-tests-2/node_modules/@isaacs/dedupe-tests-b",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/dedupe-tests-a" => EdgeOut {
      "name": "@isaacs/dedupe-tests-a",
      "spec": "1.0.1",
      "to": "node_modules/@isaacs/dedupe-tests-a",
      "type": "prod",
    },
    "@isaacs/dedupe-tests-b" => EdgeOut {
      "name": "@isaacs/dedupe-tests-b",
      "spec": "1",
      "to": "node_modules/@isaacs/dedupe-tests-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "dedupe-tests-2",
  "packageName": "@isaacs/dedupe-tests-2",
  "path": "{CWD}/test/fixtures/dedupe-tests-2",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP dedupe example - deduped because preferDedupe=true > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/dedupe-tests-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-a",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/dedupe-tests-b" => EdgeOut {
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "to": "node_modules/@isaacs/dedupe-tests-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-a",
      "name": "@isaacs/dedupe-tests-a",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-a",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-a/-/dedupe-tests-a-1.0.1.tgz",
      "version": "1.0.1",
    },
    "@isaacs/dedupe-tests-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1||2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/dedupe-tests-a",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-b",
      "name": "@isaacs/dedupe-tests-b",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-b",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/dedupe-tests-a" => EdgeOut {
      "name": "@isaacs/dedupe-tests-a",
      "spec": "1.0.1",
      "to": "node_modules/@isaacs/dedupe-tests-a",
      "type": "prod",
    },
    "@isaacs/dedupe-tests-b" => EdgeOut {
      "name": "@isaacs/dedupe-tests-b",
      "spec": "1||2",
      "to": "node_modules/@isaacs/dedupe-tests-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "dedupe-tests",
  "packageName": "@isaacs/dedupe-tests",
  "path": "{CWD}/test/fixtures/dedupe-tests",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP dedupe example - nested because legacyBundling=true > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/dedupe-tests-a" => ArboristNode {
      "children": Map {
        "@isaacs/dedupe-tests-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/dedupe-tests-a",
              "name": "@isaacs/dedupe-tests-b",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "name": "@isaacs/dedupe-tests-b",
          "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-a",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/dedupe-tests-b" => EdgeOut {
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "to": "node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-a",
      "name": "@isaacs/dedupe-tests-a",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-a",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-a/-/dedupe-tests-a-1.0.1.tgz",
      "version": "1.0.1",
    },
    "@isaacs/dedupe-tests-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1||2",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-b",
      "name": "@isaacs/dedupe-tests-b",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-b",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/dedupe-tests-a" => EdgeOut {
      "name": "@isaacs/dedupe-tests-a",
      "spec": "1.0.1",
      "to": "node_modules/@isaacs/dedupe-tests-a",
      "type": "prod",
    },
    "@isaacs/dedupe-tests-b" => EdgeOut {
      "name": "@isaacs/dedupe-tests-b",
      "spec": "1||2",
      "to": "node_modules/@isaacs/dedupe-tests-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "dedupe-tests",
  "packageName": "@isaacs/dedupe-tests",
  "path": "{CWD}/test/fixtures/dedupe-tests",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP dedupe example - not deduped > dedupe testing 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/dedupe-tests-a" => ArboristNode {
      "children": Map {
        "@isaacs/dedupe-tests-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/dedupe-tests-a",
              "name": "@isaacs/dedupe-tests-b",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "name": "@isaacs/dedupe-tests-b",
          "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-a",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/dedupe-tests-b" => EdgeOut {
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1",
          "to": "node_modules/@isaacs/dedupe-tests-a/node_modules/@isaacs/dedupe-tests-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-a",
      "name": "@isaacs/dedupe-tests-a",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-a",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-a/-/dedupe-tests-a-1.0.1.tgz",
      "version": "1.0.1",
    },
    "@isaacs/dedupe-tests-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/dedupe-tests-b",
          "spec": "1||2",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/dedupe-tests-b",
      "name": "@isaacs/dedupe-tests-b",
      "path": "{CWD}/test/fixtures/dedupe-tests/node_modules/@isaacs/dedupe-tests-b",
      "resolved": "https://registry.npmjs.org/@isaacs/dedupe-tests-b/-/dedupe-tests-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/dedupe-tests-a" => EdgeOut {
      "name": "@isaacs/dedupe-tests-a",
      "spec": "1.0.1",
      "to": "node_modules/@isaacs/dedupe-tests-a",
      "type": "prod",
    },
    "@isaacs/dedupe-tests-b" => EdgeOut {
      "name": "@isaacs/dedupe-tests-b",
      "spec": "1||2",
      "to": "node_modules/@isaacs/dedupe-tests-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "dedupe-tests",
  "packageName": "@isaacs/dedupe-tests",
  "path": "{CWD}/test/fixtures/dedupe-tests",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP deduped transitive deps with asymmetrical bin declaration > with lockfile 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/node_modules/a",
      "realpath": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:b",
          "type": "prod",
        },
      },
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/node_modules/b",
      "realpath": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:a",
      "to": "node_modules/a",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:b",
      "to": "node_modules/b",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "a",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "a/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/a/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "to": "a/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "b",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "b/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/b/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^2.0.0",
          "to": "b/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-asymmetrical-bin-with-lock",
  "packageName": "test-bin-added-transitive-deps",
  "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-with-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP deduped transitive deps with asymmetrical bin declaration > with no lockfile 1`] = `
ArboristNode {
  "children": Map {
    "@ruyadorno/package-with-added-bin" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@ruyadorno/package-with-added-bin",
      "name": "@ruyadorno/package-with-added-bin",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/node_modules/@ruyadorno/package-with-added-bin",
      "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-1.0.0.tgz",
      "version": "1.0.0",
    },
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/node_modules/a",
      "realpath": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:b",
          "type": "prod",
        },
      },
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/node_modules/b",
      "realpath": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:a",
      "to": "node_modules/a",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:b",
      "to": "node_modules/b",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "to": "node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "b",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "b/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/b/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^2.0.0",
          "to": "b/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-asymmetrical-bin-no-lock",
  "packageName": "test-bin-added-transitive-deps",
  "path": "{CWD}/test/fixtures/testing-asymmetrical-bin-no-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP detect conflicts in transitive peerOptional deps nest when peerOptional conflicts > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/test-conflicted-optional-peer-dep-has-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer",
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-conflicted-optional-peer-dep-peer" => EdgeOut {
          "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
          "spec": "1",
          "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer",
      "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-has-peer/-/test-conflicted-optional-peer-dep-has-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-conflicted-optional-peer-dep-has-peer" => EdgeOut {
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer",
          "spec": "1",
          "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer",
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer",
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-meta-peer/-/test-conflicted-optional-peer-dep-meta-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional" => ArboristNode {
      "children": Map {
        "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
              "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/test-conflicted-optional-peer-dep-peer" => EdgeOut {
              "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
              "spec": "2",
              "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
              "type": "peerOptional",
            },
          },
          "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional/-/test-conflicted-optional-peer-dep-has-peer-optional-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/test-conflicted-optional-peer-dep-peer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
              "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
              "spec": "2",
              "type": "peerOptional",
            },
          },
          "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
          "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
          "optional": true,
          "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-peer/-/test-conflicted-optional-peer-dep-peer-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional" => EdgeOut {
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "spec": "1",
          "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/-/test-conflicted-optional-peer-dep-meta-peer-optional-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/test-conflicted-optional-peer-dep-peer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer",
          "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
      "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-peer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-peer/-/test-conflicted-optional-peer-dep-peer-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer" => EdgeOut {
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer",
      "spec": "1",
      "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer",
      "type": "prod",
    },
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional" => EdgeOut {
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "spec": "1",
      "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "nest-peer-optional",
  "packageName": "@isaacs/test-conflicted-optional-peer-dep",
  "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/nest-peer-optional",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP detect conflicts in transitive peerOptional deps omit peerOptionals when not needed for conflicts > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-conflicted-optional-peer-dep-peer" => EdgeOut {
          "name": "@isaacs/test-conflicted-optional-peer-dep-peer",
          "spec": "2",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
      "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/omit-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional/-/test-conflicted-optional-peer-dep-has-peer-optional-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional" => EdgeOut {
          "name": "@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "spec": "1",
          "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-has-peer-optional",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/omit-peer-optional/node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "resolved": "https://registry.npmjs.org/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional/-/test-conflicted-optional-peer-dep-meta-peer-optional-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional" => EdgeOut {
      "name": "@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "spec": "1",
      "to": "node_modules/@isaacs/test-conflicted-optional-peer-dep-meta-peer-optional",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "omit-peer-optional",
  "packageName": "@isaacs/test-conflicted-optional-peer-dep",
  "path": "{CWD}/test/fixtures/test-conflicted-optional-peer-dep/omit-peer-optional",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do add shrinkwrapped deps when complete:true is set > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/shrinkwrapped-dep-no-lock/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/fixtures/shrinkwrapped-dep-no-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "shrinkwrapped-dep-no-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/fixtures/shrinkwrapped-dep-no-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case a > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-a-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-a-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-a-z" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-a-z",
          "spec": "1",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-a-x",
      "name": "@isaacs/testing-peer-optional-conflict-a-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/a/node_modules/@isaacs/testing-peer-optional-conflict-a-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-a-x/-/testing-peer-optional-conflict-a-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-a-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-a-y",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-a-y",
      "name": "@isaacs/testing-peer-optional-conflict-a-y",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/a/node_modules/@isaacs/testing-peer-optional-conflict-a-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-a-y/-/testing-peer-optional-conflict-a-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-a-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-a-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-a-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-a-y" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-a-y",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-a-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "a",
  "packageName": "@isaacs/testing-peer-optional-conflict-a",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/a",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case b > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-b-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-b-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-b-y" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-b-y",
          "spec": "2",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-b-x",
      "name": "@isaacs/testing-peer-optional-conflict-b-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/b/node_modules/@isaacs/testing-peer-optional-conflict-b-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-b-x/-/testing-peer-optional-conflict-b-x-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-b-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-b-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-b-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-b-y" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-b-y",
      "spec": "1",
      "to": null,
      "type": "peerOptional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "b",
  "packageName": "@isaacs/testing-peer-optional-conflict-b",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/b",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case c > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-c-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-c-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-c-z" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-c-z",
          "spec": "1.0.0",
          "to": "node_modules/@isaacs/testing-peer-optional-conflict-c-z",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-c-x",
      "name": "@isaacs/testing-peer-optional-conflict-c-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/c/node_modules/@isaacs/testing-peer-optional-conflict-c-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-c-x/-/testing-peer-optional-conflict-c-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-c-z" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-optional-conflict-c-x",
          "name": "@isaacs/testing-peer-optional-conflict-c-z",
          "spec": "1.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-c-y" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-c-y",
          "spec": "2.0.0",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-c-z",
      "name": "@isaacs/testing-peer-optional-conflict-c-z",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/c/node_modules/@isaacs/testing-peer-optional-conflict-c-z",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-c-z/-/testing-peer-optional-conflict-c-z-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-c-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-c-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-c-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-c-y" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-c-y",
      "spec": "1",
      "to": null,
      "type": "peerOptional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "c",
  "packageName": "@isaacs/testing-peer-optional-conflict-c",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/c",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case d > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-d-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-d-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-d-y" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-d-y",
          "spec": "1",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-d-x",
      "name": "@isaacs/testing-peer-optional-conflict-d-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/d/node_modules/@isaacs/testing-peer-optional-conflict-d-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-d-x/-/testing-peer-optional-conflict-d-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-d-z" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-d-z",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-d-z",
      "name": "@isaacs/testing-peer-optional-conflict-d-z",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/d/node_modules/@isaacs/testing-peer-optional-conflict-d-z",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-d-z/-/testing-peer-optional-conflict-d-z-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-d-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-d-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-d-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-d-z" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-d-z",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-d-z",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "d",
  "packageName": "@isaacs/testing-peer-optional-conflict-d",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/d",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case e > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-e-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-e-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-e-y" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-e-y",
          "spec": "1",
          "to": null,
          "type": "peerOptional",
        },
        "@isaacs/testing-peer-optional-conflict-e-z" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-e-z",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-optional-conflict-e-z",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-e-x",
      "name": "@isaacs/testing-peer-optional-conflict-e-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/e/node_modules/@isaacs/testing-peer-optional-conflict-e-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-e-x/-/testing-peer-optional-conflict-e-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-e-z" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-e-z",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-optional-conflict-e-x",
          "name": "@isaacs/testing-peer-optional-conflict-e-z",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-e-z",
      "name": "@isaacs/testing-peer-optional-conflict-e-z",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/e/node_modules/@isaacs/testing-peer-optional-conflict-e-z",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-e-z/-/testing-peer-optional-conflict-e-z-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-e-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-e-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-e-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-e-z" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-e-z",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-e-z",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "e",
  "packageName": "@isaacs/testing-peer-optional-conflict-e",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/e",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not ERESOLVE on peerOptionals that are ignored anyway case f > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-optional-conflict-f-w" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-optional-conflict-f-x",
          "name": "@isaacs/testing-peer-optional-conflict-f-w",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-f-z" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-f-z",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-optional-conflict-f-z",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-f-w",
      "name": "@isaacs/testing-peer-optional-conflict-f-w",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/f/node_modules/@isaacs/testing-peer-optional-conflict-f-w",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-f-w/-/testing-peer-optional-conflict-f-w-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-f-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-f-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-optional-conflict-f-w" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-f-w",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-optional-conflict-f-w",
          "type": "peer",
        },
        "@isaacs/testing-peer-optional-conflict-f-y" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-f-y",
          "spec": "1",
          "to": null,
          "type": "peerOptional",
        },
        "@isaacs/testing-peer-optional-conflict-f-z" => EdgeOut {
          "name": "@isaacs/testing-peer-optional-conflict-f-z",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-optional-conflict-f-z",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-f-x",
      "name": "@isaacs/testing-peer-optional-conflict-f-x",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/f/node_modules/@isaacs/testing-peer-optional-conflict-f-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-f-x/-/testing-peer-optional-conflict-f-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-optional-conflict-f-z" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-optional-conflict-f-z",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-optional-conflict-f-w",
          "name": "@isaacs/testing-peer-optional-conflict-f-z",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-optional-conflict-f-x",
          "name": "@isaacs/testing-peer-optional-conflict-f-z",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-optional-conflict-f-z",
      "name": "@isaacs/testing-peer-optional-conflict-f-z",
      "path": "{CWD}/test/fixtures/peer-optional-eresolve/f/node_modules/@isaacs/testing-peer-optional-conflict-f-z",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-optional-conflict-f-z/-/testing-peer-optional-conflict-f-z-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-optional-conflict-f-x" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-f-x",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-f-x",
      "type": "prod",
    },
    "@isaacs/testing-peer-optional-conflict-f-z" => EdgeOut {
      "name": "@isaacs/testing-peer-optional-conflict-f-z",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-optional-conflict-f-z",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "f",
  "packageName": "@isaacs/testing-peer-optional-conflict-f",
  "path": "{CWD}/test/fixtures/peer-optional-eresolve/f",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not add shrinkwrapped deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "error": "MISSING",
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": null,
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/fixtures/shrinkwrapped-dep-no-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "shrinkwrapped-dep-no-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/fixtures/shrinkwrapped-dep-no-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not fail if root peerDep looser than meta peerDep > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/test-peer-looser-than-dev-a-peer-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/test-peer-looser-than-dev-a-peer-dep",
          "spec": ">=0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/test-peer-looser-than-dev-devdep",
          "name": "@isaacs/test-peer-looser-than-dev-a-peer-dep",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/test-peer-looser-than-dev-a-peer-dep",
      "name": "@isaacs/test-peer-looser-than-dev-a-peer-dep",
      "path": "{CWD}/test/fixtures/test-peer-looser-than-dev/node_modules/@isaacs/test-peer-looser-than-dev-a-peer-dep",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/test-peer-looser-than-dev-a-peer-dep/-/test-peer-looser-than-dev-a-peer-dep-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/test-peer-looser-than-dev-devdep" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/test-peer-looser-than-dev-devdep",
          "spec": "1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@isaacs/test-peer-looser-than-dev-a-peer-dep" => EdgeOut {
          "name": "@isaacs/test-peer-looser-than-dev-a-peer-dep",
          "spec": "1",
          "to": "node_modules/@isaacs/test-peer-looser-than-dev-a-peer-dep",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/test-peer-looser-than-dev-devdep",
      "name": "@isaacs/test-peer-looser-than-dev-devdep",
      "path": "{CWD}/test/fixtures/test-peer-looser-than-dev/node_modules/@isaacs/test-peer-looser-than-dev-devdep",
      "resolved": "https://registry.npmjs.org/@isaacs/test-peer-looser-than-dev-devdep/-/test-peer-looser-than-dev-devdep-1.0.1.tgz",
      "version": "1.0.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/test-peer-looser-than-dev-a-peer-dep" => EdgeOut {
      "name": "@isaacs/test-peer-looser-than-dev-a-peer-dep",
      "spec": ">=0",
      "to": "node_modules/@isaacs/test-peer-looser-than-dev-a-peer-dep",
      "type": "peer",
    },
    "@isaacs/test-peer-looser-than-dev-devdep" => EdgeOut {
      "name": "@isaacs/test-peer-looser-than-dev-devdep",
      "spec": "1",
      "to": "node_modules/@isaacs/test-peer-looser-than-dev-devdep",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "test-peer-looser-than-dev",
  "packageName": "@isaacs/test-peer-looser-than-dev",
  "path": "{CWD}/test/fixtures/test-peer-looser-than-dev",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not update shrinkwrapped deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "shrinkwrapped-dep-with-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not update shrinkwrapped deps when complete:true is set > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "shrinkwrapped-dep-with-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP do not update shrinkwrapped deps, ignore lockfile > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "error": "MISSING",
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": null,
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "shrinkwrapped-dep-with-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/fixtures/shrinkwrapped-dep-with-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP dont get confused if root matches duped metadep > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "test-root-matches-metadep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-root-matches-metadep-x",
          "name": "test-root-matches-metadep",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "test-root-matches-metadep-x" => EdgeOut {
          "name": "test-root-matches-metadep-x",
          "spec": "1.0.0",
          "to": "node_modules/test-root-matches-metadep-x",
          "type": "prod",
        },
        "test-root-matches-metadep-y" => EdgeOut {
          "name": "test-root-matches-metadep-y",
          "spec": "1.0.0",
          "to": "node_modules/test-root-matches-metadep-y",
          "type": "prod",
        },
      },
      "location": "node_modules/test-root-matches-metadep",
      "name": "test-root-matches-metadep",
      "path": "{CWD}/test/fixtures/test-root-matches-metadep/node_modules/test-root-matches-metadep",
      "resolved": "https://registry.npmjs.org/test-root-matches-metadep/-/test-root-matches-metadep-1.0.0.tgz",
      "version": "1.0.0",
    },
    "test-root-matches-metadep-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "test-root-matches-metadep-x",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-root-matches-metadep",
          "name": "test-root-matches-metadep-x",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-root-matches-metadep-y/node_modules/test-root-matches-metadep",
          "name": "test-root-matches-metadep-x",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "test-root-matches-metadep" => EdgeOut {
          "name": "test-root-matches-metadep",
          "spec": "1.0.0",
          "to": "node_modules/test-root-matches-metadep",
          "type": "prod",
        },
      },
      "location": "node_modules/test-root-matches-metadep-x",
      "name": "test-root-matches-metadep-x",
      "path": "{CWD}/test/fixtures/test-root-matches-metadep/node_modules/test-root-matches-metadep-x",
      "resolved": "https://registry.npmjs.org/test-root-matches-metadep-x/-/test-root-matches-metadep-x-1.0.0.tgz",
      "version": "1.0.0",
    },
    "test-root-matches-metadep-y" => ArboristNode {
      "children": Map {
        "test-root-matches-metadep" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/test-root-matches-metadep-y",
              "name": "test-root-matches-metadep",
              "spec": "1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "test-root-matches-metadep-x" => EdgeOut {
              "name": "test-root-matches-metadep-x",
              "spec": "1.0.0",
              "to": "node_modules/test-root-matches-metadep-x",
              "type": "prod",
            },
            "test-root-matches-metadep-y" => EdgeOut {
              "name": "test-root-matches-metadep-y",
              "spec": "1.0.0",
              "to": "node_modules/test-root-matches-metadep-y",
              "type": "prod",
            },
          },
          "location": "node_modules/test-root-matches-metadep-y/node_modules/test-root-matches-metadep",
          "name": "test-root-matches-metadep",
          "path": "{CWD}/test/fixtures/test-root-matches-metadep/node_modules/test-root-matches-metadep-y/node_modules/test-root-matches-metadep",
          "resolved": "https://registry.npmjs.org/test-root-matches-metadep/-/test-root-matches-metadep-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "test-root-matches-metadep-y",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-root-matches-metadep",
          "name": "test-root-matches-metadep-y",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-root-matches-metadep-y/node_modules/test-root-matches-metadep",
          "name": "test-root-matches-metadep-y",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "test-root-matches-metadep" => EdgeOut {
          "name": "test-root-matches-metadep",
          "spec": "1.0.1",
          "to": "node_modules/test-root-matches-metadep-y/node_modules/test-root-matches-metadep",
          "type": "prod",
        },
      },
      "location": "node_modules/test-root-matches-metadep-y",
      "name": "test-root-matches-metadep-y",
      "path": "{CWD}/test/fixtures/test-root-matches-metadep/node_modules/test-root-matches-metadep-y",
      "resolved": "https://registry.npmjs.org/test-root-matches-metadep-y/-/test-root-matches-metadep-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "test-root-matches-metadep-x" => EdgeOut {
      "name": "test-root-matches-metadep-x",
      "spec": "1.0.0",
      "to": "node_modules/test-root-matches-metadep-x",
      "type": "prod",
    },
    "test-root-matches-metadep-y" => EdgeOut {
      "name": "test-root-matches-metadep-y",
      "spec": "1.0.0",
      "to": "node_modules/test-root-matches-metadep-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "test-root-matches-metadep",
  "path": "{CWD}/test/fixtures/test-root-matches-metadep",
  "version": "1.0.1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP force a new mkdirp (but not semver major) > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/mkdirp-pinned/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "0.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/mkdirp-pinned/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "0.5.5",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "mkdirp-pinned",
  "path": "{CWD}/test/fixtures/mkdirp-pinned",
}
`

exports[`test/arborist/build-ideal-tree.js TAP force a new nyc (and update mkdirp nicely) > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/core" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/core",
              "name": "semver",
              "spec": "^5.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/core/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/core/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
          "version": "5.7.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/core",
          "spec": "^7.7.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helpers" => EdgeOut {
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helpers",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "gensync" => EdgeOut {
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "to": "node_modules/gensync",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^2.1.2",
          "to": "node_modules/json5",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.3.2",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.4.1",
          "to": "node_modules/@babel/core/node_modules/semver",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/core",
      "name": "@babel/core",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/core",
      "resolved": "https://registry.npmjs.org/@babel/core/-/core-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/generator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-function-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-member-expression-to-functions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-member-expression-to-functions",
      "name": "@babel/helper-member-expression-to-functions",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-member-expression-to-functions",
      "resolved": "https://registry.npmjs.org/@babel/helper-member-expression-to-functions/-/helper-member-expression-to-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-imports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-imports",
      "name": "@babel/helper-module-imports",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-module-imports",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-imports/-/helper-module-imports-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-transforms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-simple-access" => EdgeOut {
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-simple-access",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-transforms",
      "name": "@babel/helper-module-transforms",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-module-transforms",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-transforms/-/helper-module-transforms-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-optimise-call-expression" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-optimise-call-expression",
      "name": "@babel/helper-optimise-call-expression",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-optimise-call-expression",
      "resolved": "https://registry.npmjs.org/@babel/helper-optimise-call-expression/-/helper-optimise-call-expression-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-replace-supers" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-member-expression-to-functions" => EdgeOut {
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-member-expression-to-functions",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-replace-supers",
      "name": "@babel/helper-replace-supers",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-replace-supers",
      "resolved": "https://registry.npmjs.org/@babel/helper-replace-supers/-/helper-replace-supers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-simple-access" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-simple-access",
      "name": "@babel/helper-simple-access",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-simple-access",
      "resolved": "https://registry.npmjs.org/@babel/helper-simple-access/-/helper-simple-access-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.11.0.tgz",
      "version": "7.11.0",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helpers" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helpers",
      "name": "@babel/helpers",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/helpers",
      "resolved": "https://registry.npmjs.org/@babel/helpers/-/helpers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/highlight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/template" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-member-expression-to-functions",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-imports",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-optimise-call-expression",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-simple-access",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@istanbuljs/load-nyc-config" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "@istanbuljs/load-nyc-config",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.3.1",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.1.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-package-type" => EdgeOut {
          "name": "get-package-type",
          "spec": "^0.1.0",
          "to": "node_modules/get-package-type",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^5.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/@istanbuljs/load-nyc-config",
      "name": "@istanbuljs/load-nyc-config",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@istanbuljs/load-nyc-config",
      "resolved": "https://registry.npmjs.org/@istanbuljs/load-nyc-config/-/load-nyc-config-1.1.0.tgz",
      "version": "1.1.0",
    },
    "@istanbuljs/schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@istanbuljs/schema",
      "name": "@istanbuljs/schema",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/@istanbuljs/schema",
      "resolved": "https://registry.npmjs.org/@istanbuljs/schema/-/schema-0.1.2.tgz",
      "version": "0.1.2",
    },
    "aggregate-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-map",
          "name": "aggregate-error",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "clean-stack" => EdgeOut {
          "name": "clean-stack",
          "spec": "^2.0.0",
          "to": "node_modules/clean-stack",
          "type": "prod",
        },
        "indent-string" => EdgeOut {
          "name": "indent-string",
          "spec": "^4.0.0",
          "to": "node_modules/indent-string",
          "type": "prod",
        },
      },
      "location": "node_modules/aggregate-error",
      "name": "aggregate-error",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/aggregate-error",
      "resolved": "https://registry.npmjs.org/aggregate-error/-/aggregate-error-3.1.0.tgz",
      "version": "3.1.0",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
      "version": "5.0.0",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "append-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^3.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-2.0.0.tgz",
      "version": "2.0.0",
    },
    "archy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "argparse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "balanced-match" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "brace-expansion" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "caching-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^5.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^4.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.0",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-4.0.0.tgz",
      "version": "4.0.0",
    },
    "camelcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@istanbuljs/load-nyc-config",
          "name": "camelcase",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "chalk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "clean-stack" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/aggregate-error",
          "name": "clean-stack",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/clean-stack",
      "name": "clean-stack",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/clean-stack",
      "resolved": "https://registry.npmjs.org/clean-stack/-/clean-stack-2.2.0.tgz",
      "version": "2.2.0",
    },
    "cliui" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "cliui",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-6.0.0.tgz",
      "version": "6.0.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "commondir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "cross-spawn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^3.1.0",
          "to": "node_modules/path-key",
          "type": "prod",
        },
        "shebang-command" => EdgeOut {
          "name": "shebang-command",
          "spec": "^2.0.0",
          "to": "node_modules/shebang-command",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.1",
          "to": "node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-7.0.3.tgz",
      "version": "7.0.3",
    },
    "debug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "decamelize" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^4.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-3.0.0.tgz",
      "version": "3.0.0",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "es6-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esprima" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.2",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^4.1.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-3.3.1.tgz",
      "version": "3.3.1",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@istanbuljs/load-nyc-config",
          "name": "find-up",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^5.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^4.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-4.1.0.tgz",
      "version": "4.1.0",
    },
    "foreground-child" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-2.0.0.tgz",
      "version": "2.0.0",
    },
    "fromentries" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/process-on-spawn",
          "name": "fromentries",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fromentries",
      "name": "fromentries",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/fromentries",
      "resolved": "https://registry.npmjs.org/fromentries/-/fromentries-1.2.1.tgz",
      "version": "1.2.1",
    },
    "fs.realpath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "gensync" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "type": "prod",
        },
      },
      "location": "node_modules/gensync",
      "name": "gensync",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/gensync",
      "resolved": "https://registry.npmjs.org/gensync/-/gensync-1.0.0-beta.1.tgz",
      "version": "1.0.0-beta.1",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "get-package-type" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@istanbuljs/load-nyc-config",
          "name": "get-package-type",
          "spec": "^0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "get-package-type",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/get-package-type",
      "name": "get-package-type",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/get-package-type",
      "resolved": "https://registry.npmjs.org/get-package-type/-/get-package-type-0.1.0.tgz",
      "version": "0.1.0",
    },
    "glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "globals" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^2.0.0",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.0",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-5.2.2.tgz",
      "version": "5.2.2",
    },
    "html-escaper" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "html-escaper",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/html-escaper",
      "name": "html-escaper",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/html-escaper",
      "resolved": "https://registry.npmjs.org/html-escaper/-/html-escaper-2.0.2.tgz",
      "version": "2.0.2",
    },
    "imurmurhash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "indent-string" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/aggregate-error",
          "name": "indent-string",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/indent-string",
      "name": "indent-string",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/indent-string",
      "resolved": "https://registry.npmjs.org/indent-string/-/indent-string-4.0.0.tgz",
      "version": "4.0.0",
    },
    "inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-typedarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-windows" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "is-windows",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/is-windows",
      "name": "is-windows",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/is-windows",
      "resolved": "https://registry.npmjs.org/is-windows/-/is-windows-1.0.2.tgz",
      "version": "1.0.2",
    },
    "isexe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0-alpha.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-3.0.0.tgz",
      "version": "3.0.0",
    },
    "istanbul-lib-hook" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^2.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-3.0.0.tgz",
      "version": "3.0.0",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.7.5",
          "to": "node_modules/@babel/core",
          "type": "prod",
        },
        "@istanbuljs/schema" => EdgeOut {
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "to": "node_modules/@istanbuljs/schema",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.3.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-4.0.3.tgz",
      "version": "4.0.3",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-processinfo",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0-alpha.1",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "p-map" => EdgeOut {
          "name": "p-map",
          "spec": "^3.0.0",
          "to": "node_modules/p-map",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^3.0.0",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.3",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-2.0.2.tgz",
      "version": "2.0.2",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "has-flag" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-report/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
          "version": "4.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/istanbul-lib-report/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
          "version": "7.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-3.0.0.tgz",
      "version": "3.0.0",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-source-maps",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-4.0.0.tgz",
      "version": "4.0.0",
    },
    "istanbul-reports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "html-escaper" => EdgeOut {
          "name": "html-escaper",
          "spec": "^2.0.0",
          "to": "node_modules/html-escaper",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-3.0.2.tgz",
      "version": "3.0.2",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@istanbuljs/load-nyc-config",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.14.0.tgz",
      "version": "3.14.0",
    },
    "jsesc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json5" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "json5",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/json5",
      "resolved": "https://registry.npmjs.org/json5/-/json5-2.1.3.tgz",
      "version": "2.1.3",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^4.1.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-5.0.0.tgz",
      "version": "5.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "lodash.flattendeep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "make-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-3.1.0.tgz",
      "version": "3.1.0",
    },
    "minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "ms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "node-preload" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "node-preload",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "process-on-spawn" => EdgeOut {
          "name": "process-on-spawn",
          "spec": "^1.0.0",
          "to": "node_modules/process-on-spawn",
          "type": "prod",
        },
      },
      "location": "node_modules/node-preload",
      "name": "node-preload",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/node-preload",
      "resolved": "https://registry.npmjs.org/node-preload/-/node-preload-0.2.1.tgz",
      "version": "0.2.1",
    },
    "nyc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "nyc",
          "spec": "15.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@istanbuljs/load-nyc-config" => EdgeOut {
          "name": "@istanbuljs/load-nyc-config",
          "spec": "^1.0.0",
          "to": "node_modules/@istanbuljs/load-nyc-config",
          "type": "prod",
        },
        "@istanbuljs/schema" => EdgeOut {
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "to": "node_modules/@istanbuljs/schema",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^4.0.0",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^3.2.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.1.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^2.0.0",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "get-package-type" => EdgeOut {
          "name": "get-package-type",
          "spec": "^0.1.0",
          "to": "node_modules/get-package-type",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^4.0.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^2.0.2",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^4.0.0",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^3.0.2",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "node-preload" => EdgeOut {
          "name": "node-preload",
          "spec": "^0.2.1",
          "to": "node_modules/node-preload",
          "type": "prod",
        },
        "p-map" => EdgeOut {
          "name": "p-map",
          "spec": "^3.0.0",
          "to": "node_modules/p-map",
          "type": "prod",
        },
        "process-on-spawn" => EdgeOut {
          "name": "process-on-spawn",
          "spec": "^1.0.0",
          "to": "node_modules/process-on-spawn",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^5.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^3.0.0",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^2.0.0",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^6.0.0",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^15.0.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-15.1.0.tgz",
      "version": "15.1.0",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
      "version": "2.3.0",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.2.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-4.1.0.tgz",
      "version": "4.1.0",
    },
    "p-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "p-map",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "p-map",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aggregate-error" => EdgeOut {
          "name": "aggregate-error",
          "spec": "^3.0.0",
          "to": "node_modules/aggregate-error",
          "type": "prod",
        },
      },
      "location": "node_modules/p-map",
      "name": "p-map",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/p-map",
      "resolved": "https://registry.npmjs.org/p-map/-/p-map-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^5.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "path-exists",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-3.1.1.tgz",
      "version": "3.1.1",
    },
    "path-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-4.2.0.tgz",
      "version": "4.2.0",
    },
    "process-on-spawn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-preload",
          "name": "process-on-spawn",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "process-on-spawn",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fromentries" => EdgeOut {
          "name": "fromentries",
          "spec": "^1.2.0",
          "to": "node_modules/fromentries",
          "type": "prod",
        },
      },
      "location": "node_modules/process-on-spawn",
      "name": "process-on-spawn",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/process-on-spawn",
      "resolved": "https://registry.npmjs.org/process-on-spawn/-/process-on-spawn-1.0.0.tgz",
      "version": "1.0.0",
    },
    "release-zalgo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "resolve",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.17.0.tgz",
      "version": "1.17.0",
    },
    "resolve-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@istanbuljs/load-nyc-config",
          "name": "resolve-from",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "resolve-from",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-5.0.0.tgz",
      "version": "5.0.0",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-3.0.2.tgz",
      "version": "3.0.2",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/convert-source-map",
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
      "version": "5.1.2",
    },
    "semver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "semver",
          "spec": "^6.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
      "version": "6.3.0",
    },
    "set-blocking" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-command" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-3.0.0.tgz",
      "version": "3.0.0",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
      "version": "0.5.7",
    },
    "spawn-wrap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^2.0.0",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "is-windows" => EdgeOut {
          "name": "is-windows",
          "spec": "^1.0.2",
          "to": "node_modules/is-windows",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^3.0.0",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.1",
          "to": "node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-2.0.0.tgz",
      "version": "2.0.0",
    },
    "sprintf-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "string-width" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
      "version": "6.0.0",
    },
    "strip-bom" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-4.0.0.tgz",
      "version": "4.0.0",
    },
    "supports-color" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "test-exclude" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@istanbuljs/schema" => EdgeOut {
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "to": "node_modules/@istanbuljs/schema",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.4",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-6.0.0.tgz",
      "version": "6.0.0",
    },
    "to-fast-properties" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "type-fest" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "type-fest",
          "spec": "^0.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
      "version": "0.8.1",
    },
    "typedarray-to-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.3",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "which" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "which",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "which",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/wrap-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/wrap-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
          "version": "4.3.0",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/wrap-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/wrap-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/wrap-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/wrap-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
      "version": "6.2.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "write-file-atomic",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.3.tgz",
      "version": "3.0.3",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yargs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^15.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^6.0.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.1.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^18.1.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-15.4.1.tgz",
      "version": "15.4.1",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^18.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/audit-nyc-mkdirp/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-18.1.3.tgz",
      "version": "18.1.3",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^0.5.1",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
    "nyc" => EdgeOut {
      "name": "nyc",
      "spec": "15.1.0",
      "to": "node_modules/nyc",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "audit-nyc-mkdirp",
  "packageName": "fooo",
  "path": "{CWD}/test/fixtures/audit-nyc-mkdirp",
}
`

exports[`test/arborist/build-ideal-tree.js TAP global > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "rimraf" => ArboristNode {
      "children": Map {
        "balanced-match" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/brace-expansion",
              "name": "balanced-match",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/balanced-match",
          "name": "balanced-match",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/balanced-match",
          "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
          "version": "1.0.0",
        },
        "brace-expansion" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/minimatch",
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "balanced-match" => EdgeOut {
              "name": "balanced-match",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/balanced-match",
              "type": "prod",
            },
            "concat-map" => EdgeOut {
              "name": "concat-map",
              "spec": "0.0.1",
              "to": "node_modules/rimraf/node_modules/concat-map",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/brace-expansion",
          "name": "brace-expansion",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/brace-expansion",
          "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
          "version": "1.1.11",
        },
        "concat-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/brace-expansion",
              "name": "concat-map",
              "spec": "0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/concat-map",
          "name": "concat-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/concat-map",
          "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
          "version": "0.0.1",
        },
        "fs.realpath" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/fs.realpath",
          "name": "fs.realpath",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/fs.realpath",
          "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
          "version": "1.0.0",
        },
        "glob" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf",
              "name": "glob",
              "spec": "^7.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/rimraf/node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/rimraf/node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.4",
              "to": "node_modules/rimraf/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/rimraf/node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
          "version": "7.1.6",
        },
        "inflight" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "inflight",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/rimraf/node_modules/once",
              "type": "prod",
            },
            "wrappy" => EdgeOut {
              "name": "wrappy",
              "spec": "1",
              "to": "node_modules/rimraf/node_modules/wrappy",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/inflight",
          "name": "inflight",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/inflight",
          "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
          "version": "1.0.6",
        },
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "inherits",
              "spec": "2",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
          "version": "2.0.4",
        },
        "minimatch" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "to": "node_modules/rimraf/node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
          "version": "3.0.4",
        },
        "once" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "once",
              "spec": "^1.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/inflight",
              "name": "once",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "wrappy" => EdgeOut {
              "name": "wrappy",
              "spec": "1",
              "to": "node_modules/rimraf/node_modules/wrappy",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/once",
          "name": "once",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/once",
          "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
          "version": "1.4.0",
        },
        "path-is-absolute" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/path-is-absolute",
          "name": "path-is-absolute",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/path-is-absolute",
          "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
          "version": "1.0.1",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/inflight",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "rimraf",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/rimraf/node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-3.0.2.tgz",
      "version": "3.0.2",
    },
  },
  "edgesOut": Map {
    "rimraf" => EdgeOut {
      "name": "rimraf",
      "spec": "*",
      "to": "node_modules/rimraf",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-global",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global",
}
`

exports[`test/arborist/build-ideal-tree.js TAP global style > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "rimraf" => ArboristNode {
      "children": Map {
        "balanced-match" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/brace-expansion",
              "name": "balanced-match",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/balanced-match",
          "name": "balanced-match",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/balanced-match",
          "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
          "version": "1.0.0",
        },
        "brace-expansion" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/minimatch",
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "balanced-match" => EdgeOut {
              "name": "balanced-match",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/balanced-match",
              "type": "prod",
            },
            "concat-map" => EdgeOut {
              "name": "concat-map",
              "spec": "0.0.1",
              "to": "node_modules/rimraf/node_modules/concat-map",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/brace-expansion",
          "name": "brace-expansion",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/brace-expansion",
          "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
          "version": "1.1.11",
        },
        "concat-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/brace-expansion",
              "name": "concat-map",
              "spec": "0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/concat-map",
          "name": "concat-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/concat-map",
          "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
          "version": "0.0.1",
        },
        "fs.realpath" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/fs.realpath",
          "name": "fs.realpath",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/fs.realpath",
          "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
          "version": "1.0.0",
        },
        "glob" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf",
              "name": "glob",
              "spec": "^7.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/rimraf/node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/rimraf/node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.4",
              "to": "node_modules/rimraf/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/rimraf/node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/rimraf/node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
          "version": "7.1.6",
        },
        "inflight" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "inflight",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/rimraf/node_modules/once",
              "type": "prod",
            },
            "wrappy" => EdgeOut {
              "name": "wrappy",
              "spec": "1",
              "to": "node_modules/rimraf/node_modules/wrappy",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/inflight",
          "name": "inflight",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/inflight",
          "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
          "version": "1.0.6",
        },
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "inherits",
              "spec": "2",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
          "version": "2.0.4",
        },
        "minimatch" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "to": "node_modules/rimraf/node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
          "version": "3.0.4",
        },
        "once" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "once",
              "spec": "^1.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/inflight",
              "name": "once",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "wrappy" => EdgeOut {
              "name": "wrappy",
              "spec": "1",
              "to": "node_modules/rimraf/node_modules/wrappy",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/once",
          "name": "once",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/once",
          "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
          "version": "1.4.0",
        },
        "path-is-absolute" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/glob",
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/path-is-absolute",
          "name": "path-is-absolute",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/path-is-absolute",
          "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
          "version": "1.0.1",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/inflight",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/rimraf/node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "rimraf",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/rimraf/node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-3.0.2.tgz",
      "version": "3.0.2",
    },
  },
  "edgesOut": Map {
    "rimraf" => EdgeOut {
      "name": "rimraf",
      "spec": "*",
      "to": "node_modules/rimraf",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-global-style",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-global-style",
}
`

exports[`test/arborist/build-ideal-tree.js TAP inflate an ancient lockfile by hitting the registry > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "acorn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn",
          "spec": "4.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/fixtures/sax/node_modules/acorn",
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-4.0.4.tgz",
      "version": "4.0.4",
    },
    "acorn-jsx" => ArboristNode {
      "children": Map {
        "acorn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/acorn-jsx",
              "name": "acorn",
              "spec": "^3.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/acorn-jsx/node_modules/acorn",
          "name": "acorn",
          "path": "{CWD}/test/fixtures/sax/node_modules/acorn-jsx/node_modules/acorn",
          "resolved": "https://registry.npmjs.org/acorn/-/acorn-3.3.0.tgz",
          "version": "3.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^3.0.4",
          "to": "node_modules/acorn-jsx/node_modules/acorn",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn-jsx",
      "name": "acorn-jsx",
      "path": "{CWD}/test/fixtures/sax/node_modules/acorn-jsx",
      "resolved": "https://registry.npmjs.org/acorn-jsx/-/acorn-jsx-3.0.1.tgz",
      "version": "3.0.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv-keywords",
          "name": "ajv",
          "spec": ">=4.10.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv",
          "spec": "^4.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "co" => EdgeOut {
          "name": "co",
          "spec": "^4.6.0",
          "to": "node_modules/co",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/sax/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-4.11.2.tgz",
      "version": "4.11.2",
    },
    "ajv-keywords" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": ">=4.10.0",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-keywords",
      "name": "ajv-keywords",
      "path": "{CWD}/test/fixtures/sax/node_modules/ajv-keywords",
      "resolved": "https://registry.npmjs.org/ajv-keywords/-/ajv-keywords-1.5.1.tgz",
      "version": "1.5.1",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-1.4.0.tgz",
      "version": "1.4.0",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/has-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.1.0.tgz",
      "version": "2.1.0",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/sax/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.9.tgz",
      "version": "1.0.9",
    },
    "array-union" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-uniq" => EdgeOut {
          "name": "array-uniq",
          "spec": "^1.0.1",
          "to": "node_modules/array-uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/fixtures/sax/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-1.0.2.tgz",
      "version": "1.0.2",
    },
    "array-uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-union",
          "name": "array-uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/array-uniq",
      "name": "array-uniq",
      "path": "{CWD}/test/fixtures/sax/node_modules/array-uniq",
      "resolved": "https://registry.npmjs.org/array-uniq/-/array-uniq-1.0.3.tgz",
      "version": "1.0.3",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "arrify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/sax/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/sax/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.3.tgz",
      "version": "0.2.3",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/sax/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-0.2.0.tgz",
      "version": "0.2.0",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/sax/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/sax/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.6.0.tgz",
      "version": "0.6.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/sax/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.6.0.tgz",
      "version": "1.6.0",
    },
    "babel-code-frame" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/babel-code-frame",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/babel-code-frame/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/babel-code-frame/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/babel-code-frame/node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/babel-code-frame",
      "name": "babel-code-frame",
      "path": "{CWD}/test/fixtures/sax/node_modules/babel-code-frame",
      "resolved": "https://registry.npmjs.org/babel-code-frame/-/babel-code-frame-6.22.0.tgz",
      "version": "6.22.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/sax/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-0.3.0.tgz",
      "version": "0.3.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.1.tgz",
      "version": "1.0.1",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/sax/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bluebird" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bluebird",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/bluebird",
      "name": "bluebird",
      "path": "{CWD}/test/fixtures/sax/node_modules/bluebird",
      "resolved": "https://registry.npmjs.org/bluebird/-/bluebird-3.5.0.tgz",
      "version": "3.5.0",
    },
    "boom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cryptiles",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/boom",
      "name": "boom",
      "path": "{CWD}/test/fixtures/sax/node_modules/boom",
      "resolved": "https://registry.npmjs.org/boom/-/boom-2.10.1.tgz",
      "version": "2.10.1",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^0.3.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/sax/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.3.tgz",
      "version": "1.1.3",
    },
    "buffer-shims" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "buffer-shims",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-shims",
      "name": "buffer-shims",
      "path": "{CWD}/test/fixtures/sax/node_modules/buffer-shims",
      "resolved": "https://registry.npmjs.org/buffer-shims/-/buffer-shims-1.0.0.tgz",
      "version": "1.0.0",
    },
    "caller-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "caller-path",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^0.2.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/fixtures/sax/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-0.1.0.tgz",
      "version": "0.1.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "callsites",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/sax/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-0.2.0.tgz",
      "version": "0.2.0",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/sax/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.11.0.tgz",
      "version": "0.11.0",
    },
    "chalk" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/chalk",
              "name": "supports-color",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/chalk/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/sax/node_modules/chalk/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "chalk",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "chalk",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "has-ansi" => EdgeOut {
          "name": "has-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/has-ansi",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^2.0.0",
          "to": "node_modules/chalk/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/sax/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.1.tgz",
      "version": "1.1.1",
    },
    "circular-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "circular-json",
          "spec": "^0.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/circular-json",
      "name": "circular-json",
      "path": "{CWD}/test/fixtures/sax/node_modules/circular-json",
      "resolved": "https://registry.npmjs.org/circular-json/-/circular-json-0.3.1.tgz",
      "version": "0.3.1",
    },
    "clean-yaml-object" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/clean-yaml-object",
      "name": "clean-yaml-object",
      "path": "{CWD}/test/fixtures/sax/node_modules/clean-yaml-object",
      "resolved": "https://registry.npmjs.org/clean-yaml-object/-/clean-yaml-object-0.1.0.tgz",
      "version": "0.1.0",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/sax/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-1.0.2.tgz",
      "version": "1.0.2",
    },
    "cli-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-width",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-width",
      "name": "cli-width",
      "path": "{CWD}/test/fixtures/sax/node_modules/cli-width",
      "resolved": "https://registry.npmjs.org/cli-width/-/cli-width-2.1.0.tgz",
      "version": "2.1.0",
    },
    "co" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "co",
          "spec": "^4.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/co",
      "name": "co",
      "path": "{CWD}/test/fixtures/sax/node_modules/co",
      "resolved": "https://registry.npmjs.org/co/-/co-4.6.0.tgz",
      "version": "4.6.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/sax/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.0.0.tgz",
      "version": "1.0.0",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/sax/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.2.tgz",
      "version": "1.1.2",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.5.tgz",
      "version": "1.0.5",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "concat-stream" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "inherits",
              "spec": "^2.0.3",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "readable-stream" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "readable-stream",
              "spec": "^2.2.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "buffer-shims" => EdgeOut {
              "name": "buffer-shims",
              "spec": "^1.0.0",
              "to": "node_modules/buffer-shims",
              "type": "prod",
            },
            "core-util-is" => EdgeOut {
              "name": "core-util-is",
              "spec": "~1.0.0",
              "to": "node_modules/core-util-is",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "~2.0.1",
              "to": "node_modules/concat-stream/node_modules/inherits",
              "type": "prod",
            },
            "isarray" => EdgeOut {
              "name": "isarray",
              "spec": "~1.0.0",
              "to": "node_modules/concat-stream/node_modules/isarray",
              "type": "prod",
            },
            "process-nextick-args" => EdgeOut {
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "to": "node_modules/concat-stream/node_modules/process-nextick-args",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "~0.10.x",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "~1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/readable-stream",
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.2.2.tgz",
          "version": "2.2.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "concat-stream",
          "spec": "^1.4.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/concat-stream/node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.2.2",
          "to": "node_modules/concat-stream/node_modules/readable-stream",
          "type": "prod",
        },
        "typedarray" => EdgeOut {
          "name": "typedarray",
          "spec": "^0.0.6",
          "to": "node_modules/typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-stream",
      "name": "concat-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream",
      "resolved": "https://registry.npmjs.org/concat-stream/-/concat-stream-1.6.0.tgz",
      "version": "1.6.0",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/sax/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "children": Map {
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls/node_modules/js-yaml",
              "name": "esprima",
              "spec": "^2.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.7.3.tgz",
          "version": "2.7.3",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "js-yaml",
              "spec": "3.6.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^2.6.0",
              "to": "node_modules/coveralls/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.6.1.tgz",
          "version": "3.6.1",
        },
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "minimist",
              "spec": "1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^2.11.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "3.6.1",
          "to": "node_modules/coveralls/node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "0.0.10",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "1.2.5",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "1.2.0",
          "to": "node_modules/coveralls/node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "2.79.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/sax/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-2.11.16.tgz",
      "version": "2.11.16",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "lru-cache" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "lru-cache",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pseudomap" => EdgeOut {
              "name": "pseudomap",
              "spec": "^1.0.1",
              "to": "node_modules/pseudomap",
              "type": "prod",
            },
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^2.0.0",
              "to": "node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "lru-cache",
          "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn/node_modules/lru-cache",
          "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.0.2.tgz",
          "version": "4.0.2",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^1.1.1",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.2.12.tgz",
          "version": "1.2.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/cross-spawn/node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "cryptiles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "cryptiles",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
      },
      "location": "node_modules/cryptiles",
      "name": "cryptiles",
      "path": "{CWD}/test/fixtures/sax/node_modules/cryptiles",
      "resolved": "https://registry.npmjs.org/cryptiles/-/cryptiles-2.0.5.tgz",
      "version": "2.0.5",
    },
    "d" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.2",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/d",
      "name": "d",
      "path": "{CWD}/test/fixtures/sax/node_modules/d",
      "resolved": "https://registry.npmjs.org/d/-/d-0.1.1.tgz",
      "version": "0.1.1",
    },
    "dashdash" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/dashdash",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/dashdash/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/dashdash/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/dashdash/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/sax/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "debug",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "debug",
          "spec": "^2.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "0.7.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/sax/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-2.2.0.tgz",
      "version": "2.2.0",
    },
    "debug-log" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "debug-log",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/debug-log",
      "name": "debug-log",
      "path": "{CWD}/test/fixtures/sax/node_modules/debug-log",
      "resolved": "https://registry.npmjs.org/debug-log/-/debug-log-1.0.1.tgz",
      "version": "1.0.1",
    },
    "deep-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "deep-is",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-is",
      "name": "deep-is",
      "path": "{CWD}/test/fixtures/sax/node_modules/deep-is",
      "resolved": "https://registry.npmjs.org/deep-is/-/deep-is-0.1.3.tgz",
      "version": "0.1.3",
    },
    "deglob" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob",
              "name": "glob",
              "spec": "^7.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/deglob/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/deglob/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/deglob/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "deglob",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/deglob/node_modules/glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.0.9",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.1.0",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.2",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
        "uniq" => EdgeOut {
          "name": "uniq",
          "spec": "^1.0.1",
          "to": "node_modules/uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/deglob",
      "name": "deglob",
      "path": "{CWD}/test/fixtures/sax/node_modules/deglob",
      "resolved": "https://registry.npmjs.org/deglob/-/deglob-2.1.0.tgz",
      "version": "2.1.0",
    },
    "del" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "del",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^5.0.0",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-path-cwd" => EdgeOut {
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-cwd",
          "type": "prod",
        },
        "is-path-in-cwd" => EdgeOut {
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-in-cwd",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.2.8",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "node_modules/del",
      "name": "del",
      "path": "{CWD}/test/fixtures/sax/node_modules/del",
      "resolved": "https://registry.npmjs.org/del/-/del-2.2.2.tgz",
      "version": "2.2.2",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "diff",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/sax/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
      "version": "1.4.0",
    },
    "doctrine" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/doctrine/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "isarray",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/fixtures/sax/node_modules/doctrine/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/doctrine/node_modules/esutils",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "^1.0.0",
          "to": "node_modules/doctrine/node_modules/isarray",
          "type": "prod",
        },
      },
      "location": "node_modules/doctrine",
      "name": "doctrine",
      "path": "{CWD}/test/fixtures/sax/node_modules/doctrine",
      "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-1.5.0.tgz",
      "version": "1.5.0",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "es5-ext" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/d",
          "name": "es5-ext",
          "spec": "~0.10.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es5-ext",
          "spec": "^0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es5-ext",
          "spec": "^0.10.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es5-ext",
      "name": "es5-ext",
      "path": "{CWD}/test/fixtures/sax/node_modules/es5-ext",
      "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.8.tgz",
      "version": "0.10.8",
    },
    "es6-iterator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-iterator",
      "name": "es6-iterator",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-iterator",
      "resolved": "https://registry.npmjs.org/es6-iterator/-/es6-iterator-2.0.0.tgz",
      "version": "2.0.0",
    },
    "es6-map" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-map/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-map/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
        "es6-symbol" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es6-symbol",
              "spec": "~3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es5-ext",
              "name": "es6-symbol",
              "spec": "~3.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "d" => EdgeOut {
              "name": "d",
              "spec": "~0.1.1",
              "to": "node_modules/d",
              "type": "prod",
            },
            "es5-ext" => EdgeOut {
              "name": "es5-ext",
              "spec": "~0.10.11",
              "to": "node_modules/es6-map/node_modules/es5-ext",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "es6-symbol",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-map/node_modules/es6-symbol",
          "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-map",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-map/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-set" => EdgeOut {
          "name": "es6-set",
          "spec": "~0.1.3",
          "to": "node_modules/es6-set",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "~3.1.0",
          "to": "node_modules/es6-map/node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-map",
      "name": "es6-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-map",
      "resolved": "https://registry.npmjs.org/es6-map/-/es6-map-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-set" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "children": Map {
            "es6-symbol" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/es6-set/node_modules/es5-ext",
                  "name": "es6-symbol",
                  "spec": "~3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "d" => EdgeOut {
                  "name": "d",
                  "spec": "~0.1.1",
                  "to": "node_modules/d",
                  "type": "prod",
                },
                "es5-ext" => EdgeOut {
                  "name": "es5-ext",
                  "spec": "~0.10.11",
                  "to": "node_modules/es6-set/node_modules/es5-ext",
                  "type": "prod",
                },
              },
              "location": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es6-symbol",
              "path": "{CWD}/test/fixtures/sax/node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
              "version": "3.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-set",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-set/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-set",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-set/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-set",
      "name": "es6-set",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-set",
      "resolved": "https://registry.npmjs.org/es6-set/-/es6-set-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-symbol" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-symbol",
      "name": "es6-symbol",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-symbol",
      "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.0.0.tgz",
      "version": "3.0.0",
    },
    "es6-weak-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.8",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-weak-map",
      "name": "es6-weak-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-weak-map",
      "resolved": "https://registry.npmjs.org/es6-weak-map/-/es6-weak-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/sax/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.3.tgz",
      "version": "1.0.3",
    },
    "escope" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/escope",
              "name": "estraverse",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/escope/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/escope/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "escope",
          "spec": "^3.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-map" => EdgeOut {
          "name": "es6-map",
          "spec": "^0.1.3",
          "to": "node_modules/es6-map",
          "type": "prod",
        },
        "es6-weak-map" => EdgeOut {
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "to": "node_modules/es6-weak-map",
          "type": "prod",
        },
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.1.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/escope/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/escope",
      "name": "escope",
      "path": "{CWD}/test/fixtures/sax/node_modules/escope",
      "resolved": "https://registry.npmjs.org/escope/-/escope-3.6.0.tgz",
      "version": "3.6.0",
    },
    "eslint" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.2.1.tgz",
          "version": "2.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "chalk",
              "spec": "^1.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "to": "node_modules/eslint/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.2",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "has-ansi" => EdgeOut {
              "name": "has-ansi",
              "spec": "^2.0.0",
              "to": "node_modules/has-ansi",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^2.0.0",
              "to": "node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.3.tgz",
          "version": "1.1.3",
        },
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "estraverse",
              "spec": "^4.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "fast-levenshtein" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/fast-levenshtein",
          "name": "fast-levenshtein",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/fast-levenshtein",
          "resolved": "https://registry.npmjs.org/fast-levenshtein/-/fast-levenshtein-2.0.6.tgz",
          "version": "2.0.6",
        },
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/eslint/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "js-yaml",
              "spec": "^3.5.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^3.1.1",
              "to": "node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.8.1.tgz",
          "version": "3.8.1",
        },
        "levn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "levn",
              "spec": "^0.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "levn",
              "spec": "~0.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/levn",
          "name": "levn",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/levn",
          "resolved": "https://registry.npmjs.org/levn/-/levn-0.3.0.tgz",
          "version": "0.3.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
        "optionator" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "optionator",
              "spec": "^0.8.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "deep-is" => EdgeOut {
              "name": "deep-is",
              "spec": "~0.1.3",
              "to": "node_modules/deep-is",
              "type": "prod",
            },
            "fast-levenshtein" => EdgeOut {
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "to": "node_modules/eslint/node_modules/fast-levenshtein",
              "type": "prod",
            },
            "levn" => EdgeOut {
              "name": "levn",
              "spec": "~0.3.0",
              "to": "node_modules/eslint/node_modules/levn",
              "type": "prod",
            },
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
            "wordwrap" => EdgeOut {
              "name": "wordwrap",
              "spec": "~1.0.0",
              "to": "node_modules/wordwrap",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/optionator",
          "name": "optionator",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/optionator",
          "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.8.2.tgz",
          "version": "0.8.2",
        },
        "strip-bom" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "strip-bom",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/strip-bom",
          "name": "strip-bom",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/strip-bom",
          "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
          "version": "3.0.0",
        },
        "type-check" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/levn",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/type-check",
          "name": "type-check",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/type-check",
          "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.3.2.tgz",
          "version": "0.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint",
          "spec": ">=3.8.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-standard",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint",
          "spec": "~3.10.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "babel-code-frame" => EdgeOut {
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "to": "node_modules/babel-code-frame",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.3",
          "to": "node_modules/eslint/node_modules/chalk",
          "type": "prod",
        },
        "concat-stream" => EdgeOut {
          "name": "concat-stream",
          "spec": "^1.4.6",
          "to": "node_modules/concat-stream",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "escope" => EdgeOut {
          "name": "escope",
          "spec": "^3.6.0",
          "to": "node_modules/escope",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^3.3.1",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.2.0",
          "to": "node_modules/eslint/node_modules/estraverse",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/eslint/node_modules/esutils",
          "type": "prod",
        },
        "file-entry-cache" => EdgeOut {
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "to": "node_modules/file-entry-cache",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/eslint/node_modules/glob",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^9.2.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.2.0",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "inquirer" => EdgeOut {
          "name": "inquirer",
          "spec": "^0.12.0",
          "to": "node_modules/inquirer",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "to": "node_modules/is-my-json-valid",
          "type": "prod",
        },
        "is-resolvable" => EdgeOut {
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "to": "node_modules/is-resolvable",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.5.1",
          "to": "node_modules/eslint/node_modules/js-yaml",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.3.0",
          "to": "node_modules/eslint/node_modules/levn",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/eslint/node_modules/lodash",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "natural-compare" => EdgeOut {
          "name": "natural-compare",
          "spec": "^1.4.0",
          "to": "node_modules/natural-compare",
          "type": "prod",
        },
        "optionator" => EdgeOut {
          "name": "optionator",
          "spec": "^0.8.2",
          "to": "node_modules/eslint/node_modules/optionator",
          "type": "prod",
        },
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
        "pluralize" => EdgeOut {
          "name": "pluralize",
          "spec": "^1.2.1",
          "to": "node_modules/pluralize",
          "type": "prod",
        },
        "progress" => EdgeOut {
          "name": "progress",
          "spec": "^1.1.8",
          "to": "node_modules/progress",
          "type": "prod",
        },
        "require-uncached" => EdgeOut {
          "name": "require-uncached",
          "spec": "^1.0.2",
          "to": "node_modules/require-uncached",
          "type": "prod",
        },
        "shelljs" => EdgeOut {
          "name": "shelljs",
          "spec": "^0.7.5",
          "to": "node_modules/shelljs",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/eslint/node_modules/strip-bom",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
        "table" => EdgeOut {
          "name": "table",
          "spec": "^3.7.8",
          "to": "node_modules/table",
          "type": "prod",
        },
        "text-table" => EdgeOut {
          "name": "text-table",
          "spec": "~0.2.0",
          "to": "node_modules/text-table",
          "type": "prod",
        },
        "user-home" => EdgeOut {
          "name": "user-home",
          "spec": "^2.0.0",
          "to": "node_modules/user-home",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint",
      "name": "eslint",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint",
      "resolved": "https://registry.npmjs.org/eslint/-/eslint-3.10.2.tgz",
      "version": "3.10.2",
    },
    "eslint-config-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.8.1",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "peer",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "to": "node_modules/eslint-plugin-standard",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard",
      "name": "eslint-config-standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-config-standard",
      "resolved": "https://registry.npmjs.org/eslint-config-standard/-/eslint-config-standard-6.2.1.tgz",
      "version": "6.2.1",
    },
    "eslint-config-standard-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard-jsx",
      "name": "eslint-config-standard-jsx",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-config-standard-jsx",
      "resolved": "https://registry.npmjs.org/eslint-config-standard-jsx/-/eslint-config-standard-jsx-3.2.0.tgz",
      "version": "3.2.0",
    },
    "eslint-plugin-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-promise",
      "name": "eslint-plugin-promise",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-promise",
      "resolved": "https://registry.npmjs.org/eslint-plugin-promise/-/eslint-plugin-promise-3.4.1.tgz",
      "version": "3.4.1",
    },
    "eslint-plugin-react" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "jsx-ast-utils" => EdgeOut {
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "to": "node_modules/jsx-ast-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-react",
      "name": "eslint-plugin-react",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-react",
      "resolved": "https://registry.npmjs.org/eslint-plugin-react/-/eslint-plugin-react-6.7.1.tgz",
      "version": "6.7.1",
    },
    "eslint-plugin-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-plugin-standard",
      "name": "eslint-plugin-standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-standard",
      "resolved": "https://registry.npmjs.org/eslint-plugin-standard/-/eslint-plugin-standard-2.0.1.tgz",
      "version": "2.0.1",
    },
    "espree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "espree",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "4.0.4",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "acorn-jsx" => EdgeOut {
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/acorn-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/espree",
      "name": "espree",
      "path": "{CWD}/test/fixtures/sax/node_modules/espree",
      "resolved": "https://registry.npmjs.org/espree/-/espree-3.4.0.tgz",
      "version": "3.4.0",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "esprima",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/sax/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-3.1.3.tgz",
      "version": "3.1.3",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "~4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/esrecurse/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.1.1.tgz",
          "version": "4.1.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "esrecurse",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "~4.1.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/fixtures/sax/node_modules/esrecurse",
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.1.0.tgz",
      "version": "4.1.0",
    },
    "event-emitter" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/event-emitter",
      "name": "event-emitter",
      "path": "{CWD}/test/fixtures/sax/node_modules/event-emitter",
      "resolved": "https://registry.npmjs.org/event-emitter/-/event-emitter-0.3.4.tgz",
      "version": "0.3.4",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/sax/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.0.2.tgz",
      "version": "1.0.2",
    },
    "exit-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "exit-hook",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/exit-hook",
      "name": "exit-hook",
      "path": "{CWD}/test/fixtures/sax/node_modules/exit-hook",
      "resolved": "https://registry.npmjs.org/exit-hook/-/exit-hook-1.1.1.tgz",
      "version": "1.1.1",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/sax/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "figures" => ArboristNode {
      "children": Map {
        "escape-string-regexp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/figures",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/figures/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/sax/node_modules/figures/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
          "version": "1.0.5",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "figures",
          "spec": "^1.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/figures/node_modules/escape-string-regexp",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/figures",
      "name": "figures",
      "path": "{CWD}/test/fixtures/sax/node_modules/figures",
      "resolved": "https://registry.npmjs.org/figures/-/figures-1.7.0.tgz",
      "version": "1.7.0",
    },
    "file-entry-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flat-cache" => EdgeOut {
          "name": "flat-cache",
          "spec": "^1.2.1",
          "to": "node_modules/flat-cache",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/file-entry-cache",
      "name": "file-entry-cache",
      "path": "{CWD}/test/fixtures/sax/node_modules/file-entry-cache",
      "resolved": "https://registry.npmjs.org/file-entry-cache/-/file-entry-cache-2.0.0.tgz",
      "version": "2.0.0",
    },
    "find-root" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/find-root",
      "name": "find-root",
      "path": "{CWD}/test/fixtures/sax/node_modules/find-root",
      "resolved": "https://registry.npmjs.org/find-root/-/find-root-1.0.0.tgz",
      "version": "1.0.0",
    },
    "flat-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "flat-cache",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "circular-json" => EdgeOut {
          "name": "circular-json",
          "spec": "^0.3.1",
          "to": "node_modules/circular-json",
          "type": "prod",
        },
        "del" => EdgeOut {
          "name": "del",
          "spec": "^2.0.2",
          "to": "node_modules/del",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "write" => EdgeOut {
          "name": "write",
          "spec": "^0.2.1",
          "to": "node_modules/write",
          "type": "prod",
        },
      },
      "location": "node_modules/flat-cache",
      "name": "flat-cache",
      "path": "{CWD}/test/fixtures/sax/node_modules/flat-cache",
      "resolved": "https://registry.npmjs.org/flat-cache/-/flat-cache-1.2.2.tgz",
      "version": "1.2.2",
    },
    "foreground-child" => ArboristNode {
      "children": Map {
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/foreground-child",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/foreground-child/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/fixtures/sax/node_modules/foreground-child/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/foreground-child/node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/sax/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/sax/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/sax/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.1.2.tgz",
      "version": "2.1.2",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/sax/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/sax/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/sax/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.1.tgz",
      "version": "1.0.1",
    },
    "generate-function" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-function",
      "name": "generate-function",
      "path": "{CWD}/test/fixtures/sax/node_modules/generate-function",
      "resolved": "https://registry.npmjs.org/generate-function/-/generate-function-2.0.0.tgz",
      "version": "2.0.0",
    },
    "generate-object-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-property" => EdgeOut {
          "name": "is-property",
          "spec": "^1.0.0",
          "to": "node_modules/is-property",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-object-property",
      "name": "generate-object-property",
      "path": "{CWD}/test/fixtures/sax/node_modules/generate-object-property",
      "resolved": "https://registry.npmjs.org/generate-object-property/-/generate-object-property-1.2.0.tgz",
      "version": "1.2.0",
    },
    "get-stdin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "get-stdin",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stdin",
      "name": "get-stdin",
      "path": "{CWD}/test/fixtures/sax/node_modules/get-stdin",
      "resolved": "https://registry.npmjs.org/get-stdin/-/get-stdin-5.0.1.tgz",
      "version": "5.0.1",
    },
    "getpass" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/getpass",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/getpass/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/getpass/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/getpass/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/sax/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.6.tgz",
      "version": "0.1.6",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/sax/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.2.tgz",
      "version": "7.1.2",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "globals",
          "spec": "^9.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/sax/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-9.14.0.tgz",
      "version": "9.14.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/globby/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/globby/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/globby/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "globby",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^1.0.1",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^1.0.0",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/globby/node_modules/glob",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/fixtures/sax/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-5.0.0.tgz",
      "version": "5.0.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/sax/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.1.11.tgz",
      "version": "4.1.11",
    },
    "graceful-readlink" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/commander",
          "name": "graceful-readlink",
          "spec": ">= 1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-readlink",
      "name": "graceful-readlink",
      "path": "{CWD}/test/fixtures/sax/node_modules/graceful-readlink",
      "resolved": "https://registry.npmjs.org/graceful-readlink/-/graceful-readlink-1.0.1.tgz",
      "version": "1.0.1",
    },
    "har-validator" => ArboristNode {
      "children": Map {
        "commander" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "commander",
              "spec": "^2.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-readlink" => EdgeOut {
              "name": "graceful-readlink",
              "spec": ">= 1.0.0",
              "to": "node_modules/graceful-readlink",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/commander",
          "name": "commander",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/commander",
          "resolved": "https://registry.npmjs.org/commander/-/commander-2.9.0.tgz",
          "version": "2.9.0",
        },
        "is-my-json-valid" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "is-my-json-valid",
              "spec": "^2.12.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "generate-function" => EdgeOut {
              "name": "generate-function",
              "spec": "^2.0.0",
              "to": "node_modules/generate-function",
              "type": "prod",
            },
            "generate-object-property" => EdgeOut {
              "name": "generate-object-property",
              "spec": "^1.1.0",
              "to": "node_modules/generate-object-property",
              "type": "prod",
            },
            "jsonpointer" => EdgeOut {
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "to": "node_modules/har-validator/node_modules/jsonpointer",
              "type": "prod",
            },
            "xtend" => EdgeOut {
              "name": "xtend",
              "spec": "^4.0.0",
              "to": "node_modules/xtend",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "is-my-json-valid",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/is-my-json-valid",
          "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
          "version": "2.15.0",
        },
        "jsonpointer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/is-my-json-valid",
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/jsonpointer",
          "name": "jsonpointer",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/jsonpointer",
          "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
          "version": "4.0.1",
        },
        "pinkie" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/pinkie-promise",
              "name": "pinkie",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie",
          "name": "pinkie",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/pinkie",
          "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
          "version": "2.0.4",
        },
        "pinkie-promise" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "pinkie-promise",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pinkie" => EdgeOut {
              "name": "pinkie",
              "spec": "^2.0.0",
              "to": "node_modules/har-validator/node_modules/pinkie",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie-promise",
          "name": "pinkie-promise",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/pinkie-promise",
          "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
          "version": "2.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~2.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "^2.9.0",
          "to": "node_modules/har-validator/node_modules/commander",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.12.4",
          "to": "node_modules/har-validator/node_modules/is-my-json-valid",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/har-validator/node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/sax/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-2.0.6.tgz",
      "version": "2.0.6",
    },
    "has-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/has-ansi",
      "name": "has-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/has-ansi",
      "resolved": "https://registry.npmjs.org/has-ansi/-/has-ansi-2.0.0.tgz",
      "version": "2.0.0",
    },
    "hawk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "hawk",
          "spec": "~3.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
        "cryptiles" => EdgeOut {
          "name": "cryptiles",
          "spec": "2.x.x",
          "to": "node_modules/cryptiles",
          "type": "prod",
        },
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
        "sntp" => EdgeOut {
          "name": "sntp",
          "spec": "1.x.x",
          "to": "node_modules/sntp",
          "type": "prod",
        },
      },
      "location": "node_modules/hawk",
      "name": "hawk",
      "path": "{CWD}/test/fixtures/sax/node_modules/hawk",
      "resolved": "https://registry.npmjs.org/hawk/-/hawk-3.1.3.tgz",
      "version": "3.1.3",
    },
    "hoek" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boom",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sntp",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "location": "node_modules/hoek",
      "name": "hoek",
      "path": "{CWD}/test/fixtures/sax/node_modules/hoek",
      "resolved": "https://registry.npmjs.org/hoek/-/hoek-2.16.3.tgz",
      "version": "2.16.3",
    },
    "home-or-tmp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "os-tmpdir" => EdgeOut {
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "to": "node_modules/os-tmpdir",
          "type": "prod",
        },
      },
      "location": "node_modules/home-or-tmp",
      "name": "home-or-tmp",
      "path": "{CWD}/test/fixtures/sax/node_modules/home-or-tmp",
      "resolved": "https://registry.npmjs.org/home-or-tmp/-/home-or-tmp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^0.2.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/sax/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ignore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "ignore",
          "spec": "^3.0.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ignore",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ignore",
      "name": "ignore",
      "path": "{CWD}/test/fixtures/sax/node_modules/ignore",
      "resolved": "https://registry.npmjs.org/ignore/-/ignore-3.2.2.tgz",
      "version": "3.2.2",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/sax/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/sax/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.4.tgz",
      "version": "1.0.4",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/sax/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.1.tgz",
      "version": "2.0.1",
    },
    "inquirer" => ArboristNode {
      "children": Map {
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/inquirer",
              "name": "lodash",
              "spec": "^4.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/inquirer/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/inquirer/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "inquirer",
          "spec": "^0.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-width" => EdgeOut {
          "name": "cli-width",
          "spec": "^2.0.0",
          "to": "node_modules/cli-width",
          "type": "prod",
        },
        "figures" => EdgeOut {
          "name": "figures",
          "spec": "^1.3.5",
          "to": "node_modules/figures",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.3.0",
          "to": "node_modules/inquirer/node_modules/lodash",
          "type": "prod",
        },
        "readline2" => EdgeOut {
          "name": "readline2",
          "spec": "^1.0.1",
          "to": "node_modules/readline2",
          "type": "prod",
        },
        "run-async" => EdgeOut {
          "name": "run-async",
          "spec": "^0.1.0",
          "to": "node_modules/run-async",
          "type": "prod",
        },
        "rx-lite" => EdgeOut {
          "name": "rx-lite",
          "spec": "^3.1.2",
          "to": "node_modules/rx-lite",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.1",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "through" => EdgeOut {
          "name": "through",
          "spec": "^2.3.6",
          "to": "node_modules/through",
          "type": "prod",
        },
      },
      "location": "node_modules/inquirer",
      "name": "inquirer",
      "path": "{CWD}/test/fixtures/sax/node_modules/inquirer",
      "resolved": "https://registry.npmjs.org/inquirer/-/inquirer-0.12.0.tgz",
      "version": "0.12.0",
    },
    "interpret" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "interpret",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/interpret",
      "name": "interpret",
      "path": "{CWD}/test/fixtures/sax/node_modules/interpret",
      "resolved": "https://registry.npmjs.org/interpret/-/interpret-1.0.1.tgz",
      "version": "1.0.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-my-json-valid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "generate-function" => EdgeOut {
          "name": "generate-function",
          "spec": "^2.0.0",
          "to": "node_modules/generate-function",
          "type": "prod",
        },
        "generate-object-property" => EdgeOut {
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "to": "node_modules/generate-object-property",
          "type": "prod",
        },
        "jsonpointer" => EdgeOut {
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "to": "node_modules/jsonpointer",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.0",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/is-my-json-valid",
      "name": "is-my-json-valid",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-my-json-valid",
      "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
      "version": "2.15.0",
    },
    "is-path-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-cwd",
      "name": "is-path-cwd",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-cwd",
      "resolved": "https://registry.npmjs.org/is-path-cwd/-/is-path-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-in-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-path-inside" => EdgeOut {
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-in-cwd",
      "name": "is-path-in-cwd",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-in-cwd",
      "resolved": "https://registry.npmjs.org/is-path-in-cwd/-/is-path-in-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-path-in-cwd",
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-inside",
      "name": "is-path-inside",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-inside",
      "resolved": "https://registry.npmjs.org/is-path-inside/-/is-path-inside-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/generate-object-property",
          "name": "is-property",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-property",
      "name": "is-property",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-property",
      "resolved": "https://registry.npmjs.org/is-property/-/is-property-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-resolvable" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tryit" => EdgeOut {
          "name": "tryit",
          "spec": "^1.0.1",
          "to": "node_modules/tryit",
          "type": "prod",
        },
      },
      "location": "node_modules/is-resolvable",
      "name": "is-resolvable",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-resolvable",
      "resolved": "https://registry.npmjs.org/is-resolvable/-/is-resolvable-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-0.0.1.tgz",
      "version": "0.0.1",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/sax/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-1.1.2.tgz",
      "version": "1.1.2",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/sax/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "jodid25519" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jodid25519",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/jodid25519",
      "name": "jodid25519",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/jodid25519",
      "resolved": "https://registry.npmjs.org/jodid25519/-/jodid25519-1.0.2.tgz",
      "version": "1.0.2",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "js-tokens",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/sax/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-3.0.1.tgz",
      "version": "3.0.1",
    },
    "js-yaml" => ArboristNode {
      "children": Map {
        "argparse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "argparse",
              "spec": "~1.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": ">= 3.2.0 < 4.0.0",
              "to": "node_modules/lodash",
              "type": "prod",
            },
            "sprintf-js" => EdgeOut {
              "name": "sprintf-js",
              "spec": "~1.0.2",
              "to": "node_modules/sprintf-js",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/argparse",
          "name": "argparse",
          "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml/node_modules/argparse",
          "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.2.tgz",
          "version": "1.0.2",
        },
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "esprima",
              "spec": "~2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.2.0.tgz",
          "version": "2.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "js-yaml",
          "spec": "^3.2.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "~1.0.2",
          "to": "node_modules/js-yaml/node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "~2.2.0",
          "to": "node_modules/js-yaml/node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.4.2.tgz",
      "version": "3.4.2",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jodid25519",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "optional",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.0.tgz",
      "version": "0.1.0",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsonify" => EdgeOut {
          "name": "jsonify",
          "spec": "~0.0.0",
          "to": "node_modules/jsonify",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stable-stringify",
      "name": "json-stable-stringify",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-stable-stringify",
      "resolved": "https://registry.npmjs.org/json-stable-stringify/-/json-stable-stringify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsonify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/json-stable-stringify",
          "name": "jsonify",
          "spec": "~0.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonify",
      "name": "jsonify",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsonify",
      "resolved": "https://registry.npmjs.org/jsonify/-/jsonify-0.0.0.tgz",
      "version": "0.0.0",
    },
    "jsonpointer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonpointer",
      "name": "jsonpointer",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsonpointer",
      "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
      "version": "4.0.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.3.6",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.3.1.tgz",
      "version": "1.3.1",
    },
    "jsx-ast-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/jsx-ast-utils",
      "name": "jsx-ast-utils",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsx-ast-utils",
      "resolved": "https://registry.npmjs.org/jsx-ast-utils/-/jsx-ast-utils-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "0.0.10",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/sax/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-0.0.10.tgz",
      "version": "0.0.10",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "lodash",
          "spec": ">= 3.2.0 < 4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/sax/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-3.10.1.tgz",
      "version": "3.10.1",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/sax/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "~1.26.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/sax/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.26.0.tgz",
      "version": "1.26.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "~1.26.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/sax/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.14.tgz",
      "version": "2.1.14",
    },
    "minimatch" => ArboristNode {
      "children": Map {
        "balanced-match" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch/node_modules/brace-expansion",
              "name": "balanced-match",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/balanced-match",
          "name": "balanced-match",
          "path": "{CWD}/test/fixtures/sax/node_modules/minimatch/node_modules/balanced-match",
          "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
          "version": "1.0.0",
        },
        "brace-expansion" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch",
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "balanced-match" => EdgeOut {
              "name": "balanced-match",
              "spec": "^1.0.0",
              "to": "node_modules/minimatch/node_modules/balanced-match",
              "type": "prod",
            },
            "concat-map" => EdgeOut {
              "name": "concat-map",
              "spec": "0.0.1",
              "to": "node_modules/concat-map",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "brace-expansion",
          "path": "{CWD}/test/fixtures/sax/node_modules/minimatch/node_modules/brace-expansion",
          "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.8.tgz",
          "version": "1.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "minimatch",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/minimatch/node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/sax/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/sax/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/sax/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "0.7.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/sax/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-0.7.1.tgz",
      "version": "0.7.1",
    },
    "mute-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "mute-stream",
          "spec": "0.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/mute-stream",
      "name": "mute-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/mute-stream",
      "resolved": "https://registry.npmjs.org/mute-stream/-/mute-stream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "natural-compare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/natural-compare",
      "name": "natural-compare",
      "path": "{CWD}/test/fixtures/sax/node_modules/natural-compare",
      "resolved": "https://registry.npmjs.org/natural-compare/-/natural-compare-1.4.0.tgz",
      "version": "1.4.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/code-point-at",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/sax/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.0.tgz",
      "version": "1.0.0",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/sax/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.8.2.tgz",
      "version": "0.8.2",
    },
    "object-assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/esrecurse",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/figures",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsx-ast-utils",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/sax/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/run-async",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/sax/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "version": "1.3.2",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/sax/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-1.1.0.tgz",
      "version": "1.1.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/sax/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.4.2.tgz",
      "version": "1.4.2",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "os-homedir",
          "spec": "1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/user-home",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/sax/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "os-tmpdir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-tmpdir",
      "name": "os-tmpdir",
      "path": "{CWD}/test/fixtures/sax/node_modules/os-tmpdir",
      "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/sax/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/sax/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/sax/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-path-inside",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-inside",
      "name": "path-is-inside",
      "path": "{CWD}/test/fixtures/sax/node_modules/path-is-inside",
      "resolved": "https://registry.npmjs.org/path-is-inside/-/path-is-inside-1.0.2.tgz",
      "version": "1.0.2",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/sax/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-2.3.0.tgz",
      "version": "2.3.0",
    },
    "pinkie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pinkie-promise",
          "name": "pinkie",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie",
      "name": "pinkie",
      "path": "{CWD}/test/fixtures/sax/node_modules/pinkie",
      "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
      "version": "2.0.4",
    },
    "pinkie-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pinkie" => EdgeOut {
          "name": "pinkie",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie-promise",
      "name": "pinkie-promise",
      "path": "{CWD}/test/fixtures/sax/node_modules/pinkie-promise",
      "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
      "version": "2.0.1",
    },
    "pkg-config" => ArboristNode {
      "children": Map {
        "xtend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-config",
              "name": "xtend",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-config/node_modules/xtend",
          "name": "xtend",
          "path": "{CWD}/test/fixtures/sax/node_modules/pkg-config/node_modules/xtend",
          "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.1.tgz",
          "version": "4.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "pkg-config",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "pkg-config",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug-log" => EdgeOut {
          "name": "debug-log",
          "spec": "^1.0.0",
          "to": "node_modules/debug-log",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.1",
          "to": "node_modules/pkg-config/node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-config",
      "name": "pkg-config",
      "path": "{CWD}/test/fixtures/sax/node_modules/pkg-config",
      "resolved": "https://registry.npmjs.org/pkg-config/-/pkg-config-1.1.1.tgz",
      "version": "1.1.1",
    },
    "pluralize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "pluralize",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pluralize",
      "name": "pluralize",
      "path": "{CWD}/test/fixtures/sax/node_modules/pluralize",
      "resolved": "https://registry.npmjs.org/pluralize/-/pluralize-1.2.1.tgz",
      "version": "1.2.1",
    },
    "prelude-ls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/levn",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/type-check",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/prelude-ls",
      "name": "prelude-ls",
      "path": "{CWD}/test/fixtures/sax/node_modules/prelude-ls",
      "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.1.2.tgz",
      "version": "1.1.2",
    },
    "process-nextick-args" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "path": "{CWD}/test/fixtures/sax/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.3.tgz",
      "version": "1.0.3",
    },
    "progress" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "progress",
          "spec": "^1.1.8",
          "type": "prod",
        },
      },
      "location": "node_modules/progress",
      "name": "progress",
      "path": "{CWD}/test/fixtures/sax/node_modules/progress",
      "resolved": "https://registry.npmjs.org/progress/-/progress-1.1.8.tgz",
      "version": "1.1.8",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/sax/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^1.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "punycode",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/sax/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
      "version": "1.4.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/sax/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.3.0.tgz",
      "version": "6.3.0",
    },
    "readable-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "readable-stream",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "readable-stream",
          "spec": "^2",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "0.0.1",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~0.10.x",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.0.2.tgz",
      "version": "2.0.2",
    },
    "readline2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "readline2",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "mute-stream" => EdgeOut {
          "name": "mute-stream",
          "spec": "0.0.5",
          "to": "node_modules/mute-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/readline2",
      "name": "readline2",
      "path": "{CWD}/test/fixtures/sax/node_modules/readline2",
      "resolved": "https://registry.npmjs.org/readline2/-/readline2-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rechoir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "rechoir",
          "spec": "^0.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.1.6",
          "to": "node_modules/resolve",
          "type": "prod",
        },
      },
      "location": "node_modules/rechoir",
      "name": "rechoir",
      "path": "{CWD}/test/fixtures/sax/node_modules/rechoir",
      "resolved": "https://registry.npmjs.org/rechoir/-/rechoir-0.6.2.tgz",
      "version": "0.6.2",
    },
    "request" => ArboristNode {
      "children": Map {
        "extend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/request",
              "name": "extend",
              "spec": "~3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/request/node_modules/extend",
          "name": "extend",
          "path": "{CWD}/test/fixtures/sax/node_modules/request/node_modules/extend",
          "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "2.79.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.2.1",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.11.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.0",
          "to": "node_modules/request/node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.1.1",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~2.0.6",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "hawk" => EdgeOut {
          "name": "hawk",
          "spec": "~3.1.3",
          "to": "node_modules/hawk",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.1.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.7",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.3.0",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "stringstream" => EdgeOut {
          "name": "stringstream",
          "spec": "~0.0.4",
          "to": "node_modules/stringstream",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.0.0",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/sax/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.79.0.tgz",
      "version": "2.79.0",
    },
    "require-uncached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "require-uncached",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^0.1.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^1.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/require-uncached",
      "name": "require-uncached",
      "path": "{CWD}/test/fixtures/sax/node_modules/require-uncached",
      "resolved": "https://registry.npmjs.org/require-uncached/-/require-uncached-1.0.3.tgz",
      "version": "1.0.3",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rechoir",
          "name": "resolve",
          "spec": "^1.1.6",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/sax/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.1.6.tgz",
      "version": "1.1.6",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "resolve-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/sax/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-1.0.1.tgz",
      "version": "1.0.1",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "exit-hook" => EdgeOut {
          "name": "exit-hook",
          "spec": "^1.0.0",
          "to": "node_modules/exit-hook",
          "type": "prod",
        },
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^1.0.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/sax/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rimraf" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf",
              "name": "glob",
              "spec": "^5.0.14",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "2 || 3",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/rimraf/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-5.0.15.tgz",
          "version": "5.0.15",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "rimraf",
          "spec": "^2.2.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^5.0.14",
          "to": "node_modules/rimraf/node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/sax/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.4.3.tgz",
      "version": "2.4.3",
    },
    "run-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "run-async",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/run-async",
      "name": "run-async",
      "path": "{CWD}/test/fixtures/sax/node_modules/run-async",
      "resolved": "https://registry.npmjs.org/run-async/-/run-async-0.1.0.tgz",
      "version": "0.1.0",
    },
    "run-parallel" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "run-parallel",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/run-parallel",
      "name": "run-parallel",
      "path": "{CWD}/test/fixtures/sax/node_modules/run-parallel",
      "resolved": "https://registry.npmjs.org/run-parallel/-/run-parallel-1.1.6.tgz",
      "version": "1.1.6",
    },
    "rx-lite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "rx-lite",
          "spec": "^3.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/rx-lite",
      "name": "rx-lite",
      "path": "{CWD}/test/fixtures/sax/node_modules/rx-lite",
      "resolved": "https://registry.npmjs.org/rx-lite/-/rx-lite-3.1.2.tgz",
      "version": "3.1.2",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.1.tgz",
      "version": "5.1.1",
    },
    "shelljs" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs",
              "name": "glob",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/shelljs/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/shelljs/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/shelljs/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "shelljs",
          "spec": "^0.7.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/shelljs/node_modules/glob",
          "type": "prod",
        },
        "interpret" => EdgeOut {
          "name": "interpret",
          "spec": "^1.0.0",
          "to": "node_modules/interpret",
          "type": "prod",
        },
        "rechoir" => EdgeOut {
          "name": "rechoir",
          "spec": "^0.6.2",
          "to": "node_modules/rechoir",
          "type": "prod",
        },
      },
      "location": "node_modules/shelljs",
      "name": "shelljs",
      "path": "{CWD}/test/fixtures/sax/node_modules/shelljs",
      "resolved": "https://registry.npmjs.org/shelljs/-/shelljs-0.7.6.tgz",
      "version": "0.7.6",
    },
    "slice-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "slice-ansi",
          "spec": "0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-0.0.4.tgz",
      "version": "0.0.4",
    },
    "sntp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "sntp",
          "spec": "1.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/sntp",
      "name": "sntp",
      "path": "{CWD}/test/fixtures/sax/node_modules/sntp",
      "resolved": "https://registry.npmjs.org/sntp/-/sntp-1.0.9.tgz",
      "version": "1.0.9",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.5.6",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/sax/node_modules/source-map-support/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.6.tgz",
          "version": "0.5.6",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/sax/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.4.15.tgz",
      "version": "0.4.15",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/sax/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/sshpk",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/sshpk/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/sshpk/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/sshpk/node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "optional",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "optional",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jodid25519" => EdgeOut {
          "name": "jodid25519",
          "spec": "^1.0.0",
          "to": "node_modules/jodid25519",
          "type": "optional",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "optional",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "optional",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/sax/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.10.2.tgz",
      "version": "1.10.2",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/sax/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.0.tgz",
      "version": "1.0.0",
    },
    "standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "standard",
          "spec": "^8.6.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "~3.10.2",
          "to": "node_modules/eslint",
          "type": "prod",
        },
        "eslint-config-standard" => EdgeOut {
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "to": "node_modules/eslint-config-standard",
          "type": "prod",
        },
        "eslint-config-standard-jsx" => EdgeOut {
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "to": "node_modules/eslint-config-standard-jsx",
          "type": "prod",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "prod",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "prod",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "to": "node_modules/eslint-plugin-standard",
          "type": "prod",
        },
        "standard-engine" => EdgeOut {
          "name": "standard-engine",
          "spec": "~5.2.0",
          "to": "node_modules/standard-engine",
          "type": "prod",
        },
      },
      "location": "node_modules/standard",
      "name": "standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/standard",
      "resolved": "https://registry.npmjs.org/standard/-/standard-8.6.0.tgz",
      "version": "8.6.0",
    },
    "standard-engine" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard-engine",
              "name": "minimist",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard-engine/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/sax/node_modules/standard-engine/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "standard-engine",
          "spec": "~5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deglob" => EdgeOut {
          "name": "deglob",
          "spec": "^2.0.0",
          "to": "node_modules/deglob",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "get-stdin" => EdgeOut {
          "name": "get-stdin",
          "spec": "^5.0.1",
          "to": "node_modules/get-stdin",
          "type": "prod",
        },
        "home-or-tmp" => EdgeOut {
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "to": "node_modules/home-or-tmp",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.1.0",
          "to": "node_modules/standard-engine/node_modules/minimist",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.0.1",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
      },
      "location": "node_modules/standard-engine",
      "name": "standard-engine",
      "path": "{CWD}/test/fixtures/sax/node_modules/standard-engine",
      "resolved": "https://registry.npmjs.org/standard-engine/-/standard-engine-5.2.0.tgz",
      "version": "5.2.0",
    },
    "string_decoder" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "path": "{CWD}/test/fixtures/sax/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-0.10.31.tgz",
      "version": "0.10.31",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "string-width",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/sax/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.1.tgz",
      "version": "1.0.1",
    },
    "stringstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "stringstream",
          "spec": "~0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/stringstream",
      "name": "stringstream",
      "path": "{CWD}/test/fixtures/sax/node_modules/stringstream",
      "resolved": "https://registry.npmjs.org/stringstream/-/stringstream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table/node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "strip-json-comments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-json-comments",
      "name": "strip-json-comments",
      "path": "{CWD}/test/fixtures/sax/node_modules/strip-json-comments",
      "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-1.0.4.tgz",
      "version": "1.0.4",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "supports-color",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/sax/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
      "version": "2.0.0",
    },
    "table" => ArboristNode {
      "children": Map {
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "string-width",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/table/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "table",
          "spec": "^3.7.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^4.7.0",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/table/node_modules/lodash",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "0.0.4",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.0.0",
          "to": "node_modules/table/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/table",
      "name": "table",
      "path": "{CWD}/test/fixtures/sax/node_modules/table",
      "resolved": "https://registry.npmjs.org/table/-/table-3.8.3.tgz",
      "version": "3.8.3",
    },
    "tap" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/inherits",
          "name": "inherits",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/isarray",
          "name": "isarray",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "nyc" => ArboristNode {
          "bundleDependencies": Array [
            "archy",
            "arrify",
            "caching-transform",
            "convert-source-map",
            "debug-log",
            "default-require-extensions",
            "find-cache-dir",
            "find-up",
            "foreground-child",
            "glob",
            "istanbul-lib-coverage",
            "istanbul-lib-hook",
            "istanbul-lib-instrument",
            "istanbul-lib-report",
            "istanbul-lib-source-maps",
            "istanbul-reports",
            "md5-hex",
            "merge-source-map",
            "micromatch",
            "mkdirp",
            "resolve-from",
            "rimraf",
            "signal-exit",
            "spawn-wrap",
            "test-exclude",
            "yargs",
            "yargs-parser",
          ],
          "children": Map {
            "align-text" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "longest" => EdgeOut {
                  "name": "longest",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/longest",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/align-text",
              "name": "align-text",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/align-text",
              "version": "0.1.4",
            },
            "amdefine" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "amdefine",
                  "spec": ">=0.0.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "name": "amdefine",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "version": "1.0.1",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "version": "2.2.1",
            },
            "append-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "default-require-extensions" => EdgeOut {
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "name": "append-transform",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "version": "0.4.0",
            },
            "archy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "archy",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "name": "archy",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/archy",
              "version": "1.0.0",
            },
            "arr-diff" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-flatten" => EdgeOut {
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "name": "arr-diff",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "version": "2.0.0",
            },
            "arr-flatten" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "name": "arr-flatten",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "version": "1.0.3",
            },
            "array-unique" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "name": "array-unique",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "version": "0.2.1",
            },
            "arrify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "name": "arrify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arrify",
              "version": "1.0.1",
            },
            "async" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "async",
                  "spec": "^1.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/async",
              "name": "async",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/async",
              "version": "1.5.2",
            },
            "babel-code-frame" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "chalk" => EdgeOut {
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "name": "babel-code-frame",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "version": "6.22.0",
            },
            "babel-generator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "detect-indent" => EdgeOut {
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "type": "prod",
                },
                "jsesc" => EdgeOut {
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "trim-right" => EdgeOut {
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "name": "babel-generator",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "version": "6.24.1",
            },
            "babel-messages" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "name": "babel-messages",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "version": "6.23.0",
            },
            "babel-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "core-js" => EdgeOut {
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/core-js",
                  "type": "prod",
                },
                "regenerator-runtime" => EdgeOut {
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "name": "babel-runtime",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "version": "6.23.0",
            },
            "babel-template" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "name": "babel-template",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "version": "6.24.1",
            },
            "babel-traverse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-code-frame" => EdgeOut {
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "type": "prod",
                },
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "globals" => EdgeOut {
                  "name": "globals",
                  "spec": "^9.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/globals",
                  "type": "prod",
                },
                "invariant" => EdgeOut {
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "name": "babel-traverse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "version": "6.24.1",
            },
            "babel-types" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "to-fast-properties" => EdgeOut {
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "name": "babel-types",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "version": "6.24.1",
            },
            "babylon" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babylon",
              "name": "babylon",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babylon",
              "version": "6.17.2",
            },
            "balanced-match" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "name": "balanced-match",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "version": "0.4.2",
            },
            "brace-expansion" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "balanced-match" => EdgeOut {
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
                  "type": "prod",
                },
                "concat-map" => EdgeOut {
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "name": "brace-expansion",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "version": "1.1.7",
            },
            "braces" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "braces",
                  "spec": "^1.8.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "expand-range" => EdgeOut {
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "type": "prod",
                },
                "preserve" => EdgeOut {
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/preserve",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/braces",
              "name": "braces",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/braces",
              "version": "1.8.5",
            },
            "builtin-modules" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "name": "builtin-modules",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "version": "1.1.1",
            },
            "caching-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "caching-transform",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-hex" => EdgeOut {
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "write-file-atomic" => EdgeOut {
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "name": "caching-transform",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "version": "1.0.1",
            },
            "center-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
                "lazy-cache" => EdgeOut {
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/center-align",
              "name": "center-align",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/center-align",
              "version": "0.1.3",
            },
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "has-ansi" => EdgeOut {
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/chalk",
              "version": "1.1.3",
            },
            "cliui" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "wordwrap" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "name": "wordwrap",
                      "spec": "0.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "name": "wordwrap",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "version": "0.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "cliui",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "center-align" => EdgeOut {
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "type": "prod",
                },
                "right-align" => EdgeOut {
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cliui",
              "name": "cliui",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cliui",
              "version": "2.1.0",
            },
            "code-point-at" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "name": "code-point-at",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "version": "1.1.0",
            },
            "commondir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/commondir",
              "name": "commondir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/commondir",
              "version": "1.0.1",
            },
            "concat-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "name": "concat-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "version": "0.0.1",
            },
            "convert-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "convert-source-map",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "name": "convert-source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "version": "1.5.0",
            },
            "core-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/core-js",
              "name": "core-js",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/core-js",
              "version": "2.4.1",
            },
            "cross-spawn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "cross-spawn",
                  "spec": "^4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "lru-cache" => EdgeOut {
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.9",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "name": "cross-spawn",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "version": "4.0.2",
            },
            "debug" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "debug",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "debug",
                  "spec": "^2.6.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ms" => EdgeOut {
                  "name": "ms",
                  "spec": "2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ms",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug",
              "name": "debug",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/debug",
              "version": "2.6.8",
            },
            "debug-log" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "debug-log",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "name": "debug-log",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "version": "1.0.1",
            },
            "decamelize" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "name": "decamelize",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "version": "1.2.0",
            },
            "default-require-extensions" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "name": "default-require-extensions",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "version": "1.0.0",
            },
            "detect-indent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "repeating" => EdgeOut {
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "name": "detect-indent",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "version": "4.0.0",
            },
            "error-ex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-arrayish" => EdgeOut {
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "name": "error-ex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "version": "1.3.1",
            },
            "escape-string-regexp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "name": "escape-string-regexp",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "version": "1.0.5",
            },
            "esutils" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/esutils",
              "name": "esutils",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/esutils",
              "version": "2.0.2",
            },
            "execa" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "execa",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "get-stream" => EdgeOut {
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "type": "prod",
                },
                "is-stream" => EdgeOut {
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
                  "type": "prod",
                },
                "npm-run-path" => EdgeOut {
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "type": "prod",
                },
                "p-finally" => EdgeOut {
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "strip-eof" => EdgeOut {
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/execa",
              "name": "execa",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/execa",
              "version": "0.5.1",
            },
            "expand-brackets" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-posix-bracket" => EdgeOut {
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "name": "expand-brackets",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "version": "0.1.5",
            },
            "expand-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fill-range" => EdgeOut {
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "name": "expand-range",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "version": "1.8.2",
            },
            "extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/extglob",
              "name": "extglob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/extglob",
              "version": "0.3.2",
            },
            "filename-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "name": "filename-regex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "version": "2.0.1",
            },
            "fill-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "isobject" => EdgeOut {
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "type": "prod",
                },
                "randomatic" => EdgeOut {
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "name": "fill-range",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "version": "2.2.3",
            },
            "find-cache-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-cache-dir",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "commondir" => EdgeOut {
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/commondir",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "pkg-dir" => EdgeOut {
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "name": "find-cache-dir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "version": "0.1.1",
            },
            "find-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-up",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "find-up",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "locate-path" => EdgeOut {
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "name": "find-up",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/find-up",
              "version": "2.1.0",
            },
            "for-in" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-in",
              "name": "for-in",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/for-in",
              "version": "1.0.2",
            },
            "for-own" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-in" => EdgeOut {
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-in",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-own",
              "name": "for-own",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/for-own",
              "version": "0.1.5",
            },
            "foreground-child" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "foreground-child",
                  "spec": "^1.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "name": "foreground-child",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "version": "1.5.6",
            },
            "fs.realpath" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "name": "fs.realpath",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "version": "1.0.0",
            },
            "get-caller-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "name": "get-caller-file",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "version": "1.0.2",
            },
            "get-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "name": "get-stream",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "version": "2.3.1",
            },
            "glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "glob",
                  "spec": "^7.0.6",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "name": "glob",
                  "spec": "^7.0.5",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fs.realpath" => EdgeOut {
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
                  "type": "prod",
                },
                "inflight" => EdgeOut {
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inherits",
                  "type": "prod",
                },
                "minimatch" => EdgeOut {
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "type": "prod",
                },
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "path-is-absolute" => EdgeOut {
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "name": "glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob",
              "version": "7.1.2",
            },
            "glob-base" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-parent" => EdgeOut {
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "name": "glob-base",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "version": "0.3.0",
            },
            "glob-parent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "name": "glob-parent",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "version": "2.0.0",
            },
            "globals" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "globals",
                  "spec": "^9.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/globals",
              "name": "globals",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/globals",
              "version": "9.17.0",
            },
            "graceful-fs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "name": "graceful-fs",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "version": "4.1.11",
            },
            "handlebars" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "source-map" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                      "name": "source-map",
                      "spec": "^0.4.4",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "amdefine" => EdgeOut {
                      "name": "amdefine",
                      "spec": ">=0.0.4",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "source-map",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "version": "0.4.4",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "async" => EdgeOut {
                  "name": "async",
                  "spec": "^1.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/async",
                  "type": "prod",
                },
                "optimist" => EdgeOut {
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.4.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-js" => EdgeOut {
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "name": "handlebars",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "version": "4.0.10",
            },
            "has-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "name": "has-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "version": "2.0.0",
            },
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "version": "1.0.0",
            },
            "hosted-git-info" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "name": "hosted-git-info",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "version": "2.4.2",
            },
            "imurmurhash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "name": "imurmurhash",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "version": "0.1.4",
            },
            "inflight" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inflight",
              "name": "inflight",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/inflight",
              "version": "1.0.6",
            },
            "inherits" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inherits",
                  "spec": "2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inherits",
              "name": "inherits",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/inherits",
              "version": "2.0.3",
            },
            "invariant" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "loose-envify" => EdgeOut {
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invariant",
              "name": "invariant",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/invariant",
              "version": "2.2.2",
            },
            "invert-kv" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "name": "invert-kv",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "version": "1.0.0",
            },
            "is-arrayish" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "name": "is-arrayish",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "version": "0.2.1",
            },
            "is-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "name": "is-buffer",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "version": "1.1.5",
            },
            "is-builtin-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "builtin-modules" => EdgeOut {
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "name": "is-builtin-module",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "version": "1.0.0",
            },
            "is-dotfile" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "name": "is-dotfile",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "version": "1.0.3",
            },
            "is-equal-shallow" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "name": "is-equal-shallow",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "version": "0.1.3",
            },
            "is-extendable" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "name": "is-extendable",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "version": "0.1.1",
            },
            "is-extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "name": "is-extglob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "version": "1.0.0",
            },
            "is-finite" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "name": "is-finite",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "version": "1.0.2",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "version": "1.0.0",
            },
            "is-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "name": "is-glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "version": "2.0.1",
            },
            "is-number" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-number",
              "name": "is-number",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-number",
              "version": "2.1.0",
            },
            "is-posix-bracket" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "name": "is-posix-bracket",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "version": "0.1.1",
            },
            "is-primitive" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "name": "is-primitive",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "version": "2.0.0",
            },
            "is-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "name": "is-stream",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "version": "1.1.0",
            },
            "is-utf8" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "name": "is-utf8",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "version": "0.2.1",
            },
            "isarray" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "name": "isarray",
                  "spec": "1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isarray",
              "name": "isarray",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isarray",
              "version": "1.0.0",
            },
            "isexe" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isexe",
              "name": "isexe",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isexe",
              "version": "2.0.0",
            },
            "isobject" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isarray",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isobject",
              "name": "isobject",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isobject",
              "version": "2.1.0",
            },
            "istanbul-lib-coverage" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "name": "istanbul-lib-coverage",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "version": "1.1.1",
            },
            "istanbul-lib-hook" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-hook",
                  "spec": "^1.0.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "append-transform" => EdgeOut {
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "name": "istanbul-lib-hook",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "version": "1.0.7",
            },
            "istanbul-lib-instrument" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-instrument",
                  "spec": "^1.7.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-generator" => EdgeOut {
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "type": "prod",
                },
                "babel-template" => EdgeOut {
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "name": "istanbul-lib-instrument",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "version": "1.7.2",
            },
            "istanbul-lib-report" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "supports-color" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                      "name": "supports-color",
                      "spec": "^3.1.2",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "has-flag" => EdgeOut {
                      "name": "has-flag",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "supports-color",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "version": "3.2.3",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-report",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "path-parse" => EdgeOut {
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^3.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "name": "istanbul-lib-report",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "version": "1.1.1",
            },
            "istanbul-lib-source-maps" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-source-maps",
                  "spec": "^1.2.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.6.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "name": "istanbul-lib-source-maps",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "version": "1.2.1",
            },
            "istanbul-reports" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-reports",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "handlebars" => EdgeOut {
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "name": "istanbul-reports",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "version": "1.1.1",
            },
            "js-tokens" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "name": "js-tokens",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "version": "3.0.1",
            },
            "jsesc" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "name": "jsesc",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "version": "1.3.0",
            },
            "kind-of" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-buffer" => EdgeOut {
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "version": "3.2.2",
            },
            "lazy-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "name": "lazy-cache",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "version": "1.0.4",
            },
            "lcid" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "invert-kv" => EdgeOut {
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lcid",
              "name": "lcid",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lcid",
              "version": "1.0.0",
            },
            "load-json-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "parse-json" => EdgeOut {
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "name": "load-json-file",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "version": "1.1.0",
            },
            "locate-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "path-exists" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                      "name": "path-exists",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "name": "path-exists",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-locate" => EdgeOut {
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "type": "prod",
                },
                "path-exists" => EdgeOut {
                  "name": "path-exists",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "name": "locate-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "version": "2.0.0",
            },
            "lodash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lodash",
              "name": "lodash",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lodash",
              "version": "4.17.4",
            },
            "longest" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "longest",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/longest",
              "name": "longest",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/longest",
              "version": "1.0.1",
            },
            "loose-envify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "name": "loose-envify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "version": "1.3.1",
            },
            "lru-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pseudomap" => EdgeOut {
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
                  "type": "prod",
                },
                "yallist" => EdgeOut {
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yallist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "name": "lru-cache",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "version": "4.0.2",
            },
            "md5-hex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-o-matic" => EdgeOut {
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "name": "md5-hex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "version": "1.3.0",
            },
            "md5-o-matic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "name": "md5-o-matic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "version": "0.1.1",
            },
            "mem" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "mem",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mem",
              "name": "mem",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mem",
              "version": "1.1.0",
            },
            "merge-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "merge-source-map",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "name": "merge-source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "version": "1.0.3",
            },
            "micromatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-diff" => EdgeOut {
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "type": "prod",
                },
                "array-unique" => EdgeOut {
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
                  "type": "prod",
                },
                "braces" => EdgeOut {
                  "name": "braces",
                  "spec": "^1.8.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "type": "prod",
                },
                "expand-brackets" => EdgeOut {
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "type": "prod",
                },
                "extglob" => EdgeOut {
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "type": "prod",
                },
                "filename-regex" => EdgeOut {
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "normalize-path" => EdgeOut {
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "type": "prod",
                },
                "object.omit" => EdgeOut {
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "type": "prod",
                },
                "parse-glob" => EdgeOut {
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "type": "prod",
                },
                "regex-cache" => EdgeOut {
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "name": "micromatch",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "version": "2.3.11",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "version": "1.1.0",
            },
            "minimatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "brace-expansion" => EdgeOut {
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "name": "minimatch",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "version": "3.0.4",
            },
            "minimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "name": "minimist",
                  "spec": "0.0.8",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimist",
              "name": "minimist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/minimist",
              "version": "0.0.8",
            },
            "mkdirp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "0.0.8",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "name": "mkdirp",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "version": "0.5.1",
            },
            "ms" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "name": "ms",
                  "spec": "2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ms",
              "name": "ms",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ms",
              "version": "2.0.0",
            },
            "normalize-package-data" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "hosted-git-info" => EdgeOut {
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
                  "type": "prod",
                },
                "is-builtin-module" => EdgeOut {
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
                "validate-npm-package-license" => EdgeOut {
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "name": "normalize-package-data",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "version": "2.3.8",
            },
            "normalize-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "remove-trailing-separator" => EdgeOut {
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "name": "normalize-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "version": "2.1.1",
            },
            "npm-run-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "path-key" => EdgeOut {
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-key",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "name": "npm-run-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "version": "2.0.2",
            },
            "number-is-nan" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "name": "number-is-nan",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "version": "1.0.1",
            },
            "object-assign" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "name": "object-assign",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "version": "4.1.1",
            },
            "object.omit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-own" => EdgeOut {
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "type": "prod",
                },
                "is-extendable" => EdgeOut {
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "name": "object.omit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "version": "2.0.1",
            },
            "once" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/once",
              "name": "once",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/once",
              "version": "1.4.0",
            },
            "optimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/optimist",
              "name": "optimist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/optimist",
              "version": "0.6.1",
            },
            "os-homedir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "name": "os-homedir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "version": "1.0.2",
            },
            "os-locale" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "execa" => EdgeOut {
                  "name": "execa",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "type": "prod",
                },
                "lcid" => EdgeOut {
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "type": "prod",
                },
                "mem" => EdgeOut {
                  "name": "mem",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "name": "os-locale",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "version": "2.0.0",
            },
            "p-finally" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "name": "p-finally",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "version": "1.0.0",
            },
            "p-limit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "name": "p-limit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "version": "1.1.0",
            },
            "p-locate" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-limit" => EdgeOut {
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "name": "p-locate",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "version": "2.0.0",
            },
            "parse-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-base" => EdgeOut {
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "type": "prod",
                },
                "is-dotfile" => EdgeOut {
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "name": "parse-glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "version": "3.0.4",
            },
            "parse-json" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "error-ex" => EdgeOut {
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "name": "parse-json",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "version": "2.2.0",
            },
            "path-exists" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "name": "path-exists",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "version": "2.1.0",
            },
            "path-is-absolute" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "name": "path-is-absolute",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "version": "1.0.1",
            },
            "path-key" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-key",
              "name": "path-key",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-key",
              "version": "2.0.1",
            },
            "path-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "name": "path-parse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "version": "1.0.5",
            },
            "path-type" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-type",
              "name": "path-type",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-type",
              "version": "1.1.0",
            },
            "pify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pify",
              "name": "pify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pify",
              "version": "2.3.0",
            },
            "pinkie" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "name": "pinkie",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "version": "2.0.4",
            },
            "pinkie-promise" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie" => EdgeOut {
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "name": "pinkie-promise",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "version": "2.0.1",
            },
            "pkg-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "name": "pkg-dir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "version": "1.0.0",
            },
            "preserve" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/preserve",
              "name": "preserve",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/preserve",
              "version": "0.2.0",
            },
            "pseudomap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "name": "pseudomap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "version": "1.0.2",
            },
            "randomatic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "name": "randomatic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "version": "1.1.6",
            },
            "read-pkg" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "load-json-file" => EdgeOut {
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "type": "prod",
                },
                "normalize-package-data" => EdgeOut {
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "type": "prod",
                },
                "path-type" => EdgeOut {
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "name": "read-pkg",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "version": "1.1.0",
            },
            "read-pkg-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "type": "prod",
                },
                "read-pkg" => EdgeOut {
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "name": "read-pkg-up",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "version": "1.0.1",
            },
            "regenerator-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "name": "regenerator-runtime",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "version": "0.10.5",
            },
            "regex-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-equal-shallow" => EdgeOut {
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "type": "prod",
                },
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "name": "regex-cache",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "version": "0.4.3",
            },
            "remove-trailing-separator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "name": "remove-trailing-separator",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "version": "1.0.1",
            },
            "repeat-element" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "name": "repeat-element",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "version": "1.1.2",
            },
            "repeat-string" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "name": "repeat-string",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "version": "1.6.1",
            },
            "repeating" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-finite" => EdgeOut {
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeating",
              "name": "repeating",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeating",
              "version": "2.0.1",
            },
            "require-directory" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "name": "require-directory",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "version": "2.1.1",
            },
            "require-main-filename" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "name": "require-main-filename",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "version": "1.0.1",
            },
            "resolve-from" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "resolve-from",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "name": "resolve-from",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "version": "2.0.0",
            },
            "right-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/right-align",
              "name": "right-align",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/right-align",
              "version": "0.1.3",
            },
            "rimraf" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "rimraf",
                  "spec": "^2.5.4",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob" => EdgeOut {
                  "name": "glob",
                  "spec": "^7.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "name": "rimraf",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "version": "2.6.1",
            },
            "semver" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "semver",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/semver",
              "name": "semver",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/semver",
              "version": "5.3.0",
            },
            "set-blocking" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "name": "set-blocking",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "version": "2.0.0",
            },
            "signal-exit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "signal-exit",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "name": "signal-exit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "version": "3.0.2",
            },
            "slide" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "slide",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/slide",
              "name": "slide",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/slide",
              "version": "1.1.6",
            },
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/source-map",
              "version": "0.5.6",
            },
            "spawn-wrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "spawn-wrap",
                  "spec": "^1.3.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "foreground-child" => EdgeOut {
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "os-homedir" => EdgeOut {
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "name": "spawn-wrap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "version": "1.3.6",
            },
            "spdx-correct" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-license-ids" => EdgeOut {
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "name": "spdx-correct",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "version": "1.0.2",
            },
            "spdx-expression-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "name": "spdx-expression-parse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "version": "1.0.4",
            },
            "spdx-license-ids" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "name": "spdx-license-ids",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "version": "1.2.2",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "is-fullwidth-code-point" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                      "name": "is-fullwidth-code-point",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "name": "is-fullwidth-code-point",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "version": "2.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/string-width",
              "version": "2.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "strip-bom" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-utf8" => EdgeOut {
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "name": "strip-bom",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "version": "2.0.0",
            },
            "strip-eof" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "name": "strip-eof",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "version": "1.0.0",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "version": "2.0.0",
            },
            "test-exclude" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "test-exclude",
                  "spec": "^4.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arrify" => EdgeOut {
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
                  "type": "prod",
                },
                "micromatch" => EdgeOut {
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "type": "prod",
                },
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "name": "test-exclude",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "version": "4.1.1",
            },
            "to-fast-properties" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "name": "to-fast-properties",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "version": "1.0.3",
            },
            "trim-right" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "name": "trim-right",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "version": "1.0.1",
            },
            "uglify-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "name": "camelcase",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "version": "1.2.1",
                },
                "yargs" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                      "name": "yargs",
                      "spec": "~3.10.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                      "type": "prod",
                    },
                    "cliui" => EdgeOut {
                      "name": "cliui",
                      "spec": "^2.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "type": "prod",
                    },
                    "decamelize" => EdgeOut {
                      "name": "decamelize",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                      "type": "prod",
                    },
                    "window-size" => EdgeOut {
                      "name": "window-size",
                      "spec": "0.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/window-size",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "yargs",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "version": "3.10.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-to-browserify" => EdgeOut {
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
                  "type": "optional",
                },
                "yargs" => EdgeOut {
                  "name": "yargs",
                  "spec": "~3.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "name": "uglify-js",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "version": "2.8.27",
            },
            "uglify-to-browserify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "name": "uglify-to-browserify",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "version": "1.0.2",
            },
            "validate-npm-package-license" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-correct" => EdgeOut {
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "type": "prod",
                },
                "spdx-expression-parse" => EdgeOut {
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "name": "validate-npm-package-license",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "version": "3.0.1",
            },
            "which" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "which",
                  "spec": "^1.2.9",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "which",
                  "spec": "^1.2.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isexe" => EdgeOut {
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isexe",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which",
              "name": "which",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/which",
              "version": "1.2.14",
            },
            "which-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which-module",
              "name": "which-module",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/which-module",
              "version": "2.0.0",
            },
            "window-size" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "window-size",
                  "spec": "0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/window-size",
              "name": "window-size",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/window-size",
              "version": "0.1.0",
            },
            "wordwrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "name": "wordwrap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "version": "0.0.3",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "string-width" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "code-point-at" => EdgeOut {
                      "name": "code-point-at",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                      "type": "prod",
                    },
                    "is-fullwidth-code-point" => EdgeOut {
                      "name": "is-fullwidth-code-point",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "string-width",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "version": "1.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "wrap-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "version": "2.1.0",
            },
            "wrappy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "name": "wrappy",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "version": "1.0.2",
            },
            "write-file-atomic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "imurmurhash" => EdgeOut {
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
                  "type": "prod",
                },
                "slide" => EdgeOut {
                  "name": "slide",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/slide",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "name": "write-file-atomic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "version": "1.3.4",
            },
            "y18n" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/y18n",
              "name": "y18n",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/y18n",
              "version": "3.2.1",
            },
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yallist",
              "version": "2.1.2",
            },
            "yargs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "version": "4.1.0",
                },
                "cliui" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "children": Map {
                    "string-width" => ArboristNode {
                      "bundled": true,
                      "bundler": "node_modules/tap/node_modules/nyc",
                      "dev": true,
                      "edgesIn": Set {
                        EdgeIn {
                          "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                          "name": "string-width",
                          "spec": "^1.0.1",
                          "type": "prod",
                        },
                      },
                      "edgesOut": Map {
                        "code-point-at" => EdgeOut {
                          "name": "code-point-at",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                          "type": "prod",
                        },
                        "is-fullwidth-code-point" => EdgeOut {
                          "name": "is-fullwidth-code-point",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                          "type": "prod",
                        },
                        "strip-ansi" => EdgeOut {
                          "name": "strip-ansi",
                          "spec": "^3.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                          "type": "prod",
                        },
                      },
                      "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "name": "string-width",
                      "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "version": "1.0.2",
                    },
                  },
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "cliui",
                      "spec": "^3.2.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "string-width" => EdgeOut {
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                    "wrap-ansi" => EdgeOut {
                      "name": "wrap-ansi",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "cliui",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "version": "3.2.0",
                },
                "load-json-file" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "graceful-fs" => EdgeOut {
                      "name": "graceful-fs",
                      "spec": "^4.1.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                      "type": "prod",
                    },
                    "parse-json" => EdgeOut {
                      "name": "parse-json",
                      "spec": "^2.2.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                      "type": "prod",
                    },
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                    "strip-bom" => EdgeOut {
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "load-json-file",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "version": "2.0.0",
                },
                "path-type" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "path-type",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "version": "2.0.0",
                },
                "read-pkg" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "load-json-file" => EdgeOut {
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "type": "prod",
                    },
                    "normalize-package-data" => EdgeOut {
                      "name": "normalize-package-data",
                      "spec": "^2.3.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                      "type": "prod",
                    },
                    "path-type" => EdgeOut {
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "read-pkg",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "version": "2.0.0",
                },
                "read-pkg-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "read-pkg-up",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "find-up" => EdgeOut {
                      "name": "find-up",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                      "type": "prod",
                    },
                    "read-pkg" => EdgeOut {
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "read-pkg-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "version": "2.0.0",
                },
                "strip-bom" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "name": "strip-bom",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "version": "3.0.0",
                },
                "yargs-parser" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "yargs-parser",
                      "spec": "^7.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "name": "yargs-parser",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "version": "7.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs",
                  "spec": "^8.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "type": "prod",
                },
                "cliui" => EdgeOut {
                  "name": "cliui",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "type": "prod",
                },
                "decamelize" => EdgeOut {
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                  "type": "prod",
                },
                "get-caller-file" => EdgeOut {
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
                  "type": "prod",
                },
                "os-locale" => EdgeOut {
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-directory" => EdgeOut {
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
                "set-blocking" => EdgeOut {
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "type": "prod",
                },
                "which-module" => EdgeOut {
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which-module",
                  "type": "prod",
                },
                "y18n" => EdgeOut {
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/y18n",
                  "type": "prod",
                },
                "yargs-parser" => EdgeOut {
                  "name": "yargs-parser",
                  "spec": "^7.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "name": "yargs",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs",
              "version": "8.0.1",
            },
            "yargs-parser" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs-parser",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "name": "yargs-parser",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "version": "5.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "nyc",
              "spec": "^11.0.2-candidate.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "archy" => EdgeOut {
              "name": "archy",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "type": "prod",
            },
            "caching-transform" => EdgeOut {
              "name": "caching-transform",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.3.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug-log" => EdgeOut {
              "name": "debug-log",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "type": "prod",
            },
            "default-require-extensions" => EdgeOut {
              "name": "default-require-extensions",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "type": "prod",
            },
            "find-cache-dir" => EdgeOut {
              "name": "find-cache-dir",
              "spec": "^0.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "type": "prod",
            },
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "type": "prod",
            },
            "foreground-child" => EdgeOut {
              "name": "foreground-child",
              "spec": "^1.5.3",
              "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "type": "prod",
            },
            "istanbul-lib-coverage" => EdgeOut {
              "name": "istanbul-lib-coverage",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "type": "prod",
            },
            "istanbul-lib-hook" => EdgeOut {
              "name": "istanbul-lib-hook",
              "spec": "^1.0.7",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "type": "prod",
            },
            "istanbul-lib-instrument" => EdgeOut {
              "name": "istanbul-lib-instrument",
              "spec": "^1.7.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "type": "prod",
            },
            "istanbul-lib-report" => EdgeOut {
              "name": "istanbul-lib-report",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "type": "prod",
            },
            "istanbul-lib-source-maps" => EdgeOut {
              "name": "istanbul-lib-source-maps",
              "spec": "^1.2.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "type": "prod",
            },
            "istanbul-reports" => EdgeOut {
              "name": "istanbul-reports",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "type": "prod",
            },
            "md5-hex" => EdgeOut {
              "name": "md5-hex",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "type": "prod",
            },
            "merge-source-map" => EdgeOut {
              "name": "merge-source-map",
              "spec": "^1.0.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "type": "prod",
            },
            "micromatch" => EdgeOut {
              "name": "micromatch",
              "spec": "^2.3.11",
              "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "type": "prod",
            },
            "mkdirp" => EdgeOut {
              "name": "mkdirp",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "type": "prod",
            },
            "rimraf" => EdgeOut {
              "name": "rimraf",
              "spec": "^2.5.4",
              "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "type": "prod",
            },
            "spawn-wrap" => EdgeOut {
              "name": "spawn-wrap",
              "spec": "^1.3.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "type": "prod",
            },
            "test-exclude" => EdgeOut {
              "name": "test-exclude",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "type": "prod",
            },
            "yargs" => EdgeOut {
              "name": "yargs",
              "spec": "^8.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "type": "prod",
            },
            "yargs-parser" => EdgeOut {
              "name": "yargs-parser",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/nyc",
          "name": "nyc",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc",
          "resolved": "https://registry.npmjs.org/nyc/-/nyc-11.0.2.tgz",
          "version": "11.0.2",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
        "string_decoder" => ArboristNode {
          "children": Map {
            "safe-buffer" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string_decoder",
                  "name": "safe-buffer",
                  "spec": "~5.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "name": "safe-buffer",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.0.1.tgz",
              "version": "5.0.1",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "string_decoder",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.0.1",
              "to": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string_decoder",
          "name": "string_decoder",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/string_decoder",
          "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.0.2.tgz",
          "version": "1.0.2",
        },
        "tap-mocha-reporter" => ArboristNode {
          "children": Map {
            "readable-stream" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/tap-mocha-reporter",
                  "name": "readable-stream",
                  "spec": "^2.1.5",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "core-util-is" => EdgeOut {
                  "name": "core-util-is",
                  "spec": "~1.0.0",
                  "to": "node_modules/core-util-is",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "~2.0.3",
                  "to": "node_modules/tap/node_modules/inherits",
                  "type": "prod",
                },
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/isarray",
                  "type": "prod",
                },
                "process-nextick-args" => EdgeOut {
                  "name": "process-nextick-args",
                  "spec": "~1.0.6",
                  "to": "node_modules/tap/node_modules/process-nextick-args",
                  "type": "prod",
                },
                "safe-buffer" => EdgeOut {
                  "name": "safe-buffer",
                  "spec": "~5.1.0",
                  "to": "node_modules/safe-buffer",
                  "type": "prod",
                },
                "string_decoder" => EdgeOut {
                  "name": "string_decoder",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/string_decoder",
                  "type": "prod",
                },
                "util-deprecate" => EdgeOut {
                  "name": "util-deprecate",
                  "spec": "~1.0.1",
                  "to": "node_modules/util-deprecate",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "readable-stream",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.2.tgz",
              "version": "2.3.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-mocha-reporter",
              "spec": "^3.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-support" => EdgeOut {
              "name": "color-support",
              "spec": "^1.1.0",
              "to": "node_modules/color-support",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^2.1.3",
              "to": "node_modules/debug",
              "type": "prod",
            },
            "diff" => EdgeOut {
              "name": "diff",
              "spec": "^1.3.2",
              "to": "node_modules/diff",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.3",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.5",
              "to": "node_modules/glob",
              "type": "prod",
            },
            "js-yaml" => EdgeOut {
              "name": "js-yaml",
              "spec": "^3.3.1",
              "to": "node_modules/js-yaml",
              "type": "prod",
            },
            "readable-stream" => EdgeOut {
              "name": "readable-stream",
              "spec": "^2.1.5",
              "to": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "type": "optional",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^5.1.0",
              "to": "node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^1.0.0",
              "to": "node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-mocha-reporter",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tap-mocha-reporter",
          "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-3.0.5.tgz",
          "version": "3.0.5",
        },
        "tmatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tmatch",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tmatch",
          "name": "tmatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tmatch",
          "resolved": "https://registry.npmjs.org/tmatch/-/tmatch-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "^10.5.1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "bluebird" => EdgeOut {
          "name": "bluebird",
          "spec": "^3.3.1",
          "to": "node_modules/bluebird",
          "type": "prod",
        },
        "clean-yaml-object" => EdgeOut {
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "to": "node_modules/clean-yaml-object",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^2.11.2",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.1",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^1.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.3.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^11.0.2-candidate.0",
          "to": "node_modules/tap/node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.4.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.0",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.4.3",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.0",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^3.0.5",
          "to": "node_modules/tap/node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^5.3.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tmatch" => EdgeOut {
          "name": "tmatch",
          "spec": "^3.1.0",
          "to": "node_modules/tap/node_modules/tmatch",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "tsame" => EdgeOut {
          "name": "tsame",
          "spec": "^1.1.2",
          "to": "node_modules/tsame",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/sax/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-10.5.1.tgz",
      "version": "10.5.1",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.2.7",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/sax/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-5.3.2.tgz",
      "version": "5.3.2",
    },
    "text-table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "text-table",
          "spec": "~0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/text-table",
      "name": "text-table",
      "path": "{CWD}/test/fixtures/sax/node_modules/text-table",
      "resolved": "https://registry.npmjs.org/text-table/-/text-table-0.2.0.tgz",
      "version": "0.2.0",
    },
    "through" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "through",
          "spec": "^2.3.6",
          "type": "prod",
        },
      },
      "location": "node_modules/through",
      "name": "through",
      "path": "{CWD}/test/fixtures/sax/node_modules/through",
      "resolved": "https://registry.npmjs.org/through/-/through-2.3.8.tgz",
      "version": "2.3.8",
    },
    "tough-cookie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/sax/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.3.2.tgz",
      "version": "2.3.2",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/sax/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "tryit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-resolvable",
          "name": "tryit",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tryit",
      "name": "tryit",
      "path": "{CWD}/test/fixtures/sax/node_modules/tryit",
      "resolved": "https://registry.npmjs.org/tryit/-/tryit-1.0.3.tgz",
      "version": "1.0.3",
    },
    "tsame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tsame",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/tsame",
      "name": "tsame",
      "path": "{CWD}/test/fixtures/sax/node_modules/tsame",
      "resolved": "https://registry.npmjs.org/tsame/-/tsame-1.1.2.tgz",
      "version": "1.1.2",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/sax/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.4.3.tgz",
      "version": "0.4.3",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "optional",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "typedarray",
          "spec": "^0.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray",
      "name": "typedarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/typedarray",
      "resolved": "https://registry.npmjs.org/typedarray/-/typedarray-0.0.6.tgz",
      "version": "0.0.6",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/sax/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/sax/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/uniq",
      "name": "uniq",
      "path": "{CWD}/test/fixtures/sax/node_modules/uniq",
      "resolved": "https://registry.npmjs.org/uniq/-/uniq-1.0.1.tgz",
      "version": "1.0.1",
    },
    "user-home" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "user-home",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
      },
      "location": "node_modules/user-home",
      "name": "user-home",
      "path": "{CWD}/test/fixtures/sax/node_modules/user-home",
      "resolved": "https://registry.npmjs.org/user-home/-/user-home-2.0.0.tgz",
      "version": "2.0.0",
    },
    "util-deprecate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "path": "{CWD}/test/fixtures/sax/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.1.tgz",
      "version": "1.0.1",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/sax/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.0.1.tgz",
      "version": "3.0.1",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.3.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/sax/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.3.6.tgz",
      "version": "1.3.6",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "wordwrap",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/fixtures/sax/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-1.0.0.tgz",
      "version": "1.0.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/sax/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.1.tgz",
      "version": "1.0.1",
    },
    "write" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "write",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/write",
      "name": "write",
      "path": "{CWD}/test/fixtures/sax/node_modules/write",
      "resolved": "https://registry.npmjs.org/write/-/write-0.2.1.tgz",
      "version": "0.2.1",
    },
    "xtend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/xtend",
      "name": "xtend",
      "path": "{CWD}/test/fixtures/sax/node_modules/xtend",
      "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/sax/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.0.0.tgz",
      "version": "2.0.0",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/sax/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "standard" => EdgeOut {
      "name": "standard",
      "spec": "^8.6.0",
      "to": "node_modules/standard",
      "type": "dev",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "^10.5.1",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "sax",
  "path": "{CWD}/test/fixtures/sax",
  "version": "1.2.4",
}
`

exports[`test/arborist/build-ideal-tree.js TAP inflate an ancient lockfile with a dep gone missing > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/this-does-not-exist-at-all" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/this-does-not-exist-at-all",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/this-does-not-exist-at-all",
      "name": "@isaacs/this-does-not-exist-at-all",
      "path": "{CWD}/test/fixtures/ancient-lockfile-invalid/node_modules/@isaacs/this-does-not-exist-at-all",
      "resolved": "https://registry.npmjs.org/@isaacs%2fthis-does-not-exist-at-all/-/this-does-not-exist-at-all-1.2.3.tgz",
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "@isaacs/this-does-not-exist-at-all" => EdgeOut {
      "name": "@isaacs/this-does-not-exist-at-all",
      "spec": "1",
      "to": "node_modules/@isaacs/this-does-not-exist-at-all",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "ancient-lockfile-invalid",
  "path": "{CWD}/test/fixtures/ancient-lockfile-invalid",
}
`

exports[`test/arborist/build-ideal-tree.js TAP inflating a link node in an old lockfile skips registry > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "link-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "link-dep",
          "spec": "file:./link-dep",
          "type": "prod",
        },
      },
      "location": "node_modules/link-dep",
      "name": "link-dep",
      "path": "{CWD}/test/fixtures/old-lock-with-link/node_modules/link-dep",
      "realpath": "{CWD}/test/fixtures/old-lock-with-link/link-dep",
      "resolved": "file:../link-dep",
      "target": ArboristNode {
        "location": "link-dep",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "link-dep" => EdgeOut {
      "name": "link-dep",
      "spec": "file:./link-dep",
      "to": "node_modules/link-dep",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "link-dep",
      "name": "link-dep",
      "path": "{CWD}/test/fixtures/old-lock-with-link/link-dep",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "old-lock-with-link",
  "path": "{CWD}/test/fixtures/old-lock-with-link",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep with a link dep > link metadeps with lockfile 1`] = `
ArboristNode {
  "children": Map {
    "app" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "app",
          "spec": "file:./app",
          "type": "prod",
        },
      },
      "location": "node_modules/app",
      "name": "app",
      "path": "{CWD}/test/fixtures/cli-750/node_modules/app",
      "realpath": "{CWD}/test/fixtures/cli-750/app",
      "resolved": "file:../app",
      "target": ArboristNode {
        "location": "app",
      },
    },
    "lib" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "app",
          "name": "lib",
          "spec": "file:../lib",
          "type": "prod",
        },
      },
      "location": "node_modules/lib",
      "name": "lib",
      "path": "{CWD}/test/fixtures/cli-750/node_modules/lib",
      "realpath": "{CWD}/test/fixtures/cli-750/lib",
      "resolved": "file:../lib",
      "target": ArboristNode {
        "location": "lib",
      },
    },
  },
  "edgesOut": Map {
    "app" => EdgeOut {
      "name": "app",
      "spec": "file:./app",
      "to": "node_modules/app",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "lib" => EdgeOut {
          "name": "lib",
          "spec": "file:../lib",
          "to": "node_modules/lib",
          "type": "prod",
        },
      },
      "location": "app",
      "name": "app",
      "path": "{CWD}/test/fixtures/cli-750/app",
    },
    ArboristNode {
      "location": "lib",
      "name": "lib",
      "path": "{CWD}/test/fixtures/cli-750/lib",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "cli-750",
  "packageName": "monorepo",
  "path": "{CWD}/test/fixtures/cli-750",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep with a link dep > link metadeps without lockfile 1`] = `
ArboristNode {
  "children": Map {
    "app" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "app",
          "spec": "file:./app",
          "type": "prod",
        },
      },
      "location": "node_modules/app",
      "name": "app",
      "path": "{CWD}/test/fixtures/cli-750/node_modules/app",
      "realpath": "{CWD}/test/fixtures/cli-750/app",
      "resolved": "file:../app",
      "target": ArboristNode {
        "location": "app",
      },
    },
    "lib" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "app",
          "name": "lib",
          "spec": "file:../lib",
          "type": "prod",
        },
      },
      "location": "node_modules/lib",
      "name": "lib",
      "path": "{CWD}/test/fixtures/cli-750/node_modules/lib",
      "realpath": "{CWD}/test/fixtures/cli-750/lib",
      "resolved": "file:../lib",
      "target": ArboristNode {
        "location": "lib",
      },
    },
  },
  "edgesOut": Map {
    "app" => EdgeOut {
      "name": "app",
      "spec": "file:./app",
      "to": "node_modules/app",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "lib" => EdgeOut {
          "name": "lib",
          "spec": "file:../lib",
          "to": "node_modules/lib",
          "type": "prod",
        },
      },
      "location": "app",
      "name": "app",
      "path": "{CWD}/test/fixtures/cli-750/app",
    },
    ArboristNode {
      "location": "lib",
      "name": "lib",
      "path": "{CWD}/test/fixtures/cli-750/lib",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "cli-750",
  "packageName": "monorepo",
  "path": "{CWD}/test/fixtures/cli-750",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep within node_modules and outside root > linky deps followed 1`] = `
ArboristNode {
  "children": Map {
    "aaaaaa" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "aaaaaa",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/aaaaaa",
      "name": "aaaaaa",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/aaaaaa",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "abbrev",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/abbrev",
        "name": "abbrev",
        "path": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
        "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      },
    },
    "abbrev" => ArboristNode {
      "location": "node_modules/abbrev",
    },
    "monorepo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "monorepo",
          "spec": "../cli-750",
          "type": "prod",
        },
      },
      "location": "node_modules/monorepo",
      "name": "monorepo",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/monorepo",
      "realpath": "{CWD}/test/fixtures/cli-750",
      "resolved": "file:../../cli-750",
      "target": ArboristNode {
        "children": Map {
          "app" => ArboristLink {
            "edgesIn": Set {
              EdgeIn {
                "from": "../cli-750",
                "name": "app",
                "spec": "file:./app",
                "type": "prod",
              },
            },
            "location": "../cli-750/node_modules/app",
            "name": "app",
            "path": "{CWD}/test/fixtures/cli-750/node_modules/app",
            "realpath": "{CWD}/test/fixtures/cli-750/app",
            "resolved": "file:../app",
            "target": ArboristNode {
              "location": "../cli-750/app",
            },
          },
          "lib" => ArboristLink {
            "edgesIn": Set {
              EdgeIn {
                "from": "../cli-750/app",
                "name": "lib",
                "spec": "file:../lib",
                "type": "prod",
              },
            },
            "location": "../cli-750/node_modules/lib",
            "name": "lib",
            "path": "{CWD}/test/fixtures/cli-750/node_modules/lib",
            "realpath": "{CWD}/test/fixtures/cli-750/lib",
            "resolved": "file:../lib",
            "target": ArboristNode {
              "location": "../cli-750/lib",
            },
          },
        },
        "edgesOut": Map {
          "app" => EdgeOut {
            "name": "app",
            "spec": "file:./app",
            "to": "../cli-750/node_modules/app",
            "type": "prod",
          },
        },
        "fsChildren": Set {
          ArboristNode {
            "edgesOut": Map {
              "lib" => EdgeOut {
                "name": "lib",
                "spec": "file:../lib",
                "to": "../cli-750/node_modules/lib",
                "type": "prod",
              },
            },
            "location": "../cli-750/app",
            "name": "app",
            "path": "{CWD}/test/fixtures/cli-750/app",
          },
          ArboristNode {
            "location": "../cli-750/lib",
            "name": "lib",
            "path": "{CWD}/test/fixtures/cli-750/lib",
          },
        },
        "location": "../cli-750",
        "name": "cli-750",
        "packageName": "monorepo",
        "path": "{CWD}/test/fixtures/cli-750",
      },
    },
    "zzzzzz" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "zzzzzz",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/zzzzzz",
      "name": "zzzzzz",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/zzzzzz",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "location": "node_modules/abbrev",
      },
    },
  },
  "edgesOut": Map {
    "aaaaaa" => EdgeOut {
      "name": "aaaaaa",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/aaaaaa",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "monorepo" => EdgeOut {
      "name": "monorepo",
      "spec": "../cli-750",
      "to": "node_modules/monorepo",
      "type": "prod",
    },
    "zzzzzz" => EdgeOut {
      "name": "zzzzzz",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/zzzzzz",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "external-link-dep",
  "path": "{CWD}/test/fixtures/external-link-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep within node_modules and outside root > linky deps followed without lockfile 1`] = `
ArboristNode {
  "children": Map {
    "aaaaaa" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "aaaaaa",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/aaaaaa",
      "name": "aaaaaa",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/aaaaaa",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "abbrev",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/abbrev",
        "name": "abbrev",
        "path": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
        "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
        "version": "1.1.1",
      },
      "version": "1.1.1",
    },
    "abbrev" => ArboristNode {
      "location": "node_modules/abbrev",
    },
    "monorepo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "monorepo",
          "spec": "../cli-750",
          "type": "prod",
        },
      },
      "location": "node_modules/monorepo",
      "name": "monorepo",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/monorepo",
      "realpath": "{CWD}/test/fixtures/cli-750",
      "resolved": "file:../../cli-750",
      "target": ArboristNode {
        "children": Map {
          "app" => ArboristLink {
            "edgesIn": Set {
              EdgeIn {
                "from": "../cli-750",
                "name": "app",
                "spec": "file:./app",
                "type": "prod",
              },
            },
            "location": "../cli-750/node_modules/app",
            "name": "app",
            "path": "{CWD}/test/fixtures/cli-750/node_modules/app",
            "realpath": "{CWD}/test/fixtures/cli-750/app",
            "resolved": "file:../app",
            "target": ArboristNode {
              "location": "../cli-750/app",
            },
          },
          "lib" => ArboristLink {
            "edgesIn": Set {
              EdgeIn {
                "from": "../cli-750/app",
                "name": "lib",
                "spec": "file:../lib",
                "type": "prod",
              },
            },
            "location": "../cli-750/node_modules/lib",
            "name": "lib",
            "path": "{CWD}/test/fixtures/cli-750/node_modules/lib",
            "realpath": "{CWD}/test/fixtures/cli-750/lib",
            "resolved": "file:../lib",
            "target": ArboristNode {
              "location": "../cli-750/lib",
            },
          },
        },
        "edgesOut": Map {
          "app" => EdgeOut {
            "name": "app",
            "spec": "file:./app",
            "to": "../cli-750/node_modules/app",
            "type": "prod",
          },
        },
        "fsChildren": Set {
          ArboristNode {
            "edgesOut": Map {
              "lib" => EdgeOut {
                "name": "lib",
                "spec": "file:../lib",
                "to": "../cli-750/node_modules/lib",
                "type": "prod",
              },
            },
            "location": "../cli-750/app",
            "name": "app",
            "path": "{CWD}/test/fixtures/cli-750/app",
          },
          ArboristNode {
            "location": "../cli-750/lib",
            "name": "lib",
            "path": "{CWD}/test/fixtures/cli-750/lib",
          },
        },
        "location": "../cli-750",
        "name": "monorepo",
        "path": "{CWD}/test/fixtures/cli-750",
      },
    },
    "zzzzzz" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "zzzzzz",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/zzzzzz",
      "name": "zzzzzz",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/zzzzzz",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "location": "node_modules/abbrev",
      },
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "aaaaaa" => EdgeOut {
      "name": "aaaaaa",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/aaaaaa",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "monorepo" => EdgeOut {
      "name": "monorepo",
      "spec": "../cli-750",
      "to": "node_modules/monorepo",
      "type": "prod",
    },
    "zzzzzz" => EdgeOut {
      "name": "zzzzzz",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/zzzzzz",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "external-link-dep",
  "path": "{CWD}/test/fixtures/external-link-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep within node_modules and outside root > linky deps with lockfile 1`] = `
ArboristNode {
  "children": Map {
    "aaaaaa" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "aaaaaa",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/aaaaaa",
      "name": "aaaaaa",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/aaaaaa",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "abbrev",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/abbrev",
        "name": "abbrev",
        "path": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
        "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      },
    },
    "abbrev" => ArboristNode {
      "location": "node_modules/abbrev",
    },
    "monorepo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "monorepo",
          "spec": "../cli-750",
          "type": "prod",
        },
      },
      "location": "node_modules/monorepo",
      "name": "monorepo",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/monorepo",
      "realpath": "{CWD}/test/fixtures/cli-750",
      "resolved": "file:../../cli-750",
      "target": ArboristNode {
        "edgesOut": Map {
          "app" => EdgeOut {
            "error": "MISSING",
            "name": "app",
            "spec": "file:./app",
            "to": null,
            "type": "prod",
          },
        },
        "location": "../cli-750",
        "name": "cli-750",
        "packageName": "monorepo",
        "path": "{CWD}/test/fixtures/cli-750",
      },
    },
    "zzzzzz" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "zzzzzz",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/zzzzzz",
      "name": "zzzzzz",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/zzzzzz",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "location": "node_modules/abbrev",
      },
    },
  },
  "edgesOut": Map {
    "aaaaaa" => EdgeOut {
      "name": "aaaaaa",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/aaaaaa",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "monorepo" => EdgeOut {
      "name": "monorepo",
      "spec": "../cli-750",
      "to": "node_modules/monorepo",
      "type": "prod",
    },
    "zzzzzz" => EdgeOut {
      "name": "zzzzzz",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/zzzzzz",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "external-link-dep",
  "path": "{CWD}/test/fixtures/external-link-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link dep within node_modules and outside root > linky deps without lockfile 1`] = `
ArboristNode {
  "children": Map {
    "aaaaaa" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "aaaaaa",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/aaaaaa",
      "name": "aaaaaa",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/aaaaaa",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "abbrev",
            "spec": "*",
            "type": "prod",
          },
        },
        "location": "node_modules/abbrev",
        "name": "abbrev",
        "path": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
        "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
        "version": "1.1.1",
      },
      "version": "1.1.1",
    },
    "abbrev" => ArboristNode {
      "location": "node_modules/abbrev",
    },
    "monorepo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "monorepo",
          "spec": "../cli-750",
          "type": "prod",
        },
      },
      "location": "node_modules/monorepo",
      "name": "monorepo",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/monorepo",
      "realpath": "{CWD}/test/fixtures/cli-750",
      "resolved": "file:../../cli-750",
      "target": ArboristNode {
        "edgesOut": Map {
          "app" => EdgeOut {
            "error": "MISSING",
            "name": "app",
            "spec": "file:./app",
            "to": null,
            "type": "prod",
          },
        },
        "location": "../cli-750",
        "name": "monorepo",
        "path": "{CWD}/test/fixtures/cli-750",
      },
    },
    "zzzzzz" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "zzzzzz",
          "spec": "file:./node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/zzzzzz",
      "name": "zzzzzz",
      "packageName": "abbrev",
      "path": "{CWD}/test/fixtures/external-link-dep/node_modules/zzzzzz",
      "realpath": "{CWD}/test/fixtures/external-link-dep/node_modules/abbrev",
      "resolved": "file:abbrev",
      "target": ArboristNode {
        "location": "node_modules/abbrev",
      },
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "aaaaaa" => EdgeOut {
      "name": "aaaaaa",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/aaaaaa",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "monorepo" => EdgeOut {
      "name": "monorepo",
      "spec": "../cli-750",
      "to": "node_modules/monorepo",
      "type": "prod",
    },
    "zzzzzz" => EdgeOut {
      "name": "zzzzzz",
      "spec": "file:./node_modules/abbrev",
      "to": "node_modules/zzzzzz",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "external-link-dep",
  "path": "{CWD}/test/fixtures/external-link-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP link meta deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-link-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dep",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "linked-dep" => EdgeOut {
          "name": "linked-dep",
          "spec": "file:target",
          "to": "node_modules/linked-dep",
          "type": "prod",
        },
      },
      "fsChildren": Set {
        ArboristNode {
          "location": "node_modules/@isaacs/testing-link-dep/target",
          "name": "linked-dep",
          "path": "{CWD}/test/fixtures/link-meta-deps-empty/node_modules/@isaacs/testing-link-dep/target",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dep",
      "name": "@isaacs/testing-link-dep",
      "path": "{CWD}/test/fixtures/link-meta-deps-empty/node_modules/@isaacs/testing-link-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dep/-/testing-link-dep-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-link-dev-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dev-dep",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dev-dep",
      "name": "@isaacs/testing-link-dev-dep",
      "path": "{CWD}/test/fixtures/link-meta-deps-empty/node_modules/@isaacs/testing-link-dev-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dev-dep/-/testing-link-dev-dep-1.0.1.tgz",
      "version": "1.0.1",
    },
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-link-dep",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/fixtures/link-meta-deps-empty/node_modules/linked-dep",
      "realpath": "{CWD}/test/fixtures/link-meta-deps-empty/node_modules/@isaacs/testing-link-dep/target",
      "resolved": "file:@isaacs/testing-link-dep/target",
      "target": ArboristNode {
        "location": "node_modules/@isaacs/testing-link-dep/target",
      },
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-link-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dep",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-link-dep",
      "type": "prod",
    },
    "@isaacs/testing-link-dev-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dev-dep",
      "spec": "^1.0.1",
      "to": "node_modules/@isaacs/testing-link-dev-dep",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "link-meta-deps-empty",
  "path": "{CWD}/test/fixtures/link-meta-deps-empty",
}
`

exports[`test/arborist/build-ideal-tree.js TAP make sure yargs works > yargs should build fine 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/compat-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-compilation-targets",
          "name": "@babel/compat-data",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/compat-data",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/compat-data",
      "name": "@babel/compat-data",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/compat-data",
      "resolved": "https://registry.npmjs.org/@babel/compat-data/-/compat-data-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/core" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-compilation-targets",
          "name": "@babel/core",
          "spec": "^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/core",
          "spec": "^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-create-regexp-features-plugin",
          "name": "@babel/core",
          "spec": "^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-class-properties",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-dynamic-import",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-export-namespace-from",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-json-strings",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-logical-assignment-operators",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-numeric-separator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-catch-binding",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-chaining",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-private-methods",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-async-generators",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-class-properties",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-dynamic-import",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-export-namespace-from",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-json-strings",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-logical-assignment-operators",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-numeric-separator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-optional-catch-binding",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-optional-chaining",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-top-level-await",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-arrow-functions",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-async-to-generator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-block-scoped-functions",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-block-scoping",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-computed-properties",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-dotall-regex",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-duplicate-keys",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-exponentiation-operator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-for-of",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-function-name",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-literals",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-member-expression-literals",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-amd",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-commonjs",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-umd",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-named-capturing-groups-regex",
          "name": "@babel/core",
          "spec": "^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-new-target",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-object-super",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-property-literals",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-regenerator",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-reserved-words",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-runtime",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-shorthand-properties",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-sticky-regex",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-template-literals",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-typeof-symbol",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-unicode-escapes",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-unicode-regex",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/core",
          "spec": "^7.11.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helpers" => EdgeOut {
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helpers",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "gensync" => EdgeOut {
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "to": "node_modules/gensync",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^2.1.2",
          "to": "node_modules/json5",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.3.2",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.4.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/core",
      "name": "@babel/core",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/core",
      "resolved": "https://registry.npmjs.org/@babel/core/-/core-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/generator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-annotate-as-pure" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-regexp-features-plugin",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-remap-async-to-generator",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-annotate-as-pure",
      "name": "@babel/helper-annotate-as-pure",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-annotate-as-pure",
      "resolved": "https://registry.npmjs.org/@babel/helper-annotate-as-pure/-/helper-annotate-as-pure-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-builder-binary-assignment-operator-visitor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-exponentiation-operator",
          "name": "@babel/helper-builder-binary-assignment-operator-visitor",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-explode-assignable-expression" => EdgeOut {
          "name": "@babel/helper-explode-assignable-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-explode-assignable-expression",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-binary-assignment-operator-visitor",
      "name": "@babel/helper-builder-binary-assignment-operator-visitor",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-builder-binary-assignment-operator-visitor",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-binary-assignment-operator-visitor/-/helper-builder-binary-assignment-operator-visitor-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-compilation-targets" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/helper-compilation-targets",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/compat-data" => EdgeOut {
          "name": "@babel/compat-data",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/compat-data",
          "type": "prod",
        },
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-validator-option" => EdgeOut {
          "name": "@babel/helper-validator-option",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-validator-option",
          "type": "prod",
        },
        "browserslist" => EdgeOut {
          "name": "browserslist",
          "spec": "^4.12.0",
          "to": "node_modules/browserslist",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.5.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-compilation-targets",
      "name": "@babel/helper-compilation-targets",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-compilation-targets",
      "resolved": "https://registry.npmjs.org/@babel/helper-compilation-targets/-/helper-compilation-targets-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-create-class-features-plugin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-class-properties",
          "name": "@babel/helper-create-class-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-private-methods",
          "name": "@babel/helper-create-class-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-member-expression-to-functions" => EdgeOut {
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-member-expression-to-functions",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-create-class-features-plugin",
      "name": "@babel/helper-create-class-features-plugin",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-create-class-features-plugin",
      "resolved": "https://registry.npmjs.org/@babel/helper-create-class-features-plugin/-/helper-create-class-features-plugin-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-create-regexp-features-plugin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-dotall-regex",
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-named-capturing-groups-regex",
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-unicode-regex",
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/helper-regex" => EdgeOut {
          "name": "@babel/helper-regex",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-regex",
          "type": "prod",
        },
        "regexpu-core" => EdgeOut {
          "name": "regexpu-core",
          "spec": "^4.7.1",
          "to": "node_modules/regexpu-core",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-create-regexp-features-plugin",
      "name": "@babel/helper-create-regexp-features-plugin",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-create-regexp-features-plugin",
      "resolved": "https://registry.npmjs.org/@babel/helper-create-regexp-features-plugin/-/helper-create-regexp-features-plugin-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-define-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-define-map",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.5",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-define-map",
      "name": "@babel/helper-define-map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-define-map",
      "resolved": "https://registry.npmjs.org/@babel/helper-define-map/-/helper-define-map-7.10.5.tgz",
      "version": "7.10.5",
    },
    "@babel/helper-explode-assignable-expression" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-binary-assignment-operator-visitor",
          "name": "@babel/helper-explode-assignable-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-explode-assignable-expression",
      "name": "@babel/helper-explode-assignable-expression",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-explode-assignable-expression",
      "resolved": "https://registry.npmjs.org/@babel/helper-explode-assignable-expression/-/helper-explode-assignable-expression-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-define-map",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-wrap-function",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-function-name",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-hoist-variables" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "@babel/helper-hoist-variables",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-hoist-variables",
      "name": "@babel/helper-hoist-variables",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-hoist-variables",
      "resolved": "https://registry.npmjs.org/@babel/helper-hoist-variables/-/helper-hoist-variables-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-member-expression-to-functions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-member-expression-to-functions",
      "name": "@babel/helper-member-expression-to-functions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-member-expression-to-functions",
      "resolved": "https://registry.npmjs.org/@babel/helper-member-expression-to-functions/-/helper-member-expression-to-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-imports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-async-to-generator",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-runtime",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-imports",
      "name": "@babel/helper-module-imports",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-module-imports",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-imports/-/helper-module-imports-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-transforms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-amd",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-commonjs",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-umd",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-simple-access" => EdgeOut {
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-simple-access",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-transforms",
      "name": "@babel/helper-module-transforms",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-module-transforms",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-transforms/-/helper-module-transforms-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-optimise-call-expression" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-optimise-call-expression",
      "name": "@babel/helper-optimise-call-expression",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-optimise-call-expression",
      "resolved": "https://registry.npmjs.org/@babel/helper-optimise-call-expression/-/helper-optimise-call-expression-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-plugin-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-class-properties",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-dynamic-import",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-export-namespace-from",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-json-strings",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-logical-assignment-operators",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-numeric-separator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-catch-binding",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-chaining",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-private-methods",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-async-generators",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-class-properties",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-dynamic-import",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-export-namespace-from",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-json-strings",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-logical-assignment-operators",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-numeric-separator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-optional-catch-binding",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-optional-chaining",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-top-level-await",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-arrow-functions",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-async-to-generator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-block-scoped-functions",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-block-scoping",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-computed-properties",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-dotall-regex",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-duplicate-keys",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-exponentiation-operator",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-for-of",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-function-name",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-literals",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-member-expression-literals",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-amd",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-commonjs",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-umd",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-new-target",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-object-super",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-property-literals",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-reserved-words",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-runtime",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-shorthand-properties",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-sticky-regex",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-template-literals",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-typeof-symbol",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-unicode-escapes",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-unicode-regex",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-plugin-utils",
      "name": "@babel/helper-plugin-utils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-plugin-utils",
      "resolved": "https://registry.npmjs.org/@babel/helper-plugin-utils/-/helper-plugin-utils-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-regexp-features-plugin",
          "name": "@babel/helper-regex",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-sticky-regex",
          "name": "@babel/helper-regex",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-regex",
      "name": "@babel/helper-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-regex",
      "resolved": "https://registry.npmjs.org/@babel/helper-regex/-/helper-regex-7.10.5.tgz",
      "version": "7.10.5",
    },
    "@babel/helper-remap-async-to-generator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "name": "@babel/helper-remap-async-to-generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-async-to-generator",
          "name": "@babel/helper-remap-async-to-generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/helper-wrap-function" => EdgeOut {
          "name": "@babel/helper-wrap-function",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-wrap-function",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-remap-async-to-generator",
      "name": "@babel/helper-remap-async-to-generator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-remap-async-to-generator",
      "resolved": "https://registry.npmjs.org/@babel/helper-remap-async-to-generator/-/helper-remap-async-to-generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-replace-supers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-object-super",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-member-expression-to-functions" => EdgeOut {
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-member-expression-to-functions",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-replace-supers",
      "name": "@babel/helper-replace-supers",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-replace-supers",
      "resolved": "https://registry.npmjs.org/@babel/helper-replace-supers/-/helper-replace-supers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-simple-access" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-commonjs",
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-simple-access",
      "name": "@babel/helper-simple-access",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-simple-access",
      "resolved": "https://registry.npmjs.org/@babel/helper-simple-access/-/helper-simple-access-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-skip-transparent-expression-wrappers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-chaining",
          "name": "@babel/helper-skip-transparent-expression-wrappers",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-spread",
          "name": "@babel/helper-skip-transparent-expression-wrappers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-skip-transparent-expression-wrappers",
      "name": "@babel/helper-skip-transparent-expression-wrappers",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-skip-transparent-expression-wrappers",
      "resolved": "https://registry.npmjs.org/@babel/helper-skip-transparent-expression-wrappers/-/helper-skip-transparent-expression-wrappers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-class-features-plugin",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.11.0.tgz",
      "version": "7.11.0",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-validator-option" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-compilation-targets",
          "name": "@babel/helper-validator-option",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/helper-validator-option",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-option",
      "name": "@babel/helper-validator-option",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-validator-option",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-option/-/helper-validator-option-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-wrap-function" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-remap-async-to-generator",
          "name": "@babel/helper-wrap-function",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-wrap-function",
      "name": "@babel/helper-wrap-function",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helper-wrap-function",
      "resolved": "https://registry.npmjs.org/@babel/helper-wrap-function/-/helper-wrap-function-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helpers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helpers",
      "name": "@babel/helpers",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/helpers",
      "resolved": "https://registry.npmjs.org/@babel/helpers/-/helpers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/highlight" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/@babel/highlight/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight",
              "name": "chalk",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "to": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^5.3.0",
              "to": "node_modules/@babel/highlight/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
          "version": "2.4.2",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/@babel/highlight/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "supports-color",
              "spec": "^5.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/@babel/highlight/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
          "version": "5.5.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/@babel/highlight/node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__core",
          "name": "@babel/parser",
          "spec": "^7.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__template",
          "name": "@babel/parser",
          "spec": "^7.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-async-generator-functions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-async-generator-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-proposal-async-generator-functions",
          "spec": "^7.10.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-remap-async-to-generator" => EdgeOut {
          "name": "@babel/helper-remap-async-to-generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-remap-async-to-generator",
          "type": "prod",
        },
        "@babel/plugin-syntax-async-generators" => EdgeOut {
          "name": "@babel/plugin-syntax-async-generators",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-async-generators",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-async-generator-functions",
      "name": "@babel/plugin-proposal-async-generator-functions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-async-generator-functions",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-async-generator-functions/-/plugin-proposal-async-generator-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-class-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-class-properties",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-class-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-class-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-class-features-plugin",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-class-properties",
      "name": "@babel/plugin-proposal-class-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-class-properties",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-class-properties/-/plugin-proposal-class-properties-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-dynamic-import" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-dynamic-import",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-dynamic-import" => EdgeOut {
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-dynamic-import",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-dynamic-import",
      "name": "@babel/plugin-proposal-dynamic-import",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-dynamic-import",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-dynamic-import/-/plugin-proposal-dynamic-import-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-export-namespace-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-export-namespace-from",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-export-namespace-from" => EdgeOut {
          "name": "@babel/plugin-syntax-export-namespace-from",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/plugin-syntax-export-namespace-from",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-export-namespace-from",
      "name": "@babel/plugin-proposal-export-namespace-from",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-export-namespace-from",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-export-namespace-from/-/plugin-proposal-export-namespace-from-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-json-strings" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-json-strings",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-proposal-json-strings",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-json-strings" => EdgeOut {
          "name": "@babel/plugin-syntax-json-strings",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-json-strings",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-json-strings",
      "name": "@babel/plugin-proposal-json-strings",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-json-strings",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-json-strings/-/plugin-proposal-json-strings-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-logical-assignment-operators" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-logical-assignment-operators",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-logical-assignment-operators" => EdgeOut {
          "name": "@babel/plugin-syntax-logical-assignment-operators",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-syntax-logical-assignment-operators",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-logical-assignment-operators",
      "name": "@babel/plugin-proposal-logical-assignment-operators",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-logical-assignment-operators",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-logical-assignment-operators/-/plugin-proposal-logical-assignment-operators-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-nullish-coalescing-operator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-nullish-coalescing-operator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-nullish-coalescing-operator" => EdgeOut {
          "name": "@babel/plugin-syntax-nullish-coalescing-operator",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
      "name": "@babel/plugin-proposal-nullish-coalescing-operator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-nullish-coalescing-operator/-/plugin-proposal-nullish-coalescing-operator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-numeric-separator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-numeric-separator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-numeric-separator" => EdgeOut {
          "name": "@babel/plugin-syntax-numeric-separator",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-syntax-numeric-separator",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-numeric-separator",
      "name": "@babel/plugin-proposal-numeric-separator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-numeric-separator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-numeric-separator/-/plugin-proposal-numeric-separator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-parameters" => EdgeOut {
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-parameters",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-object-rest-spread",
      "name": "@babel/plugin-proposal-object-rest-spread",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-object-rest-spread/-/plugin-proposal-object-rest-spread-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-optional-catch-binding" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-optional-catch-binding",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-proposal-optional-catch-binding",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-optional-catch-binding" => EdgeOut {
          "name": "@babel/plugin-syntax-optional-catch-binding",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-optional-catch-binding",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-optional-catch-binding",
      "name": "@babel/plugin-proposal-optional-catch-binding",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-optional-catch-binding",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-optional-catch-binding/-/plugin-proposal-optional-catch-binding-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-optional-chaining" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-optional-chaining",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-skip-transparent-expression-wrappers" => EdgeOut {
          "name": "@babel/helper-skip-transparent-expression-wrappers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-skip-transparent-expression-wrappers",
          "type": "prod",
        },
        "@babel/plugin-syntax-optional-chaining" => EdgeOut {
          "name": "@babel/plugin-syntax-optional-chaining",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-optional-chaining",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-optional-chaining",
      "name": "@babel/plugin-proposal-optional-chaining",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-optional-chaining",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-optional-chaining/-/plugin-proposal-optional-chaining-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-private-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-private-methods",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-class-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-class-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-class-features-plugin",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-private-methods",
      "name": "@babel/plugin-proposal-private-methods",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-private-methods",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-private-methods/-/plugin-proposal-private-methods-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-unicode-property-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.4.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-regexp-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-regexp-features-plugin",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-unicode-property-regex",
      "name": "@babel/plugin-proposal-unicode-property-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-proposal-unicode-property-regex",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-unicode-property-regex/-/plugin-proposal-unicode-property-regex-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-async-generators" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "name": "@babel/plugin-syntax-async-generators",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-async-generators",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-async-generators",
      "name": "@babel/plugin-syntax-async-generators",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-async-generators",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-async-generators/-/plugin-syntax-async-generators-7.8.4.tgz",
      "version": "7.8.4",
    },
    "@babel/plugin-syntax-class-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-class-properties",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-class-properties",
      "name": "@babel/plugin-syntax-class-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-class-properties",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-class-properties/-/plugin-syntax-class-properties-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-dynamic-import" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-dynamic-import",
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-dynamic-import",
      "name": "@babel/plugin-syntax-dynamic-import",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-dynamic-import",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-dynamic-import/-/plugin-syntax-dynamic-import-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-export-namespace-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-export-namespace-from",
          "name": "@babel/plugin-syntax-export-namespace-from",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-export-namespace-from",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-export-namespace-from",
      "name": "@babel/plugin-syntax-export-namespace-from",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-export-namespace-from",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-export-namespace-from/-/plugin-syntax-export-namespace-from-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-json-strings" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-json-strings",
          "name": "@babel/plugin-syntax-json-strings",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-json-strings",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-json-strings",
      "name": "@babel/plugin-syntax-json-strings",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-json-strings",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-json-strings/-/plugin-syntax-json-strings-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-logical-assignment-operators" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-logical-assignment-operators",
          "name": "@babel/plugin-syntax-logical-assignment-operators",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-logical-assignment-operators",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-logical-assignment-operators",
      "name": "@babel/plugin-syntax-logical-assignment-operators",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-logical-assignment-operators",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-logical-assignment-operators/-/plugin-syntax-logical-assignment-operators-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/plugin-syntax-nullish-coalescing-operator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
          "name": "@babel/plugin-syntax-nullish-coalescing-operator",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-nullish-coalescing-operator",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
      "name": "@babel/plugin-syntax-nullish-coalescing-operator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-nullish-coalescing-operator/-/plugin-syntax-nullish-coalescing-operator-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-numeric-separator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-numeric-separator",
          "name": "@babel/plugin-syntax-numeric-separator",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-numeric-separator",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-numeric-separator",
      "name": "@babel/plugin-syntax-numeric-separator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-numeric-separator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-numeric-separator/-/plugin-syntax-numeric-separator-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-object-rest-spread",
      "name": "@babel/plugin-syntax-object-rest-spread",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-object-rest-spread/-/plugin-syntax-object-rest-spread-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-optional-catch-binding" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-catch-binding",
          "name": "@babel/plugin-syntax-optional-catch-binding",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-optional-catch-binding",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-optional-catch-binding",
      "name": "@babel/plugin-syntax-optional-catch-binding",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-optional-catch-binding",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-optional-catch-binding/-/plugin-syntax-optional-catch-binding-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-optional-chaining" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-optional-chaining",
          "name": "@babel/plugin-syntax-optional-chaining",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-optional-chaining",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-optional-chaining",
      "name": "@babel/plugin-syntax-optional-chaining",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-optional-chaining",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-optional-chaining/-/plugin-syntax-optional-chaining-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-syntax-top-level-await" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-syntax-top-level-await",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-top-level-await",
      "name": "@babel/plugin-syntax-top-level-await",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-syntax-top-level-await",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-top-level-await/-/plugin-syntax-top-level-await-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-arrow-functions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-arrow-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-arrow-functions",
      "name": "@babel/plugin-transform-arrow-functions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-arrow-functions",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-arrow-functions/-/plugin-transform-arrow-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-async-to-generator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-async-to-generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-remap-async-to-generator" => EdgeOut {
          "name": "@babel/helper-remap-async-to-generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-remap-async-to-generator",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-async-to-generator",
      "name": "@babel/plugin-transform-async-to-generator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-async-to-generator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-async-to-generator/-/plugin-transform-async-to-generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-block-scoped-functions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-block-scoped-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-block-scoped-functions",
      "name": "@babel/plugin-transform-block-scoped-functions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-block-scoped-functions",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-block-scoped-functions/-/plugin-transform-block-scoped-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-block-scoping" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-block-scoping",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-block-scoping",
      "name": "@babel/plugin-transform-block-scoping",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-block-scoping",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-block-scoping/-/plugin-transform-block-scoping-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-classes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-classes",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/helper-define-map" => EdgeOut {
          "name": "@babel/helper-define-map",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-define-map",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-classes",
      "name": "@babel/plugin-transform-classes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-classes",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-classes/-/plugin-transform-classes-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-computed-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-computed-properties",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-computed-properties",
      "name": "@babel/plugin-transform-computed-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-computed-properties",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-computed-properties/-/plugin-transform-computed-properties-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-destructuring" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-destructuring",
      "name": "@babel/plugin-transform-destructuring",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-destructuring",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-destructuring/-/plugin-transform-destructuring-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-dotall-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-dotall-regex",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "@babel/plugin-transform-dotall-regex",
          "spec": "^7.4.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-regexp-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-regexp-features-plugin",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-dotall-regex",
      "name": "@babel/plugin-transform-dotall-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-dotall-regex",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-dotall-regex/-/plugin-transform-dotall-regex-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-duplicate-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-duplicate-keys",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-duplicate-keys",
      "name": "@babel/plugin-transform-duplicate-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-duplicate-keys",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-duplicate-keys/-/plugin-transform-duplicate-keys-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-exponentiation-operator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-exponentiation-operator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-builder-binary-assignment-operator-visitor" => EdgeOut {
          "name": "@babel/helper-builder-binary-assignment-operator-visitor",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-builder-binary-assignment-operator-visitor",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-exponentiation-operator",
      "name": "@babel/plugin-transform-exponentiation-operator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-exponentiation-operator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-exponentiation-operator/-/plugin-transform-exponentiation-operator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-for-of" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-for-of",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-for-of",
      "name": "@babel/plugin-transform-for-of",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-for-of",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-for-of/-/plugin-transform-for-of-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-function-name",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-function-name",
      "name": "@babel/plugin-transform-function-name",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-function-name",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-function-name/-/plugin-transform-function-name-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-literals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-literals",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-literals",
      "name": "@babel/plugin-transform-literals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-literals",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-literals/-/plugin-transform-literals-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-member-expression-literals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-member-expression-literals",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-member-expression-literals",
      "name": "@babel/plugin-transform-member-expression-literals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-member-expression-literals",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-member-expression-literals/-/plugin-transform-member-expression-literals-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-modules-amd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-modules-amd",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "babel-plugin-dynamic-import-node" => EdgeOut {
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "to": "node_modules/babel-plugin-dynamic-import-node",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-modules-amd",
      "name": "@babel/plugin-transform-modules-amd",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-modules-amd",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-modules-amd/-/plugin-transform-modules-amd-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-modules-commonjs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-modules-commonjs",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-simple-access" => EdgeOut {
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-simple-access",
          "type": "prod",
        },
        "babel-plugin-dynamic-import-node" => EdgeOut {
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "to": "node_modules/babel-plugin-dynamic-import-node",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-modules-commonjs",
      "name": "@babel/plugin-transform-modules-commonjs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-modules-commonjs",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-modules-commonjs/-/plugin-transform-modules-commonjs-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-modules-systemjs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-modules-systemjs",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-hoist-variables" => EdgeOut {
          "name": "@babel/helper-hoist-variables",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-hoist-variables",
          "type": "prod",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "babel-plugin-dynamic-import-node" => EdgeOut {
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "to": "node_modules/babel-plugin-dynamic-import-node",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-modules-systemjs",
      "name": "@babel/plugin-transform-modules-systemjs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-modules-systemjs",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-modules-systemjs/-/plugin-transform-modules-systemjs-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-modules-umd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-modules-umd",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-modules-umd",
      "name": "@babel/plugin-transform-modules-umd",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-modules-umd",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-modules-umd/-/plugin-transform-modules-umd-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-named-capturing-groups-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-named-capturing-groups-regex",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-regexp-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-regexp-features-plugin",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-named-capturing-groups-regex",
      "name": "@babel/plugin-transform-named-capturing-groups-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-named-capturing-groups-regex",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-named-capturing-groups-regex/-/plugin-transform-named-capturing-groups-regex-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-new-target" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-new-target",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-new-target",
      "name": "@babel/plugin-transform-new-target",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-new-target",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-new-target/-/plugin-transform-new-target-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-object-super" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-object-super",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-object-super",
      "name": "@babel/plugin-transform-object-super",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-object-super",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-object-super/-/plugin-transform-object-super-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-parameters" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-parameters",
      "name": "@babel/plugin-transform-parameters",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-parameters",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-parameters/-/plugin-transform-parameters-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-property-literals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-property-literals",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-property-literals",
      "name": "@babel/plugin-transform-property-literals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-property-literals",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-property-literals/-/plugin-transform-property-literals-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-regenerator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-regenerator",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "regenerator-transform" => EdgeOut {
          "name": "regenerator-transform",
          "spec": "^0.14.2",
          "to": "node_modules/regenerator-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-regenerator",
      "name": "@babel/plugin-transform-regenerator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-regenerator",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-regenerator/-/plugin-transform-regenerator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-reserved-words" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-reserved-words",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-reserved-words",
      "name": "@babel/plugin-transform-reserved-words",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-reserved-words",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-reserved-words/-/plugin-transform-reserved-words-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/plugin-transform-runtime",
          "spec": "^7.11.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.8.1",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.5.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-runtime",
      "name": "@babel/plugin-transform-runtime",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-runtime",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-runtime/-/plugin-transform-runtime-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-shorthand-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-shorthand-properties",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-shorthand-properties",
      "name": "@babel/plugin-transform-shorthand-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-shorthand-properties",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-shorthand-properties/-/plugin-transform-shorthand-properties-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-spread",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-skip-transparent-expression-wrappers" => EdgeOut {
          "name": "@babel/helper-skip-transparent-expression-wrappers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-skip-transparent-expression-wrappers",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-spread",
      "name": "@babel/plugin-transform-spread",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-spread/-/plugin-transform-spread-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-sticky-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-sticky-regex",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-regex" => EdgeOut {
          "name": "@babel/helper-regex",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-sticky-regex",
      "name": "@babel/plugin-transform-sticky-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-sticky-regex",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-sticky-regex/-/plugin-transform-sticky-regex-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-template-literals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-template-literals",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-template-literals",
      "name": "@babel/plugin-transform-template-literals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-template-literals",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-template-literals/-/plugin-transform-template-literals-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-typeof-symbol" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-typeof-symbol",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-typeof-symbol",
      "name": "@babel/plugin-transform-typeof-symbol",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-typeof-symbol",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-typeof-symbol/-/plugin-transform-typeof-symbol-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-unicode-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-unicode-escapes",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-unicode-escapes",
      "name": "@babel/plugin-transform-unicode-escapes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-unicode-escapes",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-unicode-escapes/-/plugin-transform-unicode-escapes-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-unicode-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/plugin-transform-unicode-regex",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-create-regexp-features-plugin" => EdgeOut {
          "name": "@babel/helper-create-regexp-features-plugin",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-create-regexp-features-plugin",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-unicode-regex",
      "name": "@babel/plugin-transform-unicode-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/plugin-transform-unicode-regex",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-unicode-regex/-/plugin-transform-unicode-regex-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/preset-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/preset-env",
          "spec": "^7.11.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/compat-data" => EdgeOut {
          "name": "@babel/compat-data",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/compat-data",
          "type": "prod",
        },
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-compilation-targets" => EdgeOut {
          "name": "@babel/helper-compilation-targets",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-compilation-targets",
          "type": "prod",
        },
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/helper-validator-option" => EdgeOut {
          "name": "@babel/helper-validator-option",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-validator-option",
          "type": "prod",
        },
        "@babel/plugin-proposal-async-generator-functions" => EdgeOut {
          "name": "@babel/plugin-proposal-async-generator-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "type": "prod",
        },
        "@babel/plugin-proposal-class-properties" => EdgeOut {
          "name": "@babel/plugin-proposal-class-properties",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-class-properties",
          "type": "prod",
        },
        "@babel/plugin-proposal-dynamic-import" => EdgeOut {
          "name": "@babel/plugin-proposal-dynamic-import",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-dynamic-import",
          "type": "prod",
        },
        "@babel/plugin-proposal-export-namespace-from" => EdgeOut {
          "name": "@babel/plugin-proposal-export-namespace-from",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-export-namespace-from",
          "type": "prod",
        },
        "@babel/plugin-proposal-json-strings" => EdgeOut {
          "name": "@babel/plugin-proposal-json-strings",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-json-strings",
          "type": "prod",
        },
        "@babel/plugin-proposal-logical-assignment-operators" => EdgeOut {
          "name": "@babel/plugin-proposal-logical-assignment-operators",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-logical-assignment-operators",
          "type": "prod",
        },
        "@babel/plugin-proposal-nullish-coalescing-operator" => EdgeOut {
          "name": "@babel/plugin-proposal-nullish-coalescing-operator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-nullish-coalescing-operator",
          "type": "prod",
        },
        "@babel/plugin-proposal-numeric-separator" => EdgeOut {
          "name": "@babel/plugin-proposal-numeric-separator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-numeric-separator",
          "type": "prod",
        },
        "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-proposal-optional-catch-binding" => EdgeOut {
          "name": "@babel/plugin-proposal-optional-catch-binding",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-optional-catch-binding",
          "type": "prod",
        },
        "@babel/plugin-proposal-optional-chaining" => EdgeOut {
          "name": "@babel/plugin-proposal-optional-chaining",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-optional-chaining",
          "type": "prod",
        },
        "@babel/plugin-proposal-private-methods" => EdgeOut {
          "name": "@babel/plugin-proposal-private-methods",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-private-methods",
          "type": "prod",
        },
        "@babel/plugin-proposal-unicode-property-regex" => EdgeOut {
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "type": "prod",
        },
        "@babel/plugin-syntax-async-generators" => EdgeOut {
          "name": "@babel/plugin-syntax-async-generators",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-async-generators",
          "type": "prod",
        },
        "@babel/plugin-syntax-class-properties" => EdgeOut {
          "name": "@babel/plugin-syntax-class-properties",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-syntax-class-properties",
          "type": "prod",
        },
        "@babel/plugin-syntax-dynamic-import" => EdgeOut {
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-dynamic-import",
          "type": "prod",
        },
        "@babel/plugin-syntax-export-namespace-from" => EdgeOut {
          "name": "@babel/plugin-syntax-export-namespace-from",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/plugin-syntax-export-namespace-from",
          "type": "prod",
        },
        "@babel/plugin-syntax-json-strings" => EdgeOut {
          "name": "@babel/plugin-syntax-json-strings",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-json-strings",
          "type": "prod",
        },
        "@babel/plugin-syntax-logical-assignment-operators" => EdgeOut {
          "name": "@babel/plugin-syntax-logical-assignment-operators",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-syntax-logical-assignment-operators",
          "type": "prod",
        },
        "@babel/plugin-syntax-nullish-coalescing-operator" => EdgeOut {
          "name": "@babel/plugin-syntax-nullish-coalescing-operator",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-nullish-coalescing-operator",
          "type": "prod",
        },
        "@babel/plugin-syntax-numeric-separator" => EdgeOut {
          "name": "@babel/plugin-syntax-numeric-separator",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-syntax-numeric-separator",
          "type": "prod",
        },
        "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-syntax-optional-catch-binding" => EdgeOut {
          "name": "@babel/plugin-syntax-optional-catch-binding",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-optional-catch-binding",
          "type": "prod",
        },
        "@babel/plugin-syntax-optional-chaining" => EdgeOut {
          "name": "@babel/plugin-syntax-optional-chaining",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-optional-chaining",
          "type": "prod",
        },
        "@babel/plugin-syntax-top-level-await" => EdgeOut {
          "name": "@babel/plugin-syntax-top-level-await",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-syntax-top-level-await",
          "type": "prod",
        },
        "@babel/plugin-transform-arrow-functions" => EdgeOut {
          "name": "@babel/plugin-transform-arrow-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-arrow-functions",
          "type": "prod",
        },
        "@babel/plugin-transform-async-to-generator" => EdgeOut {
          "name": "@babel/plugin-transform-async-to-generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-async-to-generator",
          "type": "prod",
        },
        "@babel/plugin-transform-block-scoped-functions" => EdgeOut {
          "name": "@babel/plugin-transform-block-scoped-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-block-scoped-functions",
          "type": "prod",
        },
        "@babel/plugin-transform-block-scoping" => EdgeOut {
          "name": "@babel/plugin-transform-block-scoping",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-block-scoping",
          "type": "prod",
        },
        "@babel/plugin-transform-classes" => EdgeOut {
          "name": "@babel/plugin-transform-classes",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-classes",
          "type": "prod",
        },
        "@babel/plugin-transform-computed-properties" => EdgeOut {
          "name": "@babel/plugin-transform-computed-properties",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-computed-properties",
          "type": "prod",
        },
        "@babel/plugin-transform-destructuring" => EdgeOut {
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-destructuring",
          "type": "prod",
        },
        "@babel/plugin-transform-dotall-regex" => EdgeOut {
          "name": "@babel/plugin-transform-dotall-regex",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-dotall-regex",
          "type": "prod",
        },
        "@babel/plugin-transform-duplicate-keys" => EdgeOut {
          "name": "@babel/plugin-transform-duplicate-keys",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-duplicate-keys",
          "type": "prod",
        },
        "@babel/plugin-transform-exponentiation-operator" => EdgeOut {
          "name": "@babel/plugin-transform-exponentiation-operator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-exponentiation-operator",
          "type": "prod",
        },
        "@babel/plugin-transform-for-of" => EdgeOut {
          "name": "@babel/plugin-transform-for-of",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-for-of",
          "type": "prod",
        },
        "@babel/plugin-transform-function-name" => EdgeOut {
          "name": "@babel/plugin-transform-function-name",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-function-name",
          "type": "prod",
        },
        "@babel/plugin-transform-literals" => EdgeOut {
          "name": "@babel/plugin-transform-literals",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-literals",
          "type": "prod",
        },
        "@babel/plugin-transform-member-expression-literals" => EdgeOut {
          "name": "@babel/plugin-transform-member-expression-literals",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-member-expression-literals",
          "type": "prod",
        },
        "@babel/plugin-transform-modules-amd" => EdgeOut {
          "name": "@babel/plugin-transform-modules-amd",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-modules-amd",
          "type": "prod",
        },
        "@babel/plugin-transform-modules-commonjs" => EdgeOut {
          "name": "@babel/plugin-transform-modules-commonjs",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-modules-commonjs",
          "type": "prod",
        },
        "@babel/plugin-transform-modules-systemjs" => EdgeOut {
          "name": "@babel/plugin-transform-modules-systemjs",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-modules-systemjs",
          "type": "prod",
        },
        "@babel/plugin-transform-modules-umd" => EdgeOut {
          "name": "@babel/plugin-transform-modules-umd",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-modules-umd",
          "type": "prod",
        },
        "@babel/plugin-transform-named-capturing-groups-regex" => EdgeOut {
          "name": "@babel/plugin-transform-named-capturing-groups-regex",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-named-capturing-groups-regex",
          "type": "prod",
        },
        "@babel/plugin-transform-new-target" => EdgeOut {
          "name": "@babel/plugin-transform-new-target",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-new-target",
          "type": "prod",
        },
        "@babel/plugin-transform-object-super" => EdgeOut {
          "name": "@babel/plugin-transform-object-super",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-object-super",
          "type": "prod",
        },
        "@babel/plugin-transform-parameters" => EdgeOut {
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-parameters",
          "type": "prod",
        },
        "@babel/plugin-transform-property-literals" => EdgeOut {
          "name": "@babel/plugin-transform-property-literals",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-property-literals",
          "type": "prod",
        },
        "@babel/plugin-transform-regenerator" => EdgeOut {
          "name": "@babel/plugin-transform-regenerator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-regenerator",
          "type": "prod",
        },
        "@babel/plugin-transform-reserved-words" => EdgeOut {
          "name": "@babel/plugin-transform-reserved-words",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-reserved-words",
          "type": "prod",
        },
        "@babel/plugin-transform-shorthand-properties" => EdgeOut {
          "name": "@babel/plugin-transform-shorthand-properties",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-shorthand-properties",
          "type": "prod",
        },
        "@babel/plugin-transform-spread" => EdgeOut {
          "name": "@babel/plugin-transform-spread",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-sticky-regex" => EdgeOut {
          "name": "@babel/plugin-transform-sticky-regex",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-sticky-regex",
          "type": "prod",
        },
        "@babel/plugin-transform-template-literals" => EdgeOut {
          "name": "@babel/plugin-transform-template-literals",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-template-literals",
          "type": "prod",
        },
        "@babel/plugin-transform-typeof-symbol" => EdgeOut {
          "name": "@babel/plugin-transform-typeof-symbol",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-typeof-symbol",
          "type": "prod",
        },
        "@babel/plugin-transform-unicode-escapes" => EdgeOut {
          "name": "@babel/plugin-transform-unicode-escapes",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-unicode-escapes",
          "type": "prod",
        },
        "@babel/plugin-transform-unicode-regex" => EdgeOut {
          "name": "@babel/plugin-transform-unicode-regex",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-unicode-regex",
          "type": "prod",
        },
        "@babel/preset-modules" => EdgeOut {
          "name": "@babel/preset-modules",
          "spec": "^0.1.3",
          "to": "node_modules/@babel/preset-modules",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "core-js-compat" => EdgeOut {
          "name": "core-js-compat",
          "spec": "^3.6.2",
          "to": "node_modules/core-js-compat",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.5.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/preset-env",
      "name": "@babel/preset-env",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/preset-env",
      "resolved": "https://registry.npmjs.org/@babel/preset-env/-/preset-env-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/preset-modules" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/preset-modules",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-proposal-unicode-property-regex" => EdgeOut {
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.4.4",
          "to": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "type": "prod",
        },
        "@babel/plugin-transform-dotall-regex" => EdgeOut {
          "name": "@babel/plugin-transform-dotall-regex",
          "spec": "^7.4.4",
          "to": "node_modules/@babel/plugin-transform-dotall-regex",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/preset-modules",
      "name": "@babel/preset-modules",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/preset-modules",
      "resolved": "https://registry.npmjs.org/@babel/preset-modules/-/preset-modules-0.1.4.tgz",
      "version": "0.1.4",
    },
    "@babel/runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@babel/runtime",
          "spec": "^7.11.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regenerator-transform",
          "name": "@babel/runtime",
          "spec": "^7.8.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerator-runtime" => EdgeOut {
          "name": "regenerator-runtime",
          "spec": "^0.13.4",
          "to": "node_modules/regenerator-runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/runtime",
      "name": "@babel/runtime",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/runtime",
      "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-wrap-function",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-wrap-function",
          "name": "@babel/traverse",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-annotate-as-pure",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-binary-assignment-operator-visitor",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-define-map",
          "name": "@babel/types",
          "spec": "^7.10.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-explode-assignable-expression",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-hoist-variables",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-member-expression-to-functions",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-imports",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-optimise-call-expression",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-remap-async-to-generator",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-simple-access",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-skip-transparent-expression-wrappers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-wrap-function",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "@babel/types",
          "spec": "^7.4.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__core",
          "name": "@babel/types",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__generator",
          "name": "@babel/types",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__template",
          "name": "@babel/types",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@types/babel__traverse",
          "name": "@babel/types",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@bcoe/v8-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "@bcoe/v8-coverage",
          "spec": "^0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@bcoe/v8-coverage",
      "name": "@bcoe/v8-coverage",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@bcoe/v8-coverage",
      "resolved": "https://registry.npmjs.org/@bcoe/v8-coverage/-/v8-coverage-0.2.3.tgz",
      "version": "0.2.3",
    },
    "@eslint/eslintrc" => ArboristNode {
      "children": Map {
        "globals" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@eslint/eslintrc",
              "name": "globals",
              "spec": "^12.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/@eslint/eslintrc/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/@eslint/eslintrc/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@eslint/eslintrc/node_modules/globals",
          "resolved": "https://registry.npmjs.org/globals/-/globals-12.4.0.tgz",
          "version": "12.4.0",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@eslint/eslintrc/node_modules/globals",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/@eslint/eslintrc/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@eslint/eslintrc/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
          "version": "0.8.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "@eslint/eslintrc",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.4",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^7.3.0",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^12.1.0",
          "to": "node_modules/@eslint/eslintrc/node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^4.0.6",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "import-fresh" => EdgeOut {
          "name": "import-fresh",
          "spec": "^3.2.1",
          "to": "node_modules/import-fresh",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "^3.1.1",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
      },
      "location": "node_modules/@eslint/eslintrc",
      "name": "@eslint/eslintrc",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@eslint/eslintrc",
      "resolved": "https://registry.npmjs.org/@eslint/eslintrc/-/eslintrc-0.1.3.tgz",
      "version": "0.1.3",
    },
    "@istanbuljs/schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@istanbuljs/schema",
      "name": "@istanbuljs/schema",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@istanbuljs/schema",
      "resolved": "https://registry.npmjs.org/@istanbuljs/schema/-/schema-0.1.2.tgz",
      "version": "0.1.2",
    },
    "@mdn/browser-compat-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "@mdn/browser-compat-data",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
      },
      "location": "node_modules/@mdn/browser-compat-data",
      "name": "@mdn/browser-compat-data",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@mdn/browser-compat-data",
      "resolved": "https://registry.npmjs.org/@mdn/browser-compat-data/-/browser-compat-data-2.0.3.tgz",
      "version": "2.0.3",
    },
    "@nodelib/fs.scandir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.walk",
          "name": "@nodelib/fs.scandir",
          "spec": "2.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.stat" => EdgeOut {
          "name": "@nodelib/fs.stat",
          "spec": "2.0.3",
          "to": "node_modules/@nodelib/fs.stat",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.9",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.scandir",
      "name": "@nodelib/fs.scandir",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@nodelib/fs.scandir",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.scandir/-/fs.scandir-2.1.3.tgz",
      "version": "2.1.3",
    },
    "@nodelib/fs.stat" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.scandir",
          "name": "@nodelib/fs.stat",
          "spec": "2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "@nodelib/fs.stat",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.stat",
      "name": "@nodelib/fs.stat",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@nodelib/fs.stat",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.stat/-/fs.stat-2.0.3.tgz",
      "version": "2.0.3",
    },
    "@nodelib/fs.walk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "@nodelib/fs.walk",
          "spec": "^1.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.scandir" => EdgeOut {
          "name": "@nodelib/fs.scandir",
          "spec": "2.1.3",
          "to": "node_modules/@nodelib/fs.scandir",
          "type": "prod",
        },
        "fastq" => EdgeOut {
          "name": "fastq",
          "spec": "^1.6.0",
          "to": "node_modules/fastq",
          "type": "prod",
        },
      },
      "location": "node_modules/@nodelib/fs.walk",
      "name": "@nodelib/fs.walk",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@nodelib/fs.walk",
      "resolved": "https://registry.npmjs.org/@nodelib/fs.walk/-/fs.walk-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@rollup/pluginutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@rollup/pluginutils",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/estree" => EdgeOut {
          "name": "@types/estree",
          "spec": "0.0.45",
          "to": "node_modules/@types/estree",
          "type": "prod",
        },
        "estree-walker" => EdgeOut {
          "name": "estree-walker",
          "spec": "^2.0.1",
          "to": "node_modules/estree-walker",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.2",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
        "rollup" => EdgeOut {
          "name": "rollup",
          "spec": "^1.20.0||^2.0.0",
          "to": "node_modules/rollup",
          "type": "peer",
        },
      },
      "location": "node_modules/@rollup/pluginutils",
      "name": "@rollup/pluginutils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@rollup/pluginutils",
      "resolved": "https://registry.npmjs.org/@rollup/pluginutils/-/pluginutils-4.0.0.tgz",
      "version": "4.0.0",
    },
    "@sindresorhus/is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "@sindresorhus/is",
          "spec": "^0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@sindresorhus/is",
      "name": "@sindresorhus/is",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@sindresorhus/is",
      "resolved": "https://registry.npmjs.org/@sindresorhus/is/-/is-0.14.0.tgz",
      "version": "0.14.0",
    },
    "@szmarczak/http-timer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "@szmarczak/http-timer",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "defer-to-connect" => EdgeOut {
          "name": "defer-to-connect",
          "spec": "^1.0.1",
          "to": "node_modules/defer-to-connect",
          "type": "prod",
        },
      },
      "location": "node_modules/@szmarczak/http-timer",
      "name": "@szmarczak/http-timer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@szmarczak/http-timer",
      "resolved": "https://registry.npmjs.org/@szmarczak/http-timer/-/http-timer-1.1.2.tgz",
      "version": "1.1.2",
    },
    "@types/babel__core" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@types/babel__core",
          "spec": "^7.1.10",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.1.0",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "@types/babel__generator" => EdgeOut {
          "name": "@types/babel__generator",
          "spec": "*",
          "to": "node_modules/@types/babel__generator",
          "type": "prod",
        },
        "@types/babel__template" => EdgeOut {
          "name": "@types/babel__template",
          "spec": "*",
          "to": "node_modules/@types/babel__template",
          "type": "prod",
        },
        "@types/babel__traverse" => EdgeOut {
          "name": "@types/babel__traverse",
          "spec": "*",
          "to": "node_modules/@types/babel__traverse",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/babel__core",
      "name": "@types/babel__core",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/babel__core",
      "resolved": "https://registry.npmjs.org/@types/babel__core/-/babel__core-7.1.10.tgz",
      "version": "7.1.10",
    },
    "@types/babel__generator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/babel__core",
          "name": "@types/babel__generator",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/babel__generator",
      "name": "@types/babel__generator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/babel__generator",
      "resolved": "https://registry.npmjs.org/@types/babel__generator/-/babel__generator-7.6.2.tgz",
      "version": "7.6.2",
    },
    "@types/babel__template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/babel__core",
          "name": "@types/babel__template",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.1.0",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/babel__template",
      "name": "@types/babel__template",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/babel__template",
      "resolved": "https://registry.npmjs.org/@types/babel__template/-/babel__template-7.0.3.tgz",
      "version": "7.0.3",
    },
    "@types/babel__traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/babel__core",
          "name": "@types/babel__traverse",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.3.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/babel__traverse",
      "name": "@types/babel__traverse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/babel__traverse",
      "resolved": "https://registry.npmjs.org/@types/babel__traverse/-/babel__traverse-7.0.15.tgz",
      "version": "7.0.15",
    },
    "@types/chai" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@types/chai",
          "spec": "^4.2.11",
          "type": "dev",
        },
      },
      "location": "node_modules/@types/chai",
      "name": "@types/chai",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/chai",
      "resolved": "https://registry.npmjs.org/@types/chai/-/chai-4.2.13.tgz",
      "version": "4.2.13",
    },
    "@types/estree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@rollup/pluginutils",
          "name": "@types/estree",
          "spec": "0.0.45",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/estree",
      "name": "@types/estree",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/estree",
      "resolved": "https://registry.npmjs.org/@types/estree/-/estree-0.0.45.tgz",
      "version": "0.0.45",
    },
    "@types/is-windows" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/furi",
          "name": "@types/is-windows",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/is-windows",
      "name": "@types/is-windows",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/is-windows",
      "resolved": "https://registry.npmjs.org/@types/is-windows/-/is-windows-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@types/istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/v8-to-istanbul",
          "name": "@types/istanbul-lib-coverage",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/istanbul-lib-coverage",
      "name": "@types/istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/@types/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.3.tgz",
      "version": "2.0.3",
    },
    "@types/json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/json-schema",
      "name": "@types/json-schema",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/json-schema",
      "resolved": "https://registry.npmjs.org/@types/json-schema/-/json-schema-7.0.6.tgz",
      "version": "7.0.6",
    },
    "@types/minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "@types/minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/minimist",
      "name": "@types/minimist",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/minimist",
      "resolved": "https://registry.npmjs.org/@types/minimist/-/minimist-1.2.0.tgz",
      "version": "1.2.0",
    },
    "@types/mocha" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@types/mocha",
          "spec": "^8.0.0",
          "type": "dev",
        },
      },
      "location": "node_modules/@types/mocha",
      "name": "@types/mocha",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/mocha",
      "resolved": "https://registry.npmjs.org/@types/mocha/-/mocha-8.0.3.tgz",
      "version": "8.0.3",
    },
    "@types/node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@types/node",
          "spec": "^14.11.2",
          "type": "dev",
        },
      },
      "location": "node_modules/@types/node",
      "name": "@types/node",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/node",
      "resolved": "https://registry.npmjs.org/@types/node/-/node-14.11.8.tgz",
      "version": "14.11.8",
    },
    "@types/normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "@types/normalize-package-data",
          "spec": "^2.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/normalize-package-data",
      "name": "@types/normalize-package-data",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/normalize-package-data",
      "resolved": "https://registry.npmjs.org/@types/normalize-package-data/-/normalize-package-data-2.4.0.tgz",
      "version": "2.4.0",
    },
    "@types/object-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "@types/object-path",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/object-path",
      "name": "@types/object-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/object-path",
      "resolved": "https://registry.npmjs.org/@types/object-path/-/object-path-0.11.0.tgz",
      "version": "0.11.0",
    },
    "@types/semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "@types/semver",
          "spec": "^7.3.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/semver",
      "name": "@types/semver",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/semver",
      "resolved": "https://registry.npmjs.org/@types/semver/-/semver-7.3.4.tgz",
      "version": "7.3.4",
    },
    "@types/ua-parser-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "@types/ua-parser-js",
          "spec": "^0.7.33",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/ua-parser-js",
      "name": "@types/ua-parser-js",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@types/ua-parser-js",
      "resolved": "https://registry.npmjs.org/@types/ua-parser-js/-/ua-parser-js-0.7.33.tgz",
      "version": "0.7.33",
    },
    "@typescript-eslint/eslint-plugin" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@typescript-eslint/eslint-plugin",
              "name": "semver",
              "spec": "^7.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@typescript-eslint/eslint-plugin/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/eslint-plugin/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
          "version": "7.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "@typescript-eslint/eslint-plugin",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/experimental-utils" => EdgeOut {
          "name": "@typescript-eslint/experimental-utils",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/experimental-utils",
          "type": "prod",
        },
        "@typescript-eslint/parser" => EdgeOut {
          "name": "@typescript-eslint/parser",
          "spec": "^4.0.0",
          "to": "node_modules/@typescript-eslint/parser",
          "type": "peer",
        },
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "functional-red-black-tree" => EdgeOut {
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "to": "node_modules/functional-red-black-tree",
          "type": "prod",
        },
        "regexpp" => EdgeOut {
          "name": "regexpp",
          "spec": "^3.0.0",
          "to": "node_modules/regexpp",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/@typescript-eslint/eslint-plugin/node_modules/semver",
          "type": "prod",
        },
        "tsutils" => EdgeOut {
          "name": "tsutils",
          "spec": "^3.17.1",
          "to": "node_modules/tsutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/eslint-plugin",
      "name": "@typescript-eslint/eslint-plugin",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/eslint-plugin",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/eslint-plugin/-/eslint-plugin-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/experimental-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/experimental-utils",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/json-schema" => EdgeOut {
          "name": "@types/json-schema",
          "spec": "^7.0.3",
          "to": "node_modules/@types/json-schema",
          "type": "prod",
        },
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/typescript-estree" => EdgeOut {
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/typescript-estree",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "*",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^5.0.0",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/experimental-utils",
      "name": "@typescript-eslint/experimental-utils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/experimental-utils",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/experimental-utils/-/experimental-utils-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/parser",
          "spec": "^4.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "@typescript-eslint/parser",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/scope-manager" => EdgeOut {
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/scope-manager",
          "type": "prod",
        },
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/typescript-estree" => EdgeOut {
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/typescript-estree",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/@typescript-eslint/parser",
      "name": "@typescript-eslint/parser",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/parser",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/parser/-/parser-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/scope-manager" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/scope-manager",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/visitor-keys" => EdgeOut {
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/scope-manager",
      "name": "@typescript-eslint/scope-manager",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/scope-manager",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/scope-manager/-/scope-manager-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/scope-manager",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/visitor-keys",
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/types",
      "name": "@typescript-eslint/types",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/types",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/types/-/types-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/typescript-estree" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@typescript-eslint/typescript-estree",
              "name": "semver",
              "spec": "^7.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@typescript-eslint/typescript-estree/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/typescript-estree/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
          "version": "7.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "@typescript-eslint/typescript-estree",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "@typescript-eslint/visitor-keys" => EdgeOut {
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/visitor-keys",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^11.0.1",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.15",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/@typescript-eslint/typescript-estree/node_modules/semver",
          "type": "prod",
        },
        "tsutils" => EdgeOut {
          "name": "tsutils",
          "spec": "^3.17.1",
          "to": "node_modules/tsutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/typescript-estree",
      "name": "@typescript-eslint/typescript-estree",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/typescript-estree",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/typescript-estree/-/typescript-estree-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@typescript-eslint/visitor-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/scope-manager",
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "@typescript-eslint/visitor-keys",
          "spec": "4.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/types" => EdgeOut {
          "name": "@typescript-eslint/types",
          "spec": "4.4.1",
          "to": "node_modules/@typescript-eslint/types",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/@typescript-eslint/visitor-keys",
      "name": "@typescript-eslint/visitor-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@typescript-eslint/visitor-keys",
      "resolved": "https://registry.npmjs.org/@typescript-eslint/visitor-keys/-/visitor-keys-4.4.1.tgz",
      "version": "4.4.1",
    },
    "@wessberg/browserslist-generator" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@wessberg/browserslist-generator",
              "name": "semver",
              "spec": "^7.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@wessberg/browserslist-generator/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/@wessberg/browserslist-generator/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
          "version": "7.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@wessberg/browserslist-generator",
          "spec": "^1.0.38",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@mdn/browser-compat-data" => EdgeOut {
          "name": "@mdn/browser-compat-data",
          "spec": "^2.0.2",
          "to": "node_modules/@mdn/browser-compat-data",
          "type": "prod",
        },
        "@types/object-path" => EdgeOut {
          "name": "@types/object-path",
          "spec": "^0.11.0",
          "to": "node_modules/@types/object-path",
          "type": "prod",
        },
        "@types/semver" => EdgeOut {
          "name": "@types/semver",
          "spec": "^7.3.4",
          "to": "node_modules/@types/semver",
          "type": "prod",
        },
        "@types/ua-parser-js" => EdgeOut {
          "name": "@types/ua-parser-js",
          "spec": "^0.7.33",
          "to": "node_modules/@types/ua-parser-js",
          "type": "prod",
        },
        "browserslist" => EdgeOut {
          "name": "browserslist",
          "spec": "4.14.5",
          "to": "node_modules/browserslist",
          "type": "prod",
        },
        "caniuse-lite" => EdgeOut {
          "name": "caniuse-lite",
          "spec": "^1.0.30001148",
          "to": "node_modules/caniuse-lite",
          "type": "prod",
        },
        "object-path" => EdgeOut {
          "name": "object-path",
          "spec": "^0.11.5",
          "to": "node_modules/object-path",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/@wessberg/browserslist-generator/node_modules/semver",
          "type": "prod",
        },
        "ua-parser-js" => EdgeOut {
          "name": "ua-parser-js",
          "spec": "^0.7.22",
          "to": "node_modules/ua-parser-js",
          "type": "prod",
        },
      },
      "location": "node_modules/@wessberg/browserslist-generator",
      "name": "@wessberg/browserslist-generator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@wessberg/browserslist-generator",
      "resolved": "https://registry.npmjs.org/@wessberg/browserslist-generator/-/browserslist-generator-1.0.39.tgz",
      "version": "1.0.39",
    },
    "@wessberg/rollup-plugin-ts" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@wessberg/rollup-plugin-ts",
          "spec": "^1.3.2",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.11.6",
          "to": "node_modules/@babel/core",
          "type": "prod",
        },
        "@babel/plugin-proposal-async-generator-functions" => EdgeOut {
          "name": "@babel/plugin-proposal-async-generator-functions",
          "spec": "^7.10.5",
          "to": "node_modules/@babel/plugin-proposal-async-generator-functions",
          "type": "prod",
        },
        "@babel/plugin-proposal-json-strings" => EdgeOut {
          "name": "@babel/plugin-proposal-json-strings",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-proposal-json-strings",
          "type": "prod",
        },
        "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-proposal-optional-catch-binding" => EdgeOut {
          "name": "@babel/plugin-proposal-optional-catch-binding",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-proposal-optional-catch-binding",
          "type": "prod",
        },
        "@babel/plugin-proposal-unicode-property-regex" => EdgeOut {
          "name": "@babel/plugin-proposal-unicode-property-regex",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/plugin-proposal-unicode-property-regex",
          "type": "prod",
        },
        "@babel/plugin-syntax-dynamic-import" => EdgeOut {
          "name": "@babel/plugin-syntax-dynamic-import",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/plugin-syntax-dynamic-import",
          "type": "prod",
        },
        "@babel/plugin-transform-runtime" => EdgeOut {
          "name": "@babel/plugin-transform-runtime",
          "spec": "^7.11.5",
          "to": "node_modules/@babel/plugin-transform-runtime",
          "type": "prod",
        },
        "@babel/preset-env" => EdgeOut {
          "name": "@babel/preset-env",
          "spec": "^7.11.5",
          "to": "node_modules/@babel/preset-env",
          "type": "prod",
        },
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.11.2",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
        "@rollup/pluginutils" => EdgeOut {
          "name": "@rollup/pluginutils",
          "spec": "^4.0.0",
          "to": "node_modules/@rollup/pluginutils",
          "type": "prod",
        },
        "@types/babel__core" => EdgeOut {
          "name": "@types/babel__core",
          "spec": "^7.1.10",
          "to": "node_modules/@types/babel__core",
          "type": "prod",
        },
        "@wessberg/browserslist-generator" => EdgeOut {
          "name": "@wessberg/browserslist-generator",
          "spec": "^1.0.38",
          "to": "node_modules/@wessberg/browserslist-generator",
          "type": "prod",
        },
        "@wessberg/stringutil" => EdgeOut {
          "name": "@wessberg/stringutil",
          "spec": "^1.0.19",
          "to": "node_modules/@wessberg/stringutil",
          "type": "prod",
        },
        "@wessberg/ts-clone-node" => EdgeOut {
          "name": "@wessberg/ts-clone-node",
          "spec": "^0.3.15",
          "to": "node_modules/@wessberg/ts-clone-node",
          "type": "prod",
        },
        "browserslist" => EdgeOut {
          "name": "browserslist",
          "spec": "^4.14.5",
          "to": "node_modules/browserslist",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "magic-string" => EdgeOut {
          "name": "magic-string",
          "spec": "^0.25.7",
          "to": "node_modules/magic-string",
          "type": "prod",
        },
        "rollup" => EdgeOut {
          "name": "rollup",
          "spec": ">=1.x || >=2.x",
          "to": "node_modules/rollup",
          "type": "peer",
        },
        "slash" => EdgeOut {
          "name": "slash",
          "spec": "^3.0.0",
          "to": "node_modules/slash",
          "type": "prod",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^2.0.1",
          "to": "node_modules/tslib",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=3.2.x || >= 4.x",
          "to": "node_modules/typescript",
          "type": "peer",
        },
      },
      "location": "node_modules/@wessberg/rollup-plugin-ts",
      "name": "@wessberg/rollup-plugin-ts",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@wessberg/rollup-plugin-ts",
      "resolved": "https://registry.npmjs.org/@wessberg/rollup-plugin-ts/-/rollup-plugin-ts-1.3.5.tgz",
      "version": "1.3.5",
    },
    "@wessberg/stringutil" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@wessberg/stringutil",
          "spec": "^1.0.19",
          "type": "prod",
        },
      },
      "location": "node_modules/@wessberg/stringutil",
      "name": "@wessberg/stringutil",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@wessberg/stringutil",
      "resolved": "https://registry.npmjs.org/@wessberg/stringutil/-/stringutil-1.0.19.tgz",
      "version": "1.0.19",
    },
    "@wessberg/ts-clone-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "@wessberg/ts-clone-node",
          "spec": "^0.3.15",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.x || ^4.x",
          "to": "node_modules/typescript",
          "type": "peer",
        },
      },
      "location": "node_modules/@wessberg/ts-clone-node",
      "name": "@wessberg/ts-clone-node",
      "path": "{CWD}/test/fixtures/yargs/node_modules/@wessberg/ts-clone-node",
      "resolved": "https://registry.npmjs.org/@wessberg/ts-clone-node/-/ts-clone-node-0.3.15.tgz",
      "version": "0.3.15",
    },
    "acorn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/acorn-jsx",
          "name": "acorn",
          "spec": "^6.0.0 || ^7.0.0 || ^8.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn",
          "spec": "^7.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/espree",
          "name": "acorn",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/acorn",
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-7.4.1.tgz",
      "version": "7.4.1",
    },
    "acorn-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^5.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^6.0.0 || ^7.0.0 || ^8.0.0",
          "to": "node_modules/acorn",
          "type": "peer",
        },
      },
      "location": "node_modules/acorn-jsx",
      "name": "acorn-jsx",
      "path": "{CWD}/test/fixtures/yargs/node_modules/acorn-jsx",
      "resolved": "https://registry.npmjs.org/acorn-jsx/-/acorn-jsx-5.3.1.tgz",
      "version": "5.3.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "ajv",
          "spec": "^6.12.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ajv",
          "spec": "^6.10.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.12.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "ajv",
          "spec": "^6.10.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv",
          "spec": "^6.10.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ansi-align" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-align/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-align/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-align/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-align/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-align/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-align/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-align",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/ansi-align/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/ansi-align/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/ansi-align/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-align/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ansi-align/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/ansi-align/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/ansi-align/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "ansi-align",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/ansi-align/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-align",
      "name": "ansi-align",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-align",
      "resolved": "https://registry.npmjs.org/ansi-align/-/ansi-align-3.0.0.tgz",
      "version": "3.0.0",
    },
    "ansi-colors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enquirer",
          "name": "ansi-colors",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "ansi-colors",
          "spec": "4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-colors",
      "name": "ansi-colors",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-colors",
      "resolved": "https://registry.npmjs.org/ansi-colors/-/ansi-colors-4.1.1.tgz",
      "version": "4.1.1",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.11.0",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.1.tgz",
      "version": "4.3.1",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
      "version": "5.0.0",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen/node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^2.0.1",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
      "version": "4.3.0",
    },
    "anymatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/fixtures/yargs/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "array-includes" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/array-includes",
              "name": "es-abstract",
              "spec": "^1.17.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/array-includes/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/array-includes/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsx-ast-utils",
          "name": "array-includes",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "array-includes",
          "spec": "^3.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "array-includes",
          "spec": "^3.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0",
          "to": "node_modules/array-includes/node_modules/es-abstract",
          "type": "prod",
        },
        "is-string" => EdgeOut {
          "name": "is-string",
          "spec": "^1.0.5",
          "to": "node_modules/is-string",
          "type": "prod",
        },
      },
      "location": "node_modules/array-includes",
      "name": "array-includes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/array-includes",
      "resolved": "https://registry.npmjs.org/array-includes/-/array-includes-3.1.1.tgz",
      "version": "3.1.1",
    },
    "array-union" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/fixtures/yargs/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-2.1.0.tgz",
      "version": "2.1.0",
    },
    "array.prototype.map" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/array.prototype.map",
              "name": "es-abstract",
              "spec": "^1.17.0-next.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/array.prototype.map/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/promise.allsettled",
          "name": "array.prototype.map",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0-next.1",
          "to": "node_modules/array.prototype.map/node_modules/es-abstract",
          "type": "prod",
        },
        "es-array-method-boxes-properly" => EdgeOut {
          "name": "es-array-method-boxes-properly",
          "spec": "^1.0.0",
          "to": "node_modules/es-array-method-boxes-properly",
          "type": "prod",
        },
        "is-string" => EdgeOut {
          "name": "is-string",
          "spec": "^1.0.4",
          "to": "node_modules/is-string",
          "type": "prod",
        },
      },
      "location": "node_modules/array.prototype.map",
      "name": "array.prototype.map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/array.prototype.map",
      "resolved": "https://registry.npmjs.org/array.prototype.map/-/array.prototype.map-1.0.2.tgz",
      "version": "1.0.2",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimist-options",
          "name": "arrify",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/yargs/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/yargs/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "assertion-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "assertion-error",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assertion-error",
      "name": "assertion-error",
      "path": "{CWD}/test/fixtures/yargs/node_modules/assertion-error",
      "resolved": "https://registry.npmjs.org/assertion-error/-/assertion-error-1.1.0.tgz",
      "version": "1.1.0",
    },
    "astral-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/yargs/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/yargs/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/yargs/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.10.1.tgz",
      "version": "1.10.1",
    },
    "babel-plugin-dynamic-import-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-amd",
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-commonjs",
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-modules-systemjs",
          "name": "babel-plugin-dynamic-import-node",
          "spec": "^2.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object.assign" => EdgeOut {
          "name": "object.assign",
          "spec": "^4.1.0",
          "to": "node_modules/object.assign",
          "type": "prod",
        },
      },
      "location": "node_modules/babel-plugin-dynamic-import-node",
      "name": "babel-plugin-dynamic-import-node",
      "path": "{CWD}/test/fixtures/yargs/node_modules/babel-plugin-dynamic-import-node",
      "resolved": "https://registry.npmjs.org/babel-plugin-dynamic-import-node/-/babel-plugin-dynamic-import-node-2.3.3.tgz",
      "version": "2.3.3",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/yargs/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/fixtures/yargs/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.1.0.tgz",
      "version": "2.1.0",
    },
    "boxen" => ArboristNode {
      "children": Map {
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/boxen",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/boxen/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/yargs/node_modules/boxen/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
          "version": "3.0.0",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/boxen",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/boxen/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/boxen/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
          "version": "0.8.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "boxen",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-align" => EdgeOut {
          "name": "ansi-align",
          "spec": "^3.0.0",
          "to": "node_modules/ansi-align",
          "type": "prod",
        },
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.3.1",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^3.0.0",
          "to": "node_modules/boxen/node_modules/chalk",
          "type": "prod",
        },
        "cli-boxes" => EdgeOut {
          "name": "cli-boxes",
          "spec": "^2.2.0",
          "to": "node_modules/cli-boxes",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "term-size" => EdgeOut {
          "name": "term-size",
          "spec": "^2.1.0",
          "to": "node_modules/term-size",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.1",
          "to": "node_modules/boxen/node_modules/type-fest",
          "type": "prod",
        },
        "widest-line" => EdgeOut {
          "name": "widest-line",
          "spec": "^3.1.0",
          "to": "node_modules/widest-line",
          "type": "prod",
        },
      },
      "location": "node_modules/boxen",
      "name": "boxen",
      "path": "{CWD}/test/fixtures/yargs/node_modules/boxen",
      "resolved": "https://registry.npmjs.org/boxen/-/boxen-4.2.0.tgz",
      "version": "4.2.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/yargs/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "braces",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/yargs/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-stdout" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "browser-stdout",
          "spec": "1.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-stdout",
      "name": "browser-stdout",
      "path": "{CWD}/test/fixtures/yargs/node_modules/browser-stdout",
      "resolved": "https://registry.npmjs.org/browser-stdout/-/browser-stdout-1.3.1.tgz",
      "version": "1.3.1",
    },
    "browserslist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-compilation-targets",
          "name": "browserslist",
          "spec": "^4.12.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "browserslist",
          "spec": "4.14.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "browserslist",
          "spec": "^4.14.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/core-js-compat",
          "name": "browserslist",
          "spec": "^4.8.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caniuse-lite" => EdgeOut {
          "name": "caniuse-lite",
          "spec": "^1.0.30001135",
          "to": "node_modules/caniuse-lite",
          "type": "prod",
        },
        "electron-to-chromium" => EdgeOut {
          "name": "electron-to-chromium",
          "spec": "^1.3.571",
          "to": "node_modules/electron-to-chromium",
          "type": "prod",
        },
        "escalade" => EdgeOut {
          "name": "escalade",
          "spec": "^3.1.0",
          "to": "node_modules/escalade",
          "type": "prod",
        },
        "node-releases" => EdgeOut {
          "name": "node-releases",
          "spec": "^1.1.61",
          "to": "node_modules/node-releases",
          "type": "prod",
        },
      },
      "location": "node_modules/browserslist",
      "name": "browserslist",
      "path": "{CWD}/test/fixtures/yargs/node_modules/browserslist",
      "resolved": "https://registry.npmjs.org/browserslist/-/browserslist-4.14.5.tgz",
      "version": "4.14.5",
    },
    "c8" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c8",
          "spec": "^7.0.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@bcoe/v8-coverage" => EdgeOut {
          "name": "@bcoe/v8-coverage",
          "spec": "^0.2.3",
          "to": "node_modules/@bcoe/v8-coverage",
          "type": "prod",
        },
        "@istanbuljs/schema" => EdgeOut {
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "to": "node_modules/@istanbuljs/schema",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^5.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^2.0.0",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "furi" => EdgeOut {
          "name": "furi",
          "spec": "^2.0.0",
          "to": "node_modules/furi",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^3.0.2",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^3.0.0",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^6.0.0",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "v8-to-istanbul" => EdgeOut {
          "name": "v8-to-istanbul",
          "spec": "^6.0.1",
          "to": "node_modules/v8-to-istanbul",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^16.0.0",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^20.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/c8",
      "name": "c8",
      "path": "{CWD}/test/fixtures/yargs/node_modules/c8",
      "resolved": "https://registry.npmjs.org/c8/-/c8-7.3.4.tgz",
      "version": "7.3.4",
    },
    "cacheable-request" => ArboristNode {
      "children": Map {
        "lowercase-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cacheable-request",
              "name": "lowercase-keys",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cacheable-request/node_modules/lowercase-keys",
          "name": "lowercase-keys",
          "path": "{CWD}/test/fixtures/yargs/node_modules/cacheable-request/node_modules/lowercase-keys",
          "resolved": "https://registry.npmjs.org/lowercase-keys/-/lowercase-keys-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "cacheable-request",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "clone-response" => EdgeOut {
          "name": "clone-response",
          "spec": "^1.0.2",
          "to": "node_modules/clone-response",
          "type": "prod",
        },
        "get-stream" => EdgeOut {
          "name": "get-stream",
          "spec": "^5.1.0",
          "to": "node_modules/get-stream",
          "type": "prod",
        },
        "http-cache-semantics" => EdgeOut {
          "name": "http-cache-semantics",
          "spec": "^4.0.0",
          "to": "node_modules/http-cache-semantics",
          "type": "prod",
        },
        "keyv" => EdgeOut {
          "name": "keyv",
          "spec": "^3.0.0",
          "to": "node_modules/keyv",
          "type": "prod",
        },
        "lowercase-keys" => EdgeOut {
          "name": "lowercase-keys",
          "spec": "^2.0.0",
          "to": "node_modules/cacheable-request/node_modules/lowercase-keys",
          "type": "prod",
        },
        "normalize-url" => EdgeOut {
          "name": "normalize-url",
          "spec": "^4.1.0",
          "to": "node_modules/normalize-url",
          "type": "prod",
        },
        "responselike" => EdgeOut {
          "name": "responselike",
          "spec": "^1.0.2",
          "to": "node_modules/responselike",
          "type": "prod",
        },
      },
      "location": "node_modules/cacheable-request",
      "name": "cacheable-request",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cacheable-request",
      "resolved": "https://registry.npmjs.org/cacheable-request/-/cacheable-request-6.1.0.tgz",
      "version": "6.1.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parent-module",
          "name": "callsites",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/yargs/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-3.1.0.tgz",
      "version": "3.1.0",
    },
    "camelcase" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "camelcase",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/camelcase-keys",
          "name": "camelcase",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/meow/node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser",
          "name": "camelcase",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/yargs/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "camelcase-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "camelcase-keys",
          "spec": "^6.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.3.1",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "map-obj" => EdgeOut {
          "name": "map-obj",
          "spec": "^4.0.0",
          "to": "node_modules/map-obj",
          "type": "prod",
        },
        "quick-lru" => EdgeOut {
          "name": "quick-lru",
          "spec": "^4.0.1",
          "to": "node_modules/quick-lru",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase-keys",
      "name": "camelcase-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/camelcase-keys",
      "resolved": "https://registry.npmjs.org/camelcase-keys/-/camelcase-keys-6.2.2.tgz",
      "version": "6.2.2",
    },
    "caniuse-lite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "caniuse-lite",
          "spec": "^1.0.30001148",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "caniuse-lite",
          "spec": "^1.0.30001135",
          "type": "prod",
        },
      },
      "location": "node_modules/caniuse-lite",
      "name": "caniuse-lite",
      "path": "{CWD}/test/fixtures/yargs/node_modules/caniuse-lite",
      "resolved": "https://registry.npmjs.org/caniuse-lite/-/caniuse-lite-1.0.30001148.tgz",
      "version": "1.0.30001148",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/yargs/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chai" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "chai",
          "spec": "^4.2.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "assertion-error" => EdgeOut {
          "name": "assertion-error",
          "spec": "^1.1.0",
          "to": "node_modules/assertion-error",
          "type": "prod",
        },
        "check-error" => EdgeOut {
          "name": "check-error",
          "spec": "^1.0.2",
          "to": "node_modules/check-error",
          "type": "prod",
        },
        "deep-eql" => EdgeOut {
          "name": "deep-eql",
          "spec": "^3.0.1",
          "to": "node_modules/deep-eql",
          "type": "prod",
        },
        "get-func-name" => EdgeOut {
          "name": "get-func-name",
          "spec": "^2.0.0",
          "to": "node_modules/get-func-name",
          "type": "prod",
        },
        "pathval" => EdgeOut {
          "name": "pathval",
          "spec": "^1.1.0",
          "to": "node_modules/pathval",
          "type": "prod",
        },
        "type-detect" => EdgeOut {
          "name": "type-detect",
          "spec": "^4.0.5",
          "to": "node_modules/type-detect",
          "type": "prod",
        },
      },
      "location": "node_modules/chai",
      "name": "chai",
      "path": "{CWD}/test/fixtures/yargs/node_modules/chai",
      "resolved": "https://registry.npmjs.org/chai/-/chai-4.2.0.tgz",
      "version": "4.2.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "chalk",
          "spec": "^4.0.0",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "chalk",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "chalk",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "chalk",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "chalk",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-symbols",
          "name": "chalk",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "chalk",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/yargs/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-4.1.0.tgz",
      "version": "4.1.0",
    },
    "chardet" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/external-editor",
          "name": "chardet",
          "spec": "^0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/chardet",
      "name": "chardet",
      "path": "{CWD}/test/fixtures/yargs/node_modules/chardet",
      "resolved": "https://registry.npmjs.org/chardet/-/chardet-0.7.0.tgz",
      "version": "0.7.0",
    },
    "check-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "check-error",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/check-error",
      "name": "check-error",
      "path": "{CWD}/test/fixtures/yargs/node_modules/check-error",
      "resolved": "https://registry.npmjs.org/check-error/-/check-error-1.0.2.tgz",
      "version": "1.0.2",
    },
    "chokidar" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "chokidar",
          "spec": "3.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.4.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/fixtures/yargs/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.4.2.tgz",
      "version": "3.4.2",
    },
    "ci-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-boxes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "cli-boxes",
          "spec": "^2.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-boxes",
      "name": "cli-boxes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cli-boxes",
      "resolved": "https://registry.npmjs.org/cli-boxes/-/cli-boxes-2.2.1.tgz",
      "version": "2.2.1",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-width",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-width",
      "name": "cli-width",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cli-width",
      "resolved": "https://registry.npmjs.org/cli-width/-/cli-width-3.0.0.tgz",
      "version": "3.0.0",
    },
    "cliui" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "cliui",
          "spec": "^7.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "cliui",
          "spec": "^7.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^7.0.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-7.0.2.tgz",
      "version": "7.0.2",
    },
    "clone-response" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "clone-response",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-response" => EdgeOut {
          "name": "mimic-response",
          "spec": "^1.0.0",
          "to": "node_modules/mimic-response",
          "type": "prod",
        },
      },
      "location": "node_modules/clone-response",
      "name": "clone-response",
      "path": "{CWD}/test/fixtures/yargs/node_modules/clone-response",
      "resolved": "https://registry.npmjs.org/clone-response/-/clone-response-1.0.2.tgz",
      "version": "1.0.2",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "~1.1.4",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/yargs/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
      "version": "2.0.1",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "~1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/yargs/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
      "version": "1.1.4",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "configstore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "configstore",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "dot-prop" => EdgeOut {
          "name": "dot-prop",
          "spec": "^5.2.0",
          "to": "node_modules/dot-prop",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "unique-string" => EdgeOut {
          "name": "unique-string",
          "spec": "^2.0.0",
          "to": "node_modules/unique-string",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.0",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "xdg-basedir" => EdgeOut {
          "name": "xdg-basedir",
          "spec": "^4.0.0",
          "to": "node_modules/xdg-basedir",
          "type": "prod",
        },
      },
      "location": "node_modules/configstore",
      "name": "configstore",
      "path": "{CWD}/test/fixtures/yargs/node_modules/configstore",
      "resolved": "https://registry.npmjs.org/configstore/-/configstore-5.0.1.tgz",
      "version": "5.0.1",
    },
    "contains-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "contains-path",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/contains-path",
      "name": "contains-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/contains-path",
      "resolved": "https://registry.npmjs.org/contains-path/-/contains-path-0.1.0.tgz",
      "version": "0.1.0",
    },
    "convert-source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/v8-to-istanbul",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-js-compat" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/core-js-compat",
              "name": "semver",
              "spec": "7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/core-js-compat/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/core-js-compat/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.0.0.tgz",
          "version": "7.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "core-js-compat",
          "spec": "^3.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "browserslist" => EdgeOut {
          "name": "browserslist",
          "spec": "^4.8.5",
          "to": "node_modules/browserslist",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "7.0.0",
          "to": "node_modules/core-js-compat/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/core-js-compat",
      "name": "core-js-compat",
      "path": "{CWD}/test/fixtures/yargs/node_modules/core-js-compat",
      "resolved": "https://registry.npmjs.org/core-js-compat/-/core-js-compat-3.6.5.tgz",
      "version": "3.6.5",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/yargs/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "coveralls",
          "spec": "^3.0.9",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.2",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/yargs/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cpr" => ArboristNode {
      "children": Map {
        "rimraf" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cpr",
              "name": "rimraf",
              "spec": "^2.5.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.1.3",
              "to": "node_modules/glob",
              "type": "prod",
            },
          },
          "location": "node_modules/cpr/node_modules/rimraf",
          "name": "rimraf",
          "path": "{CWD}/test/fixtures/yargs/node_modules/cpr/node_modules/rimraf",
          "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
          "version": "2.7.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "cpr",
          "spec": "^3.0.1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.5",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "~0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.5.4",
          "to": "node_modules/cpr/node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "node_modules/cpr",
      "name": "cpr",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cpr",
      "resolved": "https://registry.npmjs.org/cpr/-/cpr-3.0.1.tgz",
      "version": "3.0.1",
    },
    "cross-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "cross-env",
          "spec": "^7.0.2",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.1",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-env",
      "name": "cross-env",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cross-env",
      "resolved": "https://registry.npmjs.org/cross-env/-/cross-env-7.0.2.tgz",
      "version": "7.0.2",
    },
    "cross-spawn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/cross-env",
          "name": "cross-spawn",
          "spec": "^7.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "cross-spawn",
          "spec": "^7.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/execa",
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^3.1.0",
          "to": "node_modules/path-key",
          "type": "prod",
        },
        "shebang-command" => EdgeOut {
          "name": "shebang-command",
          "spec": "^2.0.0",
          "to": "node_modules/shebang-command",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.1",
          "to": "node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-7.0.3.tgz",
      "version": "7.0.3",
    },
    "crypto-random-string" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/unique-string",
          "name": "crypto-random-string",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/crypto-random-string",
      "name": "crypto-random-string",
      "path": "{CWD}/test/fixtures/yargs/node_modules/crypto-random-string",
      "resolved": "https://registry.npmjs.org/crypto-random-string/-/crypto-random-string-2.0.0.tgz",
      "version": "2.0.0",
    },
    "dashdash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/yargs/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "debug",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "debug",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/yargs/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "debug-log" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "debug-log",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/debug-log",
      "name": "debug-log",
      "path": "{CWD}/test/fixtures/yargs/node_modules/debug-log",
      "resolved": "https://registry.npmjs.org/debug-log/-/debug-log-1.0.1.tgz",
      "version": "1.0.1",
    },
    "decamelize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/decamelize-keys",
          "name": "decamelize",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/meow/node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/yargs/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "decamelize-keys" => ArboristNode {
      "children": Map {
        "map-obj" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/decamelize-keys",
              "name": "map-obj",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/decamelize-keys/node_modules/map-obj",
          "name": "map-obj",
          "path": "{CWD}/test/fixtures/yargs/node_modules/decamelize-keys/node_modules/map-obj",
          "resolved": "https://registry.npmjs.org/map-obj/-/map-obj-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "decamelize-keys",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.1.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
        "map-obj" => EdgeOut {
          "name": "map-obj",
          "spec": "^1.0.0",
          "to": "node_modules/decamelize-keys/node_modules/map-obj",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize-keys",
      "name": "decamelize-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/decamelize-keys",
      "resolved": "https://registry.npmjs.org/decamelize-keys/-/decamelize-keys-1.1.0.tgz",
      "version": "1.1.0",
    },
    "decompress-response" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "decompress-response",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-response" => EdgeOut {
          "name": "mimic-response",
          "spec": "^1.0.0",
          "to": "node_modules/mimic-response",
          "type": "prod",
        },
      },
      "location": "node_modules/decompress-response",
      "name": "decompress-response",
      "path": "{CWD}/test/fixtures/yargs/node_modules/decompress-response",
      "resolved": "https://registry.npmjs.org/decompress-response/-/decompress-response-3.3.0.tgz",
      "version": "3.3.0",
    },
    "deep-eql" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "deep-eql",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-detect" => EdgeOut {
          "name": "type-detect",
          "spec": "^4.0.0",
          "to": "node_modules/type-detect",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-eql",
      "name": "deep-eql",
      "path": "{CWD}/test/fixtures/yargs/node_modules/deep-eql",
      "resolved": "https://registry.npmjs.org/deep-eql/-/deep-eql-3.0.1.tgz",
      "version": "3.0.1",
    },
    "deep-extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rc",
          "name": "deep-extend",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-extend",
      "name": "deep-extend",
      "path": "{CWD}/test/fixtures/yargs/node_modules/deep-extend",
      "resolved": "https://registry.npmjs.org/deep-extend/-/deep-extend-0.6.0.tgz",
      "version": "0.6.0",
    },
    "deep-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "deep-is",
          "spec": "^0.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/optionator",
          "name": "deep-is",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-is",
      "name": "deep-is",
      "path": "{CWD}/test/fixtures/yargs/node_modules/deep-is",
      "resolved": "https://registry.npmjs.org/deep-is/-/deep-is-0.1.3.tgz",
      "version": "0.1.3",
    },
    "defer-to-connect" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@szmarczak/http-timer",
          "name": "defer-to-connect",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/defer-to-connect",
      "name": "defer-to-connect",
      "path": "{CWD}/test/fixtures/yargs/node_modules/defer-to-connect",
      "resolved": "https://registry.npmjs.org/defer-to-connect/-/defer-to-connect-1.1.3.tgz",
      "version": "1.1.3",
    },
    "define-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/object.assign",
          "name": "define-properties",
          "spec": "^1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.0.12",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/define-properties",
      "name": "define-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/define-properties",
      "resolved": "https://registry.npmjs.org/define-properties/-/define-properties-1.1.3.tgz",
      "version": "1.1.3",
    },
    "deglob" => ArboristNode {
      "children": Map {
        "ignore" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob",
              "name": "ignore",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/ignore",
          "name": "ignore",
          "path": "{CWD}/test/fixtures/yargs/node_modules/deglob/node_modules/ignore",
          "resolved": "https://registry.npmjs.org/ignore/-/ignore-5.1.8.tgz",
          "version": "5.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "deglob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^5.0.0",
          "to": "node_modules/deglob/node_modules/ignore",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.1.0",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.2",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
        "uniq" => EdgeOut {
          "name": "uniq",
          "spec": "^1.0.1",
          "to": "node_modules/uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/deglob",
      "name": "deglob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/deglob",
      "resolved": "https://registry.npmjs.org/deglob/-/deglob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "diff",
          "spec": "4.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/yargs/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dir-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "dir-glob",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^4.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/dir-glob",
      "name": "dir-glob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/dir-glob",
      "resolved": "https://registry.npmjs.org/dir-glob/-/dir-glob-3.0.1.tgz",
      "version": "3.0.1",
    },
    "doctrine" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "doctrine",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "doctrine",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
      },
      "location": "node_modules/doctrine",
      "name": "doctrine",
      "path": "{CWD}/test/fixtures/yargs/node_modules/doctrine",
      "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-3.0.0.tgz",
      "version": "3.0.0",
    },
    "dot-prop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "dot-prop",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-obj" => EdgeOut {
          "name": "is-obj",
          "spec": "^2.0.0",
          "to": "node_modules/is-obj",
          "type": "prod",
        },
      },
      "location": "node_modules/dot-prop",
      "name": "dot-prop",
      "path": "{CWD}/test/fixtures/yargs/node_modules/dot-prop",
      "resolved": "https://registry.npmjs.org/dot-prop/-/dot-prop-5.3.0.tgz",
      "version": "5.3.0",
    },
    "duplexer3" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "duplexer3",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/duplexer3",
      "name": "duplexer3",
      "path": "{CWD}/test/fixtures/yargs/node_modules/duplexer3",
      "resolved": "https://registry.npmjs.org/duplexer3/-/duplexer3-0.1.4.tgz",
      "version": "0.1.4",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "electron-to-chromium" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "electron-to-chromium",
          "spec": "^1.3.571",
          "type": "prod",
        },
      },
      "location": "node_modules/electron-to-chromium",
      "name": "electron-to-chromium",
      "path": "{CWD}/test/fixtures/yargs/node_modules/electron-to-chromium",
      "resolved": "https://registry.npmjs.org/electron-to-chromium/-/electron-to-chromium-1.3.582.tgz",
      "version": "1.3.582",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "end-of-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pump",
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.4.0",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/end-of-stream",
      "name": "end-of-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/end-of-stream",
      "resolved": "https://registry.npmjs.org/end-of-stream/-/end-of-stream-1.4.4.tgz",
      "version": "1.4.4",
    },
    "enquirer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "enquirer",
          "spec": "^2.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-colors" => EdgeOut {
          "name": "ansi-colors",
          "spec": "^4.1.1",
          "to": "node_modules/ansi-colors",
          "type": "prod",
        },
      },
      "location": "node_modules/enquirer",
      "name": "enquirer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/enquirer",
      "resolved": "https://registry.npmjs.org/enquirer/-/enquirer-2.3.6.tgz",
      "version": "2.3.6",
    },
    "error-ex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file/node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es-abstract" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "es-abstract",
          "spec": "^1.18.0-next.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es-to-primitive" => EdgeOut {
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "to": "node_modules/es-to-primitive",
          "type": "prod",
        },
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
        "has" => EdgeOut {
          "name": "has",
          "spec": "^1.0.3",
          "to": "node_modules/has",
          "type": "prod",
        },
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
        "is-callable" => EdgeOut {
          "name": "is-callable",
          "spec": "^1.2.2",
          "to": "node_modules/is-callable",
          "type": "prod",
        },
        "is-negative-zero" => EdgeOut {
          "name": "is-negative-zero",
          "spec": "^2.0.0",
          "to": "node_modules/is-negative-zero",
          "type": "prod",
        },
        "is-regex" => EdgeOut {
          "name": "is-regex",
          "spec": "^1.1.1",
          "to": "node_modules/is-regex",
          "type": "prod",
        },
        "object-inspect" => EdgeOut {
          "name": "object-inspect",
          "spec": "^1.8.0",
          "to": "node_modules/object-inspect",
          "type": "prod",
        },
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.1.1",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
        "object.assign" => EdgeOut {
          "name": "object.assign",
          "spec": "^4.1.1",
          "to": "node_modules/object.assign",
          "type": "prod",
        },
        "string.prototype.trimend" => EdgeOut {
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "to": "node_modules/string.prototype.trimend",
          "type": "prod",
        },
        "string.prototype.trimstart" => EdgeOut {
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "to": "node_modules/string.prototype.trimstart",
          "type": "prod",
        },
      },
      "location": "node_modules/es-abstract",
      "name": "es-abstract",
      "path": "{CWD}/test/fixtures/yargs/node_modules/es-abstract",
      "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.18.0-next.1.tgz",
      "version": "1.18.0-next.1",
    },
    "es-array-method-boxes-properly" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array.prototype.map",
          "name": "es-array-method-boxes-properly",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/es-array-method-boxes-properly",
      "name": "es-array-method-boxes-properly",
      "path": "{CWD}/test/fixtures/yargs/node_modules/es-array-method-boxes-properly",
      "resolved": "https://registry.npmjs.org/es-array-method-boxes-properly/-/es-array-method-boxes-properly-1.0.0.tgz",
      "version": "1.0.0",
    },
    "es-get-iterator" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es-get-iterator",
              "name": "es-abstract",
              "spec": "^1.17.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/es-get-iterator/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/iterate-value",
          "name": "es-get-iterator",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.4",
          "to": "node_modules/es-get-iterator/node_modules/es-abstract",
          "type": "prod",
        },
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
        "is-arguments" => EdgeOut {
          "name": "is-arguments",
          "spec": "^1.0.4",
          "to": "node_modules/is-arguments",
          "type": "prod",
        },
        "is-map" => EdgeOut {
          "name": "is-map",
          "spec": "^2.0.1",
          "to": "node_modules/is-map",
          "type": "prod",
        },
        "is-set" => EdgeOut {
          "name": "is-set",
          "spec": "^2.0.1",
          "to": "node_modules/is-set",
          "type": "prod",
        },
        "is-string" => EdgeOut {
          "name": "is-string",
          "spec": "^1.0.5",
          "to": "node_modules/is-string",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "^2.0.5",
          "to": "node_modules/isarray",
          "type": "prod",
        },
      },
      "location": "node_modules/es-get-iterator",
      "name": "es-get-iterator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/es-get-iterator",
      "resolved": "https://registry.npmjs.org/es-get-iterator/-/es-get-iterator-1.1.0.tgz",
      "version": "1.1.0",
    },
    "es-to-primitive" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-callable" => EdgeOut {
          "name": "is-callable",
          "spec": "^1.1.4",
          "to": "node_modules/is-callable",
          "type": "prod",
        },
        "is-date-object" => EdgeOut {
          "name": "is-date-object",
          "spec": "^1.0.1",
          "to": "node_modules/is-date-object",
          "type": "prod",
        },
        "is-symbol" => EdgeOut {
          "name": "is-symbol",
          "spec": "^1.0.2",
          "to": "node_modules/is-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es-to-primitive",
      "name": "es-to-primitive",
      "path": "{CWD}/test/fixtures/yargs/node_modules/es-to-primitive",
      "resolved": "https://registry.npmjs.org/es-to-primitive/-/es-to-primitive-1.2.1.tgz",
      "version": "1.2.1",
    },
    "escalade" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "escalade",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "escalade",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "escalade",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/escalade",
      "name": "escalade",
      "path": "{CWD}/test/fixtures/yargs/node_modules/escalade",
      "resolved": "https://registry.npmjs.org/escalade/-/escalade-3.1.1.tgz",
      "version": "3.1.1",
    },
    "escape-goat" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pupa",
          "name": "escape-goat",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-goat",
      "name": "escape-goat",
      "path": "{CWD}/test/fixtures/yargs/node_modules/escape-goat",
      "resolved": "https://registry.npmjs.org/escape-goat/-/escape-goat-2.1.1.tgz",
      "version": "2.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/figures",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "eslint" => ArboristNode {
      "children": Map {
        "globals" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "globals",
              "spec": "^12.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/eslint/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint/node_modules/globals",
          "resolved": "https://registry.npmjs.org/globals/-/globals-12.4.0.tgz",
          "version": "12.4.0",
        },
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "semver",
              "spec": "^7.2.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
          "version": "7.3.2",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/globals",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
          "version": "0.8.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint",
          "spec": "*",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/parser",
          "name": "eslint",
          "spec": "^5.0.0 || ^6.0.0 || ^7.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-config-prettier",
          "name": "eslint",
          "spec": ">=3.14.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-es",
          "name": "eslint",
          "spec": ">=4.19.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-node",
          "name": "eslint",
          "spec": ">=5.16.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-prettier",
          "name": "eslint",
          "spec": ">=5.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-standard",
          "name": "eslint",
          "spec": ">=5.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "eslint",
          "spec": "^7.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@eslint/eslintrc" => EdgeOut {
          "name": "@eslint/eslintrc",
          "spec": "^0.1.3",
          "to": "node_modules/@eslint/eslintrc",
          "type": "prod",
        },
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.10.0",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.2",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.0.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^3.0.0",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "enquirer" => EdgeOut {
          "name": "enquirer",
          "spec": "^2.3.5",
          "to": "node_modules/enquirer",
          "type": "prod",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.1.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-visitor-keys",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^7.3.0",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "esquery" => EdgeOut {
          "name": "esquery",
          "spec": "^1.2.0",
          "to": "node_modules/esquery",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "file-entry-cache" => EdgeOut {
          "name": "file-entry-cache",
          "spec": "^5.0.1",
          "to": "node_modules/file-entry-cache",
          "type": "prod",
        },
        "functional-red-black-tree" => EdgeOut {
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "to": "node_modules/functional-red-black-tree",
          "type": "prod",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "^5.0.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^12.1.0",
          "to": "node_modules/eslint/node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^4.0.6",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "import-fresh" => EdgeOut {
          "name": "import-fresh",
          "spec": "^3.0.0",
          "to": "node_modules/import-fresh",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.0",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "json-stable-stringify-without-jsonify" => EdgeOut {
          "name": "json-stable-stringify-without-jsonify",
          "spec": "^1.0.1",
          "to": "node_modules/json-stable-stringify-without-jsonify",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.4.1",
          "to": "node_modules/levn",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "natural-compare" => EdgeOut {
          "name": "natural-compare",
          "spec": "^1.4.0",
          "to": "node_modules/natural-compare",
          "type": "prod",
        },
        "optionator" => EdgeOut {
          "name": "optionator",
          "spec": "^0.9.1",
          "to": "node_modules/optionator",
          "type": "prod",
        },
        "progress" => EdgeOut {
          "name": "progress",
          "spec": "^2.0.0",
          "to": "node_modules/progress",
          "type": "prod",
        },
        "regexpp" => EdgeOut {
          "name": "regexpp",
          "spec": "^3.1.0",
          "to": "node_modules/regexpp",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.2.1",
          "to": "node_modules/eslint/node_modules/semver",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "^3.1.0",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
        "table" => EdgeOut {
          "name": "table",
          "spec": "^5.2.3",
          "to": "node_modules/table",
          "type": "prod",
        },
        "text-table" => EdgeOut {
          "name": "text-table",
          "spec": "^0.2.0",
          "to": "node_modules/text-table",
          "type": "prod",
        },
        "v8-compile-cache" => EdgeOut {
          "name": "v8-compile-cache",
          "spec": "^2.0.3",
          "to": "node_modules/v8-compile-cache",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint",
      "name": "eslint",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint",
      "resolved": "https://registry.npmjs.org/eslint/-/eslint-7.11.0.tgz",
      "version": "7.11.0",
    },
    "eslint-config-prettier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "eslint-config-prettier",
          "spec": "^6.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.14.1",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "get-stdin" => EdgeOut {
          "name": "get-stdin",
          "spec": "^6.0.0",
          "to": "node_modules/get-stdin",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-config-prettier",
      "name": "eslint-config-prettier",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-config-prettier",
      "resolved": "https://registry.npmjs.org/eslint-config-prettier/-/eslint-config-prettier-6.12.0.tgz",
      "version": "6.12.0",
    },
    "eslint-import-resolver-node" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-import-resolver-node",
              "name": "debug",
              "spec": "^2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/eslint-import-resolver-node/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-import-resolver-node/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-import-resolver-node/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-import-resolver-node/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-import-resolver-node/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-import-resolver-node/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "eslint-import-resolver-node",
          "spec": "^0.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.6.9",
          "to": "node_modules/eslint-import-resolver-node/node_modules/debug",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.13.1",
          "to": "node_modules/resolve",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-import-resolver-node",
      "name": "eslint-import-resolver-node",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-import-resolver-node",
      "resolved": "https://registry.npmjs.org/eslint-import-resolver-node/-/eslint-import-resolver-node-0.3.4.tgz",
      "version": "0.3.4",
    },
    "eslint-module-utils" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-module-utils",
              "name": "debug",
              "spec": "^2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/eslint-module-utils/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-module-utils/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-module-utils/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-module-utils/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-module-utils/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-module-utils/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "eslint-module-utils",
          "spec": "^2.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.6.9",
          "to": "node_modules/eslint-module-utils/node_modules/debug",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^2.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-module-utils",
      "name": "eslint-module-utils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-module-utils",
      "resolved": "https://registry.npmjs.org/eslint-module-utils/-/eslint-module-utils-2.6.0.tgz",
      "version": "2.6.0",
    },
    "eslint-plugin-es" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-node",
          "name": "eslint-plugin-es",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=4.19.1",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
        "regexpp" => EdgeOut {
          "name": "regexpp",
          "spec": "^3.0.0",
          "to": "node_modules/regexpp",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-es",
      "name": "eslint-plugin-es",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-es",
      "resolved": "https://registry.npmjs.org/eslint-plugin-es/-/eslint-plugin-es-3.0.1.tgz",
      "version": "3.0.1",
    },
    "eslint-plugin-node" => ArboristNode {
      "children": Map {
        "ignore" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-plugin-node",
              "name": "ignore",
              "spec": "^5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-plugin-node/node_modules/ignore",
          "name": "ignore",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-node/node_modules/ignore",
          "resolved": "https://registry.npmjs.org/ignore/-/ignore-5.1.8.tgz",
          "version": "5.1.8",
        },
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-plugin-node",
              "name": "semver",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-plugin-node/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-node/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "eslint-plugin-node",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=5.16.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-es" => EdgeOut {
          "name": "eslint-plugin-es",
          "spec": "^3.0.0",
          "to": "node_modules/eslint-plugin-es",
          "type": "prod",
        },
        "eslint-utils" => EdgeOut {
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "to": "node_modules/eslint-utils",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^5.1.1",
          "to": "node_modules/eslint-plugin-node/node_modules/ignore",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.1",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.1.0",
          "to": "node_modules/eslint-plugin-node/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-node",
      "name": "eslint-plugin-node",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-node",
      "resolved": "https://registry.npmjs.org/eslint-plugin-node/-/eslint-plugin-node-11.1.0.tgz",
      "version": "11.1.0",
    },
    "eslint-plugin-prettier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "eslint-plugin-prettier",
          "spec": "^3.1.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=5.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "prettier" => EdgeOut {
          "name": "prettier",
          "spec": ">=1.13.0",
          "to": "node_modules/prettier",
          "type": "peer",
        },
        "prettier-linter-helpers" => EdgeOut {
          "name": "prettier-linter-helpers",
          "spec": "^1.0.0",
          "to": "node_modules/prettier-linter-helpers",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-prettier",
      "name": "eslint-plugin-prettier",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-prettier",
      "resolved": "https://registry.npmjs.org/eslint-plugin-prettier/-/eslint-plugin-prettier-3.1.4.tgz",
      "version": "3.1.4",
    },
    "eslint-plugin-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-promise",
          "spec": "~4.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-config-standard",
          "name": "eslint-plugin-promise",
          "spec": ">=4.2.1",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-plugin-promise",
      "name": "eslint-plugin-promise",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-promise",
      "resolved": "https://registry.npmjs.org/eslint-plugin-promise/-/eslint-plugin-promise-4.2.1.tgz",
      "version": "4.2.1",
    },
    "eslint-plugin-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-standard",
          "spec": "~4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-config-standard",
          "name": "eslint-plugin-standard",
          "spec": ">=4.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=5.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-plugin-standard",
      "name": "eslint-plugin-standard",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-plugin-standard",
      "resolved": "https://registry.npmjs.org/eslint-plugin-standard/-/eslint-plugin-standard-4.0.1.tgz",
      "version": "4.0.1",
    },
    "eslint-scope" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint-scope",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-scope",
          "spec": "^5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "eslint-scope",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.3.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-scope",
      "name": "eslint-scope",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-scope",
      "resolved": "https://registry.npmjs.org/eslint-scope/-/eslint-scope-5.1.1.tgz",
      "version": "5.1.1",
    },
    "eslint-utils" => ArboristNode {
      "children": Map {
        "eslint-visitor-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint-utils",
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "name": "eslint-visitor-keys",
          "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-1.3.0.tgz",
          "version": "1.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/experimental-utils",
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-utils",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-es",
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-node",
          "name": "eslint-utils",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^1.1.0",
          "to": "node_modules/eslint-utils/node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-utils",
      "name": "eslint-utils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-utils",
      "resolved": "https://registry.npmjs.org/eslint-utils/-/eslint-utils-2.1.0.tgz",
      "version": "2.1.0",
    },
    "eslint-visitor-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/visitor-keys",
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "eslint-visitor-keys",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-visitor-keys",
      "name": "eslint-visitor-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/eslint-visitor-keys",
      "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-2.0.0.tgz",
      "version": "2.0.0",
    },
    "espree" => ArboristNode {
      "children": Map {
        "eslint-visitor-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/espree",
              "name": "eslint-visitor-keys",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/espree/node_modules/eslint-visitor-keys",
          "name": "eslint-visitor-keys",
          "path": "{CWD}/test/fixtures/yargs/node_modules/espree/node_modules/eslint-visitor-keys",
          "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-1.3.0.tgz",
          "version": "1.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "espree",
          "spec": "^7.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "espree",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^7.4.0",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "acorn-jsx" => EdgeOut {
          "name": "acorn-jsx",
          "spec": "^5.2.0",
          "to": "node_modules/acorn-jsx",
          "type": "prod",
        },
        "eslint-visitor-keys" => EdgeOut {
          "name": "eslint-visitor-keys",
          "spec": "^1.3.0",
          "to": "node_modules/espree/node_modules/eslint-visitor-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/espree",
      "name": "espree",
      "path": "{CWD}/test/fixtures/yargs/node_modules/espree",
      "resolved": "https://registry.npmjs.org/espree/-/espree-7.3.0.tgz",
      "version": "7.3.0",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/yargs/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esquery" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esquery",
              "name": "estraverse",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esquery/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/yargs/node_modules/esquery/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "esquery",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "esquery",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.1.0",
          "to": "node_modules/esquery/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esquery",
      "name": "esquery",
      "path": "{CWD}/test/fixtures/yargs/node_modules/esquery",
      "resolved": "https://registry.npmjs.org/esquery/-/esquery-1.3.1.tgz",
      "version": "1.3.1",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/yargs/node_modules/esrecurse/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "esrecurse",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.2.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/esrecurse",
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "estraverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "estraverse",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/estraverse",
      "name": "estraverse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/estraverse",
      "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "estree-walker" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@rollup/pluginutils",
          "name": "estree-walker",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/estree-walker",
      "name": "estree-walker",
      "path": "{CWD}/test/fixtures/yargs/node_modules/estree-walker",
      "resolved": "https://registry.npmjs.org/estree-walker/-/estree-walker-2.0.1.tgz",
      "version": "2.0.1",
    },
    "esutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/preset-modules",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/doctrine",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/doctrine",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react/node_modules/doctrine",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "execa" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "execa",
          "spec": "^4.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "get-stream" => EdgeOut {
          "name": "get-stream",
          "spec": "^5.0.0",
          "to": "node_modules/get-stream",
          "type": "prod",
        },
        "human-signals" => EdgeOut {
          "name": "human-signals",
          "spec": "^1.1.1",
          "to": "node_modules/human-signals",
          "type": "prod",
        },
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^2.0.0",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
        "merge-stream" => EdgeOut {
          "name": "merge-stream",
          "spec": "^2.0.0",
          "to": "node_modules/merge-stream",
          "type": "prod",
        },
        "npm-run-path" => EdgeOut {
          "name": "npm-run-path",
          "spec": "^4.0.0",
          "to": "node_modules/npm-run-path",
          "type": "prod",
        },
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "strip-final-newline" => EdgeOut {
          "name": "strip-final-newline",
          "spec": "^2.0.0",
          "to": "node_modules/strip-final-newline",
          "type": "prod",
        },
      },
      "location": "node_modules/execa",
      "name": "execa",
      "path": "{CWD}/test/fixtures/yargs/node_modules/execa",
      "resolved": "https://registry.npmjs.org/execa/-/execa-4.0.3.tgz",
      "version": "4.0.3",
    },
    "extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@mdn/browser-compat-data",
          "name": "extend",
          "spec": "3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/fixtures/yargs/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "external-editor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "external-editor",
          "spec": "^3.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chardet" => EdgeOut {
          "name": "chardet",
          "spec": "^0.7.0",
          "to": "node_modules/chardet",
          "type": "prod",
        },
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "^0.4.24",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
        "tmp" => EdgeOut {
          "name": "tmp",
          "spec": "^0.0.33",
          "to": "node_modules/tmp",
          "type": "prod",
        },
      },
      "location": "node_modules/external-editor",
      "name": "external-editor",
      "path": "{CWD}/test/fixtures/yargs/node_modules/external-editor",
      "resolved": "https://registry.npmjs.org/external-editor/-/external-editor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/yargs/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prettier-linter-helpers",
          "name": "fast-diff",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-diff",
      "name": "fast-diff",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fast-diff",
      "resolved": "https://registry.npmjs.org/fast-diff/-/fast-diff-1.2.0.tgz",
      "version": "1.2.0",
    },
    "fast-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "fast-glob",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nodelib/fs.stat" => EdgeOut {
          "name": "@nodelib/fs.stat",
          "spec": "^2.0.2",
          "to": "node_modules/@nodelib/fs.stat",
          "type": "prod",
        },
        "@nodelib/fs.walk" => EdgeOut {
          "name": "@nodelib/fs.walk",
          "spec": "^1.2.3",
          "to": "node_modules/@nodelib/fs.walk",
          "type": "prod",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "^5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "merge2" => EdgeOut {
          "name": "merge2",
          "spec": "^1.3.0",
          "to": "node_modules/merge2",
          "type": "prod",
        },
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^4.0.2",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.1",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-glob",
      "name": "fast-glob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fast-glob",
      "resolved": "https://registry.npmjs.org/fast-glob/-/fast-glob-3.2.4.tgz",
      "version": "3.2.4",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fast-levenshtein" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "fast-levenshtein",
          "spec": "^2.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/optionator",
          "name": "fast-levenshtein",
          "spec": "~2.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-levenshtein",
      "name": "fast-levenshtein",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fast-levenshtein",
      "resolved": "https://registry.npmjs.org/fast-levenshtein/-/fast-levenshtein-2.0.6.tgz",
      "version": "2.0.6",
    },
    "fastq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.walk",
          "name": "fastq",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "reusify" => EdgeOut {
          "name": "reusify",
          "spec": "^1.0.4",
          "to": "node_modules/reusify",
          "type": "prod",
        },
      },
      "location": "node_modules/fastq",
      "name": "fastq",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fastq",
      "resolved": "https://registry.npmjs.org/fastq/-/fastq-1.8.0.tgz",
      "version": "1.8.0",
    },
    "figures" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "figures",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/figures",
      "name": "figures",
      "path": "{CWD}/test/fixtures/yargs/node_modules/figures",
      "resolved": "https://registry.npmjs.org/figures/-/figures-3.2.0.tgz",
      "version": "3.2.0",
    },
    "file-entry-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flat-cache" => EdgeOut {
          "name": "flat-cache",
          "spec": "^2.0.1",
          "to": "node_modules/flat-cache",
          "type": "prod",
        },
      },
      "location": "node_modules/file-entry-cache",
      "name": "file-entry-cache",
      "path": "{CWD}/test/fixtures/yargs/node_modules/file-entry-cache",
      "resolved": "https://registry.npmjs.org/file-entry-cache/-/file-entry-cache-5.0.1.tgz",
      "version": "5.0.1",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-root" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/find-root",
      "name": "find-root",
      "path": "{CWD}/test/fixtures/yargs/node_modules/find-root",
      "resolved": "https://registry.npmjs.org/find-root/-/find-root-1.1.0.tgz",
      "version": "1.1.0",
    },
    "find-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "find-up",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "find-up",
          "spec": "5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^6.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^4.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/yargs/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-5.0.0.tgz",
      "version": "5.0.0",
    },
    "flat" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-unparser",
          "name": "flat",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-buffer" => EdgeOut {
          "name": "is-buffer",
          "spec": "~2.0.3",
          "to": "node_modules/is-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/flat",
      "name": "flat",
      "path": "{CWD}/test/fixtures/yargs/node_modules/flat",
      "resolved": "https://registry.npmjs.org/flat/-/flat-4.1.1.tgz",
      "version": "4.1.1",
    },
    "flat-cache" => ArboristNode {
      "children": Map {
        "rimraf" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/flat-cache",
              "name": "rimraf",
              "spec": "2.6.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.1.3",
              "to": "node_modules/glob",
              "type": "prod",
            },
          },
          "location": "node_modules/flat-cache/node_modules/rimraf",
          "name": "rimraf",
          "path": "{CWD}/test/fixtures/yargs/node_modules/flat-cache/node_modules/rimraf",
          "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
          "version": "2.6.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "flat-cache",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flatted" => EdgeOut {
          "name": "flatted",
          "spec": "^2.0.0",
          "to": "node_modules/flatted",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "2.6.3",
          "to": "node_modules/flat-cache/node_modules/rimraf",
          "type": "prod",
        },
        "write" => EdgeOut {
          "name": "write",
          "spec": "1.0.3",
          "to": "node_modules/write",
          "type": "prod",
        },
      },
      "location": "node_modules/flat-cache",
      "name": "flat-cache",
      "path": "{CWD}/test/fixtures/yargs/node_modules/flat-cache",
      "resolved": "https://registry.npmjs.org/flat-cache/-/flat-cache-2.0.1.tgz",
      "version": "2.0.1",
    },
    "flatted" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "flatted",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flatted",
      "name": "flatted",
      "path": "{CWD}/test/fixtures/yargs/node_modules/flatted",
      "resolved": "https://registry.npmjs.org/flatted/-/flatted-2.0.2.tgz",
      "version": "2.0.2",
    },
    "foreground-child" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "foreground-child",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^7.0.0",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/yargs/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-2.0.0.tgz",
      "version": "2.0.0",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/yargs/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/yargs/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
        EdgeIn {
          "from": "node_modules/rollup",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/fixtures/yargs/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.3.tgz",
      "version": "2.1.3",
    },
    "function-bind" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/has",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/object.assign",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-bind",
      "name": "function-bind",
      "path": "{CWD}/test/fixtures/yargs/node_modules/function-bind",
      "resolved": "https://registry.npmjs.org/function-bind/-/function-bind-1.1.1.tgz",
      "version": "1.1.1",
    },
    "functional-red-black-tree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "functional-red-black-tree",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/functional-red-black-tree",
      "name": "functional-red-black-tree",
      "path": "{CWD}/test/fixtures/yargs/node_modules/functional-red-black-tree",
      "resolved": "https://registry.npmjs.org/functional-red-black-tree/-/functional-red-black-tree-1.0.1.tgz",
      "version": "1.0.1",
    },
    "furi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "furi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/is-windows" => EdgeOut {
          "name": "@types/is-windows",
          "spec": "^1.0.0",
          "to": "node_modules/@types/is-windows",
          "type": "prod",
        },
        "is-windows" => EdgeOut {
          "name": "is-windows",
          "spec": "^1.0.2",
          "to": "node_modules/is-windows",
          "type": "prod",
        },
      },
      "location": "node_modules/furi",
      "name": "furi",
      "path": "{CWD}/test/fixtures/yargs/node_modules/furi",
      "resolved": "https://registry.npmjs.org/furi/-/furi-2.0.0.tgz",
      "version": "2.0.0",
    },
    "gensync" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "type": "prod",
        },
      },
      "location": "node_modules/gensync",
      "name": "gensync",
      "path": "{CWD}/test/fixtures/yargs/node_modules/gensync",
      "resolved": "https://registry.npmjs.org/gensync/-/gensync-1.0.0-beta.1.tgz",
      "version": "1.0.0-beta.1",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "get-caller-file",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/yargs/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "get-func-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "get-func-name",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/get-func-name",
      "name": "get-func-name",
      "path": "{CWD}/test/fixtures/yargs/node_modules/get-func-name",
      "resolved": "https://registry.npmjs.org/get-func-name/-/get-func-name-2.0.0.tgz",
      "version": "2.0.0",
    },
    "get-stdin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-prettier",
          "name": "get-stdin",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stdin",
      "name": "get-stdin",
      "path": "{CWD}/test/fixtures/yargs/node_modules/get-stdin",
      "resolved": "https://registry.npmjs.org/get-stdin/-/get-stdin-6.0.0.tgz",
      "version": "6.0.0",
    },
    "get-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "get-stream",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/execa",
          "name": "get-stream",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pump" => EdgeOut {
          "name": "pump",
          "spec": "^3.0.0",
          "to": "node_modules/pump",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stream",
      "name": "get-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/get-stream",
      "resolved": "https://registry.npmjs.org/get-stream/-/get-stream-5.2.0.tgz",
      "version": "5.2.0",
    },
    "getpass" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/yargs/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cpr/node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flat-cache/node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "glob",
          "spec": "7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "glob-parent",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "glob-parent",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "glob-parent",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.1.tgz",
      "version": "5.1.1",
    },
    "global-dirs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-installed-globally",
          "name": "global-dirs",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ini" => EdgeOut {
          "name": "ini",
          "spec": "^1.3.5",
          "to": "node_modules/ini",
          "type": "prod",
        },
      },
      "location": "node_modules/global-dirs",
      "name": "global-dirs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/global-dirs",
      "resolved": "https://registry.npmjs.org/global-dirs/-/global-dirs-2.0.1.tgz",
      "version": "2.0.1",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-classes",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "ignore" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "ignore",
              "spec": "^5.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/ignore",
          "name": "ignore",
          "path": "{CWD}/test/fixtures/yargs/node_modules/globby/node_modules/ignore",
          "resolved": "https://registry.npmjs.org/ignore/-/ignore-5.1.8.tgz",
          "version": "5.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "globby",
          "spec": "^11.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^2.1.0",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "dir-glob" => EdgeOut {
          "name": "dir-glob",
          "spec": "^3.0.1",
          "to": "node_modules/dir-glob",
          "type": "prod",
        },
        "fast-glob" => EdgeOut {
          "name": "fast-glob",
          "spec": "^3.1.1",
          "to": "node_modules/fast-glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^5.1.4",
          "to": "node_modules/globby/node_modules/ignore",
          "type": "prod",
        },
        "merge2" => EdgeOut {
          "name": "merge2",
          "spec": "^1.3.0",
          "to": "node_modules/merge2",
          "type": "prod",
        },
        "slash" => EdgeOut {
          "name": "slash",
          "spec": "^3.0.0",
          "to": "node_modules/slash",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/fixtures/yargs/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-11.0.1.tgz",
      "version": "11.0.1",
    },
    "got" => ArboristNode {
      "children": Map {
        "get-stream" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/got",
              "name": "get-stream",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pump" => EdgeOut {
              "name": "pump",
              "spec": "^3.0.0",
              "to": "node_modules/pump",
              "type": "prod",
            },
          },
          "location": "node_modules/got/node_modules/get-stream",
          "name": "get-stream",
          "path": "{CWD}/test/fixtures/yargs/node_modules/got/node_modules/get-stream",
          "resolved": "https://registry.npmjs.org/get-stream/-/get-stream-4.1.0.tgz",
          "version": "4.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-json",
          "name": "got",
          "spec": "^9.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@sindresorhus/is" => EdgeOut {
          "name": "@sindresorhus/is",
          "spec": "^0.14.0",
          "to": "node_modules/@sindresorhus/is",
          "type": "prod",
        },
        "@szmarczak/http-timer" => EdgeOut {
          "name": "@szmarczak/http-timer",
          "spec": "^1.1.2",
          "to": "node_modules/@szmarczak/http-timer",
          "type": "prod",
        },
        "cacheable-request" => EdgeOut {
          "name": "cacheable-request",
          "spec": "^6.0.0",
          "to": "node_modules/cacheable-request",
          "type": "prod",
        },
        "decompress-response" => EdgeOut {
          "name": "decompress-response",
          "spec": "^3.3.0",
          "to": "node_modules/decompress-response",
          "type": "prod",
        },
        "duplexer3" => EdgeOut {
          "name": "duplexer3",
          "spec": "^0.1.4",
          "to": "node_modules/duplexer3",
          "type": "prod",
        },
        "get-stream" => EdgeOut {
          "name": "get-stream",
          "spec": "^4.1.0",
          "to": "node_modules/got/node_modules/get-stream",
          "type": "prod",
        },
        "lowercase-keys" => EdgeOut {
          "name": "lowercase-keys",
          "spec": "^1.0.1",
          "to": "node_modules/lowercase-keys",
          "type": "prod",
        },
        "mimic-response" => EdgeOut {
          "name": "mimic-response",
          "spec": "^1.0.1",
          "to": "node_modules/mimic-response",
          "type": "prod",
        },
        "p-cancelable" => EdgeOut {
          "name": "p-cancelable",
          "spec": "^1.0.0",
          "to": "node_modules/p-cancelable",
          "type": "prod",
        },
        "to-readable-stream" => EdgeOut {
          "name": "to-readable-stream",
          "spec": "^1.0.0",
          "to": "node_modules/to-readable-stream",
          "type": "prod",
        },
        "url-parse-lax" => EdgeOut {
          "name": "url-parse-lax",
          "spec": "^3.0.0",
          "to": "node_modules/url-parse-lax",
          "type": "prod",
        },
      },
      "location": "node_modules/got",
      "name": "got",
      "path": "{CWD}/test/fixtures/yargs/node_modules/got",
      "resolved": "https://registry.npmjs.org/got/-/got-9.6.0.tgz",
      "version": "9.6.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cpr",
          "name": "graceful-fs",
          "spec": "^4.1.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "growl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "growl",
          "spec": "1.10.5",
          "type": "prod",
        },
      },
      "location": "node_modules/growl",
      "name": "growl",
      "path": "{CWD}/test/fixtures/yargs/node_modules/growl",
      "resolved": "https://registry.npmjs.org/growl/-/growl-1.10.5.tgz",
      "version": "1.10.5",
    },
    "gts" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "gts",
          "spec": "^3.0.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@typescript-eslint/eslint-plugin" => EdgeOut {
          "name": "@typescript-eslint/eslint-plugin",
          "spec": "^4.2.0",
          "to": "node_modules/@typescript-eslint/eslint-plugin",
          "type": "prod",
        },
        "@typescript-eslint/parser" => EdgeOut {
          "name": "@typescript-eslint/parser",
          "spec": "^4.2.0",
          "to": "node_modules/@typescript-eslint/parser",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^7.10.0",
          "to": "node_modules/eslint",
          "type": "prod",
        },
        "eslint-config-prettier" => EdgeOut {
          "name": "eslint-config-prettier",
          "spec": "^6.12.0",
          "to": "node_modules/eslint-config-prettier",
          "type": "prod",
        },
        "eslint-plugin-node" => EdgeOut {
          "name": "eslint-plugin-node",
          "spec": "^11.1.0",
          "to": "node_modules/eslint-plugin-node",
          "type": "prod",
        },
        "eslint-plugin-prettier" => EdgeOut {
          "name": "eslint-plugin-prettier",
          "spec": "^3.1.4",
          "to": "node_modules/eslint-plugin-prettier",
          "type": "prod",
        },
        "execa" => EdgeOut {
          "name": "execa",
          "spec": "^4.0.3",
          "to": "node_modules/execa",
          "type": "prod",
        },
        "inquirer" => EdgeOut {
          "name": "inquirer",
          "spec": "^7.3.3",
          "to": "node_modules/inquirer",
          "type": "prod",
        },
        "meow" => EdgeOut {
          "name": "meow",
          "spec": "^7.1.1",
          "to": "node_modules/meow",
          "type": "prod",
        },
        "ncp" => EdgeOut {
          "name": "ncp",
          "spec": "^2.0.0",
          "to": "node_modules/ncp",
          "type": "prod",
        },
        "prettier" => EdgeOut {
          "name": "prettier",
          "spec": "^2.1.2",
          "to": "node_modules/prettier",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^3.0.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=4",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "update-notifier" => EdgeOut {
          "name": "update-notifier",
          "spec": "^5.0.0",
          "to": "node_modules/update-notifier",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.3",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/gts",
      "name": "gts",
      "path": "{CWD}/test/fixtures/yargs/node_modules/gts",
      "resolved": "https://registry.npmjs.org/gts/-/gts-3.0.1.tgz",
      "version": "3.0.1",
    },
    "har-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/fixtures/yargs/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.3",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.5.tgz",
      "version": "5.1.5",
    },
    "hard-rejection" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "hard-rejection",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/hard-rejection",
      "name": "hard-rejection",
      "path": "{CWD}/test/fixtures/yargs/node_modules/hard-rejection",
      "resolved": "https://registry.npmjs.org/hard-rejection/-/hard-rejection-2.1.0.tgz",
      "version": "2.1.0",
    },
    "has" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
      },
      "location": "node_modules/has",
      "name": "has",
      "path": "{CWD}/test/fixtures/yargs/node_modules/has",
      "resolved": "https://registry.npmjs.org/has/-/has-1.0.3.tgz",
      "version": "1.0.3",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/yargs/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
      "version": "4.0.0",
    },
    "has-symbols" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-regex",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-symbol",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/object.assign",
          "name": "has-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/has-symbols",
      "name": "has-symbols",
      "path": "{CWD}/test/fixtures/yargs/node_modules/has-symbols",
      "resolved": "https://registry.npmjs.org/has-symbols/-/has-symbols-1.0.1.tgz",
      "version": "1.0.1",
    },
    "has-yarn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "has-yarn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-yarn",
      "name": "has-yarn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/has-yarn",
      "resolved": "https://registry.npmjs.org/has-yarn/-/has-yarn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "hashish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "hashish",
          "spec": "0.0.4",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "traverse" => EdgeOut {
          "name": "traverse",
          "spec": ">=0.2.4",
          "to": "node_modules/traverse",
          "type": "prod",
        },
      },
      "location": "node_modules/hashish",
      "name": "hashish",
      "path": "{CWD}/test/fixtures/yargs/node_modules/hashish",
      "resolved": "https://registry.npmjs.org/hashish/-/hashish-0.0.4.tgz",
      "version": "0.0.4",
    },
    "he" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "he",
          "spec": "1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/he",
      "name": "he",
      "path": "{CWD}/test/fixtures/yargs/node_modules/he",
      "resolved": "https://registry.npmjs.org/he/-/he-1.2.0.tgz",
      "version": "1.2.0",
    },
    "hosted-git-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/fixtures/yargs/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.8.tgz",
      "version": "2.8.8",
    },
    "html-escaper" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "html-escaper",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/html-escaper",
      "name": "html-escaper",
      "path": "{CWD}/test/fixtures/yargs/node_modules/html-escaper",
      "resolved": "https://registry.npmjs.org/html-escaper/-/html-escaper-2.0.2.tgz",
      "version": "2.0.2",
    },
    "http-cache-semantics" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "http-cache-semantics",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/http-cache-semantics",
      "name": "http-cache-semantics",
      "path": "{CWD}/test/fixtures/yargs/node_modules/http-cache-semantics",
      "resolved": "https://registry.npmjs.org/http-cache-semantics/-/http-cache-semantics-4.1.0.tgz",
      "version": "4.1.0",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/yargs/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "human-signals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "human-signals",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/human-signals",
      "name": "human-signals",
      "path": "{CWD}/test/fixtures/yargs/node_modules/human-signals",
      "resolved": "https://registry.npmjs.org/human-signals/-/human-signals-1.1.1.tgz",
      "version": "1.1.1",
    },
    "iconv-lite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/external-editor",
          "name": "iconv-lite",
          "spec": "^0.4.24",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/iconv-lite",
      "name": "iconv-lite",
      "path": "{CWD}/test/fixtures/yargs/node_modules/iconv-lite",
      "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
      "version": "0.4.24",
    },
    "ignore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "ignore",
          "spec": "^4.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ignore",
          "spec": "^4.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "ignore",
          "spec": "^4.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/ignore",
      "name": "ignore",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ignore",
      "resolved": "https://registry.npmjs.org/ignore/-/ignore-4.0.6.tgz",
      "version": "4.0.6",
    },
    "import-fresh" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "import-fresh",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "import-fresh",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "import-fresh",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "parent-module" => EdgeOut {
          "name": "parent-module",
          "spec": "^1.0.0",
          "to": "node_modules/parent-module",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/import-fresh",
      "name": "import-fresh",
      "path": "{CWD}/test/fixtures/yargs/node_modules/import-fresh",
      "resolved": "https://registry.npmjs.org/import-fresh/-/import-fresh-3.2.1.tgz",
      "version": "3.2.1",
    },
    "import-lazy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "import-lazy",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/import-lazy",
      "name": "import-lazy",
      "path": "{CWD}/test/fixtures/yargs/node_modules/import-lazy",
      "resolved": "https://registry.npmjs.org/import-lazy/-/import-lazy-2.1.0.tgz",
      "version": "2.1.0",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/yargs/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "indent-string" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/redent",
          "name": "indent-string",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/indent-string",
      "name": "indent-string",
      "path": "{CWD}/test/fixtures/yargs/node_modules/indent-string",
      "resolved": "https://registry.npmjs.org/indent-string/-/indent-string-4.0.0.tgz",
      "version": "4.0.0",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/yargs/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/yargs/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "ini" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/global-dirs",
          "name": "ini",
          "spec": "^1.3.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rc",
          "name": "ini",
          "spec": "~1.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ini",
      "name": "ini",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ini",
      "resolved": "https://registry.npmjs.org/ini/-/ini-1.3.5.tgz",
      "version": "1.3.5",
    },
    "inquirer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "inquirer",
          "spec": "^7.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "inquirer",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-width" => EdgeOut {
          "name": "cli-width",
          "spec": "^3.0.0",
          "to": "node_modules/cli-width",
          "type": "prod",
        },
        "external-editor" => EdgeOut {
          "name": "external-editor",
          "spec": "^3.0.3",
          "to": "node_modules/external-editor",
          "type": "prod",
        },
        "figures" => EdgeOut {
          "name": "figures",
          "spec": "^3.0.0",
          "to": "node_modules/figures",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "mute-stream" => EdgeOut {
          "name": "mute-stream",
          "spec": "0.0.8",
          "to": "node_modules/mute-stream",
          "type": "prod",
        },
        "run-async" => EdgeOut {
          "name": "run-async",
          "spec": "^2.4.0",
          "to": "node_modules/run-async",
          "type": "prod",
        },
        "rxjs" => EdgeOut {
          "name": "rxjs",
          "spec": "^6.6.0",
          "to": "node_modules/rxjs",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "through" => EdgeOut {
          "name": "through",
          "spec": "^2.3.6",
          "to": "node_modules/through",
          "type": "prod",
        },
      },
      "location": "node_modules/inquirer",
      "name": "inquirer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/inquirer",
      "resolved": "https://registry.npmjs.org/inquirer/-/inquirer-7.3.3.tgz",
      "version": "7.3.3",
    },
    "is-arguments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "is-arguments",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arguments",
      "name": "is-arguments",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-arguments",
      "resolved": "https://registry.npmjs.org/is-arguments/-/is-arguments-1.0.4.tgz",
      "version": "1.0.4",
    },
    "is-arrayish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat",
          "name": "is-buffer",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/is-buffer",
      "name": "is-buffer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-buffer",
      "resolved": "https://registry.npmjs.org/is-buffer/-/is-buffer-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-callable" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-callable",
          "spec": "^1.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/is-callable",
      "name": "is-callable",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-callable",
      "resolved": "https://registry.npmjs.org/is-callable/-/is-callable-1.2.2.tgz",
      "version": "1.2.2",
    },
    "is-ci" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-date-object" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-date-object",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-date-object",
      "name": "is-date-object",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-date-object",
      "resolved": "https://registry.npmjs.org/is-date-object/-/is-date-object-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-glob",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "is-glob",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-installed-globally" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "is-installed-globally",
          "spec": "^0.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "global-dirs" => EdgeOut {
          "name": "global-dirs",
          "spec": "^2.0.1",
          "to": "node_modules/global-dirs",
          "type": "prod",
        },
        "is-path-inside" => EdgeOut {
          "name": "is-path-inside",
          "spec": "^3.0.1",
          "to": "node_modules/is-path-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-installed-globally",
      "name": "is-installed-globally",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-installed-globally",
      "resolved": "https://registry.npmjs.org/is-installed-globally/-/is-installed-globally-0.3.2.tgz",
      "version": "0.3.2",
    },
    "is-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "is-map",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-map",
      "name": "is-map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-map",
      "resolved": "https://registry.npmjs.org/is-map/-/is-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "is-negative-zero" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-negative-zero",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-negative-zero",
      "name": "is-negative-zero",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-negative-zero",
      "resolved": "https://registry.npmjs.org/is-negative-zero/-/is-negative-zero-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-npm" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "is-npm",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-npm",
      "name": "is-npm",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-npm",
      "resolved": "https://registry.npmjs.org/is-npm/-/is-npm-5.0.0.tgz",
      "version": "5.0.0",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-obj" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dot-prop",
          "name": "is-obj",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-obj",
      "name": "is-obj",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-obj",
      "resolved": "https://registry.npmjs.org/is-obj/-/is-obj-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-path-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-installed-globally",
          "name": "is-path-inside",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-inside",
      "name": "is-path-inside",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-path-inside",
      "resolved": "https://registry.npmjs.org/is-path-inside/-/is-path-inside-3.0.2.tgz",
      "version": "3.0.2",
    },
    "is-plain-obj" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimist-options",
          "name": "is-plain-obj",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser",
          "name": "is-plain-obj",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-plain-obj",
      "name": "is-plain-obj",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-plain-obj",
      "resolved": "https://registry.npmjs.org/is-plain-obj/-/is-plain-obj-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
      },
      "location": "node_modules/is-regex",
      "name": "is-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-regex",
      "resolved": "https://registry.npmjs.org/is-regex/-/is-regex-1.1.1.tgz",
      "version": "1.1.1",
    },
    "is-set" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "is-set",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-set",
      "name": "is-set",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-set",
      "resolved": "https://registry.npmjs.org/is-set/-/is-set-2.0.1.tgz",
      "version": "2.0.1",
    },
    "is-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "is-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-string" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes",
          "name": "is-string",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map",
          "name": "is-string",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "is-string",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/is-string",
      "name": "is-string",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-string",
      "resolved": "https://registry.npmjs.org/is-string/-/is-string-1.0.5.tgz",
      "version": "1.0.5",
    },
    "is-symbol" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-symbol",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
      },
      "location": "node_modules/is-symbol",
      "name": "is-symbol",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-symbol",
      "resolved": "https://registry.npmjs.org/is-symbol/-/is-symbol-1.0.3.tgz",
      "version": "1.0.3",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-windows" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/furi",
          "name": "is-windows",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/is-windows",
      "name": "is-windows",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-windows",
      "resolved": "https://registry.npmjs.org/is-windows/-/is-windows-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-yarn-global" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "is-yarn-global",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-yarn-global",
      "name": "is-yarn-global",
      "path": "{CWD}/test/fixtures/yargs/node_modules/is-yarn-global",
      "resolved": "https://registry.npmjs.org/is-yarn-global/-/is-yarn-global-0.3.0.tgz",
      "version": "0.3.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-get-iterator",
          "name": "isarray",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "path": "{CWD}/test/fixtures/yargs/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-2.0.5.tgz",
      "version": "2.0.5",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/yargs/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/yargs/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-3.0.0.tgz",
      "version": "3.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^3.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/yargs/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-3.0.0.tgz",
      "version": "3.0.0",
    },
    "istanbul-reports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "istanbul-reports",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "html-escaper" => EdgeOut {
          "name": "html-escaper",
          "spec": "^2.0.0",
          "to": "node_modules/html-escaper",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^3.0.0",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/yargs/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-3.0.2.tgz",
      "version": "3.0.2",
    },
    "iterate-iterator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/iterate-value",
          "name": "iterate-iterator",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/iterate-iterator",
      "name": "iterate-iterator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/iterate-iterator",
      "resolved": "https://registry.npmjs.org/iterate-iterator/-/iterate-iterator-1.0.1.tgz",
      "version": "1.0.1",
    },
    "iterate-value" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/promise.allsettled",
          "name": "iterate-value",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es-get-iterator" => EdgeOut {
          "name": "es-get-iterator",
          "spec": "^1.0.2",
          "to": "node_modules/es-get-iterator",
          "type": "prod",
        },
        "iterate-iterator" => EdgeOut {
          "name": "iterate-iterator",
          "spec": "^1.0.1",
          "to": "node_modules/iterate-iterator",
          "type": "prod",
        },
      },
      "location": "node_modules/iterate-value",
      "name": "iterate-value",
      "path": "{CWD}/test/fixtures/yargs/node_modules/iterate-value",
      "resolved": "https://registry.npmjs.org/iterate-value/-/iterate-value-1.0.2.tgz",
      "version": "1.0.2",
    },
    "js-cleanup" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rollup-plugin-cleanup",
          "name": "js-cleanup",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "magic-string" => EdgeOut {
          "name": "magic-string",
          "spec": "^0.25.7",
          "to": "node_modules/magic-string",
          "type": "prod",
        },
        "perf-regexes" => EdgeOut {
          "name": "perf-regexes",
          "spec": "^1.0.1",
          "to": "node_modules/perf-regexes",
          "type": "prod",
        },
        "skip-regex" => EdgeOut {
          "name": "skip-regex",
          "spec": "^1.0.2",
          "to": "node_modules/skip-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/js-cleanup",
      "name": "js-cleanup",
      "path": "{CWD}/test/fixtures/yargs/node_modules/js-cleanup",
      "resolved": "https://registry.npmjs.org/js-cleanup/-/js-cleanup-1.2.0.tgz",
      "version": "1.2.0",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/yargs/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "js-yaml",
          "spec": "3.14.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/yargs/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.14.0.tgz",
      "version": "3.14.0",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/yargs/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/keyv",
          "name": "json-buffer",
          "spec": "3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/json-buffer",
      "name": "json-buffer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-buffer",
      "resolved": "https://registry.npmjs.org/json-buffer/-/json-buffer-3.0.0.tgz",
      "version": "3.0.0",
    },
    "json-parse-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file/node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-parse-even-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-even-better-errors",
          "spec": "^2.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-even-better-errors",
      "name": "json-parse-even-better-errors",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-parse-even-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-even-better-errors/-/json-parse-even-better-errors-2.3.1.tgz",
      "version": "2.3.1",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stable-stringify-without-jsonify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "json-stable-stringify-without-jsonify",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "json-stable-stringify-without-jsonify",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stable-stringify-without-jsonify",
      "name": "json-stable-stringify-without-jsonify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-stable-stringify-without-jsonify",
      "resolved": "https://registry.npmjs.org/json-stable-stringify-without-jsonify/-/json-stable-stringify-without-jsonify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "json5" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "json5",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/fixtures/yargs/node_modules/json5",
      "resolved": "https://registry.npmjs.org/json5/-/json5-2.1.3.tgz",
      "version": "2.1.3",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/yargs/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "jsx-ast-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "jsx-ast-utils",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-includes" => EdgeOut {
          "name": "array-includes",
          "spec": "^3.1.1",
          "to": "node_modules/array-includes",
          "type": "prod",
        },
        "object.assign" => EdgeOut {
          "name": "object.assign",
          "spec": "^4.1.0",
          "to": "node_modules/object.assign",
          "type": "prod",
        },
      },
      "location": "node_modules/jsx-ast-utils",
      "name": "jsx-ast-utils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/jsx-ast-utils",
      "resolved": "https://registry.npmjs.org/jsx-ast-utils/-/jsx-ast-utils-2.4.1.tgz",
      "version": "2.4.1",
    },
    "keyv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "keyv",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "json-buffer" => EdgeOut {
          "name": "json-buffer",
          "spec": "3.0.0",
          "to": "node_modules/json-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/keyv",
      "name": "keyv",
      "path": "{CWD}/test/fixtures/yargs/node_modules/keyv",
      "resolved": "https://registry.npmjs.org/keyv/-/keyv-3.1.0.tgz",
      "version": "3.1.0",
    },
    "kind-of" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimist-options",
          "name": "kind-of",
          "spec": "^6.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/kind-of",
      "name": "kind-of",
      "path": "{CWD}/test/fixtures/yargs/node_modules/kind-of",
      "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
      "version": "6.0.3",
    },
    "latest-version" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "latest-version",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "package-json" => EdgeOut {
          "name": "package-json",
          "spec": "^6.3.0",
          "to": "node_modules/package-json",
          "type": "prod",
        },
      },
      "location": "node_modules/latest-version",
      "name": "latest-version",
      "path": "{CWD}/test/fixtures/yargs/node_modules/latest-version",
      "resolved": "https://registry.npmjs.org/latest-version/-/latest-version-5.1.0.tgz",
      "version": "5.1.0",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "levn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "levn",
          "spec": "^0.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "levn",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
        "type-check" => EdgeOut {
          "name": "type-check",
          "spec": "~0.4.0",
          "to": "node_modules/type-check",
          "type": "prod",
        },
      },
      "location": "node_modules/levn",
      "name": "levn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/levn",
      "resolved": "https://registry.npmjs.org/levn/-/levn-0.4.1.tgz",
      "version": "0.4.1",
    },
    "lines-and-columns" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "lines-and-columns",
          "spec": "^1.1.6",
          "type": "prod",
        },
      },
      "location": "node_modules/lines-and-columns",
      "name": "lines-and-columns",
      "path": "{CWD}/test/fixtures/yargs/node_modules/lines-and-columns",
      "resolved": "https://registry.npmjs.org/lines-and-columns/-/lines-and-columns-1.1.6.tgz",
      "version": "1.1.6",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "parse-json" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "parse-json",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "error-ex" => EdgeOut {
              "name": "error-ex",
              "spec": "^1.3.1",
              "to": "node_modules/error-ex",
              "type": "prod",
            },
            "json-parse-better-errors" => EdgeOut {
              "name": "json-parse-better-errors",
              "spec": "^1.0.1",
              "to": "node_modules/json-parse-better-errors",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/parse-json",
          "name": "parse-json",
          "path": "{CWD}/test/fixtures/yargs/node_modules/load-json-file/node_modules/parse-json",
          "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
          "version": "4.0.0",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "type-fest",
              "spec": "^0.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/load-json-file/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.3.1.tgz",
          "version": "0.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-conf",
          "name": "load-json-file",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file/node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.3.0",
          "to": "node_modules/load-json-file/node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/fixtures/yargs/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-5.3.0.tgz",
      "version": "5.3.0",
    },
    "locate-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^5.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-6.0.0.tgz",
      "version": "6.0.0",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-define-map",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-regex",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "lodash",
          "spec": "^4.17.15",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "lodash",
          "spec": "^4.17.14",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "lodash",
          "spec": "^4.17.14",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/yargs/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/yargs/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "log-symbols" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "log-symbols",
          "spec": "4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
      },
      "location": "node_modules/log-symbols",
      "name": "log-symbols",
      "path": "{CWD}/test/fixtures/yargs/node_modules/log-symbols",
      "resolved": "https://registry.npmjs.org/log-symbols/-/log-symbols-4.0.0.tgz",
      "version": "4.0.0",
    },
    "loose-envify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lowercase-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "lowercase-keys",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/responselike",
          "name": "lowercase-keys",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lowercase-keys",
      "name": "lowercase-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/lowercase-keys",
      "resolved": "https://registry.npmjs.org/lowercase-keys/-/lowercase-keys-1.0.1.tgz",
      "version": "1.0.1",
    },
    "magic-string" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "magic-string",
          "spec": "^0.25.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/js-cleanup",
          "name": "magic-string",
          "spec": "^0.25.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sourcemap-codec" => EdgeOut {
          "name": "sourcemap-codec",
          "spec": "^1.4.4",
          "to": "node_modules/sourcemap-codec",
          "type": "prod",
        },
      },
      "location": "node_modules/magic-string",
      "name": "magic-string",
      "path": "{CWD}/test/fixtures/yargs/node_modules/magic-string",
      "resolved": "https://registry.npmjs.org/magic-string/-/magic-string-0.25.7.tgz",
      "version": "0.25.7",
    },
    "make-dir" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/make-dir",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/make-dir/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/make-dir/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/make-dir/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/yargs/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-3.1.0.tgz",
      "version": "3.1.0",
    },
    "map-obj" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/camelcase-keys",
          "name": "map-obj",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/map-obj",
      "name": "map-obj",
      "path": "{CWD}/test/fixtures/yargs/node_modules/map-obj",
      "resolved": "https://registry.npmjs.org/map-obj/-/map-obj-4.1.0.tgz",
      "version": "4.1.0",
    },
    "meow" => ArboristNode {
      "children": Map {
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/meow",
              "name": "type-fest",
              "spec": "^0.13.1",
              "type": "prod",
            },
          },
          "location": "node_modules/meow/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/meow/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.13.1.tgz",
          "version": "0.13.1",
        },
        "yargs-parser" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/meow",
              "name": "yargs-parser",
              "spec": "^18.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "camelcase" => EdgeOut {
              "name": "camelcase",
              "spec": "^5.0.0",
              "to": "node_modules/camelcase",
              "type": "prod",
            },
            "decamelize" => EdgeOut {
              "name": "decamelize",
              "spec": "^1.2.0",
              "to": "node_modules/decamelize",
              "type": "prod",
            },
          },
          "location": "node_modules/meow/node_modules/yargs-parser",
          "name": "yargs-parser",
          "path": "{CWD}/test/fixtures/yargs/node_modules/meow/node_modules/yargs-parser",
          "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-18.1.3.tgz",
          "version": "18.1.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "meow",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/minimist" => EdgeOut {
          "name": "@types/minimist",
          "spec": "^1.2.0",
          "to": "node_modules/@types/minimist",
          "type": "prod",
        },
        "camelcase-keys" => EdgeOut {
          "name": "camelcase-keys",
          "spec": "^6.2.2",
          "to": "node_modules/camelcase-keys",
          "type": "prod",
        },
        "decamelize-keys" => EdgeOut {
          "name": "decamelize-keys",
          "spec": "^1.1.0",
          "to": "node_modules/decamelize-keys",
          "type": "prod",
        },
        "hard-rejection" => EdgeOut {
          "name": "hard-rejection",
          "spec": "^2.1.0",
          "to": "node_modules/hard-rejection",
          "type": "prod",
        },
        "minimist-options" => EdgeOut {
          "name": "minimist-options",
          "spec": "4.1.0",
          "to": "node_modules/minimist-options",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.5.0",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^7.0.1",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "redent" => EdgeOut {
          "name": "redent",
          "spec": "^3.0.0",
          "to": "node_modules/redent",
          "type": "prod",
        },
        "trim-newlines" => EdgeOut {
          "name": "trim-newlines",
          "spec": "^3.0.0",
          "to": "node_modules/trim-newlines",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.13.1",
          "to": "node_modules/meow/node_modules/type-fest",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^18.1.3",
          "to": "node_modules/meow/node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/meow",
      "name": "meow",
      "path": "{CWD}/test/fixtures/yargs/node_modules/meow",
      "resolved": "https://registry.npmjs.org/meow/-/meow-7.1.1.tgz",
      "version": "7.1.1",
    },
    "merge-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "merge-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-stream",
      "name": "merge-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/merge-stream",
      "resolved": "https://registry.npmjs.org/merge-stream/-/merge-stream-2.0.0.tgz",
      "version": "2.0.0",
    },
    "merge2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "merge2",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "merge2",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/merge2",
      "name": "merge2",
      "path": "{CWD}/test/fixtures/yargs/node_modules/merge2",
      "resolved": "https://registry.npmjs.org/merge2/-/merge2-1.4.1.tgz",
      "version": "1.4.1",
    },
    "micromatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "micromatch",
          "spec": "^4.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "^3.0.1",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.5",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/micromatch",
      "name": "micromatch",
      "path": "{CWD}/test/fixtures/yargs/node_modules/micromatch",
      "resolved": "https://registry.npmjs.org/micromatch/-/micromatch-4.0.2.tgz",
      "version": "4.0.2",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.44.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.44.0.tgz",
      "version": "1.44.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.44.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.27.tgz",
      "version": "2.1.27",
    },
    "mimic-fn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "mimic-response" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/clone-response",
          "name": "mimic-response",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/decompress-response",
          "name": "mimic-response",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/got",
          "name": "mimic-response",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-response",
      "name": "mimic-response",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mimic-response",
      "resolved": "https://registry.npmjs.org/mimic-response/-/mimic-response-1.0.1.tgz",
      "version": "1.0.1",
    },
    "min-indent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-indent",
          "name": "min-indent",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/min-indent",
      "name": "min-indent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/min-indent",
      "resolved": "https://registry.npmjs.org/min-indent/-/min-indent-1.0.1.tgz",
      "version": "1.0.1",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-node",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "minimatch",
          "spec": "3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-node",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/yargs/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cpr",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rc",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/yargs/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "minimist-options" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "minimist-options",
          "spec": "4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^1.0.1",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "is-plain-obj" => EdgeOut {
          "name": "is-plain-obj",
          "spec": "^1.1.0",
          "to": "node_modules/is-plain-obj",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.3",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist-options",
      "name": "minimist-options",
      "path": "{CWD}/test/fixtures/yargs/node_modules/minimist-options",
      "resolved": "https://registry.npmjs.org/minimist-options/-/minimist-options-4.1.0.tgz",
      "version": "4.1.0",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cpr",
          "name": "mkdirp",
          "spec": "~0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "mocha" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/mocha/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/mocha/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/mocha/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/mocha/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/mocha/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "debug",
              "spec": "4.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
          "version": "4.1.1",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "escape-string-regexp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "escape-string-regexp",
              "spec": "4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-4.0.0.tgz",
          "version": "4.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/yargs/node_modules/find-up",
              "name": "locate-path",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^3.0.0",
              "to": "node_modules/mocha/node_modules/p-locate",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^3.0.0",
              "to": "node_modules/mocha/node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
          "version": "3.0.0",
        },
        "object.assign" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "object.assign",
              "spec": "4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "define-properties" => EdgeOut {
              "name": "define-properties",
              "spec": "^1.1.2",
              "to": "node_modules/define-properties",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.0.11",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/object.assign",
          "name": "object.assign",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/object.assign",
          "resolved": "https://registry.npmjs.org/object.assign/-/object.assign-4.1.0.tgz",
          "version": "4.1.0",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^2.0.0",
              "to": "node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
          "version": "2.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^2.0.0",
              "to": "node_modules/mocha/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
          "version": "3.0.0",
        },
        "path-exists" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/locate-path",
              "name": "path-exists",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/path-exists",
          "name": "path-exists",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/path-exists",
          "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
          "version": "3.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/mocha/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/mocha/node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/mocha/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/mocha/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/mocha/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/mocha/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/mocha/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/mocha/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "strip-json-comments" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "strip-json-comments",
              "spec": "3.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/strip-json-comments",
          "name": "strip-json-comments",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/strip-json-comments",
          "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-3.0.1.tgz",
          "version": "3.0.1",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "supports-color",
              "spec": "7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.1.0.tgz",
          "version": "7.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/mocha/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/mocha/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/mocha/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
        "y18n" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha/node_modules/yargs",
              "name": "y18n",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/y18n",
          "name": "y18n",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/y18n",
          "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
          "version": "4.0.0",
        },
        "yargs" => ArboristNode {
          "children": Map {
            "find-up" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/mocha/node_modules/yargs",
                  "name": "find-up",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "locate-path" => EdgeOut {
                  "name": "locate-path",
                  "spec": "^3.0.0",
                  "to": "node_modules/mocha/node_modules/locate-path",
                  "type": "prod",
                },
              },
              "location": "node_modules/mocha/node_modules/yargs/node_modules/find-up",
              "name": "find-up",
              "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/yargs/node_modules/find-up",
              "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
              "version": "3.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "yargs",
              "spec": "13.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cliui" => EdgeOut {
              "name": "cliui",
              "spec": "^5.0.0",
              "to": "node_modules/mocha/node_modules/cliui",
              "type": "prod",
            },
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^3.0.0",
              "to": "node_modules/mocha/node_modules/yargs/node_modules/find-up",
              "type": "prod",
            },
            "get-caller-file" => EdgeOut {
              "name": "get-caller-file",
              "spec": "^2.0.1",
              "to": "node_modules/get-caller-file",
              "type": "prod",
            },
            "require-directory" => EdgeOut {
              "name": "require-directory",
              "spec": "^2.1.1",
              "to": "node_modules/require-directory",
              "type": "prod",
            },
            "require-main-filename" => EdgeOut {
              "name": "require-main-filename",
              "spec": "^2.0.0",
              "to": "node_modules/require-main-filename",
              "type": "prod",
            },
            "set-blocking" => EdgeOut {
              "name": "set-blocking",
              "spec": "^2.0.0",
              "to": "node_modules/set-blocking",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/mocha/node_modules/string-width",
              "type": "prod",
            },
            "which-module" => EdgeOut {
              "name": "which-module",
              "spec": "^2.0.0",
              "to": "node_modules/which-module",
              "type": "prod",
            },
            "y18n" => EdgeOut {
              "name": "y18n",
              "spec": "^4.0.0",
              "to": "node_modules/mocha/node_modules/y18n",
              "type": "prod",
            },
            "yargs-parser" => EdgeOut {
              "name": "yargs-parser",
              "spec": "^13.1.2",
              "to": "node_modules/mocha/node_modules/yargs-parser",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/yargs",
          "name": "yargs",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/yargs",
          "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.2.tgz",
          "version": "13.3.2",
        },
        "yargs-parser" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mocha",
              "name": "yargs-parser",
              "spec": "13.1.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/mocha/node_modules/yargs",
              "name": "yargs-parser",
              "spec": "^13.1.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "camelcase" => EdgeOut {
              "name": "camelcase",
              "spec": "^5.0.0",
              "to": "node_modules/camelcase",
              "type": "prod",
            },
            "decamelize" => EdgeOut {
              "name": "decamelize",
              "spec": "^1.2.0",
              "to": "node_modules/decamelize",
              "type": "prod",
            },
          },
          "location": "node_modules/mocha/node_modules/yargs-parser",
          "name": "yargs-parser",
          "path": "{CWD}/test/fixtures/yargs/node_modules/mocha/node_modules/yargs-parser",
          "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.2.tgz",
          "version": "13.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mocha",
          "spec": "^8.0.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "ansi-colors" => EdgeOut {
          "name": "ansi-colors",
          "spec": "4.1.1",
          "to": "node_modules/ansi-colors",
          "type": "prod",
        },
        "browser-stdout" => EdgeOut {
          "name": "browser-stdout",
          "spec": "1.3.1",
          "to": "node_modules/browser-stdout",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "3.4.2",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "4.1.1",
          "to": "node_modules/mocha/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "4.0.2",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "4.0.0",
          "to": "node_modules/mocha/node_modules/escape-string-regexp",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "5.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "growl" => EdgeOut {
          "name": "growl",
          "spec": "1.10.5",
          "to": "node_modules/growl",
          "type": "prod",
        },
        "he" => EdgeOut {
          "name": "he",
          "spec": "1.2.0",
          "to": "node_modules/he",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "3.14.0",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "log-symbols" => EdgeOut {
          "name": "log-symbols",
          "spec": "4.0.0",
          "to": "node_modules/log-symbols",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
        "object.assign" => EdgeOut {
          "name": "object.assign",
          "spec": "4.1.0",
          "to": "node_modules/mocha/node_modules/object.assign",
          "type": "prod",
        },
        "promise.allsettled" => EdgeOut {
          "name": "promise.allsettled",
          "spec": "1.0.2",
          "to": "node_modules/promise.allsettled",
          "type": "prod",
        },
        "serialize-javascript" => EdgeOut {
          "name": "serialize-javascript",
          "spec": "4.0.0",
          "to": "node_modules/serialize-javascript",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "3.0.1",
          "to": "node_modules/mocha/node_modules/strip-json-comments",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "7.1.0",
          "to": "node_modules/mocha/node_modules/supports-color",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "wide-align" => EdgeOut {
          "name": "wide-align",
          "spec": "1.1.3",
          "to": "node_modules/wide-align",
          "type": "prod",
        },
        "workerpool" => EdgeOut {
          "name": "workerpool",
          "spec": "6.0.0",
          "to": "node_modules/workerpool",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "13.3.2",
          "to": "node_modules/mocha/node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "13.1.2",
          "to": "node_modules/mocha/node_modules/yargs-parser",
          "type": "prod",
        },
        "yargs-unparser" => EdgeOut {
          "name": "yargs-unparser",
          "spec": "1.6.1",
          "to": "node_modules/yargs-unparser",
          "type": "prod",
        },
      },
      "location": "node_modules/mocha",
      "name": "mocha",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mocha",
      "resolved": "https://registry.npmjs.org/mocha/-/mocha-8.1.3.tgz",
      "version": "8.1.3",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "mute-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "mute-stream",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/mute-stream",
      "name": "mute-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/mute-stream",
      "resolved": "https://registry.npmjs.org/mute-stream/-/mute-stream-0.0.8.tgz",
      "version": "0.0.8",
    },
    "natural-compare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/natural-compare",
      "name": "natural-compare",
      "path": "{CWD}/test/fixtures/yargs/node_modules/natural-compare",
      "resolved": "https://registry.npmjs.org/natural-compare/-/natural-compare-1.4.0.tgz",
      "version": "1.4.0",
    },
    "ncp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "ncp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ncp",
      "name": "ncp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ncp",
      "resolved": "https://registry.npmjs.org/ncp/-/ncp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "nice-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/fixtures/yargs/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-releases" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserslist",
          "name": "node-releases",
          "spec": "^1.1.61",
          "type": "prod",
        },
      },
      "location": "node_modules/node-releases",
      "name": "node-releases",
      "path": "{CWD}/test/fixtures/yargs/node_modules/node-releases",
      "resolved": "https://registry.npmjs.org/node-releases/-/node-releases-1.1.63.tgz",
      "version": "1.1.63",
    },
    "normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "normalize-package-data",
          "spec": "^2.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/fixtures/yargs/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "normalize-url" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "normalize-url",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-url",
      "name": "normalize-url",
      "path": "{CWD}/test/fixtures/yargs/node_modules/normalize-url",
      "resolved": "https://registry.npmjs.org/normalize-url/-/normalize-url-4.5.0.tgz",
      "version": "4.5.0",
    },
    "npm-run-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "npm-run-path",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^3.0.0",
          "to": "node_modules/path-key",
          "type": "prod",
        },
      },
      "location": "node_modules/npm-run-path",
      "name": "npm-run-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/npm-run-path",
      "resolved": "https://registry.npmjs.org/npm-run-path/-/npm-run-path-4.0.1.tgz",
      "version": "4.0.1",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/yargs/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "object-inspect" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-inspect",
      "name": "object-inspect",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object-inspect",
      "resolved": "https://registry.npmjs.org/object-inspect/-/object-inspect-1.8.0.tgz",
      "version": "1.8.0",
    },
    "object-keys" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/define-properties",
          "name": "object-keys",
          "spec": "^1.0.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/object.assign",
          "name": "object-keys",
          "spec": "^1.0.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-keys",
      "name": "object-keys",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object-keys",
      "resolved": "https://registry.npmjs.org/object-keys/-/object-keys-1.1.1.tgz",
      "version": "1.1.1",
    },
    "object-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "object-path",
          "spec": "^0.11.5",
          "type": "prod",
        },
      },
      "location": "node_modules/object-path",
      "name": "object-path",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object-path",
      "resolved": "https://registry.npmjs.org/object-path/-/object-path-0.11.5.tgz",
      "version": "0.11.5",
    },
    "object.assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/babel-plugin-dynamic-import-node",
          "name": "object.assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsx-ast-utils",
          "name": "object.assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.18.0-next.0",
          "to": "node_modules/es-abstract",
          "type": "prod",
        },
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.1.1",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/object.assign",
      "name": "object.assign",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object.assign",
      "resolved": "https://registry.npmjs.org/object.assign/-/object.assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "object.entries" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object.entries",
              "name": "es-abstract",
              "spec": "^1.17.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/object.entries/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/object.entries/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "object.entries",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.5",
          "to": "node_modules/object.entries/node_modules/es-abstract",
          "type": "prod",
        },
        "has" => EdgeOut {
          "name": "has",
          "spec": "^1.0.3",
          "to": "node_modules/has",
          "type": "prod",
        },
      },
      "location": "node_modules/object.entries",
      "name": "object.entries",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object.entries",
      "resolved": "https://registry.npmjs.org/object.entries/-/object.entries-1.1.2.tgz",
      "version": "1.1.2",
    },
    "object.fromentries" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object.fromentries",
              "name": "es-abstract",
              "spec": "^1.17.0-next.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/object.fromentries/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "object.fromentries",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0-next.1",
          "to": "node_modules/object.fromentries/node_modules/es-abstract",
          "type": "prod",
        },
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
        "has" => EdgeOut {
          "name": "has",
          "spec": "^1.0.3",
          "to": "node_modules/has",
          "type": "prod",
        },
      },
      "location": "node_modules/object.fromentries",
      "name": "object.fromentries",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object.fromentries",
      "resolved": "https://registry.npmjs.org/object.fromentries/-/object.fromentries-2.0.2.tgz",
      "version": "2.0.2",
    },
    "object.values" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object.values",
              "name": "es-abstract",
              "spec": "^1.17.0-next.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/object.values/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/object.values/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "object.values",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "object.values",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0-next.1",
          "to": "node_modules/object.values/node_modules/es-abstract",
          "type": "prod",
        },
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
        "has" => EdgeOut {
          "name": "has",
          "spec": "^1.0.3",
          "to": "node_modules/has",
          "type": "prod",
        },
      },
      "location": "node_modules/object.values",
      "name": "object.values",
      "path": "{CWD}/test/fixtures/yargs/node_modules/object.values",
      "resolved": "https://registry.npmjs.org/object.values/-/object.values-1.1.1.tgz",
      "version": "1.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/end-of-stream",
          "name": "once",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pump",
          "name": "once",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/yargs/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/yargs/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.2.tgz",
      "version": "5.1.2",
    },
    "optionator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "optionator",
          "spec": "^0.9.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deep-is" => EdgeOut {
          "name": "deep-is",
          "spec": "^0.1.3",
          "to": "node_modules/deep-is",
          "type": "prod",
        },
        "fast-levenshtein" => EdgeOut {
          "name": "fast-levenshtein",
          "spec": "^2.0.6",
          "to": "node_modules/fast-levenshtein",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.4.1",
          "to": "node_modules/levn",
          "type": "prod",
        },
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
        "type-check" => EdgeOut {
          "name": "type-check",
          "spec": "^0.4.0",
          "to": "node_modules/type-check",
          "type": "prod",
        },
        "word-wrap" => EdgeOut {
          "name": "word-wrap",
          "spec": "^1.2.3",
          "to": "node_modules/word-wrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optionator",
      "name": "optionator",
      "path": "{CWD}/test/fixtures/yargs/node_modules/optionator",
      "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.9.1.tgz",
      "version": "0.9.1",
    },
    "os-tmpdir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tmp",
          "name": "os-tmpdir",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/os-tmpdir",
      "name": "os-tmpdir",
      "path": "{CWD}/test/fixtures/yargs/node_modules/os-tmpdir",
      "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "p-cancelable" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "p-cancelable",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-cancelable",
      "name": "p-cancelable",
      "path": "{CWD}/test/fixtures/yargs/node_modules/p-cancelable",
      "resolved": "https://registry.npmjs.org/p-cancelable/-/p-cancelable-1.1.0.tgz",
      "version": "1.1.0",
    },
    "p-limit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/yargs/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-3.0.2.tgz",
      "version": "3.0.2",
    },
    "p-locate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^3.0.2",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/yargs/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-5.0.0.tgz",
      "version": "5.0.0",
    },
    "p-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha/node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-conf/node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up/node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/yargs/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-json" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/package-json",
              "name": "semver",
              "spec": "^6.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/package-json/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/package-json/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/latest-version",
          "name": "package-json",
          "spec": "^6.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "got" => EdgeOut {
          "name": "got",
          "spec": "^9.6.0",
          "to": "node_modules/got",
          "type": "prod",
        },
        "registry-auth-token" => EdgeOut {
          "name": "registry-auth-token",
          "spec": "^4.0.0",
          "to": "node_modules/registry-auth-token",
          "type": "prod",
        },
        "registry-url" => EdgeOut {
          "name": "registry-url",
          "spec": "^5.0.0",
          "to": "node_modules/registry-url",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.2.0",
          "to": "node_modules/package-json/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/package-json",
      "name": "package-json",
      "path": "{CWD}/test/fixtures/yargs/node_modules/package-json",
      "resolved": "https://registry.npmjs.org/package-json/-/package-json-6.5.0.tgz",
      "version": "6.5.0",
    },
    "parent-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-fresh",
          "name": "parent-module",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^3.0.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/parent-module",
      "name": "parent-module",
      "path": "{CWD}/test/fixtures/yargs/node_modules/parent-module",
      "resolved": "https://registry.npmjs.org/parent-module/-/parent-module-1.0.1.tgz",
      "version": "1.0.1",
    },
    "parse-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "parse-json",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-even-better-errors" => EdgeOut {
          "name": "json-parse-even-better-errors",
          "spec": "^2.3.0",
          "to": "node_modules/json-parse-even-better-errors",
          "type": "prod",
        },
        "lines-and-columns" => EdgeOut {
          "name": "lines-and-columns",
          "spec": "^1.1.6",
          "to": "node_modules/lines-and-columns",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/fixtures/yargs/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-5.1.0.tgz",
      "version": "5.1.0",
    },
    "path-exists" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "path-exists",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up/node_modules/find-up",
          "name": "path-exists",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/yargs/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/yargs/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/npm-run-path",
          "name": "path-key",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/yargs/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-3.1.1.tgz",
      "version": "3.1.1",
    },
    "path-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dir-glob",
          "name": "path-type",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/yargs/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-4.0.0.tgz",
      "version": "4.0.0",
    },
    "pathval" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "pathval",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pathval",
      "name": "pathval",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pathval",
      "resolved": "https://registry.npmjs.org/pathval/-/pathval-1.1.0.tgz",
      "version": "1.1.0",
    },
    "perf-regexes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-cleanup",
          "name": "perf-regexes",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/perf-regexes",
      "name": "perf-regexes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/perf-regexes",
      "resolved": "https://registry.npmjs.org/perf-regexes/-/perf-regexes-1.0.1.tgz",
      "version": "1.0.1",
    },
    "performance-now" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/fixtures/yargs/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@rollup/pluginutils",
          "name": "picomatch",
          "spec": "^2.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fast-glob",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "picomatch",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/yargs/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.2.2.tgz",
      "version": "2.2.2",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pkg-conf" => ArboristNode {
      "children": Map {
        "find-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-conf",
              "name": "find-up",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "locate-path" => EdgeOut {
              "name": "locate-path",
              "spec": "^3.0.0",
              "to": "node_modules/pkg-conf/node_modules/locate-path",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-conf/node_modules/find-up",
          "name": "find-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf/node_modules/find-up",
          "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
          "version": "3.0.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-conf/node_modules/find-up",
              "name": "locate-path",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^3.0.0",
              "to": "node_modules/pkg-conf/node_modules/p-locate",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^3.0.0",
              "to": "node_modules/pkg-conf/node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-conf/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
          "version": "3.0.0",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-conf/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^2.0.0",
              "to": "node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-conf/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
          "version": "2.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-conf/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^2.0.0",
              "to": "node_modules/pkg-conf/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-conf/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
          "version": "3.0.0",
        },
        "path-exists" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-conf/node_modules/locate-path",
              "name": "path-exists",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-conf/node_modules/path-exists",
          "name": "path-exists",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf/node_modules/path-exists",
          "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "pkg-conf",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-conf/node_modules/find-up",
          "type": "prod",
        },
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^5.2.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-conf",
      "name": "pkg-conf",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-conf",
      "resolved": "https://registry.npmjs.org/pkg-conf/-/pkg-conf-3.1.0.tgz",
      "version": "3.1.0",
    },
    "pkg-config" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "pkg-config",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug-log" => EdgeOut {
          "name": "debug-log",
          "spec": "^1.0.0",
          "to": "node_modules/debug-log",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.1",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-config",
      "name": "pkg-config",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-config",
      "resolved": "https://registry.npmjs.org/pkg-config/-/pkg-config-1.1.1.tgz",
      "version": "1.1.1",
    },
    "pkg-dir" => ArboristNode {
      "children": Map {
        "find-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir",
              "name": "find-up",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "locate-path" => EdgeOut {
              "name": "locate-path",
              "spec": "^2.0.0",
              "to": "node_modules/pkg-dir/node_modules/locate-path",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/find-up",
          "name": "find-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/find-up",
          "resolved": "https://registry.npmjs.org/find-up/-/find-up-2.1.0.tgz",
          "version": "2.1.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir/node_modules/find-up",
              "name": "locate-path",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^2.0.0",
              "to": "node_modules/pkg-dir/node_modules/p-locate",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^3.0.0",
              "to": "node_modules/pkg-dir/node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-2.0.0.tgz",
          "version": "2.0.0",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^1.0.0",
              "to": "node_modules/pkg-dir/node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-1.3.0.tgz",
          "version": "1.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^1.1.0",
              "to": "node_modules/pkg-dir/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-2.0.0.tgz",
          "version": "2.0.0",
        },
        "p-try" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir/node_modules/p-limit",
              "name": "p-try",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/p-try",
          "name": "p-try",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/p-try",
          "resolved": "https://registry.npmjs.org/p-try/-/p-try-1.0.0.tgz",
          "version": "1.0.0",
        },
        "path-exists" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-dir/node_modules/locate-path",
              "name": "path-exists",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-dir/node_modules/path-exists",
          "name": "path-exists",
          "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir/node_modules/path-exists",
          "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-module-utils",
          "name": "pkg-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^2.1.0",
          "to": "node_modules/pkg-dir/node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-2.0.0.tgz",
      "version": "2.0.0",
    },
    "prelude-ls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/levn",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/type-check",
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/prelude-ls",
      "name": "prelude-ls",
      "path": "{CWD}/test/fixtures/yargs/node_modules/prelude-ls",
      "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.2.1.tgz",
      "version": "1.2.1",
    },
    "prepend-http" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/url-parse-lax",
          "name": "prepend-http",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/prepend-http",
      "name": "prepend-http",
      "path": "{CWD}/test/fixtures/yargs/node_modules/prepend-http",
      "resolved": "https://registry.npmjs.org/prepend-http/-/prepend-http-2.0.0.tgz",
      "version": "2.0.0",
    },
    "prettier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-prettier",
          "name": "prettier",
          "spec": ">=1.13.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "prettier",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/prettier",
      "name": "prettier",
      "path": "{CWD}/test/fixtures/yargs/node_modules/prettier",
      "resolved": "https://registry.npmjs.org/prettier/-/prettier-2.1.2.tgz",
      "version": "2.1.2",
    },
    "prettier-linter-helpers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-prettier",
          "name": "prettier-linter-helpers",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-diff" => EdgeOut {
          "name": "fast-diff",
          "spec": "^1.1.2",
          "to": "node_modules/fast-diff",
          "type": "prod",
        },
      },
      "location": "node_modules/prettier-linter-helpers",
      "name": "prettier-linter-helpers",
      "path": "{CWD}/test/fixtures/yargs/node_modules/prettier-linter-helpers",
      "resolved": "https://registry.npmjs.org/prettier-linter-helpers/-/prettier-linter-helpers-1.0.0.tgz",
      "version": "1.0.0",
    },
    "progress" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "progress",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "progress",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/progress",
      "name": "progress",
      "path": "{CWD}/test/fixtures/yargs/node_modules/progress",
      "resolved": "https://registry.npmjs.org/progress/-/progress-2.0.3.tgz",
      "version": "2.0.3",
    },
    "promise.allsettled" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/promise.allsettled",
              "name": "es-abstract",
              "spec": "^1.17.0-next.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/promise.allsettled/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "promise.allsettled",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array.prototype.map" => EdgeOut {
          "name": "array.prototype.map",
          "spec": "^1.0.1",
          "to": "node_modules/array.prototype.map",
          "type": "prod",
        },
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0-next.1",
          "to": "node_modules/promise.allsettled/node_modules/es-abstract",
          "type": "prod",
        },
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
        "iterate-value" => EdgeOut {
          "name": "iterate-value",
          "spec": "^1.0.0",
          "to": "node_modules/iterate-value",
          "type": "prod",
        },
      },
      "location": "node_modules/promise.allsettled",
      "name": "promise.allsettled",
      "path": "{CWD}/test/fixtures/yargs/node_modules/promise.allsettled",
      "resolved": "https://registry.npmjs.org/promise.allsettled/-/promise.allsettled-1.0.2.tgz",
      "version": "1.0.2",
    },
    "prop-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "prop-types",
          "spec": "^15.7.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/fixtures/yargs/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "psl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.28",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/fixtures/yargs/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.8.0.tgz",
      "version": "1.8.0",
    },
    "pump" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/get-stream",
          "name": "pump",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/got/node_modules/get-stream",
          "name": "pump",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "end-of-stream" => EdgeOut {
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "to": "node_modules/end-of-stream",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.1",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/pump",
      "name": "pump",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pump",
      "resolved": "https://registry.npmjs.org/pump/-/pump-3.0.0.tgz",
      "version": "3.0.0",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/yargs/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "pupa" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "pupa",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-goat" => EdgeOut {
          "name": "escape-goat",
          "spec": "^2.0.0",
          "to": "node_modules/escape-goat",
          "type": "prod",
        },
      },
      "location": "node_modules/pupa",
      "name": "pupa",
      "path": "{CWD}/test/fixtures/yargs/node_modules/pupa",
      "resolved": "https://registry.npmjs.org/pupa/-/pupa-2.0.1.tgz",
      "version": "2.0.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "quick-lru" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/camelcase-keys",
          "name": "quick-lru",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/quick-lru",
      "name": "quick-lru",
      "path": "{CWD}/test/fixtures/yargs/node_modules/quick-lru",
      "resolved": "https://registry.npmjs.org/quick-lru/-/quick-lru-4.0.1.tgz",
      "version": "4.0.1",
    },
    "randombytes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/serialize-javascript",
          "name": "randombytes",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/randombytes",
      "name": "randombytes",
      "path": "{CWD}/test/fixtures/yargs/node_modules/randombytes",
      "resolved": "https://registry.npmjs.org/randombytes/-/randombytes-2.1.0.tgz",
      "version": "2.1.0",
    },
    "rc" => ArboristNode {
      "children": Map {
        "strip-json-comments" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rc",
              "name": "strip-json-comments",
              "spec": "~2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/rc/node_modules/strip-json-comments",
          "name": "strip-json-comments",
          "path": "{CWD}/test/fixtures/yargs/node_modules/rc/node_modules/strip-json-comments",
          "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-2.0.1.tgz",
          "version": "2.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/registry-auth-token",
          "name": "rc",
          "spec": "^1.2.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/registry-url",
          "name": "rc",
          "spec": "^1.2.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deep-extend" => EdgeOut {
          "name": "deep-extend",
          "spec": "^0.6.0",
          "to": "node_modules/deep-extend",
          "type": "prod",
        },
        "ini" => EdgeOut {
          "name": "ini",
          "spec": "~1.3.0",
          "to": "node_modules/ini",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "~2.0.1",
          "to": "node_modules/rc/node_modules/strip-json-comments",
          "type": "prod",
        },
      },
      "location": "node_modules/rc",
      "name": "rc",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rc",
      "resolved": "https://registry.npmjs.org/rc/-/rc-1.2.8.tgz",
      "version": "1.2.8",
    },
    "react-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/fixtures/yargs/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.13.1.tgz",
      "version": "16.13.1",
    },
    "read-pkg" => ArboristNode {
      "children": Map {
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg",
              "name": "type-fest",
              "spec": "^0.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.6.0.tgz",
          "version": "0.6.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/normalize-package-data" => EdgeOut {
          "name": "@types/normalize-package-data",
          "spec": "^2.4.0",
          "to": "node_modules/@types/normalize-package-data",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.5.0",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^5.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.6.0",
          "to": "node_modules/read-pkg/node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-5.2.0.tgz",
      "version": "5.2.0",
    },
    "read-pkg-up" => ArboristNode {
      "children": Map {
        "find-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg-up",
              "name": "find-up",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "locate-path" => EdgeOut {
              "name": "locate-path",
              "spec": "^5.0.0",
              "to": "node_modules/read-pkg-up/node_modules/locate-path",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^4.0.0",
              "to": "node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg-up/node_modules/find-up",
          "name": "find-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up/node_modules/find-up",
          "resolved": "https://registry.npmjs.org/find-up/-/find-up-4.1.0.tgz",
          "version": "4.1.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg-up/node_modules/find-up",
              "name": "locate-path",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^4.1.0",
              "to": "node_modules/read-pkg-up/node_modules/p-locate",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg-up/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-5.0.0.tgz",
          "version": "5.0.0",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg-up/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^2.0.0",
              "to": "node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg-up/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
          "version": "2.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg-up/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^2.2.0",
              "to": "node_modules/read-pkg-up/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg-up/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-4.1.0.tgz",
          "version": "4.1.0",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/read-pkg-up",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/read-pkg-up/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
          "version": "0.8.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "read-pkg-up",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^4.1.0",
          "to": "node_modules/read-pkg-up/node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^5.2.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.1",
          "to": "node_modules/read-pkg-up/node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/fixtures/yargs/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-7.0.1.tgz",
      "version": "7.0.1",
    },
    "readdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.1",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.4.0.tgz",
      "version": "3.4.0",
    },
    "redent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "redent",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "indent-string" => EdgeOut {
          "name": "indent-string",
          "spec": "^4.0.0",
          "to": "node_modules/indent-string",
          "type": "prod",
        },
        "strip-indent" => EdgeOut {
          "name": "strip-indent",
          "spec": "^3.0.0",
          "to": "node_modules/strip-indent",
          "type": "prod",
        },
      },
      "location": "node_modules/redent",
      "name": "redent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/redent",
      "resolved": "https://registry.npmjs.org/redent/-/redent-3.0.0.tgz",
      "version": "3.0.0",
    },
    "regenerate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regenerate-unicode-properties",
          "name": "regenerate",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "regenerate",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerate",
      "name": "regenerate",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regenerate",
      "resolved": "https://registry.npmjs.org/regenerate/-/regenerate-1.4.1.tgz",
      "version": "1.4.1",
    },
    "regenerate-unicode-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "regenerate-unicode-properties",
          "spec": "^8.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerate" => EdgeOut {
          "name": "regenerate",
          "spec": "^1.4.0",
          "to": "node_modules/regenerate",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerate-unicode-properties",
      "name": "regenerate-unicode-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regenerate-unicode-properties",
      "resolved": "https://registry.npmjs.org/regenerate-unicode-properties/-/regenerate-unicode-properties-8.2.0.tgz",
      "version": "8.2.0",
    },
    "regenerator-runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/runtime",
          "name": "regenerator-runtime",
          "spec": "^0.13.4",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-runtime",
      "name": "regenerator-runtime",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regenerator-runtime",
      "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.7.tgz",
      "version": "0.13.7",
    },
    "regenerator-transform" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-regenerator",
          "name": "regenerator-transform",
          "spec": "^0.14.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.8.4",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-transform",
      "name": "regenerator-transform",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regenerator-transform",
      "resolved": "https://registry.npmjs.org/regenerator-transform/-/regenerator-transform-0.14.5.tgz",
      "version": "0.14.5",
    },
    "regexpp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "regexpp",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "regexpp",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-es",
          "name": "regexpp",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/regexpp",
      "name": "regexpp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regexpp",
      "resolved": "https://registry.npmjs.org/regexpp/-/regexpp-3.1.0.tgz",
      "version": "3.1.0",
    },
    "regexpu-core" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-create-regexp-features-plugin",
          "name": "regexpu-core",
          "spec": "^4.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerate" => EdgeOut {
          "name": "regenerate",
          "spec": "^1.4.0",
          "to": "node_modules/regenerate",
          "type": "prod",
        },
        "regenerate-unicode-properties" => EdgeOut {
          "name": "regenerate-unicode-properties",
          "spec": "^8.2.0",
          "to": "node_modules/regenerate-unicode-properties",
          "type": "prod",
        },
        "regjsgen" => EdgeOut {
          "name": "regjsgen",
          "spec": "^0.5.1",
          "to": "node_modules/regjsgen",
          "type": "prod",
        },
        "regjsparser" => EdgeOut {
          "name": "regjsparser",
          "spec": "^0.6.4",
          "to": "node_modules/regjsparser",
          "type": "prod",
        },
        "unicode-match-property-ecmascript" => EdgeOut {
          "name": "unicode-match-property-ecmascript",
          "spec": "^1.0.4",
          "to": "node_modules/unicode-match-property-ecmascript",
          "type": "prod",
        },
        "unicode-match-property-value-ecmascript" => EdgeOut {
          "name": "unicode-match-property-value-ecmascript",
          "spec": "^1.2.0",
          "to": "node_modules/unicode-match-property-value-ecmascript",
          "type": "prod",
        },
      },
      "location": "node_modules/regexpu-core",
      "name": "regexpu-core",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regexpu-core",
      "resolved": "https://registry.npmjs.org/regexpu-core/-/regexpu-core-4.7.1.tgz",
      "version": "4.7.1",
    },
    "registry-auth-token" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-json",
          "name": "registry-auth-token",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "rc" => EdgeOut {
          "name": "rc",
          "spec": "^1.2.8",
          "to": "node_modules/rc",
          "type": "prod",
        },
      },
      "location": "node_modules/registry-auth-token",
      "name": "registry-auth-token",
      "path": "{CWD}/test/fixtures/yargs/node_modules/registry-auth-token",
      "resolved": "https://registry.npmjs.org/registry-auth-token/-/registry-auth-token-4.2.0.tgz",
      "version": "4.2.0",
    },
    "registry-url" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-json",
          "name": "registry-url",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "rc" => EdgeOut {
          "name": "rc",
          "spec": "^1.2.8",
          "to": "node_modules/rc",
          "type": "prod",
        },
      },
      "location": "node_modules/registry-url",
      "name": "registry-url",
      "path": "{CWD}/test/fixtures/yargs/node_modules/registry-url",
      "resolved": "https://registry.npmjs.org/registry-url/-/registry-url-5.1.0.tgz",
      "version": "5.1.0",
    },
    "regjsgen" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "regjsgen",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/regjsgen",
      "name": "regjsgen",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regjsgen",
      "resolved": "https://registry.npmjs.org/regjsgen/-/regjsgen-0.5.2.tgz",
      "version": "0.5.2",
    },
    "regjsparser" => ArboristNode {
      "children": Map {
        "jsesc" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/regjsparser",
              "name": "jsesc",
              "spec": "~0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/regjsparser/node_modules/jsesc",
          "name": "jsesc",
          "path": "{CWD}/test/fixtures/yargs/node_modules/regjsparser/node_modules/jsesc",
          "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-0.5.0.tgz",
          "version": "0.5.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "regjsparser",
          "spec": "^0.6.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "~0.5.0",
          "to": "node_modules/regjsparser/node_modules/jsesc",
          "type": "prod",
        },
      },
      "location": "node_modules/regjsparser",
      "name": "regjsparser",
      "path": "{CWD}/test/fixtures/yargs/node_modules/regjsparser",
      "resolved": "https://registry.npmjs.org/regjsparser/-/regjsparser-0.6.4.tgz",
      "version": "0.6.4",
    },
    "request" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.3",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/yargs/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.2.tgz",
      "version": "2.88.2",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/yargs/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/yargs/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "resolve",
          "spec": "^1.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-runtime",
          "name": "resolve",
          "spec": "^1.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-import-resolver-node",
          "name": "resolve",
          "spec": "^1.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-node",
          "name": "resolve",
          "spec": "^1.10.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "resolve",
          "spec": "^1.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-node",
          "name": "resolve",
          "spec": "^1.10.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "resolve",
          "spec": "^1.10.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/yargs/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.17.0.tgz",
      "version": "1.17.0",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-fresh",
          "name": "resolve-from",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/yargs/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
      "version": "4.0.0",
    },
    "responselike" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacheable-request",
          "name": "responselike",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lowercase-keys" => EdgeOut {
          "name": "lowercase-keys",
          "spec": "^1.0.0",
          "to": "node_modules/lowercase-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/responselike",
      "name": "responselike",
      "path": "{CWD}/test/fixtures/yargs/node_modules/responselike",
      "resolved": "https://registry.npmjs.org/responselike/-/responselike-1.0.2.tgz",
      "version": "1.0.2",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/yargs/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "reusify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fastq",
          "name": "reusify",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/reusify",
      "name": "reusify",
      "path": "{CWD}/test/fixtures/yargs/node_modules/reusify",
      "resolved": "https://registry.npmjs.org/reusify/-/reusify-1.0.4.tgz",
      "version": "1.0.4",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "rimraf",
          "spec": "^3.0.2",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/c8",
          "name": "rimraf",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "rimraf",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-3.0.2.tgz",
      "version": "3.0.2",
    },
    "rollup" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "rollup",
          "spec": "^2.23.0",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/@rollup/pluginutils",
          "name": "rollup",
          "spec": "^1.20.0||^2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "rollup",
          "spec": ">=1.x || >=2.x",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/rollup-plugin-cleanup",
          "name": "rollup",
          "spec": ">=2.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
      },
      "location": "node_modules/rollup",
      "name": "rollup",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rollup",
      "resolved": "https://registry.npmjs.org/rollup/-/rollup-2.31.0.tgz",
      "version": "2.31.0",
    },
    "rollup-plugin-cleanup" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "rollup-plugin-cleanup",
          "spec": "^3.1.1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "js-cleanup" => EdgeOut {
          "name": "js-cleanup",
          "spec": "^1.2.0",
          "to": "node_modules/js-cleanup",
          "type": "prod",
        },
        "rollup" => EdgeOut {
          "name": "rollup",
          "spec": ">=2.0",
          "to": "node_modules/rollup",
          "type": "peer",
        },
        "rollup-pluginutils" => EdgeOut {
          "name": "rollup-pluginutils",
          "spec": "^2.8.2",
          "to": "node_modules/rollup-pluginutils",
          "type": "prod",
        },
      },
      "location": "node_modules/rollup-plugin-cleanup",
      "name": "rollup-plugin-cleanup",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rollup-plugin-cleanup",
      "resolved": "https://registry.npmjs.org/rollup-plugin-cleanup/-/rollup-plugin-cleanup-3.2.1.tgz",
      "version": "3.2.1",
    },
    "rollup-pluginutils" => ArboristNode {
      "children": Map {
        "estree-walker" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rollup-pluginutils",
              "name": "estree-walker",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/rollup-pluginutils/node_modules/estree-walker",
          "name": "estree-walker",
          "path": "{CWD}/test/fixtures/yargs/node_modules/rollup-pluginutils/node_modules/estree-walker",
          "resolved": "https://registry.npmjs.org/estree-walker/-/estree-walker-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rollup-plugin-cleanup",
          "name": "rollup-pluginutils",
          "spec": "^2.8.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estree-walker" => EdgeOut {
          "name": "estree-walker",
          "spec": "^0.6.1",
          "to": "node_modules/rollup-pluginutils/node_modules/estree-walker",
          "type": "prod",
        },
      },
      "location": "node_modules/rollup-pluginutils",
      "name": "rollup-pluginutils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rollup-pluginutils",
      "resolved": "https://registry.npmjs.org/rollup-pluginutils/-/rollup-pluginutils-2.8.2.tgz",
      "version": "2.8.2",
    },
    "run-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "run-async",
          "spec": "^2.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/run-async",
      "name": "run-async",
      "path": "{CWD}/test/fixtures/yargs/node_modules/run-async",
      "resolved": "https://registry.npmjs.org/run-async/-/run-async-2.4.1.tgz",
      "version": "2.4.1",
    },
    "run-parallel" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@nodelib/fs.scandir",
          "name": "run-parallel",
          "spec": "^1.1.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "run-parallel",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/run-parallel",
      "name": "run-parallel",
      "path": "{CWD}/test/fixtures/yargs/node_modules/run-parallel",
      "resolved": "https://registry.npmjs.org/run-parallel/-/run-parallel-1.1.9.tgz",
      "version": "1.1.9",
    },
    "rxjs" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rxjs",
              "name": "tslib",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "location": "node_modules/rxjs/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/yargs/node_modules/rxjs/node_modules/tslib",
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
          "version": "1.14.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "rxjs",
          "spec": "^6.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.9.0",
          "to": "node_modules/rxjs/node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/rxjs",
      "name": "rxjs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/rxjs",
      "resolved": "https://registry.npmjs.org/rxjs/-/rxjs-6.6.3.tgz",
      "version": "6.6.3",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/convert-source-map",
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/randombytes",
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
      "version": "5.1.2",
    },
    "safer-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/iconv-lite",
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "semver",
          "spec": "^5.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-compilation-targets",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-runtime",
          "name": "semver",
          "spec": "^5.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/preset-env",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/yargs/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "semver-diff" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/semver-diff",
              "name": "semver",
              "spec": "^6.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/semver-diff/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/semver-diff/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "semver-diff",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.3.0",
          "to": "node_modules/semver-diff/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/semver-diff",
      "name": "semver-diff",
      "path": "{CWD}/test/fixtures/yargs/node_modules/semver-diff",
      "resolved": "https://registry.npmjs.org/semver-diff/-/semver-diff-3.1.1.tgz",
      "version": "3.1.1",
    },
    "serialize-javascript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "serialize-javascript",
          "spec": "4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.1.0",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
      },
      "location": "node_modules/serialize-javascript",
      "name": "serialize-javascript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/serialize-javascript",
      "resolved": "https://registry.npmjs.org/serialize-javascript/-/serialize-javascript-4.0.0.tgz",
      "version": "4.0.0",
    },
    "set-blocking" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/yargs/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/yargs/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-3.0.0.tgz",
      "version": "3.0.0",
    },
    "signal-exit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/yargs/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "skip-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-cleanup",
          "name": "skip-regex",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/skip-regex",
      "name": "skip-regex",
      "path": "{CWD}/test/fixtures/yargs/node_modules/skip-regex",
      "resolved": "https://registry.npmjs.org/skip-regex/-/skip-regex-1.0.2.tgz",
      "version": "1.0.2",
    },
    "slash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "slash",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "slash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/slash",
      "name": "slash",
      "path": "{CWD}/test/fixtures/yargs/node_modules/slash",
      "resolved": "https://registry.npmjs.org/slash/-/slash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "slice-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/slice-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/yargs/node_modules/slice-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/slice-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/yargs/node_modules/slice-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/yargs/node_modules/slice-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "slice-ansi",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "to": "node_modules/slice-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/yargs/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-2.1.0.tgz",
      "version": "2.1.0",
    },
    "source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/yargs/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
      "version": "0.5.7",
    },
    "sourcemap-codec" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/magic-string",
          "name": "sourcemap-codec",
          "spec": "^1.4.4",
          "type": "prod",
        },
      },
      "location": "node_modules/sourcemap-codec",
      "name": "sourcemap-codec",
      "path": "{CWD}/test/fixtures/yargs/node_modules/sourcemap-codec",
      "resolved": "https://registry.npmjs.org/sourcemap-codec/-/sourcemap-codec-1.4.8.tgz",
      "version": "1.4.8",
    },
    "spdx-correct" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/fixtures/yargs/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.1.tgz",
      "version": "3.1.1",
    },
    "spdx-exceptions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/fixtures/yargs/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.3.0.tgz",
      "version": "2.3.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.1.tgz",
      "version": "3.0.1",
    },
    "spdx-license-ids" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/fixtures/yargs/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.6.tgz",
      "version": "3.0.6",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/yargs/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/yargs/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "standard" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/standard/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "chalk",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "to": "node_modules/standard/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^5.3.0",
              "to": "node_modules/standard/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
          "version": "2.4.2",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/standard/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "cross-spawn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/standard/node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/standard/node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/standard/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "eslint" => ArboristNode {
          "children": Map {
            "semver" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint",
                  "name": "semver",
                  "spec": "^6.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint/node_modules/semver",
              "name": "semver",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint/node_modules/semver",
              "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
              "version": "6.3.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint",
              "spec": "~6.8.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-config-standard",
              "name": "eslint",
              "spec": ">=6.2.2",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-config-standard-jsx",
              "name": "eslint",
              "spec": ">=6.2.2",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-es",
              "name": "eslint",
              "spec": ">=4.19.1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-import",
              "name": "eslint",
              "spec": "2.x - 6.x",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-node",
              "name": "eslint",
              "spec": ">=5.16.0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-react",
              "name": "eslint",
              "spec": "^3.0.0 || ^4.0.0 || ^5.0.0 || ^6.0.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "to": "node_modules/@babel/code-frame",
              "type": "prod",
            },
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.10.0",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^2.1.0",
              "to": "node_modules/standard/node_modules/chalk",
              "type": "prod",
            },
            "cross-spawn" => EdgeOut {
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "to": "node_modules/standard/node_modules/cross-spawn",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.0.1",
              "to": "node_modules/debug",
              "type": "prod",
            },
            "doctrine" => EdgeOut {
              "name": "doctrine",
              "spec": "^3.0.0",
              "to": "node_modules/doctrine",
              "type": "prod",
            },
            "eslint-scope" => EdgeOut {
              "name": "eslint-scope",
              "spec": "^5.0.0",
              "to": "node_modules/eslint-scope",
              "type": "prod",
            },
            "eslint-utils" => EdgeOut {
              "name": "eslint-utils",
              "spec": "^1.4.3",
              "to": "node_modules/standard/node_modules/eslint-utils",
              "type": "prod",
            },
            "eslint-visitor-keys" => EdgeOut {
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "to": "node_modules/standard/node_modules/eslint-visitor-keys",
              "type": "prod",
            },
            "espree" => EdgeOut {
              "name": "espree",
              "spec": "^6.1.2",
              "to": "node_modules/standard/node_modules/espree",
              "type": "prod",
            },
            "esquery" => EdgeOut {
              "name": "esquery",
              "spec": "^1.0.1",
              "to": "node_modules/esquery",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/esutils",
              "type": "prod",
            },
            "file-entry-cache" => EdgeOut {
              "name": "file-entry-cache",
              "spec": "^5.0.1",
              "to": "node_modules/file-entry-cache",
              "type": "prod",
            },
            "functional-red-black-tree" => EdgeOut {
              "name": "functional-red-black-tree",
              "spec": "^1.0.1",
              "to": "node_modules/functional-red-black-tree",
              "type": "prod",
            },
            "glob-parent" => EdgeOut {
              "name": "glob-parent",
              "spec": "^5.0.0",
              "to": "node_modules/glob-parent",
              "type": "prod",
            },
            "globals" => EdgeOut {
              "name": "globals",
              "spec": "^12.1.0",
              "to": "node_modules/standard/node_modules/globals",
              "type": "prod",
            },
            "ignore" => EdgeOut {
              "name": "ignore",
              "spec": "^4.0.6",
              "to": "node_modules/ignore",
              "type": "prod",
            },
            "import-fresh" => EdgeOut {
              "name": "import-fresh",
              "spec": "^3.0.0",
              "to": "node_modules/import-fresh",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "inquirer" => EdgeOut {
              "name": "inquirer",
              "spec": "^7.0.0",
              "to": "node_modules/inquirer",
              "type": "prod",
            },
            "is-glob" => EdgeOut {
              "name": "is-glob",
              "spec": "^4.0.0",
              "to": "node_modules/is-glob",
              "type": "prod",
            },
            "js-yaml" => EdgeOut {
              "name": "js-yaml",
              "spec": "^3.13.1",
              "to": "node_modules/js-yaml",
              "type": "prod",
            },
            "json-stable-stringify-without-jsonify" => EdgeOut {
              "name": "json-stable-stringify-without-jsonify",
              "spec": "^1.0.1",
              "to": "node_modules/json-stable-stringify-without-jsonify",
              "type": "prod",
            },
            "levn" => EdgeOut {
              "name": "levn",
              "spec": "^0.3.0",
              "to": "node_modules/standard/node_modules/levn",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.14",
              "to": "node_modules/lodash",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.4",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "mkdirp" => EdgeOut {
              "name": "mkdirp",
              "spec": "^0.5.1",
              "to": "node_modules/mkdirp",
              "type": "prod",
            },
            "natural-compare" => EdgeOut {
              "name": "natural-compare",
              "spec": "^1.4.0",
              "to": "node_modules/natural-compare",
              "type": "prod",
            },
            "optionator" => EdgeOut {
              "name": "optionator",
              "spec": "^0.8.3",
              "to": "node_modules/standard/node_modules/optionator",
              "type": "prod",
            },
            "progress" => EdgeOut {
              "name": "progress",
              "spec": "^2.0.0",
              "to": "node_modules/progress",
              "type": "prod",
            },
            "regexpp" => EdgeOut {
              "name": "regexpp",
              "spec": "^2.0.1",
              "to": "node_modules/standard/node_modules/regexpp",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^6.1.2",
              "to": "node_modules/standard/node_modules/eslint/node_modules/semver",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/standard/node_modules/strip-ansi",
              "type": "prod",
            },
            "strip-json-comments" => EdgeOut {
              "name": "strip-json-comments",
              "spec": "^3.0.1",
              "to": "node_modules/strip-json-comments",
              "type": "prod",
            },
            "table" => EdgeOut {
              "name": "table",
              "spec": "^5.2.3",
              "to": "node_modules/table",
              "type": "prod",
            },
            "text-table" => EdgeOut {
              "name": "text-table",
              "spec": "^0.2.0",
              "to": "node_modules/text-table",
              "type": "prod",
            },
            "v8-compile-cache" => EdgeOut {
              "name": "v8-compile-cache",
              "spec": "^2.0.3",
              "to": "node_modules/v8-compile-cache",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint",
          "name": "eslint",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint",
          "resolved": "https://registry.npmjs.org/eslint/-/eslint-6.8.0.tgz",
          "version": "6.8.0",
        },
        "eslint-config-standard" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint-config-standard",
              "spec": "14.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": ">=6.2.2",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "eslint-plugin-import" => EdgeOut {
              "name": "eslint-plugin-import",
              "spec": ">=2.18.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-import",
              "type": "peer",
            },
            "eslint-plugin-node" => EdgeOut {
              "name": "eslint-plugin-node",
              "spec": ">=9.1.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-node",
              "type": "peer",
            },
            "eslint-plugin-promise" => EdgeOut {
              "name": "eslint-plugin-promise",
              "spec": ">=4.2.1",
              "to": "node_modules/eslint-plugin-promise",
              "type": "peer",
            },
            "eslint-plugin-standard" => EdgeOut {
              "name": "eslint-plugin-standard",
              "spec": ">=4.0.0",
              "to": "node_modules/eslint-plugin-standard",
              "type": "peer",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-config-standard",
          "name": "eslint-config-standard",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-config-standard",
          "resolved": "https://registry.npmjs.org/eslint-config-standard/-/eslint-config-standard-14.1.1.tgz",
          "version": "14.1.1",
        },
        "eslint-config-standard-jsx" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint-config-standard-jsx",
              "spec": "8.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": ">=6.2.2",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "eslint-plugin-react" => EdgeOut {
              "name": "eslint-plugin-react",
              "spec": ">=7.14.2",
              "to": "node_modules/standard/node_modules/eslint-plugin-react",
              "type": "peer",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-config-standard-jsx",
          "name": "eslint-config-standard-jsx",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-config-standard-jsx",
          "resolved": "https://registry.npmjs.org/eslint-config-standard-jsx/-/eslint-config-standard-jsx-8.1.0.tgz",
          "version": "8.1.0",
        },
        "eslint-plugin-es" => ArboristNode {
          "children": Map {
            "regexpp" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-es",
                  "name": "regexpp",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-es/node_modules/regexpp",
              "name": "regexpp",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-es/node_modules/regexpp",
              "resolved": "https://registry.npmjs.org/regexpp/-/regexpp-3.1.0.tgz",
              "version": "3.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-node",
              "name": "eslint-plugin-es",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": ">=4.19.1",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "eslint-utils" => EdgeOut {
              "name": "eslint-utils",
              "spec": "^1.4.2",
              "to": "node_modules/standard/node_modules/eslint-utils",
              "type": "prod",
            },
            "regexpp" => EdgeOut {
              "name": "regexpp",
              "spec": "^3.0.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-es/node_modules/regexpp",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-plugin-es",
          "name": "eslint-plugin-es",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-es",
          "resolved": "https://registry.npmjs.org/eslint-plugin-es/-/eslint-plugin-es-2.0.0.tgz",
          "version": "2.0.0",
        },
        "eslint-plugin-import" => ArboristNode {
          "children": Map {
            "debug" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-import",
                  "name": "debug",
                  "spec": "^2.6.9",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ms" => EdgeOut {
                  "name": "ms",
                  "spec": "2.0.0",
                  "to": "node_modules/standard/node_modules/ms",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/debug",
              "name": "debug",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-import/node_modules/debug",
              "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
              "version": "2.6.9",
            },
            "doctrine" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-import",
                  "name": "doctrine",
                  "spec": "1.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/esutils",
                  "type": "prod",
                },
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "^1.0.0",
                  "to": "node_modules/standard/node_modules/isarray",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/doctrine",
              "name": "doctrine",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-import/node_modules/doctrine",
              "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-1.5.0.tgz",
              "version": "1.5.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint-plugin-import",
              "spec": "~2.18.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-config-standard",
              "name": "eslint-plugin-import",
              "spec": ">=2.18.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "array-includes" => EdgeOut {
              "name": "array-includes",
              "spec": "^3.0.3",
              "to": "node_modules/array-includes",
              "type": "prod",
            },
            "contains-path" => EdgeOut {
              "name": "contains-path",
              "spec": "^0.1.0",
              "to": "node_modules/contains-path",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^2.6.9",
              "to": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/debug",
              "type": "prod",
            },
            "doctrine" => EdgeOut {
              "name": "doctrine",
              "spec": "1.5.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/doctrine",
              "type": "prod",
            },
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": "2.x - 6.x",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "eslint-import-resolver-node" => EdgeOut {
              "name": "eslint-import-resolver-node",
              "spec": "^0.3.2",
              "to": "node_modules/eslint-import-resolver-node",
              "type": "prod",
            },
            "eslint-module-utils" => EdgeOut {
              "name": "eslint-module-utils",
              "spec": "^2.4.0",
              "to": "node_modules/eslint-module-utils",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.4",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "object.values" => EdgeOut {
              "name": "object.values",
              "spec": "^1.1.0",
              "to": "node_modules/object.values",
              "type": "prod",
            },
            "read-pkg-up" => EdgeOut {
              "name": "read-pkg-up",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/read-pkg-up",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.11.0",
              "to": "node_modules/resolve",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-plugin-import",
          "name": "eslint-plugin-import",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-import",
          "resolved": "https://registry.npmjs.org/eslint-plugin-import/-/eslint-plugin-import-2.18.2.tgz",
          "version": "2.18.2",
        },
        "eslint-plugin-node" => ArboristNode {
          "children": Map {
            "ignore" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-node",
                  "name": "ignore",
                  "spec": "^5.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-node/node_modules/ignore",
              "name": "ignore",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-node/node_modules/ignore",
              "resolved": "https://registry.npmjs.org/ignore/-/ignore-5.1.8.tgz",
              "version": "5.1.8",
            },
            "semver" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-node",
                  "name": "semver",
                  "spec": "^6.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-node/node_modules/semver",
              "name": "semver",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-node/node_modules/semver",
              "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
              "version": "6.3.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint-plugin-node",
              "spec": "~10.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-config-standard",
              "name": "eslint-plugin-node",
              "spec": ">=9.1.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": ">=5.16.0",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "eslint-plugin-es" => EdgeOut {
              "name": "eslint-plugin-es",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-es",
              "type": "prod",
            },
            "eslint-utils" => EdgeOut {
              "name": "eslint-utils",
              "spec": "^1.4.2",
              "to": "node_modules/standard/node_modules/eslint-utils",
              "type": "prod",
            },
            "ignore" => EdgeOut {
              "name": "ignore",
              "spec": "^5.1.1",
              "to": "node_modules/standard/node_modules/eslint-plugin-node/node_modules/ignore",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.4",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.10.1",
              "to": "node_modules/resolve",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^6.1.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-node/node_modules/semver",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-plugin-node",
          "name": "eslint-plugin-node",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-node",
          "resolved": "https://registry.npmjs.org/eslint-plugin-node/-/eslint-plugin-node-10.0.0.tgz",
          "version": "10.0.0",
        },
        "eslint-plugin-react" => ArboristNode {
          "children": Map {
            "doctrine" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/standard/node_modules/eslint-plugin-react",
                  "name": "doctrine",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/esutils",
                  "type": "prod",
                },
              },
              "location": "node_modules/standard/node_modules/eslint-plugin-react/node_modules/doctrine",
              "name": "doctrine",
              "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-react/node_modules/doctrine",
              "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-2.1.0.tgz",
              "version": "2.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard",
              "name": "eslint-plugin-react",
              "spec": "~7.14.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-config-standard-jsx",
              "name": "eslint-plugin-react",
              "spec": ">=7.14.2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "array-includes" => EdgeOut {
              "name": "array-includes",
              "spec": "^3.0.3",
              "to": "node_modules/array-includes",
              "type": "prod",
            },
            "doctrine" => EdgeOut {
              "name": "doctrine",
              "spec": "^2.1.0",
              "to": "node_modules/standard/node_modules/eslint-plugin-react/node_modules/doctrine",
              "type": "prod",
            },
            "eslint" => EdgeOut {
              "name": "eslint",
              "spec": "^3.0.0 || ^4.0.0 || ^5.0.0 || ^6.0.0",
              "to": "node_modules/standard/node_modules/eslint",
              "type": "peer",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "jsx-ast-utils" => EdgeOut {
              "name": "jsx-ast-utils",
              "spec": "^2.1.0",
              "to": "node_modules/jsx-ast-utils",
              "type": "prod",
            },
            "object.entries" => EdgeOut {
              "name": "object.entries",
              "spec": "^1.1.0",
              "to": "node_modules/object.entries",
              "type": "prod",
            },
            "object.fromentries" => EdgeOut {
              "name": "object.fromentries",
              "spec": "^2.0.0",
              "to": "node_modules/object.fromentries",
              "type": "prod",
            },
            "object.values" => EdgeOut {
              "name": "object.values",
              "spec": "^1.1.0",
              "to": "node_modules/object.values",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.7.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.10.1",
              "to": "node_modules/resolve",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-plugin-react",
          "name": "eslint-plugin-react",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-plugin-react",
          "resolved": "https://registry.npmjs.org/eslint-plugin-react/-/eslint-plugin-react-7.14.3.tgz",
          "version": "7.14.3",
        },
        "eslint-utils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "eslint-utils",
              "spec": "^1.4.3",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-es",
              "name": "eslint-utils",
              "spec": "^1.4.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-node",
              "name": "eslint-utils",
              "spec": "^1.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "eslint-visitor-keys" => EdgeOut {
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "to": "node_modules/standard/node_modules/eslint-visitor-keys",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-utils",
          "name": "eslint-utils",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-utils",
          "resolved": "https://registry.npmjs.org/eslint-utils/-/eslint-utils-1.4.3.tgz",
          "version": "1.4.3",
        },
        "eslint-visitor-keys" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-utils",
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/espree",
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/eslint-visitor-keys",
          "name": "eslint-visitor-keys",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/eslint-visitor-keys",
          "resolved": "https://registry.npmjs.org/eslint-visitor-keys/-/eslint-visitor-keys-1.3.0.tgz",
          "version": "1.3.0",
        },
        "espree" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "espree",
              "spec": "^6.1.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "acorn" => EdgeOut {
              "name": "acorn",
              "spec": "^7.1.1",
              "to": "node_modules/acorn",
              "type": "prod",
            },
            "acorn-jsx" => EdgeOut {
              "name": "acorn-jsx",
              "spec": "^5.2.0",
              "to": "node_modules/acorn-jsx",
              "type": "prod",
            },
            "eslint-visitor-keys" => EdgeOut {
              "name": "eslint-visitor-keys",
              "spec": "^1.1.0",
              "to": "node_modules/standard/node_modules/eslint-visitor-keys",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/espree",
          "name": "espree",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/espree",
          "resolved": "https://registry.npmjs.org/espree/-/espree-6.2.1.tgz",
          "version": "6.2.1",
        },
        "find-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/read-pkg-up",
              "name": "find-up",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "locate-path" => EdgeOut {
              "name": "locate-path",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/locate-path",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/find-up",
          "name": "find-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/find-up",
          "resolved": "https://registry.npmjs.org/find-up/-/find-up-2.1.0.tgz",
          "version": "2.1.0",
        },
        "globals" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "globals",
              "spec": "^12.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/standard/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/globals",
          "resolved": "https://registry.npmjs.org/globals/-/globals-12.4.0.tgz",
          "version": "12.4.0",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/doctrine",
              "name": "isarray",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "levn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "levn",
              "spec": "^0.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/optionator",
              "name": "levn",
              "spec": "~0.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/standard/node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/standard/node_modules/type-check",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/levn",
          "name": "levn",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/levn",
          "resolved": "https://registry.npmjs.org/levn/-/levn-0.3.0.tgz",
          "version": "0.3.0",
        },
        "load-json-file" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/read-pkg",
              "name": "load-json-file",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.2",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "parse-json" => EdgeOut {
              "name": "parse-json",
              "spec": "^2.2.0",
              "to": "node_modules/standard/node_modules/parse-json",
              "type": "prod",
            },
            "pify" => EdgeOut {
              "name": "pify",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/pify",
              "type": "prod",
            },
            "strip-bom" => EdgeOut {
              "name": "strip-bom",
              "spec": "^3.0.0",
              "to": "node_modules/strip-bom",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/load-json-file",
          "name": "load-json-file",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/load-json-file",
          "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-2.0.0.tgz",
          "version": "2.0.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/find-up",
              "name": "locate-path",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/p-locate",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^3.0.0",
              "to": "node_modules/standard/node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-2.0.0.tgz",
          "version": "2.0.0",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-import/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
        "optionator" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "optionator",
              "spec": "^0.8.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "deep-is" => EdgeOut {
              "name": "deep-is",
              "spec": "~0.1.3",
              "to": "node_modules/deep-is",
              "type": "prod",
            },
            "fast-levenshtein" => EdgeOut {
              "name": "fast-levenshtein",
              "spec": "~2.0.6",
              "to": "node_modules/fast-levenshtein",
              "type": "prod",
            },
            "levn" => EdgeOut {
              "name": "levn",
              "spec": "~0.3.0",
              "to": "node_modules/standard/node_modules/levn",
              "type": "prod",
            },
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/standard/node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/standard/node_modules/type-check",
              "type": "prod",
            },
            "word-wrap" => EdgeOut {
              "name": "word-wrap",
              "spec": "~1.2.3",
              "to": "node_modules/word-wrap",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/optionator",
          "name": "optionator",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/optionator",
          "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.8.3.tgz",
          "version": "0.8.3",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^1.0.0",
              "to": "node_modules/standard/node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-1.3.0.tgz",
          "version": "1.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^1.1.0",
              "to": "node_modules/standard/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-2.0.0.tgz",
          "version": "2.0.0",
        },
        "p-try" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/p-limit",
              "name": "p-try",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/p-try",
          "name": "p-try",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/p-try",
          "resolved": "https://registry.npmjs.org/p-try/-/p-try-1.0.0.tgz",
          "version": "1.0.0",
        },
        "parse-json" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/load-json-file",
              "name": "parse-json",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "error-ex" => EdgeOut {
              "name": "error-ex",
              "spec": "^1.2.0",
              "to": "node_modules/error-ex",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/parse-json",
          "name": "parse-json",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/parse-json",
          "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-2.2.0.tgz",
          "version": "2.2.0",
        },
        "path-exists" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/locate-path",
              "name": "path-exists",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/path-exists",
          "name": "path-exists",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/path-exists",
          "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
          "version": "3.0.0",
        },
        "path-key" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/cross-spawn",
              "name": "path-key",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/path-key",
          "name": "path-key",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/path-key",
          "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
          "version": "2.0.1",
        },
        "path-type" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/read-pkg",
              "name": "path-type",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pify" => EdgeOut {
              "name": "pify",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/pify",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/path-type",
          "name": "path-type",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/path-type",
          "resolved": "https://registry.npmjs.org/path-type/-/path-type-2.0.0.tgz",
          "version": "2.0.0",
        },
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/load-json-file",
              "name": "pify",
              "spec": "^2.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/path-type",
              "name": "pify",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-2.3.0.tgz",
          "version": "2.3.0",
        },
        "prelude-ls" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/levn",
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/optionator",
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/type-check",
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/prelude-ls",
          "name": "prelude-ls",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/prelude-ls",
          "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.1.2.tgz",
          "version": "1.1.2",
        },
        "read-pkg" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/read-pkg-up",
              "name": "read-pkg",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "load-json-file" => EdgeOut {
              "name": "load-json-file",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/load-json-file",
              "type": "prod",
            },
            "normalize-package-data" => EdgeOut {
              "name": "normalize-package-data",
              "spec": "^2.3.2",
              "to": "node_modules/normalize-package-data",
              "type": "prod",
            },
            "path-type" => EdgeOut {
              "name": "path-type",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/path-type",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/read-pkg",
          "name": "read-pkg",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/read-pkg",
          "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-2.0.0.tgz",
          "version": "2.0.0",
        },
        "read-pkg-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint-plugin-import",
              "name": "read-pkg-up",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/find-up",
              "type": "prod",
            },
            "read-pkg" => EdgeOut {
              "name": "read-pkg",
              "spec": "^2.0.0",
              "to": "node_modules/standard/node_modules/read-pkg",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/read-pkg-up",
          "name": "read-pkg-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/read-pkg-up",
          "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-2.0.0.tgz",
          "version": "2.0.0",
        },
        "regexpp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "regexpp",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/regexpp",
          "name": "regexpp",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/regexpp",
          "resolved": "https://registry.npmjs.org/regexpp/-/regexpp-2.0.1.tgz",
          "version": "2.0.1",
        },
        "shebang-command" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/cross-spawn",
              "name": "shebang-command",
              "spec": "^1.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "shebang-regex" => EdgeOut {
              "name": "shebang-regex",
              "spec": "^1.0.0",
              "to": "node_modules/standard/node_modules/shebang-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/shebang-command",
          "name": "shebang-command",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/shebang-command",
          "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
          "version": "1.2.0",
        },
        "shebang-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/shebang-command",
              "name": "shebang-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/shebang-regex",
          "name": "shebang-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/shebang-regex",
          "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/eslint",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/standard/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/chalk",
              "name": "supports-color",
              "spec": "^5.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/standard/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
          "version": "5.5.0",
        },
        "type-check" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/levn",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/standard/node_modules/optionator",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/standard/node_modules/prelude-ls",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/type-check",
          "name": "type-check",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/type-check",
          "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.3.2.tgz",
          "version": "0.3.2",
        },
        "type-fest" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/globals",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/type-fest",
          "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
          "version": "0.8.1",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/standard/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standardx",
          "name": "standard",
          "spec": "^14.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "~6.8.0",
          "to": "node_modules/standard/node_modules/eslint",
          "type": "prod",
        },
        "eslint-config-standard" => EdgeOut {
          "name": "eslint-config-standard",
          "spec": "14.1.1",
          "to": "node_modules/standard/node_modules/eslint-config-standard",
          "type": "prod",
        },
        "eslint-config-standard-jsx" => EdgeOut {
          "name": "eslint-config-standard-jsx",
          "spec": "8.1.0",
          "to": "node_modules/standard/node_modules/eslint-config-standard-jsx",
          "type": "prod",
        },
        "eslint-plugin-import" => EdgeOut {
          "name": "eslint-plugin-import",
          "spec": "~2.18.0",
          "to": "node_modules/standard/node_modules/eslint-plugin-import",
          "type": "prod",
        },
        "eslint-plugin-node" => EdgeOut {
          "name": "eslint-plugin-node",
          "spec": "~10.0.0",
          "to": "node_modules/standard/node_modules/eslint-plugin-node",
          "type": "prod",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": "~4.2.1",
          "to": "node_modules/eslint-plugin-promise",
          "type": "prod",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": "~7.14.2",
          "to": "node_modules/standard/node_modules/eslint-plugin-react",
          "type": "prod",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": "~4.0.0",
          "to": "node_modules/eslint-plugin-standard",
          "type": "prod",
        },
        "standard-engine" => EdgeOut {
          "name": "standard-engine",
          "spec": "^12.0.0",
          "to": "node_modules/standard-engine",
          "type": "prod",
        },
      },
      "location": "node_modules/standard",
      "name": "standard",
      "path": "{CWD}/test/fixtures/yargs/node_modules/standard",
      "resolved": "https://registry.npmjs.org/standard/-/standard-14.3.4.tgz",
      "version": "14.3.4",
    },
    "standard-engine" => ArboristNode {
      "children": Map {
        "get-stdin" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard-engine",
              "name": "get-stdin",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard-engine/node_modules/get-stdin",
          "name": "get-stdin",
          "path": "{CWD}/test/fixtures/yargs/node_modules/standard-engine/node_modules/get-stdin",
          "resolved": "https://registry.npmjs.org/get-stdin/-/get-stdin-7.0.0.tgz",
          "version": "7.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "standard-engine",
          "spec": "^12.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standardx",
          "name": "standard-engine",
          "spec": "^12.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deglob" => EdgeOut {
          "name": "deglob",
          "spec": "^4.0.1",
          "to": "node_modules/deglob",
          "type": "prod",
        },
        "get-stdin" => EdgeOut {
          "name": "get-stdin",
          "spec": "^7.0.0",
          "to": "node_modules/standard-engine/node_modules/get-stdin",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "pkg-conf" => EdgeOut {
          "name": "pkg-conf",
          "spec": "^3.1.0",
          "to": "node_modules/pkg-conf",
          "type": "prod",
        },
      },
      "location": "node_modules/standard-engine",
      "name": "standard-engine",
      "path": "{CWD}/test/fixtures/yargs/node_modules/standard-engine",
      "resolved": "https://registry.npmjs.org/standard-engine/-/standard-engine-12.1.0.tgz",
      "version": "12.1.0",
    },
    "standardx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "standardx",
          "spec": "^5.0.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "standard" => EdgeOut {
          "name": "standard",
          "spec": "^14.0.0",
          "to": "node_modules/standard",
          "type": "prod",
        },
        "standard-engine" => EdgeOut {
          "name": "standard-engine",
          "spec": "^12.0.0",
          "to": "node_modules/standard-engine",
          "type": "prod",
        },
      },
      "location": "node_modules/standardx",
      "name": "standardx",
      "path": "{CWD}/test/fixtures/yargs/node_modules/standardx",
      "resolved": "https://registry.npmjs.org/standardx/-/standardx-5.0.0.tgz",
      "version": "5.0.0",
    },
    "string-width" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line",
          "name": "string-width",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/yargs/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "string.prototype.trimend" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string.prototype.trimend",
              "name": "es-abstract",
              "spec": "^1.17.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/string.prototype.trimend/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.5",
          "to": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/string.prototype.trimend",
      "name": "string.prototype.trimend",
      "path": "{CWD}/test/fixtures/yargs/node_modules/string.prototype.trimend",
      "resolved": "https://registry.npmjs.org/string.prototype.trimend/-/string.prototype.trimend-1.0.1.tgz",
      "version": "1.0.1",
    },
    "string.prototype.trimstart" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string.prototype.trimstart",
              "name": "es-abstract",
              "spec": "^1.17.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/fixtures/yargs/node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-includes/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/array.prototype.map/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-get-iterator/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.entries/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.fromentries/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.values/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/promise.allsettled/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.5",
          "to": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/string.prototype.trimstart",
      "name": "string.prototype.trimstart",
      "path": "{CWD}/test/fixtures/yargs/node_modules/string.prototype.trimstart",
      "resolved": "https://registry.npmjs.org/string.prototype.trimstart/-/string.prototype.trimstart-1.0.1.tgz",
      "version": "1.0.1",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/yargs/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
      "version": "6.0.0",
    },
    "strip-bom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/yargs/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "strip-final-newline" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "strip-final-newline",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-final-newline",
      "name": "strip-final-newline",
      "path": "{CWD}/test/fixtures/yargs/node_modules/strip-final-newline",
      "resolved": "https://registry.npmjs.org/strip-final-newline/-/strip-final-newline-2.0.0.tgz",
      "version": "2.0.0",
    },
    "strip-indent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/redent",
          "name": "strip-indent",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "min-indent" => EdgeOut {
          "name": "min-indent",
          "spec": "^1.0.0",
          "to": "node_modules/min-indent",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-indent",
      "name": "strip-indent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/strip-indent",
      "resolved": "https://registry.npmjs.org/strip-indent/-/strip-indent-3.0.0.tgz",
      "version": "3.0.0",
    },
    "strip-json-comments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@eslint/eslintrc",
          "name": "strip-json-comments",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-json-comments",
      "name": "strip-json-comments",
      "path": "{CWD}/test/fixtures/yargs/node_modules/strip-json-comments",
      "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-3.1.1.tgz",
      "version": "3.1.1",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen/node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^4.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/yargs/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
      "version": "7.2.0",
    },
    "table" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/table/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/table/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/table/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/table/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/table/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/table/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/yargs/node_modules/table/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/table/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/table/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "table",
          "spec": "^5.2.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "table",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.10.2",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.14",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^2.1.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/table/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/table",
      "name": "table",
      "path": "{CWD}/test/fixtures/yargs/node_modules/table",
      "resolved": "https://registry.npmjs.org/table/-/table-5.4.6.tgz",
      "version": "5.4.6",
    },
    "term-size" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "term-size",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/term-size",
      "name": "term-size",
      "path": "{CWD}/test/fixtures/yargs/node_modules/term-size",
      "resolved": "https://registry.npmjs.org/term-size/-/term-size-2.2.0.tgz",
      "version": "2.2.0",
    },
    "test-exclude" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "test-exclude",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@istanbuljs/schema" => EdgeOut {
          "name": "@istanbuljs/schema",
          "spec": "^0.1.2",
          "to": "node_modules/@istanbuljs/schema",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.4",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/yargs/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-6.0.0.tgz",
      "version": "6.0.0",
    },
    "text-table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "text-table",
          "spec": "^0.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "text-table",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/text-table",
      "name": "text-table",
      "path": "{CWD}/test/fixtures/yargs/node_modules/text-table",
      "resolved": "https://registry.npmjs.org/text-table/-/text-table-0.2.0.tgz",
      "version": "0.2.0",
    },
    "through" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "through",
          "spec": "^2.3.6",
          "type": "prod",
        },
      },
      "location": "node_modules/through",
      "name": "through",
      "path": "{CWD}/test/fixtures/yargs/node_modules/through",
      "resolved": "https://registry.npmjs.org/through/-/through-2.3.8.tgz",
      "version": "2.3.8",
    },
    "tmp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/external-editor",
          "name": "tmp",
          "spec": "^0.0.33",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-tmpdir" => EdgeOut {
          "name": "os-tmpdir",
          "spec": "~1.0.2",
          "to": "node_modules/os-tmpdir",
          "type": "prod",
        },
      },
      "location": "node_modules/tmp",
      "name": "tmp",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tmp",
      "resolved": "https://registry.npmjs.org/tmp/-/tmp-0.0.33.tgz",
      "version": "0.0.33",
    },
    "to-fast-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/yargs/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-readable-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "to-readable-stream",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-readable-stream",
      "name": "to-readable-stream",
      "path": "{CWD}/test/fixtures/yargs/node_modules/to-readable-stream",
      "resolved": "https://registry.npmjs.org/to-readable-stream/-/to-readable-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/yargs/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.28",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.5.0.tgz",
      "version": "2.5.0",
    },
    "traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hashish",
          "name": "traverse",
          "spec": ">=0.2.4",
          "type": "prod",
        },
      },
      "location": "node_modules/traverse",
      "name": "traverse",
      "path": "{CWD}/test/fixtures/yargs/node_modules/traverse",
      "resolved": "https://registry.npmjs.org/traverse/-/traverse-0.6.6.tgz",
      "version": "0.6.6",
    },
    "trim-newlines" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/meow",
          "name": "trim-newlines",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/trim-newlines",
      "name": "trim-newlines",
      "path": "{CWD}/test/fixtures/yargs/node_modules/trim-newlines",
      "resolved": "https://registry.npmjs.org/trim-newlines/-/trim-newlines-3.0.0.tgz",
      "version": "3.0.0",
    },
    "tslib" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "tslib",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tslib",
      "name": "tslib",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tslib",
      "resolved": "https://registry.npmjs.org/tslib/-/tslib-2.0.3.tgz",
      "version": "2.0.3",
    },
    "tsutils" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tsutils",
              "name": "tslib",
              "spec": "^1.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tsutils/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/yargs/node_modules/tsutils/node_modules/tslib",
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
          "version": "1.14.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@typescript-eslint/eslint-plugin",
          "name": "tsutils",
          "spec": "^3.17.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@typescript-eslint/typescript-estree",
          "name": "tsutils",
          "spec": "^3.17.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.8.1",
          "to": "node_modules/tsutils/node_modules/tslib",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.8.0 || >= 3.2.0-dev || >= 3.3.0-dev || >= 3.4.0-dev || >= 3.5.0-dev || >= 3.6.0-dev || >= 3.6.0-beta || >= 3.7.0-dev || >= 3.7.0-beta",
          "to": "node_modules/typescript",
          "type": "peer",
        },
      },
      "location": "node_modules/tsutils",
      "name": "tsutils",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tsutils",
      "resolved": "https://registry.npmjs.org/tsutils/-/tsutils-3.17.1.tgz",
      "version": "3.17.1",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/fixtures/yargs/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "type-check" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/levn",
          "name": "type-check",
          "spec": "~0.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "type-check",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prelude-ls" => EdgeOut {
          "name": "prelude-ls",
          "spec": "^1.2.1",
          "to": "node_modules/prelude-ls",
          "type": "prod",
        },
      },
      "location": "node_modules/type-check",
      "name": "type-check",
      "path": "{CWD}/test/fixtures/yargs/node_modules/type-check",
      "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.4.0.tgz",
      "version": "0.4.0",
    },
    "type-detect" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chai",
          "name": "type-detect",
          "spec": "^4.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/deep-eql",
          "name": "type-detect",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-detect",
      "name": "type-detect",
      "path": "{CWD}/test/fixtures/yargs/node_modules/type-detect",
      "resolved": "https://registry.npmjs.org/type-detect/-/type-detect-4.0.8.tgz",
      "version": "4.0.8",
    },
    "type-fest" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-escapes",
          "name": "type-fest",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/yargs/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.11.0.tgz",
      "version": "0.11.0",
    },
    "typedarray-to-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/yargs/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "typescript",
          "spec": "^4.0.2",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/rollup-plugin-ts",
          "name": "typescript",
          "spec": ">=3.2.x || >= 4.x",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@wessberg/ts-clone-node",
          "name": "typescript",
          "spec": "^3.x || ^4.x",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "typescript",
          "spec": ">=4",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tsutils",
          "name": "typescript",
          "spec": ">=2.8.0 || >= 3.2.0-dev || >= 3.3.0-dev || >= 3.4.0-dev || >= 3.5.0-dev || >= 3.6.0-dev || >= 3.6.0-beta || >= 3.7.0-dev || >= 3.7.0-beta",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.0.3.tgz",
      "version": "4.0.3",
    },
    "ua-parser-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@wessberg/browserslist-generator",
          "name": "ua-parser-js",
          "spec": "^0.7.22",
          "type": "prod",
        },
      },
      "location": "node_modules/ua-parser-js",
      "name": "ua-parser-js",
      "path": "{CWD}/test/fixtures/yargs/node_modules/ua-parser-js",
      "resolved": "https://registry.npmjs.org/ua-parser-js/-/ua-parser-js-0.7.22.tgz",
      "version": "0.7.22",
    },
    "unicode-canonical-property-names-ecmascript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/unicode-match-property-ecmascript",
          "name": "unicode-canonical-property-names-ecmascript",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-canonical-property-names-ecmascript",
      "name": "unicode-canonical-property-names-ecmascript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/unicode-canonical-property-names-ecmascript",
      "resolved": "https://registry.npmjs.org/unicode-canonical-property-names-ecmascript/-/unicode-canonical-property-names-ecmascript-1.0.4.tgz",
      "version": "1.0.4",
    },
    "unicode-match-property-ecmascript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "unicode-match-property-ecmascript",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "unicode-canonical-property-names-ecmascript" => EdgeOut {
          "name": "unicode-canonical-property-names-ecmascript",
          "spec": "^1.0.4",
          "to": "node_modules/unicode-canonical-property-names-ecmascript",
          "type": "prod",
        },
        "unicode-property-aliases-ecmascript" => EdgeOut {
          "name": "unicode-property-aliases-ecmascript",
          "spec": "^1.0.4",
          "to": "node_modules/unicode-property-aliases-ecmascript",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-match-property-ecmascript",
      "name": "unicode-match-property-ecmascript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/unicode-match-property-ecmascript",
      "resolved": "https://registry.npmjs.org/unicode-match-property-ecmascript/-/unicode-match-property-ecmascript-1.0.4.tgz",
      "version": "1.0.4",
    },
    "unicode-match-property-value-ecmascript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regexpu-core",
          "name": "unicode-match-property-value-ecmascript",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-match-property-value-ecmascript",
      "name": "unicode-match-property-value-ecmascript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/unicode-match-property-value-ecmascript",
      "resolved": "https://registry.npmjs.org/unicode-match-property-value-ecmascript/-/unicode-match-property-value-ecmascript-1.2.0.tgz",
      "version": "1.2.0",
    },
    "unicode-property-aliases-ecmascript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/unicode-match-property-ecmascript",
          "name": "unicode-property-aliases-ecmascript",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-property-aliases-ecmascript",
      "name": "unicode-property-aliases-ecmascript",
      "path": "{CWD}/test/fixtures/yargs/node_modules/unicode-property-aliases-ecmascript",
      "resolved": "https://registry.npmjs.org/unicode-property-aliases-ecmascript/-/unicode-property-aliases-ecmascript-1.1.0.tgz",
      "version": "1.1.0",
    },
    "uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/uniq",
      "name": "uniq",
      "path": "{CWD}/test/fixtures/yargs/node_modules/uniq",
      "resolved": "https://registry.npmjs.org/uniq/-/uniq-1.0.1.tgz",
      "version": "1.0.1",
    },
    "unique-string" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "unique-string",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "crypto-random-string" => EdgeOut {
          "name": "crypto-random-string",
          "spec": "^2.0.0",
          "to": "node_modules/crypto-random-string",
          "type": "prod",
        },
      },
      "location": "node_modules/unique-string",
      "name": "unique-string",
      "path": "{CWD}/test/fixtures/yargs/node_modules/unique-string",
      "resolved": "https://registry.npmjs.org/unique-string/-/unique-string-2.0.0.tgz",
      "version": "2.0.0",
    },
    "update-notifier" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/update-notifier",
              "name": "semver",
              "spec": "^7.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/update-notifier/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/yargs/node_modules/update-notifier/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
          "version": "7.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/gts",
          "name": "update-notifier",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boxen" => EdgeOut {
          "name": "boxen",
          "spec": "^4.2.0",
          "to": "node_modules/boxen",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^4.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "configstore" => EdgeOut {
          "name": "configstore",
          "spec": "^5.0.1",
          "to": "node_modules/configstore",
          "type": "prod",
        },
        "has-yarn" => EdgeOut {
          "name": "has-yarn",
          "spec": "^2.1.0",
          "to": "node_modules/has-yarn",
          "type": "prod",
        },
        "import-lazy" => EdgeOut {
          "name": "import-lazy",
          "spec": "^2.1.0",
          "to": "node_modules/import-lazy",
          "type": "prod",
        },
        "is-ci" => EdgeOut {
          "name": "is-ci",
          "spec": "^2.0.0",
          "to": "node_modules/is-ci",
          "type": "prod",
        },
        "is-installed-globally" => EdgeOut {
          "name": "is-installed-globally",
          "spec": "^0.3.1",
          "to": "node_modules/is-installed-globally",
          "type": "prod",
        },
        "is-npm" => EdgeOut {
          "name": "is-npm",
          "spec": "^5.0.0",
          "to": "node_modules/is-npm",
          "type": "prod",
        },
        "is-yarn-global" => EdgeOut {
          "name": "is-yarn-global",
          "spec": "^0.3.0",
          "to": "node_modules/is-yarn-global",
          "type": "prod",
        },
        "latest-version" => EdgeOut {
          "name": "latest-version",
          "spec": "^5.0.0",
          "to": "node_modules/latest-version",
          "type": "prod",
        },
        "pupa" => EdgeOut {
          "name": "pupa",
          "spec": "^2.0.1",
          "to": "node_modules/pupa",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^7.3.2",
          "to": "node_modules/update-notifier/node_modules/semver",
          "type": "prod",
        },
        "semver-diff" => EdgeOut {
          "name": "semver-diff",
          "spec": "^3.1.1",
          "to": "node_modules/semver-diff",
          "type": "prod",
        },
        "xdg-basedir" => EdgeOut {
          "name": "xdg-basedir",
          "spec": "^4.0.0",
          "to": "node_modules/xdg-basedir",
          "type": "prod",
        },
      },
      "location": "node_modules/update-notifier",
      "name": "update-notifier",
      "path": "{CWD}/test/fixtures/yargs/node_modules/update-notifier",
      "resolved": "https://registry.npmjs.org/update-notifier/-/update-notifier-5.0.0.tgz",
      "version": "5.0.0",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/yargs/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "url-parse-lax" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/got",
          "name": "url-parse-lax",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prepend-http" => EdgeOut {
          "name": "prepend-http",
          "spec": "^2.0.0",
          "to": "node_modules/prepend-http",
          "type": "prod",
        },
      },
      "location": "node_modules/url-parse-lax",
      "name": "url-parse-lax",
      "path": "{CWD}/test/fixtures/yargs/node_modules/url-parse-lax",
      "resolved": "https://registry.npmjs.org/url-parse-lax/-/url-parse-lax-3.0.0.tgz",
      "version": "3.0.0",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/yargs/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "v8-compile-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "v8-compile-cache",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/eslint",
          "name": "v8-compile-cache",
          "spec": "^2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/v8-compile-cache",
      "name": "v8-compile-cache",
      "path": "{CWD}/test/fixtures/yargs/node_modules/v8-compile-cache",
      "resolved": "https://registry.npmjs.org/v8-compile-cache/-/v8-compile-cache-2.1.1.tgz",
      "version": "2.1.1",
    },
    "v8-to-istanbul" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/v8-to-istanbul",
              "name": "source-map",
              "spec": "^0.7.3",
              "type": "prod",
            },
          },
          "location": "node_modules/v8-to-istanbul/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/yargs/node_modules/v8-to-istanbul/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.7.3.tgz",
          "version": "0.7.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "v8-to-istanbul",
          "spec": "^6.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/istanbul-lib-coverage" => EdgeOut {
          "name": "@types/istanbul-lib-coverage",
          "spec": "^2.0.1",
          "to": "node_modules/@types/istanbul-lib-coverage",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.7.3",
          "to": "node_modules/v8-to-istanbul/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/v8-to-istanbul",
      "name": "v8-to-istanbul",
      "path": "{CWD}/test/fixtures/yargs/node_modules/v8-to-istanbul",
      "resolved": "https://registry.npmjs.org/v8-to-istanbul/-/v8-to-istanbul-6.0.1.tgz",
      "version": "6.0.1",
    },
    "validate-npm-package-license" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/fixtures/yargs/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/yargs/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "which",
          "spec": "^2.0.0",
          "type": "dev",
        },
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "which",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "which",
          "spec": "2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/yargs/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha/node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/yargs/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "wide-align" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wide-align/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wide-align/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/wide-align/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wide-align/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wide-align/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/wide-align/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wide-align",
              "name": "string-width",
              "spec": "^1.0.2 || 2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/wide-align/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "to": "node_modules/wide-align/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/wide-align/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/yargs/node_modules/wide-align/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wide-align/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "to": "node_modules/wide-align/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wide-align/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/wide-align/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "wide-align",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.2 || 2",
          "to": "node_modules/wide-align/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/wide-align",
      "name": "wide-align",
      "path": "{CWD}/test/fixtures/yargs/node_modules/wide-align",
      "resolved": "https://registry.npmjs.org/wide-align/-/wide-align-1.1.3.tgz",
      "version": "1.1.3",
    },
    "widest-line" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boxen",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/fixtures/yargs/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "word-wrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optionator",
          "name": "word-wrap",
          "spec": "^1.2.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard/node_modules/optionator",
          "name": "word-wrap",
          "spec": "~1.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/word-wrap",
      "name": "word-wrap",
      "path": "{CWD}/test/fixtures/yargs/node_modules/word-wrap",
      "resolved": "https://registry.npmjs.org/word-wrap/-/word-wrap-1.2.3.tgz",
      "version": "1.2.3",
    },
    "workerpool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "workerpool",
          "spec": "6.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/workerpool",
      "name": "workerpool",
      "path": "{CWD}/test/fixtures/yargs/node_modules/workerpool",
      "resolved": "https://registry.npmjs.org/workerpool/-/workerpool-6.0.0.tgz",
      "version": "6.0.0",
    },
    "wrap-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "wrap-ansi",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/yargs/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-7.0.0.tgz",
      "version": "7.0.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/yargs/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "write",
          "spec": "1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/write",
      "name": "write",
      "path": "{CWD}/test/fixtures/yargs/node_modules/write",
      "resolved": "https://registry.npmjs.org/write/-/write-1.0.3.tgz",
      "version": "1.0.3",
    },
    "write-file-atomic" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "write-file-atomic",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/gts",
          "name": "write-file-atomic",
          "spec": "^3.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/yargs/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.3.tgz",
      "version": "3.0.3",
    },
    "xdg-basedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/configstore",
          "name": "xdg-basedir",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/update-notifier",
          "name": "xdg-basedir",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/xdg-basedir",
      "name": "xdg-basedir",
      "path": "{CWD}/test/fixtures/yargs/node_modules/xdg-basedir",
      "resolved": "https://registry.npmjs.org/xdg-basedir/-/xdg-basedir-4.0.0.tgz",
      "version": "4.0.0",
    },
    "xtend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "xtend",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/xtend",
      "name": "xtend",
      "path": "{CWD}/test/fixtures/yargs/node_modules/xtend",
      "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.2.tgz",
      "version": "4.0.2",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "y18n",
          "spec": "^5.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^5.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/yargs/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-5.0.3.tgz",
      "version": "5.0.3",
    },
    "yargs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/c8",
          "name": "yargs",
          "spec": "^16.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^7.0.2",
          "to": "node_modules/cliui",
          "type": "prod",
        },
        "escalade" => EdgeOut {
          "name": "escalade",
          "spec": "^3.1.1",
          "to": "node_modules/escalade",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.5",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^5.0.2",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^20.2.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/yargs/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-16.1.0.tgz",
      "version": "16.1.0",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "yargs-parser",
          "spec": "^20.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/c8",
          "name": "yargs-parser",
          "spec": "^20.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^20.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-20.2.2.tgz",
      "version": "20.2.2",
    },
    "yargs-test-extends" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "yargs-test-extends",
          "spec": "^1.0.1",
          "type": "dev",
        },
      },
      "location": "node_modules/yargs-test-extends",
      "name": "yargs-test-extends",
      "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-test-extends",
      "resolved": "https://registry.npmjs.org/yargs-test-extends/-/yargs-test-extends-1.0.1.tgz",
      "version": "1.0.1",
    },
    "yargs-unparser" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/yargs-unparser/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs-unparser/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs-unparser/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs-unparser/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/yargs-unparser/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "find-up" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/yargs",
              "name": "find-up",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "locate-path" => EdgeOut {
              "name": "locate-path",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/locate-path",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/find-up",
          "name": "find-up",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/find-up",
          "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "locate-path" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/find-up",
              "name": "locate-path",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-locate" => EdgeOut {
              "name": "p-locate",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/p-locate",
              "type": "prod",
            },
            "path-exists" => EdgeOut {
              "name": "path-exists",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/path-exists",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/locate-path",
          "name": "locate-path",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/locate-path",
          "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
          "version": "3.0.0",
        },
        "p-limit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/p-locate",
              "name": "p-limit",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-try" => EdgeOut {
              "name": "p-try",
              "spec": "^2.0.0",
              "to": "node_modules/p-try",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/p-limit",
          "name": "p-limit",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/p-limit",
          "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
          "version": "2.3.0",
        },
        "p-locate" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/locate-path",
              "name": "p-locate",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "p-limit" => EdgeOut {
              "name": "p-limit",
              "spec": "^2.0.0",
              "to": "node_modules/yargs-unparser/node_modules/p-limit",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/p-locate",
          "name": "p-locate",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/p-locate",
          "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
          "version": "3.0.0",
        },
        "path-exists" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/locate-path",
              "name": "path-exists",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/path-exists",
          "name": "path-exists",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/path-exists",
          "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
          "version": "3.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs-unparser/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/yargs-unparser/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs-unparser/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs-unparser/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/yargs-unparser/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs-unparser/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
        "y18n" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/yargs",
              "name": "y18n",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/y18n",
          "name": "y18n",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/y18n",
          "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
          "version": "4.0.0",
        },
        "yargs" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser",
              "name": "yargs",
              "spec": "^14.2.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cliui" => EdgeOut {
              "name": "cliui",
              "spec": "^5.0.0",
              "to": "node_modules/yargs-unparser/node_modules/cliui",
              "type": "prod",
            },
            "decamelize" => EdgeOut {
              "name": "decamelize",
              "spec": "^1.2.0",
              "to": "node_modules/decamelize",
              "type": "prod",
            },
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/find-up",
              "type": "prod",
            },
            "get-caller-file" => EdgeOut {
              "name": "get-caller-file",
              "spec": "^2.0.1",
              "to": "node_modules/get-caller-file",
              "type": "prod",
            },
            "require-directory" => EdgeOut {
              "name": "require-directory",
              "spec": "^2.1.1",
              "to": "node_modules/require-directory",
              "type": "prod",
            },
            "require-main-filename" => EdgeOut {
              "name": "require-main-filename",
              "spec": "^2.0.0",
              "to": "node_modules/require-main-filename",
              "type": "prod",
            },
            "set-blocking" => EdgeOut {
              "name": "set-blocking",
              "spec": "^2.0.0",
              "to": "node_modules/set-blocking",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs-unparser/node_modules/string-width",
              "type": "prod",
            },
            "which-module" => EdgeOut {
              "name": "which-module",
              "spec": "^2.0.0",
              "to": "node_modules/which-module",
              "type": "prod",
            },
            "y18n" => EdgeOut {
              "name": "y18n",
              "spec": "^4.0.0",
              "to": "node_modules/yargs-unparser/node_modules/y18n",
              "type": "prod",
            },
            "yargs-parser" => EdgeOut {
              "name": "yargs-parser",
              "spec": "^15.0.1",
              "to": "node_modules/yargs-unparser/node_modules/yargs-parser",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/yargs",
          "name": "yargs",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/yargs",
          "resolved": "https://registry.npmjs.org/yargs/-/yargs-14.2.3.tgz",
          "version": "14.2.3",
        },
        "yargs-parser" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs-unparser/node_modules/yargs",
              "name": "yargs-parser",
              "spec": "^15.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "camelcase" => EdgeOut {
              "name": "camelcase",
              "spec": "^5.0.0",
              "to": "node_modules/camelcase",
              "type": "prod",
            },
            "decamelize" => EdgeOut {
              "name": "decamelize",
              "spec": "^1.2.0",
              "to": "node_modules/decamelize",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs-unparser/node_modules/yargs-parser",
          "name": "yargs-parser",
          "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser/node_modules/yargs-parser",
          "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-15.0.1.tgz",
          "version": "15.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mocha",
          "name": "yargs-unparser",
          "spec": "1.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.3.1",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
        "flat" => EdgeOut {
          "name": "flat",
          "spec": "^4.1.0",
          "to": "node_modules/flat",
          "type": "prod",
        },
        "is-plain-obj" => EdgeOut {
          "name": "is-plain-obj",
          "spec": "^1.1.0",
          "to": "node_modules/is-plain-obj",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^14.2.3",
          "to": "node_modules/yargs-unparser/node_modules/yargs",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-unparser",
      "name": "yargs-unparser",
      "path": "{CWD}/test/fixtures/yargs/node_modules/yargs-unparser",
      "resolved": "https://registry.npmjs.org/yargs-unparser/-/yargs-unparser-1.6.1.tgz",
      "version": "1.6.1",
    },
  },
  "edgesOut": Map {
    "@types/chai" => EdgeOut {
      "name": "@types/chai",
      "spec": "^4.2.11",
      "to": "node_modules/@types/chai",
      "type": "dev",
    },
    "@types/mocha" => EdgeOut {
      "name": "@types/mocha",
      "spec": "^8.0.0",
      "to": "node_modules/@types/mocha",
      "type": "dev",
    },
    "@types/node" => EdgeOut {
      "name": "@types/node",
      "spec": "^14.11.2",
      "to": "node_modules/@types/node",
      "type": "dev",
    },
    "@wessberg/rollup-plugin-ts" => EdgeOut {
      "name": "@wessberg/rollup-plugin-ts",
      "spec": "^1.3.2",
      "to": "node_modules/@wessberg/rollup-plugin-ts",
      "type": "dev",
    },
    "c8" => EdgeOut {
      "name": "c8",
      "spec": "^7.0.0",
      "to": "node_modules/c8",
      "type": "dev",
    },
    "chai" => EdgeOut {
      "name": "chai",
      "spec": "^4.2.0",
      "to": "node_modules/chai",
      "type": "dev",
    },
    "chalk" => EdgeOut {
      "name": "chalk",
      "spec": "^4.0.0",
      "to": "node_modules/chalk",
      "type": "dev",
    },
    "cliui" => EdgeOut {
      "name": "cliui",
      "spec": "^7.0.2",
      "to": "node_modules/cliui",
      "type": "prod",
    },
    "coveralls" => EdgeOut {
      "name": "coveralls",
      "spec": "^3.0.9",
      "to": "node_modules/coveralls",
      "type": "dev",
    },
    "cpr" => EdgeOut {
      "name": "cpr",
      "spec": "^3.0.1",
      "to": "node_modules/cpr",
      "type": "dev",
    },
    "cross-env" => EdgeOut {
      "name": "cross-env",
      "spec": "^7.0.2",
      "to": "node_modules/cross-env",
      "type": "dev",
    },
    "cross-spawn" => EdgeOut {
      "name": "cross-spawn",
      "spec": "^7.0.0",
      "to": "node_modules/cross-spawn",
      "type": "dev",
    },
    "escalade" => EdgeOut {
      "name": "escalade",
      "spec": "^3.1.1",
      "to": "node_modules/escalade",
      "type": "prod",
    },
    "get-caller-file" => EdgeOut {
      "name": "get-caller-file",
      "spec": "^2.0.5",
      "to": "node_modules/get-caller-file",
      "type": "prod",
    },
    "gts" => EdgeOut {
      "name": "gts",
      "spec": "^3.0.0",
      "to": "node_modules/gts",
      "type": "dev",
    },
    "hashish" => EdgeOut {
      "name": "hashish",
      "spec": "0.0.4",
      "to": "node_modules/hashish",
      "type": "dev",
    },
    "mocha" => EdgeOut {
      "name": "mocha",
      "spec": "^8.0.0",
      "to": "node_modules/mocha",
      "type": "dev",
    },
    "require-directory" => EdgeOut {
      "name": "require-directory",
      "spec": "^2.1.1",
      "to": "node_modules/require-directory",
      "type": "prod",
    },
    "rimraf" => EdgeOut {
      "name": "rimraf",
      "spec": "^3.0.2",
      "to": "node_modules/rimraf",
      "type": "dev",
    },
    "rollup" => EdgeOut {
      "name": "rollup",
      "spec": "^2.23.0",
      "to": "node_modules/rollup",
      "type": "dev",
    },
    "rollup-plugin-cleanup" => EdgeOut {
      "name": "rollup-plugin-cleanup",
      "spec": "^3.1.1",
      "to": "node_modules/rollup-plugin-cleanup",
      "type": "dev",
    },
    "standardx" => EdgeOut {
      "name": "standardx",
      "spec": "^5.0.0",
      "to": "node_modules/standardx",
      "type": "dev",
    },
    "string-width" => EdgeOut {
      "name": "string-width",
      "spec": "^4.2.0",
      "to": "node_modules/string-width",
      "type": "prod",
    },
    "typescript" => EdgeOut {
      "name": "typescript",
      "spec": "^4.0.2",
      "to": "node_modules/typescript",
      "type": "dev",
    },
    "which" => EdgeOut {
      "name": "which",
      "spec": "^2.0.0",
      "to": "node_modules/which",
      "type": "dev",
    },
    "y18n" => EdgeOut {
      "name": "y18n",
      "spec": "^5.0.2",
      "to": "node_modules/y18n",
      "type": "prod",
    },
    "yargs-parser" => EdgeOut {
      "name": "yargs-parser",
      "spec": "^20.2.2",
      "to": "node_modules/yargs-parser",
      "type": "prod",
    },
    "yargs-test-extends" => EdgeOut {
      "name": "yargs-test-extends",
      "spec": "^1.0.1",
      "to": "node_modules/yargs-test-extends",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "yargs",
  "path": "{CWD}/test/fixtures/yargs",
  "version": "16.1.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 1 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 1 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 1 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 2 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication via add, order 2 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-via-add-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 1 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 1 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 1 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 2 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing duplication, order 2 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 1 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-ii" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-ii/-/testing-peer-dep-conflict-chain-ii-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-ii" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 1 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-ii" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-ii/-/testing-peer-dep-conflict-chain-ii-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-ii" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 1 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-ii" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-ii/-/testing-peer-dep-conflict-chain-ii-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-ii" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-ii",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-ii",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 2 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts collision forcing metadep duplication, order 2 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts conflict on root edge, order 1 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "file:.",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1",
      "resolved": "file:../..",
      "target": ArboristNode {
        "location": "",
      },
      "version": "2.3.4",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "file:.",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-a",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-1",
  "version": "2.3.4",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts conflict on root edge, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "file:.",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2",
      "resolved": "file:../..",
      "target": ArboristNode {
        "location": "",
      },
      "version": "2.3.4",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "spec": "file:.",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-d",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-conflict-on-root-edge-order-2",
  "version": "2.3.4",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts dep indirectly on conflicted peer > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-w",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-w" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-w",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-w",
      "name": "@isaacs/testing-peer-dep-conflict-chain-w",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-w",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-w/-/testing-peer-dep-conflict-chain-w-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-w" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-w",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-w",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-indirectly-on-conflicted-peer",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts dep with conflicting peers > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-p",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-p/-/testing-peer-dep-conflict-chain-p-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-dep-with-conflicting-peers",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 1 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 1 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 1 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "1",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-1",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 2 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts direct collision forcing metadep duplication, order 2 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-j" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
              "name": "@isaacs/testing-peer-dep-conflict-chain-j",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-j/-/testing-peer-dep-conflict-chain-j-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-j",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj/node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-jj/-/testing-peer-dep-conflict-chain-jj-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-j" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-j",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-j",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-jj" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-jj",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-jj",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-direct-collision-forcing-metadep-duplication-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep conflict that warns because source is target > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "peerConflicted": true,
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-p/-/testing-peer-dep-conflict-chain-p-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep conflict that warns because source is target > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-i" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-i",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-i/-/testing-peer-dep-conflict-chain-i-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "peerConflicted": true,
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target/node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-p/-/testing-peer-dep-conflict-chain-p-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-i" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-i",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-i",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-that-warns-because-source-is-target",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep conflict that warns because source is target > warnings 1`] = `
Array []
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep conflict triggering the peerConflict code path > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep conflict triggering the peerConflict code path, order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-conflict-triggering-the-peerConflict-code-path-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-4.0.0.tgz",
      "version": "4.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-4.0.0.tgz",
      "version": "4.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-4.0.0.tgz",
      "version": "4.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable order 2 > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-4.0.0.tgz",
      "version": "4.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable order 2 > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-4.0.0.tgz",
      "version": "4.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadep without conflicts, partly overlapping peerSets, resolvable order 2 > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-y",
          "spec": "4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1 || 2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2/node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-y/-/testing-peer-dep-conflict-chain-y-4.0.0.tgz",
      "version": "4.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-y" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-y",
      "spec": "4",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadep-without-conflicts-partly-overlapping-peerSets-resolvable-order-2",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadeps with conflicting peers > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "peerConflicted": true,
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-p/-/testing-peer-dep-conflict-chain-p-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadeps with conflicting peers > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-p" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "peerConflicted": true,
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-p",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers/node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-p/-/testing-peer-dep-conflict-chain-p-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-p" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-p",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-p",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-metadeps-with-conflicting-peers",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts metadeps with conflicting peers > warnings 1`] = `
Array []
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts prod dep directly on conflicted peer, full peer set, newer > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-newer",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts prod dep directly on conflicted peer, full peer set, older > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-full-peer-set-older",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts prod dep directly on conflicted peer, meta peer set, older > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-l",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-l" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-l",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-l",
      "name": "@isaacs/testing-peer-dep-conflict-chain-l",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-l",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-l/-/testing-peer-dep-conflict-chain-l-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-m" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-m",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "name": "@isaacs/testing-peer-dep-conflict-chain-m",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-m/-/testing-peer-dep-conflict-chain-m-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-l" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-l",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-l",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-m" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-m",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-meta-peer-set-older",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts prod dep directly on conflicted peer, newer > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-newer",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts prod dep directly on conflicted peer, older > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "peerConflicted": true,
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-prod-dep-directly-on-conflicted-peer-older",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts this is fine > default result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-x",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-x/-/testing-peer-dep-conflict-chain-x-3.0.0.tgz",
      "version": "3.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-x" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "spec": "3",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts this is fine > force result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-x",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-x/-/testing-peer-dep-conflict-chain-x-3.0.0.tgz",
      "version": "3.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-x" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "spec": "3",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
}
`

exports[`test/arborist/build-ideal-tree.js TAP more peer dep conflicts this is fine > strict result 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-x" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-x",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine/node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-x/-/testing-peer-dep-conflict-chain-x-3.0.0.tgz",
      "version": "3.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-x" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-x",
      "spec": "3",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-x",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-more-peer-dep-conflicts-this-is-fine",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested > nested peer deps cycle 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested > upgrade a 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested > upgrade b 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested > upgrade c, forcibly 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested/node_modules/@isaacs/peer-dep-cycle-c",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-c" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-c",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-c",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested-with-sw > nested peer deps cycle 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested-with-sw",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested-with-sw > upgrade a 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested-with-sw",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested-with-sw > upgrade b 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-a" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-a",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-a",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-b" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-b",
      "spec": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "to": "node_modules/@isaacs/peer-dep-cycle-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested-with-sw",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP nested cyclical peer deps peer-dep-cycle-nested-with-sw > upgrade c, forcibly 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/peer-dep-cycle" => ArboristNode {
      "children": Map {
        "@isaacs/peer-dep-cycle-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-b" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-a",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "name": "@isaacs/peer-dep-cycle-b",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-c" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-b",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/peer-dep-cycle-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-b",
              "name": "@isaacs/peer-dep-cycle-c",
              "spec": "1",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/peer-dep-cycle-a" => EdgeOut {
              "name": "@isaacs/peer-dep-cycle-a",
              "spec": "1",
              "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-c",
          "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "1",
          "to": "node_modules/@isaacs/peer-dep-cycle/node_modules/@isaacs/peer-dep-cycle-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle",
      "name": "@isaacs/peer-dep-cycle",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle/-/peer-dep-cycle-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/peer-dep-cycle-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-c",
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-b" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-a",
      "name": "@isaacs/peer-dep-cycle-a",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-a/-/peer-dep-cycle-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-a",
          "name": "@isaacs/peer-dep-cycle-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-c" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-b",
      "name": "@isaacs/peer-dep-cycle-b",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-b/-/peer-dep-cycle-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/peer-dep-cycle-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/peer-dep-cycle-b",
          "name": "@isaacs/peer-dep-cycle-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/peer-dep-cycle-a" => EdgeOut {
          "name": "@isaacs/peer-dep-cycle-a",
          "spec": "2",
          "to": "node_modules/@isaacs/peer-dep-cycle-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/peer-dep-cycle-c",
      "name": "@isaacs/peer-dep-cycle-c",
      "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw/node_modules/@isaacs/peer-dep-cycle-c",
      "resolved": "https://registry.npmjs.org/@isaacs/peer-dep-cycle-c/-/peer-dep-cycle-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/peer-dep-cycle" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle",
      "spec": "1",
      "to": "node_modules/@isaacs/peer-dep-cycle",
      "type": "prod",
    },
    "@isaacs/peer-dep-cycle-c" => EdgeOut {
      "name": "@isaacs/peer-dep-cycle-c",
      "spec": "2.x",
      "to": "node_modules/@isaacs/peer-dep-cycle-c",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-dep-cycle-nested-with-sw",
  "packageName": "@isaacs/peer-dep-nested",
  "path": "{CWD}/test/fixtures/peer-dep-cycle-nested-with-sw",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP no fix available > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/audit-mkdirp/mkdirp-unfixable/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/audit-mkdirp/mkdirp-unfixable/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "0.5.1",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "mkdirp-unfixable",
  "path": "{CWD}/test/fixtures/audit-mkdirp/mkdirp-unfixable",
}
`

exports[`test/arborist/build-ideal-tree.js TAP no fix available, linked top package > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/audit-mkdirp/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "mkdirp-unfixable",
          "name": "mkdirp",
          "spec": "0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/audit-mkdirp/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "mkdirp-unfixable" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp-unfixable",
          "spec": "file:mkdirp-unfixable",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp-unfixable",
      "name": "mkdirp-unfixable",
      "path": "{CWD}/test/fixtures/audit-mkdirp/node_modules/mkdirp-unfixable",
      "realpath": "{CWD}/test/fixtures/audit-mkdirp/mkdirp-unfixable",
      "resolved": "file:../mkdirp-unfixable",
      "target": ArboristNode {
        "location": "mkdirp-unfixable",
      },
    },
  },
  "edgesOut": Map {
    "mkdirp-unfixable" => EdgeOut {
      "name": "mkdirp-unfixable",
      "spec": "file:mkdirp-unfixable",
      "to": "node_modules/mkdirp-unfixable",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "mkdirp-unfixable",
      "name": "mkdirp-unfixable",
      "path": "{CWD}/test/fixtures/audit-mkdirp/mkdirp-unfixable",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "audit-mkdirp",
  "path": "{CWD}/test/fixtures/audit-mkdirp",
}
`

exports[`test/arborist/build-ideal-tree.js TAP no old lockfile warning if we write back v1 > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/old-package-lock/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/old-package-lock/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "once" => EdgeOut {
      "name": "once",
      "spec": "^1.4.0",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "old-package-lock",
  "path": "{CWD}/test/fixtures/old-package-lock",
}
`

exports[`test/arborist/build-ideal-tree.js TAP optional dependency failures > optional-dep-enotarget 1`] = `
ArboristNode {
  "edgesOut": Map {
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "9999.0000.9999",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "optional-dep-enotarget",
  "packageName": "@isaacs/optional-dep-enotarget",
  "path": "{CWD}/test/fixtures/optional-dep-enotarget",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP optional dependency failures > optional-dep-missing 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/this-does-not-exist-at-all" => EdgeOut {
      "name": "@isaacs/this-does-not-exist-at-all",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "optional-dep-missing",
  "packageName": "@isaacs/testing-optional-dep-metadata-missing",
  "path": "{CWD}/test/fixtures/optional-dep-missing",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP optional dependency failures > optional-metadep-enotarget 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/prod-dep-enotarget" => EdgeOut {
      "name": "@isaacs/prod-dep-enotarget",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "optional-metadep-enotarget",
  "path": "{CWD}/test/fixtures/optional-metadep-enotarget",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP optional dependency failures > optional-metadep-missing 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-metadata-missing" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-metadata-missing",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "optional-metadep-missing",
  "packageName": "@isaacs/testing-optional-metadep-metadata-missing",
  "path": "{CWD}/test/fixtures/optional-metadep-missing",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP optional dependency failures > optional-ok 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "optional",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "optional": true,
      "path": "{CWD}/test/fixtures/optional-ok/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "optional-ok",
  "path": "{CWD}/test/fixtures/optional-ok",
}
`

exports[`test/arborist/build-ideal-tree.js TAP override a conflict with the root dep (with force) > non-strict and force override 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "override",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-override",
  "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP override a conflict with the root dep (with force) > strict and force override 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "override",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-override",
  "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP override a conflict with the root peer dep (with force) > non-strict and force override 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "peer",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "override-peer",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-override-peer",
  "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP override a conflict with the root peer dep (with force) > strict and force override 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "error": "INVALID",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "peerConflicted": true,
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "type": "peer",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "override-peer",
  "packageName": "@isaacs/testing-peer-dep-conflict-chain-override-peer",
  "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-peer",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP pathologically nested dependency cycle > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/pathological-dep-nesting-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/pathological-dep-nesting-a",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/pathological-dep-nesting-b" => EdgeOut {
          "name": "@isaacs/pathological-dep-nesting-b",
          "spec": "1",
          "to": "node_modules/@isaacs/pathological-dep-nesting-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/pathological-dep-nesting-a",
      "name": "@isaacs/pathological-dep-nesting-a",
      "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-a",
      "resolved": "https://registry.npmjs.org/@isaacs/pathological-dep-nesting-a/-/pathological-dep-nesting-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/pathological-dep-nesting-b" => ArboristNode {
      "children": Map {
        "@isaacs/pathological-dep-nesting-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/pathological-dep-nesting-b",
              "name": "@isaacs/pathological-dep-nesting-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/pathological-dep-nesting-b" => EdgeOut {
              "name": "@isaacs/pathological-dep-nesting-b",
              "spec": "2",
              "to": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
          "name": "@isaacs/pathological-dep-nesting-a",
          "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
          "resolved": "https://registry.npmjs.org/@isaacs/pathological-dep-nesting-a/-/pathological-dep-nesting-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/pathological-dep-nesting-b" => ArboristNode {
          "children": Map {
            "@isaacs/pathological-dep-nesting-a" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
                  "name": "@isaacs/pathological-dep-nesting-a",
                  "spec": "1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/pathological-dep-nesting-b" => EdgeOut {
                  "name": "@isaacs/pathological-dep-nesting-b",
                  "spec": "1",
                  "to": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
              "name": "@isaacs/pathological-dep-nesting-a",
              "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
              "resolved": "https://registry.npmjs.org/@isaacs/pathological-dep-nesting-a/-/pathological-dep-nesting-a-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/pathological-dep-nesting-b" => ArboristLink {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
                  "name": "@isaacs/pathological-dep-nesting-b",
                  "spec": "1",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
              "name": "@isaacs/pathological-dep-nesting-b",
              "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
              "realpath": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b",
              "resolved": "file:../../../../..",
              "target": ArboristNode {
                "location": "node_modules/@isaacs/pathological-dep-nesting-b",
              },
              "version": "1.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
              "name": "@isaacs/pathological-dep-nesting-b",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/pathological-dep-nesting-a" => EdgeOut {
              "name": "@isaacs/pathological-dep-nesting-a",
              "spec": "1",
              "to": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
          "name": "@isaacs/pathological-dep-nesting-b",
          "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-b",
          "resolved": "https://registry.npmjs.org/@isaacs/pathological-dep-nesting-b/-/pathological-dep-nesting-b-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/pathological-dep-nesting-a",
          "name": "@isaacs/pathological-dep-nesting-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/pathological-dep-nesting-a" => EdgeOut {
          "name": "@isaacs/pathological-dep-nesting-a",
          "spec": "2",
          "to": "node_modules/@isaacs/pathological-dep-nesting-b/node_modules/@isaacs/pathological-dep-nesting-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/pathological-dep-nesting-b",
      "name": "@isaacs/pathological-dep-nesting-b",
      "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle/node_modules/@isaacs/pathological-dep-nesting-b",
      "resolved": "https://registry.npmjs.org/@isaacs/pathological-dep-nesting-b/-/pathological-dep-nesting-b-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/pathological-dep-nesting-a" => EdgeOut {
      "name": "@isaacs/pathological-dep-nesting-a",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/pathological-dep-nesting-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "pathological-dep-nesting-cycle",
  "path": "{CWD}/test/fixtures/pathological-dep-nesting-cycle",
}
`

exports[`test/arborist/build-ideal-tree.js TAP peer conflicts between peer sets in transitive deps get warning, x and j@1 in root, put y and j@3 in a > ERESOLVE explanation 1`] = `
Object {
  "code": "ERESOLVE",
  "current": Object {
    "dependents": Array [
      Object {
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a",
              },
              "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
              "spec": "3",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a",
          "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
          "version": "3.0.0",
        },
        "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
        "spec": "3",
        "type": "prod",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-j",
    "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
    "version": "3.0.0",
  },
  "dep": Object {
    "dependents": Array [
      Object {
        "error": "INVALID",
        "from": Object {
          "dependents": Array [
            Object {
              "from": Object {
                "dependents": Array [
                  Object {
                    "from": Object {
                      "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a",
                    },
                    "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                    "spec": "3",
                    "type": "prod",
                  },
                ],
                "isWorkspace": false,
                "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a",
                "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                "version": "3.0.0",
              },
              "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
              "spec": "",
              "type": "prod",
            },
          ],
          "isWorkspace": false,
          "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-y",
          "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
          "version": "1.0.0",
        },
        "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
        "spec": "2",
        "type": "peer",
      },
    ],
    "isWorkspace": false,
    "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-j",
    "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
    "version": "2.0.0",
    "whileInstalling": Object {
      "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-y",
      "version": "1.0.0",
    },
  },
  "edge": Object {
    "error": "INVALID",
    "from": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a",
                },
                "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                "spec": "3",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a",
            "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
            "version": "3.0.0",
          },
          "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
          "spec": "",
          "type": "prod",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-y",
      "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
      "version": "1.0.0",
    },
    "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
    "spec": "2",
    "type": "peer",
  },
  "force": false,
  "isMine": false,
  "peerConflict": Object {
    "current": Object {
      "dependents": Array [
        Object {
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a",
                },
                "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                "spec": "3",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a",
            "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
            "version": "3.0.0",
          },
          "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
          "spec": "3",
          "type": "prod",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-j",
      "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
      "version": "3.0.0",
    },
    "peer": Object {
      "dependents": Array [
        Object {
          "error": "INVALID",
          "from": Object {
            "dependents": Array [
              Object {
                "from": Object {
                  "dependents": Array [
                    Object {
                      "from": Object {
                        "location": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a",
                      },
                      "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                      "spec": "3",
                      "type": "prod",
                    },
                  ],
                  "isWorkspace": false,
                  "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a",
                  "name": "@isaacs/peer-dep-conflict-infinite-loop-a",
                  "version": "3.0.0",
                },
                "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
                "spec": "",
                "type": "prod",
              },
            ],
            "isWorkspace": false,
            "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-y",
            "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
            "version": "1.0.0",
          },
          "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
          "spec": "2",
          "type": "peer",
        },
      ],
      "isWorkspace": false,
      "location": "node_modules/@isaacs/peer-dep-conflict-infinite-loop-j",
      "name": "@isaacs/peer-dep-conflict-infinite-loop-j",
      "version": "2.0.0",
      "whileInstalling": Object {
        "name": "@isaacs/peer-dep-conflict-infinite-loop-y",
        "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-conflicts-between-peer-sets-in-transitive-deps-get-warning-x-and-j-1-in-root-put-y-and-j-3-in-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-a/node_modules/@isaacs/peer-dep-conflict-infinite-loop-y",
        "version": "1.0.0",
      },
    },
  },
  "strictPeerDeps": false,
}
`

exports[`test/arborist/build-ideal-tree.js TAP peer dep that needs to be replaced > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@pmmmwh/react-refresh-webpack-plugin" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@pmmmwh/react-refresh-webpack-plugin",
          "spec": "^0.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/webpack" => EdgeOut {
          "name": "@types/webpack",
          "spec": "4.x",
          "to": null,
          "type": "peerOptional",
        },
        "ansi-html" => EdgeOut {
          "name": "ansi-html",
          "spec": "^0.0.7",
          "to": "node_modules/ansi-html",
          "type": "prod",
        },
        "error-stack-parser" => EdgeOut {
          "name": "error-stack-parser",
          "spec": "^2.0.6",
          "to": "node_modules/error-stack-parser",
          "type": "prod",
        },
        "html-entities" => EdgeOut {
          "name": "html-entities",
          "spec": "^1.2.1",
          "to": "node_modules/html-entities",
          "type": "prod",
        },
        "native-url" => EdgeOut {
          "name": "native-url",
          "spec": "^0.2.6",
          "to": "node_modules/native-url",
          "type": "prod",
        },
        "react-refresh" => EdgeOut {
          "name": "react-refresh",
          "spec": ">=0.8.3 <0.10.0",
          "to": "node_modules/react-refresh",
          "type": "peer",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^2.6.5",
          "to": "node_modules/schema-utils",
          "type": "prod",
        },
        "sockjs-client" => EdgeOut {
          "name": "sockjs-client",
          "spec": "^1.4.0",
          "to": "node_modules/sockjs-client",
          "type": "peerOptional",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.7.3",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.13.1",
          "to": null,
          "type": "peerOptional",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": ">=4.43.0 <6.0.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
        "webpack-dev-server" => EdgeOut {
          "name": "webpack-dev-server",
          "spec": "3.x",
          "to": "node_modules/webpack-dev-server",
          "type": "peerOptional",
        },
        "webpack-hot-middleware" => EdgeOut {
          "name": "webpack-hot-middleware",
          "spec": "2.x",
          "to": null,
          "type": "peerOptional",
        },
        "webpack-plugin-serve" => EdgeOut {
          "name": "webpack-plugin-serve",
          "spec": "0.x || 1.x",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "name": "@pmmmwh/react-refresh-webpack-plugin",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "resolved": "https://registry.npmjs.org/@pmmmwh/react-refresh-webpack-plugin/-/react-refresh-webpack-plugin-0.4.3.tgz",
      "version": "0.4.3",
    },
    "@types/glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "@types/glob",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/minimatch" => EdgeOut {
          "name": "@types/minimatch",
          "spec": "*",
          "to": "node_modules/@types/minimatch",
          "type": "prod",
        },
        "@types/node" => EdgeOut {
          "name": "@types/node",
          "spec": "*",
          "to": "node_modules/@types/node",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/glob",
      "name": "@types/glob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@types/glob",
      "resolved": "https://registry.npmjs.org/@types/glob/-/glob-7.1.3.tgz",
      "version": "7.1.3",
    },
    "@types/json-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "@types/json-schema",
          "spec": "^7.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/json-schema",
      "name": "@types/json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@types/json-schema",
      "resolved": "https://registry.npmjs.org/@types/json-schema/-/json-schema-7.0.6.tgz",
      "version": "7.0.6",
    },
    "@types/minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/glob",
          "name": "@types/minimatch",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/minimatch",
      "name": "@types/minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@types/minimatch",
      "resolved": "https://registry.npmjs.org/@types/minimatch/-/minimatch-3.0.3.tgz",
      "version": "3.0.3",
    },
    "@types/node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@types/glob",
          "name": "@types/node",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/node",
      "name": "@types/node",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@types/node",
      "resolved": "https://registry.npmjs.org/@types/node/-/node-14.11.8.tgz",
      "version": "14.11.8",
    },
    "@webassemblyjs/ast" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-module-context",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/helper-module-context" => EdgeOut {
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-module-context",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/wast-parser" => EdgeOut {
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/ast",
      "name": "@webassemblyjs/ast",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/ast",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/ast/-/ast-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/floating-point-hex-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/floating-point-hex-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/floating-point-hex-parser",
      "name": "@webassemblyjs/floating-point-hex-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/floating-point-hex-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/floating-point-hex-parser/-/floating-point-hex-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-api-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-api-error",
      "name": "@webassemblyjs/helper-api-error",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-api-error",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-api-error/-/helper-api-error-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-buffer",
      "name": "@webassemblyjs/helper-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-buffer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-buffer/-/helper-buffer-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-code-frame",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/wast-printer" => EdgeOut {
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-printer",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-code-frame",
      "name": "@webassemblyjs/helper-code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-code-frame",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-code-frame/-/helper-code-frame-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-fsm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@webassemblyjs/helper-fsm",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-fsm",
      "name": "@webassemblyjs/helper-fsm",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-fsm",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-fsm/-/helper-fsm-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-module-context" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-module-context",
      "name": "@webassemblyjs/helper-module-context",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-module-context",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-module-context/-/helper-module-context-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-wasm-bytecode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-wasm-bytecode",
      "name": "@webassemblyjs/helper-wasm-bytecode",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-wasm-bytecode",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-wasm-bytecode/-/helper-wasm-bytecode-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/helper-wasm-section" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/helper-wasm-section",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/helper-wasm-section",
      "name": "@webassemblyjs/helper-wasm-section",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/helper-wasm-section",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/helper-wasm-section/-/helper-wasm-section-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/ieee754" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@xtuc/ieee754" => EdgeOut {
          "name": "@xtuc/ieee754",
          "spec": "^1.2.0",
          "to": "node_modules/@xtuc/ieee754",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/ieee754",
      "name": "@webassemblyjs/ieee754",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/ieee754",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/ieee754/-/ieee754-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/leb128" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/leb128",
      "name": "@webassemblyjs/leb128",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/leb128",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/leb128/-/leb128-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/utf8" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-gen",
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-parser",
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/utf8",
      "name": "@webassemblyjs/utf8",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/utf8",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/utf8/-/utf8-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-edit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/wasm-edit",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-section" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-section",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-section",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
        "@webassemblyjs/wasm-opt" => EdgeOut {
          "name": "@webassemblyjs/wasm-opt",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-opt",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
        "@webassemblyjs/wast-printer" => EdgeOut {
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-printer",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-edit",
      "name": "@webassemblyjs/wasm-edit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wasm-edit",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-edit/-/wasm-edit-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-gen" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-wasm-section",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/ieee754" => EdgeOut {
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ieee754",
          "type": "prod",
        },
        "@webassemblyjs/leb128" => EdgeOut {
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/leb128",
          "type": "prod",
        },
        "@webassemblyjs/utf8" => EdgeOut {
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/utf8",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-gen",
      "name": "@webassemblyjs/wasm-gen",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wasm-gen",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-gen/-/wasm-gen-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-opt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-opt",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-buffer" => EdgeOut {
          "name": "@webassemblyjs/helper-buffer",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-buffer",
          "type": "prod",
        },
        "@webassemblyjs/wasm-gen" => EdgeOut {
          "name": "@webassemblyjs/wasm-gen",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-gen",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-opt",
      "name": "@webassemblyjs/wasm-opt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wasm-opt",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-opt/-/wasm-opt-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wasm-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-opt",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-api-error" => EdgeOut {
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-api-error",
          "type": "prod",
        },
        "@webassemblyjs/helper-wasm-bytecode" => EdgeOut {
          "name": "@webassemblyjs/helper-wasm-bytecode",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-wasm-bytecode",
          "type": "prod",
        },
        "@webassemblyjs/ieee754" => EdgeOut {
          "name": "@webassemblyjs/ieee754",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ieee754",
          "type": "prod",
        },
        "@webassemblyjs/leb128" => EdgeOut {
          "name": "@webassemblyjs/leb128",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/leb128",
          "type": "prod",
        },
        "@webassemblyjs/utf8" => EdgeOut {
          "name": "@webassemblyjs/utf8",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/utf8",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wasm-parser",
      "name": "@webassemblyjs/wasm-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wasm-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wasm-parser/-/wasm-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wast-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ast",
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/floating-point-hex-parser" => EdgeOut {
          "name": "@webassemblyjs/floating-point-hex-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/floating-point-hex-parser",
          "type": "prod",
        },
        "@webassemblyjs/helper-api-error" => EdgeOut {
          "name": "@webassemblyjs/helper-api-error",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-api-error",
          "type": "prod",
        },
        "@webassemblyjs/helper-code-frame" => EdgeOut {
          "name": "@webassemblyjs/helper-code-frame",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-code-frame",
          "type": "prod",
        },
        "@webassemblyjs/helper-fsm" => EdgeOut {
          "name": "@webassemblyjs/helper-fsm",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-fsm",
          "type": "prod",
        },
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wast-parser",
      "name": "@webassemblyjs/wast-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wast-parser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wast-parser/-/wast-parser-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@webassemblyjs/wast-printer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/helper-code-frame",
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wasm-edit",
          "name": "@webassemblyjs/wast-printer",
          "spec": "1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/wast-parser" => EdgeOut {
          "name": "@webassemblyjs/wast-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wast-parser",
          "type": "prod",
        },
        "@xtuc/long" => EdgeOut {
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "to": "node_modules/@xtuc/long",
          "type": "prod",
        },
      },
      "location": "node_modules/@webassemblyjs/wast-printer",
      "name": "@webassemblyjs/wast-printer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@webassemblyjs/wast-printer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@webassemblyjs/wast-printer/-/wast-printer-1.9.0.tgz",
      "version": "1.9.0",
    },
    "@xtuc/ieee754" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/ieee754",
          "name": "@xtuc/ieee754",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@xtuc/ieee754",
      "name": "@xtuc/ieee754",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@xtuc/ieee754",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@xtuc/ieee754/-/ieee754-1.2.0.tgz",
      "version": "1.2.0",
    },
    "@xtuc/long" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@webassemblyjs/leb128",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-parser",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@webassemblyjs/wast-printer",
          "name": "@xtuc/long",
          "spec": "4.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@xtuc/long",
      "name": "@xtuc/long",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/@xtuc/long",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@xtuc/long/-/long-4.2.2.tgz",
      "version": "4.2.2",
    },
    "accepts" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compression",
          "name": "accepts",
          "spec": "~1.3.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "accepts",
          "spec": "~1.3.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index",
          "name": "accepts",
          "spec": "~1.3.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.24",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "negotiator" => EdgeOut {
          "name": "negotiator",
          "spec": "0.6.2",
          "to": "node_modules/negotiator",
          "type": "prod",
        },
      },
      "location": "node_modules/accepts",
      "name": "accepts",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/accepts",
      "resolved": "https://registry.npmjs.org/accepts/-/accepts-1.3.7.tgz",
      "version": "1.3.7",
    },
    "acorn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "acorn",
          "spec": "^6.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/acorn",
      "peer": true,
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-6.4.2.tgz",
      "version": "6.4.2",
    },
    "ajv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv-errors",
          "name": "ajv",
          "spec": ">=5.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/ajv-keywords",
          "name": "ajv",
          "spec": "^6.9.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.12.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "ajv",
          "spec": "^6.10.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server/node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "ajv",
          "spec": "^6.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ajv-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "ajv-errors",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server/node_modules/schema-utils",
          "name": "ajv-errors",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "ajv-errors",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": ">=5.0.0",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-errors",
      "name": "ajv-errors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ajv-errors",
      "resolved": "https://registry.npmjs.org/ajv-errors/-/ajv-errors-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ajv-keywords" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "ajv-keywords",
          "spec": "^3.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server/node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack/node_modules/schema-utils",
          "name": "ajv-keywords",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.9.1",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-keywords",
      "name": "ajv-keywords",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ajv-keywords",
      "resolved": "https://registry.npmjs.org/ajv-keywords/-/ajv-keywords-3.5.2.tgz",
      "version": "3.5.2",
    },
    "ansi-colors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-log",
          "name": "ansi-colors",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-colors",
      "name": "ansi-colors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ansi-colors",
      "resolved": "https://registry.npmjs.org/ansi-colors/-/ansi-colors-3.2.4.tgz",
      "version": "3.2.4",
    },
    "ansi-html" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "ansi-html",
          "spec": "^0.0.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "ansi-html",
          "spec": "0.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-html",
      "name": "ansi-html",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ansi-html",
      "resolved": "https://registry.npmjs.org/ansi-html/-/ansi-html-0.0.7.tgz",
      "version": "0.0.7",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
      "version": "2.1.1",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "anymatch" => ArboristNode {
      "children": Map {
        "normalize-path" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/anymatch",
              "name": "normalize-path",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "remove-trailing-separator" => EdgeOut {
              "name": "remove-trailing-separator",
              "spec": "^1.0.1",
              "to": "node_modules/remove-trailing-separator",
              "type": "prod",
            },
          },
          "location": "node_modules/anymatch/node_modules/normalize-path",
          "name": "normalize-path",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/anymatch/node_modules/normalize-path",
          "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-2.1.1.tgz",
          "version": "2.1.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^3.1.4",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^2.1.1",
          "to": "node_modules/anymatch/node_modules/normalize-path",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-2.0.0.tgz",
      "version": "2.0.0",
    },
    "aproba" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "aproba",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "aproba",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/run-queue",
          "name": "aproba",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/aproba",
      "name": "aproba",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/aproba",
      "peer": true,
      "resolved": "https://registry.npmjs.org/aproba/-/aproba-1.2.0.tgz",
      "version": "1.2.0",
    },
    "arr-diff" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "arr-diff",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "arr-diff",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arr-diff",
      "name": "arr-diff",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/arr-diff",
      "resolved": "https://registry.npmjs.org/arr-diff/-/arr-diff-4.0.0.tgz",
      "version": "4.0.0",
    },
    "arr-flatten" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "arr-flatten",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arr-flatten",
      "name": "arr-flatten",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/arr-flatten",
      "resolved": "https://registry.npmjs.org/arr-flatten/-/arr-flatten-1.1.0.tgz",
      "version": "1.1.0",
    },
    "arr-union" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/class-utils",
          "name": "arr-union",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/union-value",
          "name": "arr-union",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arr-union",
      "name": "arr-union",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/arr-union",
      "resolved": "https://registry.npmjs.org/arr-union/-/arr-union-3.1.0.tgz",
      "version": "3.1.0",
    },
    "array-flatten" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "array-flatten",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/array-flatten",
      "name": "array-flatten",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/array-flatten",
      "resolved": "https://registry.npmjs.org/array-flatten/-/array-flatten-2.1.2.tgz",
      "version": "2.1.2",
    },
    "array-union" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-uniq" => EdgeOut {
          "name": "array-uniq",
          "spec": "^1.0.1",
          "to": "node_modules/array-uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-1.0.2.tgz",
      "version": "1.0.2",
    },
    "array-uniq" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-union",
          "name": "array-uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/array-uniq",
      "name": "array-uniq",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/array-uniq",
      "resolved": "https://registry.npmjs.org/array-uniq/-/array-uniq-1.0.3.tgz",
      "version": "1.0.3",
    },
    "array-unique" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "array-unique",
          "spec": "^0.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "array-unique",
          "spec": "^0.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "array-unique",
          "spec": "^0.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "array-unique",
          "spec": "^0.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/array-unique",
      "name": "array-unique",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/array-unique",
      "resolved": "https://registry.npmjs.org/array-unique/-/array-unique-0.3.2.tgz",
      "version": "0.3.2",
    },
    "asn1.js" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/asn1.js",
              "name": "bn.js",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/asn1.js/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/asn1.js/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-asn1",
          "name": "asn1.js",
          "spec": "^5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.0.0",
          "to": "node_modules/asn1.js/node_modules/bn.js",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1.js",
      "name": "asn1.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/asn1.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/asn1.js/-/asn1.js-5.4.1.tgz",
      "version": "5.4.1",
    },
    "assert" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/assert/node_modules/util",
              "name": "inherits",
              "spec": "2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/assert/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/assert/node_modules/inherits",
          "peer": true,
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.1.tgz",
          "version": "2.0.1",
        },
        "util" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/assert",
              "name": "util",
              "spec": "0.10.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2.0.1",
              "to": "node_modules/assert/node_modules/inherits",
              "type": "prod",
            },
          },
          "location": "node_modules/assert/node_modules/util",
          "name": "util",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/assert/node_modules/util",
          "peer": true,
          "resolved": "https://registry.npmjs.org/util/-/util-0.10.3.tgz",
          "version": "0.10.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "assert",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "util" => EdgeOut {
          "name": "util",
          "spec": "0.10.3",
          "to": "node_modules/assert/node_modules/util",
          "type": "prod",
        },
      },
      "location": "node_modules/assert",
      "name": "assert",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/assert",
      "peer": true,
      "resolved": "https://registry.npmjs.org/assert/-/assert-1.5.0.tgz",
      "version": "1.5.0",
    },
    "assign-symbols" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch/node_modules/extend-shallow",
          "name": "assign-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch/node_modules/extend-shallow",
          "name": "assign-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regex-not/node_modules/extend-shallow",
          "name": "assign-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/split-string/node_modules/extend-shallow",
          "name": "assign-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex/node_modules/extend-shallow",
          "name": "assign-symbols",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assign-symbols",
      "name": "assign-symbols",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/assign-symbols",
      "resolved": "https://registry.npmjs.org/assign-symbols/-/assign-symbols-1.0.0.tgz",
      "version": "1.0.0",
    },
    "async" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/portfinder",
          "name": "async",
          "spec": "^2.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.14",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/async",
      "name": "async",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/async",
      "resolved": "https://registry.npmjs.org/async/-/async-2.6.3.tgz",
      "version": "2.6.3",
    },
    "async-each" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "async-each",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/async-each",
      "name": "async-each",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/async-each",
      "resolved": "https://registry.npmjs.org/async-each/-/async-each-1.0.3.tgz",
      "version": "1.0.3",
    },
    "async-limiter" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ws",
          "name": "async-limiter",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/async-limiter",
      "name": "async-limiter",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/async-limiter",
      "resolved": "https://registry.npmjs.org/async-limiter/-/async-limiter-1.0.1.tgz",
      "version": "1.0.1",
    },
    "atob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-resolve",
          "name": "atob",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/atob",
      "name": "atob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/atob",
      "resolved": "https://registry.npmjs.org/atob/-/atob-2.1.2.tgz",
      "version": "2.1.2",
    },
    "balanced-match" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "base" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/base",
              "name": "define-property",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^1.0.0",
              "to": "node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/base/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/base/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/snapdragon",
          "name": "base",
          "spec": "^0.11.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cache-base" => EdgeOut {
          "name": "cache-base",
          "spec": "^1.0.1",
          "to": "node_modules/cache-base",
          "type": "prod",
        },
        "class-utils" => EdgeOut {
          "name": "class-utils",
          "spec": "^0.3.5",
          "to": "node_modules/class-utils",
          "type": "prod",
        },
        "component-emitter" => EdgeOut {
          "name": "component-emitter",
          "spec": "^1.2.1",
          "to": "node_modules/component-emitter",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^1.0.0",
          "to": "node_modules/base/node_modules/define-property",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
        "mixin-deep" => EdgeOut {
          "name": "mixin-deep",
          "spec": "^1.2.0",
          "to": "node_modules/mixin-deep",
          "type": "prod",
        },
        "pascalcase" => EdgeOut {
          "name": "pascalcase",
          "spec": "^0.1.1",
          "to": "node_modules/pascalcase",
          "type": "prod",
        },
      },
      "location": "node_modules/base",
      "name": "base",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/base",
      "resolved": "https://registry.npmjs.org/base/-/base-0.11.2.tgz",
      "version": "0.11.2",
    },
    "base64-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/buffer",
          "name": "base64-js",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/base64-js",
      "name": "base64-js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/base64-js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/base64-js/-/base64-js-1.5.1.tgz",
      "version": "1.5.1",
    },
    "batch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/serve-index",
          "name": "batch",
          "spec": "0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/batch",
      "name": "batch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/batch",
      "resolved": "https://registry.npmjs.org/batch/-/batch-0.6.1.tgz",
      "version": "0.6.1",
    },
    "big.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/loader-utils",
          "name": "big.js",
          "spec": "^5.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/big.js",
      "name": "big.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/big.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/big.js/-/big.js-5.2.2.tgz",
      "version": "5.2.2",
    },
    "binary-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-1.13.1.tgz",
      "version": "1.13.1",
    },
    "bindings" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fsevents",
          "name": "bindings",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "file-uri-to-path" => EdgeOut {
          "name": "file-uri-to-path",
          "spec": "1.0.0",
          "to": "node_modules/file-uri-to-path",
          "type": "prod",
        },
      },
      "location": "node_modules/bindings",
      "name": "bindings",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/bindings",
      "resolved": "https://registry.npmjs.org/bindings/-/bindings-1.5.0.tgz",
      "version": "1.5.0",
    },
    "bluebird" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "bluebird",
          "spec": "^3.5.5",
          "type": "prod",
        },
      },
      "location": "node_modules/bluebird",
      "name": "bluebird",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/bluebird",
      "peer": true,
      "resolved": "https://registry.npmjs.org/bluebird/-/bluebird-3.7.2.tgz",
      "version": "3.7.2",
    },
    "bn.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "bn.js",
          "spec": "^5.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/bn.js",
      "name": "bn.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/bn.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-5.1.3.tgz",
      "version": "5.1.3",
    },
    "body-parser" => ArboristNode {
      "children": Map {
        "bytes" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/body-parser",
              "name": "bytes",
              "spec": "3.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/body-parser/node_modules/bytes",
          "name": "bytes",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/body-parser/node_modules/bytes",
          "resolved": "https://registry.npmjs.org/bytes/-/bytes-3.1.0.tgz",
          "version": "3.1.0",
        },
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/body-parser",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/body-parser/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/body-parser/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/body-parser/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/body-parser/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/body-parser/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/body-parser/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "body-parser",
          "spec": "1.19.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bytes" => EdgeOut {
          "name": "bytes",
          "spec": "3.1.0",
          "to": "node_modules/body-parser/node_modules/bytes",
          "type": "prod",
        },
        "content-type" => EdgeOut {
          "name": "content-type",
          "spec": "~1.0.4",
          "to": "node_modules/content-type",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/body-parser/node_modules/debug",
          "type": "prod",
        },
        "depd" => EdgeOut {
          "name": "depd",
          "spec": "~1.1.2",
          "to": "node_modules/depd",
          "type": "prod",
        },
        "http-errors" => EdgeOut {
          "name": "http-errors",
          "spec": "1.7.2",
          "to": "node_modules/http-errors",
          "type": "prod",
        },
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "0.4.24",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
        "on-finished" => EdgeOut {
          "name": "on-finished",
          "spec": "~2.3.0",
          "to": "node_modules/on-finished",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "6.7.0",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "raw-body" => EdgeOut {
          "name": "raw-body",
          "spec": "2.4.0",
          "to": "node_modules/raw-body",
          "type": "prod",
        },
        "type-is" => EdgeOut {
          "name": "type-is",
          "spec": "~1.6.17",
          "to": "node_modules/type-is",
          "type": "prod",
        },
      },
      "location": "node_modules/body-parser",
      "name": "body-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/body-parser",
      "resolved": "https://registry.npmjs.org/body-parser/-/body-parser-1.19.0.tgz",
      "version": "1.19.0",
    },
    "bonjour" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "bonjour",
          "spec": "^3.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-flatten" => EdgeOut {
          "name": "array-flatten",
          "spec": "^2.1.0",
          "to": "node_modules/array-flatten",
          "type": "prod",
        },
        "deep-equal" => EdgeOut {
          "name": "deep-equal",
          "spec": "^1.0.1",
          "to": "node_modules/deep-equal",
          "type": "prod",
        },
        "dns-equal" => EdgeOut {
          "name": "dns-equal",
          "spec": "^1.0.0",
          "to": "node_modules/dns-equal",
          "type": "prod",
        },
        "dns-txt" => EdgeOut {
          "name": "dns-txt",
          "spec": "^2.0.2",
          "to": "node_modules/dns-txt",
          "type": "prod",
        },
        "multicast-dns" => EdgeOut {
          "name": "multicast-dns",
          "spec": "^6.0.1",
          "to": "node_modules/multicast-dns",
          "type": "prod",
        },
        "multicast-dns-service-types" => EdgeOut {
          "name": "multicast-dns-service-types",
          "spec": "^1.1.0",
          "to": "node_modules/multicast-dns-service-types",
          "type": "prod",
        },
      },
      "location": "node_modules/bonjour",
      "name": "bonjour",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/bonjour",
      "resolved": "https://registry.npmjs.org/bonjour/-/bonjour-3.5.0.tgz",
      "version": "3.5.0",
    },
    "brace-expansion" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "^2.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "braces",
          "spec": "^2.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arr-flatten" => EdgeOut {
          "name": "arr-flatten",
          "spec": "^1.1.0",
          "to": "node_modules/arr-flatten",
          "type": "prod",
        },
        "array-unique" => EdgeOut {
          "name": "array-unique",
          "spec": "^0.3.2",
          "to": "node_modules/array-unique",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^4.0.0",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
        "repeat-element" => EdgeOut {
          "name": "repeat-element",
          "spec": "^1.1.2",
          "to": "node_modules/repeat-element",
          "type": "prod",
        },
        "snapdragon" => EdgeOut {
          "name": "snapdragon",
          "spec": "^0.8.1",
          "to": "node_modules/snapdragon",
          "type": "prod",
        },
        "snapdragon-node" => EdgeOut {
          "name": "snapdragon-node",
          "spec": "^2.0.1",
          "to": "node_modules/snapdragon-node",
          "type": "prod",
        },
        "split-string" => EdgeOut {
          "name": "split-string",
          "spec": "^3.0.2",
          "to": "node_modules/split-string",
          "type": "prod",
        },
        "to-regex" => EdgeOut {
          "name": "to-regex",
          "spec": "^3.0.1",
          "to": "node_modules/to-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-2.3.2.tgz",
      "version": "2.3.2",
    },
    "brorand" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "brorand",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/miller-rabin",
          "name": "brorand",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/brorand",
      "name": "brorand",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/brorand",
      "peer": true,
      "resolved": "https://registry.npmjs.org/brorand/-/brorand-1.1.0.tgz",
      "version": "1.1.0",
    },
    "browserify-aes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-cipher",
          "name": "browserify-aes",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-asn1",
          "name": "browserify-aes",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-xor" => EdgeOut {
          "name": "buffer-xor",
          "spec": "^1.0.3",
          "to": "node_modules/buffer-xor",
          "type": "prod",
        },
        "cipher-base" => EdgeOut {
          "name": "cipher-base",
          "spec": "^1.0.0",
          "to": "node_modules/cipher-base",
          "type": "prod",
        },
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.1.0",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "evp_bytestokey" => EdgeOut {
          "name": "evp_bytestokey",
          "spec": "^1.0.3",
          "to": "node_modules/evp_bytestokey",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-aes",
      "name": "browserify-aes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-aes",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-aes/-/browserify-aes-1.2.0.tgz",
      "version": "1.2.0",
    },
    "browserify-cipher" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "browserify-cipher",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "browserify-aes" => EdgeOut {
          "name": "browserify-aes",
          "spec": "^1.0.4",
          "to": "node_modules/browserify-aes",
          "type": "prod",
        },
        "browserify-des" => EdgeOut {
          "name": "browserify-des",
          "spec": "^1.0.0",
          "to": "node_modules/browserify-des",
          "type": "prod",
        },
        "evp_bytestokey" => EdgeOut {
          "name": "evp_bytestokey",
          "spec": "^1.0.0",
          "to": "node_modules/evp_bytestokey",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-cipher",
      "name": "browserify-cipher",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-cipher",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-cipher/-/browserify-cipher-1.0.1.tgz",
      "version": "1.0.1",
    },
    "browserify-des" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-cipher",
          "name": "browserify-des",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cipher-base" => EdgeOut {
          "name": "cipher-base",
          "spec": "^1.0.1",
          "to": "node_modules/cipher-base",
          "type": "prod",
        },
        "des.js" => EdgeOut {
          "name": "des.js",
          "spec": "^1.0.0",
          "to": "node_modules/des.js",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-des",
      "name": "browserify-des",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-des",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-des/-/browserify-des-1.0.2.tgz",
      "version": "1.0.2",
    },
    "browserify-rsa" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/browserify-rsa",
              "name": "bn.js",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/browserify-rsa/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-rsa/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "browserify-rsa",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/public-encrypt",
          "name": "browserify-rsa",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.1.0",
          "to": "node_modules/browserify-rsa/node_modules/bn.js",
          "type": "prod",
        },
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.0.1",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-rsa",
      "name": "browserify-rsa",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-rsa",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-rsa/-/browserify-rsa-4.0.1.tgz",
      "version": "4.0.1",
    },
    "browserify-sign" => ArboristNode {
      "children": Map {
        "readable-stream" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/browserify-sign",
              "name": "readable-stream",
              "spec": "^3.6.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "^2.0.3",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "^1.1.1",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "^1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/browserify-sign/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-sign/node_modules/readable-stream",
          "peer": true,
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-3.6.0.tgz",
          "version": "3.6.0",
        },
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/browserify-sign",
              "name": "safe-buffer",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/browserify-sign/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-sign/node_modules/safe-buffer",
          "peer": true,
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.1.tgz",
          "version": "5.2.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "browserify-sign",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^5.1.1",
          "to": "node_modules/bn.js",
          "type": "prod",
        },
        "browserify-rsa" => EdgeOut {
          "name": "browserify-rsa",
          "spec": "^4.0.1",
          "to": "node_modules/browserify-rsa",
          "type": "prod",
        },
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.2.0",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "create-hmac" => EdgeOut {
          "name": "create-hmac",
          "spec": "^1.1.7",
          "to": "node_modules/create-hmac",
          "type": "prod",
        },
        "elliptic" => EdgeOut {
          "name": "elliptic",
          "spec": "^6.5.3",
          "to": "node_modules/elliptic",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.4",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "parse-asn1" => EdgeOut {
          "name": "parse-asn1",
          "spec": "^5.1.5",
          "to": "node_modules/parse-asn1",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^3.6.0",
          "to": "node_modules/browserify-sign/node_modules/readable-stream",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.2.0",
          "to": "node_modules/browserify-sign/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-sign",
      "name": "browserify-sign",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-sign",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-sign/-/browserify-sign-4.2.1.tgz",
      "version": "4.2.1",
    },
    "browserify-zlib" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "browserify-zlib",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pako" => EdgeOut {
          "name": "pako",
          "spec": "~1.0.5",
          "to": "node_modules/pako",
          "type": "prod",
        },
      },
      "location": "node_modules/browserify-zlib",
      "name": "browserify-zlib",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/browserify-zlib",
      "peer": true,
      "resolved": "https://registry.npmjs.org/browserify-zlib/-/browserify-zlib-0.2.0.tgz",
      "version": "0.2.0",
    },
    "buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "buffer",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "base64-js" => EdgeOut {
          "name": "base64-js",
          "spec": "^1.0.2",
          "to": "node_modules/base64-js",
          "type": "prod",
        },
        "ieee754" => EdgeOut {
          "name": "ieee754",
          "spec": "^1.1.4",
          "to": "node_modules/ieee754",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "^1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer",
      "name": "buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/buffer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/buffer/-/buffer-4.9.2.tgz",
      "version": "4.9.2",
    },
    "buffer-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/buffer-from",
      "peer": true,
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "buffer-indexof" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dns-txt",
          "name": "buffer-indexof",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-indexof",
      "name": "buffer-indexof",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/buffer-indexof",
      "resolved": "https://registry.npmjs.org/buffer-indexof/-/buffer-indexof-1.1.1.tgz",
      "version": "1.1.1",
    },
    "buffer-xor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "buffer-xor",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-xor",
      "name": "buffer-xor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/buffer-xor",
      "peer": true,
      "resolved": "https://registry.npmjs.org/buffer-xor/-/buffer-xor-1.0.3.tgz",
      "version": "1.0.3",
    },
    "builtin-status-codes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/stream-http",
          "name": "builtin-status-codes",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/builtin-status-codes",
      "name": "builtin-status-codes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/builtin-status-codes",
      "peer": true,
      "resolved": "https://registry.npmjs.org/builtin-status-codes/-/builtin-status-codes-3.0.0.tgz",
      "version": "3.0.0",
    },
    "bytes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compression",
          "name": "bytes",
          "spec": "3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bytes",
      "name": "bytes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/bytes",
      "resolved": "https://registry.npmjs.org/bytes/-/bytes-3.0.0.tgz",
      "version": "3.0.0",
    },
    "cacache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "cacache",
          "spec": "^12.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bluebird" => EdgeOut {
          "name": "bluebird",
          "spec": "^3.5.5",
          "to": "node_modules/bluebird",
          "type": "prod",
        },
        "chownr" => EdgeOut {
          "name": "chownr",
          "spec": "^1.1.1",
          "to": "node_modules/chownr",
          "type": "prod",
        },
        "figgy-pudding" => EdgeOut {
          "name": "figgy-pudding",
          "spec": "^3.5.1",
          "to": "node_modules/figgy-pudding",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.4",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "infer-owner" => EdgeOut {
          "name": "infer-owner",
          "spec": "^1.0.3",
          "to": "node_modules/infer-owner",
          "type": "prod",
        },
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^5.1.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "mississippi" => EdgeOut {
          "name": "mississippi",
          "spec": "^3.0.0",
          "to": "node_modules/mississippi",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "move-concurrently" => EdgeOut {
          "name": "move-concurrently",
          "spec": "^1.0.1",
          "to": "node_modules/move-concurrently",
          "type": "prod",
        },
        "promise-inflight" => EdgeOut {
          "name": "promise-inflight",
          "spec": "^1.0.1",
          "to": "node_modules/promise-inflight",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "ssri" => EdgeOut {
          "name": "ssri",
          "spec": "^6.0.1",
          "to": "node_modules/ssri",
          "type": "prod",
        },
        "unique-filename" => EdgeOut {
          "name": "unique-filename",
          "spec": "^1.1.1",
          "to": "node_modules/unique-filename",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
      },
      "location": "node_modules/cacache",
      "name": "cacache",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cacache",
      "peer": true,
      "resolved": "https://registry.npmjs.org/cacache/-/cacache-12.0.4.tgz",
      "version": "12.0.4",
    },
    "cache-base" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "cache-base",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "collection-visit" => EdgeOut {
          "name": "collection-visit",
          "spec": "^1.0.0",
          "to": "node_modules/collection-visit",
          "type": "prod",
        },
        "component-emitter" => EdgeOut {
          "name": "component-emitter",
          "spec": "^1.2.1",
          "to": "node_modules/component-emitter",
          "type": "prod",
        },
        "get-value" => EdgeOut {
          "name": "get-value",
          "spec": "^2.0.6",
          "to": "node_modules/get-value",
          "type": "prod",
        },
        "has-value" => EdgeOut {
          "name": "has-value",
          "spec": "^1.0.0",
          "to": "node_modules/has-value",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
        "set-value" => EdgeOut {
          "name": "set-value",
          "spec": "^2.0.0",
          "to": "node_modules/set-value",
          "type": "prod",
        },
        "to-object-path" => EdgeOut {
          "name": "to-object-path",
          "spec": "^0.3.0",
          "to": "node_modules/to-object-path",
          "type": "prod",
        },
        "union-value" => EdgeOut {
          "name": "union-value",
          "spec": "^1.0.0",
          "to": "node_modules/union-value",
          "type": "prod",
        },
        "unset-value" => EdgeOut {
          "name": "unset-value",
          "spec": "^1.0.0",
          "to": "node_modules/unset-value",
          "type": "prod",
        },
      },
      "location": "node_modules/cache-base",
      "name": "cache-base",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cache-base",
      "resolved": "https://registry.npmjs.org/cache-base/-/cache-base-1.0.1.tgz",
      "version": "1.0.1",
    },
    "camelcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "chokidar" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/watchpack-chokidar2",
          "name": "chokidar",
          "spec": "^2.1.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "chokidar",
          "spec": "^2.1.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "^2.0.0",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "async-each" => EdgeOut {
          "name": "async-each",
          "spec": "^1.0.1",
          "to": "node_modules/async-each",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "^2.3.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "^1.2.7",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "^3.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "^1.0.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.0",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "^2.2.1",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
        "upath" => EdgeOut {
          "name": "upath",
          "spec": "^1.1.1",
          "to": "node_modules/upath",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-2.1.8.tgz",
      "version": "2.1.8",
    },
    "chownr" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "chownr",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/chownr",
      "name": "chownr",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/chownr",
      "peer": true,
      "resolved": "https://registry.npmjs.org/chownr/-/chownr-1.1.4.tgz",
      "version": "1.1.4",
    },
    "chrome-trace-event" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "chrome-trace-event",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.9.0",
          "to": "node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/chrome-trace-event",
      "name": "chrome-trace-event",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/chrome-trace-event",
      "peer": true,
      "resolved": "https://registry.npmjs.org/chrome-trace-event/-/chrome-trace-event-1.0.2.tgz",
      "version": "1.0.2",
    },
    "cipher-base" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "cipher-base",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-des",
          "name": "cipher-base",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hash",
          "name": "cipher-base",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "cipher-base",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cipher-base",
      "name": "cipher-base",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cipher-base",
      "peer": true,
      "resolved": "https://registry.npmjs.org/cipher-base/-/cipher-base-1.0.4.tgz",
      "version": "1.0.4",
    },
    "class-utils" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/class-utils",
              "name": "define-property",
              "spec": "^0.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "to": "node_modules/class-utils/node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/class-utils/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
          "version": "0.2.5",
        },
        "is-accessor-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/class-utils/node_modules/is-descriptor",
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/class-utils/node_modules/is-accessor-descriptor",
          "name": "is-accessor-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils/node_modules/is-accessor-descriptor",
          "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "is-data-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/class-utils/node_modules/is-descriptor",
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/class-utils/node_modules/is-data-descriptor",
          "name": "is-data-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils/node_modules/is-data-descriptor",
          "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
          "version": "0.1.4",
        },
        "is-descriptor" => ArboristNode {
          "children": Map {
            "kind-of" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/class-utils/node_modules/is-descriptor",
                  "name": "kind-of",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/class-utils/node_modules/is-descriptor/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils/node_modules/is-descriptor/node_modules/kind-of",
              "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/class-utils/node_modules/define-property",
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-accessor-descriptor" => EdgeOut {
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "to": "node_modules/class-utils/node_modules/is-accessor-descriptor",
              "type": "prod",
            },
            "is-data-descriptor" => EdgeOut {
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "to": "node_modules/class-utils/node_modules/is-data-descriptor",
              "type": "prod",
            },
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^5.0.0",
              "to": "node_modules/class-utils/node_modules/is-descriptor/node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/class-utils/node_modules/is-descriptor",
          "name": "is-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils/node_modules/is-descriptor",
          "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "class-utils",
          "spec": "^0.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arr-union" => EdgeOut {
          "name": "arr-union",
          "spec": "^3.1.0",
          "to": "node_modules/arr-union",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^0.2.5",
          "to": "node_modules/class-utils/node_modules/define-property",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.0",
          "to": "node_modules/isobject",
          "type": "prod",
        },
        "static-extend" => EdgeOut {
          "name": "static-extend",
          "spec": "^0.1.1",
          "to": "node_modules/static-extend",
          "type": "prod",
        },
      },
      "location": "node_modules/class-utils",
      "name": "class-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/class-utils",
      "resolved": "https://registry.npmjs.org/class-utils/-/class-utils-0.3.6.tgz",
      "version": "0.3.6",
    },
    "cliui" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cliui/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/cliui/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cliui/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "cliui",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/cliui/node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^5.1.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
      "version": "5.0.0",
    },
    "collection-visit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "collection-visit",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "map-visit" => EdgeOut {
          "name": "map-visit",
          "spec": "^1.0.0",
          "to": "node_modules/map-visit",
          "type": "prod",
        },
        "object-visit" => EdgeOut {
          "name": "object-visit",
          "spec": "^1.0.0",
          "to": "node_modules/object-visit",
          "type": "prod",
        },
      },
      "location": "node_modules/collection-visit",
      "name": "collection-visit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/collection-visit",
      "resolved": "https://registry.npmjs.org/collection-visit/-/collection-visit-1.0.0.tgz",
      "version": "1.0.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "commander" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser",
          "name": "commander",
          "spec": "^2.20.0",
          "type": "prod",
        },
      },
      "location": "node_modules/commander",
      "name": "commander",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/commander",
      "peer": true,
      "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.3.tgz",
      "version": "2.20.3",
    },
    "commondir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/commondir",
      "peer": true,
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "component-emitter" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "component-emitter",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "component-emitter",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/component-emitter",
      "name": "component-emitter",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/component-emitter",
      "resolved": "https://registry.npmjs.org/component-emitter/-/component-emitter-1.3.0.tgz",
      "version": "1.3.0",
    },
    "compressible" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compression",
          "name": "compressible",
          "spec": "~2.0.16",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": ">= 1.43.0 < 2",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/compressible",
      "name": "compressible",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/compressible",
      "resolved": "https://registry.npmjs.org/compressible/-/compressible-2.0.18.tgz",
      "version": "2.0.18",
    },
    "compression" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/compression",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/compression/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/compression/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/compression/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/compression/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/compression/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/compression/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "compression",
          "spec": "^1.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "accepts" => EdgeOut {
          "name": "accepts",
          "spec": "~1.3.5",
          "to": "node_modules/accepts",
          "type": "prod",
        },
        "bytes" => EdgeOut {
          "name": "bytes",
          "spec": "3.0.0",
          "to": "node_modules/bytes",
          "type": "prod",
        },
        "compressible" => EdgeOut {
          "name": "compressible",
          "spec": "~2.0.16",
          "to": "node_modules/compressible",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/compression/node_modules/debug",
          "type": "prod",
        },
        "on-headers" => EdgeOut {
          "name": "on-headers",
          "spec": "~1.0.2",
          "to": "node_modules/on-headers",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "vary" => EdgeOut {
          "name": "vary",
          "spec": "~1.1.2",
          "to": "node_modules/vary",
          "type": "prod",
        },
      },
      "location": "node_modules/compression",
      "name": "compression",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/compression",
      "resolved": "https://registry.npmjs.org/compression/-/compression-1.7.4.tgz",
      "version": "1.7.4",
    },
    "concat-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "concat-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "concat-stream",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.2.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "typedarray" => EdgeOut {
          "name": "typedarray",
          "spec": "^0.0.6",
          "to": "node_modules/typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-stream",
      "name": "concat-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/concat-stream",
      "peer": true,
      "resolved": "https://registry.npmjs.org/concat-stream/-/concat-stream-1.6.2.tgz",
      "version": "1.6.2",
    },
    "connect-history-api-fallback" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "connect-history-api-fallback",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/connect-history-api-fallback",
      "name": "connect-history-api-fallback",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/connect-history-api-fallback",
      "resolved": "https://registry.npmjs.org/connect-history-api-fallback/-/connect-history-api-fallback-1.6.0.tgz",
      "version": "1.6.0",
    },
    "console-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "console-browserify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/console-browserify",
      "name": "console-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/console-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/console-browserify/-/console-browserify-1.2.0.tgz",
      "version": "1.2.0",
    },
    "constants-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "constants-browserify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/constants-browserify",
      "name": "constants-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/constants-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/constants-browserify/-/constants-browserify-1.0.0.tgz",
      "version": "1.0.0",
    },
    "content-disposition" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "content-disposition",
          "spec": "0.5.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/content-disposition",
      "name": "content-disposition",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/content-disposition",
      "resolved": "https://registry.npmjs.org/content-disposition/-/content-disposition-0.5.3.tgz",
      "version": "0.5.3",
    },
    "content-type" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "content-type",
          "spec": "~1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "content-type",
          "spec": "~1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/content-type",
      "name": "content-type",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/content-type",
      "resolved": "https://registry.npmjs.org/content-type/-/content-type-1.0.4.tgz",
      "version": "1.0.4",
    },
    "cookie" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "cookie",
          "spec": "0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cookie",
      "name": "cookie",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cookie",
      "resolved": "https://registry.npmjs.org/cookie/-/cookie-0.4.0.tgz",
      "version": "0.4.0",
    },
    "cookie-signature" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "cookie-signature",
          "spec": "1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/cookie-signature",
      "name": "cookie-signature",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cookie-signature",
      "resolved": "https://registry.npmjs.org/cookie-signature/-/cookie-signature-1.0.6.tgz",
      "version": "1.0.6",
    },
    "copy-concurrently" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "copy-concurrently",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aproba" => EdgeOut {
          "name": "aproba",
          "spec": "^1.1.1",
          "to": "node_modules/aproba",
          "type": "prod",
        },
        "fs-write-stream-atomic" => EdgeOut {
          "name": "fs-write-stream-atomic",
          "spec": "^1.0.8",
          "to": "node_modules/fs-write-stream-atomic",
          "type": "prod",
        },
        "iferr" => EdgeOut {
          "name": "iferr",
          "spec": "^0.1.5",
          "to": "node_modules/iferr",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.5.4",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "run-queue" => EdgeOut {
          "name": "run-queue",
          "spec": "^1.0.0",
          "to": "node_modules/run-queue",
          "type": "prod",
        },
      },
      "location": "node_modules/copy-concurrently",
      "name": "copy-concurrently",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/copy-concurrently",
      "peer": true,
      "resolved": "https://registry.npmjs.org/copy-concurrently/-/copy-concurrently-1.0.5.tgz",
      "version": "1.0.5",
    },
    "copy-descriptor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/object-copy",
          "name": "copy-descriptor",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/copy-descriptor",
      "name": "copy-descriptor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/copy-descriptor",
      "resolved": "https://registry.npmjs.org/copy-descriptor/-/copy-descriptor-0.1.1.tgz",
      "version": "0.1.1",
    },
    "core-util-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "create-ecdh" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/create-ecdh",
              "name": "bn.js",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/create-ecdh/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/create-ecdh/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "create-ecdh",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.1.0",
          "to": "node_modules/create-ecdh/node_modules/bn.js",
          "type": "prod",
        },
        "elliptic" => EdgeOut {
          "name": "elliptic",
          "spec": "^6.5.3",
          "to": "node_modules/elliptic",
          "type": "prod",
        },
      },
      "location": "node_modules/create-ecdh",
      "name": "create-ecdh",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/create-ecdh",
      "peer": true,
      "resolved": "https://registry.npmjs.org/create-ecdh/-/create-ecdh-4.0.4.tgz",
      "version": "4.0.4",
    },
    "create-hash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "create-hash",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "create-hash",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "create-hash",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "create-hash",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pbkdf2",
          "name": "create-hash",
          "spec": "^1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/public-encrypt",
          "name": "create-hash",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cipher-base" => EdgeOut {
          "name": "cipher-base",
          "spec": "^1.0.1",
          "to": "node_modules/cipher-base",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "md5.js" => EdgeOut {
          "name": "md5.js",
          "spec": "^1.3.4",
          "to": "node_modules/md5.js",
          "type": "prod",
        },
        "ripemd160" => EdgeOut {
          "name": "ripemd160",
          "spec": "^2.0.1",
          "to": "node_modules/ripemd160",
          "type": "prod",
        },
        "sha.js" => EdgeOut {
          "name": "sha.js",
          "spec": "^2.4.0",
          "to": "node_modules/sha.js",
          "type": "prod",
        },
      },
      "location": "node_modules/create-hash",
      "name": "create-hash",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/create-hash",
      "peer": true,
      "resolved": "https://registry.npmjs.org/create-hash/-/create-hash-1.2.0.tgz",
      "version": "1.2.0",
    },
    "create-hmac" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "create-hmac",
          "spec": "^1.1.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "create-hmac",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pbkdf2",
          "name": "create-hmac",
          "spec": "^1.1.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cipher-base" => EdgeOut {
          "name": "cipher-base",
          "spec": "^1.0.3",
          "to": "node_modules/cipher-base",
          "type": "prod",
        },
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.1.0",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "ripemd160" => EdgeOut {
          "name": "ripemd160",
          "spec": "^2.0.0",
          "to": "node_modules/ripemd160",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "sha.js" => EdgeOut {
          "name": "sha.js",
          "spec": "^2.4.8",
          "to": "node_modules/sha.js",
          "type": "prod",
        },
      },
      "location": "node_modules/create-hmac",
      "name": "create-hmac",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/create-hmac",
      "peer": true,
      "resolved": "https://registry.npmjs.org/create-hmac/-/create-hmac-1.1.7.tgz",
      "version": "1.1.7",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "semver",
              "spec": "^5.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cross-spawn/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
          "version": "5.7.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "cross-spawn",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "nice-try" => EdgeOut {
          "name": "nice-try",
          "spec": "^1.0.4",
          "to": "node_modules/nice-try",
          "type": "prod",
        },
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^2.0.1",
          "to": "node_modules/path-key",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.5.0",
          "to": "node_modules/cross-spawn/node_modules/semver",
          "type": "prod",
        },
        "shebang-command" => EdgeOut {
          "name": "shebang-command",
          "spec": "^1.2.0",
          "to": "node_modules/shebang-command",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
      "version": "6.0.5",
    },
    "crypto-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "crypto-browserify",
          "spec": "^3.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "browserify-cipher" => EdgeOut {
          "name": "browserify-cipher",
          "spec": "^1.0.0",
          "to": "node_modules/browserify-cipher",
          "type": "prod",
        },
        "browserify-sign" => EdgeOut {
          "name": "browserify-sign",
          "spec": "^4.0.0",
          "to": "node_modules/browserify-sign",
          "type": "prod",
        },
        "create-ecdh" => EdgeOut {
          "name": "create-ecdh",
          "spec": "^4.0.0",
          "to": "node_modules/create-ecdh",
          "type": "prod",
        },
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.1.0",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "create-hmac" => EdgeOut {
          "name": "create-hmac",
          "spec": "^1.1.0",
          "to": "node_modules/create-hmac",
          "type": "prod",
        },
        "diffie-hellman" => EdgeOut {
          "name": "diffie-hellman",
          "spec": "^5.0.0",
          "to": "node_modules/diffie-hellman",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "pbkdf2" => EdgeOut {
          "name": "pbkdf2",
          "spec": "^3.0.3",
          "to": "node_modules/pbkdf2",
          "type": "prod",
        },
        "public-encrypt" => EdgeOut {
          "name": "public-encrypt",
          "spec": "^4.0.0",
          "to": "node_modules/public-encrypt",
          "type": "prod",
        },
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.0.0",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
        "randomfill" => EdgeOut {
          "name": "randomfill",
          "spec": "^1.0.3",
          "to": "node_modules/randomfill",
          "type": "prod",
        },
      },
      "location": "node_modules/crypto-browserify",
      "name": "crypto-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/crypto-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/crypto-browserify/-/crypto-browserify-3.12.0.tgz",
      "version": "3.12.0",
    },
    "cyclist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parallel-transform",
          "name": "cyclist",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/cyclist",
      "name": "cyclist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/cyclist",
      "peer": true,
      "resolved": "https://registry.npmjs.org/cyclist/-/cyclist-1.0.1.tgz",
      "version": "1.0.1",
    },
    "debug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "decamelize" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "decode-uri-component" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-resolve",
          "name": "decode-uri-component",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decode-uri-component",
      "name": "decode-uri-component",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/decode-uri-component",
      "resolved": "https://registry.npmjs.org/decode-uri-component/-/decode-uri-component-0.2.0.tgz",
      "version": "0.2.0",
    },
    "deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "deep-equal",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arguments" => EdgeOut {
          "name": "is-arguments",
          "spec": "^1.0.4",
          "to": "node_modules/is-arguments",
          "type": "prod",
        },
        "is-date-object" => EdgeOut {
          "name": "is-date-object",
          "spec": "^1.0.1",
          "to": "node_modules/is-date-object",
          "type": "prod",
        },
        "is-regex" => EdgeOut {
          "name": "is-regex",
          "spec": "^1.0.4",
          "to": "node_modules/is-regex",
          "type": "prod",
        },
        "object-is" => EdgeOut {
          "name": "object-is",
          "spec": "^1.0.1",
          "to": "node_modules/object-is",
          "type": "prod",
        },
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.1.1",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
        "regexp.prototype.flags" => EdgeOut {
          "name": "regexp.prototype.flags",
          "spec": "^1.2.0",
          "to": "node_modules/regexp.prototype.flags",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-equal",
      "name": "deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/deep-equal",
      "resolved": "https://registry.npmjs.org/deep-equal/-/deep-equal-1.1.1.tgz",
      "version": "1.1.1",
    },
    "default-gateway" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/internal-ip",
          "name": "default-gateway",
          "spec": "^4.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "execa" => EdgeOut {
          "name": "execa",
          "spec": "^1.0.0",
          "to": "node_modules/execa",
          "type": "prod",
        },
        "ip-regex" => EdgeOut {
          "name": "ip-regex",
          "spec": "^2.1.0",
          "to": "node_modules/ip-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/default-gateway",
      "name": "default-gateway",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/default-gateway",
      "resolved": "https://registry.npmjs.org/default-gateway/-/default-gateway-4.2.0.tgz",
      "version": "4.2.0",
    },
    "define-properties" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/object-is",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart",
          "name": "define-properties",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.0.12",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/define-properties",
      "name": "define-properties",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/define-properties",
      "resolved": "https://registry.npmjs.org/define-properties/-/define-properties-1.1.3.tgz",
      "version": "1.1.3",
    },
    "define-property" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "define-property",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "define-property",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex",
          "name": "define-property",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-descriptor" => EdgeOut {
          "name": "is-descriptor",
          "spec": "^1.0.2",
          "to": "node_modules/is-descriptor",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/define-property",
      "name": "define-property",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/define-property",
      "resolved": "https://registry.npmjs.org/define-property/-/define-property-2.0.2.tgz",
      "version": "2.0.2",
    },
    "del" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "del",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/glob" => EdgeOut {
          "name": "@types/glob",
          "spec": "^7.1.1",
          "to": "node_modules/@types/glob",
          "type": "prod",
        },
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^6.1.0",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-path-cwd" => EdgeOut {
          "name": "is-path-cwd",
          "spec": "^2.0.0",
          "to": "node_modules/is-path-cwd",
          "type": "prod",
        },
        "is-path-in-cwd" => EdgeOut {
          "name": "is-path-in-cwd",
          "spec": "^2.0.0",
          "to": "node_modules/is-path-in-cwd",
          "type": "prod",
        },
        "p-map" => EdgeOut {
          "name": "p-map",
          "spec": "^2.0.0",
          "to": "node_modules/p-map",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "node_modules/del",
      "name": "del",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/del",
      "resolved": "https://registry.npmjs.org/del/-/del-4.1.1.tgz",
      "version": "4.1.1",
    },
    "depd" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "depd",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "depd",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-errors",
          "name": "depd",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "depd",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index/node_modules/http-errors",
          "name": "depd",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/depd",
      "name": "depd",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/depd",
      "resolved": "https://registry.npmjs.org/depd/-/depd-1.1.2.tgz",
      "version": "1.1.2",
    },
    "des.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-des",
          "name": "des.js",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
      },
      "location": "node_modules/des.js",
      "name": "des.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/des.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/des.js/-/des.js-1.0.1.tgz",
      "version": "1.0.1",
    },
    "destroy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/send",
          "name": "destroy",
          "spec": "~1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/destroy",
      "name": "destroy",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/destroy",
      "resolved": "https://registry.npmjs.org/destroy/-/destroy-1.0.4.tgz",
      "version": "1.0.4",
    },
    "detect-node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy-transport",
          "name": "detect-node",
          "spec": "^2.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/detect-node",
      "name": "detect-node",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/detect-node",
      "resolved": "https://registry.npmjs.org/detect-node/-/detect-node-2.0.4.tgz",
      "version": "2.0.4",
    },
    "diffie-hellman" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/diffie-hellman",
              "name": "bn.js",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/diffie-hellman/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/diffie-hellman/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "diffie-hellman",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.1.0",
          "to": "node_modules/diffie-hellman/node_modules/bn.js",
          "type": "prod",
        },
        "miller-rabin" => EdgeOut {
          "name": "miller-rabin",
          "spec": "^4.0.0",
          "to": "node_modules/miller-rabin",
          "type": "prod",
        },
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.0.0",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
      },
      "location": "node_modules/diffie-hellman",
      "name": "diffie-hellman",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/diffie-hellman",
      "peer": true,
      "resolved": "https://registry.npmjs.org/diffie-hellman/-/diffie-hellman-5.0.3.tgz",
      "version": "5.0.3",
    },
    "dns-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "dns-equal",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/dns-equal",
      "name": "dns-equal",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/dns-equal",
      "resolved": "https://registry.npmjs.org/dns-equal/-/dns-equal-1.0.0.tgz",
      "version": "1.0.0",
    },
    "dns-packet" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/multicast-dns",
          "name": "dns-packet",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ip" => EdgeOut {
          "name": "ip",
          "spec": "^1.1.0",
          "to": "node_modules/ip",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/dns-packet",
      "name": "dns-packet",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/dns-packet",
      "resolved": "https://registry.npmjs.org/dns-packet/-/dns-packet-1.3.1.tgz",
      "version": "1.3.1",
    },
    "dns-txt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "dns-txt",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-indexof" => EdgeOut {
          "name": "buffer-indexof",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-indexof",
          "type": "prod",
        },
      },
      "location": "node_modules/dns-txt",
      "name": "dns-txt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/dns-txt",
      "resolved": "https://registry.npmjs.org/dns-txt/-/dns-txt-2.0.2.tgz",
      "version": "2.0.2",
    },
    "domain-browser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "domain-browser",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/domain-browser",
      "name": "domain-browser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/domain-browser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/domain-browser/-/domain-browser-1.2.0.tgz",
      "version": "1.2.0",
    },
    "duplexify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "duplexify",
          "spec": "^3.4.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pumpify",
          "name": "duplexify",
          "spec": "^3.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "end-of-stream" => EdgeOut {
          "name": "end-of-stream",
          "spec": "^1.0.0",
          "to": "node_modules/end-of-stream",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.0",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "stream-shift" => EdgeOut {
          "name": "stream-shift",
          "spec": "^1.0.0",
          "to": "node_modules/stream-shift",
          "type": "prod",
        },
      },
      "location": "node_modules/duplexify",
      "name": "duplexify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/duplexify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/duplexify/-/duplexify-3.7.1.tgz",
      "version": "3.7.1",
    },
    "ee-first" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/on-finished",
          "name": "ee-first",
          "spec": "1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ee-first",
      "name": "ee-first",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ee-first",
      "resolved": "https://registry.npmjs.org/ee-first/-/ee-first-1.1.1.tgz",
      "version": "1.1.1",
    },
    "elliptic" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/elliptic",
              "name": "bn.js",
              "spec": "^4.4.0",
              "type": "prod",
            },
          },
          "location": "node_modules/elliptic/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/elliptic/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "elliptic",
          "spec": "^6.5.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-ecdh",
          "name": "elliptic",
          "spec": "^6.5.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.4.0",
          "to": "node_modules/elliptic/node_modules/bn.js",
          "type": "prod",
        },
        "brorand" => EdgeOut {
          "name": "brorand",
          "spec": "^1.0.1",
          "to": "node_modules/brorand",
          "type": "prod",
        },
        "hash.js" => EdgeOut {
          "name": "hash.js",
          "spec": "^1.0.0",
          "to": "node_modules/hash.js",
          "type": "prod",
        },
        "hmac-drbg" => EdgeOut {
          "name": "hmac-drbg",
          "spec": "^1.0.0",
          "to": "node_modules/hmac-drbg",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
        "minimalistic-crypto-utils" => EdgeOut {
          "name": "minimalistic-crypto-utils",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-crypto-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/elliptic",
      "name": "elliptic",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/elliptic",
      "peer": true,
      "resolved": "https://registry.npmjs.org/elliptic/-/elliptic-6.5.3.tgz",
      "version": "6.5.3",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
      "version": "7.0.3",
    },
    "emojis-list" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/loader-utils",
          "name": "emojis-list",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emojis-list",
      "name": "emojis-list",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/emojis-list",
      "peer": true,
      "resolved": "https://registry.npmjs.org/emojis-list/-/emojis-list-3.0.0.tgz",
      "version": "3.0.0",
    },
    "encodeurl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "encodeurl",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "encodeurl",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "encodeurl",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-static",
          "name": "encodeurl",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/encodeurl",
      "name": "encodeurl",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/encodeurl",
      "resolved": "https://registry.npmjs.org/encodeurl/-/encodeurl-1.0.2.tgz",
      "version": "1.0.2",
    },
    "end-of-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/duplexify",
          "name": "end-of-stream",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pump",
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pumpify/node_modules/pump",
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-each",
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.4.0",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/end-of-stream",
      "name": "end-of-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/end-of-stream",
      "resolved": "https://registry.npmjs.org/end-of-stream/-/end-of-stream-1.4.4.tgz",
      "version": "1.4.4",
    },
    "enhanced-resolve" => ArboristNode {
      "children": Map {
        "memory-fs" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/enhanced-resolve",
              "name": "memory-fs",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "errno" => EdgeOut {
              "name": "errno",
              "spec": "^0.1.3",
              "to": "node_modules/errno",
              "type": "prod",
            },
            "readable-stream" => EdgeOut {
              "name": "readable-stream",
              "spec": "^2.0.1",
              "to": "node_modules/readable-stream",
              "type": "prod",
            },
          },
          "location": "node_modules/enhanced-resolve/node_modules/memory-fs",
          "name": "memory-fs",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/enhanced-resolve/node_modules/memory-fs",
          "peer": true,
          "resolved": "https://registry.npmjs.org/memory-fs/-/memory-fs-0.5.0.tgz",
          "version": "0.5.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "enhanced-resolve",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "memory-fs" => EdgeOut {
          "name": "memory-fs",
          "spec": "^0.5.0",
          "to": "node_modules/enhanced-resolve/node_modules/memory-fs",
          "type": "prod",
        },
        "tapable" => EdgeOut {
          "name": "tapable",
          "spec": "^1.0.0",
          "to": "node_modules/tapable",
          "type": "prod",
        },
      },
      "location": "node_modules/enhanced-resolve",
      "name": "enhanced-resolve",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/enhanced-resolve",
      "peer": true,
      "resolved": "https://registry.npmjs.org/enhanced-resolve/-/enhanced-resolve-4.3.0.tgz",
      "version": "4.3.0",
    },
    "errno" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enhanced-resolve/node_modules/memory-fs",
          "name": "errno",
          "spec": "^0.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/memory-fs",
          "name": "errno",
          "spec": "^0.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/worker-farm",
          "name": "errno",
          "spec": "~0.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "prr" => EdgeOut {
          "name": "prr",
          "spec": "~1.0.1",
          "to": "node_modules/prr",
          "type": "prod",
        },
      },
      "location": "node_modules/errno",
      "name": "errno",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/errno",
      "resolved": "https://registry.npmjs.org/errno/-/errno-0.1.7.tgz",
      "version": "0.1.7",
    },
    "error-stack-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "error-stack-parser",
          "spec": "^2.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "stackframe" => EdgeOut {
          "name": "stackframe",
          "spec": "^1.1.1",
          "to": "node_modules/stackframe",
          "type": "prod",
        },
      },
      "location": "node_modules/error-stack-parser",
      "name": "error-stack-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/error-stack-parser",
      "resolved": "https://registry.npmjs.org/error-stack-parser/-/error-stack-parser-2.0.6.tgz",
      "version": "2.0.6",
    },
    "es-abstract" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/object-is",
          "name": "es-abstract",
          "spec": "^1.18.0-next.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "es-abstract",
          "spec": "^1.18.0-next.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es-to-primitive" => EdgeOut {
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "to": "node_modules/es-to-primitive",
          "type": "prod",
        },
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
        "has" => EdgeOut {
          "name": "has",
          "spec": "^1.0.3",
          "to": "node_modules/has",
          "type": "prod",
        },
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
        "is-callable" => EdgeOut {
          "name": "is-callable",
          "spec": "^1.2.2",
          "to": "node_modules/is-callable",
          "type": "prod",
        },
        "is-negative-zero" => EdgeOut {
          "name": "is-negative-zero",
          "spec": "^2.0.0",
          "to": "node_modules/is-negative-zero",
          "type": "prod",
        },
        "is-regex" => EdgeOut {
          "name": "is-regex",
          "spec": "^1.1.1",
          "to": "node_modules/is-regex",
          "type": "prod",
        },
        "object-inspect" => EdgeOut {
          "name": "object-inspect",
          "spec": "^1.8.0",
          "to": "node_modules/object-inspect",
          "type": "prod",
        },
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.1.1",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
        "object.assign" => EdgeOut {
          "name": "object.assign",
          "spec": "^4.1.1",
          "to": "node_modules/object.assign",
          "type": "prod",
        },
        "string.prototype.trimend" => EdgeOut {
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "to": "node_modules/string.prototype.trimend",
          "type": "prod",
        },
        "string.prototype.trimstart" => EdgeOut {
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "to": "node_modules/string.prototype.trimstart",
          "type": "prod",
        },
      },
      "location": "node_modules/es-abstract",
      "name": "es-abstract",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/es-abstract",
      "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.18.0-next.1.tgz",
      "version": "1.18.0-next.1",
    },
    "es-to-primitive" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "es-to-primitive",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-callable" => EdgeOut {
          "name": "is-callable",
          "spec": "^1.1.4",
          "to": "node_modules/is-callable",
          "type": "prod",
        },
        "is-date-object" => EdgeOut {
          "name": "is-date-object",
          "spec": "^1.0.1",
          "to": "node_modules/is-date-object",
          "type": "prod",
        },
        "is-symbol" => EdgeOut {
          "name": "is-symbol",
          "spec": "^1.0.2",
          "to": "node_modules/is-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es-to-primitive",
      "name": "es-to-primitive",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/es-to-primitive",
      "resolved": "https://registry.npmjs.org/es-to-primitive/-/es-to-primitive-1.2.1.tgz",
      "version": "1.2.1",
    },
    "escape-html" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "escape-html",
          "spec": "~1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "escape-html",
          "spec": "~1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "escape-html",
          "spec": "~1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index",
          "name": "escape-html",
          "spec": "~1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-static",
          "name": "escape-html",
          "spec": "~1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-html",
      "name": "escape-html",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/escape-html",
      "resolved": "https://registry.npmjs.org/escape-html/-/escape-html-1.0.3.tgz",
      "version": "1.0.3",
    },
    "eslint-scope" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "eslint-scope",
          "spec": "^4.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.1.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-scope",
      "name": "eslint-scope",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/eslint-scope",
      "peer": true,
      "resolved": "https://registry.npmjs.org/eslint-scope/-/eslint-scope-4.0.3.tgz",
      "version": "4.0.3",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/esrecurse/node_modules/estraverse",
          "peer": true,
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "esrecurse",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^5.2.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/esrecurse",
      "peer": true,
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "estraverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-scope",
          "name": "estraverse",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/estraverse",
      "name": "estraverse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/estraverse",
      "peer": true,
      "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.3.0.tgz",
      "version": "4.3.0",
    },
    "etag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "etag",
          "spec": "~1.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "etag",
          "spec": "~1.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/etag",
      "name": "etag",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/etag",
      "resolved": "https://registry.npmjs.org/etag/-/etag-1.8.1.tgz",
      "version": "1.8.1",
    },
    "eventemitter3" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-proxy",
          "name": "eventemitter3",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eventemitter3",
      "name": "eventemitter3",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/eventemitter3",
      "resolved": "https://registry.npmjs.org/eventemitter3/-/eventemitter3-4.0.7.tgz",
      "version": "4.0.7",
    },
    "events" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "events",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/events",
      "name": "events",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/events",
      "peer": true,
      "resolved": "https://registry.npmjs.org/events/-/events-3.2.0.tgz",
      "version": "3.2.0",
    },
    "eventsource" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sockjs-client",
          "name": "eventsource",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "original" => EdgeOut {
          "name": "original",
          "spec": "^1.0.0",
          "to": "node_modules/original",
          "type": "prod",
        },
      },
      "location": "node_modules/eventsource",
      "name": "eventsource",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/eventsource",
      "resolved": "https://registry.npmjs.org/eventsource/-/eventsource-1.0.7.tgz",
      "version": "1.0.7",
    },
    "evp_bytestokey" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "evp_bytestokey",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-cipher",
          "name": "evp_bytestokey",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-asn1",
          "name": "evp_bytestokey",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "md5.js" => EdgeOut {
          "name": "md5.js",
          "spec": "^1.3.4",
          "to": "node_modules/md5.js",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/evp_bytestokey",
      "name": "evp_bytestokey",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/evp_bytestokey",
      "peer": true,
      "resolved": "https://registry.npmjs.org/evp_bytestokey/-/evp_bytestokey-1.0.3.tgz",
      "version": "1.0.3",
    },
    "execa" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-gateway",
          "name": "execa",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.0",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "get-stream" => EdgeOut {
          "name": "get-stream",
          "spec": "^4.0.0",
          "to": "node_modules/get-stream",
          "type": "prod",
        },
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.1.0",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
        "npm-run-path" => EdgeOut {
          "name": "npm-run-path",
          "spec": "^2.0.0",
          "to": "node_modules/npm-run-path",
          "type": "prod",
        },
        "p-finally" => EdgeOut {
          "name": "p-finally",
          "spec": "^1.0.0",
          "to": "node_modules/p-finally",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "strip-eof" => EdgeOut {
          "name": "strip-eof",
          "spec": "^1.0.0",
          "to": "node_modules/strip-eof",
          "type": "prod",
        },
      },
      "location": "node_modules/execa",
      "name": "execa",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/execa",
      "resolved": "https://registry.npmjs.org/execa/-/execa-1.0.0.tgz",
      "version": "1.0.0",
    },
    "expand-brackets" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets",
              "name": "debug",
              "spec": "^2.3.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/expand-brackets/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets",
              "name": "define-property",
              "spec": "^0.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "to": "node_modules/expand-brackets/node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
          "version": "0.2.5",
        },
        "is-accessor-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets/node_modules/is-descriptor",
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/is-accessor-descriptor",
          "name": "is-accessor-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/is-accessor-descriptor",
          "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "is-data-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets/node_modules/is-descriptor",
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/is-data-descriptor",
          "name": "is-data-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/is-data-descriptor",
          "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
          "version": "0.1.4",
        },
        "is-descriptor" => ArboristNode {
          "children": Map {
            "kind-of" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/expand-brackets/node_modules/is-descriptor",
                  "name": "kind-of",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/expand-brackets/node_modules/is-descriptor/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/is-descriptor/node_modules/kind-of",
              "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets/node_modules/define-property",
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-accessor-descriptor" => EdgeOut {
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "to": "node_modules/expand-brackets/node_modules/is-accessor-descriptor",
              "type": "prod",
            },
            "is-data-descriptor" => EdgeOut {
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "to": "node_modules/expand-brackets/node_modules/is-data-descriptor",
              "type": "prod",
            },
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^5.0.0",
              "to": "node_modules/expand-brackets/node_modules/is-descriptor/node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/is-descriptor",
          "name": "is-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/is-descriptor",
          "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/expand-brackets/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/expand-brackets/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "expand-brackets",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.3.3",
          "to": "node_modules/expand-brackets/node_modules/debug",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^0.2.5",
          "to": "node_modules/expand-brackets/node_modules/define-property",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "posix-character-classes" => EdgeOut {
          "name": "posix-character-classes",
          "spec": "^0.1.0",
          "to": "node_modules/posix-character-classes",
          "type": "prod",
        },
        "regex-not" => EdgeOut {
          "name": "regex-not",
          "spec": "^1.0.0",
          "to": "node_modules/regex-not",
          "type": "prod",
        },
        "snapdragon" => EdgeOut {
          "name": "snapdragon",
          "spec": "^0.8.1",
          "to": "node_modules/snapdragon",
          "type": "prod",
        },
        "to-regex" => EdgeOut {
          "name": "to-regex",
          "spec": "^3.0.1",
          "to": "node_modules/to-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/expand-brackets",
      "name": "expand-brackets",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/expand-brackets",
      "resolved": "https://registry.npmjs.org/expand-brackets/-/expand-brackets-2.1.4.tgz",
      "version": "2.1.4",
    },
    "express" => ArboristNode {
      "children": Map {
        "array-flatten" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/express",
              "name": "array-flatten",
              "spec": "1.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/express/node_modules/array-flatten",
          "name": "array-flatten",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/express/node_modules/array-flatten",
          "resolved": "https://registry.npmjs.org/array-flatten/-/array-flatten-1.1.1.tgz",
          "version": "1.1.1",
        },
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/express",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/express/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/express/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/express/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/express/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/express/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/express/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "express",
          "spec": "^4.17.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "accepts" => EdgeOut {
          "name": "accepts",
          "spec": "~1.3.7",
          "to": "node_modules/accepts",
          "type": "prod",
        },
        "array-flatten" => EdgeOut {
          "name": "array-flatten",
          "spec": "1.1.1",
          "to": "node_modules/express/node_modules/array-flatten",
          "type": "prod",
        },
        "body-parser" => EdgeOut {
          "name": "body-parser",
          "spec": "1.19.0",
          "to": "node_modules/body-parser",
          "type": "prod",
        },
        "content-disposition" => EdgeOut {
          "name": "content-disposition",
          "spec": "0.5.3",
          "to": "node_modules/content-disposition",
          "type": "prod",
        },
        "content-type" => EdgeOut {
          "name": "content-type",
          "spec": "~1.0.4",
          "to": "node_modules/content-type",
          "type": "prod",
        },
        "cookie" => EdgeOut {
          "name": "cookie",
          "spec": "0.4.0",
          "to": "node_modules/cookie",
          "type": "prod",
        },
        "cookie-signature" => EdgeOut {
          "name": "cookie-signature",
          "spec": "1.0.6",
          "to": "node_modules/cookie-signature",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/express/node_modules/debug",
          "type": "prod",
        },
        "depd" => EdgeOut {
          "name": "depd",
          "spec": "~1.1.2",
          "to": "node_modules/depd",
          "type": "prod",
        },
        "encodeurl" => EdgeOut {
          "name": "encodeurl",
          "spec": "~1.0.2",
          "to": "node_modules/encodeurl",
          "type": "prod",
        },
        "escape-html" => EdgeOut {
          "name": "escape-html",
          "spec": "~1.0.3",
          "to": "node_modules/escape-html",
          "type": "prod",
        },
        "etag" => EdgeOut {
          "name": "etag",
          "spec": "~1.8.1",
          "to": "node_modules/etag",
          "type": "prod",
        },
        "finalhandler" => EdgeOut {
          "name": "finalhandler",
          "spec": "~1.1.2",
          "to": "node_modules/finalhandler",
          "type": "prod",
        },
        "fresh" => EdgeOut {
          "name": "fresh",
          "spec": "0.5.2",
          "to": "node_modules/fresh",
          "type": "prod",
        },
        "merge-descriptors" => EdgeOut {
          "name": "merge-descriptors",
          "spec": "1.0.1",
          "to": "node_modules/merge-descriptors",
          "type": "prod",
        },
        "methods" => EdgeOut {
          "name": "methods",
          "spec": "~1.1.2",
          "to": "node_modules/methods",
          "type": "prod",
        },
        "on-finished" => EdgeOut {
          "name": "on-finished",
          "spec": "~2.3.0",
          "to": "node_modules/on-finished",
          "type": "prod",
        },
        "parseurl" => EdgeOut {
          "name": "parseurl",
          "spec": "~1.3.3",
          "to": "node_modules/parseurl",
          "type": "prod",
        },
        "path-to-regexp" => EdgeOut {
          "name": "path-to-regexp",
          "spec": "0.1.7",
          "to": "node_modules/path-to-regexp",
          "type": "prod",
        },
        "proxy-addr" => EdgeOut {
          "name": "proxy-addr",
          "spec": "~2.0.5",
          "to": "node_modules/proxy-addr",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "6.7.0",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "range-parser" => EdgeOut {
          "name": "range-parser",
          "spec": "~1.2.1",
          "to": "node_modules/range-parser",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "send" => EdgeOut {
          "name": "send",
          "spec": "0.17.1",
          "to": "node_modules/send",
          "type": "prod",
        },
        "serve-static" => EdgeOut {
          "name": "serve-static",
          "spec": "1.14.1",
          "to": "node_modules/serve-static",
          "type": "prod",
        },
        "setprototypeof" => EdgeOut {
          "name": "setprototypeof",
          "spec": "1.1.1",
          "to": "node_modules/setprototypeof",
          "type": "prod",
        },
        "statuses" => EdgeOut {
          "name": "statuses",
          "spec": "~1.5.0",
          "to": "node_modules/statuses",
          "type": "prod",
        },
        "type-is" => EdgeOut {
          "name": "type-is",
          "spec": "~1.6.18",
          "to": "node_modules/type-is",
          "type": "prod",
        },
        "utils-merge" => EdgeOut {
          "name": "utils-merge",
          "spec": "1.0.1",
          "to": "node_modules/utils-merge",
          "type": "prod",
        },
        "vary" => EdgeOut {
          "name": "vary",
          "spec": "~1.1.2",
          "to": "node_modules/vary",
          "type": "prod",
        },
      },
      "location": "node_modules/express",
      "name": "express",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/express",
      "resolved": "https://registry.npmjs.org/express/-/express-4.17.1.tgz",
      "version": "4.17.1",
    },
    "extend-shallow" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/expand-brackets",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/set-value",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon",
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extendable" => EdgeOut {
          "name": "is-extendable",
          "spec": "^0.1.0",
          "to": "node_modules/is-extendable",
          "type": "prod",
        },
      },
      "location": "node_modules/extend-shallow",
      "name": "extend-shallow",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/extend-shallow",
      "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-2.0.1.tgz",
      "version": "2.0.1",
    },
    "extglob" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/extglob",
              "name": "define-property",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^1.0.0",
              "to": "node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/extglob/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/extglob/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "extglob",
          "spec": "^2.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-unique" => EdgeOut {
          "name": "array-unique",
          "spec": "^0.3.2",
          "to": "node_modules/array-unique",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^1.0.0",
          "to": "node_modules/extglob/node_modules/define-property",
          "type": "prod",
        },
        "expand-brackets" => EdgeOut {
          "name": "expand-brackets",
          "spec": "^2.1.4",
          "to": "node_modules/expand-brackets",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "fragment-cache" => EdgeOut {
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "to": "node_modules/fragment-cache",
          "type": "prod",
        },
        "regex-not" => EdgeOut {
          "name": "regex-not",
          "spec": "^1.0.0",
          "to": "node_modules/regex-not",
          "type": "prod",
        },
        "snapdragon" => EdgeOut {
          "name": "snapdragon",
          "spec": "^0.8.1",
          "to": "node_modules/snapdragon",
          "type": "prod",
        },
        "to-regex" => EdgeOut {
          "name": "to-regex",
          "spec": "^3.0.1",
          "to": "node_modules/to-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/extglob",
      "name": "extglob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/extglob",
      "resolved": "https://registry.npmjs.org/extglob/-/extglob-2.0.4.tgz",
      "version": "2.0.4",
    },
    "fast-deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "faye-websocket" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sockjs",
          "name": "faye-websocket",
          "spec": "^0.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "websocket-driver" => EdgeOut {
          "name": "websocket-driver",
          "spec": ">=0.5.1",
          "to": "node_modules/websocket-driver",
          "type": "prod",
        },
      },
      "location": "node_modules/faye-websocket",
      "name": "faye-websocket",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/faye-websocket",
      "resolved": "https://registry.npmjs.org/faye-websocket/-/faye-websocket-0.10.0.tgz",
      "version": "0.10.0",
    },
    "figgy-pudding" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "figgy-pudding",
          "spec": "^3.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ssri",
          "name": "figgy-pudding",
          "spec": "^3.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/figgy-pudding",
      "name": "figgy-pudding",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/figgy-pudding",
      "peer": true,
      "resolved": "https://registry.npmjs.org/figgy-pudding/-/figgy-pudding-3.5.2.tgz",
      "version": "3.5.2",
    },
    "file-uri-to-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bindings",
          "name": "file-uri-to-path",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/file-uri-to-path",
      "name": "file-uri-to-path",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/file-uri-to-path",
      "resolved": "https://registry.npmjs.org/file-uri-to-path/-/file-uri-to-path-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fill-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^3.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
        "repeat-string" => EdgeOut {
          "name": "repeat-string",
          "spec": "^1.6.1",
          "to": "node_modules/repeat-string",
          "type": "prod",
        },
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^2.1.0",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-4.0.0.tgz",
      "version": "4.0.0",
    },
    "finalhandler" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/finalhandler",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/finalhandler/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/finalhandler/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/finalhandler/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/finalhandler/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/finalhandler/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/finalhandler/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "finalhandler",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/finalhandler/node_modules/debug",
          "type": "prod",
        },
        "encodeurl" => EdgeOut {
          "name": "encodeurl",
          "spec": "~1.0.2",
          "to": "node_modules/encodeurl",
          "type": "prod",
        },
        "escape-html" => EdgeOut {
          "name": "escape-html",
          "spec": "~1.0.3",
          "to": "node_modules/escape-html",
          "type": "prod",
        },
        "on-finished" => EdgeOut {
          "name": "on-finished",
          "spec": "~2.3.0",
          "to": "node_modules/on-finished",
          "type": "prod",
        },
        "parseurl" => EdgeOut {
          "name": "parseurl",
          "spec": "~1.3.3",
          "to": "node_modules/parseurl",
          "type": "prod",
        },
        "statuses" => EdgeOut {
          "name": "statuses",
          "spec": "~1.5.0",
          "to": "node_modules/statuses",
          "type": "prod",
        },
        "unpipe" => EdgeOut {
          "name": "unpipe",
          "spec": "~1.0.0",
          "to": "node_modules/unpipe",
          "type": "prod",
        },
      },
      "location": "node_modules/finalhandler",
      "name": "finalhandler",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/finalhandler",
      "resolved": "https://registry.npmjs.org/finalhandler/-/finalhandler-1.1.2.tgz",
      "version": "1.1.2",
    },
    "find-cache-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/find-cache-dir",
      "peer": true,
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "flush-write-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "flush-write-stream",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.3.6",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/flush-write-stream",
      "name": "flush-write-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/flush-write-stream",
      "peer": true,
      "resolved": "https://registry.npmjs.org/flush-write-stream/-/flush-write-stream-1.1.1.tgz",
      "version": "1.1.1",
    },
    "follow-redirects" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-proxy",
          "name": "follow-redirects",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/follow-redirects",
      "name": "follow-redirects",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/follow-redirects",
      "resolved": "https://registry.npmjs.org/follow-redirects/-/follow-redirects-1.13.0.tgz",
      "version": "1.13.0",
    },
    "for-in" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mixin-deep",
          "name": "for-in",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/for-in",
      "name": "for-in",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/for-in",
      "resolved": "https://registry.npmjs.org/for-in/-/for-in-1.0.2.tgz",
      "version": "1.0.2",
    },
    "forwarded" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/proxy-addr",
          "name": "forwarded",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/forwarded",
      "name": "forwarded",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/forwarded",
      "resolved": "https://registry.npmjs.org/forwarded/-/forwarded-0.1.2.tgz",
      "version": "0.1.2",
    },
    "fragment-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "map-cache" => EdgeOut {
          "name": "map-cache",
          "spec": "^0.2.2",
          "to": "node_modules/map-cache",
          "type": "prod",
        },
      },
      "location": "node_modules/fragment-cache",
      "name": "fragment-cache",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fragment-cache",
      "resolved": "https://registry.npmjs.org/fragment-cache/-/fragment-cache-0.2.1.tgz",
      "version": "0.2.1",
    },
    "fresh" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "fresh",
          "spec": "0.5.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "fresh",
          "spec": "0.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/fresh",
      "name": "fresh",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fresh",
      "resolved": "https://registry.npmjs.org/fresh/-/fresh-0.5.2.tgz",
      "version": "0.5.2",
    },
    "from2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "from2",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.0",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/from2",
      "name": "from2",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/from2",
      "peer": true,
      "resolved": "https://registry.npmjs.org/from2/-/from2-2.3.0.tgz",
      "version": "2.3.0",
    },
    "fs-write-stream-atomic" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "fs-write-stream-atomic",
          "spec": "^1.0.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "fs-write-stream-atomic",
          "spec": "^1.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "iferr" => EdgeOut {
          "name": "iferr",
          "spec": "^0.1.5",
          "to": "node_modules/iferr",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "1 || 2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-write-stream-atomic",
      "name": "fs-write-stream-atomic",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fs-write-stream-atomic",
      "peer": true,
      "resolved": "https://registry.npmjs.org/fs-write-stream-atomic/-/fs-write-stream-atomic-1.0.10.tgz",
      "version": "1.0.10",
    },
    "fs.realpath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "^1.2.7",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "bindings" => EdgeOut {
          "name": "bindings",
          "spec": "^1.5.0",
          "to": "node_modules/bindings",
          "type": "prod",
        },
        "nan" => EdgeOut {
          "name": "nan",
          "spec": "^2.12.1",
          "to": "node_modules/nan",
          "type": "prod",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-1.2.13.tgz",
      "version": "1.2.13",
    },
    "function-bind" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/has",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "function-bind",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-bind",
      "name": "function-bind",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/function-bind",
      "resolved": "https://registry.npmjs.org/function-bind/-/function-bind-1.1.1.tgz",
      "version": "1.1.1",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "get-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "get-stream",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pump" => EdgeOut {
          "name": "pump",
          "spec": "^3.0.0",
          "to": "node_modules/pump",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stream",
      "name": "get-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/get-stream",
      "resolved": "https://registry.npmjs.org/get-stream/-/get-stream-4.1.0.tgz",
      "version": "4.1.0",
    },
    "get-value" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "get-value",
          "spec": "^2.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/has-value",
          "name": "get-value",
          "spec": "^2.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/union-value",
          "name": "get-value",
          "spec": "^2.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unset-value/node_modules/has-value",
          "name": "get-value",
          "spec": "^2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/get-value",
      "name": "get-value",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/get-value",
      "resolved": "https://registry.npmjs.org/get-value/-/get-value-2.0.6.tgz",
      "version": "2.0.6",
    },
    "glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "glob",
          "spec": "^7.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "glob",
          "spec": "^7.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "children": Map {
        "is-glob" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/glob-parent",
              "name": "is-glob",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-extglob" => EdgeOut {
              "name": "is-extglob",
              "spec": "^2.1.0",
              "to": "node_modules/is-extglob",
              "type": "prod",
            },
          },
          "location": "node_modules/glob-parent/node_modules/is-glob",
          "name": "is-glob",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/glob-parent/node_modules/is-glob",
          "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^3.1.0",
          "to": "node_modules/glob-parent/node_modules/is-glob",
          "type": "prod",
        },
        "path-dirname" => EdgeOut {
          "name": "path-dirname",
          "spec": "^1.0.0",
          "to": "node_modules/path-dirname",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-3.1.0.tgz",
      "version": "3.1.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "pify",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/globby/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-2.3.0.tgz",
          "version": "2.3.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "globby",
          "spec": "^6.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^1.0.1",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/globby/node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-6.1.0.tgz",
      "version": "6.1.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/enhanced-resolve",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fs-write-stream-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "handle-thing" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy",
          "name": "handle-thing",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/handle-thing",
      "name": "handle-thing",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/handle-thing",
      "resolved": "https://registry.npmjs.org/handle-thing/-/handle-thing-2.0.1.tgz",
      "version": "2.0.1",
    },
    "has" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "has",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "function-bind" => EdgeOut {
          "name": "function-bind",
          "spec": "^1.1.1",
          "to": "node_modules/function-bind",
          "type": "prod",
        },
      },
      "location": "node_modules/has",
      "name": "has",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has",
      "resolved": "https://registry.npmjs.org/has/-/has-1.0.3.tgz",
      "version": "1.0.3",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "has-symbols" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-regex",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-symbol",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "has-symbols",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/has-symbols",
      "name": "has-symbols",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has-symbols",
      "resolved": "https://registry.npmjs.org/has-symbols/-/has-symbols-1.0.1.tgz",
      "version": "1.0.1",
    },
    "has-value" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "has-value",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "get-value" => EdgeOut {
          "name": "get-value",
          "spec": "^2.0.6",
          "to": "node_modules/get-value",
          "type": "prod",
        },
        "has-values" => EdgeOut {
          "name": "has-values",
          "spec": "^1.0.0",
          "to": "node_modules/has-values",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.0",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/has-value",
      "name": "has-value",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has-value",
      "resolved": "https://registry.npmjs.org/has-value/-/has-value-1.0.0.tgz",
      "version": "1.0.0",
    },
    "has-values" => ArboristNode {
      "children": Map {
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/has-values",
              "name": "kind-of",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-buffer" => EdgeOut {
              "name": "is-buffer",
              "spec": "^1.1.5",
              "to": "node_modules/is-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/has-values/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has-values/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/has-value",
          "name": "has-values",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^3.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^4.0.0",
          "to": "node_modules/has-values/node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/has-values",
      "name": "has-values",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/has-values",
      "resolved": "https://registry.npmjs.org/has-values/-/has-values-1.0.0.tgz",
      "version": "1.0.0",
    },
    "hash-base" => ArboristNode {
      "children": Map {
        "readable-stream" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/hash-base",
              "name": "readable-stream",
              "spec": "^3.6.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "^2.0.3",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "^1.1.1",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "^1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/hash-base/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hash-base/node_modules/readable-stream",
          "peer": true,
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-3.6.0.tgz",
          "version": "3.6.0",
        },
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/hash-base",
              "name": "safe-buffer",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/hash-base/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hash-base/node_modules/safe-buffer",
          "peer": true,
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.1.tgz",
          "version": "5.2.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/md5.js",
          "name": "hash-base",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ripemd160",
          "name": "hash-base",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.4",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^3.6.0",
          "to": "node_modules/hash-base/node_modules/readable-stream",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.2.0",
          "to": "node_modules/hash-base/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/hash-base",
      "name": "hash-base",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hash-base",
      "peer": true,
      "resolved": "https://registry.npmjs.org/hash-base/-/hash-base-3.1.0.tgz",
      "version": "3.1.0",
    },
    "hash.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "hash.js",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hmac-drbg",
          "name": "hash.js",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.1",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
      },
      "location": "node_modules/hash.js",
      "name": "hash.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hash.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/hash.js/-/hash.js-1.1.7.tgz",
      "version": "1.1.7",
    },
    "hmac-drbg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "hmac-drbg",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hash.js" => EdgeOut {
          "name": "hash.js",
          "spec": "^1.0.3",
          "to": "node_modules/hash.js",
          "type": "prod",
        },
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
        "minimalistic-crypto-utils" => EdgeOut {
          "name": "minimalistic-crypto-utils",
          "spec": "^1.0.1",
          "to": "node_modules/minimalistic-crypto-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/hmac-drbg",
      "name": "hmac-drbg",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hmac-drbg",
      "peer": true,
      "resolved": "https://registry.npmjs.org/hmac-drbg/-/hmac-drbg-1.0.1.tgz",
      "version": "1.0.1",
    },
    "hpack.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy-transport",
          "name": "hpack.js",
          "spec": "^2.1.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "obuf" => EdgeOut {
          "name": "obuf",
          "spec": "^1.0.0",
          "to": "node_modules/obuf",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.1",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "wbuf" => EdgeOut {
          "name": "wbuf",
          "spec": "^1.1.0",
          "to": "node_modules/wbuf",
          "type": "prod",
        },
      },
      "location": "node_modules/hpack.js",
      "name": "hpack.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/hpack.js",
      "resolved": "https://registry.npmjs.org/hpack.js/-/hpack.js-2.1.6.tgz",
      "version": "2.1.6",
    },
    "html-entities" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "html-entities",
          "spec": "^1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "html-entities",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/html-entities",
      "name": "html-entities",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/html-entities",
      "resolved": "https://registry.npmjs.org/html-entities/-/html-entities-1.3.1.tgz",
      "version": "1.3.1",
    },
    "http-deceiver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy",
          "name": "http-deceiver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/http-deceiver",
      "name": "http-deceiver",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/http-deceiver",
      "resolved": "https://registry.npmjs.org/http-deceiver/-/http-deceiver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "http-errors" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/http-errors",
              "name": "inherits",
              "spec": "2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/http-errors/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/http-errors/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "http-errors",
          "spec": "1.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/raw-body",
          "name": "http-errors",
          "spec": "1.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "http-errors",
          "spec": "~1.7.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "depd" => EdgeOut {
          "name": "depd",
          "spec": "~1.1.2",
          "to": "node_modules/depd",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2.0.3",
          "to": "node_modules/http-errors/node_modules/inherits",
          "type": "prod",
        },
        "setprototypeof" => EdgeOut {
          "name": "setprototypeof",
          "spec": "1.1.1",
          "to": "node_modules/setprototypeof",
          "type": "prod",
        },
        "statuses" => EdgeOut {
          "name": "statuses",
          "spec": ">= 1.5.0 < 2",
          "to": "node_modules/statuses",
          "type": "prod",
        },
        "toidentifier" => EdgeOut {
          "name": "toidentifier",
          "spec": "1.0.0",
          "to": "node_modules/toidentifier",
          "type": "prod",
        },
      },
      "location": "node_modules/http-errors",
      "name": "http-errors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/http-errors",
      "resolved": "https://registry.npmjs.org/http-errors/-/http-errors-1.7.2.tgz",
      "version": "1.7.2",
    },
    "http-proxy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-proxy-middleware",
          "name": "http-proxy",
          "spec": "^1.17.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eventemitter3" => EdgeOut {
          "name": "eventemitter3",
          "spec": "^4.0.0",
          "to": "node_modules/eventemitter3",
          "type": "prod",
        },
        "follow-redirects" => EdgeOut {
          "name": "follow-redirects",
          "spec": "^1.0.0",
          "to": "node_modules/follow-redirects",
          "type": "prod",
        },
        "requires-port" => EdgeOut {
          "name": "requires-port",
          "spec": "^1.0.0",
          "to": "node_modules/requires-port",
          "type": "prod",
        },
      },
      "location": "node_modules/http-proxy",
      "name": "http-proxy",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/http-proxy",
      "resolved": "https://registry.npmjs.org/http-proxy/-/http-proxy-1.18.1.tgz",
      "version": "1.18.1",
    },
    "http-proxy-middleware" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "http-proxy-middleware",
          "spec": "0.19.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "http-proxy" => EdgeOut {
          "name": "http-proxy",
          "spec": "^1.17.0",
          "to": "node_modules/http-proxy",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.0",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.11",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^3.1.10",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
      },
      "location": "node_modules/http-proxy-middleware",
      "name": "http-proxy-middleware",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/http-proxy-middleware",
      "resolved": "https://registry.npmjs.org/http-proxy-middleware/-/http-proxy-middleware-0.19.1.tgz",
      "version": "0.19.1",
    },
    "https-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "https-browserify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/https-browserify",
      "name": "https-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/https-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/https-browserify/-/https-browserify-1.0.0.tgz",
      "version": "1.0.0",
    },
    "iconv-lite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "iconv-lite",
          "spec": "0.4.24",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/raw-body",
          "name": "iconv-lite",
          "spec": "0.4.24",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/iconv-lite",
      "name": "iconv-lite",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/iconv-lite",
      "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
      "version": "0.4.24",
    },
    "ieee754" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/buffer",
          "name": "ieee754",
          "spec": "^1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/ieee754",
      "name": "ieee754",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ieee754",
      "peer": true,
      "resolved": "https://registry.npmjs.org/ieee754/-/ieee754-1.2.1.tgz",
      "version": "1.2.1",
    },
    "iferr" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "iferr",
          "spec": "^0.1.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fs-write-stream-atomic",
          "name": "iferr",
          "spec": "^0.1.5",
          "type": "prod",
        },
      },
      "location": "node_modules/iferr",
      "name": "iferr",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/iferr",
      "peer": true,
      "resolved": "https://registry.npmjs.org/iferr/-/iferr-0.1.5.tgz",
      "version": "0.1.5",
    },
    "import-local" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "import-local",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
        "resolve-cwd" => EdgeOut {
          "name": "resolve-cwd",
          "spec": "^2.0.0",
          "to": "node_modules/resolve-cwd",
          "type": "prod",
        },
      },
      "location": "node_modules/import-local",
      "name": "import-local",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/import-local",
      "resolved": "https://registry.npmjs.org/import-local/-/import-local-2.0.0.tgz",
      "version": "2.0.0",
    },
    "imurmurhash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fs-write-stream-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unique-slug",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/imurmurhash",
      "peer": true,
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "infer-owner" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "infer-owner",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/infer-owner",
      "name": "infer-owner",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/infer-owner",
      "peer": true,
      "resolved": "https://registry.npmjs.org/infer-owner/-/infer-owner-1.0.4.tgz",
      "version": "1.0.4",
    },
    "inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1.js",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-des",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "inherits",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-sign/node_modules/readable-stream",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cipher-base",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hash",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/des.js",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/duplexify",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flush-write-stream",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/from2",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash-base",
          "name": "inherits",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash-base/node_modules/readable-stream",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash.js",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hpack.js",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/md5.js",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parallel-transform",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pumpify",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ripemd160",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sha.js",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sockjs-client",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport/node_modules/readable-stream",
          "name": "inherits",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-browserify",
          "name": "inherits",
          "spec": "~2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-http",
          "name": "inherits",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "internal-ip" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "internal-ip",
          "spec": "^4.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-gateway" => EdgeOut {
          "name": "default-gateway",
          "spec": "^4.2.0",
          "to": "node_modules/default-gateway",
          "type": "prod",
        },
        "ipaddr.js" => EdgeOut {
          "name": "ipaddr.js",
          "spec": "^1.9.0",
          "to": "node_modules/ipaddr.js",
          "type": "prod",
        },
      },
      "location": "node_modules/internal-ip",
      "name": "internal-ip",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/internal-ip",
      "resolved": "https://registry.npmjs.org/internal-ip/-/internal-ip-4.3.0.tgz",
      "version": "4.3.0",
    },
    "ip" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dns-packet",
          "name": "ip",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "ip",
          "spec": "^1.1.5",
          "type": "prod",
        },
      },
      "location": "node_modules/ip",
      "name": "ip",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ip",
      "resolved": "https://registry.npmjs.org/ip/-/ip-1.1.5.tgz",
      "version": "1.1.5",
    },
    "ip-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-gateway",
          "name": "ip-regex",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ip-regex",
      "name": "ip-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ip-regex",
      "resolved": "https://registry.npmjs.org/ip-regex/-/ip-regex-2.1.0.tgz",
      "version": "2.1.0",
    },
    "ipaddr.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/internal-ip",
          "name": "ipaddr.js",
          "spec": "^1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/proxy-addr",
          "name": "ipaddr.js",
          "spec": "1.9.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ipaddr.js",
      "name": "ipaddr.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ipaddr.js",
      "resolved": "https://registry.npmjs.org/ipaddr.js/-/ipaddr.js-1.9.1.tgz",
      "version": "1.9.1",
    },
    "is-absolute-url" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "is-absolute-url",
          "spec": "^3.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/is-absolute-url",
      "name": "is-absolute-url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-absolute-url",
      "resolved": "https://registry.npmjs.org/is-absolute-url/-/is-absolute-url-3.0.3.tgz",
      "version": "3.0.3",
    },
    "is-accessor-descriptor" => ArboristNode {
      "children": Map {
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/is-accessor-descriptor",
              "name": "kind-of",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/is-accessor-descriptor/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-accessor-descriptor/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
          "version": "6.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-descriptor",
          "name": "is-accessor-descriptor",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.0",
          "to": "node_modules/is-accessor-descriptor/node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/is-accessor-descriptor",
      "name": "is-accessor-descriptor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-accessor-descriptor",
      "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-arguments" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "is-arguments",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arguments",
      "name": "is-arguments",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-arguments",
      "resolved": "https://registry.npmjs.org/is-arguments/-/is-arguments-1.0.4.tgz",
      "version": "1.0.4",
    },
    "is-binary-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^1.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-1.0.1.tgz",
      "version": "1.0.1",
    },
    "is-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/has-values/node_modules/kind-of",
          "name": "is-buffer",
          "spec": "^1.1.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/kind-of",
          "name": "is-buffer",
          "spec": "^1.1.5",
          "type": "prod",
        },
      },
      "location": "node_modules/is-buffer",
      "name": "is-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-buffer",
      "resolved": "https://registry.npmjs.org/is-buffer/-/is-buffer-1.1.6.tgz",
      "version": "1.1.6",
    },
    "is-callable" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-callable",
          "spec": "^1.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "is-callable",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/is-callable",
      "name": "is-callable",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-callable",
      "resolved": "https://registry.npmjs.org/is-callable/-/is-callable-1.2.2.tgz",
      "version": "1.2.2",
    },
    "is-data-descriptor" => ArboristNode {
      "children": Map {
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/is-data-descriptor",
              "name": "kind-of",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/is-data-descriptor/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-data-descriptor/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
          "version": "6.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-descriptor",
          "name": "is-data-descriptor",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.0",
          "to": "node_modules/is-data-descriptor/node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/is-data-descriptor",
      "name": "is-data-descriptor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-data-descriptor",
      "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-date-object" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "is-date-object",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-date-object",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-date-object",
      "name": "is-date-object",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-date-object",
      "resolved": "https://registry.npmjs.org/is-date-object/-/is-date-object-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-descriptor" => ArboristNode {
      "children": Map {
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/is-descriptor",
              "name": "kind-of",
              "spec": "^6.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/is-descriptor/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-descriptor/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
          "version": "6.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base/node_modules/define-property",
          "name": "is-descriptor",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/define-property",
          "name": "is-descriptor",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob/node_modules/define-property",
          "name": "is-descriptor",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon-node/node_modules/define-property",
          "name": "is-descriptor",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-accessor-descriptor" => EdgeOut {
          "name": "is-accessor-descriptor",
          "spec": "^1.0.0",
          "to": "node_modules/is-accessor-descriptor",
          "type": "prod",
        },
        "is-data-descriptor" => EdgeOut {
          "name": "is-data-descriptor",
          "spec": "^1.0.0",
          "to": "node_modules/is-data-descriptor",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.2",
          "to": "node_modules/is-descriptor/node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/is-descriptor",
      "name": "is-descriptor",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-descriptor",
      "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-extendable" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/extend-shallow",
          "name": "is-extendable",
          "spec": "^0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/set-value",
          "name": "is-extendable",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/union-value",
          "name": "is-extendable",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extendable",
      "name": "is-extendable",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-extendable",
      "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-0.1.1.tgz",
      "version": "0.1.1",
    },
    "is-extglob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob-parent/node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-proxy-middleware",
          "name": "is-glob",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-negative-zero" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-negative-zero",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-negative-zero",
      "name": "is-negative-zero",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-negative-zero",
      "resolved": "https://registry.npmjs.org/is-negative-zero/-/is-negative-zero-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-number" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "is-number",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/has-values",
          "name": "is-number",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^3.0.2",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-path-cwd" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-cwd",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-cwd",
      "name": "is-path-cwd",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-path-cwd",
      "resolved": "https://registry.npmjs.org/is-path-cwd/-/is-path-cwd-2.2.0.tgz",
      "version": "2.2.0",
    },
    "is-path-in-cwd" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-in-cwd",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-path-inside" => EdgeOut {
          "name": "is-path-inside",
          "spec": "^2.1.0",
          "to": "node_modules/is-path-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-in-cwd",
      "name": "is-path-in-cwd",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-path-in-cwd",
      "resolved": "https://registry.npmjs.org/is-path-in-cwd/-/is-path-in-cwd-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-path-inside" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-path-in-cwd",
          "name": "is-path-inside",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.2",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-inside",
      "name": "is-path-inside",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-path-inside",
      "resolved": "https://registry.npmjs.org/is-path-inside/-/is-path-inside-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-plain-object" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mixin-deep/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regex-not/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/set-value",
          "name": "is-plain-object",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/split-string/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex/node_modules/is-extendable",
          "name": "is-plain-object",
          "spec": "^2.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/is-plain-object",
      "name": "is-plain-object",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-plain-object",
      "resolved": "https://registry.npmjs.org/is-plain-object/-/is-plain-object-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "is-regex",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "is-regex",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
      },
      "location": "node_modules/is-regex",
      "name": "is-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-regex",
      "resolved": "https://registry.npmjs.org/is-regex/-/is-regex-1.1.1.tgz",
      "version": "1.1.1",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "is-stream",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-symbol" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-to-primitive",
          "name": "is-symbol",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
      },
      "location": "node_modules/is-symbol",
      "name": "is-symbol",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-symbol",
      "resolved": "https://registry.npmjs.org/is-symbol/-/is-symbol-1.0.3.tgz",
      "version": "1.0.3",
    },
    "is-windows" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "is-windows",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/is-windows",
      "name": "is-windows",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-windows",
      "resolved": "https://registry.npmjs.org/is-windows/-/is-windows-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-wsl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/opn",
          "name": "is-wsl",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "is-wsl",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-wsl",
      "name": "is-wsl",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/is-wsl",
      "resolved": "https://registry.npmjs.org/is-wsl/-/is-wsl-1.1.0.tgz",
      "version": "1.1.0",
    },
    "isarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/buffer",
          "name": "isarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unset-value/node_modules/has-value/node_modules/isobject",
          "name": "isarray",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isobject" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/braces",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/class-utils",
          "name": "isobject",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/define-property",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/has-value",
          "name": "isobject",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-plain-object",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object-visit",
          "name": "isobject",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.pick",
          "name": "isobject",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon-node",
          "name": "isobject",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unset-value",
          "name": "isobject",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isobject",
      "name": "isobject",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/isobject",
      "resolved": "https://registry.npmjs.org/isobject/-/isobject-3.0.1.tgz",
      "version": "3.0.1",
    },
    "json-parse-better-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "json-parse-better-errors",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/json-parse-better-errors",
      "peer": true,
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema-traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json3" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sockjs-client",
          "name": "json3",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/json3",
      "name": "json3",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/json3",
      "resolved": "https://registry.npmjs.org/json3/-/json3-3.3.3.tgz",
      "version": "3.3.3",
    },
    "json5" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/loader-utils",
          "name": "json5",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/json5",
      "peer": true,
      "resolved": "https://registry.npmjs.org/json5/-/json5-1.0.1.tgz",
      "version": "1.0.1",
    },
    "killable" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "killable",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/killable",
      "name": "killable",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/killable",
      "resolved": "https://registry.npmjs.org/killable/-/killable-1.0.1.tgz",
      "version": "1.0.1",
    },
    "kind-of" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/class-utils/node_modules/is-accessor-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/class-utils/node_modules/is-data-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/expand-brackets/node_modules/is-accessor-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/expand-brackets/node_modules/is-data-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-number",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object-copy",
          "name": "kind-of",
          "spec": "^3.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object-copy/node_modules/is-accessor-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object-copy/node_modules/is-data-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon-util",
          "name": "kind-of",
          "spec": "^3.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon/node_modules/is-accessor-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon/node_modules/is-data-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/static-extend/node_modules/is-accessor-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/static-extend/node_modules/is-data-descriptor",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-object-path",
          "name": "kind-of",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-buffer" => EdgeOut {
          "name": "is-buffer",
          "spec": "^1.1.5",
          "to": "node_modules/is-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/kind-of",
      "name": "kind-of",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/kind-of",
      "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-3.2.2.tgz",
      "version": "3.2.2",
    },
    "loader-runner" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "loader-runner",
          "spec": "^2.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/loader-runner",
      "name": "loader-runner",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/loader-runner",
      "peer": true,
      "resolved": "https://registry.npmjs.org/loader-runner/-/loader-runner-2.4.0.tgz",
      "version": "2.4.0",
    },
    "loader-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "loader-utils",
          "spec": "^1.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "big.js" => EdgeOut {
          "name": "big.js",
          "spec": "^5.2.2",
          "to": "node_modules/big.js",
          "type": "prod",
        },
        "emojis-list" => EdgeOut {
          "name": "emojis-list",
          "spec": "^3.0.0",
          "to": "node_modules/emojis-list",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^1.0.1",
          "to": "node_modules/json5",
          "type": "prod",
        },
      },
      "location": "node_modules/loader-utils",
      "name": "loader-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/loader-utils",
      "peer": true,
      "resolved": "https://registry.npmjs.org/loader-utils/-/loader-utils-1.4.0.tgz",
      "version": "1.4.0",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async",
          "name": "lodash",
          "spec": "^4.17.14",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-proxy-middleware",
          "name": "lodash",
          "spec": "^4.17.11",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "loglevel" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "loglevel",
          "spec": "^1.6.8",
          "type": "prod",
        },
      },
      "location": "node_modules/loglevel",
      "name": "loglevel",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/loglevel",
      "resolved": "https://registry.npmjs.org/loglevel/-/loglevel-1.7.0.tgz",
      "version": "1.7.0",
    },
    "lru-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "lru-cache",
          "spec": "^5.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^3.0.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/lru-cache",
      "peer": true,
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-5.1.1.tgz",
      "version": "5.1.1",
    },
    "make-dir" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/make-dir",
              "name": "semver",
              "spec": "^5.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/make-dir/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/make-dir/node_modules/semver",
          "peer": true,
          "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
          "version": "5.7.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/make-dir/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/make-dir",
      "peer": true,
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "map-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fragment-cache",
          "name": "map-cache",
          "spec": "^0.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/snapdragon",
          "name": "map-cache",
          "spec": "^0.2.2",
          "type": "prod",
        },
      },
      "location": "node_modules/map-cache",
      "name": "map-cache",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/map-cache",
      "resolved": "https://registry.npmjs.org/map-cache/-/map-cache-0.2.2.tgz",
      "version": "0.2.2",
    },
    "map-visit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/collection-visit",
          "name": "map-visit",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-visit" => EdgeOut {
          "name": "object-visit",
          "spec": "^1.0.0",
          "to": "node_modules/object-visit",
          "type": "prod",
        },
      },
      "location": "node_modules/map-visit",
      "name": "map-visit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/map-visit",
      "resolved": "https://registry.npmjs.org/map-visit/-/map-visit-1.0.0.tgz",
      "version": "1.0.0",
    },
    "md5.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-hash",
          "name": "md5.js",
          "spec": "^1.3.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/evp_bytestokey",
          "name": "md5.js",
          "spec": "^1.3.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hash-base" => EdgeOut {
          "name": "hash-base",
          "spec": "^3.0.0",
          "to": "node_modules/hash-base",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/md5.js",
      "name": "md5.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/md5.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/md5.js/-/md5.js-1.3.5.tgz",
      "version": "1.3.5",
    },
    "media-typer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/type-is",
          "name": "media-typer",
          "spec": "0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/media-typer",
      "name": "media-typer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/media-typer",
      "resolved": "https://registry.npmjs.org/media-typer/-/media-typer-0.3.0.tgz",
      "version": "0.3.0",
    },
    "memory-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "memory-fs",
          "spec": "^0.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-middleware",
          "name": "memory-fs",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "errno" => EdgeOut {
          "name": "errno",
          "spec": "^0.1.3",
          "to": "node_modules/errno",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.1",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/memory-fs",
      "name": "memory-fs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/memory-fs",
      "resolved": "https://registry.npmjs.org/memory-fs/-/memory-fs-0.4.1.tgz",
      "version": "0.4.1",
    },
    "merge-descriptors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "merge-descriptors",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-descriptors",
      "name": "merge-descriptors",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/merge-descriptors",
      "resolved": "https://registry.npmjs.org/merge-descriptors/-/merge-descriptors-1.0.1.tgz",
      "version": "1.0.1",
    },
    "methods" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "methods",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/methods",
      "name": "methods",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/methods",
      "resolved": "https://registry.npmjs.org/methods/-/methods-1.1.2.tgz",
      "version": "1.1.2",
    },
    "micromatch" => ArboristNode {
      "children": Map {
        "extend-shallow" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/micromatch",
              "name": "extend-shallow",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "assign-symbols" => EdgeOut {
              "name": "assign-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/assign-symbols",
              "type": "prod",
            },
            "is-extendable" => EdgeOut {
              "name": "is-extendable",
              "spec": "^1.0.1",
              "to": "node_modules/micromatch/node_modules/is-extendable",
              "type": "prod",
            },
          },
          "location": "node_modules/micromatch/node_modules/extend-shallow",
          "name": "extend-shallow",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/micromatch/node_modules/extend-shallow",
          "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
          "version": "3.0.2",
        },
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/micromatch/node_modules/extend-shallow",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/micromatch/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/micromatch/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/micromatch",
              "name": "kind-of",
              "spec": "^6.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/micromatch/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/micromatch/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
          "version": "6.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "micromatch",
          "spec": "^3.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-proxy-middleware",
          "name": "micromatch",
          "spec": "^3.1.10",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "micromatch",
          "spec": "^3.1.10",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "micromatch",
          "spec": "^3.1.10",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arr-diff" => EdgeOut {
          "name": "arr-diff",
          "spec": "^4.0.0",
          "to": "node_modules/arr-diff",
          "type": "prod",
        },
        "array-unique" => EdgeOut {
          "name": "array-unique",
          "spec": "^0.3.2",
          "to": "node_modules/array-unique",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "^2.3.1",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^2.0.2",
          "to": "node_modules/define-property",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^3.0.2",
          "to": "node_modules/micromatch/node_modules/extend-shallow",
          "type": "prod",
        },
        "extglob" => EdgeOut {
          "name": "extglob",
          "spec": "^2.0.4",
          "to": "node_modules/extglob",
          "type": "prod",
        },
        "fragment-cache" => EdgeOut {
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "to": "node_modules/fragment-cache",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.2",
          "to": "node_modules/micromatch/node_modules/kind-of",
          "type": "prod",
        },
        "nanomatch" => EdgeOut {
          "name": "nanomatch",
          "spec": "^1.2.9",
          "to": "node_modules/nanomatch",
          "type": "prod",
        },
        "object.pick" => EdgeOut {
          "name": "object.pick",
          "spec": "^1.3.0",
          "to": "node_modules/object.pick",
          "type": "prod",
        },
        "regex-not" => EdgeOut {
          "name": "regex-not",
          "spec": "^1.0.0",
          "to": "node_modules/regex-not",
          "type": "prod",
        },
        "snapdragon" => EdgeOut {
          "name": "snapdragon",
          "spec": "^0.8.1",
          "to": "node_modules/snapdragon",
          "type": "prod",
        },
        "to-regex" => EdgeOut {
          "name": "to-regex",
          "spec": "^3.0.2",
          "to": "node_modules/to-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/micromatch",
      "name": "micromatch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/micromatch",
      "resolved": "https://registry.npmjs.org/micromatch/-/micromatch-3.1.10.tgz",
      "version": "3.1.10",
    },
    "miller-rabin" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/miller-rabin",
              "name": "bn.js",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/miller-rabin/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/miller-rabin/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/diffie-hellman",
          "name": "miller-rabin",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.0.0",
          "to": "node_modules/miller-rabin/node_modules/bn.js",
          "type": "prod",
        },
        "brorand" => EdgeOut {
          "name": "brorand",
          "spec": "^1.0.1",
          "to": "node_modules/brorand",
          "type": "prod",
        },
      },
      "location": "node_modules/miller-rabin",
      "name": "miller-rabin",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/miller-rabin",
      "peer": true,
      "resolved": "https://registry.npmjs.org/miller-rabin/-/miller-rabin-4.0.1.tgz",
      "version": "4.0.1",
    },
    "mime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/send",
          "name": "mime",
          "spec": "1.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime",
      "name": "mime",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mime",
      "resolved": "https://registry.npmjs.org/mime/-/mime-1.6.0.tgz",
      "version": "1.6.0",
    },
    "mime-db" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compressible",
          "name": "mime-db",
          "spec": ">= 1.43.0 < 2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.44.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.44.0.tgz",
      "version": "1.44.0",
    },
    "mime-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/accepts",
          "name": "mime-types",
          "spec": "~2.1.24",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index",
          "name": "mime-types",
          "spec": "~2.1.17",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/type-is",
          "name": "mime-types",
          "spec": "~2.1.24",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.44.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.27.tgz",
      "version": "2.1.27",
    },
    "minimalistic-assert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1.js",
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/des.js",
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash.js",
          "name": "minimalistic-assert",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hmac-drbg",
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wbuf",
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/minimalistic-assert",
      "name": "minimalistic-assert",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/minimalistic-assert",
      "resolved": "https://registry.npmjs.org/minimalistic-assert/-/minimalistic-assert-1.0.1.tgz",
      "version": "1.0.1",
    },
    "minimalistic-crypto-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/elliptic",
          "name": "minimalistic-crypto-utils",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hmac-drbg",
          "name": "minimalistic-crypto-utils",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/minimalistic-crypto-utils",
      "name": "minimalistic-crypto-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/minimalistic-crypto-utils",
      "peer": true,
      "resolved": "https://registry.npmjs.org/minimalistic-crypto-utils/-/minimalistic-crypto-utils-1.0.1.tgz",
      "version": "1.0.1",
    },
    "minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mississippi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "mississippi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "concat-stream" => EdgeOut {
          "name": "concat-stream",
          "spec": "^1.5.0",
          "to": "node_modules/concat-stream",
          "type": "prod",
        },
        "duplexify" => EdgeOut {
          "name": "duplexify",
          "spec": "^3.4.2",
          "to": "node_modules/duplexify",
          "type": "prod",
        },
        "end-of-stream" => EdgeOut {
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "to": "node_modules/end-of-stream",
          "type": "prod",
        },
        "flush-write-stream" => EdgeOut {
          "name": "flush-write-stream",
          "spec": "^1.0.0",
          "to": "node_modules/flush-write-stream",
          "type": "prod",
        },
        "from2" => EdgeOut {
          "name": "from2",
          "spec": "^2.1.0",
          "to": "node_modules/from2",
          "type": "prod",
        },
        "parallel-transform" => EdgeOut {
          "name": "parallel-transform",
          "spec": "^1.1.0",
          "to": "node_modules/parallel-transform",
          "type": "prod",
        },
        "pump" => EdgeOut {
          "name": "pump",
          "spec": "^3.0.0",
          "to": "node_modules/pump",
          "type": "prod",
        },
        "pumpify" => EdgeOut {
          "name": "pumpify",
          "spec": "^1.3.3",
          "to": "node_modules/pumpify",
          "type": "prod",
        },
        "stream-each" => EdgeOut {
          "name": "stream-each",
          "spec": "^1.1.0",
          "to": "node_modules/stream-each",
          "type": "prod",
        },
        "through2" => EdgeOut {
          "name": "through2",
          "spec": "^2.0.0",
          "to": "node_modules/through2",
          "type": "prod",
        },
      },
      "location": "node_modules/mississippi",
      "name": "mississippi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mississippi",
      "peer": true,
      "resolved": "https://registry.npmjs.org/mississippi/-/mississippi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "mixin-deep" => ArboristNode {
      "children": Map {
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mixin-deep",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/mixin-deep/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mixin-deep/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "mixin-deep",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "for-in" => EdgeOut {
          "name": "for-in",
          "spec": "^1.0.2",
          "to": "node_modules/for-in",
          "type": "prod",
        },
        "is-extendable" => EdgeOut {
          "name": "is-extendable",
          "spec": "^1.0.1",
          "to": "node_modules/mixin-deep/node_modules/is-extendable",
          "type": "prod",
        },
      },
      "location": "node_modules/mixin-deep",
      "name": "mixin-deep",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mixin-deep",
      "resolved": "https://registry.npmjs.org/mixin-deep/-/mixin-deep-1.3.2.tgz",
      "version": "1.3.2",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/portfinder",
          "name": "mkdirp",
          "spec": "^0.5.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "mkdirp",
          "spec": "^0.5.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-middleware",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "move-concurrently" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "move-concurrently",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aproba" => EdgeOut {
          "name": "aproba",
          "spec": "^1.1.1",
          "to": "node_modules/aproba",
          "type": "prod",
        },
        "copy-concurrently" => EdgeOut {
          "name": "copy-concurrently",
          "spec": "^1.0.0",
          "to": "node_modules/copy-concurrently",
          "type": "prod",
        },
        "fs-write-stream-atomic" => EdgeOut {
          "name": "fs-write-stream-atomic",
          "spec": "^1.0.8",
          "to": "node_modules/fs-write-stream-atomic",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.5.4",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "run-queue" => EdgeOut {
          "name": "run-queue",
          "spec": "^1.0.3",
          "to": "node_modules/run-queue",
          "type": "prod",
        },
      },
      "location": "node_modules/move-concurrently",
      "name": "move-concurrently",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/move-concurrently",
      "peer": true,
      "resolved": "https://registry.npmjs.org/move-concurrently/-/move-concurrently-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/portfinder/node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sockjs-client/node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "multicast-dns" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "multicast-dns",
          "spec": "^6.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "dns-packet" => EdgeOut {
          "name": "dns-packet",
          "spec": "^1.3.1",
          "to": "node_modules/dns-packet",
          "type": "prod",
        },
        "thunky" => EdgeOut {
          "name": "thunky",
          "spec": "^1.0.2",
          "to": "node_modules/thunky",
          "type": "prod",
        },
      },
      "location": "node_modules/multicast-dns",
      "name": "multicast-dns",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/multicast-dns",
      "resolved": "https://registry.npmjs.org/multicast-dns/-/multicast-dns-6.2.3.tgz",
      "version": "6.2.3",
    },
    "multicast-dns-service-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bonjour",
          "name": "multicast-dns-service-types",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/multicast-dns-service-types",
      "name": "multicast-dns-service-types",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/multicast-dns-service-types",
      "resolved": "https://registry.npmjs.org/multicast-dns-service-types/-/multicast-dns-service-types-1.1.0.tgz",
      "version": "1.1.0",
    },
    "nan" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fsevents",
          "name": "nan",
          "spec": "^2.12.1",
          "type": "prod",
        },
      },
      "location": "node_modules/nan",
      "name": "nan",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nan",
      "resolved": "https://registry.npmjs.org/nan/-/nan-2.14.2.tgz",
      "version": "2.14.2",
    },
    "nanomatch" => ArboristNode {
      "children": Map {
        "extend-shallow" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nanomatch",
              "name": "extend-shallow",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "assign-symbols" => EdgeOut {
              "name": "assign-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/assign-symbols",
              "type": "prod",
            },
            "is-extendable" => EdgeOut {
              "name": "is-extendable",
              "spec": "^1.0.1",
              "to": "node_modules/nanomatch/node_modules/is-extendable",
              "type": "prod",
            },
          },
          "location": "node_modules/nanomatch/node_modules/extend-shallow",
          "name": "extend-shallow",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nanomatch/node_modules/extend-shallow",
          "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
          "version": "3.0.2",
        },
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nanomatch/node_modules/extend-shallow",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/nanomatch/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nanomatch/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
        "kind-of" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nanomatch",
              "name": "kind-of",
              "spec": "^6.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/nanomatch/node_modules/kind-of",
          "name": "kind-of",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nanomatch/node_modules/kind-of",
          "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.3.tgz",
          "version": "6.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "nanomatch",
          "spec": "^1.2.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arr-diff" => EdgeOut {
          "name": "arr-diff",
          "spec": "^4.0.0",
          "to": "node_modules/arr-diff",
          "type": "prod",
        },
        "array-unique" => EdgeOut {
          "name": "array-unique",
          "spec": "^0.3.2",
          "to": "node_modules/array-unique",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^2.0.2",
          "to": "node_modules/define-property",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^3.0.2",
          "to": "node_modules/nanomatch/node_modules/extend-shallow",
          "type": "prod",
        },
        "fragment-cache" => EdgeOut {
          "name": "fragment-cache",
          "spec": "^0.2.1",
          "to": "node_modules/fragment-cache",
          "type": "prod",
        },
        "is-windows" => EdgeOut {
          "name": "is-windows",
          "spec": "^1.0.2",
          "to": "node_modules/is-windows",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^6.0.2",
          "to": "node_modules/nanomatch/node_modules/kind-of",
          "type": "prod",
        },
        "object.pick" => EdgeOut {
          "name": "object.pick",
          "spec": "^1.3.0",
          "to": "node_modules/object.pick",
          "type": "prod",
        },
        "regex-not" => EdgeOut {
          "name": "regex-not",
          "spec": "^1.0.0",
          "to": "node_modules/regex-not",
          "type": "prod",
        },
        "snapdragon" => EdgeOut {
          "name": "snapdragon",
          "spec": "^0.8.1",
          "to": "node_modules/snapdragon",
          "type": "prod",
        },
        "to-regex" => EdgeOut {
          "name": "to-regex",
          "spec": "^3.0.1",
          "to": "node_modules/to-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/nanomatch",
      "name": "nanomatch",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nanomatch",
      "resolved": "https://registry.npmjs.org/nanomatch/-/nanomatch-1.2.13.tgz",
      "version": "1.2.13",
    },
    "native-url" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "native-url",
          "spec": "^0.2.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "querystring" => EdgeOut {
          "name": "querystring",
          "spec": "^0.2.0",
          "to": "node_modules/querystring",
          "type": "prod",
        },
      },
      "location": "node_modules/native-url",
      "name": "native-url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/native-url",
      "resolved": "https://registry.npmjs.org/native-url/-/native-url-0.2.6.tgz",
      "version": "0.2.6",
    },
    "negotiator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/accepts",
          "name": "negotiator",
          "spec": "0.6.2",
          "type": "prod",
        },
      },
      "location": "node_modules/negotiator",
      "name": "negotiator",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/negotiator",
      "resolved": "https://registry.npmjs.org/negotiator/-/negotiator-0.6.2.tgz",
      "version": "0.6.2",
    },
    "neo-async" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/watchpack",
          "name": "neo-async",
          "spec": "^2.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "neo-async",
          "spec": "^2.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/neo-async",
      "peer": true,
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.2.tgz",
      "version": "2.6.2",
    },
    "nice-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-forge" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/selfsigned",
          "name": "node-forge",
          "spec": "^0.10.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-forge",
      "name": "node-forge",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/node-forge",
      "resolved": "https://registry.npmjs.org/node-forge/-/node-forge-0.10.0.tgz",
      "version": "0.10.0",
    },
    "node-libs-browser" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/node-libs-browser",
              "name": "punycode",
              "spec": "^1.2.4",
              "type": "prod",
            },
          },
          "location": "node_modules/node-libs-browser/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/node-libs-browser/node_modules/punycode",
          "peer": true,
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "node-libs-browser",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert" => EdgeOut {
          "name": "assert",
          "spec": "^1.1.1",
          "to": "node_modules/assert",
          "type": "prod",
        },
        "browserify-zlib" => EdgeOut {
          "name": "browserify-zlib",
          "spec": "^0.2.0",
          "to": "node_modules/browserify-zlib",
          "type": "prod",
        },
        "buffer" => EdgeOut {
          "name": "buffer",
          "spec": "^4.3.0",
          "to": "node_modules/buffer",
          "type": "prod",
        },
        "console-browserify" => EdgeOut {
          "name": "console-browserify",
          "spec": "^1.1.0",
          "to": "node_modules/console-browserify",
          "type": "prod",
        },
        "constants-browserify" => EdgeOut {
          "name": "constants-browserify",
          "spec": "^1.0.0",
          "to": "node_modules/constants-browserify",
          "type": "prod",
        },
        "crypto-browserify" => EdgeOut {
          "name": "crypto-browserify",
          "spec": "^3.11.0",
          "to": "node_modules/crypto-browserify",
          "type": "prod",
        },
        "domain-browser" => EdgeOut {
          "name": "domain-browser",
          "spec": "^1.1.1",
          "to": "node_modules/domain-browser",
          "type": "prod",
        },
        "events" => EdgeOut {
          "name": "events",
          "spec": "^3.0.0",
          "to": "node_modules/events",
          "type": "prod",
        },
        "https-browserify" => EdgeOut {
          "name": "https-browserify",
          "spec": "^1.0.0",
          "to": "node_modules/https-browserify",
          "type": "prod",
        },
        "os-browserify" => EdgeOut {
          "name": "os-browserify",
          "spec": "^0.3.0",
          "to": "node_modules/os-browserify",
          "type": "prod",
        },
        "path-browserify" => EdgeOut {
          "name": "path-browserify",
          "spec": "0.0.1",
          "to": "node_modules/path-browserify",
          "type": "prod",
        },
        "process" => EdgeOut {
          "name": "process",
          "spec": "^0.11.10",
          "to": "node_modules/process",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.2.4",
          "to": "node_modules/node-libs-browser/node_modules/punycode",
          "type": "prod",
        },
        "querystring-es3" => EdgeOut {
          "name": "querystring-es3",
          "spec": "^0.2.0",
          "to": "node_modules/querystring-es3",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.3.3",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "stream-browserify" => EdgeOut {
          "name": "stream-browserify",
          "spec": "^2.0.1",
          "to": "node_modules/stream-browserify",
          "type": "prod",
        },
        "stream-http" => EdgeOut {
          "name": "stream-http",
          "spec": "^2.7.2",
          "to": "node_modules/stream-http",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "^1.0.0",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "timers-browserify" => EdgeOut {
          "name": "timers-browserify",
          "spec": "^2.0.4",
          "to": "node_modules/timers-browserify",
          "type": "prod",
        },
        "tty-browserify" => EdgeOut {
          "name": "tty-browserify",
          "spec": "0.0.0",
          "to": "node_modules/tty-browserify",
          "type": "prod",
        },
        "url" => EdgeOut {
          "name": "url",
          "spec": "^0.11.0",
          "to": "node_modules/url",
          "type": "prod",
        },
        "util" => EdgeOut {
          "name": "util",
          "spec": "^0.11.0",
          "to": "node_modules/util",
          "type": "prod",
        },
        "vm-browserify" => EdgeOut {
          "name": "vm-browserify",
          "spec": "^1.0.1",
          "to": "node_modules/vm-browserify",
          "type": "prod",
        },
      },
      "location": "node_modules/node-libs-browser",
      "name": "node-libs-browser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/node-libs-browser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/node-libs-browser/-/node-libs-browser-2.2.1.tgz",
      "version": "2.2.1",
    },
    "normalize-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "npm-run-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "npm-run-path",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-key" => EdgeOut {
          "name": "path-key",
          "spec": "^2.0.0",
          "to": "node_modules/path-key",
          "type": "prod",
        },
      },
      "location": "node_modules/npm-run-path",
      "name": "npm-run-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/npm-run-path",
      "resolved": "https://registry.npmjs.org/npm-run-path/-/npm-run-path-2.0.2.tgz",
      "version": "2.0.2",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/assert",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "object-copy" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object-copy",
              "name": "define-property",
              "spec": "^0.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "to": "node_modules/object-copy/node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/object-copy/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
          "version": "0.2.5",
        },
        "is-accessor-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object-copy/node_modules/is-descriptor",
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/object-copy/node_modules/is-accessor-descriptor",
          "name": "is-accessor-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy/node_modules/is-accessor-descriptor",
          "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "is-data-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object-copy/node_modules/is-descriptor",
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/object-copy/node_modules/is-data-descriptor",
          "name": "is-data-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy/node_modules/is-data-descriptor",
          "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
          "version": "0.1.4",
        },
        "is-descriptor" => ArboristNode {
          "children": Map {
            "kind-of" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/object-copy/node_modules/is-descriptor",
                  "name": "kind-of",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/object-copy/node_modules/is-descriptor/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy/node_modules/is-descriptor/node_modules/kind-of",
              "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/object-copy/node_modules/define-property",
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-accessor-descriptor" => EdgeOut {
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "to": "node_modules/object-copy/node_modules/is-accessor-descriptor",
              "type": "prod",
            },
            "is-data-descriptor" => EdgeOut {
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "to": "node_modules/object-copy/node_modules/is-data-descriptor",
              "type": "prod",
            },
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^5.0.0",
              "to": "node_modules/object-copy/node_modules/is-descriptor/node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/object-copy/node_modules/is-descriptor",
          "name": "is-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy/node_modules/is-descriptor",
          "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/static-extend",
          "name": "object-copy",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "copy-descriptor" => EdgeOut {
          "name": "copy-descriptor",
          "spec": "^0.1.0",
          "to": "node_modules/copy-descriptor",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^0.2.5",
          "to": "node_modules/object-copy/node_modules/define-property",
          "type": "prod",
        },
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^3.0.3",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/object-copy",
      "name": "object-copy",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-copy",
      "resolved": "https://registry.npmjs.org/object-copy/-/object-copy-0.1.0.tgz",
      "version": "0.1.0",
    },
    "object-inspect" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object-inspect",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-inspect",
      "name": "object-inspect",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-inspect",
      "resolved": "https://registry.npmjs.org/object-inspect/-/object-inspect-1.8.0.tgz",
      "version": "1.8.0",
    },
    "object-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "object-is",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.18.0-next.1",
          "to": "node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/object-is",
      "name": "object-is",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-is",
      "resolved": "https://registry.npmjs.org/object-is/-/object-is-1.1.3.tgz",
      "version": "1.1.3",
    },
    "object-keys" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/define-properties",
          "name": "object-keys",
          "spec": "^1.0.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/object.assign",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object-keys",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-keys",
      "name": "object-keys",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-keys",
      "resolved": "https://registry.npmjs.org/object-keys/-/object-keys-1.1.1.tgz",
      "version": "1.1.1",
    },
    "object-visit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/collection-visit",
          "name": "object-visit",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/map-visit",
          "name": "object-visit",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.0",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/object-visit",
      "name": "object-visit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object-visit",
      "resolved": "https://registry.npmjs.org/object-visit/-/object-visit-1.0.1.tgz",
      "version": "1.0.1",
    },
    "object.assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "object.assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.18.0-next.0",
          "to": "node_modules/es-abstract",
          "type": "prod",
        },
        "has-symbols" => EdgeOut {
          "name": "has-symbols",
          "spec": "^1.0.1",
          "to": "node_modules/has-symbols",
          "type": "prod",
        },
        "object-keys" => EdgeOut {
          "name": "object-keys",
          "spec": "^1.1.1",
          "to": "node_modules/object-keys",
          "type": "prod",
        },
      },
      "location": "node_modules/object.assign",
      "name": "object.assign",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object.assign",
      "resolved": "https://registry.npmjs.org/object.assign/-/object.assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "object.pick" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "object.pick",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "object.pick",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.1",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/object.pick",
      "name": "object.pick",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/object.pick",
      "resolved": "https://registry.npmjs.org/object.pick/-/object.pick-1.3.0.tgz",
      "version": "1.3.0",
    },
    "obuf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hpack.js",
          "name": "obuf",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport",
          "name": "obuf",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/obuf",
      "name": "obuf",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/obuf",
      "resolved": "https://registry.npmjs.org/obuf/-/obuf-1.1.2.tgz",
      "version": "1.1.2",
    },
    "on-finished" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "on-finished",
          "spec": "~2.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "on-finished",
          "spec": "~2.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "on-finished",
          "spec": "~2.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "on-finished",
          "spec": "~2.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ee-first" => EdgeOut {
          "name": "ee-first",
          "spec": "1.1.1",
          "to": "node_modules/ee-first",
          "type": "prod",
        },
      },
      "location": "node_modules/on-finished",
      "name": "on-finished",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/on-finished",
      "resolved": "https://registry.npmjs.org/on-finished/-/on-finished-2.3.0.tgz",
      "version": "2.3.0",
    },
    "on-headers" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compression",
          "name": "on-headers",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/on-headers",
      "name": "on-headers",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/on-headers",
      "resolved": "https://registry.npmjs.org/on-headers/-/on-headers-1.0.2.tgz",
      "version": "1.0.2",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/end-of-stream",
          "name": "once",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pump",
          "name": "once",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pumpify/node_modules/pump",
          "name": "once",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "opn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "opn",
          "spec": "^5.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-wsl" => EdgeOut {
          "name": "is-wsl",
          "spec": "^1.1.0",
          "to": "node_modules/is-wsl",
          "type": "prod",
        },
      },
      "location": "node_modules/opn",
      "name": "opn",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/opn",
      "resolved": "https://registry.npmjs.org/opn/-/opn-5.5.0.tgz",
      "version": "5.5.0",
    },
    "original" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eventsource",
          "name": "original",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "url-parse" => EdgeOut {
          "name": "url-parse",
          "spec": "^1.4.3",
          "to": "node_modules/url-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/original",
      "name": "original",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/original",
      "resolved": "https://registry.npmjs.org/original/-/original-1.0.2.tgz",
      "version": "1.0.2",
    },
    "os-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "os-browserify",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/os-browserify",
      "name": "os-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/os-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/os-browserify/-/os-browserify-0.3.0.tgz",
      "version": "0.3.0",
    },
    "p-finally" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "p-finally",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-finally",
      "name": "p-finally",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-finally",
      "resolved": "https://registry.npmjs.org/p-finally/-/p-finally-1.0.0.tgz",
      "version": "1.0.0",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
      "version": "2.3.0",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "p-map",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-map",
      "name": "p-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-map",
      "resolved": "https://registry.npmjs.org/p-map/-/p-map-2.1.0.tgz",
      "version": "2.1.0",
    },
    "p-retry" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "p-retry",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "retry" => EdgeOut {
          "name": "retry",
          "spec": "^0.12.0",
          "to": "node_modules/retry",
          "type": "prod",
        },
      },
      "location": "node_modules/p-retry",
      "name": "p-retry",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-retry",
      "resolved": "https://registry.npmjs.org/p-retry/-/p-retry-3.0.1.tgz",
      "version": "3.0.1",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "pako" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-zlib",
          "name": "pako",
          "spec": "~1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/pako",
      "name": "pako",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pako",
      "peer": true,
      "resolved": "https://registry.npmjs.org/pako/-/pako-1.0.11.tgz",
      "version": "1.0.11",
    },
    "parallel-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "parallel-transform",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cyclist" => EdgeOut {
          "name": "cyclist",
          "spec": "^1.0.1",
          "to": "node_modules/cyclist",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/parallel-transform",
      "name": "parallel-transform",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/parallel-transform",
      "peer": true,
      "resolved": "https://registry.npmjs.org/parallel-transform/-/parallel-transform-1.2.0.tgz",
      "version": "1.2.0",
    },
    "parse-asn1" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign",
          "name": "parse-asn1",
          "spec": "^5.1.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/public-encrypt",
          "name": "parse-asn1",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1.js" => EdgeOut {
          "name": "asn1.js",
          "spec": "^5.2.0",
          "to": "node_modules/asn1.js",
          "type": "prod",
        },
        "browserify-aes" => EdgeOut {
          "name": "browserify-aes",
          "spec": "^1.0.0",
          "to": "node_modules/browserify-aes",
          "type": "prod",
        },
        "evp_bytestokey" => EdgeOut {
          "name": "evp_bytestokey",
          "spec": "^1.0.0",
          "to": "node_modules/evp_bytestokey",
          "type": "prod",
        },
        "pbkdf2" => EdgeOut {
          "name": "pbkdf2",
          "spec": "^3.0.3",
          "to": "node_modules/pbkdf2",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-asn1",
      "name": "parse-asn1",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/parse-asn1",
      "peer": true,
      "resolved": "https://registry.npmjs.org/parse-asn1/-/parse-asn1-5.1.6.tgz",
      "version": "5.1.6",
    },
    "parseurl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "parseurl",
          "spec": "~1.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "parseurl",
          "spec": "~1.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index",
          "name": "parseurl",
          "spec": "~1.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-static",
          "name": "parseurl",
          "spec": "~1.3.3",
          "type": "prod",
        },
      },
      "location": "node_modules/parseurl",
      "name": "parseurl",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/parseurl",
      "resolved": "https://registry.npmjs.org/parseurl/-/parseurl-1.3.3.tgz",
      "version": "1.3.3",
    },
    "pascalcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/base",
          "name": "pascalcase",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pascalcase",
      "name": "pascalcase",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pascalcase",
      "resolved": "https://registry.npmjs.org/pascalcase/-/pascalcase-0.1.1.tgz",
      "version": "0.1.1",
    },
    "path-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "path-browserify",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-browserify",
      "name": "path-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/path-browserify/-/path-browserify-0.0.1.tgz",
      "version": "0.0.1",
    },
    "path-dirname" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "path-dirname",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-dirname",
      "name": "path-dirname",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-dirname",
      "resolved": "https://registry.npmjs.org/path-dirname/-/path-dirname-1.0.2.tgz",
      "version": "1.0.2",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-is-inside" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-path-inside",
          "name": "path-is-inside",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-inside",
      "name": "path-is-inside",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-is-inside",
      "resolved": "https://registry.npmjs.org/path-is-inside/-/path-is-inside-1.0.2.tgz",
      "version": "1.0.2",
    },
    "path-key" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/npm-run-path",
          "name": "path-key",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-to-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "path-to-regexp",
          "spec": "0.1.7",
          "type": "prod",
        },
      },
      "location": "node_modules/path-to-regexp",
      "name": "path-to-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/path-to-regexp",
      "resolved": "https://registry.npmjs.org/path-to-regexp/-/path-to-regexp-0.1.7.tgz",
      "version": "0.1.7",
    },
    "pbkdf2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "pbkdf2",
          "spec": "^3.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-asn1",
          "name": "pbkdf2",
          "spec": "^3.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.1.2",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "create-hmac" => EdgeOut {
          "name": "create-hmac",
          "spec": "^1.1.4",
          "to": "node_modules/create-hmac",
          "type": "prod",
        },
        "ripemd160" => EdgeOut {
          "name": "ripemd160",
          "spec": "^2.0.1",
          "to": "node_modules/ripemd160",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "sha.js" => EdgeOut {
          "name": "sha.js",
          "spec": "^2.4.8",
          "to": "node_modules/sha.js",
          "type": "prod",
        },
      },
      "location": "node_modules/pbkdf2",
      "name": "pbkdf2",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pbkdf2",
      "peer": true,
      "resolved": "https://registry.npmjs.org/pbkdf2/-/pbkdf2-3.1.1.tgz",
      "version": "3.1.1",
    },
    "picomatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/watchpack/node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/picomatch",
      "peer": true,
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.2.2.tgz",
      "version": "2.2.2",
    },
    "pify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pinkie" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pinkie-promise",
          "name": "pinkie",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie",
      "name": "pinkie",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pinkie",
      "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
      "version": "2.0.4",
    },
    "pinkie-promise" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pinkie" => EdgeOut {
          "name": "pinkie",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie-promise",
      "name": "pinkie-promise",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pinkie-promise",
      "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
      "version": "2.0.1",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/import-local",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "portfinder" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/portfinder",
              "name": "debug",
              "spec": "^3.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/portfinder/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/portfinder/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-3.2.6.tgz",
          "version": "3.2.6",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "portfinder",
          "spec": "^1.0.26",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "async" => EdgeOut {
          "name": "async",
          "spec": "^2.6.2",
          "to": "node_modules/async",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^3.1.1",
          "to": "node_modules/portfinder/node_modules/debug",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.5",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/portfinder",
      "name": "portfinder",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/portfinder",
      "resolved": "https://registry.npmjs.org/portfinder/-/portfinder-1.0.28.tgz",
      "version": "1.0.28",
    },
    "posix-character-classes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/expand-brackets",
          "name": "posix-character-classes",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/posix-character-classes",
      "name": "posix-character-classes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/posix-character-classes",
      "resolved": "https://registry.npmjs.org/posix-character-classes/-/posix-character-classes-0.1.1.tgz",
      "version": "0.1.1",
    },
    "process" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "process",
          "spec": "^0.11.10",
          "type": "prod",
        },
      },
      "location": "node_modules/process",
      "name": "process",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/process",
      "peer": true,
      "resolved": "https://registry.npmjs.org/process/-/process-0.11.10.tgz",
      "version": "0.11.10",
    },
    "process-nextick-args" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.1.tgz",
      "version": "2.0.1",
    },
    "promise-inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "promise-inflight",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/promise-inflight",
      "name": "promise-inflight",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/promise-inflight",
      "peer": true,
      "resolved": "https://registry.npmjs.org/promise-inflight/-/promise-inflight-1.0.1.tgz",
      "version": "1.0.1",
    },
    "proxy-addr" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "proxy-addr",
          "spec": "~2.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "forwarded" => EdgeOut {
          "name": "forwarded",
          "spec": "~0.1.2",
          "to": "node_modules/forwarded",
          "type": "prod",
        },
        "ipaddr.js" => EdgeOut {
          "name": "ipaddr.js",
          "spec": "1.9.1",
          "to": "node_modules/ipaddr.js",
          "type": "prod",
        },
      },
      "location": "node_modules/proxy-addr",
      "name": "proxy-addr",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/proxy-addr",
      "resolved": "https://registry.npmjs.org/proxy-addr/-/proxy-addr-2.0.6.tgz",
      "version": "2.0.6",
    },
    "prr" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/errno",
          "name": "prr",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/prr",
      "name": "prr",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/prr",
      "resolved": "https://registry.npmjs.org/prr/-/prr-1.0.1.tgz",
      "version": "1.0.1",
    },
    "public-encrypt" => ArboristNode {
      "children": Map {
        "bn.js" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/public-encrypt",
              "name": "bn.js",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/public-encrypt/node_modules/bn.js",
          "name": "bn.js",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/public-encrypt/node_modules/bn.js",
          "peer": true,
          "resolved": "https://registry.npmjs.org/bn.js/-/bn.js-4.11.9.tgz",
          "version": "4.11.9",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "public-encrypt",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bn.js" => EdgeOut {
          "name": "bn.js",
          "spec": "^4.1.0",
          "to": "node_modules/public-encrypt/node_modules/bn.js",
          "type": "prod",
        },
        "browserify-rsa" => EdgeOut {
          "name": "browserify-rsa",
          "spec": "^4.0.0",
          "to": "node_modules/browserify-rsa",
          "type": "prod",
        },
        "create-hash" => EdgeOut {
          "name": "create-hash",
          "spec": "^1.1.0",
          "to": "node_modules/create-hash",
          "type": "prod",
        },
        "parse-asn1" => EdgeOut {
          "name": "parse-asn1",
          "spec": "^5.0.0",
          "to": "node_modules/parse-asn1",
          "type": "prod",
        },
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.0.1",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/public-encrypt",
      "name": "public-encrypt",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/public-encrypt",
      "peer": true,
      "resolved": "https://registry.npmjs.org/public-encrypt/-/public-encrypt-4.0.3.tgz",
      "version": "4.0.3",
    },
    "pump" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/get-stream",
          "name": "pump",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "pump",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "end-of-stream" => EdgeOut {
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "to": "node_modules/end-of-stream",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.1",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/pump",
      "name": "pump",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pump",
      "resolved": "https://registry.npmjs.org/pump/-/pump-3.0.0.tgz",
      "version": "3.0.0",
    },
    "pumpify" => ArboristNode {
      "children": Map {
        "pump" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pumpify",
              "name": "pump",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "end-of-stream" => EdgeOut {
              "name": "end-of-stream",
              "spec": "^1.1.0",
              "to": "node_modules/end-of-stream",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.1",
              "to": "node_modules/once",
              "type": "prod",
            },
          },
          "location": "node_modules/pumpify/node_modules/pump",
          "name": "pump",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pumpify/node_modules/pump",
          "peer": true,
          "resolved": "https://registry.npmjs.org/pump/-/pump-2.0.1.tgz",
          "version": "2.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "pumpify",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "duplexify" => EdgeOut {
          "name": "duplexify",
          "spec": "^3.6.0",
          "to": "node_modules/duplexify",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "pump" => EdgeOut {
          "name": "pump",
          "spec": "^2.0.0",
          "to": "node_modules/pumpify/node_modules/pump",
          "type": "prod",
        },
      },
      "location": "node_modules/pumpify",
      "name": "pumpify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/pumpify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/pumpify/-/pumpify-1.5.1.tgz",
      "version": "1.5.1",
    },
    "punycode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "qs",
          "spec": "6.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "qs",
          "spec": "6.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.7.0.tgz",
      "version": "6.7.0",
    },
    "querystring" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/native-url",
          "name": "querystring",
          "spec": "^0.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/url",
          "name": "querystring",
          "spec": "0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/querystring",
      "name": "querystring",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/querystring",
      "resolved": "https://registry.npmjs.org/querystring/-/querystring-0.2.0.tgz",
      "version": "0.2.0",
    },
    "querystring-es3" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "querystring-es3",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/querystring-es3",
      "name": "querystring-es3",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/querystring-es3",
      "peer": true,
      "resolved": "https://registry.npmjs.org/querystring-es3/-/querystring-es3-0.2.1.tgz",
      "version": "0.2.1",
    },
    "querystringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/url-parse",
          "name": "querystringify",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/querystringify",
      "name": "querystringify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/querystringify",
      "resolved": "https://registry.npmjs.org/querystringify/-/querystringify-2.2.0.tgz",
      "version": "2.2.0",
    },
    "randombytes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-rsa",
          "name": "randombytes",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "randombytes",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/diffie-hellman",
          "name": "randombytes",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/public-encrypt",
          "name": "randombytes",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/randomfill",
          "name": "randombytes",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serialize-javascript",
          "name": "randombytes",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/randombytes",
      "name": "randombytes",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/randombytes",
      "peer": true,
      "resolved": "https://registry.npmjs.org/randombytes/-/randombytes-2.1.0.tgz",
      "version": "2.1.0",
    },
    "randomfill" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/crypto-browserify",
          "name": "randomfill",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.0.5",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/randomfill",
      "name": "randomfill",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/randomfill",
      "peer": true,
      "resolved": "https://registry.npmjs.org/randomfill/-/randomfill-1.0.4.tgz",
      "version": "1.0.4",
    },
    "range-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "range-parser",
          "spec": "~1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "range-parser",
          "spec": "~1.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-middleware",
          "name": "range-parser",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/range-parser",
      "name": "range-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/range-parser",
      "resolved": "https://registry.npmjs.org/range-parser/-/range-parser-1.2.1.tgz",
      "version": "1.2.1",
    },
    "raw-body" => ArboristNode {
      "children": Map {
        "bytes" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/raw-body",
              "name": "bytes",
              "spec": "3.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/raw-body/node_modules/bytes",
          "name": "bytes",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/raw-body/node_modules/bytes",
          "resolved": "https://registry.npmjs.org/bytes/-/bytes-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "raw-body",
          "spec": "2.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bytes" => EdgeOut {
          "name": "bytes",
          "spec": "3.1.0",
          "to": "node_modules/raw-body/node_modules/bytes",
          "type": "prod",
        },
        "http-errors" => EdgeOut {
          "name": "http-errors",
          "spec": "1.7.2",
          "to": "node_modules/http-errors",
          "type": "prod",
        },
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "0.4.24",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
        "unpipe" => EdgeOut {
          "name": "unpipe",
          "spec": "1.0.0",
          "to": "node_modules/unpipe",
          "type": "prod",
        },
      },
      "location": "node_modules/raw-body",
      "name": "raw-body",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/raw-body",
      "resolved": "https://registry.npmjs.org/raw-body/-/raw-body-2.4.0.tgz",
      "version": "2.4.0",
    },
    "react-refresh" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "react-refresh",
          "spec": ">=0.8.3 <0.10.0",
          "type": "peer",
        },
      },
      "location": "node_modules/react-refresh",
      "name": "react-refresh",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/react-refresh",
      "peer": true,
      "resolved": "https://registry.npmjs.org/react-refresh/-/react-refresh-0.9.0.tgz",
      "version": "0.9.0",
    },
    "readable-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "readable-stream",
          "spec": "^2.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/duplexify",
          "name": "readable-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/enhanced-resolve/node_modules/memory-fs",
          "name": "readable-stream",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flush-write-stream",
          "name": "readable-stream",
          "spec": "^2.3.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/from2",
          "name": "readable-stream",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fs-write-stream-atomic",
          "name": "readable-stream",
          "spec": "1 || 2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hpack.js",
          "name": "readable-stream",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/memory-fs",
          "name": "readable-stream",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "readable-stream",
          "spec": "^2.3.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parallel-transform",
          "name": "readable-stream",
          "spec": "^2.1.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "readable-stream",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-browserify",
          "name": "readable-stream",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-http",
          "name": "readable-stream",
          "spec": "^2.3.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/through2",
          "name": "readable-stream",
          "spec": "~2.3.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "~1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~1.1.1",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.7.tgz",
      "version": "2.3.7",
    },
    "readdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^3.1.10",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-2.2.1.tgz",
      "version": "2.2.1",
    },
    "regex-not" => ArboristNode {
      "children": Map {
        "extend-shallow" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/regex-not",
              "name": "extend-shallow",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "assign-symbols" => EdgeOut {
              "name": "assign-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/assign-symbols",
              "type": "prod",
            },
            "is-extendable" => EdgeOut {
              "name": "is-extendable",
              "spec": "^1.0.1",
              "to": "node_modules/regex-not/node_modules/is-extendable",
              "type": "prod",
            },
          },
          "location": "node_modules/regex-not/node_modules/extend-shallow",
          "name": "extend-shallow",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/regex-not/node_modules/extend-shallow",
          "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
          "version": "3.0.2",
        },
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/regex-not/node_modules/extend-shallow",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/regex-not/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/regex-not/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/expand-brackets",
          "name": "regex-not",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "regex-not",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "regex-not",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "regex-not",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex",
          "name": "regex-not",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^3.0.2",
          "to": "node_modules/regex-not/node_modules/extend-shallow",
          "type": "prod",
        },
        "safe-regex" => EdgeOut {
          "name": "safe-regex",
          "spec": "^1.1.0",
          "to": "node_modules/safe-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/regex-not",
      "name": "regex-not",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/regex-not",
      "resolved": "https://registry.npmjs.org/regex-not/-/regex-not-1.0.2.tgz",
      "version": "1.0.2",
    },
    "regexp.prototype.flags" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/regexp.prototype.flags",
              "name": "es-abstract",
              "spec": "^1.17.0-next.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deep-equal",
          "name": "regexp.prototype.flags",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.0-next.1",
          "to": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/regexp.prototype.flags",
      "name": "regexp.prototype.flags",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/regexp.prototype.flags",
      "resolved": "https://registry.npmjs.org/regexp.prototype.flags/-/regexp.prototype.flags-1.3.0.tgz",
      "version": "1.3.0",
    },
    "remove-trailing-separator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch/node_modules/normalize-path",
          "name": "remove-trailing-separator",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/remove-trailing-separator",
      "name": "remove-trailing-separator",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/remove-trailing-separator",
      "resolved": "https://registry.npmjs.org/remove-trailing-separator/-/remove-trailing-separator-1.1.0.tgz",
      "version": "1.1.0",
    },
    "repeat-element" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "repeat-element",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/repeat-element",
      "name": "repeat-element",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/repeat-element",
      "resolved": "https://registry.npmjs.org/repeat-element/-/repeat-element-1.1.3.tgz",
      "version": "1.1.3",
    },
    "repeat-string" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "repeat-string",
          "spec": "^1.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "repeat-string",
          "spec": "^1.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/repeat-string",
      "name": "repeat-string",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/repeat-string",
      "resolved": "https://registry.npmjs.org/repeat-string/-/repeat-string-1.6.1.tgz",
      "version": "1.6.1",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "requires-port" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-proxy",
          "name": "requires-port",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/url-parse",
          "name": "requires-port",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/requires-port",
      "name": "requires-port",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/requires-port",
      "resolved": "https://registry.npmjs.org/requires-port/-/requires-port-1.0.0.tgz",
      "version": "1.0.0",
    },
    "resolve-cwd" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-local",
          "name": "resolve-cwd",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^3.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-cwd",
      "name": "resolve-cwd",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/resolve-cwd",
      "resolved": "https://registry.npmjs.org/resolve-cwd/-/resolve-cwd-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve-cwd",
          "name": "resolve-from",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
      "version": "3.0.0",
    },
    "resolve-url" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-resolve",
          "name": "resolve-url",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-url",
      "name": "resolve-url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/resolve-url",
      "resolved": "https://registry.npmjs.org/resolve-url/-/resolve-url-0.2.1.tgz",
      "version": "0.2.1",
    },
    "ret" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/safe-regex",
          "name": "ret",
          "spec": "~0.1.10",
          "type": "prod",
        },
      },
      "location": "node_modules/ret",
      "name": "ret",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ret",
      "resolved": "https://registry.npmjs.org/ret/-/ret-0.1.15.tgz",
      "version": "0.1.15",
    },
    "retry" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-retry",
          "name": "retry",
          "spec": "^0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/retry",
      "name": "retry",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/retry",
      "resolved": "https://registry.npmjs.org/retry/-/retry-0.12.0.tgz",
      "version": "0.12.0",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "rimraf",
          "spec": "^2.5.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/del",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "rimraf",
          "spec": "^2.5.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "ripemd160" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-hash",
          "name": "ripemd160",
          "spec": "^2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "ripemd160",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pbkdf2",
          "name": "ripemd160",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hash-base" => EdgeOut {
          "name": "hash-base",
          "spec": "^3.0.0",
          "to": "node_modules/hash-base",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
      },
      "location": "node_modules/ripemd160",
      "name": "ripemd160",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ripemd160",
      "peer": true,
      "resolved": "https://registry.npmjs.org/ripemd160/-/ripemd160-2.0.2.tgz",
      "version": "2.0.2",
    },
    "run-queue" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/copy-concurrently",
          "name": "run-queue",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/move-concurrently",
          "name": "run-queue",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aproba" => EdgeOut {
          "name": "aproba",
          "spec": "^1.1.1",
          "to": "node_modules/aproba",
          "type": "prod",
        },
      },
      "location": "node_modules/run-queue",
      "name": "run-queue",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/run-queue",
      "peer": true,
      "resolved": "https://registry.npmjs.org/run-queue/-/run-queue-1.0.3.tgz",
      "version": "1.0.3",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-aes",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/browserify-des",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cipher-base",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/compression",
          "name": "safe-buffer",
          "spec": "5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/content-disposition",
          "name": "safe-buffer",
          "spec": "5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/dns-packet",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/evp_bytestokey",
          "name": "safe-buffer",
          "spec": "^5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "safe-buffer",
          "spec": "5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/md5.js",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/parse-asn1",
          "name": "safe-buffer",
          "spec": "^5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pbkdf2",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/public-encrypt",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/randombytes",
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/randomfill",
          "name": "safe-buffer",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sha.js",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string_decoder",
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
      "version": "5.1.2",
    },
    "safe-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/regex-not",
          "name": "safe-regex",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/to-regex",
          "name": "safe-regex",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ret" => EdgeOut {
          "name": "ret",
          "spec": "~0.1.10",
          "to": "node_modules/ret",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-regex",
      "name": "safe-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/safe-regex",
      "resolved": "https://registry.npmjs.org/safe-regex/-/safe-regex-1.1.0.tgz",
      "version": "1.1.0",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1.js",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/iconv-lite",
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "schema-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "schema-utils",
          "spec": "^2.6.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/json-schema" => EdgeOut {
          "name": "@types/json-schema",
          "spec": "^7.0.5",
          "to": "node_modules/@types/json-schema",
          "type": "prod",
        },
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.4",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^3.5.2",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
      },
      "location": "node_modules/schema-utils",
      "name": "schema-utils",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/schema-utils",
      "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-2.7.1.tgz",
      "version": "2.7.1",
    },
    "select-hose" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy",
          "name": "select-hose",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/select-hose",
      "name": "select-hose",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/select-hose",
      "resolved": "https://registry.npmjs.org/select-hose/-/select-hose-2.0.0.tgz",
      "version": "2.0.0",
    },
    "selfsigned" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "selfsigned",
          "spec": "^1.10.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-forge" => EdgeOut {
          "name": "node-forge",
          "spec": "^0.10.0",
          "to": "node_modules/node-forge",
          "type": "prod",
        },
      },
      "location": "node_modules/selfsigned",
      "name": "selfsigned",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/selfsigned",
      "resolved": "https://registry.npmjs.org/selfsigned/-/selfsigned-1.10.8.tgz",
      "version": "1.10.8",
    },
    "semver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "semver",
          "spec": "^6.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
      "version": "6.3.0",
    },
    "send" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "children": Map {
            "ms" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/send/node_modules/debug",
                  "name": "ms",
                  "spec": "2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/send/node_modules/debug/node_modules/ms",
              "name": "ms",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/send/node_modules/debug/node_modules/ms",
              "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
              "version": "2.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/send",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/send/node_modules/debug/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/send/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/send/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/send",
              "name": "ms",
              "spec": "2.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/send/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/send/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
          "version": "2.1.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "send",
          "spec": "0.17.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-static",
          "name": "send",
          "spec": "0.17.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/send/node_modules/debug",
          "type": "prod",
        },
        "depd" => EdgeOut {
          "name": "depd",
          "spec": "~1.1.2",
          "to": "node_modules/depd",
          "type": "prod",
        },
        "destroy" => EdgeOut {
          "name": "destroy",
          "spec": "~1.0.4",
          "to": "node_modules/destroy",
          "type": "prod",
        },
        "encodeurl" => EdgeOut {
          "name": "encodeurl",
          "spec": "~1.0.2",
          "to": "node_modules/encodeurl",
          "type": "prod",
        },
        "escape-html" => EdgeOut {
          "name": "escape-html",
          "spec": "~1.0.3",
          "to": "node_modules/escape-html",
          "type": "prod",
        },
        "etag" => EdgeOut {
          "name": "etag",
          "spec": "~1.8.1",
          "to": "node_modules/etag",
          "type": "prod",
        },
        "fresh" => EdgeOut {
          "name": "fresh",
          "spec": "0.5.2",
          "to": "node_modules/fresh",
          "type": "prod",
        },
        "http-errors" => EdgeOut {
          "name": "http-errors",
          "spec": "~1.7.2",
          "to": "node_modules/http-errors",
          "type": "prod",
        },
        "mime" => EdgeOut {
          "name": "mime",
          "spec": "1.6.0",
          "to": "node_modules/mime",
          "type": "prod",
        },
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.1",
          "to": "node_modules/send/node_modules/ms",
          "type": "prod",
        },
        "on-finished" => EdgeOut {
          "name": "on-finished",
          "spec": "~2.3.0",
          "to": "node_modules/on-finished",
          "type": "prod",
        },
        "range-parser" => EdgeOut {
          "name": "range-parser",
          "spec": "~1.2.1",
          "to": "node_modules/range-parser",
          "type": "prod",
        },
        "statuses" => EdgeOut {
          "name": "statuses",
          "spec": "~1.5.0",
          "to": "node_modules/statuses",
          "type": "prod",
        },
      },
      "location": "node_modules/send",
      "name": "send",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/send",
      "resolved": "https://registry.npmjs.org/send/-/send-0.17.1.tgz",
      "version": "0.17.1",
    },
    "serialize-javascript" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "serialize-javascript",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "randombytes" => EdgeOut {
          "name": "randombytes",
          "spec": "^2.1.0",
          "to": "node_modules/randombytes",
          "type": "prod",
        },
      },
      "location": "node_modules/serialize-javascript",
      "name": "serialize-javascript",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serialize-javascript",
      "peer": true,
      "resolved": "https://registry.npmjs.org/serialize-javascript/-/serialize-javascript-4.0.0.tgz",
      "version": "4.0.0",
    },
    "serve-index" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/serve-index",
              "name": "debug",
              "spec": "2.6.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/serve-index/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/serve-index/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "http-errors" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/serve-index",
              "name": "http-errors",
              "spec": "~1.6.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "depd" => EdgeOut {
              "name": "depd",
              "spec": "~1.1.2",
              "to": "node_modules/depd",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2.0.3",
              "to": "node_modules/serve-index/node_modules/inherits",
              "type": "prod",
            },
            "setprototypeof" => EdgeOut {
              "name": "setprototypeof",
              "spec": "1.1.0",
              "to": "node_modules/serve-index/node_modules/setprototypeof",
              "type": "prod",
            },
            "statuses" => EdgeOut {
              "name": "statuses",
              "spec": ">= 1.4.0 < 2",
              "to": "node_modules/statuses",
              "type": "prod",
            },
          },
          "location": "node_modules/serve-index/node_modules/http-errors",
          "name": "http-errors",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index/node_modules/http-errors",
          "resolved": "https://registry.npmjs.org/http-errors/-/http-errors-1.6.3.tgz",
          "version": "1.6.3",
        },
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/serve-index/node_modules/http-errors",
              "name": "inherits",
              "spec": "2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/serve-index/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/serve-index/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/serve-index/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
        "setprototypeof" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/serve-index/node_modules/http-errors",
              "name": "setprototypeof",
              "spec": "1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/serve-index/node_modules/setprototypeof",
          "name": "setprototypeof",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index/node_modules/setprototypeof",
          "resolved": "https://registry.npmjs.org/setprototypeof/-/setprototypeof-1.1.0.tgz",
          "version": "1.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "serve-index",
          "spec": "^1.9.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "accepts" => EdgeOut {
          "name": "accepts",
          "spec": "~1.3.4",
          "to": "node_modules/accepts",
          "type": "prod",
        },
        "batch" => EdgeOut {
          "name": "batch",
          "spec": "0.6.1",
          "to": "node_modules/batch",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "2.6.9",
          "to": "node_modules/serve-index/node_modules/debug",
          "type": "prod",
        },
        "escape-html" => EdgeOut {
          "name": "escape-html",
          "spec": "~1.0.3",
          "to": "node_modules/escape-html",
          "type": "prod",
        },
        "http-errors" => EdgeOut {
          "name": "http-errors",
          "spec": "~1.6.2",
          "to": "node_modules/serve-index/node_modules/http-errors",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.17",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "parseurl" => EdgeOut {
          "name": "parseurl",
          "spec": "~1.3.2",
          "to": "node_modules/parseurl",
          "type": "prod",
        },
      },
      "location": "node_modules/serve-index",
      "name": "serve-index",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-index",
      "resolved": "https://registry.npmjs.org/serve-index/-/serve-index-1.9.1.tgz",
      "version": "1.9.1",
    },
    "serve-static" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "serve-static",
          "spec": "1.14.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "encodeurl" => EdgeOut {
          "name": "encodeurl",
          "spec": "~1.0.2",
          "to": "node_modules/encodeurl",
          "type": "prod",
        },
        "escape-html" => EdgeOut {
          "name": "escape-html",
          "spec": "~1.0.3",
          "to": "node_modules/escape-html",
          "type": "prod",
        },
        "parseurl" => EdgeOut {
          "name": "parseurl",
          "spec": "~1.3.3",
          "to": "node_modules/parseurl",
          "type": "prod",
        },
        "send" => EdgeOut {
          "name": "send",
          "spec": "0.17.1",
          "to": "node_modules/send",
          "type": "prod",
        },
      },
      "location": "node_modules/serve-static",
      "name": "serve-static",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/serve-static",
      "resolved": "https://registry.npmjs.org/serve-static/-/serve-static-1.14.1.tgz",
      "version": "1.14.1",
    },
    "set-blocking" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "set-value" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "set-value",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/union-value",
          "name": "set-value",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "is-extendable" => EdgeOut {
          "name": "is-extendable",
          "spec": "^0.1.1",
          "to": "node_modules/is-extendable",
          "type": "prod",
        },
        "is-plain-object" => EdgeOut {
          "name": "is-plain-object",
          "spec": "^2.0.3",
          "to": "node_modules/is-plain-object",
          "type": "prod",
        },
        "split-string" => EdgeOut {
          "name": "split-string",
          "spec": "^3.0.1",
          "to": "node_modules/split-string",
          "type": "prod",
        },
      },
      "location": "node_modules/set-value",
      "name": "set-value",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/set-value",
      "resolved": "https://registry.npmjs.org/set-value/-/set-value-2.0.1.tgz",
      "version": "2.0.1",
    },
    "setimmediate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/timers-browserify",
          "name": "setimmediate",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/setimmediate",
      "name": "setimmediate",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/setimmediate",
      "peer": true,
      "resolved": "https://registry.npmjs.org/setimmediate/-/setimmediate-1.0.5.tgz",
      "version": "1.0.5",
    },
    "setprototypeof" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "setprototypeof",
          "spec": "1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-errors",
          "name": "setprototypeof",
          "spec": "1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/setprototypeof",
      "name": "setprototypeof",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/setprototypeof",
      "resolved": "https://registry.npmjs.org/setprototypeof/-/setprototypeof-1.1.1.tgz",
      "version": "1.1.1",
    },
    "sha.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-hash",
          "name": "sha.js",
          "spec": "^2.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/create-hmac",
          "name": "sha.js",
          "spec": "^2.4.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pbkdf2",
          "name": "sha.js",
          "spec": "^2.4.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/sha.js",
      "name": "sha.js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/sha.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/sha.js/-/sha.js-2.4.11.tgz",
      "version": "2.4.11",
    },
    "shebang-command" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "snapdragon" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon",
              "name": "debug",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/snapdragon/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon",
              "name": "define-property",
              "spec": "^0.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "to": "node_modules/snapdragon/node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
          "version": "0.2.5",
        },
        "is-accessor-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon/node_modules/is-descriptor",
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/is-accessor-descriptor",
          "name": "is-accessor-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/is-accessor-descriptor",
          "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "is-data-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon/node_modules/is-descriptor",
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/is-data-descriptor",
          "name": "is-data-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/is-data-descriptor",
          "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
          "version": "0.1.4",
        },
        "is-descriptor" => ArboristNode {
          "children": Map {
            "kind-of" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/snapdragon/node_modules/is-descriptor",
                  "name": "kind-of",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/snapdragon/node_modules/is-descriptor/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/is-descriptor/node_modules/kind-of",
              "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon/node_modules/define-property",
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-accessor-descriptor" => EdgeOut {
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "to": "node_modules/snapdragon/node_modules/is-accessor-descriptor",
              "type": "prod",
            },
            "is-data-descriptor" => EdgeOut {
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "to": "node_modules/snapdragon/node_modules/is-data-descriptor",
              "type": "prod",
            },
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^5.0.0",
              "to": "node_modules/snapdragon/node_modules/is-descriptor/node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/is-descriptor",
          "name": "is-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/is-descriptor",
          "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon",
              "name": "source-map",
              "spec": "^0.5.6",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "snapdragon",
          "spec": "^0.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/expand-brackets",
          "name": "snapdragon",
          "spec": "^0.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "snapdragon",
          "spec": "^0.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "snapdragon",
          "spec": "^0.8.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "snapdragon",
          "spec": "^0.8.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "base" => EdgeOut {
          "name": "base",
          "spec": "^0.11.1",
          "to": "node_modules/base",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.2.0",
          "to": "node_modules/snapdragon/node_modules/debug",
          "type": "prod",
        },
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^0.2.5",
          "to": "node_modules/snapdragon/node_modules/define-property",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^2.0.1",
          "to": "node_modules/extend-shallow",
          "type": "prod",
        },
        "map-cache" => EdgeOut {
          "name": "map-cache",
          "spec": "^0.2.2",
          "to": "node_modules/map-cache",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.6",
          "to": "node_modules/snapdragon/node_modules/source-map",
          "type": "prod",
        },
        "source-map-resolve" => EdgeOut {
          "name": "source-map-resolve",
          "spec": "^0.5.0",
          "to": "node_modules/source-map-resolve",
          "type": "prod",
        },
        "use" => EdgeOut {
          "name": "use",
          "spec": "^3.1.0",
          "to": "node_modules/use",
          "type": "prod",
        },
      },
      "location": "node_modules/snapdragon",
      "name": "snapdragon",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon",
      "resolved": "https://registry.npmjs.org/snapdragon/-/snapdragon-0.8.2.tgz",
      "version": "0.8.2",
    },
    "snapdragon-node" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/snapdragon-node",
              "name": "define-property",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^1.0.0",
              "to": "node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/snapdragon-node/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon-node/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "snapdragon-node",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^1.0.0",
          "to": "node_modules/snapdragon-node/node_modules/define-property",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.0",
          "to": "node_modules/isobject",
          "type": "prod",
        },
        "snapdragon-util" => EdgeOut {
          "name": "snapdragon-util",
          "spec": "^3.0.1",
          "to": "node_modules/snapdragon-util",
          "type": "prod",
        },
      },
      "location": "node_modules/snapdragon-node",
      "name": "snapdragon-node",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon-node",
      "resolved": "https://registry.npmjs.org/snapdragon-node/-/snapdragon-node-2.1.1.tgz",
      "version": "2.1.1",
    },
    "snapdragon-util" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/snapdragon-node",
          "name": "snapdragon-util",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^3.2.0",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/snapdragon-util",
      "name": "snapdragon-util",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/snapdragon-util",
      "resolved": "https://registry.npmjs.org/snapdragon-util/-/snapdragon-util-3.0.1.tgz",
      "version": "3.0.1",
    },
    "sockjs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "sockjs",
          "spec": "0.3.20",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "faye-websocket" => EdgeOut {
          "name": "faye-websocket",
          "spec": "^0.10.0",
          "to": "node_modules/faye-websocket",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.4.0",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "websocket-driver" => EdgeOut {
          "name": "websocket-driver",
          "spec": "0.6.5",
          "to": "node_modules/websocket-driver",
          "type": "prod",
        },
      },
      "location": "node_modules/sockjs",
      "name": "sockjs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/sockjs",
      "resolved": "https://registry.npmjs.org/sockjs/-/sockjs-0.3.20.tgz",
      "version": "0.3.20",
    },
    "sockjs-client" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/sockjs-client",
              "name": "debug",
              "spec": "^3.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/sockjs-client/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/sockjs-client/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-3.2.6.tgz",
          "version": "3.2.6",
        },
        "faye-websocket" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/sockjs-client",
              "name": "faye-websocket",
              "spec": "~0.11.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "websocket-driver" => EdgeOut {
              "name": "websocket-driver",
              "spec": ">=0.5.1",
              "to": "node_modules/websocket-driver",
              "type": "prod",
            },
          },
          "location": "node_modules/sockjs-client/node_modules/faye-websocket",
          "name": "faye-websocket",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/sockjs-client/node_modules/faye-websocket",
          "resolved": "https://registry.npmjs.org/faye-websocket/-/faye-websocket-0.11.3.tgz",
          "version": "0.11.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "sockjs-client",
          "spec": "^1.4.0",
          "type": "peerOptional",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "sockjs-client",
          "spec": "1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^3.2.5",
          "to": "node_modules/sockjs-client/node_modules/debug",
          "type": "prod",
        },
        "eventsource" => EdgeOut {
          "name": "eventsource",
          "spec": "^1.0.7",
          "to": "node_modules/eventsource",
          "type": "prod",
        },
        "faye-websocket" => EdgeOut {
          "name": "faye-websocket",
          "spec": "~0.11.1",
          "to": "node_modules/sockjs-client/node_modules/faye-websocket",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "json3" => EdgeOut {
          "name": "json3",
          "spec": "^3.3.2",
          "to": "node_modules/json3",
          "type": "prod",
        },
        "url-parse" => EdgeOut {
          "name": "url-parse",
          "spec": "^1.4.3",
          "to": "node_modules/url-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/sockjs-client",
      "name": "sockjs-client",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/sockjs-client",
      "resolved": "https://registry.npmjs.org/sockjs-client/-/sockjs-client-1.4.0.tgz",
      "version": "1.4.0",
    },
    "source-list-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-sources",
          "name": "source-list-map",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-list-map",
      "name": "source-list-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-list-map",
      "peer": true,
      "resolved": "https://registry.npmjs.org/source-list-map/-/source-list-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "source-map",
          "spec": "^0.7.3",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.7.3.tgz",
      "version": "0.7.3",
    },
    "source-map-resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/snapdragon",
          "name": "source-map-resolve",
          "spec": "^0.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "atob" => EdgeOut {
          "name": "atob",
          "spec": "^2.1.2",
          "to": "node_modules/atob",
          "type": "prod",
        },
        "decode-uri-component" => EdgeOut {
          "name": "decode-uri-component",
          "spec": "^0.2.0",
          "to": "node_modules/decode-uri-component",
          "type": "prod",
        },
        "resolve-url" => EdgeOut {
          "name": "resolve-url",
          "spec": "^0.2.1",
          "to": "node_modules/resolve-url",
          "type": "prod",
        },
        "source-map-url" => EdgeOut {
          "name": "source-map-url",
          "spec": "^0.4.0",
          "to": "node_modules/source-map-url",
          "type": "prod",
        },
        "urix" => EdgeOut {
          "name": "urix",
          "spec": "^0.1.0",
          "to": "node_modules/urix",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-resolve",
      "name": "source-map-resolve",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-map-resolve",
      "resolved": "https://registry.npmjs.org/source-map-resolve/-/source-map-resolve-0.5.3.tgz",
      "version": "0.5.3",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-map-support/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser",
          "name": "source-map-support",
          "spec": "~0.5.12",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-map-support",
      "peer": true,
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
      "version": "0.5.19",
    },
    "source-map-url" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-resolve",
          "name": "source-map-url",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-url",
      "name": "source-map-url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/source-map-url",
      "resolved": "https://registry.npmjs.org/source-map-url/-/source-map-url-0.4.0.tgz",
      "version": "0.4.0",
    },
    "spdy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "spdy",
          "spec": "^4.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "handle-thing" => EdgeOut {
          "name": "handle-thing",
          "spec": "^2.0.0",
          "to": "node_modules/handle-thing",
          "type": "prod",
        },
        "http-deceiver" => EdgeOut {
          "name": "http-deceiver",
          "spec": "^1.2.7",
          "to": "node_modules/http-deceiver",
          "type": "prod",
        },
        "select-hose" => EdgeOut {
          "name": "select-hose",
          "spec": "^2.0.0",
          "to": "node_modules/select-hose",
          "type": "prod",
        },
        "spdy-transport" => EdgeOut {
          "name": "spdy-transport",
          "spec": "^3.0.0",
          "to": "node_modules/spdy-transport",
          "type": "prod",
        },
      },
      "location": "node_modules/spdy",
      "name": "spdy",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/spdy",
      "resolved": "https://registry.npmjs.org/spdy/-/spdy-4.0.2.tgz",
      "version": "4.0.2",
    },
    "spdy-transport" => ArboristNode {
      "children": Map {
        "readable-stream" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spdy-transport",
              "name": "readable-stream",
              "spec": "^3.0.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "^2.0.3",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "^1.1.1",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "^1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/spdy-transport/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/spdy-transport/node_modules/readable-stream",
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-3.6.0.tgz",
          "version": "3.6.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdy",
          "name": "spdy-transport",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "detect-node" => EdgeOut {
          "name": "detect-node",
          "spec": "^2.0.4",
          "to": "node_modules/detect-node",
          "type": "prod",
        },
        "hpack.js" => EdgeOut {
          "name": "hpack.js",
          "spec": "^2.1.6",
          "to": "node_modules/hpack.js",
          "type": "prod",
        },
        "obuf" => EdgeOut {
          "name": "obuf",
          "spec": "^1.1.2",
          "to": "node_modules/obuf",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^3.0.6",
          "to": "node_modules/spdy-transport/node_modules/readable-stream",
          "type": "prod",
        },
        "wbuf" => EdgeOut {
          "name": "wbuf",
          "spec": "^1.7.3",
          "to": "node_modules/wbuf",
          "type": "prod",
        },
      },
      "location": "node_modules/spdy-transport",
      "name": "spdy-transport",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/spdy-transport",
      "resolved": "https://registry.npmjs.org/spdy-transport/-/spdy-transport-3.0.0.tgz",
      "version": "3.0.0",
    },
    "split-string" => ArboristNode {
      "children": Map {
        "extend-shallow" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/split-string",
              "name": "extend-shallow",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "assign-symbols" => EdgeOut {
              "name": "assign-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/assign-symbols",
              "type": "prod",
            },
            "is-extendable" => EdgeOut {
              "name": "is-extendable",
              "spec": "^1.0.1",
              "to": "node_modules/split-string/node_modules/is-extendable",
              "type": "prod",
            },
          },
          "location": "node_modules/split-string/node_modules/extend-shallow",
          "name": "extend-shallow",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/split-string/node_modules/extend-shallow",
          "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
          "version": "3.0.2",
        },
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/split-string/node_modules/extend-shallow",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/split-string/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/split-string/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "split-string",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/set-value",
          "name": "split-string",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^3.0.0",
          "to": "node_modules/split-string/node_modules/extend-shallow",
          "type": "prod",
        },
      },
      "location": "node_modules/split-string",
      "name": "split-string",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/split-string",
      "resolved": "https://registry.npmjs.org/split-string/-/split-string-3.1.0.tgz",
      "version": "3.1.0",
    },
    "ssri" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "ssri",
          "spec": "^6.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "figgy-pudding" => EdgeOut {
          "name": "figgy-pudding",
          "spec": "^3.5.1",
          "to": "node_modules/figgy-pudding",
          "type": "prod",
        },
      },
      "location": "node_modules/ssri",
      "name": "ssri",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ssri",
      "peer": true,
      "resolved": "https://registry.npmjs.org/ssri/-/ssri-6.0.1.tgz",
      "version": "6.0.1",
    },
    "stackframe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-stack-parser",
          "name": "stackframe",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/stackframe",
      "name": "stackframe",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/stackframe",
      "resolved": "https://registry.npmjs.org/stackframe/-/stackframe-1.2.0.tgz",
      "version": "1.2.0",
    },
    "static-extend" => ArboristNode {
      "children": Map {
        "define-property" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/static-extend",
              "name": "define-property",
              "spec": "^0.2.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-descriptor" => EdgeOut {
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "to": "node_modules/static-extend/node_modules/is-descriptor",
              "type": "prod",
            },
          },
          "location": "node_modules/static-extend/node_modules/define-property",
          "name": "define-property",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend/node_modules/define-property",
          "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
          "version": "0.2.5",
        },
        "is-accessor-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/static-extend/node_modules/is-descriptor",
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/static-extend/node_modules/is-accessor-descriptor",
          "name": "is-accessor-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend/node_modules/is-accessor-descriptor",
          "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
        "is-data-descriptor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/static-extend/node_modules/is-descriptor",
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^3.0.2",
              "to": "node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/static-extend/node_modules/is-data-descriptor",
          "name": "is-data-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend/node_modules/is-data-descriptor",
          "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
          "version": "0.1.4",
        },
        "is-descriptor" => ArboristNode {
          "children": Map {
            "kind-of" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/static-extend/node_modules/is-descriptor",
                  "name": "kind-of",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/static-extend/node_modules/is-descriptor/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend/node_modules/is-descriptor/node_modules/kind-of",
              "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/static-extend/node_modules/define-property",
              "name": "is-descriptor",
              "spec": "^0.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-accessor-descriptor" => EdgeOut {
              "name": "is-accessor-descriptor",
              "spec": "^0.1.6",
              "to": "node_modules/static-extend/node_modules/is-accessor-descriptor",
              "type": "prod",
            },
            "is-data-descriptor" => EdgeOut {
              "name": "is-data-descriptor",
              "spec": "^0.1.4",
              "to": "node_modules/static-extend/node_modules/is-data-descriptor",
              "type": "prod",
            },
            "kind-of" => EdgeOut {
              "name": "kind-of",
              "spec": "^5.0.0",
              "to": "node_modules/static-extend/node_modules/is-descriptor/node_modules/kind-of",
              "type": "prod",
            },
          },
          "location": "node_modules/static-extend/node_modules/is-descriptor",
          "name": "is-descriptor",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend/node_modules/is-descriptor",
          "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
          "version": "0.1.6",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/class-utils",
          "name": "static-extend",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^0.2.5",
          "to": "node_modules/static-extend/node_modules/define-property",
          "type": "prod",
        },
        "object-copy" => EdgeOut {
          "name": "object-copy",
          "spec": "^0.1.0",
          "to": "node_modules/object-copy",
          "type": "prod",
        },
      },
      "location": "node_modules/static-extend",
      "name": "static-extend",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/static-extend",
      "resolved": "https://registry.npmjs.org/static-extend/-/static-extend-0.1.2.tgz",
      "version": "0.1.2",
    },
    "statuses" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "statuses",
          "spec": "~1.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "statuses",
          "spec": "~1.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-errors",
          "name": "statuses",
          "spec": ">= 1.5.0 < 2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/send",
          "name": "statuses",
          "spec": "~1.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/serve-index/node_modules/http-errors",
          "name": "statuses",
          "spec": ">= 1.4.0 < 2",
          "type": "prod",
        },
      },
      "location": "node_modules/statuses",
      "name": "statuses",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/statuses",
      "resolved": "https://registry.npmjs.org/statuses/-/statuses-1.5.0.tgz",
      "version": "1.5.0",
    },
    "stream-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "stream-browserify",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/stream-browserify",
      "name": "stream-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/stream-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/stream-browserify/-/stream-browserify-2.0.2.tgz",
      "version": "2.0.2",
    },
    "stream-each" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "stream-each",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "end-of-stream" => EdgeOut {
          "name": "end-of-stream",
          "spec": "^1.1.0",
          "to": "node_modules/end-of-stream",
          "type": "prod",
        },
        "stream-shift" => EdgeOut {
          "name": "stream-shift",
          "spec": "^1.0.0",
          "to": "node_modules/stream-shift",
          "type": "prod",
        },
      },
      "location": "node_modules/stream-each",
      "name": "stream-each",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/stream-each",
      "peer": true,
      "resolved": "https://registry.npmjs.org/stream-each/-/stream-each-1.2.3.tgz",
      "version": "1.2.3",
    },
    "stream-http" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "stream-http",
          "spec": "^2.7.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "builtin-status-codes" => EdgeOut {
          "name": "builtin-status-codes",
          "spec": "^3.0.0",
          "to": "node_modules/builtin-status-codes",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.3.6",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "to-arraybuffer" => EdgeOut {
          "name": "to-arraybuffer",
          "spec": "^1.0.0",
          "to": "node_modules/to-arraybuffer",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.0",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/stream-http",
      "name": "stream-http",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/stream-http",
      "peer": true,
      "resolved": "https://registry.npmjs.org/stream-http/-/stream-http-2.8.3.tgz",
      "version": "2.8.3",
    },
    "stream-shift" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/duplexify",
          "name": "stream-shift",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/stream-each",
          "name": "stream-shift",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/stream-shift",
      "name": "stream-shift",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/stream-shift",
      "peer": true,
      "resolved": "https://registry.npmjs.org/stream-shift/-/stream-shift-1.0.1.tgz",
      "version": "1.0.1",
    },
    "string_decoder" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash-base/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "string_decoder",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
      "version": "1.1.1",
    },
    "string-width" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string-width/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/string-width/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string-width/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "string-width",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "string-width",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.1.0",
          "to": "node_modules/string-width/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string.prototype.trimend" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string.prototype.trimend",
              "name": "es-abstract",
              "spec": "^1.17.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string.prototype.trimend/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "string.prototype.trimend",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.5",
          "to": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/string.prototype.trimend",
      "name": "string.prototype.trimend",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string.prototype.trimend",
      "resolved": "https://registry.npmjs.org/string.prototype.trimend/-/string.prototype.trimend-1.0.1.tgz",
      "version": "1.0.1",
    },
    "string.prototype.trimstart" => ArboristNode {
      "children": Map {
        "es-abstract" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string.prototype.trimstart",
              "name": "es-abstract",
              "spec": "^1.17.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es-to-primitive" => EdgeOut {
              "name": "es-to-primitive",
              "spec": "^1.2.1",
              "to": "node_modules/es-to-primitive",
              "type": "prod",
            },
            "function-bind" => EdgeOut {
              "name": "function-bind",
              "spec": "^1.1.1",
              "to": "node_modules/function-bind",
              "type": "prod",
            },
            "has" => EdgeOut {
              "name": "has",
              "spec": "^1.0.3",
              "to": "node_modules/has",
              "type": "prod",
            },
            "has-symbols" => EdgeOut {
              "name": "has-symbols",
              "spec": "^1.0.1",
              "to": "node_modules/has-symbols",
              "type": "prod",
            },
            "is-callable" => EdgeOut {
              "name": "is-callable",
              "spec": "^1.2.2",
              "to": "node_modules/is-callable",
              "type": "prod",
            },
            "is-regex" => EdgeOut {
              "name": "is-regex",
              "spec": "^1.1.1",
              "to": "node_modules/is-regex",
              "type": "prod",
            },
            "object-inspect" => EdgeOut {
              "name": "object-inspect",
              "spec": "^1.8.0",
              "to": "node_modules/object-inspect",
              "type": "prod",
            },
            "object-keys" => EdgeOut {
              "name": "object-keys",
              "spec": "^1.1.1",
              "to": "node_modules/object-keys",
              "type": "prod",
            },
            "object.assign" => EdgeOut {
              "name": "object.assign",
              "spec": "^4.1.1",
              "to": "node_modules/object.assign",
              "type": "prod",
            },
            "string.prototype.trimend" => EdgeOut {
              "name": "string.prototype.trimend",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimend",
              "type": "prod",
            },
            "string.prototype.trimstart" => EdgeOut {
              "name": "string.prototype.trimstart",
              "spec": "^1.0.1",
              "to": "node_modules/string.prototype.trimstart",
              "type": "prod",
            },
          },
          "location": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "es-abstract",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "resolved": "https://registry.npmjs.org/es-abstract/-/es-abstract-1.17.7.tgz",
          "version": "1.17.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/regexp.prototype.flags/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimend/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "name": "string.prototype.trimstart",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-properties" => EdgeOut {
          "name": "define-properties",
          "spec": "^1.1.3",
          "to": "node_modules/define-properties",
          "type": "prod",
        },
        "es-abstract" => EdgeOut {
          "name": "es-abstract",
          "spec": "^1.17.5",
          "to": "node_modules/string.prototype.trimstart/node_modules/es-abstract",
          "type": "prod",
        },
      },
      "location": "node_modules/string.prototype.trimstart",
      "name": "string.prototype.trimstart",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/string.prototype.trimstart",
      "resolved": "https://registry.npmjs.org/string.prototype.trimstart/-/string.prototype.trimstart-1.0.1.tgz",
      "version": "1.0.1",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
      "version": "3.0.1",
    },
    "strip-eof" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/execa",
          "name": "strip-eof",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-eof",
      "name": "strip-eof",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/strip-eof",
      "resolved": "https://registry.npmjs.org/strip-eof/-/strip-eof-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tapable" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/enhanced-resolve",
          "name": "tapable",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "tapable",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/tapable",
      "name": "tapable",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/tapable",
      "peer": true,
      "resolved": "https://registry.npmjs.org/tapable/-/tapable-1.1.3.tgz",
      "version": "1.1.3",
    },
    "terser" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser",
              "name": "source-map",
              "spec": "~0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/terser/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/terser/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "terser",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "^2.20.0",
          "to": "node_modules/commander",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.6.1",
          "to": "node_modules/terser/node_modules/source-map",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "~0.5.12",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/terser",
      "name": "terser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/terser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/terser/-/terser-4.8.0.tgz",
      "version": "4.8.0",
    },
    "terser-webpack-plugin" => ArboristNode {
      "children": Map {
        "schema-utils" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser-webpack-plugin",
              "name": "schema-utils",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.1.0",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "ajv-errors" => EdgeOut {
              "name": "ajv-errors",
              "spec": "^1.0.0",
              "to": "node_modules/ajv-errors",
              "type": "prod",
            },
            "ajv-keywords" => EdgeOut {
              "name": "ajv-keywords",
              "spec": "^3.1.0",
              "to": "node_modules/ajv-keywords",
              "type": "prod",
            },
          },
          "location": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "name": "schema-utils",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "peer": true,
          "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-1.0.0.tgz",
          "version": "1.0.0",
        },
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/terser-webpack-plugin",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/terser-webpack-plugin/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/terser-webpack-plugin/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "terser-webpack-plugin",
          "spec": "^1.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cacache" => EdgeOut {
          "name": "cacache",
          "spec": "^12.0.2",
          "to": "node_modules/cacache",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "is-wsl" => EdgeOut {
          "name": "is-wsl",
          "spec": "^1.1.0",
          "to": "node_modules/is-wsl",
          "type": "prod",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^1.0.0",
          "to": "node_modules/terser-webpack-plugin/node_modules/schema-utils",
          "type": "prod",
        },
        "serialize-javascript" => EdgeOut {
          "name": "serialize-javascript",
          "spec": "^4.0.0",
          "to": "node_modules/serialize-javascript",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/terser-webpack-plugin/node_modules/source-map",
          "type": "prod",
        },
        "terser" => EdgeOut {
          "name": "terser",
          "spec": "^4.1.2",
          "to": "node_modules/terser",
          "type": "prod",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": "^4.0.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
        "webpack-sources" => EdgeOut {
          "name": "webpack-sources",
          "spec": "^1.4.0",
          "to": "node_modules/webpack-sources",
          "type": "prod",
        },
        "worker-farm" => EdgeOut {
          "name": "worker-farm",
          "spec": "^1.7.0",
          "to": "node_modules/worker-farm",
          "type": "prod",
        },
      },
      "location": "node_modules/terser-webpack-plugin",
      "name": "terser-webpack-plugin",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/terser-webpack-plugin",
      "peer": true,
      "resolved": "https://registry.npmjs.org/terser-webpack-plugin/-/terser-webpack-plugin-1.4.5.tgz",
      "version": "1.4.5",
    },
    "through2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mississippi",
          "name": "through2",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "~2.3.6",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "~4.0.1",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/through2",
      "name": "through2",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/through2",
      "peer": true,
      "resolved": "https://registry.npmjs.org/through2/-/through2-2.0.5.tgz",
      "version": "2.0.5",
    },
    "thunky" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/multicast-dns",
          "name": "thunky",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/thunky",
      "name": "thunky",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/thunky",
      "resolved": "https://registry.npmjs.org/thunky/-/thunky-1.1.0.tgz",
      "version": "1.1.0",
    },
    "timers-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "timers-browserify",
          "spec": "^2.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "setimmediate" => EdgeOut {
          "name": "setimmediate",
          "spec": "^1.0.4",
          "to": "node_modules/setimmediate",
          "type": "prod",
        },
      },
      "location": "node_modules/timers-browserify",
      "name": "timers-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/timers-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/timers-browserify/-/timers-browserify-2.0.12.tgz",
      "version": "2.0.12",
    },
    "to-arraybuffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/stream-http",
          "name": "to-arraybuffer",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-arraybuffer",
      "name": "to-arraybuffer",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-arraybuffer",
      "peer": true,
      "resolved": "https://registry.npmjs.org/to-arraybuffer/-/to-arraybuffer-1.0.1.tgz",
      "version": "1.0.1",
    },
    "to-object-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "to-object-path",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "^3.0.2",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "location": "node_modules/to-object-path",
      "name": "to-object-path",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-object-path",
      "resolved": "https://registry.npmjs.org/to-object-path/-/to-object-path-0.3.0.tgz",
      "version": "0.3.0",
    },
    "to-regex" => ArboristNode {
      "children": Map {
        "extend-shallow" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/to-regex",
              "name": "extend-shallow",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "assign-symbols" => EdgeOut {
              "name": "assign-symbols",
              "spec": "^1.0.0",
              "to": "node_modules/assign-symbols",
              "type": "prod",
            },
            "is-extendable" => EdgeOut {
              "name": "is-extendable",
              "spec": "^1.0.1",
              "to": "node_modules/to-regex/node_modules/is-extendable",
              "type": "prod",
            },
          },
          "location": "node_modules/to-regex/node_modules/extend-shallow",
          "name": "extend-shallow",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-regex/node_modules/extend-shallow",
          "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
          "version": "3.0.2",
        },
        "is-extendable" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/to-regex/node_modules/extend-shallow",
              "name": "is-extendable",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-plain-object" => EdgeOut {
              "name": "is-plain-object",
              "spec": "^2.0.4",
              "to": "node_modules/is-plain-object",
              "type": "prod",
            },
          },
          "location": "node_modules/to-regex/node_modules/is-extendable",
          "name": "is-extendable",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-regex/node_modules/is-extendable",
          "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "to-regex",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/expand-brackets",
          "name": "to-regex",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/extglob",
          "name": "to-regex",
          "spec": "^3.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/micromatch",
          "name": "to-regex",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nanomatch",
          "name": "to-regex",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "define-property" => EdgeOut {
          "name": "define-property",
          "spec": "^2.0.2",
          "to": "node_modules/define-property",
          "type": "prod",
        },
        "extend-shallow" => EdgeOut {
          "name": "extend-shallow",
          "spec": "^3.0.2",
          "to": "node_modules/to-regex/node_modules/extend-shallow",
          "type": "prod",
        },
        "regex-not" => EdgeOut {
          "name": "regex-not",
          "spec": "^1.0.2",
          "to": "node_modules/regex-not",
          "type": "prod",
        },
        "safe-regex" => EdgeOut {
          "name": "safe-regex",
          "spec": "^1.1.0",
          "to": "node_modules/safe-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex",
      "name": "to-regex",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-regex",
      "resolved": "https://registry.npmjs.org/to-regex/-/to-regex-3.0.2.tgz",
      "version": "3.0.2",
    },
    "to-regex-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^3.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
        "repeat-string" => EdgeOut {
          "name": "repeat-string",
          "spec": "^1.6.1",
          "to": "node_modules/repeat-string",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-2.1.1.tgz",
      "version": "2.1.1",
    },
    "toidentifier" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-errors",
          "name": "toidentifier",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/toidentifier",
      "name": "toidentifier",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/toidentifier",
      "resolved": "https://registry.npmjs.org/toidentifier/-/toidentifier-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tslib" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chrome-trace-event",
          "name": "tslib",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tslib",
      "name": "tslib",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/tslib",
      "peer": true,
      "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
      "version": "1.14.1",
    },
    "tty-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "tty-browserify",
          "spec": "0.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tty-browserify",
      "name": "tty-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/tty-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/tty-browserify/-/tty-browserify-0.0.0.tgz",
      "version": "0.0.0",
    },
    "type-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/body-parser",
          "name": "type-is",
          "spec": "~1.6.17",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "type-is",
          "spec": "~1.6.18",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "media-typer" => EdgeOut {
          "name": "media-typer",
          "spec": "0.3.0",
          "to": "node_modules/media-typer",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.24",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/type-is",
      "name": "type-is",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/type-is",
      "resolved": "https://registry.npmjs.org/type-is/-/type-is-1.6.18.tgz",
      "version": "1.6.18",
    },
    "typedarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "typedarray",
          "spec": "^0.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray",
      "name": "typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/typedarray",
      "peer": true,
      "resolved": "https://registry.npmjs.org/typedarray/-/typedarray-0.0.6.tgz",
      "version": "0.0.6",
    },
    "union-value" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "union-value",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arr-union" => EdgeOut {
          "name": "arr-union",
          "spec": "^3.1.0",
          "to": "node_modules/arr-union",
          "type": "prod",
        },
        "get-value" => EdgeOut {
          "name": "get-value",
          "spec": "^2.0.6",
          "to": "node_modules/get-value",
          "type": "prod",
        },
        "is-extendable" => EdgeOut {
          "name": "is-extendable",
          "spec": "^0.1.1",
          "to": "node_modules/is-extendable",
          "type": "prod",
        },
        "set-value" => EdgeOut {
          "name": "set-value",
          "spec": "^2.0.1",
          "to": "node_modules/set-value",
          "type": "prod",
        },
      },
      "location": "node_modules/union-value",
      "name": "union-value",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/union-value",
      "resolved": "https://registry.npmjs.org/union-value/-/union-value-1.0.1.tgz",
      "version": "1.0.1",
    },
    "unique-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "unique-filename",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "unique-slug" => EdgeOut {
          "name": "unique-slug",
          "spec": "^2.0.0",
          "to": "node_modules/unique-slug",
          "type": "prod",
        },
      },
      "location": "node_modules/unique-filename",
      "name": "unique-filename",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unique-filename",
      "peer": true,
      "resolved": "https://registry.npmjs.org/unique-filename/-/unique-filename-1.1.1.tgz",
      "version": "1.1.1",
    },
    "unique-slug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/unique-filename",
          "name": "unique-slug",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
      },
      "location": "node_modules/unique-slug",
      "name": "unique-slug",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unique-slug",
      "peer": true,
      "resolved": "https://registry.npmjs.org/unique-slug/-/unique-slug-2.0.2.tgz",
      "version": "2.0.2",
    },
    "unpipe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/finalhandler",
          "name": "unpipe",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/raw-body",
          "name": "unpipe",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/unpipe",
      "name": "unpipe",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unpipe",
      "resolved": "https://registry.npmjs.org/unpipe/-/unpipe-1.0.0.tgz",
      "version": "1.0.0",
    },
    "unset-value" => ArboristNode {
      "children": Map {
        "has-value" => ArboristNode {
          "children": Map {
            "isobject" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/unset-value/node_modules/has-value",
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "1.0.0",
                  "to": "node_modules/isarray",
                  "type": "prod",
                },
              },
              "location": "node_modules/unset-value/node_modules/has-value/node_modules/isobject",
              "name": "isobject",
              "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unset-value/node_modules/has-value/node_modules/isobject",
              "resolved": "https://registry.npmjs.org/isobject/-/isobject-2.1.0.tgz",
              "version": "2.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unset-value",
              "name": "has-value",
              "spec": "^0.3.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "get-value" => EdgeOut {
              "name": "get-value",
              "spec": "^2.0.3",
              "to": "node_modules/get-value",
              "type": "prod",
            },
            "has-values" => EdgeOut {
              "name": "has-values",
              "spec": "^0.1.4",
              "to": "node_modules/unset-value/node_modules/has-values",
              "type": "prod",
            },
            "isobject" => EdgeOut {
              "name": "isobject",
              "spec": "^2.0.0",
              "to": "node_modules/unset-value/node_modules/has-value/node_modules/isobject",
              "type": "prod",
            },
          },
          "location": "node_modules/unset-value/node_modules/has-value",
          "name": "has-value",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unset-value/node_modules/has-value",
          "resolved": "https://registry.npmjs.org/has-value/-/has-value-0.3.1.tgz",
          "version": "0.3.1",
        },
        "has-values" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unset-value/node_modules/has-value",
              "name": "has-values",
              "spec": "^0.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/unset-value/node_modules/has-values",
          "name": "has-values",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unset-value/node_modules/has-values",
          "resolved": "https://registry.npmjs.org/has-values/-/has-values-0.1.4.tgz",
          "version": "0.1.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cache-base",
          "name": "unset-value",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-value" => EdgeOut {
          "name": "has-value",
          "spec": "^0.3.1",
          "to": "node_modules/unset-value/node_modules/has-value",
          "type": "prod",
        },
        "isobject" => EdgeOut {
          "name": "isobject",
          "spec": "^3.0.0",
          "to": "node_modules/isobject",
          "type": "prod",
        },
      },
      "location": "node_modules/unset-value",
      "name": "unset-value",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/unset-value",
      "resolved": "https://registry.npmjs.org/unset-value/-/unset-value-1.0.0.tgz",
      "version": "1.0.0",
    },
    "upath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "upath",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/upath",
      "name": "upath",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/upath",
      "resolved": "https://registry.npmjs.org/upath/-/upath-1.2.0.tgz",
      "version": "1.2.0",
    },
    "uri-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "urix" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-resolve",
          "name": "urix",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/urix",
      "name": "urix",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/urix",
      "resolved": "https://registry.npmjs.org/urix/-/urix-0.1.0.tgz",
      "version": "0.1.0",
    },
    "url" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/url",
              "name": "punycode",
              "spec": "1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/url/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/url/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.3.2.tgz",
          "version": "1.3.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "url",
          "spec": "^0.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "url",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "1.3.2",
          "to": "node_modules/url/node_modules/punycode",
          "type": "prod",
        },
        "querystring" => EdgeOut {
          "name": "querystring",
          "spec": "0.2.0",
          "to": "node_modules/querystring",
          "type": "prod",
        },
      },
      "location": "node_modules/url",
      "name": "url",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/url",
      "resolved": "https://registry.npmjs.org/url/-/url-0.11.0.tgz",
      "version": "0.11.0",
    },
    "url-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/original",
          "name": "url-parse",
          "spec": "^1.4.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sockjs-client",
          "name": "url-parse",
          "spec": "^1.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "querystringify" => EdgeOut {
          "name": "querystringify",
          "spec": "^2.1.1",
          "to": "node_modules/querystringify",
          "type": "prod",
        },
        "requires-port" => EdgeOut {
          "name": "requires-port",
          "spec": "^1.0.0",
          "to": "node_modules/requires-port",
          "type": "prod",
        },
      },
      "location": "node_modules/url-parse",
      "name": "url-parse",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/url-parse",
      "resolved": "https://registry.npmjs.org/url-parse/-/url-parse-1.4.7.tgz",
      "version": "1.4.7",
    },
    "use" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/snapdragon",
          "name": "use",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/use",
      "name": "use",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/use",
      "resolved": "https://registry.npmjs.org/use/-/use-3.1.1.tgz",
      "version": "3.1.1",
    },
    "util" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/util",
              "name": "inherits",
              "spec": "2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/util/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/util/node_modules/inherits",
          "peer": true,
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "util",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2.0.3",
          "to": "node_modules/util/node_modules/inherits",
          "type": "prod",
        },
      },
      "location": "node_modules/util",
      "name": "util",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/util",
      "peer": true,
      "resolved": "https://registry.npmjs.org/util/-/util-0.11.1.tgz",
      "version": "0.11.1",
    },
    "util-deprecate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/browserify-sign/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hash-base/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
      "version": "1.0.2",
    },
    "utils-merge" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/express",
          "name": "utils-merge",
          "spec": "1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/utils-merge",
      "name": "utils-merge",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/utils-merge",
      "resolved": "https://registry.npmjs.org/utils-merge/-/utils-merge-1.0.1.tgz",
      "version": "1.0.1",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sockjs",
          "name": "uuid",
          "spec": "^3.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-log",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "vary" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/compression",
          "name": "vary",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/express",
          "name": "vary",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/vary",
      "name": "vary",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/vary",
      "resolved": "https://registry.npmjs.org/vary/-/vary-1.1.2.tgz",
      "version": "1.1.2",
    },
    "vm-browserify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-libs-browser",
          "name": "vm-browserify",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vm-browserify",
      "name": "vm-browserify",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/vm-browserify",
      "peer": true,
      "resolved": "https://registry.npmjs.org/vm-browserify/-/vm-browserify-1.1.2.tgz",
      "version": "1.1.2",
    },
    "watchpack" => ArboristNode {
      "children": Map {
        "anymatch" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "anymatch",
              "spec": "~3.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "normalize-path" => EdgeOut {
              "name": "normalize-path",
              "spec": "^3.0.0",
              "to": "node_modules/normalize-path",
              "type": "prod",
            },
            "picomatch" => EdgeOut {
              "name": "picomatch",
              "spec": "^2.0.4",
              "to": "node_modules/picomatch",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/anymatch",
          "name": "anymatch",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/anymatch",
          "peer": true,
          "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
          "version": "3.1.1",
        },
        "binary-extensions" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/is-binary-path",
              "name": "binary-extensions",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/binary-extensions",
          "name": "binary-extensions",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/binary-extensions",
          "peer": true,
          "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.1.0.tgz",
          "version": "2.1.0",
        },
        "braces" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "braces",
              "spec": "~3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fill-range" => EdgeOut {
              "name": "fill-range",
              "spec": "^7.0.1",
              "to": "node_modules/watchpack/node_modules/fill-range",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/braces",
          "name": "braces",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/braces",
          "peer": true,
          "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
          "version": "3.0.2",
        },
        "chokidar" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack",
              "name": "chokidar",
              "spec": "^3.4.1",
              "type": "optional",
            },
          },
          "edgesOut": Map {
            "anymatch" => EdgeOut {
              "name": "anymatch",
              "spec": "~3.1.1",
              "to": "node_modules/watchpack/node_modules/anymatch",
              "type": "prod",
            },
            "braces" => EdgeOut {
              "name": "braces",
              "spec": "~3.0.2",
              "to": "node_modules/watchpack/node_modules/braces",
              "type": "prod",
            },
            "fsevents" => EdgeOut {
              "name": "fsevents",
              "spec": "~2.1.2",
              "to": "node_modules/watchpack/node_modules/fsevents",
              "type": "optional",
            },
            "glob-parent" => EdgeOut {
              "name": "glob-parent",
              "spec": "~5.1.0",
              "to": "node_modules/watchpack/node_modules/glob-parent",
              "type": "prod",
            },
            "is-binary-path" => EdgeOut {
              "name": "is-binary-path",
              "spec": "~2.1.0",
              "to": "node_modules/watchpack/node_modules/is-binary-path",
              "type": "prod",
            },
            "is-glob" => EdgeOut {
              "name": "is-glob",
              "spec": "~4.0.1",
              "to": "node_modules/is-glob",
              "type": "prod",
            },
            "normalize-path" => EdgeOut {
              "name": "normalize-path",
              "spec": "~3.0.0",
              "to": "node_modules/normalize-path",
              "type": "prod",
            },
            "readdirp" => EdgeOut {
              "name": "readdirp",
              "spec": "~3.5.0",
              "to": "node_modules/watchpack/node_modules/readdirp",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/chokidar",
          "name": "chokidar",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/chokidar",
          "peer": true,
          "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.4.3.tgz",
          "version": "3.4.3",
        },
        "fill-range" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/braces",
              "name": "fill-range",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "to-regex-range" => EdgeOut {
              "name": "to-regex-range",
              "spec": "^5.0.1",
              "to": "node_modules/watchpack/node_modules/to-regex-range",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/fill-range",
          "name": "fill-range",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/fill-range",
          "peer": true,
          "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
          "version": "7.0.1",
        },
        "fsevents" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "fsevents",
              "spec": "~2.1.2",
              "type": "optional",
            },
          },
          "location": "node_modules/watchpack/node_modules/fsevents",
          "name": "fsevents",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/fsevents",
          "peer": true,
          "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.3.tgz",
          "version": "2.1.3",
        },
        "glob-parent" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "glob-parent",
              "spec": "~5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-glob" => EdgeOut {
              "name": "is-glob",
              "spec": "^4.0.1",
              "to": "node_modules/is-glob",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/glob-parent",
          "name": "glob-parent",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/glob-parent",
          "peer": true,
          "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.1.tgz",
          "version": "5.1.1",
        },
        "is-binary-path" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "is-binary-path",
              "spec": "~2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "binary-extensions" => EdgeOut {
              "name": "binary-extensions",
              "spec": "^2.0.0",
              "to": "node_modules/watchpack/node_modules/binary-extensions",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/is-binary-path",
          "name": "is-binary-path",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/is-binary-path",
          "peer": true,
          "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
          "version": "2.1.0",
        },
        "is-number" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/to-regex-range",
              "name": "is-number",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/is-number",
          "name": "is-number",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/is-number",
          "peer": true,
          "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
          "version": "7.0.0",
        },
        "readdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/chokidar",
              "name": "readdirp",
              "spec": "~3.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "picomatch" => EdgeOut {
              "name": "picomatch",
              "spec": "^2.2.1",
              "to": "node_modules/picomatch",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/readdirp",
          "name": "readdirp",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/readdirp",
          "peer": true,
          "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.5.0.tgz",
          "version": "3.5.0",
        },
        "to-regex-range" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/watchpack/node_modules/fill-range",
              "name": "to-regex-range",
              "spec": "^5.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-number" => EdgeOut {
              "name": "is-number",
              "spec": "^7.0.0",
              "to": "node_modules/watchpack/node_modules/is-number",
              "type": "prod",
            },
          },
          "location": "node_modules/watchpack/node_modules/to-regex-range",
          "name": "to-regex-range",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack/node_modules/to-regex-range",
          "peer": true,
          "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
          "version": "5.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "watchpack",
          "spec": "^1.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.4.1",
          "to": "node_modules/watchpack/node_modules/chokidar",
          "type": "optional",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.5.0",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "watchpack-chokidar2" => EdgeOut {
          "name": "watchpack-chokidar2",
          "spec": "^2.0.0",
          "to": "node_modules/watchpack-chokidar2",
          "type": "optional",
        },
      },
      "location": "node_modules/watchpack",
      "name": "watchpack",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack",
      "peer": true,
      "resolved": "https://registry.npmjs.org/watchpack/-/watchpack-1.7.4.tgz",
      "version": "1.7.4",
    },
    "watchpack-chokidar2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/watchpack",
          "name": "watchpack-chokidar2",
          "spec": "^2.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^2.1.8",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
      },
      "location": "node_modules/watchpack-chokidar2",
      "name": "watchpack-chokidar2",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/watchpack-chokidar2",
      "peer": true,
      "resolved": "https://registry.npmjs.org/watchpack-chokidar2/-/watchpack-chokidar2-2.0.1.tgz",
      "version": "2.0.1",
    },
    "wbuf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hpack.js",
          "name": "wbuf",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdy-transport",
          "name": "wbuf",
          "spec": "^1.7.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimalistic-assert" => EdgeOut {
          "name": "minimalistic-assert",
          "spec": "^1.0.0",
          "to": "node_modules/minimalistic-assert",
          "type": "prod",
        },
      },
      "location": "node_modules/wbuf",
      "name": "wbuf",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/wbuf",
      "resolved": "https://registry.npmjs.org/wbuf/-/wbuf-1.7.3.tgz",
      "version": "1.7.3",
    },
    "webpack" => ArboristNode {
      "children": Map {
        "schema-utils" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack",
              "name": "schema-utils",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.1.0",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "ajv-errors" => EdgeOut {
              "name": "ajv-errors",
              "spec": "^1.0.0",
              "to": "node_modules/ajv-errors",
              "type": "prod",
            },
            "ajv-keywords" => EdgeOut {
              "name": "ajv-keywords",
              "spec": "^3.1.0",
              "to": "node_modules/ajv-keywords",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack/node_modules/schema-utils",
          "name": "schema-utils",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack/node_modules/schema-utils",
          "peer": true,
          "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "webpack",
          "spec": ">=4.43.0 <6.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "webpack",
          "spec": "^4.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-middleware",
          "name": "webpack",
          "spec": "^4.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "webpack",
          "spec": "^4.0.0 || ^5.0.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@webassemblyjs/ast" => EdgeOut {
          "name": "@webassemblyjs/ast",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/ast",
          "type": "prod",
        },
        "@webassemblyjs/helper-module-context" => EdgeOut {
          "name": "@webassemblyjs/helper-module-context",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/helper-module-context",
          "type": "prod",
        },
        "@webassemblyjs/wasm-edit" => EdgeOut {
          "name": "@webassemblyjs/wasm-edit",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-edit",
          "type": "prod",
        },
        "@webassemblyjs/wasm-parser" => EdgeOut {
          "name": "@webassemblyjs/wasm-parser",
          "spec": "1.9.0",
          "to": "node_modules/@webassemblyjs/wasm-parser",
          "type": "prod",
        },
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^6.4.1",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.10.2",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^3.4.1",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
        "chrome-trace-event" => EdgeOut {
          "name": "chrome-trace-event",
          "spec": "^1.0.2",
          "to": "node_modules/chrome-trace-event",
          "type": "prod",
        },
        "enhanced-resolve" => EdgeOut {
          "name": "enhanced-resolve",
          "spec": "^4.3.0",
          "to": "node_modules/enhanced-resolve",
          "type": "prod",
        },
        "eslint-scope" => EdgeOut {
          "name": "eslint-scope",
          "spec": "^4.0.3",
          "to": "node_modules/eslint-scope",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.2",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
        "loader-runner" => EdgeOut {
          "name": "loader-runner",
          "spec": "^2.4.0",
          "to": "node_modules/loader-runner",
          "type": "prod",
        },
        "loader-utils" => EdgeOut {
          "name": "loader-utils",
          "spec": "^1.2.3",
          "to": "node_modules/loader-utils",
          "type": "prod",
        },
        "memory-fs" => EdgeOut {
          "name": "memory-fs",
          "spec": "^0.4.1",
          "to": "node_modules/memory-fs",
          "type": "prod",
        },
        "micromatch" => EdgeOut {
          "name": "micromatch",
          "spec": "^3.1.10",
          "to": "node_modules/micromatch",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.3",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.1",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "node-libs-browser" => EdgeOut {
          "name": "node-libs-browser",
          "spec": "^2.2.1",
          "to": "node_modules/node-libs-browser",
          "type": "prod",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^1.0.0",
          "to": "node_modules/webpack/node_modules/schema-utils",
          "type": "prod",
        },
        "tapable" => EdgeOut {
          "name": "tapable",
          "spec": "^1.1.3",
          "to": "node_modules/tapable",
          "type": "prod",
        },
        "terser-webpack-plugin" => EdgeOut {
          "name": "terser-webpack-plugin",
          "spec": "^1.4.3",
          "to": "node_modules/terser-webpack-plugin",
          "type": "prod",
        },
        "watchpack" => EdgeOut {
          "name": "watchpack",
          "spec": "^1.7.4",
          "to": "node_modules/watchpack",
          "type": "prod",
        },
        "webpack-sources" => EdgeOut {
          "name": "webpack-sources",
          "spec": "^1.4.1",
          "to": "node_modules/webpack-sources",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack",
      "name": "webpack",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack",
      "peer": true,
      "resolved": "https://registry.npmjs.org/webpack/-/webpack-4.44.2.tgz",
      "version": "4.44.2",
    },
    "webpack-dev-middleware" => ArboristNode {
      "children": Map {
        "mime" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack-dev-middleware",
              "name": "mime",
              "spec": "^2.4.4",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack-dev-middleware/node_modules/mime",
          "name": "mime",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-dev-middleware/node_modules/mime",
          "resolved": "https://registry.npmjs.org/mime/-/mime-2.4.6.tgz",
          "version": "2.4.6",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "webpack-dev-middleware",
          "spec": "^3.7.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "memory-fs" => EdgeOut {
          "name": "memory-fs",
          "spec": "^0.4.1",
          "to": "node_modules/memory-fs",
          "type": "prod",
        },
        "mime" => EdgeOut {
          "name": "mime",
          "spec": "^2.4.4",
          "to": "node_modules/webpack-dev-middleware/node_modules/mime",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "range-parser" => EdgeOut {
          "name": "range-parser",
          "spec": "^1.2.1",
          "to": "node_modules/range-parser",
          "type": "prod",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": "^4.0.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
        "webpack-log" => EdgeOut {
          "name": "webpack-log",
          "spec": "^2.0.0",
          "to": "node_modules/webpack-log",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack-dev-middleware",
      "name": "webpack-dev-middleware",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-dev-middleware",
      "resolved": "https://registry.npmjs.org/webpack-dev-middleware/-/webpack-dev-middleware-3.7.2.tgz",
      "version": "3.7.2",
    },
    "webpack-dev-server" => ArboristNode {
      "children": Map {
        "schema-utils" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack-dev-server",
              "name": "schema-utils",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ajv" => EdgeOut {
              "name": "ajv",
              "spec": "^6.1.0",
              "to": "node_modules/ajv",
              "type": "prod",
            },
            "ajv-errors" => EdgeOut {
              "name": "ajv-errors",
              "spec": "^1.0.0",
              "to": "node_modules/ajv-errors",
              "type": "prod",
            },
            "ajv-keywords" => EdgeOut {
              "name": "ajv-keywords",
              "spec": "^3.1.0",
              "to": "node_modules/ajv-keywords",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack-dev-server/node_modules/schema-utils",
          "name": "schema-utils",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-dev-server/node_modules/schema-utils",
          "resolved": "https://registry.npmjs.org/schema-utils/-/schema-utils-1.0.0.tgz",
          "version": "1.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack-dev-server",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack-dev-server/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-dev-server/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "webpack-dev-server",
          "spec": "^3.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
          "name": "webpack-dev-server",
          "spec": "3.x",
          "type": "peerOptional",
        },
      },
      "edgesOut": Map {
        "ansi-html" => EdgeOut {
          "name": "ansi-html",
          "spec": "0.0.7",
          "to": "node_modules/ansi-html",
          "type": "prod",
        },
        "bonjour" => EdgeOut {
          "name": "bonjour",
          "spec": "^3.5.0",
          "to": "node_modules/bonjour",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^2.1.8",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "compression" => EdgeOut {
          "name": "compression",
          "spec": "^1.7.4",
          "to": "node_modules/compression",
          "type": "prod",
        },
        "connect-history-api-fallback" => EdgeOut {
          "name": "connect-history-api-fallback",
          "spec": "^1.6.0",
          "to": "node_modules/connect-history-api-fallback",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "del" => EdgeOut {
          "name": "del",
          "spec": "^4.1.1",
          "to": "node_modules/del",
          "type": "prod",
        },
        "express" => EdgeOut {
          "name": "express",
          "spec": "^4.17.1",
          "to": "node_modules/express",
          "type": "prod",
        },
        "html-entities" => EdgeOut {
          "name": "html-entities",
          "spec": "^1.3.1",
          "to": "node_modules/html-entities",
          "type": "prod",
        },
        "http-proxy-middleware" => EdgeOut {
          "name": "http-proxy-middleware",
          "spec": "0.19.1",
          "to": "node_modules/http-proxy-middleware",
          "type": "prod",
        },
        "import-local" => EdgeOut {
          "name": "import-local",
          "spec": "^2.0.0",
          "to": "node_modules/import-local",
          "type": "prod",
        },
        "internal-ip" => EdgeOut {
          "name": "internal-ip",
          "spec": "^4.3.0",
          "to": "node_modules/internal-ip",
          "type": "prod",
        },
        "ip" => EdgeOut {
          "name": "ip",
          "spec": "^1.1.5",
          "to": "node_modules/ip",
          "type": "prod",
        },
        "is-absolute-url" => EdgeOut {
          "name": "is-absolute-url",
          "spec": "^3.0.3",
          "to": "node_modules/is-absolute-url",
          "type": "prod",
        },
        "killable" => EdgeOut {
          "name": "killable",
          "spec": "^1.0.1",
          "to": "node_modules/killable",
          "type": "prod",
        },
        "loglevel" => EdgeOut {
          "name": "loglevel",
          "spec": "^1.6.8",
          "to": "node_modules/loglevel",
          "type": "prod",
        },
        "opn" => EdgeOut {
          "name": "opn",
          "spec": "^5.5.0",
          "to": "node_modules/opn",
          "type": "prod",
        },
        "p-retry" => EdgeOut {
          "name": "p-retry",
          "spec": "^3.0.1",
          "to": "node_modules/p-retry",
          "type": "prod",
        },
        "portfinder" => EdgeOut {
          "name": "portfinder",
          "spec": "^1.0.26",
          "to": "node_modules/portfinder",
          "type": "prod",
        },
        "schema-utils" => EdgeOut {
          "name": "schema-utils",
          "spec": "^1.0.0",
          "to": "node_modules/webpack-dev-server/node_modules/schema-utils",
          "type": "prod",
        },
        "selfsigned" => EdgeOut {
          "name": "selfsigned",
          "spec": "^1.10.7",
          "to": "node_modules/selfsigned",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.3.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "serve-index" => EdgeOut {
          "name": "serve-index",
          "spec": "^1.9.1",
          "to": "node_modules/serve-index",
          "type": "prod",
        },
        "sockjs" => EdgeOut {
          "name": "sockjs",
          "spec": "0.3.20",
          "to": "node_modules/sockjs",
          "type": "prod",
        },
        "sockjs-client" => EdgeOut {
          "name": "sockjs-client",
          "spec": "1.4.0",
          "to": "node_modules/sockjs-client",
          "type": "prod",
        },
        "spdy" => EdgeOut {
          "name": "spdy",
          "spec": "^4.0.2",
          "to": "node_modules/spdy",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/webpack-dev-server/node_modules/supports-color",
          "type": "prod",
        },
        "url" => EdgeOut {
          "name": "url",
          "spec": "^0.11.0",
          "to": "node_modules/url",
          "type": "prod",
        },
        "webpack" => EdgeOut {
          "name": "webpack",
          "spec": "^4.0.0 || ^5.0.0",
          "to": "node_modules/webpack",
          "type": "peer",
        },
        "webpack-dev-middleware" => EdgeOut {
          "name": "webpack-dev-middleware",
          "spec": "^3.7.2",
          "to": "node_modules/webpack-dev-middleware",
          "type": "prod",
        },
        "webpack-log" => EdgeOut {
          "name": "webpack-log",
          "spec": "^2.0.0",
          "to": "node_modules/webpack-log",
          "type": "prod",
        },
        "ws" => EdgeOut {
          "name": "ws",
          "spec": "^6.2.1",
          "to": "node_modules/ws",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.3.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack-dev-server",
      "name": "webpack-dev-server",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-dev-server",
      "resolved": "https://registry.npmjs.org/webpack-dev-server/-/webpack-dev-server-3.11.0.tgz",
      "version": "3.11.0",
    },
    "webpack-log" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-middleware",
          "name": "webpack-log",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "webpack-log",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-colors" => EdgeOut {
          "name": "ansi-colors",
          "spec": "^3.0.0",
          "to": "node_modules/ansi-colors",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack-log",
      "name": "webpack-log",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-log",
      "resolved": "https://registry.npmjs.org/webpack-log/-/webpack-log-2.0.0.tgz",
      "version": "2.0.0",
    },
    "webpack-sources" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/webpack-sources",
              "name": "source-map",
              "spec": "~0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/webpack-sources/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-sources/node_modules/source-map",
          "peer": true,
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "webpack-sources",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/webpack",
          "name": "webpack-sources",
          "spec": "^1.4.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-list-map" => EdgeOut {
          "name": "source-list-map",
          "spec": "^2.0.0",
          "to": "node_modules/source-list-map",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.6.1",
          "to": "node_modules/webpack-sources/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/webpack-sources",
      "name": "webpack-sources",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/webpack-sources",
      "peer": true,
      "resolved": "https://registry.npmjs.org/webpack-sources/-/webpack-sources-1.4.3.tgz",
      "version": "1.4.3",
    },
    "websocket-driver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/faye-websocket",
          "name": "websocket-driver",
          "spec": ">=0.5.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sockjs",
          "name": "websocket-driver",
          "spec": "0.6.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sockjs-client/node_modules/faye-websocket",
          "name": "websocket-driver",
          "spec": ">=0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "websocket-extensions" => EdgeOut {
          "name": "websocket-extensions",
          "spec": ">=0.1.1",
          "to": "node_modules/websocket-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/websocket-driver",
      "name": "websocket-driver",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/websocket-driver",
      "resolved": "https://registry.npmjs.org/websocket-driver/-/websocket-driver-0.6.5.tgz",
      "version": "0.6.5",
    },
    "websocket-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/websocket-driver",
          "name": "websocket-extensions",
          "spec": ">=0.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/websocket-extensions",
      "name": "websocket-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/websocket-extensions",
      "resolved": "https://registry.npmjs.org/websocket-extensions/-/websocket-extensions-0.1.4.tgz",
      "version": "0.1.4",
    },
    "which" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "which",
          "spec": "^1.2.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
      "version": "1.3.1",
    },
    "which-module" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "worker-farm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/terser-webpack-plugin",
          "name": "worker-farm",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "errno" => EdgeOut {
          "name": "errno",
          "spec": "~0.1.7",
          "to": "node_modules/errno",
          "type": "prod",
        },
      },
      "location": "node_modules/worker-farm",
      "name": "worker-farm",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/worker-farm",
      "peer": true,
      "resolved": "https://registry.npmjs.org/worker-farm/-/worker-farm-1.7.0.tgz",
      "version": "1.7.0",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/wrap-ansi/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/wrap-ansi/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/wrap-ansi/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "wrap-ansi",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.0.0",
          "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
      "version": "5.1.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "ws" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "ws",
          "spec": "^6.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "async-limiter" => EdgeOut {
          "name": "async-limiter",
          "spec": "~1.0.0",
          "to": "node_modules/async-limiter",
          "type": "prod",
        },
      },
      "location": "node_modules/ws",
      "name": "ws",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/ws",
      "resolved": "https://registry.npmjs.org/ws/-/ws-6.2.1.tgz",
      "version": "6.2.1",
    },
    "xtend" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/stream-http",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/through2",
          "name": "xtend",
          "spec": "~4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/xtend",
      "name": "xtend",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/xtend",
      "peer": true,
      "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.2.tgz",
      "version": "4.0.2",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cacache",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/yallist",
      "peer": true,
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.1.1.tgz",
      "version": "3.1.1",
    },
    "yargs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/webpack-dev-server",
          "name": "yargs",
          "spec": "^13.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.2.tgz",
      "version": "13.3.2",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.2.tgz",
      "version": "13.1.2",
    },
  },
  "edgesOut": Map {
    "@pmmmwh/react-refresh-webpack-plugin" => EdgeOut {
      "name": "@pmmmwh/react-refresh-webpack-plugin",
      "spec": "^0.4.3",
      "to": "node_modules/@pmmmwh/react-refresh-webpack-plugin",
      "type": "prod",
    },
    "webpack-dev-server" => EdgeOut {
      "name": "webpack-dev-server",
      "spec": "^3.11.0",
      "to": "node_modules/webpack-dev-server",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-peer-dep-that-needs-to-be-replaced",
}
`

exports[`test/arborist/build-ideal-tree.js TAP peerOptionals that are devDeps or explicit request > should install the abbrev dep 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "peerOptional",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "optional": true,
      "path": "{CWD}/test/fixtures/peer-optional-installs/node_modules/abbrev",
      "peer": true,
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "*",
          "type": "dev",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/peer-optional-installs/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "peerOptional",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "*",
      "to": "node_modules/wrappy",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-optional-installs",
  "path": "{CWD}/test/fixtures/peer-optional-installs",
}
`

exports[`test/arborist/build-ideal-tree.js TAP peerOptionals that are devDeps or explicit request > should install the wrappy dep, and not remove from peerDeps 1`] = `
ArboristNode {
  "children": Map {
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "*",
          "type": "dev",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/peer-optional-installs/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": null,
      "type": "peerOptional",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "*",
      "to": "node_modules/wrappy",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "peer-optional-installs",
  "path": "{CWD}/test/fixtures/peer-optional-installs",
}
`

exports[`test/arborist/build-ideal-tree.js TAP properly assign fsParent when paths have .. in them > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:../../a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/z/node_modules/a",
      "realpath": "{CWD}/test/fixtures/fs-parent-dots/x/a",
      "resolved": "file:../../../a",
      "target": ArboristNode {
        "location": "../../a",
        "name": "a",
        "path": "{CWD}/test/fixtures/fs-parent-dots/x/a",
        "version": "1.2.3",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:../b",
          "type": "prod",
        },
      },
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/z/node_modules/b",
      "realpath": "{CWD}/test/fixtures/fs-parent-dots/x/y/b",
      "resolved": "file:../../b",
      "target": ArboristNode {
        "location": "../b",
        "name": "b",
        "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/b",
        "version": "1.2.3",
      },
      "version": "1.2.3",
    },
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "x",
          "spec": "file:../..",
          "type": "prod",
        },
      },
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/z/node_modules/x",
      "realpath": "{CWD}/test/fixtures/fs-parent-dots/x",
      "resolved": "file:../../..",
      "target": ArboristNode {
        "fsChildren": Set {
          ArboristNode {
            "location": "../../a",
          },
          ArboristNode {
            "fsChildren": Set {
              ArboristNode {
                "location": "../b",
              },
            },
            "location": "..",
            "name": "y",
            "path": "{CWD}/test/fixtures/fs-parent-dots/x/y",
            "version": "1.2.3",
          },
        },
        "location": "../..",
        "name": "x",
        "path": "{CWD}/test/fixtures/fs-parent-dots/x",
        "version": "1.2.3",
      },
      "version": "1.2.3",
    },
    "y" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "y",
          "spec": "file:..",
          "type": "prod",
        },
      },
      "location": "node_modules/y",
      "name": "y",
      "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/z/node_modules/y",
      "realpath": "{CWD}/test/fixtures/fs-parent-dots/x/y",
      "resolved": "file:../..",
      "target": ArboristNode {
        "location": "..",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:../../a",
      "to": "node_modules/a",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:../b",
      "to": "node_modules/b",
      "type": "prod",
    },
    "x" => EdgeOut {
      "name": "x",
      "spec": "file:../..",
      "to": "node_modules/x",
      "type": "prod",
    },
    "y" => EdgeOut {
      "name": "y",
      "spec": "file:..",
      "to": "node_modules/y",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "z",
  "path": "{CWD}/test/fixtures/fs-parent-dots/x/y/z",
}
`

exports[`test/arborist/build-ideal-tree.js TAP push conflicted peer deps deeper in to the tree to solve > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "name": "@isaacs/testing-peer-dep-conflict-chain-e",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-override" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-override",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-dep-conflict-chain-v" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override",
      "name": "@isaacs/testing-peer-dep-conflict-chain-override",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-override",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-override/-/testing-peer-dep-conflict-chain-override-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-v" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override",
          "name": "@isaacs/testing-peer-dep-conflict-chain-v",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "name": "@isaacs/testing-peer-dep-conflict-chain-v",
      "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep/node_modules/@isaacs/testing-peer-dep-conflict-chain-v",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-v/-/testing-peer-dep-conflict-chain-v-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-override" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-override",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-override",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "override-dep",
  "path": "{CWD}/test/fixtures/testing-peer-dep-conflict-chain/override-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP replace a link with a matching link when the current one is wrong > replace incorrect with correct 1`] = `
ArboristNode {
  "children": Map {
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "x",
          "spec": "file:correct/x",
          "type": "prod",
        },
      },
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/node_modules/x",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/correct/x",
      "resolved": "file:../correct/x",
      "target": ArboristNode {
        "location": "correct/x",
      },
      "version": "1.2.3",
    },
    "y" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "y",
          "spec": "file:correct/x",
          "type": "prod",
        },
      },
      "location": "node_modules/y",
      "name": "y",
      "packageName": "x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/node_modules/y",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/correct/x",
      "resolved": "file:../correct/x",
      "target": ArboristNode {
        "location": "correct/x",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "x" => EdgeOut {
      "name": "x",
      "spec": "file:correct/x",
      "to": "node_modules/x",
      "type": "prod",
    },
    "y" => EdgeOut {
      "name": "y",
      "spec": "file:correct/x",
      "to": "node_modules/y",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "correct/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/correct/x",
      "version": "1.2.3",
    },
    ArboristNode {
      "dev": true,
      "extraneous": true,
      "location": "incorrect/x",
      "name": "x",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong/incorrect/x",
      "peer": true,
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-replace-a-link-with-a-matching-link-when-the-current-one-is-wrong",
}
`

exports[`test/arborist/build-ideal-tree.js TAP respect the yarn.lock file > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/yarn-lock-mkdirp/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.2",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "yarn-lock-mkdirp",
  "path": "{CWD}/test/fixtures/yarn-lock-mkdirp",
}
`

exports[`test/arborist/build-ideal-tree.js TAP respect the yarn.lock file version, if lacking resolved > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/yarn-lock-mkdirp-no-resolved/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.2",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "yarn-lock-mkdirp-no-resolved",
  "path": "{CWD}/test/fixtures/yarn-lock-mkdirp-no-resolved",
}
`

exports[`test/arborist/build-ideal-tree.js TAP store files with a custom indenting > must match snapshot 1`] = `
{
	"name": "tab-indented-package-json",
	"version": "1.0.0",
	"lockfileVersion": 3,
	"requires": true,
	"packages": {
		"": {
			"name": "tab-indented-package-json",
			"version": "1.0.0",
			"dependencies": {
				"abbrev": "^1.0.0"
			}
		},
		"node_modules/abbrev": {
			"version": "1.1.1",
			"resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
			"integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
		}
	}
}

`

exports[`test/arborist/build-ideal-tree.js TAP tap vs react15 > build ideal tree with tap collision 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/core" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/core",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helpers" => EdgeOut {
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helpers",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "gensync" => EdgeOut {
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "to": "node_modules/gensync",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^2.1.2",
          "to": "node_modules/json5",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.3.2",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.4.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/core",
      "name": "@babel/core",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/core",
      "resolved": "https://registry.npmjs.org/@babel/core/-/core-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/generator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-annotate-as-pure" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-annotate-as-pure",
      "name": "@babel/helper-annotate-as-pure",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-annotate-as-pure",
      "resolved": "https://registry.npmjs.org/@babel/helper-annotate-as-pure/-/helper-annotate-as-pure-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-builder-react-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-react-jsx",
      "name": "@babel/helper-builder-react-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-builder-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-react-jsx/-/helper-builder-react-jsx-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-builder-react-jsx-experimental" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-builder-react-jsx-experimental",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-react-jsx-experimental",
      "name": "@babel/helper-builder-react-jsx-experimental",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-builder-react-jsx-experimental",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-react-jsx-experimental/-/helper-builder-react-jsx-experimental-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-member-expression-to-functions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-member-expression-to-functions",
      "name": "@babel/helper-member-expression-to-functions",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-member-expression-to-functions",
      "resolved": "https://registry.npmjs.org/@babel/helper-member-expression-to-functions/-/helper-member-expression-to-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-imports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-imports",
      "name": "@babel/helper-module-imports",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-module-imports",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-imports/-/helper-module-imports-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-transforms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-simple-access" => EdgeOut {
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-simple-access",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-transforms",
      "name": "@babel/helper-module-transforms",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-module-transforms",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-transforms/-/helper-module-transforms-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-optimise-call-expression" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-optimise-call-expression",
      "name": "@babel/helper-optimise-call-expression",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-optimise-call-expression",
      "resolved": "https://registry.npmjs.org/@babel/helper-optimise-call-expression/-/helper-optimise-call-expression-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-plugin-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-plugin-utils",
      "name": "@babel/helper-plugin-utils",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-plugin-utils",
      "resolved": "https://registry.npmjs.org/@babel/helper-plugin-utils/-/helper-plugin-utils-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-replace-supers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-member-expression-to-functions" => EdgeOut {
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-member-expression-to-functions",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-replace-supers",
      "name": "@babel/helper-replace-supers",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-replace-supers",
      "resolved": "https://registry.npmjs.org/@babel/helper-replace-supers/-/helper-replace-supers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-simple-access" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-simple-access",
      "name": "@babel/helper-simple-access",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-simple-access",
      "resolved": "https://registry.npmjs.org/@babel/helper-simple-access/-/helper-simple-access-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.11.0.tgz",
      "version": "7.11.0",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helpers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helpers",
      "name": "@babel/helpers",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/helpers",
      "resolved": "https://registry.npmjs.org/@babel/helpers/-/helpers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/highlight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-parameters" => EdgeOut {
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-parameters",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-object-rest-spread",
      "name": "@babel/plugin-proposal-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-proposal-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-object-rest-spread/-/plugin-proposal-object-rest-spread-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-jsx",
      "name": "@babel/plugin-syntax-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-syntax-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-jsx/-/plugin-syntax-jsx-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-object-rest-spread",
      "name": "@babel/plugin-syntax-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-syntax-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-object-rest-spread/-/plugin-syntax-object-rest-spread-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-transform-destructuring" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-destructuring",
      "name": "@babel/plugin-transform-destructuring",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-transform-destructuring",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-destructuring/-/plugin-transform-destructuring-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-parameters" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-parameters",
      "name": "@babel/plugin-transform-parameters",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-transform-parameters",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-parameters/-/plugin-transform-parameters-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-react-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-builder-react-jsx" => EdgeOut {
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-builder-react-jsx",
          "type": "prod",
        },
        "@babel/helper-builder-react-jsx-experimental" => EdgeOut {
          "name": "@babel/helper-builder-react-jsx-experimental",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-jsx" => EdgeOut {
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-syntax-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-react-jsx",
      "name": "@babel/plugin-transform-react-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/plugin-transform-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-react-jsx/-/plugin-transform-react-jsx-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-annotate-as-pure",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-member-expression-to-functions",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-imports",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-optimise-call-expression",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-simple-access",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@types/yoga-layout" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yoga-layout-prebuilt",
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/yoga-layout",
      "name": "@types/yoga-layout",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/@types/yoga-layout",
      "resolved": "https://registry.npmjs.org/@types/yoga-layout/-/yoga-layout-1.9.2.tgz",
      "version": "1.9.2",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.12.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.11.0",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.1.tgz",
      "version": "4.3.1",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
      "version": "3.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "ansicolors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cardinal",
          "name": "ansicolors",
          "spec": "~0.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ansicolors",
      "name": "ansicolors",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ansicolors",
      "resolved": "https://registry.npmjs.org/ansicolors/-/ansicolors-0.3.2.tgz",
      "version": "0.3.2",
    },
    "anymatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.3.tgz",
      "version": "4.1.3",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "arrify",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-2.0.1.tgz",
      "version": "2.0.1",
    },
    "asap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/promise",
          "name": "asap",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asap",
      "name": "asap",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/asap",
      "resolved": "https://registry.npmjs.org/asap/-/asap-2.0.6.tgz",
      "version": "2.0.6",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "astral-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "auto-bind" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "auto-bind",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/auto-bind",
      "name": "auto-bind",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/auto-bind",
      "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-4.0.0.tgz",
      "version": "4.0.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.10.1.tgz",
      "version": "1.10.1",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.1.0.tgz",
      "version": "2.1.0",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "caller-callsite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^2.0.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-callsite",
      "name": "caller-callsite",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/caller-callsite",
      "resolved": "https://registry.npmjs.org/caller-callsite/-/caller-callsite-2.0.0.tgz",
      "version": "2.0.0",
    },
    "caller-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "caller-path",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-callsite" => EdgeOut {
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "to": "node_modules/caller-callsite",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-2.0.0.tgz",
      "version": "2.0.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-callsite",
          "name": "callsites",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-2.0.0.tgz",
      "version": "2.0.0",
    },
    "camelcase" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "cardinal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "cardinal",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansicolors" => EdgeOut {
          "name": "ansicolors",
          "spec": "~0.3.2",
          "to": "node_modules/ansicolors",
          "type": "prod",
        },
        "redeyed" => EdgeOut {
          "name": "redeyed",
          "spec": "~2.1.0",
          "to": "node_modules/redeyed",
          "type": "prod",
        },
      },
      "location": "node_modules/cardinal",
      "name": "cardinal",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cardinal",
      "resolved": "https://registry.npmjs.org/cardinal/-/cardinal-2.1.1.tgz",
      "version": "2.1.1",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "chokidar" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.5.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.4.3.tgz",
      "version": "3.4.3",
    },
    "ci-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-truncate" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cli-truncate/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cli-truncate/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-truncate/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cli-truncate/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cli-truncate/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-truncate/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
          "version": "3.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cli-truncate",
              "name": "string-width",
              "spec": "^4.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/cli-truncate/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/cli-truncate/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cli-truncate/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-truncate/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cli-truncate/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/cli-truncate/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cli-truncate/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-truncate/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/cli-truncate/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-truncate",
      "name": "cli-truncate",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cli-truncate",
      "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-2.1.0.tgz",
      "version": "2.1.0",
    },
    "cliui" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commondir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "core-js",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-js",
      "name": "core-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/core-js",
      "resolved": "https://registry.npmjs.org/core-js/-/core-js-1.2.7.tgz",
      "version": "1.2.7",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.2",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cp-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "create-react-class" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "create-react-class",
          "spec": "^15.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.3.1",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/create-react-class",
      "name": "create-react-class",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/create-react-class",
      "resolved": "https://registry.npmjs.org/create-react-class/-/create-react-class-15.7.0.tgz",
      "version": "15.7.0",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "decamelize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.2.tgz",
      "version": "4.0.2",
    },
    "diff-frag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate/node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line/node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "encoding" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "encoding",
          "spec": "^0.1.11",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "^0.6.2",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
      },
      "location": "node_modules/encoding",
      "name": "encoding",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/encoding",
      "resolved": "https://registry.npmjs.org/encoding/-/encoding-0.1.13.tgz",
      "version": "0.1.13",
    },
    "error-ex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/redeyed",
          "name": "esprima",
          "spec": "~4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fbjs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "fbjs",
          "spec": "^0.8.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "core-js" => EdgeOut {
          "name": "core-js",
          "spec": "^1.0.0",
          "to": "node_modules/core-js",
          "type": "prod",
        },
        "isomorphic-fetch" => EdgeOut {
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "to": "node_modules/isomorphic-fetch",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.0.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "promise" => EdgeOut {
          "name": "promise",
          "spec": "^7.1.1",
          "to": "node_modules/promise",
          "type": "prod",
        },
        "setimmediate" => EdgeOut {
          "name": "setimmediate",
          "spec": "^1.0.5",
          "to": "node_modules/setimmediate",
          "type": "prod",
        },
        "ua-parser-js" => EdgeOut {
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "to": "node_modules/ua-parser-js",
          "type": "prod",
        },
      },
      "location": "node_modules/fbjs",
      "name": "fbjs",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fbjs",
      "resolved": "https://registry.npmjs.org/fbjs/-/fbjs-0.8.17.tgz",
      "version": "0.8.17",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.136.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.136.0.tgz",
      "version": "0.136.0",
    },
    "flow-remove-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.136.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.136.0.tgz",
      "version": "2.136.0",
    },
    "foreground-child" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.3.tgz",
      "version": "2.1.3",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "gensync" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "type": "prod",
        },
      },
      "location": "node_modules/gensync",
      "name": "gensync",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/gensync",
      "resolved": "https://registry.npmjs.org/gensync/-/gensync-1.0.0-beta.1.tgz",
      "version": "1.0.0-beta.1",
    },
    "get-caller-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.1.tgz",
      "version": "5.1.1",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "har-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.3",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.5.tgz",
      "version": "5.1.5",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.8.tgz",
      "version": "2.8.8",
    },
    "html-escaper" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "html-escaper",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/html-escaper",
      "name": "html-escaper",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/html-escaper",
      "resolved": "https://registry.npmjs.org/html-escaper/-/html-escaper-2.0.2.tgz",
      "version": "2.0.2",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "iconv-lite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/encoding",
          "name": "iconv-lite",
          "spec": "^0.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3.0.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/iconv-lite",
      "name": "iconv-lite",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/iconv-lite",
      "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.6.2.tgz",
      "version": "0.6.2",
    },
    "import-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "import-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "import-jsx",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/core",
          "type": "prod",
        },
        "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-destructuring" => EdgeOut {
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "to": "node_modules/@babel/plugin-transform-destructuring",
          "type": "prod",
        },
        "@babel/plugin-transform-react-jsx" => EdgeOut {
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "to": "node_modules/@babel/plugin-transform-react-jsx",
          "type": "prod",
        },
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^2.0.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^3.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/import-jsx",
      "name": "import-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/import-jsx",
      "resolved": "https://registry.npmjs.org/import-jsx/-/import-jsx-3.1.0.tgz",
      "version": "3.1.0",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-arrayish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-ci" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/log-update/node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isomorphic-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-fetch" => EdgeOut {
          "name": "node-fetch",
          "spec": "^1.0.1",
          "to": "node_modules/node-fetch",
          "type": "prod",
        },
        "whatwg-fetch" => EdgeOut {
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "to": "node_modules/whatwg-fetch",
          "type": "prod",
        },
      },
      "location": "node_modules/isomorphic-fetch",
      "name": "isomorphic-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/isomorphic-fetch",
      "resolved": "https://registry.npmjs.org/isomorphic-fetch/-/isomorphic-fetch-2.2.1.tgz",
      "version": "2.2.1",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-source-maps",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "html-escaper" => EdgeOut {
          "name": "html-escaper",
          "spec": "^2.0.0",
          "to": "node_modules/html-escaper",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.7.tgz",
      "version": "2.2.7",
    },
    "jackspeak" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.14.0.tgz",
      "version": "3.14.0",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "json5" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "json5",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/json5",
      "resolved": "https://registry.npmjs.org/json5/-/json5-2.1.3.tgz",
      "version": "2.1.3",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "lodash.flattendeep" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "lodash.throttle" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.throttle",
      "name": "lodash.throttle",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/lodash.throttle",
      "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
      "version": "4.1.1",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "log-update" => ArboristNode {
      "children": Map {
        "ansi-escapes" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/ansi-escapes",
          "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
          "version": "3.2.0",
        },
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cli-cursor" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/cli-cursor",
          "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
          "version": "2.1.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "mimic-fn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/mimic-fn",
          "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
          "version": "1.2.0",
        },
        "onetime" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "to": "node_modules/log-update/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/onetime",
          "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
          "version": "2.0.1",
        },
        "restore-cursor" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/restore-cursor",
          "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/log-update/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/log-update/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/log-update/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "log-update",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^3.2.0",
          "to": "node_modules/log-update/node_modules/ansi-escapes",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^2.1.0",
          "to": "node_modules/log-update/node_modules/cli-cursor",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^5.0.0",
          "to": "node_modules/log-update/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/log-update",
      "name": "log-update",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/log-update",
      "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.4.0.tgz",
      "version": "3.4.0",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "loose-envify",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "loose-envify",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.6.tgz",
      "version": "1.3.6",
    },
    "merge-source-map" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/merge-source-map",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/merge-source-map/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/merge-source-map/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/merge-source-map/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.44.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.44.0.tgz",
      "version": "1.44.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.44.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.27.tgz",
      "version": "2.1.27",
    },
    "mimic-fn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "minipass",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.3.tgz",
      "version": "3.1.3",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "ms",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "nested-error-stacks" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "node-fetch",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "encoding" => EdgeOut {
          "name": "encoding",
          "spec": "^0.1.11",
          "to": "node_modules/encoding",
          "type": "prod",
        },
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/node-fetch",
      "name": "node-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/node-fetch",
      "resolved": "https://registry.npmjs.org/node-fetch/-/node-fetch-1.7.3.tgz",
      "version": "1.7.3",
    },
    "node-modules-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "children": Map {
        "resolve-from" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nyc",
              "name": "resolve-from",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/nyc/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/nyc/node_modules/resolve-from",
          "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/nyc/node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.2.tgz",
      "version": "5.1.2",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.2.tgz",
      "version": "1.5.2",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
      "version": "2.3.0",
    },
    "p-locate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.2.2.tgz",
      "version": "2.2.2",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "promise" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "promise",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asap" => EdgeOut {
          "name": "asap",
          "spec": "~2.0.3",
          "to": "node_modules/asap",
          "type": "prod",
        },
      },
      "location": "node_modules/promise",
      "name": "promise",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/promise",
      "resolved": "https://registry.npmjs.org/promise/-/promise-7.3.1.tgz",
      "version": "7.3.1",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.5.10",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.28",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.8.0.tgz",
      "version": "1.8.0",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "punycode",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "react",
          "spec": "15",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "create-react-class" => EdgeOut {
          "name": "create-react-class",
          "spec": "^15.6.0",
          "to": "node_modules/create-react-class",
          "type": "prod",
        },
        "fbjs" => EdgeOut {
          "name": "fbjs",
          "spec": "^0.8.9",
          "to": "node_modules/fbjs",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.5.10",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-15.7.0.tgz",
      "version": "15.7.0",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.13.1.tgz",
      "version": "16.13.1",
    },
    "read-pkg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.1",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.5.0.tgz",
      "version": "3.5.0",
    },
    "redeyed" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cardinal",
          "name": "redeyed",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "~4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/redeyed",
      "name": "redeyed",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/redeyed",
      "resolved": "https://registry.npmjs.org/redeyed/-/redeyed-2.1.1.tgz",
      "version": "2.1.1",
    },
    "release-zalgo" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.3",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.2.tgz",
      "version": "2.88.2",
    },
    "require-directory" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "resolve",
          "spec": "^1.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.17.0.tgz",
      "version": "1.17.0",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "resolve-from",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
      "version": "3.0.0",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/convert-source-map",
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
      "version": "5.1.2",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/iconv-lite",
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "scheduler" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/scheduler",
      "name": "scheduler",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/scheduler",
      "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.18.0.tgz",
      "version": "0.18.0",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "semver",
          "spec": "^5.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "setimmediate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "setimmediate",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/setimmediate",
      "name": "setimmediate",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/setimmediate",
      "resolved": "https://registry.npmjs.org/setimmediate/-/setimmediate-1.0.5.tgz",
      "version": "1.0.5",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "slice-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/slice-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/slice-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
          "version": "4.3.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/slice-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/slice-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/slice-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/slice-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^2.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi/node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
      "version": "0.5.7",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/source-map-support/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.17",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
      "version": "0.5.19",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.1.tgz",
      "version": "3.1.1",
    },
    "spdx-exceptions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.3.0.tgz",
      "version": "2.3.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.1.tgz",
      "version": "3.0.1",
    },
    "spdx-license-ids" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.6.tgz",
      "version": "3.0.6",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/string-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "astral-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "astral-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/string-length/node_modules/astral-regex",
          "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/string-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/string-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/string-length/node_modules/astral-regex",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/string-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-length",
      "name": "string-length",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/string-length",
      "resolved": "https://registry.npmjs.org/string-length/-/string-length-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "string-width",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
      "version": "2.1.1",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^3.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
      "version": "4.0.0",
    },
    "strip-bom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "tap" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/tap/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
          "version": "4.3.0",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/tap/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
          "version": "3.0.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/tap/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
          "version": "4.0.0",
        },
        "ink" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ink",
              "spec": "^2.6.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": null,
              "type": "peerOptional",
            },
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "to": "node_modules/ansi-escapes",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^2.0.1",
              "to": "node_modules/arrify",
              "type": "prod",
            },
            "auto-bind" => EdgeOut {
              "name": "auto-bind",
              "spec": "^4.0.0",
              "to": "node_modules/auto-bind",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/cli-cursor",
              "type": "prod",
            },
            "cli-truncate" => EdgeOut {
              "name": "cli-truncate",
              "spec": "^2.1.0",
              "to": "node_modules/cli-truncate",
              "type": "prod",
            },
            "is-ci" => EdgeOut {
              "name": "is-ci",
              "spec": "^2.0.0",
              "to": "node_modules/is-ci",
              "type": "prod",
            },
            "lodash.throttle" => EdgeOut {
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "to": "node_modules/lodash.throttle",
              "type": "prod",
            },
            "log-update" => EdgeOut {
              "name": "log-update",
              "spec": "^3.0.0",
              "to": "node_modules/log-update",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "react-reconciler" => EdgeOut {
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "to": "node_modules/tap/node_modules/react-reconciler",
              "type": "prod",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/scheduler",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/slice-ansi",
              "type": "prod",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/string-length",
              "type": "prod",
            },
            "widest-line" => EdgeOut {
              "name": "widest-line",
              "spec": "^3.1.0",
              "to": "node_modules/widest-line",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "to": "node_modules/tap/node_modules/wrap-ansi",
              "type": "prod",
            },
            "yoga-layout-prebuilt" => EdgeOut {
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "to": "node_modules/yoga-layout-prebuilt",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ink",
          "name": "ink",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/ink",
          "resolved": "https://registry.npmjs.org/ink/-/ink-2.7.1.tgz",
          "version": "2.7.1",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
          "version": "3.0.0",
        },
        "react" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "react",
              "spec": "^16.12.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react",
              "spec": ">=16.8.0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "react",
              "spec": "^16.0.0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "react",
              "spec": "^16.8.6",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react",
          "name": "react",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/react",
          "resolved": "https://registry.npmjs.org/react/-/react-16.14.0.tgz",
          "version": "16.14.0",
        },
        "react-reconciler" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.0.0",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/scheduler",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-reconciler",
          "name": "react-reconciler",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/react-reconciler",
          "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.24.0.tgz",
          "version": "0.24.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
          "version": "7.2.0",
        },
        "treport" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "treport",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cardinal" => EdgeOut {
              "name": "cardinal",
              "spec": "^2.1.1",
              "to": "node_modules/cardinal",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "import-jsx" => EdgeOut {
              "name": "import-jsx",
              "spec": "^3.1.0",
              "to": "node_modules/import-jsx",
              "type": "prod",
            },
            "ink" => EdgeOut {
              "name": "ink",
              "spec": "^2.6.0",
              "to": "node_modules/tap/node_modules/ink",
              "type": "prod",
            },
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.2",
              "to": "node_modules/ms",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.8.6",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/string-length",
              "type": "prod",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^10.0.1",
              "to": "node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^2.0.2",
              "to": "node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/treport",
          "name": "treport",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/treport",
          "resolved": "https://registry.npmjs.org/treport/-/treport-1.0.2.tgz",
          "version": "1.0.2",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
          "version": "6.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "unbundled",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.8",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/tap/node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/tap/node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "name": "treport",
          "spec": "^1.0.0",
          "to": "node_modules/tap/node_modules/treport",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.2-unbundled.tgz",
      "version": "14.10.2-unbundled",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "escape-string-regexp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "escape-string-regexp",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap-mocha-reporter/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^2.0.0",
          "to": "node_modules/tap-mocha-reporter/node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^2.0.2",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^10.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "tap-parser",
          "spec": "^10.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.1.0.tgz",
      "version": "10.1.0",
    },
    "tap-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.28",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.5.0.tgz",
      "version": "2.5.0",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.17",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.7",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "3.1.1",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.10.2.tgz",
      "version": "8.10.2",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "type-fest" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-escapes",
          "name": "type-fest",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.11.0.tgz",
      "version": "0.11.0",
    },
    "typedarray-to-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "typescript",
          "spec": ">=2.7",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.9.7.tgz",
      "version": "3.9.7",
    },
    "ua-parser-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "type": "prod",
        },
      },
      "location": "node_modules/ua-parser-js",
      "name": "ua-parser-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/ua-parser-js",
      "resolved": "https://registry.npmjs.org/ua-parser-js/-/ua-parser-js-0.7.22.tgz",
      "version": "0.7.22",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "unicode-length",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.0.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-2.0.2.tgz",
      "version": "2.0.2",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "validate-npm-package-license" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "whatwg-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "type": "prod",
        },
      },
      "location": "node_modules/whatwg-fetch",
      "name": "whatwg-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/whatwg-fetch",
      "resolved": "https://registry.npmjs.org/whatwg-fetch/-/whatwg-fetch-3.4.1.tgz",
      "version": "3.4.1",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "widest-line" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/widest-line/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/widest-line/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/widest-line/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/widest-line/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/widest-line/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/widest-line/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
          "version": "3.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/widest-line",
              "name": "string-width",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/widest-line/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/widest-line/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/widest-line/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/widest-line/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/widest-line/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/widest-line/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/widest-line/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/widest-line/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/widest-line/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrap-ansi/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "number-is-nan" => EdgeOut {
              "name": "number-is-nan",
              "spec": "^1.0.0",
              "to": "node_modules/number-is-nan",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
          "version": "1.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "code-point-at" => EdgeOut {
              "name": "code-point-at",
              "spec": "^1.0.0",
              "to": "node_modules/code-point-at",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^1.0.0",
              "to": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrap-ansi/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
          "version": "1.0.2",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/wrap-ansi/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrap-ansi/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.1",
          "to": "node_modules/wrap-ansi/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
      "version": "2.1.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.3.tgz",
      "version": "3.0.3",
    },
    "y18n" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yaml",
          "spec": "^1.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.10.0.tgz",
      "version": "1.10.0",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.2.tgz",
      "version": "13.3.2",
    },
    "yargs-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.2.tgz",
      "version": "13.1.2",
    },
    "yn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
    "yoga-layout-prebuilt" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/yoga-layout" => EdgeOut {
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "to": "node_modules/@types/yoga-layout",
          "type": "prod",
        },
      },
      "location": "node_modules/yoga-layout-prebuilt",
      "name": "yoga-layout-prebuilt",
      "path": "{CWD}/test/fixtures/tap-react15-collision/node_modules/yoga-layout-prebuilt",
      "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.10.0.tgz",
      "version": "1.10.0",
    },
  },
  "edgesOut": Map {
    "react" => EdgeOut {
      "name": "react",
      "spec": "15",
      "to": "node_modules/react",
      "type": "prod",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "unbundled",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-react15-collision",
  "packageName": "tap-14.7-react-15-peerdep-collision",
  "path": "{CWD}/test/fixtures/tap-react15-collision",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP tap vs react15 with legacy shrinkwrap > tap collision with legacy sw file 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.5.5.tgz",
      "version": "7.5.5",
    },
    "@babel/core" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/core",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/core/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/core/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/core",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.7.7",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helpers" => EdgeOut {
          "name": "@babel/helpers",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helpers",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.7",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^2.1.0",
          "to": "node_modules/json5",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.3.2",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.4.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/core/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/core",
      "name": "@babel/core",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/core",
      "resolved": "https://registry.npmjs.org/@babel/core/-/core-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/generator" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/generator",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/generator/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/generator/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/generator",
          "spec": "^7.7.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/generator/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/helper-builder-react-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.0",
          "to": "node_modules/esutils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-react-jsx",
      "name": "@babel/helper-builder-react-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helper-builder-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-react-jsx/-/helper-builder-react-jsx-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-plugin-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-plugin-utils",
      "name": "@babel/helper-plugin-utils",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helper-plugin-utils",
      "resolved": "https://registry.npmjs.org/@babel/helper-plugin-utils/-/helper-plugin-utils-7.0.0.tgz",
      "version": "7.0.0",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helpers" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helpers",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helpers",
      "name": "@babel/helpers",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/helpers",
      "resolved": "https://registry.npmjs.org/@babel/helpers/-/helpers-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/highlight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.5.0.tgz",
      "version": "7.5.0",
    },
    "@babel/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/parser",
          "spec": "^7.7.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-object-rest-spread",
      "name": "@babel/plugin-proposal-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/plugin-proposal-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-object-rest-spread/-/plugin-proposal-object-rest-spread-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/plugin-syntax-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-jsx",
      "name": "@babel/plugin-syntax-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/plugin-syntax-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-jsx/-/plugin-syntax-jsx-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-object-rest-spread",
      "name": "@babel/plugin-syntax-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/plugin-syntax-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-object-rest-spread/-/plugin-syntax-object-rest-spread-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/plugin-transform-destructuring" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-destructuring",
      "name": "@babel/plugin-transform-destructuring",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/plugin-transform-destructuring",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-destructuring/-/plugin-transform-destructuring-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/plugin-transform-react-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-builder-react-jsx" => EdgeOut {
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-builder-react-jsx",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-jsx" => EdgeOut {
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/plugin-syntax-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-react-jsx",
      "name": "@babel/plugin-transform-react-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/plugin-transform-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-react-jsx/-/plugin-transform-react-jsx-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yaml",
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerator-runtime" => EdgeOut {
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "to": "node_modules/regenerator-runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/runtime",
      "name": "@babel/runtime",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/runtime",
      "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/traverse",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/traverse",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@types/color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink/node_modules/ansi-styles",
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/slice-ansi/node_modules/ansi-styles",
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport/node_modules/ansi-styles",
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi/node_modules/ansi-styles",
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/color-name",
      "name": "@types/color-name",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/@types/color-name",
      "resolved": "https://registry.npmjs.org/@types/color-name/-/color-name-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.2.tgz",
      "version": "6.10.2",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.1",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.0.tgz",
      "version": "4.3.0",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
      "version": "5.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "ansicolors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cardinal",
          "name": "ansicolors",
          "spec": "~0.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ansicolors",
      "name": "ansicolors",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ansicolors",
      "resolved": "https://registry.npmjs.org/ansicolors/-/ansicolors-0.3.2.tgz",
      "version": "0.3.2",
    },
    "anymatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.2.tgz",
      "version": "4.1.2",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "arrify",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-2.0.1.tgz",
      "version": "2.0.1",
    },
    "asap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/promise",
          "name": "asap",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asap",
      "name": "asap",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/asap",
      "resolved": "https://registry.npmjs.org/asap/-/asap-2.0.6.tgz",
      "version": "2.0.6",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "astral-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "auto-bind" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "auto-bind",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": ">=16.8.0",
          "to": null,
          "type": "peerOptional",
        },
      },
      "location": "node_modules/auto-bind",
      "name": "auto-bind",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/auto-bind",
      "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-3.0.0.tgz",
      "version": "3.0.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.9.0.tgz",
      "version": "1.9.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "caller-callsite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^2.0.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-callsite",
      "name": "caller-callsite",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/caller-callsite",
      "resolved": "https://registry.npmjs.org/caller-callsite/-/caller-callsite-2.0.0.tgz",
      "version": "2.0.0",
    },
    "caller-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "caller-path",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-callsite" => EdgeOut {
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "to": "node_modules/caller-callsite",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-2.0.0.tgz",
      "version": "2.0.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-callsite",
          "name": "callsites",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-2.0.0.tgz",
      "version": "2.0.0",
    },
    "camelcase" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "cardinal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/treport",
          "name": "cardinal",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansicolors" => EdgeOut {
          "name": "ansicolors",
          "spec": "~0.3.2",
          "to": "node_modules/ansicolors",
          "type": "prod",
        },
        "redeyed" => EdgeOut {
          "name": "redeyed",
          "spec": "~2.1.0",
          "to": "node_modules/redeyed",
          "type": "prod",
        },
      },
      "location": "node_modules/cardinal",
      "name": "cardinal",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cardinal",
      "resolved": "https://registry.npmjs.org/cardinal/-/cardinal-2.1.1.tgz",
      "version": "2.1.1",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "chokidar" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.3.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.3.1.tgz",
      "version": "3.3.1",
    },
    "ci-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-truncate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-truncate",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-truncate",
      "name": "cli-truncate",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cli-truncate",
      "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-2.1.0.tgz",
      "version": "2.1.0",
    },
    "cliui" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "string-width",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/cliui/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "to": "node_modules/cliui/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "to": "node_modules/cliui/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
          "version": "4.0.0",
        },
        "wrap-ansi" => ArboristNode {
          "children": Map {
            "ansi-regex" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
              "version": "2.1.1",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
              "version": "1.0.0",
            },
            "string-width" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "code-point-at" => EdgeOut {
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "to": "node_modules/code-point-at",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
              "version": "1.0.2",
            },
            "strip-ansi" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
              "version": "3.0.1",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^1.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
          "version": "2.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/cliui/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/cliui/node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/cliui/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commander" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "commander",
          "spec": "~2.20.3",
          "type": "prod",
        },
      },
      "location": "node_modules/commander",
      "name": "commander",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/commander",
      "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.3.tgz",
      "version": "2.20.3",
    },
    "commondir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/convert-source-map",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/convert-source-map/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/convert-source-map/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/convert-source-map/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "core-js",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-js",
      "name": "core-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/core-js",
      "resolved": "https://registry.npmjs.org/core-js/-/core-js-1.2.7.tgz",
      "version": "1.2.7",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.9.tgz",
      "version": "3.0.9",
    },
    "cp-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "create-react-class" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "create-react-class",
          "spec": "^15.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fbjs" => EdgeOut {
          "name": "fbjs",
          "spec": "^0.8.9",
          "to": "node_modules/fbjs",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.3.1",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/create-react-class",
      "name": "create-react-class",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/create-react-class",
      "resolved": "https://registry.npmjs.org/create-react-class/-/create-react-class-15.6.3.tgz",
      "version": "15.6.3",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
      "version": "4.1.1",
    },
    "decamelize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.1.tgz",
      "version": "4.0.1",
    },
    "diff-frag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "encoding" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "encoding",
          "spec": "^0.1.11",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "~0.4.13",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
      },
      "location": "node_modules/encoding",
      "name": "encoding",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/encoding",
      "resolved": "https://registry.npmjs.org/encoding/-/encoding-0.1.12.tgz",
      "version": "0.1.12",
    },
    "error-ex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/redeyed",
          "name": "esprima",
          "spec": "~4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "esutils",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
      "version": "2.0.1",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fbjs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "fbjs",
          "spec": "^0.8.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "fbjs",
          "spec": "^0.8.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "core-js" => EdgeOut {
          "name": "core-js",
          "spec": "^1.0.0",
          "to": "node_modules/core-js",
          "type": "prod",
        },
        "isomorphic-fetch" => EdgeOut {
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "to": "node_modules/isomorphic-fetch",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.0.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "promise" => EdgeOut {
          "name": "promise",
          "spec": "^7.1.1",
          "to": "node_modules/promise",
          "type": "prod",
        },
        "setimmediate" => EdgeOut {
          "name": "setimmediate",
          "spec": "^1.0.5",
          "to": "node_modules/setimmediate",
          "type": "prod",
        },
        "ua-parser-js" => EdgeOut {
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "to": "node_modules/ua-parser-js",
          "type": "prod",
        },
      },
      "location": "node_modules/fbjs",
      "name": "fbjs",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fbjs",
      "resolved": "https://registry.npmjs.org/fbjs/-/fbjs-0.8.17.tgz",
      "version": "0.8.17",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.114.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.114.0.tgz",
      "version": "0.114.0",
    },
    "flow-remove-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.114.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.114.0.tgz",
      "version": "2.114.0",
    },
    "foreground-child" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.2.tgz",
      "version": "2.1.2",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "get-caller-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.0.tgz",
      "version": "5.1.0",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.3.tgz",
      "version": "4.2.3",
    },
    "handlebars" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "handlebars",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.0",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "optimist" => EdgeOut {
          "name": "optimist",
          "spec": "^0.6.1",
          "to": "node_modules/optimist",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "uglify-js" => EdgeOut {
          "name": "uglify-js",
          "spec": "^3.1.4",
          "to": "node_modules/uglify-js",
          "type": "optional",
        },
      },
      "location": "node_modules/handlebars",
      "name": "handlebars",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/handlebars",
      "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.5.3.tgz",
      "version": "4.5.3",
    },
    "har-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.5.5",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
      "version": "5.1.3",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.5.tgz",
      "version": "2.8.5",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "iconv-lite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/encoding",
          "name": "iconv-lite",
          "spec": "~0.4.13",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/iconv-lite",
      "name": "iconv-lite",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/iconv-lite",
      "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
      "version": "0.4.24",
    },
    "import-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "import-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport",
          "name": "import-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/core",
          "type": "prod",
        },
        "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-destructuring" => EdgeOut {
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "to": "node_modules/@babel/plugin-transform-destructuring",
          "type": "prod",
        },
        "@babel/plugin-transform-react-jsx" => EdgeOut {
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "to": "node_modules/@babel/plugin-transform-react-jsx",
          "type": "prod",
        },
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^2.0.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^3.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/import-jsx",
      "name": "import-jsx",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/import-jsx",
      "resolved": "https://registry.npmjs.org/import-jsx/-/import-jsx-3.0.0.tgz",
      "version": "3.0.0",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "ink" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/color-name" => EdgeOut {
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "to": "node_modules/@types/color-name",
              "type": "prod",
            },
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/ink/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.2.0.tgz",
          "version": "4.2.0",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/ink/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/ink/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
          "version": "3.0.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/ink/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
          "version": "4.0.0",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/ink/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/ink/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/ink/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.1.0.tgz",
          "version": "7.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ink",
          "spec": "^2.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport",
          "name": "ink",
          "spec": "^2.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "error": "MISSING",
          "name": "@types/react",
          "spec": ">=16.8.0",
          "to": null,
          "type": "peer",
        },
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^2.0.1",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "auto-bind" => EdgeOut {
          "name": "auto-bind",
          "spec": "^3.0.0",
          "to": "node_modules/auto-bind",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^3.0.0",
          "to": "node_modules/ink/node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-truncate" => EdgeOut {
          "name": "cli-truncate",
          "spec": "^2.0.0",
          "to": "node_modules/cli-truncate",
          "type": "prod",
        },
        "is-ci" => EdgeOut {
          "name": "is-ci",
          "spec": "^2.0.0",
          "to": "node_modules/is-ci",
          "type": "prod",
        },
        "lodash.throttle" => EdgeOut {
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "to": "node_modules/lodash.throttle",
          "type": "prod",
        },
        "log-update" => EdgeOut {
          "name": "log-update",
          "spec": "^3.0.0",
          "to": "node_modules/log-update",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "error": "INVALID",
          "name": "react",
          "spec": ">=16.8.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "react-reconciler" => EdgeOut {
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "to": "node_modules/react-reconciler",
          "type": "prod",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-length" => EdgeOut {
          "name": "string-length",
          "spec": "^3.1.0",
          "to": "node_modules/string-length",
          "type": "prod",
        },
        "widest-line" => EdgeOut {
          "name": "widest-line",
          "spec": "^3.1.0",
          "to": "node_modules/widest-line",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
        "yoga-layout-prebuilt" => EdgeOut {
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "to": "node_modules/yoga-layout-prebuilt",
          "type": "prod",
        },
      },
      "location": "node_modules/ink",
      "name": "ink",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ink",
      "resolved": "https://registry.npmjs.org/ink/-/ink-2.6.0.tgz",
      "version": "2.6.0",
    },
    "is-arrayish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-ci" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isomorphic-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-fetch" => EdgeOut {
          "name": "node-fetch",
          "spec": "^1.0.1",
          "to": "node_modules/node-fetch",
          "type": "prod",
        },
        "whatwg-fetch" => EdgeOut {
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "to": "node_modules/whatwg-fetch",
          "type": "prod",
        },
      },
      "location": "node_modules/isomorphic-fetch",
      "name": "isomorphic-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/isomorphic-fetch",
      "resolved": "https://registry.npmjs.org/isomorphic-fetch/-/isomorphic-fetch-2.2.1.tgz",
      "version": "2.2.1",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "handlebars" => EdgeOut {
          "name": "handlebars",
          "spec": "^4.1.2",
          "to": "node_modules/handlebars",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.6.tgz",
      "version": "2.2.6",
    },
    "jackspeak" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
      "version": "3.13.1",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "json5" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "json5",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/json5",
      "resolved": "https://registry.npmjs.org/json5/-/json5-2.1.1.tgz",
      "version": "2.1.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.15.tgz",
      "version": "4.17.15",
    },
    "lodash.flattendeep" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "lodash.throttle" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.throttle",
      "name": "lodash.throttle",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/lodash.throttle",
      "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
      "version": "4.1.1",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "log-update" => ArboristNode {
      "children": Map {
        "ansi-escapes" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/ansi-escapes",
          "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
          "version": "3.2.0",
        },
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cli-cursor" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/cli-cursor",
          "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
          "version": "2.1.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "mimic-fn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/mimic-fn",
          "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
          "version": "1.2.0",
        },
        "onetime" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "to": "node_modules/log-update/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/onetime",
          "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
          "version": "2.0.1",
        },
        "restore-cursor" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/restore-cursor",
          "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/log-update/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/log-update/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/log-update/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "log-update",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^3.2.0",
          "to": "node_modules/log-update/node_modules/ansi-escapes",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^2.1.0",
          "to": "node_modules/log-update/node_modules/cli-cursor",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^5.0.0",
          "to": "node_modules/log-update/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/log-update",
      "name": "log-update",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/log-update",
      "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.3.0.tgz",
      "version": "3.3.0",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "loose-envify",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "loose-envify",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
      "version": "1.3.5",
    },
    "merge-source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.42.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.42.0.tgz",
      "version": "1.42.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.42.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.25.tgz",
      "version": "2.1.25",
    },
    "mimic-fn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
      "version": "1.2.0",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "minipass",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.1.tgz",
      "version": "3.1.1",
    },
    "mkdirp" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "node_modules/mkdirp/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/mkdirp/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/mkdirp/node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport",
          "name": "ms",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "neo-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "neo-async",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/neo-async",
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.1.tgz",
      "version": "2.6.1",
    },
    "nested-error-stacks" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "node-fetch",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "encoding" => EdgeOut {
          "name": "encoding",
          "spec": "^0.1.11",
          "to": "node_modules/encoding",
          "type": "prod",
        },
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/node-fetch",
      "name": "node-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/node-fetch",
      "resolved": "https://registry.npmjs.org/node-fetch/-/node-fetch-1.7.3.tgz",
      "version": "1.7.3",
    },
    "node-modules-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "children": Map {
        "resolve-from" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nyc",
              "name": "resolve-from",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/nyc/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/nyc/node_modules/resolve-from",
          "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/nyc/node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.0.tgz",
      "version": "5.1.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
      "version": "1.5.1",
    },
    "optimist" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/optimist",
              "name": "minimist",
              "spec": "~0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/optimist/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/optimist/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.10.tgz",
          "version": "0.0.10",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "optimist",
          "spec": "^0.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "~0.0.1",
          "to": "node_modules/optimist/node_modules/minimist",
          "type": "prod",
        },
        "wordwrap" => EdgeOut {
          "name": "wordwrap",
          "spec": "~0.0.2",
          "to": "node_modules/wordwrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optimist",
      "name": "optimist",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/optimist",
      "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
      "version": "0.6.1",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.1.tgz",
      "version": "2.2.1",
    },
    "p-locate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.1.1.tgz",
      "version": "2.1.1",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "process-nextick-args" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.1.tgz",
      "version": "2.0.1",
    },
    "promise" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "promise",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asap" => EdgeOut {
          "name": "asap",
          "spec": "~2.0.3",
          "to": "node_modules/asap",
          "type": "prod",
        },
      },
      "location": "node_modules/promise",
      "name": "promise",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/promise",
      "resolved": "https://registry.npmjs.org/promise/-/promise-7.3.1.tgz",
      "version": "7.3.1",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.5.10",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.24",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.7.0.tgz",
      "version": "1.7.0",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/treport/node_modules/unicode-length",
          "name": "punycode",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "react",
          "spec": "15",
          "type": "prod",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/treport",
          "name": "react",
          "spec": "^16.8.6",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "create-react-class" => EdgeOut {
          "name": "create-react-class",
          "spec": "^15.6.0",
          "to": "node_modules/create-react-class",
          "type": "prod",
        },
        "fbjs" => EdgeOut {
          "name": "fbjs",
          "spec": "^0.8.9",
          "to": "node_modules/fbjs",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.5.10",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-15.6.2.tgz",
      "version": "15.6.2",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.12.0.tgz",
      "version": "16.12.0",
    },
    "react-reconciler" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "error": "INVALID",
          "name": "react",
          "spec": "^16.0.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
      },
      "location": "node_modules/react-reconciler",
      "name": "react-reconciler",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/react-reconciler",
      "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.24.0.tgz",
      "version": "0.24.0",
    },
    "read-pkg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readable-stream" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/readable-stream",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/readable-stream/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/readable-stream/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "readable-stream",
          "spec": "^2.1.5",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "~1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/readable-stream/node_modules/safe-buffer",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~1.1.1",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
      "version": "2.3.6",
    },
    "readdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.7",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.3.0.tgz",
      "version": "3.3.0",
    },
    "redeyed" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cardinal",
          "name": "redeyed",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "~4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/redeyed",
      "name": "redeyed",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/redeyed",
      "resolved": "https://registry.npmjs.org/redeyed/-/redeyed-2.1.1.tgz",
      "version": "2.1.1",
    },
    "regenerator-runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/runtime",
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-runtime",
      "name": "regenerator-runtime",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/regenerator-runtime",
      "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.3.tgz",
      "version": "0.13.3",
    },
    "release-zalgo" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.0",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
      "version": "2.88.0",
    },
    "require-directory" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "resolve",
          "spec": "^1.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.14.1.tgz",
      "version": "1.14.1",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "resolve-from",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
      "version": "3.0.0",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.0.tgz",
      "version": "5.2.0",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/iconv-lite",
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "scheduler" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/scheduler",
      "name": "scheduler",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/scheduler",
      "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.18.0.tgz",
      "version": "0.18.0",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "semver",
          "spec": "^5.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "setimmediate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "setimmediate",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/setimmediate",
      "name": "setimmediate",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/setimmediate",
      "resolved": "https://registry.npmjs.org/setimmediate/-/setimmediate-1.0.5.tgz",
      "version": "1.0.5",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
      "version": "3.0.2",
    },
    "slice-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi",
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/color-name" => EdgeOut {
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "to": "node_modules/@types/color-name",
              "type": "prod",
            },
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/slice-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/slice-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.2.0.tgz",
          "version": "4.2.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/slice-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/slice-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/slice-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/slice-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/slice-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^2.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/merge-source-map",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "source-map",
          "spec": "^0.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "source-map",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
      "version": "0.6.1",
    },
    "source-map-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.16.tgz",
      "version": "0.5.16",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
      "version": "3.1.0",
    },
    "spdx-exceptions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
      "version": "2.2.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
      "version": "3.0.0",
    },
    "spdx-license-ids" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.5.tgz",
      "version": "3.0.5",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string_decoder" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string_decoder",
              "name": "safe-buffer",
              "spec": "~5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string_decoder/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string_decoder/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "to": "node_modules/string_decoder/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
      "version": "1.1.1",
    },
    "string-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "astral-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "astral-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string-length/node_modules/astral-regex",
          "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/string-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/string-length/node_modules/astral-regex",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/string-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-length",
      "name": "string-length",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string-length",
      "resolved": "https://registry.npmjs.org/string-length/-/string-length-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line",
          "name": "string-width",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
      "version": "6.0.0",
    },
    "strip-bom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "tap" => ArboristNode {
      "children": Map {
        "react" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "react",
              "spec": "^16.12.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/prop-types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react",
          "name": "react",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap/node_modules/react",
          "resolved": "https://registry.npmjs.org/react/-/react-16.12.0.tgz",
          "version": "16.12.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "unbundled",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.8",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/tap/node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "name": "treport",
          "spec": "^1.0.0",
          "to": "node_modules/treport",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.2-unbundled.tgz",
      "version": "14.10.2-unbundled",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "debug",
              "spec": "^2.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/tap-mocha-reporter/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-mocha-reporter/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "diff" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "diff",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/diff",
          "name": "diff",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-mocha-reporter/node_modules/diff",
          "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
          "version": "1.4.0",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-mocha-reporter/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.3",
          "to": "node_modules/tap-mocha-reporter/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^1.3.2",
          "to": "node_modules/tap-mocha-reporter/node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^1.0.0",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.0.tgz",
      "version": "5.0.0",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^10.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/treport",
          "name": "tap-parser",
          "spec": "^10.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.0.1.tgz",
      "version": "10.0.1",
    },
    "tap-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tough-cookie",
              "name": "punycode",
              "spec": "^1.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tough-cookie/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tough-cookie/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.24",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/tough-cookie/node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
      "version": "2.4.3",
    },
    "treport" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/color-name" => EdgeOut {
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "to": "node_modules/@types/color-name",
              "type": "prod",
            },
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/treport/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.2.0.tgz",
          "version": "4.2.0",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/treport/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/treport/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
          "version": "3.0.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/treport/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
        "has-flag" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
          "version": "4.0.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/treport/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/treport/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.1.0.tgz",
          "version": "7.1.0",
        },
        "unicode-length" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/treport",
              "name": "unicode-length",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "punycode" => EdgeOut {
              "name": "punycode",
              "spec": "^2.0.0",
              "to": "node_modules/punycode",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "to": "node_modules/treport/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/treport/node_modules/unicode-length",
          "name": "unicode-length",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport/node_modules/unicode-length",
          "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-2.0.2.tgz",
          "version": "2.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "treport",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cardinal" => EdgeOut {
          "name": "cardinal",
          "spec": "^2.1.1",
          "to": "node_modules/cardinal",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^3.0.0",
          "to": "node_modules/treport/node_modules/chalk",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/ink",
          "type": "prod",
        },
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
        "react" => EdgeOut {
          "error": "INVALID",
          "name": "react",
          "spec": "^16.8.6",
          "to": "node_modules/react",
          "type": "peer",
        },
        "string-length" => EdgeOut {
          "name": "string-length",
          "spec": "^3.1.0",
          "to": "node_modules/string-length",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^2.0.2",
          "to": "node_modules/treport/node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/treport",
      "name": "treport",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/treport",
      "resolved": "https://registry.npmjs.org/treport/-/treport-1.0.1.tgz",
      "version": "1.0.1",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.0",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "^3.0.0",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.5.4.tgz",
      "version": "8.5.4",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "type-fest" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-escapes",
          "name": "type-fest",
          "spec": "^0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
      "version": "0.8.1",
    },
    "typedarray-to-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "typescript",
          "spec": ">=2.0",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.7.4.tgz",
      "version": "3.7.4",
    },
    "ua-parser-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "type": "prod",
        },
      },
      "location": "node_modules/ua-parser-js",
      "name": "ua-parser-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/ua-parser-js",
      "resolved": "https://registry.npmjs.org/ua-parser-js/-/ua-parser-js-0.7.20.tgz",
      "version": "0.7.20",
    },
    "uglify-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "uglify-js",
          "spec": "^3.1.4",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "~2.20.3",
          "to": "node_modules/commander",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/uglify-js",
      "name": "uglify-js",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/uglify-js",
      "resolved": "https://registry.npmjs.org/uglify-js/-/uglify-js-3.7.3.tgz",
      "version": "3.7.3",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "punycode",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/unicode-length/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/unicode-length/node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
      "version": "4.2.2",
    },
    "util-deprecate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
      "version": "1.0.2",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.3.tgz",
      "version": "3.3.3",
    },
    "validate-npm-package-license" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "whatwg-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "type": "prod",
        },
      },
      "location": "node_modules/whatwg-fetch",
      "name": "whatwg-fetch",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/whatwg-fetch",
      "resolved": "https://registry.npmjs.org/whatwg-fetch/-/whatwg-fetch-3.0.0.tgz",
      "version": "3.0.0",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "widest-line" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optimist",
          "name": "wordwrap",
          "spec": "~0.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
      "version": "0.0.3",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/color-name" => EdgeOut {
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "to": "node_modules/@types/color-name",
              "type": "prod",
            },
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^2.0.1",
              "to": "node_modules/wrap-ansi/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wrap-ansi/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.2.0.tgz",
          "version": "4.2.0",
        },
        "color-convert" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "~1.1.4",
              "to": "node_modules/wrap-ansi/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wrap-ansi/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
          "version": "2.0.1",
        },
        "color-name" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/color-convert",
              "name": "color-name",
              "spec": "~1.1.4",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wrap-ansi/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
          "version": "1.1.4",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/wrap-ansi/node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
      "version": "6.2.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.1.tgz",
      "version": "3.0.1",
    },
    "y18n" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yaml",
          "spec": "^1.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.7.2.tgz",
      "version": "1.7.2",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/yargs/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.0.tgz",
      "version": "13.3.0",
    },
    "yargs-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.1.tgz",
      "version": "13.1.1",
    },
    "yn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
    "yoga-layout-prebuilt" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "type": "prod",
        },
      },
      "location": "node_modules/yoga-layout-prebuilt",
      "name": "yoga-layout-prebuilt",
      "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw/node_modules/yoga-layout-prebuilt",
      "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.9.3.tgz",
      "version": "1.9.3",
    },
  },
  "edgesOut": Map {
    "react" => EdgeOut {
      "name": "react",
      "spec": "15",
      "to": "node_modules/react",
      "type": "prod",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "unbundled",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-react15-collision-legacy-sw",
  "packageName": "tap-14.7-react-15-peerdep-collision",
  "path": "{CWD}/test/fixtures/tap-react15-collision-legacy-sw",
  "version": "1.2.3",
}
`

exports[`test/arborist/build-ideal-tree.js TAP tarball deps with transitive tarball deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-file-transitive-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-file-transitive-a",
          "spec": "file:testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-file-transitive-b" => EdgeOut {
          "name": "@isaacs/testing-file-transitive-b",
          "spec": "file:../testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
          "to": "node_modules/@isaacs/testing-file-transitive-b",
          "type": "prod",
        },
        "@isaacs/testing-file-transitive-c" => EdgeOut {
          "name": "@isaacs/testing-file-transitive-c",
          "spec": "file:./c/isaacs-testing-file-transitive-c-1.0.0.tgz",
          "to": "node_modules/@isaacs/testing-file-transitive-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-a",
      "name": "@isaacs/testing-file-transitive-a",
      "path": "{CWD}/test/fixtures/tarball-dependencies/node_modules/@isaacs/testing-file-transitive-a",
      "resolved": "file:{CWD}/test/fixtures/tarball-dependencies/testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-file-transitive-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-file-transitive-a",
          "name": "@isaacs/testing-file-transitive-b",
          "spec": "file:../testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-b",
      "name": "@isaacs/testing-file-transitive-b",
      "path": "{CWD}/test/fixtures/tarball-dependencies/node_modules/@isaacs/testing-file-transitive-b",
      "resolved": "file:{CWD}/test/fixtures/tarball-dependencies/testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-file-transitive-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-file-transitive-a",
          "name": "@isaacs/testing-file-transitive-c",
          "spec": "file:./c/isaacs-testing-file-transitive-c-1.0.0.tgz",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-c",
      "name": "@isaacs/testing-file-transitive-c",
      "path": "{CWD}/test/fixtures/tarball-dependencies/node_modules/@isaacs/testing-file-transitive-c",
      "resolved": "file:{CWD}/test/fixtures/tarball-dependencies/testing-file-transitive-a/c/isaacs-testing-file-transitive-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-file-transitive-a" => EdgeOut {
      "name": "@isaacs/testing-file-transitive-a",
      "spec": "file:testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
      "to": "node_modules/@isaacs/testing-file-transitive-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tarball-dependencies",
  "path": "{CWD}/test/fixtures/tarball-dependencies",
}
`

exports[`test/arborist/build-ideal-tree.js TAP testing-peer-deps nested > build ideal tree 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-deps-d" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps",
      "name": "@isaacs/testing-peer-deps",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps/-/testing-peer-deps-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-deps-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-b" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-a",
      "name": "@isaacs/testing-peer-deps-a",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps" => EdgeOut {
      "name": "@isaacs/testing-peer-deps",
      "spec": "2 || 3",
      "to": "node_modules/@isaacs/testing-peer-deps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-peer-deps-nested",
  "path": "{CWD}/test/fixtures/testing-peer-deps-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP testing-peer-deps nested > can update a peer dep cycle 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-b" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-b",
          "type": "prod",
        },
        "@isaacs/testing-peer-deps-d" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps",
      "name": "@isaacs/testing-peer-deps",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps/-/testing-peer-deps-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/fixtures/testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps" => EdgeOut {
      "name": "@isaacs/testing-peer-deps",
      "spec": "2 || 3",
      "to": "node_modules/@isaacs/testing-peer-deps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-peer-deps-nested",
  "path": "{CWD}/test/fixtures/testing-peer-deps-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP testing-peer-deps package > build ideal tree with peer deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps-b" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-deps-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-deps-d" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-d",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-deps-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-peer-deps",
  "packageName": "@isaacs/testing-peer-deps",
  "path": "{CWD}/test/fixtures/testing-peer-deps",
  "version": "2.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP testing-peer-deps package with symlinked root > build ideal tree with peer deps 1`] = `
ArboristLink {
  "isProjectRoot": true,
  "location": "../testing-peer-deps-link",
  "name": "testing-peer-deps-link",
  "packageName": "@isaacs/testing-peer-deps",
  "path": "{CWD}/test/fixtures/testing-peer-deps-link",
  "realpath": "{CWD}/test/fixtures/testing-peer-deps",
  "resolved": "file:testing-peer-deps",
  "target": ArboristNode {
    "children": Map {
      "@isaacs/testing-peer-deps-b" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-peer-deps-b",
            "spec": "1",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-peer-deps-c" => EdgeOut {
            "name": "@isaacs/testing-peer-deps-c",
            "spec": "1",
            "to": "node_modules/@isaacs/testing-peer-deps-c",
            "type": "peer",
          },
        },
        "location": "node_modules/@isaacs/testing-peer-deps-b",
        "name": "@isaacs/testing-peer-deps-b",
        "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-b",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
        "version": "1.2.4",
      },
      "@isaacs/testing-peer-deps-c" => ArboristNode {
        "edgesIn": Set {
          EdgeIn {
            "from": "node_modules/@isaacs/testing-peer-deps-b",
            "name": "@isaacs/testing-peer-deps-c",
            "spec": "1",
            "type": "peer",
          },
        },
        "location": "node_modules/@isaacs/testing-peer-deps-c",
        "name": "@isaacs/testing-peer-deps-c",
        "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-c",
        "peer": true,
        "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
        "version": "1.2.3",
      },
      "@isaacs/testing-peer-deps-d" => ArboristNode {
        "children": Map {
          "@isaacs/testing-peer-deps-a" => ArboristNode {
            "edgesIn": Set {
              EdgeIn {
                "from": "node_modules/@isaacs/testing-peer-deps-d",
                "name": "@isaacs/testing-peer-deps-a",
                "spec": "2",
                "type": "prod",
              },
            },
            "edgesOut": Map {
              "@isaacs/testing-peer-deps-b" => EdgeOut {
                "name": "@isaacs/testing-peer-deps-b",
                "spec": "2",
                "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
                "type": "peer",
              },
            },
            "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
            "name": "@isaacs/testing-peer-deps-a",
            "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
            "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
            "version": "2.0.0",
          },
          "@isaacs/testing-peer-deps-b" => ArboristNode {
            "edgesIn": Set {
              EdgeIn {
                "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
                "name": "@isaacs/testing-peer-deps-b",
                "spec": "2",
                "type": "peer",
              },
            },
            "edgesOut": Map {
              "@isaacs/testing-peer-deps-c" => EdgeOut {
                "name": "@isaacs/testing-peer-deps-c",
                "spec": "2",
                "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
                "type": "peer",
              },
            },
            "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
            "name": "@isaacs/testing-peer-deps-b",
            "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
            "peer": true,
            "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
            "version": "2.0.1",
          },
          "@isaacs/testing-peer-deps-c" => ArboristNode {
            "edgesIn": Set {
              EdgeIn {
                "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
                "name": "@isaacs/testing-peer-deps-c",
                "spec": "2",
                "type": "peer",
              },
            },
            "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
            "name": "@isaacs/testing-peer-deps-c",
            "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
            "peer": true,
            "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
            "version": "2.0.0",
          },
        },
        "edgesIn": Set {
          EdgeIn {
            "from": "",
            "name": "@isaacs/testing-peer-deps-d",
            "spec": "2",
            "type": "prod",
          },
        },
        "edgesOut": Map {
          "@isaacs/testing-peer-deps-a" => EdgeOut {
            "name": "@isaacs/testing-peer-deps-a",
            "spec": "2",
            "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
            "type": "prod",
          },
        },
        "location": "node_modules/@isaacs/testing-peer-deps-d",
        "name": "@isaacs/testing-peer-deps-d",
        "path": "{CWD}/test/fixtures/testing-peer-deps/node_modules/@isaacs/testing-peer-deps-d",
        "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
        "version": "2.0.0",
      },
    },
    "edgesOut": Map {
      "@isaacs/testing-peer-deps-b" => EdgeOut {
        "name": "@isaacs/testing-peer-deps-b",
        "spec": "1",
        "to": "node_modules/@isaacs/testing-peer-deps-b",
        "type": "prod",
      },
      "@isaacs/testing-peer-deps-d" => EdgeOut {
        "name": "@isaacs/testing-peer-deps-d",
        "spec": "2",
        "to": "node_modules/@isaacs/testing-peer-deps-d",
        "type": "prod",
      },
    },
    "isProjectRoot": true,
    "location": "",
    "name": "testing-peer-deps",
    "packageName": "@isaacs/testing-peer-deps",
    "path": "{CWD}/test/fixtures/testing-peer-deps",
    "version": "2.0.0",
  },
  "version": "2.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP testing-peer-deps-overlap package > build ideal tree with overlapping peer dep ranges 1`] = `
ArboristNode {
  "children": Map {
    "@angular-slider/ngx-slider" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@angular-slider/ngx-slider",
          "spec": "^2.0.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@angular/common" => EdgeOut {
          "name": "@angular/common",
          "spec": ">=6.1.0",
          "to": "node_modules/@angular/common",
          "type": "peer",
        },
        "@angular/core" => EdgeOut {
          "name": "@angular/core",
          "spec": ">=6.1.0",
          "to": "node_modules/@angular/core",
          "type": "peer",
        },
        "@angular/forms" => EdgeOut {
          "name": "@angular/forms",
          "spec": ">=6.1.0",
          "to": "node_modules/@angular/forms",
          "type": "peer",
        },
        "detect-passive-events" => EdgeOut {
          "name": "detect-passive-events",
          "spec": "^1.0.4",
          "to": "node_modules/detect-passive-events",
          "type": "prod",
        },
        "rxjs" => EdgeOut {
          "name": "rxjs",
          "spec": "^6.5.2",
          "to": "node_modules/rxjs",
          "type": "prod",
        },
        "rxjs-compat" => EdgeOut {
          "name": "rxjs-compat",
          "spec": "^6.5.2",
          "to": "node_modules/rxjs-compat",
          "type": "prod",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.9.0",
          "to": "node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/@angular-slider/ngx-slider",
      "name": "@angular-slider/ngx-slider",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular-slider/ngx-slider",
      "resolved": "https://registry.npmjs.org/@angular-slider/ngx-slider/-/ngx-slider-2.0.3.tgz",
      "version": "2.0.3",
    },
    "@angular/common" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@angular/common",
              "name": "tslib",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@angular/common/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/common/node_modules/tslib",
          "peer": true,
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "@angular/common",
          "spec": ">=6.1.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/forms",
          "name": "@angular/common",
          "spec": "10.2.5",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/platform-browser",
          "name": "@angular/common",
          "spec": "10.2.5",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@angular/core" => EdgeOut {
          "name": "@angular/core",
          "spec": "10.2.5",
          "to": "node_modules/@angular/core",
          "type": "peer",
        },
        "rxjs" => EdgeOut {
          "name": "rxjs",
          "spec": "^6.5.3",
          "to": "node_modules/rxjs",
          "type": "peer",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^2.0.0",
          "to": "node_modules/@angular/common/node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/@angular/common",
      "name": "@angular/common",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/common",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@angular/common/-/common-10.2.5.tgz",
      "version": "10.2.5",
    },
    "@angular/core" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@angular/core",
              "name": "tslib",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@angular/core/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/core/node_modules/tslib",
          "peer": true,
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "@angular/core",
          "spec": ">=6.1.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/common",
          "name": "@angular/core",
          "spec": "10.2.5",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/forms",
          "name": "@angular/core",
          "spec": "10.2.5",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/platform-browser",
          "name": "@angular/core",
          "spec": "10.2.5",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "rxjs" => EdgeOut {
          "name": "rxjs",
          "spec": "^6.5.3",
          "to": "node_modules/rxjs",
          "type": "peer",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^2.0.0",
          "to": "node_modules/@angular/core/node_modules/tslib",
          "type": "prod",
        },
        "zone.js" => EdgeOut {
          "name": "zone.js",
          "spec": "~0.10.3",
          "to": "node_modules/zone.js",
          "type": "peer",
        },
      },
      "location": "node_modules/@angular/core",
      "name": "@angular/core",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/core",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@angular/core/-/core-10.2.5.tgz",
      "version": "10.2.5",
    },
    "@angular/forms" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@angular/forms",
              "name": "tslib",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@angular/forms/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/forms/node_modules/tslib",
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@angular/forms",
          "spec": "^10.2.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "@angular/forms",
          "spec": ">=6.1.0",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@angular/common" => EdgeOut {
          "name": "@angular/common",
          "spec": "10.2.5",
          "to": "node_modules/@angular/common",
          "type": "peer",
        },
        "@angular/core" => EdgeOut {
          "name": "@angular/core",
          "spec": "10.2.5",
          "to": "node_modules/@angular/core",
          "type": "peer",
        },
        "@angular/platform-browser" => EdgeOut {
          "name": "@angular/platform-browser",
          "spec": "10.2.5",
          "to": "node_modules/@angular/platform-browser",
          "type": "peer",
        },
        "rxjs" => EdgeOut {
          "name": "rxjs",
          "spec": "^6.5.3",
          "to": "node_modules/rxjs",
          "type": "peer",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^2.0.0",
          "to": "node_modules/@angular/forms/node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/@angular/forms",
      "name": "@angular/forms",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/forms",
      "resolved": "https://registry.npmjs.org/@angular/forms/-/forms-10.2.5.tgz",
      "version": "10.2.5",
    },
    "@angular/platform-browser" => ArboristNode {
      "children": Map {
        "tslib" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@angular/platform-browser",
              "name": "tslib",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@angular/platform-browser/node_modules/tslib",
          "name": "tslib",
          "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/platform-browser/node_modules/tslib",
          "peer": true,
          "resolved": "https://registry.npmjs.org/tslib/-/tslib-2.0.3.tgz",
          "version": "2.0.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular/forms",
          "name": "@angular/platform-browser",
          "spec": "10.2.5",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@angular/animations" => EdgeOut {
          "name": "@angular/animations",
          "spec": "10.2.5",
          "to": null,
          "type": "peerOptional",
        },
        "@angular/common" => EdgeOut {
          "name": "@angular/common",
          "spec": "10.2.5",
          "to": "node_modules/@angular/common",
          "type": "peer",
        },
        "@angular/core" => EdgeOut {
          "name": "@angular/core",
          "spec": "10.2.5",
          "to": "node_modules/@angular/core",
          "type": "peer",
        },
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^2.0.0",
          "to": "node_modules/@angular/platform-browser/node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/@angular/platform-browser",
      "name": "@angular/platform-browser",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/@angular/platform-browser",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@angular/platform-browser/-/platform-browser-10.2.5.tgz",
      "version": "10.2.5",
    },
    "detect-passive-events" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "detect-passive-events",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/detect-passive-events",
      "name": "detect-passive-events",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/detect-passive-events",
      "resolved": "https://registry.npmjs.org/detect-passive-events/-/detect-passive-events-1.0.5.tgz",
      "version": "1.0.5",
    },
    "rxjs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "rxjs",
          "spec": "^6.5.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@angular/common",
          "name": "rxjs",
          "spec": "^6.5.3",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/core",
          "name": "rxjs",
          "spec": "^6.5.3",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@angular/forms",
          "name": "rxjs",
          "spec": "^6.5.3",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "tslib" => EdgeOut {
          "name": "tslib",
          "spec": "^1.9.0",
          "to": "node_modules/tslib",
          "type": "prod",
        },
      },
      "location": "node_modules/rxjs",
      "name": "rxjs",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/rxjs",
      "resolved": "https://registry.npmjs.org/rxjs/-/rxjs-6.6.3.tgz",
      "version": "6.6.3",
    },
    "rxjs-compat" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "rxjs-compat",
          "spec": "^6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/rxjs-compat",
      "name": "rxjs-compat",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/rxjs-compat",
      "resolved": "https://registry.npmjs.org/rxjs-compat/-/rxjs-compat-6.6.7.tgz",
      "version": "6.6.7",
    },
    "tslib" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular-slider/ngx-slider",
          "name": "tslib",
          "spec": "^1.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rxjs",
          "name": "tslib",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tslib",
      "name": "tslib",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/tslib",
      "resolved": "https://registry.npmjs.org/tslib/-/tslib-1.14.1.tgz",
      "version": "1.14.1",
    },
    "zone.js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@angular/core",
          "name": "zone.js",
          "spec": "~0.10.3",
          "type": "peer",
        },
      },
      "location": "node_modules/zone.js",
      "name": "zone.js",
      "path": "{CWD}/test/fixtures/testing-peer-deps-overlap/node_modules/zone.js",
      "peer": true,
      "resolved": "https://registry.npmjs.org/zone.js/-/zone.js-0.10.3.tgz",
      "version": "0.10.3",
    },
  },
  "edgesOut": Map {
    "@angular-slider/ngx-slider" => EdgeOut {
      "name": "@angular-slider/ngx-slider",
      "spec": "^2.0.3",
      "to": "node_modules/@angular-slider/ngx-slider",
      "type": "prod",
    },
    "@angular/forms" => EdgeOut {
      "name": "@angular/forms",
      "spec": "^10.2.4",
      "to": "node_modules/@angular/forms",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "testing-peer-deps-overlap",
  "path": "{CWD}/test/fixtures/testing-peer-deps-overlap",
}
`

exports[`test/arborist/build-ideal-tree.js TAP transitive conflicted peer dependency > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-transitive-conflicted-peer-a" => ArboristNode {
      "children": Map {
        "@isaacs/testing-transitive-conflicted-peer-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
              "name": "@isaacs/testing-transitive-conflicted-peer-c",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-transitive-conflicted-peer-b",
              "peerConflicted": true,
              "spec": "1",
              "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "name": "@isaacs/testing-transitive-conflicted-peer-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-c/-/testing-transitive-conflicted-peer-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-transitive-conflicted-peer-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "1||2",
          "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
          "type": "peer",
        },
        "@isaacs/testing-transitive-conflicted-peer-c" => EdgeOut {
          "name": "@isaacs/testing-transitive-conflicted-peer-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "name": "@isaacs/testing-transitive-conflicted-peer-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-a/-/testing-transitive-conflicted-peer-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-transitive-conflicted-peer-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "1||2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "name": "@isaacs/testing-transitive-conflicted-peer-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-b/-/testing-transitive-conflicted-peer-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-transitive-conflicted-peer-a" => EdgeOut {
      "name": "@isaacs/testing-transitive-conflicted-peer-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "type": "prod",
    },
    "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
      "name": "@isaacs/testing-transitive-conflicted-peer-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency",
}
`

exports[`test/arborist/build-ideal-tree.js TAP transitive conflicted peer dependency > must match snapshot 2`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-transitive-conflicted-peer-a" => ArboristNode {
      "children": Map {
        "@isaacs/testing-transitive-conflicted-peer-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
              "name": "@isaacs/testing-transitive-conflicted-peer-c",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
              "error": "INVALID",
              "name": "@isaacs/testing-transitive-conflicted-peer-b",
              "peerConflicted": true,
              "spec": "1",
              "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "name": "@isaacs/testing-transitive-conflicted-peer-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-c/-/testing-transitive-conflicted-peer-c-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-transitive-conflicted-peer-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "1||2",
          "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
          "type": "peer",
        },
        "@isaacs/testing-transitive-conflicted-peer-c" => EdgeOut {
          "name": "@isaacs/testing-transitive-conflicted-peer-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "name": "@isaacs/testing-transitive-conflicted-peer-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-a/-/testing-transitive-conflicted-peer-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-transitive-conflicted-peer-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "spec": "1||2",
          "type": "peer",
        },
        EdgeIn {
          "error": "INVALID",
          "from": "node_modules/@isaacs/testing-transitive-conflicted-peer-a/node_modules/@isaacs/testing-transitive-conflicted-peer-c",
          "name": "@isaacs/testing-transitive-conflicted-peer-b",
          "peerConflicted": true,
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "name": "@isaacs/testing-transitive-conflicted-peer-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency/node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-transitive-conflicted-peer-b/-/testing-transitive-conflicted-peer-b-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-transitive-conflicted-peer-a" => EdgeOut {
      "name": "@isaacs/testing-transitive-conflicted-peer-a",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-a",
      "type": "prod",
    },
    "@isaacs/testing-transitive-conflicted-peer-b" => EdgeOut {
      "name": "@isaacs/testing-transitive-conflicted-peer-b",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-transitive-conflicted-peer-b",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-transitive-conflicted-peer-dependency",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update a global space that contains a link > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "once" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/target",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/node_modules/once",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/target",
      "resolved": "file:../target",
      "target": ArboristNode {
        "location": "target",
      },
      "version": "1.0.0-foo",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/target",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "target",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link/target",
      "version": "1.0.0-foo",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-a-global-space-that-contains-a-link",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update flow outdated > update everything 1`] = `
ArboristNode {
  "children": Map {
    "flow-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-parser",
          "spec": "0.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.136.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.136.0.tgz",
      "version": "0.136.0",
    },
    "flow-remove-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.136.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.136.0.tgz",
      "version": "2.136.0",
    },
    "node-modules-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "pirates" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "vlq" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
  },
  "edgesOut": Map {
    "flow-parser" => EdgeOut {
      "name": "flow-parser",
      "spec": "0.x",
      "to": "node_modules/flow-parser",
      "type": "prod",
    },
    "flow-remove-types" => EdgeOut {
      "name": "flow-remove-types",
      "spec": "^2.112.0",
      "to": "node_modules/flow-remove-types",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "flow-outdated",
  "path": "{CWD}/test/fixtures/flow-outdated",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update flow outdated > update flow parser 1`] = `
ArboristNode {
  "children": Map {
    "flow-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-parser",
          "spec": "0.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.112.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.112.0.tgz",
      "version": "0.112.0",
    },
    "flow-remove-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.112.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.112.0.tgz",
      "version": "2.112.0",
    },
    "node-modules-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "pirates" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "vlq" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/flow-outdated/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.1.tgz",
      "version": "0.2.1",
    },
  },
  "edgesOut": Map {
    "flow-parser" => EdgeOut {
      "name": "flow-parser",
      "spec": "0.x",
      "to": "node_modules/flow-parser",
      "type": "prod",
    },
    "flow-remove-types" => EdgeOut {
      "name": "flow-remove-types",
      "spec": "^2.112.0",
      "to": "node_modules/flow-remove-types",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "flow-outdated",
  "path": "{CWD}/test/fixtures/flow-outdated",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global > update a single dep, also fixes the invalid node 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-dev-optional-flags" => ArboristNode {
      "children": Map {
        "own-or" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "own-or",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "name": "own-or",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "version": "1.0.0",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "wrappy",
              "spec": "^1.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-dev-optional-flags",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "^1.0.2",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-dev-optional-flags",
      "name": "@isaacs/testing-dev-optional-flags",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags",
      "version": "1.0.0",
    },
    "once" => ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/once/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once/node_modules/wrappy",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/once/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-dev-optional-flags",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-update-global",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global > update all the deps 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-dev-optional-flags" => ArboristNode {
      "children": Map {
        "own-or" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "own-or",
              "spec": "^1.0.0",
              "type": "optional",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "name": "own-or",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
          "version": "1.0.0",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "wrappy",
              "spec": "^1.0.2",
              "type": "optional",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "name": "wrappy",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-dev-optional-flags",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "type": "optional",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "^1.0.2",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "type": "optional",
        },
      },
      "location": "node_modules/@isaacs/testing-dev-optional-flags",
      "name": "@isaacs/testing-dev-optional-flags",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-dev-optional-flags/-/testing-dev-optional-flags-1.0.0.tgz",
      "version": "1.0.0",
    },
    "once" => ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/once/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/once/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-dev-optional-flags",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-update-global",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global > updating missing dep should have no effect, but fix the invalid node 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-dev-optional-flags" => ArboristNode {
      "children": Map {
        "own-or" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "own-or",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "name": "own-or",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "version": "1.0.0",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "wrappy",
              "spec": "^1.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-dev-optional-flags",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "^1.0.2",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-dev-optional-flags",
      "name": "@isaacs/testing-dev-optional-flags",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags",
      "version": "1.0.0",
    },
    "once" => ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/once/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once/node_modules/wrappy",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/once/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once",
      "version": "1.3.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-dev-optional-flags",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-update-global",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global > updating sub-dep has no effect, but fixes the invalid node 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-dev-optional-flags" => ArboristNode {
      "children": Map {
        "own-or" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "own-or",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "name": "own-or",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "version": "1.0.0",
        },
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-dev-optional-flags",
              "name": "wrappy",
              "spec": "^1.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-dev-optional-flags",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/own-or",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "^1.0.2",
          "to": "node_modules/@isaacs/testing-dev-optional-flags/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-dev-optional-flags",
      "name": "@isaacs/testing-dev-optional-flags",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/@isaacs/testing-dev-optional-flags",
      "version": "1.0.0",
    },
    "once" => ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/once/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once/node_modules/wrappy",
          "version": "1.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/once/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global/node_modules/once",
      "version": "1.3.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-dev-optional-flags",
      "type": "prod",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-update-global",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global space single dep > should update global dependencies 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global-space-single-dep/global-prefix/lib/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "lib",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global-space-single-dep/global-prefix/lib",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global when nothing in global > update with empty node_modules 1`] = `
ArboristNode {
  "isProjectRoot": true,
  "location": "",
  "name": "empty_nm",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global-when-nothing-in-global/empty_nm",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update global when nothing in global > update without node_modules 1`] = `
ArboristNode {
  "isProjectRoot": true,
  "location": "",
  "name": "no_nm",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-update-global-when-nothing-in-global/no_nm",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update mkdirp to non-minimist-using version > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/deprecated-dep/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^0.5.1 <0.5.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/deprecated-dep/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.3.tgz",
      "version": "0.5.3",
    },
  },
  "edgesOut": Map {
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^0.5.1 <0.5.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "deprecated-dep",
  "path": "{CWD}/test/fixtures/deprecated-dep",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update tap and flow > update everything 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/core" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/core",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-module-transforms" => EdgeOut {
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-transforms",
          "type": "prod",
        },
        "@babel/helpers" => EdgeOut {
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helpers",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "gensync" => EdgeOut {
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "to": "node_modules/gensync",
          "type": "prod",
        },
        "json5" => EdgeOut {
          "name": "json5",
          "spec": "^2.1.2",
          "to": "node_modules/json5",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.3.2",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.4.1",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/core",
      "name": "@babel/core",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/core",
      "resolved": "https://registry.npmjs.org/@babel/core/-/core-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/generator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-annotate-as-pure" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-annotate-as-pure",
      "name": "@babel/helper-annotate-as-pure",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-annotate-as-pure",
      "resolved": "https://registry.npmjs.org/@babel/helper-annotate-as-pure/-/helper-annotate-as-pure-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-builder-react-jsx" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-react-jsx",
      "name": "@babel/helper-builder-react-jsx",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-builder-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-react-jsx/-/helper-builder-react-jsx-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-builder-react-jsx-experimental" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-builder-react-jsx-experimental",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-annotate-as-pure" => EdgeOut {
          "name": "@babel/helper-annotate-as-pure",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-annotate-as-pure",
          "type": "prod",
        },
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-builder-react-jsx-experimental",
      "name": "@babel/helper-builder-react-jsx-experimental",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-builder-react-jsx-experimental",
      "resolved": "https://registry.npmjs.org/@babel/helper-builder-react-jsx-experimental/-/helper-builder-react-jsx-experimental-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-function-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-member-expression-to-functions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-member-expression-to-functions",
      "name": "@babel/helper-member-expression-to-functions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-member-expression-to-functions",
      "resolved": "https://registry.npmjs.org/@babel/helper-member-expression-to-functions/-/helper-member-expression-to-functions-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-imports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-imports",
      "name": "@babel/helper-module-imports",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-module-imports",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-imports/-/helper-module-imports-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-module-transforms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helper-module-transforms",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-module-imports" => EdgeOut {
          "name": "@babel/helper-module-imports",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-module-imports",
          "type": "prod",
        },
        "@babel/helper-replace-supers" => EdgeOut {
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-replace-supers",
          "type": "prod",
        },
        "@babel/helper-simple-access" => EdgeOut {
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-simple-access",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-module-transforms",
      "name": "@babel/helper-module-transforms",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-module-transforms",
      "resolved": "https://registry.npmjs.org/@babel/helper-module-transforms/-/helper-module-transforms-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-optimise-call-expression" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-optimise-call-expression",
      "name": "@babel/helper-optimise-call-expression",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-optimise-call-expression",
      "resolved": "https://registry.npmjs.org/@babel/helper-optimise-call-expression/-/helper-optimise-call-expression-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-plugin-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-parameters",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-plugin-utils",
      "name": "@babel/helper-plugin-utils",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-plugin-utils",
      "resolved": "https://registry.npmjs.org/@babel/helper-plugin-utils/-/helper-plugin-utils-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helper-replace-supers" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-replace-supers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-member-expression-to-functions" => EdgeOut {
          "name": "@babel/helper-member-expression-to-functions",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-member-expression-to-functions",
          "type": "prod",
        },
        "@babel/helper-optimise-call-expression" => EdgeOut {
          "name": "@babel/helper-optimise-call-expression",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-optimise-call-expression",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-replace-supers",
      "name": "@babel/helper-replace-supers",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-replace-supers",
      "resolved": "https://registry.npmjs.org/@babel/helper-replace-supers/-/helper-replace-supers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-simple-access" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-simple-access",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-simple-access",
      "name": "@babel/helper-simple-access",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-simple-access",
      "resolved": "https://registry.npmjs.org/@babel/helper-simple-access/-/helper-simple-access-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.11.0.tgz",
      "version": "7.11.0",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/helpers" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/helpers",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helpers",
      "name": "@babel/helpers",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helpers",
      "resolved": "https://registry.npmjs.org/@babel/helpers/-/helpers-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/highlight" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/@babel/highlight/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "chalk" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight",
              "name": "chalk",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "to": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "to": "node_modules/@babel/highlight/node_modules/escape-string-regexp",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^5.3.0",
              "to": "node_modules/@babel/highlight/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
          "version": "2.4.2",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/@babel/highlight/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "escape-string-regexp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
          "version": "1.0.5",
        },
        "has-flag" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "supports-color",
              "spec": "^5.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/@babel/highlight/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
          "version": "5.5.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.10.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/@babel/highlight/node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/plugin-syntax-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-parameters" => EdgeOut {
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-transform-parameters",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-proposal-object-rest-spread",
      "name": "@babel/plugin-proposal-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-proposal-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-proposal-object-rest-spread/-/plugin-proposal-object-rest-spread-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-jsx" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-jsx",
      "name": "@babel/plugin-syntax-jsx",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-syntax-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-jsx/-/plugin-syntax-jsx-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "spec": "^7.8.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.8.0",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-syntax-object-rest-spread",
      "name": "@babel/plugin-syntax-object-rest-spread",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-syntax-object-rest-spread",
      "resolved": "https://registry.npmjs.org/@babel/plugin-syntax-object-rest-spread/-/plugin-syntax-object-rest-spread-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/plugin-transform-destructuring" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-destructuring",
      "name": "@babel/plugin-transform-destructuring",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-transform-destructuring",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-destructuring/-/plugin-transform-destructuring-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-parameters" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-transform-parameters",
          "spec": "^7.12.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-parameters",
      "name": "@babel/plugin-transform-parameters",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-transform-parameters",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-parameters/-/plugin-transform-parameters-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/plugin-transform-react-jsx" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.0.0-0",
          "to": "node_modules/@babel/core",
          "type": "peer",
        },
        "@babel/helper-builder-react-jsx" => EdgeOut {
          "name": "@babel/helper-builder-react-jsx",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-builder-react-jsx",
          "type": "prod",
        },
        "@babel/helper-builder-react-jsx-experimental" => EdgeOut {
          "name": "@babel/helper-builder-react-jsx-experimental",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "type": "prod",
        },
        "@babel/helper-plugin-utils" => EdgeOut {
          "name": "@babel/helper-plugin-utils",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-plugin-utils",
          "type": "prod",
        },
        "@babel/plugin-syntax-jsx" => EdgeOut {
          "name": "@babel/plugin-syntax-jsx",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/plugin-syntax-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/plugin-transform-react-jsx",
      "name": "@babel/plugin-transform-react-jsx",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/plugin-transform-react-jsx",
      "resolved": "https://registry.npmjs.org/@babel/plugin-transform-react-jsx/-/plugin-transform-react-jsx-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/template" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/template",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.10.4.tgz",
      "version": "7.10.4",
    },
    "@babel/traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/traverse",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.11.0",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.12.1",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@babel/types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-annotate-as-pure",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-builder-react-jsx-experimental",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-member-expression-to-functions",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-imports",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-optimise-call-expression",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-replace-supers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-simple-access",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.11.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helpers",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.10.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.12.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.10.4",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.19",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.12.1.tgz",
      "version": "7.12.1",
    },
    "@types/yoga-layout" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yoga-layout-prebuilt",
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/yoga-layout",
      "name": "@types/yoga-layout",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@types/yoga-layout",
      "resolved": "https://registry.npmjs.org/@types/yoga-layout/-/yoga-layout-1.9.2.tgz",
      "version": "1.9.2",
    },
    "ajv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.12.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.6.tgz",
      "version": "6.12.6",
    },
    "ansi-escapes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.11.0",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.1.tgz",
      "version": "4.3.1",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
      "version": "4.1.0",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^2.0.1",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.3.0.tgz",
      "version": "4.3.0",
    },
    "anymatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.3.tgz",
      "version": "4.1.3",
    },
    "argparse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "arrify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "arrify",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-2.0.1.tgz",
      "version": "2.0.1",
    },
    "asn1" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "astral-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "auto-bind" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "auto-bind",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/auto-bind",
      "name": "auto-bind",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/auto-bind",
      "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-4.0.0.tgz",
      "version": "4.0.0",
    },
    "aws-sign2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.10.1.tgz",
      "version": "1.10.1",
    },
    "balanced-match" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.1.0.tgz",
      "version": "2.1.0",
    },
    "bind-obj-methods" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "caller-callsite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^2.0.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-callsite",
      "name": "caller-callsite",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caller-callsite",
      "resolved": "https://registry.npmjs.org/caller-callsite/-/caller-callsite-2.0.0.tgz",
      "version": "2.0.0",
    },
    "caller-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "caller-path",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-callsite" => EdgeOut {
          "name": "caller-callsite",
          "spec": "^2.0.0",
          "to": "node_modules/caller-callsite",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-2.0.0.tgz",
      "version": "2.0.0",
    },
    "callsites" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-callsite",
          "name": "callsites",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-2.0.0.tgz",
      "version": "2.0.0",
    },
    "camelcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "caseless" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "chalk",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
      "version": "3.0.0",
    },
    "chokidar" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.5.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.4.3.tgz",
      "version": "3.4.3",
    },
    "ci-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-truncate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-truncate",
      "name": "cli-truncate",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cli-truncate",
      "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-2.1.0.tgz",
      "version": "2.1.0",
    },
    "cliui" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "string-width",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/cliui/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "to": "node_modules/cliui/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "to": "node_modules/cliui/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
          "version": "4.0.0",
        },
        "wrap-ansi" => ArboristNode {
          "children": Map {
            "ansi-regex" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
              "version": "2.1.1",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
              "version": "1.0.0",
            },
            "string-width" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "code-point-at" => EdgeOut {
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "to": "node_modules/code-point-at",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
              "version": "1.0.2",
            },
            "strip-ansi" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
              "version": "3.0.1",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^1.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
          "version": "2.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/cliui/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/cliui/node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/cliui/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "~1.1.4",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
      "version": "2.0.1",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "~1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
      "version": "1.1.4",
    },
    "color-support" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commondir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "convert-source-map",
          "spec": "^1.7.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-util-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.11",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.2",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cp-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "2.1.2",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.2.0.tgz",
      "version": "4.2.0",
    },
    "decamelize" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.2.tgz",
      "version": "4.0.2",
    },
    "diff-frag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "error-ex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "esm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "events-to-array" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.3.tgz",
      "version": "3.1.3",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fill-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-parser",
          "spec": "0.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.136.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.136.0.tgz",
      "version": "0.136.0",
    },
    "flow-remove-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.136.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.136.0.tgz",
      "version": "2.136.0",
    },
    "foreground-child" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.3.tgz",
      "version": "2.1.3",
    },
    "function-loop" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "gensync" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "gensync",
          "spec": "^1.0.0-beta.1",
          "type": "prod",
        },
      },
      "location": "node_modules/gensync",
      "name": "gensync",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/gensync",
      "resolved": "https://registry.npmjs.org/gensync/-/gensync-1.0.0-beta.1.tgz",
      "version": "1.0.0-beta.1",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.1.tgz",
      "version": "5.1.1",
    },
    "globals" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.4.tgz",
      "version": "4.2.4",
    },
    "har-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.12.3",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.5.tgz",
      "version": "5.1.5",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
      "version": "4.0.0",
    },
    "hasha" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.8.tgz",
      "version": "2.8.8",
    },
    "html-escaper" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "html-escaper",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/html-escaper",
      "name": "html-escaper",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/html-escaper",
      "resolved": "https://registry.npmjs.org/html-escaper/-/html-escaper-2.0.2.tgz",
      "version": "2.0.2",
    },
    "http-signature" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "import-jsx" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "import-jsx",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/core" => EdgeOut {
          "name": "@babel/core",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/core",
          "type": "prod",
        },
        "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
          "name": "@babel/plugin-proposal-object-rest-spread",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/plugin-proposal-object-rest-spread",
          "type": "prod",
        },
        "@babel/plugin-transform-destructuring" => EdgeOut {
          "name": "@babel/plugin-transform-destructuring",
          "spec": "^7.5.0",
          "to": "node_modules/@babel/plugin-transform-destructuring",
          "type": "prod",
        },
        "@babel/plugin-transform-react-jsx" => EdgeOut {
          "name": "@babel/plugin-transform-react-jsx",
          "spec": "^7.3.0",
          "to": "node_modules/@babel/plugin-transform-react-jsx",
          "type": "prod",
        },
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^2.0.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^3.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/import-jsx",
      "name": "import-jsx",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/import-jsx",
      "resolved": "https://registry.npmjs.org/import-jsx/-/import-jsx-3.1.0.tgz",
      "version": "3.1.0",
    },
    "imurmurhash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "ink" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "ink",
          "spec": "^2.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ink",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": ">=16.8.0",
          "to": null,
          "type": "peerOptional",
        },
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^2.0.1",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "auto-bind" => EdgeOut {
          "name": "auto-bind",
          "spec": "^4.0.0",
          "to": "node_modules/auto-bind",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^3.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-truncate" => EdgeOut {
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "to": "node_modules/cli-truncate",
          "type": "prod",
        },
        "is-ci" => EdgeOut {
          "name": "is-ci",
          "spec": "^2.0.0",
          "to": "node_modules/is-ci",
          "type": "prod",
        },
        "lodash.throttle" => EdgeOut {
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "to": "node_modules/lodash.throttle",
          "type": "prod",
        },
        "log-update" => EdgeOut {
          "name": "log-update",
          "spec": "^3.0.0",
          "to": "node_modules/log-update",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": ">=16.8.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "react-reconciler" => EdgeOut {
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "to": "node_modules/react-reconciler",
          "type": "prod",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-length" => EdgeOut {
          "name": "string-length",
          "spec": "^3.1.0",
          "to": "node_modules/string-length",
          "type": "prod",
        },
        "widest-line" => EdgeOut {
          "name": "widest-line",
          "spec": "^3.1.0",
          "to": "node_modules/widest-line",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
        "yoga-layout-prebuilt" => EdgeOut {
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "to": "node_modules/yoga-layout-prebuilt",
          "type": "prod",
        },
      },
      "location": "node_modules/ink",
      "name": "ink",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ink",
      "resolved": "https://registry.npmjs.org/ink/-/ink-2.7.1.tgz",
      "version": "2.7.1",
    },
    "is-arrayish" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-ci" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-extglob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isstream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "has-flag" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/istanbul-lib-report/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-source-maps",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/istanbul-lib-source-maps/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "html-escaper" => EdgeOut {
          "name": "html-escaper",
          "spec": "^2.0.0",
          "to": "node_modules/html-escaper",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.7.tgz",
      "version": "2.2.7",
    },
    "jackspeak" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.14.0.tgz",
      "version": "3.14.0",
    },
    "jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "json5" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "json5",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/json5",
      "name": "json5",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json5",
      "resolved": "https://registry.npmjs.org/json5/-/json5-2.1.3.tgz",
      "version": "2.1.3",
    },
    "jsprim" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-module-transforms",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.19",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.20.tgz",
      "version": "4.17.20",
    },
    "lodash.flattendeep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "lodash.throttle" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.throttle",
      "name": "lodash.throttle",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash.throttle",
      "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
      "version": "4.1.1",
    },
    "log-driver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "log-update" => ArboristNode {
      "children": Map {
        "ansi-escapes" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/ansi-escapes",
          "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
          "version": "3.2.0",
        },
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/log-update/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cli-cursor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/cli-cursor",
          "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
          "version": "2.1.0",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/log-update/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "emoji-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "mimic-fn" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/mimic-fn",
          "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
          "version": "1.2.0",
        },
        "onetime" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "to": "node_modules/log-update/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/onetime",
          "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
          "version": "2.0.1",
        },
        "restore-cursor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/restore-cursor",
          "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/log-update/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/log-update/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/log-update/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "log-update",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^3.2.0",
          "to": "node_modules/log-update/node_modules/ansi-escapes",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^2.1.0",
          "to": "node_modules/log-update/node_modules/cli-cursor",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^5.0.0",
          "to": "node_modules/log-update/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/log-update",
      "name": "log-update",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update",
      "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.4.0.tgz",
      "version": "3.4.0",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.6.tgz",
      "version": "1.3.6",
    },
    "merge-source-map" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/merge-source-map",
              "name": "source-map",
              "spec": "^0.6.1",
              "type": "prod",
            },
          },
          "location": "node_modules/merge-source-map/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/merge-source-map/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/merge-source-map/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.44.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.44.0.tgz",
      "version": "1.44.0",
    },
    "mime-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.44.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.27.tgz",
      "version": "2.1.27",
    },
    "mimic-fn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/json5",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "minipass",
          "spec": "^3.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.3.tgz",
      "version": "3.1.3",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "ms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "nested-error-stacks" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-modules-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "children": Map {
        "resolve-from" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nyc",
              "name": "resolve-from",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/nyc/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nyc/node_modules/resolve-from",
          "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/nyc/node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "onetime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.2.tgz",
      "version": "5.1.2",
    },
    "opener" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.2.tgz",
      "version": "1.5.2",
    },
    "os-homedir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.3.0.tgz",
      "version": "2.3.0",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.2.2.tgz",
      "version": "2.2.2",
    },
    "pify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.28",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.8.0.tgz",
      "version": "1.8.0",
    },
    "punycode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "punycode",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "react",
          "spec": "^16.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-16.14.0.tgz",
      "version": "16.14.0",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.13.1.tgz",
      "version": "16.13.1",
    },
    "react-reconciler" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.0.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
      },
      "location": "node_modules/react-reconciler",
      "name": "react-reconciler",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react-reconciler",
      "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.24.0.tgz",
      "version": "0.24.0",
    },
    "read-pkg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.2.1",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.5.0.tgz",
      "version": "3.5.0",
    },
    "release-zalgo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.3",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.2.tgz",
      "version": "2.88.2",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "resolve",
          "spec": "^1.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.17.0.tgz",
      "version": "1.17.0",
    },
    "resolve-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/import-jsx",
          "name": "resolve-from",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
      "version": "3.0.0",
    },
    "restore-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/convert-source-map",
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
      "version": "5.1.2",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "scheduler" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/scheduler",
      "name": "scheduler",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/scheduler",
      "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.18.0.tgz",
      "version": "0.18.0",
    },
    "semver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "semver",
          "spec": "^5.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-command" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "slice-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^2.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/core",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "source-map",
          "spec": "^0.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
      "version": "0.5.7",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/source-map-support/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
          "version": "0.6.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.17",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
      "version": "0.5.19",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.1.tgz",
      "version": "3.1.1",
    },
    "spdx-exceptions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.3.0.tgz",
      "version": "2.3.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.1.tgz",
      "version": "3.0.1",
    },
    "spdx-license-ids" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.6.tgz",
      "version": "3.0.6",
    },
    "sprintf-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string-length" => ArboristNode {
      "children": Map {
        "astral-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "astral-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length/node_modules/astral-regex",
          "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/string-length/node_modules/astral-regex",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-length",
      "name": "string-length",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length",
      "resolved": "https://registry.npmjs.org/string-length/-/string-length-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string-width" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-width/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/string-width/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-width/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-width/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line",
          "name": "string-width",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/string-width/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/log-update/node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^5.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-length",
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/cliui",
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^5.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
      "version": "5.2.0",
    },
    "strip-bom" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^4.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.2.0.tgz",
      "version": "7.2.0",
    },
    "tap" => ArboristNode {
      "bundleDependencies": Array [
        "ink",
        "treport",
        "@types/react",
      ],
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "14",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "error": "MISSING",
          "name": "@types/react",
          "spec": "^16.9.16",
          "to": null,
          "type": "prod",
        },
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.11",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.1.0",
          "to": "node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.6.0",
          "to": "node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.4",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "error": "MISSING",
          "name": "treport",
          "spec": "^1.0.2",
          "to": null,
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.8.tgz",
      "version": "14.10.8",
    },
    "tap-mocha-reporter" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^2.0.0",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^2.0.2",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tap-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^10.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.1.0.tgz",
      "version": "10.1.0",
    },
    "tap-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.28",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.5.0.tgz",
      "version": "2.5.0",
    },
    "trivial-deferred" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.17",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.7",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "3.1.1",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.10.2.tgz",
      "version": "8.10.2",
    },
    "tunnel-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "type-fest" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-escapes",
          "name": "type-fest",
          "spec": "^0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.11.0.tgz",
      "version": "0.11.0",
    },
    "typedarray-to-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "typescript",
          "spec": ">=2.7",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.9.7.tgz",
      "version": "3.9.7",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.0.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-2.0.2.tgz",
      "version": "2.0.2",
    },
    "uri-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.4.0.tgz",
      "version": "4.4.0",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "validate-npm-package-license" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "which" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "widest-line" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrap-ansi/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
          "version": "5.0.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/wrap-ansi/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrap-ansi/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
          "version": "6.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
      "version": "6.2.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.3.tgz",
      "version": "3.0.3",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yaml",
          "spec": "^1.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.10.0.tgz",
      "version": "1.10.0",
    },
    "yapool" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/yargs/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cliui" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/yargs/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "emoji-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/yargs/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/yargs/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.2.tgz",
      "version": "13.3.2",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.2.tgz",
      "version": "13.1.2",
    },
    "yn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
    "yoga-layout-prebuilt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/yoga-layout" => EdgeOut {
          "name": "@types/yoga-layout",
          "spec": "1.9.2",
          "to": "node_modules/@types/yoga-layout",
          "type": "prod",
        },
      },
      "location": "node_modules/yoga-layout-prebuilt",
      "name": "yoga-layout-prebuilt",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yoga-layout-prebuilt",
      "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.10.0.tgz",
      "version": "1.10.0",
    },
  },
  "edgesOut": Map {
    "flow-parser" => EdgeOut {
      "name": "flow-parser",
      "spec": "0.x",
      "to": "node_modules/flow-parser",
      "type": "prod",
    },
    "flow-remove-types" => EdgeOut {
      "name": "flow-remove-types",
      "spec": "^2.112.0",
      "to": "node_modules/flow-remove-types",
      "type": "prod",
    },
    "ink" => EdgeOut {
      "name": "ink",
      "spec": "^2.6.0",
      "to": "node_modules/ink",
      "type": "prod",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "14",
      "to": "node_modules/tap",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-and-flow",
  "path": "{CWD}/test/fixtures/tap-and-flow",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP update tap and flow > update ink 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.5.5.tgz",
      "version": "7.5.5",
    },
    "@babel/generator" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/generator",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/generator/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/generator/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/generator/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-function-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/highlight" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/@babel/highlight/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "chalk" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight",
              "name": "chalk",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "to": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^5.3.0",
              "to": "node_modules/@babel/highlight/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
          "version": "2.4.2",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/@babel/highlight/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "has-flag" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/highlight/node_modules/chalk",
              "name": "supports-color",
              "spec": "^5.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/@babel/highlight/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/highlight/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
          "version": "5.5.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/@babel/highlight/node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.5.0.tgz",
      "version": "7.5.0",
    },
    "@babel/parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.7.5.tgz",
      "version": "7.7.5",
    },
    "@babel/runtime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yaml",
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerator-runtime" => EdgeOut {
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "to": "node_modules/regenerator-runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/runtime",
      "name": "@babel/runtime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/runtime",
      "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.7.6.tgz",
      "version": "7.7.6",
    },
    "@babel/template" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@types/color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@types/color-name",
      "name": "@types/color-name",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/@types/color-name",
      "resolved": "https://registry.npmjs.org/@types/color-name/-/color-name-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ajv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.2.tgz",
      "version": "6.10.2",
    },
    "ansi-escapes" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "type-fest" => EdgeOut {
          "name": "type-fest",
          "spec": "^0.8.1",
          "to": "node_modules/type-fest",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-4.3.0.tgz",
      "version": "4.3.0",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-5.0.0.tgz",
      "version": "5.0.0",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/color-name" => EdgeOut {
          "name": "@types/color-name",
          "spec": "^1.1.1",
          "to": "node_modules/@types/color-name",
          "type": "prod",
        },
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^2.0.1",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-4.2.0.tgz",
      "version": "4.2.0",
    },
    "anymatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.2.tgz",
      "version": "4.1.2",
    },
    "argparse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "arrify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "arrify",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-2.0.1.tgz",
      "version": "2.0.1",
    },
    "asn1" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "astral-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "astral-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/astral-regex",
      "name": "astral-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/astral-regex",
      "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "auto-bind" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "auto-bind",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/auto-bind",
      "name": "auto-bind",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/auto-bind",
      "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-4.0.0.tgz",
      "version": "4.0.0",
    },
    "aws-sign2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.9.0.tgz",
      "version": "1.9.0",
    },
    "balanced-match" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "bind-obj-methods" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "camelcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "caseless" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "chalk",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^7.1.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-3.0.0.tgz",
      "version": "3.0.0",
    },
    "chokidar" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.3.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.3.1.tgz",
      "version": "3.3.1",
    },
    "ci-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-ci",
          "name": "ci-info",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ci-info",
      "name": "ci-info",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ci-info",
      "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
      "version": "2.0.0",
    },
    "cli-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "cli-truncate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.2.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-truncate",
      "name": "cli-truncate",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cli-truncate",
      "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-2.1.0.tgz",
      "version": "2.1.0",
    },
    "cliui" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "string-width",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/cliui/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "to": "node_modules/cliui/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "to": "node_modules/cliui/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
          "version": "4.0.0",
        },
        "wrap-ansi" => ArboristNode {
          "children": Map {
            "ansi-regex" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
              "version": "2.1.1",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
              "version": "1.0.0",
            },
            "string-width" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "code-point-at" => EdgeOut {
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "to": "node_modules/code-point-at",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
              "version": "1.0.2",
            },
            "strip-ansi" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
              "version": "3.0.1",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^1.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
          "version": "2.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/cliui/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/cliui/node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/cliui/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "~1.1.4",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-2.0.1.tgz",
      "version": "2.0.1",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "~1.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.4.tgz",
      "version": "1.1.4",
    },
    "color-support" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commander" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "commander",
          "spec": "~2.20.3",
          "type": "prod",
        },
      },
      "location": "node_modules/commander",
      "name": "commander",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/commander",
      "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.3.tgz",
      "version": "2.20.3",
    },
    "commondir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/convert-source-map",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/convert-source-map/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/convert-source-map/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/convert-source-map/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-util-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.9.tgz",
      "version": "3.0.9",
    },
    "cp-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
      "version": "4.1.1",
    },
    "decamelize" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.1.tgz",
      "version": "4.0.1",
    },
    "diff-frag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-8.0.0.tgz",
      "version": "8.0.0",
    },
    "error-ex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esutils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "events-to-array" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
      "version": "2.0.1",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fill-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-parser",
          "spec": "0.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.114.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.114.0.tgz",
      "version": "0.114.0",
    },
    "flow-remove-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.114.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.114.0.tgz",
      "version": "2.114.0",
    },
    "foreground-child" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.2.tgz",
      "version": "2.1.2",
    },
    "function-loop" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.0.tgz",
      "version": "5.1.0",
    },
    "globals" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.3.tgz",
      "version": "4.2.3",
    },
    "handlebars" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "handlebars",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.0",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "optimist" => EdgeOut {
          "name": "optimist",
          "spec": "^0.6.1",
          "to": "node_modules/optimist",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "uglify-js" => EdgeOut {
          "name": "uglify-js",
          "spec": "^3.1.4",
          "to": "node_modules/uglify-js",
          "type": "optional",
        },
      },
      "location": "node_modules/handlebars",
      "name": "handlebars",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/handlebars",
      "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.5.3.tgz",
      "version": "4.5.3",
    },
    "har-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.5.5",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
      "version": "5.1.3",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-4.0.0.tgz",
      "version": "4.0.0",
    },
    "hasha" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.5.tgz",
      "version": "2.8.5",
    },
    "http-signature" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "imurmurhash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "ink" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "ink",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": ">=16.8.0",
          "to": null,
          "type": "peerOptional",
        },
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^4.2.1",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^2.0.1",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "auto-bind" => EdgeOut {
          "name": "auto-bind",
          "spec": "^4.0.0",
          "to": "node_modules/auto-bind",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^3.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^3.1.0",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-truncate" => EdgeOut {
          "name": "cli-truncate",
          "spec": "^2.1.0",
          "to": "node_modules/cli-truncate",
          "type": "prod",
        },
        "is-ci" => EdgeOut {
          "name": "is-ci",
          "spec": "^2.0.0",
          "to": "node_modules/is-ci",
          "type": "prod",
        },
        "lodash.throttle" => EdgeOut {
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "to": "node_modules/lodash.throttle",
          "type": "prod",
        },
        "log-update" => EdgeOut {
          "name": "log-update",
          "spec": "^3.0.0",
          "to": "node_modules/log-update",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": ">=16.8.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "react-reconciler" => EdgeOut {
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "to": "node_modules/react-reconciler",
          "type": "prod",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-length" => EdgeOut {
          "name": "string-length",
          "spec": "^3.1.0",
          "to": "node_modules/string-length",
          "type": "prod",
        },
        "widest-line" => EdgeOut {
          "name": "widest-line",
          "spec": "^3.1.0",
          "to": "node_modules/widest-line",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
        "yoga-layout-prebuilt" => EdgeOut {
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "to": "node_modules/yoga-layout-prebuilt",
          "type": "prod",
        },
      },
      "location": "node_modules/ink",
      "name": "ink",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ink",
      "resolved": "https://registry.npmjs.org/ink/-/ink-2.7.1.tgz",
      "version": "2.7.1",
    },
    "is-arrayish" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-ci" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "is-ci",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ci-info" => EdgeOut {
          "name": "ci-info",
          "spec": "^2.0.0",
          "to": "node_modules/ci-info",
          "type": "prod",
        },
      },
      "location": "node_modules/is-ci",
      "name": "is-ci",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-ci",
      "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-extglob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/slice-ansi",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-3.0.0.tgz",
      "version": "3.0.0",
    },
    "is-glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isstream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "has-flag" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report/node_modules/has-flag",
          "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
          "version": "3.0.0",
        },
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/istanbul-lib-report/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "handlebars" => EdgeOut {
          "name": "handlebars",
          "spec": "^4.1.2",
          "to": "node_modules/handlebars",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.6.tgz",
      "version": "2.2.6",
    },
    "jackspeak" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
      "version": "3.13.1",
    },
    "jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsprim" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.15.tgz",
      "version": "4.17.15",
    },
    "lodash.flattendeep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "lodash.throttle" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "lodash.throttle",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.throttle",
      "name": "lodash.throttle",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lodash.throttle",
      "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
      "version": "4.1.1",
    },
    "log-driver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "log-update" => ArboristNode {
      "children": Map {
        "ansi-escapes" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/ansi-escapes",
          "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
          "version": "3.2.0",
        },
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/log-update/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cli-cursor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/cli-cursor",
          "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
          "version": "2.1.0",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/log-update/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "emoji-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "mimic-fn" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/mimic-fn",
          "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
          "version": "1.2.0",
        },
        "onetime" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^1.0.0",
              "to": "node_modules/log-update/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/onetime",
          "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
          "version": "2.0.1",
        },
        "restore-cursor" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/restore-cursor",
          "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/log-update/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/log-update/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/log-update/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/log-update/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/log-update",
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/log-update/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/log-update/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/log-update/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/log-update/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "log-update",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^3.2.0",
          "to": "node_modules/log-update/node_modules/ansi-escapes",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^2.1.0",
          "to": "node_modules/log-update/node_modules/cli-cursor",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^5.0.0",
          "to": "node_modules/log-update/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/log-update",
      "name": "log-update",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/log-update",
      "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.3.0.tgz",
      "version": "3.3.0",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
      "version": "1.3.5",
    },
    "merge-source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.42.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.42.0.tgz",
      "version": "1.42.0",
    },
    "mime-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.42.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.25.tgz",
      "version": "2.1.25",
    },
    "mimic-fn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/onetime",
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mimic-fn",
      "name": "mimic-fn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mimic-fn",
      "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
      "version": "2.1.0",
    },
    "minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
      "version": "1.2.0",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.1.tgz",
      "version": "3.1.1",
    },
    "mkdirp" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "node_modules/mkdirp/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mkdirp/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/mkdirp/node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "neo-async" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "neo-async",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/neo-async",
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.1.tgz",
      "version": "2.6.1",
    },
    "nested-error-stacks" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-modules-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/scheduler",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "onetime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mimic-fn" => EdgeOut {
          "name": "mimic-fn",
          "spec": "^2.1.0",
          "to": "node_modules/mimic-fn",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-5.1.0.tgz",
      "version": "5.1.0",
    },
    "opener" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
      "version": "1.5.1",
    },
    "optimist" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/optimist",
              "name": "minimist",
              "spec": "~0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/optimist/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/optimist/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.10.tgz",
          "version": "0.0.10",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "optimist",
          "spec": "^0.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "~0.0.1",
          "to": "node_modules/optimist/node_modules/minimist",
          "type": "prod",
        },
        "wordwrap" => EdgeOut {
          "name": "wordwrap",
          "spec": "~0.0.2",
          "to": "node_modules/wordwrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optimist",
      "name": "optimist",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/optimist",
      "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
      "version": "0.6.1",
    },
    "os-homedir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.1.tgz",
      "version": "2.2.1",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.1.1.tgz",
      "version": "2.1.1",
    },
    "pify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "process-nextick-args" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.1.tgz",
      "version": "2.0.1",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.24",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.6.0.tgz",
      "version": "1.6.0",
    },
    "punycode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "react",
          "spec": "^16.12.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "react",
          "spec": "^16.8.6",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-16.12.0.tgz",
      "version": "16.12.0",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.12.0.tgz",
      "version": "16.12.0",
    },
    "react-reconciler" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "react-reconciler",
          "spec": "^0.24.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.0.0",
          "to": "node_modules/react",
          "type": "peer",
        },
        "scheduler" => EdgeOut {
          "name": "scheduler",
          "spec": "^0.18.0",
          "to": "node_modules/scheduler",
          "type": "prod",
        },
      },
      "location": "node_modules/react-reconciler",
      "name": "react-reconciler",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/react-reconciler",
      "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.24.0.tgz",
      "version": "0.24.0",
    },
    "read-pkg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readable-stream" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/readable-stream",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/readable-stream/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/readable-stream/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "readable-stream",
          "spec": "^2.1.5",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "~1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/readable-stream/node_modules/safe-buffer",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~1.1.1",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
      "version": "2.3.6",
    },
    "readdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.7",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.3.0.tgz",
      "version": "3.3.0",
    },
    "regenerator-runtime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/runtime",
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-runtime",
      "name": "regenerator-runtime",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/regenerator-runtime",
      "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.3.tgz",
      "version": "0.13.3",
    },
    "release-zalgo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.0",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
      "version": "2.88.0",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.14.0.tgz",
      "version": "1.14.0",
    },
    "resolve-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "resolve-from",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
      "version": "4.0.0",
    },
    "restore-cursor" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^5.1.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-3.1.0.tgz",
      "version": "3.1.0",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.0.tgz",
      "version": "5.2.0",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "scheduler" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react-reconciler",
          "name": "scheduler",
          "spec": "^0.18.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/scheduler",
      "name": "scheduler",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/scheduler",
      "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.18.0.tgz",
      "version": "0.18.0",
    },
    "semver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-command" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/log-update/node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
      "version": "3.0.2",
    },
    "slice-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ink",
          "name": "slice-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^2.0.0",
          "to": "node_modules/astral-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/merge-source-map",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "source-map",
          "spec": "^0.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "source-map",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
      "version": "0.6.1",
    },
    "source-map-support" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.16.tgz",
      "version": "0.5.16",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
      "version": "3.1.0",
    },
    "spdx-exceptions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
      "version": "2.2.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
      "version": "3.0.0",
    },
    "spdx-license-ids" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.5.tgz",
      "version": "3.0.5",
    },
    "sprintf-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string_decoder" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string_decoder",
              "name": "safe-buffer",
              "spec": "~5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string_decoder/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string_decoder/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "to": "node_modules/string_decoder/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
      "version": "1.1.1",
    },
    "string-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "astral-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "astral-regex",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length/node_modules/astral-regex",
          "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
          "version": "1.0.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string-length",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/string-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/string-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "string-length",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "astral-regex" => EdgeOut {
          "name": "astral-regex",
          "spec": "^1.0.0",
          "to": "node_modules/string-length/node_modules/astral-regex",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^5.2.0",
          "to": "node_modules/string-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-length",
      "name": "string-length",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-length",
      "resolved": "https://registry.npmjs.org/string-length/-/string-length-3.1.0.tgz",
      "version": "3.1.0",
    },
    "string-width" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-truncate",
          "name": "string-width",
          "spec": "^4.2.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/widest-line",
          "name": "string-width",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "string-width",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "emoji-regex" => EdgeOut {
          "name": "emoji-regex",
          "spec": "^8.0.0",
          "to": "node_modules/emoji-regex",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^3.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-4.2.0.tgz",
      "version": "4.2.0",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/wrap-ansi",
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^5.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-6.0.0.tgz",
      "version": "6.0.0",
    },
    "strip-bom" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^7.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^4.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-7.1.0.tgz",
      "version": "7.1.0",
    },
    "tap" => ArboristNode {
      "bundleDependencies": Array [
        "ink",
        "treport",
        "@types/react",
      ],
      "children": Map {
        "@babel/code-frame" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/highlight" => EdgeOut {
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/code-frame",
          "name": "@babel/code-frame",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/code-frame",
          "version": "7.5.5",
        },
        "@babel/core" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/core",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/core",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helpers" => EdgeOut {
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helpers",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "to": "node_modules/tap/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "json5" => EdgeOut {
              "name": "json5",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/json5",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.3.2",
              "to": "node_modules/tap/node_modules/resolve",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.4.1",
              "to": "node_modules/tap/node_modules/semver",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/core",
          "name": "@babel/core",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/core",
          "version": "7.7.5",
        },
        "@babel/generator" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "jsesc" => EdgeOut {
              "name": "jsesc",
              "spec": "^2.5.1",
              "to": "node_modules/tap/node_modules/jsesc",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/generator",
          "name": "@babel/generator",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/generator",
          "version": "7.7.4",
        },
        "@babel/helper-builder-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "version": "7.7.4",
        },
        "@babel/helper-function-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/helper-get-function-arity" => EdgeOut {
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-function-name",
          "name": "@babel/helper-function-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helper-function-name",
          "version": "7.7.4",
        },
        "@babel/helper-get-function-arity" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "name": "@babel/helper-get-function-arity",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "version": "7.7.4",
        },
        "@babel/helper-plugin-utils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "name": "@babel/helper-plugin-utils",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "version": "7.0.0",
        },
        "@babel/helper-split-export-declaration" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/helper-split-export-declaration",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "version": "7.7.4",
        },
        "@babel/helpers" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helpers",
          "name": "@babel/helpers",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/helpers",
          "version": "7.7.4",
        },
        "@babel/highlight" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight",
                  "name": "chalk",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "version": "2.4.2",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "has-flag" => EdgeOut {
                  "name": "has-flag",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/has-flag",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "version": "5.5.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/code-frame",
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/highlight",
          "name": "@babel/highlight",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/highlight",
          "version": "7.5.0",
        },
        "@babel/parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/parser",
          "name": "@babel/parser",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/parser",
          "version": "7.7.5",
        },
        "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-destructuring" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/plugin-transform-destructuring",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-builder-react-jsx" => EdgeOut {
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "type": "prod",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-jsx" => EdgeOut {
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "version": "7.7.4",
        },
        "@babel/runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/yaml",
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "regenerator-runtime" => EdgeOut {
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "to": "node_modules/tap/node_modules/regenerator-runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/runtime",
          "name": "@babel/runtime",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/runtime",
          "version": "7.7.6",
        },
        "@babel/template" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/template",
          "name": "@babel/template",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/template",
          "version": "7.7.4",
        },
        "@babel/traverse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helper-function-name" => EdgeOut {
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-function-name",
              "type": "prod",
            },
            "@babel/helper-split-export-declaration" => EdgeOut {
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "globals" => EdgeOut {
              "name": "globals",
              "spec": "^11.1.0",
              "to": "node_modules/tap/node_modules/globals",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/traverse",
          "name": "@babel/traverse",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/traverse",
          "version": "7.7.4",
        },
        "@babel/types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "to-fast-properties" => EdgeOut {
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/to-fast-properties",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/types",
          "name": "@babel/types",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@babel/types",
          "version": "7.7.4",
        },
        "@types/color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/color-name",
          "name": "@types/color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@types/color-name",
          "version": "1.1.1",
        },
        "@types/prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "@types/prop-types",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/prop-types",
          "name": "@types/prop-types",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@types/prop-types",
          "version": "15.7.3",
        },
        "@types/react" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "@types/react",
              "spec": "^16.9.16",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/auto-bind",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peerOptional",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@types/prop-types" => EdgeOut {
              "name": "@types/prop-types",
              "spec": "*",
              "to": "node_modules/tap/node_modules/@types/prop-types",
              "type": "prod",
            },
            "csstype" => EdgeOut {
              "name": "csstype",
              "spec": "^2.2.0",
              "to": "node_modules/tap/node_modules/csstype",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/react",
          "name": "@types/react",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/@types/react",
          "version": "16.9.16",
        },
        "ansi-escapes" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/tap/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ansi-escapes",
          "version": "4.3.0",
        },
        "ansi-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ansi-regex",
          "version": "5.0.0",
        },
        "ansi-styles" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/tap/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ansi-styles",
          "version": "3.2.1",
        },
        "ansicolors" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "ansicolors",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansicolors",
          "name": "ansicolors",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ansicolors",
          "version": "0.3.2",
        },
        "arrify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "arrify",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/arrify",
          "name": "arrify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/arrify",
          "version": "2.0.1",
        },
        "astral-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "astral-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/astral-regex",
          "version": "2.0.0",
        },
        "auto-bind" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "auto-bind",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peerOptional",
            },
          },
          "location": "node_modules/tap/node_modules/auto-bind",
          "name": "auto-bind",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/auto-bind",
          "version": "3.0.0",
        },
        "caller-callsite" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-path",
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "callsites" => EdgeOut {
              "name": "callsites",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/callsites",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-callsite",
          "name": "caller-callsite",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/caller-callsite",
          "version": "2.0.0",
        },
        "caller-path" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "caller-path",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "caller-callsite" => EdgeOut {
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-callsite",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-path",
          "name": "caller-path",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/caller-path",
          "version": "2.0.0",
        },
        "callsites" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-callsite",
              "name": "callsites",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/callsites",
          "name": "callsites",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/callsites",
          "version": "2.0.0",
        },
        "cardinal" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "cardinal",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansicolors" => EdgeOut {
              "name": "ansicolors",
              "spec": "~0.3.2",
              "to": "node_modules/tap/node_modules/ansicolors",
              "type": "prod",
            },
            "redeyed" => EdgeOut {
              "name": "redeyed",
              "spec": "~2.1.0",
              "to": "node_modules/tap/node_modules/redeyed",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cardinal",
          "name": "cardinal",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/cardinal",
          "version": "2.1.1",
        },
        "chalk" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/chalk/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/tap/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/chalk",
          "version": "3.0.0",
        },
        "ci-info" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/is-ci",
              "name": "ci-info",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ci-info",
          "name": "ci-info",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ci-info",
          "version": "2.0.0",
        },
        "cli-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/cli-cursor",
          "version": "3.1.0",
        },
        "cli-truncate" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.2.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-truncate",
          "name": "cli-truncate",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/cli-truncate",
          "version": "2.1.0",
        },
        "color-convert" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/tap/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/color-convert",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/color-name",
          "version": "1.1.3",
        },
        "convert-source-map" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "safe-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/convert-source-map",
                  "name": "safe-buffer",
                  "spec": "~5.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "name": "safe-buffer",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "version": "5.1.2",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "to": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/convert-source-map",
          "name": "convert-source-map",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/convert-source-map",
          "version": "1.7.0",
        },
        "csstype" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "csstype",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/csstype",
          "name": "csstype",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/csstype",
          "version": "2.6.8",
        },
        "debug" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/debug",
          "version": "4.1.1",
        },
        "emoji-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/emoji-regex",
          "version": "8.0.0",
        },
        "escape-string-regexp" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/escape-string-regexp",
          "version": "1.0.5",
        },
        "esprima" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/redeyed",
              "name": "esprima",
              "spec": "~4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/esprima",
          "version": "4.0.1",
        },
        "esutils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "esutils",
              "spec": "^2.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/esutils",
          "version": "2.0.3",
        },
        "events-to-array" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "events-to-array",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/events-to-array",
          "name": "events-to-array",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/events-to-array",
          "version": "1.1.2",
        },
        "globals" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "globals",
              "spec": "^11.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/globals",
          "version": "11.12.0",
        },
        "has-flag" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/has-flag",
          "version": "3.0.0",
        },
        "import-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "prod",
            },
            "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "type": "prod",
            },
            "@babel/plugin-transform-destructuring" => EdgeOut {
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "type": "prod",
            },
            "@babel/plugin-transform-react-jsx" => EdgeOut {
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "type": "prod",
            },
            "caller-path" => EdgeOut {
              "name": "caller-path",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-path",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/resolve-from",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/import-jsx",
          "name": "import-jsx",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/import-jsx",
          "version": "3.0.0",
        },
        "ink" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peer",
            },
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "to": "node_modules/tap/node_modules/ansi-escapes",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^2.0.1",
              "to": "node_modules/tap/node_modules/arrify",
              "type": "prod",
            },
            "auto-bind" => EdgeOut {
              "name": "auto-bind",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/auto-bind",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/cli-cursor",
              "type": "prod",
            },
            "cli-truncate" => EdgeOut {
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/cli-truncate",
              "type": "prod",
            },
            "is-ci" => EdgeOut {
              "name": "is-ci",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/is-ci",
              "type": "prod",
            },
            "lodash.throttle" => EdgeOut {
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/lodash.throttle",
              "type": "prod",
            },
            "log-update" => EdgeOut {
              "name": "log-update",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/log-update",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": ">=16.8.0",
              "to": "node_modules/react",
              "type": "peer",
            },
            "react-reconciler" => EdgeOut {
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "to": "node_modules/tap/node_modules/react-reconciler",
              "type": "prod",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "widest-line" => EdgeOut {
              "name": "widest-line",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/widest-line",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "to": "node_modules/tap/node_modules/wrap-ansi",
              "type": "prod",
            },
            "yoga-layout-prebuilt" => EdgeOut {
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "to": "node_modules/tap/node_modules/yoga-layout-prebuilt",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ink",
          "name": "ink",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ink",
          "version": "2.6.0",
        },
        "is-ci" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "is-ci",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ci-info" => EdgeOut {
              "name": "ci-info",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/ci-info",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-ci",
          "name": "is-ci",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/is-ci",
          "version": "2.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/is-fullwidth-code-point",
          "version": "3.0.0",
        },
        "js-tokens" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "js-tokens",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/loose-envify",
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/js-tokens",
          "name": "js-tokens",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/js-tokens",
          "version": "4.0.0",
        },
        "jsesc" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "jsesc",
              "spec": "^2.5.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/jsesc",
          "name": "jsesc",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/jsesc",
          "version": "2.5.2",
        },
        "json5" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "json5",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/json5",
          "name": "json5",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/json5",
          "version": "2.1.1",
        },
        "lodash" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/lodash",
          "version": "4.17.15",
        },
        "lodash.throttle" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash.throttle",
          "name": "lodash.throttle",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/lodash.throttle",
          "version": "4.1.1",
        },
        "log-update" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-escapes" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "ansi-escapes",
                  "spec": "^3.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "name": "ansi-escapes",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "version": "3.2.0",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "cli-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "cli-cursor",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "restore-cursor" => EdgeOut {
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "name": "cli-cursor",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "version": "2.1.0",
            },
            "emoji-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "name": "emoji-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "version": "7.0.3",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "version": "2.0.0",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "version": "1.2.0",
            },
            "onetime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/onetime",
              "name": "onetime",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/onetime",
              "version": "2.0.1",
            },
            "restore-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "onetime" => EdgeOut {
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "restore-cursor",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "version": "2.0.0",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "emoji-regex" => EdgeOut {
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/string-width",
              "version": "3.1.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "version": "5.2.0",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "wrap-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "version": "5.1.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "log-update",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/log-update",
          "name": "log-update",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/log-update",
          "version": "3.3.0",
        },
        "loose-envify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "loose-envify",
              "spec": "^1.4.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/loose-envify",
          "name": "loose-envify",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/loose-envify",
          "version": "1.4.0",
        },
        "mimic-fn" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/mimic-fn",
          "version": "2.1.0",
        },
        "minimist" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/json5",
              "name": "minimist",
              "spec": "^1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/minimist",
          "version": "1.2.0",
        },
        "minipass" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/minipass",
                  "name": "yallist",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/minipass/node_modules/yallist",
              "version": "4.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "minipass",
              "spec": "^3.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "minipass",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minipass",
          "name": "minipass",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/minipass",
          "version": "3.1.1",
        },
        "ms" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/debug",
              "name": "ms",
              "spec": "^2.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ms",
              "spec": "^2.1.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/ms",
          "version": "2.1.2",
        },
        "object-assign" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/object-assign",
          "name": "object-assign",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/object-assign",
          "version": "4.1.1",
        },
        "onetime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/onetime",
          "version": "5.1.0",
        },
        "path-parse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/resolve",
              "name": "path-parse",
              "spec": "^1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/path-parse",
          "name": "path-parse",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/path-parse",
          "version": "1.0.6",
        },
        "prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.4.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "react-is" => EdgeOut {
              "name": "react-is",
              "spec": "^16.8.1",
              "to": "node_modules/tap/node_modules/react-is",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/prop-types",
          "name": "prop-types",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/prop-types",
          "version": "15.7.2",
        },
        "punycode" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "punycode",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/punycode",
          "version": "2.1.1",
        },
        "react-is" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "react-is",
              "spec": "^16.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-is",
          "name": "react-is",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/react-is",
          "version": "16.12.0",
        },
        "react-reconciler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.0.0",
              "to": "node_modules/react",
              "type": "peer",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-reconciler",
          "name": "react-reconciler",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/react-reconciler",
          "version": "0.24.0",
        },
        "redeyed" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "redeyed",
              "spec": "~2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "~4.0.0",
              "to": "node_modules/tap/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/redeyed",
          "name": "redeyed",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/redeyed",
          "version": "2.1.1",
        },
        "regenerator-runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/runtime",
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/regenerator-runtime",
          "name": "regenerator-runtime",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/regenerator-runtime",
          "version": "0.13.3",
        },
        "resolve" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "resolve",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "path-parse" => EdgeOut {
              "name": "path-parse",
              "spec": "^1.0.6",
              "to": "node_modules/tap/node_modules/path-parse",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve",
          "name": "resolve",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/resolve",
          "version": "1.13.1",
        },
        "resolve-from" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "resolve-from",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/resolve-from",
          "version": "3.0.0",
        },
        "restore-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^5.1.0",
              "to": "node_modules/tap/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/restore-cursor",
          "version": "3.1.0",
        },
        "scheduler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/scheduler",
          "name": "scheduler",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/scheduler",
          "version": "0.18.0",
        },
        "semver" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "semver",
              "spec": "^5.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/semver",
          "version": "5.7.1",
        },
        "signal-exit" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/signal-exit",
          "version": "3.0.2",
        },
        "slice-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/astral-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/slice-ansi",
          "name": "slice-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/slice-ansi",
          "version": "3.0.0",
        },
        "string-length" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "astral-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "astral-regex",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "name": "astral-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "version": "1.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "strip-ansi",
                  "spec": "^5.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "version": "5.2.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-length",
          "name": "string-length",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/string-length",
          "version": "3.1.0",
        },
        "string-width" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "string-width",
              "spec": "^4.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/widest-line",
              "name": "string-width",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/tap/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/string-width",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/strip-ansi",
          "version": "6.0.0",
        },
        "supports-color" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "version": "4.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/supports-color",
          "version": "7.1.0",
        },
        "tap-parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "events-to-array" => EdgeOut {
              "name": "events-to-array",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/events-to-array",
              "type": "prod",
            },
            "minipass" => EdgeOut {
              "name": "minipass",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/minipass",
              "type": "prod",
            },
            "tap-yaml" => EdgeOut {
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/tap-yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-parser",
          "name": "tap-parser",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/tap-parser",
          "version": "10.0.1",
        },
        "tap-yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yaml" => EdgeOut {
              "name": "yaml",
              "spec": "^1.5.0",
              "to": "node_modules/tap/node_modules/yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-yaml",
          "name": "tap-yaml",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/tap-yaml",
          "version": "1.0.0",
        },
        "to-fast-properties" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/to-fast-properties",
          "name": "to-fast-properties",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/to-fast-properties",
          "version": "2.0.0",
        },
        "treport" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "unicode-length" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport",
                  "name": "unicode-length",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "punycode" => EdgeOut {
                  "name": "punycode",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/punycode",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "unicode-length",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "version": "2.0.2",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "treport",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cardinal" => EdgeOut {
              "name": "cardinal",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/cardinal",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "import-jsx" => EdgeOut {
              "name": "import-jsx",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/import-jsx",
              "type": "prod",
            },
            "ink" => EdgeOut {
              "name": "ink",
              "spec": "^2.5.0",
              "to": "node_modules/tap/node_modules/ink",
              "type": "prod",
            },
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.2",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.8.6",
              "to": "node_modules/react",
              "type": "peer",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^10.0.1",
              "to": "node_modules/tap/node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/treport",
          "name": "treport",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/treport",
          "version": "1.0.1",
        },
        "type-fest" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-escapes",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/type-fest",
          "version": "0.8.1",
        },
        "widest-line" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "widest-line",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/widest-line",
          "name": "widest-line",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/widest-line",
          "version": "3.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/wrap-ansi",
          "version": "6.2.0",
        },
        "yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "yaml",
              "spec": "^1.7.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-yaml",
              "name": "yaml",
              "spec": "^1.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/runtime" => EdgeOut {
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "to": "node_modules/tap/node_modules/@babel/runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yaml",
          "name": "yaml",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/yaml",
          "version": "1.7.2",
        },
        "yoga-layout-prebuilt" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yoga-layout-prebuilt",
          "name": "yoga-layout-prebuilt",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap/node_modules/yoga-layout-prebuilt",
          "version": "1.9.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "14",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": "^16.9.16",
          "to": "node_modules/tap/node_modules/@types/react",
          "type": "prod",
        },
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.8",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/tap/node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/tap/node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap/node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap/node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "name": "treport",
          "spec": "^1.0.1",
          "to": "node_modules/tap/node_modules/treport",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/tap/node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.5.tgz",
      "version": "14.10.5",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "debug",
              "spec": "^2.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/tap-mocha-reporter/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-mocha-reporter/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "diff" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "diff",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/diff",
          "name": "diff",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-mocha-reporter/node_modules/diff",
          "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
          "version": "1.4.0",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-mocha-reporter/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.3",
          "to": "node_modules/tap-mocha-reporter/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^1.3.2",
          "to": "node_modules/tap-mocha-reporter/node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^1.0.0",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.0.tgz",
      "version": "5.0.0",
    },
    "tap-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.0.1.tgz",
      "version": "10.0.1",
    },
    "tap-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tough-cookie",
              "name": "punycode",
              "spec": "^1.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tough-cookie/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tough-cookie/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.24",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/tough-cookie/node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
      "version": "2.4.3",
    },
    "trivial-deferred" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.0",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "^3.0.0",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.5.4.tgz",
      "version": "8.5.4",
    },
    "tunnel-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "type-fest" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-escapes",
          "name": "type-fest",
          "spec": "^0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/type-fest",
      "name": "type-fest",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/type-fest",
      "resolved": "https://registry.npmjs.org/type-fest/-/type-fest-0.8.1.tgz",
      "version": "0.8.1",
    },
    "typedarray-to-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "typescript",
          "spec": ">=2.0",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.7.3.tgz",
      "version": "3.7.3",
    },
    "uglify-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "uglify-js",
          "spec": "^3.1.4",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "~2.20.3",
          "to": "node_modules/commander",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/uglify-js",
      "name": "uglify-js",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/uglify-js",
      "resolved": "https://registry.npmjs.org/uglify-js/-/uglify-js-3.7.2.tgz",
      "version": "3.7.2",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "punycode" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "punycode",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/unicode-length/node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uri-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
      "version": "4.2.2",
    },
    "util-deprecate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "optional": true,
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
      "version": "1.0.2",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.3.tgz",
      "version": "3.3.3",
    },
    "validate-npm-package-license" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "which" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "widest-line" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "widest-line",
          "spec": "^3.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.0.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/widest-line",
      "name": "widest-line",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/widest-line",
      "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-3.1.0.tgz",
      "version": "3.1.0",
    },
    "wordwrap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optimist",
          "name": "wordwrap",
          "spec": "~0.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
      "version": "0.0.3",
    },
    "wrap-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "wrap-ansi",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^4.0.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^4.1.0",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^6.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-6.2.0.tgz",
      "version": "6.2.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.1.tgz",
      "version": "3.0.1",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.7.2.tgz",
      "version": "1.7.2",
    },
    "yapool" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "ansi-styles" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/yargs/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
          "version": "3.2.1",
        },
        "cliui" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "color-convert" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/yargs/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/color-convert",
          "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/color-name",
          "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
          "version": "1.1.3",
        },
        "emoji-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/yargs/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/yargs/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.0.tgz",
      "version": "13.3.0",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.1.tgz",
      "version": "13.1.1",
    },
    "yn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
    "yoga-layout-prebuilt" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ink",
          "name": "yoga-layout-prebuilt",
          "spec": "^1.9.3",
          "type": "prod",
        },
      },
      "location": "node_modules/yoga-layout-prebuilt",
      "name": "yoga-layout-prebuilt",
      "path": "{CWD}/test/fixtures/tap-and-flow/node_modules/yoga-layout-prebuilt",
      "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.9.3.tgz",
      "version": "1.9.3",
    },
  },
  "edgesOut": Map {
    "flow-parser" => EdgeOut {
      "name": "flow-parser",
      "spec": "0.x",
      "to": "node_modules/flow-parser",
      "type": "prod",
    },
    "flow-remove-types" => EdgeOut {
      "name": "flow-remove-types",
      "spec": "^2.112.0",
      "to": "node_modules/flow-remove-types",
      "type": "prod",
    },
    "ink" => EdgeOut {
      "name": "ink",
      "spec": "^2.6.0",
      "to": "node_modules/ink",
      "type": "prod",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "14",
      "to": "node_modules/tap",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-and-flow",
  "path": "{CWD}/test/fixtures/tap-and-flow",
  "version": "1.0.0",
}
`

exports[`test/arborist/build-ideal-tree.js TAP upgrade a partly overlapping peer set > should be able to upgrade dep, nesting the conflict 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "3",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "3",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "name": "@isaacs/testing-peer-dep-conflict-chain-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "spec": "3",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "3",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "spec": "3",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "3",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "name": "@isaacs/testing-peer-dep-conflict-chain-c",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "spec": "3",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "spec": "3",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "name": "@isaacs/testing-peer-dep-conflict-chain-d",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-dep-conflict-chain-m" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-dep-conflict-chain-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "name": "@isaacs/testing-peer-dep-conflict-chain-a",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-a/-/testing-peer-dep-conflict-chain-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "name": "@isaacs/testing-peer-dep-conflict-chain-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-c" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "name": "@isaacs/testing-peer-dep-conflict-chain-b",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-b/-/testing-peer-dep-conflict-chain-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
              "name": "@isaacs/testing-peer-dep-conflict-chain-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-d" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "name": "@isaacs/testing-peer-dep-conflict-chain-c",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-c/-/testing-peer-dep-conflict-chain-c-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-c",
              "name": "@isaacs/testing-peer-dep-conflict-chain-d",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "name": "@isaacs/testing-peer-dep-conflict-chain-d",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-d/-/testing-peer-dep-conflict-chain-d-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-dep-conflict-chain-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-d",
              "name": "@isaacs/testing-peer-dep-conflict-chain-e",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-dep-conflict-chain-a" => EdgeOut {
              "name": "@isaacs/testing-peer-dep-conflict-chain-a",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-a",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-e/-/testing-peer-dep-conflict-chain-e-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-dep-conflict-chain-m",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-dep-conflict-chain-e" => EdgeOut {
          "name": "@isaacs/testing-peer-dep-conflict-chain-e",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m/node_modules/@isaacs/testing-peer-dep-conflict-chain-e",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "name": "@isaacs/testing-peer-dep-conflict-chain-m",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set/node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-dep-conflict-chain-m/-/testing-peer-dep-conflict-chain-m-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-dep-conflict-chain-b" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-b",
      "spec": "3",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-dep-conflict-chain-m" => EdgeOut {
      "name": "@isaacs/testing-peer-dep-conflict-chain-m",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-dep-conflict-chain-m",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-upgrade-a-partly-overlapping-peer-set",
}
`

exports[`test/arborist/build-ideal-tree.js TAP warn for ancient lockfile, even if we use v1 > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "acorn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn",
          "spec": "4.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/fixtures/sax/node_modules/acorn",
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-4.0.4.tgz",
      "version": "4.0.4",
    },
    "acorn-jsx" => ArboristNode {
      "children": Map {
        "acorn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/acorn-jsx",
              "name": "acorn",
              "spec": "^3.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/acorn-jsx/node_modules/acorn",
          "name": "acorn",
          "path": "{CWD}/test/fixtures/sax/node_modules/acorn-jsx/node_modules/acorn",
          "resolved": "https://registry.npmjs.org/acorn/-/acorn-3.3.0.tgz",
          "version": "3.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^3.0.4",
          "to": "node_modules/acorn-jsx/node_modules/acorn",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn-jsx",
      "name": "acorn-jsx",
      "path": "{CWD}/test/fixtures/sax/node_modules/acorn-jsx",
      "resolved": "https://registry.npmjs.org/acorn-jsx/-/acorn-jsx-3.0.1.tgz",
      "version": "3.0.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv-keywords",
          "name": "ajv",
          "spec": ">=4.10.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv",
          "spec": "^4.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "co" => EdgeOut {
          "name": "co",
          "spec": "^4.6.0",
          "to": "node_modules/co",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/fixtures/sax/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-4.11.2.tgz",
      "version": "4.11.2",
    },
    "ajv-keywords" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": ">=4.10.0",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-keywords",
      "name": "ajv-keywords",
      "path": "{CWD}/test/fixtures/sax/node_modules/ajv-keywords",
      "resolved": "https://registry.npmjs.org/ajv-keywords/-/ajv-keywords-1.5.1.tgz",
      "version": "1.5.1",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-1.4.0.tgz",
      "version": "1.4.0",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/has-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/fixtures/sax/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.1.0.tgz",
      "version": "2.1.0",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/fixtures/sax/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.9.tgz",
      "version": "1.0.9",
    },
    "array-union" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-uniq" => EdgeOut {
          "name": "array-uniq",
          "spec": "^1.0.1",
          "to": "node_modules/array-uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/fixtures/sax/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-1.0.2.tgz",
      "version": "1.0.2",
    },
    "array-uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-union",
          "name": "array-uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/array-uniq",
      "name": "array-uniq",
      "path": "{CWD}/test/fixtures/sax/node_modules/array-uniq",
      "resolved": "https://registry.npmjs.org/array-uniq/-/array-uniq-1.0.3.tgz",
      "version": "1.0.3",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "arrify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/fixtures/sax/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/fixtures/sax/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.3.tgz",
      "version": "0.2.3",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/fixtures/sax/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-0.2.0.tgz",
      "version": "0.2.0",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/fixtures/sax/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/fixtures/sax/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.6.0.tgz",
      "version": "0.6.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/fixtures/sax/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.6.0.tgz",
      "version": "1.6.0",
    },
    "babel-code-frame" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/babel-code-frame",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/babel-code-frame/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/babel-code-frame/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/babel-code-frame/node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/babel-code-frame",
      "name": "babel-code-frame",
      "path": "{CWD}/test/fixtures/sax/node_modules/babel-code-frame",
      "resolved": "https://registry.npmjs.org/babel-code-frame/-/babel-code-frame-6.22.0.tgz",
      "version": "6.22.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/fixtures/sax/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-0.3.0.tgz",
      "version": "0.3.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.1.tgz",
      "version": "1.0.1",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/fixtures/sax/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bluebird" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bluebird",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/bluebird",
      "name": "bluebird",
      "path": "{CWD}/test/fixtures/sax/node_modules/bluebird",
      "resolved": "https://registry.npmjs.org/bluebird/-/bluebird-3.5.0.tgz",
      "version": "3.5.0",
    },
    "boom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cryptiles",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/boom",
      "name": "boom",
      "path": "{CWD}/test/fixtures/sax/node_modules/boom",
      "resolved": "https://registry.npmjs.org/boom/-/boom-2.10.1.tgz",
      "version": "2.10.1",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^0.3.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/fixtures/sax/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.3.tgz",
      "version": "1.1.3",
    },
    "buffer-shims" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "buffer-shims",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-shims",
      "name": "buffer-shims",
      "path": "{CWD}/test/fixtures/sax/node_modules/buffer-shims",
      "resolved": "https://registry.npmjs.org/buffer-shims/-/buffer-shims-1.0.0.tgz",
      "version": "1.0.0",
    },
    "caller-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "caller-path",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^0.2.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/fixtures/sax/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-0.1.0.tgz",
      "version": "0.1.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "callsites",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/fixtures/sax/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-0.2.0.tgz",
      "version": "0.2.0",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/fixtures/sax/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.11.0.tgz",
      "version": "0.11.0",
    },
    "chalk" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/chalk",
              "name": "supports-color",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/chalk/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/fixtures/sax/node_modules/chalk/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "chalk",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "chalk",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "has-ansi" => EdgeOut {
          "name": "has-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/has-ansi",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^2.0.0",
          "to": "node_modules/chalk/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/fixtures/sax/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.1.tgz",
      "version": "1.1.1",
    },
    "circular-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "circular-json",
          "spec": "^0.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/circular-json",
      "name": "circular-json",
      "path": "{CWD}/test/fixtures/sax/node_modules/circular-json",
      "resolved": "https://registry.npmjs.org/circular-json/-/circular-json-0.3.1.tgz",
      "version": "0.3.1",
    },
    "clean-yaml-object" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/clean-yaml-object",
      "name": "clean-yaml-object",
      "path": "{CWD}/test/fixtures/sax/node_modules/clean-yaml-object",
      "resolved": "https://registry.npmjs.org/clean-yaml-object/-/clean-yaml-object-0.1.0.tgz",
      "version": "0.1.0",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/fixtures/sax/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-1.0.2.tgz",
      "version": "1.0.2",
    },
    "cli-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-width",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-width",
      "name": "cli-width",
      "path": "{CWD}/test/fixtures/sax/node_modules/cli-width",
      "resolved": "https://registry.npmjs.org/cli-width/-/cli-width-2.1.0.tgz",
      "version": "2.1.0",
    },
    "co" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "co",
          "spec": "^4.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/co",
      "name": "co",
      "path": "{CWD}/test/fixtures/sax/node_modules/co",
      "resolved": "https://registry.npmjs.org/co/-/co-4.6.0.tgz",
      "version": "4.6.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/fixtures/sax/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.0.0.tgz",
      "version": "1.0.0",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/fixtures/sax/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.2.tgz",
      "version": "1.1.2",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.5.tgz",
      "version": "1.0.5",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "concat-stream" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "inherits",
              "spec": "^2.0.3",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "readable-stream" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "readable-stream",
              "spec": "^2.2.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "buffer-shims" => EdgeOut {
              "name": "buffer-shims",
              "spec": "^1.0.0",
              "to": "node_modules/buffer-shims",
              "type": "prod",
            },
            "core-util-is" => EdgeOut {
              "name": "core-util-is",
              "spec": "~1.0.0",
              "to": "node_modules/core-util-is",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "~2.0.1",
              "to": "node_modules/concat-stream/node_modules/inherits",
              "type": "prod",
            },
            "isarray" => EdgeOut {
              "name": "isarray",
              "spec": "~1.0.0",
              "to": "node_modules/concat-stream/node_modules/isarray",
              "type": "prod",
            },
            "process-nextick-args" => EdgeOut {
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "to": "node_modules/concat-stream/node_modules/process-nextick-args",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "~0.10.x",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "~1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream/node_modules/readable-stream",
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.2.2.tgz",
          "version": "2.2.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "concat-stream",
          "spec": "^1.4.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/concat-stream/node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.2.2",
          "to": "node_modules/concat-stream/node_modules/readable-stream",
          "type": "prod",
        },
        "typedarray" => EdgeOut {
          "name": "typedarray",
          "spec": "^0.0.6",
          "to": "node_modules/typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-stream",
      "name": "concat-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/concat-stream",
      "resolved": "https://registry.npmjs.org/concat-stream/-/concat-stream-1.6.0.tgz",
      "version": "1.6.0",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/fixtures/sax/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "children": Map {
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls/node_modules/js-yaml",
              "name": "esprima",
              "spec": "^2.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.7.3.tgz",
          "version": "2.7.3",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "js-yaml",
              "spec": "3.6.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^2.6.0",
              "to": "node_modules/coveralls/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.6.1.tgz",
          "version": "3.6.1",
        },
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "minimist",
              "spec": "1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/sax/node_modules/coveralls/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^2.11.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "3.6.1",
          "to": "node_modules/coveralls/node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "0.0.10",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "1.2.5",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "1.2.0",
          "to": "node_modules/coveralls/node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "2.79.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/fixtures/sax/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-2.11.16.tgz",
      "version": "2.11.16",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "lru-cache" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "lru-cache",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pseudomap" => EdgeOut {
              "name": "pseudomap",
              "spec": "^1.0.1",
              "to": "node_modules/pseudomap",
              "type": "prod",
            },
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^2.0.0",
              "to": "node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "lru-cache",
          "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn/node_modules/lru-cache",
          "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.0.2.tgz",
          "version": "4.0.2",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^1.1.1",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.2.12.tgz",
          "version": "1.2.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/cross-spawn/node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/fixtures/sax/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "cryptiles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "cryptiles",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
      },
      "location": "node_modules/cryptiles",
      "name": "cryptiles",
      "path": "{CWD}/test/fixtures/sax/node_modules/cryptiles",
      "resolved": "https://registry.npmjs.org/cryptiles/-/cryptiles-2.0.5.tgz",
      "version": "2.0.5",
    },
    "d" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.2",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/d",
      "name": "d",
      "path": "{CWD}/test/fixtures/sax/node_modules/d",
      "resolved": "https://registry.npmjs.org/d/-/d-0.1.1.tgz",
      "version": "0.1.1",
    },
    "dashdash" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/dashdash",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/dashdash/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/dashdash/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/dashdash/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/fixtures/sax/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "debug",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "debug",
          "spec": "^2.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "0.7.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/fixtures/sax/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-2.2.0.tgz",
      "version": "2.2.0",
    },
    "debug-log" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "debug-log",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/debug-log",
      "name": "debug-log",
      "path": "{CWD}/test/fixtures/sax/node_modules/debug-log",
      "resolved": "https://registry.npmjs.org/debug-log/-/debug-log-1.0.1.tgz",
      "version": "1.0.1",
    },
    "deep-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "deep-is",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-is",
      "name": "deep-is",
      "path": "{CWD}/test/fixtures/sax/node_modules/deep-is",
      "resolved": "https://registry.npmjs.org/deep-is/-/deep-is-0.1.3.tgz",
      "version": "0.1.3",
    },
    "deglob" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob",
              "name": "glob",
              "spec": "^7.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/deglob/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/deglob/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/deglob/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "deglob",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/deglob/node_modules/glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.0.9",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.1.0",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.2",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
        "uniq" => EdgeOut {
          "name": "uniq",
          "spec": "^1.0.1",
          "to": "node_modules/uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/deglob",
      "name": "deglob",
      "path": "{CWD}/test/fixtures/sax/node_modules/deglob",
      "resolved": "https://registry.npmjs.org/deglob/-/deglob-2.1.0.tgz",
      "version": "2.1.0",
    },
    "del" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "del",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^5.0.0",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-path-cwd" => EdgeOut {
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-cwd",
          "type": "prod",
        },
        "is-path-in-cwd" => EdgeOut {
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-in-cwd",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.2.8",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "node_modules/del",
      "name": "del",
      "path": "{CWD}/test/fixtures/sax/node_modules/del",
      "resolved": "https://registry.npmjs.org/del/-/del-2.2.2.tgz",
      "version": "2.2.2",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "diff",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/fixtures/sax/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
      "version": "1.4.0",
    },
    "doctrine" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/doctrine/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "isarray",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/fixtures/sax/node_modules/doctrine/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/doctrine/node_modules/esutils",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "^1.0.0",
          "to": "node_modules/doctrine/node_modules/isarray",
          "type": "prod",
        },
      },
      "location": "node_modules/doctrine",
      "name": "doctrine",
      "path": "{CWD}/test/fixtures/sax/node_modules/doctrine",
      "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-1.5.0.tgz",
      "version": "1.5.0",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "es5-ext" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/d",
          "name": "es5-ext",
          "spec": "~0.10.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es5-ext",
          "spec": "^0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es5-ext",
          "spec": "^0.10.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es5-ext",
      "name": "es5-ext",
      "path": "{CWD}/test/fixtures/sax/node_modules/es5-ext",
      "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.8.tgz",
      "version": "0.10.8",
    },
    "es6-iterator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-iterator",
      "name": "es6-iterator",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-iterator",
      "resolved": "https://registry.npmjs.org/es6-iterator/-/es6-iterator-2.0.0.tgz",
      "version": "2.0.0",
    },
    "es6-map" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-map/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-map/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
        "es6-symbol" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es6-symbol",
              "spec": "~3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es5-ext",
              "name": "es6-symbol",
              "spec": "~3.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "d" => EdgeOut {
              "name": "d",
              "spec": "~0.1.1",
              "to": "node_modules/d",
              "type": "prod",
            },
            "es5-ext" => EdgeOut {
              "name": "es5-ext",
              "spec": "~0.10.11",
              "to": "node_modules/es6-map/node_modules/es5-ext",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "es6-symbol",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-map/node_modules/es6-symbol",
          "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-map",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-map/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-set" => EdgeOut {
          "name": "es6-set",
          "spec": "~0.1.3",
          "to": "node_modules/es6-set",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "~3.1.0",
          "to": "node_modules/es6-map/node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-map",
      "name": "es6-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-map",
      "resolved": "https://registry.npmjs.org/es6-map/-/es6-map-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-set" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "children": Map {
            "es6-symbol" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/es6-set/node_modules/es5-ext",
                  "name": "es6-symbol",
                  "spec": "~3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "d" => EdgeOut {
                  "name": "d",
                  "spec": "~0.1.1",
                  "to": "node_modules/d",
                  "type": "prod",
                },
                "es5-ext" => EdgeOut {
                  "name": "es5-ext",
                  "spec": "~0.10.11",
                  "to": "node_modules/es6-set/node_modules/es5-ext",
                  "type": "prod",
                },
              },
              "location": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es6-symbol",
              "path": "{CWD}/test/fixtures/sax/node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
              "version": "3.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-set",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/fixtures/sax/node_modules/es6-set/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-set",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-set/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-set",
      "name": "es6-set",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-set",
      "resolved": "https://registry.npmjs.org/es6-set/-/es6-set-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-symbol" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-symbol",
      "name": "es6-symbol",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-symbol",
      "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.0.0.tgz",
      "version": "3.0.0",
    },
    "es6-weak-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.8",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-weak-map",
      "name": "es6-weak-map",
      "path": "{CWD}/test/fixtures/sax/node_modules/es6-weak-map",
      "resolved": "https://registry.npmjs.org/es6-weak-map/-/es6-weak-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/fixtures/sax/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.3.tgz",
      "version": "1.0.3",
    },
    "escope" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/escope",
              "name": "estraverse",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/escope/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/escope/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "escope",
          "spec": "^3.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-map" => EdgeOut {
          "name": "es6-map",
          "spec": "^0.1.3",
          "to": "node_modules/es6-map",
          "type": "prod",
        },
        "es6-weak-map" => EdgeOut {
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "to": "node_modules/es6-weak-map",
          "type": "prod",
        },
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.1.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/escope/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/escope",
      "name": "escope",
      "path": "{CWD}/test/fixtures/sax/node_modules/escope",
      "resolved": "https://registry.npmjs.org/escope/-/escope-3.6.0.tgz",
      "version": "3.6.0",
    },
    "eslint" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.2.1.tgz",
          "version": "2.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "chalk",
              "spec": "^1.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "to": "node_modules/eslint/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.2",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "has-ansi" => EdgeOut {
              "name": "has-ansi",
              "spec": "^2.0.0",
              "to": "node_modules/has-ansi",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^2.0.0",
              "to": "node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.3.tgz",
          "version": "1.1.3",
        },
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "estraverse",
              "spec": "^4.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "fast-levenshtein" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/fast-levenshtein",
          "name": "fast-levenshtein",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/fast-levenshtein",
          "resolved": "https://registry.npmjs.org/fast-levenshtein/-/fast-levenshtein-2.0.6.tgz",
          "version": "2.0.6",
        },
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/eslint/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "js-yaml",
              "spec": "^3.5.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^3.1.1",
              "to": "node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.8.1.tgz",
          "version": "3.8.1",
        },
        "levn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "levn",
              "spec": "^0.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "levn",
              "spec": "~0.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/levn",
          "name": "levn",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/levn",
          "resolved": "https://registry.npmjs.org/levn/-/levn-0.3.0.tgz",
          "version": "0.3.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
        "optionator" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "optionator",
              "spec": "^0.8.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "deep-is" => EdgeOut {
              "name": "deep-is",
              "spec": "~0.1.3",
              "to": "node_modules/deep-is",
              "type": "prod",
            },
            "fast-levenshtein" => EdgeOut {
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "to": "node_modules/eslint/node_modules/fast-levenshtein",
              "type": "prod",
            },
            "levn" => EdgeOut {
              "name": "levn",
              "spec": "~0.3.0",
              "to": "node_modules/eslint/node_modules/levn",
              "type": "prod",
            },
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
            "wordwrap" => EdgeOut {
              "name": "wordwrap",
              "spec": "~1.0.0",
              "to": "node_modules/wordwrap",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/optionator",
          "name": "optionator",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/optionator",
          "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.8.2.tgz",
          "version": "0.8.2",
        },
        "strip-bom" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "strip-bom",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/strip-bom",
          "name": "strip-bom",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/strip-bom",
          "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
          "version": "3.0.0",
        },
        "type-check" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/levn",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/type-check",
          "name": "type-check",
          "path": "{CWD}/test/fixtures/sax/node_modules/eslint/node_modules/type-check",
          "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.3.2.tgz",
          "version": "0.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint",
          "spec": ">=3.8.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-standard",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint",
          "spec": "~3.10.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "babel-code-frame" => EdgeOut {
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "to": "node_modules/babel-code-frame",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.3",
          "to": "node_modules/eslint/node_modules/chalk",
          "type": "prod",
        },
        "concat-stream" => EdgeOut {
          "name": "concat-stream",
          "spec": "^1.4.6",
          "to": "node_modules/concat-stream",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "escope" => EdgeOut {
          "name": "escope",
          "spec": "^3.6.0",
          "to": "node_modules/escope",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^3.3.1",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.2.0",
          "to": "node_modules/eslint/node_modules/estraverse",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/eslint/node_modules/esutils",
          "type": "prod",
        },
        "file-entry-cache" => EdgeOut {
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "to": "node_modules/file-entry-cache",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/eslint/node_modules/glob",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^9.2.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.2.0",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "inquirer" => EdgeOut {
          "name": "inquirer",
          "spec": "^0.12.0",
          "to": "node_modules/inquirer",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "to": "node_modules/is-my-json-valid",
          "type": "prod",
        },
        "is-resolvable" => EdgeOut {
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "to": "node_modules/is-resolvable",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.5.1",
          "to": "node_modules/eslint/node_modules/js-yaml",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.3.0",
          "to": "node_modules/eslint/node_modules/levn",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/eslint/node_modules/lodash",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "natural-compare" => EdgeOut {
          "name": "natural-compare",
          "spec": "^1.4.0",
          "to": "node_modules/natural-compare",
          "type": "prod",
        },
        "optionator" => EdgeOut {
          "name": "optionator",
          "spec": "^0.8.2",
          "to": "node_modules/eslint/node_modules/optionator",
          "type": "prod",
        },
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
        "pluralize" => EdgeOut {
          "name": "pluralize",
          "spec": "^1.2.1",
          "to": "node_modules/pluralize",
          "type": "prod",
        },
        "progress" => EdgeOut {
          "name": "progress",
          "spec": "^1.1.8",
          "to": "node_modules/progress",
          "type": "prod",
        },
        "require-uncached" => EdgeOut {
          "name": "require-uncached",
          "spec": "^1.0.2",
          "to": "node_modules/require-uncached",
          "type": "prod",
        },
        "shelljs" => EdgeOut {
          "name": "shelljs",
          "spec": "^0.7.5",
          "to": "node_modules/shelljs",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/eslint/node_modules/strip-bom",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
        "table" => EdgeOut {
          "name": "table",
          "spec": "^3.7.8",
          "to": "node_modules/table",
          "type": "prod",
        },
        "text-table" => EdgeOut {
          "name": "text-table",
          "spec": "~0.2.0",
          "to": "node_modules/text-table",
          "type": "prod",
        },
        "user-home" => EdgeOut {
          "name": "user-home",
          "spec": "^2.0.0",
          "to": "node_modules/user-home",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint",
      "name": "eslint",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint",
      "resolved": "https://registry.npmjs.org/eslint/-/eslint-3.10.2.tgz",
      "version": "3.10.2",
    },
    "eslint-config-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.8.1",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "peer",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "to": "node_modules/eslint-plugin-standard",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard",
      "name": "eslint-config-standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-config-standard",
      "resolved": "https://registry.npmjs.org/eslint-config-standard/-/eslint-config-standard-6.2.1.tgz",
      "version": "6.2.1",
    },
    "eslint-config-standard-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard-jsx",
      "name": "eslint-config-standard-jsx",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-config-standard-jsx",
      "resolved": "https://registry.npmjs.org/eslint-config-standard-jsx/-/eslint-config-standard-jsx-3.2.0.tgz",
      "version": "3.2.0",
    },
    "eslint-plugin-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-promise",
      "name": "eslint-plugin-promise",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-promise",
      "resolved": "https://registry.npmjs.org/eslint-plugin-promise/-/eslint-plugin-promise-3.4.1.tgz",
      "version": "3.4.1",
    },
    "eslint-plugin-react" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "jsx-ast-utils" => EdgeOut {
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "to": "node_modules/jsx-ast-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-react",
      "name": "eslint-plugin-react",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-react",
      "resolved": "https://registry.npmjs.org/eslint-plugin-react/-/eslint-plugin-react-6.7.1.tgz",
      "version": "6.7.1",
    },
    "eslint-plugin-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-plugin-standard",
      "name": "eslint-plugin-standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/eslint-plugin-standard",
      "resolved": "https://registry.npmjs.org/eslint-plugin-standard/-/eslint-plugin-standard-2.0.1.tgz",
      "version": "2.0.1",
    },
    "espree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "espree",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "4.0.4",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "acorn-jsx" => EdgeOut {
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/acorn-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/espree",
      "name": "espree",
      "path": "{CWD}/test/fixtures/sax/node_modules/espree",
      "resolved": "https://registry.npmjs.org/espree/-/espree-3.4.0.tgz",
      "version": "3.4.0",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "esprima",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/fixtures/sax/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-3.1.3.tgz",
      "version": "3.1.3",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "~4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/fixtures/sax/node_modules/esrecurse/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.1.1.tgz",
          "version": "4.1.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "esrecurse",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "~4.1.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/fixtures/sax/node_modules/esrecurse",
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.1.0.tgz",
      "version": "4.1.0",
    },
    "event-emitter" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/event-emitter",
      "name": "event-emitter",
      "path": "{CWD}/test/fixtures/sax/node_modules/event-emitter",
      "resolved": "https://registry.npmjs.org/event-emitter/-/event-emitter-0.3.4.tgz",
      "version": "0.3.4",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/fixtures/sax/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.0.2.tgz",
      "version": "1.0.2",
    },
    "exit-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "exit-hook",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/exit-hook",
      "name": "exit-hook",
      "path": "{CWD}/test/fixtures/sax/node_modules/exit-hook",
      "resolved": "https://registry.npmjs.org/exit-hook/-/exit-hook-1.1.1.tgz",
      "version": "1.1.1",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/fixtures/sax/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "figures" => ArboristNode {
      "children": Map {
        "escape-string-regexp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/figures",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/figures/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/fixtures/sax/node_modules/figures/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
          "version": "1.0.5",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "figures",
          "spec": "^1.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/figures/node_modules/escape-string-regexp",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/figures",
      "name": "figures",
      "path": "{CWD}/test/fixtures/sax/node_modules/figures",
      "resolved": "https://registry.npmjs.org/figures/-/figures-1.7.0.tgz",
      "version": "1.7.0",
    },
    "file-entry-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flat-cache" => EdgeOut {
          "name": "flat-cache",
          "spec": "^1.2.1",
          "to": "node_modules/flat-cache",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/file-entry-cache",
      "name": "file-entry-cache",
      "path": "{CWD}/test/fixtures/sax/node_modules/file-entry-cache",
      "resolved": "https://registry.npmjs.org/file-entry-cache/-/file-entry-cache-2.0.0.tgz",
      "version": "2.0.0",
    },
    "find-root" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/find-root",
      "name": "find-root",
      "path": "{CWD}/test/fixtures/sax/node_modules/find-root",
      "resolved": "https://registry.npmjs.org/find-root/-/find-root-1.0.0.tgz",
      "version": "1.0.0",
    },
    "flat-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "flat-cache",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "circular-json" => EdgeOut {
          "name": "circular-json",
          "spec": "^0.3.1",
          "to": "node_modules/circular-json",
          "type": "prod",
        },
        "del" => EdgeOut {
          "name": "del",
          "spec": "^2.0.2",
          "to": "node_modules/del",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "write" => EdgeOut {
          "name": "write",
          "spec": "^0.2.1",
          "to": "node_modules/write",
          "type": "prod",
        },
      },
      "location": "node_modules/flat-cache",
      "name": "flat-cache",
      "path": "{CWD}/test/fixtures/sax/node_modules/flat-cache",
      "resolved": "https://registry.npmjs.org/flat-cache/-/flat-cache-1.2.2.tgz",
      "version": "1.2.2",
    },
    "foreground-child" => ArboristNode {
      "children": Map {
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/foreground-child",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/foreground-child/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/fixtures/sax/node_modules/foreground-child/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/foreground-child/node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/fixtures/sax/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/fixtures/sax/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/fixtures/sax/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.1.2.tgz",
      "version": "2.1.2",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/fixtures/sax/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/fixtures/sax/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/fixtures/sax/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.1.tgz",
      "version": "1.0.1",
    },
    "generate-function" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-function",
      "name": "generate-function",
      "path": "{CWD}/test/fixtures/sax/node_modules/generate-function",
      "resolved": "https://registry.npmjs.org/generate-function/-/generate-function-2.0.0.tgz",
      "version": "2.0.0",
    },
    "generate-object-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-property" => EdgeOut {
          "name": "is-property",
          "spec": "^1.0.0",
          "to": "node_modules/is-property",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-object-property",
      "name": "generate-object-property",
      "path": "{CWD}/test/fixtures/sax/node_modules/generate-object-property",
      "resolved": "https://registry.npmjs.org/generate-object-property/-/generate-object-property-1.2.0.tgz",
      "version": "1.2.0",
    },
    "get-stdin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "get-stdin",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stdin",
      "name": "get-stdin",
      "path": "{CWD}/test/fixtures/sax/node_modules/get-stdin",
      "resolved": "https://registry.npmjs.org/get-stdin/-/get-stdin-5.0.1.tgz",
      "version": "5.0.1",
    },
    "getpass" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/getpass",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/getpass/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/getpass/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/getpass/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/fixtures/sax/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.6.tgz",
      "version": "0.1.6",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/fixtures/sax/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.2.tgz",
      "version": "7.1.2",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "globals",
          "spec": "^9.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/fixtures/sax/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-9.14.0.tgz",
      "version": "9.14.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/globby/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/globby/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/globby/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "globby",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^1.0.1",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^1.0.0",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/globby/node_modules/glob",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/fixtures/sax/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-5.0.0.tgz",
      "version": "5.0.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/fixtures/sax/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.1.11.tgz",
      "version": "4.1.11",
    },
    "graceful-readlink" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/commander",
          "name": "graceful-readlink",
          "spec": ">= 1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-readlink",
      "name": "graceful-readlink",
      "path": "{CWD}/test/fixtures/sax/node_modules/graceful-readlink",
      "resolved": "https://registry.npmjs.org/graceful-readlink/-/graceful-readlink-1.0.1.tgz",
      "version": "1.0.1",
    },
    "har-validator" => ArboristNode {
      "children": Map {
        "commander" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "commander",
              "spec": "^2.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-readlink" => EdgeOut {
              "name": "graceful-readlink",
              "spec": ">= 1.0.0",
              "to": "node_modules/graceful-readlink",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/commander",
          "name": "commander",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/commander",
          "resolved": "https://registry.npmjs.org/commander/-/commander-2.9.0.tgz",
          "version": "2.9.0",
        },
        "is-my-json-valid" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "is-my-json-valid",
              "spec": "^2.12.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "generate-function" => EdgeOut {
              "name": "generate-function",
              "spec": "^2.0.0",
              "to": "node_modules/generate-function",
              "type": "prod",
            },
            "generate-object-property" => EdgeOut {
              "name": "generate-object-property",
              "spec": "^1.1.0",
              "to": "node_modules/generate-object-property",
              "type": "prod",
            },
            "jsonpointer" => EdgeOut {
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "to": "node_modules/har-validator/node_modules/jsonpointer",
              "type": "prod",
            },
            "xtend" => EdgeOut {
              "name": "xtend",
              "spec": "^4.0.0",
              "to": "node_modules/xtend",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "is-my-json-valid",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/is-my-json-valid",
          "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
          "version": "2.15.0",
        },
        "jsonpointer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/is-my-json-valid",
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/jsonpointer",
          "name": "jsonpointer",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/jsonpointer",
          "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
          "version": "4.0.1",
        },
        "pinkie" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/pinkie-promise",
              "name": "pinkie",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie",
          "name": "pinkie",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/pinkie",
          "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
          "version": "2.0.4",
        },
        "pinkie-promise" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "pinkie-promise",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pinkie" => EdgeOut {
              "name": "pinkie",
              "spec": "^2.0.0",
              "to": "node_modules/har-validator/node_modules/pinkie",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie-promise",
          "name": "pinkie-promise",
          "path": "{CWD}/test/fixtures/sax/node_modules/har-validator/node_modules/pinkie-promise",
          "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
          "version": "2.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~2.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "^2.9.0",
          "to": "node_modules/har-validator/node_modules/commander",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.12.4",
          "to": "node_modules/har-validator/node_modules/is-my-json-valid",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/har-validator/node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/fixtures/sax/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-2.0.6.tgz",
      "version": "2.0.6",
    },
    "has-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/has-ansi",
      "name": "has-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/has-ansi",
      "resolved": "https://registry.npmjs.org/has-ansi/-/has-ansi-2.0.0.tgz",
      "version": "2.0.0",
    },
    "hawk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "hawk",
          "spec": "~3.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
        "cryptiles" => EdgeOut {
          "name": "cryptiles",
          "spec": "2.x.x",
          "to": "node_modules/cryptiles",
          "type": "prod",
        },
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
        "sntp" => EdgeOut {
          "name": "sntp",
          "spec": "1.x.x",
          "to": "node_modules/sntp",
          "type": "prod",
        },
      },
      "location": "node_modules/hawk",
      "name": "hawk",
      "path": "{CWD}/test/fixtures/sax/node_modules/hawk",
      "resolved": "https://registry.npmjs.org/hawk/-/hawk-3.1.3.tgz",
      "version": "3.1.3",
    },
    "hoek" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boom",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sntp",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "location": "node_modules/hoek",
      "name": "hoek",
      "path": "{CWD}/test/fixtures/sax/node_modules/hoek",
      "resolved": "https://registry.npmjs.org/hoek/-/hoek-2.16.3.tgz",
      "version": "2.16.3",
    },
    "home-or-tmp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "os-tmpdir" => EdgeOut {
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "to": "node_modules/os-tmpdir",
          "type": "prod",
        },
      },
      "location": "node_modules/home-or-tmp",
      "name": "home-or-tmp",
      "path": "{CWD}/test/fixtures/sax/node_modules/home-or-tmp",
      "resolved": "https://registry.npmjs.org/home-or-tmp/-/home-or-tmp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^0.2.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/fixtures/sax/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ignore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "ignore",
          "spec": "^3.0.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ignore",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ignore",
      "name": "ignore",
      "path": "{CWD}/test/fixtures/sax/node_modules/ignore",
      "resolved": "https://registry.npmjs.org/ignore/-/ignore-3.2.2.tgz",
      "version": "3.2.2",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/fixtures/sax/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/fixtures/sax/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.4.tgz",
      "version": "1.0.4",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/fixtures/sax/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.1.tgz",
      "version": "2.0.1",
    },
    "inquirer" => ArboristNode {
      "children": Map {
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/inquirer",
              "name": "lodash",
              "spec": "^4.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/inquirer/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/inquirer/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "inquirer",
          "spec": "^0.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-width" => EdgeOut {
          "name": "cli-width",
          "spec": "^2.0.0",
          "to": "node_modules/cli-width",
          "type": "prod",
        },
        "figures" => EdgeOut {
          "name": "figures",
          "spec": "^1.3.5",
          "to": "node_modules/figures",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.3.0",
          "to": "node_modules/inquirer/node_modules/lodash",
          "type": "prod",
        },
        "readline2" => EdgeOut {
          "name": "readline2",
          "spec": "^1.0.1",
          "to": "node_modules/readline2",
          "type": "prod",
        },
        "run-async" => EdgeOut {
          "name": "run-async",
          "spec": "^0.1.0",
          "to": "node_modules/run-async",
          "type": "prod",
        },
        "rx-lite" => EdgeOut {
          "name": "rx-lite",
          "spec": "^3.1.2",
          "to": "node_modules/rx-lite",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.1",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "through" => EdgeOut {
          "name": "through",
          "spec": "^2.3.6",
          "to": "node_modules/through",
          "type": "prod",
        },
      },
      "location": "node_modules/inquirer",
      "name": "inquirer",
      "path": "{CWD}/test/fixtures/sax/node_modules/inquirer",
      "resolved": "https://registry.npmjs.org/inquirer/-/inquirer-0.12.0.tgz",
      "version": "0.12.0",
    },
    "interpret" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "interpret",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/interpret",
      "name": "interpret",
      "path": "{CWD}/test/fixtures/sax/node_modules/interpret",
      "resolved": "https://registry.npmjs.org/interpret/-/interpret-1.0.1.tgz",
      "version": "1.0.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-my-json-valid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "generate-function" => EdgeOut {
          "name": "generate-function",
          "spec": "^2.0.0",
          "to": "node_modules/generate-function",
          "type": "prod",
        },
        "generate-object-property" => EdgeOut {
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "to": "node_modules/generate-object-property",
          "type": "prod",
        },
        "jsonpointer" => EdgeOut {
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "to": "node_modules/jsonpointer",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.0",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/is-my-json-valid",
      "name": "is-my-json-valid",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-my-json-valid",
      "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
      "version": "2.15.0",
    },
    "is-path-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-cwd",
      "name": "is-path-cwd",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-cwd",
      "resolved": "https://registry.npmjs.org/is-path-cwd/-/is-path-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-in-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-path-inside" => EdgeOut {
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-in-cwd",
      "name": "is-path-in-cwd",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-in-cwd",
      "resolved": "https://registry.npmjs.org/is-path-in-cwd/-/is-path-in-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-path-in-cwd",
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-inside",
      "name": "is-path-inside",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-path-inside",
      "resolved": "https://registry.npmjs.org/is-path-inside/-/is-path-inside-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/generate-object-property",
          "name": "is-property",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-property",
      "name": "is-property",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-property",
      "resolved": "https://registry.npmjs.org/is-property/-/is-property-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-resolvable" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tryit" => EdgeOut {
          "name": "tryit",
          "spec": "^1.0.1",
          "to": "node_modules/tryit",
          "type": "prod",
        },
      },
      "location": "node_modules/is-resolvable",
      "name": "is-resolvable",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-resolvable",
      "resolved": "https://registry.npmjs.org/is-resolvable/-/is-resolvable-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-0.0.1.tgz",
      "version": "0.0.1",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/fixtures/sax/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-1.1.2.tgz",
      "version": "1.1.2",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/fixtures/sax/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "jodid25519" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jodid25519",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/jodid25519",
      "name": "jodid25519",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/jodid25519",
      "resolved": "https://registry.npmjs.org/jodid25519/-/jodid25519-1.0.2.tgz",
      "version": "1.0.2",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "js-tokens",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/fixtures/sax/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-3.0.1.tgz",
      "version": "3.0.1",
    },
    "js-yaml" => ArboristNode {
      "children": Map {
        "argparse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "argparse",
              "spec": "~1.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": ">= 3.2.0 < 4.0.0",
              "to": "node_modules/lodash",
              "type": "prod",
            },
            "sprintf-js" => EdgeOut {
              "name": "sprintf-js",
              "spec": "~1.0.2",
              "to": "node_modules/sprintf-js",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/argparse",
          "name": "argparse",
          "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml/node_modules/argparse",
          "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.2.tgz",
          "version": "1.0.2",
        },
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "esprima",
              "spec": "~2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.2.0.tgz",
          "version": "2.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "js-yaml",
          "spec": "^3.2.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "~1.0.2",
          "to": "node_modules/js-yaml/node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "~2.2.0",
          "to": "node_modules/js-yaml/node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/fixtures/sax/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.4.2.tgz",
      "version": "3.4.2",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jodid25519",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "optional",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.0.tgz",
      "version": "0.1.0",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsonify" => EdgeOut {
          "name": "jsonify",
          "spec": "~0.0.0",
          "to": "node_modules/jsonify",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stable-stringify",
      "name": "json-stable-stringify",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-stable-stringify",
      "resolved": "https://registry.npmjs.org/json-stable-stringify/-/json-stable-stringify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/fixtures/sax/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsonify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/json-stable-stringify",
          "name": "jsonify",
          "spec": "~0.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonify",
      "name": "jsonify",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsonify",
      "resolved": "https://registry.npmjs.org/jsonify/-/jsonify-0.0.0.tgz",
      "version": "0.0.0",
    },
    "jsonpointer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonpointer",
      "name": "jsonpointer",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsonpointer",
      "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
      "version": "4.0.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.3.6",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.3.1.tgz",
      "version": "1.3.1",
    },
    "jsx-ast-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/jsx-ast-utils",
      "name": "jsx-ast-utils",
      "path": "{CWD}/test/fixtures/sax/node_modules/jsx-ast-utils",
      "resolved": "https://registry.npmjs.org/jsx-ast-utils/-/jsx-ast-utils-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "0.0.10",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/fixtures/sax/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-0.0.10.tgz",
      "version": "0.0.10",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "lodash",
          "spec": ">= 3.2.0 < 4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/fixtures/sax/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-3.10.1.tgz",
      "version": "3.10.1",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/fixtures/sax/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "~1.26.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/fixtures/sax/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.26.0.tgz",
      "version": "1.26.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "~1.26.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/fixtures/sax/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.14.tgz",
      "version": "2.1.14",
    },
    "minimatch" => ArboristNode {
      "children": Map {
        "balanced-match" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch/node_modules/brace-expansion",
              "name": "balanced-match",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/balanced-match",
          "name": "balanced-match",
          "path": "{CWD}/test/fixtures/sax/node_modules/minimatch/node_modules/balanced-match",
          "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
          "version": "1.0.0",
        },
        "brace-expansion" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch",
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "balanced-match" => EdgeOut {
              "name": "balanced-match",
              "spec": "^1.0.0",
              "to": "node_modules/minimatch/node_modules/balanced-match",
              "type": "prod",
            },
            "concat-map" => EdgeOut {
              "name": "concat-map",
              "spec": "0.0.1",
              "to": "node_modules/concat-map",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "brace-expansion",
          "path": "{CWD}/test/fixtures/sax/node_modules/minimatch/node_modules/brace-expansion",
          "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.8.tgz",
          "version": "1.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "minimatch",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/minimatch/node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/fixtures/sax/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/fixtures/sax/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/fixtures/sax/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "0.7.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/fixtures/sax/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-0.7.1.tgz",
      "version": "0.7.1",
    },
    "mute-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "mute-stream",
          "spec": "0.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/mute-stream",
      "name": "mute-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/mute-stream",
      "resolved": "https://registry.npmjs.org/mute-stream/-/mute-stream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "natural-compare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/natural-compare",
      "name": "natural-compare",
      "path": "{CWD}/test/fixtures/sax/node_modules/natural-compare",
      "resolved": "https://registry.npmjs.org/natural-compare/-/natural-compare-1.4.0.tgz",
      "version": "1.4.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/code-point-at",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/fixtures/sax/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.0.tgz",
      "version": "1.0.0",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/fixtures/sax/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.8.2.tgz",
      "version": "0.8.2",
    },
    "object-assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/esrecurse",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/figures",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsx-ast-utils",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/fixtures/sax/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/run-async",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/sax/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "version": "1.3.2",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/fixtures/sax/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-1.1.0.tgz",
      "version": "1.1.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/fixtures/sax/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.4.2.tgz",
      "version": "1.4.2",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "os-homedir",
          "spec": "1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/user-home",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/fixtures/sax/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "os-tmpdir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-tmpdir",
      "name": "os-tmpdir",
      "path": "{CWD}/test/fixtures/sax/node_modules/os-tmpdir",
      "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/fixtures/sax/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/fixtures/sax/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/fixtures/sax/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-path-inside",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-inside",
      "name": "path-is-inside",
      "path": "{CWD}/test/fixtures/sax/node_modules/path-is-inside",
      "resolved": "https://registry.npmjs.org/path-is-inside/-/path-is-inside-1.0.2.tgz",
      "version": "1.0.2",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/fixtures/sax/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-2.3.0.tgz",
      "version": "2.3.0",
    },
    "pinkie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pinkie-promise",
          "name": "pinkie",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie",
      "name": "pinkie",
      "path": "{CWD}/test/fixtures/sax/node_modules/pinkie",
      "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
      "version": "2.0.4",
    },
    "pinkie-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pinkie" => EdgeOut {
          "name": "pinkie",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie-promise",
      "name": "pinkie-promise",
      "path": "{CWD}/test/fixtures/sax/node_modules/pinkie-promise",
      "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
      "version": "2.0.1",
    },
    "pkg-config" => ArboristNode {
      "children": Map {
        "xtend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-config",
              "name": "xtend",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-config/node_modules/xtend",
          "name": "xtend",
          "path": "{CWD}/test/fixtures/sax/node_modules/pkg-config/node_modules/xtend",
          "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.1.tgz",
          "version": "4.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "pkg-config",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "pkg-config",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug-log" => EdgeOut {
          "name": "debug-log",
          "spec": "^1.0.0",
          "to": "node_modules/debug-log",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.1",
          "to": "node_modules/pkg-config/node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-config",
      "name": "pkg-config",
      "path": "{CWD}/test/fixtures/sax/node_modules/pkg-config",
      "resolved": "https://registry.npmjs.org/pkg-config/-/pkg-config-1.1.1.tgz",
      "version": "1.1.1",
    },
    "pluralize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "pluralize",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pluralize",
      "name": "pluralize",
      "path": "{CWD}/test/fixtures/sax/node_modules/pluralize",
      "resolved": "https://registry.npmjs.org/pluralize/-/pluralize-1.2.1.tgz",
      "version": "1.2.1",
    },
    "prelude-ls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/levn",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/type-check",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/prelude-ls",
      "name": "prelude-ls",
      "path": "{CWD}/test/fixtures/sax/node_modules/prelude-ls",
      "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.1.2.tgz",
      "version": "1.1.2",
    },
    "process-nextick-args" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "path": "{CWD}/test/fixtures/sax/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.3.tgz",
      "version": "1.0.3",
    },
    "progress" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "progress",
          "spec": "^1.1.8",
          "type": "prod",
        },
      },
      "location": "node_modules/progress",
      "name": "progress",
      "path": "{CWD}/test/fixtures/sax/node_modules/progress",
      "resolved": "https://registry.npmjs.org/progress/-/progress-1.1.8.tgz",
      "version": "1.1.8",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/fixtures/sax/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^1.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "punycode",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/fixtures/sax/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
      "version": "1.4.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/fixtures/sax/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.3.0.tgz",
      "version": "6.3.0",
    },
    "readable-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "readable-stream",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "readable-stream",
          "spec": "^2",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "0.0.1",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~0.10.x",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "path": "{CWD}/test/fixtures/sax/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.0.2.tgz",
      "version": "2.0.2",
    },
    "readline2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "readline2",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "mute-stream" => EdgeOut {
          "name": "mute-stream",
          "spec": "0.0.5",
          "to": "node_modules/mute-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/readline2",
      "name": "readline2",
      "path": "{CWD}/test/fixtures/sax/node_modules/readline2",
      "resolved": "https://registry.npmjs.org/readline2/-/readline2-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rechoir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "rechoir",
          "spec": "^0.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.1.6",
          "to": "node_modules/resolve",
          "type": "prod",
        },
      },
      "location": "node_modules/rechoir",
      "name": "rechoir",
      "path": "{CWD}/test/fixtures/sax/node_modules/rechoir",
      "resolved": "https://registry.npmjs.org/rechoir/-/rechoir-0.6.2.tgz",
      "version": "0.6.2",
    },
    "request" => ArboristNode {
      "children": Map {
        "extend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/request",
              "name": "extend",
              "spec": "~3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/request/node_modules/extend",
          "name": "extend",
          "path": "{CWD}/test/fixtures/sax/node_modules/request/node_modules/extend",
          "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "2.79.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.2.1",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.11.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.0",
          "to": "node_modules/request/node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.1.1",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~2.0.6",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "hawk" => EdgeOut {
          "name": "hawk",
          "spec": "~3.1.3",
          "to": "node_modules/hawk",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.1.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.7",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.3.0",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "stringstream" => EdgeOut {
          "name": "stringstream",
          "spec": "~0.0.4",
          "to": "node_modules/stringstream",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.0.0",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/fixtures/sax/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.79.0.tgz",
      "version": "2.79.0",
    },
    "require-uncached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "require-uncached",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^0.1.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^1.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/require-uncached",
      "name": "require-uncached",
      "path": "{CWD}/test/fixtures/sax/node_modules/require-uncached",
      "resolved": "https://registry.npmjs.org/require-uncached/-/require-uncached-1.0.3.tgz",
      "version": "1.0.3",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rechoir",
          "name": "resolve",
          "spec": "^1.1.6",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/fixtures/sax/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.1.6.tgz",
      "version": "1.1.6",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "resolve-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/fixtures/sax/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-1.0.1.tgz",
      "version": "1.0.1",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "exit-hook" => EdgeOut {
          "name": "exit-hook",
          "spec": "^1.0.0",
          "to": "node_modules/exit-hook",
          "type": "prod",
        },
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^1.0.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/fixtures/sax/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rimraf" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf",
              "name": "glob",
              "spec": "^5.0.14",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "2 || 3",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/rimraf/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-5.0.15.tgz",
          "version": "5.0.15",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "rimraf",
          "spec": "^2.2.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^5.0.14",
          "to": "node_modules/rimraf/node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/fixtures/sax/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.4.3.tgz",
      "version": "2.4.3",
    },
    "run-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "run-async",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/run-async",
      "name": "run-async",
      "path": "{CWD}/test/fixtures/sax/node_modules/run-async",
      "resolved": "https://registry.npmjs.org/run-async/-/run-async-0.1.0.tgz",
      "version": "0.1.0",
    },
    "run-parallel" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "run-parallel",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/run-parallel",
      "name": "run-parallel",
      "path": "{CWD}/test/fixtures/sax/node_modules/run-parallel",
      "resolved": "https://registry.npmjs.org/run-parallel/-/run-parallel-1.1.6.tgz",
      "version": "1.1.6",
    },
    "rx-lite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "rx-lite",
          "spec": "^3.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/rx-lite",
      "name": "rx-lite",
      "path": "{CWD}/test/fixtures/sax/node_modules/rx-lite",
      "resolved": "https://registry.npmjs.org/rx-lite/-/rx-lite-3.1.2.tgz",
      "version": "3.1.2",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.1.tgz",
      "version": "5.1.1",
    },
    "shelljs" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs",
              "name": "glob",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/shelljs/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/fixtures/sax/node_modules/shelljs/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/shelljs/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "shelljs",
          "spec": "^0.7.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/shelljs/node_modules/glob",
          "type": "prod",
        },
        "interpret" => EdgeOut {
          "name": "interpret",
          "spec": "^1.0.0",
          "to": "node_modules/interpret",
          "type": "prod",
        },
        "rechoir" => EdgeOut {
          "name": "rechoir",
          "spec": "^0.6.2",
          "to": "node_modules/rechoir",
          "type": "prod",
        },
      },
      "location": "node_modules/shelljs",
      "name": "shelljs",
      "path": "{CWD}/test/fixtures/sax/node_modules/shelljs",
      "resolved": "https://registry.npmjs.org/shelljs/-/shelljs-0.7.6.tgz",
      "version": "0.7.6",
    },
    "slice-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "slice-ansi",
          "spec": "0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-0.0.4.tgz",
      "version": "0.0.4",
    },
    "sntp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "sntp",
          "spec": "1.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/sntp",
      "name": "sntp",
      "path": "{CWD}/test/fixtures/sax/node_modules/sntp",
      "resolved": "https://registry.npmjs.org/sntp/-/sntp-1.0.9.tgz",
      "version": "1.0.9",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.5.6",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/fixtures/sax/node_modules/source-map-support/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.6.tgz",
          "version": "0.5.6",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/fixtures/sax/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.4.15.tgz",
      "version": "0.4.15",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/fixtures/sax/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/sshpk",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/sshpk/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/fixtures/sax/node_modules/sshpk/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/sshpk/node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "optional",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "optional",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jodid25519" => EdgeOut {
          "name": "jodid25519",
          "spec": "^1.0.0",
          "to": "node_modules/jodid25519",
          "type": "optional",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "optional",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "optional",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/fixtures/sax/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.10.2.tgz",
      "version": "1.10.2",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/fixtures/sax/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.0.tgz",
      "version": "1.0.0",
    },
    "standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "standard",
          "spec": "^8.6.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "~3.10.2",
          "to": "node_modules/eslint",
          "type": "prod",
        },
        "eslint-config-standard" => EdgeOut {
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "to": "node_modules/eslint-config-standard",
          "type": "prod",
        },
        "eslint-config-standard-jsx" => EdgeOut {
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "to": "node_modules/eslint-config-standard-jsx",
          "type": "prod",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "prod",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "prod",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "to": "node_modules/eslint-plugin-standard",
          "type": "prod",
        },
        "standard-engine" => EdgeOut {
          "name": "standard-engine",
          "spec": "~5.2.0",
          "to": "node_modules/standard-engine",
          "type": "prod",
        },
      },
      "location": "node_modules/standard",
      "name": "standard",
      "path": "{CWD}/test/fixtures/sax/node_modules/standard",
      "resolved": "https://registry.npmjs.org/standard/-/standard-8.6.0.tgz",
      "version": "8.6.0",
    },
    "standard-engine" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard-engine",
              "name": "minimist",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard-engine/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/fixtures/sax/node_modules/standard-engine/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "standard-engine",
          "spec": "~5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deglob" => EdgeOut {
          "name": "deglob",
          "spec": "^2.0.0",
          "to": "node_modules/deglob",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "get-stdin" => EdgeOut {
          "name": "get-stdin",
          "spec": "^5.0.1",
          "to": "node_modules/get-stdin",
          "type": "prod",
        },
        "home-or-tmp" => EdgeOut {
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "to": "node_modules/home-or-tmp",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.1.0",
          "to": "node_modules/standard-engine/node_modules/minimist",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.0.1",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
      },
      "location": "node_modules/standard-engine",
      "name": "standard-engine",
      "path": "{CWD}/test/fixtures/sax/node_modules/standard-engine",
      "resolved": "https://registry.npmjs.org/standard-engine/-/standard-engine-5.2.0.tgz",
      "version": "5.2.0",
    },
    "string_decoder" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "path": "{CWD}/test/fixtures/sax/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-0.10.31.tgz",
      "version": "0.10.31",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "string-width",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/fixtures/sax/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.1.tgz",
      "version": "1.0.1",
    },
    "stringstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "stringstream",
          "spec": "~0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/stringstream",
      "name": "stringstream",
      "path": "{CWD}/test/fixtures/sax/node_modules/stringstream",
      "resolved": "https://registry.npmjs.org/stringstream/-/stringstream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table/node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/fixtures/sax/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "strip-json-comments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-json-comments",
      "name": "strip-json-comments",
      "path": "{CWD}/test/fixtures/sax/node_modules/strip-json-comments",
      "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-1.0.4.tgz",
      "version": "1.0.4",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "supports-color",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/fixtures/sax/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
      "version": "2.0.0",
    },
    "table" => ArboristNode {
      "children": Map {
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "string-width",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/table/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/fixtures/sax/node_modules/table/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "table",
          "spec": "^3.7.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^4.7.0",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/table/node_modules/lodash",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "0.0.4",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.0.0",
          "to": "node_modules/table/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/table",
      "name": "table",
      "path": "{CWD}/test/fixtures/sax/node_modules/table",
      "resolved": "https://registry.npmjs.org/table/-/table-3.8.3.tgz",
      "version": "3.8.3",
    },
    "tap" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/inherits",
          "name": "inherits",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/isarray",
          "name": "isarray",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "nyc" => ArboristNode {
          "bundleDependencies": Array [
            "archy",
            "arrify",
            "caching-transform",
            "convert-source-map",
            "debug-log",
            "default-require-extensions",
            "find-cache-dir",
            "find-up",
            "foreground-child",
            "glob",
            "istanbul-lib-coverage",
            "istanbul-lib-hook",
            "istanbul-lib-instrument",
            "istanbul-lib-report",
            "istanbul-lib-source-maps",
            "istanbul-reports",
            "md5-hex",
            "merge-source-map",
            "micromatch",
            "mkdirp",
            "resolve-from",
            "rimraf",
            "signal-exit",
            "spawn-wrap",
            "test-exclude",
            "yargs",
            "yargs-parser",
          ],
          "children": Map {
            "align-text" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "longest" => EdgeOut {
                  "name": "longest",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/longest",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/align-text",
              "name": "align-text",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/align-text",
              "version": "0.1.4",
            },
            "amdefine" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "amdefine",
                  "spec": ">=0.0.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "name": "amdefine",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "version": "1.0.1",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "version": "2.2.1",
            },
            "append-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "default-require-extensions" => EdgeOut {
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "name": "append-transform",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "version": "0.4.0",
            },
            "archy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "archy",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "name": "archy",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/archy",
              "version": "1.0.0",
            },
            "arr-diff" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-flatten" => EdgeOut {
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "name": "arr-diff",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "version": "2.0.0",
            },
            "arr-flatten" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "name": "arr-flatten",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "version": "1.0.3",
            },
            "array-unique" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "name": "array-unique",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "version": "0.2.1",
            },
            "arrify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "name": "arrify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/arrify",
              "version": "1.0.1",
            },
            "async" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "async",
                  "spec": "^1.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/async",
              "name": "async",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/async",
              "version": "1.5.2",
            },
            "babel-code-frame" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "chalk" => EdgeOut {
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "name": "babel-code-frame",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "version": "6.22.0",
            },
            "babel-generator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "detect-indent" => EdgeOut {
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "type": "prod",
                },
                "jsesc" => EdgeOut {
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "trim-right" => EdgeOut {
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "name": "babel-generator",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "version": "6.24.1",
            },
            "babel-messages" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "name": "babel-messages",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "version": "6.23.0",
            },
            "babel-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "core-js" => EdgeOut {
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/core-js",
                  "type": "prod",
                },
                "regenerator-runtime" => EdgeOut {
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "name": "babel-runtime",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "version": "6.23.0",
            },
            "babel-template" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "name": "babel-template",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "version": "6.24.1",
            },
            "babel-traverse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-code-frame" => EdgeOut {
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "type": "prod",
                },
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "globals" => EdgeOut {
                  "name": "globals",
                  "spec": "^9.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/globals",
                  "type": "prod",
                },
                "invariant" => EdgeOut {
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "name": "babel-traverse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "version": "6.24.1",
            },
            "babel-types" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "to-fast-properties" => EdgeOut {
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "name": "babel-types",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "version": "6.24.1",
            },
            "babylon" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babylon",
              "name": "babylon",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/babylon",
              "version": "6.17.2",
            },
            "balanced-match" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "name": "balanced-match",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "version": "0.4.2",
            },
            "brace-expansion" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "balanced-match" => EdgeOut {
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
                  "type": "prod",
                },
                "concat-map" => EdgeOut {
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "name": "brace-expansion",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "version": "1.1.7",
            },
            "braces" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "braces",
                  "spec": "^1.8.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "expand-range" => EdgeOut {
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "type": "prod",
                },
                "preserve" => EdgeOut {
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/preserve",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/braces",
              "name": "braces",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/braces",
              "version": "1.8.5",
            },
            "builtin-modules" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "name": "builtin-modules",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "version": "1.1.1",
            },
            "caching-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "caching-transform",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-hex" => EdgeOut {
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "write-file-atomic" => EdgeOut {
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "name": "caching-transform",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "version": "1.0.1",
            },
            "center-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
                "lazy-cache" => EdgeOut {
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/center-align",
              "name": "center-align",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/center-align",
              "version": "0.1.3",
            },
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "has-ansi" => EdgeOut {
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/chalk",
              "version": "1.1.3",
            },
            "cliui" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "wordwrap" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "name": "wordwrap",
                      "spec": "0.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "name": "wordwrap",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "version": "0.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "cliui",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "center-align" => EdgeOut {
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "type": "prod",
                },
                "right-align" => EdgeOut {
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cliui",
              "name": "cliui",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cliui",
              "version": "2.1.0",
            },
            "code-point-at" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "name": "code-point-at",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "version": "1.1.0",
            },
            "commondir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/commondir",
              "name": "commondir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/commondir",
              "version": "1.0.1",
            },
            "concat-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "name": "concat-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "version": "0.0.1",
            },
            "convert-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "convert-source-map",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "name": "convert-source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "version": "1.5.0",
            },
            "core-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/core-js",
              "name": "core-js",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/core-js",
              "version": "2.4.1",
            },
            "cross-spawn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "cross-spawn",
                  "spec": "^4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "lru-cache" => EdgeOut {
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.9",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "name": "cross-spawn",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "version": "4.0.2",
            },
            "debug" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "debug",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "debug",
                  "spec": "^2.6.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ms" => EdgeOut {
                  "name": "ms",
                  "spec": "2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ms",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug",
              "name": "debug",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/debug",
              "version": "2.6.8",
            },
            "debug-log" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "debug-log",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "name": "debug-log",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "version": "1.0.1",
            },
            "decamelize" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "name": "decamelize",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "version": "1.2.0",
            },
            "default-require-extensions" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "name": "default-require-extensions",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "version": "1.0.0",
            },
            "detect-indent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "repeating" => EdgeOut {
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "name": "detect-indent",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "version": "4.0.0",
            },
            "error-ex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-arrayish" => EdgeOut {
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "name": "error-ex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "version": "1.3.1",
            },
            "escape-string-regexp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "name": "escape-string-regexp",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "version": "1.0.5",
            },
            "esutils" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/esutils",
              "name": "esutils",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/esutils",
              "version": "2.0.2",
            },
            "execa" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "execa",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "get-stream" => EdgeOut {
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "type": "prod",
                },
                "is-stream" => EdgeOut {
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
                  "type": "prod",
                },
                "npm-run-path" => EdgeOut {
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "type": "prod",
                },
                "p-finally" => EdgeOut {
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "strip-eof" => EdgeOut {
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/execa",
              "name": "execa",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/execa",
              "version": "0.5.1",
            },
            "expand-brackets" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-posix-bracket" => EdgeOut {
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "name": "expand-brackets",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "version": "0.1.5",
            },
            "expand-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fill-range" => EdgeOut {
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "name": "expand-range",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "version": "1.8.2",
            },
            "extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/extglob",
              "name": "extglob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/extglob",
              "version": "0.3.2",
            },
            "filename-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "name": "filename-regex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "version": "2.0.1",
            },
            "fill-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "isobject" => EdgeOut {
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "type": "prod",
                },
                "randomatic" => EdgeOut {
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "name": "fill-range",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "version": "2.2.3",
            },
            "find-cache-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-cache-dir",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "commondir" => EdgeOut {
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/commondir",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "pkg-dir" => EdgeOut {
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "name": "find-cache-dir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "version": "0.1.1",
            },
            "find-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-up",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "find-up",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "locate-path" => EdgeOut {
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "name": "find-up",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/find-up",
              "version": "2.1.0",
            },
            "for-in" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-in",
              "name": "for-in",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/for-in",
              "version": "1.0.2",
            },
            "for-own" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-in" => EdgeOut {
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-in",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-own",
              "name": "for-own",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/for-own",
              "version": "0.1.5",
            },
            "foreground-child" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "foreground-child",
                  "spec": "^1.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "name": "foreground-child",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "version": "1.5.6",
            },
            "fs.realpath" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "name": "fs.realpath",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "version": "1.0.0",
            },
            "get-caller-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "name": "get-caller-file",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "version": "1.0.2",
            },
            "get-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "name": "get-stream",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "version": "2.3.1",
            },
            "glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "glob",
                  "spec": "^7.0.6",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "name": "glob",
                  "spec": "^7.0.5",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fs.realpath" => EdgeOut {
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
                  "type": "prod",
                },
                "inflight" => EdgeOut {
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inherits",
                  "type": "prod",
                },
                "minimatch" => EdgeOut {
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "type": "prod",
                },
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "path-is-absolute" => EdgeOut {
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "name": "glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob",
              "version": "7.1.2",
            },
            "glob-base" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-parent" => EdgeOut {
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "name": "glob-base",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "version": "0.3.0",
            },
            "glob-parent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "name": "glob-parent",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "version": "2.0.0",
            },
            "globals" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "globals",
                  "spec": "^9.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/globals",
              "name": "globals",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/globals",
              "version": "9.17.0",
            },
            "graceful-fs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "name": "graceful-fs",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "version": "4.1.11",
            },
            "handlebars" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "source-map" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                      "name": "source-map",
                      "spec": "^0.4.4",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "amdefine" => EdgeOut {
                      "name": "amdefine",
                      "spec": ">=0.0.4",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "source-map",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "version": "0.4.4",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "async" => EdgeOut {
                  "name": "async",
                  "spec": "^1.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/async",
                  "type": "prod",
                },
                "optimist" => EdgeOut {
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.4.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-js" => EdgeOut {
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "name": "handlebars",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "version": "4.0.10",
            },
            "has-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "name": "has-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "version": "2.0.0",
            },
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "version": "1.0.0",
            },
            "hosted-git-info" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "name": "hosted-git-info",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "version": "2.4.2",
            },
            "imurmurhash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "name": "imurmurhash",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "version": "0.1.4",
            },
            "inflight" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inflight",
              "name": "inflight",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/inflight",
              "version": "1.0.6",
            },
            "inherits" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inherits",
                  "spec": "2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inherits",
              "name": "inherits",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/inherits",
              "version": "2.0.3",
            },
            "invariant" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "loose-envify" => EdgeOut {
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invariant",
              "name": "invariant",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/invariant",
              "version": "2.2.2",
            },
            "invert-kv" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "name": "invert-kv",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "version": "1.0.0",
            },
            "is-arrayish" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "name": "is-arrayish",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "version": "0.2.1",
            },
            "is-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "name": "is-buffer",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "version": "1.1.5",
            },
            "is-builtin-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "builtin-modules" => EdgeOut {
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "name": "is-builtin-module",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "version": "1.0.0",
            },
            "is-dotfile" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "name": "is-dotfile",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "version": "1.0.3",
            },
            "is-equal-shallow" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "name": "is-equal-shallow",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "version": "0.1.3",
            },
            "is-extendable" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "name": "is-extendable",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "version": "0.1.1",
            },
            "is-extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "name": "is-extglob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "version": "1.0.0",
            },
            "is-finite" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "name": "is-finite",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "version": "1.0.2",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "version": "1.0.0",
            },
            "is-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "name": "is-glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "version": "2.0.1",
            },
            "is-number" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-number",
              "name": "is-number",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-number",
              "version": "2.1.0",
            },
            "is-posix-bracket" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "name": "is-posix-bracket",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "version": "0.1.1",
            },
            "is-primitive" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "name": "is-primitive",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "version": "2.0.0",
            },
            "is-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "name": "is-stream",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "version": "1.1.0",
            },
            "is-utf8" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "name": "is-utf8",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "version": "0.2.1",
            },
            "isarray" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "name": "isarray",
                  "spec": "1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isarray",
              "name": "isarray",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isarray",
              "version": "1.0.0",
            },
            "isexe" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isexe",
              "name": "isexe",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isexe",
              "version": "2.0.0",
            },
            "isobject" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isarray",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isobject",
              "name": "isobject",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/isobject",
              "version": "2.1.0",
            },
            "istanbul-lib-coverage" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "name": "istanbul-lib-coverage",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "version": "1.1.1",
            },
            "istanbul-lib-hook" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-hook",
                  "spec": "^1.0.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "append-transform" => EdgeOut {
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "name": "istanbul-lib-hook",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "version": "1.0.7",
            },
            "istanbul-lib-instrument" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-instrument",
                  "spec": "^1.7.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-generator" => EdgeOut {
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "type": "prod",
                },
                "babel-template" => EdgeOut {
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "name": "istanbul-lib-instrument",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "version": "1.7.2",
            },
            "istanbul-lib-report" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "supports-color" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                      "name": "supports-color",
                      "spec": "^3.1.2",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "has-flag" => EdgeOut {
                      "name": "has-flag",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "supports-color",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "version": "3.2.3",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-report",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "path-parse" => EdgeOut {
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^3.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "name": "istanbul-lib-report",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "version": "1.1.1",
            },
            "istanbul-lib-source-maps" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-source-maps",
                  "spec": "^1.2.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.6.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "name": "istanbul-lib-source-maps",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "version": "1.2.1",
            },
            "istanbul-reports" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-reports",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "handlebars" => EdgeOut {
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "name": "istanbul-reports",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "version": "1.1.1",
            },
            "js-tokens" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "name": "js-tokens",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "version": "3.0.1",
            },
            "jsesc" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "name": "jsesc",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "version": "1.3.0",
            },
            "kind-of" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-buffer" => EdgeOut {
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "version": "3.2.2",
            },
            "lazy-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "name": "lazy-cache",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "version": "1.0.4",
            },
            "lcid" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "invert-kv" => EdgeOut {
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lcid",
              "name": "lcid",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lcid",
              "version": "1.0.0",
            },
            "load-json-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "parse-json" => EdgeOut {
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "name": "load-json-file",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "version": "1.1.0",
            },
            "locate-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "path-exists" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                      "name": "path-exists",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "name": "path-exists",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-locate" => EdgeOut {
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "type": "prod",
                },
                "path-exists" => EdgeOut {
                  "name": "path-exists",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "name": "locate-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "version": "2.0.0",
            },
            "lodash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lodash",
              "name": "lodash",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lodash",
              "version": "4.17.4",
            },
            "longest" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "longest",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/longest",
              "name": "longest",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/longest",
              "version": "1.0.1",
            },
            "loose-envify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "name": "loose-envify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "version": "1.3.1",
            },
            "lru-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pseudomap" => EdgeOut {
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
                  "type": "prod",
                },
                "yallist" => EdgeOut {
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yallist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "name": "lru-cache",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "version": "4.0.2",
            },
            "md5-hex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-o-matic" => EdgeOut {
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "name": "md5-hex",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "version": "1.3.0",
            },
            "md5-o-matic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "name": "md5-o-matic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "version": "0.1.1",
            },
            "mem" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "mem",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mem",
              "name": "mem",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mem",
              "version": "1.1.0",
            },
            "merge-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "merge-source-map",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "name": "merge-source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "version": "1.0.3",
            },
            "micromatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-diff" => EdgeOut {
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "type": "prod",
                },
                "array-unique" => EdgeOut {
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
                  "type": "prod",
                },
                "braces" => EdgeOut {
                  "name": "braces",
                  "spec": "^1.8.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "type": "prod",
                },
                "expand-brackets" => EdgeOut {
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "type": "prod",
                },
                "extglob" => EdgeOut {
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "type": "prod",
                },
                "filename-regex" => EdgeOut {
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "normalize-path" => EdgeOut {
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "type": "prod",
                },
                "object.omit" => EdgeOut {
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "type": "prod",
                },
                "parse-glob" => EdgeOut {
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "type": "prod",
                },
                "regex-cache" => EdgeOut {
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "name": "micromatch",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "version": "2.3.11",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "version": "1.1.0",
            },
            "minimatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "brace-expansion" => EdgeOut {
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "name": "minimatch",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "version": "3.0.4",
            },
            "minimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "name": "minimist",
                  "spec": "0.0.8",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimist",
              "name": "minimist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/minimist",
              "version": "0.0.8",
            },
            "mkdirp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "0.0.8",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "name": "mkdirp",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "version": "0.5.1",
            },
            "ms" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "name": "ms",
                  "spec": "2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ms",
              "name": "ms",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/ms",
              "version": "2.0.0",
            },
            "normalize-package-data" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "hosted-git-info" => EdgeOut {
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
                  "type": "prod",
                },
                "is-builtin-module" => EdgeOut {
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
                "validate-npm-package-license" => EdgeOut {
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "name": "normalize-package-data",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "version": "2.3.8",
            },
            "normalize-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "remove-trailing-separator" => EdgeOut {
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "name": "normalize-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "version": "2.1.1",
            },
            "npm-run-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "path-key" => EdgeOut {
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-key",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "name": "npm-run-path",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "version": "2.0.2",
            },
            "number-is-nan" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "name": "number-is-nan",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "version": "1.0.1",
            },
            "object-assign" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "name": "object-assign",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "version": "4.1.1",
            },
            "object.omit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-own" => EdgeOut {
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "type": "prod",
                },
                "is-extendable" => EdgeOut {
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "name": "object.omit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "version": "2.0.1",
            },
            "once" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/once",
              "name": "once",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/once",
              "version": "1.4.0",
            },
            "optimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/optimist",
              "name": "optimist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/optimist",
              "version": "0.6.1",
            },
            "os-homedir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "name": "os-homedir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "version": "1.0.2",
            },
            "os-locale" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "execa" => EdgeOut {
                  "name": "execa",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "type": "prod",
                },
                "lcid" => EdgeOut {
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "type": "prod",
                },
                "mem" => EdgeOut {
                  "name": "mem",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "name": "os-locale",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "version": "2.0.0",
            },
            "p-finally" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "name": "p-finally",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "version": "1.0.0",
            },
            "p-limit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "name": "p-limit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "version": "1.1.0",
            },
            "p-locate" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-limit" => EdgeOut {
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "name": "p-locate",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "version": "2.0.0",
            },
            "parse-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-base" => EdgeOut {
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "type": "prod",
                },
                "is-dotfile" => EdgeOut {
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "name": "parse-glob",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "version": "3.0.4",
            },
            "parse-json" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "error-ex" => EdgeOut {
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "name": "parse-json",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "version": "2.2.0",
            },
            "path-exists" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "name": "path-exists",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "version": "2.1.0",
            },
            "path-is-absolute" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "name": "path-is-absolute",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "version": "1.0.1",
            },
            "path-key" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-key",
              "name": "path-key",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-key",
              "version": "2.0.1",
            },
            "path-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "name": "path-parse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "version": "1.0.5",
            },
            "path-type" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-type",
              "name": "path-type",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/path-type",
              "version": "1.1.0",
            },
            "pify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pify",
              "name": "pify",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pify",
              "version": "2.3.0",
            },
            "pinkie" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "name": "pinkie",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "version": "2.0.4",
            },
            "pinkie-promise" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie" => EdgeOut {
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "name": "pinkie-promise",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "version": "2.0.1",
            },
            "pkg-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "name": "pkg-dir",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "version": "1.0.0",
            },
            "preserve" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/preserve",
              "name": "preserve",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/preserve",
              "version": "0.2.0",
            },
            "pseudomap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "name": "pseudomap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "version": "1.0.2",
            },
            "randomatic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "name": "randomatic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "version": "1.1.6",
            },
            "read-pkg" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "load-json-file" => EdgeOut {
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "type": "prod",
                },
                "normalize-package-data" => EdgeOut {
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "type": "prod",
                },
                "path-type" => EdgeOut {
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "name": "read-pkg",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "version": "1.1.0",
            },
            "read-pkg-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "type": "prod",
                },
                "read-pkg" => EdgeOut {
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "name": "read-pkg-up",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "version": "1.0.1",
            },
            "regenerator-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "name": "regenerator-runtime",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "version": "0.10.5",
            },
            "regex-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-equal-shallow" => EdgeOut {
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "type": "prod",
                },
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "name": "regex-cache",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "version": "0.4.3",
            },
            "remove-trailing-separator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "name": "remove-trailing-separator",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "version": "1.0.1",
            },
            "repeat-element" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "name": "repeat-element",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "version": "1.1.2",
            },
            "repeat-string" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "name": "repeat-string",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "version": "1.6.1",
            },
            "repeating" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-finite" => EdgeOut {
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeating",
              "name": "repeating",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/repeating",
              "version": "2.0.1",
            },
            "require-directory" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "name": "require-directory",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "version": "2.1.1",
            },
            "require-main-filename" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "name": "require-main-filename",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "version": "1.0.1",
            },
            "resolve-from" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "resolve-from",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "name": "resolve-from",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "version": "2.0.0",
            },
            "right-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/right-align",
              "name": "right-align",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/right-align",
              "version": "0.1.3",
            },
            "rimraf" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "rimraf",
                  "spec": "^2.5.4",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob" => EdgeOut {
                  "name": "glob",
                  "spec": "^7.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "name": "rimraf",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "version": "2.6.1",
            },
            "semver" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "semver",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/semver",
              "name": "semver",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/semver",
              "version": "5.3.0",
            },
            "set-blocking" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "name": "set-blocking",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "version": "2.0.0",
            },
            "signal-exit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "signal-exit",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "name": "signal-exit",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "version": "3.0.2",
            },
            "slide" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "slide",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/slide",
              "name": "slide",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/slide",
              "version": "1.1.6",
            },
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/source-map",
              "version": "0.5.6",
            },
            "spawn-wrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "spawn-wrap",
                  "spec": "^1.3.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "foreground-child" => EdgeOut {
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "os-homedir" => EdgeOut {
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "name": "spawn-wrap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "version": "1.3.6",
            },
            "spdx-correct" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-license-ids" => EdgeOut {
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "name": "spdx-correct",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "version": "1.0.2",
            },
            "spdx-expression-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "name": "spdx-expression-parse",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "version": "1.0.4",
            },
            "spdx-license-ids" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "name": "spdx-license-ids",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "version": "1.2.2",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "is-fullwidth-code-point" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                      "name": "is-fullwidth-code-point",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "name": "is-fullwidth-code-point",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "version": "2.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/string-width",
              "version": "2.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "strip-bom" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-utf8" => EdgeOut {
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "name": "strip-bom",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "version": "2.0.0",
            },
            "strip-eof" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "name": "strip-eof",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "version": "1.0.0",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "version": "2.0.0",
            },
            "test-exclude" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "test-exclude",
                  "spec": "^4.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arrify" => EdgeOut {
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
                  "type": "prod",
                },
                "micromatch" => EdgeOut {
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "type": "prod",
                },
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "name": "test-exclude",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "version": "4.1.1",
            },
            "to-fast-properties" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "name": "to-fast-properties",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "version": "1.0.3",
            },
            "trim-right" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "name": "trim-right",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "version": "1.0.1",
            },
            "uglify-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "name": "camelcase",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "version": "1.2.1",
                },
                "yargs" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                      "name": "yargs",
                      "spec": "~3.10.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                      "type": "prod",
                    },
                    "cliui" => EdgeOut {
                      "name": "cliui",
                      "spec": "^2.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "type": "prod",
                    },
                    "decamelize" => EdgeOut {
                      "name": "decamelize",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                      "type": "prod",
                    },
                    "window-size" => EdgeOut {
                      "name": "window-size",
                      "spec": "0.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/window-size",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "yargs",
                  "optional": true,
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "version": "3.10.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-to-browserify" => EdgeOut {
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
                  "type": "optional",
                },
                "yargs" => EdgeOut {
                  "name": "yargs",
                  "spec": "~3.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "name": "uglify-js",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "version": "2.8.27",
            },
            "uglify-to-browserify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "name": "uglify-to-browserify",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "version": "1.0.2",
            },
            "validate-npm-package-license" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-correct" => EdgeOut {
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "type": "prod",
                },
                "spdx-expression-parse" => EdgeOut {
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "name": "validate-npm-package-license",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "version": "3.0.1",
            },
            "which" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "which",
                  "spec": "^1.2.9",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "which",
                  "spec": "^1.2.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isexe" => EdgeOut {
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isexe",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which",
              "name": "which",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/which",
              "version": "1.2.14",
            },
            "which-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which-module",
              "name": "which-module",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/which-module",
              "version": "2.0.0",
            },
            "window-size" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "window-size",
                  "spec": "0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/window-size",
              "name": "window-size",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/window-size",
              "version": "0.1.0",
            },
            "wordwrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "name": "wordwrap",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "version": "0.0.3",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "string-width" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "code-point-at" => EdgeOut {
                      "name": "code-point-at",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                      "type": "prod",
                    },
                    "is-fullwidth-code-point" => EdgeOut {
                      "name": "is-fullwidth-code-point",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "string-width",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "version": "1.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "wrap-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "version": "2.1.0",
            },
            "wrappy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "name": "wrappy",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "version": "1.0.2",
            },
            "write-file-atomic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "imurmurhash" => EdgeOut {
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
                  "type": "prod",
                },
                "slide" => EdgeOut {
                  "name": "slide",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/slide",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "name": "write-file-atomic",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "version": "1.3.4",
            },
            "y18n" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/y18n",
              "name": "y18n",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/y18n",
              "version": "3.2.1",
            },
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yallist",
              "version": "2.1.2",
            },
            "yargs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "version": "4.1.0",
                },
                "cliui" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "children": Map {
                    "string-width" => ArboristNode {
                      "bundled": true,
                      "bundler": "node_modules/tap/node_modules/nyc",
                      "dev": true,
                      "edgesIn": Set {
                        EdgeIn {
                          "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                          "name": "string-width",
                          "spec": "^1.0.1",
                          "type": "prod",
                        },
                      },
                      "edgesOut": Map {
                        "code-point-at" => EdgeOut {
                          "name": "code-point-at",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                          "type": "prod",
                        },
                        "is-fullwidth-code-point" => EdgeOut {
                          "name": "is-fullwidth-code-point",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                          "type": "prod",
                        },
                        "strip-ansi" => EdgeOut {
                          "name": "strip-ansi",
                          "spec": "^3.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                          "type": "prod",
                        },
                      },
                      "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "name": "string-width",
                      "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "version": "1.0.2",
                    },
                  },
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "cliui",
                      "spec": "^3.2.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "string-width" => EdgeOut {
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                    "wrap-ansi" => EdgeOut {
                      "name": "wrap-ansi",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "cliui",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "version": "3.2.0",
                },
                "load-json-file" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "graceful-fs" => EdgeOut {
                      "name": "graceful-fs",
                      "spec": "^4.1.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                      "type": "prod",
                    },
                    "parse-json" => EdgeOut {
                      "name": "parse-json",
                      "spec": "^2.2.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                      "type": "prod",
                    },
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                    "strip-bom" => EdgeOut {
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "load-json-file",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "version": "2.0.0",
                },
                "path-type" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "path-type",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "version": "2.0.0",
                },
                "read-pkg" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "load-json-file" => EdgeOut {
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "type": "prod",
                    },
                    "normalize-package-data" => EdgeOut {
                      "name": "normalize-package-data",
                      "spec": "^2.3.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                      "type": "prod",
                    },
                    "path-type" => EdgeOut {
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "read-pkg",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "version": "2.0.0",
                },
                "read-pkg-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "read-pkg-up",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "find-up" => EdgeOut {
                      "name": "find-up",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                      "type": "prod",
                    },
                    "read-pkg" => EdgeOut {
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "read-pkg-up",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "version": "2.0.0",
                },
                "strip-bom" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "name": "strip-bom",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "version": "3.0.0",
                },
                "yargs-parser" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "yargs-parser",
                      "spec": "^7.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "name": "yargs-parser",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "version": "7.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs",
                  "spec": "^8.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "type": "prod",
                },
                "cliui" => EdgeOut {
                  "name": "cliui",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "type": "prod",
                },
                "decamelize" => EdgeOut {
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                  "type": "prod",
                },
                "get-caller-file" => EdgeOut {
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
                  "type": "prod",
                },
                "os-locale" => EdgeOut {
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-directory" => EdgeOut {
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
                "set-blocking" => EdgeOut {
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "type": "prod",
                },
                "which-module" => EdgeOut {
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which-module",
                  "type": "prod",
                },
                "y18n" => EdgeOut {
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/y18n",
                  "type": "prod",
                },
                "yargs-parser" => EdgeOut {
                  "name": "yargs-parser",
                  "spec": "^7.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "name": "yargs",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs",
              "version": "8.0.1",
            },
            "yargs-parser" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs-parser",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "name": "yargs-parser",
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "version": "5.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "nyc",
              "spec": "^11.0.2-candidate.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "archy" => EdgeOut {
              "name": "archy",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "type": "prod",
            },
            "caching-transform" => EdgeOut {
              "name": "caching-transform",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.3.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug-log" => EdgeOut {
              "name": "debug-log",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "type": "prod",
            },
            "default-require-extensions" => EdgeOut {
              "name": "default-require-extensions",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "type": "prod",
            },
            "find-cache-dir" => EdgeOut {
              "name": "find-cache-dir",
              "spec": "^0.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "type": "prod",
            },
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "type": "prod",
            },
            "foreground-child" => EdgeOut {
              "name": "foreground-child",
              "spec": "^1.5.3",
              "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "type": "prod",
            },
            "istanbul-lib-coverage" => EdgeOut {
              "name": "istanbul-lib-coverage",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "type": "prod",
            },
            "istanbul-lib-hook" => EdgeOut {
              "name": "istanbul-lib-hook",
              "spec": "^1.0.7",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "type": "prod",
            },
            "istanbul-lib-instrument" => EdgeOut {
              "name": "istanbul-lib-instrument",
              "spec": "^1.7.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "type": "prod",
            },
            "istanbul-lib-report" => EdgeOut {
              "name": "istanbul-lib-report",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "type": "prod",
            },
            "istanbul-lib-source-maps" => EdgeOut {
              "name": "istanbul-lib-source-maps",
              "spec": "^1.2.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "type": "prod",
            },
            "istanbul-reports" => EdgeOut {
              "name": "istanbul-reports",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "type": "prod",
            },
            "md5-hex" => EdgeOut {
              "name": "md5-hex",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "type": "prod",
            },
            "merge-source-map" => EdgeOut {
              "name": "merge-source-map",
              "spec": "^1.0.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "type": "prod",
            },
            "micromatch" => EdgeOut {
              "name": "micromatch",
              "spec": "^2.3.11",
              "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "type": "prod",
            },
            "mkdirp" => EdgeOut {
              "name": "mkdirp",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "type": "prod",
            },
            "rimraf" => EdgeOut {
              "name": "rimraf",
              "spec": "^2.5.4",
              "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "type": "prod",
            },
            "spawn-wrap" => EdgeOut {
              "name": "spawn-wrap",
              "spec": "^1.3.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "type": "prod",
            },
            "test-exclude" => EdgeOut {
              "name": "test-exclude",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "type": "prod",
            },
            "yargs" => EdgeOut {
              "name": "yargs",
              "spec": "^8.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "type": "prod",
            },
            "yargs-parser" => EdgeOut {
              "name": "yargs-parser",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/nyc",
          "name": "nyc",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/nyc",
          "resolved": "https://registry.npmjs.org/nyc/-/nyc-11.0.2.tgz",
          "version": "11.0.2",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
        "string_decoder" => ArboristNode {
          "children": Map {
            "safe-buffer" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string_decoder",
                  "name": "safe-buffer",
                  "spec": "~5.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "name": "safe-buffer",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.0.1.tgz",
              "version": "5.0.1",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "string_decoder",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.0.1",
              "to": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string_decoder",
          "name": "string_decoder",
          "optional": true,
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/string_decoder",
          "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.0.2.tgz",
          "version": "1.0.2",
        },
        "tap-mocha-reporter" => ArboristNode {
          "children": Map {
            "readable-stream" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/tap-mocha-reporter",
                  "name": "readable-stream",
                  "spec": "^2.1.5",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "core-util-is" => EdgeOut {
                  "name": "core-util-is",
                  "spec": "~1.0.0",
                  "to": "node_modules/core-util-is",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "~2.0.3",
                  "to": "node_modules/tap/node_modules/inherits",
                  "type": "prod",
                },
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/isarray",
                  "type": "prod",
                },
                "process-nextick-args" => EdgeOut {
                  "name": "process-nextick-args",
                  "spec": "~1.0.6",
                  "to": "node_modules/tap/node_modules/process-nextick-args",
                  "type": "prod",
                },
                "safe-buffer" => EdgeOut {
                  "name": "safe-buffer",
                  "spec": "~5.1.0",
                  "to": "node_modules/safe-buffer",
                  "type": "prod",
                },
                "string_decoder" => EdgeOut {
                  "name": "string_decoder",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/string_decoder",
                  "type": "prod",
                },
                "util-deprecate" => EdgeOut {
                  "name": "util-deprecate",
                  "spec": "~1.0.1",
                  "to": "node_modules/util-deprecate",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "readable-stream",
              "optional": true,
              "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.2.tgz",
              "version": "2.3.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-mocha-reporter",
              "spec": "^3.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-support" => EdgeOut {
              "name": "color-support",
              "spec": "^1.1.0",
              "to": "node_modules/color-support",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^2.1.3",
              "to": "node_modules/debug",
              "type": "prod",
            },
            "diff" => EdgeOut {
              "name": "diff",
              "spec": "^1.3.2",
              "to": "node_modules/diff",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.3",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.5",
              "to": "node_modules/glob",
              "type": "prod",
            },
            "js-yaml" => EdgeOut {
              "name": "js-yaml",
              "spec": "^3.3.1",
              "to": "node_modules/js-yaml",
              "type": "prod",
            },
            "readable-stream" => EdgeOut {
              "name": "readable-stream",
              "spec": "^2.1.5",
              "to": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "type": "optional",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^5.1.0",
              "to": "node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^1.0.0",
              "to": "node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-mocha-reporter",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tap-mocha-reporter",
          "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-3.0.5.tgz",
          "version": "3.0.5",
        },
        "tmatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tmatch",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tmatch",
          "name": "tmatch",
          "path": "{CWD}/test/fixtures/sax/node_modules/tap/node_modules/tmatch",
          "resolved": "https://registry.npmjs.org/tmatch/-/tmatch-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "^10.5.1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "bluebird" => EdgeOut {
          "name": "bluebird",
          "spec": "^3.3.1",
          "to": "node_modules/bluebird",
          "type": "prod",
        },
        "clean-yaml-object" => EdgeOut {
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "to": "node_modules/clean-yaml-object",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^2.11.2",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.1",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^1.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.3.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^11.0.2-candidate.0",
          "to": "node_modules/tap/node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.4.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.0",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.4.3",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.0",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^3.0.5",
          "to": "node_modules/tap/node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^5.3.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tmatch" => EdgeOut {
          "name": "tmatch",
          "spec": "^3.1.0",
          "to": "node_modules/tap/node_modules/tmatch",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "tsame" => EdgeOut {
          "name": "tsame",
          "spec": "^1.1.2",
          "to": "node_modules/tsame",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/fixtures/sax/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-10.5.1.tgz",
      "version": "10.5.1",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.2.7",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/fixtures/sax/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-5.3.2.tgz",
      "version": "5.3.2",
    },
    "text-table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "text-table",
          "spec": "~0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/text-table",
      "name": "text-table",
      "path": "{CWD}/test/fixtures/sax/node_modules/text-table",
      "resolved": "https://registry.npmjs.org/text-table/-/text-table-0.2.0.tgz",
      "version": "0.2.0",
    },
    "through" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "through",
          "spec": "^2.3.6",
          "type": "prod",
        },
      },
      "location": "node_modules/through",
      "name": "through",
      "path": "{CWD}/test/fixtures/sax/node_modules/through",
      "resolved": "https://registry.npmjs.org/through/-/through-2.3.8.tgz",
      "version": "2.3.8",
    },
    "tough-cookie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/fixtures/sax/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.3.2.tgz",
      "version": "2.3.2",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/fixtures/sax/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "tryit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-resolvable",
          "name": "tryit",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tryit",
      "name": "tryit",
      "path": "{CWD}/test/fixtures/sax/node_modules/tryit",
      "resolved": "https://registry.npmjs.org/tryit/-/tryit-1.0.3.tgz",
      "version": "1.0.3",
    },
    "tsame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tsame",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/tsame",
      "name": "tsame",
      "path": "{CWD}/test/fixtures/sax/node_modules/tsame",
      "resolved": "https://registry.npmjs.org/tsame/-/tsame-1.1.2.tgz",
      "version": "1.1.2",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/fixtures/sax/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.4.3.tgz",
      "version": "0.4.3",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "optional",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "optional": true,
      "path": "{CWD}/test/fixtures/sax/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "typedarray",
          "spec": "^0.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray",
      "name": "typedarray",
      "path": "{CWD}/test/fixtures/sax/node_modules/typedarray",
      "resolved": "https://registry.npmjs.org/typedarray/-/typedarray-0.0.6.tgz",
      "version": "0.0.6",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/fixtures/sax/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/fixtures/sax/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/uniq",
      "name": "uniq",
      "path": "{CWD}/test/fixtures/sax/node_modules/uniq",
      "resolved": "https://registry.npmjs.org/uniq/-/uniq-1.0.1.tgz",
      "version": "1.0.1",
    },
    "user-home" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "user-home",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
      },
      "location": "node_modules/user-home",
      "name": "user-home",
      "path": "{CWD}/test/fixtures/sax/node_modules/user-home",
      "resolved": "https://registry.npmjs.org/user-home/-/user-home-2.0.0.tgz",
      "version": "2.0.0",
    },
    "util-deprecate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "path": "{CWD}/test/fixtures/sax/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.1.tgz",
      "version": "1.0.1",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/sax/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.0.1.tgz",
      "version": "3.0.1",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.3.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/fixtures/sax/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.3.6.tgz",
      "version": "1.3.6",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "wordwrap",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/fixtures/sax/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-1.0.0.tgz",
      "version": "1.0.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/sax/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.1.tgz",
      "version": "1.0.1",
    },
    "write" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "write",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/write",
      "name": "write",
      "path": "{CWD}/test/fixtures/sax/node_modules/write",
      "resolved": "https://registry.npmjs.org/write/-/write-0.2.1.tgz",
      "version": "0.2.1",
    },
    "xtend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/xtend",
      "name": "xtend",
      "path": "{CWD}/test/fixtures/sax/node_modules/xtend",
      "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/fixtures/sax/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.0.0.tgz",
      "version": "2.0.0",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/fixtures/sax/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "standard" => EdgeOut {
      "name": "standard",
      "spec": "^8.6.0",
      "to": "node_modules/standard",
      "type": "dev",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "^10.5.1",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "sax",
  "path": "{CWD}/test/fixtures/sax",
  "version": "1.2.4",
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should allow adding a workspace as a dep to a workspace > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "workspace-a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "workspace-a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/workspace-a",
      "name": "workspace-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/node_modules/workspace-a",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-a",
      "resolved": "file:../workspace-a",
      "target": ArboristNode {
        "location": "workspace-a",
      },
      "version": "1.0.0",
    },
    "workspace-b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "workspace-b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "workspace-a",
          "name": "workspace-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/workspace-b",
      "name": "workspace-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/node_modules/workspace-b",
      "realpath": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-b",
      "resolved": "file:../workspace-b",
      "target": ArboristNode {
        "location": "workspace-b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "workspace-a" => EdgeOut {
      "name": "workspace-a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-a",
      "to": "node_modules/workspace-a",
      "type": "workspace",
    },
    "workspace-b" => EdgeOut {
      "name": "workspace-b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-b",
      "to": "node_modules/workspace-b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "workspace-b" => EdgeOut {
          "name": "workspace-b",
          "spec": "*",
          "to": "node_modules/workspace-b",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "workspace-a",
      "name": "workspace-a",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "workspace-b",
      "name": "workspace-b",
      "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace/workspace-b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace",
  "packageName": "root",
  "path": "{CWD}/test/arborist/tap-testdir-build-ideal-tree-workspaces-should-allow-adding-a-workspace-as-a-dep-to-a-workspace",
  "workspaces": Map {
    "workspace-a" => "workspace-a",
    "workspace-b" => "workspace-b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should ignore nested node_modules folders > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-ignore-nm/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-ignore-nm/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-ignore-nm/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-ignore-nm/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "c" => ArboristNode {
          "dev": true,
          "extraneous": true,
          "location": "packages/a/node_modules/c",
          "name": "c",
          "optional": true,
          "path": "{CWD}/test/fixtures/workspaces-ignore-nm/packages/a/node_modules/c",
          "peer": true,
          "version": "1.0.0",
        },
        "nested-workspace" => ArboristNode {
          "dev": true,
          "extraneous": true,
          "location": "packages/a/node_modules/nested-workspace",
          "name": "nested-workspace",
          "optional": true,
          "path": "{CWD}/test/fixtures/workspaces-ignore-nm/packages/a/node_modules/nested-workspace",
          "peer": true,
          "version": "1.0.0",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-ignore-nm/packages/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-ignore-nm",
  "packageName": "workspace-ignore-nm",
  "path": "{CWD}/test/fixtures/workspaces-ignore-nm",
  "workspaces": Map {
    "a" => "packages/a",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install a simple example > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-simple/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-simple/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-simple/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-simple/b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "a",
          "name": "b",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-simple/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-simple/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-simple/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-simple/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "b" => EdgeOut {
          "name": "b",
          "spec": "^1.0.0",
          "to": "node_modules/b",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-simple/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-simple/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-simple",
  "packageName": "workspace-simple",
  "path": "{CWD}/test/fixtures/workspaces-simple",
  "workspaces": Map {
    "a" => "a",
    "b" => "b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install a simple scoped pkg example > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@ruyadorno/scoped-a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@ruyadorno/scoped-a",
          "spec": "file:{CWD}/test/fixtures/workspaces-scoped-pkg/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/@ruyadorno/scoped-a",
      "name": "@ruyadorno/scoped-a",
      "path": "{CWD}/test/fixtures/workspaces-scoped-pkg/node_modules/@ruyadorno/scoped-a",
      "realpath": "{CWD}/test/fixtures/workspaces-scoped-pkg/packages/a",
      "resolved": "file:../../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "@ruyadorno/scoped-b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@ruyadorno/scoped-b",
          "spec": "file:{CWD}/test/fixtures/workspaces-scoped-pkg/packages/b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "@ruyadorno/scoped-b",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/@ruyadorno/scoped-b",
      "name": "@ruyadorno/scoped-b",
      "path": "{CWD}/test/fixtures/workspaces-scoped-pkg/node_modules/@ruyadorno/scoped-b",
      "realpath": "{CWD}/test/fixtures/workspaces-scoped-pkg/packages/b",
      "resolved": "file:../../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@ruyadorno/scoped-a" => EdgeOut {
      "name": "@ruyadorno/scoped-a",
      "spec": "file:{CWD}/test/fixtures/workspaces-scoped-pkg/packages/a",
      "to": "node_modules/@ruyadorno/scoped-a",
      "type": "workspace",
    },
    "@ruyadorno/scoped-b" => EdgeOut {
      "name": "@ruyadorno/scoped-b",
      "spec": "file:{CWD}/test/fixtures/workspaces-scoped-pkg/packages/b",
      "to": "node_modules/@ruyadorno/scoped-b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@ruyadorno/scoped-b" => EdgeOut {
          "name": "@ruyadorno/scoped-b",
          "spec": "^1.0.0",
          "to": "node_modules/@ruyadorno/scoped-b",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "packageName": "@ruyadorno/scoped-a",
      "path": "{CWD}/test/fixtures/workspaces-scoped-pkg/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "packageName": "@ruyadorno/scoped-b",
      "path": "{CWD}/test/fixtures/workspaces-scoped-pkg/packages/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-scoped-pkg",
  "packageName": "workspace-simple",
  "path": "{CWD}/test/fixtures/workspaces-scoped-pkg",
  "workspaces": Map {
    "@ruyadorno/scoped-a" => "packages/a",
    "@ruyadorno/scoped-b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install conflicting dep versions > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-conflicting-versions/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-conflicting-versions/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-conflicting-versions/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-conflicting-versions/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "abbrev",
              "spec": "=1.0.4",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/a/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "=1.0.4",
          "to": "packages/a/node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b",
              "name": "abbrev",
              "spec": "=1.1.0",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/b/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.0.tgz",
          "version": "1.1.0",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "=1.1.0",
          "to": "packages/b/node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-conflicting-versions/packages/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-conflicting-versions",
  "packageName": "workspaces-conflicting-deps",
  "path": "{CWD}/test/fixtures/workspaces-conflicting-versions",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install from registry on version not satisfied > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "abbrev" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "file:{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/abbrev",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied/node_modules/abbrev",
      "realpath": "{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/abbrev",
      "resolved": "file:../packages/abbrev",
      "target": ArboristNode {
        "location": "packages/abbrev",
      },
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "file:{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/abbrev",
      "to": "node_modules/abbrev",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "abbrev",
              "spec": "=1.1.1",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/a/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
          "version": "1.1.1",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "=1.1.1",
          "to": "packages/a/node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied/packages/abbrev",
      "version": "2.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-version-unsatisfied",
  "path": "{CWD}/test/fixtures/workspaces-version-unsatisfied",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
    "abbrev" => "packages/abbrev",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install shared dependencies into root folder > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-shared-deps/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/b",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-shared-deps/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.0.0",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/node_modules/c",
      "realpath": "{CWD}/test/fixtures/workspaces-shared-deps/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.0.0",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/c",
          "name": "uuid",
          "spec": "=3.3.3",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.3.tgz",
      "version": "3.3.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/fixtures/workspaces-shared-deps/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.1.1",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.1.1",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/packages/b",
      "version": "1.0.0",
    },
    ArboristNode {
      "edgesOut": Map {
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "=3.3.3",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-shared-deps/packages/c",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-shared-deps",
  "path": "{CWD}/test/fixtures/workspaces-shared-deps",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should install workspace transitive dependencies > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-transitive-deps/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-transitive-deps/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-transitive-deps/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a",
          "name": "once",
          "spec": "^1.4.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/fixtures/workspaces-transitive-deps/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/fixtures/workspaces-transitive-deps/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-transitive-deps/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.4.0",
          "to": "node_modules/once",
          "type": "dev",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-transitive-deps/packages/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-transitive-deps",
  "path": "{CWD}/test/fixtures/workspaces-transitive-deps",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should link top level nested workspaces > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-top-level-link/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-top-level-link/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-top-level-link/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-top-level-link/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-top-level-link/packages/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-top-level-link",
  "path": "{CWD}/test/fixtures/workspaces-top-level-link",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should prefer linking nested workspaces > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-prefer-linking/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-prefer-linking/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-prefer-linking/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "abbrev" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "file:{CWD}/test/fixtures/workspaces-prefer-linking/packages/abbrev",
          "type": "workspace",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-prefer-linking/node_modules/abbrev",
      "realpath": "{CWD}/test/fixtures/workspaces-prefer-linking/packages/abbrev",
      "resolved": "file:../packages/abbrev",
      "target": ArboristNode {
        "location": "packages/abbrev",
      },
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-prefer-linking/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "file:{CWD}/test/fixtures/workspaces-prefer-linking/packages/abbrev",
      "to": "node_modules/abbrev",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-prefer-linking/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/fixtures/workspaces-prefer-linking/packages/abbrev",
      "version": "2.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-prefer-linking",
  "path": "{CWD}/test/fixtures/workspaces-prefer-linking",
  "version": "1.0.0",
  "workspaces": Map {
    "a" => "packages/a",
    "abbrev" => "packages/abbrev",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should update a simple example > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-simple/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-simple/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-simple/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-simple/b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "a",
          "name": "b",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-simple/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-simple/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-simple/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-simple/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "b" => EdgeOut {
          "name": "b",
          "spec": "^1.0.0",
          "to": "node_modules/b",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-simple/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-simple/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-simple",
  "packageName": "workspace-simple",
  "path": "{CWD}/test/fixtures/workspaces-simple",
  "workspaces": Map {
    "a" => "a",
    "b" => "b",
  },
}
`

exports[`test/arborist/build-ideal-tree.js TAP workspaces should work with files spec > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/fixtures/workspaces-with-files-spec/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/node_modules/a",
      "realpath": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/fixtures/workspaces-with-files-spec/packages/b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "d",
          "name": "b",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/node_modules/b",
      "realpath": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.0.0",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:packages/c",
          "type": "prod",
        },
      },
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/node_modules/c",
      "realpath": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.0.0",
    },
    "d" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "d",
          "spec": "file:d",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "d",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/d",
      "name": "d",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/node_modules/d",
      "realpath": "{CWD}/test/fixtures/workspaces-with-files-spec/d",
      "resolved": "file:../d",
      "target": ArboristNode {
        "location": "d",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/fixtures/workspaces-with-files-spec/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/fixtures/workspaces-with-files-spec/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:packages/c",
      "to": "node_modules/c",
      "type": "prod",
    },
    "d" => EdgeOut {
      "name": "d",
      "spec": "file:d",
      "to": "node_modules/d",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "b" => EdgeOut {
          "name": "b",
          "spec": "^1.0.0",
          "to": "node_modules/b",
          "type": "prod",
        },
      },
      "location": "d",
      "name": "d",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/d",
      "version": "1.0.0",
    },
    ArboristNode {
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^1.0.0",
          "to": "node_modules/d",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/b",
      "version": "1.0.0",
    },
    ArboristNode {
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/fixtures/workspaces-with-files-spec/packages/c",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "workspaces-with-files-spec",
  "path": "{CWD}/test/fixtures/workspaces-with-files-spec",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`
