// generated from test/fixtures/audit-fix-old-tap
module.exports = t => {
  const path = t.testdir({
  "audit.json": JSON.stringify({
    "actions": [
      {
        "isMajor": true,
        "action": "install",
        "resolves": [
          {
            "id": 1179,
            "path": "mkdirp>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "mkdirp",
        "target": "1.0.4"
      },
      {
        "isMajor": true,
        "action": "install",
        "resolves": [
          {
            "id": 1179,
            "path": "tap>chokidar>fsevents>node-pre-gyp>mkdirp>minimist",
            "dev": false,
            "optional": true,
            "bundled": true
          },
          {
            "id": 1179,
            "path": "tap>chokidar>fsevents>node-pre-gyp>tar>mkdirp>minimist",
            "dev": false,
            "optional": true,
            "bundled": true
          },
          {
            "id": 1179,
            "path": "tap>chokidar>fsevents>node-pre-gyp>rc>minimist",
            "dev": false,
            "optional": true,
            "bundled": true
          }
        ],
        "module": "tap",
        "target": "14.10.7"
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>braces>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>cache-base>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "set-value",
        "target": "2.0.1",
        "depth": 10
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>braces>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1012,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>cache-base>union-value>set-value",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "union-value",
        "target": "1.0.1",
        "depth": 10
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1013,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>braces>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1013,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>mixin-deep",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "mixin-deep",
        "target": "1.3.2",
        "depth": 9
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-generator>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-generator>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-template>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-template>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-traverse>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-template>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-template>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-plugin-transform-react-jsx>babel-helper-builder-react-jsx>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-plugin-transform-react-jsx>babel-helper-builder-react-jsx>babel-types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-generator>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-generator>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-template>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-template>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-helpers>babel-template>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-template>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-template>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>babel-register>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>babel-register>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>import-jsx>babel-core>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>treport>import-jsx>babel-core>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/generator>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/generator>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/template>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/template>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/helper-get-function-arity>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-split-export-declaration>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/types>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/generator>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/generator>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1065,
            "path": "tap>nyc>istanbul-lib-instrument>@babel/traverse>lodash",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "lodash",
        "target": "4.17.15",
        "depth": 9
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1164,
            "path": "tap>nyc>istanbul-reports>handlebars",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1300,
            "path": "tap>nyc>istanbul-reports>handlebars",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1316,
            "path": "tap>nyc>istanbul-reports>handlebars",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1324,
            "path": "tap>nyc>istanbul-reports>handlebars",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1325,
            "path": "tap>nyc>istanbul-reports>handlebars",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "handlebars",
        "target": "4.7.6",
        "depth": 4
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1179,
            "path": "tap>import-jsx>babel-core>babel-register>mkdirp>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1179,
            "path": "tap>treport>import-jsx>babel-core>babel-register>mkdirp>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1179,
            "path": "tap>mkdirp>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "mkdirp",
        "target": "0.5.5",
        "depth": 6
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1179,
            "path": "tap>nyc>spawn-wrap>mkdirp>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1179,
            "path": "tap>nyc>istanbul-reports>handlebars>optimist>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "nyc",
        "target": "14.1.1",
        "depth": 2
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1179,
            "path": "tap>coveralls>minimist",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "minimist",
        "target": "1.2.5",
        "depth": 3
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>is-accessor-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>is-data-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>braces>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>anymatch>micromatch>nanomatch>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1490,
            "path": "tap>chokidar>readdirp>micromatch>nanomatch>kind-of",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "kind-of",
        "target": "6.0.3",
        "depth": 12
      },
      {
        "action": "update",
        "resolves": [
          {
            "id": 1500,
            "path": "tap>nyc>yargs>yargs-parser",
            "dev": false,
            "optional": false,
            "bundled": false
          },
          {
            "id": 1500,
            "path": "tap>nyc>yargs-parser",
            "dev": false,
            "optional": false,
            "bundled": false
          }
        ],
        "module": "yargs-parser",
        "target": "13.1.2",
        "depth": 4
      }
    ],
    "advisories": {
      "1012": {
        "findings": [
          {
            "version": "2.0.0",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>cache-base>set-value",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>cache-base>set-value",
              "tap>chokidar>braces>snapdragon>base>cache-base>set-value",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>set-value",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>set-value",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>cache-base>set-value",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>cache-base>set-value",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>cache-base>set-value",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>cache-base>set-value",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>cache-base>set-value",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>cache-base>set-value"
            ]
          },
          {
            "version": "0.4.3",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>braces>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>cache-base>union-value>set-value",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>cache-base>union-value>set-value"
            ]
          }
        ],
        "id": 1012,
        "created": "2019-06-20T14:35:11.974Z",
        "updated": "2019-07-12T21:43:14.145Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "Jon Schlinkert"
        },
        "reported_by": {
          "link": "",
          "name": "Jon Schlinkert"
        },
        "module_name": "set-value",
        "cves": [
          "CVE-2019-10747"
        ],
        "vulnerable_versions": "<2.0.1 || >=3.0.0 <3.0.1",
        "patched_versions": ">=2.0.1 <3.0.0 || >=3.0.1",
        "overview": "Versions of `set-value` prior to 3.0.1 or 2.0.1 are vulnerable to Prototype Pollution. The `set` function fails to validate which Object properties it updates. This allows attackers to modify the prototype of Object, causing the addition or modification of an existing property on all objects.\n\n",
        "recommendation": "If you are using `set-value` 3.x, upgrade to version 3.0.1 or later.\nIf you are using `set-value` 2.x, upgrade to version 2.0.1 or later.\n",
        "references": "- [Snyk Report](https://snyk.io/vuln/SNYK-JS-SETVALUE-450213)",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 5,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1012"
      },
      "1013": {
        "findings": [
          {
            "version": "1.3.1",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>mixin-deep",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>mixin-deep",
              "tap>chokidar>braces>snapdragon>base>mixin-deep",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>mixin-deep",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>mixin-deep",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>mixin-deep",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>mixin-deep",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>mixin-deep",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>mixin-deep",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>mixin-deep",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>mixin-deep"
            ]
          }
        ],
        "id": 1013,
        "created": "2019-06-20T14:51:39.425Z",
        "updated": "2019-07-12T18:53:21.152Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "Snyk Security Team"
        },
        "reported_by": {
          "link": "",
          "name": "Snyk Security Team"
        },
        "module_name": "mixin-deep",
        "cves": [
          "CVE-2019-10746"
        ],
        "vulnerable_versions": "<1.3.2 || >=2.0.0 <2.0.1",
        "patched_versions": ">=1.3.2 <2.0.0 || >=2.0.1",
        "overview": "Versions of `mixin-deep` prior to 2.0.1 or 1.3.2 are vulnerable to Prototype Pollution. The `mixinDeep` function fails to validate which Object properties it updates. This allows attackers to modify the prototype of Object, causing the addition or modification of an existing property on all objects.\n\n",
        "recommendation": "If you are using `mixin-deep` 2.x, upgrade to version 2.0.1 or later.\nIf you are using `mixin-deep` 1.x, upgrade to version 1.3.2 or later.",
        "references": "- [Snyk Report](https://snyk.io/vuln/SNYK-JS-MIXINDEEP-450212)",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 5,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1013"
      },
      "1065": {
        "findings": [
          {
            "version": "4.17.11",
            "paths": [
              "tap>import-jsx>babel-core>babel-generator>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-generator>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-template>babel-traverse>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-template>babel-traverse>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-traverse>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-traverse>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-template>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-template>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-types>lodash",
              "tap>treport>import-jsx>babel-core>babel-types>lodash",
              "tap>import-jsx>babel-plugin-transform-react-jsx>babel-helper-builder-react-jsx>babel-types>lodash",
              "tap>treport>import-jsx>babel-plugin-transform-react-jsx>babel-helper-builder-react-jsx>babel-types>lodash",
              "tap>import-jsx>babel-core>babel-generator>lodash",
              "tap>treport>import-jsx>babel-core>babel-generator>lodash",
              "tap>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>lodash",
              "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>babel-traverse>lodash",
              "tap>import-jsx>babel-core>babel-template>babel-traverse>lodash",
              "tap>treport>import-jsx>babel-core>babel-template>babel-traverse>lodash",
              "tap>import-jsx>babel-core>babel-traverse>lodash",
              "tap>treport>import-jsx>babel-core>babel-traverse>lodash",
              "tap>import-jsx>babel-core>babel-helpers>babel-template>lodash",
              "tap>treport>import-jsx>babel-core>babel-helpers>babel-template>lodash",
              "tap>import-jsx>babel-core>babel-template>lodash",
              "tap>treport>import-jsx>babel-core>babel-template>lodash",
              "tap>import-jsx>babel-core>babel-register>lodash",
              "tap>treport>import-jsx>babel-core>babel-register>lodash",
              "tap>import-jsx>babel-core>lodash",
              "tap>treport>import-jsx>babel-core>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/generator>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/generator>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/template>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/template>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/helper-get-function-arity>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-function-name>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/helper-split-export-declaration>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/types>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/generator>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>@babel/generator>lodash",
              "tap>nyc>istanbul-lib-instrument>@babel/traverse>lodash"
            ]
          }
        ],
        "id": 1065,
        "created": "2019-07-15T17:22:56.990Z",
        "updated": "2019-07-15T17:25:05.721Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "Snyk Security Team"
        },
        "reported_by": {
          "link": "",
          "name": "Snyk Security Team"
        },
        "module_name": "lodash",
        "cves": [
          "CVE-2019-10744"
        ],
        "vulnerable_versions": "<4.17.12",
        "patched_versions": ">=4.17.12",
        "overview": "Versions of `lodash` before 4.17.12 are vulnerable to Prototype Pollution.  The function `defaultsDeep` allows a malicious user to modify the prototype of `Object` via `{constructor: {prototype: {...}}}` causing the addition or modification of an existing property that will exist on all objects.\n\n",
        "recommendation": "Update to version 4.17.12 or later.",
        "references": "- [Snyk Advisory](https://snyk.io/vuln/SNYK-JS-LODASH-450202)",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 3,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1065"
      },
      "1164": {
        "findings": [
          {
            "version": "4.1.2",
            "paths": [
              "tap>nyc>istanbul-reports>handlebars"
            ]
          }
        ],
        "id": 1164,
        "created": "2019-09-16T15:14:43.509Z",
        "updated": "2020-02-26T17:57:03.589Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "itszn",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "itszn",
          "email": ""
        },
        "module_name": "handlebars",
        "cves": [
          "CVE-2019-19919"
        ],
        "vulnerable_versions": "<3.0.8 || >=4.0.0 <4.3.0",
        "patched_versions": ">=3.0.8 <4.0.0 || >=4.3.0",
        "overview": "Versions of `handlebars` prior to 3.0.8 or 4.3.0 are vulnerable to Prototype Pollution leading to Remote Code Execution. Templates may alter an Objects' `__proto__` and `__defineGetter__` properties, which may allow an attacker to execute arbitrary code through crafted payloads.",
        "recommendation": "Upgrade to version 3.0.8, 4.3.0 or later.",
        "references": "- [CVE](https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-19919)",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 6,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1164"
      },
      "1179": {
        "findings": [
          {
            "version": "0.0.8",
            "paths": [
              "mkdirp>minimist",
              "tap>import-jsx>babel-core>babel-register>mkdirp>minimist",
              "tap>treport>import-jsx>babel-core>babel-register>mkdirp>minimist",
              "tap>mkdirp>minimist",
              "tap>nyc>spawn-wrap>mkdirp>minimist",
              "tap>nyc>istanbul-reports>handlebars>optimist>minimist"
            ]
          },
          {
            "version": "0.0.8",
            "paths": [
              "tap>chokidar>fsevents>node-pre-gyp>mkdirp>minimist",
              "tap>chokidar>fsevents>node-pre-gyp>tar>mkdirp>minimist"
            ]
          },
          {
            "version": "1.2.0",
            "paths": [
              "tap>chokidar>fsevents>node-pre-gyp>rc>minimist"
            ]
          },
          {
            "version": "1.2.0",
            "paths": [
              "tap>coveralls>minimist"
            ]
          }
        ],
        "id": 1179,
        "created": "2019-09-23T15:01:43.049Z",
        "updated": "2020-03-18T19:41:45.921Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "https://www.checkmarx.com/resources/blog/",
          "name": "Checkmarx Research Team",
          "email": ""
        },
        "reported_by": {
          "link": "https://www.checkmarx.com/resources/blog/",
          "name": "Checkmarx Research Team",
          "email": ""
        },
        "module_name": "minimist",
        "cves": [],
        "vulnerable_versions": "<0.2.1 || >=1.0.0 <1.2.3",
        "patched_versions": ">=0.2.1 <1.0.0 || >=1.2.3",
        "overview": "Affected versions of `minimist` are vulnerable to prototype pollution. Arguments are not properly sanitized, allowing an attacker to modify the prototype of `Object`, causing the addition or modification of an existing property that will exist on all objects.  \nParsing the argument `--__proto__.y=Polluted` adds a `y` property with value `Polluted` to all objects. The argument `--__proto__=Polluted` raises and uncaught error and crashes the application.  \nThis is exploitable if attackers have control over the arguments being passed to `minimist`.\n",
        "recommendation": "Upgrade to versions 0.2.1, 1.2.3 or later.",
        "references": "- [GitHub commit 1](https://github.com/substack/minimist/commit/4cf1354839cb972e38496d35e12f806eea92c11f#diff-a1e0ee62c91705696ddb71aa30ad4f95)\n- [GitHub commit 2](https://github.com/substack/minimist/commit/63e7ed05aa4b1889ec2f3b196426db4500cbda94)",
        "access": "public",
        "severity": "low",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 1,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1179"
      },
      "1300": {
        "findings": [
          {
            "version": "4.1.2",
            "paths": [
              "tap>nyc>istanbul-reports>handlebars"
            ]
          }
        ],
        "id": 1300,
        "created": "2019-10-30T15:57:14.032Z",
        "updated": "2020-02-24T16:20:25.230Z",
        "deleted": null,
        "title": "Denial of Service",
        "found_by": {
          "link": "",
          "name": "Nils Knappmeier",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "Nils Knappmeier",
          "email": ""
        },
        "module_name": "handlebars",
        "cves": [],
        "vulnerable_versions": ">=4.0.0 <4.4.5",
        "patched_versions": ">=4.4.5",
        "overview": "Affected versions of `handlebars` are vulnerable to Denial of Service. The package's parser may be forced into an endless loop while processing specially-crafted templates. This may allow attackers to exhaust system resources leading to Denial of Service.",
        "recommendation": "Upgrade to version 4.4.5 or later.",
        "references": "",
        "access": "public",
        "severity": "moderate",
        "cwe": "CWE-400",
        "metadata": {
          "module_type": "",
          "exploitability": 5,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1300"
      },
      "1316": {
        "findings": [
          {
            "version": "4.1.2",
            "paths": [
              "tap>nyc>istanbul-reports>handlebars"
            ]
          }
        ],
        "id": 1316,
        "created": "2019-11-14T15:29:41.991Z",
        "updated": "2020-02-26T17:53:41.428Z",
        "deleted": null,
        "title": "Arbitrary Code Execution",
        "found_by": {
          "link": "",
          "name": "François Lajeunesse-Robert",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "François Lajeunesse-Robert",
          "email": ""
        },
        "module_name": "handlebars",
        "cves": [],
        "vulnerable_versions": "<3.0.8 || >=4.0.0 <4.5.2",
        "patched_versions": ">=3.0.8 <4.0.0 || >=4.5.2",
        "overview": "Versions of `handlebars` prior to 3.0.8 or 4.5.2 are vulnerable to Arbitrary Code Execution. The package's lookup helper fails to properly validate templates, allowing attackers to submit templates that execute arbitrary JavaScript in the system. It can be used to run arbitrary code in a server processing Handlebars templates or on a victim's browser (effectively serving as Cross-Site Scripting).\n\nThe following template can be used to demonstrate the vulnerability:  \n```{{#with \"constructor\"}}\n\t{{#with split as |a|}}\n\t\t{{pop (push \"alert('Vulnerable Handlebars JS');\")}}\n\t\t{{#with (concat (lookup join (slice 0 1)))}}\n\t\t\t{{#each (slice 2 3)}}\n\t\t\t\t{{#with (apply 0 a)}}\n\t\t\t\t\t{{.}}\n\t\t\t\t{{/with}}\n\t\t\t{{/each}}\n\t\t{{/with}}\n\t{{/with}}\n{{/with}}```",
        "recommendation": "Upgrade to version 3.0.8, 4.5.2 or later.",
        "references": "",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-79",
        "metadata": {
          "module_type": "",
          "exploitability": 6,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1316"
      },
      "1324": {
        "findings": [
          {
            "version": "4.1.2",
            "paths": [
              "tap>nyc>istanbul-reports>handlebars"
            ]
          }
        ],
        "id": 1324,
        "created": "2019-11-18T19:06:46.461Z",
        "updated": "2020-02-26T17:51:49.573Z",
        "deleted": null,
        "title": "Arbitrary Code Execution",
        "found_by": {
          "link": "",
          "name": "Unknown",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "Unknown",
          "email": ""
        },
        "module_name": "handlebars",
        "cves": [],
        "vulnerable_versions": "<3.0.8 || >=4.0.0 <4.5.3",
        "patched_versions": ">=3.0.8 <4.0.0 || >=4.5.3",
        "overview": "Versions of `handlebars` prior to 3.0.8 or 4.5.3 are vulnerable to Arbitrary Code Execution. The package's lookup helper fails to properly validate templates, allowing attackers to submit templates that execute arbitrary JavaScript in the system. It is due to an incomplete fix for a [previous issue](https://www.npmjs.com/advisories/1316). This vulnerability can be used to run arbitrary code in a server processing Handlebars templates or on a victim's browser (effectively serving as Cross-Site Scripting).",
        "recommendation": "Upgrade to version 3.0.8, 4.5.3 or later.",
        "references": "",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-79",
        "metadata": {
          "module_type": "",
          "exploitability": 4,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1324"
      },
      "1325": {
        "findings": [
          {
            "version": "4.1.2",
            "paths": [
              "tap>nyc>istanbul-reports>handlebars"
            ]
          }
        ],
        "id": 1325,
        "created": "2019-11-18T19:42:01.445Z",
        "updated": "2020-02-26T17:55:51.120Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "Vladyslav Babkin",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "Vladyslav Babkin",
          "email": ""
        },
        "module_name": "handlebars",
        "cves": [],
        "vulnerable_versions": "<3.0.8 || >=4.0.0 <4.5.3",
        "patched_versions": ">=3.0.8 <4.0.0 || >=4.5.3",
        "overview": "Versions of `handlebars` prior to 3.0.8 or 4.5.3 are vulnerable to prototype pollution. It is possible to add or modify properties to the Object prototype through a malicious template. This may allow attackers to crash the application or execute Arbitrary Code in specific conditions.",
        "recommendation": "Upgrade to version 3.0.8, 4.5.3 or later.",
        "references": "",
        "access": "public",
        "severity": "high",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 4,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1325"
      },
      "1490": {
        "findings": [
          {
            "version": "6.0.2",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>braces>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>is-accessor-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>is-accessor-descriptor>kind-of"
            ]
          },
          {
            "version": "6.0.2",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>braces>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>is-data-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>is-data-descriptor>kind-of"
            ]
          },
          {
            "version": "6.0.2",
            "paths": [
              "tap>chokidar>anymatch>micromatch>braces>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>snapdragon>base>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>snapdragon-node>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>snapdragon-node>define-property>is-descriptor>kind-of",
              "tap>chokidar>braces>snapdragon-node>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>braces>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>braces>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>braces>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>expand-brackets>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>to-regex>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>nanomatch>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>define-property>is-descriptor>kind-of",
              "tap>chokidar>anymatch>micromatch>extglob>define-property>is-descriptor>kind-of",
              "tap>chokidar>readdirp>micromatch>extglob>define-property>is-descriptor>kind-of"
            ]
          },
          {
            "version": "6.0.2",
            "paths": [
              "tap>chokidar>anymatch>micromatch>kind-of",
              "tap>chokidar>readdirp>micromatch>kind-of"
            ]
          },
          {
            "version": "6.0.2",
            "paths": [
              "tap>chokidar>anymatch>micromatch>nanomatch>kind-of",
              "tap>chokidar>readdirp>micromatch>nanomatch>kind-of"
            ]
          }
        ],
        "id": 1490,
        "created": "2020-03-06T21:22:12.305Z",
        "updated": "2020-03-09T14:12:20.619Z",
        "deleted": null,
        "title": "Validation Bypass",
        "found_by": {
          "link": "",
          "name": "Feng Xiao",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "Feng Xiao",
          "email": ""
        },
        "module_name": "kind-of",
        "cves": [],
        "vulnerable_versions": ">=6.0.0 <6.0.3",
        "patched_versions": ">=6.0.3",
        "overview": "Versions of `kind-of` 6.x prior to 6.0.3 are vulnerable to a Validation Bypass. A maliciously crafted object can alter the result of the type check, allowing attackers to bypass the type checking validation. ",
        "recommendation": "Upgrade to versions 6.0.3 or later.",
        "references": "- [GitHub issue](https://github.com/jonschlinkert/kind-of/issues/30)",
        "access": "public",
        "severity": "low",
        "cwe": "CWE-20",
        "metadata": {
          "module_type": "",
          "exploitability": 3,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1490"
      },
      "1500": {
        "findings": [
          {
            "version": "13.0.0",
            "paths": [
              "tap>nyc>yargs>yargs-parser",
              "tap>nyc>yargs-parser"
            ]
          }
        ],
        "id": 1500,
        "created": "2020-03-26T19:21:50.174Z",
        "updated": "2020-05-01T01:05:15.020Z",
        "deleted": null,
        "title": "Prototype Pollution",
        "found_by": {
          "link": "",
          "name": "Snyk Security Team",
          "email": ""
        },
        "reported_by": {
          "link": "",
          "name": "Snyk Security Team",
          "email": ""
        },
        "module_name": "yargs-parser",
        "cves": [],
        "vulnerable_versions": "<13.1.2 || >=14.0.0 <15.0.1 || >=16.0.0 <18.1.2",
        "patched_versions": ">=13.1.2 <14.0.0 || >=15.0.1 <16.0.0 || >=18.1.2",
        "overview": "Affected versions of `yargs-parser` are vulnerable to prototype pollution. Arguments are not properly sanitized, allowing an attacker to modify the prototype of `Object`, causing the addition or modification of an existing property that will exist on all objects.  \nParsing the argument `--foo.__proto__.bar baz'` adds a `bar` property with value `baz` to all objects. This is only exploitable if attackers have control over the arguments being passed to `yargs-parser`.\n",
        "recommendation": "Upgrade to versions 13.1.2, 15.0.1, 18.1.1 or later.",
        "references": "- [Snyk Report](https://snyk.io/vuln/SNYK-JS-YARGSPARSER-560381)",
        "access": "public",
        "severity": "low",
        "cwe": "CWE-471",
        "metadata": {
          "module_type": "",
          "exploitability": 1,
          "affected_components": ""
        },
        "url": "https://npmjs.com/advisories/1500"
      }
    },
    "muted": [],
    "metadata": {
      "vulnerabilities": {
        "info": 0,
        "low": 109,
        "moderate": 1,
        "high": 81,
        "critical": 0
      },
      "dependencies": 3312,
      "devDependencies": 0,
      "optionalDependencies": 100,
      "totalDependencies": 3412
    },
    "runId": "2a39f254-0964-4e87-8bbf-c4c806b60e1c"
  }),
  "package-lock.json": JSON.stringify({
    "name": "audit-fix-old-tap-14",
    "lockfileVersion": 2,
    "requires": true,
    "packages": {
      "": {
        "dependencies": {
          "mkdirp": "^0.5.1",
          "tap": "^13.0.3"
        }
      },
      "node_modules/@babel/code-frame": {
        "name": "@babel/code-frame",
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.0.0.tgz",
        "integrity": "sha512-OfC2uemaknXr87bdLUkWog7nYuliM9Ij5HUcajsVcMCpQrcLmtxRbVFTIqmcSkSeYRBFBRxs2FiUqFJDLdiebA==",
        "license": "MIT",
        "dependencies": {
          "@babel/highlight": "^7.0.0"
        }
      },
      "node_modules/@babel/generator": {
        "name": "@babel/generator",
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.4.0.tgz",
        "integrity": "sha512-/v5I+a1jhGSKLgZDcmAUZ4K/VePi43eRkUs3yePW1HB1iANOD5tqJXwGSG4BZhSksP8J9ejSlwGeTiiOFZOrXQ==",
        "license": "MIT",
        "dependencies": {
          "@babel/types": "^7.4.0",
          "jsesc": "^2.5.1",
          "lodash": "^4.17.11",
          "source-map": "^0.5.0",
          "trim-right": "^1.0.1"
        }
      },
      "node_modules/@babel/generator/node_modules/jsesc": {
        "name": "jsesc",
        "version": "2.5.2",
        "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
        "integrity": "sha512-OYu7XEzjkCQ3C5Ps3QIZsQfNpqoJyZZA99wd9aWd05NCtC5pWOkShK2mkL6HXQR6/Cy2lbNdPlZBpuQHXE63gA==",
        "license": "MIT",
        "bin": {
          "jsesc": "bin/jsesc"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@babel/helper-function-name": {
        "name": "@babel/helper-function-name",
        "version": "7.1.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.1.0.tgz",
        "integrity": "sha512-A95XEoCpb3TO+KZzJ4S/5uW5fNe26DjBGqf1o9ucyLyCmi1dXq/B3c8iaWTfBk3VvetUxl16e8tIrd5teOCfGw==",
        "license": "MIT",
        "dependencies": {
          "@babel/helper-get-function-arity": "^7.0.0",
          "@babel/template": "^7.1.0",
          "@babel/types": "^7.0.0"
        }
      },
      "node_modules/@babel/helper-get-function-arity": {
        "name": "@babel/helper-get-function-arity",
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.0.0.tgz",
        "integrity": "sha512-r2DbJeg4svYvt3HOS74U4eWKsUAMRH01Z1ds1zx8KNTPtpTL5JAsdFv8BNyOpVqdFhHkkRDIg5B4AsxmkjAlmQ==",
        "license": "MIT",
        "dependencies": {
          "@babel/types": "^7.0.0"
        }
      },
      "node_modules/@babel/helper-split-export-declaration": {
        "name": "@babel/helper-split-export-declaration",
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.4.0.tgz",
        "integrity": "sha512-7Cuc6JZiYShaZnybDmfwhY4UYHzI6rlqhWjaIqbsJGsIqPimEYy5uh3akSRLMg65LSdSEnJ8a8/bWQN6u2oMGw==",
        "license": "MIT",
        "dependencies": {
          "@babel/types": "^7.4.0"
        }
      },
      "node_modules/@babel/highlight": {
        "name": "@babel/highlight",
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.0.0.tgz",
        "integrity": "sha512-UFMC4ZeFC48Tpvj7C8UgLvtkaUuovQX+5xNWrsIoMG8o2z+XFKjKaN9iVmS84dPwVN00W4wPmqvYoZF3EGAsfw==",
        "license": "MIT",
        "dependencies": {
          "chalk": "^2.0.0",
          "esutils": "^2.0.2",
          "js-tokens": "^4.0.0"
        }
      },
      "node_modules/@babel/highlight/node_modules/ansi-styles": {
        "name": "ansi-styles",
        "version": "3.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
        "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
        "license": "MIT",
        "dependencies": {
          "color-convert": "^1.9.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@babel/highlight/node_modules/chalk": {
        "name": "chalk",
        "version": "2.4.2",
        "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
        "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^3.2.1",
          "escape-string-regexp": "^1.0.5",
          "supports-color": "^5.3.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@babel/highlight/node_modules/js-tokens": {
        "name": "js-tokens",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
        "integrity": "sha512-RdJUflcE3cUzKiMqQgsCu06FPu9UdIJO0beYbPhHN4k6apgJtifcoCtT9bcxOpYBtpD2kCM6Sbzg4CausW/PKQ==",
        "license": "MIT"
      },
      "node_modules/@babel/highlight/node_modules/supports-color": {
        "name": "supports-color",
        "version": "5.5.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
        "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
        "license": "MIT",
        "dependencies": {
          "has-flag": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@babel/parser": {
        "name": "@babel/parser",
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.4.3.tgz",
        "integrity": "sha512-gxpEUhTS1sGA63EGQGuA+WESPR/6tz6ng7tSHFCmaTJK/cGK8y37cBTspX+U2xCAue2IQVvF6Z0oigmjwD8YGQ==",
        "license": "MIT",
        "bin": {
          "parser": "bin/babel-parser.js"
        },
        "engines": {
          "node": ">=6.0.0"
        }
      },
      "node_modules/@babel/runtime": {
        "name": "@babel/runtime",
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.4.3.tgz",
        "integrity": "sha512-9lsJwJLxDh/T3Q3SZszfWOTkk3pHbkmH+3KY+zwIDmsNlxsumuhS2TH3NIpktU4kNvfzy+k3eLT7aTJSPTo0OA==",
        "license": "MIT",
        "dependencies": {
          "regenerator-runtime": "^0.13.2"
        }
      },
      "node_modules/@babel/runtime/node_modules/regenerator-runtime": {
        "name": "regenerator-runtime",
        "version": "0.13.2",
        "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.2.tgz",
        "integrity": "sha512-S/TQAZJO+D3m9xeN1WTI8dLKBBiRgXBlTJvbWjCThHWZj9EvHK70Ff50/tYj2J/fvBY6JtFVwRuazHN2E7M9BA==",
        "license": "MIT"
      },
      "node_modules/@babel/template": {
        "name": "@babel/template",
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.4.0.tgz",
        "integrity": "sha512-SOWwxxClTTh5NdbbYZ0BmaBVzxzTh2tO/TeLTbF6MO6EzVhHTnff8CdBXx3mEtazFBoysmEM6GU/wF+SuSx4Fw==",
        "license": "MIT",
        "dependencies": {
          "@babel/code-frame": "^7.0.0",
          "@babel/parser": "^7.4.0",
          "@babel/types": "^7.4.0"
        }
      },
      "node_modules/@babel/traverse": {
        "name": "@babel/traverse",
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.4.3.tgz",
        "integrity": "sha512-HmA01qrtaCwwJWpSKpA948cBvU5BrmviAief/b3AVw936DtcdsTexlbyzNuDnthwhOQ37xshn7hvQaEQk7ISYQ==",
        "license": "MIT",
        "dependencies": {
          "@babel/code-frame": "^7.0.0",
          "@babel/generator": "^7.4.0",
          "@babel/helper-function-name": "^7.1.0",
          "@babel/helper-split-export-declaration": "^7.4.0",
          "@babel/parser": "^7.4.3",
          "@babel/types": "^7.4.0",
          "debug": "^4.1.0",
          "globals": "^11.1.0",
          "lodash": "^4.17.11"
        }
      },
      "node_modules/@babel/traverse/node_modules/debug": {
        "name": "debug",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
        "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
        "license": "MIT",
        "dependencies": {
          "ms": "^2.1.1"
        }
      },
      "node_modules/@babel/traverse/node_modules/globals": {
        "name": "globals",
        "version": "11.11.0",
        "resolved": "https://registry.npmjs.org/globals/-/globals-11.11.0.tgz",
        "integrity": "sha512-WHq43gS+6ufNOEqlrDBxVEbb8ntfXrfAUU2ZOpCxrBdGKW3gyv8mCxAfIBD0DroPKGrJ2eSsXsLtY9MPntsyTw==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@babel/traverse/node_modules/ms": {
        "name": "ms",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
        "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg==",
        "license": "MIT"
      },
      "node_modules/@babel/types": {
        "name": "@babel/types",
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.4.0.tgz",
        "integrity": "sha512-aPvkXyU2SPOnztlgo8n9cEiXW755mgyvueUPcpStqdzoSPm0fjO0vQBjLkt3JKJW7ufikfcnMTTPsN1xaTsBPA==",
        "license": "MIT",
        "dependencies": {
          "esutils": "^2.0.2",
          "lodash": "^4.17.11",
          "to-fast-properties": "^2.0.0"
        }
      },
      "node_modules/@babel/types/node_modules/to-fast-properties": {
        "name": "to-fast-properties",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
        "integrity": "sha1-3F5pjL0HkmW8c+A3doGk5Og/YW4=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/@types/prop-types": {
        "name": "@types/prop-types",
        "version": "15.7.1",
        "resolved": "https://registry.npmjs.org/@types/prop-types/-/prop-types-15.7.1.tgz",
        "integrity": "sha512-CFzn9idOEpHrgdw8JsoTkaDDyRWk1jrzIV8djzcgpq0y9tG4B4lFT+Nxh52DVpDXV+n4+NPNv7M1Dj5uMp6XFg==",
        "license": "MIT"
      },
      "node_modules/@types/react": {
        "name": "@types/react",
        "version": "16.8.14",
        "resolved": "https://registry.npmjs.org/@types/react/-/react-16.8.14.tgz",
        "integrity": "sha512-26tFVJ1omGmzIdFTFmnC5zhz1GTaqCjxgUxV4KzWvsybF42P7/j4RBn6UeO3KbHPXqKWZszMXMoI65xIWm954A==",
        "license": "MIT",
        "dependencies": {
          "@types/prop-types": "*",
          "csstype": "^2.2.0"
        }
      },
      "node_modules/ajv": {
        "name": "ajv",
        "version": "6.10.0",
        "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.0.tgz",
        "integrity": "sha512-nffhOpkymDECQyR0mnsUtoCE8RlX38G0rYP+wgLWFyZuUyuuojSSvi/+euOiQBIn63whYwYVIIH1TvE3tu4OEg==",
        "license": "MIT",
        "dependencies": {
          "fast-deep-equal": "^2.0.1",
          "fast-json-stable-stringify": "^2.0.0",
          "json-schema-traverse": "^0.4.1",
          "uri-js": "^4.2.2"
        }
      },
      "node_modules/ansi-escapes": {
        "name": "ansi-escapes",
        "version": "3.2.0",
        "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
        "integrity": "sha512-cBhpre4ma+U0T1oM5fXg7Dy1Jw7zzwv7lt/GoCpr+hDQJoYnKVPLL4dCvSEFMmQurOQvSrwT7SL/DAlhBI97RQ==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
        "integrity": "sha1-w7M6te42DYbg5ijwRorn7yfWVN8=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/ansi-styles": {
        "name": "ansi-styles",
        "version": "2.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.2.1.tgz",
        "integrity": "sha1-tDLdM1i2NM914eRmQ2gkBTPB3b4=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/ansicolors": {
        "name": "ansicolors",
        "version": "0.3.2",
        "resolved": "https://registry.npmjs.org/ansicolors/-/ansicolors-0.3.2.tgz",
        "integrity": "sha1-ZlWX3oap/+Oqm/vmyuXG6kJrSXk=",
        "license": "MIT"
      },
      "node_modules/anymatch": {
        "name": "anymatch",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-2.0.0.tgz",
        "integrity": "sha512-5teOsQWABXHHBFP9y3skS5P3d/WfWXpv3FUpy+LorMrNYaT9pI4oLMQX7jzQ2KklNpGpWHzdCXTDT2Y3XGlZBw==",
        "license": "ISC",
        "dependencies": {
          "micromatch": "^3.1.4",
          "normalize-path": "^2.1.1"
        }
      },
      "node_modules/anymatch/node_modules/normalize-path": {
        "name": "normalize-path",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-2.1.1.tgz",
        "integrity": "sha1-GrKLVW4Zg2Oowab35vogE3/mrtk=",
        "license": "MIT",
        "dependencies": {
          "remove-trailing-separator": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/append-transform": {
        "name": "append-transform",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
        "integrity": "sha512-P009oYkeHyU742iSZJzZZywj4QRJdnTWffaKuJQLablCZ1uz6/cW4yaRgcDaoQ+uwOxxnt0gRUcwfsNP2ri0gw==",
        "license": "MIT",
        "dependencies": {
          "default-require-extensions": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/archy": {
        "name": "archy",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
        "integrity": "sha1-+cjBN1fMHde8N5rHeyxipcKGjEA=",
        "license": "MIT"
      },
      "node_modules/arg": {
        "name": "arg",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.0.tgz",
        "integrity": "sha512-ZWc51jO3qegGkVh8Hwpv636EkbesNV5ZNQPCtRa+0qytRYPEs9IYT9qITY9buezqUH5uqyzlWLcufrzU2rffdg==",
        "license": "MIT"
      },
      "node_modules/argparse": {
        "name": "argparse",
        "version": "1.0.10",
        "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
        "integrity": "sha512-o5Roy6tNG4SL/FOkCAN6RzjiakZS25RLYFrcMttJqbdd8BWrnA+fGz57iN5Pb06pvBGvl5gQ0B48dJlslXvoTg==",
        "license": "MIT",
        "dependencies": {
          "sprintf-js": "~1.0.2"
        }
      },
      "node_modules/arr-diff": {
        "name": "arr-diff",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/arr-diff/-/arr-diff-4.0.0.tgz",
        "integrity": "sha1-1kYQdP6/7HHn4VI1dhoyml3HxSA=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/arr-flatten": {
        "name": "arr-flatten",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/arr-flatten/-/arr-flatten-1.1.0.tgz",
        "integrity": "sha512-L3hKV5R/p5o81R7O02IGnwpDmkp6E982XhtbuwSe3O4qOtMMMtodicASA1Cny2U+aCXcNpml+m4dPsvsJ3jatg==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/arr-union": {
        "name": "arr-union",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/arr-union/-/arr-union-3.1.0.tgz",
        "integrity": "sha1-45sJrqne+Gao8gbiiK9jkZuuOcQ=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/array-unique": {
        "name": "array-unique",
        "version": "0.3.2",
        "resolved": "https://registry.npmjs.org/array-unique/-/array-unique-0.3.2.tgz",
        "integrity": "sha1-qJS3XUvE9s1nnvMkSp/Y9Gri1Cg=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/arrify": {
        "name": "arrify",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
        "integrity": "sha1-iYUI2iIm84DfkEcoRWhJwVAaSw0=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/asn1": {
        "name": "asn1",
        "version": "0.2.4",
        "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
        "integrity": "sha512-jxwzQpLQjSmWXgwaCZE9Nz+glAG01yF1QnWgbhGwHI5A6FRIEY6IVqtHhIepHqI7/kyEyQEagBC5mBEFlIYvdg==",
        "license": "MIT",
        "dependencies": {
          "safer-buffer": "~2.1.0"
        }
      },
      "node_modules/assert-plus": {
        "name": "assert-plus",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
        "integrity": "sha1-8S4PPF13sLHN2RRpQuTpbB5N1SU=",
        "license": "MIT",
        "engines": {
          "node": ">=0.8"
        }
      },
      "node_modules/assign-symbols": {
        "name": "assign-symbols",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/assign-symbols/-/assign-symbols-1.0.0.tgz",
        "integrity": "sha1-WWZ/QfrdTyDMvCu5a41Pf3jsA2c=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/astral-regex": {
        "name": "astral-regex",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
        "integrity": "sha512-+Ryf6g3BKoRc7jfp7ad8tM4TtMiaWvbF/1/sQcZPkkS7ag3D5nMBCe2UfOTONtAkaG0tO0ij3C5Lwmf1EiyjHg==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/async-each": {
        "name": "async-each",
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/async-each/-/async-each-1.0.3.tgz",
        "integrity": "sha512-z/WhQ5FPySLdvREByI2vZiTWwCnF0moMJ1hK9YQwDTHKh6I7/uSckMetoRGb5UBZPC1z0jlw+n/XCgjeH7y1AQ==",
        "license": "MIT"
      },
      "node_modules/async-hook-domain": {
        "name": "async-hook-domain",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.0.tgz",
        "integrity": "sha512-NH7V97d1yCbIanu2oDLyPT2GFNct0esPeJyRfkk8J5hTztHVSQp4UiNfL2O42sCA9XZPU8OgHvzOmt9ewBhVqA==",
        "license": "ISC",
        "dependencies": {
          "source-map-support": "^0.5.11"
        }
      },
      "node_modules/asynckit": {
        "name": "asynckit",
        "version": "0.4.0",
        "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
        "integrity": "sha1-x57Zf380y48robyXkLzDZkdLS3k=",
        "license": "MIT"
      },
      "node_modules/atob": {
        "name": "atob",
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/atob/-/atob-2.1.2.tgz",
        "integrity": "sha512-Wm6ukoaOGJi/73p/cl2GvLjTI5JM1k/O14isD73YML8StrH/7/lRFgmg8nICZgD3bZZvjwCGxtMOD3wWNAu8cg==",
        "license": "(MIT OR Apache-2.0)",
        "bin": {
          "atob": "bin/atob.js"
        },
        "engines": {
          "node": ">= 4.5.0"
        }
      },
      "node_modules/auto-bind": {
        "name": "auto-bind",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-2.1.0.tgz",
        "integrity": "sha512-qZuFvkes1eh9lB2mg8/HG18C+5GIO51r+RrCSst/lh+i5B1CtVlkhTE488M805Nr3dKl0sM/pIFKSKUIlg3zUg==",
        "license": "MIT",
        "dependencies": {
          "@types/react": "^16.8.12"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/aws-sign2": {
        "name": "aws-sign2",
        "version": "0.7.0",
        "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
        "integrity": "sha1-tG6JCTSpWR8tL2+G1+ap8bP+dqg=",
        "license": "Apache-2.0",
        "engines": {
          "node": "*"
        }
      },
      "node_modules/aws4": {
        "name": "aws4",
        "version": "1.8.0",
        "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.8.0.tgz",
        "integrity": "sha512-ReZxvNHIOv88FlT7rxcXIIC0fPt4KZqZbOlivyWtXLt8ESx84zd3kMC6iK5jVeS2qt+g7ftS7ye4fi06X5rtRQ==",
        "license": "MIT"
      },
      "node_modules/babel-code-frame": {
        "name": "babel-code-frame",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-code-frame/-/babel-code-frame-6.26.0.tgz",
        "integrity": "sha1-Y/1D99weO7fONZR9uP42mj9Yx0s=",
        "license": "MIT",
        "dependencies": {
          "chalk": "^1.1.3",
          "esutils": "^2.0.2",
          "js-tokens": "^3.0.2"
        }
      },
      "node_modules/babel-core": {
        "name": "babel-core",
        "version": "6.26.3",
        "resolved": "https://registry.npmjs.org/babel-core/-/babel-core-6.26.3.tgz",
        "integrity": "sha512-6jyFLuDmeidKmUEb3NM+/yawG0M2bDZ9Z1qbZP59cyHLz8kYGKYwpJP0UwUKKUiTRNvxfLesJnTedqczP7cTDA==",
        "license": "MIT",
        "dependencies": {
          "babel-code-frame": "^6.26.0",
          "babel-generator": "^6.26.0",
          "babel-helpers": "^6.24.1",
          "babel-messages": "^6.23.0",
          "babel-register": "^6.26.0",
          "babel-runtime": "^6.26.0",
          "babel-template": "^6.26.0",
          "babel-traverse": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "convert-source-map": "^1.5.1",
          "debug": "^2.6.9",
          "json5": "^0.5.1",
          "lodash": "^4.17.4",
          "minimatch": "^3.0.4",
          "path-is-absolute": "^1.0.1",
          "private": "^0.1.8",
          "slash": "^1.0.0",
          "source-map": "^0.5.7"
        }
      },
      "node_modules/babel-generator": {
        "name": "babel-generator",
        "version": "6.26.1",
        "resolved": "https://registry.npmjs.org/babel-generator/-/babel-generator-6.26.1.tgz",
        "integrity": "sha512-HyfwY6ApZj7BYTcJURpM5tznulaBvyio7/0d4zFOeMPUmfxkCjHocCuoLa2SAGzBI8AREcH3eP3758F672DppA==",
        "license": "MIT",
        "dependencies": {
          "babel-messages": "^6.23.0",
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "detect-indent": "^4.0.0",
          "jsesc": "^1.3.0",
          "lodash": "^4.17.4",
          "source-map": "^0.5.7",
          "trim-right": "^1.0.1"
        }
      },
      "node_modules/babel-helper-builder-react-jsx": {
        "name": "babel-helper-builder-react-jsx",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-helper-builder-react-jsx/-/babel-helper-builder-react-jsx-6.26.0.tgz",
        "integrity": "sha1-Of+DE7dci2Xc7/HzHTg+D/KkCKA=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "esutils": "^2.0.2"
        }
      },
      "node_modules/babel-helpers": {
        "name": "babel-helpers",
        "version": "6.24.1",
        "resolved": "https://registry.npmjs.org/babel-helpers/-/babel-helpers-6.24.1.tgz",
        "integrity": "sha1-NHHenK7DiOXIUOWX5Yom3fN2ArI=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.22.0",
          "babel-template": "^6.24.1"
        }
      },
      "node_modules/babel-messages": {
        "name": "babel-messages",
        "version": "6.23.0",
        "resolved": "https://registry.npmjs.org/babel-messages/-/babel-messages-6.23.0.tgz",
        "integrity": "sha1-8830cDhYA1sqKVHG7F7fbGLyYw4=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.22.0"
        }
      },
      "node_modules/babel-plugin-syntax-jsx": {
        "name": "babel-plugin-syntax-jsx",
        "version": "6.18.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-syntax-jsx/-/babel-plugin-syntax-jsx-6.18.0.tgz",
        "integrity": "sha1-CvMqmm4Tyno/1QaeYtew9Y0NiUY=",
        "license": "MIT"
      },
      "node_modules/babel-plugin-syntax-object-rest-spread": {
        "name": "babel-plugin-syntax-object-rest-spread",
        "version": "6.13.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-syntax-object-rest-spread/-/babel-plugin-syntax-object-rest-spread-6.13.0.tgz",
        "integrity": "sha1-/WU28rzhODb/o6VFjEkDpZe7O/U=",
        "license": "MIT"
      },
      "node_modules/babel-plugin-transform-es2015-destructuring": {
        "name": "babel-plugin-transform-es2015-destructuring",
        "version": "6.23.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-es2015-destructuring/-/babel-plugin-transform-es2015-destructuring-6.23.0.tgz",
        "integrity": "sha1-mXux8auWf2gtKwh2/jWNYOdlxW0=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.22.0"
        }
      },
      "node_modules/babel-plugin-transform-object-rest-spread": {
        "name": "babel-plugin-transform-object-rest-spread",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-object-rest-spread/-/babel-plugin-transform-object-rest-spread-6.26.0.tgz",
        "integrity": "sha1-DzZpLVD+9rfi1LOsFHgTepY7ewY=",
        "license": "MIT",
        "dependencies": {
          "babel-plugin-syntax-object-rest-spread": "^6.8.0",
          "babel-runtime": "^6.26.0"
        }
      },
      "node_modules/babel-plugin-transform-react-jsx": {
        "name": "babel-plugin-transform-react-jsx",
        "version": "6.24.1",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-react-jsx/-/babel-plugin-transform-react-jsx-6.24.1.tgz",
        "integrity": "sha1-hAoCjn30YN/DotKfDA2R9jduZqM=",
        "license": "MIT",
        "dependencies": {
          "babel-helper-builder-react-jsx": "^6.24.1",
          "babel-plugin-syntax-jsx": "^6.8.0",
          "babel-runtime": "^6.22.0"
        }
      },
      "node_modules/babel-register": {
        "name": "babel-register",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-register/-/babel-register-6.26.0.tgz",
        "integrity": "sha1-btAhFz4vy0htestFxgCahW9kcHE=",
        "license": "MIT",
        "dependencies": {
          "babel-core": "^6.26.0",
          "babel-runtime": "^6.26.0",
          "core-js": "^2.5.0",
          "home-or-tmp": "^2.0.0",
          "lodash": "^4.17.4",
          "mkdirp": "^0.5.1",
          "source-map-support": "^0.4.15"
        }
      },
      "node_modules/babel-register/node_modules/source-map-support": {
        "name": "source-map-support",
        "version": "0.4.18",
        "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.4.18.tgz",
        "integrity": "sha512-try0/JqxPLF9nOjvSta7tVondkP5dwgyLDjVoyMDlmjugT2lRZ1OfsrYTkCd2hkDnJTKRbO/Rl3orm8vlsUzbA==",
        "license": "MIT",
        "dependencies": {
          "source-map": "^0.5.6"
        }
      },
      "node_modules/babel-runtime": {
        "name": "babel-runtime",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-runtime/-/babel-runtime-6.26.0.tgz",
        "integrity": "sha1-llxwWGaOgrVde/4E/yM3vItWR/4=",
        "license": "MIT",
        "dependencies": {
          "core-js": "^2.4.0",
          "regenerator-runtime": "^0.11.0"
        }
      },
      "node_modules/babel-template": {
        "name": "babel-template",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-template/-/babel-template-6.26.0.tgz",
        "integrity": "sha1-3gPi0WOWsGn0bdn/+FIfsaDjXgI=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.26.0",
          "babel-traverse": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "lodash": "^4.17.4"
        }
      },
      "node_modules/babel-traverse": {
        "name": "babel-traverse",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-traverse/-/babel-traverse-6.26.0.tgz",
        "integrity": "sha1-RqnL1+3MYsjlwGTi0tjQ9ANXZu4=",
        "license": "MIT",
        "dependencies": {
          "babel-code-frame": "^6.26.0",
          "babel-messages": "^6.23.0",
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "debug": "^2.6.8",
          "globals": "^9.18.0",
          "invariant": "^2.2.2",
          "lodash": "^4.17.4"
        }
      },
      "node_modules/babel-types": {
        "name": "babel-types",
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-types/-/babel-types-6.26.0.tgz",
        "integrity": "sha1-o7Bz+Uq0nrb6Vc1lInozQ4BjJJc=",
        "license": "MIT",
        "dependencies": {
          "babel-runtime": "^6.26.0",
          "esutils": "^2.0.2",
          "lodash": "^4.17.4",
          "to-fast-properties": "^1.0.3"
        }
      },
      "node_modules/babylon": {
        "name": "babylon",
        "version": "6.18.0",
        "resolved": "https://registry.npmjs.org/babylon/-/babylon-6.18.0.tgz",
        "integrity": "sha512-q/UEjfGJ2Cm3oKV71DJz9d25TPnq5rhBVL2Q4fA5wcC3jcrdn7+SssEybFIxwAvvP+YCsCYNKughoF33GxgycQ==",
        "license": "MIT",
        "bin": {
          "babylon": "bin/babylon.js"
        }
      },
      "node_modules/balanced-match": {
        "name": "balanced-match",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
        "integrity": "sha1-ibTRmasr7kneFk6gK4nORi1xt2c=",
        "license": "MIT"
      },
      "node_modules/base": {
        "name": "base",
        "version": "0.11.2",
        "resolved": "https://registry.npmjs.org/base/-/base-0.11.2.tgz",
        "integrity": "sha512-5T6P4xPgpp0YDFvSWwEZ4NoE3aM4QBQXDzmVbraCkFj8zHM+mba8SyqB5DbZWyR7mYHo6Y7BdQo3MoA4m0TeQg==",
        "license": "MIT",
        "dependencies": {
          "cache-base": "^1.0.1",
          "class-utils": "^0.3.5",
          "component-emitter": "^1.2.1",
          "define-property": "^1.0.0",
          "isobject": "^3.0.1",
          "mixin-deep": "^1.2.0",
          "pascalcase": "^0.1.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/base/node_modules/define-property": {
        "name": "define-property",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
        "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/bcrypt-pbkdf": {
        "name": "bcrypt-pbkdf",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
        "integrity": "sha1-pDAdOJtqQ/m2f/PKEaP2Y342Dp4=",
        "license": "BSD-3-Clause",
        "dependencies": {
          "tweetnacl": "^0.14.3"
        }
      },
      "node_modules/binary-extensions": {
        "name": "binary-extensions",
        "version": "1.13.1",
        "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-1.13.1.tgz",
        "integrity": "sha512-Un7MIEDdUC5gNpcGDV97op1Ywk748MpHcFTHoYs6qnj1Z3j7I53VG3nwZhKzoBZmbdRNnb6WRdFlwl7tSDuZGw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/bind-obj-methods": {
        "name": "bind-obj-methods",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
        "integrity": "sha512-3/qRXczDi2Cdbz6jE+W3IflJOutRVica8frpBn14de1mBOkzDo+6tY33kNhvkw54Kn3PzRRD2VnGbGPcTAk4sw==",
        "license": "ISC"
      },
      "node_modules/brace-expansion": {
        "name": "brace-expansion",
        "version": "1.1.11",
        "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
        "integrity": "sha512-iCuPHDFgrHX7H2vEI/5xpz07zSHB00TpugqhmYtVmMO6518mCuRMoOYFldEBl0g187ufozdaHgWKcYFb61qGiA==",
        "license": "MIT",
        "dependencies": {
          "balanced-match": "^1.0.0",
          "concat-map": "0.0.1"
        }
      },
      "node_modules/braces": {
        "name": "braces",
        "version": "2.3.2",
        "resolved": "https://registry.npmjs.org/braces/-/braces-2.3.2.tgz",
        "integrity": "sha512-aNdbnj9P8PjdXU4ybaWLK2IF3jc/EoDYbC7AazW6to3TRsfXxscC9UXOB5iDiEQrkyIbWp2SLQda4+QAa7nc3w==",
        "license": "MIT",
        "dependencies": {
          "arr-flatten": "^1.1.0",
          "array-unique": "^0.3.2",
          "extend-shallow": "^2.0.1",
          "fill-range": "^4.0.0",
          "isobject": "^3.0.1",
          "repeat-element": "^1.1.2",
          "snapdragon": "^0.8.1",
          "snapdragon-node": "^2.0.1",
          "split-string": "^3.0.2",
          "to-regex": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/browser-process-hrtime": {
        "name": "browser-process-hrtime",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
        "integrity": "sha512-9o5UecI3GhkpM6DrXr69PblIuWxPKk9Y0jHBRhdocZ2y7YECBFCsHm79Pr3OyR2AvjhDkabFJaDJMYRazHgsow==",
        "license": "BSD-2-Clause"
      },
      "node_modules/buffer-from": {
        "name": "buffer-from",
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
        "integrity": "sha512-MQcXEUbCKtEo7bhqEs6560Hyd4XaovZlO/k9V3hjVUF/zwW7KBVdSK4gIt/bzwS9MbR5qob+F5jusZsb0YQK2A==",
        "license": "MIT"
      },
      "node_modules/cache-base": {
        "name": "cache-base",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/cache-base/-/cache-base-1.0.1.tgz",
        "integrity": "sha512-AKcdTnFSWATd5/GCPRxr2ChwIJ85CeyrEyjRHlKxQ56d4XJMGym0uAiKn0xbLOGOl3+yRpOTi484dVCEc5AUzQ==",
        "license": "MIT",
        "dependencies": {
          "collection-visit": "^1.0.0",
          "component-emitter": "^1.2.1",
          "get-value": "^2.0.6",
          "has-value": "^1.0.0",
          "isobject": "^3.0.1",
          "set-value": "^2.0.0",
          "to-object-path": "^0.3.0",
          "union-value": "^1.0.0",
          "unset-value": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/caching-transform": {
        "name": "caching-transform",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
        "integrity": "sha512-Mtgcv3lh3U0zRii/6qVgQODdPA4G3zhG+jtbCWj39RXuUFTMzH0vcdMtaJS1jPowd+It2Pqr6y3NJMQqOqCE2w==",
        "license": "MIT",
        "dependencies": {
          "hasha": "^3.0.0",
          "make-dir": "^2.0.0",
          "package-hash": "^3.0.0",
          "write-file-atomic": "^2.4.2"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/caller-callsite": {
        "name": "caller-callsite",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/caller-callsite/-/caller-callsite-2.0.0.tgz",
        "integrity": "sha1-hH4PzgoiN1CpoCfFSzNzGtMVQTQ=",
        "license": "MIT",
        "dependencies": {
          "callsites": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/caller-path": {
        "name": "caller-path",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-2.0.0.tgz",
        "integrity": "sha1-Ro+DBE42mrIBD6xfBs7uFbsssfQ=",
        "license": "MIT",
        "dependencies": {
          "caller-callsite": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/callsites": {
        "name": "callsites",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/callsites/-/callsites-2.0.0.tgz",
        "integrity": "sha1-BuuE8A7qQT2oav/vrL/7Ngk7PFA=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/camelcase": {
        "name": "camelcase",
        "version": "5.3.1",
        "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
        "integrity": "sha512-L28STB170nwWS63UjtlEOE3dldQApaJXZkOI1uMFfzf3rRuPegHaHesyee+YxQ+W6SvRDQV6UrdOdRiR153wJg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/capture-stack-trace": {
        "name": "capture-stack-trace",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/capture-stack-trace/-/capture-stack-trace-1.0.1.tgz",
        "integrity": "sha512-mYQLZnx5Qt1JgB1WEiMCf2647plpGeQ2NMR/5L0HNZzGQo4fuSPnK+wjfPnKZV0aiJDgzmWqqkV/g7JD+DW0qw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/cardinal": {
        "name": "cardinal",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/cardinal/-/cardinal-2.1.1.tgz",
        "integrity": "sha1-fMEFXYItISlU0HsIXeolHMe8VQU=",
        "license": "MIT",
        "dependencies": {
          "ansicolors": "~0.3.2",
          "redeyed": "~2.1.0"
        },
        "bin": {
          "cdl": "bin/cdl.js"
        }
      },
      "node_modules/caseless": {
        "name": "caseless",
        "version": "0.12.0",
        "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
        "integrity": "sha1-G2gcIf+EAzyCZUMJBolCDRhxUdw=",
        "license": "Apache-2.0"
      },
      "node_modules/chalk": {
        "name": "chalk",
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.3.tgz",
        "integrity": "sha1-qBFcVeSnAv5NFQq9OHKCKn4J/Jg=",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^2.2.1",
          "escape-string-regexp": "^1.0.2",
          "has-ansi": "^2.0.0",
          "strip-ansi": "^3.0.0",
          "supports-color": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/chokidar": {
        "name": "chokidar",
        "version": "2.1.5",
        "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-2.1.5.tgz",
        "integrity": "sha512-i0TprVWp+Kj4WRPtInjexJ8Q+BqTE909VpH8xVhXrJkoc5QC8VO9TryGOqTr+2hljzc1sC62t22h5tZePodM/A==",
        "deprecated": "Chokidar 2 will break on node v14+. Upgrade to chokidar 3 with 15x less dependencies.",
        "license": "MIT",
        "dependencies": {
          "anymatch": "^2.0.0",
          "async-each": "^1.0.1",
          "braces": "^2.3.2",
          "fsevents": "^1.2.7",
          "glob-parent": "^3.1.0",
          "inherits": "^2.0.3",
          "is-binary-path": "^1.0.0",
          "is-glob": "^4.0.0",
          "normalize-path": "^3.0.0",
          "path-is-absolute": "^1.0.0",
          "readdirp": "^2.2.1",
          "upath": "^1.1.1"
        },
        "optionalDependencies": {
          "fsevents": "^1.2.7"
        }
      },
      "node_modules/ci-info": {
        "name": "ci-info",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
        "integrity": "sha512-5tK7EtrZ0N+OLFMthtqOj4fI2Jeb88C4CAZPu25LDVUgXJ0A3Js4PMGqrn0JU1W0Mh1/Z8wZzYPxqUrXeBboCQ==",
        "license": "MIT"
      },
      "node_modules/class-utils": {
        "name": "class-utils",
        "version": "0.3.6",
        "resolved": "https://registry.npmjs.org/class-utils/-/class-utils-0.3.6.tgz",
        "integrity": "sha512-qOhPa/Fj7s6TY8H8esGu5QNpMMQxz79h+urzrNYN6mn+9BnxlDGf5QZ+XeCDsxSjPqsSR56XOZOJmpeurnLMeg==",
        "license": "MIT",
        "dependencies": {
          "arr-union": "^3.1.0",
          "define-property": "^0.2.5",
          "isobject": "^3.0.0",
          "static-extend": "^0.1.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/class-utils/node_modules/define-property": {
        "name": "define-property",
        "version": "0.2.5",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
        "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/class-utils/node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
        "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/class-utils/node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
        "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/class-utils/node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
        "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^0.1.6",
          "is-data-descriptor": "^0.1.4",
          "kind-of": "^5.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/class-utils/node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
        "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/cli-cursor": {
        "name": "cli-cursor",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
        "integrity": "sha1-s12sN2R5+sw+lHR9QdDQ9SOP/LU=",
        "license": "MIT",
        "dependencies": {
          "restore-cursor": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/cli-truncate": {
        "name": "cli-truncate",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-1.1.0.tgz",
        "integrity": "sha512-bAtZo0u82gCfaAGfSNxUdTI9mNyza7D8w4CVCcaOsy7sgwDzvx6ekr6cuWJqY3UGzgnQ1+4wgENup5eIhgxEYA==",
        "license": "MIT",
        "dependencies": {
          "slice-ansi": "^1.0.0",
          "string-width": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/cliui": {
        "name": "cliui",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
        "integrity": "sha512-4FG+RSG9DL7uEwRUZXZn3SS34DiDPfzP0VOiEwtUWlE+AR2EIg+hSyvrIgUUfhdgR/UkAeW2QHgeP+hWrXs7jQ==",
        "license": "ISC",
        "dependencies": {
          "string-width": "^2.1.1",
          "strip-ansi": "^4.0.0",
          "wrap-ansi": "^2.0.0"
        }
      },
      "node_modules/cliui/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
        "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/cliui/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
        "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/code-point-at": {
        "name": "code-point-at",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
        "integrity": "sha1-DQcLTQQ6W+ozovGkDi7bPZpMz3c=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/collection-visit": {
        "name": "collection-visit",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/collection-visit/-/collection-visit-1.0.0.tgz",
        "integrity": "sha1-S8A3PBZLwykbTTaMgpzxqApZ3KA=",
        "license": "MIT",
        "dependencies": {
          "map-visit": "^1.0.0",
          "object-visit": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/color-convert": {
        "name": "color-convert",
        "version": "1.9.3",
        "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
        "integrity": "sha512-QfAUtd+vFdAtFQcC8CCyYt1fYWxSqAiK2cSD6zDB8N3cpsEBAvRxp9zOGg6G/SHHJYAT88/az/IuDGALsNVbGg==",
        "license": "MIT",
        "dependencies": {
          "color-name": "1.1.3"
        }
      },
      "node_modules/color-name": {
        "name": "color-name",
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
        "integrity": "sha1-p9BVi9icQveV3UIyj3QIMcpTvCU=",
        "license": "MIT"
      },
      "node_modules/color-support": {
        "name": "color-support",
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
        "integrity": "sha512-qiBjkpbMLO/HL68y+lh4q0/O1MZFj2RX6X/KmMa3+gJD3z+WwI1ZzDHysvqHGS3mP6mznPckpXmw1nI9cJjyRg==",
        "license": "ISC",
        "bin": {
          "color-support": "bin.js"
        }
      },
      "node_modules/combined-stream": {
        "name": "combined-stream",
        "version": "1.0.7",
        "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.7.tgz",
        "integrity": "sha512-brWl9y6vOB1xYPZcpZde3N9zDByXTosAeMDo4p1wzo6UMOX4vumB+TP1RZ76sfE6Md68Q0NJSrE/gbezd4Ul+w==",
        "license": "MIT",
        "dependencies": {
          "delayed-stream": "~1.0.0"
        },
        "engines": {
          "node": ">= 0.8"
        }
      },
      "node_modules/commander": {
        "name": "commander",
        "version": "2.20.0",
        "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.0.tgz",
        "integrity": "sha512-7j2y+40w61zy6YC2iRNpUe/NwhNyoXrYpHMrSunaMG64nRnaf96zO/KMQR4OyN/UnE5KLyEBnKHd4aG3rskjpQ==",
        "license": "MIT",
        "optional": true
      },
      "node_modules/commondir": {
        "name": "commondir",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
        "integrity": "sha1-3dgA2gxmEnOTzKWVDqloo6rxJTs=",
        "license": "MIT"
      },
      "node_modules/component-emitter": {
        "name": "component-emitter",
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/component-emitter/-/component-emitter-1.3.0.tgz",
        "integrity": "sha512-Rd3se6QB+sO1TwqZjscQrurpEPIfO0/yYnSin6Q/rD3mOutHvUrCAhJub3r90uNb+SESBuE0QYoB90YdfatsRg==",
        "license": "MIT"
      },
      "node_modules/concat-map": {
        "name": "concat-map",
        "version": "0.0.1",
        "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
        "integrity": "sha1-2Klr13/Wjfd5OnMDajug1UBdR3s=",
        "license": "MIT"
      },
      "node_modules/convert-source-map": {
        "name": "convert-source-map",
        "version": "1.6.0",
        "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.6.0.tgz",
        "integrity": "sha512-eFu7XigvxdZ1ETfbgPBohgyQ/Z++C0eEhTor0qRwBw9unw+L0/6V8wkSuGgzdThkiS5lSpdptOQPD8Ak40a+7A==",
        "license": "MIT",
        "dependencies": {
          "safe-buffer": "~5.1.1"
        }
      },
      "node_modules/copy-descriptor": {
        "name": "copy-descriptor",
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/copy-descriptor/-/copy-descriptor-0.1.1.tgz",
        "integrity": "sha1-Z29us8OZl8LuGsOpJP1hJHSPV40=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/core-js": {
        "name": "core-js",
        "version": "2.6.5",
        "resolved": "https://registry.npmjs.org/core-js/-/core-js-2.6.5.tgz",
        "integrity": "sha512-klh/kDpwX8hryYL14M9w/xei6vrv6sE8gTHDG7/T/+SEovB/G4ejwcfE/CBzO6Edsu+OETZMZ3wcX/EjUkrl5A==",
        "deprecated": "core-js@<3 is no longer maintained and not recommended for usage due to the number of issues. Please, upgrade your dependencies to the actual version of core-js@3.",
        "license": "MIT"
      },
      "node_modules/core-util-is": {
        "name": "core-util-is",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
        "integrity": "sha1-tf1UIgqivFq1eqtxQMlAdUUDwac=",
        "license": "MIT"
      },
      "node_modules/coveralls": {
        "name": "coveralls",
        "version": "3.0.3",
        "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.3.tgz",
        "integrity": "sha512-viNfeGlda2zJr8Gj1zqXpDMRjw9uM54p7wzZdvLRyOgnAfCe974Dq4veZkjJdxQXbmdppu6flEajFYseHYaUhg==",
        "license": "BSD-2-Clause",
        "dependencies": {
          "growl": "~> 1.10.0",
          "js-yaml": "^3.11.0",
          "lcov-parse": "^0.0.10",
          "log-driver": "^1.2.7",
          "minimist": "^1.2.0",
          "request": "^2.86.0"
        },
        "bin": {
          "coveralls": "bin/coveralls.js"
        },
        "engines": {
          "node": ">=4.0.0"
        }
      },
      "node_modules/coveralls/node_modules/minimist": {
        "name": "minimist",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
        "integrity": "sha1-o1AIsg9BOD7sH7kU9M1d95omQoQ=",
        "license": "MIT"
      },
      "node_modules/cp-file": {
        "name": "cp-file",
        "version": "6.2.0",
        "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
        "integrity": "sha512-fmvV4caBnofhPe8kOcitBwSn2f39QLjnAnGq3gO9dfd75mUytzKNZB1hde6QHunW2Rt+OwuBOMc3i1tNElbszA==",
        "license": "MIT",
        "dependencies": {
          "graceful-fs": "^4.1.2",
          "make-dir": "^2.0.0",
          "nested-error-stacks": "^2.0.0",
          "pify": "^4.0.1",
          "safe-buffer": "^5.0.1"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/cross-spawn": {
        "name": "cross-spawn",
        "version": "4.0.2",
        "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
        "integrity": "sha1-e5JHYhwjrf3ThWAEqCPL45dCTUE=",
        "license": "MIT",
        "dependencies": {
          "lru-cache": "^4.0.1",
          "which": "^1.2.9"
        }
      },
      "node_modules/csstype": {
        "name": "csstype",
        "version": "2.6.4",
        "resolved": "https://registry.npmjs.org/csstype/-/csstype-2.6.4.tgz",
        "integrity": "sha512-lAJUJP3M6HxFXbqtGRc0iZrdyeN+WzOWeY0q/VnFzI+kqVrYIzC7bWlKqCW7oCIdzoPkvfp82EVvrTlQ8zsWQg==",
        "license": "MIT"
      },
      "node_modules/dashdash": {
        "name": "dashdash",
        "version": "1.14.1",
        "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
        "integrity": "sha1-hTz6D3y+L+1d4gMmuN1YEDX24vA=",
        "license": "MIT",
        "dependencies": {
          "assert-plus": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10"
        }
      },
      "node_modules/debug": {
        "name": "debug",
        "version": "2.6.9",
        "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
        "integrity": "sha512-bC7ElrdJaJnPbAP+1EotYvqZsb3ecl5wi6Bfi6BJTUcNowp6cvspg0jXznRTKDjm/E7AdgFBVeAPVMNcKGsHMA==",
        "license": "MIT",
        "dependencies": {
          "ms": "2.0.0"
        }
      },
      "node_modules/decamelize": {
        "name": "decamelize",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
        "integrity": "sha1-9lNNFRSCabIDUue+4m9QH5oZEpA=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/decode-uri-component": {
        "name": "decode-uri-component",
        "version": "0.2.0",
        "resolved": "https://registry.npmjs.org/decode-uri-component/-/decode-uri-component-0.2.0.tgz",
        "integrity": "sha1-6zkTMzRYd1y4TNGh+uBiEGu4dUU=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10"
        }
      },
      "node_modules/default-require-extensions": {
        "name": "default-require-extensions",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
        "integrity": "sha1-9fj7sYp9bVCyH2QfZJ67Uiz+JPc=",
        "license": "MIT",
        "dependencies": {
          "strip-bom": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/define-property": {
        "name": "define-property",
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-2.0.2.tgz",
        "integrity": "sha512-jwK2UV4cnPpbcG7+VRARKTZPUWowwXA8bzH5NP6ud0oeAxyYPuGZUAC7hMugpCdz4BeSZl2Dl9k66CHJ/46ZYQ==",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^1.0.2",
          "isobject": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/delayed-stream": {
        "name": "delayed-stream",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
        "integrity": "sha1-3zrhmayt+31ECqrgsp4icrJOxhk=",
        "license": "MIT",
        "engines": {
          "node": ">=0.4.0"
        }
      },
      "node_modules/detect-indent": {
        "name": "detect-indent",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/detect-indent/-/detect-indent-4.0.0.tgz",
        "integrity": "sha1-920GQ1LN9Docts5hnE7jqUdd4gg=",
        "license": "MIT",
        "dependencies": {
          "repeating": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/diff": {
        "name": "diff",
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
        "integrity": "sha1-fyjS657nsVqX79ic5j3P2qPMur8=",
        "engines": {
          "node": ">=0.3.1"
        }
      },
      "node_modules/domain-browser": {
        "name": "domain-browser",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/domain-browser/-/domain-browser-1.2.0.tgz",
        "integrity": "sha512-jnjyiM6eRyZl2H+W8Q/zLMA481hzi0eszAaBUzIVnmYVDBbnLxVNnfu1HgEBvCbL+71FrxMl3E6lpKH7Ge3OXA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.4",
          "npm": ">=1.2"
        }
      },
      "node_modules/ecc-jsbn": {
        "name": "ecc-jsbn",
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
        "integrity": "sha1-OoOpBOVDUyh4dMVkt1SThoSamMk=",
        "license": "MIT",
        "dependencies": {
          "jsbn": "~0.1.0",
          "safer-buffer": "^2.1.0"
        }
      },
      "node_modules/emoji-regex": {
        "name": "emoji-regex",
        "version": "7.0.3",
        "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
        "integrity": "sha512-CwBLREIQ7LvYFB0WyRvwhq5N5qPhc6PMjD6bYggFlI5YyDgl+0vxq5VHbMOFqLg7hfWzmu8T5Z1QofhmTIhItA==",
        "license": "MIT"
      },
      "node_modules/end-of-stream": {
        "name": "end-of-stream",
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/end-of-stream/-/end-of-stream-1.4.1.tgz",
        "integrity": "sha512-1MkrZNvWTKCaigbn+W15elq2BB/L22nqrSY5DKlo3X6+vclJm8Bb5djXJBmEX6fS3+zCh/F4VBK5Z2KxJt4s2Q==",
        "license": "MIT",
        "dependencies": {
          "once": "^1.4.0"
        }
      },
      "node_modules/error-ex": {
        "name": "error-ex",
        "version": "1.3.2",
        "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
        "integrity": "sha512-7dFHNmqeFSEt2ZBsCriorKnn3Z2pj+fd9kmI6QoWw4//DL+icEBfc0U7qJCisqrTsKTjw4fNFy2pW9OqStD84g==",
        "license": "MIT",
        "dependencies": {
          "is-arrayish": "^0.2.1"
        }
      },
      "node_modules/es6-error": {
        "name": "es6-error",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
        "integrity": "sha512-Um/+FxMr9CISWh0bi5Zv0iOD+4cFh5qLeks1qhAopKVAJw3drgKbKySikp7wGhDL0HPeaja0P5ULZrxLkniUVg==",
        "license": "MIT"
      },
      "node_modules/escape-string-regexp": {
        "name": "escape-string-regexp",
        "version": "1.0.5",
        "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
        "integrity": "sha1-G2HAViGQqN/2rjuyzwIAyhMLhtQ=",
        "license": "MIT",
        "engines": {
          "node": ">=0.8.0"
        }
      },
      "node_modules/esm": {
        "name": "esm",
        "version": "3.2.22",
        "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.22.tgz",
        "integrity": "sha512-z8YG7U44L82j1XrdEJcqZOLUnjxco8pO453gKOlaMD1/md1n/5QrscAmYG+oKUspsmDLuBFZrpbxI6aQ67yRxA==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/esprima": {
        "name": "esprima",
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
        "integrity": "sha512-eGuFFw7Upda+g4p+QHvnW0RyTX/SVeJBDM/gCtMARO0cLuT2HcEKnTPvhjV6aGeqrCB/sbNop0Kszm0jsaWU4A==",
        "license": "BSD-2-Clause",
        "bin": {
          "esparse": "bin/esparse.js",
          "esvalidate": "bin/esvalidate.js"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/esutils": {
        "name": "esutils",
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
        "integrity": "sha1-Cr9PHKpbyx96nYrMbepPqqBLrJs=",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/events-to-array": {
        "name": "events-to-array",
        "version": "1.1.2",
        "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
        "integrity": "sha1-LUH1Y+H+QA7Uli/hpNXGp1Od9/Y=",
        "license": "ISC"
      },
      "node_modules/execa": {
        "name": "execa",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/execa/-/execa-1.0.0.tgz",
        "integrity": "sha512-adbxcyWV46qiHyvSp50TKt05tB4tK3HcmF7/nxfAdhnox83seTDbwnaqKO4sXRy7roHAIFqJP/Rw/AuEbX61LA==",
        "license": "MIT",
        "dependencies": {
          "cross-spawn": "^6.0.0",
          "get-stream": "^4.0.0",
          "is-stream": "^1.1.0",
          "npm-run-path": "^2.0.0",
          "p-finally": "^1.0.0",
          "signal-exit": "^3.0.0",
          "strip-eof": "^1.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/execa/node_modules/cross-spawn": {
        "name": "cross-spawn",
        "version": "6.0.5",
        "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
        "integrity": "sha512-eTVLrBSt7fjbDygz805pMnstIs2VTBNkRm0qxZd+M7A5XDdxVRWO5MxGBXZhjY4cqLYLdtrGqRf8mBPmzwSpWQ==",
        "license": "MIT",
        "dependencies": {
          "nice-try": "^1.0.4",
          "path-key": "^2.0.1",
          "semver": "^5.5.0",
          "shebang-command": "^1.2.0",
          "which": "^1.2.9"
        },
        "engines": {
          "node": ">=4.8"
        }
      },
      "node_modules/execa/node_modules/semver": {
        "name": "semver",
        "version": "5.7.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
        "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/expand-brackets": {
        "name": "expand-brackets",
        "version": "2.1.4",
        "resolved": "https://registry.npmjs.org/expand-brackets/-/expand-brackets-2.1.4.tgz",
        "integrity": "sha1-t3c14xXOMPa27/D4OwQVGiJEliI=",
        "license": "MIT",
        "dependencies": {
          "debug": "^2.3.3",
          "define-property": "^0.2.5",
          "extend-shallow": "^2.0.1",
          "posix-character-classes": "^0.1.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/expand-brackets/node_modules/define-property": {
        "name": "define-property",
        "version": "0.2.5",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
        "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/expand-brackets/node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
        "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/expand-brackets/node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
        "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/expand-brackets/node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
        "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^0.1.6",
          "is-data-descriptor": "^0.1.4",
          "kind-of": "^5.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/expand-brackets/node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
        "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/extend": {
        "name": "extend",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
        "integrity": "sha512-fjquC59cD7CyW6urNXK0FBufkZcoiGG80wTuPujX590cB5Ttln20E2UB4S/WARVqhXffZl2LNgS+gQdPIIim/g==",
        "license": "MIT"
      },
      "node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-2.0.1.tgz",
        "integrity": "sha1-Ua99YUrZqfYQ6huvu5idaxxWiQ8=",
        "license": "MIT",
        "dependencies": {
          "is-extendable": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/extglob": {
        "name": "extglob",
        "version": "2.0.4",
        "resolved": "https://registry.npmjs.org/extglob/-/extglob-2.0.4.tgz",
        "integrity": "sha512-Nmb6QXkELsuBr24CJSkilo6UHHgbekK5UiZgfE6UHD3Eb27YC6oD+bhcT+tJ6cl8dmsgdQxnWlcry8ksBIBLpw==",
        "license": "MIT",
        "dependencies": {
          "array-unique": "^0.3.2",
          "define-property": "^1.0.0",
          "expand-brackets": "^2.1.4",
          "extend-shallow": "^2.0.1",
          "fragment-cache": "^0.2.1",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/extglob/node_modules/define-property": {
        "name": "define-property",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
        "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/extsprintf": {
        "name": "extsprintf",
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
        "integrity": "sha1-lpGEQOMEGnpBT4xS48V06zw+HgU=",
        "engines": [
          "node >=0.6.0"
        ],
        "license": "MIT"
      },
      "node_modules/fast-deep-equal": {
        "name": "fast-deep-equal",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
        "integrity": "sha1-ewUhjd+WZ79/Nwv3/bLLFf3Qqkk=",
        "license": "MIT"
      },
      "node_modules/fast-json-stable-stringify": {
        "name": "fast-json-stable-stringify",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.0.0.tgz",
        "integrity": "sha1-1RQsDK7msRifh9OnYREGT4bIu/I=",
        "license": "MIT"
      },
      "node_modules/fill-range": {
        "name": "fill-range",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-4.0.0.tgz",
        "integrity": "sha1-1USBHUKPmOsGpj3EAtJAPDKMOPc=",
        "license": "MIT",
        "dependencies": {
          "extend-shallow": "^2.0.1",
          "is-number": "^3.0.0",
          "repeat-string": "^1.6.1",
          "to-regex-range": "^2.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/find-cache-dir": {
        "name": "find-cache-dir",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
        "integrity": "sha512-Tq6PixE0w/VMFfCgbONnkiQIVol/JJL7nRMi20fqzA4NRs9AfeqMGeRdPi3wIhYkxjeBaWh2rxwapn5Tu3IqOQ==",
        "license": "MIT",
        "dependencies": {
          "commondir": "^1.0.1",
          "make-dir": "^2.0.0",
          "pkg-dir": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/find-up": {
        "name": "find-up",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
        "integrity": "sha512-1yD6RmLI1XBfxugvORwlck6f75tYL+iR0jqwsOrOxMZyGYqUuDhJ0l4AXdO1iX/FTs9cBAMEk1gWSEx1kSbylg==",
        "license": "MIT",
        "dependencies": {
          "locate-path": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/findit": {
        "name": "findit",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
        "integrity": "sha1-ZQnwEmr0wXhVHPqZOU4DLhOk1W4=",
        "license": "MIT"
      },
      "node_modules/for-in": {
        "name": "for-in",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/for-in/-/for-in-1.0.2.tgz",
        "integrity": "sha1-gQaNKVqBQuwKxybG4iAMMPttXoA=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/foreground-child": {
        "name": "foreground-child",
        "version": "1.5.6",
        "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
        "integrity": "sha1-T9ca0t/elnibmApcCilZN8svXOk=",
        "license": "ISC",
        "dependencies": {
          "cross-spawn": "^4",
          "signal-exit": "^3.0.0"
        }
      },
      "node_modules/forever-agent": {
        "name": "forever-agent",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
        "integrity": "sha1-+8cfDEGt6zf5bFd60e1C2P2sypE=",
        "license": "Apache-2.0",
        "engines": {
          "node": "*"
        }
      },
      "node_modules/form-data": {
        "name": "form-data",
        "version": "2.3.3",
        "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
        "integrity": "sha512-1lLKB2Mu3aGP1Q/2eCOx0fNbRMe7XdwktwOruhfqqd0rIJWwN4Dh+E3hrPSlDCXnSR7UtZ1N38rVXm+6+MEhJQ==",
        "license": "MIT",
        "dependencies": {
          "asynckit": "^0.4.0",
          "combined-stream": "^1.0.6",
          "mime-types": "^2.1.12"
        },
        "engines": {
          "node": ">= 0.12"
        }
      },
      "node_modules/fragment-cache": {
        "name": "fragment-cache",
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/fragment-cache/-/fragment-cache-0.2.1.tgz",
        "integrity": "sha1-QpD60n8T6Jvn8zeZxrxaCr//DRk=",
        "license": "MIT",
        "dependencies": {
          "map-cache": "^0.2.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fs-exists-cached": {
        "name": "fs-exists-cached",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
        "integrity": "sha1-zyVVTKBQ3EmuZla0HeQiWJidy84=",
        "license": "ISC"
      },
      "node_modules/fs.realpath": {
        "name": "fs.realpath",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
        "integrity": "sha1-FQStJSMVjKpA20onh8sBQRmU6k8=",
        "license": "ISC"
      },
      "node_modules/fsevents": {
        "name": "fsevents",
        "version": "1.2.8",
        "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-1.2.8.tgz",
        "integrity": "sha512-tPvHgPGB7m40CZ68xqFGkKuzN+RnpGmSV+hgeKxhRpbxdqKXUFJGC3yonBOLzQBcJyGpdZFDfCsdOC2KFsXzeA==",
        "bundleDependencies": [
          "node-pre-gyp"
        ],
        "deprecated": "fsevents 1 will break on node v14+ and could be using insecure binaries. Upgrade to fsevents 2.",
        "hasInstallScript": true,
        "license": "MIT",
        "optional": true,
        "os": [
          "darwin"
        ],
        "dependencies": {
          "nan": "^2.12.1",
          "node-pre-gyp": "^0.12.0"
        },
        "engines": {
          "node": ">=4.0"
        }
      },
      "node_modules/fsevents/node_modules/abbrev": {
        "name": "abbrev",
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
        "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
        "integrity": "sha1-w7M6te42DYbg5ijwRorn7yfWVN8=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/aproba": {
        "name": "aproba",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/aproba/-/aproba-1.2.0.tgz",
        "integrity": "sha512-Y9J6ZjXtoYh8RnXVCMOU/ttDmk1aBjunq9vO0ta5x85WDQiQfUF9sIPBITdbiiIVcBo03Hi3jMxigBtsddlXRw==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/are-we-there-yet": {
        "name": "are-we-there-yet",
        "version": "1.1.5",
        "resolved": "https://registry.npmjs.org/are-we-there-yet/-/are-we-there-yet-1.1.5.tgz",
        "integrity": "sha512-5hYdAkZlcG8tOLujVDTgCT+uPX0VnpAH28gWsLfzpXYm7wP6mp5Q/gYyR7YQ0cKVJcXJnl3j2kpBan13PtQf6w==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "delegates": "^1.0.0",
          "readable-stream": "^2.0.6"
        }
      },
      "node_modules/fsevents/node_modules/balanced-match": {
        "name": "balanced-match",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
        "integrity": "sha1-ibTRmasr7kneFk6gK4nORi1xt2c=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/brace-expansion": {
        "name": "brace-expansion",
        "version": "1.1.11",
        "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
        "integrity": "sha512-iCuPHDFgrHX7H2vEI/5xpz07zSHB00TpugqhmYtVmMO6518mCuRMoOYFldEBl0g187ufozdaHgWKcYFb61qGiA==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "balanced-match": "^1.0.0",
          "concat-map": "0.0.1"
        }
      },
      "node_modules/fsevents/node_modules/chownr": {
        "name": "chownr",
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/chownr/-/chownr-1.1.1.tgz",
        "integrity": "sha512-j38EvO5+LHX84jlo6h4UzmOwi0UgW61WRyPtJz4qaadK5eY3BTS5TY/S1Stc3Uk2lIM6TPevAlULiEJwie860g==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/code-point-at": {
        "name": "code-point-at",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
        "integrity": "sha1-DQcLTQQ6W+ozovGkDi7bPZpMz3c=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/concat-map": {
        "name": "concat-map",
        "version": "0.0.1",
        "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
        "integrity": "sha1-2Klr13/Wjfd5OnMDajug1UBdR3s=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/console-control-strings": {
        "name": "console-control-strings",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/console-control-strings/-/console-control-strings-1.1.0.tgz",
        "integrity": "sha1-PXz0Rk22RG6mRL9LOVB/mFEAjo4=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/core-util-is": {
        "name": "core-util-is",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
        "integrity": "sha1-tf1UIgqivFq1eqtxQMlAdUUDwac=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/debug": {
        "name": "debug",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
        "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "ms": "^2.1.1"
        }
      },
      "node_modules/fsevents/node_modules/deep-extend": {
        "name": "deep-extend",
        "version": "0.6.0",
        "resolved": "https://registry.npmjs.org/deep-extend/-/deep-extend-0.6.0.tgz",
        "integrity": "sha512-LOHxIOaPYdHlJRtCQfDIVZtfw/ufM8+rVj649RIHzcm/vGwQRXFt6OPqIFWsm2XEMrNIEtWR64sY1LEKD2vAOA==",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=4.0.0"
        }
      },
      "node_modules/fsevents/node_modules/delegates": {
        "name": "delegates",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/delegates/-/delegates-1.0.0.tgz",
        "integrity": "sha1-hMbhWbgZBP3KWaDvRM2HDTElD5o=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/detect-libc": {
        "name": "detect-libc",
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/detect-libc/-/detect-libc-1.0.3.tgz",
        "integrity": "sha1-+hN8S9aY7fVc1c0CrFWfkaTEups=",
        "inBundle": true,
        "license": "Apache-2.0",
        "bin": {
          "detect-libc": "bin/detect-libc.js"
        },
        "engines": {
          "node": ">=0.10"
        }
      },
      "node_modules/fsevents/node_modules/fs-minipass": {
        "name": "fs-minipass",
        "version": "1.2.5",
        "resolved": "https://registry.npmjs.org/fs-minipass/-/fs-minipass-1.2.5.tgz",
        "integrity": "sha512-JhBl0skXjUPCFH7x6x61gQxrKyXsxB5gcgePLZCwfyCGGsTISMoIeObbrvVeP6Xmyaudw4TT43qV2Gz+iyd2oQ==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "minipass": "^2.2.1"
        }
      },
      "node_modules/fsevents/node_modules/fs.realpath": {
        "name": "fs.realpath",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
        "integrity": "sha1-FQStJSMVjKpA20onh8sBQRmU6k8=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/gauge": {
        "name": "gauge",
        "version": "2.7.4",
        "resolved": "https://registry.npmjs.org/gauge/-/gauge-2.7.4.tgz",
        "integrity": "sha1-LANAXHU4w51+s3sxcCLjJfsBi/c=",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "aproba": "^1.0.3",
          "console-control-strings": "^1.0.0",
          "has-unicode": "^2.0.0",
          "object-assign": "^4.1.0",
          "signal-exit": "^3.0.0",
          "string-width": "^1.0.1",
          "strip-ansi": "^3.0.1",
          "wide-align": "^1.1.0"
        }
      },
      "node_modules/fsevents/node_modules/glob": {
        "name": "glob",
        "version": "7.1.3",
        "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.3.tgz",
        "integrity": "sha512-vcfuiIxogLV4DlGBHIUOwI0IbrJ8HWPc4MU7HzviGeNho/UJDfi6B5p3sHeWIQ0KGIU0Jpxi5ZHxemQfLkkAwQ==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "fs.realpath": "^1.0.0",
          "inflight": "^1.0.4",
          "inherits": "2",
          "minimatch": "^3.0.4",
          "once": "^1.3.0",
          "path-is-absolute": "^1.0.0"
        },
        "engines": {
          "node": "*"
        }
      },
      "node_modules/fsevents/node_modules/has-unicode": {
        "name": "has-unicode",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/has-unicode/-/has-unicode-2.0.1.tgz",
        "integrity": "sha1-4Ob+aijPUROIVeCG0Wkedx3iqLk=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/iconv-lite": {
        "name": "iconv-lite",
        "version": "0.4.24",
        "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
        "integrity": "sha512-v3MXnZAcvnywkTUEZomIActle7RXXeedOR31wwl7VlyoXO4Qi9arvSenNQWne1TcRwhCL1HwLI21bEqdpj8/rA==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "safer-buffer": ">= 2.1.2 < 3"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/ignore-walk": {
        "name": "ignore-walk",
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/ignore-walk/-/ignore-walk-3.0.1.tgz",
        "integrity": "sha512-DTVlMx3IYPe0/JJcYP7Gxg7ttZZu3IInhuEhbchuqneY9wWe5Ojy2mXLBaQFUQmo0AW2r3qG7m1mg86js+gnlQ==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "minimatch": "^3.0.4"
        }
      },
      "node_modules/fsevents/node_modules/inflight": {
        "name": "inflight",
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
        "integrity": "sha1-Sb1jMdfQLQwJvJEKEHW6gWW1bfk=",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "once": "^1.3.0",
          "wrappy": "1"
        }
      },
      "node_modules/fsevents/node_modules/inherits": {
        "name": "inherits",
        "version": "2.0.3",
        "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
        "integrity": "sha1-Yzwsg+PaQqUC9SRmAiSA9CCCYd4=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/ini": {
        "name": "ini",
        "version": "1.3.5",
        "resolved": "https://registry.npmjs.org/ini/-/ini-1.3.5.tgz",
        "integrity": "sha512-RZY5huIKCMRWDUqZlEi72f/lmXKMvuszcMBduliQ3nnWbx9X/ZBQO7DijMEYS9EhHBb2qacRUMtC7svLwe0lcw==",
        "inBundle": true,
        "license": "ISC",
        "engines": {
          "node": "*"
        }
      },
      "node_modules/fsevents/node_modules/is-fullwidth-code-point": {
        "name": "is-fullwidth-code-point",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
        "integrity": "sha1-754xOG8DGn8NZDr4L95QxFfvAMs=",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "number-is-nan": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/isarray": {
        "name": "isarray",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
        "integrity": "sha1-u5NdSFgsuhaMBoNJV6VKPgcSTxE=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/minimatch": {
        "name": "minimatch",
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
        "integrity": "sha512-yJHVQEhyqPLUTgt9B83PXu6W3rx4MvvHvSUvToogpwoGDOUQ+yDrR0HRot+yOCdCO7u4hX3pWft6kWBBcqh0UA==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "brace-expansion": "^1.1.7"
        },
        "engines": {
          "node": "*"
        }
      },
      "node_modules/fsevents/node_modules/minimist": {
        "name": "minimist",
        "version": "0.0.8",
        "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
        "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/minipass": {
        "name": "minipass",
        "version": "2.3.5",
        "resolved": "https://registry.npmjs.org/minipass/-/minipass-2.3.5.tgz",
        "integrity": "sha512-Gi1W4k059gyRbyVUZQ4mEqLm0YIUiGYfvxhF6SIlk3ui1WVxMTGfGdQ2SInh3PDrRTVvPKgULkpJtT4RH10+VA==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "safe-buffer": "^5.1.2",
          "yallist": "^3.0.0"
        }
      },
      "node_modules/fsevents/node_modules/minizlib": {
        "name": "minizlib",
        "version": "1.2.1",
        "resolved": "https://registry.npmjs.org/minizlib/-/minizlib-1.2.1.tgz",
        "integrity": "sha512-7+4oTUOWKg7AuL3vloEWekXY2/D20cevzsrNT2kGWm+39J9hGTCBv8VI5Pm5lXZ/o3/mdR4f8rflAPhnQb8mPA==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "minipass": "^2.2.1"
        }
      },
      "node_modules/fsevents/node_modules/mkdirp": {
        "name": "mkdirp",
        "version": "0.5.1",
        "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
        "integrity": "sha1-MAV0OOrGz3+MR2fzhkjWaX11yQM=",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "minimist": "0.0.8"
        },
        "bin": {
          "mkdirp": "bin/cmd.js"
        }
      },
      "node_modules/fsevents/node_modules/ms": {
        "name": "ms",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
        "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg==",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/needle": {
        "name": "needle",
        "version": "2.3.0",
        "resolved": "https://registry.npmjs.org/needle/-/needle-2.3.0.tgz",
        "integrity": "sha512-QBZu7aAFR0522EyaXZM0FZ9GLpq6lvQ3uq8gteiDUp7wKdy0lSd2hPlgFwVuW1CBkfEs9PfDQsQzZghLs/psdg==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "debug": "^4.1.0",
          "iconv-lite": "^0.4.4",
          "sax": "^1.2.4"
        },
        "bin": {
          "needle": "bin/needle"
        },
        "engines": {
          "node": ">= 4.4.x"
        }
      },
      "node_modules/fsevents/node_modules/node-pre-gyp": {
        "name": "node-pre-gyp",
        "version": "0.12.0",
        "resolved": "https://registry.npmjs.org/node-pre-gyp/-/node-pre-gyp-0.12.0.tgz",
        "integrity": "sha512-4KghwV8vH5k+g2ylT+sLTjy5wmUOb9vPhnM8NHvRf9dHmnW/CndrFXy2aRPaPST6dugXSdHXfeaHQm77PIz/1A==",
        "inBundle": true,
        "license": "BSD-3-Clause",
        "dependencies": {
          "detect-libc": "^1.0.2",
          "mkdirp": "^0.5.1",
          "needle": "^2.2.1",
          "nopt": "^4.0.1",
          "npm-packlist": "^1.1.6",
          "npmlog": "^4.0.2",
          "rc": "^1.2.7",
          "rimraf": "^2.6.1",
          "semver": "^5.3.0",
          "tar": "^4"
        },
        "bin": {
          "node-pre-gyp": "bin/node-pre-gyp"
        }
      },
      "node_modules/fsevents/node_modules/nopt": {
        "name": "nopt",
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/nopt/-/nopt-4.0.1.tgz",
        "integrity": "sha1-0NRoWv1UFRk8jHUFYC0NF81kR00=",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "abbrev": "1",
          "osenv": "^0.1.4"
        },
        "bin": {
          "nopt": "bin/nopt.js"
        }
      },
      "node_modules/fsevents/node_modules/npm-bundled": {
        "name": "npm-bundled",
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/npm-bundled/-/npm-bundled-1.0.6.tgz",
        "integrity": "sha512-8/JCaftHwbd//k6y2rEWp6k1wxVfpFzB6t1p825+cUb7Ym2XQfhwIC5KwhrvzZRJu+LtDE585zVaS32+CGtf0g==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/npm-packlist": {
        "name": "npm-packlist",
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/npm-packlist/-/npm-packlist-1.4.1.tgz",
        "integrity": "sha512-+TcdO7HJJ8peiiYhvPxsEDhF3PJFGUGRcFsGve3vxvxdcpO2Z4Z7rkosRM0kWj6LfbK/P0gu3dzk5RU1ffvFcw==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "ignore-walk": "^3.0.1",
          "npm-bundled": "^1.0.1"
        }
      },
      "node_modules/fsevents/node_modules/npmlog": {
        "name": "npmlog",
        "version": "4.1.2",
        "resolved": "https://registry.npmjs.org/npmlog/-/npmlog-4.1.2.tgz",
        "integrity": "sha512-2uUqazuKlTaSI/dC8AzicUck7+IrEaOnN/e0jd3Xtt1KcGpwx30v50mL7oPyr/h9bL3E4aZccVwpwP+5W9Vjkg==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "are-we-there-yet": "~1.1.2",
          "console-control-strings": "~1.1.0",
          "gauge": "~2.7.3",
          "set-blocking": "~2.0.0"
        }
      },
      "node_modules/fsevents/node_modules/number-is-nan": {
        "name": "number-is-nan",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
        "integrity": "sha1-CXtgK1NCKlIsGvuHkDGDNpQaAR0=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/object-assign": {
        "name": "object-assign",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
        "integrity": "sha1-IQmtx5ZYh8/AXLvUQsrIv7s2CGM=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/once": {
        "name": "once",
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
        "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "wrappy": "1"
        }
      },
      "node_modules/fsevents/node_modules/os-homedir": {
        "name": "os-homedir",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
        "integrity": "sha1-/7xJiDNuDoM94MFox+8VISGqf7M=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/os-tmpdir": {
        "name": "os-tmpdir",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
        "integrity": "sha1-u+Z0BseaqFxc/sdm/lc0VV36EnQ=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/osenv": {
        "name": "osenv",
        "version": "0.1.5",
        "resolved": "https://registry.npmjs.org/osenv/-/osenv-0.1.5.tgz",
        "integrity": "sha512-0CWcCECdMVc2Rw3U5w9ZjqX6ga6ubk1xDVKxtBQPK7wis/0F2r9T6k4ydGYhecl7YUBxBVxhL5oisPsNxAPe2g==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "os-homedir": "^1.0.0",
          "os-tmpdir": "^1.0.0"
        }
      },
      "node_modules/fsevents/node_modules/path-is-absolute": {
        "name": "path-is-absolute",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
        "integrity": "sha1-F0uSaHNVNP+8es5r9TpanhtcX18=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/process-nextick-args": {
        "name": "process-nextick-args",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.0.tgz",
        "integrity": "sha512-MtEC1TqN0EU5nephaJ4rAtThHtC86dNN9qCuEhtshvpVBkAW5ZO7BASN9REnF9eoXGcRub+pFuKEpOHE+HbEMw==",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/rc": {
        "name": "rc",
        "version": "1.2.8",
        "resolved": "https://registry.npmjs.org/rc/-/rc-1.2.8.tgz",
        "integrity": "sha512-y3bGgqKj3QBdxLbLkomlohkvsA8gdAiUQlSBJnBhfn+BPxg4bc62d8TcBW15wavDfgexCgccckhcZvywyQYPOw==",
        "inBundle": true,
        "license": "(BSD-2-Clause OR MIT OR Apache-2.0)",
        "dependencies": {
          "deep-extend": "^0.6.0",
          "ini": "~1.3.0",
          "minimist": "^1.2.0",
          "strip-json-comments": "~2.0.1"
        },
        "bin": {
          "rc": "cli.js"
        }
      },
      "node_modules/fsevents/node_modules/rc/node_modules/minimist": {
        "name": "minimist",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
        "integrity": "sha1-o1AIsg9BOD7sH7kU9M1d95omQoQ=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/readable-stream": {
        "name": "readable-stream",
        "version": "2.3.6",
        "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
        "integrity": "sha512-tQtKA9WIAhBF3+VLAseyMqZeBjW0AHJoxOtYqSUZNJxauErmLbVm2FW1y+J/YA9dUrAC39ITejlZWhVIwawkKw==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "core-util-is": "~1.0.0",
          "inherits": "~2.0.3",
          "isarray": "~1.0.0",
          "process-nextick-args": "~2.0.0",
          "safe-buffer": "~5.1.1",
          "string_decoder": "~1.1.1",
          "util-deprecate": "~1.0.1"
        }
      },
      "node_modules/fsevents/node_modules/rimraf": {
        "name": "rimraf",
        "version": "2.6.3",
        "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
        "integrity": "sha512-mwqeW5XsA2qAejG46gYdENaxXjx9onRNCfn7L0duuP4hCuTIi/QO7PDK07KJfp1d+izWPrzEJDcSqBa0OZQriA==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "glob": "^7.1.3"
        },
        "bin": {
          "rimraf": "bin.js"
        }
      },
      "node_modules/fsevents/node_modules/safe-buffer": {
        "name": "safe-buffer",
        "version": "5.1.2",
        "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
        "integrity": "sha512-Gd2UZBJDkXlY7GbJxfsE8/nvKkUEU1G38c1siN6QP6a9PT9MmHB8GnpscSmMJSoF8LOIrt8ud/wPtojys4G6+g==",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/safer-buffer": {
        "name": "safer-buffer",
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
        "integrity": "sha512-YZo3K82SD7Riyi0E1EQPojLz7kpepnSQI9IyPbHHg1XXXevb5dJI7tpyN2ADxGcQbHG7vcyRHk0cbwqcQriUtg==",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/sax": {
        "name": "sax",
        "version": "1.2.4",
        "resolved": "https://registry.npmjs.org/sax/-/sax-1.2.4.tgz",
        "integrity": "sha512-NqVDv9TpANUjFm0N8uM5GxL36UgKi9/atZw+x7YFnQ8ckwFGKrl4xX4yWtrey3UJm5nP1kUbnYgLopqWNSRhWw==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/semver": {
        "name": "semver",
        "version": "5.7.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
        "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
        "inBundle": true,
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/fsevents/node_modules/set-blocking": {
        "name": "set-blocking",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
        "integrity": "sha1-BF+XgtARrppoA93TgrJDkrPYkPc=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/signal-exit": {
        "name": "signal-exit",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
        "integrity": "sha1-tf3AjxKH6hF4Yo5BXiUTK3NkbG0=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/string_decoder": {
        "name": "string_decoder",
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
        "integrity": "sha512-n/ShnvDi6FHbbVfviro+WojiFzv+s8MPMHBczVePfUpDJLwoLT0ht1l4YwBCbi8pJAveEEdnkHyPyTP/mzRfwg==",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "safe-buffer": "~5.1.0"
        }
      },
      "node_modules/fsevents/node_modules/string-width": {
        "name": "string-width",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
        "integrity": "sha1-EYvfW4zcUaKn5w0hHgfisLmxB9M=",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "code-point-at": "^1.0.0",
          "is-fullwidth-code-point": "^1.0.0",
          "strip-ansi": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
        "integrity": "sha1-ajhfuIU9lS1f8F0Oiq+UJ43GPc8=",
        "inBundle": true,
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/strip-json-comments": {
        "name": "strip-json-comments",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-2.0.1.tgz",
        "integrity": "sha1-PFMZQukIwml8DsNEhYwobHygpgo=",
        "inBundle": true,
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/fsevents/node_modules/tar": {
        "name": "tar",
        "version": "4.4.8",
        "resolved": "https://registry.npmjs.org/tar/-/tar-4.4.8.tgz",
        "integrity": "sha512-LzHF64s5chPQQS0IYBn9IN5h3i98c12bo4NCO7e0sGM2llXQ3p2FGC5sdENN4cTW48O915Sh+x+EXx7XW96xYQ==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "chownr": "^1.1.1",
          "fs-minipass": "^1.2.5",
          "minipass": "^2.3.4",
          "minizlib": "^1.1.1",
          "mkdirp": "^0.5.0",
          "safe-buffer": "^5.1.2",
          "yallist": "^3.0.2"
        },
        "engines": {
          "node": ">=4.5"
        }
      },
      "node_modules/fsevents/node_modules/util-deprecate": {
        "name": "util-deprecate",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
        "integrity": "sha1-RQ1Nyfpw3nMnYvvS1KKJgUGaDM8=",
        "inBundle": true,
        "license": "MIT"
      },
      "node_modules/fsevents/node_modules/wide-align": {
        "name": "wide-align",
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/wide-align/-/wide-align-1.1.3.tgz",
        "integrity": "sha512-QGkOQc8XL6Bt5PwnsExKBPuMKBxnGxWWW3fU55Xt4feHozMUhdUMaBCk290qpm/wG5u/RSKzwdAC4i51YigihA==",
        "inBundle": true,
        "license": "ISC",
        "dependencies": {
          "string-width": "^1.0.2 || 2"
        }
      },
      "node_modules/fsevents/node_modules/wrappy": {
        "name": "wrappy",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
        "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8=",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/fsevents/node_modules/yallist": {
        "name": "yallist",
        "version": "3.0.3",
        "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.0.3.tgz",
        "integrity": "sha512-S+Zk8DEWE6oKpV+vI3qWkaK+jSbIK86pCwe2IF/xwIpQ8jEuxpw9NyaGjmp9+BoJv5FV2piqCDcoCtStppiq2A==",
        "inBundle": true,
        "license": "ISC"
      },
      "node_modules/function-loop": {
        "name": "function-loop",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
        "integrity": "sha512-Iw4MzMfS3udk/rqxTiDDCllhGwlOrsr50zViTOO/W6lS/9y6B1J0BD2VZzrnWUYBJsl3aeqjgR5v7bWWhZSYbA==",
        "license": "ISC"
      },
      "node_modules/get-caller-file": {
        "name": "get-caller-file",
        "version": "2.0.5",
        "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
        "integrity": "sha512-DyFP3BM/3YHTQOCUL/w0OZHR0lpKeGrxotcHWcqNEdnltqFwXVfhEBQ94eIo34AfQpo0rGki4cyIiftY06h2Fg==",
        "license": "ISC",
        "engines": {
          "node": "6.* || 8.* || >= 10.*"
        }
      },
      "node_modules/get-stream": {
        "name": "get-stream",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/get-stream/-/get-stream-4.1.0.tgz",
        "integrity": "sha512-GMat4EJ5161kIy2HevLlr4luNjBgvmj413KaQA7jt4V8B4RDsfpHk7WQ9GVqfYyyx8OS/L66Kox+rJRNklLK7w==",
        "license": "MIT",
        "dependencies": {
          "pump": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/get-value": {
        "name": "get-value",
        "version": "2.0.6",
        "resolved": "https://registry.npmjs.org/get-value/-/get-value-2.0.6.tgz",
        "integrity": "sha1-3BXKHGcjh8p2vTesCjlbogQqLCg=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/getpass": {
        "name": "getpass",
        "version": "0.1.7",
        "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
        "integrity": "sha1-Xv+OPmhNVprkyysSgmBOi6YhSfo=",
        "license": "MIT",
        "dependencies": {
          "assert-plus": "^1.0.0"
        }
      },
      "node_modules/glob": {
        "name": "glob",
        "version": "7.1.3",
        "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.3.tgz",
        "integrity": "sha512-vcfuiIxogLV4DlGBHIUOwI0IbrJ8HWPc4MU7HzviGeNho/UJDfi6B5p3sHeWIQ0KGIU0Jpxi5ZHxemQfLkkAwQ==",
        "license": "ISC",
        "dependencies": {
          "fs.realpath": "^1.0.0",
          "inflight": "^1.0.4",
          "inherits": "2",
          "minimatch": "^3.0.4",
          "once": "^1.3.0",
          "path-is-absolute": "^1.0.0"
        },
        "engines": {
          "node": "*"
        }
      },
      "node_modules/glob-parent": {
        "name": "glob-parent",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-3.1.0.tgz",
        "integrity": "sha1-nmr2KZ2NO9K9QEMIMr0RPfkGxa4=",
        "license": "ISC",
        "dependencies": {
          "is-glob": "^3.1.0",
          "path-dirname": "^1.0.0"
        }
      },
      "node_modules/glob-parent/node_modules/is-glob": {
        "name": "is-glob",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-3.1.0.tgz",
        "integrity": "sha1-e6WuJCF4BKxwcHuWkiVnSGzD6Eo=",
        "license": "MIT",
        "dependencies": {
          "is-extglob": "^2.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/globals": {
        "name": "globals",
        "version": "9.18.0",
        "resolved": "https://registry.npmjs.org/globals/-/globals-9.18.0.tgz",
        "integrity": "sha512-S0nG3CLEQiY/ILxqtztTWH/3iRRdyBLw6KMDxnKMchrtbj2OFmehVh0WUCfW3DUrIgx/qFrJPICrq4Z4sTR9UQ==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/graceful-fs": {
        "name": "graceful-fs",
        "version": "4.1.15",
        "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.1.15.tgz",
        "integrity": "sha512-6uHUhOPEBgQ24HM+r6b/QwWfZq+yiFcipKFrOFiBEnWdy5sdzYoi+pJeQaPI5qOLRFqWmAXUPQNsielzdLoecA==",
        "license": "ISC"
      },
      "node_modules/growl": {
        "name": "growl",
        "version": "1.10.5",
        "resolved": "https://registry.npmjs.org/growl/-/growl-1.10.5.tgz",
        "integrity": "sha512-qBr4OuELkhPenW6goKVXiv47US3clb3/IbuWF9KNKEijAy9oeHxU9IgzjvJhHkUzhaj7rOUD7+YGWqUjLp5oSA==",
        "license": "MIT",
        "engines": {
          "node": ">=4.x"
        }
      },
      "node_modules/handlebars": {
        "name": "handlebars",
        "version": "4.1.2",
        "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.1.2.tgz",
        "integrity": "sha512-nvfrjqvt9xQ8Z/w0ijewdD/vvWDTOweBUm96NTr66Wfvo1mJenBLwcYmPs3TIBP5ruzYGD7Hx/DaM9RmhroGPw==",
        "license": "MIT",
        "dependencies": {
          "neo-async": "^2.6.0",
          "optimist": "^0.6.1",
          "source-map": "^0.6.1",
          "uglify-js": "^3.1.4"
        },
        "bin": {
          "handlebars": "bin/handlebars"
        },
        "engines": {
          "node": ">=0.4.7"
        },
        "optionalDependencies": {
          "uglify-js": "^3.1.4"
        }
      },
      "node_modules/handlebars/node_modules/source-map": {
        "name": "source-map",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
        "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/har-schema": {
        "name": "har-schema",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
        "integrity": "sha1-qUwiJOvKwEeCoNkDVSHyRzW37JI=",
        "license": "ISC",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/har-validator": {
        "name": "har-validator",
        "version": "5.1.3",
        "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
        "integrity": "sha512-sNvOCzEQNr/qrvJgc3UG/kD4QtlHycrzwS+6mfTrrSq97BvaYcPZZI1ZSqGSPR73Cxn4LKTD4PttRwfU7jWq5g==",
        "license": "MIT",
        "dependencies": {
          "ajv": "^6.5.5",
          "har-schema": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/has-ansi": {
        "name": "has-ansi",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/has-ansi/-/has-ansi-2.0.0.tgz",
        "integrity": "sha1-NPUEnOHs3ysGSa8+8k5F7TVBbZE=",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/has-flag": {
        "name": "has-flag",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
        "integrity": "sha1-tdRU3CGZriJWmfNGfloH87lVuv0=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/has-value": {
        "name": "has-value",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/has-value/-/has-value-1.0.0.tgz",
        "integrity": "sha1-GLKB2lhbHFxR3vJMkw7SmgvmsXc=",
        "license": "MIT",
        "dependencies": {
          "get-value": "^2.0.6",
          "has-values": "^1.0.0",
          "isobject": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/has-values": {
        "name": "has-values",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/has-values/-/has-values-1.0.0.tgz",
        "integrity": "sha1-lbC2P+whRmGab+V/51Yo1aOe/k8=",
        "license": "MIT",
        "dependencies": {
          "is-number": "^3.0.0",
          "kind-of": "^4.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/has-values/node_modules/kind-of": {
        "name": "kind-of",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-4.0.0.tgz",
        "integrity": "sha1-IIE989cSkosgc3hpGkUGb65y3Vc=",
        "license": "MIT",
        "dependencies": {
          "is-buffer": "^1.1.5"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/hasha": {
        "name": "hasha",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
        "integrity": "sha1-UqMvq4Vp1BymmmH/GiFPjrfIvTk=",
        "license": "MIT",
        "dependencies": {
          "is-stream": "^1.0.1"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/home-or-tmp": {
        "name": "home-or-tmp",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/home-or-tmp/-/home-or-tmp-2.0.0.tgz",
        "integrity": "sha1-42w/LSyufXRqhX440Y1fMqeILbg=",
        "license": "MIT",
        "dependencies": {
          "os-homedir": "^1.0.0",
          "os-tmpdir": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/hosted-git-info": {
        "name": "hosted-git-info",
        "version": "2.7.1",
        "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.7.1.tgz",
        "integrity": "sha512-7T/BxH19zbcCTa8XkMlbK5lTo1WtgkFi3GvdWEyNuc4Vex7/9Dqbnpsf4JMydcfj9HCg4zUWFTL3Za6lapg5/w==",
        "license": "ISC"
      },
      "node_modules/http-signature": {
        "name": "http-signature",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
        "integrity": "sha1-muzZJRFHcvPZW2WmCruPfBj7rOE=",
        "license": "MIT",
        "dependencies": {
          "assert-plus": "^1.0.0",
          "jsprim": "^1.2.2",
          "sshpk": "^1.7.0"
        },
        "engines": {
          "node": ">=0.8",
          "npm": ">=1.3.7"
        }
      },
      "node_modules/import-jsx": {
        "name": "import-jsx",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/import-jsx/-/import-jsx-2.0.0.tgz",
        "integrity": "sha512-xmrgtiRnAdjIaRzKwsHut54FA8nx59WqN4MpQvPFr/8yD6BamavkmKHrA5dotAlnIiF4uqMzg/lA5yhPdpIXsA==",
        "license": "MIT",
        "dependencies": {
          "babel-core": "^6.25.0",
          "babel-plugin-transform-es2015-destructuring": "^6.23.0",
          "babel-plugin-transform-object-rest-spread": "^6.23.0",
          "babel-plugin-transform-react-jsx": "^6.24.1",
          "caller-path": "^2.0.0",
          "resolve-from": "^3.0.0"
        },
        "engines": {
          "node": ">= 4"
        }
      },
      "node_modules/imurmurhash": {
        "name": "imurmurhash",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
        "integrity": "sha1-khi5srkoojixPcT7a21XbyMUU+o=",
        "license": "MIT",
        "engines": {
          "node": ">=0.8.19"
        }
      },
      "node_modules/inflight": {
        "name": "inflight",
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
        "integrity": "sha1-Sb1jMdfQLQwJvJEKEHW6gWW1bfk=",
        "license": "ISC",
        "dependencies": {
          "once": "^1.3.0",
          "wrappy": "1"
        }
      },
      "node_modules/inherits": {
        "name": "inherits",
        "version": "2.0.3",
        "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
        "integrity": "sha1-Yzwsg+PaQqUC9SRmAiSA9CCCYd4=",
        "license": "ISC"
      },
      "node_modules/ink": {
        "name": "ink",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ink/-/ink-2.1.1.tgz",
        "integrity": "sha512-vP1yE/uJoiY6uB9yHalczUA02I9fg7xDUbTEZitPK5y6dvnPo9a/6UWqIB2uCYkHOhEZMN+D/TsVr4v2sz8qYA==",
        "license": "MIT",
        "dependencies": {
          "@types/react": "^16.8.6",
          "arrify": "^1.0.1",
          "auto-bind": "^2.0.0",
          "chalk": "^2.4.1",
          "cli-cursor": "^2.1.0",
          "cli-truncate": "^1.1.0",
          "is-ci": "^2.0.0",
          "lodash.throttle": "^4.1.1",
          "log-update": "^3.0.0",
          "prop-types": "^15.6.2",
          "react-reconciler": "^0.20.0",
          "scheduler": "^0.13.2",
          "signal-exit": "^3.0.2",
          "slice-ansi": "^1.0.0",
          "string-length": "^2.0.0",
          "widest-line": "^2.0.0",
          "wrap-ansi": "^5.0.0",
          "yoga-layout-prebuilt": "^1.9.3"
        },
        "engines": {
          "node": ">=8"
        },
        "peerDependencies": {
          "react": ">=16.8.0"
        }
      },
      "node_modules/ink/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
        "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/ink/node_modules/ansi-styles": {
        "name": "ansi-styles",
        "version": "3.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
        "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
        "license": "MIT",
        "dependencies": {
          "color-convert": "^1.9.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/ink/node_modules/chalk": {
        "name": "chalk",
        "version": "2.4.2",
        "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
        "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^3.2.1",
          "escape-string-regexp": "^1.0.5",
          "supports-color": "^5.3.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/ink/node_modules/string-width": {
        "name": "string-width",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
        "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
        "license": "MIT",
        "dependencies": {
          "emoji-regex": "^7.0.1",
          "is-fullwidth-code-point": "^2.0.0",
          "strip-ansi": "^5.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/ink/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "5.2.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
        "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^4.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/ink/node_modules/supports-color": {
        "name": "supports-color",
        "version": "5.5.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
        "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
        "license": "MIT",
        "dependencies": {
          "has-flag": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/ink/node_modules/wrap-ansi": {
        "name": "wrap-ansi",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
        "integrity": "sha512-QC1/iN/2/RPVJ5jYK8BGttj5z83LmSKmvbvrXPNCLZSEb32KKVDJDl/MOt2N01qU2H/FkzEa9PKto1BqDjtd7Q==",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^3.2.0",
          "string-width": "^3.0.0",
          "strip-ansi": "^5.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/invariant": {
        "name": "invariant",
        "version": "2.2.4",
        "resolved": "https://registry.npmjs.org/invariant/-/invariant-2.2.4.tgz",
        "integrity": "sha512-phJfQVBuaJM5raOpJjSfkiD6BpbCE4Ns//LaXl6wGYtUBY83nWS6Rf9tXm2e8VaK60JEjYldbPif/A2B1C2gNA==",
        "license": "MIT",
        "dependencies": {
          "loose-envify": "^1.0.0"
        }
      },
      "node_modules/invert-kv": {
        "name": "invert-kv",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/invert-kv/-/invert-kv-2.0.0.tgz",
        "integrity": "sha512-wPVv/y/QQ/Uiirj/vh3oP+1Ww+AWehmi1g5fFWGPF6IpCBCDVrhgHRMvrLfdYcwDh3QJbGXDW4JAuzxElLSqKA==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-1.0.0.tgz",
        "integrity": "sha512-m5hnHTkcVsPfqx3AKlyttIPb7J+XykHvJP2B9bZDjlhLIoEq4XoK64Vg7boZlVWYK6LUY94dYPEE7Lh0ZkZKcQ==",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^6.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-accessor-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "6.0.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
        "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-arrayish": {
        "name": "is-arrayish",
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
        "integrity": "sha1-d8mYQFJ6qOyxqLppe4BkWnqSap0=",
        "license": "MIT"
      },
      "node_modules/is-binary-path": {
        "name": "is-binary-path",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-1.0.1.tgz",
        "integrity": "sha1-dfFmQrSA8YenEcgUFh/TpKdlWJg=",
        "license": "MIT",
        "dependencies": {
          "binary-extensions": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-buffer": {
        "name": "is-buffer",
        "version": "1.1.6",
        "resolved": "https://registry.npmjs.org/is-buffer/-/is-buffer-1.1.6.tgz",
        "integrity": "sha512-NcdALwpXkTm5Zvvbk7owOUSvVvBKDgKP5/ewfXEznmQFfs4ZRmanOeKBTjRVjka3QFoN6XJ+9F3USqfHqTaU5w==",
        "license": "MIT"
      },
      "node_modules/is-ci": {
        "name": "is-ci",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
        "integrity": "sha512-YfJT7rkpQB0updsdHLGWrvhBJfcfzNNawYDNIyQXJz0IViGf75O8EBPKSdvw2rF+LGCsX4FZ8tcr3b19LcZq4w==",
        "license": "MIT",
        "dependencies": {
          "ci-info": "^2.0.0"
        },
        "bin": {
          "is-ci": "bin.js"
        }
      },
      "node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-1.0.0.tgz",
        "integrity": "sha512-jbRXy1FmtAoCjQkVmIVYwuuqDFUbaOeDjmed1tOGPrsMhtJA4rD9tkgA0F1qJ3gRFRXcHYVkdeaP50Q5rE/jLQ==",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^6.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-data-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "6.0.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
        "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-1.0.2.tgz",
        "integrity": "sha512-2eis5WqQGV7peooDyLmNEPUrps9+SXX5c9pL3xEB+4e9HnGuDa7mB7kHxHw4CbqS9k1T2hOH3miL8n8WtiYVtg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^1.0.0",
          "is-data-descriptor": "^1.0.0",
          "kind-of": "^6.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "6.0.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
        "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-0.1.1.tgz",
        "integrity": "sha1-YrEQ4omkcUGOPsNqYX1HLjAd/Ik=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-extglob": {
        "name": "is-extglob",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
        "integrity": "sha1-qIwCU1eR8C7TfHahueqXc8gz+MI=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-finite": {
        "name": "is-finite",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-finite/-/is-finite-1.0.2.tgz",
        "integrity": "sha1-zGZ3aVYCvlUO8R6LSqYwU0K20Ko=",
        "license": "MIT",
        "dependencies": {
          "number-is-nan": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-fullwidth-code-point": {
        "name": "is-fullwidth-code-point",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
        "integrity": "sha1-o7MKXE8ZkYMWeqq5O+764937ZU8=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/is-glob": {
        "name": "is-glob",
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
        "integrity": "sha512-5G0tKtBTFImOqDnLB2hG6Bp2qcKEFduo4tZu9MT/H6NQv/ghhy30o55ufafxJ/LdH79LLs2Kfrn85TLKyA7BUg==",
        "license": "MIT",
        "dependencies": {
          "is-extglob": "^2.1.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-number": {
        "name": "is-number",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/is-number/-/is-number-3.0.0.tgz",
        "integrity": "sha1-JP1iAaR4LPUFYcgQJ2r8fRLXEZU=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-plain-object": {
        "name": "is-plain-object",
        "version": "2.0.4",
        "resolved": "https://registry.npmjs.org/is-plain-object/-/is-plain-object-2.0.4.tgz",
        "integrity": "sha512-h5PpgXkWitc38BBMYawTYMWJHFZJVnBquFE57xFpjB8pJFiF6gZ+bU+WyI/yqXiFR5mdLsgYNaPe8uao6Uv9Og==",
        "license": "MIT",
        "dependencies": {
          "isobject": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-stream": {
        "name": "is-stream",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
        "integrity": "sha1-EtSj3U5o4Lec6428hBc66A2RykQ=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/is-typedarray": {
        "name": "is-typedarray",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
        "integrity": "sha1-5HnICFjfDBsR3dppQPlgEfzaSpo=",
        "license": "MIT"
      },
      "node_modules/is-windows": {
        "name": "is-windows",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-windows/-/is-windows-1.0.2.tgz",
        "integrity": "sha512-eXK1UInq2bPmjyX6e3VHIzMLobc4J94i4AWn+Hpq3OU5KkrRC96OAcR3PRJ/pGu6m8TRnBHP9dkXQVsT/COVIA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/isarray": {
        "name": "isarray",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
        "integrity": "sha1-u5NdSFgsuhaMBoNJV6VKPgcSTxE=",
        "license": "MIT"
      },
      "node_modules/isexe": {
        "name": "isexe",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
        "integrity": "sha1-6PvzdNxVb/iUehDcsFctYz8s+hA=",
        "license": "ISC"
      },
      "node_modules/isobject": {
        "name": "isobject",
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/isobject/-/isobject-3.0.1.tgz",
        "integrity": "sha1-TkMekrEalzFjaqH5yNHMvP2reN8=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/isstream": {
        "name": "isstream",
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
        "integrity": "sha1-R+Y/evVa+m+S4VAOaQ64uFKcCZo=",
        "license": "MIT"
      },
      "node_modules/istanbul-lib-coverage": {
        "name": "istanbul-lib-coverage",
        "version": "2.0.5",
        "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
        "integrity": "sha512-8aXznuEPCJvGnMSRft4udDRDtb1V3pkQkMMI5LI+6HuQz5oQ4J2UFn1H82raA3qJtyOLkkwVqICBQkjnGtn5mA==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-hook": {
        "name": "istanbul-lib-hook",
        "version": "2.0.7",
        "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
        "integrity": "sha512-vrRztU9VRRFDyC+aklfLoeXyNdTfga2EI3udDGn4cZ6fpSXpHLV9X6CHvfoMCPtggg8zvDDmC4b9xfu0z6/llA==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "append-transform": "^1.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-instrument": {
        "name": "istanbul-lib-instrument",
        "version": "3.3.0",
        "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
        "integrity": "sha512-5nnIN4vo5xQZHdXno/YDXJ0G+I3dAm4XgzfSVTPLQpj/zAV2dV6Juy0yaf10/zrJOJeHoN3fraFe+XRq2bFVZA==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "@babel/generator": "^7.4.0",
          "@babel/parser": "^7.4.3",
          "@babel/template": "^7.4.0",
          "@babel/traverse": "^7.4.3",
          "@babel/types": "^7.4.0",
          "istanbul-lib-coverage": "^2.0.5",
          "semver": "^6.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-processinfo": {
        "name": "istanbul-lib-processinfo",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
        "integrity": "sha512-FY0cPmWa4WoQNlvB8VOcafiRoB5nB+l2Pz2xGuXHRSy1KM8QFOYfz/rN+bGMCAeejrY3mrpF5oJHcN0s/garCg==",
        "license": "ISC",
        "dependencies": {
          "archy": "^1.0.0",
          "cross-spawn": "^6.0.5",
          "istanbul-lib-coverage": "^2.0.3",
          "rimraf": "^2.6.3",
          "uuid": "^3.3.2"
        }
      },
      "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn": {
        "name": "cross-spawn",
        "version": "6.0.5",
        "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
        "integrity": "sha512-eTVLrBSt7fjbDygz805pMnstIs2VTBNkRm0qxZd+M7A5XDdxVRWO5MxGBXZhjY4cqLYLdtrGqRf8mBPmzwSpWQ==",
        "license": "MIT",
        "dependencies": {
          "nice-try": "^1.0.4",
          "path-key": "^2.0.1",
          "semver": "^5.5.0",
          "shebang-command": "^1.2.0",
          "which": "^1.2.9"
        },
        "engines": {
          "node": ">=4.8"
        }
      },
      "node_modules/istanbul-lib-processinfo/node_modules/semver": {
        "name": "semver",
        "version": "5.7.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
        "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/istanbul-lib-report": {
        "name": "istanbul-lib-report",
        "version": "2.0.8",
        "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
        "integrity": "sha512-fHBeG573EIihhAblwgxrSenp0Dby6tJMFR/HvlerBsrCTD5bkUuoNtn3gVh29ZCS824cGGBPn7Sg7cNk+2xUsQ==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "istanbul-lib-coverage": "^2.0.5",
          "make-dir": "^2.1.0",
          "supports-color": "^6.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-report/node_modules/supports-color": {
        "name": "supports-color",
        "version": "6.1.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
        "integrity": "sha512-qe1jfm1Mg7Nq/NSh6XE24gPXROEVsWHxC1LIx//XNlD9iw7YZQGjZNjYN7xGaEG6iKdA8EtNFW6R0gjnVXp+wQ==",
        "license": "MIT",
        "dependencies": {
          "has-flag": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-source-maps": {
        "name": "istanbul-lib-source-maps",
        "version": "3.0.6",
        "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
        "integrity": "sha512-R47KzMtDJH6X4/YW9XTx+jrLnZnscW4VpNN+1PViSYTejLVPWv7oov+Duf8YQSPyVRUvueQqz1TcsC6mooZTXw==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "debug": "^4.1.1",
          "istanbul-lib-coverage": "^2.0.5",
          "make-dir": "^2.1.0",
          "rimraf": "^2.6.3",
          "source-map": "^0.6.1"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/istanbul-lib-source-maps/node_modules/debug": {
        "name": "debug",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
        "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
        "license": "MIT",
        "dependencies": {
          "ms": "^2.1.1"
        }
      },
      "node_modules/istanbul-lib-source-maps/node_modules/ms": {
        "name": "ms",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
        "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg==",
        "license": "MIT"
      },
      "node_modules/istanbul-lib-source-maps/node_modules/source-map": {
        "name": "source-map",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
        "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/istanbul-reports": {
        "name": "istanbul-reports",
        "version": "2.2.4",
        "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.4.tgz",
        "integrity": "sha512-QCHGyZEK0bfi9GR215QSm+NJwFKEShbtc7tfbUdLAEzn3kKhLDDZqvljn8rPZM9v8CEOhzL1nlYoO4r1ryl67w==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "handlebars": "^4.1.2"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/jackspeak": {
        "name": "jackspeak",
        "version": "1.3.7",
        "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.3.7.tgz",
        "integrity": "sha512-Z4iSFpaCV7Cocpcl5t9/UyPkisxenbmaqminyTgK6lDDMXcm9EvIZ9Bwr/uFbGOjfWlz1UZwKwFY5AvtgNlHuw==",
        "license": "ISC",
        "dependencies": {
          "cliui": "^4.1.0"
        },
        "engines": {
          "node": ">=8"
        }
      },
      "node_modules/js-tokens": {
        "name": "js-tokens",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-3.0.2.tgz",
        "integrity": "sha1-mGbfOVECEw449/mWvOtlRDIJwls=",
        "license": "MIT"
      },
      "node_modules/js-yaml": {
        "name": "js-yaml",
        "version": "3.13.1",
        "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
        "integrity": "sha512-YfbcO7jXDdyj0DGxYVSlSeQNHbD7XPWvrVWeVUujrQEoZzWJIRrCPoyk6kL6IAjAG2IolMK4T0hNUe0HOUs5Jw==",
        "license": "MIT",
        "dependencies": {
          "argparse": "^1.0.7",
          "esprima": "^4.0.0"
        },
        "bin": {
          "js-yaml": "bin/js-yaml.js"
        }
      },
      "node_modules/jsbn": {
        "name": "jsbn",
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
        "integrity": "sha1-peZUwuWi3rXyAdls77yoDA7y9RM=",
        "license": "MIT"
      },
      "node_modules/jsesc": {
        "name": "jsesc",
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-1.3.0.tgz",
        "integrity": "sha1-RsP+yMGJKxKwgz25vHYiF226s0s=",
        "license": "MIT",
        "bin": {
          "jsesc": "bin/jsesc"
        }
      },
      "node_modules/json-parse-better-errors": {
        "name": "json-parse-better-errors",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
        "integrity": "sha512-mrqyZKfX5EhL7hvqcV6WG1yYjnjeuYDzDhhcAAUrq8Po85NBQBJP+ZDUT75qZQ98IkUoBqdkExkukOU7Ts2wrw==",
        "license": "MIT"
      },
      "node_modules/json-schema": {
        "name": "json-schema",
        "version": "0.2.3",
        "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
        "integrity": "sha1-tIDIkuWaLwWVTOcnvT8qTogvnhM="
      },
      "node_modules/json-schema-traverse": {
        "name": "json-schema-traverse",
        "version": "0.4.1",
        "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
        "integrity": "sha512-xbbCH5dCYU5T8LcEhhuh7HJ88HXuW3qsI3Y0zOZFKfZEHcpWiHU/Jxzk629Brsab/mMiHQti9wMP+845RPe3Vg==",
        "license": "MIT"
      },
      "node_modules/json-stringify-safe": {
        "name": "json-stringify-safe",
        "version": "5.0.1",
        "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
        "integrity": "sha1-Epai1Y/UXxmg9s4B1lcB4sc1tus=",
        "license": "ISC"
      },
      "node_modules/json5": {
        "name": "json5",
        "version": "0.5.1",
        "resolved": "https://registry.npmjs.org/json5/-/json5-0.5.1.tgz",
        "integrity": "sha1-Hq3nrMASA0rYTiOWdn6tn6VJWCE=",
        "license": "MIT",
        "bin": {
          "json5": "lib/cli.js"
        }
      },
      "node_modules/jsprim": {
        "name": "jsprim",
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
        "integrity": "sha1-MT5mvB5cwG5Di8G3SZwuXFastqI=",
        "engines": [
          "node >=0.6.0"
        ],
        "license": "MIT",
        "dependencies": {
          "assert-plus": "1.0.0",
          "extsprintf": "1.3.0",
          "json-schema": "0.2.3",
          "verror": "1.10.0"
        }
      },
      "node_modules/kind-of": {
        "name": "kind-of",
        "version": "3.2.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-3.2.2.tgz",
        "integrity": "sha1-MeohpzS6ubuw8yRm2JOupR5KPGQ=",
        "license": "MIT",
        "dependencies": {
          "is-buffer": "^1.1.5"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/lcid": {
        "name": "lcid",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/lcid/-/lcid-2.0.0.tgz",
        "integrity": "sha512-avPEb8P8EGnwXKClwsNUgryVjllcRqtMYa49NTsbQagYuT1DcXnl1915oxWjoyGrXR6zH/Y0Zc96xWsPcoDKeA==",
        "license": "MIT",
        "dependencies": {
          "invert-kv": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/lcov-parse": {
        "name": "lcov-parse",
        "version": "0.0.10",
        "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-0.0.10.tgz",
        "integrity": "sha1-GwuP+ayceIklBYK3C3ExXZ2m2aM=",
        "license": "BSD-3-Clause"
      },
      "node_modules/load-json-file": {
        "name": "load-json-file",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
        "integrity": "sha1-L19Fq5HjMhYjT9U62rZo607AmTs=",
        "license": "MIT",
        "dependencies": {
          "graceful-fs": "^4.1.2",
          "parse-json": "^4.0.0",
          "pify": "^3.0.0",
          "strip-bom": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/load-json-file/node_modules/pify": {
        "name": "pify",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
        "integrity": "sha1-5aSs0sEB/fPZpNB/DbxNtJ3SgXY=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/locate-path": {
        "name": "locate-path",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
        "integrity": "sha512-7AO748wWnIhNqAuaty2ZWHkQHRSNfPVIsPIfwEOWO22AmaoVrWavlOcMR5nzTLNYvp36X220/maaRsrec1G65A==",
        "license": "MIT",
        "dependencies": {
          "p-locate": "^3.0.0",
          "path-exists": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/lodash": {
        "name": "lodash",
        "version": "4.17.11",
        "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.11.tgz",
        "integrity": "sha512-cQKh8igo5QUhZ7lg38DYWAxMvjSAKG0A8wGSVimP07SIUEK2UO+arSRKbRZWtelMtN5V0Hkwh5ryOto/SshYIg==",
        "license": "MIT"
      },
      "node_modules/lodash.flattendeep": {
        "name": "lodash.flattendeep",
        "version": "4.4.0",
        "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
        "integrity": "sha1-+wMJF/hqMTTlvJvsDWngAT3f7bI=",
        "license": "MIT"
      },
      "node_modules/lodash.throttle": {
        "name": "lodash.throttle",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
        "integrity": "sha1-wj6RtxAkKscMN/HhzaknTMOb8vQ=",
        "license": "MIT"
      },
      "node_modules/log-driver": {
        "name": "log-driver",
        "version": "1.2.7",
        "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
        "integrity": "sha512-U7KCmLdqsGHBLeWqYlFA0V0Sl6P08EE1ZrmA9cxjUE0WVqT9qnyVDPz1kzpFEP0jdJuFnasWIfSd7fsaNXkpbg==",
        "license": "ISC",
        "engines": {
          "node": ">=0.8.6"
        }
      },
      "node_modules/log-update": {
        "name": "log-update",
        "version": "3.2.0",
        "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.2.0.tgz",
        "integrity": "sha512-KJ6zAPIHWo7Xg1jYror6IUDFJBq1bQ4Bi4wAEp2y/0ScjBBVi/g0thr0sUVhuvuXauWzczt7T2QHghPDNnKBuw==",
        "license": "MIT",
        "dependencies": {
          "ansi-escapes": "^3.2.0",
          "cli-cursor": "^2.1.0",
          "wrap-ansi": "^5.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/log-update/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
        "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/log-update/node_modules/ansi-styles": {
        "name": "ansi-styles",
        "version": "3.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
        "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
        "license": "MIT",
        "dependencies": {
          "color-convert": "^1.9.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/log-update/node_modules/string-width": {
        "name": "string-width",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
        "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
        "license": "MIT",
        "dependencies": {
          "emoji-regex": "^7.0.1",
          "is-fullwidth-code-point": "^2.0.0",
          "strip-ansi": "^5.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/log-update/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "5.2.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
        "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^4.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/log-update/node_modules/wrap-ansi": {
        "name": "wrap-ansi",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
        "integrity": "sha512-QC1/iN/2/RPVJ5jYK8BGttj5z83LmSKmvbvrXPNCLZSEb32KKVDJDl/MOt2N01qU2H/FkzEa9PKto1BqDjtd7Q==",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^3.2.0",
          "string-width": "^3.0.0",
          "strip-ansi": "^5.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/loose-envify": {
        "name": "loose-envify",
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
        "integrity": "sha512-lyuxPGr/Wfhrlem2CL/UcnUc1zcqKAImBDzukY7Y5F/yQiNdko6+fRLevlw1HgMySw7f611UIY408EtxRSoK3Q==",
        "license": "MIT",
        "dependencies": {
          "js-tokens": "^3.0.0 || ^4.0.0"
        },
        "bin": {
          "loose-envify": "cli.js"
        }
      },
      "node_modules/lru-cache": {
        "name": "lru-cache",
        "version": "4.1.5",
        "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
        "integrity": "sha512-sWZlbEP2OsHNkXrMl5GYk/jKk70MBng6UU4YI/qGDYbgf6YbP4EvmqISbXCoJiRKs+1bSpFHVgQxvJ17F2li5g==",
        "license": "ISC",
        "dependencies": {
          "pseudomap": "^1.0.2",
          "yallist": "^2.1.2"
        }
      },
      "node_modules/make-dir": {
        "name": "make-dir",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
        "integrity": "sha512-LS9X+dc8KLxXCb8dni79fLIIUA5VyZoyjSMCwTluaXA0o27cCK0bhXkpgw+sTXVpPy/lSO57ilRixqk0vDmtRA==",
        "license": "MIT",
        "dependencies": {
          "pify": "^4.0.1",
          "semver": "^5.6.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/make-dir/node_modules/semver": {
        "name": "semver",
        "version": "5.7.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
        "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/make-error": {
        "name": "make-error",
        "version": "1.3.5",
        "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
        "integrity": "sha512-c3sIjNUow0+8swNwVpqoH4YCShKNFkMaw6oH1mNS2haDZQqkeZFlHS3dhoeEbKKmJB4vXpJucU6oH75aDYeE9g==",
        "license": "ISC"
      },
      "node_modules/map-age-cleaner": {
        "name": "map-age-cleaner",
        "version": "0.1.3",
        "resolved": "https://registry.npmjs.org/map-age-cleaner/-/map-age-cleaner-0.1.3.tgz",
        "integrity": "sha512-bJzx6nMoP6PDLPBFmg7+xRKeFZvFboMrGlxmNj9ClvX53KrmvM5bXFXEWjbz4cz1AFn+jWJ9z/DJSz7hrs0w3w==",
        "license": "MIT",
        "dependencies": {
          "p-defer": "^1.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/map-cache": {
        "name": "map-cache",
        "version": "0.2.2",
        "resolved": "https://registry.npmjs.org/map-cache/-/map-cache-0.2.2.tgz",
        "integrity": "sha1-wyq9C9ZSXZsFFkW7TyasXcmKDb8=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/map-visit": {
        "name": "map-visit",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/map-visit/-/map-visit-1.0.0.tgz",
        "integrity": "sha1-7Nyo8TFE5mDxtb1B8S80edmN+48=",
        "license": "MIT",
        "dependencies": {
          "object-visit": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/mem": {
        "name": "mem",
        "version": "4.3.0",
        "resolved": "https://registry.npmjs.org/mem/-/mem-4.3.0.tgz",
        "integrity": "sha512-qX2bG48pTqYRVmDB37rn/6PT7LcR8T7oAX3bf99u1Tt1nzxYfxkgqDwUwolPlXweM0XzBOBFzSx4kfp7KP1s/w==",
        "license": "MIT",
        "dependencies": {
          "map-age-cleaner": "^0.1.1",
          "mimic-fn": "^2.0.0",
          "p-is-promise": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/mem/node_modules/mimic-fn": {
        "name": "mimic-fn",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
        "integrity": "sha512-OqbOk5oEQeAZ8WXWydlu9HJjz9WVdEIvamMCcXmuqUYjTknH/sqsWvhQ3vgwKFRR1HpjvNBKQ37nbJgYzGqGcg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/merge-source-map": {
        "name": "merge-source-map",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
        "integrity": "sha512-Qkcp7P2ygktpMPh2mCQZaf3jhN6D3Z/qVZHSdWvQ+2Ef5HgRAPBO57A77+ENm0CPx2+1Ce/MYKi3ymqdfuqibw==",
        "license": "MIT",
        "dependencies": {
          "source-map": "^0.6.1"
        }
      },
      "node_modules/merge-source-map/node_modules/source-map": {
        "name": "source-map",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
        "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/micromatch": {
        "name": "micromatch",
        "version": "3.1.10",
        "resolved": "https://registry.npmjs.org/micromatch/-/micromatch-3.1.10.tgz",
        "integrity": "sha512-MWikgl9n9M3w+bpsY3He8L+w9eF9338xRl8IAO5viDizwSzziFEyUzo2xrrloB64ADbTf8uA8vRqqttDTOmccg==",
        "license": "MIT",
        "dependencies": {
          "arr-diff": "^4.0.0",
          "array-unique": "^0.3.2",
          "braces": "^2.3.1",
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "extglob": "^2.0.4",
          "fragment-cache": "^0.2.1",
          "kind-of": "^6.0.2",
          "nanomatch": "^1.2.9",
          "object.pick": "^1.3.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/micromatch/node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
        "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
        "license": "MIT",
        "dependencies": {
          "assign-symbols": "^1.0.0",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/micromatch/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/micromatch/node_modules/kind-of": {
        "name": "kind-of",
        "version": "6.0.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
        "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/mime-db": {
        "name": "mime-db",
        "version": "1.40.0",
        "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.40.0.tgz",
        "integrity": "sha512-jYdeOMPy9vnxEqFRRo6ZvTZ8d9oPb+k18PKoYNYUe2stVEBPPwsln/qWzdbmaIvnhZ9v2P+CuecK+fpUfsV2mA==",
        "license": "MIT",
        "engines": {
          "node": ">= 0.6"
        }
      },
      "node_modules/mime-types": {
        "name": "mime-types",
        "version": "2.1.24",
        "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.24.tgz",
        "integrity": "sha512-WaFHS3MCl5fapm3oLxU4eYDw77IQM2ACcxQ9RIxfaC3ooc6PFuBMGZZsYpvoXS5D5QTWPieo1jjLdAm3TBP3cQ==",
        "license": "MIT",
        "dependencies": {
          "mime-db": "1.40.0"
        },
        "engines": {
          "node": ">= 0.6"
        }
      },
      "node_modules/mimic-fn": {
        "name": "mimic-fn",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
        "integrity": "sha512-jf84uxzwiuiIVKiOLpfYk7N46TSy8ubTonmneY9vrpHNAnp0QBt2BxWV9dO3/j+BoVAb+a5G6YDPW3M5HOdMWQ==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/minimatch": {
        "name": "minimatch",
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
        "integrity": "sha512-yJHVQEhyqPLUTgt9B83PXu6W3rx4MvvHvSUvToogpwoGDOUQ+yDrR0HRot+yOCdCO7u4hX3pWft6kWBBcqh0UA==",
        "license": "ISC",
        "dependencies": {
          "brace-expansion": "^1.1.7"
        },
        "engines": {
          "node": "*"
        }
      },
      "node_modules/minimist": {
        "name": "minimist",
        "version": "0.0.8",
        "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
        "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0=",
        "license": "MIT"
      },
      "node_modules/minipass": {
        "name": "minipass",
        "version": "2.3.5",
        "resolved": "https://registry.npmjs.org/minipass/-/minipass-2.3.5.tgz",
        "integrity": "sha512-Gi1W4k059gyRbyVUZQ4mEqLm0YIUiGYfvxhF6SIlk3ui1WVxMTGfGdQ2SInh3PDrRTVvPKgULkpJtT4RH10+VA==",
        "license": "ISC",
        "dependencies": {
          "safe-buffer": "^5.1.2",
          "yallist": "^3.0.0"
        }
      },
      "node_modules/minipass/node_modules/yallist": {
        "name": "yallist",
        "version": "3.0.3",
        "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.0.3.tgz",
        "integrity": "sha512-S+Zk8DEWE6oKpV+vI3qWkaK+jSbIK86pCwe2IF/xwIpQ8jEuxpw9NyaGjmp9+BoJv5FV2piqCDcoCtStppiq2A==",
        "license": "ISC"
      },
      "node_modules/mixin-deep": {
        "name": "mixin-deep",
        "version": "1.3.1",
        "resolved": "https://registry.npmjs.org/mixin-deep/-/mixin-deep-1.3.1.tgz",
        "integrity": "sha512-8ZItLHeEgaqEvd5lYBXfm4EZSFCX29Jb9K+lAHhDKzReKBQKj3R+7NOF6tjqYi9t4oI8VUfaWITJQm86wnXGNQ==",
        "deprecated": "Critical bug fixed in v2.0.1, please upgrade to the latest version.",
        "license": "MIT",
        "dependencies": {
          "for-in": "^1.0.2",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/mixin-deep/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/mkdirp": {
        "name": "mkdirp",
        "version": "0.5.1",
        "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
        "integrity": "sha1-MAV0OOrGz3+MR2fzhkjWaX11yQM=",
        "deprecated": "Legacy versions of mkdirp are no longer supported. Please update to mkdirp 1.x. (Note that the API surface has changed to use Promises in 1.x.)",
        "license": "MIT",
        "dependencies": {
          "minimist": "0.0.8"
        },
        "bin": {
          "mkdirp": "bin/cmd.js"
        }
      },
      "node_modules/ms": {
        "name": "ms",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
        "integrity": "sha1-VgiurfwAvmwpAd9fmGF4jeDVl8g=",
        "license": "MIT"
      },
      "node_modules/nan": {
        "name": "nan",
        "version": "2.13.2",
        "resolved": "https://registry.npmjs.org/nan/-/nan-2.13.2.tgz",
        "integrity": "sha512-TghvYc72wlMGMVMluVo9WRJc0mB8KxxF/gZ4YYFy7V2ZQX9l7rgbPg7vjS9mt6U5HXODVFVI2bOduCzwOMv/lw==",
        "license": "MIT",
        "optional": true
      },
      "node_modules/nanomatch": {
        "name": "nanomatch",
        "version": "1.2.13",
        "resolved": "https://registry.npmjs.org/nanomatch/-/nanomatch-1.2.13.tgz",
        "integrity": "sha512-fpoe2T0RbHwBTBUOftAfBPaDEi06ufaUai0mE6Yn1kacc3SnTErfb/h+X94VXzI64rKFHYImXSvdwGGCmwOqCA==",
        "license": "MIT",
        "dependencies": {
          "arr-diff": "^4.0.0",
          "array-unique": "^0.3.2",
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "fragment-cache": "^0.2.1",
          "is-windows": "^1.0.2",
          "kind-of": "^6.0.2",
          "object.pick": "^1.3.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/nanomatch/node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
        "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
        "license": "MIT",
        "dependencies": {
          "assign-symbols": "^1.0.0",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/nanomatch/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/nanomatch/node_modules/kind-of": {
        "name": "kind-of",
        "version": "6.0.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
        "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/neo-async": {
        "name": "neo-async",
        "version": "2.6.0",
        "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.0.tgz",
        "integrity": "sha512-MFh0d/Wa7vkKO3Y3LlacqAEeHK0mckVqzDieUKTT+KGxi+zIpeVsFxymkIiRpbpDziHc290Xr9A1O4Om7otoRA==",
        "license": "MIT"
      },
      "node_modules/nested-error-stacks": {
        "name": "nested-error-stacks",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
        "integrity": "sha512-AO81vsIO1k1sM4Zrd6Hu7regmJN1NSiAja10gc4bX3F0wd+9rQmcuHQaHVQCYIEC8iFXnE+mavh23GOt7wBgug==",
        "license": "MIT"
      },
      "node_modules/nice-try": {
        "name": "nice-try",
        "version": "1.0.5",
        "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
        "integrity": "sha512-1nh45deeb5olNY7eX82BkPO7SSxR5SSYJiPTrTdFUVYwAl8CKMA5N9PjTYkHiRjisVcxcQ1HXdLhx2qxxJzLNQ==",
        "license": "MIT"
      },
      "node_modules/normalize-package-data": {
        "name": "normalize-package-data",
        "version": "2.5.0",
        "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
        "integrity": "sha512-/5CMN3T0R4XTj4DcGaexo+roZSdSFW/0AOOTROrjxzCG1wrWXEsGbRKevjlIL+ZDE4sZlJr5ED4YW0yqmkK+eA==",
        "license": "BSD-2-Clause",
        "dependencies": {
          "hosted-git-info": "^2.1.4",
          "resolve": "^1.10.0",
          "semver": "2 || 3 || 4 || 5",
          "validate-npm-package-license": "^3.0.1"
        }
      },
      "node_modules/normalize-package-data/node_modules/semver": {
        "name": "semver",
        "version": "5.7.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
        "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/normalize-path": {
        "name": "normalize-path",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
        "integrity": "sha512-6eZs5Ls3WtCisHWp9S2GUy8dqkpGi4BVSz3GaqiE6ezub0512ESztXUwUB6C6IKbQkY2Pnb/mD4WYojCRwcwLA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/npm-run-path": {
        "name": "npm-run-path",
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/npm-run-path/-/npm-run-path-2.0.2.tgz",
        "integrity": "sha1-NakjLfo11wZ7TLLd8jV7GHFTbF8=",
        "license": "MIT",
        "dependencies": {
          "path-key": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/number-is-nan": {
        "name": "number-is-nan",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
        "integrity": "sha1-CXtgK1NCKlIsGvuHkDGDNpQaAR0=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/nyc": {
        "name": "nyc",
        "version": "14.1.0",
        "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.0.tgz",
        "integrity": "sha512-iy9fEV8Emevz3z/AanIZsoGa8F4U2p0JKevZ/F0sk+/B2r9E6Qn+EPs0bpxEhnAt6UPlTL8mQZIaSJy8sK0ZFw==",
        "license": "ISC",
        "dependencies": {
          "archy": "^1.0.0",
          "caching-transform": "^3.0.2",
          "convert-source-map": "^1.6.0",
          "cp-file": "^6.2.0",
          "find-cache-dir": "^2.1.0",
          "find-up": "^3.0.0",
          "foreground-child": "^1.5.6",
          "glob": "^7.1.3",
          "istanbul-lib-coverage": "^2.0.5",
          "istanbul-lib-hook": "^2.0.7",
          "istanbul-lib-instrument": "^3.3.0",
          "istanbul-lib-report": "^2.0.8",
          "istanbul-lib-source-maps": "^3.0.6",
          "istanbul-reports": "^2.2.4",
          "js-yaml": "^3.13.1",
          "make-dir": "^2.1.0",
          "merge-source-map": "^1.1.0",
          "resolve-from": "^4.0.0",
          "rimraf": "^2.6.3",
          "signal-exit": "^3.0.2",
          "spawn-wrap": "^1.4.2",
          "test-exclude": "^5.2.3",
          "uuid": "^3.3.2",
          "yargs": "^13.2.2",
          "yargs-parser": "^13.0.0"
        },
        "bin": {
          "nyc": "bin/nyc.js"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/nyc/node_modules/resolve-from": {
        "name": "resolve-from",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
        "integrity": "sha512-pb/MYmXstAkysRFx8piNI1tGFNQIFA3vkE3Gq4EuA1dF6gHp/+vgZqsCGJapvy8N3Q+4o7FwvquPJcnZ7RYy4g==",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/oauth-sign": {
        "name": "oauth-sign",
        "version": "0.9.0",
        "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
        "integrity": "sha512-fexhUFFPTGV8ybAtSIGbV6gOkSv8UtRbDBnAyLQw4QPKkgNlsH2ByPGtMUqdWkos6YCRmAqViwgZrJc/mRDzZQ==",
        "license": "Apache-2.0",
        "engines": {
          "node": "*"
        }
      },
      "node_modules/object-assign": {
        "name": "object-assign",
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
        "integrity": "sha1-IQmtx5ZYh8/AXLvUQsrIv7s2CGM=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy": {
        "name": "object-copy",
        "version": "0.1.0",
        "resolved": "https://registry.npmjs.org/object-copy/-/object-copy-0.1.0.tgz",
        "integrity": "sha1-fn2Fi3gb18mRpBupde04EnVOmYw=",
        "license": "MIT",
        "dependencies": {
          "copy-descriptor": "^0.1.0",
          "define-property": "^0.2.5",
          "kind-of": "^3.0.3"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy/node_modules/define-property": {
        "name": "define-property",
        "version": "0.2.5",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
        "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy/node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
        "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy/node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
        "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy/node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
        "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^0.1.6",
          "is-data-descriptor": "^0.1.4",
          "kind-of": "^5.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-copy/node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
        "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object-visit": {
        "name": "object-visit",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/object-visit/-/object-visit-1.0.1.tgz",
        "integrity": "sha1-95xEk68MU3e1n+OdOV5BBC3QRbs=",
        "license": "MIT",
        "dependencies": {
          "isobject": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/object.pick": {
        "name": "object.pick",
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/object.pick/-/object.pick-1.3.0.tgz",
        "integrity": "sha1-h6EKxMFpS9Lhy/U1kaZhQftd10c=",
        "license": "MIT",
        "dependencies": {
          "isobject": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/once": {
        "name": "once",
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
        "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
        "license": "ISC",
        "dependencies": {
          "wrappy": "1"
        }
      },
      "node_modules/onetime": {
        "name": "onetime",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
        "integrity": "sha1-BnQoIw/WdEOyeUsiu6UotoZ5YtQ=",
        "license": "MIT",
        "dependencies": {
          "mimic-fn": "^1.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/opener": {
        "name": "opener",
        "version": "1.5.1",
        "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
        "integrity": "sha512-goYSy5c2UXE4Ra1xixabeVh1guIX/ZV/YokJksb6q2lubWu6UbvPQ20p542/sFIll1nl8JnCyK9oBaOcCWXwvA==",
        "license": "(WTFPL OR MIT)",
        "bin": {
          "opener": "bin/opener-bin.js"
        }
      },
      "node_modules/optimist": {
        "name": "optimist",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
        "integrity": "sha1-2j6nRob6IaGaERwybpDrFaAZZoY=",
        "license": "MIT/X11",
        "dependencies": {
          "minimist": "~0.0.1",
          "wordwrap": "~0.0.2"
        }
      },
      "node_modules/os-homedir": {
        "name": "os-homedir",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
        "integrity": "sha1-/7xJiDNuDoM94MFox+8VISGqf7M=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/os-locale": {
        "name": "os-locale",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/os-locale/-/os-locale-3.1.0.tgz",
        "integrity": "sha512-Z8l3R4wYWM40/52Z+S265okfFj8Kt2cC2MKY+xNi3kFs+XGI7WXu/I309QQQYbRW4ijiZ+yxs9pqEhJh0DqW3Q==",
        "license": "MIT",
        "dependencies": {
          "execa": "^1.0.0",
          "lcid": "^2.0.0",
          "mem": "^4.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/os-tmpdir": {
        "name": "os-tmpdir",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
        "integrity": "sha1-u+Z0BseaqFxc/sdm/lc0VV36EnQ=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/own-or": {
        "name": "own-or",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
        "integrity": "sha1-Tod/vtqaLsgAD7wLyuOWRe6L+Nw=",
        "license": "ISC"
      },
      "node_modules/own-or-env": {
        "name": "own-or-env",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
        "integrity": "sha512-y8qULRbRAlL6x2+M0vIe7jJbJx/kmUTzYonRAa2ayesR2qWLswninkVyeJe4x3IEXhdgoNodzjQRKAoEs6Fmrw==",
        "license": "ISC",
        "dependencies": {
          "own-or": "^1.0.0"
        }
      },
      "node_modules/p-defer": {
        "name": "p-defer",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/p-defer/-/p-defer-1.0.0.tgz",
        "integrity": "sha1-n26xgvbJqozXQwBKfU+WsZaw+ww=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/p-finally": {
        "name": "p-finally",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/p-finally/-/p-finally-1.0.0.tgz",
        "integrity": "sha1-P7z7FbiZpEEjs0ttzBi3JDNqLK4=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/p-is-promise": {
        "name": "p-is-promise",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/p-is-promise/-/p-is-promise-2.1.0.tgz",
        "integrity": "sha512-Y3W0wlRPK8ZMRbNq97l4M5otioeA5lm1z7bkNkxCka8HSPjR0xRWmpCmc9utiaLP9Jb1eD8BgeIxTW4AIF45Pg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/p-limit": {
        "name": "p-limit",
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.0.tgz",
        "integrity": "sha512-pZbTJpoUsCzV48Mc9Nh51VbwO0X9cuPFE8gYwx9BTCt9SF8/b7Zljd2fVgOxhIF/HDTKgpVzs+GPhyKfjLLFRQ==",
        "license": "MIT",
        "dependencies": {
          "p-try": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/p-locate": {
        "name": "p-locate",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
        "integrity": "sha512-x+12w/To+4GFfgJhBEpiDcLozRJGegY+Ei7/z0tSLkMmxGZNybVMSfWj9aJn8Z5Fc7dBUNJOOVgPv2H7IwulSQ==",
        "license": "MIT",
        "dependencies": {
          "p-limit": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/p-try": {
        "name": "p-try",
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
        "integrity": "sha512-R4nPAVTAU0B9D35/Gk3uJf/7XYbQcyohSKdvAxIRSNghFl4e71hVoGnBNQz9cWaXxO2I10KTC+3jMdvvoKw6dQ==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/package-hash": {
        "name": "package-hash",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
        "integrity": "sha512-lOtmukMDVvtkL84rJHI7dpTYq+0rli8N2wlnqUcBuDWCfVhRUfOmnR9SsoHFMLpACvEV60dX7rd0rFaYDZI+FA==",
        "license": "ISC",
        "dependencies": {
          "graceful-fs": "^4.1.15",
          "hasha": "^3.0.0",
          "lodash.flattendeep": "^4.4.0",
          "release-zalgo": "^1.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/parse-json": {
        "name": "parse-json",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
        "integrity": "sha1-vjX1Qlvh9/bHRxhPmKeIy5lHfuA=",
        "license": "MIT",
        "dependencies": {
          "error-ex": "^1.3.1",
          "json-parse-better-errors": "^1.0.1"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/pascalcase": {
        "name": "pascalcase",
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/pascalcase/-/pascalcase-0.1.1.tgz",
        "integrity": "sha1-s2PlXoAGym/iF4TS2yK9FdeRfxQ=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/path-dirname": {
        "name": "path-dirname",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/path-dirname/-/path-dirname-1.0.2.tgz",
        "integrity": "sha1-zDPSTVJeCZpTiMAzbG4yuRYGCeA=",
        "license": "MIT"
      },
      "node_modules/path-exists": {
        "name": "path-exists",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
        "integrity": "sha1-zg6+ql94yxiSXqfYENe1mwEP1RU=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/path-is-absolute": {
        "name": "path-is-absolute",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
        "integrity": "sha1-F0uSaHNVNP+8es5r9TpanhtcX18=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/path-key": {
        "name": "path-key",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
        "integrity": "sha1-QRyttXTFoUDTpLGRDUDYDMn0C0A=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/path-parse": {
        "name": "path-parse",
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
        "integrity": "sha512-GSmOT2EbHrINBf9SR7CDELwlJ8AENk3Qn7OikK4nFYAu3Ote2+JYNVvkpAEQm3/TLNEJFD/xZJjzyxg3KBWOzw==",
        "license": "MIT"
      },
      "node_modules/path-type": {
        "name": "path-type",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
        "integrity": "sha512-T2ZUsdZFHgA3u4e5PfPbjd7HDDpxPnQb5jN0SrDsjNSuVXHJqtwTnWqG0B1jZrgmJ/7lj1EmVIByWt1gxGkWvg==",
        "license": "MIT",
        "dependencies": {
          "pify": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/path-type/node_modules/pify": {
        "name": "pify",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
        "integrity": "sha1-5aSs0sEB/fPZpNB/DbxNtJ3SgXY=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/performance-now": {
        "name": "performance-now",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
        "integrity": "sha1-Ywn04OX6kT7BxpMHrjZLSzd8nns=",
        "license": "MIT"
      },
      "node_modules/pify": {
        "name": "pify",
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
        "integrity": "sha512-uB80kBFb/tfd68bVleG9T5GGsGPjJrLAUpR5PZIrhBnIaRTQRjqdJSsIKkOP6OAIFbj7GOrcudc5pNjZ+geV2g==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/pkg-dir": {
        "name": "pkg-dir",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
        "integrity": "sha512-/E57AYkoeQ25qkxMj5PBOVgF8Kiu/h7cYS30Z5+R7WaiCCBfLq58ZI/dSeaEKb9WVJV5n/03QwrN3IeWIFllvw==",
        "license": "MIT",
        "dependencies": {
          "find-up": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/posix-character-classes": {
        "name": "posix-character-classes",
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/posix-character-classes/-/posix-character-classes-0.1.1.tgz",
        "integrity": "sha1-AerA/jta9xoqbAL+q7jB/vfgDqs=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/private": {
        "name": "private",
        "version": "0.1.8",
        "resolved": "https://registry.npmjs.org/private/-/private-0.1.8.tgz",
        "integrity": "sha512-VvivMrbvd2nKkiG38qjULzlc+4Vx4wm/whI9pQD35YrARNnhxeiRktSOhSukRLFNlzg6Br/cJPet5J/u19r/mg==",
        "license": "MIT",
        "engines": {
          "node": ">= 0.6"
        }
      },
      "node_modules/process-nextick-args": {
        "name": "process-nextick-args",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.0.tgz",
        "integrity": "sha512-MtEC1TqN0EU5nephaJ4rAtThHtC86dNN9qCuEhtshvpVBkAW5ZO7BASN9REnF9eoXGcRub+pFuKEpOHE+HbEMw==",
        "license": "MIT"
      },
      "node_modules/prop-types": {
        "name": "prop-types",
        "version": "15.7.2",
        "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
        "integrity": "sha512-8QQikdH7//R2vurIJSutZ1smHYTcLpRWEOlHnzcWHmBYrOGUysKwSsrC89BCiFj3CbrfJ/nXFdJepOVrY1GCHQ==",
        "license": "MIT",
        "dependencies": {
          "loose-envify": "^1.4.0",
          "object-assign": "^4.1.1",
          "react-is": "^16.8.1"
        }
      },
      "node_modules/pseudomap": {
        "name": "pseudomap",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
        "integrity": "sha1-8FKijacOYYkX7wqKw0wa5aaChrM=",
        "license": "ISC"
      },
      "node_modules/psl": {
        "name": "psl",
        "version": "1.1.31",
        "resolved": "https://registry.npmjs.org/psl/-/psl-1.1.31.tgz",
        "integrity": "sha512-/6pt4+C+T+wZUieKR620OpzN/LlnNKuWjy1iFLQ/UG35JqHlR/89MP1d96dUfkf6Dne3TuLQzOYEYshJ+Hx8mw==",
        "license": "MIT"
      },
      "node_modules/pump": {
        "name": "pump",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pump/-/pump-3.0.0.tgz",
        "integrity": "sha512-LwZy+p3SFs1Pytd/jYct4wpv49HiYCqd9Rlc5ZVdk0V+8Yzv6jR5Blk3TRmPL1ft69TxP0IMZGJ+WPFU2BFhww==",
        "license": "MIT",
        "dependencies": {
          "end-of-stream": "^1.1.0",
          "once": "^1.3.1"
        }
      },
      "node_modules/punycode": {
        "name": "punycode",
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
        "integrity": "sha1-wNWmOycYgArY4esPpSachN1BhF4=",
        "license": "MIT"
      },
      "node_modules/qs": {
        "name": "qs",
        "version": "6.5.2",
        "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
        "integrity": "sha512-N5ZAX4/LxJmF+7wN74pUD6qAh9/wnvdQcjq9TZjevvXzSUo7bfmw91saqMjzGS2xq91/odN2dW/WOl7qQHNDGA==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.6"
        }
      },
      "node_modules/react": {
        "name": "react",
        "version": "16.8.6",
        "resolved": "https://registry.npmjs.org/react/-/react-16.8.6.tgz",
        "integrity": "sha512-pC0uMkhLaHm11ZSJULfOBqV4tIZkx87ZLvbbQYunNixAAvjnC+snJCg0XQXn9VIsttVsbZP/H/ewzgsd5fxKXw==",
        "license": "MIT",
        "dependencies": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1",
          "prop-types": "^15.6.2",
          "scheduler": "^0.13.6"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/react-is": {
        "name": "react-is",
        "version": "16.8.6",
        "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.8.6.tgz",
        "integrity": "sha512-aUk3bHfZ2bRSVFFbbeVS4i+lNPZr3/WM5jT2J5omUVV1zzcs1nAaf3l51ctA5FFvCRbhrH0bdAsRRQddFJZPtA==",
        "license": "MIT"
      },
      "node_modules/react-reconciler": {
        "name": "react-reconciler",
        "version": "0.20.4",
        "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.20.4.tgz",
        "integrity": "sha512-kxERc4H32zV2lXMg/iMiwQHOtyqf15qojvkcZ5Ja2CPkjVohHw9k70pdDBwrnQhLVetUJBSYyqU3yqrlVTOajA==",
        "license": "MIT",
        "dependencies": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1",
          "prop-types": "^15.6.2",
          "scheduler": "^0.13.6"
        },
        "engines": {
          "node": ">=0.10.0"
        },
        "peerDependencies": {
          "react": "^16.0.0"
        }
      },
      "node_modules/read-pkg": {
        "name": "read-pkg",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
        "integrity": "sha1-nLxoaXj+5l0WwA4rGcI3/Pbjg4k=",
        "license": "MIT",
        "dependencies": {
          "load-json-file": "^4.0.0",
          "normalize-package-data": "^2.3.2",
          "path-type": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/read-pkg-up": {
        "name": "read-pkg-up",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
        "integrity": "sha512-6etQSH7nJGsK0RbG/2TeDzZFa8shjQ1um+SwQQ5cwKy0dhSXdOncEhb1CPpvQG4h7FyOV6EB6YlV0yJvZQNAkA==",
        "license": "MIT",
        "dependencies": {
          "find-up": "^3.0.0",
          "read-pkg": "^3.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/readable-stream": {
        "name": "readable-stream",
        "version": "2.3.6",
        "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
        "integrity": "sha512-tQtKA9WIAhBF3+VLAseyMqZeBjW0AHJoxOtYqSUZNJxauErmLbVm2FW1y+J/YA9dUrAC39ITejlZWhVIwawkKw==",
        "license": "MIT",
        "dependencies": {
          "core-util-is": "~1.0.0",
          "inherits": "~2.0.3",
          "isarray": "~1.0.0",
          "process-nextick-args": "~2.0.0",
          "safe-buffer": "~5.1.1",
          "string_decoder": "~1.1.1",
          "util-deprecate": "~1.0.1"
        }
      },
      "node_modules/readdirp": {
        "name": "readdirp",
        "version": "2.2.1",
        "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-2.2.1.tgz",
        "integrity": "sha512-1JU/8q+VgFZyxwrJ+SVIOsh+KywWGpds3NTqikiKpDMZWScmAYyKIgqkO+ARvNWJfXeXR1zxz7aHF4u4CyH6vQ==",
        "license": "MIT",
        "dependencies": {
          "graceful-fs": "^4.1.11",
          "micromatch": "^3.1.10",
          "readable-stream": "^2.0.2"
        },
        "engines": {
          "node": ">=0.10"
        }
      },
      "node_modules/redeyed": {
        "name": "redeyed",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/redeyed/-/redeyed-2.1.1.tgz",
        "integrity": "sha1-iYS1gV2ZyyIEacme7v/jiRPmzAs=",
        "license": "MIT",
        "dependencies": {
          "esprima": "~4.0.0"
        }
      },
      "node_modules/regenerator-runtime": {
        "name": "regenerator-runtime",
        "version": "0.11.1",
        "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.11.1.tgz",
        "integrity": "sha512-MguG95oij0fC3QV3URf4V2SDYGJhJnJGqvIIgdECeODCT98wSWDAJ94SSuVpYQUoTcGUIL6L4yNB7j1DFFHSBg==",
        "license": "MIT"
      },
      "node_modules/regex-not": {
        "name": "regex-not",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/regex-not/-/regex-not-1.0.2.tgz",
        "integrity": "sha512-J6SDjUgDxQj5NusnOtdFxDwN/+HWykR8GELwctJ7mdqhcyy1xEc4SRFHUXvxTp661YaVKAjfRLZ9cCqS6tn32A==",
        "license": "MIT",
        "dependencies": {
          "extend-shallow": "^3.0.2",
          "safe-regex": "^1.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/regex-not/node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
        "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
        "license": "MIT",
        "dependencies": {
          "assign-symbols": "^1.0.0",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/regex-not/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/release-zalgo": {
        "name": "release-zalgo",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
        "integrity": "sha1-CXALflB0Mpc5Mw5TXFqQ+2eFFzA=",
        "license": "ISC",
        "dependencies": {
          "es6-error": "^4.0.1"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/remove-trailing-separator": {
        "name": "remove-trailing-separator",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/remove-trailing-separator/-/remove-trailing-separator-1.1.0.tgz",
        "integrity": "sha1-wkvOKig62tW8P1jg1IJJuSN52O8=",
        "license": "ISC"
      },
      "node_modules/repeat-element": {
        "name": "repeat-element",
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/repeat-element/-/repeat-element-1.1.3.tgz",
        "integrity": "sha512-ahGq0ZnV5m5XtZLMb+vP76kcAM5nkLqk0lpqAuojSKGgQtn4eRi4ZZGm2olo2zKFH+sMsWaqOCW1dqAnOru72g==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/repeat-string": {
        "name": "repeat-string",
        "version": "1.6.1",
        "resolved": "https://registry.npmjs.org/repeat-string/-/repeat-string-1.6.1.tgz",
        "integrity": "sha1-jcrkcOHIirwtYA//Sndihtp15jc=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10"
        }
      },
      "node_modules/repeating": {
        "name": "repeating",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/repeating/-/repeating-2.0.1.tgz",
        "integrity": "sha1-UhTFOpJtNVJwdSf7q0FdvAjQbdo=",
        "license": "MIT",
        "dependencies": {
          "is-finite": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/request": {
        "name": "request",
        "version": "2.88.0",
        "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
        "integrity": "sha512-NAqBSrijGLZdM0WZNsInLJpkJokL72XYjUpnB0iwsRgxh7dB6COrHnTBNwN0E+lHDAJzu7kLAkDeY08z2/A0hg==",
        "deprecated": "request has been deprecated, see https://github.com/request/request/issues/3142",
        "license": "Apache-2.0",
        "dependencies": {
          "aws-sign2": "~0.7.0",
          "aws4": "^1.8.0",
          "caseless": "~0.12.0",
          "combined-stream": "~1.0.6",
          "extend": "~3.0.2",
          "forever-agent": "~0.6.1",
          "form-data": "~2.3.2",
          "har-validator": "~5.1.0",
          "http-signature": "~1.2.0",
          "is-typedarray": "~1.0.0",
          "isstream": "~0.1.2",
          "json-stringify-safe": "~5.0.1",
          "mime-types": "~2.1.19",
          "oauth-sign": "~0.9.0",
          "performance-now": "^2.1.0",
          "qs": "~6.5.2",
          "safe-buffer": "^5.1.2",
          "tough-cookie": "~2.4.3",
          "tunnel-agent": "^0.6.0",
          "uuid": "^3.3.2"
        },
        "engines": {
          "node": ">= 4"
        }
      },
      "node_modules/require-directory": {
        "name": "require-directory",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
        "integrity": "sha1-jGStX9MNqxyXbiNE/+f3kqam30I=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/require-main-filename": {
        "name": "require-main-filename",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
        "integrity": "sha512-NKN5kMDylKuldxYLSUfrbo5Tuzh4hd+2E8NPPX02mZtn1VuREQToYe/ZdlJy+J3uCpfaiGF05e7B8W0iXbQHmg==",
        "license": "ISC"
      },
      "node_modules/resolve": {
        "name": "resolve",
        "version": "1.10.1",
        "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.10.1.tgz",
        "integrity": "sha512-KuIe4mf++td/eFb6wkaPbMDnP6kObCaEtIDuHOUED6MNUo4K670KZUHuuvYPZDxNF0WVLw49n06M2m2dXphEzA==",
        "license": "MIT",
        "dependencies": {
          "path-parse": "^1.0.6"
        }
      },
      "node_modules/resolve-from": {
        "name": "resolve-from",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
        "integrity": "sha1-six699nWiBvItuZTM17rywoYh0g=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/resolve-url": {
        "name": "resolve-url",
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/resolve-url/-/resolve-url-0.2.1.tgz",
        "integrity": "sha1-LGN/53yJOv0qZj/iGqkIAGjiBSo=",
        "deprecated": "https://github.com/lydell/resolve-url#deprecated",
        "license": "MIT"
      },
      "node_modules/restore-cursor": {
        "name": "restore-cursor",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
        "integrity": "sha1-n37ih/gv0ybU/RYpI9YhKe7g368=",
        "license": "MIT",
        "dependencies": {
          "onetime": "^2.0.0",
          "signal-exit": "^3.0.2"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/ret": {
        "name": "ret",
        "version": "0.1.15",
        "resolved": "https://registry.npmjs.org/ret/-/ret-0.1.15.tgz",
        "integrity": "sha512-TTlYpa+OL+vMMNG24xSlQGEJ3B/RzEfUlLct7b5G/ytav+wPrplCpVMFuwzXbkecJrb6IYo1iFb0S9v37754mg==",
        "license": "MIT",
        "engines": {
          "node": ">=0.12"
        }
      },
      "node_modules/rimraf": {
        "name": "rimraf",
        "version": "2.6.3",
        "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
        "integrity": "sha512-mwqeW5XsA2qAejG46gYdENaxXjx9onRNCfn7L0duuP4hCuTIi/QO7PDK07KJfp1d+izWPrzEJDcSqBa0OZQriA==",
        "license": "ISC",
        "dependencies": {
          "glob": "^7.1.3"
        },
        "bin": {
          "rimraf": "bin.js"
        }
      },
      "node_modules/safe-buffer": {
        "name": "safe-buffer",
        "version": "5.1.2",
        "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
        "integrity": "sha512-Gd2UZBJDkXlY7GbJxfsE8/nvKkUEU1G38c1siN6QP6a9PT9MmHB8GnpscSmMJSoF8LOIrt8ud/wPtojys4G6+g==",
        "license": "MIT"
      },
      "node_modules/safe-regex": {
        "name": "safe-regex",
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/safe-regex/-/safe-regex-1.1.0.tgz",
        "integrity": "sha1-QKNmnzsHfR6UPURinhV91IAjvy4=",
        "license": "MIT",
        "dependencies": {
          "ret": "~0.1.10"
        }
      },
      "node_modules/safer-buffer": {
        "name": "safer-buffer",
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
        "integrity": "sha512-YZo3K82SD7Riyi0E1EQPojLz7kpepnSQI9IyPbHHg1XXXevb5dJI7tpyN2ADxGcQbHG7vcyRHk0cbwqcQriUtg==",
        "license": "MIT"
      },
      "node_modules/scheduler": {
        "name": "scheduler",
        "version": "0.13.6",
        "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.13.6.tgz",
        "integrity": "sha512-IWnObHt413ucAYKsD9J1QShUKkbKLQQHdxRyw73sw4FN26iWr3DY/H34xGPe4nmL1DwXyWmSWmMrA9TfQbE/XQ==",
        "license": "MIT",
        "dependencies": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1"
        }
      },
      "node_modules/semver": {
        "name": "semver",
        "version": "6.0.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-6.0.0.tgz",
        "integrity": "sha512-0UewU+9rFapKFnlbirLi3byoOuhrSsli/z/ihNnvM24vgF+8sNBiI1LZPBSH9wJKUwaUbw+s3hToDLCXkrghrQ==",
        "license": "ISC",
        "bin": {
          "semver": "bin/semver"
        }
      },
      "node_modules/set-blocking": {
        "name": "set-blocking",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
        "integrity": "sha1-BF+XgtARrppoA93TgrJDkrPYkPc=",
        "license": "ISC"
      },
      "node_modules/set-value": {
        "name": "set-value",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/set-value/-/set-value-2.0.0.tgz",
        "integrity": "sha512-hw0yxk9GT/Hr5yJEYnHNKYXkIA8mVJgd9ditYZCe16ZczcaELYYcfvaXesNACk2O8O0nTiPQcQhGUQj8JLzeeg==",
        "deprecated": "Critical bug fixed in v3.0.1, please upgrade to the latest version.",
        "license": "MIT",
        "dependencies": {
          "extend-shallow": "^2.0.1",
          "is-extendable": "^0.1.1",
          "is-plain-object": "^2.0.3",
          "split-string": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/shebang-command": {
        "name": "shebang-command",
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
        "integrity": "sha1-RKrGW2lbAzmJaMOfNj/uXer98eo=",
        "license": "MIT",
        "dependencies": {
          "shebang-regex": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/shebang-regex": {
        "name": "shebang-regex",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
        "integrity": "sha1-2kL0l0DAtC2yypcoVxyxkMmO/qM=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/signal-exit": {
        "name": "signal-exit",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
        "integrity": "sha1-tf3AjxKH6hF4Yo5BXiUTK3NkbG0=",
        "license": "ISC"
      },
      "node_modules/slash": {
        "name": "slash",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/slash/-/slash-1.0.0.tgz",
        "integrity": "sha1-xB8vbDn8FtHNF61LXYlhFK5HDVU=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/slice-ansi": {
        "name": "slice-ansi",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-1.0.0.tgz",
        "integrity": "sha512-POqxBK6Lb3q6s047D/XsDVNPnF9Dl8JSaqe9h9lURl0OdNqy/ujDrOiIHtsqXMGbWWTIomRzAMaTyawAU//Reg==",
        "license": "MIT",
        "dependencies": {
          "is-fullwidth-code-point": "^2.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/snapdragon": {
        "name": "snapdragon",
        "version": "0.8.2",
        "resolved": "https://registry.npmjs.org/snapdragon/-/snapdragon-0.8.2.tgz",
        "integrity": "sha512-FtyOnWN/wCHTVXOMwvSv26d+ko5vWlIDD6zoUJ7LW8vh+ZBC8QdljveRP+crNrtBwioEUWy/4dMtbBjA4ioNlg==",
        "license": "MIT",
        "dependencies": {
          "base": "^0.11.1",
          "debug": "^2.2.0",
          "define-property": "^0.2.5",
          "extend-shallow": "^2.0.1",
          "map-cache": "^0.2.2",
          "source-map": "^0.5.6",
          "source-map-resolve": "^0.5.0",
          "use": "^3.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon-node": {
        "name": "snapdragon-node",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/snapdragon-node/-/snapdragon-node-2.1.1.tgz",
        "integrity": "sha512-O27l4xaMYt/RSQ5TR3vpWCAB5Kb/czIcqUFOM/C4fYcLnbZUc1PkjTAMjof2pBWaSTwOUd6qUHcFGVGj7aIwnw==",
        "license": "MIT",
        "dependencies": {
          "define-property": "^1.0.0",
          "isobject": "^3.0.0",
          "snapdragon-util": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon-node/node_modules/define-property": {
        "name": "define-property",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
        "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon-util": {
        "name": "snapdragon-util",
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/snapdragon-util/-/snapdragon-util-3.0.1.tgz",
        "integrity": "sha512-mbKkMdQKsjX4BAL4bRYTj21edOf8cN7XHdYUJEe+Zn99hVEYcMvKPct1IqNe7+AZPirn8BCDOQBHQZknqmKlZQ==",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.2.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon/node_modules/define-property": {
        "name": "define-property",
        "version": "0.2.5",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
        "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon/node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
        "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon/node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
        "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon/node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
        "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^0.1.6",
          "is-data-descriptor": "^0.1.4",
          "kind-of": "^5.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/snapdragon/node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
        "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/source-map": {
        "name": "source-map",
        "version": "0.5.7",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
        "integrity": "sha1-igOdLRAh0i0eoUyA2OpGi6LvP8w=",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/source-map-resolve": {
        "name": "source-map-resolve",
        "version": "0.5.2",
        "resolved": "https://registry.npmjs.org/source-map-resolve/-/source-map-resolve-0.5.2.tgz",
        "integrity": "sha512-MjqsvNwyz1s0k81Goz/9vRBe9SZdB09Bdw+/zYyO+3CuPk6fouTaxscHkgtE8jKvf01kVfl8riHzERQ/kefaSA==",
        "license": "MIT",
        "dependencies": {
          "atob": "^2.1.1",
          "decode-uri-component": "^0.2.0",
          "resolve-url": "^0.2.1",
          "source-map-url": "^0.4.0",
          "urix": "^0.1.0"
        }
      },
      "node_modules/source-map-support": {
        "name": "source-map-support",
        "version": "0.5.12",
        "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.12.tgz",
        "integrity": "sha512-4h2Pbvyy15EE02G+JOZpUCmqWJuqrs+sEkzewTm++BPi7Hvn/HwcqLAcNxYAyI0x13CpPPn+kMjl+hplXMHITQ==",
        "license": "MIT",
        "dependencies": {
          "buffer-from": "^1.0.0",
          "source-map": "^0.6.0"
        }
      },
      "node_modules/source-map-support/node_modules/source-map": {
        "name": "source-map",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
        "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/source-map-url": {
        "name": "source-map-url",
        "version": "0.4.0",
        "resolved": "https://registry.npmjs.org/source-map-url/-/source-map-url-0.4.0.tgz",
        "integrity": "sha1-PpNdfd1zYxuXZZlW1VEo6HtQhKM=",
        "license": "MIT"
      },
      "node_modules/spawn-wrap": {
        "name": "spawn-wrap",
        "version": "1.4.2",
        "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.2.tgz",
        "integrity": "sha512-vMwR3OmmDhnxCVxM8M+xO/FtIp6Ju/mNaDfCMMW7FDcLRTPFWUswec4LXJHTJE2hwTI9O0YBfygu4DalFl7Ylg==",
        "license": "ISC",
        "dependencies": {
          "foreground-child": "^1.5.6",
          "mkdirp": "^0.5.0",
          "os-homedir": "^1.0.1",
          "rimraf": "^2.6.2",
          "signal-exit": "^3.0.2",
          "which": "^1.3.0"
        }
      },
      "node_modules/spdx-correct": {
        "name": "spdx-correct",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
        "integrity": "sha512-lr2EZCctC2BNR7j7WzJ2FpDznxky1sjfxvvYEyzxNyb6lZXHODmEoJeFu4JupYlkfha1KZpJyoqiJ7pgA1qq8Q==",
        "license": "Apache-2.0",
        "dependencies": {
          "spdx-expression-parse": "^3.0.0",
          "spdx-license-ids": "^3.0.0"
        }
      },
      "node_modules/spdx-exceptions": {
        "name": "spdx-exceptions",
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
        "integrity": "sha512-2XQACfElKi9SlVb1CYadKDXvoajPgBVPn/gOQLrTvHdElaVhr7ZEbqJaRnJLVNeaI4cMEAgVCeBMKF6MWRDCRA==",
        "license": "CC-BY-3.0"
      },
      "node_modules/spdx-expression-parse": {
        "name": "spdx-expression-parse",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
        "integrity": "sha512-Yg6D3XpRD4kkOmTpdgbUiEJFKghJH03fiC1OPll5h/0sO6neh2jqRDVHOQ4o/LMea0tgCkbMgea5ip/e+MkWyg==",
        "license": "MIT",
        "dependencies": {
          "spdx-exceptions": "^2.1.0",
          "spdx-license-ids": "^3.0.0"
        }
      },
      "node_modules/spdx-license-ids": {
        "name": "spdx-license-ids",
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.4.tgz",
        "integrity": "sha512-7j8LYJLeY/Yb6ACbQ7F76qy5jHkp0U6jgBfJsk97bwWlVUnUWsAgpyaCvo17h0/RQGnQ036tVDomiwoI4pDkQA==",
        "license": "CC0-1.0"
      },
      "node_modules/split-string": {
        "name": "split-string",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/split-string/-/split-string-3.1.0.tgz",
        "integrity": "sha512-NzNVhJDYpwceVVii8/Hu6DKfD2G+NrQHlS/V/qgv763EYudVwEcMQNxd2lh+0VrUByXN/oJkl5grOhYWvQUYiw==",
        "license": "MIT",
        "dependencies": {
          "extend-shallow": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/split-string/node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
        "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
        "license": "MIT",
        "dependencies": {
          "assign-symbols": "^1.0.0",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/split-string/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/sprintf-js": {
        "name": "sprintf-js",
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
        "integrity": "sha1-BOaSb2YolTVPPdAVIDYzuFcpfiw=",
        "license": "BSD-3-Clause"
      },
      "node_modules/sshpk": {
        "name": "sshpk",
        "version": "1.16.1",
        "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
        "integrity": "sha512-HXXqVUq7+pcKeLqqZj6mHFUMvXtOJt1uoUx09pFW6011inTMxqI8BA8PM95myrIyyKwdnzjdFjLiE6KBPVtJIg==",
        "license": "MIT",
        "dependencies": {
          "asn1": "~0.2.3",
          "assert-plus": "^1.0.0",
          "bcrypt-pbkdf": "^1.0.0",
          "dashdash": "^1.12.0",
          "ecc-jsbn": "~0.1.1",
          "getpass": "^0.1.1",
          "jsbn": "~0.1.0",
          "safer-buffer": "^2.0.2",
          "tweetnacl": "~0.14.0"
        },
        "bin": {
          "sshpk-conv": "bin/sshpk-conv",
          "sshpk-sign": "bin/sshpk-sign",
          "sshpk-verify": "bin/sshpk-verify"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/stack-utils": {
        "name": "stack-utils",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
        "integrity": "sha512-MTX+MeG5U994cazkjd/9KNAapsHnibjMLnfXodlkXw76JEea0UiNzrqidzo1emMwk7w5Qhc9jd4Bn9TBb1MFwA==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend": {
        "name": "static-extend",
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/static-extend/-/static-extend-0.1.2.tgz",
        "integrity": "sha1-YICcOcv/VTNyJv1eC1IPNB8ftcY=",
        "license": "MIT",
        "dependencies": {
          "define-property": "^0.2.5",
          "object-copy": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend/node_modules/define-property": {
        "name": "define-property",
        "version": "0.2.5",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
        "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
        "license": "MIT",
        "dependencies": {
          "is-descriptor": "^0.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend/node_modules/is-accessor-descriptor": {
        "name": "is-accessor-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
        "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend/node_modules/is-data-descriptor": {
        "name": "is-data-descriptor",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
        "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend/node_modules/is-descriptor": {
        "name": "is-descriptor",
        "version": "0.1.6",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
        "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
        "license": "MIT",
        "dependencies": {
          "is-accessor-descriptor": "^0.1.6",
          "is-data-descriptor": "^0.1.4",
          "kind-of": "^5.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/static-extend/node_modules/is-descriptor/node_modules/kind-of": {
        "name": "kind-of",
        "version": "5.1.0",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
        "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/string_decoder": {
        "name": "string_decoder",
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
        "integrity": "sha512-n/ShnvDi6FHbbVfviro+WojiFzv+s8MPMHBczVePfUpDJLwoLT0ht1l4YwBCbi8pJAveEEdnkHyPyTP/mzRfwg==",
        "license": "MIT",
        "dependencies": {
          "safe-buffer": "~5.1.0"
        }
      },
      "node_modules/string-length": {
        "name": "string-length",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/string-length/-/string-length-2.0.0.tgz",
        "integrity": "sha1-1A27aGo6zpYMHP/KVivyxF+DY+0=",
        "license": "MIT",
        "dependencies": {
          "astral-regex": "^1.0.0",
          "strip-ansi": "^4.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/string-length/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
        "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/string-length/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
        "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/string-width": {
        "name": "string-width",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
        "integrity": "sha512-nOqH59deCq9SRHlxq1Aw85Jnt4w6KvLKqWVik6oA9ZklXLNIOlqg4F2yrT1MVaTjAqvVwdfeZ7w7aCvJD7ugkw==",
        "license": "MIT",
        "dependencies": {
          "is-fullwidth-code-point": "^2.0.0",
          "strip-ansi": "^4.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/string-width/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
        "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/string-width/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
        "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
        "integrity": "sha1-ajhfuIU9lS1f8F0Oiq+UJ43GPc8=",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/strip-bom": {
        "name": "strip-bom",
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
        "integrity": "sha1-IzTBjpx1n3vdVv3vfprj1YjmjtM=",
        "license": "MIT",
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/strip-eof": {
        "name": "strip-eof",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/strip-eof/-/strip-eof-1.0.0.tgz",
        "integrity": "sha1-u0P/VZim6wXYm1n80SnJgzE2Br8=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/supports-color": {
        "name": "supports-color",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
        "integrity": "sha1-U10EXOa2Nj+kARcIRimZXp3zJMc=",
        "license": "MIT",
        "engines": {
          "node": ">=0.8.0"
        }
      },
      "node_modules/tap": {
        "name": "tap",
        "version": "13.0.3",
        "resolved": "https://registry.npmjs.org/tap/-/tap-13.0.3.tgz",
        "integrity": "sha512-G9S2OUsE97gdHk9iRGZkPKBh/+to7P1Tqi5kTUtKDrVA2sE2cmUMpmp0izlVtqxMRTVjiVmhLVpMdHitwoHEHQ==",
        "license": "ISC",
        "dependencies": {
          "async-hook-domain": "^1.1.0",
          "bind-obj-methods": "^2.0.0",
          "browser-process-hrtime": "^1.0.0",
          "capture-stack-trace": "^1.0.0",
          "chokidar": "^2.1.5",
          "color-support": "^1.1.0",
          "coveralls": "^3.0.3",
          "domain-browser": "^1.2.0",
          "esm": "^3.2.22",
          "findit": "^2.0.0",
          "foreground-child": "^1.3.3",
          "fs-exists-cached": "^1.0.0",
          "function-loop": "^1.0.2",
          "glob": "^7.1.3",
          "import-jsx": "^2.0.0",
          "isexe": "^2.0.0",
          "istanbul-lib-processinfo": "^1.0.0",
          "jackspeak": "^1.3.7",
          "minipass": "^2.3.5",
          "mkdirp": "^0.5.1",
          "nyc": "^14.1.0",
          "opener": "^1.5.1",
          "own-or": "^1.0.0",
          "own-or-env": "^1.0.1",
          "rimraf": "^2.6.3",
          "signal-exit": "^3.0.0",
          "source-map-support": "^0.5.12",
          "stack-utils": "^1.0.2",
          "tap-mocha-reporter": "^4.0.1",
          "tap-parser": "^9.3.2",
          "tap-yaml": "^1.0.0",
          "tcompare": "^2.2.0",
          "treport": "^0.3.0",
          "trivial-deferred": "^1.0.1",
          "ts-node": "^8.1.0",
          "typescript": "^3.4.3",
          "which": "^1.3.1",
          "write-file-atomic": "^2.4.2",
          "yaml": "^1.5.0",
          "yapool": "^1.0.0"
        },
        "bin": {
          "tap": "bin/run.js"
        },
        "engines": {
          "node": ">=8"
        }
      },
      "node_modules/tap-mocha-reporter": {
        "name": "tap-mocha-reporter",
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-4.0.1.tgz",
        "integrity": "sha512-/KfXaaYeSPn8qBi5Be8WSIP3iKV83s2uj2vzImJAXmjNu22kzqZ+1Dv1riYWa53sPCiyo1R1w1jbJrftF8SpcQ==",
        "license": "ISC",
        "dependencies": {
          "color-support": "^1.1.0",
          "debug": "^2.1.3",
          "diff": "^1.3.2",
          "escape-string-regexp": "^1.0.3",
          "glob": "^7.0.5",
          "readable-stream": "^2.1.5",
          "tap-parser": "^8.0.0",
          "tap-yaml": "0 || 1",
          "unicode-length": "^1.0.0"
        },
        "bin": {
          "tap-mocha-reporter": "index.js"
        },
        "optionalDependencies": {
          "readable-stream": "^2.1.5"
        }
      },
      "node_modules/tap-mocha-reporter/node_modules/tap-parser": {
        "name": "tap-parser",
        "version": "8.1.0",
        "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-8.1.0.tgz",
        "integrity": "sha512-GgOzgDwThYLxhVR83RbS1JUR1TxcT+jfZsrETgPAvFdr12lUOnuvrHOBaUQgpkAp6ZyeW6r2Nwd91t88M0ru3w==",
        "license": "MIT",
        "dependencies": {
          "events-to-array": "^1.0.1",
          "minipass": "^2.2.0",
          "tap-yaml": "0 || 1"
        },
        "bin": {
          "tap-parser": "bin/cmd.js"
        }
      },
      "node_modules/tap-parser": {
        "name": "tap-parser",
        "version": "9.3.2",
        "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-9.3.2.tgz",
        "integrity": "sha512-bQ76sD6ZP9loxdk/KiqXgWDEfjJYiZUj0a7ElnLMSny7Q8G72UMcOQee85j5ddKHA9fzGAoL6cfJbg3rur7S5g==",
        "license": "MIT",
        "dependencies": {
          "events-to-array": "^1.0.1",
          "minipass": "^2.2.0",
          "tap-yaml": "^1.0.0"
        },
        "bin": {
          "tap-parser": "bin/cmd.js"
        }
      },
      "node_modules/tap-yaml": {
        "name": "tap-yaml",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
        "integrity": "sha512-Rxbx4EnrWkYk0/ztcm5u3/VznbyFJpyXO12dDBHKWiDVxy7O2Qw6MRrwO5H6Ww0U5YhRY/4C/VzWmFPhBQc4qQ==",
        "license": "ISC",
        "dependencies": {
          "yaml": "^1.5.0"
        }
      },
      "node_modules/tcompare": {
        "name": "tcompare",
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-2.2.0.tgz",
        "integrity": "sha512-+Mr0UBIE3ncNn0wJvKsw8ph61QoaDvR6Q8WkxWIHxWLcKf8SHGyTTkGLMX//4NKQ/Pe1Uu64oXYJsxLyAXXWdA==",
        "license": "ISC"
      },
      "node_modules/test-exclude": {
        "name": "test-exclude",
        "version": "5.2.3",
        "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
        "integrity": "sha512-M+oxtseCFO3EDtAaGH7iiej3CBkzXqFMbzqYAACdzKui4eZA+pq3tZEwChvOdNfa7xxy8BfbmgJSIr43cC/+2g==",
        "license": "ISC",
        "dependencies": {
          "glob": "^7.1.3",
          "minimatch": "^3.0.4",
          "read-pkg-up": "^4.0.0",
          "require-main-filename": "^2.0.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/to-fast-properties": {
        "name": "to-fast-properties",
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-1.0.3.tgz",
        "integrity": "sha1-uDVx+k2MJbguIxsG46MFXeTKGkc=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/to-object-path": {
        "name": "to-object-path",
        "version": "0.3.0",
        "resolved": "https://registry.npmjs.org/to-object-path/-/to-object-path-0.3.0.tgz",
        "integrity": "sha1-KXWIt7Dn4KwI4E5nL4XB9JmeF68=",
        "license": "MIT",
        "dependencies": {
          "kind-of": "^3.0.2"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/to-regex": {
        "name": "to-regex",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/to-regex/-/to-regex-3.0.2.tgz",
        "integrity": "sha512-FWtleNAtZ/Ki2qtqej2CXTOayOH9bHDQF+Q48VpWyDXjbYxA4Yz8iDB31zXOBUlOHHKidDbqGVrTUvQMPmBGBw==",
        "license": "MIT",
        "dependencies": {
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "regex-not": "^1.0.2",
          "safe-regex": "^1.1.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/to-regex-range": {
        "name": "to-regex-range",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-2.1.1.tgz",
        "integrity": "sha1-fIDBe53+vlmeJzZ+DU3VWQFB2zg=",
        "license": "MIT",
        "dependencies": {
          "is-number": "^3.0.0",
          "repeat-string": "^1.6.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/to-regex/node_modules/extend-shallow": {
        "name": "extend-shallow",
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
        "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
        "license": "MIT",
        "dependencies": {
          "assign-symbols": "^1.0.0",
          "is-extendable": "^1.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/to-regex/node_modules/is-extendable": {
        "name": "is-extendable",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
        "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
        "license": "MIT",
        "dependencies": {
          "is-plain-object": "^2.0.4"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/tough-cookie": {
        "name": "tough-cookie",
        "version": "2.4.3",
        "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
        "integrity": "sha512-Q5srk/4vDM54WJsJio3XNn6K2sCG+CQ8G5Wz6bZhRZoAe/+TxjWB/GlFAnYEbkYVlON9FMk/fE3h2RLpPXo4lQ==",
        "license": "BSD-3-Clause",
        "dependencies": {
          "psl": "^1.1.24",
          "punycode": "^1.4.1"
        },
        "engines": {
          "node": ">=0.8"
        }
      },
      "node_modules/treport": {
        "name": "treport",
        "version": "0.3.0",
        "resolved": "https://registry.npmjs.org/treport/-/treport-0.3.0.tgz",
        "integrity": "sha512-THr7NS5iLJewcAiaBkxAuCVoakw84hAVY9n2kFNZqlFKHrZeGw8sNR4VhmT23JB1/JnCmPcIOmooTdYYVTI5hA==",
        "license": "ISC",
        "dependencies": {
          "cardinal": "^2.1.1",
          "chalk": "^2.4.2",
          "import-jsx": "^2.0.0",
          "ink": "^2.1.1",
          "ms": "^2.1.1",
          "react": "^16.8.6",
          "string-length": "^2.0.0",
          "tap-parser": "^9.3.2",
          "unicode-length": "^2.0.1"
        }
      },
      "node_modules/treport/node_modules/ansi-styles": {
        "name": "ansi-styles",
        "version": "3.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
        "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
        "license": "MIT",
        "dependencies": {
          "color-convert": "^1.9.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/treport/node_modules/chalk": {
        "name": "chalk",
        "version": "2.4.2",
        "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
        "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
        "license": "MIT",
        "dependencies": {
          "ansi-styles": "^3.2.1",
          "escape-string-regexp": "^1.0.5",
          "supports-color": "^5.3.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/treport/node_modules/ms": {
        "name": "ms",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
        "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg==",
        "license": "MIT"
      },
      "node_modules/treport/node_modules/punycode": {
        "name": "punycode",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
        "integrity": "sha512-XRsRjdf+j5ml+y/6GKHPZbrF/8p2Yga0JPtdqTIY2Xe5ohJPD9saDJJLPvp9+NSBprVvevdXZybnj2cv8OEd0A==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/treport/node_modules/supports-color": {
        "name": "supports-color",
        "version": "5.5.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
        "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
        "license": "MIT",
        "dependencies": {
          "has-flag": "^3.0.0"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/treport/node_modules/unicode-length": {
        "name": "unicode-length",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-2.0.1.tgz",
        "integrity": "sha1-hN3DCcIyNjX2QuhtRI6ersCEwEo=",
        "license": "MIT",
        "dependencies": {
          "punycode": "^2.0.0",
          "strip-ansi": "^3.0.1"
        }
      },
      "node_modules/trim-right": {
        "name": "trim-right",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/trim-right/-/trim-right-1.0.1.tgz",
        "integrity": "sha1-yy4SAwZ+DI3h9hQJS5/kVwTqYAM=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/trivial-deferred": {
        "name": "trivial-deferred",
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
        "integrity": "sha1-N21NKdlR1jaKb3oK6FwvTV4GWPM=",
        "license": "ISC"
      },
      "node_modules/ts-node": {
        "name": "ts-node",
        "version": "8.1.0",
        "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.1.0.tgz",
        "integrity": "sha512-34jpuOrxDuf+O6iW1JpgTRDFynUZ1iEqtYruBqh35gICNjN8x+LpVcPAcwzLPi9VU6mdA3ym+x233nZmZp445A==",
        "license": "MIT",
        "dependencies": {
          "arg": "^4.1.0",
          "diff": "^3.1.0",
          "make-error": "^1.1.1",
          "source-map-support": "^0.5.6",
          "yn": "^3.0.0"
        },
        "bin": {
          "ts-node": "dist/bin.js"
        },
        "engines": {
          "node": ">=4.2.0"
        },
        "peerDependencies": {
          "typescript": ">=2.0"
        }
      },
      "node_modules/ts-node/node_modules/diff": {
        "name": "diff",
        "version": "3.5.0",
        "resolved": "https://registry.npmjs.org/diff/-/diff-3.5.0.tgz",
        "integrity": "sha512-A46qtFgd+g7pDZinpnwiRJtxbC1hpgf0uzP3iG89scHk0AUC7A1TGxf5OiiOUv/JMZR8GOt8hL900hV0bOy5xA==",
        "license": "BSD-3-Clause",
        "engines": {
          "node": ">=0.3.1"
        }
      },
      "node_modules/tunnel-agent": {
        "name": "tunnel-agent",
        "version": "0.6.0",
        "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
        "integrity": "sha1-J6XeoGs2sEoKmWZ3SykIaPD8QP0=",
        "license": "Apache-2.0",
        "dependencies": {
          "safe-buffer": "^5.0.1"
        },
        "engines": {
          "node": "*"
        }
      },
      "node_modules/tweetnacl": {
        "name": "tweetnacl",
        "version": "0.14.5",
        "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
        "integrity": "sha1-WuaBd/GS1EViadEIr6k/+HQ/T2Q=",
        "license": "Unlicense"
      },
      "node_modules/typescript": {
        "name": "typescript",
        "version": "3.4.5",
        "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.4.5.tgz",
        "integrity": "sha512-YycBxUb49UUhdNMU5aJ7z5Ej2XGmaIBL0x34vZ82fn3hGvD+bgrMrVDpatgz2f7YxUMJxMkbWxJZeAvDxVe7Vw==",
        "license": "Apache-2.0",
        "bin": {
          "tsc": "bin/tsc",
          "tsserver": "bin/tsserver"
        },
        "engines": {
          "node": ">=4.2.0"
        }
      },
      "node_modules/uglify-js": {
        "name": "uglify-js",
        "version": "3.5.8",
        "resolved": "https://registry.npmjs.org/uglify-js/-/uglify-js-3.5.8.tgz",
        "integrity": "sha512-GFSjB1nZIzoIq70qvDRtWRORHX3vFkAnyK/rDExc0BN7r9+/S+Voz3t/fwJuVfjppAMz+ceR2poE7tkhvnVwQQ==",
        "license": "BSD-2-Clause",
        "optional": true,
        "dependencies": {
          "commander": "~2.20.0",
          "source-map": "~0.6.1"
        },
        "bin": {
          "uglifyjs": "bin/uglifyjs"
        },
        "engines": {
          "node": ">=0.8.0"
        }
      },
      "node_modules/uglify-js/node_modules/source-map": {
        "name": "source-map",
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
        "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
        "license": "BSD-3-Clause",
        "optional": true,
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/unicode-length": {
        "name": "unicode-length",
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
        "integrity": "sha1-Wtp6f+1RhBpBijKM8UlHisg1irs=",
        "license": "MIT",
        "dependencies": {
          "punycode": "^1.3.2",
          "strip-ansi": "^3.0.1"
        }
      },
      "node_modules/union-value": {
        "name": "union-value",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/union-value/-/union-value-1.0.0.tgz",
        "integrity": "sha1-XHHDTLW61dzr4+oM0IIHulqhrqQ=",
        "license": "MIT",
        "dependencies": {
          "arr-union": "^3.1.0",
          "get-value": "^2.0.6",
          "is-extendable": "^0.1.1",
          "set-value": "^0.4.3"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/union-value/node_modules/set-value": {
        "name": "set-value",
        "version": "0.4.3",
        "resolved": "https://registry.npmjs.org/set-value/-/set-value-0.4.3.tgz",
        "integrity": "sha1-fbCPnT0i3H945Trzw79GZuzfzPE=",
        "deprecated": "Critical bug fixed in v3.0.1, please upgrade to the latest version.",
        "license": "MIT",
        "dependencies": {
          "extend-shallow": "^2.0.1",
          "is-extendable": "^0.1.1",
          "is-plain-object": "^2.0.1",
          "to-object-path": "^0.3.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/unset-value": {
        "name": "unset-value",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/unset-value/-/unset-value-1.0.0.tgz",
        "integrity": "sha1-g3aHP30jNRef+x5vw6jtDfyKtVk=",
        "license": "MIT",
        "dependencies": {
          "has-value": "^0.3.1",
          "isobject": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/unset-value/node_modules/has-value": {
        "name": "has-value",
        "version": "0.3.1",
        "resolved": "https://registry.npmjs.org/has-value/-/has-value-0.3.1.tgz",
        "integrity": "sha1-ex9YutpiyoJ+wKIHgCVlSEWZXh8=",
        "license": "MIT",
        "dependencies": {
          "get-value": "^2.0.3",
          "has-values": "^0.1.4",
          "isobject": "^2.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/unset-value/node_modules/has-value/node_modules/isobject": {
        "name": "isobject",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/isobject/-/isobject-2.1.0.tgz",
        "integrity": "sha1-8GVWEJaj8dou9GJy+BXIQNh+DIk=",
        "license": "MIT",
        "dependencies": {
          "isarray": "1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/unset-value/node_modules/has-values": {
        "name": "has-values",
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/has-values/-/has-values-0.1.4.tgz",
        "integrity": "sha1-bWHeldkd/Km5oCCJrThL/49it3E=",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/upath": {
        "name": "upath",
        "version": "1.1.2",
        "resolved": "https://registry.npmjs.org/upath/-/upath-1.1.2.tgz",
        "integrity": "sha512-kXpym8nmDmlCBr7nKdIx8P2jNBa+pBpIUFRnKJ4dr8htyYGJFokkr2ZvERRtUN+9SY+JqXouNgUPtv6JQva/2Q==",
        "license": "MIT",
        "engines": {
          "node": ">=4",
          "yarn": "*"
        }
      },
      "node_modules/uri-js": {
        "name": "uri-js",
        "version": "4.2.2",
        "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
        "integrity": "sha512-KY9Frmirql91X2Qgjry0Wd4Y+YTdrdZheS8TFwvkbLWf/G5KNJDCh6pKL5OZctEW4+0Baa5idK2ZQuELRwPznQ==",
        "license": "BSD-2-Clause",
        "dependencies": {
          "punycode": "^2.1.0"
        }
      },
      "node_modules/uri-js/node_modules/punycode": {
        "name": "punycode",
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
        "integrity": "sha512-XRsRjdf+j5ml+y/6GKHPZbrF/8p2Yga0JPtdqTIY2Xe5ohJPD9saDJJLPvp9+NSBprVvevdXZybnj2cv8OEd0A==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/urix": {
        "name": "urix",
        "version": "0.1.0",
        "resolved": "https://registry.npmjs.org/urix/-/urix-0.1.0.tgz",
        "integrity": "sha1-2pN/emLiH+wf0Y1Js1wpNQZ6bHI=",
        "deprecated": "Please see https://github.com/lydell/urix#deprecated",
        "license": "MIT"
      },
      "node_modules/use": {
        "name": "use",
        "version": "3.1.1",
        "resolved": "https://registry.npmjs.org/use/-/use-3.1.1.tgz",
        "integrity": "sha512-cwESVXlO3url9YWlFW/TA9cshCEhtu7IKJ/p5soJ/gGpj7vbvFrAY/eIioQ6Dw23KjZhYgiIo8HOs1nQ2vr/oQ==",
        "license": "MIT",
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/util-deprecate": {
        "name": "util-deprecate",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
        "integrity": "sha1-RQ1Nyfpw3nMnYvvS1KKJgUGaDM8=",
        "license": "MIT"
      },
      "node_modules/uuid": {
        "name": "uuid",
        "version": "3.3.2",
        "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.2.tgz",
        "integrity": "sha512-yXJmeNaw3DnnKAOKJE51sL/ZaYfWJRl1pK9dr19YFCu0ObS231AB1/LbqTKRAQ5kw8A90rA6fr4riOUpTZvQZA==",
        "license": "MIT",
        "bin": {
          "uuid": "bin/uuid"
        }
      },
      "node_modules/validate-npm-package-license": {
        "name": "validate-npm-package-license",
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
        "integrity": "sha512-DpKm2Ui/xN7/HQKCtpZxoRWBhZ9Z0kqtygG8XCgNQ8ZlDnxuQmWhj566j8fN4Cu3/JmbhsDo7fcAJq4s9h27Ew==",
        "license": "Apache-2.0",
        "dependencies": {
          "spdx-correct": "^3.0.0",
          "spdx-expression-parse": "^3.0.0"
        }
      },
      "node_modules/verror": {
        "name": "verror",
        "version": "1.10.0",
        "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
        "integrity": "sha1-OhBcoXBTr1XW4nDB+CiGguGNpAA=",
        "engines": [
          "node >=0.6.0"
        ],
        "license": "MIT",
        "dependencies": {
          "assert-plus": "^1.0.0",
          "core-util-is": "1.0.2",
          "extsprintf": "^1.2.0"
        }
      },
      "node_modules/which": {
        "name": "which",
        "version": "1.3.1",
        "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
        "integrity": "sha512-HxJdYWq1MTIQbJ3nw0cqssHoTNU267KlrDuGZ1WYlxDStUtKUhOaJmh112/TZmHxxUfuJqPXSOm7tDyas0OSIQ==",
        "license": "ISC",
        "dependencies": {
          "isexe": "^2.0.0"
        },
        "bin": {
          "which": "bin/which"
        }
      },
      "node_modules/which-module": {
        "name": "which-module",
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
        "integrity": "sha1-2e8H3Od7mQK4o6j6SzHD4/fm6Ho=",
        "license": "ISC"
      },
      "node_modules/widest-line": {
        "name": "widest-line",
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-2.0.1.tgz",
        "integrity": "sha512-Ba5m9/Fa4Xt9eb2ELXt77JxVDV8w7qQrH0zS/TWSJdLyAwQjWoOzpzj5lwVftDz6n/EOu3tNACS84v509qwnJA==",
        "license": "MIT",
        "dependencies": {
          "string-width": "^2.1.1"
        },
        "engines": {
          "node": ">=4"
        }
      },
      "node_modules/wordwrap": {
        "name": "wordwrap",
        "version": "0.0.3",
        "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
        "integrity": "sha1-o9XabNXAvAAI03I0u68b7WMFkQc=",
        "license": "MIT",
        "engines": {
          "node": ">=0.4.0"
        }
      },
      "node_modules/wrap-ansi": {
        "name": "wrap-ansi",
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
        "integrity": "sha1-2Pw9KE3QV5T+hJc8rs3Rz4JP3YU=",
        "license": "MIT",
        "dependencies": {
          "string-width": "^1.0.1",
          "strip-ansi": "^3.0.1"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point": {
        "name": "is-fullwidth-code-point",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
        "integrity": "sha1-754xOG8DGn8NZDr4L95QxFfvAMs=",
        "license": "MIT",
        "dependencies": {
          "number-is-nan": "^1.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/wrap-ansi/node_modules/string-width": {
        "name": "string-width",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
        "integrity": "sha1-EYvfW4zcUaKn5w0hHgfisLmxB9M=",
        "license": "MIT",
        "dependencies": {
          "code-point-at": "^1.0.0",
          "is-fullwidth-code-point": "^1.0.0",
          "strip-ansi": "^3.0.0"
        },
        "engines": {
          "node": ">=0.10.0"
        }
      },
      "node_modules/wrappy": {
        "name": "wrappy",
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
        "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8=",
        "license": "ISC"
      },
      "node_modules/write-file-atomic": {
        "name": "write-file-atomic",
        "version": "2.4.2",
        "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.2.tgz",
        "integrity": "sha512-s0b6vB3xIVRLWywa6X9TOMA7k9zio0TMOsl9ZnDkliA/cfJlpHXAscj0gbHVJiTdIuAYpIyqS5GW91fqm6gG5g==",
        "license": "ISC",
        "dependencies": {
          "graceful-fs": "^4.1.11",
          "imurmurhash": "^0.1.4",
          "signal-exit": "^3.0.2"
        }
      },
      "node_modules/y18n": {
        "name": "y18n",
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
        "integrity": "sha512-r9S/ZyXu/Xu9q1tYlpsLIsa3EeLXXk0VwlxqTcFRfg9EhMW+17kbt9G0NrgCmhGb5vT2hyhJZLfDGx+7+5Uj/w==",
        "license": "ISC"
      },
      "node_modules/yallist": {
        "name": "yallist",
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
        "integrity": "sha1-HBH5IY8HYImkfdUS+TxmmaaoHVI=",
        "license": "ISC"
      },
      "node_modules/yaml": {
        "name": "yaml",
        "version": "1.5.0",
        "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.5.0.tgz",
        "integrity": "sha512-nKxSWOa7vxAP2pikrGxbkZsG/garQseRiLn9mIDjzwoQsyVy7ZWIpLoARejnINGGLA4fttuzRFFNxxbsztdJgw==",
        "license": "ISC",
        "dependencies": {
          "@babel/runtime": "^7.4.3"
        },
        "engines": {
          "node": ">= 6"
        }
      },
      "node_modules/yapool": {
        "name": "yapool",
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
        "integrity": "sha1-9pPymjFbUNmp2iZGp6ZkXJaYW2o=",
        "license": "ISC"
      },
      "node_modules/yargs": {
        "name": "yargs",
        "version": "13.2.2",
        "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.2.2.tgz",
        "integrity": "sha512-WyEoxgyTD3w5XRpAQNYUB9ycVH/PQrToaTXdYXRdOXvEy1l19br+VJsc0vcO8PTGg5ro/l/GY7F/JMEBmI0BxA==",
        "license": "MIT",
        "dependencies": {
          "cliui": "^4.0.0",
          "find-up": "^3.0.0",
          "get-caller-file": "^2.0.1",
          "os-locale": "^3.1.0",
          "require-directory": "^2.1.1",
          "require-main-filename": "^2.0.0",
          "set-blocking": "^2.0.0",
          "string-width": "^3.0.0",
          "which-module": "^2.0.0",
          "y18n": "^4.0.0",
          "yargs-parser": "^13.0.0"
        }
      },
      "node_modules/yargs-parser": {
        "name": "yargs-parser",
        "version": "13.0.0",
        "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.0.0.tgz",
        "integrity": "sha512-w2LXjoL8oRdRQN+hOyppuXs+V/fVAYtpcrRxZuF7Kt/Oc+Jr2uAcVntaUTNT6w5ihoWfFDpNY8CPx1QskxZ/pw==",
        "license": "ISC",
        "dependencies": {
          "camelcase": "^5.0.0",
          "decamelize": "^1.2.0"
        }
      },
      "node_modules/yargs/node_modules/ansi-regex": {
        "name": "ansi-regex",
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
        "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/yargs/node_modules/string-width": {
        "name": "string-width",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
        "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
        "license": "MIT",
        "dependencies": {
          "emoji-regex": "^7.0.1",
          "is-fullwidth-code-point": "^2.0.0",
          "strip-ansi": "^5.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/yargs/node_modules/strip-ansi": {
        "name": "strip-ansi",
        "version": "5.2.0",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
        "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
        "license": "MIT",
        "dependencies": {
          "ansi-regex": "^4.1.0"
        },
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/yn": {
        "name": "yn",
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.0.tgz",
        "integrity": "sha512-kKfnnYkbTfrAdd0xICNFw7Atm8nKpLcLv9AZGEt+kczL/WQVai4e2V6ZN8U/O+iI6WrNuJjNNOyu4zfhl9D3Hg==",
        "license": "MIT",
        "engines": {
          "node": ">=6"
        }
      },
      "node_modules/yoga-layout-prebuilt": {
        "name": "yoga-layout-prebuilt",
        "version": "1.9.3",
        "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.9.3.tgz",
        "integrity": "sha512-9SNQpwuEh2NucU83i2KMZnONVudZ86YNcFk9tq74YaqrQfgJWO3yB9uzH1tAg8iqh5c9F5j0wuyJ2z72wcum2w==",
        "license": "MIT",
        "engines": {
          "node": ">=8"
        }
      }
    },
    "dependencies": {
      "@babel/code-frame": {
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.0.0.tgz",
        "integrity": "sha512-OfC2uemaknXr87bdLUkWog7nYuliM9Ij5HUcajsVcMCpQrcLmtxRbVFTIqmcSkSeYRBFBRxs2FiUqFJDLdiebA==",
        "requires": {
          "@babel/highlight": "^7.0.0"
        }
      },
      "@babel/generator": {
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.4.0.tgz",
        "integrity": "sha512-/v5I+a1jhGSKLgZDcmAUZ4K/VePi43eRkUs3yePW1HB1iANOD5tqJXwGSG4BZhSksP8J9ejSlwGeTiiOFZOrXQ==",
        "requires": {
          "@babel/types": "^7.4.0",
          "jsesc": "^2.5.1",
          "lodash": "^4.17.11",
          "source-map": "^0.5.0",
          "trim-right": "^1.0.1"
        },
        "dependencies": {
          "jsesc": {
            "version": "2.5.2",
            "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
            "integrity": "sha512-OYu7XEzjkCQ3C5Ps3QIZsQfNpqoJyZZA99wd9aWd05NCtC5pWOkShK2mkL6HXQR6/Cy2lbNdPlZBpuQHXE63gA=="
          }
        }
      },
      "@babel/helper-function-name": {
        "version": "7.1.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.1.0.tgz",
        "integrity": "sha512-A95XEoCpb3TO+KZzJ4S/5uW5fNe26DjBGqf1o9ucyLyCmi1dXq/B3c8iaWTfBk3VvetUxl16e8tIrd5teOCfGw==",
        "requires": {
          "@babel/helper-get-function-arity": "^7.0.0",
          "@babel/template": "^7.1.0",
          "@babel/types": "^7.0.0"
        }
      },
      "@babel/helper-get-function-arity": {
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.0.0.tgz",
        "integrity": "sha512-r2DbJeg4svYvt3HOS74U4eWKsUAMRH01Z1ds1zx8KNTPtpTL5JAsdFv8BNyOpVqdFhHkkRDIg5B4AsxmkjAlmQ==",
        "requires": {
          "@babel/types": "^7.0.0"
        }
      },
      "@babel/helper-split-export-declaration": {
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.4.0.tgz",
        "integrity": "sha512-7Cuc6JZiYShaZnybDmfwhY4UYHzI6rlqhWjaIqbsJGsIqPimEYy5uh3akSRLMg65LSdSEnJ8a8/bWQN6u2oMGw==",
        "requires": {
          "@babel/types": "^7.4.0"
        }
      },
      "@babel/highlight": {
        "version": "7.0.0",
        "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.0.0.tgz",
        "integrity": "sha512-UFMC4ZeFC48Tpvj7C8UgLvtkaUuovQX+5xNWrsIoMG8o2z+XFKjKaN9iVmS84dPwVN00W4wPmqvYoZF3EGAsfw==",
        "requires": {
          "chalk": "^2.0.0",
          "esutils": "^2.0.2",
          "js-tokens": "^4.0.0"
        },
        "dependencies": {
          "ansi-styles": {
            "version": "3.2.1",
            "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
            "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
            "requires": {
              "color-convert": "^1.9.0"
            }
          },
          "chalk": {
            "version": "2.4.2",
            "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
            "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
            "requires": {
              "ansi-styles": "^3.2.1",
              "escape-string-regexp": "^1.0.5",
              "supports-color": "^5.3.0"
            }
          },
          "js-tokens": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
            "integrity": "sha512-RdJUflcE3cUzKiMqQgsCu06FPu9UdIJO0beYbPhHN4k6apgJtifcoCtT9bcxOpYBtpD2kCM6Sbzg4CausW/PKQ=="
          },
          "supports-color": {
            "version": "5.5.0",
            "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
            "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
            "requires": {
              "has-flag": "^3.0.0"
            }
          }
        }
      },
      "@babel/parser": {
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.4.3.tgz",
        "integrity": "sha512-gxpEUhTS1sGA63EGQGuA+WESPR/6tz6ng7tSHFCmaTJK/cGK8y37cBTspX+U2xCAue2IQVvF6Z0oigmjwD8YGQ=="
      },
      "@babel/runtime": {
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.4.3.tgz",
        "integrity": "sha512-9lsJwJLxDh/T3Q3SZszfWOTkk3pHbkmH+3KY+zwIDmsNlxsumuhS2TH3NIpktU4kNvfzy+k3eLT7aTJSPTo0OA==",
        "requires": {
          "regenerator-runtime": "^0.13.2"
        },
        "dependencies": {
          "regenerator-runtime": {
            "version": "0.13.2",
            "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.2.tgz",
            "integrity": "sha512-S/TQAZJO+D3m9xeN1WTI8dLKBBiRgXBlTJvbWjCThHWZj9EvHK70Ff50/tYj2J/fvBY6JtFVwRuazHN2E7M9BA=="
          }
        }
      },
      "@babel/template": {
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.4.0.tgz",
        "integrity": "sha512-SOWwxxClTTh5NdbbYZ0BmaBVzxzTh2tO/TeLTbF6MO6EzVhHTnff8CdBXx3mEtazFBoysmEM6GU/wF+SuSx4Fw==",
        "requires": {
          "@babel/code-frame": "^7.0.0",
          "@babel/parser": "^7.4.0",
          "@babel/types": "^7.4.0"
        }
      },
      "@babel/traverse": {
        "version": "7.4.3",
        "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.4.3.tgz",
        "integrity": "sha512-HmA01qrtaCwwJWpSKpA948cBvU5BrmviAief/b3AVw936DtcdsTexlbyzNuDnthwhOQ37xshn7hvQaEQk7ISYQ==",
        "requires": {
          "@babel/code-frame": "^7.0.0",
          "@babel/generator": "^7.4.0",
          "@babel/helper-function-name": "^7.1.0",
          "@babel/helper-split-export-declaration": "^7.4.0",
          "@babel/parser": "^7.4.3",
          "@babel/types": "^7.4.0",
          "debug": "^4.1.0",
          "globals": "^11.1.0",
          "lodash": "^4.17.11"
        },
        "dependencies": {
          "debug": {
            "version": "4.1.1",
            "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
            "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
            "requires": {
              "ms": "^2.1.1"
            }
          },
          "globals": {
            "version": "11.11.0",
            "resolved": "https://registry.npmjs.org/globals/-/globals-11.11.0.tgz",
            "integrity": "sha512-WHq43gS+6ufNOEqlrDBxVEbb8ntfXrfAUU2ZOpCxrBdGKW3gyv8mCxAfIBD0DroPKGrJ2eSsXsLtY9MPntsyTw=="
          },
          "ms": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
            "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg=="
          }
        }
      },
      "@babel/types": {
        "version": "7.4.0",
        "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.4.0.tgz",
        "integrity": "sha512-aPvkXyU2SPOnztlgo8n9cEiXW755mgyvueUPcpStqdzoSPm0fjO0vQBjLkt3JKJW7ufikfcnMTTPsN1xaTsBPA==",
        "requires": {
          "esutils": "^2.0.2",
          "lodash": "^4.17.11",
          "to-fast-properties": "^2.0.0"
        },
        "dependencies": {
          "to-fast-properties": {
            "version": "2.0.0",
            "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
            "integrity": "sha1-3F5pjL0HkmW8c+A3doGk5Og/YW4="
          }
        }
      },
      "@types/prop-types": {
        "version": "15.7.1",
        "resolved": "https://registry.npmjs.org/@types/prop-types/-/prop-types-15.7.1.tgz",
        "integrity": "sha512-CFzn9idOEpHrgdw8JsoTkaDDyRWk1jrzIV8djzcgpq0y9tG4B4lFT+Nxh52DVpDXV+n4+NPNv7M1Dj5uMp6XFg=="
      },
      "@types/react": {
        "version": "16.8.14",
        "resolved": "https://registry.npmjs.org/@types/react/-/react-16.8.14.tgz",
        "integrity": "sha512-26tFVJ1omGmzIdFTFmnC5zhz1GTaqCjxgUxV4KzWvsybF42P7/j4RBn6UeO3KbHPXqKWZszMXMoI65xIWm954A==",
        "requires": {
          "@types/prop-types": "*",
          "csstype": "^2.2.0"
        }
      },
      "ajv": {
        "version": "6.10.0",
        "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.0.tgz",
        "integrity": "sha512-nffhOpkymDECQyR0mnsUtoCE8RlX38G0rYP+wgLWFyZuUyuuojSSvi/+euOiQBIn63whYwYVIIH1TvE3tu4OEg==",
        "requires": {
          "fast-deep-equal": "^2.0.1",
          "fast-json-stable-stringify": "^2.0.0",
          "json-schema-traverse": "^0.4.1",
          "uri-js": "^4.2.2"
        }
      },
      "ansi-escapes": {
        "version": "3.2.0",
        "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-3.2.0.tgz",
        "integrity": "sha512-cBhpre4ma+U0T1oM5fXg7Dy1Jw7zzwv7lt/GoCpr+hDQJoYnKVPLL4dCvSEFMmQurOQvSrwT7SL/DAlhBI97RQ=="
      },
      "ansi-regex": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
        "integrity": "sha1-w7M6te42DYbg5ijwRorn7yfWVN8="
      },
      "ansi-styles": {
        "version": "2.2.1",
        "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.2.1.tgz",
        "integrity": "sha1-tDLdM1i2NM914eRmQ2gkBTPB3b4="
      },
      "ansicolors": {
        "version": "0.3.2",
        "resolved": "https://registry.npmjs.org/ansicolors/-/ansicolors-0.3.2.tgz",
        "integrity": "sha1-ZlWX3oap/+Oqm/vmyuXG6kJrSXk="
      },
      "anymatch": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-2.0.0.tgz",
        "integrity": "sha512-5teOsQWABXHHBFP9y3skS5P3d/WfWXpv3FUpy+LorMrNYaT9pI4oLMQX7jzQ2KklNpGpWHzdCXTDT2Y3XGlZBw==",
        "requires": {
          "micromatch": "^3.1.4",
          "normalize-path": "^2.1.1"
        },
        "dependencies": {
          "normalize-path": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-2.1.1.tgz",
            "integrity": "sha1-GrKLVW4Zg2Oowab35vogE3/mrtk=",
            "requires": {
              "remove-trailing-separator": "^1.0.1"
            }
          }
        }
      },
      "append-transform": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
        "integrity": "sha512-P009oYkeHyU742iSZJzZZywj4QRJdnTWffaKuJQLablCZ1uz6/cW4yaRgcDaoQ+uwOxxnt0gRUcwfsNP2ri0gw==",
        "requires": {
          "default-require-extensions": "^2.0.0"
        }
      },
      "archy": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
        "integrity": "sha1-+cjBN1fMHde8N5rHeyxipcKGjEA="
      },
      "arg": {
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.0.tgz",
        "integrity": "sha512-ZWc51jO3qegGkVh8Hwpv636EkbesNV5ZNQPCtRa+0qytRYPEs9IYT9qITY9buezqUH5uqyzlWLcufrzU2rffdg=="
      },
      "argparse": {
        "version": "1.0.10",
        "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
        "integrity": "sha512-o5Roy6tNG4SL/FOkCAN6RzjiakZS25RLYFrcMttJqbdd8BWrnA+fGz57iN5Pb06pvBGvl5gQ0B48dJlslXvoTg==",
        "requires": {
          "sprintf-js": "~1.0.2"
        }
      },
      "arr-diff": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/arr-diff/-/arr-diff-4.0.0.tgz",
        "integrity": "sha1-1kYQdP6/7HHn4VI1dhoyml3HxSA="
      },
      "arr-flatten": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/arr-flatten/-/arr-flatten-1.1.0.tgz",
        "integrity": "sha512-L3hKV5R/p5o81R7O02IGnwpDmkp6E982XhtbuwSe3O4qOtMMMtodicASA1Cny2U+aCXcNpml+m4dPsvsJ3jatg=="
      },
      "arr-union": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/arr-union/-/arr-union-3.1.0.tgz",
        "integrity": "sha1-45sJrqne+Gao8gbiiK9jkZuuOcQ="
      },
      "array-unique": {
        "version": "0.3.2",
        "resolved": "https://registry.npmjs.org/array-unique/-/array-unique-0.3.2.tgz",
        "integrity": "sha1-qJS3XUvE9s1nnvMkSp/Y9Gri1Cg="
      },
      "arrify": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
        "integrity": "sha1-iYUI2iIm84DfkEcoRWhJwVAaSw0="
      },
      "asn1": {
        "version": "0.2.4",
        "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
        "integrity": "sha512-jxwzQpLQjSmWXgwaCZE9Nz+glAG01yF1QnWgbhGwHI5A6FRIEY6IVqtHhIepHqI7/kyEyQEagBC5mBEFlIYvdg==",
        "requires": {
          "safer-buffer": "~2.1.0"
        }
      },
      "assert-plus": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
        "integrity": "sha1-8S4PPF13sLHN2RRpQuTpbB5N1SU="
      },
      "assign-symbols": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/assign-symbols/-/assign-symbols-1.0.0.tgz",
        "integrity": "sha1-WWZ/QfrdTyDMvCu5a41Pf3jsA2c="
      },
      "astral-regex": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/astral-regex/-/astral-regex-1.0.0.tgz",
        "integrity": "sha512-+Ryf6g3BKoRc7jfp7ad8tM4TtMiaWvbF/1/sQcZPkkS7ag3D5nMBCe2UfOTONtAkaG0tO0ij3C5Lwmf1EiyjHg=="
      },
      "async-each": {
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/async-each/-/async-each-1.0.3.tgz",
        "integrity": "sha512-z/WhQ5FPySLdvREByI2vZiTWwCnF0moMJ1hK9YQwDTHKh6I7/uSckMetoRGb5UBZPC1z0jlw+n/XCgjeH7y1AQ=="
      },
      "async-hook-domain": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.0.tgz",
        "integrity": "sha512-NH7V97d1yCbIanu2oDLyPT2GFNct0esPeJyRfkk8J5hTztHVSQp4UiNfL2O42sCA9XZPU8OgHvzOmt9ewBhVqA==",
        "requires": {
          "source-map-support": "^0.5.11"
        }
      },
      "asynckit": {
        "version": "0.4.0",
        "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
        "integrity": "sha1-x57Zf380y48robyXkLzDZkdLS3k="
      },
      "atob": {
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/atob/-/atob-2.1.2.tgz",
        "integrity": "sha512-Wm6ukoaOGJi/73p/cl2GvLjTI5JM1k/O14isD73YML8StrH/7/lRFgmg8nICZgD3bZZvjwCGxtMOD3wWNAu8cg=="
      },
      "auto-bind": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/auto-bind/-/auto-bind-2.1.0.tgz",
        "integrity": "sha512-qZuFvkes1eh9lB2mg8/HG18C+5GIO51r+RrCSst/lh+i5B1CtVlkhTE488M805Nr3dKl0sM/pIFKSKUIlg3zUg==",
        "requires": {
          "@types/react": "^16.8.12"
        }
      },
      "aws-sign2": {
        "version": "0.7.0",
        "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
        "integrity": "sha1-tG6JCTSpWR8tL2+G1+ap8bP+dqg="
      },
      "aws4": {
        "version": "1.8.0",
        "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.8.0.tgz",
        "integrity": "sha512-ReZxvNHIOv88FlT7rxcXIIC0fPt4KZqZbOlivyWtXLt8ESx84zd3kMC6iK5jVeS2qt+g7ftS7ye4fi06X5rtRQ=="
      },
      "babel-code-frame": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-code-frame/-/babel-code-frame-6.26.0.tgz",
        "integrity": "sha1-Y/1D99weO7fONZR9uP42mj9Yx0s=",
        "requires": {
          "chalk": "^1.1.3",
          "esutils": "^2.0.2",
          "js-tokens": "^3.0.2"
        }
      },
      "babel-core": {
        "version": "6.26.3",
        "resolved": "https://registry.npmjs.org/babel-core/-/babel-core-6.26.3.tgz",
        "integrity": "sha512-6jyFLuDmeidKmUEb3NM+/yawG0M2bDZ9Z1qbZP59cyHLz8kYGKYwpJP0UwUKKUiTRNvxfLesJnTedqczP7cTDA==",
        "requires": {
          "babel-code-frame": "^6.26.0",
          "babel-generator": "^6.26.0",
          "babel-helpers": "^6.24.1",
          "babel-messages": "^6.23.0",
          "babel-register": "^6.26.0",
          "babel-runtime": "^6.26.0",
          "babel-template": "^6.26.0",
          "babel-traverse": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "convert-source-map": "^1.5.1",
          "debug": "^2.6.9",
          "json5": "^0.5.1",
          "lodash": "^4.17.4",
          "minimatch": "^3.0.4",
          "path-is-absolute": "^1.0.1",
          "private": "^0.1.8",
          "slash": "^1.0.0",
          "source-map": "^0.5.7"
        }
      },
      "babel-generator": {
        "version": "6.26.1",
        "resolved": "https://registry.npmjs.org/babel-generator/-/babel-generator-6.26.1.tgz",
        "integrity": "sha512-HyfwY6ApZj7BYTcJURpM5tznulaBvyio7/0d4zFOeMPUmfxkCjHocCuoLa2SAGzBI8AREcH3eP3758F672DppA==",
        "requires": {
          "babel-messages": "^6.23.0",
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "detect-indent": "^4.0.0",
          "jsesc": "^1.3.0",
          "lodash": "^4.17.4",
          "source-map": "^0.5.7",
          "trim-right": "^1.0.1"
        }
      },
      "babel-helper-builder-react-jsx": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-helper-builder-react-jsx/-/babel-helper-builder-react-jsx-6.26.0.tgz",
        "integrity": "sha1-Of+DE7dci2Xc7/HzHTg+D/KkCKA=",
        "requires": {
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "esutils": "^2.0.2"
        }
      },
      "babel-helpers": {
        "version": "6.24.1",
        "resolved": "https://registry.npmjs.org/babel-helpers/-/babel-helpers-6.24.1.tgz",
        "integrity": "sha1-NHHenK7DiOXIUOWX5Yom3fN2ArI=",
        "requires": {
          "babel-runtime": "^6.22.0",
          "babel-template": "^6.24.1"
        }
      },
      "babel-messages": {
        "version": "6.23.0",
        "resolved": "https://registry.npmjs.org/babel-messages/-/babel-messages-6.23.0.tgz",
        "integrity": "sha1-8830cDhYA1sqKVHG7F7fbGLyYw4=",
        "requires": {
          "babel-runtime": "^6.22.0"
        }
      },
      "babel-plugin-syntax-jsx": {
        "version": "6.18.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-syntax-jsx/-/babel-plugin-syntax-jsx-6.18.0.tgz",
        "integrity": "sha1-CvMqmm4Tyno/1QaeYtew9Y0NiUY="
      },
      "babel-plugin-syntax-object-rest-spread": {
        "version": "6.13.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-syntax-object-rest-spread/-/babel-plugin-syntax-object-rest-spread-6.13.0.tgz",
        "integrity": "sha1-/WU28rzhODb/o6VFjEkDpZe7O/U="
      },
      "babel-plugin-transform-es2015-destructuring": {
        "version": "6.23.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-es2015-destructuring/-/babel-plugin-transform-es2015-destructuring-6.23.0.tgz",
        "integrity": "sha1-mXux8auWf2gtKwh2/jWNYOdlxW0=",
        "requires": {
          "babel-runtime": "^6.22.0"
        }
      },
      "babel-plugin-transform-object-rest-spread": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-object-rest-spread/-/babel-plugin-transform-object-rest-spread-6.26.0.tgz",
        "integrity": "sha1-DzZpLVD+9rfi1LOsFHgTepY7ewY=",
        "requires": {
          "babel-plugin-syntax-object-rest-spread": "^6.8.0",
          "babel-runtime": "^6.26.0"
        }
      },
      "babel-plugin-transform-react-jsx": {
        "version": "6.24.1",
        "resolved": "https://registry.npmjs.org/babel-plugin-transform-react-jsx/-/babel-plugin-transform-react-jsx-6.24.1.tgz",
        "integrity": "sha1-hAoCjn30YN/DotKfDA2R9jduZqM=",
        "requires": {
          "babel-helper-builder-react-jsx": "^6.24.1",
          "babel-plugin-syntax-jsx": "^6.8.0",
          "babel-runtime": "^6.22.0"
        }
      },
      "babel-register": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-register/-/babel-register-6.26.0.tgz",
        "integrity": "sha1-btAhFz4vy0htestFxgCahW9kcHE=",
        "requires": {
          "babel-core": "^6.26.0",
          "babel-runtime": "^6.26.0",
          "core-js": "^2.5.0",
          "home-or-tmp": "^2.0.0",
          "lodash": "^4.17.4",
          "mkdirp": "^0.5.1",
          "source-map-support": "^0.4.15"
        },
        "dependencies": {
          "source-map-support": {
            "version": "0.4.18",
            "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.4.18.tgz",
            "integrity": "sha512-try0/JqxPLF9nOjvSta7tVondkP5dwgyLDjVoyMDlmjugT2lRZ1OfsrYTkCd2hkDnJTKRbO/Rl3orm8vlsUzbA==",
            "requires": {
              "source-map": "^0.5.6"
            }
          }
        }
      },
      "babel-runtime": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-runtime/-/babel-runtime-6.26.0.tgz",
        "integrity": "sha1-llxwWGaOgrVde/4E/yM3vItWR/4=",
        "requires": {
          "core-js": "^2.4.0",
          "regenerator-runtime": "^0.11.0"
        }
      },
      "babel-template": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-template/-/babel-template-6.26.0.tgz",
        "integrity": "sha1-3gPi0WOWsGn0bdn/+FIfsaDjXgI=",
        "requires": {
          "babel-runtime": "^6.26.0",
          "babel-traverse": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "lodash": "^4.17.4"
        }
      },
      "babel-traverse": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-traverse/-/babel-traverse-6.26.0.tgz",
        "integrity": "sha1-RqnL1+3MYsjlwGTi0tjQ9ANXZu4=",
        "requires": {
          "babel-code-frame": "^6.26.0",
          "babel-messages": "^6.23.0",
          "babel-runtime": "^6.26.0",
          "babel-types": "^6.26.0",
          "babylon": "^6.18.0",
          "debug": "^2.6.8",
          "globals": "^9.18.0",
          "invariant": "^2.2.2",
          "lodash": "^4.17.4"
        }
      },
      "babel-types": {
        "version": "6.26.0",
        "resolved": "https://registry.npmjs.org/babel-types/-/babel-types-6.26.0.tgz",
        "integrity": "sha1-o7Bz+Uq0nrb6Vc1lInozQ4BjJJc=",
        "requires": {
          "babel-runtime": "^6.26.0",
          "esutils": "^2.0.2",
          "lodash": "^4.17.4",
          "to-fast-properties": "^1.0.3"
        }
      },
      "babylon": {
        "version": "6.18.0",
        "resolved": "https://registry.npmjs.org/babylon/-/babylon-6.18.0.tgz",
        "integrity": "sha512-q/UEjfGJ2Cm3oKV71DJz9d25TPnq5rhBVL2Q4fA5wcC3jcrdn7+SssEybFIxwAvvP+YCsCYNKughoF33GxgycQ=="
      },
      "balanced-match": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
        "integrity": "sha1-ibTRmasr7kneFk6gK4nORi1xt2c="
      },
      "base": {
        "version": "0.11.2",
        "resolved": "https://registry.npmjs.org/base/-/base-0.11.2.tgz",
        "integrity": "sha512-5T6P4xPgpp0YDFvSWwEZ4NoE3aM4QBQXDzmVbraCkFj8zHM+mba8SyqB5DbZWyR7mYHo6Y7BdQo3MoA4m0TeQg==",
        "requires": {
          "cache-base": "^1.0.1",
          "class-utils": "^0.3.5",
          "component-emitter": "^1.2.1",
          "define-property": "^1.0.0",
          "isobject": "^3.0.1",
          "mixin-deep": "^1.2.0",
          "pascalcase": "^0.1.1"
        },
        "dependencies": {
          "define-property": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
            "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
            "requires": {
              "is-descriptor": "^1.0.0"
            }
          }
        }
      },
      "bcrypt-pbkdf": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
        "integrity": "sha1-pDAdOJtqQ/m2f/PKEaP2Y342Dp4=",
        "requires": {
          "tweetnacl": "^0.14.3"
        }
      },
      "binary-extensions": {
        "version": "1.13.1",
        "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-1.13.1.tgz",
        "integrity": "sha512-Un7MIEDdUC5gNpcGDV97op1Ywk748MpHcFTHoYs6qnj1Z3j7I53VG3nwZhKzoBZmbdRNnb6WRdFlwl7tSDuZGw=="
      },
      "bind-obj-methods": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
        "integrity": "sha512-3/qRXczDi2Cdbz6jE+W3IflJOutRVica8frpBn14de1mBOkzDo+6tY33kNhvkw54Kn3PzRRD2VnGbGPcTAk4sw=="
      },
      "brace-expansion": {
        "version": "1.1.11",
        "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
        "integrity": "sha512-iCuPHDFgrHX7H2vEI/5xpz07zSHB00TpugqhmYtVmMO6518mCuRMoOYFldEBl0g187ufozdaHgWKcYFb61qGiA==",
        "requires": {
          "balanced-match": "^1.0.0",
          "concat-map": "0.0.1"
        }
      },
      "braces": {
        "version": "2.3.2",
        "resolved": "https://registry.npmjs.org/braces/-/braces-2.3.2.tgz",
        "integrity": "sha512-aNdbnj9P8PjdXU4ybaWLK2IF3jc/EoDYbC7AazW6to3TRsfXxscC9UXOB5iDiEQrkyIbWp2SLQda4+QAa7nc3w==",
        "requires": {
          "arr-flatten": "^1.1.0",
          "array-unique": "^0.3.2",
          "extend-shallow": "^2.0.1",
          "fill-range": "^4.0.0",
          "isobject": "^3.0.1",
          "repeat-element": "^1.1.2",
          "snapdragon": "^0.8.1",
          "snapdragon-node": "^2.0.1",
          "split-string": "^3.0.2",
          "to-regex": "^3.0.1"
        }
      },
      "browser-process-hrtime": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
        "integrity": "sha512-9o5UecI3GhkpM6DrXr69PblIuWxPKk9Y0jHBRhdocZ2y7YECBFCsHm79Pr3OyR2AvjhDkabFJaDJMYRazHgsow=="
      },
      "buffer-from": {
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
        "integrity": "sha512-MQcXEUbCKtEo7bhqEs6560Hyd4XaovZlO/k9V3hjVUF/zwW7KBVdSK4gIt/bzwS9MbR5qob+F5jusZsb0YQK2A=="
      },
      "cache-base": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/cache-base/-/cache-base-1.0.1.tgz",
        "integrity": "sha512-AKcdTnFSWATd5/GCPRxr2ChwIJ85CeyrEyjRHlKxQ56d4XJMGym0uAiKn0xbLOGOl3+yRpOTi484dVCEc5AUzQ==",
        "requires": {
          "collection-visit": "^1.0.0",
          "component-emitter": "^1.2.1",
          "get-value": "^2.0.6",
          "has-value": "^1.0.0",
          "isobject": "^3.0.1",
          "set-value": "^2.0.0",
          "to-object-path": "^0.3.0",
          "union-value": "^1.0.0",
          "unset-value": "^1.0.0"
        }
      },
      "caching-transform": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
        "integrity": "sha512-Mtgcv3lh3U0zRii/6qVgQODdPA4G3zhG+jtbCWj39RXuUFTMzH0vcdMtaJS1jPowd+It2Pqr6y3NJMQqOqCE2w==",
        "requires": {
          "hasha": "^3.0.0",
          "make-dir": "^2.0.0",
          "package-hash": "^3.0.0",
          "write-file-atomic": "^2.4.2"
        }
      },
      "caller-callsite": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/caller-callsite/-/caller-callsite-2.0.0.tgz",
        "integrity": "sha1-hH4PzgoiN1CpoCfFSzNzGtMVQTQ=",
        "requires": {
          "callsites": "^2.0.0"
        }
      },
      "caller-path": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-2.0.0.tgz",
        "integrity": "sha1-Ro+DBE42mrIBD6xfBs7uFbsssfQ=",
        "requires": {
          "caller-callsite": "^2.0.0"
        }
      },
      "callsites": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/callsites/-/callsites-2.0.0.tgz",
        "integrity": "sha1-BuuE8A7qQT2oav/vrL/7Ngk7PFA="
      },
      "camelcase": {
        "version": "5.3.1",
        "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
        "integrity": "sha512-L28STB170nwWS63UjtlEOE3dldQApaJXZkOI1uMFfzf3rRuPegHaHesyee+YxQ+W6SvRDQV6UrdOdRiR153wJg=="
      },
      "capture-stack-trace": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/capture-stack-trace/-/capture-stack-trace-1.0.1.tgz",
        "integrity": "sha512-mYQLZnx5Qt1JgB1WEiMCf2647plpGeQ2NMR/5L0HNZzGQo4fuSPnK+wjfPnKZV0aiJDgzmWqqkV/g7JD+DW0qw=="
      },
      "cardinal": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/cardinal/-/cardinal-2.1.1.tgz",
        "integrity": "sha1-fMEFXYItISlU0HsIXeolHMe8VQU=",
        "requires": {
          "ansicolors": "~0.3.2",
          "redeyed": "~2.1.0"
        }
      },
      "caseless": {
        "version": "0.12.0",
        "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
        "integrity": "sha1-G2gcIf+EAzyCZUMJBolCDRhxUdw="
      },
      "chalk": {
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.3.tgz",
        "integrity": "sha1-qBFcVeSnAv5NFQq9OHKCKn4J/Jg=",
        "requires": {
          "ansi-styles": "^2.2.1",
          "escape-string-regexp": "^1.0.2",
          "has-ansi": "^2.0.0",
          "strip-ansi": "^3.0.0",
          "supports-color": "^2.0.0"
        }
      },
      "chokidar": {
        "version": "2.1.5",
        "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-2.1.5.tgz",
        "integrity": "sha512-i0TprVWp+Kj4WRPtInjexJ8Q+BqTE909VpH8xVhXrJkoc5QC8VO9TryGOqTr+2hljzc1sC62t22h5tZePodM/A==",
        "requires": {
          "anymatch": "^2.0.0",
          "async-each": "^1.0.1",
          "braces": "^2.3.2",
          "fsevents": "^1.2.7",
          "glob-parent": "^3.1.0",
          "inherits": "^2.0.3",
          "is-binary-path": "^1.0.0",
          "is-glob": "^4.0.0",
          "normalize-path": "^3.0.0",
          "path-is-absolute": "^1.0.0",
          "readdirp": "^2.2.1",
          "upath": "^1.1.1"
        }
      },
      "ci-info": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/ci-info/-/ci-info-2.0.0.tgz",
        "integrity": "sha512-5tK7EtrZ0N+OLFMthtqOj4fI2Jeb88C4CAZPu25LDVUgXJ0A3Js4PMGqrn0JU1W0Mh1/Z8wZzYPxqUrXeBboCQ=="
      },
      "class-utils": {
        "version": "0.3.6",
        "resolved": "https://registry.npmjs.org/class-utils/-/class-utils-0.3.6.tgz",
        "integrity": "sha512-qOhPa/Fj7s6TY8H8esGu5QNpMMQxz79h+urzrNYN6mn+9BnxlDGf5QZ+XeCDsxSjPqsSR56XOZOJmpeurnLMeg==",
        "requires": {
          "arr-union": "^3.1.0",
          "define-property": "^0.2.5",
          "isobject": "^3.0.0",
          "static-extend": "^0.1.1"
        },
        "dependencies": {
          "define-property": {
            "version": "0.2.5",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
            "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
            "requires": {
              "is-descriptor": "^0.1.0"
            }
          },
          "is-accessor-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
            "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-data-descriptor": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
            "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
            "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
            "requires": {
              "is-accessor-descriptor": "^0.1.6",
              "is-data-descriptor": "^0.1.4",
              "kind-of": "^5.0.0"
            },
            "dependencies": {
              "kind-of": {
                "version": "5.1.0",
                "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
                "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw=="
              }
            }
          }
        }
      },
      "cli-cursor": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-2.1.0.tgz",
        "integrity": "sha1-s12sN2R5+sw+lHR9QdDQ9SOP/LU=",
        "requires": {
          "restore-cursor": "^2.0.0"
        }
      },
      "cli-truncate": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/cli-truncate/-/cli-truncate-1.1.0.tgz",
        "integrity": "sha512-bAtZo0u82gCfaAGfSNxUdTI9mNyza7D8w4CVCcaOsy7sgwDzvx6ekr6cuWJqY3UGzgnQ1+4wgENup5eIhgxEYA==",
        "requires": {
          "slice-ansi": "^1.0.0",
          "string-width": "^2.0.0"
        }
      },
      "cliui": {
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
        "integrity": "sha512-4FG+RSG9DL7uEwRUZXZn3SS34DiDPfzP0VOiEwtUWlE+AR2EIg+hSyvrIgUUfhdgR/UkAeW2QHgeP+hWrXs7jQ==",
        "requires": {
          "string-width": "^2.1.1",
          "strip-ansi": "^4.0.0",
          "wrap-ansi": "^2.0.0"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "3.0.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
            "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg="
          },
          "strip-ansi": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
            "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
            "requires": {
              "ansi-regex": "^3.0.0"
            }
          }
        }
      },
      "code-point-at": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
        "integrity": "sha1-DQcLTQQ6W+ozovGkDi7bPZpMz3c="
      },
      "collection-visit": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/collection-visit/-/collection-visit-1.0.0.tgz",
        "integrity": "sha1-S8A3PBZLwykbTTaMgpzxqApZ3KA=",
        "requires": {
          "map-visit": "^1.0.0",
          "object-visit": "^1.0.0"
        }
      },
      "color-convert": {
        "version": "1.9.3",
        "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
        "integrity": "sha512-QfAUtd+vFdAtFQcC8CCyYt1fYWxSqAiK2cSD6zDB8N3cpsEBAvRxp9zOGg6G/SHHJYAT88/az/IuDGALsNVbGg==",
        "requires": {
          "color-name": "1.1.3"
        }
      },
      "color-name": {
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
        "integrity": "sha1-p9BVi9icQveV3UIyj3QIMcpTvCU="
      },
      "color-support": {
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
        "integrity": "sha512-qiBjkpbMLO/HL68y+lh4q0/O1MZFj2RX6X/KmMa3+gJD3z+WwI1ZzDHysvqHGS3mP6mznPckpXmw1nI9cJjyRg=="
      },
      "combined-stream": {
        "version": "1.0.7",
        "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.7.tgz",
        "integrity": "sha512-brWl9y6vOB1xYPZcpZde3N9zDByXTosAeMDo4p1wzo6UMOX4vumB+TP1RZ76sfE6Md68Q0NJSrE/gbezd4Ul+w==",
        "requires": {
          "delayed-stream": "~1.0.0"
        }
      },
      "commander": {
        "version": "2.20.0",
        "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.0.tgz",
        "integrity": "sha512-7j2y+40w61zy6YC2iRNpUe/NwhNyoXrYpHMrSunaMG64nRnaf96zO/KMQR4OyN/UnE5KLyEBnKHd4aG3rskjpQ==",
        "optional": true
      },
      "commondir": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
        "integrity": "sha1-3dgA2gxmEnOTzKWVDqloo6rxJTs="
      },
      "component-emitter": {
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/component-emitter/-/component-emitter-1.3.0.tgz",
        "integrity": "sha512-Rd3se6QB+sO1TwqZjscQrurpEPIfO0/yYnSin6Q/rD3mOutHvUrCAhJub3r90uNb+SESBuE0QYoB90YdfatsRg=="
      },
      "concat-map": {
        "version": "0.0.1",
        "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
        "integrity": "sha1-2Klr13/Wjfd5OnMDajug1UBdR3s="
      },
      "convert-source-map": {
        "version": "1.6.0",
        "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.6.0.tgz",
        "integrity": "sha512-eFu7XigvxdZ1ETfbgPBohgyQ/Z++C0eEhTor0qRwBw9unw+L0/6V8wkSuGgzdThkiS5lSpdptOQPD8Ak40a+7A==",
        "requires": {
          "safe-buffer": "~5.1.1"
        }
      },
      "copy-descriptor": {
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/copy-descriptor/-/copy-descriptor-0.1.1.tgz",
        "integrity": "sha1-Z29us8OZl8LuGsOpJP1hJHSPV40="
      },
      "core-js": {
        "version": "2.6.5",
        "resolved": "https://registry.npmjs.org/core-js/-/core-js-2.6.5.tgz",
        "integrity": "sha512-klh/kDpwX8hryYL14M9w/xei6vrv6sE8gTHDG7/T/+SEovB/G4ejwcfE/CBzO6Edsu+OETZMZ3wcX/EjUkrl5A=="
      },
      "core-util-is": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
        "integrity": "sha1-tf1UIgqivFq1eqtxQMlAdUUDwac="
      },
      "coveralls": {
        "version": "3.0.3",
        "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.3.tgz",
        "integrity": "sha512-viNfeGlda2zJr8Gj1zqXpDMRjw9uM54p7wzZdvLRyOgnAfCe974Dq4veZkjJdxQXbmdppu6flEajFYseHYaUhg==",
        "requires": {
          "growl": "~> 1.10.0",
          "js-yaml": "^3.11.0",
          "lcov-parse": "^0.0.10",
          "log-driver": "^1.2.7",
          "minimist": "^1.2.0",
          "request": "^2.86.0"
        },
        "dependencies": {
          "minimist": {
            "version": "1.2.0",
            "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
            "integrity": "sha1-o1AIsg9BOD7sH7kU9M1d95omQoQ="
          }
        }
      },
      "cp-file": {
        "version": "6.2.0",
        "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
        "integrity": "sha512-fmvV4caBnofhPe8kOcitBwSn2f39QLjnAnGq3gO9dfd75mUytzKNZB1hde6QHunW2Rt+OwuBOMc3i1tNElbszA==",
        "requires": {
          "graceful-fs": "^4.1.2",
          "make-dir": "^2.0.0",
          "nested-error-stacks": "^2.0.0",
          "pify": "^4.0.1",
          "safe-buffer": "^5.0.1"
        }
      },
      "cross-spawn": {
        "version": "4.0.2",
        "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
        "integrity": "sha1-e5JHYhwjrf3ThWAEqCPL45dCTUE=",
        "requires": {
          "lru-cache": "^4.0.1",
          "which": "^1.2.9"
        }
      },
      "csstype": {
        "version": "2.6.4",
        "resolved": "https://registry.npmjs.org/csstype/-/csstype-2.6.4.tgz",
        "integrity": "sha512-lAJUJP3M6HxFXbqtGRc0iZrdyeN+WzOWeY0q/VnFzI+kqVrYIzC7bWlKqCW7oCIdzoPkvfp82EVvrTlQ8zsWQg=="
      },
      "dashdash": {
        "version": "1.14.1",
        "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
        "integrity": "sha1-hTz6D3y+L+1d4gMmuN1YEDX24vA=",
        "requires": {
          "assert-plus": "^1.0.0"
        }
      },
      "debug": {
        "version": "2.6.9",
        "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
        "integrity": "sha512-bC7ElrdJaJnPbAP+1EotYvqZsb3ecl5wi6Bfi6BJTUcNowp6cvspg0jXznRTKDjm/E7AdgFBVeAPVMNcKGsHMA==",
        "requires": {
          "ms": "2.0.0"
        }
      },
      "decamelize": {
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
        "integrity": "sha1-9lNNFRSCabIDUue+4m9QH5oZEpA="
      },
      "decode-uri-component": {
        "version": "0.2.0",
        "resolved": "https://registry.npmjs.org/decode-uri-component/-/decode-uri-component-0.2.0.tgz",
        "integrity": "sha1-6zkTMzRYd1y4TNGh+uBiEGu4dUU="
      },
      "default-require-extensions": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
        "integrity": "sha1-9fj7sYp9bVCyH2QfZJ67Uiz+JPc=",
        "requires": {
          "strip-bom": "^3.0.0"
        }
      },
      "define-property": {
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/define-property/-/define-property-2.0.2.tgz",
        "integrity": "sha512-jwK2UV4cnPpbcG7+VRARKTZPUWowwXA8bzH5NP6ud0oeAxyYPuGZUAC7hMugpCdz4BeSZl2Dl9k66CHJ/46ZYQ==",
        "requires": {
          "is-descriptor": "^1.0.2",
          "isobject": "^3.0.1"
        }
      },
      "delayed-stream": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
        "integrity": "sha1-3zrhmayt+31ECqrgsp4icrJOxhk="
      },
      "detect-indent": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/detect-indent/-/detect-indent-4.0.0.tgz",
        "integrity": "sha1-920GQ1LN9Docts5hnE7jqUdd4gg=",
        "requires": {
          "repeating": "^2.0.0"
        }
      },
      "diff": {
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
        "integrity": "sha1-fyjS657nsVqX79ic5j3P2qPMur8="
      },
      "domain-browser": {
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/domain-browser/-/domain-browser-1.2.0.tgz",
        "integrity": "sha512-jnjyiM6eRyZl2H+W8Q/zLMA481hzi0eszAaBUzIVnmYVDBbnLxVNnfu1HgEBvCbL+71FrxMl3E6lpKH7Ge3OXA=="
      },
      "ecc-jsbn": {
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
        "integrity": "sha1-OoOpBOVDUyh4dMVkt1SThoSamMk=",
        "requires": {
          "jsbn": "~0.1.0",
          "safer-buffer": "^2.1.0"
        }
      },
      "emoji-regex": {
        "version": "7.0.3",
        "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
        "integrity": "sha512-CwBLREIQ7LvYFB0WyRvwhq5N5qPhc6PMjD6bYggFlI5YyDgl+0vxq5VHbMOFqLg7hfWzmu8T5Z1QofhmTIhItA=="
      },
      "end-of-stream": {
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/end-of-stream/-/end-of-stream-1.4.1.tgz",
        "integrity": "sha512-1MkrZNvWTKCaigbn+W15elq2BB/L22nqrSY5DKlo3X6+vclJm8Bb5djXJBmEX6fS3+zCh/F4VBK5Z2KxJt4s2Q==",
        "requires": {
          "once": "^1.4.0"
        }
      },
      "error-ex": {
        "version": "1.3.2",
        "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
        "integrity": "sha512-7dFHNmqeFSEt2ZBsCriorKnn3Z2pj+fd9kmI6QoWw4//DL+icEBfc0U7qJCisqrTsKTjw4fNFy2pW9OqStD84g==",
        "requires": {
          "is-arrayish": "^0.2.1"
        }
      },
      "es6-error": {
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
        "integrity": "sha512-Um/+FxMr9CISWh0bi5Zv0iOD+4cFh5qLeks1qhAopKVAJw3drgKbKySikp7wGhDL0HPeaja0P5ULZrxLkniUVg=="
      },
      "escape-string-regexp": {
        "version": "1.0.5",
        "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
        "integrity": "sha1-G2HAViGQqN/2rjuyzwIAyhMLhtQ="
      },
      "esm": {
        "version": "3.2.22",
        "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.22.tgz",
        "integrity": "sha512-z8YG7U44L82j1XrdEJcqZOLUnjxco8pO453gKOlaMD1/md1n/5QrscAmYG+oKUspsmDLuBFZrpbxI6aQ67yRxA=="
      },
      "esprima": {
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
        "integrity": "sha512-eGuFFw7Upda+g4p+QHvnW0RyTX/SVeJBDM/gCtMARO0cLuT2HcEKnTPvhjV6aGeqrCB/sbNop0Kszm0jsaWU4A=="
      },
      "esutils": {
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
        "integrity": "sha1-Cr9PHKpbyx96nYrMbepPqqBLrJs="
      },
      "events-to-array": {
        "version": "1.1.2",
        "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
        "integrity": "sha1-LUH1Y+H+QA7Uli/hpNXGp1Od9/Y="
      },
      "execa": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/execa/-/execa-1.0.0.tgz",
        "integrity": "sha512-adbxcyWV46qiHyvSp50TKt05tB4tK3HcmF7/nxfAdhnox83seTDbwnaqKO4sXRy7roHAIFqJP/Rw/AuEbX61LA==",
        "requires": {
          "cross-spawn": "^6.0.0",
          "get-stream": "^4.0.0",
          "is-stream": "^1.1.0",
          "npm-run-path": "^2.0.0",
          "p-finally": "^1.0.0",
          "signal-exit": "^3.0.0",
          "strip-eof": "^1.0.0"
        },
        "dependencies": {
          "cross-spawn": {
            "version": "6.0.5",
            "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
            "integrity": "sha512-eTVLrBSt7fjbDygz805pMnstIs2VTBNkRm0qxZd+M7A5XDdxVRWO5MxGBXZhjY4cqLYLdtrGqRf8mBPmzwSpWQ==",
            "requires": {
              "nice-try": "^1.0.4",
              "path-key": "^2.0.1",
              "semver": "^5.5.0",
              "shebang-command": "^1.2.0",
              "which": "^1.2.9"
            }
          },
          "semver": {
            "version": "5.7.0",
            "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
            "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA=="
          }
        }
      },
      "expand-brackets": {
        "version": "2.1.4",
        "resolved": "https://registry.npmjs.org/expand-brackets/-/expand-brackets-2.1.4.tgz",
        "integrity": "sha1-t3c14xXOMPa27/D4OwQVGiJEliI=",
        "requires": {
          "debug": "^2.3.3",
          "define-property": "^0.2.5",
          "extend-shallow": "^2.0.1",
          "posix-character-classes": "^0.1.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "dependencies": {
          "define-property": {
            "version": "0.2.5",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
            "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
            "requires": {
              "is-descriptor": "^0.1.0"
            }
          },
          "is-accessor-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
            "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-data-descriptor": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
            "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
            "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
            "requires": {
              "is-accessor-descriptor": "^0.1.6",
              "is-data-descriptor": "^0.1.4",
              "kind-of": "^5.0.0"
            },
            "dependencies": {
              "kind-of": {
                "version": "5.1.0",
                "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
                "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw=="
              }
            }
          }
        }
      },
      "extend": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
        "integrity": "sha512-fjquC59cD7CyW6urNXK0FBufkZcoiGG80wTuPujX590cB5Ttln20E2UB4S/WARVqhXffZl2LNgS+gQdPIIim/g=="
      },
      "extend-shallow": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-2.0.1.tgz",
        "integrity": "sha1-Ua99YUrZqfYQ6huvu5idaxxWiQ8=",
        "requires": {
          "is-extendable": "^0.1.0"
        }
      },
      "extglob": {
        "version": "2.0.4",
        "resolved": "https://registry.npmjs.org/extglob/-/extglob-2.0.4.tgz",
        "integrity": "sha512-Nmb6QXkELsuBr24CJSkilo6UHHgbekK5UiZgfE6UHD3Eb27YC6oD+bhcT+tJ6cl8dmsgdQxnWlcry8ksBIBLpw==",
        "requires": {
          "array-unique": "^0.3.2",
          "define-property": "^1.0.0",
          "expand-brackets": "^2.1.4",
          "extend-shallow": "^2.0.1",
          "fragment-cache": "^0.2.1",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "dependencies": {
          "define-property": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
            "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
            "requires": {
              "is-descriptor": "^1.0.0"
            }
          }
        }
      },
      "extsprintf": {
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
        "integrity": "sha1-lpGEQOMEGnpBT4xS48V06zw+HgU="
      },
      "fast-deep-equal": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
        "integrity": "sha1-ewUhjd+WZ79/Nwv3/bLLFf3Qqkk="
      },
      "fast-json-stable-stringify": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.0.0.tgz",
        "integrity": "sha1-1RQsDK7msRifh9OnYREGT4bIu/I="
      },
      "fill-range": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-4.0.0.tgz",
        "integrity": "sha1-1USBHUKPmOsGpj3EAtJAPDKMOPc=",
        "requires": {
          "extend-shallow": "^2.0.1",
          "is-number": "^3.0.0",
          "repeat-string": "^1.6.1",
          "to-regex-range": "^2.1.0"
        }
      },
      "find-cache-dir": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
        "integrity": "sha512-Tq6PixE0w/VMFfCgbONnkiQIVol/JJL7nRMi20fqzA4NRs9AfeqMGeRdPi3wIhYkxjeBaWh2rxwapn5Tu3IqOQ==",
        "requires": {
          "commondir": "^1.0.1",
          "make-dir": "^2.0.0",
          "pkg-dir": "^3.0.0"
        }
      },
      "find-up": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
        "integrity": "sha512-1yD6RmLI1XBfxugvORwlck6f75tYL+iR0jqwsOrOxMZyGYqUuDhJ0l4AXdO1iX/FTs9cBAMEk1gWSEx1kSbylg==",
        "requires": {
          "locate-path": "^3.0.0"
        }
      },
      "findit": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
        "integrity": "sha1-ZQnwEmr0wXhVHPqZOU4DLhOk1W4="
      },
      "for-in": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/for-in/-/for-in-1.0.2.tgz",
        "integrity": "sha1-gQaNKVqBQuwKxybG4iAMMPttXoA="
      },
      "foreground-child": {
        "version": "1.5.6",
        "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
        "integrity": "sha1-T9ca0t/elnibmApcCilZN8svXOk=",
        "requires": {
          "cross-spawn": "^4",
          "signal-exit": "^3.0.0"
        }
      },
      "forever-agent": {
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
        "integrity": "sha1-+8cfDEGt6zf5bFd60e1C2P2sypE="
      },
      "form-data": {
        "version": "2.3.3",
        "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
        "integrity": "sha512-1lLKB2Mu3aGP1Q/2eCOx0fNbRMe7XdwktwOruhfqqd0rIJWwN4Dh+E3hrPSlDCXnSR7UtZ1N38rVXm+6+MEhJQ==",
        "requires": {
          "asynckit": "^0.4.0",
          "combined-stream": "^1.0.6",
          "mime-types": "^2.1.12"
        }
      },
      "fragment-cache": {
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/fragment-cache/-/fragment-cache-0.2.1.tgz",
        "integrity": "sha1-QpD60n8T6Jvn8zeZxrxaCr//DRk=",
        "requires": {
          "map-cache": "^0.2.2"
        }
      },
      "fs-exists-cached": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
        "integrity": "sha1-zyVVTKBQ3EmuZla0HeQiWJidy84="
      },
      "fs.realpath": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
        "integrity": "sha1-FQStJSMVjKpA20onh8sBQRmU6k8="
      },
      "fsevents": {
        "version": "1.2.8",
        "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-1.2.8.tgz",
        "integrity": "sha512-tPvHgPGB7m40CZ68xqFGkKuzN+RnpGmSV+hgeKxhRpbxdqKXUFJGC3yonBOLzQBcJyGpdZFDfCsdOC2KFsXzeA==",
        "optional": true,
        "requires": {
          "nan": "^2.12.1",
          "node-pre-gyp": "^0.12.0"
        },
        "dependencies": {
          "abbrev": {
            "version": "1.1.1",
            "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
            "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==",
            "bundled": true
          },
          "ansi-regex": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
            "integrity": "sha1-w7M6te42DYbg5ijwRorn7yfWVN8=",
            "bundled": true
          },
          "aproba": {
            "version": "1.2.0",
            "resolved": "https://registry.npmjs.org/aproba/-/aproba-1.2.0.tgz",
            "integrity": "sha512-Y9J6ZjXtoYh8RnXVCMOU/ttDmk1aBjunq9vO0ta5x85WDQiQfUF9sIPBITdbiiIVcBo03Hi3jMxigBtsddlXRw==",
            "bundled": true
          },
          "are-we-there-yet": {
            "version": "1.1.5",
            "resolved": "https://registry.npmjs.org/are-we-there-yet/-/are-we-there-yet-1.1.5.tgz",
            "integrity": "sha512-5hYdAkZlcG8tOLujVDTgCT+uPX0VnpAH28gWsLfzpXYm7wP6mp5Q/gYyR7YQ0cKVJcXJnl3j2kpBan13PtQf6w==",
            "bundled": true,
            "requires": {
              "delegates": "^1.0.0",
              "readable-stream": "^2.0.6"
            }
          },
          "balanced-match": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
            "integrity": "sha1-ibTRmasr7kneFk6gK4nORi1xt2c=",
            "bundled": true
          },
          "brace-expansion": {
            "version": "1.1.11",
            "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
            "integrity": "sha512-iCuPHDFgrHX7H2vEI/5xpz07zSHB00TpugqhmYtVmMO6518mCuRMoOYFldEBl0g187ufozdaHgWKcYFb61qGiA==",
            "bundled": true,
            "requires": {
              "balanced-match": "^1.0.0",
              "concat-map": "0.0.1"
            }
          },
          "chownr": {
            "version": "1.1.1",
            "resolved": "https://registry.npmjs.org/chownr/-/chownr-1.1.1.tgz",
            "integrity": "sha512-j38EvO5+LHX84jlo6h4UzmOwi0UgW61WRyPtJz4qaadK5eY3BTS5TY/S1Stc3Uk2lIM6TPevAlULiEJwie860g==",
            "bundled": true
          },
          "code-point-at": {
            "version": "1.1.0",
            "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
            "integrity": "sha1-DQcLTQQ6W+ozovGkDi7bPZpMz3c=",
            "bundled": true
          },
          "concat-map": {
            "version": "0.0.1",
            "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
            "integrity": "sha1-2Klr13/Wjfd5OnMDajug1UBdR3s=",
            "bundled": true
          },
          "console-control-strings": {
            "version": "1.1.0",
            "resolved": "https://registry.npmjs.org/console-control-strings/-/console-control-strings-1.1.0.tgz",
            "integrity": "sha1-PXz0Rk22RG6mRL9LOVB/mFEAjo4=",
            "bundled": true
          },
          "core-util-is": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
            "integrity": "sha1-tf1UIgqivFq1eqtxQMlAdUUDwac=",
            "bundled": true
          },
          "debug": {
            "version": "4.1.1",
            "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
            "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
            "bundled": true,
            "requires": {
              "ms": "^2.1.1"
            }
          },
          "deep-extend": {
            "version": "0.6.0",
            "resolved": "https://registry.npmjs.org/deep-extend/-/deep-extend-0.6.0.tgz",
            "integrity": "sha512-LOHxIOaPYdHlJRtCQfDIVZtfw/ufM8+rVj649RIHzcm/vGwQRXFt6OPqIFWsm2XEMrNIEtWR64sY1LEKD2vAOA==",
            "bundled": true
          },
          "delegates": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/delegates/-/delegates-1.0.0.tgz",
            "integrity": "sha1-hMbhWbgZBP3KWaDvRM2HDTElD5o=",
            "bundled": true
          },
          "detect-libc": {
            "version": "1.0.3",
            "resolved": "https://registry.npmjs.org/detect-libc/-/detect-libc-1.0.3.tgz",
            "integrity": "sha1-+hN8S9aY7fVc1c0CrFWfkaTEups=",
            "bundled": true
          },
          "fs-minipass": {
            "version": "1.2.5",
            "resolved": "https://registry.npmjs.org/fs-minipass/-/fs-minipass-1.2.5.tgz",
            "integrity": "sha512-JhBl0skXjUPCFH7x6x61gQxrKyXsxB5gcgePLZCwfyCGGsTISMoIeObbrvVeP6Xmyaudw4TT43qV2Gz+iyd2oQ==",
            "bundled": true,
            "requires": {
              "minipass": "^2.2.1"
            }
          },
          "fs.realpath": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
            "integrity": "sha1-FQStJSMVjKpA20onh8sBQRmU6k8=",
            "bundled": true
          },
          "gauge": {
            "version": "2.7.4",
            "resolved": "https://registry.npmjs.org/gauge/-/gauge-2.7.4.tgz",
            "integrity": "sha1-LANAXHU4w51+s3sxcCLjJfsBi/c=",
            "bundled": true,
            "requires": {
              "aproba": "^1.0.3",
              "console-control-strings": "^1.0.0",
              "has-unicode": "^2.0.0",
              "object-assign": "^4.1.0",
              "signal-exit": "^3.0.0",
              "string-width": "^1.0.1",
              "strip-ansi": "^3.0.1",
              "wide-align": "^1.1.0"
            }
          },
          "glob": {
            "version": "7.1.3",
            "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.3.tgz",
            "integrity": "sha512-vcfuiIxogLV4DlGBHIUOwI0IbrJ8HWPc4MU7HzviGeNho/UJDfi6B5p3sHeWIQ0KGIU0Jpxi5ZHxemQfLkkAwQ==",
            "bundled": true,
            "requires": {
              "fs.realpath": "^1.0.0",
              "inflight": "^1.0.4",
              "inherits": "2",
              "minimatch": "^3.0.4",
              "once": "^1.3.0",
              "path-is-absolute": "^1.0.0"
            }
          },
          "has-unicode": {
            "version": "2.0.1",
            "resolved": "https://registry.npmjs.org/has-unicode/-/has-unicode-2.0.1.tgz",
            "integrity": "sha1-4Ob+aijPUROIVeCG0Wkedx3iqLk=",
            "bundled": true
          },
          "iconv-lite": {
            "version": "0.4.24",
            "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
            "integrity": "sha512-v3MXnZAcvnywkTUEZomIActle7RXXeedOR31wwl7VlyoXO4Qi9arvSenNQWne1TcRwhCL1HwLI21bEqdpj8/rA==",
            "bundled": true,
            "requires": {
              "safer-buffer": ">= 2.1.2 < 3"
            }
          },
          "ignore-walk": {
            "version": "3.0.1",
            "resolved": "https://registry.npmjs.org/ignore-walk/-/ignore-walk-3.0.1.tgz",
            "integrity": "sha512-DTVlMx3IYPe0/JJcYP7Gxg7ttZZu3IInhuEhbchuqneY9wWe5Ojy2mXLBaQFUQmo0AW2r3qG7m1mg86js+gnlQ==",
            "bundled": true,
            "requires": {
              "minimatch": "^3.0.4"
            }
          },
          "inflight": {
            "version": "1.0.6",
            "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
            "integrity": "sha1-Sb1jMdfQLQwJvJEKEHW6gWW1bfk=",
            "bundled": true,
            "requires": {
              "once": "^1.3.0",
              "wrappy": "1"
            }
          },
          "inherits": {
            "version": "2.0.3",
            "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
            "integrity": "sha1-Yzwsg+PaQqUC9SRmAiSA9CCCYd4=",
            "bundled": true
          },
          "ini": {
            "version": "1.3.5",
            "resolved": "https://registry.npmjs.org/ini/-/ini-1.3.5.tgz",
            "integrity": "sha512-RZY5huIKCMRWDUqZlEi72f/lmXKMvuszcMBduliQ3nnWbx9X/ZBQO7DijMEYS9EhHBb2qacRUMtC7svLwe0lcw==",
            "bundled": true
          },
          "is-fullwidth-code-point": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
            "integrity": "sha1-754xOG8DGn8NZDr4L95QxFfvAMs=",
            "bundled": true,
            "requires": {
              "number-is-nan": "^1.0.0"
            }
          },
          "isarray": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
            "integrity": "sha1-u5NdSFgsuhaMBoNJV6VKPgcSTxE=",
            "bundled": true
          },
          "minimatch": {
            "version": "3.0.4",
            "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
            "integrity": "sha512-yJHVQEhyqPLUTgt9B83PXu6W3rx4MvvHvSUvToogpwoGDOUQ+yDrR0HRot+yOCdCO7u4hX3pWft6kWBBcqh0UA==",
            "bundled": true,
            "requires": {
              "brace-expansion": "^1.1.7"
            }
          },
          "minimist": {
            "version": "0.0.8",
            "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
            "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0=",
            "bundled": true
          },
          "minipass": {
            "version": "2.3.5",
            "resolved": "https://registry.npmjs.org/minipass/-/minipass-2.3.5.tgz",
            "integrity": "sha512-Gi1W4k059gyRbyVUZQ4mEqLm0YIUiGYfvxhF6SIlk3ui1WVxMTGfGdQ2SInh3PDrRTVvPKgULkpJtT4RH10+VA==",
            "bundled": true,
            "requires": {
              "safe-buffer": "^5.1.2",
              "yallist": "^3.0.0"
            }
          },
          "minizlib": {
            "version": "1.2.1",
            "resolved": "https://registry.npmjs.org/minizlib/-/minizlib-1.2.1.tgz",
            "integrity": "sha512-7+4oTUOWKg7AuL3vloEWekXY2/D20cevzsrNT2kGWm+39J9hGTCBv8VI5Pm5lXZ/o3/mdR4f8rflAPhnQb8mPA==",
            "bundled": true,
            "requires": {
              "minipass": "^2.2.1"
            }
          },
          "mkdirp": {
            "version": "0.5.1",
            "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
            "integrity": "sha1-MAV0OOrGz3+MR2fzhkjWaX11yQM=",
            "bundled": true,
            "requires": {
              "minimist": "0.0.8"
            }
          },
          "ms": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
            "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg==",
            "bundled": true
          },
          "needle": {
            "version": "2.3.0",
            "resolved": "https://registry.npmjs.org/needle/-/needle-2.3.0.tgz",
            "integrity": "sha512-QBZu7aAFR0522EyaXZM0FZ9GLpq6lvQ3uq8gteiDUp7wKdy0lSd2hPlgFwVuW1CBkfEs9PfDQsQzZghLs/psdg==",
            "bundled": true,
            "requires": {
              "debug": "^4.1.0",
              "iconv-lite": "^0.4.4",
              "sax": "^1.2.4"
            }
          },
          "node-pre-gyp": {
            "version": "0.12.0",
            "resolved": "https://registry.npmjs.org/node-pre-gyp/-/node-pre-gyp-0.12.0.tgz",
            "integrity": "sha512-4KghwV8vH5k+g2ylT+sLTjy5wmUOb9vPhnM8NHvRf9dHmnW/CndrFXy2aRPaPST6dugXSdHXfeaHQm77PIz/1A==",
            "bundled": true,
            "requires": {
              "detect-libc": "^1.0.2",
              "mkdirp": "^0.5.1",
              "needle": "^2.2.1",
              "nopt": "^4.0.1",
              "npm-packlist": "^1.1.6",
              "npmlog": "^4.0.2",
              "rc": "^1.2.7",
              "rimraf": "^2.6.1",
              "semver": "^5.3.0",
              "tar": "^4"
            }
          },
          "nopt": {
            "version": "4.0.1",
            "resolved": "https://registry.npmjs.org/nopt/-/nopt-4.0.1.tgz",
            "integrity": "sha1-0NRoWv1UFRk8jHUFYC0NF81kR00=",
            "bundled": true,
            "requires": {
              "abbrev": "1",
              "osenv": "^0.1.4"
            }
          },
          "npm-bundled": {
            "version": "1.0.6",
            "resolved": "https://registry.npmjs.org/npm-bundled/-/npm-bundled-1.0.6.tgz",
            "integrity": "sha512-8/JCaftHwbd//k6y2rEWp6k1wxVfpFzB6t1p825+cUb7Ym2XQfhwIC5KwhrvzZRJu+LtDE585zVaS32+CGtf0g==",
            "bundled": true
          },
          "npm-packlist": {
            "version": "1.4.1",
            "resolved": "https://registry.npmjs.org/npm-packlist/-/npm-packlist-1.4.1.tgz",
            "integrity": "sha512-+TcdO7HJJ8peiiYhvPxsEDhF3PJFGUGRcFsGve3vxvxdcpO2Z4Z7rkosRM0kWj6LfbK/P0gu3dzk5RU1ffvFcw==",
            "bundled": true,
            "requires": {
              "ignore-walk": "^3.0.1",
              "npm-bundled": "^1.0.1"
            }
          },
          "npmlog": {
            "version": "4.1.2",
            "resolved": "https://registry.npmjs.org/npmlog/-/npmlog-4.1.2.tgz",
            "integrity": "sha512-2uUqazuKlTaSI/dC8AzicUck7+IrEaOnN/e0jd3Xtt1KcGpwx30v50mL7oPyr/h9bL3E4aZccVwpwP+5W9Vjkg==",
            "bundled": true,
            "requires": {
              "are-we-there-yet": "~1.1.2",
              "console-control-strings": "~1.1.0",
              "gauge": "~2.7.3",
              "set-blocking": "~2.0.0"
            }
          },
          "number-is-nan": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
            "integrity": "sha1-CXtgK1NCKlIsGvuHkDGDNpQaAR0=",
            "bundled": true
          },
          "object-assign": {
            "version": "4.1.1",
            "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
            "integrity": "sha1-IQmtx5ZYh8/AXLvUQsrIv7s2CGM=",
            "bundled": true
          },
          "once": {
            "version": "1.4.0",
            "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
            "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
            "bundled": true,
            "requires": {
              "wrappy": "1"
            }
          },
          "os-homedir": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
            "integrity": "sha1-/7xJiDNuDoM94MFox+8VISGqf7M=",
            "bundled": true
          },
          "os-tmpdir": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
            "integrity": "sha1-u+Z0BseaqFxc/sdm/lc0VV36EnQ=",
            "bundled": true
          },
          "osenv": {
            "version": "0.1.5",
            "resolved": "https://registry.npmjs.org/osenv/-/osenv-0.1.5.tgz",
            "integrity": "sha512-0CWcCECdMVc2Rw3U5w9ZjqX6ga6ubk1xDVKxtBQPK7wis/0F2r9T6k4ydGYhecl7YUBxBVxhL5oisPsNxAPe2g==",
            "bundled": true,
            "requires": {
              "os-homedir": "^1.0.0",
              "os-tmpdir": "^1.0.0"
            }
          },
          "path-is-absolute": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
            "integrity": "sha1-F0uSaHNVNP+8es5r9TpanhtcX18=",
            "bundled": true
          },
          "process-nextick-args": {
            "version": "2.0.0",
            "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.0.tgz",
            "integrity": "sha512-MtEC1TqN0EU5nephaJ4rAtThHtC86dNN9qCuEhtshvpVBkAW5ZO7BASN9REnF9eoXGcRub+pFuKEpOHE+HbEMw==",
            "bundled": true
          },
          "rc": {
            "version": "1.2.8",
            "resolved": "https://registry.npmjs.org/rc/-/rc-1.2.8.tgz",
            "integrity": "sha512-y3bGgqKj3QBdxLbLkomlohkvsA8gdAiUQlSBJnBhfn+BPxg4bc62d8TcBW15wavDfgexCgccckhcZvywyQYPOw==",
            "bundled": true,
            "requires": {
              "deep-extend": "^0.6.0",
              "ini": "~1.3.0",
              "minimist": "^1.2.0",
              "strip-json-comments": "~2.0.1"
            },
            "dependencies": {
              "minimist": {
                "version": "1.2.0",
                "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
                "integrity": "sha1-o1AIsg9BOD7sH7kU9M1d95omQoQ=",
                "bundled": true
              }
            }
          },
          "readable-stream": {
            "version": "2.3.6",
            "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
            "integrity": "sha512-tQtKA9WIAhBF3+VLAseyMqZeBjW0AHJoxOtYqSUZNJxauErmLbVm2FW1y+J/YA9dUrAC39ITejlZWhVIwawkKw==",
            "bundled": true,
            "requires": {
              "core-util-is": "~1.0.0",
              "inherits": "~2.0.3",
              "isarray": "~1.0.0",
              "process-nextick-args": "~2.0.0",
              "safe-buffer": "~5.1.1",
              "string_decoder": "~1.1.1",
              "util-deprecate": "~1.0.1"
            }
          },
          "rimraf": {
            "version": "2.6.3",
            "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
            "integrity": "sha512-mwqeW5XsA2qAejG46gYdENaxXjx9onRNCfn7L0duuP4hCuTIi/QO7PDK07KJfp1d+izWPrzEJDcSqBa0OZQriA==",
            "bundled": true,
            "requires": {
              "glob": "^7.1.3"
            }
          },
          "safe-buffer": {
            "version": "5.1.2",
            "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
            "integrity": "sha512-Gd2UZBJDkXlY7GbJxfsE8/nvKkUEU1G38c1siN6QP6a9PT9MmHB8GnpscSmMJSoF8LOIrt8ud/wPtojys4G6+g==",
            "bundled": true
          },
          "safer-buffer": {
            "version": "2.1.2",
            "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
            "integrity": "sha512-YZo3K82SD7Riyi0E1EQPojLz7kpepnSQI9IyPbHHg1XXXevb5dJI7tpyN2ADxGcQbHG7vcyRHk0cbwqcQriUtg==",
            "bundled": true
          },
          "sax": {
            "version": "1.2.4",
            "resolved": "https://registry.npmjs.org/sax/-/sax-1.2.4.tgz",
            "integrity": "sha512-NqVDv9TpANUjFm0N8uM5GxL36UgKi9/atZw+x7YFnQ8ckwFGKrl4xX4yWtrey3UJm5nP1kUbnYgLopqWNSRhWw==",
            "bundled": true
          },
          "semver": {
            "version": "5.7.0",
            "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
            "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA==",
            "bundled": true
          },
          "set-blocking": {
            "version": "2.0.0",
            "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
            "integrity": "sha1-BF+XgtARrppoA93TgrJDkrPYkPc=",
            "bundled": true
          },
          "signal-exit": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
            "integrity": "sha1-tf3AjxKH6hF4Yo5BXiUTK3NkbG0=",
            "bundled": true
          },
          "string_decoder": {
            "version": "1.1.1",
            "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
            "integrity": "sha512-n/ShnvDi6FHbbVfviro+WojiFzv+s8MPMHBczVePfUpDJLwoLT0ht1l4YwBCbi8pJAveEEdnkHyPyTP/mzRfwg==",
            "bundled": true,
            "requires": {
              "safe-buffer": "~5.1.0"
            }
          },
          "string-width": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
            "integrity": "sha1-EYvfW4zcUaKn5w0hHgfisLmxB9M=",
            "bundled": true,
            "requires": {
              "code-point-at": "^1.0.0",
              "is-fullwidth-code-point": "^1.0.0",
              "strip-ansi": "^3.0.0"
            }
          },
          "strip-ansi": {
            "version": "3.0.1",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
            "integrity": "sha1-ajhfuIU9lS1f8F0Oiq+UJ43GPc8=",
            "bundled": true,
            "requires": {
              "ansi-regex": "^2.0.0"
            }
          },
          "strip-json-comments": {
            "version": "2.0.1",
            "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-2.0.1.tgz",
            "integrity": "sha1-PFMZQukIwml8DsNEhYwobHygpgo=",
            "bundled": true
          },
          "tar": {
            "version": "4.4.8",
            "resolved": "https://registry.npmjs.org/tar/-/tar-4.4.8.tgz",
            "integrity": "sha512-LzHF64s5chPQQS0IYBn9IN5h3i98c12bo4NCO7e0sGM2llXQ3p2FGC5sdENN4cTW48O915Sh+x+EXx7XW96xYQ==",
            "bundled": true,
            "requires": {
              "chownr": "^1.1.1",
              "fs-minipass": "^1.2.5",
              "minipass": "^2.3.4",
              "minizlib": "^1.1.1",
              "mkdirp": "^0.5.0",
              "safe-buffer": "^5.1.2",
              "yallist": "^3.0.2"
            }
          },
          "util-deprecate": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
            "integrity": "sha1-RQ1Nyfpw3nMnYvvS1KKJgUGaDM8=",
            "bundled": true
          },
          "wide-align": {
            "version": "1.1.3",
            "resolved": "https://registry.npmjs.org/wide-align/-/wide-align-1.1.3.tgz",
            "integrity": "sha512-QGkOQc8XL6Bt5PwnsExKBPuMKBxnGxWWW3fU55Xt4feHozMUhdUMaBCk290qpm/wG5u/RSKzwdAC4i51YigihA==",
            "bundled": true,
            "requires": {
              "string-width": "^1.0.2 || 2"
            }
          },
          "wrappy": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
            "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8=",
            "bundled": true
          },
          "yallist": {
            "version": "3.0.3",
            "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.0.3.tgz",
            "integrity": "sha512-S+Zk8DEWE6oKpV+vI3qWkaK+jSbIK86pCwe2IF/xwIpQ8jEuxpw9NyaGjmp9+BoJv5FV2piqCDcoCtStppiq2A==",
            "bundled": true
          }
        }
      },
      "function-loop": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
        "integrity": "sha512-Iw4MzMfS3udk/rqxTiDDCllhGwlOrsr50zViTOO/W6lS/9y6B1J0BD2VZzrnWUYBJsl3aeqjgR5v7bWWhZSYbA=="
      },
      "get-caller-file": {
        "version": "2.0.5",
        "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
        "integrity": "sha512-DyFP3BM/3YHTQOCUL/w0OZHR0lpKeGrxotcHWcqNEdnltqFwXVfhEBQ94eIo34AfQpo0rGki4cyIiftY06h2Fg=="
      },
      "get-stream": {
        "version": "4.1.0",
        "resolved": "https://registry.npmjs.org/get-stream/-/get-stream-4.1.0.tgz",
        "integrity": "sha512-GMat4EJ5161kIy2HevLlr4luNjBgvmj413KaQA7jt4V8B4RDsfpHk7WQ9GVqfYyyx8OS/L66Kox+rJRNklLK7w==",
        "requires": {
          "pump": "^3.0.0"
        }
      },
      "get-value": {
        "version": "2.0.6",
        "resolved": "https://registry.npmjs.org/get-value/-/get-value-2.0.6.tgz",
        "integrity": "sha1-3BXKHGcjh8p2vTesCjlbogQqLCg="
      },
      "getpass": {
        "version": "0.1.7",
        "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
        "integrity": "sha1-Xv+OPmhNVprkyysSgmBOi6YhSfo=",
        "requires": {
          "assert-plus": "^1.0.0"
        }
      },
      "glob": {
        "version": "7.1.3",
        "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.3.tgz",
        "integrity": "sha512-vcfuiIxogLV4DlGBHIUOwI0IbrJ8HWPc4MU7HzviGeNho/UJDfi6B5p3sHeWIQ0KGIU0Jpxi5ZHxemQfLkkAwQ==",
        "requires": {
          "fs.realpath": "^1.0.0",
          "inflight": "^1.0.4",
          "inherits": "2",
          "minimatch": "^3.0.4",
          "once": "^1.3.0",
          "path-is-absolute": "^1.0.0"
        }
      },
      "glob-parent": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-3.1.0.tgz",
        "integrity": "sha1-nmr2KZ2NO9K9QEMIMr0RPfkGxa4=",
        "requires": {
          "is-glob": "^3.1.0",
          "path-dirname": "^1.0.0"
        },
        "dependencies": {
          "is-glob": {
            "version": "3.1.0",
            "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-3.1.0.tgz",
            "integrity": "sha1-e6WuJCF4BKxwcHuWkiVnSGzD6Eo=",
            "requires": {
              "is-extglob": "^2.1.0"
            }
          }
        }
      },
      "globals": {
        "version": "9.18.0",
        "resolved": "https://registry.npmjs.org/globals/-/globals-9.18.0.tgz",
        "integrity": "sha512-S0nG3CLEQiY/ILxqtztTWH/3iRRdyBLw6KMDxnKMchrtbj2OFmehVh0WUCfW3DUrIgx/qFrJPICrq4Z4sTR9UQ=="
      },
      "graceful-fs": {
        "version": "4.1.15",
        "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.1.15.tgz",
        "integrity": "sha512-6uHUhOPEBgQ24HM+r6b/QwWfZq+yiFcipKFrOFiBEnWdy5sdzYoi+pJeQaPI5qOLRFqWmAXUPQNsielzdLoecA=="
      },
      "growl": {
        "version": "1.10.5",
        "resolved": "https://registry.npmjs.org/growl/-/growl-1.10.5.tgz",
        "integrity": "sha512-qBr4OuELkhPenW6goKVXiv47US3clb3/IbuWF9KNKEijAy9oeHxU9IgzjvJhHkUzhaj7rOUD7+YGWqUjLp5oSA=="
      },
      "handlebars": {
        "version": "4.1.2",
        "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.1.2.tgz",
        "integrity": "sha512-nvfrjqvt9xQ8Z/w0ijewdD/vvWDTOweBUm96NTr66Wfvo1mJenBLwcYmPs3TIBP5ruzYGD7Hx/DaM9RmhroGPw==",
        "requires": {
          "neo-async": "^2.6.0",
          "optimist": "^0.6.1",
          "source-map": "^0.6.1",
          "uglify-js": "^3.1.4"
        },
        "dependencies": {
          "source-map": {
            "version": "0.6.1",
            "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
            "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g=="
          }
        }
      },
      "har-schema": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
        "integrity": "sha1-qUwiJOvKwEeCoNkDVSHyRzW37JI="
      },
      "har-validator": {
        "version": "5.1.3",
        "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
        "integrity": "sha512-sNvOCzEQNr/qrvJgc3UG/kD4QtlHycrzwS+6mfTrrSq97BvaYcPZZI1ZSqGSPR73Cxn4LKTD4PttRwfU7jWq5g==",
        "requires": {
          "ajv": "^6.5.5",
          "har-schema": "^2.0.0"
        }
      },
      "has-ansi": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/has-ansi/-/has-ansi-2.0.0.tgz",
        "integrity": "sha1-NPUEnOHs3ysGSa8+8k5F7TVBbZE=",
        "requires": {
          "ansi-regex": "^2.0.0"
        }
      },
      "has-flag": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
        "integrity": "sha1-tdRU3CGZriJWmfNGfloH87lVuv0="
      },
      "has-value": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/has-value/-/has-value-1.0.0.tgz",
        "integrity": "sha1-GLKB2lhbHFxR3vJMkw7SmgvmsXc=",
        "requires": {
          "get-value": "^2.0.6",
          "has-values": "^1.0.0",
          "isobject": "^3.0.0"
        }
      },
      "has-values": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/has-values/-/has-values-1.0.0.tgz",
        "integrity": "sha1-lbC2P+whRmGab+V/51Yo1aOe/k8=",
        "requires": {
          "is-number": "^3.0.0",
          "kind-of": "^4.0.0"
        },
        "dependencies": {
          "kind-of": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-4.0.0.tgz",
            "integrity": "sha1-IIE989cSkosgc3hpGkUGb65y3Vc=",
            "requires": {
              "is-buffer": "^1.1.5"
            }
          }
        }
      },
      "hasha": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
        "integrity": "sha1-UqMvq4Vp1BymmmH/GiFPjrfIvTk=",
        "requires": {
          "is-stream": "^1.0.1"
        }
      },
      "home-or-tmp": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/home-or-tmp/-/home-or-tmp-2.0.0.tgz",
        "integrity": "sha1-42w/LSyufXRqhX440Y1fMqeILbg=",
        "requires": {
          "os-homedir": "^1.0.0",
          "os-tmpdir": "^1.0.1"
        }
      },
      "hosted-git-info": {
        "version": "2.7.1",
        "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.7.1.tgz",
        "integrity": "sha512-7T/BxH19zbcCTa8XkMlbK5lTo1WtgkFi3GvdWEyNuc4Vex7/9Dqbnpsf4JMydcfj9HCg4zUWFTL3Za6lapg5/w=="
      },
      "http-signature": {
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
        "integrity": "sha1-muzZJRFHcvPZW2WmCruPfBj7rOE=",
        "requires": {
          "assert-plus": "^1.0.0",
          "jsprim": "^1.2.2",
          "sshpk": "^1.7.0"
        }
      },
      "import-jsx": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/import-jsx/-/import-jsx-2.0.0.tgz",
        "integrity": "sha512-xmrgtiRnAdjIaRzKwsHut54FA8nx59WqN4MpQvPFr/8yD6BamavkmKHrA5dotAlnIiF4uqMzg/lA5yhPdpIXsA==",
        "requires": {
          "babel-core": "^6.25.0",
          "babel-plugin-transform-es2015-destructuring": "^6.23.0",
          "babel-plugin-transform-object-rest-spread": "^6.23.0",
          "babel-plugin-transform-react-jsx": "^6.24.1",
          "caller-path": "^2.0.0",
          "resolve-from": "^3.0.0"
        }
      },
      "imurmurhash": {
        "version": "0.1.4",
        "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
        "integrity": "sha1-khi5srkoojixPcT7a21XbyMUU+o="
      },
      "inflight": {
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
        "integrity": "sha1-Sb1jMdfQLQwJvJEKEHW6gWW1bfk=",
        "requires": {
          "once": "^1.3.0",
          "wrappy": "1"
        }
      },
      "inherits": {
        "version": "2.0.3",
        "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
        "integrity": "sha1-Yzwsg+PaQqUC9SRmAiSA9CCCYd4="
      },
      "ink": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/ink/-/ink-2.1.1.tgz",
        "integrity": "sha512-vP1yE/uJoiY6uB9yHalczUA02I9fg7xDUbTEZitPK5y6dvnPo9a/6UWqIB2uCYkHOhEZMN+D/TsVr4v2sz8qYA==",
        "requires": {
          "@types/react": "^16.8.6",
          "arrify": "^1.0.1",
          "auto-bind": "^2.0.0",
          "chalk": "^2.4.1",
          "cli-cursor": "^2.1.0",
          "cli-truncate": "^1.1.0",
          "is-ci": "^2.0.0",
          "lodash.throttle": "^4.1.1",
          "log-update": "^3.0.0",
          "prop-types": "^15.6.2",
          "react": ">=16.8.0",
          "react-reconciler": "^0.20.0",
          "scheduler": "^0.13.2",
          "signal-exit": "^3.0.2",
          "slice-ansi": "^1.0.0",
          "string-length": "^2.0.0",
          "widest-line": "^2.0.0",
          "wrap-ansi": "^5.0.0",
          "yoga-layout-prebuilt": "^1.9.3"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "4.1.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
            "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg=="
          },
          "ansi-styles": {
            "version": "3.2.1",
            "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
            "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
            "requires": {
              "color-convert": "^1.9.0"
            }
          },
          "chalk": {
            "version": "2.4.2",
            "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
            "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
            "requires": {
              "ansi-styles": "^3.2.1",
              "escape-string-regexp": "^1.0.5",
              "supports-color": "^5.3.0"
            }
          },
          "string-width": {
            "version": "3.1.0",
            "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
            "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
            "requires": {
              "emoji-regex": "^7.0.1",
              "is-fullwidth-code-point": "^2.0.0",
              "strip-ansi": "^5.1.0"
            }
          },
          "strip-ansi": {
            "version": "5.2.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
            "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
            "requires": {
              "ansi-regex": "^4.1.0"
            }
          },
          "supports-color": {
            "version": "5.5.0",
            "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
            "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
            "requires": {
              "has-flag": "^3.0.0"
            }
          },
          "wrap-ansi": {
            "version": "5.1.0",
            "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
            "integrity": "sha512-QC1/iN/2/RPVJ5jYK8BGttj5z83LmSKmvbvrXPNCLZSEb32KKVDJDl/MOt2N01qU2H/FkzEa9PKto1BqDjtd7Q==",
            "requires": {
              "ansi-styles": "^3.2.0",
              "string-width": "^3.0.0",
              "strip-ansi": "^5.0.0"
            }
          }
        }
      },
      "invariant": {
        "version": "2.2.4",
        "resolved": "https://registry.npmjs.org/invariant/-/invariant-2.2.4.tgz",
        "integrity": "sha512-phJfQVBuaJM5raOpJjSfkiD6BpbCE4Ns//LaXl6wGYtUBY83nWS6Rf9tXm2e8VaK60JEjYldbPif/A2B1C2gNA==",
        "requires": {
          "loose-envify": "^1.0.0"
        }
      },
      "invert-kv": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/invert-kv/-/invert-kv-2.0.0.tgz",
        "integrity": "sha512-wPVv/y/QQ/Uiirj/vh3oP+1Ww+AWehmi1g5fFWGPF6IpCBCDVrhgHRMvrLfdYcwDh3QJbGXDW4JAuzxElLSqKA=="
      },
      "is-accessor-descriptor": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-1.0.0.tgz",
        "integrity": "sha512-m5hnHTkcVsPfqx3AKlyttIPb7J+XykHvJP2B9bZDjlhLIoEq4XoK64Vg7boZlVWYK6LUY94dYPEE7Lh0ZkZKcQ==",
        "requires": {
          "kind-of": "^6.0.0"
        },
        "dependencies": {
          "kind-of": {
            "version": "6.0.2",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
            "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA=="
          }
        }
      },
      "is-arrayish": {
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
        "integrity": "sha1-d8mYQFJ6qOyxqLppe4BkWnqSap0="
      },
      "is-binary-path": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-1.0.1.tgz",
        "integrity": "sha1-dfFmQrSA8YenEcgUFh/TpKdlWJg=",
        "requires": {
          "binary-extensions": "^1.0.0"
        }
      },
      "is-buffer": {
        "version": "1.1.6",
        "resolved": "https://registry.npmjs.org/is-buffer/-/is-buffer-1.1.6.tgz",
        "integrity": "sha512-NcdALwpXkTm5Zvvbk7owOUSvVvBKDgKP5/ewfXEznmQFfs4ZRmanOeKBTjRVjka3QFoN6XJ+9F3USqfHqTaU5w=="
      },
      "is-ci": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/is-ci/-/is-ci-2.0.0.tgz",
        "integrity": "sha512-YfJT7rkpQB0updsdHLGWrvhBJfcfzNNawYDNIyQXJz0IViGf75O8EBPKSdvw2rF+LGCsX4FZ8tcr3b19LcZq4w==",
        "requires": {
          "ci-info": "^2.0.0"
        }
      },
      "is-data-descriptor": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-1.0.0.tgz",
        "integrity": "sha512-jbRXy1FmtAoCjQkVmIVYwuuqDFUbaOeDjmed1tOGPrsMhtJA4rD9tkgA0F1qJ3gRFRXcHYVkdeaP50Q5rE/jLQ==",
        "requires": {
          "kind-of": "^6.0.0"
        },
        "dependencies": {
          "kind-of": {
            "version": "6.0.2",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
            "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA=="
          }
        }
      },
      "is-descriptor": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-1.0.2.tgz",
        "integrity": "sha512-2eis5WqQGV7peooDyLmNEPUrps9+SXX5c9pL3xEB+4e9HnGuDa7mB7kHxHw4CbqS9k1T2hOH3miL8n8WtiYVtg==",
        "requires": {
          "is-accessor-descriptor": "^1.0.0",
          "is-data-descriptor": "^1.0.0",
          "kind-of": "^6.0.2"
        },
        "dependencies": {
          "kind-of": {
            "version": "6.0.2",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
            "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA=="
          }
        }
      },
      "is-extendable": {
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-0.1.1.tgz",
        "integrity": "sha1-YrEQ4omkcUGOPsNqYX1HLjAd/Ik="
      },
      "is-extglob": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
        "integrity": "sha1-qIwCU1eR8C7TfHahueqXc8gz+MI="
      },
      "is-finite": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-finite/-/is-finite-1.0.2.tgz",
        "integrity": "sha1-zGZ3aVYCvlUO8R6LSqYwU0K20Ko=",
        "requires": {
          "number-is-nan": "^1.0.0"
        }
      },
      "is-fullwidth-code-point": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
        "integrity": "sha1-o7MKXE8ZkYMWeqq5O+764937ZU8="
      },
      "is-glob": {
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
        "integrity": "sha512-5G0tKtBTFImOqDnLB2hG6Bp2qcKEFduo4tZu9MT/H6NQv/ghhy30o55ufafxJ/LdH79LLs2Kfrn85TLKyA7BUg==",
        "requires": {
          "is-extglob": "^2.1.1"
        }
      },
      "is-number": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/is-number/-/is-number-3.0.0.tgz",
        "integrity": "sha1-JP1iAaR4LPUFYcgQJ2r8fRLXEZU=",
        "requires": {
          "kind-of": "^3.0.2"
        }
      },
      "is-plain-object": {
        "version": "2.0.4",
        "resolved": "https://registry.npmjs.org/is-plain-object/-/is-plain-object-2.0.4.tgz",
        "integrity": "sha512-h5PpgXkWitc38BBMYawTYMWJHFZJVnBquFE57xFpjB8pJFiF6gZ+bU+WyI/yqXiFR5mdLsgYNaPe8uao6Uv9Og==",
        "requires": {
          "isobject": "^3.0.1"
        }
      },
      "is-stream": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
        "integrity": "sha1-EtSj3U5o4Lec6428hBc66A2RykQ="
      },
      "is-typedarray": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
        "integrity": "sha1-5HnICFjfDBsR3dppQPlgEfzaSpo="
      },
      "is-windows": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/is-windows/-/is-windows-1.0.2.tgz",
        "integrity": "sha512-eXK1UInq2bPmjyX6e3VHIzMLobc4J94i4AWn+Hpq3OU5KkrRC96OAcR3PRJ/pGu6m8TRnBHP9dkXQVsT/COVIA=="
      },
      "isarray": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
        "integrity": "sha1-u5NdSFgsuhaMBoNJV6VKPgcSTxE="
      },
      "isexe": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
        "integrity": "sha1-6PvzdNxVb/iUehDcsFctYz8s+hA="
      },
      "isobject": {
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/isobject/-/isobject-3.0.1.tgz",
        "integrity": "sha1-TkMekrEalzFjaqH5yNHMvP2reN8="
      },
      "isstream": {
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
        "integrity": "sha1-R+Y/evVa+m+S4VAOaQ64uFKcCZo="
      },
      "istanbul-lib-coverage": {
        "version": "2.0.5",
        "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
        "integrity": "sha512-8aXznuEPCJvGnMSRft4udDRDtb1V3pkQkMMI5LI+6HuQz5oQ4J2UFn1H82raA3qJtyOLkkwVqICBQkjnGtn5mA=="
      },
      "istanbul-lib-hook": {
        "version": "2.0.7",
        "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
        "integrity": "sha512-vrRztU9VRRFDyC+aklfLoeXyNdTfga2EI3udDGn4cZ6fpSXpHLV9X6CHvfoMCPtggg8zvDDmC4b9xfu0z6/llA==",
        "requires": {
          "append-transform": "^1.0.0"
        }
      },
      "istanbul-lib-instrument": {
        "version": "3.3.0",
        "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
        "integrity": "sha512-5nnIN4vo5xQZHdXno/YDXJ0G+I3dAm4XgzfSVTPLQpj/zAV2dV6Juy0yaf10/zrJOJeHoN3fraFe+XRq2bFVZA==",
        "requires": {
          "@babel/generator": "^7.4.0",
          "@babel/parser": "^7.4.3",
          "@babel/template": "^7.4.0",
          "@babel/traverse": "^7.4.3",
          "@babel/types": "^7.4.0",
          "istanbul-lib-coverage": "^2.0.5",
          "semver": "^6.0.0"
        }
      },
      "istanbul-lib-processinfo": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
        "integrity": "sha512-FY0cPmWa4WoQNlvB8VOcafiRoB5nB+l2Pz2xGuXHRSy1KM8QFOYfz/rN+bGMCAeejrY3mrpF5oJHcN0s/garCg==",
        "requires": {
          "archy": "^1.0.0",
          "cross-spawn": "^6.0.5",
          "istanbul-lib-coverage": "^2.0.3",
          "rimraf": "^2.6.3",
          "uuid": "^3.3.2"
        },
        "dependencies": {
          "cross-spawn": {
            "version": "6.0.5",
            "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
            "integrity": "sha512-eTVLrBSt7fjbDygz805pMnstIs2VTBNkRm0qxZd+M7A5XDdxVRWO5MxGBXZhjY4cqLYLdtrGqRf8mBPmzwSpWQ==",
            "requires": {
              "nice-try": "^1.0.4",
              "path-key": "^2.0.1",
              "semver": "^5.5.0",
              "shebang-command": "^1.2.0",
              "which": "^1.2.9"
            }
          },
          "semver": {
            "version": "5.7.0",
            "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
            "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA=="
          }
        }
      },
      "istanbul-lib-report": {
        "version": "2.0.8",
        "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
        "integrity": "sha512-fHBeG573EIihhAblwgxrSenp0Dby6tJMFR/HvlerBsrCTD5bkUuoNtn3gVh29ZCS824cGGBPn7Sg7cNk+2xUsQ==",
        "requires": {
          "istanbul-lib-coverage": "^2.0.5",
          "make-dir": "^2.1.0",
          "supports-color": "^6.1.0"
        },
        "dependencies": {
          "supports-color": {
            "version": "6.1.0",
            "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
            "integrity": "sha512-qe1jfm1Mg7Nq/NSh6XE24gPXROEVsWHxC1LIx//XNlD9iw7YZQGjZNjYN7xGaEG6iKdA8EtNFW6R0gjnVXp+wQ==",
            "requires": {
              "has-flag": "^3.0.0"
            }
          }
        }
      },
      "istanbul-lib-source-maps": {
        "version": "3.0.6",
        "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
        "integrity": "sha512-R47KzMtDJH6X4/YW9XTx+jrLnZnscW4VpNN+1PViSYTejLVPWv7oov+Duf8YQSPyVRUvueQqz1TcsC6mooZTXw==",
        "requires": {
          "debug": "^4.1.1",
          "istanbul-lib-coverage": "^2.0.5",
          "make-dir": "^2.1.0",
          "rimraf": "^2.6.3",
          "source-map": "^0.6.1"
        },
        "dependencies": {
          "debug": {
            "version": "4.1.1",
            "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
            "integrity": "sha512-pYAIzeRo8J6KPEaJ0VWOh5Pzkbw/RetuzehGM7QRRX5he4fPHx2rdKMB256ehJCkX+XRQm16eZLqLNS8RSZXZw==",
            "requires": {
              "ms": "^2.1.1"
            }
          },
          "ms": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
            "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg=="
          },
          "source-map": {
            "version": "0.6.1",
            "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
            "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g=="
          }
        }
      },
      "istanbul-reports": {
        "version": "2.2.4",
        "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.4.tgz",
        "integrity": "sha512-QCHGyZEK0bfi9GR215QSm+NJwFKEShbtc7tfbUdLAEzn3kKhLDDZqvljn8rPZM9v8CEOhzL1nlYoO4r1ryl67w==",
        "requires": {
          "handlebars": "^4.1.2"
        }
      },
      "jackspeak": {
        "version": "1.3.7",
        "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.3.7.tgz",
        "integrity": "sha512-Z4iSFpaCV7Cocpcl5t9/UyPkisxenbmaqminyTgK6lDDMXcm9EvIZ9Bwr/uFbGOjfWlz1UZwKwFY5AvtgNlHuw==",
        "requires": {
          "cliui": "^4.1.0"
        }
      },
      "js-tokens": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-3.0.2.tgz",
        "integrity": "sha1-mGbfOVECEw449/mWvOtlRDIJwls="
      },
      "js-yaml": {
        "version": "3.13.1",
        "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
        "integrity": "sha512-YfbcO7jXDdyj0DGxYVSlSeQNHbD7XPWvrVWeVUujrQEoZzWJIRrCPoyk6kL6IAjAG2IolMK4T0hNUe0HOUs5Jw==",
        "requires": {
          "argparse": "^1.0.7",
          "esprima": "^4.0.0"
        }
      },
      "jsbn": {
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
        "integrity": "sha1-peZUwuWi3rXyAdls77yoDA7y9RM="
      },
      "jsesc": {
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-1.3.0.tgz",
        "integrity": "sha1-RsP+yMGJKxKwgz25vHYiF226s0s="
      },
      "json-parse-better-errors": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
        "integrity": "sha512-mrqyZKfX5EhL7hvqcV6WG1yYjnjeuYDzDhhcAAUrq8Po85NBQBJP+ZDUT75qZQ98IkUoBqdkExkukOU7Ts2wrw=="
      },
      "json-schema": {
        "version": "0.2.3",
        "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
        "integrity": "sha1-tIDIkuWaLwWVTOcnvT8qTogvnhM="
      },
      "json-schema-traverse": {
        "version": "0.4.1",
        "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
        "integrity": "sha512-xbbCH5dCYU5T8LcEhhuh7HJ88HXuW3qsI3Y0zOZFKfZEHcpWiHU/Jxzk629Brsab/mMiHQti9wMP+845RPe3Vg=="
      },
      "json-stringify-safe": {
        "version": "5.0.1",
        "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
        "integrity": "sha1-Epai1Y/UXxmg9s4B1lcB4sc1tus="
      },
      "json5": {
        "version": "0.5.1",
        "resolved": "https://registry.npmjs.org/json5/-/json5-0.5.1.tgz",
        "integrity": "sha1-Hq3nrMASA0rYTiOWdn6tn6VJWCE="
      },
      "jsprim": {
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
        "integrity": "sha1-MT5mvB5cwG5Di8G3SZwuXFastqI=",
        "requires": {
          "assert-plus": "1.0.0",
          "extsprintf": "1.3.0",
          "json-schema": "0.2.3",
          "verror": "1.10.0"
        }
      },
      "kind-of": {
        "version": "3.2.2",
        "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-3.2.2.tgz",
        "integrity": "sha1-MeohpzS6ubuw8yRm2JOupR5KPGQ=",
        "requires": {
          "is-buffer": "^1.1.5"
        }
      },
      "lcid": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/lcid/-/lcid-2.0.0.tgz",
        "integrity": "sha512-avPEb8P8EGnwXKClwsNUgryVjllcRqtMYa49NTsbQagYuT1DcXnl1915oxWjoyGrXR6zH/Y0Zc96xWsPcoDKeA==",
        "requires": {
          "invert-kv": "^2.0.0"
        }
      },
      "lcov-parse": {
        "version": "0.0.10",
        "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-0.0.10.tgz",
        "integrity": "sha1-GwuP+ayceIklBYK3C3ExXZ2m2aM="
      },
      "load-json-file": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
        "integrity": "sha1-L19Fq5HjMhYjT9U62rZo607AmTs=",
        "requires": {
          "graceful-fs": "^4.1.2",
          "parse-json": "^4.0.0",
          "pify": "^3.0.0",
          "strip-bom": "^3.0.0"
        },
        "dependencies": {
          "pify": {
            "version": "3.0.0",
            "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
            "integrity": "sha1-5aSs0sEB/fPZpNB/DbxNtJ3SgXY="
          }
        }
      },
      "locate-path": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
        "integrity": "sha512-7AO748wWnIhNqAuaty2ZWHkQHRSNfPVIsPIfwEOWO22AmaoVrWavlOcMR5nzTLNYvp36X220/maaRsrec1G65A==",
        "requires": {
          "p-locate": "^3.0.0",
          "path-exists": "^3.0.0"
        }
      },
      "lodash": {
        "version": "4.17.11",
        "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.11.tgz",
        "integrity": "sha512-cQKh8igo5QUhZ7lg38DYWAxMvjSAKG0A8wGSVimP07SIUEK2UO+arSRKbRZWtelMtN5V0Hkwh5ryOto/SshYIg=="
      },
      "lodash.flattendeep": {
        "version": "4.4.0",
        "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
        "integrity": "sha1-+wMJF/hqMTTlvJvsDWngAT3f7bI="
      },
      "lodash.throttle": {
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/lodash.throttle/-/lodash.throttle-4.1.1.tgz",
        "integrity": "sha1-wj6RtxAkKscMN/HhzaknTMOb8vQ="
      },
      "log-driver": {
        "version": "1.2.7",
        "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
        "integrity": "sha512-U7KCmLdqsGHBLeWqYlFA0V0Sl6P08EE1ZrmA9cxjUE0WVqT9qnyVDPz1kzpFEP0jdJuFnasWIfSd7fsaNXkpbg=="
      },
      "log-update": {
        "version": "3.2.0",
        "resolved": "https://registry.npmjs.org/log-update/-/log-update-3.2.0.tgz",
        "integrity": "sha512-KJ6zAPIHWo7Xg1jYror6IUDFJBq1bQ4Bi4wAEp2y/0ScjBBVi/g0thr0sUVhuvuXauWzczt7T2QHghPDNnKBuw==",
        "requires": {
          "ansi-escapes": "^3.2.0",
          "cli-cursor": "^2.1.0",
          "wrap-ansi": "^5.0.0"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "4.1.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
            "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg=="
          },
          "ansi-styles": {
            "version": "3.2.1",
            "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
            "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
            "requires": {
              "color-convert": "^1.9.0"
            }
          },
          "string-width": {
            "version": "3.1.0",
            "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
            "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
            "requires": {
              "emoji-regex": "^7.0.1",
              "is-fullwidth-code-point": "^2.0.0",
              "strip-ansi": "^5.1.0"
            }
          },
          "strip-ansi": {
            "version": "5.2.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
            "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
            "requires": {
              "ansi-regex": "^4.1.0"
            }
          },
          "wrap-ansi": {
            "version": "5.1.0",
            "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
            "integrity": "sha512-QC1/iN/2/RPVJ5jYK8BGttj5z83LmSKmvbvrXPNCLZSEb32KKVDJDl/MOt2N01qU2H/FkzEa9PKto1BqDjtd7Q==",
            "requires": {
              "ansi-styles": "^3.2.0",
              "string-width": "^3.0.0",
              "strip-ansi": "^5.0.0"
            }
          }
        }
      },
      "loose-envify": {
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
        "integrity": "sha512-lyuxPGr/Wfhrlem2CL/UcnUc1zcqKAImBDzukY7Y5F/yQiNdko6+fRLevlw1HgMySw7f611UIY408EtxRSoK3Q==",
        "requires": {
          "js-tokens": "^3.0.0 || ^4.0.0"
        }
      },
      "lru-cache": {
        "version": "4.1.5",
        "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
        "integrity": "sha512-sWZlbEP2OsHNkXrMl5GYk/jKk70MBng6UU4YI/qGDYbgf6YbP4EvmqISbXCoJiRKs+1bSpFHVgQxvJ17F2li5g==",
        "requires": {
          "pseudomap": "^1.0.2",
          "yallist": "^2.1.2"
        }
      },
      "make-dir": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
        "integrity": "sha512-LS9X+dc8KLxXCb8dni79fLIIUA5VyZoyjSMCwTluaXA0o27cCK0bhXkpgw+sTXVpPy/lSO57ilRixqk0vDmtRA==",
        "requires": {
          "pify": "^4.0.1",
          "semver": "^5.6.0"
        },
        "dependencies": {
          "semver": {
            "version": "5.7.0",
            "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
            "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA=="
          }
        }
      },
      "make-error": {
        "version": "1.3.5",
        "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
        "integrity": "sha512-c3sIjNUow0+8swNwVpqoH4YCShKNFkMaw6oH1mNS2haDZQqkeZFlHS3dhoeEbKKmJB4vXpJucU6oH75aDYeE9g=="
      },
      "map-age-cleaner": {
        "version": "0.1.3",
        "resolved": "https://registry.npmjs.org/map-age-cleaner/-/map-age-cleaner-0.1.3.tgz",
        "integrity": "sha512-bJzx6nMoP6PDLPBFmg7+xRKeFZvFboMrGlxmNj9ClvX53KrmvM5bXFXEWjbz4cz1AFn+jWJ9z/DJSz7hrs0w3w==",
        "requires": {
          "p-defer": "^1.0.0"
        }
      },
      "map-cache": {
        "version": "0.2.2",
        "resolved": "https://registry.npmjs.org/map-cache/-/map-cache-0.2.2.tgz",
        "integrity": "sha1-wyq9C9ZSXZsFFkW7TyasXcmKDb8="
      },
      "map-visit": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/map-visit/-/map-visit-1.0.0.tgz",
        "integrity": "sha1-7Nyo8TFE5mDxtb1B8S80edmN+48=",
        "requires": {
          "object-visit": "^1.0.0"
        }
      },
      "mem": {
        "version": "4.3.0",
        "resolved": "https://registry.npmjs.org/mem/-/mem-4.3.0.tgz",
        "integrity": "sha512-qX2bG48pTqYRVmDB37rn/6PT7LcR8T7oAX3bf99u1Tt1nzxYfxkgqDwUwolPlXweM0XzBOBFzSx4kfp7KP1s/w==",
        "requires": {
          "map-age-cleaner": "^0.1.1",
          "mimic-fn": "^2.0.0",
          "p-is-promise": "^2.0.0"
        },
        "dependencies": {
          "mimic-fn": {
            "version": "2.1.0",
            "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-2.1.0.tgz",
            "integrity": "sha512-OqbOk5oEQeAZ8WXWydlu9HJjz9WVdEIvamMCcXmuqUYjTknH/sqsWvhQ3vgwKFRR1HpjvNBKQ37nbJgYzGqGcg=="
          }
        }
      },
      "merge-source-map": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
        "integrity": "sha512-Qkcp7P2ygktpMPh2mCQZaf3jhN6D3Z/qVZHSdWvQ+2Ef5HgRAPBO57A77+ENm0CPx2+1Ce/MYKi3ymqdfuqibw==",
        "requires": {
          "source-map": "^0.6.1"
        },
        "dependencies": {
          "source-map": {
            "version": "0.6.1",
            "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
            "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g=="
          }
        }
      },
      "micromatch": {
        "version": "3.1.10",
        "resolved": "https://registry.npmjs.org/micromatch/-/micromatch-3.1.10.tgz",
        "integrity": "sha512-MWikgl9n9M3w+bpsY3He8L+w9eF9338xRl8IAO5viDizwSzziFEyUzo2xrrloB64ADbTf8uA8vRqqttDTOmccg==",
        "requires": {
          "arr-diff": "^4.0.0",
          "array-unique": "^0.3.2",
          "braces": "^2.3.1",
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "extglob": "^2.0.4",
          "fragment-cache": "^0.2.1",
          "kind-of": "^6.0.2",
          "nanomatch": "^1.2.9",
          "object.pick": "^1.3.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.2"
        },
        "dependencies": {
          "extend-shallow": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
            "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
            "requires": {
              "assign-symbols": "^1.0.0",
              "is-extendable": "^1.0.1"
            }
          },
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          },
          "kind-of": {
            "version": "6.0.2",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
            "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA=="
          }
        }
      },
      "mime-db": {
        "version": "1.40.0",
        "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.40.0.tgz",
        "integrity": "sha512-jYdeOMPy9vnxEqFRRo6ZvTZ8d9oPb+k18PKoYNYUe2stVEBPPwsln/qWzdbmaIvnhZ9v2P+CuecK+fpUfsV2mA=="
      },
      "mime-types": {
        "version": "2.1.24",
        "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.24.tgz",
        "integrity": "sha512-WaFHS3MCl5fapm3oLxU4eYDw77IQM2ACcxQ9RIxfaC3ooc6PFuBMGZZsYpvoXS5D5QTWPieo1jjLdAm3TBP3cQ==",
        "requires": {
          "mime-db": "1.40.0"
        }
      },
      "mimic-fn": {
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/mimic-fn/-/mimic-fn-1.2.0.tgz",
        "integrity": "sha512-jf84uxzwiuiIVKiOLpfYk7N46TSy8ubTonmneY9vrpHNAnp0QBt2BxWV9dO3/j+BoVAb+a5G6YDPW3M5HOdMWQ=="
      },
      "minimatch": {
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
        "integrity": "sha512-yJHVQEhyqPLUTgt9B83PXu6W3rx4MvvHvSUvToogpwoGDOUQ+yDrR0HRot+yOCdCO7u4hX3pWft6kWBBcqh0UA==",
        "requires": {
          "brace-expansion": "^1.1.7"
        }
      },
      "minimist": {
        "version": "0.0.8",
        "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
        "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0="
      },
      "minipass": {
        "version": "2.3.5",
        "resolved": "https://registry.npmjs.org/minipass/-/minipass-2.3.5.tgz",
        "integrity": "sha512-Gi1W4k059gyRbyVUZQ4mEqLm0YIUiGYfvxhF6SIlk3ui1WVxMTGfGdQ2SInh3PDrRTVvPKgULkpJtT4RH10+VA==",
        "requires": {
          "safe-buffer": "^5.1.2",
          "yallist": "^3.0.0"
        },
        "dependencies": {
          "yallist": {
            "version": "3.0.3",
            "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.0.3.tgz",
            "integrity": "sha512-S+Zk8DEWE6oKpV+vI3qWkaK+jSbIK86pCwe2IF/xwIpQ8jEuxpw9NyaGjmp9+BoJv5FV2piqCDcoCtStppiq2A=="
          }
        }
      },
      "mixin-deep": {
        "version": "1.3.1",
        "resolved": "https://registry.npmjs.org/mixin-deep/-/mixin-deep-1.3.1.tgz",
        "integrity": "sha512-8ZItLHeEgaqEvd5lYBXfm4EZSFCX29Jb9K+lAHhDKzReKBQKj3R+7NOF6tjqYi9t4oI8VUfaWITJQm86wnXGNQ==",
        "requires": {
          "for-in": "^1.0.2",
          "is-extendable": "^1.0.1"
        },
        "dependencies": {
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          }
        }
      },
      "mkdirp": {
        "version": "0.5.1",
        "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
        "integrity": "sha1-MAV0OOrGz3+MR2fzhkjWaX11yQM=",
        "requires": {
          "minimist": "0.0.8"
        }
      },
      "ms": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
        "integrity": "sha1-VgiurfwAvmwpAd9fmGF4jeDVl8g="
      },
      "nan": {
        "version": "2.13.2",
        "resolved": "https://registry.npmjs.org/nan/-/nan-2.13.2.tgz",
        "integrity": "sha512-TghvYc72wlMGMVMluVo9WRJc0mB8KxxF/gZ4YYFy7V2ZQX9l7rgbPg7vjS9mt6U5HXODVFVI2bOduCzwOMv/lw==",
        "optional": true
      },
      "nanomatch": {
        "version": "1.2.13",
        "resolved": "https://registry.npmjs.org/nanomatch/-/nanomatch-1.2.13.tgz",
        "integrity": "sha512-fpoe2T0RbHwBTBUOftAfBPaDEi06ufaUai0mE6Yn1kacc3SnTErfb/h+X94VXzI64rKFHYImXSvdwGGCmwOqCA==",
        "requires": {
          "arr-diff": "^4.0.0",
          "array-unique": "^0.3.2",
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "fragment-cache": "^0.2.1",
          "is-windows": "^1.0.2",
          "kind-of": "^6.0.2",
          "object.pick": "^1.3.0",
          "regex-not": "^1.0.0",
          "snapdragon": "^0.8.1",
          "to-regex": "^3.0.1"
        },
        "dependencies": {
          "extend-shallow": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
            "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
            "requires": {
              "assign-symbols": "^1.0.0",
              "is-extendable": "^1.0.1"
            }
          },
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          },
          "kind-of": {
            "version": "6.0.2",
            "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.2.tgz",
            "integrity": "sha512-s5kLOcnH0XqDO+FvuaLX8DDjZ18CGFk7VygH40QoKPUQhW4e2rvM0rwUq0t8IQDOwYSeLK01U90OjzBTme2QqA=="
          }
        }
      },
      "neo-async": {
        "version": "2.6.0",
        "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.0.tgz",
        "integrity": "sha512-MFh0d/Wa7vkKO3Y3LlacqAEeHK0mckVqzDieUKTT+KGxi+zIpeVsFxymkIiRpbpDziHc290Xr9A1O4Om7otoRA=="
      },
      "nested-error-stacks": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
        "integrity": "sha512-AO81vsIO1k1sM4Zrd6Hu7regmJN1NSiAja10gc4bX3F0wd+9rQmcuHQaHVQCYIEC8iFXnE+mavh23GOt7wBgug=="
      },
      "nice-try": {
        "version": "1.0.5",
        "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
        "integrity": "sha512-1nh45deeb5olNY7eX82BkPO7SSxR5SSYJiPTrTdFUVYwAl8CKMA5N9PjTYkHiRjisVcxcQ1HXdLhx2qxxJzLNQ=="
      },
      "normalize-package-data": {
        "version": "2.5.0",
        "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
        "integrity": "sha512-/5CMN3T0R4XTj4DcGaexo+roZSdSFW/0AOOTROrjxzCG1wrWXEsGbRKevjlIL+ZDE4sZlJr5ED4YW0yqmkK+eA==",
        "requires": {
          "hosted-git-info": "^2.1.4",
          "resolve": "^1.10.0",
          "semver": "2 || 3 || 4 || 5",
          "validate-npm-package-license": "^3.0.1"
        },
        "dependencies": {
          "semver": {
            "version": "5.7.0",
            "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.0.tgz",
            "integrity": "sha512-Ya52jSX2u7QKghxeoFGpLwCtGlt7j0oY9DYb5apt9nPlJ42ID+ulTXESnt/qAQcoSERyZ5sl3LDIOw0nAn/5DA=="
          }
        }
      },
      "normalize-path": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
        "integrity": "sha512-6eZs5Ls3WtCisHWp9S2GUy8dqkpGi4BVSz3GaqiE6ezub0512ESztXUwUB6C6IKbQkY2Pnb/mD4WYojCRwcwLA=="
      },
      "npm-run-path": {
        "version": "2.0.2",
        "resolved": "https://registry.npmjs.org/npm-run-path/-/npm-run-path-2.0.2.tgz",
        "integrity": "sha1-NakjLfo11wZ7TLLd8jV7GHFTbF8=",
        "requires": {
          "path-key": "^2.0.0"
        }
      },
      "number-is-nan": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
        "integrity": "sha1-CXtgK1NCKlIsGvuHkDGDNpQaAR0="
      },
      "nyc": {
        "version": "14.1.0",
        "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.0.tgz",
        "integrity": "sha512-iy9fEV8Emevz3z/AanIZsoGa8F4U2p0JKevZ/F0sk+/B2r9E6Qn+EPs0bpxEhnAt6UPlTL8mQZIaSJy8sK0ZFw==",
        "requires": {
          "archy": "^1.0.0",
          "caching-transform": "^3.0.2",
          "convert-source-map": "^1.6.0",
          "cp-file": "^6.2.0",
          "find-cache-dir": "^2.1.0",
          "find-up": "^3.0.0",
          "foreground-child": "^1.5.6",
          "glob": "^7.1.3",
          "istanbul-lib-coverage": "^2.0.5",
          "istanbul-lib-hook": "^2.0.7",
          "istanbul-lib-instrument": "^3.3.0",
          "istanbul-lib-report": "^2.0.8",
          "istanbul-lib-source-maps": "^3.0.6",
          "istanbul-reports": "^2.2.4",
          "js-yaml": "^3.13.1",
          "make-dir": "^2.1.0",
          "merge-source-map": "^1.1.0",
          "resolve-from": "^4.0.0",
          "rimraf": "^2.6.3",
          "signal-exit": "^3.0.2",
          "spawn-wrap": "^1.4.2",
          "test-exclude": "^5.2.3",
          "uuid": "^3.3.2",
          "yargs": "^13.2.2",
          "yargs-parser": "^13.0.0"
        },
        "dependencies": {
          "resolve-from": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
            "integrity": "sha512-pb/MYmXstAkysRFx8piNI1tGFNQIFA3vkE3Gq4EuA1dF6gHp/+vgZqsCGJapvy8N3Q+4o7FwvquPJcnZ7RYy4g=="
          }
        }
      },
      "oauth-sign": {
        "version": "0.9.0",
        "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
        "integrity": "sha512-fexhUFFPTGV8ybAtSIGbV6gOkSv8UtRbDBnAyLQw4QPKkgNlsH2ByPGtMUqdWkos6YCRmAqViwgZrJc/mRDzZQ=="
      },
      "object-assign": {
        "version": "4.1.1",
        "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
        "integrity": "sha1-IQmtx5ZYh8/AXLvUQsrIv7s2CGM="
      },
      "object-copy": {
        "version": "0.1.0",
        "resolved": "https://registry.npmjs.org/object-copy/-/object-copy-0.1.0.tgz",
        "integrity": "sha1-fn2Fi3gb18mRpBupde04EnVOmYw=",
        "requires": {
          "copy-descriptor": "^0.1.0",
          "define-property": "^0.2.5",
          "kind-of": "^3.0.3"
        },
        "dependencies": {
          "define-property": {
            "version": "0.2.5",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
            "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
            "requires": {
              "is-descriptor": "^0.1.0"
            }
          },
          "is-accessor-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
            "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-data-descriptor": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
            "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
            "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
            "requires": {
              "is-accessor-descriptor": "^0.1.6",
              "is-data-descriptor": "^0.1.4",
              "kind-of": "^5.0.0"
            },
            "dependencies": {
              "kind-of": {
                "version": "5.1.0",
                "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
                "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw=="
              }
            }
          }
        }
      },
      "object-visit": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/object-visit/-/object-visit-1.0.1.tgz",
        "integrity": "sha1-95xEk68MU3e1n+OdOV5BBC3QRbs=",
        "requires": {
          "isobject": "^3.0.0"
        }
      },
      "object.pick": {
        "version": "1.3.0",
        "resolved": "https://registry.npmjs.org/object.pick/-/object.pick-1.3.0.tgz",
        "integrity": "sha1-h6EKxMFpS9Lhy/U1kaZhQftd10c=",
        "requires": {
          "isobject": "^3.0.1"
        }
      },
      "once": {
        "version": "1.4.0",
        "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
        "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
        "requires": {
          "wrappy": "1"
        }
      },
      "onetime": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/onetime/-/onetime-2.0.1.tgz",
        "integrity": "sha1-BnQoIw/WdEOyeUsiu6UotoZ5YtQ=",
        "requires": {
          "mimic-fn": "^1.0.0"
        }
      },
      "opener": {
        "version": "1.5.1",
        "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
        "integrity": "sha512-goYSy5c2UXE4Ra1xixabeVh1guIX/ZV/YokJksb6q2lubWu6UbvPQ20p542/sFIll1nl8JnCyK9oBaOcCWXwvA=="
      },
      "optimist": {
        "version": "0.6.1",
        "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
        "integrity": "sha1-2j6nRob6IaGaERwybpDrFaAZZoY=",
        "requires": {
          "minimist": "~0.0.1",
          "wordwrap": "~0.0.2"
        }
      },
      "os-homedir": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
        "integrity": "sha1-/7xJiDNuDoM94MFox+8VISGqf7M="
      },
      "os-locale": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/os-locale/-/os-locale-3.1.0.tgz",
        "integrity": "sha512-Z8l3R4wYWM40/52Z+S265okfFj8Kt2cC2MKY+xNi3kFs+XGI7WXu/I309QQQYbRW4ijiZ+yxs9pqEhJh0DqW3Q==",
        "requires": {
          "execa": "^1.0.0",
          "lcid": "^2.0.0",
          "mem": "^4.0.0"
        }
      },
      "os-tmpdir": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
        "integrity": "sha1-u+Z0BseaqFxc/sdm/lc0VV36EnQ="
      },
      "own-or": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
        "integrity": "sha1-Tod/vtqaLsgAD7wLyuOWRe6L+Nw="
      },
      "own-or-env": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
        "integrity": "sha512-y8qULRbRAlL6x2+M0vIe7jJbJx/kmUTzYonRAa2ayesR2qWLswninkVyeJe4x3IEXhdgoNodzjQRKAoEs6Fmrw==",
        "requires": {
          "own-or": "^1.0.0"
        }
      },
      "p-defer": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/p-defer/-/p-defer-1.0.0.tgz",
        "integrity": "sha1-n26xgvbJqozXQwBKfU+WsZaw+ww="
      },
      "p-finally": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/p-finally/-/p-finally-1.0.0.tgz",
        "integrity": "sha1-P7z7FbiZpEEjs0ttzBi3JDNqLK4="
      },
      "p-is-promise": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/p-is-promise/-/p-is-promise-2.1.0.tgz",
        "integrity": "sha512-Y3W0wlRPK8ZMRbNq97l4M5otioeA5lm1z7bkNkxCka8HSPjR0xRWmpCmc9utiaLP9Jb1eD8BgeIxTW4AIF45Pg=="
      },
      "p-limit": {
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.0.tgz",
        "integrity": "sha512-pZbTJpoUsCzV48Mc9Nh51VbwO0X9cuPFE8gYwx9BTCt9SF8/b7Zljd2fVgOxhIF/HDTKgpVzs+GPhyKfjLLFRQ==",
        "requires": {
          "p-try": "^2.0.0"
        }
      },
      "p-locate": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
        "integrity": "sha512-x+12w/To+4GFfgJhBEpiDcLozRJGegY+Ei7/z0tSLkMmxGZNybVMSfWj9aJn8Z5Fc7dBUNJOOVgPv2H7IwulSQ==",
        "requires": {
          "p-limit": "^2.0.0"
        }
      },
      "p-try": {
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
        "integrity": "sha512-R4nPAVTAU0B9D35/Gk3uJf/7XYbQcyohSKdvAxIRSNghFl4e71hVoGnBNQz9cWaXxO2I10KTC+3jMdvvoKw6dQ=="
      },
      "package-hash": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
        "integrity": "sha512-lOtmukMDVvtkL84rJHI7dpTYq+0rli8N2wlnqUcBuDWCfVhRUfOmnR9SsoHFMLpACvEV60dX7rd0rFaYDZI+FA==",
        "requires": {
          "graceful-fs": "^4.1.15",
          "hasha": "^3.0.0",
          "lodash.flattendeep": "^4.4.0",
          "release-zalgo": "^1.0.0"
        }
      },
      "parse-json": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
        "integrity": "sha1-vjX1Qlvh9/bHRxhPmKeIy5lHfuA=",
        "requires": {
          "error-ex": "^1.3.1",
          "json-parse-better-errors": "^1.0.1"
        }
      },
      "pascalcase": {
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/pascalcase/-/pascalcase-0.1.1.tgz",
        "integrity": "sha1-s2PlXoAGym/iF4TS2yK9FdeRfxQ="
      },
      "path-dirname": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/path-dirname/-/path-dirname-1.0.2.tgz",
        "integrity": "sha1-zDPSTVJeCZpTiMAzbG4yuRYGCeA="
      },
      "path-exists": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
        "integrity": "sha1-zg6+ql94yxiSXqfYENe1mwEP1RU="
      },
      "path-is-absolute": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
        "integrity": "sha1-F0uSaHNVNP+8es5r9TpanhtcX18="
      },
      "path-key": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
        "integrity": "sha1-QRyttXTFoUDTpLGRDUDYDMn0C0A="
      },
      "path-parse": {
        "version": "1.0.6",
        "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
        "integrity": "sha512-GSmOT2EbHrINBf9SR7CDELwlJ8AENk3Qn7OikK4nFYAu3Ote2+JYNVvkpAEQm3/TLNEJFD/xZJjzyxg3KBWOzw=="
      },
      "path-type": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
        "integrity": "sha512-T2ZUsdZFHgA3u4e5PfPbjd7HDDpxPnQb5jN0SrDsjNSuVXHJqtwTnWqG0B1jZrgmJ/7lj1EmVIByWt1gxGkWvg==",
        "requires": {
          "pify": "^3.0.0"
        },
        "dependencies": {
          "pify": {
            "version": "3.0.0",
            "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
            "integrity": "sha1-5aSs0sEB/fPZpNB/DbxNtJ3SgXY="
          }
        }
      },
      "performance-now": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
        "integrity": "sha1-Ywn04OX6kT7BxpMHrjZLSzd8nns="
      },
      "pify": {
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
        "integrity": "sha512-uB80kBFb/tfd68bVleG9T5GGsGPjJrLAUpR5PZIrhBnIaRTQRjqdJSsIKkOP6OAIFbj7GOrcudc5pNjZ+geV2g=="
      },
      "pkg-dir": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
        "integrity": "sha512-/E57AYkoeQ25qkxMj5PBOVgF8Kiu/h7cYS30Z5+R7WaiCCBfLq58ZI/dSeaEKb9WVJV5n/03QwrN3IeWIFllvw==",
        "requires": {
          "find-up": "^3.0.0"
        }
      },
      "posix-character-classes": {
        "version": "0.1.1",
        "resolved": "https://registry.npmjs.org/posix-character-classes/-/posix-character-classes-0.1.1.tgz",
        "integrity": "sha1-AerA/jta9xoqbAL+q7jB/vfgDqs="
      },
      "private": {
        "version": "0.1.8",
        "resolved": "https://registry.npmjs.org/private/-/private-0.1.8.tgz",
        "integrity": "sha512-VvivMrbvd2nKkiG38qjULzlc+4Vx4wm/whI9pQD35YrARNnhxeiRktSOhSukRLFNlzg6Br/cJPet5J/u19r/mg=="
      },
      "process-nextick-args": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.0.tgz",
        "integrity": "sha512-MtEC1TqN0EU5nephaJ4rAtThHtC86dNN9qCuEhtshvpVBkAW5ZO7BASN9REnF9eoXGcRub+pFuKEpOHE+HbEMw=="
      },
      "prop-types": {
        "version": "15.7.2",
        "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
        "integrity": "sha512-8QQikdH7//R2vurIJSutZ1smHYTcLpRWEOlHnzcWHmBYrOGUysKwSsrC89BCiFj3CbrfJ/nXFdJepOVrY1GCHQ==",
        "requires": {
          "loose-envify": "^1.4.0",
          "object-assign": "^4.1.1",
          "react-is": "^16.8.1"
        }
      },
      "pseudomap": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
        "integrity": "sha1-8FKijacOYYkX7wqKw0wa5aaChrM="
      },
      "psl": {
        "version": "1.1.31",
        "resolved": "https://registry.npmjs.org/psl/-/psl-1.1.31.tgz",
        "integrity": "sha512-/6pt4+C+T+wZUieKR620OpzN/LlnNKuWjy1iFLQ/UG35JqHlR/89MP1d96dUfkf6Dne3TuLQzOYEYshJ+Hx8mw=="
      },
      "pump": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/pump/-/pump-3.0.0.tgz",
        "integrity": "sha512-LwZy+p3SFs1Pytd/jYct4wpv49HiYCqd9Rlc5ZVdk0V+8Yzv6jR5Blk3TRmPL1ft69TxP0IMZGJ+WPFU2BFhww==",
        "requires": {
          "end-of-stream": "^1.1.0",
          "once": "^1.3.1"
        }
      },
      "punycode": {
        "version": "1.4.1",
        "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
        "integrity": "sha1-wNWmOycYgArY4esPpSachN1BhF4="
      },
      "qs": {
        "version": "6.5.2",
        "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
        "integrity": "sha512-N5ZAX4/LxJmF+7wN74pUD6qAh9/wnvdQcjq9TZjevvXzSUo7bfmw91saqMjzGS2xq91/odN2dW/WOl7qQHNDGA=="
      },
      "react": {
        "version": "16.8.6",
        "resolved": "https://registry.npmjs.org/react/-/react-16.8.6.tgz",
        "integrity": "sha512-pC0uMkhLaHm11ZSJULfOBqV4tIZkx87ZLvbbQYunNixAAvjnC+snJCg0XQXn9VIsttVsbZP/H/ewzgsd5fxKXw==",
        "requires": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1",
          "prop-types": "^15.6.2",
          "scheduler": "^0.13.6"
        }
      },
      "react-is": {
        "version": "16.8.6",
        "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.8.6.tgz",
        "integrity": "sha512-aUk3bHfZ2bRSVFFbbeVS4i+lNPZr3/WM5jT2J5omUVV1zzcs1nAaf3l51ctA5FFvCRbhrH0bdAsRRQddFJZPtA=="
      },
      "react-reconciler": {
        "version": "0.20.4",
        "resolved": "https://registry.npmjs.org/react-reconciler/-/react-reconciler-0.20.4.tgz",
        "integrity": "sha512-kxERc4H32zV2lXMg/iMiwQHOtyqf15qojvkcZ5Ja2CPkjVohHw9k70pdDBwrnQhLVetUJBSYyqU3yqrlVTOajA==",
        "requires": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1",
          "prop-types": "^15.6.2",
          "react": "^16.0.0",
          "scheduler": "^0.13.6"
        }
      },
      "read-pkg": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
        "integrity": "sha1-nLxoaXj+5l0WwA4rGcI3/Pbjg4k=",
        "requires": {
          "load-json-file": "^4.0.0",
          "normalize-package-data": "^2.3.2",
          "path-type": "^3.0.0"
        }
      },
      "read-pkg-up": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
        "integrity": "sha512-6etQSH7nJGsK0RbG/2TeDzZFa8shjQ1um+SwQQ5cwKy0dhSXdOncEhb1CPpvQG4h7FyOV6EB6YlV0yJvZQNAkA==",
        "requires": {
          "find-up": "^3.0.0",
          "read-pkg": "^3.0.0"
        }
      },
      "readable-stream": {
        "version": "2.3.6",
        "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
        "integrity": "sha512-tQtKA9WIAhBF3+VLAseyMqZeBjW0AHJoxOtYqSUZNJxauErmLbVm2FW1y+J/YA9dUrAC39ITejlZWhVIwawkKw==",
        "requires": {
          "core-util-is": "~1.0.0",
          "inherits": "~2.0.3",
          "isarray": "~1.0.0",
          "process-nextick-args": "~2.0.0",
          "safe-buffer": "~5.1.1",
          "string_decoder": "~1.1.1",
          "util-deprecate": "~1.0.1"
        }
      },
      "readdirp": {
        "version": "2.2.1",
        "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-2.2.1.tgz",
        "integrity": "sha512-1JU/8q+VgFZyxwrJ+SVIOsh+KywWGpds3NTqikiKpDMZWScmAYyKIgqkO+ARvNWJfXeXR1zxz7aHF4u4CyH6vQ==",
        "requires": {
          "graceful-fs": "^4.1.11",
          "micromatch": "^3.1.10",
          "readable-stream": "^2.0.2"
        }
      },
      "redeyed": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/redeyed/-/redeyed-2.1.1.tgz",
        "integrity": "sha1-iYS1gV2ZyyIEacme7v/jiRPmzAs=",
        "requires": {
          "esprima": "~4.0.0"
        }
      },
      "regenerator-runtime": {
        "version": "0.11.1",
        "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.11.1.tgz",
        "integrity": "sha512-MguG95oij0fC3QV3URf4V2SDYGJhJnJGqvIIgdECeODCT98wSWDAJ94SSuVpYQUoTcGUIL6L4yNB7j1DFFHSBg=="
      },
      "regex-not": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/regex-not/-/regex-not-1.0.2.tgz",
        "integrity": "sha512-J6SDjUgDxQj5NusnOtdFxDwN/+HWykR8GELwctJ7mdqhcyy1xEc4SRFHUXvxTp661YaVKAjfRLZ9cCqS6tn32A==",
        "requires": {
          "extend-shallow": "^3.0.2",
          "safe-regex": "^1.1.0"
        },
        "dependencies": {
          "extend-shallow": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
            "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
            "requires": {
              "assign-symbols": "^1.0.0",
              "is-extendable": "^1.0.1"
            }
          },
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          }
        }
      },
      "release-zalgo": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
        "integrity": "sha1-CXALflB0Mpc5Mw5TXFqQ+2eFFzA=",
        "requires": {
          "es6-error": "^4.0.1"
        }
      },
      "remove-trailing-separator": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/remove-trailing-separator/-/remove-trailing-separator-1.1.0.tgz",
        "integrity": "sha1-wkvOKig62tW8P1jg1IJJuSN52O8="
      },
      "repeat-element": {
        "version": "1.1.3",
        "resolved": "https://registry.npmjs.org/repeat-element/-/repeat-element-1.1.3.tgz",
        "integrity": "sha512-ahGq0ZnV5m5XtZLMb+vP76kcAM5nkLqk0lpqAuojSKGgQtn4eRi4ZZGm2olo2zKFH+sMsWaqOCW1dqAnOru72g=="
      },
      "repeat-string": {
        "version": "1.6.1",
        "resolved": "https://registry.npmjs.org/repeat-string/-/repeat-string-1.6.1.tgz",
        "integrity": "sha1-jcrkcOHIirwtYA//Sndihtp15jc="
      },
      "repeating": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/repeating/-/repeating-2.0.1.tgz",
        "integrity": "sha1-UhTFOpJtNVJwdSf7q0FdvAjQbdo=",
        "requires": {
          "is-finite": "^1.0.0"
        }
      },
      "request": {
        "version": "2.88.0",
        "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
        "integrity": "sha512-NAqBSrijGLZdM0WZNsInLJpkJokL72XYjUpnB0iwsRgxh7dB6COrHnTBNwN0E+lHDAJzu7kLAkDeY08z2/A0hg==",
        "requires": {
          "aws-sign2": "~0.7.0",
          "aws4": "^1.8.0",
          "caseless": "~0.12.0",
          "combined-stream": "~1.0.6",
          "extend": "~3.0.2",
          "forever-agent": "~0.6.1",
          "form-data": "~2.3.2",
          "har-validator": "~5.1.0",
          "http-signature": "~1.2.0",
          "is-typedarray": "~1.0.0",
          "isstream": "~0.1.2",
          "json-stringify-safe": "~5.0.1",
          "mime-types": "~2.1.19",
          "oauth-sign": "~0.9.0",
          "performance-now": "^2.1.0",
          "qs": "~6.5.2",
          "safe-buffer": "^5.1.2",
          "tough-cookie": "~2.4.3",
          "tunnel-agent": "^0.6.0",
          "uuid": "^3.3.2"
        }
      },
      "require-directory": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
        "integrity": "sha1-jGStX9MNqxyXbiNE/+f3kqam30I="
      },
      "require-main-filename": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
        "integrity": "sha512-NKN5kMDylKuldxYLSUfrbo5Tuzh4hd+2E8NPPX02mZtn1VuREQToYe/ZdlJy+J3uCpfaiGF05e7B8W0iXbQHmg=="
      },
      "resolve": {
        "version": "1.10.1",
        "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.10.1.tgz",
        "integrity": "sha512-KuIe4mf++td/eFb6wkaPbMDnP6kObCaEtIDuHOUED6MNUo4K670KZUHuuvYPZDxNF0WVLw49n06M2m2dXphEzA==",
        "requires": {
          "path-parse": "^1.0.6"
        }
      },
      "resolve-from": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-3.0.0.tgz",
        "integrity": "sha1-six699nWiBvItuZTM17rywoYh0g="
      },
      "resolve-url": {
        "version": "0.2.1",
        "resolved": "https://registry.npmjs.org/resolve-url/-/resolve-url-0.2.1.tgz",
        "integrity": "sha1-LGN/53yJOv0qZj/iGqkIAGjiBSo="
      },
      "restore-cursor": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-2.0.0.tgz",
        "integrity": "sha1-n37ih/gv0ybU/RYpI9YhKe7g368=",
        "requires": {
          "onetime": "^2.0.0",
          "signal-exit": "^3.0.2"
        }
      },
      "ret": {
        "version": "0.1.15",
        "resolved": "https://registry.npmjs.org/ret/-/ret-0.1.15.tgz",
        "integrity": "sha512-TTlYpa+OL+vMMNG24xSlQGEJ3B/RzEfUlLct7b5G/ytav+wPrplCpVMFuwzXbkecJrb6IYo1iFb0S9v37754mg=="
      },
      "rimraf": {
        "version": "2.6.3",
        "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.6.3.tgz",
        "integrity": "sha512-mwqeW5XsA2qAejG46gYdENaxXjx9onRNCfn7L0duuP4hCuTIi/QO7PDK07KJfp1d+izWPrzEJDcSqBa0OZQriA==",
        "requires": {
          "glob": "^7.1.3"
        }
      },
      "safe-buffer": {
        "version": "5.1.2",
        "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
        "integrity": "sha512-Gd2UZBJDkXlY7GbJxfsE8/nvKkUEU1G38c1siN6QP6a9PT9MmHB8GnpscSmMJSoF8LOIrt8ud/wPtojys4G6+g=="
      },
      "safe-regex": {
        "version": "1.1.0",
        "resolved": "https://registry.npmjs.org/safe-regex/-/safe-regex-1.1.0.tgz",
        "integrity": "sha1-QKNmnzsHfR6UPURinhV91IAjvy4=",
        "requires": {
          "ret": "~0.1.10"
        }
      },
      "safer-buffer": {
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
        "integrity": "sha512-YZo3K82SD7Riyi0E1EQPojLz7kpepnSQI9IyPbHHg1XXXevb5dJI7tpyN2ADxGcQbHG7vcyRHk0cbwqcQriUtg=="
      },
      "scheduler": {
        "version": "0.13.6",
        "resolved": "https://registry.npmjs.org/scheduler/-/scheduler-0.13.6.tgz",
        "integrity": "sha512-IWnObHt413ucAYKsD9J1QShUKkbKLQQHdxRyw73sw4FN26iWr3DY/H34xGPe4nmL1DwXyWmSWmMrA9TfQbE/XQ==",
        "requires": {
          "loose-envify": "^1.1.0",
          "object-assign": "^4.1.1"
        }
      },
      "semver": {
        "version": "6.0.0",
        "resolved": "https://registry.npmjs.org/semver/-/semver-6.0.0.tgz",
        "integrity": "sha512-0UewU+9rFapKFnlbirLi3byoOuhrSsli/z/ihNnvM24vgF+8sNBiI1LZPBSH9wJKUwaUbw+s3hToDLCXkrghrQ=="
      },
      "set-blocking": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
        "integrity": "sha1-BF+XgtARrppoA93TgrJDkrPYkPc="
      },
      "set-value": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/set-value/-/set-value-2.0.0.tgz",
        "integrity": "sha512-hw0yxk9GT/Hr5yJEYnHNKYXkIA8mVJgd9ditYZCe16ZczcaELYYcfvaXesNACk2O8O0nTiPQcQhGUQj8JLzeeg==",
        "requires": {
          "extend-shallow": "^2.0.1",
          "is-extendable": "^0.1.1",
          "is-plain-object": "^2.0.3",
          "split-string": "^3.0.1"
        }
      },
      "shebang-command": {
        "version": "1.2.0",
        "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
        "integrity": "sha1-RKrGW2lbAzmJaMOfNj/uXer98eo=",
        "requires": {
          "shebang-regex": "^1.0.0"
        }
      },
      "shebang-regex": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
        "integrity": "sha1-2kL0l0DAtC2yypcoVxyxkMmO/qM="
      },
      "signal-exit": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
        "integrity": "sha1-tf3AjxKH6hF4Yo5BXiUTK3NkbG0="
      },
      "slash": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/slash/-/slash-1.0.0.tgz",
        "integrity": "sha1-xB8vbDn8FtHNF61LXYlhFK5HDVU="
      },
      "slice-ansi": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-1.0.0.tgz",
        "integrity": "sha512-POqxBK6Lb3q6s047D/XsDVNPnF9Dl8JSaqe9h9lURl0OdNqy/ujDrOiIHtsqXMGbWWTIomRzAMaTyawAU//Reg==",
        "requires": {
          "is-fullwidth-code-point": "^2.0.0"
        }
      },
      "snapdragon": {
        "version": "0.8.2",
        "resolved": "https://registry.npmjs.org/snapdragon/-/snapdragon-0.8.2.tgz",
        "integrity": "sha512-FtyOnWN/wCHTVXOMwvSv26d+ko5vWlIDD6zoUJ7LW8vh+ZBC8QdljveRP+crNrtBwioEUWy/4dMtbBjA4ioNlg==",
        "requires": {
          "base": "^0.11.1",
          "debug": "^2.2.0",
          "define-property": "^0.2.5",
          "extend-shallow": "^2.0.1",
          "map-cache": "^0.2.2",
          "source-map": "^0.5.6",
          "source-map-resolve": "^0.5.0",
          "use": "^3.1.0"
        },
        "dependencies": {
          "define-property": {
            "version": "0.2.5",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
            "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
            "requires": {
              "is-descriptor": "^0.1.0"
            }
          },
          "is-accessor-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
            "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-data-descriptor": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
            "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
            "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
            "requires": {
              "is-accessor-descriptor": "^0.1.6",
              "is-data-descriptor": "^0.1.4",
              "kind-of": "^5.0.0"
            },
            "dependencies": {
              "kind-of": {
                "version": "5.1.0",
                "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
                "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw=="
              }
            }
          }
        }
      },
      "snapdragon-node": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/snapdragon-node/-/snapdragon-node-2.1.1.tgz",
        "integrity": "sha512-O27l4xaMYt/RSQ5TR3vpWCAB5Kb/czIcqUFOM/C4fYcLnbZUc1PkjTAMjof2pBWaSTwOUd6qUHcFGVGj7aIwnw==",
        "requires": {
          "define-property": "^1.0.0",
          "isobject": "^3.0.0",
          "snapdragon-util": "^3.0.1"
        },
        "dependencies": {
          "define-property": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-1.0.0.tgz",
            "integrity": "sha1-dp66rz9KY6rTr56NMEybvnm/sOY=",
            "requires": {
              "is-descriptor": "^1.0.0"
            }
          }
        }
      },
      "snapdragon-util": {
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/snapdragon-util/-/snapdragon-util-3.0.1.tgz",
        "integrity": "sha512-mbKkMdQKsjX4BAL4bRYTj21edOf8cN7XHdYUJEe+Zn99hVEYcMvKPct1IqNe7+AZPirn8BCDOQBHQZknqmKlZQ==",
        "requires": {
          "kind-of": "^3.2.0"
        }
      },
      "source-map": {
        "version": "0.5.7",
        "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
        "integrity": "sha1-igOdLRAh0i0eoUyA2OpGi6LvP8w="
      },
      "source-map-resolve": {
        "version": "0.5.2",
        "resolved": "https://registry.npmjs.org/source-map-resolve/-/source-map-resolve-0.5.2.tgz",
        "integrity": "sha512-MjqsvNwyz1s0k81Goz/9vRBe9SZdB09Bdw+/zYyO+3CuPk6fouTaxscHkgtE8jKvf01kVfl8riHzERQ/kefaSA==",
        "requires": {
          "atob": "^2.1.1",
          "decode-uri-component": "^0.2.0",
          "resolve-url": "^0.2.1",
          "source-map-url": "^0.4.0",
          "urix": "^0.1.0"
        }
      },
      "source-map-support": {
        "version": "0.5.12",
        "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.12.tgz",
        "integrity": "sha512-4h2Pbvyy15EE02G+JOZpUCmqWJuqrs+sEkzewTm++BPi7Hvn/HwcqLAcNxYAyI0x13CpPPn+kMjl+hplXMHITQ==",
        "requires": {
          "buffer-from": "^1.0.0",
          "source-map": "^0.6.0"
        },
        "dependencies": {
          "source-map": {
            "version": "0.6.1",
            "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
            "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g=="
          }
        }
      },
      "source-map-url": {
        "version": "0.4.0",
        "resolved": "https://registry.npmjs.org/source-map-url/-/source-map-url-0.4.0.tgz",
        "integrity": "sha1-PpNdfd1zYxuXZZlW1VEo6HtQhKM="
      },
      "spawn-wrap": {
        "version": "1.4.2",
        "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.2.tgz",
        "integrity": "sha512-vMwR3OmmDhnxCVxM8M+xO/FtIp6Ju/mNaDfCMMW7FDcLRTPFWUswec4LXJHTJE2hwTI9O0YBfygu4DalFl7Ylg==",
        "requires": {
          "foreground-child": "^1.5.6",
          "mkdirp": "^0.5.0",
          "os-homedir": "^1.0.1",
          "rimraf": "^2.6.2",
          "signal-exit": "^3.0.2",
          "which": "^1.3.0"
        }
      },
      "spdx-correct": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
        "integrity": "sha512-lr2EZCctC2BNR7j7WzJ2FpDznxky1sjfxvvYEyzxNyb6lZXHODmEoJeFu4JupYlkfha1KZpJyoqiJ7pgA1qq8Q==",
        "requires": {
          "spdx-expression-parse": "^3.0.0",
          "spdx-license-ids": "^3.0.0"
        }
      },
      "spdx-exceptions": {
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
        "integrity": "sha512-2XQACfElKi9SlVb1CYadKDXvoajPgBVPn/gOQLrTvHdElaVhr7ZEbqJaRnJLVNeaI4cMEAgVCeBMKF6MWRDCRA=="
      },
      "spdx-expression-parse": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
        "integrity": "sha512-Yg6D3XpRD4kkOmTpdgbUiEJFKghJH03fiC1OPll5h/0sO6neh2jqRDVHOQ4o/LMea0tgCkbMgea5ip/e+MkWyg==",
        "requires": {
          "spdx-exceptions": "^2.1.0",
          "spdx-license-ids": "^3.0.0"
        }
      },
      "spdx-license-ids": {
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.4.tgz",
        "integrity": "sha512-7j8LYJLeY/Yb6ACbQ7F76qy5jHkp0U6jgBfJsk97bwWlVUnUWsAgpyaCvo17h0/RQGnQ036tVDomiwoI4pDkQA=="
      },
      "split-string": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/split-string/-/split-string-3.1.0.tgz",
        "integrity": "sha512-NzNVhJDYpwceVVii8/Hu6DKfD2G+NrQHlS/V/qgv763EYudVwEcMQNxd2lh+0VrUByXN/oJkl5grOhYWvQUYiw==",
        "requires": {
          "extend-shallow": "^3.0.0"
        },
        "dependencies": {
          "extend-shallow": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
            "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
            "requires": {
              "assign-symbols": "^1.0.0",
              "is-extendable": "^1.0.1"
            }
          },
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          }
        }
      },
      "sprintf-js": {
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
        "integrity": "sha1-BOaSb2YolTVPPdAVIDYzuFcpfiw="
      },
      "sshpk": {
        "version": "1.16.1",
        "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
        "integrity": "sha512-HXXqVUq7+pcKeLqqZj6mHFUMvXtOJt1uoUx09pFW6011inTMxqI8BA8PM95myrIyyKwdnzjdFjLiE6KBPVtJIg==",
        "requires": {
          "asn1": "~0.2.3",
          "assert-plus": "^1.0.0",
          "bcrypt-pbkdf": "^1.0.0",
          "dashdash": "^1.12.0",
          "ecc-jsbn": "~0.1.1",
          "getpass": "^0.1.1",
          "jsbn": "~0.1.0",
          "safer-buffer": "^2.0.2",
          "tweetnacl": "~0.14.0"
        }
      },
      "stack-utils": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
        "integrity": "sha512-MTX+MeG5U994cazkjd/9KNAapsHnibjMLnfXodlkXw76JEea0UiNzrqidzo1emMwk7w5Qhc9jd4Bn9TBb1MFwA=="
      },
      "static-extend": {
        "version": "0.1.2",
        "resolved": "https://registry.npmjs.org/static-extend/-/static-extend-0.1.2.tgz",
        "integrity": "sha1-YICcOcv/VTNyJv1eC1IPNB8ftcY=",
        "requires": {
          "define-property": "^0.2.5",
          "object-copy": "^0.1.0"
        },
        "dependencies": {
          "define-property": {
            "version": "0.2.5",
            "resolved": "https://registry.npmjs.org/define-property/-/define-property-0.2.5.tgz",
            "integrity": "sha1-w1se+RjsPJkPmlvFe+BKrOxcgRY=",
            "requires": {
              "is-descriptor": "^0.1.0"
            }
          },
          "is-accessor-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-accessor-descriptor/-/is-accessor-descriptor-0.1.6.tgz",
            "integrity": "sha1-qeEss66Nh2cn7u84Q/igiXtcmNY=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-data-descriptor": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/is-data-descriptor/-/is-data-descriptor-0.1.4.tgz",
            "integrity": "sha1-C17mSDiOLIYCgueT8YVv7D8wG1Y=",
            "requires": {
              "kind-of": "^3.0.2"
            }
          },
          "is-descriptor": {
            "version": "0.1.6",
            "resolved": "https://registry.npmjs.org/is-descriptor/-/is-descriptor-0.1.6.tgz",
            "integrity": "sha512-avDYr0SB3DwO9zsMov0gKCESFYqCnE4hq/4z3TdUlukEy5t9C0YRq7HLrsN52NAcqXKaepeCD0n+B0arnVG3Hg==",
            "requires": {
              "is-accessor-descriptor": "^0.1.6",
              "is-data-descriptor": "^0.1.4",
              "kind-of": "^5.0.0"
            },
            "dependencies": {
              "kind-of": {
                "version": "5.1.0",
                "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-5.1.0.tgz",
                "integrity": "sha512-NGEErnH6F2vUuXDh+OlbcKW7/wOcfdRHaZ7VWtqCztfHri/++YKmP51OdWeGPuqCOba6kk2OTe5d02VmTB80Pw=="
              }
            }
          }
        }
      },
      "string_decoder": {
        "version": "1.1.1",
        "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
        "integrity": "sha512-n/ShnvDi6FHbbVfviro+WojiFzv+s8MPMHBczVePfUpDJLwoLT0ht1l4YwBCbi8pJAveEEdnkHyPyTP/mzRfwg==",
        "requires": {
          "safe-buffer": "~5.1.0"
        }
      },
      "string-length": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/string-length/-/string-length-2.0.0.tgz",
        "integrity": "sha1-1A27aGo6zpYMHP/KVivyxF+DY+0=",
        "requires": {
          "astral-regex": "^1.0.0",
          "strip-ansi": "^4.0.0"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "3.0.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
            "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg="
          },
          "strip-ansi": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
            "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
            "requires": {
              "ansi-regex": "^3.0.0"
            }
          }
        }
      },
      "string-width": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
        "integrity": "sha512-nOqH59deCq9SRHlxq1Aw85Jnt4w6KvLKqWVik6oA9ZklXLNIOlqg4F2yrT1MVaTjAqvVwdfeZ7w7aCvJD7ugkw==",
        "requires": {
          "is-fullwidth-code-point": "^2.0.0",
          "strip-ansi": "^4.0.0"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "3.0.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
            "integrity": "sha1-7QMXwyIGT3lGbAKWa922Bas32Zg="
          },
          "strip-ansi": {
            "version": "4.0.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
            "integrity": "sha1-qEeQIusaw2iocTibY1JixQXuNo8=",
            "requires": {
              "ansi-regex": "^3.0.0"
            }
          }
        }
      },
      "strip-ansi": {
        "version": "3.0.1",
        "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
        "integrity": "sha1-ajhfuIU9lS1f8F0Oiq+UJ43GPc8=",
        "requires": {
          "ansi-regex": "^2.0.0"
        }
      },
      "strip-bom": {
        "version": "3.0.0",
        "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
        "integrity": "sha1-IzTBjpx1n3vdVv3vfprj1YjmjtM="
      },
      "strip-eof": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/strip-eof/-/strip-eof-1.0.0.tgz",
        "integrity": "sha1-u0P/VZim6wXYm1n80SnJgzE2Br8="
      },
      "supports-color": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
        "integrity": "sha1-U10EXOa2Nj+kARcIRimZXp3zJMc="
      },
      "tap": {
        "version": "13.0.3",
        "resolved": "https://registry.npmjs.org/tap/-/tap-13.0.3.tgz",
        "integrity": "sha512-G9S2OUsE97gdHk9iRGZkPKBh/+to7P1Tqi5kTUtKDrVA2sE2cmUMpmp0izlVtqxMRTVjiVmhLVpMdHitwoHEHQ==",
        "requires": {
          "async-hook-domain": "^1.1.0",
          "bind-obj-methods": "^2.0.0",
          "browser-process-hrtime": "^1.0.0",
          "capture-stack-trace": "^1.0.0",
          "chokidar": "^2.1.5",
          "color-support": "^1.1.0",
          "coveralls": "^3.0.3",
          "domain-browser": "^1.2.0",
          "esm": "^3.2.22",
          "findit": "^2.0.0",
          "foreground-child": "^1.3.3",
          "fs-exists-cached": "^1.0.0",
          "function-loop": "^1.0.2",
          "glob": "^7.1.3",
          "import-jsx": "^2.0.0",
          "isexe": "^2.0.0",
          "istanbul-lib-processinfo": "^1.0.0",
          "jackspeak": "^1.3.7",
          "minipass": "^2.3.5",
          "mkdirp": "^0.5.1",
          "nyc": "^14.1.0",
          "opener": "^1.5.1",
          "own-or": "^1.0.0",
          "own-or-env": "^1.0.1",
          "rimraf": "^2.6.3",
          "signal-exit": "^3.0.0",
          "source-map-support": "^0.5.12",
          "stack-utils": "^1.0.2",
          "tap-mocha-reporter": "^4.0.1",
          "tap-parser": "^9.3.2",
          "tap-yaml": "^1.0.0",
          "tcompare": "^2.2.0",
          "treport": "^0.3.0",
          "trivial-deferred": "^1.0.1",
          "ts-node": "^8.1.0",
          "typescript": "^3.4.3",
          "which": "^1.3.1",
          "write-file-atomic": "^2.4.2",
          "yaml": "^1.5.0",
          "yapool": "^1.0.0"
        }
      },
      "tap-mocha-reporter": {
        "version": "4.0.1",
        "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-4.0.1.tgz",
        "integrity": "sha512-/KfXaaYeSPn8qBi5Be8WSIP3iKV83s2uj2vzImJAXmjNu22kzqZ+1Dv1riYWa53sPCiyo1R1w1jbJrftF8SpcQ==",
        "requires": {
          "color-support": "^1.1.0",
          "debug": "^2.1.3",
          "diff": "^1.3.2",
          "escape-string-regexp": "^1.0.3",
          "glob": "^7.0.5",
          "readable-stream": "^2.1.5",
          "tap-parser": "^8.0.0",
          "tap-yaml": "0 || 1",
          "unicode-length": "^1.0.0"
        },
        "dependencies": {
          "tap-parser": {
            "version": "8.1.0",
            "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-8.1.0.tgz",
            "integrity": "sha512-GgOzgDwThYLxhVR83RbS1JUR1TxcT+jfZsrETgPAvFdr12lUOnuvrHOBaUQgpkAp6ZyeW6r2Nwd91t88M0ru3w==",
            "requires": {
              "events-to-array": "^1.0.1",
              "minipass": "^2.2.0",
              "tap-yaml": "0 || 1"
            }
          }
        }
      },
      "tap-parser": {
        "version": "9.3.2",
        "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-9.3.2.tgz",
        "integrity": "sha512-bQ76sD6ZP9loxdk/KiqXgWDEfjJYiZUj0a7ElnLMSny7Q8G72UMcOQee85j5ddKHA9fzGAoL6cfJbg3rur7S5g==",
        "requires": {
          "events-to-array": "^1.0.1",
          "minipass": "^2.2.0",
          "tap-yaml": "^1.0.0"
        }
      },
      "tap-yaml": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
        "integrity": "sha512-Rxbx4EnrWkYk0/ztcm5u3/VznbyFJpyXO12dDBHKWiDVxy7O2Qw6MRrwO5H6Ww0U5YhRY/4C/VzWmFPhBQc4qQ==",
        "requires": {
          "yaml": "^1.5.0"
        }
      },
      "tcompare": {
        "version": "2.2.0",
        "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-2.2.0.tgz",
        "integrity": "sha512-+Mr0UBIE3ncNn0wJvKsw8ph61QoaDvR6Q8WkxWIHxWLcKf8SHGyTTkGLMX//4NKQ/Pe1Uu64oXYJsxLyAXXWdA=="
      },
      "test-exclude": {
        "version": "5.2.3",
        "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
        "integrity": "sha512-M+oxtseCFO3EDtAaGH7iiej3CBkzXqFMbzqYAACdzKui4eZA+pq3tZEwChvOdNfa7xxy8BfbmgJSIr43cC/+2g==",
        "requires": {
          "glob": "^7.1.3",
          "minimatch": "^3.0.4",
          "read-pkg-up": "^4.0.0",
          "require-main-filename": "^2.0.0"
        }
      },
      "to-fast-properties": {
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-1.0.3.tgz",
        "integrity": "sha1-uDVx+k2MJbguIxsG46MFXeTKGkc="
      },
      "to-object-path": {
        "version": "0.3.0",
        "resolved": "https://registry.npmjs.org/to-object-path/-/to-object-path-0.3.0.tgz",
        "integrity": "sha1-KXWIt7Dn4KwI4E5nL4XB9JmeF68=",
        "requires": {
          "kind-of": "^3.0.2"
        }
      },
      "to-regex": {
        "version": "3.0.2",
        "resolved": "https://registry.npmjs.org/to-regex/-/to-regex-3.0.2.tgz",
        "integrity": "sha512-FWtleNAtZ/Ki2qtqej2CXTOayOH9bHDQF+Q48VpWyDXjbYxA4Yz8iDB31zXOBUlOHHKidDbqGVrTUvQMPmBGBw==",
        "requires": {
          "define-property": "^2.0.2",
          "extend-shallow": "^3.0.2",
          "regex-not": "^1.0.2",
          "safe-regex": "^1.1.0"
        },
        "dependencies": {
          "extend-shallow": {
            "version": "3.0.2",
            "resolved": "https://registry.npmjs.org/extend-shallow/-/extend-shallow-3.0.2.tgz",
            "integrity": "sha1-Jqcarwc7OfshJxcnRhMcJwQCjbg=",
            "requires": {
              "assign-symbols": "^1.0.0",
              "is-extendable": "^1.0.1"
            }
          },
          "is-extendable": {
            "version": "1.0.1",
            "resolved": "https://registry.npmjs.org/is-extendable/-/is-extendable-1.0.1.tgz",
            "integrity": "sha512-arnXMxT1hhoKo9k1LZdmlNyJdDDfy2v0fXjFlmok4+i8ul/6WlbVge9bhM74OpNPQPMGUToDtz+KXa1PneJxOA==",
            "requires": {
              "is-plain-object": "^2.0.4"
            }
          }
        }
      },
      "to-regex-range": {
        "version": "2.1.1",
        "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-2.1.1.tgz",
        "integrity": "sha1-fIDBe53+vlmeJzZ+DU3VWQFB2zg=",
        "requires": {
          "is-number": "^3.0.0",
          "repeat-string": "^1.6.1"
        }
      },
      "tough-cookie": {
        "version": "2.4.3",
        "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
        "integrity": "sha512-Q5srk/4vDM54WJsJio3XNn6K2sCG+CQ8G5Wz6bZhRZoAe/+TxjWB/GlFAnYEbkYVlON9FMk/fE3h2RLpPXo4lQ==",
        "requires": {
          "psl": "^1.1.24",
          "punycode": "^1.4.1"
        }
      },
      "treport": {
        "version": "0.3.0",
        "resolved": "https://registry.npmjs.org/treport/-/treport-0.3.0.tgz",
        "integrity": "sha512-THr7NS5iLJewcAiaBkxAuCVoakw84hAVY9n2kFNZqlFKHrZeGw8sNR4VhmT23JB1/JnCmPcIOmooTdYYVTI5hA==",
        "requires": {
          "cardinal": "^2.1.1",
          "chalk": "^2.4.2",
          "import-jsx": "^2.0.0",
          "ink": "^2.1.1",
          "ms": "^2.1.1",
          "react": "^16.8.6",
          "string-length": "^2.0.0",
          "tap-parser": "^9.3.2",
          "unicode-length": "^2.0.1"
        },
        "dependencies": {
          "ansi-styles": {
            "version": "3.2.1",
            "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
            "integrity": "sha512-VT0ZI6kZRdTh8YyJw3SMbYm/u+NqfsAxEpWO0Pf9sq8/e94WxxOpPKx9FR1FlyCtOVDNOQ+8ntlqFxiRc+r5qA==",
            "requires": {
              "color-convert": "^1.9.0"
            }
          },
          "chalk": {
            "version": "2.4.2",
            "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
            "integrity": "sha512-Mti+f9lpJNcwF4tWV8/OrTTtF1gZi+f8FqlyAdouralcFWFQWF2+NgCHShjkCb+IFBLq9buZwE1xckQU4peSuQ==",
            "requires": {
              "ansi-styles": "^3.2.1",
              "escape-string-regexp": "^1.0.5",
              "supports-color": "^5.3.0"
            }
          },
          "ms": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.1.tgz",
            "integrity": "sha512-tgp+dl5cGk28utYktBsrFqA7HKgrhgPsg6Z/EfhWI4gl1Hwq8B/GmY/0oXZ6nF8hDVesS/FpnYaD/kOWhYQvyg=="
          },
          "punycode": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
            "integrity": "sha512-XRsRjdf+j5ml+y/6GKHPZbrF/8p2Yga0JPtdqTIY2Xe5ohJPD9saDJJLPvp9+NSBprVvevdXZybnj2cv8OEd0A=="
          },
          "supports-color": {
            "version": "5.5.0",
            "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
            "integrity": "sha512-QjVjwdXIt408MIiAqCX4oUKsgU2EqAGzs2Ppkm4aQYbjm+ZEWEcW4SfFNTr4uMNZma0ey4f5lgLrkB0aX0QMow==",
            "requires": {
              "has-flag": "^3.0.0"
            }
          },
          "unicode-length": {
            "version": "2.0.1",
            "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-2.0.1.tgz",
            "integrity": "sha1-hN3DCcIyNjX2QuhtRI6ersCEwEo=",
            "requires": {
              "punycode": "^2.0.0",
              "strip-ansi": "^3.0.1"
            }
          }
        }
      },
      "trim-right": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/trim-right/-/trim-right-1.0.1.tgz",
        "integrity": "sha1-yy4SAwZ+DI3h9hQJS5/kVwTqYAM="
      },
      "trivial-deferred": {
        "version": "1.0.1",
        "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
        "integrity": "sha1-N21NKdlR1jaKb3oK6FwvTV4GWPM="
      },
      "ts-node": {
        "version": "8.1.0",
        "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.1.0.tgz",
        "integrity": "sha512-34jpuOrxDuf+O6iW1JpgTRDFynUZ1iEqtYruBqh35gICNjN8x+LpVcPAcwzLPi9VU6mdA3ym+x233nZmZp445A==",
        "requires": {
          "arg": "^4.1.0",
          "diff": "^3.1.0",
          "make-error": "^1.1.1",
          "source-map-support": "^0.5.6",
          "typescript": ">=2.0",
          "yn": "^3.0.0"
        },
        "dependencies": {
          "diff": {
            "version": "3.5.0",
            "resolved": "https://registry.npmjs.org/diff/-/diff-3.5.0.tgz",
            "integrity": "sha512-A46qtFgd+g7pDZinpnwiRJtxbC1hpgf0uzP3iG89scHk0AUC7A1TGxf5OiiOUv/JMZR8GOt8hL900hV0bOy5xA=="
          }
        }
      },
      "tunnel-agent": {
        "version": "0.6.0",
        "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
        "integrity": "sha1-J6XeoGs2sEoKmWZ3SykIaPD8QP0=",
        "requires": {
          "safe-buffer": "^5.0.1"
        }
      },
      "tweetnacl": {
        "version": "0.14.5",
        "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
        "integrity": "sha1-WuaBd/GS1EViadEIr6k/+HQ/T2Q="
      },
      "typescript": {
        "version": "3.4.5",
        "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.4.5.tgz",
        "integrity": "sha512-YycBxUb49UUhdNMU5aJ7z5Ej2XGmaIBL0x34vZ82fn3hGvD+bgrMrVDpatgz2f7YxUMJxMkbWxJZeAvDxVe7Vw=="
      },
      "uglify-js": {
        "version": "3.5.8",
        "resolved": "https://registry.npmjs.org/uglify-js/-/uglify-js-3.5.8.tgz",
        "integrity": "sha512-GFSjB1nZIzoIq70qvDRtWRORHX3vFkAnyK/rDExc0BN7r9+/S+Voz3t/fwJuVfjppAMz+ceR2poE7tkhvnVwQQ==",
        "optional": true,
        "requires": {
          "commander": "~2.20.0",
          "source-map": "~0.6.1"
        },
        "dependencies": {
          "source-map": {
            "version": "0.6.1",
            "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
            "integrity": "sha512-UjgapumWlbMhkBgzT7Ykc5YXUT46F0iKu8SGXq0bcwP5dz/h0Plj6enJqjz1Zbq2l5WaqYnrVbwWOWMyF3F47g==",
            "optional": true
          }
        }
      },
      "unicode-length": {
        "version": "1.0.3",
        "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
        "integrity": "sha1-Wtp6f+1RhBpBijKM8UlHisg1irs=",
        "requires": {
          "punycode": "^1.3.2",
          "strip-ansi": "^3.0.1"
        }
      },
      "union-value": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/union-value/-/union-value-1.0.0.tgz",
        "integrity": "sha1-XHHDTLW61dzr4+oM0IIHulqhrqQ=",
        "requires": {
          "arr-union": "^3.1.0",
          "get-value": "^2.0.6",
          "is-extendable": "^0.1.1",
          "set-value": "^0.4.3"
        },
        "dependencies": {
          "set-value": {
            "version": "0.4.3",
            "resolved": "https://registry.npmjs.org/set-value/-/set-value-0.4.3.tgz",
            "integrity": "sha1-fbCPnT0i3H945Trzw79GZuzfzPE=",
            "requires": {
              "extend-shallow": "^2.0.1",
              "is-extendable": "^0.1.1",
              "is-plain-object": "^2.0.1",
              "to-object-path": "^0.3.0"
            }
          }
        }
      },
      "unset-value": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/unset-value/-/unset-value-1.0.0.tgz",
        "integrity": "sha1-g3aHP30jNRef+x5vw6jtDfyKtVk=",
        "requires": {
          "has-value": "^0.3.1",
          "isobject": "^3.0.0"
        },
        "dependencies": {
          "has-value": {
            "version": "0.3.1",
            "resolved": "https://registry.npmjs.org/has-value/-/has-value-0.3.1.tgz",
            "integrity": "sha1-ex9YutpiyoJ+wKIHgCVlSEWZXh8=",
            "requires": {
              "get-value": "^2.0.3",
              "has-values": "^0.1.4",
              "isobject": "^2.0.0"
            },
            "dependencies": {
              "isobject": {
                "version": "2.1.0",
                "resolved": "https://registry.npmjs.org/isobject/-/isobject-2.1.0.tgz",
                "integrity": "sha1-8GVWEJaj8dou9GJy+BXIQNh+DIk=",
                "requires": {
                  "isarray": "1.0.0"
                }
              }
            }
          },
          "has-values": {
            "version": "0.1.4",
            "resolved": "https://registry.npmjs.org/has-values/-/has-values-0.1.4.tgz",
            "integrity": "sha1-bWHeldkd/Km5oCCJrThL/49it3E="
          }
        }
      },
      "upath": {
        "version": "1.1.2",
        "resolved": "https://registry.npmjs.org/upath/-/upath-1.1.2.tgz",
        "integrity": "sha512-kXpym8nmDmlCBr7nKdIx8P2jNBa+pBpIUFRnKJ4dr8htyYGJFokkr2ZvERRtUN+9SY+JqXouNgUPtv6JQva/2Q=="
      },
      "uri-js": {
        "version": "4.2.2",
        "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
        "integrity": "sha512-KY9Frmirql91X2Qgjry0Wd4Y+YTdrdZheS8TFwvkbLWf/G5KNJDCh6pKL5OZctEW4+0Baa5idK2ZQuELRwPznQ==",
        "requires": {
          "punycode": "^2.1.0"
        },
        "dependencies": {
          "punycode": {
            "version": "2.1.1",
            "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
            "integrity": "sha512-XRsRjdf+j5ml+y/6GKHPZbrF/8p2Yga0JPtdqTIY2Xe5ohJPD9saDJJLPvp9+NSBprVvevdXZybnj2cv8OEd0A=="
          }
        }
      },
      "urix": {
        "version": "0.1.0",
        "resolved": "https://registry.npmjs.org/urix/-/urix-0.1.0.tgz",
        "integrity": "sha1-2pN/emLiH+wf0Y1Js1wpNQZ6bHI="
      },
      "use": {
        "version": "3.1.1",
        "resolved": "https://registry.npmjs.org/use/-/use-3.1.1.tgz",
        "integrity": "sha512-cwESVXlO3url9YWlFW/TA9cshCEhtu7IKJ/p5soJ/gGpj7vbvFrAY/eIioQ6Dw23KjZhYgiIo8HOs1nQ2vr/oQ=="
      },
      "util-deprecate": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
        "integrity": "sha1-RQ1Nyfpw3nMnYvvS1KKJgUGaDM8="
      },
      "uuid": {
        "version": "3.3.2",
        "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.2.tgz",
        "integrity": "sha512-yXJmeNaw3DnnKAOKJE51sL/ZaYfWJRl1pK9dr19YFCu0ObS231AB1/LbqTKRAQ5kw8A90rA6fr4riOUpTZvQZA=="
      },
      "validate-npm-package-license": {
        "version": "3.0.4",
        "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
        "integrity": "sha512-DpKm2Ui/xN7/HQKCtpZxoRWBhZ9Z0kqtygG8XCgNQ8ZlDnxuQmWhj566j8fN4Cu3/JmbhsDo7fcAJq4s9h27Ew==",
        "requires": {
          "spdx-correct": "^3.0.0",
          "spdx-expression-parse": "^3.0.0"
        }
      },
      "verror": {
        "version": "1.10.0",
        "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
        "integrity": "sha1-OhBcoXBTr1XW4nDB+CiGguGNpAA=",
        "requires": {
          "assert-plus": "^1.0.0",
          "core-util-is": "1.0.2",
          "extsprintf": "^1.2.0"
        }
      },
      "which": {
        "version": "1.3.1",
        "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
        "integrity": "sha512-HxJdYWq1MTIQbJ3nw0cqssHoTNU267KlrDuGZ1WYlxDStUtKUhOaJmh112/TZmHxxUfuJqPXSOm7tDyas0OSIQ==",
        "requires": {
          "isexe": "^2.0.0"
        }
      },
      "which-module": {
        "version": "2.0.0",
        "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
        "integrity": "sha1-2e8H3Od7mQK4o6j6SzHD4/fm6Ho="
      },
      "widest-line": {
        "version": "2.0.1",
        "resolved": "https://registry.npmjs.org/widest-line/-/widest-line-2.0.1.tgz",
        "integrity": "sha512-Ba5m9/Fa4Xt9eb2ELXt77JxVDV8w7qQrH0zS/TWSJdLyAwQjWoOzpzj5lwVftDz6n/EOu3tNACS84v509qwnJA==",
        "requires": {
          "string-width": "^2.1.1"
        }
      },
      "wordwrap": {
        "version": "0.0.3",
        "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
        "integrity": "sha1-o9XabNXAvAAI03I0u68b7WMFkQc="
      },
      "wrap-ansi": {
        "version": "2.1.0",
        "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
        "integrity": "sha1-2Pw9KE3QV5T+hJc8rs3Rz4JP3YU=",
        "requires": {
          "string-width": "^1.0.1",
          "strip-ansi": "^3.0.1"
        },
        "dependencies": {
          "is-fullwidth-code-point": {
            "version": "1.0.0",
            "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
            "integrity": "sha1-754xOG8DGn8NZDr4L95QxFfvAMs=",
            "requires": {
              "number-is-nan": "^1.0.0"
            }
          },
          "string-width": {
            "version": "1.0.2",
            "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
            "integrity": "sha1-EYvfW4zcUaKn5w0hHgfisLmxB9M=",
            "requires": {
              "code-point-at": "^1.0.0",
              "is-fullwidth-code-point": "^1.0.0",
              "strip-ansi": "^3.0.0"
            }
          }
        }
      },
      "wrappy": {
        "version": "1.0.2",
        "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
        "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
      },
      "write-file-atomic": {
        "version": "2.4.2",
        "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.2.tgz",
        "integrity": "sha512-s0b6vB3xIVRLWywa6X9TOMA7k9zio0TMOsl9ZnDkliA/cfJlpHXAscj0gbHVJiTdIuAYpIyqS5GW91fqm6gG5g==",
        "requires": {
          "graceful-fs": "^4.1.11",
          "imurmurhash": "^0.1.4",
          "signal-exit": "^3.0.2"
        }
      },
      "y18n": {
        "version": "4.0.0",
        "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
        "integrity": "sha512-r9S/ZyXu/Xu9q1tYlpsLIsa3EeLXXk0VwlxqTcFRfg9EhMW+17kbt9G0NrgCmhGb5vT2hyhJZLfDGx+7+5Uj/w=="
      },
      "yallist": {
        "version": "2.1.2",
        "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
        "integrity": "sha1-HBH5IY8HYImkfdUS+TxmmaaoHVI="
      },
      "yaml": {
        "version": "1.5.0",
        "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.5.0.tgz",
        "integrity": "sha512-nKxSWOa7vxAP2pikrGxbkZsG/garQseRiLn9mIDjzwoQsyVy7ZWIpLoARejnINGGLA4fttuzRFFNxxbsztdJgw==",
        "requires": {
          "@babel/runtime": "^7.4.3"
        }
      },
      "yapool": {
        "version": "1.0.0",
        "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
        "integrity": "sha1-9pPymjFbUNmp2iZGp6ZkXJaYW2o="
      },
      "yargs": {
        "version": "13.2.2",
        "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.2.2.tgz",
        "integrity": "sha512-WyEoxgyTD3w5XRpAQNYUB9ycVH/PQrToaTXdYXRdOXvEy1l19br+VJsc0vcO8PTGg5ro/l/GY7F/JMEBmI0BxA==",
        "requires": {
          "cliui": "^4.0.0",
          "find-up": "^3.0.0",
          "get-caller-file": "^2.0.1",
          "os-locale": "^3.1.0",
          "require-directory": "^2.1.1",
          "require-main-filename": "^2.0.0",
          "set-blocking": "^2.0.0",
          "string-width": "^3.0.0",
          "which-module": "^2.0.0",
          "y18n": "^4.0.0",
          "yargs-parser": "^13.0.0"
        },
        "dependencies": {
          "ansi-regex": {
            "version": "4.1.0",
            "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
            "integrity": "sha512-1apePfXM1UOSqw0o9IiFAovVz9M5S1Dg+4TrDwfMewQ6p/rmMueb7tWZjQ1rx4Loy1ArBggoqGpfqqdI4rondg=="
          },
          "string-width": {
            "version": "3.1.0",
            "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
            "integrity": "sha512-vafcv6KjVZKSgz06oM/H6GDBrAtz8vdhQakGjFIvNrHA6y3HCF1CInLy+QLq8dTJPQ1b+KDUqDFctkdRW44e1w==",
            "requires": {
              "emoji-regex": "^7.0.1",
              "is-fullwidth-code-point": "^2.0.0",
              "strip-ansi": "^5.1.0"
            }
          },
          "strip-ansi": {
            "version": "5.2.0",
            "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
            "integrity": "sha512-DuRs1gKbBqsMKIZlrffwlug8MHkcnpjs5VPmL1PAh+mA30U0DTotfDZ0d2UUsXpPmPmMMJ6W773MaA3J+lbiWA==",
            "requires": {
              "ansi-regex": "^4.1.0"
            }
          }
        }
      },
      "yargs-parser": {
        "version": "13.0.0",
        "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.0.0.tgz",
        "integrity": "sha512-w2LXjoL8oRdRQN+hOyppuXs+V/fVAYtpcrRxZuF7Kt/Oc+Jr2uAcVntaUTNT6w5ihoWfFDpNY8CPx1QskxZ/pw==",
        "requires": {
          "camelcase": "^5.0.0",
          "decamelize": "^1.2.0"
        }
      },
      "yn": {
        "version": "3.1.0",
        "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.0.tgz",
        "integrity": "sha512-kKfnnYkbTfrAdd0xICNFw7Atm8nKpLcLv9AZGEt+kczL/WQVai4e2V6ZN8U/O+iI6WrNuJjNNOyu4zfhl9D3Hg=="
      },
      "yoga-layout-prebuilt": {
        "version": "1.9.3",
        "resolved": "https://registry.npmjs.org/yoga-layout-prebuilt/-/yoga-layout-prebuilt-1.9.3.tgz",
        "integrity": "sha512-9SNQpwuEh2NucU83i2KMZnONVudZ86YNcFk9tq74YaqrQfgJWO3yB9uzH1tAg8iqh5c9F5j0wuyJ2z72wcum2w=="
      }
    }
  }),
  "package.json": JSON.stringify({
    "dependencies": {
      "mkdirp": "^0.5.1",
      "tap": "^13.0.3"
    }
  })
})
  return path
}
