/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/arborist/reify.js TAP a workspace with a duplicated nested conflicted dep > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "bar" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "bar",
          "spec": "file:packages/bar",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/foo",
          "name": "bar",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bar",
      "name": "bar",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/node_modules/bar",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/packages/bar",
      "resolved": "file:../packages/bar",
      "target": ArboristNode {
        "location": "packages/bar",
      },
      "version": "1.2.3",
    },
    "foo" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "foo",
          "spec": "file:packages/foo",
          "type": "prod",
        },
      },
      "location": "node_modules/foo",
      "name": "foo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/node_modules/foo",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/packages/foo",
      "resolved": "file:../packages/foo",
      "target": ArboristNode {
        "location": "packages/foo",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/bar",
          "name": "mkdirp",
          "spec": "0.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
  },
  "edgesOut": Map {
    "bar" => EdgeOut {
      "name": "bar",
      "spec": "file:packages/bar",
      "to": "node_modules/bar",
      "type": "prod",
    },
    "foo" => EdgeOut {
      "name": "foo",
      "spec": "file:packages/foo",
      "to": "node_modules/foo",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "0.x",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "packages/bar",
      "name": "bar",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/packages/bar",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/foo",
              "name": "mkdirp",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "packages/foo/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/packages/foo/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesOut": Map {
        "bar" => EdgeOut {
          "name": "bar",
          "spec": "^1.0.0",
          "to": "node_modules/bar",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "1",
          "to": "packages/foo/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "packages/foo",
      "name": "foo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep/packages/foo",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep",
  "path": "{CWD}/test/arborist/tap-testdir-reify-a-workspace-with-a-duplicated-nested-conflicted-dep",
}
`

exports[`test/arborist/reify.js TAP add a dep present in the tree, with v1 shrinkwrap > must match snapshot 1`] = `
{"dependencies":{"once":"^1.4.0","wrappy":"^1.0.2"}}
`

exports[`test/arborist/reify.js TAP add a new pkg to a prefix that needs to be mkdirpd > should output a successful tree in mkdirp folder 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-a-new-pkg-to-a-prefix-that-needs-to-be-mkdirpd/missing/path/to/root/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "root",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-a-new-pkg-to-a-prefix-that-needs-to-be-mkdirpd/missing/path/to/root",
}
`

exports[`test/arborist/reify.js TAP add a new pkg to a prefix that needs to be mkdirpd > should place expected lockfile file into place 1`] = `
{
  "name": "root",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP add a new pkg to a prefix that needs to be mkdirpd > should place expected package.json file into place 1`] = `
{
  "dependencies": {
    "abbrev": "^1.1.1"
  }
}

`

exports[`test/arborist/reify.js TAP add deps to workspaces add a to root > lockfile added workspace as dep 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-a-to-root",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "a": "^1.2.3",
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/minimist": Object {
      "integrity": "sha512-FM9nNUYrRBAELZQT3xeZQ7fmMOBg6nWNmJKTcgsJeaLstP/UODVpGsr5OhXhhXg6f+qtJ8uiZ+PUxkDWcgIXLw==",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/a/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "^1.2.5",
      },
      "integrity": "sha512-NKmAlESf6jMGym1++R0Ra7wvhV+wFW63FaSOFPwRahvea0gMUcGUhVeAg/0BC0wiv9ih5NYPB1Wn1UEI1/L+xQ==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add a to root > package.json added workspace as dep 1`] = `
Object {
  "dependencies": Object {
    "a": "^1.2.3",
    "mkdirp": "^1.0.4",
  },
  "workspaces": Array [
    "packages/*",
  ],
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add a to root > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-a-to-root",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-a-to-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/minimist": Object {
      "integrity": "sha512-FM9nNUYrRBAELZQT3xeZQ7fmMOBg6nWNmJKTcgsJeaLstP/UODVpGsr5OhXhhXg6f+qtJ8uiZ+PUxkDWcgIXLw==",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/a/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "^1.2.5",
      },
      "integrity": "sha512-NKmAlESf6jMGym1++R0Ra7wvhV+wFW63FaSOFPwRahvea0gMUcGUhVeAg/0BC0wiv9ih5NYPB1Wn1UEI1/L+xQ==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "packages/b": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/b/node_modules/minimist": Object {
      "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0=",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "packages/b/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "0.0.8",
      },
      "deprecated": "Legacy versions of mkdirp are no longer supported. Please update to mkdirp 1.x. (Note that the API surface has changed to use Promises in 1.x.)",
      "integrity": "sha1-HXMHam35hs2TROFecfzAWkyavxI=",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
      "version": "0.5.0",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b > package.json b 1`] = `
Object {
  "dependencies": Object {
    "mkdirp": "^0.5.0",
  },
  "name": "b",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b/node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "0.0.8",
              "to": "packages/b/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
          "version": "0.5.0",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/b/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b, empty start > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/minimist": Object {
      "integrity": "sha512-FM9nNUYrRBAELZQT3xeZQ7fmMOBg6nWNmJKTcgsJeaLstP/UODVpGsr5OhXhhXg6f+qtJ8uiZ+PUxkDWcgIXLw==",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/a/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "^1.2.5",
      },
      "integrity": "sha512-NKmAlESf6jMGym1++R0Ra7wvhV+wFW63FaSOFPwRahvea0gMUcGUhVeAg/0BC0wiv9ih5NYPB1Wn1UEI1/L+xQ==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "packages/b": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/b/node_modules/minimist": Object {
      "integrity": "sha1-hX/Kv8M5fSYluCKCYuhqp6ARsF0=",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "packages/b/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "0.0.8",
      },
      "deprecated": "Legacy versions of mkdirp are no longer supported. Please update to mkdirp 1.x. (Note that the API surface has changed to use Promises in 1.x.)",
      "integrity": "sha1-HXMHam35hs2TROFecfzAWkyavxI=",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
      "version": "0.5.0",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b, empty start > package.json b 1`] = `
Object {
  "dependencies": Object {
    "mkdirp": "^0.5.0",
  },
  "name": "b",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces add mkdirp 0.5.0 to b, empty start > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "error": "MISSING",
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/a",
      "to": null,
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "error": "MISSING",
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": null,
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b/node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/b",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "0.0.8",
              "to": "packages/b/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/b/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.0.tgz",
          "version": "0.5.0",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/b/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-add-mkdirp-0.5.0-to-b-empty-start",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces no args > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-no-args",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/minimist": Object {
      "integrity": "sha512-FM9nNUYrRBAELZQT3xeZQ7fmMOBg6nWNmJKTcgsJeaLstP/UODVpGsr5OhXhhXg6f+qtJ8uiZ+PUxkDWcgIXLw==",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^0.5.0",
      },
      "version": "1.2.3",
    },
    "packages/a/node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "dependencies": Object {
        "minimist": "^1.2.5",
      },
      "integrity": "sha512-NKmAlESf6jMGym1++R0Ra7wvhV+wFW63FaSOFPwRahvea0gMUcGUhVeAg/0BC0wiv9ih5NYPB1Wn1UEI1/L+xQ==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
      "version": "0.5.5",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces no args > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/a/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.5.tgz",
          "version": "0.5.5",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "packages/a/node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-no-args",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-no-args",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "version": "1.2.3",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a > package.json a 1`] = `
Object {
  "name": "a",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a, empty start > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "version": "1.2.3",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a, empty start > package.json a 1`] = `
Object {
  "name": "a",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces remove mkdirp from a, empty start > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/packages/b",
      "to": null,
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "error": "MISSING",
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": null,
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start/packages/a",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-remove-mkdirp-from-a-empty-start",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "version": "1.2.3",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root > package.json a 1`] = `
Object {
  "dependencies": Object {
    "mkdirp": "^1.0.4",
  },
  "name": "a",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.2.3",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^1.0.4",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root/packages/b",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root, empty start > lockfile 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "workspaces": Array [
        "packages/*",
      ],
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "packages/a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "packages/b",
    },
    "node_modules/mkdirp": Object {
      "bin": Object {
        "mkdirp": "bin/cmd.js",
      },
      "engines": Object {
        "node": ">=10",
      },
      "integrity": "sha512-vVqVZQyf3WLx2Shd0qJ9xuvqgAyKPLAiqITEtqW0oIUjzo3PePDd6fW9iFz30ef7Ysp/oiWqbhszeGWW2T6Gzw==",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
    "packages/a": Object {
      "dependencies": Object {
        "mkdirp": "^1.0.4",
      },
      "version": "1.2.3",
    },
    "packages/b": Object {
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root, empty start > package.json a 1`] = `
Object {
  "dependencies": Object {
    "mkdirp": "^1.0.4",
  },
  "name": "a",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP add deps to workspaces upgrade mkdirp in a, dedupe on root, empty start > returned tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/a",
          "name": "mkdirp",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.4.tgz",
      "version": "1.0.4",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/packages/b",
      "to": null,
      "type": "workspace",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.4",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^1.0.4",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start/packages/a",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-deps-to-workspaces-upgrade-mkdirp-in-a-dedupe-on-root-empty-start",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP add local dep with existing dev + peer/optional > tree 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristLink {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "file:../dep",
          "type": "dev",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-add-local-dep-with-existing-dev-peer-optional/project/node_modules/abbrev",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-add-local-dep-with-existing-dev-peer-optional/dep",
      "resolved": "file:../../dep",
      "target": ArboristNode {
        "dev": true,
        "location": "../dep",
        "name": "abbrev",
        "path": "{CWD}/test/arborist/tap-testdir-reify-add-local-dep-with-existing-dev-peer-optional/dep",
        "version": "1.0.0",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "file:../dep",
      "to": "node_modules/abbrev",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "project",
  "path": "{CWD}/test/arborist/tap-testdir-reify-add-local-dep-with-existing-dev-peer-optional/project",
}
`

exports[`test/arborist/reify.js TAP add multiple pkgs in a specific order > should alphabetically sort dependencies 1`] = `
{"name":"multiple-pkgs","dependencies":{"abbrev":"^1.1.1","wrappy":"^1.0.2"}}
`

exports[`test/arborist/reify.js TAP add multiple pkgs in a specific order > should alphabetically sort new added dep 1`] = `
{"name":"multiple-pkgs","dependencies":{"abbrev":"^1.1.1","once":"^1.4.0","wrappy":"^1.0.2"}}
`

exports[`test/arborist/reify.js TAP add spec * with semver prefix range gets updated > must match snapshot 1`] = `
{
  "dependencies": {
    "latest-is-prerelease": "^1.0.0-beta-rc.3"
  }
}

`

exports[`test/arborist/reify.js TAP adding an unresolvable optional dep is OK > must match snapshot 1`] = `
ArboristNode {
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "999999",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-adding-an-unresolvable-optional-dep-is-OK",
  "path": "{CWD}/test/arborist/tap-testdir-reify-adding-an-unresolvable-optional-dep-is-OK",
}
`

exports[`test/arborist/reify.js TAP bad shrinkwrap file > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps-b" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-deps-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-deps-d" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-d",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-deps-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-bad-shrinkwrap-file",
  "packageName": "@isaacs/testing-peer-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-bad-shrinkwrap-file",
  "version": "2.0.0",
}
`

exports[`test/arborist/reify.js TAP collide case-variant dep names > tree 1 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-collide-case-variant-dep-names/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "1",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-collide-case-variant-dep-names",
  "path": "{CWD}/test/arborist/tap-testdir-reify-collide-case-variant-dep-names",
}
`

exports[`test/arborist/reify.js TAP collide case-variant dep names > tree 2 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-collide-case-variant-dep-names/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-collide-case-variant-dep-names",
  "path": "{CWD}/test/arborist/tap-testdir-reify-collide-case-variant-dep-names",
}
`

exports[`test/arborist/reify.js TAP create link deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-create-link-deps/node_modules/linked-dep",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-create-link-deps/target",
      "resolved": "file:../target",
      "target": ArboristNode {
        "location": "target",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "linked-dep" => EdgeOut {
      "name": "linked-dep",
      "spec": "file:target",
      "to": "node_modules/linked-dep",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "target",
      "name": "target",
      "packageName": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-create-link-deps/target",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-create-link-deps",
  "packageName": "@isaacs/testing-link-dep",
  "path": "{CWD}/test/arborist/tap-testdir-reify-create-link-deps",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dev, optional, devOptional flags and omissions dev > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "own-or" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "wrappy" => ArboristNode {
      "devOptional": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "^1.0.2",
          "type": "optional",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "error": "MISSING",
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": null,
      "type": "dev",
    },
    "once" => EdgeOut {
      "error": "MISSING",
      "name": "once",
      "spec": "^1.4.0",
      "to": null,
      "type": "dev",
    },
    "own-or" => EdgeOut {
      "name": "own-or",
      "spec": "^1.0.0",
      "to": "node_modules/own-or",
      "type": "optional",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "^1.0.2",
      "to": "node_modules/wrappy",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev",
  "packageName": "@isaacs/testing-dev-optional-flags",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dev, optional, devOptional flags and omissions dev,optional > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "error": "MISSING",
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": null,
      "type": "dev",
    },
    "once" => EdgeOut {
      "error": "MISSING",
      "name": "once",
      "spec": "^1.4.0",
      "to": null,
      "type": "dev",
    },
    "own-or" => EdgeOut {
      "name": "own-or",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "^1.0.2",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev-optional",
  "packageName": "@isaacs/testing-dev-optional-flags",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-dev-optional",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dev, optional, devOptional flags and omissions optional > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "dev",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-optional/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "^1.4.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-optional/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "devOptional": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "wrappy",
          "spec": "^1.0.2",
          "type": "optional",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-optional/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "dev",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "^1.4.0",
      "to": "node_modules/once",
      "type": "dev",
    },
    "own-or" => EdgeOut {
      "name": "own-or",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
    "wrappy" => EdgeOut {
      "name": "wrappy",
      "spec": "^1.0.2",
      "to": "node_modules/wrappy",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-optional",
  "packageName": "@isaacs/testing-dev-optional-flags",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dev-optional-devOptional-flags-and-omissions-optional",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP do not add shrinkwrapped deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-add-shrinkwrapped-deps/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-add-shrinkwrapped-deps/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-do-not-add-shrinkwrapped-deps",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-add-shrinkwrapped-deps",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP do not delete root-bundled deps in global update > after first install 1`] = `
{
  "name": "dep"
}

`

exports[`test/arborist/reify.js TAP do not delete root-bundled deps in global update > after second install 1`] = `
{
  "name": "dep"
}

`

exports[`test/arborist/reify.js TAP do not excessively duplicate bundled metadeps > hidden lockfile 1`] = `
{
  "name": "@isaacs/bundle-metadep-duplication-root",
  "version": "1.0.0",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "node_modules/@isaacs/bundle-metadep-duplication-x": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/@isaacs/bundle-metadep-duplication-x/-/bundle-metadep-duplication-x-1.0.2.tgz",
      "integrity": "sha512-mlCA3lXLaraoJFrnZ1sBZeXy4qdYpubKH8jZB5KIjCCwcWPgOui/EWTWnf/wf0Q08V0qr8xmG63wGQgEa+hkSw==",
      "bundleDependencies": [
        "@isaacs/bundle-metadep-duplication-y"
      ],
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-y": "1",
        "@isaacs/bundle-metadep-duplication-z": "1",
        "abbrev": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-z": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-z": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-a": "1"
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP do not excessively duplicate bundled metadeps > normal lockfile 1`] = `
{
  "name": "@isaacs/bundle-metadep-duplication-root",
  "version": "1.0.0",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "name": "@isaacs/bundle-metadep-duplication-root",
      "version": "1.0.0",
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-x": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/@isaacs/bundle-metadep-duplication-x/-/bundle-metadep-duplication-x-1.0.2.tgz",
      "integrity": "sha512-mlCA3lXLaraoJFrnZ1sBZeXy4qdYpubKH8jZB5KIjCCwcWPgOui/EWTWnf/wf0Q08V0qr8xmG63wGQgEa+hkSw==",
      "bundleDependencies": [
        "@isaacs/bundle-metadep-duplication-y"
      ],
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-y": "1",
        "@isaacs/bundle-metadep-duplication-z": "1",
        "abbrev": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-z": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-z": "1"
      }
    },
    "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z": {
      "version": "1.0.0",
      "inBundle": true,
      "dependencies": {
        "@isaacs/bundle-metadep-duplication-a": "1"
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP do not excessively duplicate bundled metadeps > tree 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/bundle-metadep-duplication-x" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/bundle-metadep-duplication-y",
      ],
      "children": Map {
        "@isaacs/bundle-metadep-duplication-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/bundle-metadep-duplication-x",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
              "name": "@isaacs/bundle-metadep-duplication-a",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/bundle-metadep-duplication-z" => EdgeOut {
              "name": "@isaacs/bundle-metadep-duplication-z",
              "spec": "1",
              "to": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a",
          "name": "@isaacs/bundle-metadep-duplication-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps/node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a",
          "version": "1.0.0",
        },
        "@isaacs/bundle-metadep-duplication-y" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/bundle-metadep-duplication-x",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/bundle-metadep-duplication-x",
              "name": "@isaacs/bundle-metadep-duplication-y",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/bundle-metadep-duplication-z" => EdgeOut {
              "name": "@isaacs/bundle-metadep-duplication-z",
              "spec": "1",
              "to": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y",
          "name": "@isaacs/bundle-metadep-duplication-y",
          "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps/node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y",
          "version": "1.0.0",
        },
        "@isaacs/bundle-metadep-duplication-z" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/bundle-metadep-duplication-x",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/bundle-metadep-duplication-x",
              "name": "@isaacs/bundle-metadep-duplication-z",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a",
              "name": "@isaacs/bundle-metadep-duplication-z",
              "spec": "1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y",
              "name": "@isaacs/bundle-metadep-duplication-z",
              "spec": "1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/bundle-metadep-duplication-a" => EdgeOut {
              "name": "@isaacs/bundle-metadep-duplication-a",
              "spec": "1",
              "to": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-a",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
          "name": "@isaacs/bundle-metadep-duplication-z",
          "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps/node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/bundle-metadep-duplication-x",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/bundle-metadep-duplication-y" => EdgeOut {
          "name": "@isaacs/bundle-metadep-duplication-y",
          "spec": "1",
          "to": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-y",
          "type": "prod",
        },
        "@isaacs/bundle-metadep-duplication-z" => EdgeOut {
          "name": "@isaacs/bundle-metadep-duplication-z",
          "spec": "1",
          "to": "node_modules/@isaacs/bundle-metadep-duplication-x/node_modules/@isaacs/bundle-metadep-duplication-z",
          "type": "prod",
        },
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "1",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/bundle-metadep-duplication-x",
      "name": "@isaacs/bundle-metadep-duplication-x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps/node_modules/@isaacs/bundle-metadep-duplication-x",
      "resolved": "https://registry.npmjs.org/@isaacs/bundle-metadep-duplication-x/-/bundle-metadep-duplication-x-1.0.2.tgz",
      "version": "1.0.2",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/bundle-metadep-duplication-x",
          "name": "abbrev",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/bundle-metadep-duplication-x" => EdgeOut {
      "name": "@isaacs/bundle-metadep-duplication-x",
      "spec": "1",
      "to": "node_modules/@isaacs/bundle-metadep-duplication-x",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps",
  "packageName": "@isaacs/bundle-metadep-duplication-root",
  "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-excessively-duplicate-bundled-metadeps",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP do not install optional deps with mismatched platform specifications > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "platform-specifying-test-package" => EdgeOut {
      "name": "platform-specifying-test-package",
      "spec": "1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-do-not-install-optional-deps-with-mismatched-platform-specifications",
  "packageName": "platform-test",
  "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-install-optional-deps-with-mismatched-platform-specifications",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP do not update shrinkwrapped deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-update-shrinkwrapped-deps/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-update-shrinkwrapped-deps/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-do-not-update-shrinkwrapped-deps",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-do-not-update-shrinkwrapped-deps",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dry run, do not get anything wet link-dep-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-dep-empty/node_modules/linked-dep",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-dep-empty/target",
      "resolved": "file:../target",
      "target": ArboristNode {
        "location": "target",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "linked-dep" => EdgeOut {
      "name": "linked-dep",
      "spec": "file:target",
      "to": "node_modules/linked-dep",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "target",
      "name": "target",
      "packageName": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-dep-empty/target",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dry-run-do-not-get-anything-wet-link-dep-empty",
  "packageName": "@isaacs/testing-link-dep",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-dep-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dry run, do not get anything wet link-meta-deps-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-link-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dep",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "linked-dep" => EdgeOut {
          "name": "linked-dep",
          "spec": "file:target",
          "to": "node_modules/linked-dep",
          "type": "prod",
        },
      },
      "fsChildren": Set {
        ArboristNode {
          "location": "node_modules/@isaacs/testing-link-dep/target",
          "name": "linked-dep",
          "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty/node_modules/@isaacs/testing-link-dep/target",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dep",
      "name": "@isaacs/testing-link-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty/node_modules/@isaacs/testing-link-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dep/-/testing-link-dep-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-link-dev-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dev-dep",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dev-dep",
      "name": "@isaacs/testing-link-dev-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty/node_modules/@isaacs/testing-link-dev-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dev-dep/-/testing-link-dev-dep-1.0.1.tgz",
      "version": "1.0.1",
    },
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-link-dep",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty/node_modules/linked-dep",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty/node_modules/@isaacs/testing-link-dep/target",
      "resolved": "file:@isaacs/testing-link-dep/target",
      "target": ArboristNode {
        "location": "node_modules/@isaacs/testing-link-dep/target",
      },
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-link-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dep",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-link-dep",
      "type": "prod",
    },
    "@isaacs/testing-link-dev-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dev-dep",
      "spec": "^1.0.1",
      "to": "node_modules/@isaacs/testing-link-dev-dep",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-link-meta-deps-empty",
}
`

exports[`test/arborist/reify.js TAP dry run, do not get anything wet shrinkwrapped-dep-no-lock-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-no-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-no-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-no-lock-empty",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-no-lock-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dry run, do not get anything wet shrinkwrapped-dep-with-lock-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-with-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-with-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-with-lock-empty",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-shrinkwrapped-dep-with-lock-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP dry run, do not get anything wet testing-bundledeps-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-b" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "name": "@isaacs/testing-bundledeps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-dry-run-do-not-get-anything-wet-testing-bundledeps-empty",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP failing script means install failure, unless ignoreScripts prod-dep-allinstall-fail --ignore-scripts > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-allinstall" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-allinstall",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-allinstall",
      "name": "@isaacs/testing-fail-allinstall",
      "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-allinstall-fail---ignore-scripts/node_modules/@isaacs/testing-fail-allinstall",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-allinstall/-/testing-fail-allinstall-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-allinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-allinstall",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-allinstall",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-allinstall-fail---ignore-scripts",
  "packageName": "@isaacs/testing-prod-dep-allinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-allinstall-fail---ignore-scripts",
  "version": "1.0.1",
}
`

exports[`test/arborist/reify.js TAP failing script means install failure, unless ignoreScripts prod-dep-install-fail --ignore-scripts > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-install" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-install",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-install",
      "name": "@isaacs/testing-fail-install",
      "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-install-fail---ignore-scripts/node_modules/@isaacs/testing-fail-install",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-install/-/testing-fail-install-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-install" => EdgeOut {
      "name": "@isaacs/testing-fail-install",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-install",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-install-fail---ignore-scripts",
  "packageName": "@isaacs/testing-prod-dep-install-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-install-fail---ignore-scripts",
  "version": "1.0.1",
}
`

exports[`test/arborist/reify.js TAP failing script means install failure, unless ignoreScripts prod-dep-postinstall-fail --ignore-scripts > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-postinstall" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-postinstall",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-postinstall",
      "name": "@isaacs/testing-fail-postinstall",
      "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-postinstall-fail---ignore-scripts/node_modules/@isaacs/testing-fail-postinstall",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-postinstall/-/testing-fail-postinstall-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-postinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-postinstall",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-postinstall",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-postinstall-fail---ignore-scripts",
  "packageName": "@isaacs/testing-prod-dep-postinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-postinstall-fail---ignore-scripts",
  "version": "1.0.1",
}
`

exports[`test/arborist/reify.js TAP failing script means install failure, unless ignoreScripts prod-dep-preinstall-fail --ignore-scripts > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-preinstall" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-preinstall",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-preinstall",
      "name": "@isaacs/testing-fail-preinstall",
      "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-preinstall-fail---ignore-scripts/node_modules/@isaacs/testing-fail-preinstall",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-preinstall/-/testing-fail-preinstall-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-preinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-preinstall",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-preinstall",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-preinstall-fail---ignore-scripts",
  "packageName": "@isaacs/testing-prod-dep-preinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-failing-script-means-install-failure-unless-ignoreScripts-prod-dep-preinstall-fail---ignore-scripts",
  "version": "1.0.1",
}
`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > hidden lockfile - c 1`] = `
{
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "node_modules/c": {
      "resolved": "packages/c",
      "link": true
    },
    "node_modules/wrappy": {
      "version": "1.0.0",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "integrity": "sha1-iq5PxrTNa+MqRVOYW88ys+4THk4="
    },
    "packages/c": {
      "version": "1.2.3",
      "dependencies": {
        "wrappy": "1.0.0"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > hidden lockfile - c, old x, removed a 1`] = `
{
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "apps/x": {
      "version": "1.2.3"
    },
    "node_modules/c": {
      "resolved": "packages/c",
      "link": true
    },
    "node_modules/wrappy": {
      "version": "1.0.0",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "integrity": "sha1-iq5PxrTNa+MqRVOYW88ys+4THk4="
    },
    "node_modules/x": {
      "resolved": "apps/x",
      "link": true
    },
    "packages/c": {
      "version": "1.2.3",
      "dependencies": {
        "wrappy": "1.0.0"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > hidden lockfile - c, x 1`] = `
{
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "apps/x": {
      "version": "1.2.3"
    },
    "node_modules/c": {
      "resolved": "packages/c",
      "link": true
    },
    "node_modules/wrappy": {
      "version": "1.0.0",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "integrity": "sha1-iq5PxrTNa+MqRVOYW88ys+4THk4="
    },
    "node_modules/x": {
      "resolved": "apps/x",
      "link": true
    },
    "packages/c": {
      "version": "1.2.3",
      "dependencies": {
        "wrappy": "1.0.0"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > hidden lockfile - c, x, a 1`] = `
{
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "apps/x": {
      "version": "1.2.3"
    },
    "node_modules/a": {
      "resolved": "packages/a",
      "link": true
    },
    "node_modules/c": {
      "resolved": "packages/c",
      "link": true
    },
    "node_modules/once": {
      "version": "1.4.0",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.0",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "integrity": "sha1-iq5PxrTNa+MqRVOYW88ys+4THk4="
    },
    "node_modules/x": {
      "resolved": "apps/x",
      "link": true
    },
    "packages/a": {
      "version": "1.2.3",
      "dependencies": {
        "once": "",
        "wrappy": "1.0.2"
      }
    },
    "packages/a/node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    },
    "packages/c": {
      "version": "1.2.3",
      "dependencies": {
        "wrappy": "1.0.0"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > hidden lockfile - foo/x linked, c, old x, removed a 1`] = `
{
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "apps/x": {
      "version": "1.2.3"
    },
    "foo/x": {
      "version": "1.2.3",
      "extraneous": true
    },
    "node_modules/c": {
      "resolved": "packages/c",
      "link": true
    },
    "node_modules/foox": {
      "resolved": "foo/x",
      "link": true
    },
    "node_modules/wrappy": {
      "version": "1.0.0",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "integrity": "sha1-iq5PxrTNa+MqRVOYW88ys+4THk4="
    },
    "node_modules/x": {
      "resolved": "apps/x",
      "link": true
    },
    "packages/c": {
      "version": "1.2.3",
      "dependencies": {
        "wrappy": "1.0.0"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > reify the a workspace after reifying c 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/c",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "packages/c",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "x",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/x",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "resolved": "file:../apps/x",
      "target": ArboristNode {
        "location": "apps/x",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/b",
      "to": null,
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "x" => EdgeOut {
      "name": "x",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "to": "node_modules/x",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "apps/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "version": "1.2.3",
    },
    ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "packages/a",
              "name": "wrappy",
              "spec": "1.0.2",
              "type": "prod",
            },
          },
          "location": "packages/a/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "*",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.2",
          "to": "packages/a/node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.0",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces",
  "workspaces": Map {
    "x" => "apps/x",
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > reify the c workspace only 1`] = `
ArboristNode {
  "children": Map {
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/c",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/c",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "error": "MISSING",
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
      "to": null,
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/b",
      "to": null,
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "x" => EdgeOut {
      "error": "MISSING",
      "name": "x",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "to": null,
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.0",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces",
  "workspaces": Map {
    "x" => "apps/x",
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > reify the workspaces, foo/x linked, c, old x, removed a 1`] = `
ArboristNode {
  "children": Map {
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/c",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "foox" => ArboristLink {
      "dev": true,
      "extraneous": true,
      "location": "node_modules/foox",
      "name": "foox",
      "optional": true,
      "packageName": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/foox",
      "peer": true,
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
      "resolved": "file:../foo/x",
      "target": ArboristNode {
        "location": "foo/x",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/c",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "",
          "name": "x",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/x",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "resolved": "file:../apps/x",
      "target": ArboristNode {
        "location": "apps/x",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/b",
      "to": null,
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "x" => EdgeOut {
      "error": "INVALID",
      "name": "x",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
      "to": "node_modules/x",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "apps/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "version": "1.2.3",
    },
    ArboristNode {
      "dev": true,
      "extraneous": true,
      "location": "foo/x",
      "name": "x",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
      "peer": true,
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.0",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces",
  "workspaces": Map {
    "x" => "foo/x",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > reify the workspaces, removing a and leaving c and old x in place 1`] = `
ArboristNode {
  "children": Map {
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/c",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/c",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "error": "INVALID",
          "from": "",
          "name": "x",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/x",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "resolved": "file:../apps/x",
      "target": ArboristNode {
        "location": "apps/x",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/b",
      "to": null,
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "x" => EdgeOut {
      "error": "INVALID",
      "name": "x",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/foo/x",
      "to": "node_modules/x",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "apps/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.0",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces",
  "workspaces": Map {
    "x" => "foo/x",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/reify.js TAP filtered reification in workspaces > reify the x workspace after reifying c 1`] = `
ArboristNode {
  "children": Map {
    "c" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "c",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/c",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "resolved": "file:../packages/c",
      "target": ArboristNode {
        "location": "packages/c",
      },
      "version": "1.2.3",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/c",
          "name": "wrappy",
          "spec": "1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "x" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "x",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/node_modules/x",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "resolved": "file:../apps/x",
      "target": ArboristNode {
        "location": "apps/x",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "error": "MISSING",
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/a",
      "to": null,
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/b",
      "to": null,
      "type": "workspace",
    },
    "c" => EdgeOut {
      "name": "c",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "to": "node_modules/c",
      "type": "workspace",
    },
    "x" => EdgeOut {
      "name": "x",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "to": "node_modules/x",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "apps/x",
      "name": "x",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/apps/x",
      "version": "1.2.3",
    },
    ArboristNode {
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1.0.0",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/c",
      "name": "c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces/packages/c",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-filtered-reification-in-workspaces",
  "path": "{CWD}/test/arborist/tap-testdir-reify-filtered-reification-in-workspaces",
  "workspaces": Map {
    "x" => "apps/x",
    "a" => "packages/a",
    "b" => "packages/b",
    "c" => "packages/c",
  },
}
`

exports[`test/arborist/reify.js TAP includeWorkspaceRoot in addition to workspace > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.1",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/packages/b",
      "to": null,
      "type": "workspace",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "*",
          "to": "node_modules/abbrev",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace/packages/a",
      "version": "1.0.1",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace",
  "path": "{CWD}/test/arborist/tap-testdir-reify-includeWorkspaceRoot-in-addition-to-workspace",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP just the shrinkwrap cli-750-fresh > must match snapshot 1`] = `
{
  "name": "monorepo",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "name": "monorepo",
      "dependencies": {
        "app": "file:./app"
      }
    },
    "app": {
      "dependencies": {
        "lib": "file:../lib"
      }
    },
    "lib": {},
    "node_modules/app": {
      "resolved": "app",
      "link": true
    },
    "node_modules/lib": {
      "resolved": "lib",
      "link": true
    }
  }
}

`

exports[`test/arborist/reify.js TAP just the shrinkwrap yarn-lock-mkdirp > must match snapshot 1`] = `
{
  "name": "tap-testdir-reify-just-the-shrinkwrap-yarn-lock-mkdirp",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "mkdirp": "^1.0.2"
      }
    },
    "node_modules/mkdirp": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.2.tgz",
      "integrity": "sha512-N2REVrJ/X/jGPfit2d7zea2J1pf7EAR5chIUcfHffAZ7gmlam5U65sAm76+o4ntQbSRdTjYf7qZz3chuHlwXEA==",
      "bin": {
        "mkdirp": "bin/cmd.js"
      },
      "engines": {
        "node": ">=10"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP link deps already in place > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-deps-already-in-place/node_modules/linked-dep",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-deps-already-in-place/target",
      "resolved": "file:../target",
      "target": ArboristNode {
        "location": "target",
      },
      "version": "1.2.3",
    },
  },
  "edgesOut": Map {
    "linked-dep" => EdgeOut {
      "name": "linked-dep",
      "spec": "file:target",
      "to": "node_modules/linked-dep",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "target",
      "name": "target",
      "packageName": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-deps-already-in-place/target",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-link-deps-already-in-place",
  "packageName": "@isaacs/testing-link-dep",
  "path": "{CWD}/test/arborist/tap-testdir-reify-link-deps-already-in-place",
  "version": "2.0.0",
}
`

exports[`test/arborist/reify.js TAP link meta deps, fresh install > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-link-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dep",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "linked-dep" => EdgeOut {
          "name": "linked-dep",
          "spec": "file:target",
          "to": "node_modules/linked-dep",
          "type": "prod",
        },
      },
      "fsChildren": Set {
        ArboristNode {
          "location": "node_modules/@isaacs/testing-link-dep/target",
          "name": "linked-dep",
          "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install/node_modules/@isaacs/testing-link-dep/target",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dep",
      "name": "@isaacs/testing-link-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install/node_modules/@isaacs/testing-link-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dep/-/testing-link-dep-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-link-dev-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dev-dep",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dev-dep",
      "name": "@isaacs/testing-link-dev-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install/node_modules/@isaacs/testing-link-dev-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dev-dep/-/testing-link-dev-dep-1.0.1.tgz",
      "version": "1.0.1",
    },
    "linked-dep" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-link-dep",
          "name": "linked-dep",
          "spec": "file:target",
          "type": "prod",
        },
      },
      "location": "node_modules/linked-dep",
      "name": "linked-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install/node_modules/linked-dep",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install/node_modules/@isaacs/testing-link-dep/target",
      "resolved": "file:@isaacs/testing-link-dep/target",
      "target": ArboristNode {
        "location": "node_modules/@isaacs/testing-link-dep/target",
      },
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-link-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dep",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-link-dep",
      "type": "prod",
    },
    "@isaacs/testing-link-dev-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dev-dep",
      "spec": "^1.0.1",
      "to": "node_modules/@isaacs/testing-link-dev-dep",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-link-meta-deps-fresh-install",
  "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-fresh-install",
}
`

exports[`test/arborist/reify.js TAP link meta deps, update > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-link-dep" => ArboristNode {
      "children": Map {
        "linked-dep" => ArboristLink {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-link-dep",
              "name": "linked-dep",
              "spec": "file:target",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-link-dep/node_modules/linked-dep",
          "name": "linked-dep",
          "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update/node_modules/@isaacs/testing-link-dep/node_modules/linked-dep",
          "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update/node_modules/@isaacs/testing-link-dep/target",
          "resolved": "file:../target",
          "target": ArboristNode {
            "location": "node_modules/@isaacs/testing-link-dep/target",
          },
          "version": "1.2.3",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dep",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "linked-dep" => EdgeOut {
          "name": "linked-dep",
          "spec": "file:target",
          "to": "node_modules/@isaacs/testing-link-dep/node_modules/linked-dep",
          "type": "prod",
        },
      },
      "fsChildren": Set {
        ArboristNode {
          "location": "node_modules/@isaacs/testing-link-dep/target",
          "name": "target",
          "packageName": "linked-dep",
          "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update/node_modules/@isaacs/testing-link-dep/target",
          "version": "1.2.3",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dep",
      "name": "@isaacs/testing-link-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update/node_modules/@isaacs/testing-link-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dep/-/testing-link-dep-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-link-dev-dep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-link-dev-dep",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-link-dev-dep",
      "name": "@isaacs/testing-link-dev-dep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update/node_modules/@isaacs/testing-link-dev-dep",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-link-dev-dep/-/testing-link-dev-dep-2.0.1.tgz",
      "version": "2.0.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-link-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dep",
      "spec": "^2.0.0",
      "to": "node_modules/@isaacs/testing-link-dep",
      "type": "prod",
    },
    "@isaacs/testing-link-dev-dep" => EdgeOut {
      "name": "@isaacs/testing-link-dev-dep",
      "spec": "^2.0.1",
      "to": "node_modules/@isaacs/testing-link-dev-dep",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-link-meta-deps-update",
  "path": "{CWD}/test/arborist/tap-testdir-reify-link-meta-deps-update",
}
`

exports[`test/arborist/reify.js TAP link metadep cli-750 > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "app" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "app",
          "spec": "file:./app",
          "type": "prod",
        },
      },
      "location": "node_modules/app",
      "name": "app",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/node_modules/app",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/app",
      "resolved": "file:../app",
      "target": ArboristNode {
        "location": "app",
      },
    },
    "lib" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "app",
          "name": "lib",
          "spec": "file:../lib",
          "type": "prod",
        },
      },
      "location": "node_modules/lib",
      "name": "lib",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/node_modules/lib",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/lib",
      "resolved": "file:../lib",
      "target": ArboristNode {
        "location": "lib",
      },
    },
  },
  "edgesOut": Map {
    "app" => EdgeOut {
      "name": "app",
      "spec": "file:./app",
      "to": "node_modules/app",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "lib" => EdgeOut {
          "name": "lib",
          "spec": "file:../lib",
          "to": "node_modules/lib",
          "type": "prod",
        },
      },
      "location": "app",
      "name": "app",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/app",
    },
    ArboristNode {
      "location": "lib",
      "name": "lib",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750/lib",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-link-metadep-cli-750",
  "packageName": "monorepo",
  "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750",
}
`

exports[`test/arborist/reify.js TAP link metadep cli-750-fresh > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "app" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "app",
          "spec": "file:./app",
          "type": "prod",
        },
      },
      "location": "node_modules/app",
      "name": "app",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/node_modules/app",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/app",
      "resolved": "file:../app",
      "target": ArboristNode {
        "location": "app",
      },
    },
    "lib" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "app",
          "name": "lib",
          "spec": "file:../lib",
          "type": "prod",
        },
      },
      "location": "node_modules/lib",
      "name": "lib",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/node_modules/lib",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/lib",
      "resolved": "file:../lib",
      "target": ArboristNode {
        "location": "lib",
      },
    },
  },
  "edgesOut": Map {
    "app" => EdgeOut {
      "name": "app",
      "spec": "file:./app",
      "to": "node_modules/app",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "lib" => EdgeOut {
          "name": "lib",
          "spec": "file:../lib",
          "to": "node_modules/lib",
          "type": "prod",
        },
      },
      "location": "app",
      "name": "app",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/app",
    },
    ArboristNode {
      "location": "lib",
      "name": "lib",
      "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh/lib",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-link-metadep-cli-750-fresh",
  "packageName": "monorepo",
  "path": "{CWD}/test/arborist/tap-testdir-reify-link-metadep-cli-750-fresh",
}
`

exports[`test/arborist/reify.js TAP modules bundled by the root should be installed > must match snapshot 1`] = `
{
  "name": "child",
  "version": "1.2.3"
}

`

exports[`test/arborist/reify.js TAP move aside symlink clutter > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "latest",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-move-aside-symlink-clutter/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "latest",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-move-aside-symlink-clutter",
  "path": "{CWD}/test/arborist/tap-testdir-reify-move-aside-symlink-clutter",
}
`

exports[`test/arborist/reify.js TAP multiple bundles at the same level > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.5.5.tgz",
      "version": "7.5.5",
    },
    "@babel/generator" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/generator",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/generator/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/generator/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/generator/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/helper-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/highlight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.5.0.tgz",
      "version": "7.5.0",
    },
    "@babel/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yaml",
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerator-runtime" => EdgeOut {
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "to": "node_modules/regenerator-runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/runtime",
      "name": "@babel/runtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/runtime",
      "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-b" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "name": "@isaacs/testing-bundledeps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-b",
      "name": "@isaacs/testing-bundledeps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@isaacs/testing-bundledeps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-bundledeps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-b" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-b",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps-b",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-c",
      "name": "@isaacs/testing-bundledeps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/@isaacs/testing-bundledeps-c",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-1.0.0.tgz",
      "version": "1.0.0",
    },
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "optional",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.2.tgz",
      "version": "6.10.2",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
      "version": "3.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "anymatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.2.tgz",
      "version": "4.1.2",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.9.0.tgz",
      "version": "1.9.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "camelcase" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "chokidar" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": "node_modules/fsevents",
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.3.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.3.1.tgz",
      "version": "3.3.1",
    },
    "cliui" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commander" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "commander",
          "spec": "~2.20.3",
          "type": "prod",
        },
      },
      "location": "node_modules/commander",
      "name": "commander",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/commander",
      "resolved": "https://registry.npmjs.org/commander/-/commander-2.20.3.tgz",
      "version": "2.20.3",
    },
    "commondir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/convert-source-map",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/convert-source-map/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/convert-source-map/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/convert-source-map/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.9.tgz",
      "version": "3.0.9",
    },
    "cp-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
      "version": "4.1.1",
    },
    "decamelize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.1.tgz",
      "version": "4.0.1",
    },
    "diff-frag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
      "version": "7.0.3",
    },
    "error-ex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
      "version": "2.0.1",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.114.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.114.0.tgz",
      "version": "0.114.0",
    },
    "flow-remove-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.114.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.114.0.tgz",
      "version": "2.114.0",
    },
    "foreground-child" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fsevents" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "fsevents",
          "spec": "~2.1.2",
          "type": "optional",
        },
      },
      "location": "node_modules/fsevents",
      "name": "fsevents",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/fsevents",
      "resolved": "https://registry.npmjs.org/fsevents/-/fsevents-2.1.2.tgz",
      "version": "2.1.2",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "get-caller-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.0.tgz",
      "version": "5.1.0",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.3.tgz",
      "version": "4.2.3",
    },
    "handlebars" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "handlebars",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.0",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "optimist" => EdgeOut {
          "name": "optimist",
          "spec": "^0.6.1",
          "to": "node_modules/optimist",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "uglify-js" => EdgeOut {
          "name": "uglify-js",
          "spec": "^3.1.4",
          "to": "node_modules/uglify-js",
          "type": "optional",
        },
      },
      "location": "node_modules/handlebars",
      "name": "handlebars",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/handlebars",
      "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.5.3.tgz",
      "version": "4.5.3",
    },
    "har-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.5.5",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
      "version": "5.1.3",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.5.tgz",
      "version": "2.8.5",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-arrayish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "handlebars" => EdgeOut {
          "name": "handlebars",
          "spec": "^4.1.2",
          "to": "node_modules/handlebars",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.6.tgz",
      "version": "2.2.6",
    },
    "jackspeak" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
      "version": "3.13.1",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.15.tgz",
      "version": "4.17.15",
    },
    "lodash.flattendeep" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "loose-envify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
      "version": "1.3.5",
    },
    "merge-source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.42.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.42.0.tgz",
      "version": "1.42.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.42.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.25.tgz",
      "version": "2.1.25",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
      "version": "1.2.0",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.1.tgz",
      "version": "3.1.1",
    },
    "mkdirp" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "node_modules/mkdirp/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/mkdirp/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/mkdirp/node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "neo-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "neo-async",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/neo-async",
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.1.tgz",
      "version": "2.6.1",
    },
    "nested-error-stacks" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-modules-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
      "version": "1.5.1",
    },
    "optimist" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/optimist",
              "name": "minimist",
              "spec": "~0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/optimist/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/optimist/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.10.tgz",
          "version": "0.0.10",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "optimist",
          "spec": "^0.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "~0.0.1",
          "to": "node_modules/optimist/node_modules/minimist",
          "type": "prod",
        },
        "wordwrap" => EdgeOut {
          "name": "wordwrap",
          "spec": "~0.0.2",
          "to": "node_modules/wordwrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optimist",
      "name": "optimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/optimist",
      "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
      "version": "0.6.1",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.1.tgz",
      "version": "2.2.1",
    },
    "p-locate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.1.1.tgz",
      "version": "2.1.1",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "process-nextick-args" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.1.tgz",
      "version": "2.0.1",
    },
    "prop-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.24",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.7.0.tgz",
      "version": "1.7.0",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "react",
          "spec": "^16.12.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/ink",
          "name": "react",
          "spec": ">=16.8.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/react-reconciler",
          "name": "react",
          "spec": "^16.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/treport",
          "name": "react",
          "spec": "^16.8.6",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.6.2",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-16.12.0.tgz",
      "version": "16.12.0",
    },
    "react-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.12.0.tgz",
      "version": "16.12.0",
    },
    "read-pkg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readable-stream" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/readable-stream",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/readable-stream/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/readable-stream/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "readable-stream",
          "spec": "^2.1.5",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "~1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/readable-stream/node_modules/safe-buffer",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~1.1.1",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.6.tgz",
      "version": "2.3.6",
    },
    "readdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.7",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.3.0.tgz",
      "version": "3.3.0",
    },
    "regenerator-runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/runtime",
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-runtime",
      "name": "regenerator-runtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/regenerator-runtime",
      "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.3.tgz",
      "version": "0.13.3",
    },
    "release-zalgo" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.0",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
      "version": "2.88.0",
    },
    "require-directory" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.14.1.tgz",
      "version": "1.14.1",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "resolve-from",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
      "version": "4.0.0",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.0.tgz",
      "version": "5.2.0",
    },
    "safer-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
      "version": "3.0.2",
    },
    "source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/merge-source-map",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "source-map",
          "spec": "^0.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uglify-js",
          "name": "source-map",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
      "version": "0.6.1",
    },
    "source-map-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.16.tgz",
      "version": "0.5.16",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
      "version": "3.1.0",
    },
    "spdx-exceptions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
      "version": "2.2.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
      "version": "3.0.0",
    },
    "spdx-license-ids" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.5.tgz",
      "version": "3.0.5",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string_decoder" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string_decoder",
              "name": "safe-buffer",
              "spec": "~5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string_decoder/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/string_decoder/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "to": "node_modules/string_decoder/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
      "version": "1.1.1",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "string-width",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
      "version": "2.1.1",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^3.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
      "version": "4.0.0",
    },
    "strip-bom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "tap" => ArboristNode {
      "bundleDependencies": Array [
        "@types/react",
        "import-jsx",
        "ink",
        "minipass",
        "signal-exit",
        "tap-parser",
        "tap-yaml",
        "treport",
        "yaml",
      ],
      "children": Map {
        "@babel/code-frame" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/highlight" => EdgeOut {
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/code-frame",
          "name": "@babel/code-frame",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/code-frame",
          "version": "7.5.5",
        },
        "@babel/core" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/core",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/core",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helpers" => EdgeOut {
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helpers",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "to": "node_modules/tap/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "json5" => EdgeOut {
              "name": "json5",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/json5",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.3.2",
              "to": "node_modules/tap/node_modules/resolve",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.4.1",
              "to": "node_modules/tap/node_modules/semver",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/core",
          "name": "@babel/core",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/core",
          "version": "7.7.5",
        },
        "@babel/generator" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "jsesc" => EdgeOut {
              "name": "jsesc",
              "spec": "^2.5.1",
              "to": "node_modules/tap/node_modules/jsesc",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/generator",
          "name": "@babel/generator",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/generator",
          "version": "7.7.4",
        },
        "@babel/helper-builder-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "version": "7.7.4",
        },
        "@babel/helper-function-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/helper-get-function-arity" => EdgeOut {
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-function-name",
          "name": "@babel/helper-function-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helper-function-name",
          "version": "7.7.4",
        },
        "@babel/helper-get-function-arity" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "name": "@babel/helper-get-function-arity",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "version": "7.7.4",
        },
        "@babel/helper-plugin-utils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "name": "@babel/helper-plugin-utils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "version": "7.0.0",
        },
        "@babel/helper-split-export-declaration" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/helper-split-export-declaration",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "version": "7.7.4",
        },
        "@babel/helpers" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helpers",
          "name": "@babel/helpers",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/helpers",
          "version": "7.7.4",
        },
        "@babel/highlight" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight",
                  "name": "chalk",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "version": "2.4.2",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "has-flag" => EdgeOut {
                  "name": "has-flag",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/has-flag",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "version": "5.5.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/code-frame",
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/highlight",
          "name": "@babel/highlight",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/highlight",
          "version": "7.5.0",
        },
        "@babel/parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/parser",
          "name": "@babel/parser",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/parser",
          "version": "7.7.5",
        },
        "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-destructuring" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/plugin-transform-destructuring",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-builder-react-jsx" => EdgeOut {
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "type": "prod",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-jsx" => EdgeOut {
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "version": "7.7.4",
        },
        "@babel/runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/yaml",
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "regenerator-runtime" => EdgeOut {
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "to": "node_modules/tap/node_modules/regenerator-runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/runtime",
          "name": "@babel/runtime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/runtime",
          "version": "7.7.6",
        },
        "@babel/template" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/template",
          "name": "@babel/template",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/template",
          "version": "7.7.4",
        },
        "@babel/traverse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helper-function-name" => EdgeOut {
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-function-name",
              "type": "prod",
            },
            "@babel/helper-split-export-declaration" => EdgeOut {
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "globals" => EdgeOut {
              "name": "globals",
              "spec": "^11.1.0",
              "to": "node_modules/tap/node_modules/globals",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/traverse",
          "name": "@babel/traverse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/traverse",
          "version": "7.7.4",
        },
        "@babel/types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "to-fast-properties" => EdgeOut {
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/to-fast-properties",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/types",
          "name": "@babel/types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@babel/types",
          "version": "7.7.4",
        },
        "@types/color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/color-name",
          "name": "@types/color-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@types/color-name",
          "version": "1.1.1",
        },
        "@types/prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "@types/prop-types",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/prop-types",
          "name": "@types/prop-types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@types/prop-types",
          "version": "15.7.3",
        },
        "@types/react" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "@types/react",
              "spec": "^16.9.16",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/auto-bind",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peerOptional",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@types/prop-types" => EdgeOut {
              "name": "@types/prop-types",
              "spec": "*",
              "to": "node_modules/tap/node_modules/@types/prop-types",
              "type": "prod",
            },
            "csstype" => EdgeOut {
              "name": "csstype",
              "spec": "^2.2.0",
              "to": "node_modules/tap/node_modules/csstype",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/react",
          "name": "@types/react",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/@types/react",
          "version": "16.9.16",
        },
        "ansi-escapes" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/tap/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ansi-escapes",
          "version": "4.3.0",
        },
        "ansi-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ansi-regex",
          "version": "5.0.0",
        },
        "ansi-styles" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/tap/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ansi-styles",
          "version": "3.2.1",
        },
        "ansicolors" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "ansicolors",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansicolors",
          "name": "ansicolors",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ansicolors",
          "version": "0.3.2",
        },
        "arrify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "arrify",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/arrify",
          "name": "arrify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/arrify",
          "version": "2.0.1",
        },
        "astral-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "astral-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/astral-regex",
          "version": "2.0.0",
        },
        "auto-bind" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "auto-bind",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peerOptional",
            },
          },
          "location": "node_modules/tap/node_modules/auto-bind",
          "name": "auto-bind",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/auto-bind",
          "version": "3.0.0",
        },
        "caller-callsite" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-path",
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "callsites" => EdgeOut {
              "name": "callsites",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/callsites",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-callsite",
          "name": "caller-callsite",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/caller-callsite",
          "version": "2.0.0",
        },
        "caller-path" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "caller-path",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "caller-callsite" => EdgeOut {
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-callsite",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-path",
          "name": "caller-path",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/caller-path",
          "version": "2.0.0",
        },
        "callsites" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-callsite",
              "name": "callsites",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/callsites",
          "name": "callsites",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/callsites",
          "version": "2.0.0",
        },
        "cardinal" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "cardinal",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansicolors" => EdgeOut {
              "name": "ansicolors",
              "spec": "~0.3.2",
              "to": "node_modules/tap/node_modules/ansicolors",
              "type": "prod",
            },
            "redeyed" => EdgeOut {
              "name": "redeyed",
              "spec": "~2.1.0",
              "to": "node_modules/tap/node_modules/redeyed",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cardinal",
          "name": "cardinal",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/cardinal",
          "version": "2.1.1",
        },
        "chalk" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/chalk/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/tap/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/chalk",
          "version": "3.0.0",
        },
        "ci-info" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/is-ci",
              "name": "ci-info",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ci-info",
          "name": "ci-info",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ci-info",
          "version": "2.0.0",
        },
        "cli-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/cli-cursor",
          "version": "3.1.0",
        },
        "cli-truncate" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.2.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-truncate",
          "name": "cli-truncate",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/cli-truncate",
          "version": "2.1.0",
        },
        "color-convert" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/tap/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/color-convert",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/color-name",
          "version": "1.1.3",
        },
        "convert-source-map" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "safe-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/convert-source-map",
                  "name": "safe-buffer",
                  "spec": "~5.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "name": "safe-buffer",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "version": "5.1.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "to": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/convert-source-map",
          "name": "convert-source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/convert-source-map",
          "version": "1.7.0",
        },
        "csstype" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "csstype",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/csstype",
          "name": "csstype",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/csstype",
          "version": "2.6.8",
        },
        "debug" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/debug",
          "version": "4.1.1",
        },
        "emoji-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/emoji-regex",
          "version": "8.0.0",
        },
        "escape-string-regexp" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/escape-string-regexp",
          "version": "1.0.5",
        },
        "esprima" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/redeyed",
              "name": "esprima",
              "spec": "~4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/esprima",
          "version": "4.0.1",
        },
        "esutils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "esutils",
              "spec": "^2.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/esutils",
          "version": "2.0.3",
        },
        "events-to-array" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "events-to-array",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/events-to-array",
          "name": "events-to-array",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/events-to-array",
          "version": "1.1.2",
        },
        "globals" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "globals",
              "spec": "^11.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/globals",
          "version": "11.12.0",
        },
        "has-flag" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/has-flag",
          "version": "3.0.0",
        },
        "import-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "prod",
            },
            "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "type": "prod",
            },
            "@babel/plugin-transform-destructuring" => EdgeOut {
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "type": "prod",
            },
            "@babel/plugin-transform-react-jsx" => EdgeOut {
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "type": "prod",
            },
            "caller-path" => EdgeOut {
              "name": "caller-path",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-path",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/resolve-from",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/import-jsx",
          "name": "import-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/import-jsx",
          "version": "3.0.0",
        },
        "ink" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peer",
            },
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "to": "node_modules/tap/node_modules/ansi-escapes",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^2.0.1",
              "to": "node_modules/tap/node_modules/arrify",
              "type": "prod",
            },
            "auto-bind" => EdgeOut {
              "name": "auto-bind",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/auto-bind",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/cli-cursor",
              "type": "prod",
            },
            "cli-truncate" => EdgeOut {
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/cli-truncate",
              "type": "prod",
            },
            "is-ci" => EdgeOut {
              "name": "is-ci",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/is-ci",
              "type": "prod",
            },
            "lodash.throttle" => EdgeOut {
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/lodash.throttle",
              "type": "prod",
            },
            "log-update" => EdgeOut {
              "name": "log-update",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/log-update",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": ">=16.8.0",
              "to": "node_modules/react",
              "type": "peer",
            },
            "react-reconciler" => EdgeOut {
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "to": "node_modules/tap/node_modules/react-reconciler",
              "type": "prod",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "widest-line" => EdgeOut {
              "name": "widest-line",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/widest-line",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "to": "node_modules/tap/node_modules/wrap-ansi",
              "type": "prod",
            },
            "yoga-layout-prebuilt" => EdgeOut {
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "to": "node_modules/tap/node_modules/yoga-layout-prebuilt",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ink",
          "name": "ink",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ink",
          "version": "2.6.0",
        },
        "is-ci" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "is-ci",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ci-info" => EdgeOut {
              "name": "ci-info",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/ci-info",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-ci",
          "name": "is-ci",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/is-ci",
          "version": "2.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/is-fullwidth-code-point",
          "version": "3.0.0",
        },
        "js-tokens" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "js-tokens",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/loose-envify",
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/js-tokens",
          "name": "js-tokens",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/js-tokens",
          "version": "4.0.0",
        },
        "jsesc" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "jsesc",
              "spec": "^2.5.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/jsesc",
          "name": "jsesc",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/jsesc",
          "version": "2.5.2",
        },
        "json5" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "json5",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/json5",
          "name": "json5",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/json5",
          "version": "2.1.1",
        },
        "lodash" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/lodash",
          "version": "4.17.15",
        },
        "lodash.throttle" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash.throttle",
          "name": "lodash.throttle",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/lodash.throttle",
          "version": "4.1.1",
        },
        "log-update" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-escapes" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "ansi-escapes",
                  "spec": "^3.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "name": "ansi-escapes",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "version": "3.2.0",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "cli-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "cli-cursor",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "restore-cursor" => EdgeOut {
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "name": "cli-cursor",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "version": "2.1.0",
            },
            "emoji-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "name": "emoji-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "version": "7.0.3",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "version": "2.0.0",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "version": "1.2.0",
            },
            "onetime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/onetime",
              "name": "onetime",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/onetime",
              "version": "2.0.1",
            },
            "restore-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "onetime" => EdgeOut {
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "restore-cursor",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "version": "2.0.0",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "emoji-regex" => EdgeOut {
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/string-width",
              "version": "3.1.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "version": "5.2.0",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "wrap-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "version": "5.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "log-update",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/log-update",
          "name": "log-update",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/log-update",
          "version": "3.3.0",
        },
        "loose-envify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "loose-envify",
              "spec": "^1.4.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/loose-envify",
          "name": "loose-envify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/loose-envify",
          "version": "1.4.0",
        },
        "mimic-fn" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/mimic-fn",
          "version": "2.1.0",
        },
        "minimist" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/json5",
              "name": "minimist",
              "spec": "^1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/minimist",
          "version": "1.2.0",
        },
        "minipass" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/minipass",
                  "name": "yallist",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/minipass/node_modules/yallist",
              "version": "4.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "minipass",
              "spec": "^3.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "minipass",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minipass",
          "name": "minipass",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/minipass",
          "version": "3.1.1",
        },
        "ms" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/debug",
              "name": "ms",
              "spec": "^2.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ms",
              "spec": "^2.1.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/ms",
          "version": "2.1.2",
        },
        "object-assign" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/object-assign",
          "name": "object-assign",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/object-assign",
          "version": "4.1.1",
        },
        "onetime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/onetime",
          "version": "5.1.0",
        },
        "path-parse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/resolve",
              "name": "path-parse",
              "spec": "^1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/path-parse",
          "name": "path-parse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/path-parse",
          "version": "1.0.6",
        },
        "prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.4.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "react-is" => EdgeOut {
              "name": "react-is",
              "spec": "^16.8.1",
              "to": "node_modules/tap/node_modules/react-is",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/prop-types",
          "name": "prop-types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/prop-types",
          "version": "15.7.2",
        },
        "punycode" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "punycode",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/punycode",
          "version": "2.1.1",
        },
        "react-is" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "react-is",
              "spec": "^16.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-is",
          "name": "react-is",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/react-is",
          "version": "16.12.0",
        },
        "react-reconciler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.0.0",
              "to": "node_modules/react",
              "type": "peer",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-reconciler",
          "name": "react-reconciler",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/react-reconciler",
          "version": "0.24.0",
        },
        "redeyed" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "redeyed",
              "spec": "~2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "~4.0.0",
              "to": "node_modules/tap/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/redeyed",
          "name": "redeyed",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/redeyed",
          "version": "2.1.1",
        },
        "regenerator-runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/runtime",
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/regenerator-runtime",
          "name": "regenerator-runtime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/regenerator-runtime",
          "version": "0.13.3",
        },
        "resolve" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "resolve",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "path-parse" => EdgeOut {
              "name": "path-parse",
              "spec": "^1.0.6",
              "to": "node_modules/tap/node_modules/path-parse",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve",
          "name": "resolve",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/resolve",
          "version": "1.13.1",
        },
        "resolve-from" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "resolve-from",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/resolve-from",
          "version": "3.0.0",
        },
        "restore-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^5.1.0",
              "to": "node_modules/tap/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/restore-cursor",
          "version": "3.1.0",
        },
        "scheduler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/scheduler",
          "name": "scheduler",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/scheduler",
          "version": "0.18.0",
        },
        "semver" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "semver",
              "spec": "^5.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/semver",
          "version": "5.7.1",
        },
        "signal-exit" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/signal-exit",
          "version": "3.0.2",
        },
        "slice-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/astral-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/slice-ansi",
          "name": "slice-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/slice-ansi",
          "version": "3.0.0",
        },
        "string-length" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "astral-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "astral-regex",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "name": "astral-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "version": "1.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "strip-ansi",
                  "spec": "^5.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "version": "5.2.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-length",
          "name": "string-length",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/string-length",
          "version": "3.1.0",
        },
        "string-width" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "string-width",
              "spec": "^4.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/widest-line",
              "name": "string-width",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/tap/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/string-width",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/strip-ansi",
          "version": "6.0.0",
        },
        "supports-color" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "version": "4.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/supports-color",
          "version": "7.1.0",
        },
        "tap-parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "events-to-array" => EdgeOut {
              "name": "events-to-array",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/events-to-array",
              "type": "prod",
            },
            "minipass" => EdgeOut {
              "name": "minipass",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/minipass",
              "type": "prod",
            },
            "tap-yaml" => EdgeOut {
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/tap-yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-parser",
          "name": "tap-parser",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/tap-parser",
          "version": "10.0.1",
        },
        "tap-yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yaml" => EdgeOut {
              "name": "yaml",
              "spec": "^1.5.0",
              "to": "node_modules/tap/node_modules/yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-yaml",
          "name": "tap-yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/tap-yaml",
          "version": "1.0.0",
        },
        "to-fast-properties" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/to-fast-properties",
          "name": "to-fast-properties",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/to-fast-properties",
          "version": "2.0.0",
        },
        "treport" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "unicode-length" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport",
                  "name": "unicode-length",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "punycode" => EdgeOut {
                  "name": "punycode",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/punycode",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "unicode-length",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "version": "2.0.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "treport",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cardinal" => EdgeOut {
              "name": "cardinal",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/cardinal",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "import-jsx" => EdgeOut {
              "name": "import-jsx",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/import-jsx",
              "type": "prod",
            },
            "ink" => EdgeOut {
              "name": "ink",
              "spec": "^2.5.0",
              "to": "node_modules/tap/node_modules/ink",
              "type": "prod",
            },
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.2",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.8.6",
              "to": "node_modules/react",
              "type": "peer",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^10.0.1",
              "to": "node_modules/tap/node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/treport",
          "name": "treport",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/treport",
          "version": "1.0.1",
        },
        "type-fest" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-escapes",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/type-fest",
          "version": "0.8.1",
        },
        "widest-line" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "widest-line",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/widest-line",
          "name": "widest-line",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/widest-line",
          "version": "3.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/wrap-ansi",
          "version": "6.2.0",
        },
        "yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "yaml",
              "spec": "^1.7.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-yaml",
              "name": "yaml",
              "spec": "^1.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/runtime" => EdgeOut {
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "to": "node_modules/tap/node_modules/@babel/runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yaml",
          "name": "yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/yaml",
          "version": "1.7.2",
        },
        "yoga-layout-prebuilt" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yoga-layout-prebuilt",
          "name": "yoga-layout-prebuilt",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap/node_modules/yoga-layout-prebuilt",
          "version": "1.9.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "^14.10.5",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": "^16.9.16",
          "to": "node_modules/tap/node_modules/@types/react",
          "type": "prod",
        },
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.8",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/tap/node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/tap/node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap/node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap/node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "name": "treport",
          "spec": "^1.0.1",
          "to": "node_modules/tap/node_modules/treport",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/tap/node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.5.tgz",
      "version": "14.10.5",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "debug",
              "spec": "^2.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/tap-mocha-reporter/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-mocha-reporter/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "diff" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "diff",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/diff",
          "name": "diff",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-mocha-reporter/node_modules/diff",
          "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
          "version": "1.4.0",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-mocha-reporter/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.3",
          "to": "node_modules/tap-mocha-reporter/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^1.3.2",
          "to": "node_modules/tap-mocha-reporter/node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^1.0.0",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.0.tgz",
      "version": "5.0.0",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.0.1.tgz",
      "version": "10.0.1",
    },
    "tap-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tough-cookie",
              "name": "punycode",
              "spec": "^1.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tough-cookie/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tough-cookie/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.24",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/tough-cookie/node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
      "version": "2.4.3",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "^3.0.0",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.5.4.tgz",
      "version": "8.5.4",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typedarray-to-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.7.4.tgz",
      "version": "3.7.4",
    },
    "uglify-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "uglify-js",
          "spec": "^3.1.4",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "~2.20.3",
          "to": "node_modules/commander",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "~0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/uglify-js",
      "name": "uglify-js",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/uglify-js",
      "resolved": "https://registry.npmjs.org/uglify-js/-/uglify-js-3.7.3.tgz",
      "version": "3.7.3",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "punycode",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/unicode-length/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/unicode-length/node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
      "version": "4.2.2",
    },
    "util-deprecate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
      "version": "1.0.2",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.3.tgz",
      "version": "3.3.3",
    },
    "validate-npm-package-license" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optimist",
          "name": "wordwrap",
          "spec": "~0.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
      "version": "0.0.3",
    },
    "wrap-ansi" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrap-ansi/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "number-is-nan" => EdgeOut {
              "name": "number-is-nan",
              "spec": "^1.0.0",
              "to": "node_modules/number-is-nan",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
          "version": "1.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "code-point-at" => EdgeOut {
              "name": "code-point-at",
              "spec": "^1.0.0",
              "to": "node_modules/code-point-at",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^1.0.0",
              "to": "node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrap-ansi/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
          "version": "1.0.2",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/wrap-ansi/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/wrap-ansi/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrap-ansi/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui",
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.1",
          "to": "node_modules/wrap-ansi/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/wrap-ansi/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/wrap-ansi",
      "name": "wrap-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrap-ansi",
      "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
      "version": "2.1.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.1.tgz",
      "version": "3.0.1",
    },
    "y18n" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.7.2.tgz",
      "version": "1.7.2",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.0.tgz",
      "version": "13.3.0",
    },
    "yargs-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.1.tgz",
      "version": "13.1.1",
    },
    "yn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "optional",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "^14.10.5",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-multiple-bundles-at-the-same-level",
  "packageName": "two-bundled-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-multiple-bundles-at-the-same-level",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP no saveType: dev w/ compatible optional > must match snapshot 1`] = `
{"optionalDependencies":{"abbrev":"*"},"devDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: dev w/ compatible peer > must match snapshot 1`] = `
{"peerDependencies":{"abbrev":"*"},"devDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: dev w/ incompatible optional > must match snapshot 1`] = `
{"optionalDependencies":{"abbrev":"^1.1.1"},"devDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: dev w/ incompatible peer > must match snapshot 1`] = `
{"peerDependencies":{"abbrev":"^1.1.1"},"devDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: optional only > must match snapshot 1`] = `
{"optionalDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: peer only > must match snapshot 1`] = `
{"peerDependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no saveType: prod w/ peer > must match snapshot 1`] = `
{"dependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP no workspace > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-no-workspace/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-no-workspace/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "error": "MISSING",
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-no-workspace/packages/a",
      "to": null,
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-no-workspace/packages/b",
      "to": null,
      "type": "workspace",
    },
    "once" => EdgeOut {
      "name": "once",
      "spec": "*",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-no-workspace",
  "path": "{CWD}/test/arborist/tap-testdir-reify-no-workspace",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP node_modules may not be a symlink > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-node_modules-may-not-be-a-symlink/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-node_modules-may-not-be-a-symlink",
  "path": "{CWD}/test/arborist/tap-testdir-reify-node_modules-may-not-be-a-symlink",
}
`

exports[`test/arborist/reify.js TAP node_modules may not be a symlink > must match snapshot 2`] = `
Array [
  Array [
    "warn",
    "reify",
    "Removing non-directory",
    "{CWD}/test/arborist/tap-testdir-reify-node_modules-may-not-be-a-symlink/node_modules",
  ],
]
`

exports[`test/arborist/reify.js TAP omit peer deps > finished timers 1`] = `
Array [
  "arborist:ctor",
  "arborist:ctor",
  "build",
  "build:deps",
  "build:queue",
  "idealTree",
  "idealTree:#root",
  "idealTree:buildDeps",
  "idealTree:fixDepFlags",
  "idealTree:init",
  "idealTree:node_modules/@isaacs/testing-peer-deps-b",
  "idealTree:node_modules/@isaacs/testing-peer-deps-c",
  "idealTree:node_modules/@isaacs/testing-peer-deps-d",
  "idealTree:node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
  "idealTree:node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
  "idealTree:node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
  "idealTree:userRequests",
  "reify",
  "reify:build",
  "reify:createSparse",
  "reify:diffTrees",
  "reify:loadBundles",
  "reify:loadTrees",
  "reify:retireShallow",
  "reify:save",
  "reify:trash",
  "reify:trashOmits",
  "reify:unpack",
  "reify:unretire",
  "reifyNode:node_modules/@isaacs/testing-peer-deps-b",
  "reifyNode:node_modules/@isaacs/testing-peer-deps-d",
  "reifyNode:node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
]
`

exports[`test/arborist/reify.js TAP omits when both dev and optional flags are set dev > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "error": "MISSING",
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": null,
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-omits-when-both-dev-and-optional-flags-are-set-dev",
  "packageName": "@isaacs/testing-dev-optional-flags-2",
  "path": "{CWD}/test/arborist/tap-testdir-reify-omits-when-both-dev-and-optional-flags-are-set-dev",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP omits when both dev and optional flags are set optional > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-dev-optional-flags" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-dev-optional-flags",
          "spec": "*",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": null,
          "type": "optional",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "^1.0.2",
          "to": null,
          "type": "optional",
        },
      },
      "location": "node_modules/@isaacs/testing-dev-optional-flags",
      "name": "@isaacs/testing-dev-optional-flags",
      "path": "{CWD}/test/arborist/tap-testdir-reify-omits-when-both-dev-and-optional-flags-are-set-optional/node_modules/@isaacs/testing-dev-optional-flags",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-dev-optional-flags/-/testing-dev-optional-flags-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-dev-optional-flags" => EdgeOut {
      "name": "@isaacs/testing-dev-optional-flags",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-dev-optional-flags",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-omits-when-both-dev-and-optional-flags-are-set-optional",
  "packageName": "@isaacs/testing-dev-optional-flags-2",
  "path": "{CWD}/test/arborist/tap-testdir-reify-omits-when-both-dev-and-optional-flags-are-set-optional",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-allinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-allinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-allinstall",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-allinstall-fail-save-false",
  "packageName": "optional-dep-allinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-allinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-allinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-allinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-allinstall",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-allinstall-fail-save-true",
  "packageName": "optional-dep-allinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-allinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-install-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-install" => EdgeOut {
      "name": "@isaacs/testing-fail-install",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-install-fail-save-false",
  "packageName": "optional-dep-install-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-install-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-install-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-install" => EdgeOut {
      "name": "@isaacs/testing-fail-install",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-install-fail-save-true",
  "packageName": "optional-dep-install-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-install-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-postinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-postinstall" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-postinstall",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-postinstall",
      "name": "@isaacs/testing-fail-postinstall",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-false/node_modules/@isaacs/testing-fail-postinstall",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-postinstall/-/testing-fail-postinstall-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-postinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-postinstall",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-postinstall",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-false",
  "packageName": "optional-dep-postinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-postinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-fail-postinstall" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-fail-postinstall",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "location": "node_modules/@isaacs/testing-fail-postinstall",
      "name": "@isaacs/testing-fail-postinstall",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-true/node_modules/@isaacs/testing-fail-postinstall",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-fail-postinstall/-/testing-fail-postinstall-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-fail-postinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-postinstall",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/testing-fail-postinstall",
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-true",
  "packageName": "optional-dep-postinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-postinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-preinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-preinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-preinstall",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-preinstall-fail-save-false",
  "packageName": "optional-dep-preinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-preinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-preinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-fail-preinstall" => EdgeOut {
      "name": "@isaacs/testing-fail-preinstall",
      "spec": "^1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-preinstall-fail-save-true",
  "packageName": "optional-dep-preinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-preinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-tgz-missing save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-missing-tgz" => EdgeOut {
      "name": "@isaacs/testing-missing-tgz",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-tgz-missing-save-false",
  "packageName": "@isaacs/testing-optional-dep-tgz-missing",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-tgz-missing-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-dep-tgz-missing save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-missing-tgz" => EdgeOut {
      "name": "@isaacs/testing-missing-tgz",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-dep-tgz-missing-save-true",
  "packageName": "@isaacs/testing-optional-dep-tgz-missing",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-dep-tgz-missing-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-allinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-allinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-allinstall-fail",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-allinstall-fail-save-false",
  "packageName": "optional-metadep-allinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-allinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-allinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-allinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-allinstall-fail",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-allinstall-fail-save-true",
  "packageName": "optional-metadep-allinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-allinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-install-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-install-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-install-fail",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-install-fail-save-false",
  "packageName": "optional-metadep-install-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-install-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-install-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-install-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-install-fail",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-install-fail-save-true",
  "packageName": "optional-metadep-install-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-install-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-postinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-postinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-postinstall-fail",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-postinstall-fail-save-false",
  "packageName": "optional-metadep-postinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-postinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-postinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-postinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-postinstall-fail",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-postinstall-fail-save-true",
  "packageName": "optional-metadep-postinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-postinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-preinstall-fail save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-preinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-preinstall-fail",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-preinstall-fail-save-false",
  "packageName": "optional-metadep-preinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-preinstall-fail-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-preinstall-fail save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-preinstall-fail" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-preinstall-fail",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-preinstall-fail-save-true",
  "packageName": "optional-metadep-preinstall-fail",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-preinstall-fail-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-tgz-missing save=false > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-tgz-missing" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-tgz-missing",
      "spec": "*",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-tgz-missing-save-false",
  "packageName": "@isaacs/testing-optional-metadep-tgz-missing",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-tgz-missing-save-false",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP optional dependency failures optional-metadep-tgz-missing save=true > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "@isaacs/testing-prod-dep-tgz-missing" => EdgeOut {
      "name": "@isaacs/testing-prod-dep-tgz-missing",
      "spec": "^1.0.1",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-optional-dependency-failures-optional-metadep-tgz-missing-save-true",
  "packageName": "@isaacs/testing-optional-metadep-tgz-missing",
  "path": "{CWD}/test/arborist/tap-testdir-reify-optional-dependency-failures-optional-metadep-tgz-missing-save-true",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP packageLockOnly can add deps > must match snapshot 1`] = `
{
  "dependencies": {
    "abbrev": "^1.1.1"
  }
}

`

exports[`test/arborist/reify.js TAP packageLockOnly can add deps > must match snapshot 2`] = `
{
  "name": "tap-testdir-reify-packageLockOnly-can-add-deps",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP project with bundled deps and a link dep on itself > result 1`] = `
ArboristNode {
  "bundleDependencies": Array [
    "abbrev",
  ],
  "children": Map {
    "@isaacs/testing-bundle-self-link" => ArboristLink {
      "bundleDependencies": Array [
        "abbrev",
      ],
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundle-self-link",
          "spec": "file:.",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundle-self-link",
      "name": "@isaacs/testing-bundle-self-link",
      "path": "{CWD}/test/arborist/tap-testdir-reify-project-with-bundled-deps-and-a-link-dep-on-itself/node_modules/@isaacs/testing-bundle-self-link",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-project-with-bundled-deps-and-a-link-dep-on-itself",
      "resolved": "file:../..",
      "target": ArboristNode {
        "location": "",
      },
      "version": "1.0.0",
    },
    "abbrev" => ArboristNode {
      "bundled": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "*",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-project-with-bundled-deps-and-a-link-dep-on-itself/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundle-self-link" => EdgeOut {
      "name": "@isaacs/testing-bundle-self-link",
      "spec": "file:.",
      "to": "node_modules/@isaacs/testing-bundle-self-link",
      "type": "prod",
    },
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "*",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-project-with-bundled-deps-and-a-link-dep-on-itself",
  "packageName": "@isaacs/testing-bundle-self-link",
  "path": "{CWD}/test/arborist/tap-testdir-reify-project-with-bundled-deps-and-a-link-dep-on-itself",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP reify audit only workspace deps when reifying workspace > resulting tree 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.2.3",
    },
    "kind-of" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "packages/a",
          "name": "kind-of",
          "spec": "6.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/kind-of",
      "name": "kind-of",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/node_modules/kind-of",
      "resolved": "https://registry.npmjs.org/kind-of/-/kind-of-6.0.0.tgz",
      "version": "6.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "error": "MISSING",
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/packages/b",
      "to": null,
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "kind-of" => EdgeOut {
          "name": "kind-of",
          "spec": "6.0.0",
          "to": "node_modules/kind-of",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace/packages/a",
      "version": "1.2.3",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-audit-only-workspace-deps-when-reifying-workspace",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP reify from old package-lock with bins > should add bins entry to package-lock packages entry 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "old-package-lock-with-bins",
  "packages": Object {
    "": Object {
      "dependencies": Object {
        "ruy": "^1.0.0",
      },
      "name": "old-package-lock-with-bins",
      "version": "1.0.0",
    },
    "node_modules/ruy": Object {
      "bin": Object {
        "ruy": "bin/index.js",
      },
      "integrity": "sha512-VYppDTCM6INWUMKlWiKws4nVMuCNU5h+xjF6lj/0y90rLq017/m8aEpNy4zQSZFV2qz66U/hRZwwlSLJ5l5JMQ==",
      "resolved": "https://registry.npmjs.org/ruy/-/ruy-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "requires": true,
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP reify properly with all deps when lockfile is ancient > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "acorn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn",
          "spec": "4.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn",
      "name": "acorn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/acorn",
      "resolved": "https://registry.npmjs.org/acorn/-/acorn-4.0.4.tgz",
      "version": "4.0.4",
    },
    "acorn-jsx" => ArboristNode {
      "children": Map {
        "acorn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/acorn-jsx",
              "name": "acorn",
              "spec": "^3.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/acorn-jsx/node_modules/acorn",
          "name": "acorn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/acorn-jsx/node_modules/acorn",
          "resolved": "https://registry.npmjs.org/acorn/-/acorn-3.3.0.tgz",
          "version": "3.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/espree",
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "^3.0.4",
          "to": "node_modules/acorn-jsx/node_modules/acorn",
          "type": "prod",
        },
      },
      "location": "node_modules/acorn-jsx",
      "name": "acorn-jsx",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/acorn-jsx",
      "resolved": "https://registry.npmjs.org/acorn-jsx/-/acorn-jsx-3.0.1.tgz",
      "version": "3.0.1",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv-keywords",
          "name": "ajv",
          "spec": ">=4.10.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv",
          "spec": "^4.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "co" => EdgeOut {
          "name": "co",
          "spec": "^4.6.0",
          "to": "node_modules/co",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-4.11.2.tgz",
      "version": "4.11.2",
    },
    "ajv-keywords" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": ">=4.10.0",
          "to": "node_modules/ajv",
          "type": "peer",
        },
      },
      "location": "node_modules/ajv-keywords",
      "name": "ajv-keywords",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ajv-keywords",
      "resolved": "https://registry.npmjs.org/ajv-keywords/-/ajv-keywords-1.5.1.tgz",
      "version": "1.5.1",
    },
    "ansi-escapes" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-escapes",
      "name": "ansi-escapes",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ansi-escapes",
      "resolved": "https://registry.npmjs.org/ansi-escapes/-/ansi-escapes-1.4.0.tgz",
      "version": "1.4.0",
    },
    "ansi-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/has-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.0.0.tgz",
      "version": "2.0.0",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.1.0.tgz",
      "version": "2.1.0",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.9.tgz",
      "version": "1.0.9",
    },
    "array-union" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "array-union",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-uniq" => EdgeOut {
          "name": "array-uniq",
          "spec": "^1.0.1",
          "to": "node_modules/array-uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/array-union",
      "name": "array-union",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/array-union",
      "resolved": "https://registry.npmjs.org/array-union/-/array-union-1.0.2.tgz",
      "version": "1.0.2",
    },
    "array-uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/array-union",
          "name": "array-uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/array-uniq",
      "name": "array-uniq",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/array-uniq",
      "resolved": "https://registry.npmjs.org/array-uniq/-/array-uniq-1.0.3.tgz",
      "version": "1.0.3",
    },
    "arrify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/globby",
          "name": "arrify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arrify",
      "name": "arrify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/arrify",
      "resolved": "https://registry.npmjs.org/arrify/-/arrify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.3.tgz",
      "version": "0.2.3",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-0.2.0.tgz",
      "version": "0.2.0",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.6.0.tgz",
      "version": "0.6.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.6.0.tgz",
      "version": "1.6.0",
    },
    "babel-code-frame" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/babel-code-frame",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/babel-code-frame/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/babel-code-frame/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/babel-code-frame/node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/babel-code-frame",
      "name": "babel-code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/babel-code-frame",
      "resolved": "https://registry.npmjs.org/babel-code-frame/-/babel-code-frame-6.22.0.tgz",
      "version": "6.22.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^0.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-0.3.0.tgz",
      "version": "0.3.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.1.tgz",
      "version": "1.0.1",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bluebird" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bluebird",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/bluebird",
      "name": "bluebird",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/bluebird",
      "resolved": "https://registry.npmjs.org/bluebird/-/bluebird-3.5.0.tgz",
      "version": "3.5.0",
    },
    "boom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cryptiles",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "boom",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/boom",
      "name": "boom",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/boom",
      "resolved": "https://registry.npmjs.org/boom/-/boom-2.10.1.tgz",
      "version": "2.10.1",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^0.3.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.3.tgz",
      "version": "1.1.3",
    },
    "buffer-shims" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "buffer-shims",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-shims",
      "name": "buffer-shims",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/buffer-shims",
      "resolved": "https://registry.npmjs.org/buffer-shims/-/buffer-shims-1.0.0.tgz",
      "version": "1.0.0",
    },
    "caller-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "caller-path",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "callsites" => EdgeOut {
          "name": "callsites",
          "spec": "^0.2.0",
          "to": "node_modules/callsites",
          "type": "prod",
        },
      },
      "location": "node_modules/caller-path",
      "name": "caller-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/caller-path",
      "resolved": "https://registry.npmjs.org/caller-path/-/caller-path-0.1.0.tgz",
      "version": "0.1.0",
    },
    "callsites" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caller-path",
          "name": "callsites",
          "spec": "^0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/callsites",
      "name": "callsites",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/callsites",
      "resolved": "https://registry.npmjs.org/callsites/-/callsites-0.2.0.tgz",
      "version": "0.2.0",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.11.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.11.0.tgz",
      "version": "0.11.0",
    },
    "chalk" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/chalk",
              "name": "supports-color",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/chalk/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/chalk/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "chalk",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "chalk",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table",
          "name": "chalk",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^2.1.0",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "has-ansi" => EdgeOut {
          "name": "has-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/has-ansi",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^2.0.0",
          "to": "node_modules/chalk/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.1.tgz",
      "version": "1.1.1",
    },
    "circular-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "circular-json",
          "spec": "^0.3.1",
          "type": "prod",
        },
      },
      "location": "node_modules/circular-json",
      "name": "circular-json",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/circular-json",
      "resolved": "https://registry.npmjs.org/circular-json/-/circular-json-0.3.1.tgz",
      "version": "0.3.1",
    },
    "clean-yaml-object" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/clean-yaml-object",
      "name": "clean-yaml-object",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/clean-yaml-object",
      "resolved": "https://registry.npmjs.org/clean-yaml-object/-/clean-yaml-object-0.1.0.tgz",
      "version": "0.1.0",
    },
    "cli-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "restore-cursor" => EdgeOut {
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/restore-cursor",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-cursor",
      "name": "cli-cursor",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cli-cursor",
      "resolved": "https://registry.npmjs.org/cli-cursor/-/cli-cursor-1.0.2.tgz",
      "version": "1.0.2",
    },
    "cli-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "cli-width",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/cli-width",
      "name": "cli-width",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cli-width",
      "resolved": "https://registry.npmjs.org/cli-width/-/cli-width-2.1.0.tgz",
      "version": "2.1.0",
    },
    "co" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "co",
          "spec": "^4.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/co",
      "name": "co",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/co",
      "resolved": "https://registry.npmjs.org/co/-/co-4.6.0.tgz",
      "version": "4.6.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.0.0.tgz",
      "version": "1.0.0",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.2.tgz",
      "version": "1.1.2",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.5.tgz",
      "version": "1.0.5",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "concat-stream" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "inherits",
              "spec": "^2.0.3",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/inherits",
          "name": "inherits",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-stream/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-stream/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-stream/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "readable-stream" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/concat-stream",
              "name": "readable-stream",
              "spec": "^2.2.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "buffer-shims" => EdgeOut {
              "name": "buffer-shims",
              "spec": "^1.0.0",
              "to": "node_modules/buffer-shims",
              "type": "prod",
            },
            "core-util-is" => EdgeOut {
              "name": "core-util-is",
              "spec": "~1.0.0",
              "to": "node_modules/core-util-is",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "~2.0.1",
              "to": "node_modules/concat-stream/node_modules/inherits",
              "type": "prod",
            },
            "isarray" => EdgeOut {
              "name": "isarray",
              "spec": "~1.0.0",
              "to": "node_modules/concat-stream/node_modules/isarray",
              "type": "prod",
            },
            "process-nextick-args" => EdgeOut {
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "to": "node_modules/concat-stream/node_modules/process-nextick-args",
              "type": "prod",
            },
            "string_decoder" => EdgeOut {
              "name": "string_decoder",
              "spec": "~0.10.x",
              "to": "node_modules/string_decoder",
              "type": "prod",
            },
            "util-deprecate" => EdgeOut {
              "name": "util-deprecate",
              "spec": "~1.0.1",
              "to": "node_modules/util-deprecate",
              "type": "prod",
            },
          },
          "location": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "readable-stream",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-stream/node_modules/readable-stream",
          "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.2.2.tgz",
          "version": "2.2.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "concat-stream",
          "spec": "^1.4.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "^2.0.3",
          "to": "node_modules/concat-stream/node_modules/inherits",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.2.2",
          "to": "node_modules/concat-stream/node_modules/readable-stream",
          "type": "prod",
        },
        "typedarray" => EdgeOut {
          "name": "typedarray",
          "spec": "^0.0.6",
          "to": "node_modules/typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-stream",
      "name": "concat-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/concat-stream",
      "resolved": "https://registry.npmjs.org/concat-stream/-/concat-stream-1.6.0.tgz",
      "version": "1.6.0",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "children": Map {
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls/node_modules/js-yaml",
              "name": "esprima",
              "spec": "^2.6.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/coveralls/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.7.3.tgz",
          "version": "2.7.3",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "js-yaml",
              "spec": "3.6.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^2.6.0",
              "to": "node_modules/coveralls/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/coveralls/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.6.1.tgz",
          "version": "3.6.1",
        },
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/coveralls",
              "name": "minimist",
              "spec": "1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/coveralls/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/coveralls/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^2.11.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "3.6.1",
          "to": "node_modules/coveralls/node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "0.0.10",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "1.2.5",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "1.2.0",
          "to": "node_modules/coveralls/node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "2.79.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-2.11.16.tgz",
      "version": "2.11.16",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "lru-cache" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "lru-cache",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pseudomap" => EdgeOut {
              "name": "pseudomap",
              "spec": "^1.0.1",
              "to": "node_modules/pseudomap",
              "type": "prod",
            },
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^2.0.0",
              "to": "node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "lru-cache",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cross-spawn/node_modules/lru-cache",
          "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.0.2.tgz",
          "version": "4.0.2",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^1.1.1",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.2.12.tgz",
          "version": "1.2.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/cross-spawn/node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "cryptiles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "cryptiles",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
      },
      "location": "node_modules/cryptiles",
      "name": "cryptiles",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/cryptiles",
      "resolved": "https://registry.npmjs.org/cryptiles/-/cryptiles-2.0.5.tgz",
      "version": "2.0.5",
    },
    "d" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "d",
          "spec": "^0.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "d",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.2",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/d",
      "name": "d",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/d",
      "resolved": "https://registry.npmjs.org/d/-/d-0.1.1.tgz",
      "version": "0.1.1",
    },
    "dashdash" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/dashdash",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/dashdash/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/dashdash/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/dashdash/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "debug",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "debug",
          "spec": "^2.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "0.7.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-2.2.0.tgz",
      "version": "2.2.0",
    },
    "debug-log" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "debug-log",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/debug-log",
      "name": "debug-log",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/debug-log",
      "resolved": "https://registry.npmjs.org/debug-log/-/debug-log-1.0.1.tgz",
      "version": "1.0.1",
    },
    "deep-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "deep-is",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/deep-is",
      "name": "deep-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/deep-is",
      "resolved": "https://registry.npmjs.org/deep-is/-/deep-is-0.1.3.tgz",
      "version": "0.1.3",
    },
    "deglob" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob",
              "name": "glob",
              "spec": "^7.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/deglob/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/deglob/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/deglob/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/deglob/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/deglob/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "deglob",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/deglob/node_modules/glob",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.0.9",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.1.0",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
        "run-parallel" => EdgeOut {
          "name": "run-parallel",
          "spec": "^1.1.2",
          "to": "node_modules/run-parallel",
          "type": "prod",
        },
        "uniq" => EdgeOut {
          "name": "uniq",
          "spec": "^1.0.1",
          "to": "node_modules/uniq",
          "type": "prod",
        },
      },
      "location": "node_modules/deglob",
      "name": "deglob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/deglob",
      "resolved": "https://registry.npmjs.org/deglob/-/deglob-2.1.0.tgz",
      "version": "2.1.0",
    },
    "del" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "del",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "globby" => EdgeOut {
          "name": "globby",
          "spec": "^5.0.0",
          "to": "node_modules/globby",
          "type": "prod",
        },
        "is-path-cwd" => EdgeOut {
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-cwd",
          "type": "prod",
        },
        "is-path-in-cwd" => EdgeOut {
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-in-cwd",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.2.8",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
      },
      "location": "node_modules/del",
      "name": "del",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/del",
      "resolved": "https://registry.npmjs.org/del/-/del-2.2.2.tgz",
      "version": "2.2.2",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "diff",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
      "version": "1.4.0",
    },
    "doctrine" => ArboristNode {
      "children": Map {
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/doctrine/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/doctrine",
              "name": "isarray",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/doctrine/node_modules/isarray",
          "name": "isarray",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/doctrine/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "doctrine",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/doctrine/node_modules/esutils",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "^1.0.0",
          "to": "node_modules/doctrine/node_modules/isarray",
          "type": "prod",
        },
      },
      "location": "node_modules/doctrine",
      "name": "doctrine",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/doctrine",
      "resolved": "https://registry.npmjs.org/doctrine/-/doctrine-1.5.0.tgz",
      "version": "1.5.0",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "es5-ext" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/d",
          "name": "es5-ext",
          "spec": "~0.10.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es5-ext",
          "spec": "^0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-symbol",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es5-ext",
          "spec": "^0.10.8",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/event-emitter",
          "name": "es5-ext",
          "spec": "~0.10.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es5-ext",
      "name": "es5-ext",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es5-ext",
      "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.8.tgz",
      "version": "0.10.8",
    },
    "es6-iterator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-iterator",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-iterator",
      "name": "es6-iterator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-iterator",
      "resolved": "https://registry.npmjs.org/es6-iterator/-/es6-iterator-2.0.0.tgz",
      "version": "2.0.0",
    },
    "es6-map" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-map/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-map/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
        "es6-symbol" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-map",
              "name": "es6-symbol",
              "spec": "~3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-map/node_modules/es5-ext",
              "name": "es6-symbol",
              "spec": "~3.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "d" => EdgeOut {
              "name": "d",
              "spec": "~0.1.1",
              "to": "node_modules/d",
              "type": "prod",
            },
            "es5-ext" => EdgeOut {
              "name": "es5-ext",
              "spec": "~0.10.11",
              "to": "node_modules/es6-map/node_modules/es5-ext",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-map/node_modules/es6-symbol",
          "name": "es6-symbol",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-map/node_modules/es6-symbol",
          "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-map",
          "spec": "^0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-map/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-set" => EdgeOut {
          "name": "es6-set",
          "spec": "~0.1.3",
          "to": "node_modules/es6-set",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "~3.1.0",
          "to": "node_modules/es6-map/node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-map",
      "name": "es6-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-map",
      "resolved": "https://registry.npmjs.org/es6-map/-/es6-map-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-set" => ArboristNode {
      "children": Map {
        "es5-ext" => ArboristNode {
          "children": Map {
            "es6-symbol" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/es6-set/node_modules/es5-ext",
                  "name": "es6-symbol",
                  "spec": "~3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "d" => EdgeOut {
                  "name": "d",
                  "spec": "~0.1.1",
                  "to": "node_modules/d",
                  "type": "prod",
                },
                "es5-ext" => EdgeOut {
                  "name": "es5-ext",
                  "spec": "~0.10.11",
                  "to": "node_modules/es6-set/node_modules/es5-ext",
                  "type": "prod",
                },
              },
              "location": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es6-symbol",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.1.0.tgz",
              "version": "3.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/es6-set",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "name": "es5-ext",
              "spec": "~0.10.11",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "es6-iterator" => EdgeOut {
              "name": "es6-iterator",
              "spec": "2",
              "to": "node_modules/es6-iterator",
              "type": "prod",
            },
            "es6-symbol" => EdgeOut {
              "name": "es6-symbol",
              "spec": "~3.1",
              "to": "node_modules/es6-set/node_modules/es5-ext/node_modules/es6-symbol",
              "type": "prod",
            },
          },
          "location": "node_modules/es6-set/node_modules/es5-ext",
          "name": "es5-ext",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-set/node_modules/es5-ext",
          "resolved": "https://registry.npmjs.org/es5-ext/-/es5-ext-0.10.12.tgz",
          "version": "0.10.12",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "es6-set",
          "spec": "~0.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.11",
          "to": "node_modules/es6-set/node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
        "event-emitter" => EdgeOut {
          "name": "event-emitter",
          "spec": "~0.3.4",
          "to": "node_modules/event-emitter",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-set",
      "name": "es6-set",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-set",
      "resolved": "https://registry.npmjs.org/es6-set/-/es6-set-0.1.4.tgz",
      "version": "0.1.4",
    },
    "es6-symbol" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es5-ext",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-iterator",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-weak-map",
          "name": "es6-symbol",
          "spec": "3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-symbol",
      "name": "es6-symbol",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-symbol",
      "resolved": "https://registry.npmjs.org/es6-symbol/-/es6-symbol-3.0.0.tgz",
      "version": "3.0.0",
    },
    "es6-weak-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "^0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "^0.10.8",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
        "es6-iterator" => EdgeOut {
          "name": "es6-iterator",
          "spec": "2",
          "to": "node_modules/es6-iterator",
          "type": "prod",
        },
        "es6-symbol" => EdgeOut {
          "name": "es6-symbol",
          "spec": "3",
          "to": "node_modules/es6-symbol",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-weak-map",
      "name": "es6-weak-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/es6-weak-map",
      "resolved": "https://registry.npmjs.org/es6-weak-map/-/es6-weak-map-2.0.1.tgz",
      "version": "2.0.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.3.tgz",
      "version": "1.0.3",
    },
    "escope" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/escope",
              "name": "estraverse",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/escope/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/escope/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "escope",
          "spec": "^3.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-map" => EdgeOut {
          "name": "es6-map",
          "spec": "^0.1.3",
          "to": "node_modules/es6-map",
          "type": "prod",
        },
        "es6-weak-map" => EdgeOut {
          "name": "es6-weak-map",
          "spec": "^2.0.1",
          "to": "node_modules/es6-weak-map",
          "type": "prod",
        },
        "esrecurse" => EdgeOut {
          "name": "esrecurse",
          "spec": "^4.1.0",
          "to": "node_modules/esrecurse",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.1.1",
          "to": "node_modules/escope/node_modules/estraverse",
          "type": "prod",
        },
      },
      "location": "node_modules/escope",
      "name": "escope",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/escope",
      "resolved": "https://registry.npmjs.org/escope/-/escope-3.6.0.tgz",
      "version": "3.6.0",
    },
    "eslint" => ArboristNode {
      "children": Map {
        "ansi-styles" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/ansi-styles",
          "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-2.2.1.tgz",
          "version": "2.2.1",
        },
        "chalk" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "chalk",
              "spec": "^1.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^2.2.1",
              "to": "node_modules/eslint/node_modules/ansi-styles",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.2",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "has-ansi" => EdgeOut {
              "name": "has-ansi",
              "spec": "^2.0.0",
              "to": "node_modules/has-ansi",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^2.0.0",
              "to": "node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/chalk",
          "resolved": "https://registry.npmjs.org/chalk/-/chalk-1.1.3.tgz",
          "version": "1.1.3",
        },
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "estraverse",
              "spec": "^4.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.2.0.tgz",
          "version": "4.2.0",
        },
        "esutils" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/esutils",
          "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.2.tgz",
          "version": "2.0.2",
        },
        "fast-levenshtein" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/fast-levenshtein",
          "name": "fast-levenshtein",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/fast-levenshtein",
          "resolved": "https://registry.npmjs.org/fast-levenshtein/-/fast-levenshtein-2.0.6.tgz",
          "version": "2.0.6",
        },
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/eslint/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "js-yaml" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "js-yaml",
              "spec": "^3.5.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "argparse" => EdgeOut {
              "name": "argparse",
              "spec": "^1.0.7",
              "to": "node_modules/argparse",
              "type": "prod",
            },
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "^3.1.1",
              "to": "node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/js-yaml",
          "name": "js-yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/js-yaml",
          "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.8.1.tgz",
          "version": "3.8.1",
        },
        "levn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "levn",
              "spec": "^0.3.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "levn",
              "spec": "~0.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/levn",
          "name": "levn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/levn",
          "resolved": "https://registry.npmjs.org/levn/-/levn-0.3.0.tgz",
          "version": "0.3.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
        "optionator" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "optionator",
              "spec": "^0.8.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "deep-is" => EdgeOut {
              "name": "deep-is",
              "spec": "~0.1.3",
              "to": "node_modules/deep-is",
              "type": "prod",
            },
            "fast-levenshtein" => EdgeOut {
              "name": "fast-levenshtein",
              "spec": "~2.0.4",
              "to": "node_modules/eslint/node_modules/fast-levenshtein",
              "type": "prod",
            },
            "levn" => EdgeOut {
              "name": "levn",
              "spec": "~0.3.0",
              "to": "node_modules/eslint/node_modules/levn",
              "type": "prod",
            },
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
            "type-check" => EdgeOut {
              "name": "type-check",
              "spec": "~0.3.2",
              "to": "node_modules/eslint/node_modules/type-check",
              "type": "prod",
            },
            "wordwrap" => EdgeOut {
              "name": "wordwrap",
              "spec": "~1.0.0",
              "to": "node_modules/wordwrap",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/optionator",
          "name": "optionator",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/optionator",
          "resolved": "https://registry.npmjs.org/optionator/-/optionator-0.8.2.tgz",
          "version": "0.8.2",
        },
        "strip-bom" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint",
              "name": "strip-bom",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/strip-bom",
          "name": "strip-bom",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/strip-bom",
          "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
          "version": "3.0.0",
        },
        "type-check" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/eslint/node_modules/levn",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/eslint/node_modules/optionator",
              "name": "type-check",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "prelude-ls" => EdgeOut {
              "name": "prelude-ls",
              "spec": "~1.1.2",
              "to": "node_modules/prelude-ls",
              "type": "prod",
            },
          },
          "location": "node_modules/eslint/node_modules/type-check",
          "name": "type-check",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint/node_modules/type-check",
          "resolved": "https://registry.npmjs.org/type-check/-/type-check-0.3.2.tgz",
          "version": "0.3.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint",
          "spec": ">=3.8.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/eslint-plugin-standard",
          "name": "eslint",
          "spec": ">=3.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint",
          "spec": "~3.10.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "babel-code-frame" => EdgeOut {
          "name": "babel-code-frame",
          "spec": "^6.16.0",
          "to": "node_modules/babel-code-frame",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.3",
          "to": "node_modules/eslint/node_modules/chalk",
          "type": "prod",
        },
        "concat-stream" => EdgeOut {
          "name": "concat-stream",
          "spec": "^1.4.6",
          "to": "node_modules/concat-stream",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "escope" => EdgeOut {
          "name": "escope",
          "spec": "^3.6.0",
          "to": "node_modules/escope",
          "type": "prod",
        },
        "espree" => EdgeOut {
          "name": "espree",
          "spec": "^3.3.1",
          "to": "node_modules/espree",
          "type": "prod",
        },
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "^4.2.0",
          "to": "node_modules/eslint/node_modules/estraverse",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/eslint/node_modules/esutils",
          "type": "prod",
        },
        "file-entry-cache" => EdgeOut {
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "to": "node_modules/file-entry-cache",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/eslint/node_modules/glob",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^9.2.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "ignore" => EdgeOut {
          "name": "ignore",
          "spec": "^3.2.0",
          "to": "node_modules/ignore",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "inquirer" => EdgeOut {
          "name": "inquirer",
          "spec": "^0.12.0",
          "to": "node_modules/inquirer",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "to": "node_modules/is-my-json-valid",
          "type": "prod",
        },
        "is-resolvable" => EdgeOut {
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "to": "node_modules/is-resolvable",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.5.1",
          "to": "node_modules/eslint/node_modules/js-yaml",
          "type": "prod",
        },
        "json-stable-stringify" => EdgeOut {
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "to": "node_modules/json-stable-stringify",
          "type": "prod",
        },
        "levn" => EdgeOut {
          "name": "levn",
          "spec": "^0.3.0",
          "to": "node_modules/eslint/node_modules/levn",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/eslint/node_modules/lodash",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "natural-compare" => EdgeOut {
          "name": "natural-compare",
          "spec": "^1.4.0",
          "to": "node_modules/natural-compare",
          "type": "prod",
        },
        "optionator" => EdgeOut {
          "name": "optionator",
          "spec": "^0.8.2",
          "to": "node_modules/eslint/node_modules/optionator",
          "type": "prod",
        },
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
        "pluralize" => EdgeOut {
          "name": "pluralize",
          "spec": "^1.2.1",
          "to": "node_modules/pluralize",
          "type": "prod",
        },
        "progress" => EdgeOut {
          "name": "progress",
          "spec": "^1.1.8",
          "to": "node_modules/progress",
          "type": "prod",
        },
        "require-uncached" => EdgeOut {
          "name": "require-uncached",
          "spec": "^1.0.2",
          "to": "node_modules/require-uncached",
          "type": "prod",
        },
        "shelljs" => EdgeOut {
          "name": "shelljs",
          "spec": "^0.7.5",
          "to": "node_modules/shelljs",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/eslint/node_modules/strip-bom",
          "type": "prod",
        },
        "strip-json-comments" => EdgeOut {
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "to": "node_modules/strip-json-comments",
          "type": "prod",
        },
        "table" => EdgeOut {
          "name": "table",
          "spec": "^3.7.8",
          "to": "node_modules/table",
          "type": "prod",
        },
        "text-table" => EdgeOut {
          "name": "text-table",
          "spec": "~0.2.0",
          "to": "node_modules/text-table",
          "type": "prod",
        },
        "user-home" => EdgeOut {
          "name": "user-home",
          "spec": "^2.0.0",
          "to": "node_modules/user-home",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint",
      "name": "eslint",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint",
      "resolved": "https://registry.npmjs.org/eslint/-/eslint-3.10.2.tgz",
      "version": "3.10.2",
    },
    "eslint-config-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.8.1",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "peer",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "to": "node_modules/eslint-plugin-standard",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard",
      "name": "eslint-config-standard",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint-config-standard",
      "resolved": "https://registry.npmjs.org/eslint-config-standard/-/eslint-config-standard-6.2.1.tgz",
      "version": "6.2.1",
    },
    "eslint-config-standard-jsx" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-config-standard-jsx",
      "name": "eslint-config-standard-jsx",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint-config-standard-jsx",
      "resolved": "https://registry.npmjs.org/eslint-config-standard-jsx/-/eslint-config-standard-jsx-3.2.0.tgz",
      "version": "3.2.0",
    },
    "eslint-plugin-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-promise",
          "spec": ">=3.3.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-promise",
      "name": "eslint-plugin-promise",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint-plugin-promise",
      "resolved": "https://registry.npmjs.org/eslint-plugin-promise/-/eslint-plugin-promise-3.4.1.tgz",
      "version": "3.4.1",
    },
    "eslint-plugin-react" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard-jsx",
          "name": "eslint-plugin-react",
          "spec": ">=2.7.1",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "doctrine" => EdgeOut {
          "name": "doctrine",
          "spec": "^1.2.2",
          "to": "node_modules/doctrine",
          "type": "prod",
        },
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "^2.0.0 || ^3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
        "jsx-ast-utils" => EdgeOut {
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "to": "node_modules/jsx-ast-utils",
          "type": "prod",
        },
      },
      "location": "node_modules/eslint-plugin-react",
      "name": "eslint-plugin-react",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint-plugin-react",
      "resolved": "https://registry.npmjs.org/eslint-plugin-react/-/eslint-plugin-react-6.7.1.tgz",
      "version": "6.7.1",
    },
    "eslint-plugin-standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-config-standard",
          "name": "eslint-plugin-standard",
          "spec": ">=2.0.0",
          "type": "peer",
        },
        EdgeIn {
          "from": "node_modules/standard",
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": ">=3.0.0",
          "to": "node_modules/eslint",
          "type": "peer",
        },
      },
      "location": "node_modules/eslint-plugin-standard",
      "name": "eslint-plugin-standard",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/eslint-plugin-standard",
      "resolved": "https://registry.npmjs.org/eslint-plugin-standard/-/eslint-plugin-standard-2.0.1.tgz",
      "version": "2.0.1",
    },
    "espree" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "espree",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "acorn" => EdgeOut {
          "name": "acorn",
          "spec": "4.0.4",
          "to": "node_modules/acorn",
          "type": "prod",
        },
        "acorn-jsx" => EdgeOut {
          "name": "acorn-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/acorn-jsx",
          "type": "prod",
        },
      },
      "location": "node_modules/espree",
      "name": "espree",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/espree",
      "resolved": "https://registry.npmjs.org/espree/-/espree-3.4.0.tgz",
      "version": "3.4.0",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/js-yaml",
          "name": "esprima",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-3.1.3.tgz",
      "version": "3.1.3",
    },
    "esrecurse" => ArboristNode {
      "children": Map {
        "estraverse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/esrecurse",
              "name": "estraverse",
              "spec": "~4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/esrecurse/node_modules/estraverse",
          "name": "estraverse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/esrecurse/node_modules/estraverse",
          "resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.1.1.tgz",
          "version": "4.1.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/escope",
          "name": "esrecurse",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "estraverse" => EdgeOut {
          "name": "estraverse",
          "spec": "~4.1.0",
          "to": "node_modules/esrecurse/node_modules/estraverse",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/esrecurse",
      "name": "esrecurse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/esrecurse",
      "resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.1.0.tgz",
      "version": "4.1.0",
    },
    "event-emitter" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/es6-map",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/es6-set",
          "name": "event-emitter",
          "spec": "~0.3.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "d" => EdgeOut {
          "name": "d",
          "spec": "~0.1.1",
          "to": "node_modules/d",
          "type": "prod",
        },
        "es5-ext" => EdgeOut {
          "name": "es5-ext",
          "spec": "~0.10.7",
          "to": "node_modules/es5-ext",
          "type": "prod",
        },
      },
      "location": "node_modules/event-emitter",
      "name": "event-emitter",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/event-emitter",
      "resolved": "https://registry.npmjs.org/event-emitter/-/event-emitter-0.3.4.tgz",
      "version": "0.3.4",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.0.2.tgz",
      "version": "1.0.2",
    },
    "exit-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "exit-hook",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/exit-hook",
      "name": "exit-hook",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/exit-hook",
      "resolved": "https://registry.npmjs.org/exit-hook/-/exit-hook-1.1.1.tgz",
      "version": "1.1.1",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "figures" => ArboristNode {
      "children": Map {
        "escape-string-regexp" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/figures",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/figures/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/figures/node_modules/escape-string-regexp",
          "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
          "version": "1.0.5",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "figures",
          "spec": "^1.3.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/figures/node_modules/escape-string-regexp",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/figures",
      "name": "figures",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/figures",
      "resolved": "https://registry.npmjs.org/figures/-/figures-1.7.0.tgz",
      "version": "1.7.0",
    },
    "file-entry-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "file-entry-cache",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flat-cache" => EdgeOut {
          "name": "flat-cache",
          "spec": "^1.2.1",
          "to": "node_modules/flat-cache",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/file-entry-cache",
      "name": "file-entry-cache",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/file-entry-cache",
      "resolved": "https://registry.npmjs.org/file-entry-cache/-/file-entry-cache-2.0.0.tgz",
      "version": "2.0.0",
    },
    "find-root" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-config",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "find-root",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/find-root",
      "name": "find-root",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/find-root",
      "resolved": "https://registry.npmjs.org/find-root/-/find-root-1.0.0.tgz",
      "version": "1.0.0",
    },
    "flat-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "flat-cache",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "circular-json" => EdgeOut {
          "name": "circular-json",
          "spec": "^0.3.1",
          "to": "node_modules/circular-json",
          "type": "prod",
        },
        "del" => EdgeOut {
          "name": "del",
          "spec": "^2.0.2",
          "to": "node_modules/del",
          "type": "prod",
        },
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "write" => EdgeOut {
          "name": "write",
          "spec": "^0.2.1",
          "to": "node_modules/write",
          "type": "prod",
        },
      },
      "location": "node_modules/flat-cache",
      "name": "flat-cache",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/flat-cache",
      "resolved": "https://registry.npmjs.org/flat-cache/-/flat-cache-1.2.2.tgz",
      "version": "1.2.2",
    },
    "foreground-child" => ArboristNode {
      "children": Map {
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/foreground-child",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/foreground-child/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/foreground-child/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/foreground-child/node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.1.2.tgz",
      "version": "2.1.2",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.1.tgz",
      "version": "1.0.1",
    },
    "generate-function" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-function",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-function",
      "name": "generate-function",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/generate-function",
      "resolved": "https://registry.npmjs.org/generate-function/-/generate-function-2.0.0.tgz",
      "version": "2.0.0",
    },
    "generate-object-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-property" => EdgeOut {
          "name": "is-property",
          "spec": "^1.0.0",
          "to": "node_modules/is-property",
          "type": "prod",
        },
      },
      "location": "node_modules/generate-object-property",
      "name": "generate-object-property",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/generate-object-property",
      "resolved": "https://registry.npmjs.org/generate-object-property/-/generate-object-property-1.2.0.tgz",
      "version": "1.2.0",
    },
    "get-stdin" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "get-stdin",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-stdin",
      "name": "get-stdin",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/get-stdin",
      "resolved": "https://registry.npmjs.org/get-stdin/-/get-stdin-5.0.1.tgz",
      "version": "5.0.1",
    },
    "getpass" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/getpass",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/getpass/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/getpass/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/getpass/node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.6.tgz",
      "version": "0.1.6",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.2.tgz",
      "version": "7.1.2",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "globals",
          "spec": "^9.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-9.14.0.tgz",
      "version": "9.14.0",
    },
    "globby" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby",
              "name": "glob",
              "spec": "^7.0.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/globby/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/globby/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/globby/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/globby/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/globby/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "globby",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "array-union" => EdgeOut {
          "name": "array-union",
          "spec": "^1.0.1",
          "to": "node_modules/array-union",
          "type": "prod",
        },
        "arrify" => EdgeOut {
          "name": "arrify",
          "spec": "^1.0.0",
          "to": "node_modules/arrify",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.3",
          "to": "node_modules/globby/node_modules/glob",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.0.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^2.0.0",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/globby",
      "name": "globby",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/globby",
      "resolved": "https://registry.npmjs.org/globby/-/globby-5.0.0.tgz",
      "version": "5.0.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.1.11.tgz",
      "version": "4.1.11",
    },
    "graceful-readlink" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/commander",
          "name": "graceful-readlink",
          "spec": ">= 1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-readlink",
      "name": "graceful-readlink",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/graceful-readlink",
      "resolved": "https://registry.npmjs.org/graceful-readlink/-/graceful-readlink-1.0.1.tgz",
      "version": "1.0.1",
    },
    "har-validator" => ArboristNode {
      "children": Map {
        "commander" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "commander",
              "spec": "^2.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-readlink" => EdgeOut {
              "name": "graceful-readlink",
              "spec": ">= 1.0.0",
              "to": "node_modules/graceful-readlink",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/commander",
          "name": "commander",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator/node_modules/commander",
          "resolved": "https://registry.npmjs.org/commander/-/commander-2.9.0.tgz",
          "version": "2.9.0",
        },
        "is-my-json-valid" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "is-my-json-valid",
              "spec": "^2.12.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "generate-function" => EdgeOut {
              "name": "generate-function",
              "spec": "^2.0.0",
              "to": "node_modules/generate-function",
              "type": "prod",
            },
            "generate-object-property" => EdgeOut {
              "name": "generate-object-property",
              "spec": "^1.1.0",
              "to": "node_modules/generate-object-property",
              "type": "prod",
            },
            "jsonpointer" => EdgeOut {
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "to": "node_modules/har-validator/node_modules/jsonpointer",
              "type": "prod",
            },
            "xtend" => EdgeOut {
              "name": "xtend",
              "spec": "^4.0.0",
              "to": "node_modules/xtend",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "is-my-json-valid",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator/node_modules/is-my-json-valid",
          "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
          "version": "2.15.0",
        },
        "jsonpointer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/is-my-json-valid",
              "name": "jsonpointer",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/jsonpointer",
          "name": "jsonpointer",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator/node_modules/jsonpointer",
          "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
          "version": "4.0.1",
        },
        "pinkie" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator/node_modules/pinkie-promise",
              "name": "pinkie",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie",
          "name": "pinkie",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator/node_modules/pinkie",
          "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
          "version": "2.0.4",
        },
        "pinkie-promise" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/har-validator",
              "name": "pinkie-promise",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "pinkie" => EdgeOut {
              "name": "pinkie",
              "spec": "^2.0.0",
              "to": "node_modules/har-validator/node_modules/pinkie",
              "type": "prod",
            },
          },
          "location": "node_modules/har-validator/node_modules/pinkie-promise",
          "name": "pinkie-promise",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator/node_modules/pinkie-promise",
          "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
          "version": "2.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~2.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "commander" => EdgeOut {
          "name": "commander",
          "spec": "^2.9.0",
          "to": "node_modules/har-validator/node_modules/commander",
          "type": "prod",
        },
        "is-my-json-valid" => EdgeOut {
          "name": "is-my-json-valid",
          "spec": "^2.12.4",
          "to": "node_modules/har-validator/node_modules/is-my-json-valid",
          "type": "prod",
        },
        "pinkie-promise" => EdgeOut {
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "to": "node_modules/har-validator/node_modules/pinkie-promise",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-2.0.6.tgz",
      "version": "2.0.6",
    },
    "has-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "has-ansi",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/has-ansi",
      "name": "has-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/has-ansi",
      "resolved": "https://registry.npmjs.org/has-ansi/-/has-ansi-2.0.0.tgz",
      "version": "2.0.0",
    },
    "hawk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "hawk",
          "spec": "~3.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "boom" => EdgeOut {
          "name": "boom",
          "spec": "2.x.x",
          "to": "node_modules/boom",
          "type": "prod",
        },
        "cryptiles" => EdgeOut {
          "name": "cryptiles",
          "spec": "2.x.x",
          "to": "node_modules/cryptiles",
          "type": "prod",
        },
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
        "sntp" => EdgeOut {
          "name": "sntp",
          "spec": "1.x.x",
          "to": "node_modules/sntp",
          "type": "prod",
        },
      },
      "location": "node_modules/hawk",
      "name": "hawk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/hawk",
      "resolved": "https://registry.npmjs.org/hawk/-/hawk-3.1.3.tgz",
      "version": "3.1.3",
    },
    "hoek" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/boom",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sntp",
          "name": "hoek",
          "spec": "2.x.x",
          "type": "prod",
        },
      },
      "location": "node_modules/hoek",
      "name": "hoek",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/hoek",
      "resolved": "https://registry.npmjs.org/hoek/-/hoek-2.16.3.tgz",
      "version": "2.16.3",
    },
    "home-or-tmp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "os-tmpdir" => EdgeOut {
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "to": "node_modules/os-tmpdir",
          "type": "prod",
        },
      },
      "location": "node_modules/home-or-tmp",
      "name": "home-or-tmp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/home-or-tmp",
      "resolved": "https://registry.npmjs.org/home-or-tmp/-/home-or-tmp-2.0.0.tgz",
      "version": "2.0.0",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^0.2.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.1.1.tgz",
      "version": "1.1.1",
    },
    "ignore" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "ignore",
          "spec": "^3.0.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "ignore",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ignore",
      "name": "ignore",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ignore",
      "resolved": "https://registry.npmjs.org/ignore/-/ignore-3.2.2.tgz",
      "version": "3.2.2",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.4.tgz",
      "version": "1.0.4",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.1.tgz",
      "version": "2.0.1",
    },
    "inquirer" => ArboristNode {
      "children": Map {
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/inquirer",
              "name": "lodash",
              "spec": "^4.3.0",
              "type": "prod",
            },
          },
          "location": "node_modules/inquirer/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/inquirer/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "inquirer",
          "spec": "^0.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-escapes" => EdgeOut {
          "name": "ansi-escapes",
          "spec": "^1.1.0",
          "to": "node_modules/ansi-escapes",
          "type": "prod",
        },
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "cli-cursor" => EdgeOut {
          "name": "cli-cursor",
          "spec": "^1.0.1",
          "to": "node_modules/cli-cursor",
          "type": "prod",
        },
        "cli-width" => EdgeOut {
          "name": "cli-width",
          "spec": "^2.0.0",
          "to": "node_modules/cli-width",
          "type": "prod",
        },
        "figures" => EdgeOut {
          "name": "figures",
          "spec": "^1.3.5",
          "to": "node_modules/figures",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.3.0",
          "to": "node_modules/inquirer/node_modules/lodash",
          "type": "prod",
        },
        "readline2" => EdgeOut {
          "name": "readline2",
          "spec": "^1.0.1",
          "to": "node_modules/readline2",
          "type": "prod",
        },
        "run-async" => EdgeOut {
          "name": "run-async",
          "spec": "^0.1.0",
          "to": "node_modules/run-async",
          "type": "prod",
        },
        "rx-lite" => EdgeOut {
          "name": "rx-lite",
          "spec": "^3.1.2",
          "to": "node_modules/rx-lite",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^1.0.1",
          "to": "node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
        "through" => EdgeOut {
          "name": "through",
          "spec": "^2.3.6",
          "to": "node_modules/through",
          "type": "prod",
        },
      },
      "location": "node_modules/inquirer",
      "name": "inquirer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/inquirer",
      "resolved": "https://registry.npmjs.org/inquirer/-/inquirer-0.12.0.tgz",
      "version": "0.12.0",
    },
    "interpret" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "interpret",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/interpret",
      "name": "interpret",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/interpret",
      "resolved": "https://registry.npmjs.org/interpret/-/interpret-1.0.1.tgz",
      "version": "1.0.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "number-is-nan" => EdgeOut {
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "to": "node_modules/number-is-nan",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-my-json-valid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-my-json-valid",
          "spec": "^2.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "generate-function" => EdgeOut {
          "name": "generate-function",
          "spec": "^2.0.0",
          "to": "node_modules/generate-function",
          "type": "prod",
        },
        "generate-object-property" => EdgeOut {
          "name": "generate-object-property",
          "spec": "^1.1.0",
          "to": "node_modules/generate-object-property",
          "type": "prod",
        },
        "jsonpointer" => EdgeOut {
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "to": "node_modules/jsonpointer",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.0",
          "to": "node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/is-my-json-valid",
      "name": "is-my-json-valid",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-my-json-valid",
      "resolved": "https://registry.npmjs.org/is-my-json-valid/-/is-my-json-valid-2.15.0.tgz",
      "version": "2.15.0",
    },
    "is-path-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-cwd",
      "name": "is-path-cwd",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-path-cwd",
      "resolved": "https://registry.npmjs.org/is-path-cwd/-/is-path-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-in-cwd" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "is-path-in-cwd",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-path-inside" => EdgeOut {
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "to": "node_modules/is-path-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-in-cwd",
      "name": "is-path-in-cwd",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-path-in-cwd",
      "resolved": "https://registry.npmjs.org/is-path-in-cwd/-/is-path-in-cwd-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-path-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-path-in-cwd",
          "name": "is-path-inside",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-is-inside" => EdgeOut {
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "to": "node_modules/path-is-inside",
          "type": "prod",
        },
      },
      "location": "node_modules/is-path-inside",
      "name": "is-path-inside",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-path-inside",
      "resolved": "https://registry.npmjs.org/is-path-inside/-/is-path-inside-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-property" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/generate-object-property",
          "name": "is-property",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-property",
      "name": "is-property",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-property",
      "resolved": "https://registry.npmjs.org/is-property/-/is-property-1.0.2.tgz",
      "version": "1.0.2",
    },
    "is-resolvable" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "is-resolvable",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tryit" => EdgeOut {
          "name": "tryit",
          "spec": "^1.0.1",
          "to": "node_modules/tryit",
          "type": "prod",
        },
      },
      "location": "node_modules/is-resolvable",
      "name": "is-resolvable",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-resolvable",
      "resolved": "https://registry.npmjs.org/is-resolvable/-/is-resolvable-1.0.0.tgz",
      "version": "1.0.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-0.0.1.tgz",
      "version": "0.0.1",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^1.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-1.1.2.tgz",
      "version": "1.1.2",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "jodid25519" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jodid25519",
          "spec": "^1.0.0",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
      },
      "location": "node_modules/jodid25519",
      "name": "jodid25519",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jodid25519",
      "resolved": "https://registry.npmjs.org/jodid25519/-/jodid25519-1.0.2.tgz",
      "version": "1.0.2",
    },
    "js-tokens" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/babel-code-frame",
          "name": "js-tokens",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-3.0.1.tgz",
      "version": "3.0.1",
    },
    "js-yaml" => ArboristNode {
      "children": Map {
        "argparse" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "argparse",
              "spec": "~1.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": ">= 3.2.0 < 4.0.0",
              "to": "node_modules/lodash",
              "type": "prod",
            },
            "sprintf-js" => EdgeOut {
              "name": "sprintf-js",
              "spec": "~1.0.2",
              "to": "node_modules/sprintf-js",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/argparse",
          "name": "argparse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/js-yaml/node_modules/argparse",
          "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.2.tgz",
          "version": "1.0.2",
        },
        "esprima" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/js-yaml",
              "name": "esprima",
              "spec": "~2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/js-yaml/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/js-yaml/node_modules/esprima",
          "resolved": "https://registry.npmjs.org/esprima/-/esprima-2.2.0.tgz",
          "version": "2.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "js-yaml",
          "spec": "^3.2.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "~1.0.2",
          "to": "node_modules/js-yaml/node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "~2.2.0",
          "to": "node_modules/js-yaml/node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.4.2.tgz",
      "version": "3.4.2",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jodid25519",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "optional",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.0.tgz",
      "version": "0.1.0",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-stable-stringify",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "json-stable-stringify",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsonify" => EdgeOut {
          "name": "jsonify",
          "spec": "~0.0.0",
          "to": "node_modules/jsonify",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stable-stringify",
      "name": "json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/json-stable-stringify",
      "resolved": "https://registry.npmjs.org/json-stable-stringify/-/json-stable-stringify-1.0.1.tgz",
      "version": "1.0.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsonify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/json-stable-stringify",
          "name": "jsonify",
          "spec": "~0.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonify",
      "name": "jsonify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jsonify",
      "resolved": "https://registry.npmjs.org/jsonify/-/jsonify-0.0.0.tgz",
      "version": "0.0.0",
    },
    "jsonpointer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "jsonpointer",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsonpointer",
      "name": "jsonpointer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jsonpointer",
      "resolved": "https://registry.npmjs.org/jsonpointer/-/jsonpointer-4.0.1.tgz",
      "version": "4.0.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.3.6",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.3.1.tgz",
      "version": "1.3.1",
    },
    "jsx-ast-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint-plugin-react",
          "name": "jsx-ast-utils",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/jsx-ast-utils",
      "name": "jsx-ast-utils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/jsx-ast-utils",
      "resolved": "https://registry.npmjs.org/jsx-ast-utils/-/jsx-ast-utils-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "0.0.10",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-0.0.10.tgz",
      "version": "0.0.10",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "lodash",
          "spec": ">= 3.2.0 < 4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-3.10.1.tgz",
      "version": "3.10.1",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.5.tgz",
      "version": "1.2.5",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "~1.26.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.26.0.tgz",
      "version": "1.26.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "~1.26.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.14.tgz",
      "version": "2.1.14",
    },
    "minimatch" => ArboristNode {
      "children": Map {
        "balanced-match" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch/node_modules/brace-expansion",
              "name": "balanced-match",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/balanced-match",
          "name": "balanced-match",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/minimatch/node_modules/balanced-match",
          "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
          "version": "1.0.0",
        },
        "brace-expansion" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minimatch",
              "name": "brace-expansion",
              "spec": "^1.1.7",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "balanced-match" => EdgeOut {
              "name": "balanced-match",
              "spec": "^1.0.0",
              "to": "node_modules/minimatch/node_modules/balanced-match",
              "type": "prod",
            },
            "concat-map" => EdgeOut {
              "name": "concat-map",
              "spec": "0.0.1",
              "to": "node_modules/concat-map",
              "type": "prod",
            },
          },
          "location": "node_modules/minimatch/node_modules/brace-expansion",
          "name": "brace-expansion",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/minimatch/node_modules/brace-expansion",
          "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.8.tgz",
          "version": "1.1.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "minimatch",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/minimatch/node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mkdirp",
          "name": "minimist",
          "spec": "0.0.8",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
      "version": "0.0.8",
    },
    "mkdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "0.7.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-0.7.1.tgz",
      "version": "0.7.1",
    },
    "mute-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readline2",
          "name": "mute-stream",
          "spec": "0.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/mute-stream",
      "name": "mute-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/mute-stream",
      "resolved": "https://registry.npmjs.org/mute-stream/-/mute-stream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "natural-compare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "natural-compare",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/natural-compare",
      "name": "natural-compare",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/natural-compare",
      "resolved": "https://registry.npmjs.org/natural-compare/-/natural-compare-1.4.0.tgz",
      "version": "1.4.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/code-point-at",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.0.tgz",
      "version": "1.0.0",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.8.2.tgz",
      "version": "0.8.2",
    },
    "object-assign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/esrecurse",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/figures",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/file-entry-cache",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "object-assign",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsx-ast-utils",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/run-async",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "version": "1.3.2",
    },
    "onetime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/restore-cursor",
          "name": "onetime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/onetime",
      "name": "onetime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/onetime",
      "resolved": "https://registry.npmjs.org/onetime/-/onetime-1.1.0.tgz",
      "version": "1.1.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.4.2.tgz",
      "version": "1.4.2",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "os-homedir",
          "spec": "1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/user-home",
          "name": "os-homedir",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "os-tmpdir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/home-or-tmp",
          "name": "os-tmpdir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-tmpdir",
      "name": "os-tmpdir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/os-tmpdir",
      "resolved": "https://registry.npmjs.org/os-tmpdir/-/os-tmpdir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/shelljs/node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.0.tgz",
      "version": "1.0.0",
    },
    "path-is-inside" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-path-inside",
          "name": "path-is-inside",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-inside",
      "name": "path-is-inside",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/path-is-inside",
      "resolved": "https://registry.npmjs.org/path-is-inside/-/path-is-inside-1.0.2.tgz",
      "version": "1.0.2",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-2.3.0.tgz",
      "version": "2.3.0",
    },
    "pinkie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pinkie-promise",
          "name": "pinkie",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie",
      "name": "pinkie",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pinkie",
      "resolved": "https://registry.npmjs.org/pinkie/-/pinkie-2.0.4.tgz",
      "version": "2.0.4",
    },
    "pinkie-promise" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/globby",
          "name": "pinkie-promise",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pinkie" => EdgeOut {
          "name": "pinkie",
          "spec": "^2.0.0",
          "to": "node_modules/pinkie",
          "type": "prod",
        },
      },
      "location": "node_modules/pinkie-promise",
      "name": "pinkie-promise",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pinkie-promise",
      "resolved": "https://registry.npmjs.org/pinkie-promise/-/pinkie-promise-2.0.1.tgz",
      "version": "2.0.1",
    },
    "pkg-config" => ArboristNode {
      "children": Map {
        "xtend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/pkg-config",
              "name": "xtend",
              "spec": "^4.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/pkg-config/node_modules/xtend",
          "name": "xtend",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pkg-config/node_modules/xtend",
          "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.1.tgz",
          "version": "4.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "pkg-config",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/standard-engine",
          "name": "pkg-config",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug-log" => EdgeOut {
          "name": "debug-log",
          "spec": "^1.0.0",
          "to": "node_modules/debug-log",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "xtend" => EdgeOut {
          "name": "xtend",
          "spec": "^4.0.1",
          "to": "node_modules/pkg-config/node_modules/xtend",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-config",
      "name": "pkg-config",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pkg-config",
      "resolved": "https://registry.npmjs.org/pkg-config/-/pkg-config-1.1.1.tgz",
      "version": "1.1.1",
    },
    "pluralize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "pluralize",
          "spec": "^1.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pluralize",
      "name": "pluralize",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pluralize",
      "resolved": "https://registry.npmjs.org/pluralize/-/pluralize-1.2.1.tgz",
      "version": "1.2.1",
    },
    "prelude-ls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/levn",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/type-check",
          "name": "prelude-ls",
          "spec": "~1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/prelude-ls",
      "name": "prelude-ls",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/prelude-ls",
      "resolved": "https://registry.npmjs.org/prelude-ls/-/prelude-ls-1.1.2.tgz",
      "version": "1.1.2",
    },
    "process-nextick-args" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.3.tgz",
      "version": "1.0.3",
    },
    "progress" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "progress",
          "spec": "^1.1.8",
          "type": "prod",
        },
      },
      "location": "node_modules/progress",
      "name": "progress",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/progress",
      "resolved": "https://registry.npmjs.org/progress/-/progress-1.1.8.tgz",
      "version": "1.1.8",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^1.4.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "punycode",
          "spec": "^1.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
      "version": "1.4.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.3.0",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.3.0.tgz",
      "version": "6.3.0",
    },
    "readable-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "readable-stream",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "readable-stream",
          "spec": "^2",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.1",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "0.0.1",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~1.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~0.10.x",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.0.2.tgz",
      "version": "2.0.2",
    },
    "readline2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "readline2",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "mute-stream" => EdgeOut {
          "name": "mute-stream",
          "spec": "0.0.5",
          "to": "node_modules/mute-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/readline2",
      "name": "readline2",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/readline2",
      "resolved": "https://registry.npmjs.org/readline2/-/readline2-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rechoir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shelljs",
          "name": "rechoir",
          "spec": "^0.6.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.1.6",
          "to": "node_modules/resolve",
          "type": "prod",
        },
      },
      "location": "node_modules/rechoir",
      "name": "rechoir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/rechoir",
      "resolved": "https://registry.npmjs.org/rechoir/-/rechoir-0.6.2.tgz",
      "version": "0.6.2",
    },
    "request" => ArboristNode {
      "children": Map {
        "extend" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/request",
              "name": "extend",
              "spec": "~3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/request/node_modules/extend",
          "name": "extend",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/request/node_modules/extend",
          "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "2.79.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.6.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.2.1",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.11.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.5",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.0",
          "to": "node_modules/request/node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.1.1",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~2.0.6",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "hawk" => EdgeOut {
          "name": "hawk",
          "spec": "~3.1.3",
          "to": "node_modules/hawk",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.1.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.7",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.8.1",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.3.0",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "stringstream" => EdgeOut {
          "name": "stringstream",
          "spec": "~0.0.4",
          "to": "node_modules/stringstream",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.0.0",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.79.0.tgz",
      "version": "2.79.0",
    },
    "require-uncached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "require-uncached",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "caller-path" => EdgeOut {
          "name": "caller-path",
          "spec": "^0.1.0",
          "to": "node_modules/caller-path",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^1.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
      },
      "location": "node_modules/require-uncached",
      "name": "require-uncached",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/require-uncached",
      "resolved": "https://registry.npmjs.org/require-uncached/-/require-uncached-1.0.3.tgz",
      "version": "1.0.3",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/rechoir",
          "name": "resolve",
          "spec": "^1.1.6",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.1.6.tgz",
      "version": "1.1.6",
    },
    "resolve-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/require-uncached",
          "name": "resolve-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-1.0.1.tgz",
      "version": "1.0.1",
    },
    "restore-cursor" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cli-cursor",
          "name": "restore-cursor",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "exit-hook" => EdgeOut {
          "name": "exit-hook",
          "spec": "^1.0.0",
          "to": "node_modules/exit-hook",
          "type": "prod",
        },
        "onetime" => EdgeOut {
          "name": "onetime",
          "spec": "^1.0.0",
          "to": "node_modules/onetime",
          "type": "prod",
        },
      },
      "location": "node_modules/restore-cursor",
      "name": "restore-cursor",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/restore-cursor",
      "resolved": "https://registry.npmjs.org/restore-cursor/-/restore-cursor-1.0.1.tgz",
      "version": "1.0.1",
    },
    "rimraf" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/rimraf",
              "name": "glob",
              "spec": "^5.0.14",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "2 || 3",
              "to": "node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/rimraf/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/rimraf/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-5.0.15.tgz",
          "version": "5.0.15",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/del",
          "name": "rimraf",
          "spec": "^2.2.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^5.0.14",
          "to": "node_modules/rimraf/node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.4.3.tgz",
      "version": "2.4.3",
    },
    "run-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "run-async",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
      },
      "location": "node_modules/run-async",
      "name": "run-async",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/run-async",
      "resolved": "https://registry.npmjs.org/run-async/-/run-async-0.1.0.tgz",
      "version": "0.1.0",
    },
    "run-parallel" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "run-parallel",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/run-parallel",
      "name": "run-parallel",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/run-parallel",
      "resolved": "https://registry.npmjs.org/run-parallel/-/run-parallel-1.1.6.tgz",
      "version": "1.1.6",
    },
    "rx-lite" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "rx-lite",
          "spec": "^3.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/rx-lite",
      "name": "rx-lite",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/rx-lite",
      "resolved": "https://registry.npmjs.org/rx-lite/-/rx-lite-3.1.2.tgz",
      "version": "3.1.2",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.1.tgz",
      "version": "5.1.1",
    },
    "shelljs" => ArboristNode {
      "children": Map {
        "glob" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs",
              "name": "glob",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "fs.realpath" => EdgeOut {
              "name": "fs.realpath",
              "spec": "^1.0.0",
              "to": "node_modules/fs.realpath",
              "type": "prod",
            },
            "inflight" => EdgeOut {
              "name": "inflight",
              "spec": "^1.0.4",
              "to": "node_modules/inflight",
              "type": "prod",
            },
            "inherits" => EdgeOut {
              "name": "inherits",
              "spec": "2",
              "to": "node_modules/inherits",
              "type": "prod",
            },
            "minimatch" => EdgeOut {
              "name": "minimatch",
              "spec": "^3.0.2",
              "to": "node_modules/shelljs/node_modules/minimatch",
              "type": "prod",
            },
            "once" => EdgeOut {
              "name": "once",
              "spec": "^1.3.0",
              "to": "node_modules/once",
              "type": "prod",
            },
            "path-is-absolute" => EdgeOut {
              "name": "path-is-absolute",
              "spec": "^1.0.0",
              "to": "node_modules/path-is-absolute",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/glob",
          "name": "glob",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/shelljs/node_modules/glob",
          "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.1.tgz",
          "version": "7.1.1",
        },
        "minimatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/shelljs/node_modules/glob",
              "name": "minimatch",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "brace-expansion" => EdgeOut {
              "name": "brace-expansion",
              "spec": "^1.0.0",
              "to": "node_modules/brace-expansion",
              "type": "prod",
            },
          },
          "location": "node_modules/shelljs/node_modules/minimatch",
          "name": "minimatch",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/shelljs/node_modules/minimatch",
          "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.3.tgz",
          "version": "3.0.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "shelljs",
          "spec": "^0.7.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/shelljs/node_modules/glob",
          "type": "prod",
        },
        "interpret" => EdgeOut {
          "name": "interpret",
          "spec": "^1.0.0",
          "to": "node_modules/interpret",
          "type": "prod",
        },
        "rechoir" => EdgeOut {
          "name": "rechoir",
          "spec": "^0.6.2",
          "to": "node_modules/rechoir",
          "type": "prod",
        },
      },
      "location": "node_modules/shelljs",
      "name": "shelljs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/shelljs",
      "resolved": "https://registry.npmjs.org/shelljs/-/shelljs-0.7.6.tgz",
      "version": "0.7.6",
    },
    "slice-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/table",
          "name": "slice-ansi",
          "spec": "0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/slice-ansi",
      "name": "slice-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/slice-ansi",
      "resolved": "https://registry.npmjs.org/slice-ansi/-/slice-ansi-0.0.4.tgz",
      "version": "0.0.4",
    },
    "sntp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hawk",
          "name": "sntp",
          "spec": "1.x.x",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hoek" => EdgeOut {
          "name": "hoek",
          "spec": "2.x.x",
          "to": "node_modules/hoek",
          "type": "prod",
        },
      },
      "location": "node_modules/sntp",
      "name": "sntp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/sntp",
      "resolved": "https://registry.npmjs.org/sntp/-/sntp-1.0.9.tgz",
      "version": "1.0.9",
    },
    "source-map-support" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/source-map-support",
              "name": "source-map",
              "spec": "^0.5.6",
              "type": "prod",
            },
          },
          "location": "node_modules/source-map-support/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/source-map-support/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.6.tgz",
          "version": "0.5.6",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.4.15.tgz",
      "version": "0.4.15",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/js-yaml/node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "children": Map {
        "assert-plus" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/sshpk",
              "name": "assert-plus",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/sshpk/node_modules/assert-plus",
          "name": "assert-plus",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/sshpk/node_modules/assert-plus",
          "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/sshpk/node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "optional",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "optional",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jodid25519" => EdgeOut {
          "name": "jodid25519",
          "spec": "^1.0.0",
          "to": "node_modules/jodid25519",
          "type": "optional",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "optional",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "optional",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.10.2.tgz",
      "version": "1.10.2",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.0.tgz",
      "version": "1.0.0",
    },
    "standard" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "standard",
          "spec": "^8.6.0",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "eslint" => EdgeOut {
          "name": "eslint",
          "spec": "~3.10.2",
          "to": "node_modules/eslint",
          "type": "prod",
        },
        "eslint-config-standard" => EdgeOut {
          "name": "eslint-config-standard",
          "spec": "6.2.1",
          "to": "node_modules/eslint-config-standard",
          "type": "prod",
        },
        "eslint-config-standard-jsx" => EdgeOut {
          "name": "eslint-config-standard-jsx",
          "spec": "3.2.0",
          "to": "node_modules/eslint-config-standard-jsx",
          "type": "prod",
        },
        "eslint-plugin-promise" => EdgeOut {
          "name": "eslint-plugin-promise",
          "spec": "~3.4.0",
          "to": "node_modules/eslint-plugin-promise",
          "type": "prod",
        },
        "eslint-plugin-react" => EdgeOut {
          "name": "eslint-plugin-react",
          "spec": "~6.7.1",
          "to": "node_modules/eslint-plugin-react",
          "type": "prod",
        },
        "eslint-plugin-standard" => EdgeOut {
          "name": "eslint-plugin-standard",
          "spec": "~2.0.1",
          "to": "node_modules/eslint-plugin-standard",
          "type": "prod",
        },
        "standard-engine" => EdgeOut {
          "name": "standard-engine",
          "spec": "~5.2.0",
          "to": "node_modules/standard-engine",
          "type": "prod",
        },
      },
      "location": "node_modules/standard",
      "name": "standard",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/standard",
      "resolved": "https://registry.npmjs.org/standard/-/standard-8.6.0.tgz",
      "version": "8.6.0",
    },
    "standard-engine" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/standard-engine",
              "name": "minimist",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/standard-engine/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/standard-engine/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
          "version": "1.2.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/standard",
          "name": "standard-engine",
          "spec": "~5.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "deglob" => EdgeOut {
          "name": "deglob",
          "spec": "^2.0.0",
          "to": "node_modules/deglob",
          "type": "prod",
        },
        "find-root" => EdgeOut {
          "name": "find-root",
          "spec": "^1.0.0",
          "to": "node_modules/find-root",
          "type": "prod",
        },
        "get-stdin" => EdgeOut {
          "name": "get-stdin",
          "spec": "^5.0.1",
          "to": "node_modules/get-stdin",
          "type": "prod",
        },
        "home-or-tmp" => EdgeOut {
          "name": "home-or-tmp",
          "spec": "^2.0.0",
          "to": "node_modules/home-or-tmp",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.1.0",
          "to": "node_modules/standard-engine/node_modules/minimist",
          "type": "prod",
        },
        "pkg-config" => EdgeOut {
          "name": "pkg-config",
          "spec": "^1.0.1",
          "to": "node_modules/pkg-config",
          "type": "prod",
        },
      },
      "location": "node_modules/standard-engine",
      "name": "standard-engine",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/standard-engine",
      "resolved": "https://registry.npmjs.org/standard-engine/-/standard-engine-5.2.0.tgz",
      "version": "5.2.0",
    },
    "string_decoder" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~0.10.x",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-0.10.31.tgz",
      "version": "0.10.31",
    },
    "string-width" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "string-width",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "code-point-at" => EdgeOut {
          "name": "code-point-at",
          "spec": "^1.0.0",
          "to": "node_modules/code-point-at",
          "type": "prod",
        },
        "is-fullwidth-code-point" => EdgeOut {
          "name": "is-fullwidth-code-point",
          "spec": "^1.0.0",
          "to": "node_modules/is-fullwidth-code-point",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/string-width",
      "name": "string-width",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/string-width",
      "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.1.tgz",
      "version": "1.0.1",
    },
    "stringstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "stringstream",
          "spec": "~0.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/stringstream",
      "name": "stringstream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/stringstream",
      "resolved": "https://registry.npmjs.org/stringstream/-/stringstream-0.0.5.tgz",
      "version": "0.0.5",
    },
    "strip-ansi" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/table/node_modules/string-width",
          "name": "strip-ansi",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.0.tgz",
      "version": "3.0.0",
    },
    "strip-json-comments" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "strip-json-comments",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-json-comments",
      "name": "strip-json-comments",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/strip-json-comments",
      "resolved": "https://registry.npmjs.org/strip-json-comments/-/strip-json-comments-1.0.4.tgz",
      "version": "1.0.4",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/chalk",
          "name": "supports-color",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-2.0.0.tgz",
      "version": "2.0.0",
    },
    "table" => ArboristNode {
      "children": Map {
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/table/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "lodash" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "lodash",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/table/node_modules/lodash",
          "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.4.tgz",
          "version": "4.17.4",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/table",
              "name": "string-width",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/table/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/table/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/table/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "table",
          "spec": "^3.7.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^4.7.0",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "ajv-keywords" => EdgeOut {
          "name": "ajv-keywords",
          "spec": "^1.0.0",
          "to": "node_modules/ajv-keywords",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^1.1.1",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.0.0",
          "to": "node_modules/table/node_modules/lodash",
          "type": "prod",
        },
        "slice-ansi" => EdgeOut {
          "name": "slice-ansi",
          "spec": "0.0.4",
          "to": "node_modules/slice-ansi",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.0.0",
          "to": "node_modules/table/node_modules/string-width",
          "type": "prod",
        },
      },
      "location": "node_modules/table",
      "name": "table",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/table",
      "resolved": "https://registry.npmjs.org/table/-/table-3.8.3.tgz",
      "version": "3.8.3",
    },
    "tap" => ArboristNode {
      "children": Map {
        "inherits" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "inherits",
              "spec": "~2.0.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/inherits",
          "name": "inherits",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/inherits",
          "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.3.tgz",
          "version": "2.0.3",
        },
        "isarray" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "isarray",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/isarray",
          "name": "isarray",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/isarray",
          "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
          "version": "1.0.0",
        },
        "nyc" => ArboristNode {
          "bundleDependencies": Array [
            "archy",
            "arrify",
            "caching-transform",
            "convert-source-map",
            "debug-log",
            "default-require-extensions",
            "find-cache-dir",
            "find-up",
            "foreground-child",
            "glob",
            "istanbul-lib-coverage",
            "istanbul-lib-hook",
            "istanbul-lib-instrument",
            "istanbul-lib-report",
            "istanbul-lib-source-maps",
            "istanbul-reports",
            "md5-hex",
            "merge-source-map",
            "micromatch",
            "mkdirp",
            "resolve-from",
            "rimraf",
            "signal-exit",
            "spawn-wrap",
            "test-exclude",
            "yargs",
            "yargs-parser",
          ],
          "children": Map {
            "align-text" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "longest" => EdgeOut {
                  "name": "longest",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/longest",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/align-text",
              "name": "align-text",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/align-text",
              "version": "0.1.4",
            },
            "amdefine" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "amdefine",
                  "spec": ">=0.0.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "name": "amdefine",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/amdefine",
              "version": "1.0.1",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
              "version": "2.2.1",
            },
            "append-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "default-require-extensions" => EdgeOut {
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "name": "append-transform",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/append-transform",
              "version": "0.4.0",
            },
            "archy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "archy",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "name": "archy",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/archy",
              "version": "1.0.0",
            },
            "arr-diff" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-flatten" => EdgeOut {
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "name": "arr-diff",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/arr-diff",
              "version": "2.0.0",
            },
            "arr-flatten" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "name": "arr-flatten",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "name": "arr-flatten",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/arr-flatten",
              "version": "1.0.3",
            },
            "array-unique" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "name": "array-unique",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/array-unique",
              "version": "0.2.1",
            },
            "arrify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "name": "arrify",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/arrify",
              "version": "1.0.1",
            },
            "async" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "async",
                  "spec": "^1.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/async",
              "name": "async",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/async",
              "version": "1.5.2",
            },
            "babel-code-frame" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "chalk" => EdgeOut {
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "name": "babel-code-frame",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
              "version": "6.22.0",
            },
            "babel-generator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "detect-indent" => EdgeOut {
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "type": "prod",
                },
                "jsesc" => EdgeOut {
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "trim-right" => EdgeOut {
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "name": "babel-generator",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-generator",
              "version": "6.24.1",
            },
            "babel-messages" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "name": "babel-messages",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-messages",
              "version": "6.23.0",
            },
            "babel-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "core-js" => EdgeOut {
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/core-js",
                  "type": "prod",
                },
                "regenerator-runtime" => EdgeOut {
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "name": "babel-runtime",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
              "version": "6.23.0",
            },
            "babel-template" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "name": "babel-template",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-template",
              "version": "6.24.1",
            },
            "babel-traverse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-traverse",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-code-frame" => EdgeOut {
                  "name": "babel-code-frame",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "type": "prod",
                },
                "babel-messages" => EdgeOut {
                  "name": "babel-messages",
                  "spec": "^6.23.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-messages",
                  "type": "prod",
                },
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "globals" => EdgeOut {
                  "name": "globals",
                  "spec": "^9.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/globals",
                  "type": "prod",
                },
                "invariant" => EdgeOut {
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "name": "babel-traverse",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
              "version": "6.24.1",
            },
            "babel-types" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babel-types",
                  "spec": "^6.24.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-runtime" => EdgeOut {
                  "name": "babel-runtime",
                  "spec": "^6.22.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "type": "prod",
                },
                "esutils" => EdgeOut {
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/esutils",
                  "type": "prod",
                },
                "lodash" => EdgeOut {
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lodash",
                  "type": "prod",
                },
                "to-fast-properties" => EdgeOut {
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "name": "babel-types",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babel-types",
              "version": "6.24.1",
            },
            "babylon" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "babylon",
                  "spec": "^6.11.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "babylon",
                  "spec": "^6.15.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/babylon",
              "name": "babylon",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/babylon",
              "version": "6.17.2",
            },
            "balanced-match" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "name": "balanced-match",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/balanced-match",
              "version": "0.4.2",
            },
            "brace-expansion" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "balanced-match" => EdgeOut {
                  "name": "balanced-match",
                  "spec": "^0.4.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/balanced-match",
                  "type": "prod",
                },
                "concat-map" => EdgeOut {
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "name": "brace-expansion",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
              "version": "1.1.7",
            },
            "braces" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "braces",
                  "spec": "^1.8.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "expand-range" => EdgeOut {
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "type": "prod",
                },
                "preserve" => EdgeOut {
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/preserve",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/braces",
              "name": "braces",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/braces",
              "version": "1.8.5",
            },
            "builtin-modules" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "name": "builtin-modules",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
              "version": "1.1.1",
            },
            "caching-transform" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "caching-transform",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-hex" => EdgeOut {
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "write-file-atomic" => EdgeOut {
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "name": "caching-transform",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "version": "1.0.1",
            },
            "center-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
                "lazy-cache" => EdgeOut {
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/center-align",
              "name": "center-align",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/center-align",
              "version": "0.1.3",
            },
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "chalk",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^2.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "has-ansi" => EdgeOut {
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/chalk",
              "version": "1.1.3",
            },
            "cliui" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "wordwrap" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "name": "wordwrap",
                      "spec": "0.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "name": "wordwrap",
                  "optional": true,
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "version": "0.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "cliui",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "center-align" => EdgeOut {
                  "name": "center-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "type": "prod",
                },
                "right-align" => EdgeOut {
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/right-align",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cliui/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cliui",
              "name": "cliui",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/cliui",
              "version": "2.1.0",
            },
            "code-point-at" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "name": "code-point-at",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/code-point-at",
              "version": "1.1.0",
            },
            "commondir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/commondir",
              "name": "commondir",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/commondir",
              "version": "1.0.1",
            },
            "concat-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "name": "concat-map",
                  "spec": "0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "name": "concat-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/concat-map",
              "version": "0.0.1",
            },
            "convert-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "convert-source-map",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "name": "convert-source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "version": "1.5.0",
            },
            "core-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "core-js",
                  "spec": "^2.4.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/core-js",
              "name": "core-js",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/core-js",
              "version": "2.4.1",
            },
            "cross-spawn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "cross-spawn",
                  "spec": "^4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "lru-cache" => EdgeOut {
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.9",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "name": "cross-spawn",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
              "version": "4.0.2",
            },
            "debug" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "debug",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "debug",
                  "spec": "^2.6.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ms" => EdgeOut {
                  "name": "ms",
                  "spec": "2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ms",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug",
              "name": "debug",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/debug",
              "version": "2.6.8",
            },
            "debug-log" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "debug-log",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "name": "debug-log",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "version": "1.0.1",
            },
            "decamelize" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "name": "decamelize",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/decamelize",
              "version": "1.2.0",
            },
            "default-require-extensions" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "name": "default-require-extensions",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "name": "default-require-extensions",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "version": "1.0.0",
            },
            "detect-indent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "detect-indent",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "repeating" => EdgeOut {
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "name": "detect-indent",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/detect-indent",
              "version": "4.0.0",
            },
            "error-ex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-arrayish" => EdgeOut {
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "name": "error-ex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/error-ex",
              "version": "1.3.1",
            },
            "escape-string-regexp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "escape-string-regexp",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "name": "escape-string-regexp",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/escape-string-regexp",
              "version": "1.0.5",
            },
            "esutils" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "esutils",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/esutils",
              "name": "esutils",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/esutils",
              "version": "2.0.2",
            },
            "execa" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "execa",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "get-stream" => EdgeOut {
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "type": "prod",
                },
                "is-stream" => EdgeOut {
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
                  "type": "prod",
                },
                "npm-run-path" => EdgeOut {
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "type": "prod",
                },
                "p-finally" => EdgeOut {
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "strip-eof" => EdgeOut {
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/execa",
              "name": "execa",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/execa",
              "version": "0.5.1",
            },
            "expand-brackets" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-posix-bracket" => EdgeOut {
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "name": "expand-brackets",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
              "version": "0.1.5",
            },
            "expand-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "expand-range",
                  "spec": "^1.8.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fill-range" => EdgeOut {
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "name": "expand-range",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/expand-range",
              "version": "1.8.2",
            },
            "extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/extglob",
              "name": "extglob",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/extglob",
              "version": "0.3.2",
            },
            "filename-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "name": "filename-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/filename-regex",
              "version": "2.0.1",
            },
            "fill-range" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-range",
                  "name": "fill-range",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "isobject" => EdgeOut {
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "type": "prod",
                },
                "randomatic" => EdgeOut {
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "type": "prod",
                },
                "repeat-element" => EdgeOut {
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
                  "type": "prod",
                },
                "repeat-string" => EdgeOut {
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "name": "fill-range",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/fill-range",
              "version": "2.2.3",
            },
            "find-cache-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-cache-dir",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "commondir" => EdgeOut {
                  "name": "commondir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/commondir",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "pkg-dir" => EdgeOut {
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "name": "find-cache-dir",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "version": "0.1.1",
            },
            "find-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "find-up",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "find-up",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "locate-path" => EdgeOut {
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "name": "find-up",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/find-up",
              "version": "2.1.0",
            },
            "for-in" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-in",
              "name": "for-in",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/for-in",
              "version": "1.0.2",
            },
            "for-own" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-in" => EdgeOut {
                  "name": "for-in",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-in",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/for-own",
              "name": "for-own",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/for-own",
              "version": "0.1.5",
            },
            "foreground-child" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "foreground-child",
                  "spec": "^1.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "cross-spawn" => EdgeOut {
                  "name": "cross-spawn",
                  "spec": "^4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "name": "foreground-child",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "version": "1.5.6",
            },
            "fs.realpath" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "name": "fs.realpath",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
              "version": "1.0.0",
            },
            "get-caller-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "name": "get-caller-file",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
              "version": "1.0.2",
            },
            "get-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "get-stream",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "name": "get-stream",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/get-stream",
              "version": "2.3.1",
            },
            "glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "glob",
                  "spec": "^7.0.6",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "name": "glob",
                  "spec": "^7.0.5",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "fs.realpath" => EdgeOut {
                  "name": "fs.realpath",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/fs.realpath",
                  "type": "prod",
                },
                "inflight" => EdgeOut {
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/inherits",
                  "type": "prod",
                },
                "minimatch" => EdgeOut {
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
                  "type": "prod",
                },
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "path-is-absolute" => EdgeOut {
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "name": "glob",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/glob",
              "version": "7.1.2",
            },
            "glob-base" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-parent" => EdgeOut {
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "name": "glob-base",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/glob-base",
              "version": "0.3.0",
            },
            "glob-parent" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "glob-parent",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "name": "glob-parent",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/glob-parent",
              "version": "2.0.0",
            },
            "globals" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "globals",
                  "spec": "^9.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/globals",
              "name": "globals",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/globals",
              "version": "9.17.0",
            },
            "graceful-fs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "name": "graceful-fs",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
              "version": "4.1.11",
            },
            "handlebars" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "source-map" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                      "name": "source-map",
                      "spec": "^0.4.4",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "amdefine" => EdgeOut {
                      "name": "amdefine",
                      "spec": ">=0.0.4",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/amdefine",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "name": "source-map",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "version": "0.4.4",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "async" => EdgeOut {
                  "name": "async",
                  "spec": "^1.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/async",
                  "type": "prod",
                },
                "optimist" => EdgeOut {
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.4.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-js" => EdgeOut {
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "name": "handlebars",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/handlebars",
              "version": "4.0.10",
            },
            "has-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "has-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "name": "has-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/has-ansi",
              "version": "2.0.0",
            },
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/has-flag",
              "version": "1.0.0",
            },
            "hosted-git-info" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "name": "hosted-git-info",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
              "version": "2.4.2",
            },
            "imurmurhash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "name": "imurmurhash",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
              "version": "0.1.4",
            },
            "inflight" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inflight",
                  "spec": "^1.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "once" => EdgeOut {
                  "name": "once",
                  "spec": "^1.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "type": "prod",
                },
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inflight",
              "name": "inflight",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/inflight",
              "version": "1.0.6",
            },
            "inherits" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "inherits",
                  "spec": "2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/inherits",
              "name": "inherits",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/inherits",
              "version": "2.0.3",
            },
            "invariant" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "invariant",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "loose-envify" => EdgeOut {
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invariant",
              "name": "invariant",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/invariant",
              "version": "2.2.2",
            },
            "invert-kv" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "name": "invert-kv",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/invert-kv",
              "version": "1.0.0",
            },
            "is-arrayish" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "name": "is-arrayish",
                  "spec": "^0.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "name": "is-arrayish",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-arrayish",
              "version": "0.2.1",
            },
            "is-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "name": "is-buffer",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-buffer",
              "version": "1.1.5",
            },
            "is-builtin-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "builtin-modules" => EdgeOut {
                  "name": "builtin-modules",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/builtin-modules",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "name": "is-builtin-module",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
              "version": "1.0.0",
            },
            "is-dotfile" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "name": "is-dotfile",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
              "version": "1.0.3",
            },
            "is-equal-shallow" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "name": "is-equal-shallow",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
              "version": "0.1.3",
            },
            "is-extendable" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "name": "is-extendable",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-extendable",
              "version": "0.1.1",
            },
            "is-extglob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "name": "is-extglob",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-extglob",
              "version": "1.0.0",
            },
            "is-finite" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/repeating",
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "name": "is-finite",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-finite",
              "version": "1.0.2",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
              "version": "1.0.0",
            },
            "is-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob-parent",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "name": "is-glob",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-glob",
              "version": "2.0.1",
            },
            "is-number" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "is-number",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-number",
              "name": "is-number",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-number",
              "version": "2.1.0",
            },
            "is-posix-bracket" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "name": "is-posix-bracket",
                  "spec": "^0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "name": "is-posix-bracket",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-posix-bracket",
              "version": "0.1.1",
            },
            "is-primitive" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "name": "is-primitive",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-primitive",
              "version": "2.0.0",
            },
            "is-stream" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "is-stream",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "name": "is-stream",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-stream",
              "version": "1.1.0",
            },
            "is-utf8" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "name": "is-utf8",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/is-utf8",
              "version": "0.2.1",
            },
            "isarray" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/isobject",
                  "name": "isarray",
                  "spec": "1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isarray",
              "name": "isarray",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/isarray",
              "version": "1.0.0",
            },
            "isexe" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isexe",
              "name": "isexe",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/isexe",
              "version": "2.0.0",
            },
            "isobject" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "isobject",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isarray",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/isobject",
              "name": "isobject",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/isobject",
              "version": "2.1.0",
            },
            "istanbul-lib-coverage" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "name": "istanbul-lib-coverage",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "version": "1.1.1",
            },
            "istanbul-lib-hook" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-hook",
                  "spec": "^1.0.7",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "append-transform" => EdgeOut {
                  "name": "append-transform",
                  "spec": "^0.4.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/append-transform",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "name": "istanbul-lib-hook",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "version": "1.0.7",
            },
            "istanbul-lib-instrument" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-instrument",
                  "spec": "^1.7.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "babel-generator" => EdgeOut {
                  "name": "babel-generator",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "type": "prod",
                },
                "babel-template" => EdgeOut {
                  "name": "babel-template",
                  "spec": "^6.16.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "type": "prod",
                },
                "babel-traverse" => EdgeOut {
                  "name": "babel-traverse",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "type": "prod",
                },
                "babel-types" => EdgeOut {
                  "name": "babel-types",
                  "spec": "^6.18.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "type": "prod",
                },
                "babylon" => EdgeOut {
                  "name": "babylon",
                  "spec": "^6.13.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/babylon",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "name": "istanbul-lib-instrument",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "version": "1.7.2",
            },
            "istanbul-lib-report" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "supports-color" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                      "name": "supports-color",
                      "spec": "^3.1.2",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "has-flag" => EdgeOut {
                      "name": "has-flag",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/has-flag",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "name": "supports-color",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "version": "3.2.3",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-report",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "path-parse" => EdgeOut {
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^3.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "name": "istanbul-lib-report",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "version": "1.1.1",
            },
            "istanbul-lib-source-maps" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-lib-source-maps",
                  "spec": "^1.2.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "debug" => EdgeOut {
                  "name": "debug",
                  "spec": "^2.6.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "type": "prod",
                },
                "istanbul-lib-coverage" => EdgeOut {
                  "name": "istanbul-lib-coverage",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "name": "istanbul-lib-source-maps",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "version": "1.2.1",
            },
            "istanbul-reports" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "istanbul-reports",
                  "spec": "^1.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "handlebars" => EdgeOut {
                  "name": "handlebars",
                  "spec": "^4.0.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "name": "istanbul-reports",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "version": "1.1.1",
            },
            "js-tokens" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-code-frame",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "name": "js-tokens",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/js-tokens",
              "version": "3.0.1",
            },
            "jsesc" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "jsesc",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "name": "jsesc",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/jsesc",
              "version": "1.3.0",
            },
            "kind-of" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-buffer" => EdgeOut {
                  "name": "is-buffer",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-buffer",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "name": "kind-of",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/kind-of",
              "version": "3.2.2",
            },
            "lazy-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/center-align",
                  "name": "lazy-cache",
                  "spec": "^1.0.3",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "name": "lazy-cache",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/lazy-cache",
              "version": "1.0.4",
            },
            "lcid" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "invert-kv" => EdgeOut {
                  "name": "invert-kv",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/invert-kv",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lcid",
              "name": "lcid",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/lcid",
              "version": "1.0.0",
            },
            "load-json-file" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "parse-json" => EdgeOut {
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
                "strip-bom" => EdgeOut {
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "name": "load-json-file",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/load-json-file",
              "version": "1.1.0",
            },
            "locate-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "path-exists" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                      "name": "path-exists",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "name": "path-exists",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                  "name": "locate-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-locate" => EdgeOut {
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "type": "prod",
                },
                "path-exists" => EdgeOut {
                  "name": "path-exists",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/locate-path/node_modules/path-exists",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "name": "locate-path",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/locate-path",
              "version": "2.0.0",
            },
            "lodash" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-template",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-traverse",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "lodash",
                  "spec": "^4.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lodash",
              "name": "lodash",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/lodash",
              "version": "4.17.4",
            },
            "longest" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "longest",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/longest",
              "name": "longest",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/longest",
              "version": "1.0.1",
            },
            "loose-envify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/invariant",
                  "name": "loose-envify",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "js-tokens" => EdgeOut {
                  "name": "js-tokens",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/js-tokens",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "name": "loose-envify",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/loose-envify",
              "version": "1.3.1",
            },
            "lru-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "lru-cache",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pseudomap" => EdgeOut {
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
                  "type": "prod",
                },
                "yallist" => EdgeOut {
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yallist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "name": "lru-cache",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/lru-cache",
              "version": "4.0.2",
            },
            "md5-hex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "md5-hex",
                  "spec": "^1.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "md5-o-matic" => EdgeOut {
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "name": "md5-hex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "version": "1.3.0",
            },
            "md5-o-matic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
                  "name": "md5-o-matic",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "name": "md5-o-matic",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/md5-o-matic",
              "version": "0.1.1",
            },
            "mem" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "name": "mem",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mem",
              "name": "mem",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/mem",
              "version": "1.1.0",
            },
            "merge-source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "merge-source-map",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "name": "merge-source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "version": "1.0.3",
            },
            "micromatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arr-diff" => EdgeOut {
                  "name": "arr-diff",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arr-diff",
                  "type": "prod",
                },
                "array-unique" => EdgeOut {
                  "name": "array-unique",
                  "spec": "^0.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/array-unique",
                  "type": "prod",
                },
                "braces" => EdgeOut {
                  "name": "braces",
                  "spec": "^1.8.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "type": "prod",
                },
                "expand-brackets" => EdgeOut {
                  "name": "expand-brackets",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/expand-brackets",
                  "type": "prod",
                },
                "extglob" => EdgeOut {
                  "name": "extglob",
                  "spec": "^0.3.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/extglob",
                  "type": "prod",
                },
                "filename-regex" => EdgeOut {
                  "name": "filename-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/filename-regex",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
                "normalize-path" => EdgeOut {
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "type": "prod",
                },
                "object.omit" => EdgeOut {
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
                  "type": "prod",
                },
                "parse-glob" => EdgeOut {
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
                  "type": "prod",
                },
                "regex-cache" => EdgeOut {
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "name": "micromatch",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "version": "2.3.11",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/mimic-fn",
              "version": "1.1.0",
            },
            "minimatch" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "minimatch",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "brace-expansion" => EdgeOut {
                  "name": "brace-expansion",
                  "spec": "^1.1.7",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/brace-expansion",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "name": "minimatch",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/minimatch",
              "version": "3.0.4",
            },
            "minimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "name": "minimist",
                  "spec": "0.0.8",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/minimist",
              "name": "minimist",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/minimist",
              "version": "0.0.8",
            },
            "mkdirp" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "mkdirp",
                  "spec": "^0.5.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "0.0.8",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "name": "mkdirp",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "version": "0.5.1",
            },
            "ms" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/debug",
                  "name": "ms",
                  "spec": "2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/ms",
              "name": "ms",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/ms",
              "version": "2.0.0",
            },
            "normalize-package-data" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "hosted-git-info" => EdgeOut {
                  "name": "hosted-git-info",
                  "spec": "^2.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/hosted-git-info",
                  "type": "prod",
                },
                "is-builtin-module" => EdgeOut {
                  "name": "is-builtin-module",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-builtin-module",
                  "type": "prod",
                },
                "semver" => EdgeOut {
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/semver",
                  "type": "prod",
                },
                "validate-npm-package-license" => EdgeOut {
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "name": "normalize-package-data",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
              "version": "2.3.8",
            },
            "normalize-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "normalize-path",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "remove-trailing-separator" => EdgeOut {
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "name": "normalize-path",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/normalize-path",
              "version": "2.1.1",
            },
            "npm-run-path" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "npm-run-path",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "path-key" => EdgeOut {
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-key",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "name": "npm-run-path",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
              "version": "2.0.2",
            },
            "number-is-nan" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "name": "number-is-nan",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/number-is-nan",
              "version": "1.0.1",
            },
            "object-assign" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "object-assign",
                  "spec": "^4.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "name": "object-assign",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/object-assign",
              "version": "4.1.1",
            },
            "object.omit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "object.omit",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "for-own" => EdgeOut {
                  "name": "for-own",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/for-own",
                  "type": "prod",
                },
                "is-extendable" => EdgeOut {
                  "name": "is-extendable",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extendable",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "name": "object.omit",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/object.omit",
              "version": "2.0.1",
            },
            "once" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "once",
                  "spec": "^1.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "wrappy" => EdgeOut {
                  "name": "wrappy",
                  "spec": "1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/once",
              "name": "once",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/once",
              "version": "1.4.0",
            },
            "optimist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "optimist",
                  "spec": "^0.6.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "minimist" => EdgeOut {
                  "name": "minimist",
                  "spec": "~0.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/minimist",
                  "type": "prod",
                },
                "wordwrap" => EdgeOut {
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/optimist",
              "name": "optimist",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/optimist",
              "version": "0.6.1",
            },
            "os-homedir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "name": "os-homedir",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/os-homedir",
              "version": "1.0.2",
            },
            "os-locale" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "execa" => EdgeOut {
                  "name": "execa",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "type": "prod",
                },
                "lcid" => EdgeOut {
                  "name": "lcid",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/lcid",
                  "type": "prod",
                },
                "mem" => EdgeOut {
                  "name": "mem",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mem",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "name": "os-locale",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/os-locale",
              "version": "2.0.0",
            },
            "p-finally" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "p-finally",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "name": "p-finally",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/p-finally",
              "version": "1.0.0",
            },
            "p-limit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "name": "p-limit",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/p-limit",
              "version": "1.1.0",
            },
            "p-locate" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/locate-path",
                  "name": "p-locate",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "p-limit" => EdgeOut {
                  "name": "p-limit",
                  "spec": "^1.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/p-limit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "name": "p-locate",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/p-locate",
              "version": "2.0.0",
            },
            "parse-glob" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "parse-glob",
                  "spec": "^3.0.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob-base" => EdgeOut {
                  "name": "glob-base",
                  "spec": "^0.3.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob-base",
                  "type": "prod",
                },
                "is-dotfile" => EdgeOut {
                  "name": "is-dotfile",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-dotfile",
                  "type": "prod",
                },
                "is-extglob" => EdgeOut {
                  "name": "is-extglob",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-extglob",
                  "type": "prod",
                },
                "is-glob" => EdgeOut {
                  "name": "is-glob",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "name": "parse-glob",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/parse-glob",
              "version": "3.0.4",
            },
            "parse-json" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "parse-json",
                  "spec": "^2.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "error-ex" => EdgeOut {
                  "name": "error-ex",
                  "spec": "^1.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/error-ex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "name": "parse-json",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/parse-json",
              "version": "2.2.0",
            },
            "path-exists" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "path-exists",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "name": "path-exists",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/path-exists",
              "version": "2.1.0",
            },
            "path-is-absolute" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "name": "path-is-absolute",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "name": "path-is-absolute",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/path-is-absolute",
              "version": "1.0.1",
            },
            "path-key" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/npm-run-path",
                  "name": "path-key",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-key",
              "name": "path-key",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/path-key",
              "version": "2.0.1",
            },
            "path-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
                  "name": "path-parse",
                  "spec": "^1.0.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "name": "path-parse",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/path-parse",
              "version": "1.0.5",
            },
            "path-type" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "pify" => EdgeOut {
                  "name": "pify",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                  "type": "prod",
                },
                "pinkie-promise" => EdgeOut {
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/path-type",
              "name": "path-type",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/path-type",
              "version": "1.1.0",
            },
            "pify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "pify",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pify",
              "name": "pify",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pify",
              "version": "2.3.0",
            },
            "pinkie" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "name": "pinkie",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pinkie",
              "version": "2.0.4",
            },
            "pinkie-promise" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/get-stream",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "pinkie-promise",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "pinkie" => EdgeOut {
                  "name": "pinkie",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "name": "pinkie-promise",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
              "version": "2.0.1",
            },
            "pkg-dir" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
                  "name": "pkg-dir",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir/node_modules/find-up",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "name": "pkg-dir",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pkg-dir",
              "version": "1.0.0",
            },
            "preserve" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "preserve",
                  "spec": "^0.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/preserve",
              "name": "preserve",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/preserve",
              "version": "0.2.0",
            },
            "pseudomap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "pseudomap",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "name": "pseudomap",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/pseudomap",
              "version": "1.0.2",
            },
            "randomatic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "randomatic",
                  "spec": "^1.1.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-number" => EdgeOut {
                  "name": "is-number",
                  "spec": "^2.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-number",
                  "type": "prod",
                },
                "kind-of" => EdgeOut {
                  "name": "kind-of",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/kind-of",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "name": "randomatic",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/randomatic",
              "version": "1.1.6",
            },
            "read-pkg" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "load-json-file" => EdgeOut {
                  "name": "load-json-file",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "type": "prod",
                },
                "normalize-package-data" => EdgeOut {
                  "name": "normalize-package-data",
                  "spec": "^2.3.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "type": "prod",
                },
                "path-type" => EdgeOut {
                  "name": "path-type",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/path-type",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "name": "read-pkg",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/read-pkg",
              "version": "1.1.0",
            },
            "read-pkg-up" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "find-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                      "name": "find-up",
                      "spec": "^1.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "path-exists" => EdgeOut {
                      "name": "path-exists",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/path-exists",
                      "type": "prod",
                    },
                    "pinkie-promise" => EdgeOut {
                      "name": "pinkie-promise",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pinkie-promise",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "name": "find-up",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "version": "1.1.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "find-up" => EdgeOut {
                  "name": "find-up",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up/node_modules/find-up",
                  "type": "prod",
                },
                "read-pkg" => EdgeOut {
                  "name": "read-pkg",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "name": "read-pkg-up",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
              "version": "1.0.1",
            },
            "regenerator-runtime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-runtime",
                  "name": "regenerator-runtime",
                  "spec": "^0.10.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "name": "regenerator-runtime",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/regenerator-runtime",
              "version": "0.10.5",
            },
            "regex-cache" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "name": "regex-cache",
                  "spec": "^0.4.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-equal-shallow" => EdgeOut {
                  "name": "is-equal-shallow",
                  "spec": "^0.1.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-equal-shallow",
                  "type": "prod",
                },
                "is-primitive" => EdgeOut {
                  "name": "is-primitive",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-primitive",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "name": "regex-cache",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/regex-cache",
              "version": "0.4.3",
            },
            "remove-trailing-separator" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-path",
                  "name": "remove-trailing-separator",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "name": "remove-trailing-separator",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/remove-trailing-separator",
              "version": "1.0.1",
            },
            "repeat-element" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/braces",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-element",
                  "spec": "^1.1.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "name": "repeat-element",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/repeat-element",
              "version": "1.1.2",
            },
            "repeat-string" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/fill-range",
                  "name": "repeat-string",
                  "spec": "^1.5.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "name": "repeat-string",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/repeat-string",
              "version": "1.6.1",
            },
            "repeating" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/detect-indent",
                  "name": "repeating",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-finite" => EdgeOut {
                  "name": "is-finite",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-finite",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/repeating",
              "name": "repeating",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/repeating",
              "version": "2.0.1",
            },
            "require-directory" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "name": "require-directory",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/require-directory",
              "version": "2.1.1",
            },
            "require-main-filename" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "name": "require-main-filename",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
              "version": "1.0.1",
            },
            "resolve-from" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "resolve-from",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "name": "resolve-from",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "version": "2.0.0",
            },
            "right-align" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                  "name": "right-align",
                  "spec": "^0.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "align-text" => EdgeOut {
                  "name": "align-text",
                  "spec": "^0.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/align-text",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/right-align",
              "name": "right-align",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/right-align",
              "version": "0.1.3",
            },
            "rimraf" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "rimraf",
                  "spec": "^2.5.4",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "rimraf",
                  "spec": "^2.6.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "glob" => EdgeOut {
                  "name": "glob",
                  "spec": "^7.0.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "name": "rimraf",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "version": "2.6.1",
            },
            "semver" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
                  "name": "semver",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "semver",
                  "spec": "2 || 3 || 4 || 5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/semver",
              "name": "semver",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/semver",
              "version": "5.3.0",
            },
            "set-blocking" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "name": "set-blocking",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/set-blocking",
              "version": "2.0.0",
            },
            "signal-exit" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "signal-exit",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "name": "signal-exit",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "name": "signal-exit",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "version": "3.0.2",
            },
            "slide" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
                  "name": "slide",
                  "spec": "^1.1.5",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/slide",
              "name": "slide",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/slide",
              "version": "1.1.6",
            },
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
                  "name": "source-map",
                  "spec": "^0.5.3",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/source-map",
              "version": "0.5.6",
            },
            "spawn-wrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "spawn-wrap",
                  "spec": "^1.3.6",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "foreground-child" => EdgeOut {
                  "name": "foreground-child",
                  "spec": "^1.5.6",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
                  "type": "prod",
                },
                "mkdirp" => EdgeOut {
                  "name": "mkdirp",
                  "spec": "^0.5.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
                  "type": "prod",
                },
                "os-homedir" => EdgeOut {
                  "name": "os-homedir",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-homedir",
                  "type": "prod",
                },
                "rimraf" => EdgeOut {
                  "name": "rimraf",
                  "spec": "^2.3.3",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
                  "type": "prod",
                },
                "which" => EdgeOut {
                  "name": "which",
                  "spec": "^1.2.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "name": "spawn-wrap",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "version": "1.3.6",
            },
            "spdx-correct" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-license-ids" => EdgeOut {
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "name": "spdx-correct",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
              "version": "1.0.2",
            },
            "spdx-expression-parse" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "name": "spdx-expression-parse",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
              "version": "1.0.4",
            },
            "spdx-license-ids" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "name": "spdx-license-ids",
                  "spec": "^1.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "name": "spdx-license-ids",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/spdx-license-ids",
              "version": "1.2.2",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "is-fullwidth-code-point" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                      "name": "is-fullwidth-code-point",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "name": "is-fullwidth-code-point",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "version": "2.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/string-width",
              "version": "2.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "strip-bom" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/load-json-file",
                  "name": "strip-bom",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "is-utf8" => EdgeOut {
                  "name": "is-utf8",
                  "spec": "^0.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/is-utf8",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "name": "strip-bom",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/strip-bom",
              "version": "2.0.0",
            },
            "strip-eof" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/execa",
                  "name": "strip-eof",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "name": "strip-eof",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/strip-eof",
              "version": "1.0.0",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/supports-color",
              "version": "2.0.0",
            },
            "test-exclude" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "test-exclude",
                  "spec": "^4.1.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "arrify" => EdgeOut {
                  "name": "arrify",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
                  "type": "prod",
                },
                "micromatch" => EdgeOut {
                  "name": "micromatch",
                  "spec": "^2.3.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
                  "type": "prod",
                },
                "object-assign" => EdgeOut {
                  "name": "object-assign",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/object-assign",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "name": "test-exclude",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "version": "4.1.1",
            },
            "to-fast-properties" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-types",
                  "name": "to-fast-properties",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "name": "to-fast-properties",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/to-fast-properties",
              "version": "1.0.3",
            },
            "trim-right" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/babel-generator",
                  "name": "trim-right",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "name": "trim-right",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/trim-right",
              "version": "1.0.1",
            },
            "uglify-js" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "name": "camelcase",
                  "optional": true,
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                  "version": "1.2.1",
                },
                "yargs" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                      "name": "yargs",
                      "spec": "~3.10.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^1.0.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/camelcase",
                      "type": "prod",
                    },
                    "cliui" => EdgeOut {
                      "name": "cliui",
                      "spec": "^2.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/cliui",
                      "type": "prod",
                    },
                    "decamelize" => EdgeOut {
                      "name": "decamelize",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                      "type": "prod",
                    },
                    "window-size" => EdgeOut {
                      "name": "window-size",
                      "spec": "0.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/window-size",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "yargs",
                  "optional": true,
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "version": "3.10.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/handlebars",
                  "name": "uglify-js",
                  "spec": "^2.6",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "source-map" => EdgeOut {
                  "name": "source-map",
                  "spec": "~0.5.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/source-map",
                  "type": "prod",
                },
                "uglify-to-browserify" => EdgeOut {
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
                  "type": "optional",
                },
                "yargs" => EdgeOut {
                  "name": "yargs",
                  "spec": "~3.10.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "name": "uglify-js",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/uglify-js",
              "version": "2.8.27",
            },
            "uglify-to-browserify" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js",
                  "name": "uglify-to-browserify",
                  "spec": "~1.0.0",
                  "type": "optional",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "name": "uglify-to-browserify",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/uglify-to-browserify",
              "version": "1.0.2",
            },
            "validate-npm-package-license" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                  "name": "validate-npm-package-license",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "spdx-correct" => EdgeOut {
                  "name": "spdx-correct",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-correct",
                  "type": "prod",
                },
                "spdx-expression-parse" => EdgeOut {
                  "name": "spdx-expression-parse",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/spdx-expression-parse",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "name": "validate-npm-package-license",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/validate-npm-package-license",
              "version": "3.0.1",
            },
            "which" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/cross-spawn",
                  "name": "which",
                  "spec": "^1.2.9",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
                  "name": "which",
                  "spec": "^1.2.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "isexe" => EdgeOut {
                  "name": "isexe",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/isexe",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which",
              "name": "which",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/which",
              "version": "1.2.14",
            },
            "which-module" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/which-module",
              "name": "which-module",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/which-module",
              "version": "2.0.0",
            },
            "window-size" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/uglify-js/node_modules/yargs",
                  "name": "window-size",
                  "spec": "0.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/window-size",
              "name": "window-size",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/window-size",
              "version": "0.1.0",
            },
            "wordwrap" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/optimist",
                  "name": "wordwrap",
                  "spec": "~0.0.2",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "name": "wordwrap",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/wordwrap",
              "version": "0.0.3",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "string-width" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "code-point-at" => EdgeOut {
                      "name": "code-point-at",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                      "type": "prod",
                    },
                    "is-fullwidth-code-point" => EdgeOut {
                      "name": "is-fullwidth-code-point",
                      "spec": "^1.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "string-width",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "version": "1.0.2",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "wrap-ansi",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
              "version": "2.1.0",
            },
            "wrappy" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/inflight",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/once",
                  "name": "wrappy",
                  "spec": "1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "name": "wrappy",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/wrappy",
              "version": "1.0.2",
            },
            "write-file-atomic" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
                  "name": "write-file-atomic",
                  "spec": "^1.1.4",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "graceful-fs" => EdgeOut {
                  "name": "graceful-fs",
                  "spec": "^4.1.11",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                  "type": "prod",
                },
                "imurmurhash" => EdgeOut {
                  "name": "imurmurhash",
                  "spec": "^0.1.4",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/imurmurhash",
                  "type": "prod",
                },
                "slide" => EdgeOut {
                  "name": "slide",
                  "spec": "^1.1.5",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/slide",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "name": "write-file-atomic",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/write-file-atomic",
              "version": "1.3.4",
            },
            "y18n" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/y18n",
              "name": "y18n",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/y18n",
              "version": "3.2.1",
            },
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc/node_modules/lru-cache",
                  "name": "yallist",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yallist",
              "version": "2.1.2",
            },
            "yargs" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "version": "4.1.0",
                },
                "cliui" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "children": Map {
                    "string-width" => ArboristNode {
                      "bundled": true,
                      "bundler": "node_modules/tap/node_modules/nyc",
                      "dev": true,
                      "edgesIn": Set {
                        EdgeIn {
                          "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                          "name": "string-width",
                          "spec": "^1.0.1",
                          "type": "prod",
                        },
                      },
                      "edgesOut": Map {
                        "code-point-at" => EdgeOut {
                          "name": "code-point-at",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/code-point-at",
                          "type": "prod",
                        },
                        "is-fullwidth-code-point" => EdgeOut {
                          "name": "is-fullwidth-code-point",
                          "spec": "^1.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/is-fullwidth-code-point",
                          "type": "prod",
                        },
                        "strip-ansi" => EdgeOut {
                          "name": "strip-ansi",
                          "spec": "^3.0.0",
                          "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                          "type": "prod",
                        },
                      },
                      "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "name": "string-width",
                      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "version": "1.0.2",
                    },
                  },
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "cliui",
                      "spec": "^3.2.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "string-width" => EdgeOut {
                      "name": "string-width",
                      "spec": "^1.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui/node_modules/string-width",
                      "type": "prod",
                    },
                    "strip-ansi" => EdgeOut {
                      "name": "strip-ansi",
                      "spec": "^3.0.1",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/strip-ansi",
                      "type": "prod",
                    },
                    "wrap-ansi" => EdgeOut {
                      "name": "wrap-ansi",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/wrap-ansi",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "name": "cliui",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "version": "3.2.0",
                },
                "load-json-file" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "graceful-fs" => EdgeOut {
                      "name": "graceful-fs",
                      "spec": "^4.1.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/graceful-fs",
                      "type": "prod",
                    },
                    "parse-json" => EdgeOut {
                      "name": "parse-json",
                      "spec": "^2.2.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/parse-json",
                      "type": "prod",
                    },
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                    "strip-bom" => EdgeOut {
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "name": "load-json-file",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                  "version": "2.0.0",
                },
                "path-type" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "pify" => EdgeOut {
                      "name": "pify",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/pify",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "name": "path-type",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                  "version": "2.0.0",
                },
                "read-pkg" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "load-json-file" => EdgeOut {
                      "name": "load-json-file",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "type": "prod",
                    },
                    "normalize-package-data" => EdgeOut {
                      "name": "normalize-package-data",
                      "spec": "^2.3.2",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/normalize-package-data",
                      "type": "prod",
                    },
                    "path-type" => EdgeOut {
                      "name": "path-type",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/path-type",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "name": "read-pkg",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                  "version": "2.0.0",
                },
                "read-pkg-up" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "read-pkg-up",
                      "spec": "^2.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "find-up" => EdgeOut {
                      "name": "find-up",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
                      "type": "prod",
                    },
                    "read-pkg" => EdgeOut {
                      "name": "read-pkg",
                      "spec": "^2.0.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "name": "read-pkg-up",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "version": "2.0.0",
                },
                "strip-bom" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/load-json-file",
                      "name": "strip-bom",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "name": "strip-bom",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/strip-bom",
                  "version": "3.0.0",
                },
                "yargs-parser" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs",
                      "name": "yargs-parser",
                      "spec": "^7.0.0",
                      "type": "prod",
                    },
                  },
                  "edgesOut": Map {
                    "camelcase" => EdgeOut {
                      "name": "camelcase",
                      "spec": "^4.1.0",
                      "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "name": "yargs-parser",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "version": "7.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs",
                  "spec": "^8.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/camelcase",
                  "type": "prod",
                },
                "cliui" => EdgeOut {
                  "name": "cliui",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/cliui",
                  "type": "prod",
                },
                "decamelize" => EdgeOut {
                  "name": "decamelize",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/decamelize",
                  "type": "prod",
                },
                "get-caller-file" => EdgeOut {
                  "name": "get-caller-file",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/get-caller-file",
                  "type": "prod",
                },
                "os-locale" => EdgeOut {
                  "name": "os-locale",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/os-locale",
                  "type": "prod",
                },
                "read-pkg-up" => EdgeOut {
                  "name": "read-pkg-up",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/read-pkg-up",
                  "type": "prod",
                },
                "require-directory" => EdgeOut {
                  "name": "require-directory",
                  "spec": "^2.1.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-directory",
                  "type": "prod",
                },
                "require-main-filename" => EdgeOut {
                  "name": "require-main-filename",
                  "spec": "^1.0.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/require-main-filename",
                  "type": "prod",
                },
                "set-blocking" => EdgeOut {
                  "name": "set-blocking",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/set-blocking",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/string-width",
                  "type": "prod",
                },
                "which-module" => EdgeOut {
                  "name": "which-module",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/which-module",
                  "type": "prod",
                },
                "y18n" => EdgeOut {
                  "name": "y18n",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/y18n",
                  "type": "prod",
                },
                "yargs-parser" => EdgeOut {
                  "name": "yargs-parser",
                  "spec": "^7.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs/node_modules/yargs-parser",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "name": "yargs",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs",
              "version": "8.0.1",
            },
            "yargs-parser" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap/node_modules/nyc",
              "children": Map {
                "camelcase" => ArboristNode {
                  "bundled": true,
                  "bundler": "node_modules/tap/node_modules/nyc",
                  "dev": true,
                  "edgesIn": Set {
                    EdgeIn {
                      "from": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
                      "name": "camelcase",
                      "spec": "^3.0.0",
                      "type": "prod",
                    },
                  },
                  "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "name": "camelcase",
                  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "version": "3.0.0",
                },
              },
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/nyc",
                  "name": "yargs-parser",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "camelcase" => EdgeOut {
                  "name": "camelcase",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser/node_modules/camelcase",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "name": "yargs-parser",
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "version": "5.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "nyc",
              "spec": "^11.0.2-candidate.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "archy" => EdgeOut {
              "name": "archy",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/archy",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/arrify",
              "type": "prod",
            },
            "caching-transform" => EdgeOut {
              "name": "caching-transform",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/caching-transform",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.3.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug-log" => EdgeOut {
              "name": "debug-log",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/debug-log",
              "type": "prod",
            },
            "default-require-extensions" => EdgeOut {
              "name": "default-require-extensions",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/default-require-extensions",
              "type": "prod",
            },
            "find-cache-dir" => EdgeOut {
              "name": "find-cache-dir",
              "spec": "^0.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-cache-dir",
              "type": "prod",
            },
            "find-up" => EdgeOut {
              "name": "find-up",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/find-up",
              "type": "prod",
            },
            "foreground-child" => EdgeOut {
              "name": "foreground-child",
              "spec": "^1.5.3",
              "to": "node_modules/tap/node_modules/nyc/node_modules/foreground-child",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/glob",
              "type": "prod",
            },
            "istanbul-lib-coverage" => EdgeOut {
              "name": "istanbul-lib-coverage",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-coverage",
              "type": "prod",
            },
            "istanbul-lib-hook" => EdgeOut {
              "name": "istanbul-lib-hook",
              "spec": "^1.0.7",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-hook",
              "type": "prod",
            },
            "istanbul-lib-instrument" => EdgeOut {
              "name": "istanbul-lib-instrument",
              "spec": "^1.7.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-instrument",
              "type": "prod",
            },
            "istanbul-lib-report" => EdgeOut {
              "name": "istanbul-lib-report",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-report",
              "type": "prod",
            },
            "istanbul-lib-source-maps" => EdgeOut {
              "name": "istanbul-lib-source-maps",
              "spec": "^1.2.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-lib-source-maps",
              "type": "prod",
            },
            "istanbul-reports" => EdgeOut {
              "name": "istanbul-reports",
              "spec": "^1.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/istanbul-reports",
              "type": "prod",
            },
            "md5-hex" => EdgeOut {
              "name": "md5-hex",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/md5-hex",
              "type": "prod",
            },
            "merge-source-map" => EdgeOut {
              "name": "merge-source-map",
              "spec": "^1.0.2",
              "to": "node_modules/tap/node_modules/nyc/node_modules/merge-source-map",
              "type": "prod",
            },
            "micromatch" => EdgeOut {
              "name": "micromatch",
              "spec": "^2.3.11",
              "to": "node_modules/tap/node_modules/nyc/node_modules/micromatch",
              "type": "prod",
            },
            "mkdirp" => EdgeOut {
              "name": "mkdirp",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/mkdirp",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/resolve-from",
              "type": "prod",
            },
            "rimraf" => EdgeOut {
              "name": "rimraf",
              "spec": "^2.5.4",
              "to": "node_modules/tap/node_modules/nyc/node_modules/rimraf",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/signal-exit",
              "type": "prod",
            },
            "spawn-wrap" => EdgeOut {
              "name": "spawn-wrap",
              "spec": "^1.3.6",
              "to": "node_modules/tap/node_modules/nyc/node_modules/spawn-wrap",
              "type": "prod",
            },
            "test-exclude" => EdgeOut {
              "name": "test-exclude",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/test-exclude",
              "type": "prod",
            },
            "yargs" => EdgeOut {
              "name": "yargs",
              "spec": "^8.0.1",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs",
              "type": "prod",
            },
            "yargs-parser" => EdgeOut {
              "name": "yargs-parser",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/nyc/node_modules/yargs-parser",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/nyc",
          "name": "nyc",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/nyc",
          "resolved": "https://registry.npmjs.org/nyc/-/nyc-11.0.2.tgz",
          "version": "11.0.2",
        },
        "process-nextick-args" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "process-nextick-args",
              "spec": "~1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/process-nextick-args",
          "name": "process-nextick-args",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/process-nextick-args",
          "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-1.0.7.tgz",
          "version": "1.0.7",
        },
        "signal-exit" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/signal-exit",
          "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
          "version": "3.0.2",
        },
        "string_decoder" => ArboristNode {
          "children": Map {
            "safe-buffer" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string_decoder",
                  "name": "safe-buffer",
                  "spec": "~5.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "name": "safe-buffer",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.0.1.tgz",
              "version": "5.0.1",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "string_decoder",
              "spec": "~1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.0.1",
              "to": "node_modules/tap/node_modules/string_decoder/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string_decoder",
          "name": "string_decoder",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/string_decoder",
          "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.0.2.tgz",
          "version": "1.0.2",
        },
        "tap-mocha-reporter" => ArboristNode {
          "children": Map {
            "readable-stream" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/tap-mocha-reporter",
                  "name": "readable-stream",
                  "spec": "^2.1.5",
                  "type": "optional",
                },
              },
              "edgesOut": Map {
                "core-util-is" => EdgeOut {
                  "name": "core-util-is",
                  "spec": "~1.0.0",
                  "to": "node_modules/core-util-is",
                  "type": "prod",
                },
                "inherits" => EdgeOut {
                  "name": "inherits",
                  "spec": "~2.0.3",
                  "to": "node_modules/tap/node_modules/inherits",
                  "type": "prod",
                },
                "isarray" => EdgeOut {
                  "name": "isarray",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/isarray",
                  "type": "prod",
                },
                "process-nextick-args" => EdgeOut {
                  "name": "process-nextick-args",
                  "spec": "~1.0.6",
                  "to": "node_modules/tap/node_modules/process-nextick-args",
                  "type": "prod",
                },
                "safe-buffer" => EdgeOut {
                  "name": "safe-buffer",
                  "spec": "~5.1.0",
                  "to": "node_modules/safe-buffer",
                  "type": "prod",
                },
                "string_decoder" => EdgeOut {
                  "name": "string_decoder",
                  "spec": "~1.0.0",
                  "to": "node_modules/tap/node_modules/string_decoder",
                  "type": "prod",
                },
                "util-deprecate" => EdgeOut {
                  "name": "util-deprecate",
                  "spec": "~1.0.1",
                  "to": "node_modules/util-deprecate",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "name": "readable-stream",
              "optional": true,
              "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.2.tgz",
              "version": "2.3.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-mocha-reporter",
              "spec": "^3.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-support" => EdgeOut {
              "name": "color-support",
              "spec": "^1.1.0",
              "to": "node_modules/color-support",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^2.1.3",
              "to": "node_modules/debug",
              "type": "prod",
            },
            "diff" => EdgeOut {
              "name": "diff",
              "spec": "^1.3.2",
              "to": "node_modules/diff",
              "type": "prod",
            },
            "escape-string-regexp" => EdgeOut {
              "name": "escape-string-regexp",
              "spec": "^1.0.3",
              "to": "node_modules/escape-string-regexp",
              "type": "prod",
            },
            "glob" => EdgeOut {
              "name": "glob",
              "spec": "^7.0.5",
              "to": "node_modules/glob",
              "type": "prod",
            },
            "js-yaml" => EdgeOut {
              "name": "js-yaml",
              "spec": "^3.3.1",
              "to": "node_modules/js-yaml",
              "type": "prod",
            },
            "readable-stream" => EdgeOut {
              "name": "readable-stream",
              "spec": "^2.1.5",
              "to": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
              "type": "optional",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^5.1.0",
              "to": "node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^1.0.0",
              "to": "node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-mocha-reporter",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/tap-mocha-reporter",
          "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-3.0.5.tgz",
          "version": "3.0.5",
        },
        "tmatch" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tmatch",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tmatch",
          "name": "tmatch",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap/node_modules/tmatch",
          "resolved": "https://registry.npmjs.org/tmatch/-/tmatch-3.1.0.tgz",
          "version": "3.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "^10.5.1",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^1.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "bluebird" => EdgeOut {
          "name": "bluebird",
          "spec": "^3.3.1",
          "to": "node_modules/bluebird",
          "type": "prod",
        },
        "clean-yaml-object" => EdgeOut {
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "to": "node_modules/clean-yaml-object",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^2.11.2",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.1",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.0",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^1.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.3.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^11.0.2-candidate.0",
          "to": "node_modules/tap/node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.4.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.0",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.0.2",
          "to": "node_modules/readable-stream",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.4.3",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.0",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^3.0.5",
          "to": "node_modules/tap/node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^5.3.1",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tmatch" => EdgeOut {
          "name": "tmatch",
          "spec": "^3.1.0",
          "to": "node_modules/tap/node_modules/tmatch",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "tsame" => EdgeOut {
          "name": "tsame",
          "spec": "^1.1.2",
          "to": "node_modules/tsame",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-10.5.1.tgz",
      "version": "10.5.1",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^5.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.2.7",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-5.3.2.tgz",
      "version": "5.3.2",
    },
    "text-table" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "text-table",
          "spec": "~0.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/text-table",
      "name": "text-table",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/text-table",
      "resolved": "https://registry.npmjs.org/text-table/-/text-table-0.2.0.tgz",
      "version": "0.2.0",
    },
    "through" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inquirer",
          "name": "through",
          "spec": "^2.3.6",
          "type": "prod",
        },
      },
      "location": "node_modules/through",
      "name": "through",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/through",
      "resolved": "https://registry.npmjs.org/through/-/through-2.3.8.tgz",
      "version": "2.3.8",
    },
    "tough-cookie" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.3.2.tgz",
      "version": "2.3.2",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "tryit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-resolvable",
          "name": "tryit",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tryit",
      "name": "tryit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tryit",
      "resolved": "https://registry.npmjs.org/tryit/-/tryit-1.0.3.tgz",
      "version": "1.0.3",
    },
    "tsame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tsame",
          "spec": "^1.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/tsame",
      "name": "tsame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tsame",
      "resolved": "https://registry.npmjs.org/tsame/-/tsame-1.1.2.tgz",
      "version": "1.1.2",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "~0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.4.3.tgz",
      "version": "0.4.3",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "optional",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream",
          "name": "typedarray",
          "spec": "^0.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray",
      "name": "typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/typedarray",
      "resolved": "https://registry.npmjs.org/typedarray/-/typedarray-0.0.6.tgz",
      "version": "0.0.6",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uniq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/deglob",
          "name": "uniq",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/uniq",
      "name": "uniq",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/uniq",
      "resolved": "https://registry.npmjs.org/uniq/-/uniq-1.0.1.tgz",
      "version": "1.0.1",
    },
    "user-home" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint",
          "name": "user-home",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.0",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
      },
      "location": "node_modules/user-home",
      "name": "user-home",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/user-home",
      "resolved": "https://registry.npmjs.org/user-home/-/user-home-2.0.0.tgz",
      "version": "2.0.0",
    },
    "util-deprecate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/concat-stream/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap/node_modules/tap-mocha-reporter/node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.1.tgz",
      "version": "1.0.1",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.0.1.tgz",
      "version": "3.0.1",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.3.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.0.2",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.3.6.tgz",
      "version": "1.3.6",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/eslint/node_modules/optionator",
          "name": "wordwrap",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-1.0.0.tgz",
      "version": "1.0.0",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.1.tgz",
      "version": "1.0.1",
    },
    "write" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flat-cache",
          "name": "write",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/write",
      "name": "write",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/write",
      "resolved": "https://registry.npmjs.org/write/-/write-0.2.1.tgz",
      "version": "0.2.1",
    },
    "xtend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator/node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/is-my-json-valid",
          "name": "xtend",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/xtend",
      "name": "xtend",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/xtend",
      "resolved": "https://registry.npmjs.org/xtend/-/xtend-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.0.0.tgz",
      "version": "2.0.0",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "standard" => EdgeOut {
      "name": "standard",
      "spec": "^8.6.0",
      "to": "node_modules/standard",
      "type": "dev",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "^10.5.1",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient",
  "packageName": "sax",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reify-properly-with-all-deps-when-lockfile-is-ancient",
  "version": "1.2.4",
}
`

exports[`test/arborist/reify.js TAP reifying with shronk warp dep shrinkwrapped-dep-no-lock > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP reifying with shronk warp dep shrinkwrapped-dep-no-lock-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock-empty",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-no-lock-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP reifying with shronk warp dep shrinkwrapped-dep-with-lock > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP reifying with shronk warp dep shrinkwrapped-dep-with-lock-empty > must match snapshot 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/shrinkwrapped-dependency" => ArboristNode {
      "children": Map {
        "abbrev" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/shrinkwrapped-dependency",
              "name": "abbrev",
              "spec": "^1.0.4",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "name": "abbrev",
          "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
          "version": "1.0.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/shrinkwrapped-dependency",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "abbrev" => EdgeOut {
          "name": "abbrev",
          "spec": "^1.0.4",
          "to": "node_modules/@isaacs/shrinkwrapped-dependency/node_modules/abbrev",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@isaacs/shrinkwrapped-dependency",
      "name": "@isaacs/shrinkwrapped-dependency",
      "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock-empty/node_modules/@isaacs/shrinkwrapped-dependency",
      "resolved": "https://registry.npmjs.org/@isaacs/shrinkwrapped-dependency/-/shrinkwrapped-dependency-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/shrinkwrapped-dependency" => EdgeOut {
      "name": "@isaacs/shrinkwrapped-dependency",
      "spec": "^1.0.0",
      "to": "node_modules/@isaacs/shrinkwrapped-dependency",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock-empty",
  "packageName": "shrinkwrap",
  "path": "{CWD}/test/arborist/tap-testdir-reify-reifying-with-shronk-warp-dep-shrinkwrapped-dep-with-lock-empty",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP rollbacks fail removing retired and deleted nodes > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps-parent" => ArboristNode {
      "children": Map {
        "@isaacs/testing-bundledeps" => ArboristNode {
          "bundleDependencies": Array [
            "@isaacs/testing-bundledeps-a",
          ],
          "children": Map {
            "@isaacs/testing-bundledeps-a" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-a",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-a",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-b" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "name": "@isaacs/testing-bundledeps-b",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-c" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-c",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "name": "@isaacs/testing-bundledeps-c",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
              "version": "2.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps-parent",
              "name": "@isaacs/testing-bundledeps",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-a" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "type": "prod",
            },
            "@isaacs/testing-bundledeps-c" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-c",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps",
          "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-parent",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps" => EdgeOut {
          "name": "@isaacs/testing-bundledeps",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-parent",
      "name": "@isaacs/testing-bundledeps-parent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes/node_modules/@isaacs/testing-bundledeps-parent",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-parent/-/testing-bundledeps-parent-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-parent" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-parent",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-parent",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes",
  "packageName": "testing-bundledeps-3",
  "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-removing-retired-and-deleted-nodes",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP rollbacks fail retiring node, but then rimraf fixes it > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps-parent" => ArboristNode {
      "children": Map {
        "@isaacs/testing-bundledeps" => ArboristNode {
          "bundleDependencies": Array [
            "@isaacs/testing-bundledeps-a",
          ],
          "children": Map {
            "@isaacs/testing-bundledeps-a" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-a",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-a",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-b" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "name": "@isaacs/testing-bundledeps-b",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-c" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-c",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "name": "@isaacs/testing-bundledeps-c",
              "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
              "version": "2.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps-parent",
              "name": "@isaacs/testing-bundledeps",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-a" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "type": "prod",
            },
            "@isaacs/testing-bundledeps-c" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-c",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps",
          "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-parent",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps" => EdgeOut {
          "name": "@isaacs/testing-bundledeps",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-parent",
      "name": "@isaacs/testing-bundledeps-parent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it/node_modules/@isaacs/testing-bundledeps-parent",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-parent/-/testing-bundledeps-parent-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-parent" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-parent",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-parent",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it",
  "packageName": "testing-bundledeps-3",
  "path": "{CWD}/test/arborist/tap-testdir-reify-rollbacks-fail-retiring-node-but-then-rimraf-fixes-it",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP running lifecycle scripts of unchanged link nodes on reify > result 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:./a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-running-lifecycle-scripts-of-unchanged-link-nodes-on-reify/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-running-lifecycle-scripts-of-unchanged-link-nodes-on-reify/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:./a",
      "to": "node_modules/a",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-running-lifecycle-scripts-of-unchanged-link-nodes-on-reify/a",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-running-lifecycle-scripts-of-unchanged-link-nodes-on-reify",
  "packageName": "link-dep-lifecycle-scripts",
  "path": "{CWD}/test/arborist/tap-testdir-reify-running-lifecycle-scripts-of-unchanged-link-nodes-on-reify",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP save complete lockfile on update-all > should have abbrev 1.0.4 1`] = `
{
  "name": "save-package-lock-after-update-test",
  "version": "1.0.0",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "name": "save-package-lock-after-update-test",
      "version": "1.0.0",
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "node_modules/abbrev": {
      "version": "1.0.4",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
      "integrity": "sha1-vVWuXkE7oXIu5Mq6H26hBBSlns0="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save complete lockfile on update-all > should update, but not drop root metadata 1`] = `
{
  "name": "save-package-lock-after-update-test",
  "version": "1.0.0",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "name": "save-package-lock-after-update-test",
      "version": "1.0.0",
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save package.json on update should not save any with save=false and package-lock=false > should update lockfile with many deps updated package.json save=false 1`] = `
{
  "name": "workspaces-need-update",
  "lockfileVersion": 2,
  "requires": true,
  "packages": {
    "": {
      "workspaces": [
        "a",
        "b"
      ],
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "a": {
      "dependencies": {
        "abbrev": "^1.0.4",
        "once": "^1.3.2"
      }
    },
    "b": {
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "node_modules/a": {
      "resolved": "a",
      "link": true
    },
    "node_modules/abbrev": {
      "version": "1.0.4",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
      "integrity": "sha1-vVWuXkE7oXIu5Mq6H26hBBSlns0="
    },
    "node_modules/b": {
      "resolved": "b",
      "link": true
    },
    "node_modules/once": {
      "version": "1.3.2",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "integrity": "sha1-2P7sqTsDnsHc3ud0HJK9rF4oCBs=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  }
}
`

exports[`test/arborist/reify.js TAP save package.json on update should not save many deps in multiple package.json when using save=false > should update lockfile with many deps updated package.json save=false 1`] = `
{
  "name": "tap-testdir-reify-save-package.json-on-update-should-not-save-many-deps-in-multiple-package.json-when-using-save-false",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "workspaces": [
        "a",
        "b"
      ],
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "a": {
      "dependencies": {
        "abbrev": "^1.0.4",
        "once": "^1.3.2"
      }
    },
    "b": {
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "node_modules/a": {
      "resolved": "a",
      "link": true
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    },
    "node_modules/b": {
      "resolved": "b",
      "link": true
    },
    "node_modules/once": {
      "version": "1.4.0",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save package.json on update should save many deps in multiple package.json when using save=true > should update lockfile with many deps updated package.json save=true 1`] = `
{
  "name": "tap-testdir-reify-save-package.json-on-update-should-save-many-deps-in-multiple-package.json-when-using-save-true",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "workspaces": [
        "a",
        "b"
      ],
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "a": {
      "dependencies": {
        "abbrev": "^1.1.1",
        "once": "^1.4.0"
      }
    },
    "b": {
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "node_modules/a": {
      "resolved": "a",
      "link": true
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    },
    "node_modules/b": {
      "resolved": "b",
      "link": true
    },
    "node_modules/once": {
      "version": "1.4.0",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save package.json on update should update named dep across multiple package.json using save=true > should update lockfile with many deps updated package.json save=true 1`] = `
{
  "name": "tap-testdir-reify-save-package.json-on-update-should-update-named-dep-across-multiple-package.json-using-save-true",
  "lockfileVersion": 2,
  "requires": true,
  "packages": {
    "": {
      "workspaces": [
        "a",
        "b"
      ],
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "a": {
      "dependencies": {
        "abbrev": "^1.1.1",
        "once": "^1.3.2"
      }
    },
    "b": {
      "dependencies": {
        "abbrev": "^1.1.1"
      }
    },
    "node_modules/a": {
      "resolved": "a",
      "link": true
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    },
    "node_modules/b": {
      "resolved": "b",
      "link": true
    },
    "node_modules/once": {
      "version": "1.3.2",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "integrity": "sha1-2P7sqTsDnsHc3ud0HJK9rF4oCBs=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  },
  "dependencies": {
    "a": {
      "version": "file:a",
      "requires": {
        "abbrev": "^1.0.4",
        "once": "^1.3.2"
      }
    },
    "abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    },
    "b": {
      "version": "file:b",
      "requires": {
        "abbrev": "^1.0.4"
      }
    },
    "once": {
      "version": "1.3.2",
      "resolved": "https://registry.npmjs.org/once/-/once-1.3.2.tgz",
      "integrity": "sha1-2P7sqTsDnsHc3ud0HJK9rF4oCBs=",
      "requires": {
        "wrappy": "1"
      }
    },
    "wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save package.json on update should update single named dep across multiple package.json using save=true > should update lockfile with single dep updated package.json save=true 1`] = `
{
  "name": "tap-testdir-reify-save-package.json-on-update-should-update-single-named-dep-across-multiple-package.json-using-save-true",
  "lockfileVersion": 2,
  "requires": true,
  "packages": {
    "": {
      "workspaces": [
        "a",
        "b"
      ],
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "a": {
      "dependencies": {
        "abbrev": "^1.0.4",
        "once": "^1.4.0"
      }
    },
    "b": {
      "dependencies": {
        "abbrev": "^1.0.4"
      }
    },
    "node_modules/a": {
      "resolved": "a",
      "link": true
    },
    "node_modules/abbrev": {
      "version": "1.0.4",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
      "integrity": "sha1-vVWuXkE7oXIu5Mq6H26hBBSlns0="
    },
    "node_modules/b": {
      "resolved": "b",
      "link": true
    },
    "node_modules/once": {
      "version": "1.4.0",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
      "dependencies": {
        "wrappy": "1"
      }
    },
    "node_modules/wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  },
  "dependencies": {
    "a": {
      "version": "file:a",
      "requires": {
        "abbrev": "^1.0.4",
        "once": "^1.3.2"
      }
    },
    "abbrev": {
      "version": "1.0.4",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.0.4.tgz",
      "integrity": "sha1-vVWuXkE7oXIu5Mq6H26hBBSlns0="
    },
    "b": {
      "version": "file:b",
      "requires": {
        "abbrev": "^1.0.4"
      }
    },
    "once": {
      "version": "1.4.0",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "integrity": "sha1-WDsap3WWHUsROsF9nFC6753Xa9E=",
      "requires": {
        "wrappy": "1"
      }
    },
    "wrappy": {
      "version": "1.0.2",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "integrity": "sha1-tSQ9jz7BqjXxNkYFvA0QNuMKtp8="
    }
  }
}

`

exports[`test/arborist/reify.js TAP save proper lockfile with bins when upgrading lockfile complete=false > should upgrade, with bins in place 1`] = `
{
  "name": "tap-testdir-reify-save-proper-lockfile-with-bins-when-upgrading-lockfile-complete-false",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "semver": "^7.3.2"
      }
    },
    "node_modules/semver": {
      "version": "7.3.2",
      "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
      "integrity": "sha512-OrOb32TeeambH6UrhtShmF7CRDqhL6/5XpPNp2DuRH6+9QLw/orhp72j87v8Qa1ScDkvrrBNpZcDejAirJmfXQ==",
      "bin": {
        "semver": "bin/semver.js"
      },
      "engines": {
        "node": ">=10"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP save proper lockfile with bins when upgrading lockfile complete=true > should upgrade, with bins in place 1`] = `
{
  "name": "tap-testdir-reify-save-proper-lockfile-with-bins-when-upgrading-lockfile-complete-true",
  "lockfileVersion": 3,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "semver": "^7.3.2"
      }
    },
    "node_modules/semver": {
      "version": "7.3.2",
      "resolved": "https://registry.npmjs.org/semver/-/semver-7.3.2.tgz",
      "integrity": "sha512-OrOb32TeeambH6UrhtShmF7CRDqhL6/5XpPNp2DuRH6+9QLw/orhp72j87v8Qa1ScDkvrrBNpZcDejAirJmfXQ==",
      "bin": {
        "semver": "bin/semver.js"
      },
      "engines": {
        "node": ">=10"
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP save-prod, with optional > must match snapshot 1`] = `
{"dependencies":{"abbrev":"^1.1.1"}}
`

exports[`test/arborist/reify.js TAP saveBundle > must match snapshot 1`] = `
{"dependencies":{"abbrev":"^1.1.1"},"bundleDependencies":["abbrev"]}
`

exports[`test/arborist/reify.js TAP saving the ideal tree save some stuff > lock after save 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "tap-testdir-reify-saving-the-ideal-tree-save-some-stuff",
  "packages": Object {
    "": Object {
      "bundleDependencies": Array [
        "a",
        "b",
        "c",
      ],
      "dependencies": Object {
        "a": "github:foo/bar#baz",
        "b": "^1.2.3",
        "d": "npm:c@1.x <1.9.9",
        "e": "*",
        "f": "git+https://user:pass@github.com/baz/quux.git#asdf",
        "g": "*",
        "h": "~1.2.3",
      },
      "devDependencies": Object {
        "c": "git+ssh://git@githost.com:a/b/c.git#master",
      },
      "workspaces": Array [
        "e",
      ],
    },
    "e": Object {
      "extraneous": true,
    },
    "node_modules/a": Object {
      "extraneous": true,
      "inBundle": true,
      "resolved": "git+ssh://git@github.com/foo/bar.git#71f3ccfefba85d2048484569dba8c1829f6f41d7",
    },
    "node_modules/b": Object {
      "extraneous": true,
      "inBundle": true,
      "resolved": "https://registry.npmjs.org/b/-/b-1.2.3.tgz",
      "version": "1.2.3",
    },
    "node_modules/c": Object {
      "extraneous": true,
      "inBundle": true,
      "resolved": "git+ssh://git@githost.com:a/b/c.git#71f3ccfefba85d2048484569dba8c1829f6f41d7",
    },
    "node_modules/d": Object {
      "extraneous": true,
      "name": "c",
      "resolved": "https://registry.npmjs.org/c/-/c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "node_modules/e": Object {
      "link": true,
      "resolved": "e",
    },
    "node_modules/f": Object {
      "extraneous": true,
      "resolved": "git+https://user:pass@github.com/baz/quux.git#71f3ccfefba85d2048484569dba8c1829f6f41d7",
      "version": "1.2.3",
    },
    "node_modules/g": Object {
      "extraneous": true,
      "resolved": "https://registry.npmjs.org/g/-/g-1.2.3.tgz",
    },
    "node_modules/h": Object {
      "extraneous": true,
      "resolved": "https://registry.npmjs.org/h/-/h-1.2.3.tgz",
      "version": "1.2.3",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP scoped registries > should preserve original resolved value 1`] = `
@ruyadorno/theoretically-private-pkg@https://npm.pkg.github.com/@ruyadorno/theoretically-private-pkg/-/theoretically-private-pkg-1.2.3.tgz
`

exports[`test/arborist/reify.js TAP still do not install optional deps with mismatched platform specifications even when forced > expect resolving Promise 1`] = `
ArboristNode {
  "edgesOut": Map {
    "platform-specifying-test-package" => EdgeOut {
      "name": "platform-specifying-test-package",
      "spec": "1.0.0",
      "to": null,
      "type": "optional",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-still-do-not-install-optional-deps-with-mismatched-platform-specifications-even-when-forced",
  "packageName": "platform-test",
  "path": "{CWD}/test/arborist/tap-testdir-reify-still-do-not-install-optional-deps-with-mismatched-platform-specifications-even-when-forced",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP store files with a custom indenting > must match snapshot 1`] = `
{
	"name": "tab-indented-package-json",
	"version": "1.0.0",
	"dependencies": {
		"abbrev": "^1.0.0"
	}
}

`

exports[`test/arborist/reify.js TAP store files with a custom indenting > must match snapshot 2`] = `
{
	"name": "tab-indented-package-json",
	"version": "1.0.0",
	"lockfileVersion": 3,
	"requires": true,
	"packages": {
		"": {
			"name": "tab-indented-package-json",
			"version": "1.0.0",
			"dependencies": {
				"abbrev": "^1.0.0"
			}
		},
		"node_modules/abbrev": {
			"version": "1.1.1",
			"resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
			"integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
		}
	}
}

`

exports[`test/arborist/reify.js TAP tarball deps with transitive tarball deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-file-transitive-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-file-transitive-a",
          "spec": "file:testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-file-transitive-b" => EdgeOut {
          "name": "@isaacs/testing-file-transitive-b",
          "spec": "file:../testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
          "to": "node_modules/@isaacs/testing-file-transitive-b",
          "type": "prod",
        },
        "@isaacs/testing-file-transitive-c" => EdgeOut {
          "name": "@isaacs/testing-file-transitive-c",
          "spec": "file:./c/isaacs-testing-file-transitive-c-1.0.0.tgz",
          "to": "node_modules/@isaacs/testing-file-transitive-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-a",
      "name": "@isaacs/testing-file-transitive-a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/node_modules/@isaacs/testing-file-transitive-a",
      "resolved": "file:{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-file-transitive-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-file-transitive-a",
          "name": "@isaacs/testing-file-transitive-b",
          "spec": "file:../testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-b",
      "name": "@isaacs/testing-file-transitive-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/node_modules/@isaacs/testing-file-transitive-b",
      "resolved": "file:{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/testing-file-transitive-b/isaacs-testing-file-transitive-b-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@isaacs/testing-file-transitive-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-file-transitive-a",
          "name": "@isaacs/testing-file-transitive-c",
          "spec": "file:./c/isaacs-testing-file-transitive-c-1.0.0.tgz",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-file-transitive-c",
      "name": "@isaacs/testing-file-transitive-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/node_modules/@isaacs/testing-file-transitive-c",
      "resolved": "file:{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps/testing-file-transitive-a/c/isaacs-testing-file-transitive-c-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-file-transitive-a" => EdgeOut {
      "name": "@isaacs/testing-file-transitive-a",
      "spec": "file:testing-file-transitive-a/isaacs-testing-file-transitive-a-1.0.0.tgz",
      "to": "node_modules/@isaacs/testing-file-transitive-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-tarball-deps-with-transitive-tarball-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-tarball-deps-with-transitive-tarball-deps",
}
`

exports[`test/arborist/reify.js TAP testing-peer-deps nested > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
        "@isaacs/testing-peer-deps-d" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps",
      "name": "@isaacs/testing-peer-deps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps/-/testing-peer-deps-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-deps-a" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-b" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-b",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-a",
      "name": "@isaacs/testing-peer-deps-a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-a",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-b",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps" => EdgeOut {
      "name": "@isaacs/testing-peer-deps",
      "spec": "2 || 3",
      "to": "node_modules/@isaacs/testing-peer-deps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-testing-peer-deps-nested",
  "packageName": "testing-peer-deps-nested",
  "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP testing-peer-deps nested with update > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps",
          "spec": "2 || 3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-b" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-b",
          "type": "prod",
        },
        "@isaacs/testing-peer-deps-d" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps",
      "name": "@isaacs/testing-peer-deps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update/node_modules/@isaacs/testing-peer-deps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps/-/testing-peer-deps-3.0.0.tgz",
      "version": "3.0.0",
    },
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "2",
          "type": "peer",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
      "version": "2.0.1",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "2",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
      "version": "2.0.0",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps" => EdgeOut {
      "name": "@isaacs/testing-peer-deps",
      "spec": "2 || 3",
      "to": "node_modules/@isaacs/testing-peer-deps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-testing-peer-deps-nested-with-update",
  "packageName": "testing-peer-deps-nested",
  "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-nested-with-update",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP testing-peer-deps package > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-peer-deps-b" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-b",
          "spec": "1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-c" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "to": "node_modules/@isaacs/testing-peer-deps-c",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-b",
      "name": "@isaacs/testing-peer-deps-b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-b",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-1.2.4.tgz",
      "version": "1.2.4",
    },
    "@isaacs/testing-peer-deps-c" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-c",
          "spec": "1",
          "type": "peer",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-c",
      "name": "@isaacs/testing-peer-deps-c",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-c",
      "peer": true,
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-1.2.3.tgz",
      "version": "1.2.3",
    },
    "@isaacs/testing-peer-deps-d" => ArboristNode {
      "children": Map {
        "@isaacs/testing-peer-deps-a" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d",
              "name": "@isaacs/testing-peer-deps-a",
              "spec": "2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-b" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "name": "@isaacs/testing-peer-deps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-a/-/testing-peer-deps-a-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@isaacs/testing-peer-deps-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
              "name": "@isaacs/testing-peer-deps-b",
              "spec": "2",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-peer-deps-c" => EdgeOut {
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "name": "@isaacs/testing-peer-deps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-b/-/testing-peer-deps-b-2.0.1.tgz",
          "version": "2.0.1",
        },
        "@isaacs/testing-peer-deps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-b",
              "name": "@isaacs/testing-peer-deps-c",
              "spec": "2",
              "type": "peer",
            },
          },
          "location": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "name": "@isaacs/testing-peer-deps-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-c",
          "peer": true,
          "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-c/-/testing-peer-deps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-peer-deps-d",
          "spec": "2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-peer-deps-a" => EdgeOut {
          "name": "@isaacs/testing-peer-deps-a",
          "spec": "2",
          "to": "node_modules/@isaacs/testing-peer-deps-d/node_modules/@isaacs/testing-peer-deps-a",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-peer-deps-d",
      "name": "@isaacs/testing-peer-deps-d",
      "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package/node_modules/@isaacs/testing-peer-deps-d",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-peer-deps-d/-/testing-peer-deps-d-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-peer-deps-b" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-b",
      "spec": "1",
      "to": "node_modules/@isaacs/testing-peer-deps-b",
      "type": "prod",
    },
    "@isaacs/testing-peer-deps-d" => EdgeOut {
      "name": "@isaacs/testing-peer-deps-d",
      "spec": "2",
      "to": "node_modules/@isaacs/testing-peer-deps-d",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-testing-peer-deps-package",
  "packageName": "@isaacs/testing-peer-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-testing-peer-deps-package",
  "version": "2.0.0",
}
`

exports[`test/arborist/reify.js TAP tracks changes of shrinkwrapped dep correctly > install added the correct tree 1`] = `
ArboristNode {
  "children": Map {
    "@nlf/shrinkwrapped-dep-updates-a" => ArboristNode {
      "children": Map {
        "@nlf/shrinkwrapped-dep-updates-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-b",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "name": "@nlf/shrinkwrapped-dep-updates-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-b/-/shrinkwrapped-dep-updates-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@nlf/shrinkwrapped-dep-updates-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-c",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "name": "@nlf/shrinkwrapped-dep-updates-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-c/-/shrinkwrapped-dep-updates-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@nlf/shrinkwrapped-dep-updates-d" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-d",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-d",
          "name": "@nlf/shrinkwrapped-dep-updates-d",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-d",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-d/-/shrinkwrapped-dep-updates-d-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@nlf/shrinkwrapped-dep-updates-a",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nlf/shrinkwrapped-dep-updates-b" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-b",
          "spec": "^1.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "type": "prod",
        },
        "@nlf/shrinkwrapped-dep-updates-c" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-c",
          "spec": "^1.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "type": "prod",
        },
        "@nlf/shrinkwrapped-dep-updates-d" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-d",
          "spec": "^1.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-d",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "name": "@nlf/shrinkwrapped-dep-updates-a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-a/-/shrinkwrapped-dep-updates-a-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@nlf/shrinkwrapped-dep-updates-a" => EdgeOut {
      "name": "@nlf/shrinkwrapped-dep-updates-a",
      "spec": "^1.0.0",
      "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly",
  "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly",
}
`

exports[`test/arborist/reify.js TAP tracks changes of shrinkwrapped dep correctly > installing new version brings in the correct children 1`] = `
ArboristNode {
  "children": Map {
    "@nlf/shrinkwrapped-dep-updates-a" => ArboristNode {
      "children": Map {
        "@nlf/shrinkwrapped-dep-updates-b" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-b",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "name": "@nlf/shrinkwrapped-dep-updates-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-b/-/shrinkwrapped-dep-updates-b-2.0.0.tgz",
          "version": "2.0.0",
        },
        "@nlf/shrinkwrapped-dep-updates-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-c",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "name": "@nlf/shrinkwrapped-dep-updates-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-c/-/shrinkwrapped-dep-updates-c-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@nlf/shrinkwrapped-dep-updates-e" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
              "name": "@nlf/shrinkwrapped-dep-updates-e",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "inShrinkwrap": true,
          "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-e",
          "name": "@nlf/shrinkwrapped-dep-updates-e",
          "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-e",
          "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-e/-/shrinkwrapped-dep-updates-e-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@nlf/shrinkwrapped-dep-updates-a",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@nlf/shrinkwrapped-dep-updates-b" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-b",
          "spec": "^2.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-b",
          "type": "prod",
        },
        "@nlf/shrinkwrapped-dep-updates-c" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-c",
          "spec": "^1.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-c",
          "type": "prod",
        },
        "@nlf/shrinkwrapped-dep-updates-e" => EdgeOut {
          "name": "@nlf/shrinkwrapped-dep-updates-e",
          "spec": "^1.0.0",
          "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a/node_modules/@nlf/shrinkwrapped-dep-updates-e",
          "type": "prod",
        },
      },
      "hasShrinkwrap": true,
      "location": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "name": "@nlf/shrinkwrapped-dep-updates-a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly/node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "resolved": "https://registry.npmjs.org/@nlf/shrinkwrapped-dep-updates-a/-/shrinkwrapped-dep-updates-a-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@nlf/shrinkwrapped-dep-updates-a" => EdgeOut {
      "name": "@nlf/shrinkwrapped-dep-updates-a",
      "spec": "^2.0.0",
      "to": "node_modules/@nlf/shrinkwrapped-dep-updates-a",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly",
  "path": "{CWD}/test/arborist/tap-testdir-reify-tracks-changes-of-shrinkwrapped-dep-correctly",
}
`

exports[`test/arborist/reify.js TAP transitive deps containing asymmetrical bin no lockfile > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@ruyadorno/package-with-added-bin" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@ruyadorno/package-with-added-bin",
      "name": "@ruyadorno/package-with-added-bin",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/node_modules/@ruyadorno/package-with-added-bin",
      "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-1.0.0.tgz",
      "version": "1.0.0",
    },
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:b",
          "type": "prod",
        },
      },
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:a",
      "to": "node_modules/a",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:b",
      "to": "node_modules/b",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "to": "node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "b",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "b/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/b/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^2.0.0",
          "to": "b/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile",
  "packageName": "test-bin-added-transitive-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-no-lockfile",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP transitive deps containing asymmetrical bin with lockfile > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:a",
          "type": "prod",
        },
      },
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:b",
          "type": "prod",
        },
      },
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:a",
      "to": "node_modules/a",
      "type": "prod",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:b",
      "to": "node_modules/b",
      "type": "prod",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "a",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "location": "a/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/a/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^1.0.0",
          "to": "a/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "children": Map {
        "@ruyadorno/package-with-added-bin" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "b",
              "name": "@ruyadorno/package-with-added-bin",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "b/node_modules/@ruyadorno/package-with-added-bin",
          "name": "@ruyadorno/package-with-added-bin",
          "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/b/node_modules/@ruyadorno/package-with-added-bin",
          "resolved": "https://registry.npmjs.org/@ruyadorno/package-with-added-bin/-/package-with-added-bin-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/package-with-added-bin" => EdgeOut {
          "name": "@ruyadorno/package-with-added-bin",
          "spec": "^2.0.0",
          "to": "b/node_modules/@ruyadorno/package-with-added-bin",
          "type": "prod",
        },
      },
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile",
  "packageName": "test-bin-added-transitive-deps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-transitive-deps-containing-asymmetrical-bin-with-lockfile",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP update a bundling node without updating all of its deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.5.5.tgz",
      "version": "7.5.5",
    },
    "@babel/generator" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/generator",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/generator/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/generator/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/generator/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/helper-function-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/highlight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.5.0.tgz",
      "version": "7.5.0",
    },
    "@babel/parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yaml",
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "regenerator-runtime" => EdgeOut {
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "to": "node_modules/regenerator-runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/runtime",
      "name": "@babel/runtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/runtime",
      "resolved": "https://registry.npmjs.org/@babel/runtime/-/runtime-7.7.7.tgz",
      "version": "7.7.7",
    },
    "@babel/template" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.0.0",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.5.5",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.7.4",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.7.4.tgz",
      "version": "7.7.4",
    },
    "@babel/types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.7.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "esutils" => EdgeOut {
          "name": "esutils",
          "spec": "^2.0.2",
          "to": "node_modules/esutils",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.7.4.tgz",
      "version": "7.7.4",
    },
    "ajv" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.10.2.tgz",
      "version": "6.10.2",
    },
    "ansi-styles" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "anymatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "anymatch",
          "spec": "~3.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "^3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.4",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/anymatch",
      "name": "anymatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/anymatch",
      "resolved": "https://registry.npmjs.org/anymatch/-/anymatch-3.1.1.tgz",
      "version": "3.1.1",
    },
    "append-transform" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.2.tgz",
      "version": "4.1.2",
    },
    "argparse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "asap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/promise",
          "name": "asap",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/asap",
      "name": "asap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/asap",
      "resolved": "https://registry.npmjs.org/asap/-/asap-2.0.6.tgz",
      "version": "2.0.6",
    },
    "asn1" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "async-hook-domain" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.11",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
      },
      "location": "node_modules/async-hook-domain",
      "name": "async-hook-domain",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/async-hook-domain",
      "resolved": "https://registry.npmjs.org/async-hook-domain/-/async-hook-domain-1.1.3.tgz",
      "version": "1.1.3",
    },
    "asynckit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.9.0.tgz",
      "version": "1.9.0",
    },
    "balanced-match" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "binary-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-binary-path",
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/binary-extensions",
      "name": "binary-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/binary-extensions",
      "resolved": "https://registry.npmjs.org/binary-extensions/-/binary-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "bind-obj-methods" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "braces" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "braces",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fill-range" => EdgeOut {
          "name": "fill-range",
          "spec": "^7.0.1",
          "to": "node_modules/fill-range",
          "type": "prod",
        },
      },
      "location": "node_modules/braces",
      "name": "braces",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/braces",
      "resolved": "https://registry.npmjs.org/braces/-/braces-3.0.2.tgz",
      "version": "3.0.2",
    },
    "browser-process-hrtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "children": Map {
        "write-file-atomic" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/caching-transform",
              "name": "write-file-atomic",
              "spec": "^2.4.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "graceful-fs" => EdgeOut {
              "name": "graceful-fs",
              "spec": "^4.1.11",
              "to": "node_modules/graceful-fs",
              "type": "prod",
            },
            "imurmurhash" => EdgeOut {
              "name": "imurmurhash",
              "spec": "^0.1.4",
              "to": "node_modules/imurmurhash",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "write-file-atomic",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/caching-transform/node_modules/write-file-atomic",
          "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
          "version": "2.4.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/caching-transform/node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "camelcase" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "caseless" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "chokidar" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "chokidar",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "anymatch" => EdgeOut {
          "name": "anymatch",
          "spec": "~3.1.1",
          "to": "node_modules/anymatch",
          "type": "prod",
        },
        "braces" => EdgeOut {
          "name": "braces",
          "spec": "~3.0.2",
          "to": "node_modules/braces",
          "type": "prod",
        },
        "fsevents" => EdgeOut {
          "name": "fsevents",
          "spec": "~2.1.2",
          "to": null,
          "type": "optional",
        },
        "glob-parent" => EdgeOut {
          "name": "glob-parent",
          "spec": "~5.1.0",
          "to": "node_modules/glob-parent",
          "type": "prod",
        },
        "is-binary-path" => EdgeOut {
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "to": "node_modules/is-binary-path",
          "type": "prod",
        },
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "~4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
        "normalize-path" => EdgeOut {
          "name": "normalize-path",
          "spec": "~3.0.0",
          "to": "node_modules/normalize-path",
          "type": "prod",
        },
        "readdirp" => EdgeOut {
          "name": "readdirp",
          "spec": "~3.3.0",
          "to": "node_modules/readdirp",
          "type": "prod",
        },
      },
      "location": "node_modules/chokidar",
      "name": "chokidar",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/chokidar",
      "resolved": "https://registry.npmjs.org/chokidar/-/chokidar-3.3.1.tgz",
      "version": "3.3.1",
    },
    "cliui" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-3.0.0.tgz",
          "version": "3.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "string-width",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/cliui/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "to": "node_modules/cliui/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-2.1.1.tgz",
          "version": "2.1.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/cliui/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^3.0.0",
              "to": "node_modules/cliui/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-4.0.0.tgz",
          "version": "4.0.0",
        },
        "wrap-ansi" => ArboristNode {
          "children": Map {
            "ansi-regex" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
              "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
              "version": "2.1.1",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "number-is-nan" => EdgeOut {
                  "name": "number-is-nan",
                  "spec": "^1.0.0",
                  "to": "node_modules/number-is-nan",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
              "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-1.0.0.tgz",
              "version": "1.0.0",
            },
            "string-width" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^1.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "code-point-at" => EdgeOut {
                  "name": "code-point-at",
                  "spec": "^1.0.0",
                  "to": "node_modules/code-point-at",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^1.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "resolved": "https://registry.npmjs.org/string-width/-/string-width-1.0.2.tgz",
              "version": "1.0.2",
            },
            "strip-ansi" => ArboristNode {
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
              "version": "3.0.1",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^1.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "to": "node_modules/cliui/node_modules/wrap-ansi/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/cliui/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-2.1.0.tgz",
          "version": "2.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jackspeak",
          "name": "cliui",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^2.1.1",
          "to": "node_modules/cliui/node_modules/string-width",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^4.0.0",
          "to": "node_modules/cliui/node_modules/strip-ansi",
          "type": "prod",
        },
        "wrap-ansi" => EdgeOut {
          "name": "wrap-ansi",
          "spec": "^2.0.0",
          "to": "node_modules/cliui/node_modules/wrap-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/cliui",
      "name": "cliui",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cliui",
      "resolved": "https://registry.npmjs.org/cliui/-/cliui-4.1.0.tgz",
      "version": "4.1.0",
    },
    "code-point-at" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/string-width",
          "name": "code-point-at",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/code-point-at",
      "name": "code-point-at",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/code-point-at",
      "resolved": "https://registry.npmjs.org/code-point-at/-/code-point-at-1.1.0.tgz",
      "version": "1.1.0",
    },
    "color-convert" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "color-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commondir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/convert-source-map",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/convert-source-map/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/convert-source-map/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/convert-source-map/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "core-js",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/core-js",
      "name": "core-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/core-js",
      "resolved": "https://registry.npmjs.org/core-js/-/core-js-1.2.7.tgz",
      "version": "1.2.7",
    },
    "core-util-is" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.0",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.9.tgz",
      "version": "3.0.9",
    },
    "cp-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "create-react-class" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "create-react-class",
          "spec": "^15.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fbjs" => EdgeOut {
          "name": "fbjs",
          "spec": "^0.8.9",
          "to": "node_modules/fbjs",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.3.1",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
      },
      "location": "node_modules/create-react-class",
      "name": "create-react-class",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/create-react-class",
      "resolved": "https://registry.npmjs.org/create-react-class/-/create-react-class-15.6.3.tgz",
      "version": "15.6.3",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
      "version": "4.1.1",
    },
    "decamelize" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.1.tgz",
      "version": "4.0.1",
    },
    "diff-frag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tcompare",
          "name": "diff-frag",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff-frag",
      "name": "diff-frag",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/diff-frag",
      "resolved": "https://registry.npmjs.org/diff-frag/-/diff-frag-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ecc-jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "encoding" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "encoding",
          "spec": "^0.1.11",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "iconv-lite" => EdgeOut {
          "name": "iconv-lite",
          "spec": "~0.4.13",
          "to": "node_modules/iconv-lite",
          "type": "prod",
        },
      },
      "location": "node_modules/encoding",
      "name": "encoding",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/encoding",
      "resolved": "https://registry.npmjs.org/encoding/-/encoding-0.1.12.tgz",
      "version": "0.1.12",
    },
    "error-ex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.25",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "esutils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "esutils",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/esutils",
      "name": "esutils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/esutils",
      "resolved": "https://registry.npmjs.org/esutils/-/esutils-2.0.3.tgz",
      "version": "2.0.3",
    },
    "events-to-array" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-2.0.1.tgz",
      "version": "2.0.1",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "fbjs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "fbjs",
          "spec": "^0.8.9",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "fbjs",
          "spec": "^0.8.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "core-js" => EdgeOut {
          "name": "core-js",
          "spec": "^1.0.0",
          "to": "node_modules/core-js",
          "type": "prod",
        },
        "isomorphic-fetch" => EdgeOut {
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "to": "node_modules/isomorphic-fetch",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.0.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "promise" => EdgeOut {
          "name": "promise",
          "spec": "^7.1.1",
          "to": "node_modules/promise",
          "type": "prod",
        },
        "setimmediate" => EdgeOut {
          "name": "setimmediate",
          "spec": "^1.0.5",
          "to": "node_modules/setimmediate",
          "type": "prod",
        },
        "ua-parser-js" => EdgeOut {
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "to": "node_modules/ua-parser-js",
          "type": "prod",
        },
      },
      "location": "node_modules/fbjs",
      "name": "fbjs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fbjs",
      "resolved": "https://registry.npmjs.org/fbjs/-/fbjs-0.8.17.tgz",
      "version": "0.8.17",
    },
    "fill-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/braces",
          "name": "fill-range",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "to-regex-range" => EdgeOut {
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "to": "node_modules/to-regex-range",
          "type": "prod",
        },
      },
      "location": "node_modules/fill-range",
      "name": "fill-range",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fill-range",
      "resolved": "https://registry.npmjs.org/fill-range/-/fill-range-7.0.1.tgz",
      "version": "7.0.1",
    },
    "find-cache-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "findit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "findit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/findit",
      "name": "findit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/findit",
      "resolved": "https://registry.npmjs.org/findit/-/findit-2.0.0.tgz",
      "version": "2.0.0",
    },
    "flow-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "flow-parser",
          "spec": "^0.114.0",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-parser",
      "name": "flow-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/flow-parser",
      "resolved": "https://registry.npmjs.org/flow-parser/-/flow-parser-0.114.0.tgz",
      "version": "0.114.0",
    },
    "flow-remove-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "flow-parser" => EdgeOut {
          "name": "flow-parser",
          "spec": "^0.114.0",
          "to": "node_modules/flow-parser",
          "type": "prod",
        },
        "pirates" => EdgeOut {
          "name": "pirates",
          "spec": "^3.0.2",
          "to": "node_modules/pirates",
          "type": "prod",
        },
        "vlq" => EdgeOut {
          "name": "vlq",
          "spec": "^0.2.1",
          "to": "node_modules/vlq",
          "type": "prod",
        },
      },
      "location": "node_modules/flow-remove-types",
      "name": "flow-remove-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/flow-remove-types",
      "resolved": "https://registry.npmjs.org/flow-remove-types/-/flow-remove-types-2.114.0.tgz",
      "version": "2.114.0",
    },
    "foreground-child" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "function-loop" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "get-caller-file" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "glob-parent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "glob-parent",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-glob" => EdgeOut {
          "name": "is-glob",
          "spec": "^4.0.1",
          "to": "node_modules/is-glob",
          "type": "prod",
        },
      },
      "location": "node_modules/glob-parent",
      "name": "glob-parent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/glob-parent",
      "resolved": "https://registry.npmjs.org/glob-parent/-/glob-parent-5.1.0.tgz",
      "version": "5.1.0",
    },
    "globals" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.3.tgz",
      "version": "4.2.3",
    },
    "handlebars" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "handlebars",
          "spec": "^4.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "neo-async" => EdgeOut {
          "name": "neo-async",
          "spec": "^2.6.0",
          "to": "node_modules/neo-async",
          "type": "prod",
        },
        "optimist" => EdgeOut {
          "name": "optimist",
          "spec": "^0.6.1",
          "to": "node_modules/optimist",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
        "uglify-js" => EdgeOut {
          "name": "uglify-js",
          "spec": "^3.1.4",
          "to": null,
          "type": "optional",
        },
      },
      "location": "node_modules/handlebars",
      "name": "handlebars",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/handlebars",
      "resolved": "https://registry.npmjs.org/handlebars/-/handlebars-4.5.3.tgz",
      "version": "4.5.3",
    },
    "har-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.5.5",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
      "version": "5.1.3",
    },
    "has-flag" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.5.tgz",
      "version": "2.8.5",
    },
    "http-signature" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "iconv-lite" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/encoding",
          "name": "iconv-lite",
          "spec": "~0.4.13",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/iconv-lite",
      "name": "iconv-lite",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/iconv-lite",
      "resolved": "https://registry.npmjs.org/iconv-lite/-/iconv-lite-0.4.24.tgz",
      "version": "0.4.24",
    },
    "imurmurhash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-arrayish" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-binary-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-binary-path",
          "spec": "~2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "binary-extensions" => EdgeOut {
          "name": "binary-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/binary-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/is-binary-path",
      "name": "is-binary-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-binary-path",
      "resolved": "https://registry.npmjs.org/is-binary-path/-/is-binary-path-2.1.0.tgz",
      "version": "2.1.0",
    },
    "is-extglob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/is-glob",
          "name": "is-extglob",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-extglob",
      "name": "is-extglob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-extglob",
      "resolved": "https://registry.npmjs.org/is-extglob/-/is-extglob-2.1.1.tgz",
      "version": "2.1.1",
    },
    "is-glob" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "is-glob",
          "spec": "~4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/glob-parent",
          "name": "is-glob",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-extglob" => EdgeOut {
          "name": "is-extglob",
          "spec": "^2.1.1",
          "to": "node_modules/is-extglob",
          "type": "prod",
        },
      },
      "location": "node_modules/is-glob",
      "name": "is-glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-glob",
      "resolved": "https://registry.npmjs.org/is-glob/-/is-glob-4.0.1.tgz",
      "version": "4.0.1",
    },
    "is-number" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/to-regex-range",
          "name": "is-number",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-number",
      "name": "is-number",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-number",
      "resolved": "https://registry.npmjs.org/is-number/-/is-number-7.0.0.tgz",
      "version": "7.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/node-fetch",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/typedarray-to-buffer",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isomorphic-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "isomorphic-fetch",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-fetch" => EdgeOut {
          "name": "node-fetch",
          "spec": "^1.0.1",
          "to": "node_modules/node-fetch",
          "type": "prod",
        },
        "whatwg-fetch" => EdgeOut {
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "to": "node_modules/whatwg-fetch",
          "type": "prod",
        },
      },
      "location": "node_modules/isomorphic-fetch",
      "name": "isomorphic-fetch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/isomorphic-fetch",
      "resolved": "https://registry.npmjs.org/isomorphic-fetch/-/isomorphic-fetch-2.2.1.tgz",
      "version": "2.2.1",
    },
    "isstream" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-processinfo" => ArboristNode {
      "children": Map {
        "cross-spawn" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo",
              "name": "cross-spawn",
              "spec": "^6.0.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "nice-try" => EdgeOut {
              "name": "nice-try",
              "spec": "^1.0.4",
              "to": "node_modules/nice-try",
              "type": "prod",
            },
            "path-key" => EdgeOut {
              "name": "path-key",
              "spec": "^2.0.1",
              "to": "node_modules/path-key",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.5.0",
              "to": "node_modules/semver",
              "type": "prod",
            },
            "shebang-command" => EdgeOut {
              "name": "shebang-command",
              "spec": "^1.2.0",
              "to": "node_modules/shebang-command",
              "type": "prod",
            },
            "which" => EdgeOut {
              "name": "which",
              "spec": "^1.2.9",
              "to": "node_modules/istanbul-lib-processinfo/node_modules/which",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "cross-spawn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-6.0.5.tgz",
          "version": "6.0.5",
        },
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-processinfo/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-processinfo/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^6.0.5",
          "to": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.3",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-processinfo",
      "name": "istanbul-lib-processinfo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-processinfo",
      "resolved": "https://registry.npmjs.org/istanbul-lib-processinfo/-/istanbul-lib-processinfo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "handlebars" => EdgeOut {
          "name": "handlebars",
          "spec": "^4.1.2",
          "to": "node_modules/handlebars",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.6.tgz",
      "version": "2.2.6",
    },
    "jackspeak" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "jackspeak",
          "spec": "^1.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^4.1.0",
          "to": "node_modules/cliui",
          "type": "prod",
        },
      },
      "location": "node_modules/jackspeak",
      "name": "jackspeak",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/jackspeak",
      "resolved": "https://registry.npmjs.org/jackspeak/-/jackspeak-1.4.0.tgz",
      "version": "1.4.0",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/loose-envify",
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
      "version": "3.13.1",
    },
    "jsbn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsprim" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.15.tgz",
      "version": "4.17.15",
    },
    "lodash.flattendeep" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "log-driver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "loose-envify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "loose-envify",
          "spec": "^1.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "loose-envify",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "loose-envify",
          "spec": "^1.4.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "loose-envify",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^3.0.0 || ^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/loose-envify",
      "name": "loose-envify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/loose-envify",
      "resolved": "https://registry.npmjs.org/loose-envify/-/loose-envify-1.4.0.tgz",
      "version": "1.4.0",
    },
    "lru-cache" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.5.tgz",
      "version": "1.3.5",
    },
    "merge-source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.42.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.42.0.tgz",
      "version": "1.42.0",
    },
    "mime-types" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.42.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.25.tgz",
      "version": "2.1.25",
    },
    "minimatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.0.tgz",
      "version": "1.2.0",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^4.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-3.1.1.tgz",
      "version": "3.1.1",
    },
    "mkdirp" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/mkdirp",
              "name": "minimist",
              "spec": "0.0.8",
              "type": "prod",
            },
          },
          "location": "node_modules/mkdirp/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/mkdirp/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.8.tgz",
          "version": "0.0.8",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "mkdirp",
          "spec": "^0.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "mkdirp",
          "spec": "^0.5.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "0.0.8",
          "to": "node_modules/mkdirp/node_modules/minimist",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.1.tgz",
      "version": "0.5.1",
    },
    "ms" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "neo-async" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "neo-async",
          "spec": "^2.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/neo-async",
      "name": "neo-async",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/neo-async",
      "resolved": "https://registry.npmjs.org/neo-async/-/neo-async-2.6.1.tgz",
      "version": "2.6.1",
    },
    "nested-error-stacks" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "nice-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "nice-try",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "location": "node_modules/nice-try",
      "name": "nice-try",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/nice-try",
      "resolved": "https://registry.npmjs.org/nice-try/-/nice-try-1.0.5.tgz",
      "version": "1.0.5",
    },
    "node-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "node-fetch",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "encoding" => EdgeOut {
          "name": "encoding",
          "spec": "^0.1.11",
          "to": "node_modules/encoding",
          "type": "prod",
        },
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/node-fetch",
      "name": "node-fetch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/node-fetch",
      "resolved": "https://registry.npmjs.org/node-fetch/-/node-fetch-1.7.3.tgz",
      "version": "1.7.3",
    },
    "node-modules-regexp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/pirates",
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/node-modules-regexp",
      "name": "node-modules-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/node-modules-regexp",
      "resolved": "https://registry.npmjs.org/node-modules-regexp/-/node-modules-regexp-1.0.0.tgz",
      "version": "1.0.0",
    },
    "normalize-package-data" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "normalize-path" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "normalize-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "normalize-path",
          "spec": "~3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-path",
      "name": "normalize-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/normalize-path",
      "resolved": "https://registry.npmjs.org/normalize-path/-/normalize-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "number-is-nan" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cliui/node_modules/wrap-ansi/node_modules/is-fullwidth-code-point",
          "name": "number-is-nan",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/number-is-nan",
      "name": "number-is-nan",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/number-is-nan",
      "resolved": "https://registry.npmjs.org/number-is-nan/-/number-is-nan-1.0.1.tgz",
      "version": "1.0.1",
    },
    "nyc" => ArboristNode {
      "children": Map {
        "resolve-from" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/nyc",
              "name": "resolve-from",
              "spec": "^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/nyc/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/nyc/node_modules/resolve-from",
          "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
          "version": "4.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/nyc/node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "object-assign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/create-react-class",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "object-assign",
          "spec": "^4.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/react",
          "name": "object-assign",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/object-assign",
      "name": "object-assign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/object-assign",
      "resolved": "https://registry.npmjs.org/object-assign/-/object-assign-4.1.1.tgz",
      "version": "4.1.1",
    },
    "once" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "opener" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
      "version": "1.5.1",
    },
    "optimist" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/optimist",
              "name": "minimist",
              "spec": "~0.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/optimist/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/optimist/node_modules/minimist",
          "resolved": "https://registry.npmjs.org/minimist/-/minimist-0.0.10.tgz",
          "version": "0.0.10",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "optimist",
          "spec": "^0.6.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "~0.0.1",
          "to": "node_modules/optimist/node_modules/minimist",
          "type": "prod",
        },
        "wordwrap" => EdgeOut {
          "name": "wordwrap",
          "spec": "~0.0.2",
          "to": "node_modules/wordwrap",
          "type": "prod",
        },
      },
      "location": "node_modules/optimist",
      "name": "optimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/optimist",
      "resolved": "https://registry.npmjs.org/optimist/-/optimist-0.6.1.tgz",
      "version": "0.6.1",
    },
    "os-homedir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.1.tgz",
      "version": "2.2.1",
    },
    "p-locate" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-key" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "path-key",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/path-key",
      "name": "path-key",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-key",
      "resolved": "https://registry.npmjs.org/path-key/-/path-key-2.0.1.tgz",
      "version": "2.0.1",
    },
    "path-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "picomatch" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/anymatch",
          "name": "picomatch",
          "spec": "^2.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readdirp",
          "name": "picomatch",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "location": "node_modules/picomatch",
      "name": "picomatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/picomatch",
      "resolved": "https://registry.npmjs.org/picomatch/-/picomatch-2.1.1.tgz",
      "version": "2.1.1",
    },
    "pify" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pirates" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "pirates",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "node-modules-regexp" => EdgeOut {
          "name": "node-modules-regexp",
          "spec": "^1.0.0",
          "to": "node_modules/node-modules-regexp",
          "type": "prod",
        },
      },
      "location": "node_modules/pirates",
      "name": "pirates",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/pirates",
      "resolved": "https://registry.npmjs.org/pirates/-/pirates-3.0.2.tgz",
      "version": "3.0.2",
    },
    "pkg-dir" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "promise" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "promise",
          "spec": "^7.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asap" => EdgeOut {
          "name": "asap",
          "spec": "~2.0.3",
          "to": "node_modules/asap",
          "type": "prod",
        },
      },
      "location": "node_modules/promise",
      "name": "promise",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/promise",
      "resolved": "https://registry.npmjs.org/promise/-/promise-7.3.1.tgz",
      "version": "7.3.1",
    },
    "prop-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/react",
          "name": "prop-types",
          "spec": "^15.5.10",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.4.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.1",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "react-is" => EdgeOut {
          "name": "react-is",
          "spec": "^16.8.1",
          "to": "node_modules/react-is",
          "type": "prod",
        },
      },
      "location": "node_modules/prop-types",
      "name": "prop-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/prop-types",
      "resolved": "https://registry.npmjs.org/prop-types/-/prop-types-15.7.2.tgz",
      "version": "15.7.2",
    },
    "pseudomap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.24",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.7.0.tgz",
      "version": "1.7.0",
    },
    "punycode" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "react" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "react",
          "spec": "15",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "create-react-class" => EdgeOut {
          "name": "create-react-class",
          "spec": "^15.6.0",
          "to": "node_modules/create-react-class",
          "type": "prod",
        },
        "fbjs" => EdgeOut {
          "name": "fbjs",
          "spec": "^0.8.9",
          "to": "node_modules/fbjs",
          "type": "prod",
        },
        "loose-envify" => EdgeOut {
          "name": "loose-envify",
          "spec": "^1.1.0",
          "to": "node_modules/loose-envify",
          "type": "prod",
        },
        "object-assign" => EdgeOut {
          "name": "object-assign",
          "spec": "^4.1.0",
          "to": "node_modules/object-assign",
          "type": "prod",
        },
        "prop-types" => EdgeOut {
          "name": "prop-types",
          "spec": "^15.5.10",
          "to": "node_modules/prop-types",
          "type": "prod",
        },
      },
      "location": "node_modules/react",
      "name": "react",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/react",
      "resolved": "https://registry.npmjs.org/react/-/react-15.6.2.tgz",
      "version": "15.6.2",
    },
    "react-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/prop-types",
          "name": "react-is",
          "spec": "^16.8.1",
          "type": "prod",
        },
      },
      "location": "node_modules/react-is",
      "name": "react-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/react-is",
      "resolved": "https://registry.npmjs.org/react-is/-/react-is-16.12.0.tgz",
      "version": "16.12.0",
    },
    "read-pkg" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readdirp" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chokidar",
          "name": "readdirp",
          "spec": "~3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "picomatch" => EdgeOut {
          "name": "picomatch",
          "spec": "^2.0.7",
          "to": "node_modules/picomatch",
          "type": "prod",
        },
      },
      "location": "node_modules/readdirp",
      "name": "readdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/readdirp",
      "resolved": "https://registry.npmjs.org/readdirp/-/readdirp-3.3.0.tgz",
      "version": "3.3.0",
    },
    "regenerator-runtime" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/runtime",
          "name": "regenerator-runtime",
          "spec": "^0.13.2",
          "type": "prod",
        },
      },
      "location": "node_modules/regenerator-runtime",
      "name": "regenerator-runtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/regenerator-runtime",
      "resolved": "https://registry.npmjs.org/regenerator-runtime/-/regenerator-runtime-0.13.3.tgz",
      "version": "0.13.3",
    },
    "release-zalgo" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.0",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.0.tgz",
      "version": "2.88.0",
    },
    "require-directory" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.14.1.tgz",
      "version": "1.14.1",
    },
    "rimraf" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.7.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.0.tgz",
      "version": "5.2.0",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/iconv-lite",
          "name": "safer-buffer",
          "spec": ">= 2.1.2 < 3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "semver" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "semver",
          "spec": "^5.5.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "setimmediate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "setimmediate",
          "spec": "^1.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/setimmediate",
      "name": "setimmediate",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/setimmediate",
      "resolved": "https://registry.npmjs.org/setimmediate/-/setimmediate-1.0.5.tgz",
      "version": "1.0.5",
    },
    "shebang-command" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo/node_modules/cross-spawn",
          "name": "shebang-command",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "shebang-regex" => EdgeOut {
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "to": "node_modules/shebang-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-command",
      "name": "shebang-command",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/shebang-command",
      "resolved": "https://registry.npmjs.org/shebang-command/-/shebang-command-1.2.0.tgz",
      "version": "1.2.0",
    },
    "shebang-regex" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/shebang-command",
          "name": "shebang-regex",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/shebang-regex",
      "name": "shebang-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/shebang-regex",
      "resolved": "https://registry.npmjs.org/shebang-regex/-/shebang-regex-1.0.0.tgz",
      "version": "1.0.0",
    },
    "signal-exit" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform/node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.2.tgz",
      "version": "3.0.2",
    },
    "source-map" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/handlebars",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/merge-source-map",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "source-map",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
      "version": "0.6.1",
    },
    "source-map-support" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/async-hook-domain",
          "name": "source-map-support",
          "spec": "^0.5.11",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.16",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.16.tgz",
      "version": "0.5.16",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
      "version": "3.1.0",
    },
    "spdx-exceptions" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
      "version": "2.2.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
      "version": "3.0.0",
    },
    "spdx-license-ids" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.5.tgz",
      "version": "3.0.5",
    },
    "sprintf-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "strip-bom" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "tap" => ArboristNode {
      "bundleDependencies": Array [
        "ink",
        "treport",
        "@types/react",
      ],
      "children": Map {
        "@babel/code-frame" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/highlight" => EdgeOut {
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/code-frame",
          "name": "@babel/code-frame",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/code-frame",
          "version": "7.5.5",
        },
        "@babel/core" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/core",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/core",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helpers" => EdgeOut {
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helpers",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "convert-source-map" => EdgeOut {
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "to": "node_modules/tap/node_modules/convert-source-map",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "json5" => EdgeOut {
              "name": "json5",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/json5",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "resolve" => EdgeOut {
              "name": "resolve",
              "spec": "^1.3.2",
              "to": "node_modules/tap/node_modules/resolve",
              "type": "prod",
            },
            "semver" => EdgeOut {
              "name": "semver",
              "spec": "^5.4.1",
              "to": "node_modules/tap/node_modules/semver",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/core/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/core",
          "name": "@babel/core",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/core",
          "version": "7.7.5",
        },
        "@babel/generator" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "source-map" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/generator",
                  "name": "source-map",
                  "spec": "^0.5.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "name": "source-map",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "version": "0.5.7",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "jsesc" => EdgeOut {
              "name": "jsesc",
              "spec": "^2.5.1",
              "to": "node_modules/tap/node_modules/jsesc",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "source-map" => EdgeOut {
              "name": "source-map",
              "spec": "^0.5.0",
              "to": "node_modules/tap/node_modules/@babel/generator/node_modules/source-map",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/generator",
          "name": "@babel/generator",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/generator",
          "version": "7.7.4",
        },
        "@babel/helper-builder-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "name": "@babel/helper-builder-react-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
          "version": "7.7.4",
        },
        "@babel/helper-function-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/helper-get-function-arity" => EdgeOut {
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "type": "prod",
            },
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-function-name",
          "name": "@babel/helper-function-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helper-function-name",
          "version": "7.7.4",
        },
        "@babel/helper-get-function-arity" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/helper-get-function-arity",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "name": "@babel/helper-get-function-arity",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helper-get-function-arity",
          "version": "7.7.4",
        },
        "@babel/helper-plugin-utils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "name": "@babel/helper-plugin-utils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helper-plugin-utils",
          "version": "7.0.0",
        },
        "@babel/helper-split-export-declaration" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/helper-split-export-declaration",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helper-split-export-declaration",
          "version": "7.7.4",
        },
        "@babel/helpers" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/helpers",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/template" => EdgeOut {
              "name": "@babel/template",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/template",
              "type": "prod",
            },
            "@babel/traverse" => EdgeOut {
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/traverse",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/helpers",
          "name": "@babel/helpers",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/helpers",
          "version": "7.7.4",
        },
        "@babel/highlight" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "chalk" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight",
                  "name": "chalk",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.1",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "escape-string-regexp" => EdgeOut {
                  "name": "escape-string-regexp",
                  "spec": "^1.0.5",
                  "to": "node_modules/tap/node_modules/escape-string-regexp",
                  "type": "prod",
                },
                "supports-color" => EdgeOut {
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "chalk",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "version": "2.4.2",
            },
            "supports-color" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
                  "name": "supports-color",
                  "spec": "^5.3.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "has-flag" => EdgeOut {
                  "name": "has-flag",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/has-flag",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "supports-color",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "version": "5.5.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/code-frame",
              "name": "@babel/highlight",
              "spec": "^7.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "type": "prod",
            },
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/highlight",
          "name": "@babel/highlight",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/highlight",
          "version": "7.5.0",
        },
        "@babel/parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/parser",
              "spec": "^7.7.5",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/parser",
          "name": "@babel/parser",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/parser",
          "version": "7.7.5",
        },
        "@babel/plugin-proposal-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "name": "@babel/plugin-proposal-object-rest-spread",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "name": "@babel/plugin-syntax-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
          "version": "7.7.4",
        },
        "@babel/plugin-syntax-object-rest-spread" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "name": "@babel/plugin-syntax-object-rest-spread",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "name": "@babel/plugin-syntax-object-rest-spread",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/plugin-syntax-object-rest-spread",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-destructuring" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "name": "@babel/plugin-transform-destructuring",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
          "version": "7.7.4",
        },
        "@babel/plugin-transform-react-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.0.0-0",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "peer",
            },
            "@babel/helper-builder-react-jsx" => EdgeOut {
              "name": "@babel/helper-builder-react-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "type": "prod",
            },
            "@babel/helper-plugin-utils" => EdgeOut {
              "name": "@babel/helper-plugin-utils",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/helper-plugin-utils",
              "type": "prod",
            },
            "@babel/plugin-syntax-jsx" => EdgeOut {
              "name": "@babel/plugin-syntax-jsx",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/plugin-syntax-jsx",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "name": "@babel/plugin-transform-react-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
          "version": "7.7.4",
        },
        "@babel/runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/yaml",
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "regenerator-runtime" => EdgeOut {
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "to": "node_modules/tap/node_modules/regenerator-runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/runtime",
          "name": "@babel/runtime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/runtime",
          "version": "7.7.6",
        },
        "@babel/template" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/template",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.0.0",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/template",
          "name": "@babel/template",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/template",
          "version": "7.7.4",
        },
        "@babel/traverse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/traverse",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/code-frame" => EdgeOut {
              "name": "@babel/code-frame",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/code-frame",
              "type": "prod",
            },
            "@babel/generator" => EdgeOut {
              "name": "@babel/generator",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/generator",
              "type": "prod",
            },
            "@babel/helper-function-name" => EdgeOut {
              "name": "@babel/helper-function-name",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-function-name",
              "type": "prod",
            },
            "@babel/helper-split-export-declaration" => EdgeOut {
              "name": "@babel/helper-split-export-declaration",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "type": "prod",
            },
            "@babel/parser" => EdgeOut {
              "name": "@babel/parser",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/parser",
              "type": "prod",
            },
            "@babel/types" => EdgeOut {
              "name": "@babel/types",
              "spec": "^7.7.4",
              "to": "node_modules/tap/node_modules/@babel/types",
              "type": "prod",
            },
            "debug" => EdgeOut {
              "name": "debug",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/debug",
              "type": "prod",
            },
            "globals" => EdgeOut {
              "name": "globals",
              "spec": "^11.1.0",
              "to": "node_modules/tap/node_modules/globals",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/traverse",
          "name": "@babel/traverse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/traverse",
          "version": "7.7.4",
        },
        "@babel/types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-function-name",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-get-function-arity",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-split-export-declaration",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helpers",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/template",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "@babel/types",
              "spec": "^7.7.4",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esutils" => EdgeOut {
              "name": "esutils",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/esutils",
              "type": "prod",
            },
            "lodash" => EdgeOut {
              "name": "lodash",
              "spec": "^4.17.13",
              "to": "node_modules/tap/node_modules/lodash",
              "type": "prod",
            },
            "to-fast-properties" => EdgeOut {
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/to-fast-properties",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@babel/types",
          "name": "@babel/types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@babel/types",
          "version": "7.7.4",
        },
        "@types/color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "@types/color-name",
              "spec": "^1.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/color-name",
          "name": "@types/color-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@types/color-name",
          "version": "1.1.1",
        },
        "@types/prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "@types/prop-types",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/prop-types",
          "name": "@types/prop-types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@types/prop-types",
          "version": "15.7.3",
        },
        "@types/react" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "@types/react",
              "spec": "^16.9.16",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/auto-bind",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peerOptional",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "@types/react",
              "spec": ">=16.8.0",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "@types/prop-types" => EdgeOut {
              "name": "@types/prop-types",
              "spec": "*",
              "to": "node_modules/tap/node_modules/@types/prop-types",
              "type": "prod",
            },
            "csstype" => EdgeOut {
              "name": "csstype",
              "spec": "^2.2.0",
              "to": "node_modules/tap/node_modules/csstype",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/@types/react",
          "name": "@types/react",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/@types/react",
          "version": "16.9.16",
        },
        "ansi-escapes" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "type-fest" => EdgeOut {
              "name": "type-fest",
              "spec": "^0.8.1",
              "to": "node_modules/tap/node_modules/type-fest",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-escapes",
          "name": "ansi-escapes",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ansi-escapes",
          "version": "4.3.0",
        },
        "ansi-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ansi-regex",
          "version": "5.0.0",
        },
        "ansi-styles" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "ansi-styles",
              "spec": "^3.2.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-convert" => EdgeOut {
              "name": "color-convert",
              "spec": "^1.9.0",
              "to": "node_modules/tap/node_modules/color-convert",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansi-styles",
          "name": "ansi-styles",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ansi-styles",
          "version": "3.2.1",
        },
        "ansicolors" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "ansicolors",
              "spec": "~0.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ansicolors",
          "name": "ansicolors",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ansicolors",
          "version": "0.3.2",
        },
        "arrify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "arrify",
              "spec": "^2.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/arrify",
          "name": "arrify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/arrify",
          "version": "2.0.1",
        },
        "astral-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "astral-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/astral-regex",
          "name": "astral-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/astral-regex",
          "version": "2.0.0",
        },
        "auto-bind" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "auto-bind",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peerOptional",
            },
          },
          "location": "node_modules/tap/node_modules/auto-bind",
          "name": "auto-bind",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/auto-bind",
          "version": "3.0.0",
        },
        "caller-callsite" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-path",
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "callsites" => EdgeOut {
              "name": "callsites",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/callsites",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-callsite",
          "name": "caller-callsite",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/caller-callsite",
          "version": "2.0.0",
        },
        "caller-path" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "caller-path",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "caller-callsite" => EdgeOut {
              "name": "caller-callsite",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-callsite",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/caller-path",
          "name": "caller-path",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/caller-path",
          "version": "2.0.0",
        },
        "callsites" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/caller-callsite",
              "name": "callsites",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/callsites",
          "name": "callsites",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/callsites",
          "version": "2.0.0",
        },
        "cardinal" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "cardinal",
              "spec": "^2.1.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansicolors" => EdgeOut {
              "name": "ansicolors",
              "spec": "~0.3.2",
              "to": "node_modules/tap/node_modules/ansicolors",
              "type": "prod",
            },
            "redeyed" => EdgeOut {
              "name": "redeyed",
              "spec": "~2.1.0",
              "to": "node_modules/tap/node_modules/redeyed",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cardinal",
          "name": "cardinal",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/cardinal",
          "version": "2.1.1",
        },
        "chalk" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk",
                  "name": "ansi-styles",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/chalk/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/chalk/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/chalk/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/chalk/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/chalk/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "chalk",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/chalk/node_modules/ansi-styles",
              "type": "prod",
            },
            "supports-color" => EdgeOut {
              "name": "supports-color",
              "spec": "^7.1.0",
              "to": "node_modules/tap/node_modules/supports-color",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/chalk",
          "name": "chalk",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/chalk",
          "version": "3.0.0",
        },
        "ci-info" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/is-ci",
              "name": "ci-info",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ci-info",
          "name": "ci-info",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ci-info",
          "version": "2.0.0",
        },
        "cli-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "restore-cursor" => EdgeOut {
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/restore-cursor",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-cursor",
          "name": "cli-cursor",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/cli-cursor",
          "version": "3.1.0",
        },
        "cli-truncate" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.2.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/cli-truncate",
          "name": "cli-truncate",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/cli-truncate",
          "version": "2.1.0",
        },
        "color-convert" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-styles",
              "name": "color-convert",
              "spec": "^1.9.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "color-name" => EdgeOut {
              "name": "color-name",
              "spec": "1.1.3",
              "to": "node_modules/tap/node_modules/color-name",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-convert",
          "name": "color-convert",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/color-convert",
          "version": "1.9.3",
        },
        "color-name" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/color-convert",
              "name": "color-name",
              "spec": "1.1.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/color-name",
          "name": "color-name",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/color-name",
          "version": "1.1.3",
        },
        "convert-source-map" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "safe-buffer" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/convert-source-map",
                  "name": "safe-buffer",
                  "spec": "~5.1.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "name": "safe-buffer",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "version": "5.1.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "convert-source-map",
              "spec": "^1.7.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "safe-buffer" => EdgeOut {
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "to": "node_modules/tap/node_modules/convert-source-map/node_modules/safe-buffer",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/convert-source-map",
          "name": "convert-source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/convert-source-map",
          "version": "1.7.0",
        },
        "csstype" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@types/react",
              "name": "csstype",
              "spec": "^2.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/csstype",
          "name": "csstype",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/csstype",
          "version": "2.6.8",
        },
        "debug" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "debug",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/debug",
          "version": "4.1.1",
        },
        "emoji-regex" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/emoji-regex",
          "version": "8.0.0",
        },
        "escape-string-regexp" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/chalk",
              "name": "escape-string-regexp",
              "spec": "^1.0.5",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/escape-string-regexp",
          "name": "escape-string-regexp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/escape-string-regexp",
          "version": "1.0.5",
        },
        "esprima" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/redeyed",
              "name": "esprima",
              "spec": "~4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esprima",
          "name": "esprima",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/esprima",
          "version": "4.0.1",
        },
        "esutils" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/helper-builder-react-jsx",
              "name": "esutils",
              "spec": "^2.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "esutils",
              "spec": "^2.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/esutils",
          "name": "esutils",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/esutils",
          "version": "2.0.3",
        },
        "events-to-array" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "events-to-array",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/events-to-array",
          "name": "events-to-array",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/events-to-array",
          "version": "1.1.2",
        },
        "globals" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "globals",
              "spec": "^11.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/globals",
          "name": "globals",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/globals",
          "version": "11.12.0",
        },
        "has-flag" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight/node_modules/supports-color",
              "name": "has-flag",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/has-flag",
          "name": "has-flag",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/has-flag",
          "version": "3.0.0",
        },
        "import-jsx" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "import-jsx",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/core" => EdgeOut {
              "name": "@babel/core",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/core",
              "type": "prod",
            },
            "@babel/plugin-proposal-object-rest-spread" => EdgeOut {
              "name": "@babel/plugin-proposal-object-rest-spread",
              "spec": "^7.5.5",
              "to": "node_modules/tap/node_modules/@babel/plugin-proposal-object-rest-spread",
              "type": "prod",
            },
            "@babel/plugin-transform-destructuring" => EdgeOut {
              "name": "@babel/plugin-transform-destructuring",
              "spec": "^7.5.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-destructuring",
              "type": "prod",
            },
            "@babel/plugin-transform-react-jsx" => EdgeOut {
              "name": "@babel/plugin-transform-react-jsx",
              "spec": "^7.3.0",
              "to": "node_modules/tap/node_modules/@babel/plugin-transform-react-jsx",
              "type": "prod",
            },
            "caller-path" => EdgeOut {
              "name": "caller-path",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/caller-path",
              "type": "prod",
            },
            "resolve-from" => EdgeOut {
              "name": "resolve-from",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/resolve-from",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/import-jsx",
          "name": "import-jsx",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/import-jsx",
          "version": "3.0.0",
        },
        "ink" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ink",
              "spec": "^2.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@types/react" => EdgeOut {
              "name": "@types/react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/@types/react",
              "type": "peer",
            },
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^4.2.1",
              "to": "node_modules/tap/node_modules/ansi-escapes",
              "type": "prod",
            },
            "arrify" => EdgeOut {
              "name": "arrify",
              "spec": "^2.0.1",
              "to": "node_modules/tap/node_modules/arrify",
              "type": "prod",
            },
            "auto-bind" => EdgeOut {
              "name": "auto-bind",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/auto-bind",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/cli-cursor",
              "type": "prod",
            },
            "cli-truncate" => EdgeOut {
              "name": "cli-truncate",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/cli-truncate",
              "type": "prod",
            },
            "is-ci" => EdgeOut {
              "name": "is-ci",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/is-ci",
              "type": "prod",
            },
            "lodash.throttle" => EdgeOut {
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/lodash.throttle",
              "type": "prod",
            },
            "log-update" => EdgeOut {
              "name": "log-update",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/log-update",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": ">=16.8.0",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "react-reconciler" => EdgeOut {
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "to": "node_modules/tap/node_modules/react-reconciler",
              "type": "prod",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
            "slice-ansi" => EdgeOut {
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi",
              "type": "prod",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "widest-line" => EdgeOut {
              "name": "widest-line",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/widest-line",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "to": "node_modules/tap/node_modules/wrap-ansi",
              "type": "prod",
            },
            "yoga-layout-prebuilt" => EdgeOut {
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "to": "node_modules/tap/node_modules/yoga-layout-prebuilt",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ink",
          "name": "ink",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ink",
          "version": "2.6.0",
        },
        "is-ci" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "is-ci",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ci-info" => EdgeOut {
              "name": "ci-info",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/ci-info",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-ci",
          "name": "is-ci",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/is-ci",
          "version": "2.0.0",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/slice-ansi",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/is-fullwidth-code-point",
          "version": "3.0.0",
        },
        "js-tokens" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/highlight",
              "name": "js-tokens",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/loose-envify",
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/js-tokens",
          "name": "js-tokens",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/js-tokens",
          "version": "4.0.0",
        },
        "jsesc" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "jsesc",
              "spec": "^2.5.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/jsesc",
          "name": "jsesc",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/jsesc",
          "version": "2.5.2",
        },
        "json5" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "json5",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.0",
              "to": "node_modules/tap/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/json5",
          "name": "json5",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/json5",
          "version": "2.1.1",
        },
        "lodash" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/generator",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/traverse",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "lodash",
              "spec": "^4.17.13",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash",
          "name": "lodash",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/lodash",
          "version": "4.17.15",
        },
        "lodash.throttle" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "lodash.throttle",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/lodash.throttle",
          "name": "lodash.throttle",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/lodash.throttle",
          "version": "4.1.1",
        },
        "log-update" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-escapes" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "ansi-escapes",
                  "spec": "^3.2.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "name": "ansi-escapes",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "version": "3.2.0",
            },
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "cli-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "cli-cursor",
                  "spec": "^2.1.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "restore-cursor" => EdgeOut {
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "name": "cli-cursor",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "version": "2.1.0",
            },
            "emoji-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "name": "emoji-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
              "version": "7.0.3",
            },
            "is-fullwidth-code-point" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "name": "is-fullwidth-code-point",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
              "version": "2.0.0",
            },
            "mimic-fn" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "name": "mimic-fn",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
              "version": "1.2.0",
            },
            "onetime" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "mimic-fn" => EdgeOut {
                  "name": "mimic-fn",
                  "spec": "^1.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/mimic-fn",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/onetime",
              "name": "onetime",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/onetime",
              "version": "2.0.1",
            },
            "restore-cursor" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
                  "name": "restore-cursor",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "onetime" => EdgeOut {
                  "name": "onetime",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/onetime",
                  "type": "prod",
                },
                "signal-exit" => EdgeOut {
                  "name": "signal-exit",
                  "spec": "^3.0.2",
                  "to": "node_modules/tap/node_modules/signal-exit",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "restore-cursor",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "version": "2.0.0",
            },
            "string-width" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "emoji-regex" => EdgeOut {
                  "name": "emoji-regex",
                  "spec": "^7.0.1",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/emoji-regex",
                  "type": "prod",
                },
                "is-fullwidth-code-point" => EdgeOut {
                  "name": "is-fullwidth-code-point",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/is-fullwidth-code-point",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/string-width",
              "name": "string-width",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/string-width",
              "version": "3.1.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "name": "strip-ansi",
                  "spec": "^5.1.0",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
              "version": "5.2.0",
            },
            "wrap-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/log-update",
                  "name": "wrap-ansi",
                  "spec": "^5.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-styles" => EdgeOut {
                  "name": "ansi-styles",
                  "spec": "^3.2.0",
                  "to": "node_modules/tap/node_modules/ansi-styles",
                  "type": "prod",
                },
                "string-width" => EdgeOut {
                  "name": "string-width",
                  "spec": "^3.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/string-width",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^5.0.0",
                  "to": "node_modules/tap/node_modules/log-update/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "name": "wrap-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "version": "5.1.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "log-update",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-escapes" => EdgeOut {
              "name": "ansi-escapes",
              "spec": "^3.2.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/ansi-escapes",
              "type": "prod",
            },
            "cli-cursor" => EdgeOut {
              "name": "cli-cursor",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/cli-cursor",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/log-update/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/log-update",
          "name": "log-update",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/log-update",
          "version": "3.3.0",
        },
        "loose-envify" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "loose-envify",
              "spec": "^1.4.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "loose-envify",
              "spec": "^1.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "js-tokens" => EdgeOut {
              "name": "js-tokens",
              "spec": "^3.0.0 || ^4.0.0",
              "to": "node_modules/tap/node_modules/js-tokens",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/loose-envify",
          "name": "loose-envify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/loose-envify",
          "version": "1.4.0",
        },
        "mimic-fn" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/onetime",
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/mimic-fn",
          "name": "mimic-fn",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/mimic-fn",
          "version": "2.1.0",
        },
        "minimist" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/json5",
              "name": "minimist",
              "spec": "^1.2.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/minimist",
          "version": "1.2.0",
        },
        "minipass" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "yallist" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/minipass",
                  "name": "yallist",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "name": "yallist",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/minipass/node_modules/yallist",
              "version": "4.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "minipass",
              "spec": "^3.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "minipass",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yallist" => EdgeOut {
              "name": "yallist",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/minipass/node_modules/yallist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minipass",
          "name": "minipass",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/minipass",
          "version": "3.1.1",
        },
        "ms" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/debug",
              "name": "ms",
              "spec": "^2.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "ms",
              "spec": "^2.1.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/ms",
          "version": "2.1.2",
        },
        "object-assign" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/scheduler",
              "name": "object-assign",
              "spec": "^4.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/object-assign",
          "name": "object-assign",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/object-assign",
          "version": "4.1.1",
        },
        "onetime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "onetime",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "mimic-fn" => EdgeOut {
              "name": "mimic-fn",
              "spec": "^2.1.0",
              "to": "node_modules/tap/node_modules/mimic-fn",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/onetime",
          "name": "onetime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/onetime",
          "version": "5.1.0",
        },
        "path-parse" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/resolve",
              "name": "path-parse",
              "spec": "^1.0.6",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/path-parse",
          "name": "path-parse",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/path-parse",
          "version": "1.0.6",
        },
        "prop-types" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "prop-types",
              "spec": "^15.6.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.4.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "react-is" => EdgeOut {
              "name": "react-is",
              "spec": "^16.8.1",
              "to": "node_modules/tap/node_modules/react-is",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/prop-types",
          "name": "prop-types",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/prop-types",
          "version": "15.7.2",
        },
        "punycode" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "punycode",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/punycode",
          "version": "2.1.1",
        },
        "react" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "react",
              "spec": "^16.12.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react",
              "spec": ">=16.8.0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "react",
              "spec": "^16.0.0",
              "type": "peer",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "react",
              "spec": "^16.8.6",
              "type": "peer",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react",
          "name": "react",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/react",
          "resolved": "https://registry.npmjs.org/react/-/react-16.12.0.tgz",
          "version": "16.12.0",
        },
        "react-is" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/prop-types",
              "name": "react-is",
              "spec": "^16.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-is",
          "name": "react-is",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/react-is",
          "version": "16.12.0",
        },
        "react-reconciler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "react-reconciler",
              "spec": "^0.24.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
            "prop-types" => EdgeOut {
              "name": "prop-types",
              "spec": "^15.6.2",
              "to": "node_modules/tap/node_modules/prop-types",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.0.0",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "scheduler" => EdgeOut {
              "name": "scheduler",
              "spec": "^0.18.0",
              "to": "node_modules/tap/node_modules/scheduler",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/react-reconciler",
          "name": "react-reconciler",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/react-reconciler",
          "version": "0.24.0",
        },
        "redeyed" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cardinal",
              "name": "redeyed",
              "spec": "~2.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "esprima" => EdgeOut {
              "name": "esprima",
              "spec": "~4.0.0",
              "to": "node_modules/tap/node_modules/esprima",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/redeyed",
          "name": "redeyed",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/redeyed",
          "version": "2.1.1",
        },
        "regenerator-runtime" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/runtime",
              "name": "regenerator-runtime",
              "spec": "^0.13.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/regenerator-runtime",
          "name": "regenerator-runtime",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/regenerator-runtime",
          "version": "0.13.3",
        },
        "resolve" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "resolve",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "path-parse" => EdgeOut {
              "name": "path-parse",
              "spec": "^1.0.6",
              "to": "node_modules/tap/node_modules/path-parse",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve",
          "name": "resolve",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/resolve",
          "version": "1.13.1",
        },
        "resolve-from" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/import-jsx",
              "name": "resolve-from",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/resolve-from",
          "name": "resolve-from",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/resolve-from",
          "version": "3.0.0",
        },
        "restore-cursor" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-cursor",
              "name": "restore-cursor",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "onetime" => EdgeOut {
              "name": "onetime",
              "spec": "^5.1.0",
              "to": "node_modules/tap/node_modules/onetime",
              "type": "prod",
            },
            "signal-exit" => EdgeOut {
              "name": "signal-exit",
              "spec": "^3.0.2",
              "to": "node_modules/tap/node_modules/signal-exit",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/restore-cursor",
          "name": "restore-cursor",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/restore-cursor",
          "version": "3.1.0",
        },
        "scheduler" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/react-reconciler",
              "name": "scheduler",
              "spec": "^0.18.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "loose-envify" => EdgeOut {
              "name": "loose-envify",
              "spec": "^1.1.0",
              "to": "node_modules/tap/node_modules/loose-envify",
              "type": "prod",
            },
            "object-assign" => EdgeOut {
              "name": "object-assign",
              "spec": "^4.1.1",
              "to": "node_modules/tap/node_modules/object-assign",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/scheduler",
          "name": "scheduler",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/scheduler",
          "version": "0.18.0",
        },
        "semver" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/core",
              "name": "semver",
              "spec": "^5.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/semver",
          "version": "5.7.1",
        },
        "signal-exit" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "signal-exit",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/log-update/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/restore-cursor",
              "name": "signal-exit",
              "spec": "^3.0.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/signal-exit",
          "name": "signal-exit",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/signal-exit",
          "version": "3.0.2",
        },
        "slice-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/slice-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/slice-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "slice-ansi",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/slice-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^2.0.0",
              "to": "node_modules/tap/node_modules/astral-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/slice-ansi",
          "name": "slice-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/slice-ansi",
          "version": "3.0.0",
        },
        "string-length" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
              "version": "4.1.0",
            },
            "astral-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "astral-regex",
                  "spec": "^1.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "name": "astral-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "version": "1.0.0",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/string-length",
                  "name": "strip-ansi",
                  "spec": "^5.2.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^4.1.0",
                  "to": "node_modules/tap/node_modules/string-length/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "version": "5.2.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "string-length",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "astral-regex" => EdgeOut {
              "name": "astral-regex",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/astral-regex",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/tap/node_modules/string-length/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-length",
          "name": "string-length",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/string-length",
          "version": "3.1.0",
        },
        "string-width" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/cli-truncate",
              "name": "string-width",
              "spec": "^4.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/widest-line",
              "name": "string-width",
              "spec": "^4.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^8.0.0",
              "to": "node_modules/tap/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/string-width",
          "version": "4.2.0",
        },
        "strip-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^5.0.0",
              "to": "node_modules/tap/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/strip-ansi",
          "version": "6.0.0",
        },
        "supports-color" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "has-flag" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/supports-color",
                  "name": "has-flag",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "name": "has-flag",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "version": "4.0.0",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/chalk",
              "name": "supports-color",
              "spec": "^7.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/supports-color/node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/supports-color",
          "version": "7.1.0",
        },
        "tap-parser" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/treport",
              "name": "tap-parser",
              "spec": "^10.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "events-to-array" => EdgeOut {
              "name": "events-to-array",
              "spec": "^1.0.1",
              "to": "node_modules/tap/node_modules/events-to-array",
              "type": "prod",
            },
            "minipass" => EdgeOut {
              "name": "minipass",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/minipass",
              "type": "prod",
            },
            "tap-yaml" => EdgeOut {
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "to": "node_modules/tap/node_modules/tap-yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-parser",
          "name": "tap-parser",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/tap-parser",
          "version": "10.0.1",
        },
        "tap-yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-parser",
              "name": "tap-yaml",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "yaml" => EdgeOut {
              "name": "yaml",
              "spec": "^1.5.0",
              "to": "node_modules/tap/node_modules/yaml",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/tap-yaml",
          "name": "tap-yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/tap-yaml",
          "version": "1.0.0",
        },
        "to-fast-properties" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/@babel/types",
              "name": "to-fast-properties",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/to-fast-properties",
          "name": "to-fast-properties",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/to-fast-properties",
          "version": "2.0.0",
        },
        "treport" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-regex" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "name": "ansi-regex",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/treport/node_modules/ansi-regex",
              "version": "2.1.1",
            },
            "strip-ansi" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "ansi-regex" => EdgeOut {
                  "name": "ansi-regex",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/treport/node_modules/ansi-regex",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "name": "strip-ansi",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/treport/node_modules/strip-ansi",
              "version": "3.0.1",
            },
            "unicode-length" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/treport",
                  "name": "unicode-length",
                  "spec": "^2.0.2",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "punycode" => EdgeOut {
                  "name": "punycode",
                  "spec": "^2.0.0",
                  "to": "node_modules/tap/node_modules/punycode",
                  "type": "prod",
                },
                "strip-ansi" => EdgeOut {
                  "name": "strip-ansi",
                  "spec": "^3.0.1",
                  "to": "node_modules/tap/node_modules/treport/node_modules/strip-ansi",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "name": "unicode-length",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "version": "2.0.2",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "treport",
              "spec": "^1.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "cardinal" => EdgeOut {
              "name": "cardinal",
              "spec": "^2.1.1",
              "to": "node_modules/tap/node_modules/cardinal",
              "type": "prod",
            },
            "chalk" => EdgeOut {
              "name": "chalk",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/chalk",
              "type": "prod",
            },
            "import-jsx" => EdgeOut {
              "name": "import-jsx",
              "spec": "^3.0.0",
              "to": "node_modules/tap/node_modules/import-jsx",
              "type": "prod",
            },
            "ink" => EdgeOut {
              "name": "ink",
              "spec": "^2.5.0",
              "to": "node_modules/tap/node_modules/ink",
              "type": "prod",
            },
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "^2.1.2",
              "to": "node_modules/tap/node_modules/ms",
              "type": "prod",
            },
            "react" => EdgeOut {
              "name": "react",
              "spec": "^16.8.6",
              "to": "node_modules/tap/node_modules/react",
              "type": "peer",
            },
            "string-length" => EdgeOut {
              "name": "string-length",
              "spec": "^3.1.0",
              "to": "node_modules/tap/node_modules/string-length",
              "type": "prod",
            },
            "tap-parser" => EdgeOut {
              "name": "tap-parser",
              "spec": "^10.0.1",
              "to": "node_modules/tap/node_modules/tap-parser",
              "type": "prod",
            },
            "unicode-length" => EdgeOut {
              "name": "unicode-length",
              "spec": "^2.0.2",
              "to": "node_modules/tap/node_modules/treport/node_modules/unicode-length",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/treport",
          "name": "treport",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/treport",
          "version": "1.0.1",
        },
        "type-fest" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ansi-escapes",
              "name": "type-fest",
              "spec": "^0.8.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/type-fest",
          "name": "type-fest",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/type-fest",
          "version": "0.8.1",
        },
        "widest-line" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "widest-line",
              "spec": "^3.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/widest-line",
          "name": "widest-line",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/widest-line",
          "version": "3.1.0",
        },
        "wrap-ansi" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "children": Map {
            "ansi-styles" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi",
                  "name": "ansi-styles",
                  "spec": "^4.0.0",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@types/color-name" => EdgeOut {
                  "name": "@types/color-name",
                  "spec": "^1.1.1",
                  "to": "node_modules/tap/node_modules/@types/color-name",
                  "type": "prod",
                },
                "color-convert" => EdgeOut {
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "name": "ansi-styles",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "version": "4.2.0",
            },
            "color-convert" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
                  "name": "color-convert",
                  "spec": "^2.0.1",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "color-name" => EdgeOut {
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "name": "color-convert",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
              "version": "2.0.1",
            },
            "color-name" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/tap",
              "dev": true,
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-convert",
                  "name": "color-name",
                  "spec": "~1.1.4",
                  "type": "prod",
                },
              },
              "location": "node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "name": "color-name",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/wrap-ansi/node_modules/color-name",
              "version": "1.1.4",
            },
          },
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "wrap-ansi",
              "spec": "^6.2.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^4.0.0",
              "to": "node_modules/tap/node_modules/wrap-ansi/node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^4.1.0",
              "to": "node_modules/tap/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^6.0.0",
              "to": "node_modules/tap/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/wrap-ansi",
          "version": "6.2.0",
        },
        "yaml" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "yaml",
              "spec": "^1.7.2",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/tap/node_modules/tap-yaml",
              "name": "yaml",
              "spec": "^1.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@babel/runtime" => EdgeOut {
              "name": "@babel/runtime",
              "spec": "^7.6.3",
              "to": "node_modules/tap/node_modules/@babel/runtime",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yaml",
          "name": "yaml",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/yaml",
          "version": "1.7.2",
        },
        "yoga-layout-prebuilt" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/tap",
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/ink",
              "name": "yoga-layout-prebuilt",
              "spec": "^1.9.3",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/yoga-layout-prebuilt",
          "name": "yoga-layout-prebuilt",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap/node_modules/yoga-layout-prebuilt",
          "version": "1.9.3",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "^14.10.5",
          "type": "dev",
        },
      },
      "edgesOut": Map {
        "@types/react" => EdgeOut {
          "name": "@types/react",
          "spec": "^16.9.16",
          "to": "node_modules/tap/node_modules/@types/react",
          "type": "prod",
        },
        "async-hook-domain" => EdgeOut {
          "name": "async-hook-domain",
          "spec": "^1.1.3",
          "to": "node_modules/async-hook-domain",
          "type": "prod",
        },
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "chokidar" => EdgeOut {
          "name": "chokidar",
          "spec": "^3.3.0",
          "to": "node_modules/chokidar",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.8",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.25",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "findit" => EdgeOut {
          "name": "findit",
          "spec": "^2.0.0",
          "to": "node_modules/findit",
          "type": "prod",
        },
        "flow-remove-types" => EdgeOut {
          "name": "flow-remove-types",
          "spec": "^2.112.0",
          "to": "node_modules/flow-remove-types",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.2",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.6",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "import-jsx" => EdgeOut {
          "name": "import-jsx",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/import-jsx",
          "type": "prod",
        },
        "ink" => EdgeOut {
          "name": "ink",
          "spec": "^2.5.0",
          "to": "node_modules/tap/node_modules/ink",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "istanbul-lib-processinfo" => EdgeOut {
          "name": "istanbul-lib-processinfo",
          "spec": "^1.0.0",
          "to": "node_modules/istanbul-lib-processinfo",
          "type": "prod",
        },
        "jackspeak" => EdgeOut {
          "name": "jackspeak",
          "spec": "^1.4.0",
          "to": "node_modules/jackspeak",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.1.1",
          "to": "node_modules/tap/node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.1.1",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "react" => EdgeOut {
          "name": "react",
          "spec": "^16.12.0",
          "to": "node_modules/tap/node_modules/react",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.7.1",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/tap/node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.16",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.1",
          "to": "node_modules/tap/node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap/node_modules/tap-yaml",
          "type": "prod",
        },
        "tcompare" => EdgeOut {
          "name": "tcompare",
          "spec": "^3.0.0",
          "to": "node_modules/tcompare",
          "type": "prod",
        },
        "treport" => EdgeOut {
          "name": "treport",
          "spec": "^1.0.1",
          "to": "node_modules/tap/node_modules/treport",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.5.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.7.2",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^2.0.2",
          "to": "node_modules/which",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.7.2",
          "to": "node_modules/tap/node_modules/yaml",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-14.10.5.tgz",
      "version": "14.10.5",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "debug",
              "spec": "^2.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/tap-mocha-reporter/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-mocha-reporter/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "diff" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "diff",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/diff",
          "name": "diff",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-mocha-reporter/node_modules/diff",
          "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
          "version": "1.4.0",
        },
        "ms" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-mocha-reporter/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.3",
          "to": "node_modules/tap-mocha-reporter/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^1.3.2",
          "to": "node_modules/tap-mocha-reporter/node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": null,
          "type": "optional",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^10.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^1.0.0",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-5.0.0.tgz",
      "version": "5.0.0",
    },
    "tap-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-parser",
          "spec": "^10.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^3.0.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "tap-yaml" => EdgeOut {
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "to": "node_modules/tap-yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-10.0.1.tgz",
      "version": "10.0.1",
    },
    "tap-yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "tap-yaml",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "yaml" => EdgeOut {
          "name": "yaml",
          "spec": "^1.5.0",
          "to": "node_modules/yaml",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-yaml",
      "name": "tap-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tap-yaml",
      "resolved": "https://registry.npmjs.org/tap-yaml/-/tap-yaml-1.0.0.tgz",
      "version": "1.0.0",
    },
    "tcompare" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tcompare",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "diff-frag" => EdgeOut {
          "name": "diff-frag",
          "spec": "^1.0.1",
          "to": "node_modules/diff-frag",
          "type": "prod",
        },
      },
      "location": "node_modules/tcompare",
      "name": "tcompare",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tcompare",
      "resolved": "https://registry.npmjs.org/tcompare/-/tcompare-3.0.4.tgz",
      "version": "3.0.4",
    },
    "test-exclude" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "to-fast-properties" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "to-regex-range" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fill-range",
          "name": "to-regex-range",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-number" => EdgeOut {
          "name": "is-number",
          "spec": "^7.0.0",
          "to": "node_modules/is-number",
          "type": "prod",
        },
      },
      "location": "node_modules/to-regex-range",
      "name": "to-regex-range",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/to-regex-range",
      "resolved": "https://registry.npmjs.org/to-regex-range/-/to-regex-range-5.0.1.tgz",
      "version": "5.0.1",
    },
    "tough-cookie" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tough-cookie",
              "name": "punycode",
              "spec": "^1.4.1",
              "type": "prod",
            },
          },
          "location": "node_modules/tough-cookie/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tough-cookie/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.24",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.4.1",
          "to": "node_modules/tough-cookie/node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.4.3.tgz",
      "version": "2.4.3",
    },
    "trivial-deferred" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.5.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": ">=2.0",
          "to": "node_modules/typescript",
          "type": "peer",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "^3.0.0",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.5.4.tgz",
      "version": "8.5.4",
    },
    "tunnel-agent" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typedarray-to-buffer" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
      },
      "location": "node_modules/typedarray-to-buffer",
      "name": "typedarray-to-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/typedarray-to-buffer",
      "resolved": "https://registry.npmjs.org/typedarray-to-buffer/-/typedarray-to-buffer-3.1.5.tgz",
      "version": "3.1.5",
    },
    "typescript" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.7.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "typescript",
          "spec": ">=2.0",
          "type": "peer",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.7.4.tgz",
      "version": "3.7.4",
    },
    "ua-parser-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/fbjs",
          "name": "ua-parser-js",
          "spec": "^0.7.18",
          "type": "prod",
        },
      },
      "location": "node_modules/ua-parser-js",
      "name": "ua-parser-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/ua-parser-js",
      "resolved": "https://registry.npmjs.org/ua-parser-js/-/ua-parser-js-0.7.20.tgz",
      "version": "0.7.20",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/unicode-length/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
          "version": "2.1.1",
        },
        "punycode" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "punycode",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/unicode-length/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "strip-ansi",
              "spec": "^3.0.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^2.0.0",
              "to": "node_modules/unicode-length/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/unicode-length/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
          "version": "3.0.1",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/unicode-length/node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/unicode-length/node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uri-js" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
      "version": "4.2.2",
    },
    "uuid" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-processinfo",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.3.3.tgz",
      "version": "3.3.3",
    },
    "validate-npm-package-license" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "vlq" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/flow-remove-types",
          "name": "vlq",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/vlq",
      "name": "vlq",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/vlq",
      "resolved": "https://registry.npmjs.org/vlq/-/vlq-0.2.3.tgz",
      "version": "0.2.3",
    },
    "whatwg-fetch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/isomorphic-fetch",
          "name": "whatwg-fetch",
          "spec": ">=0.10.0",
          "type": "prod",
        },
      },
      "location": "node_modules/whatwg-fetch",
      "name": "whatwg-fetch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/whatwg-fetch",
      "resolved": "https://registry.npmjs.org/whatwg-fetch/-/whatwg-fetch-3.0.0.tgz",
      "version": "3.0.0",
    },
    "which" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "which",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
      },
      "location": "node_modules/which",
      "name": "which",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/which",
      "resolved": "https://registry.npmjs.org/which/-/which-2.0.2.tgz",
      "version": "2.0.2",
    },
    "which-module" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "wordwrap" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/optimist",
          "name": "wordwrap",
          "spec": "~0.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/wordwrap",
      "name": "wordwrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/wordwrap",
      "resolved": "https://registry.npmjs.org/wordwrap/-/wordwrap-0.0.3.tgz",
      "version": "0.0.3",
    },
    "wrappy" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "^1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "typedarray-to-buffer" => EdgeOut {
          "name": "typedarray-to-buffer",
          "spec": "^3.1.5",
          "to": "node_modules/typedarray-to-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-3.0.1.tgz",
      "version": "3.0.1",
    },
    "y18n" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yaml" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-yaml",
          "name": "yaml",
          "spec": "^1.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/runtime" => EdgeOut {
          "name": "@babel/runtime",
          "spec": "^7.6.3",
          "to": "node_modules/@babel/runtime",
          "type": "prod",
        },
      },
      "location": "node_modules/yaml",
      "name": "yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yaml",
      "resolved": "https://registry.npmjs.org/yaml/-/yaml-1.7.2.tgz",
      "version": "1.7.2",
    },
    "yapool" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cliui" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "emoji-regex" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/emoji-regex",
          "name": "emoji-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/emoji-regex",
          "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
          "version": "7.0.3",
        },
        "is-fullwidth-code-point" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/is-fullwidth-code-point",
          "name": "is-fullwidth-code-point",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/is-fullwidth-code-point",
          "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
          "version": "2.0.0",
        },
        "string-width" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/yargs/node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/yargs/node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "dev": true,
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.0.tgz",
      "version": "13.3.0",
    },
    "yargs-parser" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.1.tgz",
      "version": "13.1.1",
    },
    "yn" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
  },
  "edgesOut": Map {
    "react" => EdgeOut {
      "name": "react",
      "spec": "15",
      "to": "node_modules/react",
      "type": "prod",
    },
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "^14.10.5",
      "to": "node_modules/tap",
      "type": "dev",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps",
  "packageName": "tap-14.7-react-15-peerdep-collision",
  "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-bundling-node-without-updating-all-of-its-deps",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP update a child of a node with bundled deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps" => ArboristNode {
      "bundleDependencies": Array [
        "@isaacs/testing-bundledeps-a",
      ],
      "children": Map {
        "@isaacs/testing-bundledeps-a" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "name": "@isaacs/testing-bundledeps-a",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-b" => ArboristNode {
          "bundled": true,
          "bundler": "node_modules/@isaacs/testing-bundledeps",
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "name": "@isaacs/testing-bundledeps-b",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
          "version": "1.0.0",
        },
        "@isaacs/testing-bundledeps-c" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps",
              "name": "@isaacs/testing-bundledeps-c",
              "spec": "*",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-b" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-b",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
          "name": "@isaacs/testing-bundledeps-c",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
          "version": "2.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps-a" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-a",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
          "type": "prod",
        },
        "@isaacs/testing-bundledeps-c" => EdgeOut {
          "name": "@isaacs/testing-bundledeps-c",
          "spec": "*",
          "to": "node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps",
      "name": "@isaacs/testing-bundledeps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps/node_modules/@isaacs/testing-bundledeps",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps" => EdgeOut {
      "name": "@isaacs/testing-bundledeps",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps",
  "packageName": "testing-bundledeps",
  "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-child-of-a-node-with-bundled-deps",
  "version": "1.2.3",
}
`

exports[`test/arborist/reify.js TAP update a node without updating a child that has bundle deps > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@isaacs/testing-bundledeps-parent" => ArboristNode {
      "children": Map {
        "@isaacs/testing-bundledeps" => ArboristNode {
          "bundleDependencies": Array [
            "@isaacs/testing-bundledeps-a",
          ],
          "children": Map {
            "@isaacs/testing-bundledeps-a" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-a",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "name": "@isaacs/testing-bundledeps-a",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-a/-/testing-bundledeps-a-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-b" => ArboristNode {
              "bundled": true,
              "bundler": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "name": "@isaacs/testing-bundledeps-b",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-b/-/testing-bundledeps-b-1.0.0.tgz",
              "version": "1.0.0",
            },
            "@isaacs/testing-bundledeps-c" => ArboristNode {
              "edgesIn": Set {
                EdgeIn {
                  "from": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
                  "name": "@isaacs/testing-bundledeps-c",
                  "spec": "*",
                  "type": "prod",
                },
              },
              "edgesOut": Map {
                "@isaacs/testing-bundledeps-b" => EdgeOut {
                  "name": "@isaacs/testing-bundledeps-b",
                  "spec": "*",
                  "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-b",
                  "type": "prod",
                },
              },
              "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "name": "@isaacs/testing-bundledeps-c",
              "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-c/-/testing-bundledeps-c-2.0.0.tgz",
              "version": "2.0.0",
            },
          },
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@isaacs/testing-bundledeps-parent",
              "name": "@isaacs/testing-bundledeps",
              "spec": "^1.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "@isaacs/testing-bundledeps-a" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-a",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-a",
              "type": "prod",
            },
            "@isaacs/testing-bundledeps-c" => EdgeOut {
              "name": "@isaacs/testing-bundledeps-c",
              "spec": "*",
              "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps/node_modules/@isaacs/testing-bundledeps-c",
              "type": "prod",
            },
          },
          "location": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "name": "@isaacs/testing-bundledeps",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps/node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps/-/testing-bundledeps-1.0.0.tgz",
          "version": "1.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "@isaacs/testing-bundledeps-parent",
          "spec": "*",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@isaacs/testing-bundledeps" => EdgeOut {
          "name": "@isaacs/testing-bundledeps",
          "spec": "^1.0.0",
          "to": "node_modules/@isaacs/testing-bundledeps-parent/node_modules/@isaacs/testing-bundledeps",
          "type": "prod",
        },
      },
      "location": "node_modules/@isaacs/testing-bundledeps-parent",
      "name": "@isaacs/testing-bundledeps-parent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps/node_modules/@isaacs/testing-bundledeps-parent",
      "resolved": "https://registry.npmjs.org/@isaacs/testing-bundledeps-parent/-/testing-bundledeps-parent-2.0.0.tgz",
      "version": "2.0.0",
    },
  },
  "edgesOut": Map {
    "@isaacs/testing-bundledeps-parent" => EdgeOut {
      "name": "@isaacs/testing-bundledeps-parent",
      "spec": "*",
      "to": "node_modules/@isaacs/testing-bundledeps-parent",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps",
  "packageName": "testing-bundledeps-3",
  "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-a-child-that-has-bundle-deps",
  "version": "1.0.0",
}
`

exports[`test/arborist/reify.js TAP update a node without updating its children > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "once" => ArboristNode {
      "children": Map {
        "wrappy" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/once",
              "name": "wrappy",
              "spec": "1",
              "type": "prod",
            },
          },
          "location": "node_modules/once/node_modules/wrappy",
          "name": "wrappy",
          "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-its-children/node_modules/once/node_modules/wrappy",
          "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
          "version": "1.0.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "once",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/once/node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-its-children/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
  },
  "edgesOut": Map {
    "once" => EdgeOut {
      "name": "once",
      "spec": "^1.3.1",
      "to": "node_modules/once",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-update-a-node-without-updating-its-children",
  "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-node-without-updating-its-children",
}
`

exports[`test/arborist/reify.js TAP update a yarn.lock file > add abbrev 1`] = `
ArboristNode {
  "children": Map {
    "abbrev" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-yarn.lock-file/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "mkdirp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "mkdirp",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/mkdirp",
      "name": "mkdirp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-yarn.lock-file/node_modules/mkdirp",
      "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.2.tgz",
      "version": "1.0.2",
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.1.1",
      "to": "node_modules/abbrev",
      "type": "prod",
    },
    "mkdirp" => EdgeOut {
      "name": "mkdirp",
      "spec": "^1.0.2",
      "to": "node_modules/mkdirp",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-update-a-yarn.lock-file",
  "path": "{CWD}/test/arborist/tap-testdir-reify-update-a-yarn.lock-file",
}
`

exports[`test/arborist/reify.js TAP update a yarn.lock file > updated yarn lock 1`] = `
# THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY.
# yarn lockfile v1


abbrev@^1.1.1:
  version "1.1.1"
  resolved "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz"
  integrity sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==

mkdirp@^1.0.2:
  version "1.0.2"
  resolved "https://registry.npmjs.org/mkdirp/-/mkdirp-1.0.2.tgz"
  integrity sha512-N2REVrJ/X/jGPfit2d7zea2J1pf7EAR5chIUcfHffAZ7gmlam5U65sAm76+o4ntQbSRdTjYf7qZz3chuHlwXEA==

`

exports[`test/arborist/reify.js TAP warn and correct if damaged data in lockfile first pass logs > "fixed" lockfile 1`] = `
{
  "name": "tap-testdir-reify-warn-and-correct-if-damaged-data-in-lockfile",
  "lockfileVersion": 2,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "abbrev": ""
      }
    }
  }
}

`

exports[`test/arborist/reify.js TAP warn and correct if damaged data in lockfile second pass just does the right thing > actually fixed lockfile 1`] = `
{
  "name": "tap-testdir-reify-warn-and-correct-if-damaged-data-in-lockfile",
  "lockfileVersion": 2,
  "requires": true,
  "packages": {
    "": {
      "dependencies": {
        "abbrev": ""
      }
    },
    "node_modules/abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  },
  "dependencies": {
    "abbrev": {
      "version": "1.1.1",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q=="
    }
  }
}

`

exports[`test/arborist/reify.js TAP weirdly broken lockfile without resolved value > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "@babel/code-frame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/code-frame",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/code-frame",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/highlight" => EdgeOut {
          "name": "@babel/highlight",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/highlight",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/code-frame",
      "name": "@babel/code-frame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/code-frame",
      "resolved": "https://registry.npmjs.org/@babel/code-frame/-/code-frame-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/generator" => ArboristNode {
      "children": Map {
        "source-map" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/@babel/generator",
              "name": "source-map",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "location": "node_modules/@babel/generator/node_modules/source-map",
          "name": "source-map",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/generator/node_modules/source-map",
          "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.5.7.tgz",
          "version": "0.5.7",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/generator",
          "spec": "^7.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "jsesc" => EdgeOut {
          "name": "jsesc",
          "spec": "^2.5.1",
          "to": "node_modules/jsesc",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.5.0",
          "to": "node_modules/@babel/generator/node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/generator",
      "name": "@babel/generator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/generator",
      "resolved": "https://registry.npmjs.org/@babel/generator/-/generator-7.9.4.tgz",
      "version": "7.9.4",
    },
    "@babel/helper-function-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-function-name",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-get-function-arity" => EdgeOut {
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/helper-get-function-arity",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-function-name",
      "name": "@babel/helper-function-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/helper-function-name",
      "resolved": "https://registry.npmjs.org/@babel/helper-function-name/-/helper-function-name-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/helper-get-function-arity" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/helper-get-function-arity",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-get-function-arity",
      "name": "@babel/helper-get-function-arity",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/helper-get-function-arity",
      "resolved": "https://registry.npmjs.org/@babel/helper-get-function-arity/-/helper-get-function-arity-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/helper-split-export-declaration" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-split-export-declaration",
      "name": "@babel/helper-split-export-declaration",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/helper-split-export-declaration",
      "resolved": "https://registry.npmjs.org/@babel/helper-split-export-declaration/-/helper-split-export-declaration-7.8.3.tgz",
      "version": "7.8.3",
    },
    "@babel/helper-validator-identifier" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/helper-validator-identifier",
      "name": "@babel/helper-validator-identifier",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/helper-validator-identifier",
      "resolved": "https://registry.npmjs.org/@babel/helper-validator-identifier/-/helper-validator-identifier-7.9.0.tgz",
      "version": "7.9.0",
    },
    "@babel/highlight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/code-frame",
          "name": "@babel/highlight",
          "spec": "^7.8.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "chalk" => EdgeOut {
          "name": "chalk",
          "spec": "^2.0.0",
          "to": "node_modules/chalk",
          "type": "prod",
        },
        "js-tokens" => EdgeOut {
          "name": "js-tokens",
          "spec": "^4.0.0",
          "to": "node_modules/js-tokens",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/highlight",
      "name": "@babel/highlight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/highlight",
      "resolved": "https://registry.npmjs.org/@babel/highlight/-/highlight-7.9.0.tgz",
      "version": "7.9.0",
    },
    "@babel/parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/parser",
          "spec": "^7.8.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/parser",
          "spec": "^7.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/parser",
      "name": "@babel/parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/parser",
      "resolved": "https://registry.npmjs.org/@babel/parser/-/parser-7.9.4.tgz",
      "version": "7.9.4",
    },
    "@babel/template" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/template",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/template",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.8.6",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.8.6",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/template",
      "name": "@babel/template",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/template",
      "resolved": "https://registry.npmjs.org/@babel/template/-/template-7.8.6.tgz",
      "version": "7.8.6",
    },
    "@babel/traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/code-frame" => EdgeOut {
          "name": "@babel/code-frame",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/code-frame",
          "type": "prod",
        },
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/helper-function-name" => EdgeOut {
          "name": "@babel/helper-function-name",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/helper-function-name",
          "type": "prod",
        },
        "@babel/helper-split-export-declaration" => EdgeOut {
          "name": "@babel/helper-split-export-declaration",
          "spec": "^7.8.3",
          "to": "node_modules/@babel/helper-split-export-declaration",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.0",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "globals" => EdgeOut {
          "name": "globals",
          "spec": "^11.1.0",
          "to": "node_modules/globals",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/traverse",
      "name": "@babel/traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/traverse",
      "resolved": "https://registry.npmjs.org/@babel/traverse/-/traverse-7.9.0.tgz",
      "version": "7.9.0",
    },
    "@babel/types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "@babel/types",
          "spec": "^7.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-function-name",
          "name": "@babel/types",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-get-function-arity",
          "name": "@babel/types",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/helper-split-export-declaration",
          "name": "@babel/types",
          "spec": "^7.8.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/template",
          "name": "@babel/types",
          "spec": "^7.8.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "@babel/types",
          "spec": "^7.9.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "@babel/types",
          "spec": "^7.4.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/helper-validator-identifier" => EdgeOut {
          "name": "@babel/helper-validator-identifier",
          "spec": "^7.9.0",
          "to": "node_modules/@babel/helper-validator-identifier",
          "type": "prod",
        },
        "lodash" => EdgeOut {
          "name": "lodash",
          "spec": "^4.17.13",
          "to": "node_modules/lodash",
          "type": "prod",
        },
        "to-fast-properties" => EdgeOut {
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "to": "node_modules/to-fast-properties",
          "type": "prod",
        },
      },
      "location": "node_modules/@babel/types",
      "name": "@babel/types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/@babel/types",
      "resolved": "https://registry.npmjs.org/@babel/types/-/types-7.9.0.tgz",
      "version": "7.9.0",
    },
    "ajv" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "ajv",
          "spec": "^6.5.5",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fast-deep-equal" => EdgeOut {
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "to": "node_modules/fast-deep-equal",
          "type": "prod",
        },
        "fast-json-stable-stringify" => EdgeOut {
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "to": "node_modules/fast-json-stable-stringify",
          "type": "prod",
        },
        "json-schema-traverse" => EdgeOut {
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "to": "node_modules/json-schema-traverse",
          "type": "prod",
        },
        "uri-js" => EdgeOut {
          "name": "uri-js",
          "spec": "^4.2.2",
          "to": "node_modules/uri-js",
          "type": "prod",
        },
      },
      "location": "node_modules/ajv",
      "name": "ajv",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ajv",
      "resolved": "https://registry.npmjs.org/ajv/-/ajv-6.12.0.tgz",
      "version": "6.12.0",
    },
    "ansi-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/strip-ansi",
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-regex",
      "name": "ansi-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ansi-regex",
      "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-2.1.1.tgz",
      "version": "2.1.1",
    },
    "ansi-styles" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "ansi-styles",
          "spec": "^3.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-convert" => EdgeOut {
          "name": "color-convert",
          "spec": "^1.9.0",
          "to": "node_modules/color-convert",
          "type": "prod",
        },
      },
      "location": "node_modules/ansi-styles",
      "name": "ansi-styles",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ansi-styles",
      "resolved": "https://registry.npmjs.org/ansi-styles/-/ansi-styles-3.2.1.tgz",
      "version": "3.2.1",
    },
    "append-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-hook",
          "name": "append-transform",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "default-require-extensions" => EdgeOut {
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "to": "node_modules/default-require-extensions",
          "type": "prod",
        },
      },
      "location": "node_modules/append-transform",
      "name": "append-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/append-transform",
      "resolved": "https://registry.npmjs.org/append-transform/-/append-transform-1.0.0.tgz",
      "version": "1.0.0",
    },
    "archy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "archy",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/archy",
      "name": "archy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/archy",
      "resolved": "https://registry.npmjs.org/archy/-/archy-1.0.0.tgz",
      "version": "1.0.0",
    },
    "arg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "arg",
          "spec": "^4.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/arg",
      "name": "arg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/arg",
      "resolved": "https://registry.npmjs.org/arg/-/arg-4.1.3.tgz",
      "version": "4.1.3",
    },
    "argparse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "argparse",
          "spec": "^1.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "sprintf-js" => EdgeOut {
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "to": "node_modules/sprintf-js",
          "type": "prod",
        },
      },
      "location": "node_modules/argparse",
      "name": "argparse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/argparse",
      "resolved": "https://registry.npmjs.org/argparse/-/argparse-1.0.10.tgz",
      "version": "1.0.10",
    },
    "asn1" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "asn1",
          "spec": "~0.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/asn1",
      "name": "asn1",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/asn1",
      "resolved": "https://registry.npmjs.org/asn1/-/asn1-0.2.4.tgz",
      "version": "0.2.4",
    },
    "assert-plus" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/dashdash",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/getpass",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "assert-plus",
          "spec": "1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "assert-plus",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/assert-plus",
      "name": "assert-plus",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/assert-plus",
      "resolved": "https://registry.npmjs.org/assert-plus/-/assert-plus-1.0.0.tgz",
      "version": "1.0.0",
    },
    "asynckit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "asynckit",
          "spec": "^0.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/asynckit",
      "name": "asynckit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/asynckit",
      "resolved": "https://registry.npmjs.org/asynckit/-/asynckit-0.4.0.tgz",
      "version": "0.4.0",
    },
    "aws-sign2" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws-sign2",
      "name": "aws-sign2",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/aws-sign2",
      "resolved": "https://registry.npmjs.org/aws-sign2/-/aws-sign2-0.7.0.tgz",
      "version": "0.7.0",
    },
    "aws4" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "aws4",
          "spec": "^1.8.0",
          "type": "prod",
        },
      },
      "location": "node_modules/aws4",
      "name": "aws4",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/aws4",
      "resolved": "https://registry.npmjs.org/aws4/-/aws4-1.9.1.tgz",
      "version": "1.9.1",
    },
    "balanced-match" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "balanced-match",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/balanced-match",
      "name": "balanced-match",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/balanced-match",
      "resolved": "https://registry.npmjs.org/balanced-match/-/balanced-match-1.0.0.tgz",
      "version": "1.0.0",
    },
    "bcrypt-pbkdf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/bcrypt-pbkdf",
      "name": "bcrypt-pbkdf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/bcrypt-pbkdf",
      "resolved": "https://registry.npmjs.org/bcrypt-pbkdf/-/bcrypt-pbkdf-1.0.2.tgz",
      "version": "1.0.2",
    },
    "bind-obj-methods" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/bind-obj-methods",
      "name": "bind-obj-methods",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/bind-obj-methods",
      "resolved": "https://registry.npmjs.org/bind-obj-methods/-/bind-obj-methods-2.0.0.tgz",
      "version": "2.0.0",
    },
    "brace-expansion" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/minimatch",
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "balanced-match" => EdgeOut {
          "name": "balanced-match",
          "spec": "^1.0.0",
          "to": "node_modules/balanced-match",
          "type": "prod",
        },
        "concat-map" => EdgeOut {
          "name": "concat-map",
          "spec": "0.0.1",
          "to": "node_modules/concat-map",
          "type": "prod",
        },
      },
      "location": "node_modules/brace-expansion",
      "name": "brace-expansion",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/brace-expansion",
      "resolved": "https://registry.npmjs.org/brace-expansion/-/brace-expansion-1.1.11.tgz",
      "version": "1.1.11",
    },
    "browser-process-hrtime" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/browser-process-hrtime",
      "name": "browser-process-hrtime",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/browser-process-hrtime",
      "resolved": "https://registry.npmjs.org/browser-process-hrtime/-/browser-process-hrtime-1.0.0.tgz",
      "version": "1.0.0",
    },
    "buffer-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "buffer-from",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/buffer-from",
      "name": "buffer-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/buffer-from",
      "resolved": "https://registry.npmjs.org/buffer-from/-/buffer-from-1.1.1.tgz",
      "version": "1.1.1",
    },
    "caching-transform" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "caching-transform",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "package-hash" => EdgeOut {
          "name": "package-hash",
          "spec": "^3.0.0",
          "to": "node_modules/package-hash",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
      },
      "location": "node_modules/caching-transform",
      "name": "caching-transform",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/caching-transform",
      "resolved": "https://registry.npmjs.org/caching-transform/-/caching-transform-3.0.2.tgz",
      "version": "3.0.2",
    },
    "camelcase" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "camelcase",
          "spec": "^5.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/camelcase",
      "name": "camelcase",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/camelcase",
      "resolved": "https://registry.npmjs.org/camelcase/-/camelcase-5.3.1.tgz",
      "version": "5.3.1",
    },
    "capture-stack-trace" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "capture-stack-trace",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/capture-stack-trace",
      "name": "capture-stack-trace",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/capture-stack-trace",
      "resolved": "https://registry.npmjs.org/capture-stack-trace/-/capture-stack-trace-1.0.1.tgz",
      "version": "1.0.1",
    },
    "caseless" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "caseless",
          "spec": "~0.12.0",
          "type": "prod",
        },
      },
      "location": "node_modules/caseless",
      "name": "caseless",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/caseless",
      "resolved": "https://registry.npmjs.org/caseless/-/caseless-0.12.0.tgz",
      "version": "0.12.0",
    },
    "chalk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "chalk",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-styles" => EdgeOut {
          "name": "ansi-styles",
          "spec": "^3.2.1",
          "to": "node_modules/ansi-styles",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^5.3.0",
          "to": "node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/chalk",
      "name": "chalk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/chalk",
      "resolved": "https://registry.npmjs.org/chalk/-/chalk-2.4.2.tgz",
      "version": "2.4.2",
    },
    "clean-yaml-object" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/clean-yaml-object",
      "name": "clean-yaml-object",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/clean-yaml-object",
      "resolved": "https://registry.npmjs.org/clean-yaml-object/-/clean-yaml-object-0.1.0.tgz",
      "version": "0.1.0",
    },
    "color-convert" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ansi-styles",
          "name": "color-convert",
          "spec": "^1.9.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-name" => EdgeOut {
          "name": "color-name",
          "spec": "1.1.3",
          "to": "node_modules/color-name",
          "type": "prod",
        },
      },
      "location": "node_modules/color-convert",
      "name": "color-convert",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/color-convert",
      "resolved": "https://registry.npmjs.org/color-convert/-/color-convert-1.9.3.tgz",
      "version": "1.9.3",
    },
    "color-name" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/color-convert",
          "name": "color-name",
          "spec": "1.1.3",
          "type": "prod",
        },
      },
      "location": "node_modules/color-name",
      "name": "color-name",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/color-name",
      "resolved": "https://registry.npmjs.org/color-name/-/color-name-1.1.3.tgz",
      "version": "1.1.3",
    },
    "color-support" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "color-support",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/color-support",
      "name": "color-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/color-support",
      "resolved": "https://registry.npmjs.org/color-support/-/color-support-1.1.3.tgz",
      "version": "1.1.3",
    },
    "combined-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "combined-stream",
          "spec": "^1.0.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "combined-stream",
          "spec": "~1.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "delayed-stream" => EdgeOut {
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "to": "node_modules/delayed-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/combined-stream",
      "name": "combined-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/combined-stream",
      "resolved": "https://registry.npmjs.org/combined-stream/-/combined-stream-1.0.8.tgz",
      "version": "1.0.8",
    },
    "commondir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "commondir",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/commondir",
      "name": "commondir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/commondir",
      "resolved": "https://registry.npmjs.org/commondir/-/commondir-1.0.1.tgz",
      "version": "1.0.1",
    },
    "concat-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/brace-expansion",
          "name": "concat-map",
          "spec": "0.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/concat-map",
      "name": "concat-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/concat-map",
      "resolved": "https://registry.npmjs.org/concat-map/-/concat-map-0.0.1.tgz",
      "version": "0.0.1",
    },
    "convert-source-map" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/convert-source-map",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/convert-source-map/node_modules/safe-buffer",
          "name": "safe-buffer",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/convert-source-map/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/convert-source-map/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/convert-source-map",
      "name": "convert-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/convert-source-map",
      "resolved": "https://registry.npmjs.org/convert-source-map/-/convert-source-map-1.7.0.tgz",
      "version": "1.7.0",
    },
    "core-util-is" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "core-util-is",
          "spec": "~1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "core-util-is",
          "spec": "1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/core-util-is",
      "name": "core-util-is",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/core-util-is",
      "resolved": "https://registry.npmjs.org/core-util-is/-/core-util-is-1.0.2.tgz",
      "version": "1.0.2",
    },
    "coveralls" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "coveralls",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "lcov-parse" => EdgeOut {
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "to": "node_modules/lcov-parse",
          "type": "prod",
        },
        "log-driver" => EdgeOut {
          "name": "log-driver",
          "spec": "^1.2.7",
          "to": "node_modules/log-driver",
          "type": "prod",
        },
        "minimist" => EdgeOut {
          "name": "minimist",
          "spec": "^1.2.5",
          "to": "node_modules/minimist",
          "type": "prod",
        },
        "request" => EdgeOut {
          "name": "request",
          "spec": "^2.88.0",
          "to": "node_modules/request",
          "type": "prod",
        },
      },
      "location": "node_modules/coveralls",
      "name": "coveralls",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/coveralls",
      "resolved": "https://registry.npmjs.org/coveralls/-/coveralls-3.0.11.tgz",
      "version": "3.0.11",
    },
    "cp-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "cp-file",
          "spec": "^6.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "nested-error-stacks" => EdgeOut {
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "to": "node_modules/nested-error-stacks",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/cp-file",
      "name": "cp-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/cp-file",
      "resolved": "https://registry.npmjs.org/cp-file/-/cp-file-6.2.0.tgz",
      "version": "6.2.0",
    },
    "cross-spawn" => ArboristNode {
      "children": Map {
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/cross-spawn",
              "name": "which",
              "spec": "^1.2.9",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/cross-spawn/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/cross-spawn/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "cross-spawn",
          "spec": "^4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "lru-cache" => EdgeOut {
          "name": "lru-cache",
          "spec": "^4.0.1",
          "to": "node_modules/lru-cache",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.2.9",
          "to": "node_modules/cross-spawn/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/cross-spawn",
      "name": "cross-spawn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/cross-spawn",
      "resolved": "https://registry.npmjs.org/cross-spawn/-/cross-spawn-4.0.2.tgz",
      "version": "4.0.2",
    },
    "dashdash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "dashdash",
          "spec": "^1.12.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/dashdash",
      "name": "dashdash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/dashdash",
      "resolved": "https://registry.npmjs.org/dashdash/-/dashdash-1.14.1.tgz",
      "version": "1.14.1",
    },
    "debug" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "debug",
          "spec": "^4.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "debug",
          "spec": "^4.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ms" => EdgeOut {
          "name": "ms",
          "spec": "^2.1.1",
          "to": "node_modules/ms",
          "type": "prod",
        },
      },
      "location": "node_modules/debug",
      "name": "debug",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/debug",
      "resolved": "https://registry.npmjs.org/debug/-/debug-4.1.1.tgz",
      "version": "4.1.1",
    },
    "decamelize" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs-parser",
          "name": "decamelize",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/decamelize",
      "name": "decamelize",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/decamelize",
      "resolved": "https://registry.npmjs.org/decamelize/-/decamelize-1.2.0.tgz",
      "version": "1.2.0",
    },
    "default-require-extensions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/append-transform",
          "name": "default-require-extensions",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/default-require-extensions",
      "name": "default-require-extensions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/default-require-extensions",
      "resolved": "https://registry.npmjs.org/default-require-extensions/-/default-require-extensions-2.0.0.tgz",
      "version": "2.0.0",
    },
    "delayed-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/combined-stream",
          "name": "delayed-stream",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/delayed-stream",
      "name": "delayed-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/delayed-stream",
      "resolved": "https://registry.npmjs.org/delayed-stream/-/delayed-stream-1.0.0.tgz",
      "version": "1.0.0",
    },
    "diff" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "diff",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/diff",
      "name": "diff",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/diff",
      "resolved": "https://registry.npmjs.org/diff/-/diff-4.0.2.tgz",
      "version": "4.0.2",
    },
    "domain-browser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "domain-browser",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/domain-browser",
      "name": "domain-browser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/domain-browser",
      "resolved": "https://registry.npmjs.org/domain-browser/-/domain-browser-1.2.0.tgz",
      "version": "1.2.0",
    },
    "ecc-jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/ecc-jsbn",
      "name": "ecc-jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ecc-jsbn",
      "resolved": "https://registry.npmjs.org/ecc-jsbn/-/ecc-jsbn-0.1.2.tgz",
      "version": "0.1.2",
    },
    "emoji-regex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "emoji-regex",
          "spec": "^7.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/emoji-regex",
      "name": "emoji-regex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/emoji-regex",
      "resolved": "https://registry.npmjs.org/emoji-regex/-/emoji-regex-7.0.3.tgz",
      "version": "7.0.3",
    },
    "error-ex" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "error-ex",
          "spec": "^1.3.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-arrayish" => EdgeOut {
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "to": "node_modules/is-arrayish",
          "type": "prod",
        },
      },
      "location": "node_modules/error-ex",
      "name": "error-ex",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/error-ex",
      "resolved": "https://registry.npmjs.org/error-ex/-/error-ex-1.3.2.tgz",
      "version": "1.3.2",
    },
    "es6-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/release-zalgo",
          "name": "es6-error",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/es6-error",
      "name": "es6-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/es6-error",
      "resolved": "https://registry.npmjs.org/es6-error/-/es6-error-4.1.1.tgz",
      "version": "4.1.1",
    },
    "escape-string-regexp" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "escape-string-regexp",
          "spec": "^1.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/escape-string-regexp",
      "name": "escape-string-regexp",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/escape-string-regexp",
      "resolved": "https://registry.npmjs.org/escape-string-regexp/-/escape-string-regexp-1.0.5.tgz",
      "version": "1.0.5",
    },
    "esm" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "esm",
          "spec": "^3.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/esm",
      "name": "esm",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/esm",
      "resolved": "https://registry.npmjs.org/esm/-/esm-3.2.25.tgz",
      "version": "3.2.25",
    },
    "esprima" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/js-yaml",
          "name": "esprima",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/esprima",
      "name": "esprima",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/esprima",
      "resolved": "https://registry.npmjs.org/esprima/-/esprima-4.0.1.tgz",
      "version": "4.0.1",
    },
    "events-to-array" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "events-to-array",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/events-to-array",
      "name": "events-to-array",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/events-to-array",
      "resolved": "https://registry.npmjs.org/events-to-array/-/events-to-array-1.1.2.tgz",
      "version": "1.1.2",
    },
    "extend" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "extend",
          "spec": "~3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/extend",
      "name": "extend",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/extend",
      "resolved": "https://registry.npmjs.org/extend/-/extend-3.0.2.tgz",
      "version": "3.0.2",
    },
    "extsprintf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "extsprintf",
          "spec": "1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/verror",
          "name": "extsprintf",
          "spec": "^1.2.0",
          "type": "prod",
        },
      },
      "location": "node_modules/extsprintf",
      "name": "extsprintf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/extsprintf",
      "resolved": "https://registry.npmjs.org/extsprintf/-/extsprintf-1.3.0.tgz",
      "version": "1.3.0",
    },
    "fast-deep-equal" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-deep-equal",
          "spec": "^3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-deep-equal",
      "name": "fast-deep-equal",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/fast-deep-equal",
      "resolved": "https://registry.npmjs.org/fast-deep-equal/-/fast-deep-equal-3.1.1.tgz",
      "version": "3.1.1",
    },
    "fast-json-stable-stringify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "fast-json-stable-stringify",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fast-json-stable-stringify",
      "name": "fast-json-stable-stringify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/fast-json-stable-stringify",
      "resolved": "https://registry.npmjs.org/fast-json-stable-stringify/-/fast-json-stable-stringify-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-cache-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "commondir" => EdgeOut {
          "name": "commondir",
          "spec": "^1.0.1",
          "to": "node_modules/commondir",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.0.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "pkg-dir" => EdgeOut {
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "to": "node_modules/pkg-dir",
          "type": "prod",
        },
      },
      "location": "node_modules/find-cache-dir",
      "name": "find-cache-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/find-cache-dir",
      "resolved": "https://registry.npmjs.org/find-cache-dir/-/find-cache-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "find-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/pkg-dir",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "find-up",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "locate-path" => EdgeOut {
          "name": "locate-path",
          "spec": "^3.0.0",
          "to": "node_modules/locate-path",
          "type": "prod",
        },
      },
      "location": "node_modules/find-up",
      "name": "find-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/find-up",
      "resolved": "https://registry.npmjs.org/find-up/-/find-up-3.0.0.tgz",
      "version": "3.0.0",
    },
    "foreground-child" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "foreground-child",
          "spec": "^1.5.6",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "foreground-child",
          "spec": "^1.3.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cross-spawn" => EdgeOut {
          "name": "cross-spawn",
          "spec": "^4",
          "to": "node_modules/cross-spawn",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/foreground-child",
      "name": "foreground-child",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/foreground-child",
      "resolved": "https://registry.npmjs.org/foreground-child/-/foreground-child-1.5.6.tgz",
      "version": "1.5.6",
    },
    "forever-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "forever-agent",
          "spec": "~0.6.1",
          "type": "prod",
        },
      },
      "location": "node_modules/forever-agent",
      "name": "forever-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/forever-agent",
      "resolved": "https://registry.npmjs.org/forever-agent/-/forever-agent-0.6.1.tgz",
      "version": "0.6.1",
    },
    "form-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "form-data",
          "spec": "~2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asynckit" => EdgeOut {
          "name": "asynckit",
          "spec": "^0.4.0",
          "to": "node_modules/asynckit",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "^1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "^2.1.12",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
      },
      "location": "node_modules/form-data",
      "name": "form-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/form-data",
      "resolved": "https://registry.npmjs.org/form-data/-/form-data-2.3.3.tgz",
      "version": "2.3.3",
    },
    "fs-exists-cached" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs-exists-cached",
      "name": "fs-exists-cached",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/fs-exists-cached",
      "resolved": "https://registry.npmjs.org/fs-exists-cached/-/fs-exists-cached-1.0.0.tgz",
      "version": "1.0.0",
    },
    "fs.realpath" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/fs.realpath",
      "name": "fs.realpath",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/fs.realpath",
      "resolved": "https://registry.npmjs.org/fs.realpath/-/fs.realpath-1.0.0.tgz",
      "version": "1.0.0",
    },
    "function-loop" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "function-loop",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/function-loop",
      "name": "function-loop",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/function-loop",
      "resolved": "https://registry.npmjs.org/function-loop/-/function-loop-1.0.2.tgz",
      "version": "1.0.2",
    },
    "get-caller-file" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/get-caller-file",
      "name": "get-caller-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/get-caller-file",
      "resolved": "https://registry.npmjs.org/get-caller-file/-/get-caller-file-2.0.5.tgz",
      "version": "2.0.5",
    },
    "getpass" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "getpass",
          "spec": "^0.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
      },
      "location": "node_modules/getpass",
      "name": "getpass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/getpass",
      "resolved": "https://registry.npmjs.org/getpass/-/getpass-0.1.7.tgz",
      "version": "0.1.7",
    },
    "glob" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/rimraf",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "glob",
          "spec": "^7.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "glob",
          "spec": "^7.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "fs.realpath" => EdgeOut {
          "name": "fs.realpath",
          "spec": "^1.0.0",
          "to": "node_modules/fs.realpath",
          "type": "prod",
        },
        "inflight" => EdgeOut {
          "name": "inflight",
          "spec": "^1.0.4",
          "to": "node_modules/inflight",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "2",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "path-is-absolute" => EdgeOut {
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "to": "node_modules/path-is-absolute",
          "type": "prod",
        },
      },
      "location": "node_modules/glob",
      "name": "glob",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/glob",
      "resolved": "https://registry.npmjs.org/glob/-/glob-7.1.6.tgz",
      "version": "7.1.6",
    },
    "globals" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "globals",
          "spec": "^11.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/globals",
      "name": "globals",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/globals",
      "resolved": "https://registry.npmjs.org/globals/-/globals-11.12.0.tgz",
      "version": "11.12.0",
    },
    "graceful-fs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "type": "prod",
        },
      },
      "location": "node_modules/graceful-fs",
      "name": "graceful-fs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/graceful-fs",
      "resolved": "https://registry.npmjs.org/graceful-fs/-/graceful-fs-4.2.3.tgz",
      "version": "4.2.3",
    },
    "har-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/har-validator",
          "name": "har-schema",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/har-schema",
      "name": "har-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/har-schema",
      "resolved": "https://registry.npmjs.org/har-schema/-/har-schema-2.0.0.tgz",
      "version": "2.0.0",
    },
    "har-validator" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "har-validator",
          "spec": "~5.1.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ajv" => EdgeOut {
          "name": "ajv",
          "spec": "^6.5.5",
          "to": "node_modules/ajv",
          "type": "prod",
        },
        "har-schema" => EdgeOut {
          "name": "har-schema",
          "spec": "^2.0.0",
          "to": "node_modules/har-schema",
          "type": "prod",
        },
      },
      "location": "node_modules/har-validator",
      "name": "har-validator",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/har-validator",
      "resolved": "https://registry.npmjs.org/har-validator/-/har-validator-5.1.3.tgz",
      "version": "5.1.3",
    },
    "has-flag" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/supports-color",
          "name": "has-flag",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/has-flag",
      "name": "has-flag",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/has-flag",
      "resolved": "https://registry.npmjs.org/has-flag/-/has-flag-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hasha" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "hasha",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "is-stream" => EdgeOut {
          "name": "is-stream",
          "spec": "^1.0.1",
          "to": "node_modules/is-stream",
          "type": "prod",
        },
      },
      "location": "node_modules/hasha",
      "name": "hasha",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/hasha",
      "resolved": "https://registry.npmjs.org/hasha/-/hasha-3.0.0.tgz",
      "version": "3.0.0",
    },
    "hosted-git-info" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/hosted-git-info",
      "name": "hosted-git-info",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/hosted-git-info",
      "resolved": "https://registry.npmjs.org/hosted-git-info/-/hosted-git-info-2.8.8.tgz",
      "version": "2.8.8",
    },
    "html-escaper" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-reports",
          "name": "html-escaper",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/html-escaper",
      "name": "html-escaper",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/html-escaper",
      "resolved": "https://registry.npmjs.org/html-escaper/-/html-escaper-2.0.1.tgz",
      "version": "2.0.1",
    },
    "http-signature" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "http-signature",
          "spec": "~1.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "jsprim" => EdgeOut {
          "name": "jsprim",
          "spec": "^1.2.2",
          "to": "node_modules/jsprim",
          "type": "prod",
        },
        "sshpk" => EdgeOut {
          "name": "sshpk",
          "spec": "^1.7.0",
          "to": "node_modules/sshpk",
          "type": "prod",
        },
      },
      "location": "node_modules/http-signature",
      "name": "http-signature",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/http-signature",
      "resolved": "https://registry.npmjs.org/http-signature/-/http-signature-1.2.0.tgz",
      "version": "1.2.0",
    },
    "imurmurhash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "type": "prod",
        },
      },
      "location": "node_modules/imurmurhash",
      "name": "imurmurhash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/imurmurhash",
      "resolved": "https://registry.npmjs.org/imurmurhash/-/imurmurhash-0.1.4.tgz",
      "version": "0.1.4",
    },
    "inflight" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inflight",
          "spec": "^1.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "once" => EdgeOut {
          "name": "once",
          "spec": "^1.3.0",
          "to": "node_modules/once",
          "type": "prod",
        },
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/inflight",
      "name": "inflight",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/inflight",
      "resolved": "https://registry.npmjs.org/inflight/-/inflight-1.0.6.tgz",
      "version": "1.0.6",
    },
    "inherits" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "inherits",
          "spec": "2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "inherits",
          "spec": "~2.0.3",
          "type": "prod",
        },
      },
      "location": "node_modules/inherits",
      "name": "inherits",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/inherits",
      "resolved": "https://registry.npmjs.org/inherits/-/inherits-2.0.4.tgz",
      "version": "2.0.4",
    },
    "is-arrayish" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/error-ex",
          "name": "is-arrayish",
          "spec": "^0.2.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-arrayish",
      "name": "is-arrayish",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/is-arrayish",
      "resolved": "https://registry.npmjs.org/is-arrayish/-/is-arrayish-0.2.1.tgz",
      "version": "0.2.1",
    },
    "is-fullwidth-code-point" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs/node_modules/string-width",
          "name": "is-fullwidth-code-point",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-fullwidth-code-point",
      "name": "is-fullwidth-code-point",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/is-fullwidth-code-point",
      "resolved": "https://registry.npmjs.org/is-fullwidth-code-point/-/is-fullwidth-code-point-2.0.0.tgz",
      "version": "2.0.0",
    },
    "is-stream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/hasha",
          "name": "is-stream",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/is-stream",
      "name": "is-stream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/is-stream",
      "resolved": "https://registry.npmjs.org/is-stream/-/is-stream-1.1.0.tgz",
      "version": "1.1.0",
    },
    "is-typedarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/is-typedarray",
      "name": "is-typedarray",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/is-typedarray",
      "resolved": "https://registry.npmjs.org/is-typedarray/-/is-typedarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isarray" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "isarray",
          "spec": "~1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isarray",
      "name": "isarray",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/isarray",
      "resolved": "https://registry.npmjs.org/isarray/-/isarray-1.0.0.tgz",
      "version": "1.0.0",
    },
    "isexe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/which",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "isexe",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/isexe",
      "name": "isexe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/isexe",
      "resolved": "https://registry.npmjs.org/isexe/-/isexe-2.0.0.tgz",
      "version": "2.0.0",
    },
    "isstream" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "isstream",
          "spec": "~0.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/isstream",
      "name": "isstream",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/isstream",
      "resolved": "https://registry.npmjs.org/isstream/-/isstream-0.1.2.tgz",
      "version": "0.1.2",
    },
    "istanbul-lib-coverage" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-instrument",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-coverage",
      "name": "istanbul-lib-coverage",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-coverage",
      "resolved": "https://registry.npmjs.org/istanbul-lib-coverage/-/istanbul-lib-coverage-2.0.5.tgz",
      "version": "2.0.5",
    },
    "istanbul-lib-hook" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "append-transform" => EdgeOut {
          "name": "append-transform",
          "spec": "^1.0.0",
          "to": "node_modules/append-transform",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-hook",
      "name": "istanbul-lib-hook",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-hook",
      "resolved": "https://registry.npmjs.org/istanbul-lib-hook/-/istanbul-lib-hook-2.0.7.tgz",
      "version": "2.0.7",
    },
    "istanbul-lib-instrument" => ArboristNode {
      "children": Map {
        "semver" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-instrument",
              "name": "semver",
              "spec": "^6.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "name": "semver",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-instrument/node_modules/semver",
          "resolved": "https://registry.npmjs.org/semver/-/semver-6.3.0.tgz",
          "version": "6.3.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@babel/generator" => EdgeOut {
          "name": "@babel/generator",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/generator",
          "type": "prod",
        },
        "@babel/parser" => EdgeOut {
          "name": "@babel/parser",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/parser",
          "type": "prod",
        },
        "@babel/template" => EdgeOut {
          "name": "@babel/template",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/template",
          "type": "prod",
        },
        "@babel/traverse" => EdgeOut {
          "name": "@babel/traverse",
          "spec": "^7.4.3",
          "to": "node_modules/@babel/traverse",
          "type": "prod",
        },
        "@babel/types" => EdgeOut {
          "name": "@babel/types",
          "spec": "^7.4.0",
          "to": "node_modules/@babel/types",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^6.0.0",
          "to": "node_modules/istanbul-lib-instrument/node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-instrument",
      "name": "istanbul-lib-instrument",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-instrument",
      "resolved": "https://registry.npmjs.org/istanbul-lib-instrument/-/istanbul-lib-instrument-3.3.0.tgz",
      "version": "3.3.0",
    },
    "istanbul-lib-report" => ArboristNode {
      "children": Map {
        "supports-color" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/istanbul-lib-report",
              "name": "supports-color",
              "spec": "^6.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "has-flag" => EdgeOut {
              "name": "has-flag",
              "spec": "^3.0.0",
              "to": "node_modules/has-flag",
              "type": "prod",
            },
          },
          "location": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "name": "supports-color",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-report/node_modules/supports-color",
          "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-6.1.0.tgz",
          "version": "6.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "supports-color" => EdgeOut {
          "name": "supports-color",
          "spec": "^6.1.0",
          "to": "node_modules/istanbul-lib-report/node_modules/supports-color",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-report",
      "name": "istanbul-lib-report",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-report",
      "resolved": "https://registry.npmjs.org/istanbul-lib-report/-/istanbul-lib-report-2.0.8.tgz",
      "version": "2.0.8",
    },
    "istanbul-lib-source-maps" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^4.1.1",
          "to": "node_modules/debug",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-lib-source-maps",
      "name": "istanbul-lib-source-maps",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-lib-source-maps",
      "resolved": "https://registry.npmjs.org/istanbul-lib-source-maps/-/istanbul-lib-source-maps-3.0.6.tgz",
      "version": "3.0.6",
    },
    "istanbul-reports" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "html-escaper" => EdgeOut {
          "name": "html-escaper",
          "spec": "^2.0.0",
          "to": "node_modules/html-escaper",
          "type": "prod",
        },
      },
      "location": "node_modules/istanbul-reports",
      "name": "istanbul-reports",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/istanbul-reports",
      "resolved": "https://registry.npmjs.org/istanbul-reports/-/istanbul-reports-2.2.7.tgz",
      "version": "2.2.7",
    },
    "js-tokens" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/highlight",
          "name": "js-tokens",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/js-tokens",
      "name": "js-tokens",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/js-tokens",
      "resolved": "https://registry.npmjs.org/js-tokens/-/js-tokens-4.0.0.tgz",
      "version": "4.0.0",
    },
    "js-yaml" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "js-yaml",
          "spec": "^3.13.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "js-yaml",
          "spec": "^3.3.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "name": "js-yaml",
          "spec": "^3.2.7",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "js-yaml",
          "spec": "^3.2.7",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "argparse" => EdgeOut {
          "name": "argparse",
          "spec": "^1.0.7",
          "to": "node_modules/argparse",
          "type": "prod",
        },
        "esprima" => EdgeOut {
          "name": "esprima",
          "spec": "^4.0.0",
          "to": "node_modules/esprima",
          "type": "prod",
        },
      },
      "location": "node_modules/js-yaml",
      "name": "js-yaml",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/js-yaml",
      "resolved": "https://registry.npmjs.org/js-yaml/-/js-yaml-3.13.1.tgz",
      "version": "3.13.1",
    },
    "jsbn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "jsbn",
          "spec": "~0.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/jsbn",
      "name": "jsbn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/jsbn",
      "resolved": "https://registry.npmjs.org/jsbn/-/jsbn-0.1.1.tgz",
      "version": "0.1.1",
    },
    "jsesc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "jsesc",
          "spec": "^2.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/jsesc",
      "name": "jsesc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/jsesc",
      "resolved": "https://registry.npmjs.org/jsesc/-/jsesc-2.5.2.tgz",
      "version": "2.5.2",
    },
    "json-parse-better-errors" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/parse-json",
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-parse-better-errors",
      "name": "json-parse-better-errors",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/json-parse-better-errors",
      "resolved": "https://registry.npmjs.org/json-parse-better-errors/-/json-parse-better-errors-1.0.2.tgz",
      "version": "1.0.2",
    },
    "json-schema" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "json-schema",
          "spec": "0.2.3",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema",
      "name": "json-schema",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/json-schema",
      "resolved": "https://registry.npmjs.org/json-schema/-/json-schema-0.2.3.tgz",
      "version": "0.2.3",
    },
    "json-schema-traverse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "json-schema-traverse",
          "spec": "^0.4.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-schema-traverse",
      "name": "json-schema-traverse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/json-schema-traverse",
      "resolved": "https://registry.npmjs.org/json-schema-traverse/-/json-schema-traverse-0.4.1.tgz",
      "version": "0.4.1",
    },
    "json-stringify-safe" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/json-stringify-safe",
      "name": "json-stringify-safe",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/json-stringify-safe",
      "resolved": "https://registry.npmjs.org/json-stringify-safe/-/json-stringify-safe-5.0.1.tgz",
      "version": "5.0.1",
    },
    "jsprim" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "jsprim",
          "spec": "^1.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "1.3.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
        "json-schema" => EdgeOut {
          "name": "json-schema",
          "spec": "0.2.3",
          "to": "node_modules/json-schema",
          "type": "prod",
        },
        "verror" => EdgeOut {
          "name": "verror",
          "spec": "1.10.0",
          "to": "node_modules/verror",
          "type": "prod",
        },
      },
      "location": "node_modules/jsprim",
      "name": "jsprim",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/jsprim",
      "resolved": "https://registry.npmjs.org/jsprim/-/jsprim-1.4.1.tgz",
      "version": "1.4.1",
    },
    "lcov-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "lcov-parse",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lcov-parse",
      "name": "lcov-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/lcov-parse",
      "resolved": "https://registry.npmjs.org/lcov-parse/-/lcov-parse-1.0.0.tgz",
      "version": "1.0.0",
    },
    "load-json-file" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/load-json-file",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/load-json-file/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/load-json-file/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "load-json-file",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.2",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "parse-json" => EdgeOut {
          "name": "parse-json",
          "spec": "^4.0.0",
          "to": "node_modules/parse-json",
          "type": "prod",
        },
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/load-json-file/node_modules/pify",
          "type": "prod",
        },
        "strip-bom" => EdgeOut {
          "name": "strip-bom",
          "spec": "^3.0.0",
          "to": "node_modules/strip-bom",
          "type": "prod",
        },
      },
      "location": "node_modules/load-json-file",
      "name": "load-json-file",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/load-json-file",
      "resolved": "https://registry.npmjs.org/load-json-file/-/load-json-file-4.0.0.tgz",
      "version": "4.0.0",
    },
    "locate-path" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-up",
          "name": "locate-path",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-locate" => EdgeOut {
          "name": "p-locate",
          "spec": "^3.0.0",
          "to": "node_modules/p-locate",
          "type": "prod",
        },
        "path-exists" => EdgeOut {
          "name": "path-exists",
          "spec": "^3.0.0",
          "to": "node_modules/path-exists",
          "type": "prod",
        },
      },
      "location": "node_modules/locate-path",
      "name": "locate-path",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/locate-path",
      "resolved": "https://registry.npmjs.org/locate-path/-/locate-path-3.0.0.tgz",
      "version": "3.0.0",
    },
    "lodash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/generator",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/traverse",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "lodash",
          "spec": "^4.17.13",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash",
      "name": "lodash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/lodash",
      "resolved": "https://registry.npmjs.org/lodash/-/lodash-4.17.15.tgz",
      "version": "4.17.15",
    },
    "lodash.flattendeep" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "type": "prod",
        },
      },
      "location": "node_modules/lodash.flattendeep",
      "name": "lodash.flattendeep",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/lodash.flattendeep",
      "resolved": "https://registry.npmjs.org/lodash.flattendeep/-/lodash.flattendeep-4.4.0.tgz",
      "version": "4.4.0",
    },
    "log-driver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "log-driver",
          "spec": "^1.2.7",
          "type": "prod",
        },
      },
      "location": "node_modules/log-driver",
      "name": "log-driver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/log-driver",
      "resolved": "https://registry.npmjs.org/log-driver/-/log-driver-1.2.7.tgz",
      "version": "1.2.7",
    },
    "lru-cache" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cross-spawn",
          "name": "lru-cache",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pseudomap" => EdgeOut {
          "name": "pseudomap",
          "spec": "^1.0.2",
          "to": "node_modules/pseudomap",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^2.1.2",
          "to": "node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/lru-cache",
      "name": "lru-cache",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/lru-cache",
      "resolved": "https://registry.npmjs.org/lru-cache/-/lru-cache-4.1.5.tgz",
      "version": "4.1.5",
    },
    "make-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "make-dir",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-report",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "make-dir",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^4.0.1",
          "to": "node_modules/pify",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "^5.6.0",
          "to": "node_modules/semver",
          "type": "prod",
        },
      },
      "location": "node_modules/make-dir",
      "name": "make-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/make-dir",
      "resolved": "https://registry.npmjs.org/make-dir/-/make-dir-2.1.0.tgz",
      "version": "2.1.0",
    },
    "make-error" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "make-error",
          "spec": "^1.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/make-error",
      "name": "make-error",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/make-error",
      "resolved": "https://registry.npmjs.org/make-error/-/make-error-1.3.6.tgz",
      "version": "1.3.6",
    },
    "merge-source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.1",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/merge-source-map",
      "name": "merge-source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/merge-source-map",
      "resolved": "https://registry.npmjs.org/merge-source-map/-/merge-source-map-1.1.0.tgz",
      "version": "1.1.0",
    },
    "mime-db" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/mime-types",
          "name": "mime-db",
          "spec": "1.43.0",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-db",
      "name": "mime-db",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/mime-db",
      "resolved": "https://registry.npmjs.org/mime-db/-/mime-db-1.43.0.tgz",
      "version": "1.43.0",
    },
    "mime-types" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/form-data",
          "name": "mime-types",
          "spec": "^2.1.12",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "mime-types",
          "spec": "~2.1.19",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "mime-db" => EdgeOut {
          "name": "mime-db",
          "spec": "1.43.0",
          "to": "node_modules/mime-db",
          "type": "prod",
        },
      },
      "location": "node_modules/mime-types",
      "name": "mime-types",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/mime-types",
      "resolved": "https://registry.npmjs.org/mime-types/-/mime-types-2.1.26.tgz",
      "version": "2.1.26",
    },
    "minimatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "minimatch",
          "spec": "^3.0.4",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "brace-expansion" => EdgeOut {
          "name": "brace-expansion",
          "spec": "^1.1.7",
          "to": "node_modules/brace-expansion",
          "type": "prod",
        },
      },
      "location": "node_modules/minimatch",
      "name": "minimatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/minimatch",
      "resolved": "https://registry.npmjs.org/minimatch/-/minimatch-3.0.4.tgz",
      "version": "3.0.4",
    },
    "minimist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap/node_modules/mkdirp",
          "name": "minimist",
          "spec": "^1.2.5",
          "type": "prod",
        },
      },
      "location": "node_modules/minimist",
      "name": "minimist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/minimist",
      "resolved": "https://registry.npmjs.org/minimist/-/minimist-1.2.5.tgz",
      "version": "1.2.5",
    },
    "minipass" => ArboristNode {
      "children": Map {
        "yallist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/minipass",
              "name": "yallist",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/minipass/node_modules/yallist",
          "name": "yallist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/minipass/node_modules/yallist",
          "resolved": "https://registry.npmjs.org/yallist/-/yallist-3.1.1.tgz",
          "version": "3.1.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "minipass",
          "spec": "^2.3.5",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap-parser",
          "name": "minipass",
          "spec": "^2.2.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "yallist" => EdgeOut {
          "name": "yallist",
          "spec": "^3.0.0",
          "to": "node_modules/minipass/node_modules/yallist",
          "type": "prod",
        },
      },
      "location": "node_modules/minipass",
      "name": "minipass",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/minipass",
      "resolved": "https://registry.npmjs.org/minipass/-/minipass-2.9.0.tgz",
      "version": "2.9.0",
    },
    "ms" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/debug",
          "name": "ms",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/ms",
      "name": "ms",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ms",
      "resolved": "https://registry.npmjs.org/ms/-/ms-2.1.2.tgz",
      "version": "2.1.2",
    },
    "nested-error-stacks" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "nested-error-stacks",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/nested-error-stacks",
      "name": "nested-error-stacks",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/nested-error-stacks",
      "resolved": "https://registry.npmjs.org/nested-error-stacks/-/nested-error-stacks-2.1.0.tgz",
      "version": "2.1.0",
    },
    "normalize-package-data" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "hosted-git-info" => EdgeOut {
          "name": "hosted-git-info",
          "spec": "^2.1.4",
          "to": "node_modules/hosted-git-info",
          "type": "prod",
        },
        "resolve" => EdgeOut {
          "name": "resolve",
          "spec": "^1.10.0",
          "to": "node_modules/resolve",
          "type": "prod",
        },
        "semver" => EdgeOut {
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "to": "node_modules/semver",
          "type": "prod",
        },
        "validate-npm-package-license" => EdgeOut {
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "to": "node_modules/validate-npm-package-license",
          "type": "prod",
        },
      },
      "location": "node_modules/normalize-package-data",
      "name": "normalize-package-data",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/normalize-package-data",
      "resolved": "https://registry.npmjs.org/normalize-package-data/-/normalize-package-data-2.5.0.tgz",
      "version": "2.5.0",
    },
    "nyc" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "nyc",
          "spec": "^14.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "archy" => EdgeOut {
          "name": "archy",
          "spec": "^1.0.0",
          "to": "node_modules/archy",
          "type": "prod",
        },
        "caching-transform" => EdgeOut {
          "name": "caching-transform",
          "spec": "^3.0.2",
          "to": "node_modules/caching-transform",
          "type": "prod",
        },
        "convert-source-map" => EdgeOut {
          "name": "convert-source-map",
          "spec": "^1.6.0",
          "to": "node_modules/convert-source-map",
          "type": "prod",
        },
        "cp-file" => EdgeOut {
          "name": "cp-file",
          "spec": "^6.2.0",
          "to": "node_modules/cp-file",
          "type": "prod",
        },
        "find-cache-dir" => EdgeOut {
          "name": "find-cache-dir",
          "spec": "^2.1.0",
          "to": "node_modules/find-cache-dir",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "istanbul-lib-coverage" => EdgeOut {
          "name": "istanbul-lib-coverage",
          "spec": "^2.0.5",
          "to": "node_modules/istanbul-lib-coverage",
          "type": "prod",
        },
        "istanbul-lib-hook" => EdgeOut {
          "name": "istanbul-lib-hook",
          "spec": "^2.0.7",
          "to": "node_modules/istanbul-lib-hook",
          "type": "prod",
        },
        "istanbul-lib-instrument" => EdgeOut {
          "name": "istanbul-lib-instrument",
          "spec": "^3.3.0",
          "to": "node_modules/istanbul-lib-instrument",
          "type": "prod",
        },
        "istanbul-lib-report" => EdgeOut {
          "name": "istanbul-lib-report",
          "spec": "^2.0.8",
          "to": "node_modules/istanbul-lib-report",
          "type": "prod",
        },
        "istanbul-lib-source-maps" => EdgeOut {
          "name": "istanbul-lib-source-maps",
          "spec": "^3.0.6",
          "to": "node_modules/istanbul-lib-source-maps",
          "type": "prod",
        },
        "istanbul-reports" => EdgeOut {
          "name": "istanbul-reports",
          "spec": "^2.2.4",
          "to": "node_modules/istanbul-reports",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "make-dir" => EdgeOut {
          "name": "make-dir",
          "spec": "^2.1.0",
          "to": "node_modules/make-dir",
          "type": "prod",
        },
        "merge-source-map" => EdgeOut {
          "name": "merge-source-map",
          "spec": "^1.1.0",
          "to": "node_modules/merge-source-map",
          "type": "prod",
        },
        "resolve-from" => EdgeOut {
          "name": "resolve-from",
          "spec": "^4.0.0",
          "to": "node_modules/resolve-from",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "spawn-wrap" => EdgeOut {
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "to": "node_modules/spawn-wrap",
          "type": "prod",
        },
        "test-exclude" => EdgeOut {
          "name": "test-exclude",
          "spec": "^5.2.3",
          "to": "node_modules/test-exclude",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
        "yargs" => EdgeOut {
          "name": "yargs",
          "spec": "^13.2.2",
          "to": "node_modules/yargs",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/nyc",
      "name": "nyc",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/nyc",
      "resolved": "https://registry.npmjs.org/nyc/-/nyc-14.1.1.tgz",
      "version": "14.1.1",
    },
    "oauth-sign" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "type": "prod",
        },
      },
      "location": "node_modules/oauth-sign",
      "name": "oauth-sign",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/oauth-sign",
      "resolved": "https://registry.npmjs.org/oauth-sign/-/oauth-sign-0.9.0.tgz",
      "version": "0.9.0",
    },
    "once" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "once",
          "spec": "^1.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "wrappy" => EdgeOut {
          "name": "wrappy",
          "spec": "1",
          "to": "node_modules/wrappy",
          "type": "prod",
        },
      },
      "location": "node_modules/once",
      "name": "once",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/once",
      "resolved": "https://registry.npmjs.org/once/-/once-1.4.0.tgz",
      "version": "1.4.0",
    },
    "opener" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "opener",
          "spec": "^1.5.1",
          "type": "prod",
        },
      },
      "location": "node_modules/opener",
      "name": "opener",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/opener",
      "resolved": "https://registry.npmjs.org/opener/-/opener-1.5.1.tgz",
      "version": "1.5.1",
    },
    "os-homedir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "os-homedir",
          "spec": "^1.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "os-homedir",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/os-homedir",
      "name": "os-homedir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/os-homedir",
      "resolved": "https://registry.npmjs.org/os-homedir/-/os-homedir-1.0.2.tgz",
      "version": "1.0.2",
    },
    "own-or" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/own-or-env",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or",
      "name": "own-or",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/own-or",
      "resolved": "https://registry.npmjs.org/own-or/-/own-or-1.0.0.tgz",
      "version": "1.0.0",
    },
    "own-or-env" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "own-or-env",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
      },
      "location": "node_modules/own-or-env",
      "name": "own-or-env",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/own-or-env",
      "resolved": "https://registry.npmjs.org/own-or-env/-/own-or-env-1.0.1.tgz",
      "version": "1.0.1",
    },
    "p-limit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-locate",
          "name": "p-limit",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-try" => EdgeOut {
          "name": "p-try",
          "spec": "^2.0.0",
          "to": "node_modules/p-try",
          "type": "prod",
        },
      },
      "location": "node_modules/p-limit",
      "name": "p-limit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/p-limit",
      "resolved": "https://registry.npmjs.org/p-limit/-/p-limit-2.2.2.tgz",
      "version": "2.2.2",
    },
    "p-locate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "p-locate",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "p-limit" => EdgeOut {
          "name": "p-limit",
          "spec": "^2.0.0",
          "to": "node_modules/p-limit",
          "type": "prod",
        },
      },
      "location": "node_modules/p-locate",
      "name": "p-locate",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/p-locate",
      "resolved": "https://registry.npmjs.org/p-locate/-/p-locate-3.0.0.tgz",
      "version": "3.0.0",
    },
    "p-try" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/p-limit",
          "name": "p-try",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/p-try",
      "name": "p-try",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/p-try",
      "resolved": "https://registry.npmjs.org/p-try/-/p-try-2.2.0.tgz",
      "version": "2.2.0",
    },
    "package-hash" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "package-hash",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.15",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "hasha" => EdgeOut {
          "name": "hasha",
          "spec": "^3.0.0",
          "to": "node_modules/hasha",
          "type": "prod",
        },
        "lodash.flattendeep" => EdgeOut {
          "name": "lodash.flattendeep",
          "spec": "^4.4.0",
          "to": "node_modules/lodash.flattendeep",
          "type": "prod",
        },
        "release-zalgo" => EdgeOut {
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "to": "node_modules/release-zalgo",
          "type": "prod",
        },
      },
      "location": "node_modules/package-hash",
      "name": "package-hash",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/package-hash",
      "resolved": "https://registry.npmjs.org/package-hash/-/package-hash-3.0.0.tgz",
      "version": "3.0.0",
    },
    "parse-json" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "parse-json",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "error-ex" => EdgeOut {
          "name": "error-ex",
          "spec": "^1.3.1",
          "to": "node_modules/error-ex",
          "type": "prod",
        },
        "json-parse-better-errors" => EdgeOut {
          "name": "json-parse-better-errors",
          "spec": "^1.0.1",
          "to": "node_modules/json-parse-better-errors",
          "type": "prod",
        },
      },
      "location": "node_modules/parse-json",
      "name": "parse-json",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/parse-json",
      "resolved": "https://registry.npmjs.org/parse-json/-/parse-json-4.0.0.tgz",
      "version": "4.0.0",
    },
    "path-exists" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/locate-path",
          "name": "path-exists",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-exists",
      "name": "path-exists",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/path-exists",
      "resolved": "https://registry.npmjs.org/path-exists/-/path-exists-3.0.0.tgz",
      "version": "3.0.0",
    },
    "path-is-absolute" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/glob",
          "name": "path-is-absolute",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/path-is-absolute",
      "name": "path-is-absolute",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/path-is-absolute",
      "resolved": "https://registry.npmjs.org/path-is-absolute/-/path-is-absolute-1.0.1.tgz",
      "version": "1.0.1",
    },
    "path-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/resolve",
          "name": "path-parse",
          "spec": "^1.0.6",
          "type": "prod",
        },
      },
      "location": "node_modules/path-parse",
      "name": "path-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/path-parse",
      "resolved": "https://registry.npmjs.org/path-parse/-/path-parse-1.0.6.tgz",
      "version": "1.0.6",
    },
    "path-type" => ArboristNode {
      "children": Map {
        "pify" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/path-type",
              "name": "pify",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/path-type/node_modules/pify",
          "name": "pify",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/path-type/node_modules/pify",
          "resolved": "https://registry.npmjs.org/pify/-/pify-3.0.0.tgz",
          "version": "3.0.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg",
          "name": "path-type",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "pify" => EdgeOut {
          "name": "pify",
          "spec": "^3.0.0",
          "to": "node_modules/path-type/node_modules/pify",
          "type": "prod",
        },
      },
      "location": "node_modules/path-type",
      "name": "path-type",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/path-type",
      "resolved": "https://registry.npmjs.org/path-type/-/path-type-3.0.0.tgz",
      "version": "3.0.0",
    },
    "performance-now" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "performance-now",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/performance-now",
      "name": "performance-now",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/performance-now",
      "resolved": "https://registry.npmjs.org/performance-now/-/performance-now-2.1.0.tgz",
      "version": "2.1.0",
    },
    "pify" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "pify",
          "spec": "^4.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/pify",
      "name": "pify",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/pify",
      "resolved": "https://registry.npmjs.org/pify/-/pify-4.0.1.tgz",
      "version": "4.0.1",
    },
    "pkg-dir" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/find-cache-dir",
          "name": "pkg-dir",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
      },
      "location": "node_modules/pkg-dir",
      "name": "pkg-dir",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/pkg-dir",
      "resolved": "https://registry.npmjs.org/pkg-dir/-/pkg-dir-3.0.0.tgz",
      "version": "3.0.0",
    },
    "process-nextick-args" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/process-nextick-args",
      "name": "process-nextick-args",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/process-nextick-args",
      "resolved": "https://registry.npmjs.org/process-nextick-args/-/process-nextick-args-2.0.1.tgz",
      "version": "2.0.1",
    },
    "pseudomap" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "pseudomap",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/pseudomap",
      "name": "pseudomap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/pseudomap",
      "resolved": "https://registry.npmjs.org/pseudomap/-/pseudomap-1.0.2.tgz",
      "version": "1.0.2",
    },
    "psl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "psl",
          "spec": "^1.1.28",
          "type": "prod",
        },
      },
      "location": "node_modules/psl",
      "name": "psl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/psl",
      "resolved": "https://registry.npmjs.org/psl/-/psl-1.8.0.tgz",
      "version": "1.8.0",
    },
    "punycode" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tough-cookie",
          "name": "punycode",
          "spec": "^2.1.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/uri-js",
          "name": "punycode",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/punycode",
      "name": "punycode",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/punycode",
      "resolved": "https://registry.npmjs.org/punycode/-/punycode-2.1.1.tgz",
      "version": "2.1.1",
    },
    "qs" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "qs",
          "spec": "~6.5.2",
          "type": "prod",
        },
      },
      "location": "node_modules/qs",
      "name": "qs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/qs",
      "resolved": "https://registry.npmjs.org/qs/-/qs-6.5.2.tgz",
      "version": "6.5.2",
    },
    "read-pkg" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/read-pkg-up",
          "name": "read-pkg",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "load-json-file" => EdgeOut {
          "name": "load-json-file",
          "spec": "^4.0.0",
          "to": "node_modules/load-json-file",
          "type": "prod",
        },
        "normalize-package-data" => EdgeOut {
          "name": "normalize-package-data",
          "spec": "^2.3.2",
          "to": "node_modules/normalize-package-data",
          "type": "prod",
        },
        "path-type" => EdgeOut {
          "name": "path-type",
          "spec": "^3.0.0",
          "to": "node_modules/path-type",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg",
      "name": "read-pkg",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/read-pkg",
      "resolved": "https://registry.npmjs.org/read-pkg/-/read-pkg-3.0.0.tgz",
      "version": "3.0.0",
    },
    "read-pkg-up" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "read-pkg" => EdgeOut {
          "name": "read-pkg",
          "spec": "^3.0.0",
          "to": "node_modules/read-pkg",
          "type": "prod",
        },
      },
      "location": "node_modules/read-pkg-up",
      "name": "read-pkg-up",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/read-pkg-up",
      "resolved": "https://registry.npmjs.org/read-pkg-up/-/read-pkg-up-4.0.0.tgz",
      "version": "4.0.0",
    },
    "readable-stream" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/readable-stream",
              "name": "safe-buffer",
              "spec": "~5.1.1",
              "type": "prod",
            },
          },
          "location": "node_modules/readable-stream/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/readable-stream/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "readable-stream",
          "spec": "^2.1.5",
          "type": "optional",
        },
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "name": "readable-stream",
          "spec": "^2",
          "type": "optional",
        },
      },
      "edgesOut": Map {
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "~1.0.0",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "inherits" => EdgeOut {
          "name": "inherits",
          "spec": "~2.0.3",
          "to": "node_modules/inherits",
          "type": "prod",
        },
        "isarray" => EdgeOut {
          "name": "isarray",
          "spec": "~1.0.0",
          "to": "node_modules/isarray",
          "type": "prod",
        },
        "process-nextick-args" => EdgeOut {
          "name": "process-nextick-args",
          "spec": "~2.0.0",
          "to": "node_modules/process-nextick-args",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.1",
          "to": "node_modules/readable-stream/node_modules/safe-buffer",
          "type": "prod",
        },
        "string_decoder" => EdgeOut {
          "name": "string_decoder",
          "spec": "~1.1.1",
          "to": "node_modules/string_decoder",
          "type": "prod",
        },
        "util-deprecate" => EdgeOut {
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "to": "node_modules/util-deprecate",
          "type": "prod",
        },
      },
      "location": "node_modules/readable-stream",
      "name": "readable-stream",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/readable-stream",
      "resolved": "https://registry.npmjs.org/readable-stream/-/readable-stream-2.3.7.tgz",
      "version": "2.3.7",
    },
    "release-zalgo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/package-hash",
          "name": "release-zalgo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "es6-error" => EdgeOut {
          "name": "es6-error",
          "spec": "^4.0.1",
          "to": "node_modules/es6-error",
          "type": "prod",
        },
      },
      "location": "node_modules/release-zalgo",
      "name": "release-zalgo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/release-zalgo",
      "resolved": "https://registry.npmjs.org/release-zalgo/-/release-zalgo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "request" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/coveralls",
          "name": "request",
          "spec": "^2.88.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "aws-sign2" => EdgeOut {
          "name": "aws-sign2",
          "spec": "~0.7.0",
          "to": "node_modules/aws-sign2",
          "type": "prod",
        },
        "aws4" => EdgeOut {
          "name": "aws4",
          "spec": "^1.8.0",
          "to": "node_modules/aws4",
          "type": "prod",
        },
        "caseless" => EdgeOut {
          "name": "caseless",
          "spec": "~0.12.0",
          "to": "node_modules/caseless",
          "type": "prod",
        },
        "combined-stream" => EdgeOut {
          "name": "combined-stream",
          "spec": "~1.0.6",
          "to": "node_modules/combined-stream",
          "type": "prod",
        },
        "extend" => EdgeOut {
          "name": "extend",
          "spec": "~3.0.2",
          "to": "node_modules/extend",
          "type": "prod",
        },
        "forever-agent" => EdgeOut {
          "name": "forever-agent",
          "spec": "~0.6.1",
          "to": "node_modules/forever-agent",
          "type": "prod",
        },
        "form-data" => EdgeOut {
          "name": "form-data",
          "spec": "~2.3.2",
          "to": "node_modules/form-data",
          "type": "prod",
        },
        "har-validator" => EdgeOut {
          "name": "har-validator",
          "spec": "~5.1.3",
          "to": "node_modules/har-validator",
          "type": "prod",
        },
        "http-signature" => EdgeOut {
          "name": "http-signature",
          "spec": "~1.2.0",
          "to": "node_modules/http-signature",
          "type": "prod",
        },
        "is-typedarray" => EdgeOut {
          "name": "is-typedarray",
          "spec": "~1.0.0",
          "to": "node_modules/is-typedarray",
          "type": "prod",
        },
        "isstream" => EdgeOut {
          "name": "isstream",
          "spec": "~0.1.2",
          "to": "node_modules/isstream",
          "type": "prod",
        },
        "json-stringify-safe" => EdgeOut {
          "name": "json-stringify-safe",
          "spec": "~5.0.1",
          "to": "node_modules/json-stringify-safe",
          "type": "prod",
        },
        "mime-types" => EdgeOut {
          "name": "mime-types",
          "spec": "~2.1.19",
          "to": "node_modules/mime-types",
          "type": "prod",
        },
        "oauth-sign" => EdgeOut {
          "name": "oauth-sign",
          "spec": "~0.9.0",
          "to": "node_modules/oauth-sign",
          "type": "prod",
        },
        "performance-now" => EdgeOut {
          "name": "performance-now",
          "spec": "^2.1.0",
          "to": "node_modules/performance-now",
          "type": "prod",
        },
        "qs" => EdgeOut {
          "name": "qs",
          "spec": "~6.5.2",
          "to": "node_modules/qs",
          "type": "prod",
        },
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
        "tough-cookie" => EdgeOut {
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "to": "node_modules/tough-cookie",
          "type": "prod",
        },
        "tunnel-agent" => EdgeOut {
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "to": "node_modules/tunnel-agent",
          "type": "prod",
        },
        "uuid" => EdgeOut {
          "name": "uuid",
          "spec": "^3.3.2",
          "to": "node_modules/uuid",
          "type": "prod",
        },
      },
      "location": "node_modules/request",
      "name": "request",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/request",
      "resolved": "https://registry.npmjs.org/request/-/request-2.88.2.tgz",
      "version": "2.88.2",
    },
    "require-directory" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-directory",
          "spec": "^2.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/require-directory",
      "name": "require-directory",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/require-directory",
      "resolved": "https://registry.npmjs.org/require-directory/-/require-directory-2.1.1.tgz",
      "version": "2.1.1",
    },
    "require-main-filename" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/test-exclude",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/require-main-filename",
      "name": "require-main-filename",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/require-main-filename",
      "resolved": "https://registry.npmjs.org/require-main-filename/-/require-main-filename-2.0.0.tgz",
      "version": "2.0.0",
    },
    "resolve" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "resolve",
          "spec": "^1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "path-parse" => EdgeOut {
          "name": "path-parse",
          "spec": "^1.0.6",
          "to": "node_modules/path-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve",
      "name": "resolve",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/resolve",
      "resolved": "https://registry.npmjs.org/resolve/-/resolve-1.15.1.tgz",
      "version": "1.15.1",
    },
    "resolve-from" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "resolve-from",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/resolve-from",
      "name": "resolve-from",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/resolve-from",
      "resolved": "https://registry.npmjs.org/resolve-from/-/resolve-from-4.0.0.tgz",
      "version": "4.0.0",
    },
    "rimraf" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "rimraf",
          "spec": "^2.6.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "rimraf",
          "spec": "^2.6.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
      },
      "location": "node_modules/rimraf",
      "name": "rimraf",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/rimraf",
      "resolved": "https://registry.npmjs.org/rimraf/-/rimraf-2.7.1.tgz",
      "version": "2.7.1",
    },
    "safe-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/cp-file",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/minipass",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "safe-buffer",
          "spec": "^5.1.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tunnel-agent",
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/safe-buffer",
      "name": "safe-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/safe-buffer",
      "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.2.0.tgz",
      "version": "5.2.0",
    },
    "safer-buffer" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/asn1",
          "name": "safer-buffer",
          "spec": "~2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ecc-jsbn",
          "name": "safer-buffer",
          "spec": "^2.1.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/safer-buffer",
      "name": "safer-buffer",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/safer-buffer",
      "resolved": "https://registry.npmjs.org/safer-buffer/-/safer-buffer-2.1.2.tgz",
      "version": "2.1.2",
    },
    "semver" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/make-dir",
          "name": "semver",
          "spec": "^5.6.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "semver",
          "spec": "2 || 3 || 4 || 5",
          "type": "prod",
        },
      },
      "location": "node_modules/semver",
      "name": "semver",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/semver",
      "resolved": "https://registry.npmjs.org/semver/-/semver-5.7.1.tgz",
      "version": "5.7.1",
    },
    "set-blocking" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "set-blocking",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/set-blocking",
      "name": "set-blocking",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/set-blocking",
      "resolved": "https://registry.npmjs.org/set-blocking/-/set-blocking-2.0.0.tgz",
      "version": "2.0.0",
    },
    "signal-exit" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/foreground-child",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spawn-wrap",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "signal-exit",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/write-file-atomic",
          "name": "signal-exit",
          "spec": "^3.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/signal-exit",
      "name": "signal-exit",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/signal-exit",
      "resolved": "https://registry.npmjs.org/signal-exit/-/signal-exit-3.0.3.tgz",
      "version": "3.0.3",
    },
    "source-map" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/istanbul-lib-source-maps",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/merge-source-map",
          "name": "source-map",
          "spec": "^0.6.1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/source-map-support",
          "name": "source-map",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map",
      "name": "source-map",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/source-map",
      "resolved": "https://registry.npmjs.org/source-map/-/source-map-0.6.1.tgz",
      "version": "0.6.1",
    },
    "source-map-support" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "source-map-support",
          "spec": "^0.5.10",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "source-map-support",
          "spec": "^0.5.6",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "buffer-from" => EdgeOut {
          "name": "buffer-from",
          "spec": "^1.0.0",
          "to": "node_modules/buffer-from",
          "type": "prod",
        },
        "source-map" => EdgeOut {
          "name": "source-map",
          "spec": "^0.6.0",
          "to": "node_modules/source-map",
          "type": "prod",
        },
      },
      "location": "node_modules/source-map-support",
      "name": "source-map-support",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/source-map-support",
      "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.16.tgz",
      "version": "0.5.16",
    },
    "spawn-wrap" => ArboristNode {
      "children": Map {
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "mkdirp",
              "spec": "^0.5.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spawn-wrap/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.4.tgz",
          "version": "0.5.4",
        },
        "which" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/spawn-wrap",
              "name": "which",
              "spec": "^1.3.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "isexe" => EdgeOut {
              "name": "isexe",
              "spec": "^2.0.0",
              "to": "node_modules/isexe",
              "type": "prod",
            },
          },
          "location": "node_modules/spawn-wrap/node_modules/which",
          "name": "which",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spawn-wrap/node_modules/which",
          "resolved": "https://registry.npmjs.org/which/-/which-1.3.1.tgz",
          "version": "1.3.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "spawn-wrap",
          "spec": "^1.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.5.6",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.0",
          "to": "node_modules/spawn-wrap/node_modules/mkdirp",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.1",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.2",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "which" => EdgeOut {
          "name": "which",
          "spec": "^1.3.0",
          "to": "node_modules/spawn-wrap/node_modules/which",
          "type": "prod",
        },
      },
      "location": "node_modules/spawn-wrap",
      "name": "spawn-wrap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spawn-wrap",
      "resolved": "https://registry.npmjs.org/spawn-wrap/-/spawn-wrap-1.4.3.tgz",
      "version": "1.4.3",
    },
    "spdx-correct" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-correct",
      "name": "spdx-correct",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spdx-correct",
      "resolved": "https://registry.npmjs.org/spdx-correct/-/spdx-correct-3.1.0.tgz",
      "version": "3.1.0",
    },
    "spdx-exceptions" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-exceptions",
      "name": "spdx-exceptions",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spdx-exceptions",
      "resolved": "https://registry.npmjs.org/spdx-exceptions/-/spdx-exceptions-2.2.0.tgz",
      "version": "2.2.0",
    },
    "spdx-expression-parse" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/validate-npm-package-license",
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-exceptions" => EdgeOut {
          "name": "spdx-exceptions",
          "spec": "^2.1.0",
          "to": "node_modules/spdx-exceptions",
          "type": "prod",
        },
        "spdx-license-ids" => EdgeOut {
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-license-ids",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-expression-parse",
      "name": "spdx-expression-parse",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spdx-expression-parse",
      "resolved": "https://registry.npmjs.org/spdx-expression-parse/-/spdx-expression-parse-3.0.0.tgz",
      "version": "3.0.0",
    },
    "spdx-license-ids" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/spdx-correct",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/spdx-expression-parse",
          "name": "spdx-license-ids",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/spdx-license-ids",
      "name": "spdx-license-ids",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/spdx-license-ids",
      "resolved": "https://registry.npmjs.org/spdx-license-ids/-/spdx-license-ids-3.0.5.tgz",
      "version": "3.0.5",
    },
    "sprintf-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/argparse",
          "name": "sprintf-js",
          "spec": "~1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/sprintf-js",
      "name": "sprintf-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/sprintf-js",
      "resolved": "https://registry.npmjs.org/sprintf-js/-/sprintf-js-1.0.3.tgz",
      "version": "1.0.3",
    },
    "sshpk" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/http-signature",
          "name": "sshpk",
          "spec": "^1.7.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "asn1" => EdgeOut {
          "name": "asn1",
          "spec": "~0.2.3",
          "to": "node_modules/asn1",
          "type": "prod",
        },
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "bcrypt-pbkdf" => EdgeOut {
          "name": "bcrypt-pbkdf",
          "spec": "^1.0.0",
          "to": "node_modules/bcrypt-pbkdf",
          "type": "prod",
        },
        "dashdash" => EdgeOut {
          "name": "dashdash",
          "spec": "^1.12.0",
          "to": "node_modules/dashdash",
          "type": "prod",
        },
        "ecc-jsbn" => EdgeOut {
          "name": "ecc-jsbn",
          "spec": "~0.1.1",
          "to": "node_modules/ecc-jsbn",
          "type": "prod",
        },
        "getpass" => EdgeOut {
          "name": "getpass",
          "spec": "^0.1.1",
          "to": "node_modules/getpass",
          "type": "prod",
        },
        "jsbn" => EdgeOut {
          "name": "jsbn",
          "spec": "~0.1.0",
          "to": "node_modules/jsbn",
          "type": "prod",
        },
        "safer-buffer" => EdgeOut {
          "name": "safer-buffer",
          "spec": "^2.0.2",
          "to": "node_modules/safer-buffer",
          "type": "prod",
        },
        "tweetnacl" => EdgeOut {
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "to": "node_modules/tweetnacl",
          "type": "prod",
        },
      },
      "location": "node_modules/sshpk",
      "name": "sshpk",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/sshpk",
      "resolved": "https://registry.npmjs.org/sshpk/-/sshpk-1.16.1.tgz",
      "version": "1.16.1",
    },
    "stack-utils" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "stack-utils",
          "spec": "^1.0.2",
          "type": "prod",
        },
      },
      "location": "node_modules/stack-utils",
      "name": "stack-utils",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/stack-utils",
      "resolved": "https://registry.npmjs.org/stack-utils/-/stack-utils-1.0.2.tgz",
      "version": "1.0.2",
    },
    "string_decoder" => ArboristNode {
      "children": Map {
        "safe-buffer" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/string_decoder",
              "name": "safe-buffer",
              "spec": "~5.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/string_decoder/node_modules/safe-buffer",
          "name": "safe-buffer",
          "optional": true,
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/string_decoder/node_modules/safe-buffer",
          "resolved": "https://registry.npmjs.org/safe-buffer/-/safe-buffer-5.1.2.tgz",
          "version": "5.1.2",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "string_decoder",
          "spec": "~1.1.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "~5.1.0",
          "to": "node_modules/string_decoder/node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/string_decoder",
      "name": "string_decoder",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/string_decoder",
      "resolved": "https://registry.npmjs.org/string_decoder/-/string_decoder-1.1.1.tgz",
      "version": "1.1.1",
    },
    "strip-ansi" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/unicode-length",
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "ansi-regex" => EdgeOut {
          "name": "ansi-regex",
          "spec": "^2.0.0",
          "to": "node_modules/ansi-regex",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-ansi",
      "name": "strip-ansi",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/strip-ansi",
      "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-3.0.1.tgz",
      "version": "3.0.1",
    },
    "strip-bom" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/default-require-extensions",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/load-json-file",
          "name": "strip-bom",
          "spec": "^3.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/strip-bom",
      "name": "strip-bom",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/strip-bom",
      "resolved": "https://registry.npmjs.org/strip-bom/-/strip-bom-3.0.0.tgz",
      "version": "3.0.0",
    },
    "supports-color" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/chalk",
          "name": "supports-color",
          "spec": "^5.3.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "has-flag" => EdgeOut {
          "name": "has-flag",
          "spec": "^3.0.0",
          "to": "node_modules/has-flag",
          "type": "prod",
        },
      },
      "location": "node_modules/supports-color",
      "name": "supports-color",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/supports-color",
      "resolved": "https://registry.npmjs.org/supports-color/-/supports-color-5.5.0.tgz",
      "version": "5.5.0",
    },
    "tap" => ArboristNode {
      "children": Map {
        "minimist" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap/node_modules/mkdirp",
              "name": "minimist",
              "spec": "^1.2.5",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/minimist",
          "name": "minimist",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap/node_modules/minimist",
          "version": "1.2.5",
        },
        "mkdirp" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap",
              "name": "mkdirp",
              "spec": "^0.5.1",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "minimist" => EdgeOut {
              "name": "minimist",
              "spec": "^1.2.5",
              "to": "node_modules/tap/node_modules/minimist",
              "type": "prod",
            },
          },
          "location": "node_modules/tap/node_modules/mkdirp",
          "name": "mkdirp",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap/node_modules/mkdirp",
          "resolved": "https://registry.npmjs.org/mkdirp/-/mkdirp-0.5.4.tgz",
          "version": "0.5.4",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "tap",
          "spec": "12",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "bind-obj-methods" => EdgeOut {
          "name": "bind-obj-methods",
          "spec": "^2.0.0",
          "to": "node_modules/bind-obj-methods",
          "type": "prod",
        },
        "browser-process-hrtime" => EdgeOut {
          "name": "browser-process-hrtime",
          "spec": "^1.0.0",
          "to": "node_modules/browser-process-hrtime",
          "type": "prod",
        },
        "capture-stack-trace" => EdgeOut {
          "name": "capture-stack-trace",
          "spec": "^1.0.0",
          "to": "node_modules/capture-stack-trace",
          "type": "prod",
        },
        "clean-yaml-object" => EdgeOut {
          "name": "clean-yaml-object",
          "spec": "^0.1.0",
          "to": "node_modules/clean-yaml-object",
          "type": "prod",
        },
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "coveralls" => EdgeOut {
          "name": "coveralls",
          "spec": "^3.0.2",
          "to": "node_modules/coveralls",
          "type": "prod",
        },
        "domain-browser" => EdgeOut {
          "name": "domain-browser",
          "spec": "^1.2.0",
          "to": "node_modules/domain-browser",
          "type": "prod",
        },
        "esm" => EdgeOut {
          "name": "esm",
          "spec": "^3.2.5",
          "to": "node_modules/esm",
          "type": "prod",
        },
        "foreground-child" => EdgeOut {
          "name": "foreground-child",
          "spec": "^1.3.3",
          "to": "node_modules/foreground-child",
          "type": "prod",
        },
        "fs-exists-cached" => EdgeOut {
          "name": "fs-exists-cached",
          "spec": "^1.0.0",
          "to": "node_modules/fs-exists-cached",
          "type": "prod",
        },
        "function-loop" => EdgeOut {
          "name": "function-loop",
          "spec": "^1.0.1",
          "to": "node_modules/function-loop",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "isexe" => EdgeOut {
          "name": "isexe",
          "spec": "^2.0.0",
          "to": "node_modules/isexe",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.13.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^2.3.5",
          "to": "node_modules/minipass",
          "type": "prod",
        },
        "mkdirp" => EdgeOut {
          "name": "mkdirp",
          "spec": "^0.5.1",
          "to": "node_modules/tap/node_modules/mkdirp",
          "type": "prod",
        },
        "nyc" => EdgeOut {
          "name": "nyc",
          "spec": "^14.0.0",
          "to": "node_modules/nyc",
          "type": "prod",
        },
        "opener" => EdgeOut {
          "name": "opener",
          "spec": "^1.5.1",
          "to": "node_modules/opener",
          "type": "prod",
        },
        "os-homedir" => EdgeOut {
          "name": "os-homedir",
          "spec": "^1.0.2",
          "to": "node_modules/os-homedir",
          "type": "prod",
        },
        "own-or" => EdgeOut {
          "name": "own-or",
          "spec": "^1.0.0",
          "to": "node_modules/own-or",
          "type": "prod",
        },
        "own-or-env" => EdgeOut {
          "name": "own-or-env",
          "spec": "^1.0.1",
          "to": "node_modules/own-or-env",
          "type": "prod",
        },
        "rimraf" => EdgeOut {
          "name": "rimraf",
          "spec": "^2.6.3",
          "to": "node_modules/rimraf",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.0",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.10",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "stack-utils" => EdgeOut {
          "name": "stack-utils",
          "spec": "^1.0.2",
          "to": "node_modules/stack-utils",
          "type": "prod",
        },
        "tap-mocha-reporter" => EdgeOut {
          "name": "tap-mocha-reporter",
          "spec": "^3.0.9",
          "to": "node_modules/tap-mocha-reporter",
          "type": "prod",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^7.0.0",
          "to": "node_modules/tap-parser",
          "type": "prod",
        },
        "tmatch" => EdgeOut {
          "name": "tmatch",
          "spec": "^4.0.0",
          "to": "node_modules/tmatch",
          "type": "prod",
        },
        "trivial-deferred" => EdgeOut {
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "to": "node_modules/trivial-deferred",
          "type": "prod",
        },
        "ts-node" => EdgeOut {
          "name": "ts-node",
          "spec": "^8.0.2",
          "to": "node_modules/ts-node",
          "type": "prod",
        },
        "tsame" => EdgeOut {
          "name": "tsame",
          "spec": "^2.0.1",
          "to": "node_modules/tsame",
          "type": "prod",
        },
        "typescript" => EdgeOut {
          "name": "typescript",
          "spec": "^3.3.3",
          "to": "node_modules/typescript",
          "type": "prod",
        },
        "write-file-atomic" => EdgeOut {
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "to": "node_modules/write-file-atomic",
          "type": "prod",
        },
        "yapool" => EdgeOut {
          "name": "yapool",
          "spec": "^1.0.0",
          "to": "node_modules/yapool",
          "type": "prod",
        },
      },
      "location": "node_modules/tap",
      "name": "tap",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap",
      "resolved": "https://registry.npmjs.org/tap/-/tap-12.7.0.tgz",
      "version": "12.7.0",
    },
    "tap-mocha-reporter" => ArboristNode {
      "children": Map {
        "debug" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "debug",
              "spec": "^2.1.3",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ms" => EdgeOut {
              "name": "ms",
              "spec": "2.0.0",
              "to": "node_modules/tap-mocha-reporter/node_modules/ms",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/debug",
          "name": "debug",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-mocha-reporter/node_modules/debug",
          "resolved": "https://registry.npmjs.org/debug/-/debug-2.6.9.tgz",
          "version": "2.6.9",
        },
        "diff" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "diff",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/diff",
          "name": "diff",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-mocha-reporter/node_modules/diff",
          "resolved": "https://registry.npmjs.org/diff/-/diff-1.4.0.tgz",
          "version": "1.4.0",
        },
        "ms" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter/node_modules/debug",
              "name": "ms",
              "spec": "2.0.0",
              "type": "prod",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/ms",
          "name": "ms",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-mocha-reporter/node_modules/ms",
          "resolved": "https://registry.npmjs.org/ms/-/ms-2.0.0.tgz",
          "version": "2.0.0",
        },
        "tap-parser" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/tap-mocha-reporter",
              "name": "tap-parser",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "events-to-array" => EdgeOut {
              "name": "events-to-array",
              "spec": "^1.0.1",
              "to": "node_modules/events-to-array",
              "type": "prod",
            },
            "js-yaml" => EdgeOut {
              "name": "js-yaml",
              "spec": "^3.2.7",
              "to": "node_modules/js-yaml",
              "type": "prod",
            },
            "readable-stream" => EdgeOut {
              "name": "readable-stream",
              "spec": "^2",
              "to": "node_modules/readable-stream",
              "type": "optional",
            },
          },
          "location": "node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "name": "tap-parser",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-5.4.0.tgz",
          "version": "5.4.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-mocha-reporter",
          "spec": "^3.0.9",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "color-support" => EdgeOut {
          "name": "color-support",
          "spec": "^1.1.0",
          "to": "node_modules/color-support",
          "type": "prod",
        },
        "debug" => EdgeOut {
          "name": "debug",
          "spec": "^2.1.3",
          "to": "node_modules/tap-mocha-reporter/node_modules/debug",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^1.3.2",
          "to": "node_modules/tap-mocha-reporter/node_modules/diff",
          "type": "prod",
        },
        "escape-string-regexp" => EdgeOut {
          "name": "escape-string-regexp",
          "spec": "^1.0.3",
          "to": "node_modules/escape-string-regexp",
          "type": "prod",
        },
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.0.5",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.3.1",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "readable-stream" => EdgeOut {
          "name": "readable-stream",
          "spec": "^2.1.5",
          "to": "node_modules/readable-stream",
          "type": "optional",
        },
        "tap-parser" => EdgeOut {
          "name": "tap-parser",
          "spec": "^5.1.0",
          "to": "node_modules/tap-mocha-reporter/node_modules/tap-parser",
          "type": "prod",
        },
        "unicode-length" => EdgeOut {
          "name": "unicode-length",
          "spec": "^1.0.0",
          "to": "node_modules/unicode-length",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-mocha-reporter",
      "name": "tap-mocha-reporter",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-mocha-reporter",
      "resolved": "https://registry.npmjs.org/tap-mocha-reporter/-/tap-mocha-reporter-3.0.9.tgz",
      "version": "3.0.9",
    },
    "tap-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tap-parser",
          "spec": "^7.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "events-to-array" => EdgeOut {
          "name": "events-to-array",
          "spec": "^1.0.1",
          "to": "node_modules/events-to-array",
          "type": "prod",
        },
        "js-yaml" => EdgeOut {
          "name": "js-yaml",
          "spec": "^3.2.7",
          "to": "node_modules/js-yaml",
          "type": "prod",
        },
        "minipass" => EdgeOut {
          "name": "minipass",
          "spec": "^2.2.0",
          "to": "node_modules/minipass",
          "type": "prod",
        },
      },
      "location": "node_modules/tap-parser",
      "name": "tap-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tap-parser",
      "resolved": "https://registry.npmjs.org/tap-parser/-/tap-parser-7.0.0.tgz",
      "version": "7.0.0",
    },
    "test-exclude" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "test-exclude",
          "spec": "^5.2.3",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "glob" => EdgeOut {
          "name": "glob",
          "spec": "^7.1.3",
          "to": "node_modules/glob",
          "type": "prod",
        },
        "minimatch" => EdgeOut {
          "name": "minimatch",
          "spec": "^3.0.4",
          "to": "node_modules/minimatch",
          "type": "prod",
        },
        "read-pkg-up" => EdgeOut {
          "name": "read-pkg-up",
          "spec": "^4.0.0",
          "to": "node_modules/read-pkg-up",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
      },
      "location": "node_modules/test-exclude",
      "name": "test-exclude",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/test-exclude",
      "resolved": "https://registry.npmjs.org/test-exclude/-/test-exclude-5.2.3.tgz",
      "version": "5.2.3",
    },
    "tmatch" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tmatch",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tmatch",
      "name": "tmatch",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tmatch",
      "resolved": "https://registry.npmjs.org/tmatch/-/tmatch-4.0.0.tgz",
      "version": "4.0.0",
    },
    "to-fast-properties" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@babel/types",
          "name": "to-fast-properties",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/to-fast-properties",
      "name": "to-fast-properties",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/to-fast-properties",
      "resolved": "https://registry.npmjs.org/to-fast-properties/-/to-fast-properties-2.0.0.tgz",
      "version": "2.0.0",
    },
    "tough-cookie" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tough-cookie",
          "spec": "~2.5.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "psl" => EdgeOut {
          "name": "psl",
          "spec": "^1.1.28",
          "to": "node_modules/psl",
          "type": "prod",
        },
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.1",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/tough-cookie",
      "name": "tough-cookie",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tough-cookie",
      "resolved": "https://registry.npmjs.org/tough-cookie/-/tough-cookie-2.5.0.tgz",
      "version": "2.5.0",
    },
    "trivial-deferred" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "trivial-deferred",
          "spec": "^1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/trivial-deferred",
      "name": "trivial-deferred",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/trivial-deferred",
      "resolved": "https://registry.npmjs.org/trivial-deferred/-/trivial-deferred-1.0.1.tgz",
      "version": "1.0.1",
    },
    "ts-node" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "ts-node",
          "spec": "^8.0.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "arg" => EdgeOut {
          "name": "arg",
          "spec": "^4.1.0",
          "to": "node_modules/arg",
          "type": "prod",
        },
        "diff" => EdgeOut {
          "name": "diff",
          "spec": "^4.0.1",
          "to": "node_modules/diff",
          "type": "prod",
        },
        "make-error" => EdgeOut {
          "name": "make-error",
          "spec": "^1.1.1",
          "to": "node_modules/make-error",
          "type": "prod",
        },
        "source-map-support" => EdgeOut {
          "name": "source-map-support",
          "spec": "^0.5.6",
          "to": "node_modules/source-map-support",
          "type": "prod",
        },
        "yn" => EdgeOut {
          "name": "yn",
          "spec": "3.1.1",
          "to": "node_modules/yn",
          "type": "prod",
        },
      },
      "location": "node_modules/ts-node",
      "name": "ts-node",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/ts-node",
      "resolved": "https://registry.npmjs.org/ts-node/-/ts-node-8.8.1.tgz",
      "version": "8.8.1",
    },
    "tsame" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "tsame",
          "spec": "^2.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/tsame",
      "name": "tsame",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tsame",
      "resolved": "https://registry.npmjs.org/tsame/-/tsame-2.0.1.tgz",
      "version": "2.0.1",
    },
    "tunnel-agent" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/request",
          "name": "tunnel-agent",
          "spec": "^0.6.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "safe-buffer" => EdgeOut {
          "name": "safe-buffer",
          "spec": "^5.0.1",
          "to": "node_modules/safe-buffer",
          "type": "prod",
        },
      },
      "location": "node_modules/tunnel-agent",
      "name": "tunnel-agent",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tunnel-agent",
      "resolved": "https://registry.npmjs.org/tunnel-agent/-/tunnel-agent-0.6.0.tgz",
      "version": "0.6.0",
    },
    "tweetnacl" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/bcrypt-pbkdf",
          "name": "tweetnacl",
          "spec": "^0.14.3",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/sshpk",
          "name": "tweetnacl",
          "spec": "~0.14.0",
          "type": "prod",
        },
      },
      "location": "node_modules/tweetnacl",
      "name": "tweetnacl",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/tweetnacl",
      "resolved": "https://registry.npmjs.org/tweetnacl/-/tweetnacl-0.14.5.tgz",
      "version": "0.14.5",
    },
    "typescript" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "typescript",
          "spec": "^3.3.3",
          "type": "prod",
        },
      },
      "location": "node_modules/typescript",
      "name": "typescript",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/typescript",
      "resolved": "https://registry.npmjs.org/typescript/-/typescript-3.8.3.tgz",
      "version": "3.8.3",
    },
    "unicode-length" => ArboristNode {
      "children": Map {
        "punycode" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/unicode-length",
              "name": "punycode",
              "spec": "^1.3.2",
              "type": "prod",
            },
          },
          "location": "node_modules/unicode-length/node_modules/punycode",
          "name": "punycode",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/unicode-length/node_modules/punycode",
          "resolved": "https://registry.npmjs.org/punycode/-/punycode-1.4.1.tgz",
          "version": "1.4.1",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap-mocha-reporter",
          "name": "unicode-length",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^1.3.2",
          "to": "node_modules/unicode-length/node_modules/punycode",
          "type": "prod",
        },
        "strip-ansi" => EdgeOut {
          "name": "strip-ansi",
          "spec": "^3.0.1",
          "to": "node_modules/strip-ansi",
          "type": "prod",
        },
      },
      "location": "node_modules/unicode-length",
      "name": "unicode-length",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/unicode-length",
      "resolved": "https://registry.npmjs.org/unicode-length/-/unicode-length-1.0.3.tgz",
      "version": "1.0.3",
    },
    "uri-js" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ajv",
          "name": "uri-js",
          "spec": "^4.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "punycode" => EdgeOut {
          "name": "punycode",
          "spec": "^2.1.0",
          "to": "node_modules/punycode",
          "type": "prod",
        },
      },
      "location": "node_modules/uri-js",
      "name": "uri-js",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/uri-js",
      "resolved": "https://registry.npmjs.org/uri-js/-/uri-js-4.2.2.tgz",
      "version": "4.2.2",
    },
    "util-deprecate" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/readable-stream",
          "name": "util-deprecate",
          "spec": "~1.0.1",
          "type": "prod",
        },
      },
      "location": "node_modules/util-deprecate",
      "name": "util-deprecate",
      "optional": true,
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/util-deprecate",
      "resolved": "https://registry.npmjs.org/util-deprecate/-/util-deprecate-1.0.2.tgz",
      "version": "1.0.2",
    },
    "uuid" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/request",
          "name": "uuid",
          "spec": "^3.3.2",
          "type": "prod",
        },
      },
      "location": "node_modules/uuid",
      "name": "uuid",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/uuid",
      "resolved": "https://registry.npmjs.org/uuid/-/uuid-3.4.0.tgz",
      "version": "3.4.0",
    },
    "validate-npm-package-license" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/normalize-package-data",
          "name": "validate-npm-package-license",
          "spec": "^3.0.1",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "spdx-correct" => EdgeOut {
          "name": "spdx-correct",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-correct",
          "type": "prod",
        },
        "spdx-expression-parse" => EdgeOut {
          "name": "spdx-expression-parse",
          "spec": "^3.0.0",
          "to": "node_modules/spdx-expression-parse",
          "type": "prod",
        },
      },
      "location": "node_modules/validate-npm-package-license",
      "name": "validate-npm-package-license",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/validate-npm-package-license",
      "resolved": "https://registry.npmjs.org/validate-npm-package-license/-/validate-npm-package-license-3.0.4.tgz",
      "version": "3.0.4",
    },
    "verror" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/jsprim",
          "name": "verror",
          "spec": "1.10.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "assert-plus" => EdgeOut {
          "name": "assert-plus",
          "spec": "^1.0.0",
          "to": "node_modules/assert-plus",
          "type": "prod",
        },
        "core-util-is" => EdgeOut {
          "name": "core-util-is",
          "spec": "1.0.2",
          "to": "node_modules/core-util-is",
          "type": "prod",
        },
        "extsprintf" => EdgeOut {
          "name": "extsprintf",
          "spec": "^1.2.0",
          "to": "node_modules/extsprintf",
          "type": "prod",
        },
      },
      "location": "node_modules/verror",
      "name": "verror",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/verror",
      "resolved": "https://registry.npmjs.org/verror/-/verror-1.10.0.tgz",
      "version": "1.10.0",
    },
    "which-module" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "which-module",
          "spec": "^2.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/which-module",
      "name": "which-module",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/which-module",
      "resolved": "https://registry.npmjs.org/which-module/-/which-module-2.0.0.tgz",
      "version": "2.0.0",
    },
    "wrappy" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/inflight",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/once",
          "name": "wrappy",
          "spec": "1",
          "type": "prod",
        },
      },
      "location": "node_modules/wrappy",
      "name": "wrappy",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/wrappy",
      "resolved": "https://registry.npmjs.org/wrappy/-/wrappy-1.0.2.tgz",
      "version": "1.0.2",
    },
    "write-file-atomic" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/caching-transform",
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/tap",
          "name": "write-file-atomic",
          "spec": "^2.4.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "graceful-fs" => EdgeOut {
          "name": "graceful-fs",
          "spec": "^4.1.11",
          "to": "node_modules/graceful-fs",
          "type": "prod",
        },
        "imurmurhash" => EdgeOut {
          "name": "imurmurhash",
          "spec": "^0.1.4",
          "to": "node_modules/imurmurhash",
          "type": "prod",
        },
        "signal-exit" => EdgeOut {
          "name": "signal-exit",
          "spec": "^3.0.2",
          "to": "node_modules/signal-exit",
          "type": "prod",
        },
      },
      "location": "node_modules/write-file-atomic",
      "name": "write-file-atomic",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/write-file-atomic",
      "resolved": "https://registry.npmjs.org/write-file-atomic/-/write-file-atomic-2.4.3.tgz",
      "version": "2.4.3",
    },
    "y18n" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "y18n",
          "spec": "^4.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/y18n",
      "name": "y18n",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/y18n",
      "resolved": "https://registry.npmjs.org/y18n/-/y18n-4.0.0.tgz",
      "version": "4.0.0",
    },
    "yallist" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/lru-cache",
          "name": "yallist",
          "spec": "^2.1.2",
          "type": "prod",
        },
      },
      "location": "node_modules/yallist",
      "name": "yallist",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yallist",
      "resolved": "https://registry.npmjs.org/yallist/-/yallist-2.1.2.tgz",
      "version": "2.1.2",
    },
    "yapool" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/tap",
          "name": "yapool",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/yapool",
      "name": "yapool",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yapool",
      "resolved": "https://registry.npmjs.org/yapool/-/yapool-1.0.0.tgz",
      "version": "1.0.0",
    },
    "yargs" => ArboristNode {
      "children": Map {
        "ansi-regex" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/strip-ansi",
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/ansi-regex",
          "name": "ansi-regex",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs/node_modules/ansi-regex",
          "resolved": "https://registry.npmjs.org/ansi-regex/-/ansi-regex-4.1.0.tgz",
          "version": "4.1.0",
        },
        "cliui" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "cliui",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.1.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
            "wrap-ansi" => EdgeOut {
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/wrap-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/cliui",
          "name": "cliui",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs/node_modules/cliui",
          "resolved": "https://registry.npmjs.org/cliui/-/cliui-5.0.0.tgz",
          "version": "5.0.0",
        },
        "string-width" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "string-width",
              "spec": "^3.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "string-width",
              "spec": "^3.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "emoji-regex" => EdgeOut {
              "name": "emoji-regex",
              "spec": "^7.0.1",
              "to": "node_modules/emoji-regex",
              "type": "prod",
            },
            "is-fullwidth-code-point" => EdgeOut {
              "name": "is-fullwidth-code-point",
              "spec": "^2.0.0",
              "to": "node_modules/is-fullwidth-code-point",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/string-width",
          "name": "string-width",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs/node_modules/string-width",
          "resolved": "https://registry.npmjs.org/string-width/-/string-width-3.1.0.tgz",
          "version": "3.1.0",
        },
        "strip-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "strip-ansi",
              "spec": "^5.2.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/string-width",
              "name": "strip-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
            EdgeIn {
              "from": "node_modules/yargs/node_modules/wrap-ansi",
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-regex" => EdgeOut {
              "name": "ansi-regex",
              "spec": "^4.1.0",
              "to": "node_modules/yargs/node_modules/ansi-regex",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/strip-ansi",
          "name": "strip-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs/node_modules/strip-ansi",
          "resolved": "https://registry.npmjs.org/strip-ansi/-/strip-ansi-5.2.0.tgz",
          "version": "5.2.0",
        },
        "wrap-ansi" => ArboristNode {
          "edgesIn": Set {
            EdgeIn {
              "from": "node_modules/yargs/node_modules/cliui",
              "name": "wrap-ansi",
              "spec": "^5.1.0",
              "type": "prod",
            },
          },
          "edgesOut": Map {
            "ansi-styles" => EdgeOut {
              "name": "ansi-styles",
              "spec": "^3.2.0",
              "to": "node_modules/ansi-styles",
              "type": "prod",
            },
            "string-width" => EdgeOut {
              "name": "string-width",
              "spec": "^3.0.0",
              "to": "node_modules/yargs/node_modules/string-width",
              "type": "prod",
            },
            "strip-ansi" => EdgeOut {
              "name": "strip-ansi",
              "spec": "^5.0.0",
              "to": "node_modules/yargs/node_modules/strip-ansi",
              "type": "prod",
            },
          },
          "location": "node_modules/yargs/node_modules/wrap-ansi",
          "name": "wrap-ansi",
          "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs/node_modules/wrap-ansi",
          "resolved": "https://registry.npmjs.org/wrap-ansi/-/wrap-ansi-5.1.0.tgz",
          "version": "5.1.0",
        },
      },
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs",
          "spec": "^13.2.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "cliui" => EdgeOut {
          "name": "cliui",
          "spec": "^5.0.0",
          "to": "node_modules/yargs/node_modules/cliui",
          "type": "prod",
        },
        "find-up" => EdgeOut {
          "name": "find-up",
          "spec": "^3.0.0",
          "to": "node_modules/find-up",
          "type": "prod",
        },
        "get-caller-file" => EdgeOut {
          "name": "get-caller-file",
          "spec": "^2.0.1",
          "to": "node_modules/get-caller-file",
          "type": "prod",
        },
        "require-directory" => EdgeOut {
          "name": "require-directory",
          "spec": "^2.1.1",
          "to": "node_modules/require-directory",
          "type": "prod",
        },
        "require-main-filename" => EdgeOut {
          "name": "require-main-filename",
          "spec": "^2.0.0",
          "to": "node_modules/require-main-filename",
          "type": "prod",
        },
        "set-blocking" => EdgeOut {
          "name": "set-blocking",
          "spec": "^2.0.0",
          "to": "node_modules/set-blocking",
          "type": "prod",
        },
        "string-width" => EdgeOut {
          "name": "string-width",
          "spec": "^3.0.0",
          "to": "node_modules/yargs/node_modules/string-width",
          "type": "prod",
        },
        "which-module" => EdgeOut {
          "name": "which-module",
          "spec": "^2.0.0",
          "to": "node_modules/which-module",
          "type": "prod",
        },
        "y18n" => EdgeOut {
          "name": "y18n",
          "spec": "^4.0.0",
          "to": "node_modules/y18n",
          "type": "prod",
        },
        "yargs-parser" => EdgeOut {
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "to": "node_modules/yargs-parser",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs",
      "name": "yargs",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs",
      "resolved": "https://registry.npmjs.org/yargs/-/yargs-13.3.2.tgz",
      "version": "13.3.2",
    },
    "yargs-parser" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/nyc",
          "name": "yargs-parser",
          "spec": "^13.0.0",
          "type": "prod",
        },
        EdgeIn {
          "from": "node_modules/yargs",
          "name": "yargs-parser",
          "spec": "^13.1.2",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "camelcase" => EdgeOut {
          "name": "camelcase",
          "spec": "^5.0.0",
          "to": "node_modules/camelcase",
          "type": "prod",
        },
        "decamelize" => EdgeOut {
          "name": "decamelize",
          "spec": "^1.2.0",
          "to": "node_modules/decamelize",
          "type": "prod",
        },
      },
      "location": "node_modules/yargs-parser",
      "name": "yargs-parser",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yargs-parser",
      "resolved": "https://registry.npmjs.org/yargs-parser/-/yargs-parser-13.1.2.tgz",
      "version": "13.1.2",
    },
    "yn" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/ts-node",
          "name": "yn",
          "spec": "3.1.1",
          "type": "prod",
        },
      },
      "location": "node_modules/yn",
      "name": "yn",
      "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value/node_modules/yn",
      "resolved": "https://registry.npmjs.org/yn/-/yn-3.1.1.tgz",
      "version": "3.1.1",
    },
  },
  "edgesOut": Map {
    "tap" => EdgeOut {
      "name": "tap",
      "spec": "12",
      "to": "node_modules/tap",
      "type": "prod",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value",
  "path": "{CWD}/test/arborist/tap-testdir-reify-weirdly-broken-lockfile-without-resolved-value",
}
`

exports[`test/arborist/reify.js TAP workspaces add new workspaces dep > should update package-lock with new added dep 1`] = `
Object {
  "dependencies": Object {
    "a": Object {
      "requires": Object {
        "abbrev": "^1.1.1",
      },
      "version": "file:a",
    },
    "abbrev": Object {
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "lockfileVersion": 2,
  "name": "workspaces-add-new-dep",
  "packages": Object {
    "": Object {
      "name": "workspaces-add-new-dep",
      "workspaces": Array [
        "a",
      ],
    },
    "a": Object {
      "dependencies": Object {
        "abbrev": "^1.1.1",
      },
      "version": "1.0.0",
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "a",
    },
    "node_modules/abbrev": Object {
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP workspaces reify from an actual loaded workspace env > should not clean up entire nm folder for no reason 1`] = `
ArboristNode {
  "children": Map {
    "@ruyadorno/dep-bar" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "a",
          "name": "@ruyadorno/dep-bar",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "edgesOut": Map {
        "@ruyadorno/dep-foo" => EdgeOut {
          "name": "@ruyadorno/dep-foo",
          "spec": "^1.0.0",
          "to": "node_modules/@ruyadorno/dep-foo",
          "type": "prod",
        },
      },
      "location": "node_modules/@ruyadorno/dep-bar",
      "name": "@ruyadorno/dep-bar",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/node_modules/@ruyadorno/dep-bar",
      "resolved": "https://registry.npmjs.org/@ruyadorno/dep-bar/-/dep-bar-1.0.0.tgz",
      "version": "1.0.0",
    },
    "@ruyadorno/dep-foo" => ArboristNode {
      "edgesIn": Set {
        EdgeIn {
          "from": "node_modules/@ruyadorno/dep-bar",
          "name": "@ruyadorno/dep-foo",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "location": "node_modules/@ruyadorno/dep-foo",
      "name": "@ruyadorno/dep-foo",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/node_modules/@ruyadorno/dep-foo",
      "resolved": "https://registry.npmjs.org/@ruyadorno/dep-foo/-/dep-foo-1.0.0.tgz",
      "version": "1.0.0",
    },
    "abbrev" => ArboristNode {
      "dev": true,
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "abbrev",
          "spec": "^1.0.0",
          "type": "dev",
        },
      },
      "location": "node_modules/abbrev",
      "name": "abbrev",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/node_modules/abbrev",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "pkg-a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "pkg-a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/a",
          "type": "workspace",
        },
      },
      "location": "node_modules/pkg-a",
      "name": "pkg-a",
      "packageName": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/node_modules/pkg-a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
    },
  },
  "edgesOut": Map {
    "abbrev" => EdgeOut {
      "name": "abbrev",
      "spec": "^1.0.0",
      "to": "node_modules/abbrev",
      "type": "dev",
    },
    "pkg-a" => EdgeOut {
      "name": "pkg-a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/a",
      "to": "node_modules/pkg-a",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "@ruyadorno/dep-bar" => EdgeOut {
          "name": "@ruyadorno/dep-bar",
          "spec": "^1.0.0",
          "to": "node_modules/@ruyadorno/dep-bar",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "packageName": "pkg-a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env/a",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env",
  "packageName": "workspaces-non-simplistic",
  "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-from-an-actual-loaded-workspace-env",
  "workspaces": Map {
    "pkg-a" => "a",
  },
}
`

exports[`test/arborist/reify.js TAP workspaces reify simple-workspaces > should reify simple workspaces 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/a",
      "resolved": "file:../a",
      "target": ArboristNode {
        "location": "a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/b",
          "type": "workspace",
        },
        EdgeIn {
          "from": "a",
          "name": "b",
          "spec": "^1.0.0",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/b",
      "resolved": "file:../b",
      "target": ArboristNode {
        "location": "b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "edgesOut": Map {
        "b" => EdgeOut {
          "name": "b",
          "spec": "^1.0.0",
          "to": "node_modules/b",
          "type": "prod",
        },
      },
      "isWorkspace": true,
      "location": "a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-workspaces-reify-simple-workspaces",
  "packageName": "workspace-simple",
  "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-simple-workspaces",
  "workspaces": Map {
    "a" => "a",
    "b" => "b",
  },
}
`

exports[`test/arborist/reify.js TAP workspaces reify workspaces bin files > expect resolving Promise 1`] = `
ArboristNode {
  "children": Map {
    "a" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "a",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/a",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/node_modules/a",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/a",
      "resolved": "file:../packages/a",
      "target": ArboristNode {
        "location": "packages/a",
      },
      "version": "1.0.0",
    },
    "b" => ArboristLink {
      "edgesIn": Set {
        EdgeIn {
          "from": "",
          "name": "b",
          "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/b",
          "type": "workspace",
        },
      },
      "isWorkspace": true,
      "location": "node_modules/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/node_modules/b",
      "realpath": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/b",
      "resolved": "file:../packages/b",
      "target": ArboristNode {
        "location": "packages/b",
      },
      "version": "1.0.0",
    },
  },
  "edgesOut": Map {
    "a" => EdgeOut {
      "name": "a",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/a",
      "to": "node_modules/a",
      "type": "workspace",
    },
    "b" => EdgeOut {
      "name": "b",
      "spec": "file:{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/b",
      "to": "node_modules/b",
      "type": "workspace",
    },
  },
  "fsChildren": Set {
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/a",
      "name": "a",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/a",
      "version": "1.0.0",
    },
    ArboristNode {
      "isWorkspace": true,
      "location": "packages/b",
      "name": "b",
      "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files/packages/b",
      "version": "1.0.0",
    },
  },
  "isProjectRoot": true,
  "location": "",
  "name": "tap-testdir-reify-workspaces-reify-workspaces-bin-files",
  "packageName": "workspace-duplicate",
  "path": "{CWD}/test/arborist/tap-testdir-reify-workspaces-reify-workspaces-bin-files",
  "workspaces": Map {
    "a" => "packages/a",
    "b" => "packages/b",
  },
}
`

exports[`test/arborist/reify.js TAP workspaces reify workspaces lockfile > should lock workspaces config 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "workspace-simple",
  "packages": Object {
    "": Object {
      "name": "workspace-simple",
      "workspaces": Array [
        "a",
        "b",
      ],
    },
    "a": Object {
      "dependencies": Object {
        "b": "^1.0.0",
      },
      "version": "1.0.0",
    },
    "b": Object {
      "version": "1.0.0",
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "a",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "b",
    },
  },
  "requires": true,
}
`

exports[`test/arborist/reify.js TAP workspaces root as-a-workspace > should produce expected package-lock file 1`] = `
Object {
  "lockfileVersion": 3,
  "name": "workspaces-root-linked",
  "packages": Object {
    "": Object {
      "name": "workspaces-root-linked",
      "version": "1.0.0",
      "workspaces": Array [
        ".",
        "a",
        "b",
      ],
    },
    "a": Object {
      "dependencies": Object {
        "abbrev": "^1.0.0",
      },
      "version": "1.0.0",
    },
    "b": Object {
      "version": "1.0.0",
    },
    "node_modules/a": Object {
      "link": true,
      "resolved": "a",
    },
    "node_modules/abbrev": Object {
      "integrity": "sha512-nne9/IiQ/hzIhY6pdDnbBtz7DjPTKrY00P/zvPSm5pOFkl6xuGrGnXn/VtTNNfNtAfZ9/1RtehkszU9qcTii0Q==",
      "resolved": "https://registry.npmjs.org/abbrev/-/abbrev-1.1.1.tgz",
      "version": "1.1.1",
    },
    "node_modules/b": Object {
      "link": true,
      "resolved": "b",
    },
    "node_modules/workspaces-root-linked": Object {
      "link": true,
      "resolved": "",
    },
  },
  "requires": true,
  "version": "1.0.0",
}
`
